// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.notifications");

var bpm_social_messages = {};

var connectedToBayeuxServer = false;
var logoutOngoing = false;
var currentUser;
window["BPMCometDConnectionEstablished"] = false;

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.SimpleMarkupUtils");

function connectToBayeuxServer(){
 if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) {// If Safari on iPad don't use CometD but allow if Chrome on iPad
    return
  }

  var FUNC_NAME = "connectToBayeuxServer";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, "connecting to  Bayeux server", "notifications", FUNC_NAME);
  window["collaborationContainerDiv"] = dojo.byId("processPortalCollaborationBanner");

  dojo.require("dojox.cometd");
  dojo.require("com.ibm.bpm.social.Aether");
  dojo.require("com.ibm.bpm.social.portal.Topics");
  dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

  dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
  bpm_social_messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

  // configure bayeux server url and other client options
  var configObject = {
    url: window.endpointURLObject.processPortalNotificationJS +"/cometd"
  };

  //workaround for defect 78650
  if (com.ibm.bpm.social.util.common.Util.isIPad) {
    configObject.maxNetworkDelay = 1000000;
  }
  
  dojox.cometd.configure(configObject);

  // disable the websocket transport
  dojox.cometd.websocketEnabled = false;

  if (typeof OnlineTrackingController === "function") {
    // Controller for online tracking
    // Trigger onConnectionBroken 1 minute after connection is lost.
    new OnlineTrackingController({
      cometd: dojox.cometd,
      userFunction: function(){
        var user = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (user != null) {
          return user["userName"];
        }
        else {
          return false;
        }
      },
      logFunction: function(message, cName, fName){
        com.ibm.bpm.social.util.common.Log.log(2, message, cName ? cName : "", fName ? fName : "");
      },
      timeout: 30000,
      update: true
    /* make sure new status is requested upon handshake not taking into account the timeout */});
  }

  // add a listener to the /meta/connect channel to inform us of connection events
  dojox.cometd.addListener("/meta/connect", handleOnMetaConnectMessage);
  dojox.cometd.addListener("/meta/disconnect", handleOnMetadisconnectMessage);

  // connect to bayuex server
  dojox.cometd.handshake();

  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function disconnectFromBayeuxServer() {
  logoutOngoing = true;
  if (typeof OnlineTrackingController === "function" && typeof OnlineTrackingController.instance === "object") {
    OnlineTrackingController.instance.logout();
  }
  dojox.cometd.disconnect();  
};

if (window.disconnectFromBayeuxServer == undefined) {
	window.disconnectFromBayeuxServer = disconnectFromBayeuxServer;
}

function handleOnMetadisconnectMessage(message){
  var FUNC_NAME = "handleOnMetadisconnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (logoutOngoing) {
    com.ibm.bpm.social.util.common.Log.log(2, "logout ongoing => no reconnection attempt", "notifications", FUNC_NAME);
  } else {
    // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
    // com.ibm.bpm.social.util.common.Log.log(2,"Meta Connect disconnect Message: "+ message, "notifications", FUNC_NAME);
    window["cometDForCollab"] = null;
    connectedToBayeuxServer = false;

    dojo.require("dojo.cookie");
    if (typeof (dojo.cookie("com_ibm_bpm_bspace_logout")) == "undefined" &&
      typeof (dojo.cookie("com_ibm_bpm_portal_unload")) == "undefined") {
      // if we're not logging out, reconnect to bayuex server
      dojox.cometd.handshake();
    }
  }   
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};
function handleOnMetaConnectMessage(message){
  var FUNC_NAME = "handleOnMetaConnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
  // com.ibm.bpm.social.util.common.Log.log(2, "Meta Connect Message: " + message, "notifications", FUNC_NAME);
  var wasPreviouslyConnected = connectedToBayeuxServer;
  connectedToBayeuxServer = (message.successful === true);
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage connectedToBayeuxServer:"+connectedToBayeuxServer, "notifications", "handleOnMetadisconnectMessage");
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage wasPreviouslyConnected:"+wasPreviouslyConnected, "notifications", "handleOnMetadisconnectMessage");
  if (!wasPreviouslyConnected && connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Success", "notifications", FUNC_NAME);

    handleConnectionSucceeded(message);
  }
  else if (wasPreviouslyConnected && !connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Broken", "notifications", FUNC_NAME);
    handleConnectionBroken(message);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleConnectionSucceeded(message){
  var FUNC_NAME = "handleConnectionSucceeded";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = true;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, setting windows variable cometd", "notifications",
    "handleConnectionSucceeded");
  window["cometDForCollab"] = dojox.cometd;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, calling subscribe to Channels", "notifications",
    "handleConnectionSucceeded");
  subscribeToChannels();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function getUserInfo(){
  var FUNC_NAME = "getUserInfo";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var userData = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return userData;
};

function handleConnectionBroken(message){
  var FUNC_NAME = "handleConnectionBroken";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = false;
  window["cometDForCollab"] = null;

  //Do a check here to see if the reason behind the connection break is a 401 Response.  If that is the case,
  //then redirect to the login page.
  var xhr = message.xhr;
  if (typeof xhr != "undefined") {
    var status = xhr.status;
    if (typeof status != "undefined" && status == 401) {
      //Perform REDIRECT to login page
      com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
    }
  }
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleFailedRestConnection(message){
  var FUNC_NAME = "handleFailedRestConnection";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToChannels(){
  var FUNC_NAME = "subscribeToChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //console.group("subscribeToChannels");
  if (!currentUser) {
    var userData = getUserInfo();
    currentUser = userData;
  }
  //com.ibm.bpm.social.util.common.Log.log(2, "User Info: " + userData, "notifications", "subscribeToChannels");
  //com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), "notifications", "subscribeToChannels");
  dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), handleNotificationMessage);
  var membershipsAll = userData.memberships;
  
  //remove duplicates
  var memberships = [];
  dojo.forEach(membershipsAll, function(item, i) {
    if(dojo.indexOf(memberships, item)  == -1) {
      memberships.push(item);
    }
  });
  
  if (memberships) {
    //batch the subscriptions so that we are not making multiple HTTP requests for each group subscription
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        // com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), "notifications", "subscribeToChannels");
        dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), handleNotificationMessage);
      }
    });
  }

  //
  // subscribe to the task list channels in order for automatic refresh of the task list
  subscribeToTaskListChannels(currentUser.userName, memberships);

  console.groupEnd();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToTaskListChannels(userName, memberships){
  var FUNC_NAME = "subscribeToTaskListChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // task list push events for user
  dojox.cometd.subscribe('/bpm/process_portal/task_list/user/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName), handleTaskNotificationMessage);

  // task list push events for groups
  if (memberships) {
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        dojox.cometd.subscribe('/bpm/process_portal/task_list/group/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(memberships[i]), handleTaskNotificationMessage);
      }
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    switch (data.kind) {
      case "TASK_RESOURCE_ASSIGNED" :
        handle_TASK_RESOURCE_ASSIGNED(data);
        break;
      case "TASK_COLLABORATION_INVITE" :
        handle_TASK_COLLABORATION_INVITE(data);
        break;
      case "PROCESS_COMMENT_TAGGED" :
        handle_PROCESS_COMMENT_TAGGED(data);
        break;
      default:
        break;
    }
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleTaskNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleTaskNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED, [incomingMessage]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_RESOURCE_ASSIGNED(data){
  var FUNC_NAME = "handle_TASK_RESOURCE_ASSIGNED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (!data.task.resource || !data.task.id) {
    return;
  }

  // check if task notification should be ignored, e.g. claim for inline completion
  if (!com.ibm.bpm.social.notifications._isIgnoreAssignedNotification(data.task.id)) {
    var shouldDisplayPopup = true;
    var notificationPopUpText = "";

    switch (data.task.resource.type) {
      case "ROLE" :
        //notificationMessage = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask,{'groupname': data.task.resource.displayName});
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var displayName =
          com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.resource.displayName);
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask, {
          'groupname': displayName,
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To My Group'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      case "USER" :
        //notificationMessage = bpm_social_messages.userReceivedNewTask;
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.userReceivedNewTask, {
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To Me'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      default:
        break;
    }

    if (shouldDisplayPopup) {
      //notificationPopUpText = "<div>${0} <a id='"+link_id+"' href='#' dojoattachevent='onclick: _handleActionClick'>${1}</a></div>";
      var aetherParams = {};
      aetherParams.message = notificationPopUpText;
      aetherParams.containerDivId = "messageNotificationDiv";
      aetherParams.position = "top-right";
      aetherParams.fade = true;
      aetherParams.parent = this;
      aetherParams.displayCloseIcon = true;
      aetherParams.eventParam = data;
      var message = new com.ibm.bpm.social.Aether(aetherParams);
      message.fireEvent = dojo.hitch(this, fireEvent);
    }
  }
  else {
    com.ibm.bpm.social.util.common.Log.log(2, "ignoring TASK_RESOURCE_ASSIGNED for " + data.task.id, "notifications",
      FUNC_NAME);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_COLLABORATION_INVITE(data){
  var FUNC_NAME = "handle_TASK_COLLABORATION_INVITE";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.resource.id);
  var task_link_id = getUniqueID("launch_task_action");
  var process_link_id = getUniqueID("launch_process_action");
  //apply BTD to the notifications message
  var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
  var actor = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.resource.displayName);
  var processName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
  var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.collaboration.message);

  var notificationPopUpText = dojo.string.substitute(bpm_social_messages.collaborationInviteMessage, {
    'taskName': taskName,
    'taskLinkID': task_link_id,
    'processLinkID': process_link_id,
    'actor': actor,
    'processName': processName,
    'messageBody': com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.task.collaboration.message)
  });
  var aetherParams = {};
  aetherParams.message = notificationPopUpText;
  aetherParams.containerDivId = "messageNotificationDiv";
  aetherParams.position = "top-right";
  aetherParams.fade = true;
  aetherParams.parent = this;
  aetherParams.displayCloseIcon = true;
  if (invitorInfo && invitorInfo.image) {
    aetherParams.img_src = invitorInfo.image;
  }
  aetherParams.eventParam = data;
  var message = new com.ibm.bpm.social.Aether(aetherParams);
  message.fireEvent = dojo.hitch(this, fireEvent);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_PROCESS_COMMENT_TAGGED(data){
  var FUNC_NAME = "handle_PROCESS_COMMENT_TAGGED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);
  dojo.publish("/com/ibm/bpm/social/updateStream/" + data.process.instance.id, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.process.instance.comment.author.id);
  com.ibm.bpm.social.util.common.Log.log(2, invitorInfo, "notifications", "handle_PROCESS_COMMENT_TAGGED");

  var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  if (usr == null || usr.userPreferences == null || usr.userPreferences['Portal Notification Post Mention'] != "false") {
    var link_id = getUniqueID("launch_process_action");
    var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
    var actor =
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.comment.author.name);
    var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.process.instance.comment.text));

    // Check if a user is mentioned in the text,
    // but make sure to not use the mentions text for other user then the logged in user.
    var owner= com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var messageText = bpm_social_messages.UserMentionedMessage;
    if (data.process.instance.comment.text && (data.process.instance.comment.text.indexOf("\"bpm-user-tag\"") < 0 ||
          data.process.instance.comment.text.indexOf("objId=\""+owner.userName+"\"") < 0)) {
      messageText = bpm_social_messages.UserCommentedMessage;
    }


    var notificationPopUpText = dojo.string.substitute(messageText, {
      'taskName': taskName,
      'linkID': link_id,
      'actor': actor,
      'messageBody': body
    });
    var aetherParams = {};
    aetherParams.message = notificationPopUpText;
    aetherParams.containerDivId = "messageNotificationDiv";
    aetherParams.position = "top-right";
    aetherParams.fade = true;
    aetherParams.parent = this;
    aetherParams.displayCloseIcon = true;
    if (invitorInfo && invitorInfo.image) {
      aetherParams.img_src = invitorInfo.image;
    }
    aetherParams.eventParam = data;
    var message = new com.ibm.bpm.social.Aether(aetherParams);
    message.fireEvent = dojo.hitch(this, fireEvent);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function fireEvent(link_id, param){//custom override
  var FUNC_NAME = "fireEvent";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var eventType = link_id.split("::")[0];

  switch (eventType) {
    case "launch_task_action" :
      checkCollaborationSessionAvailablity(param.task.id);
      break;
    case "launch_process_action" :
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        "instanceId": param.process.instance.id
      }]);
      break;
    default:
      break;
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

/* checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
function checkCollaborationSessionAvailablity(taskID){
  var FUNC_NAME = "checkCollaborationSessionAvailablity";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // Response will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  if (taskID) {
    /* /bpm/notification/roster/task/ */
    dojo.xhrGet({
      //url: this.getProxyUrl()+"/bpm/notification/roster/task/"+taskID,
      url: window.endpointURLObject.processPortalNotificationJS +"/roster/task/" + taskID,
      handleAs: "json",
      preventCache: true,
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: dojo.hitch(this, function(data, xhrArgs){
        checkNumberOfParticipants(data, xhrArgs, taskID);
      }),
      error: dojo.hitch(this, this.handleFailedRestConnection)
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


/* checkNumberOfParticipants: See if there aren't too many participants in this task already. */
function checkNumberOfParticipants(data, xhrArgs, taskID){
  var FUNC_NAME = "checkNumberOfParticipants";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // data will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  // or this:
  // {"message":"NO_SESSION_FOR_TASK"}
  if (!data) {
    return;
  }
  // If there's no collab session, then data won't contain an element called 'participants',
  // and in that case we go ahead and publish the show task topic.
  var doPublishShowTaskTopic = false;
  if (data.participants) {
    var participants = data.participants;
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var currentUser;
    if (userProfileObj) {
      currentUser = userProfileObj.userName;
    }
    if (participants && (participants.length < 2 || _currentUserIsAlreadyInSession(currentUser, participants))) {
      // publish the show task topic
      doPublishShowTaskTopic = true;
    }
    else {
      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(bpm_social_messages.collaborationSessionFull, function(){
        return;
      });
    }
  }
  else {
    doPublishShowTaskTopic = true;
  }
  if (doPublishShowTaskTopic) {

    //Defect 133537:  We should use the navigation API here instead of the show task completion topic, as this request
    //Might happen while on a dashboard page.  In which case, the proper pre requisite steps are not taken and the 
    //details page is not shown correctly.

    // publish the show task topic
    // dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
    //   "taskId": taskID
    // }]);

    dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION, 
      ['/dashboards/TWP/BPM_WORK?tw.local.view=taskcompletion&tw.local.taskid=' + taskID]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


function _currentUserIsAlreadyInSession(currentUser, participants){
  var FUNC_NAME = "_currentUserIsAlreadyInSession";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (currentUser && participants) {
    for ( var i in participants) {
      if (participants[i].userName == currentUser) {
        com.ibm.bpm.social.util.common.Log.logExit(1, "returning true", "notifications", FUNC_NAME);
        return true;
      }
    }

  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "returning false", "notifications", FUNC_NAME);
  return false;
};

function getUniqueID(prefix){
  var FUNC_NAME = "getUniqueID";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var currentTime = new Date();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return prefix + "::" + currentTime.getTime();
};


/** ********************************COLLABORATION UTILS END************************************************** */

com.ibm.bpm.social.notifications._ignoreIncomingAssigned = new Array();

/**
 * Prepare ignore of TASK_RESOURCE_ASSIGNED notification for a task id, e.g. for inline completion
 *
 * @param {String} taskID
 * @returns {void} nothing
 */
com.ibm.bpm.social.notifications.setIgnoreAssignedNotification = function(taskID){
  com.ibm.bpm.social.util.common.Log.log(2, taskID, "notifications", "setIgnoreAssignedNotification");
  com.ibm.bpm.social.notifications._ignoreIncomingAssigned.push(taskID);
};

/**
 * @param taskID
 * @returns {Boolean} true if TASK_RESOURCE_ASSIGNED notification for a task id should be ignored
 */
com.ibm.bpm.social.notifications._isIgnoreAssignedNotification = function(taskID){
  var ignore = false;
  var index = dojo.indexOf(com.ibm.bpm.social.notifications._ignoreIncomingAssigned, taskID);
  if (index >= 0) {
    com.ibm.bpm.social.notifications._ignoreIncomingAssigned.splice(index, 1);
    ignore = true;
  }
  com.ibm.bpm.social.util.common.Log.log(2, taskID + " - " + ignore, "notifications", "_isIgnoreAssignedNotification");
  return ignore;
};

com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX = "/bpm/ProcessPortal/notifications";

com.ibm.bpm.social.notifications.getUserNotificationChannelName = function(userName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName);
}

com.ibm.bpm.social.notifications.getGroupNotificationChannelName = function(groupName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(groupName);
}

/** ********************************************************************************** */

dojo.addOnLoad(connectToBayeuxServer);

var unload = function(event){
  if(window.exitCollaboration) {
    window.exitCollaboration(event);
  }

  if (!window.logoutInProgress) {
    dojox.cometd.disconnect();
  }
};
dojo.addOnUnload(unload);
