// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

/**
 * OnlineTracking {class implemented with plain JavaScript}
 * 
 * Constructor: //
 * 
 * @anker {object instaneof HTMLDivElement} anke that is used.
 * @userid {string} the userid of the user as need for the channel name ("/user/"+userid+(online|offline))
 * 
 */
function OnlineTracking(userid){
  this.id = this.counter.number++;
  this.logFunction = OnlineTrackingController.instance.logFunction;
  this.logFunction("new OnlineTracking[" + this.id + "]", "OnlineTracking", "<init>");
  this.userid = userid;
  // this.anker = anker;	
  this.connected = false;
  this.coachesNG = false;
  // Detecting the context
  var context = null;
  if (window != window.parent && window.parent["cometDForCollab"]) {
    this.coachesNG = true;
    this.context = window.parent;
    this.cometd = window.parent["cometDForCollab"];
  }
  else {
    this.coachesNG = false;
    this.context = window;
    this.cometd = window["cometDForCollab"];
  }
  this.isRegistered = false;
  /*
  if (context == null) {
  	// Environment not ready
  	// throw "NOT READY";
  } else {
  	// create registry if not yet available
  	if (!context.onlineTrackingRegistry) {
  		context.onlineTrackingRegistry = {};
  	} 
  	this.registry = context.onlineTrackingRegistry;		
  }
  */
  //
  // Calculate the channel names
  //
  this.onlineBroadcastChannel = "/user/" + this.userid + "/online";
  this.offlineBroadcastChannel = "/user/" + this.userid + "/offline";
  this.onlineBroadcastChannelHandle = null;
  this.offlineBroadcastChannelHandle = null;
}


// Do not pollute the global namespace therefore wrap value in an anonymous function
// to create the function on a higher scope level.
OnlineTracking.prototype =
  (function(){
    // Create a mutuable object for the counter
    var definition = {
      counter: {
        number: 1
      },
      serviceUserChannel: "/service/users",
      getUser: function(){
        return this.userid;
      },
      online: function(message){
        if (this.isRegistered) {
          // Do not call these method of instance that are
          // waiting to be removed from the registry
          this._onlineImpl(message);
          this.onOnline(message);
        }
      },
      onOnline: function(message){},
      _onlineImpl: function(message){},
      offline: function(message){
        if (this.isRegistered) {
          // Do not call these method of instance that are
          // waiting to be removed from the registry
          this._offlineImpl(message);
          this.onOffline(message);
        }
      },
      onOffline: function(message){},
      _offlineImpl: function(message){},
      connectionBroken: function(){
        this._connectionBrokenImpl();
        this.onConnectionBroken();
      },
      onConnectionBroken: function(){},
      _connectionBrokenImpl: function(){
        this.connected = false;
        this._offlineImpl();
      },
      connectionEstablished: function(){
        this._connectionEstablishedImpl();
        this.onConnectionEstablished();
      },
      onConnectionEstablished: function(){},
      _connectionEstablishedImpl: function(){
        this.connected = true;
      },
      isConnected: function(){
        return this.connected;
      },
      isRegistered: function(){
        return this.isRegistered;
      },
      unregister: function(){
        // remove instance from registry
        var controller = this.context.OnlineTrackingController.instance;
        controller.unregister(this.id, this.userid);
        this.isRegistered = false;
      }
    };
    definition.activate =
      function(initialize){
        if (typeof initialize === "undefined") {
          initialize = true;
        }
        this.logFunction("[" + this.id + "," + initialize + "])", "OnlineTracking", "activate");
        // Add to registry
        if (!this.isRegistered) {
          var controller = this.context.OnlineTrackingController.instance;
          var online = controller.register(this.id, this.userid, this);
          this.isRegistered = true;
          if (online) {
            this.online();
          }
        }
        try {
          // The following calls could throw an exception if the handshake has not been happened										
          this.onlineBroadcastChannelHandle = this.cometd.subscribe(this.onlineBroadcastChannel, this, function(message){
            this.online(message);
          });
          this
            .logFunction("[" + this.id + "] -> subscribes to " + this.onlineBroadcastChannel, "OnlineTracking", "activate");
          this.offlineBroadcastChannelHandle = this.cometd.subscribe(this.offlineBroadcastChannel, this, function(message){
            this.offline(message);
          });
          this.logFunction("[" + this.id + "] -> subscribes to " + this.offlineBroadcastChannel, "OnlineTracking",
            "activate");
          // 
          // In case a connect has not been established a exception would have been thrown
          // Initially the instance would still be disconnected (i.e. this.connectedEstablished not triggered)
          //									
          if (initialize) {
            var payload = [this.getUser()];
            // Request initial update of user state
            this.cometd.publish(this.serviceUserChannel, JSON.stringify(payload));
            this.logFunction(
              "registerOnlineTracking publishes on /service/users payload = " + JSON.stringify(payload) + ";",
              "OnlineTracking", "activate");
          }
          if (!this.connected) {
            this.connectionEstablished();
          }
        }
        catch (exception) {
          this.logFunction(" subscribe failed due missing connection", "activate", "OnlineTracking", "activate");
          // handshake has not yet occured, item is already in registry
          // and the subscriptions will be handled by the global registerOnlineTracking function
        }
      };
    definition.deactivate =
      function(){
        if (this.onlineBroadcastChannelHandle != null) {
          this.cometd.unsubscribe(this.onlineBroadcastChannelHandle);
          this.onlineBroadcastChannelHandle = null;
          this.logFunction("[" + this.id + "] -> unsubscribes to " + this.onlineBroadcastChannel, "OnlineTracking",
            "deactivate");
        }
        if (this.offlineBroadcastChannelHandle != null) {
          this.cometd.unsubscribe(this.offlineBroadcastChannelHandle);
          this.offlineBroadcastChannelHandle = null;
          this.logFunction("[" + this.id + "] -> unsubscribes to " + this.offlineBroadcastChannel, "OnlineTracking",
            "deactivate");
        }
      };
    return definition;
  }());


/**
 * OnlineTrackingController {Singleton} need to be instanciated where the dojox.cometd.handshake() or
 * dojox.cometd.init() is initiated.
 * 
 * Constructor:
 * 
 * @cometd {object instanceof dojox.cometd} the cometd handle to use.
 * @timeout {number) ms to wait until the onConnectionBroken is called on the OnlineTracking items
 * @userFunction {function) function to retrieve the user which is triggered when handshake occurs.
 * @userName {string} The user name
 * @update {boolean) if true, any handshake triggers a update for all OnlineTracking items otherwise only if
 *         necessary
 * 
 * 
 */
function OnlineTrackingController(props){
  this.connected = true;
  this.userName = null;
  this.userFunction = null;
  this.garabageCollectionTimeout = null;
  this.cometd = props.cometd;
  if (typeof props.update === "boolean") {
    this.update = props.update;
  }
  else {
    this.update = false;
  }
  if (typeof props.timeout === "number") {
    this.timeout = props.timeout;
  }
  else {
    // 2 minutes default
    this.timeout = 120000;
  }
  if (typeof props.userName === "string") {
    this.userName = props.userName;
  }
  if (typeof props.userFunction === "function") {
    this.userFunction = props.userFunction;
  }
  if (typeof props.logFunction === "function") {
    this.logFunction = props.logFunction;
  }
  // Singleton pattern enforcement
  var instance = this;
  OnlineTrackingController.instance = this;
  // Establish the listener for meta events
  this.init();
}

OnlineTrackingController.prototype =
  (function(){
    return {

      onlineChannelReg: /\/user\/(.+)\/online/,
      offlineChannelReg: /\/user\/(.+)\/offline/,

      logFunction: function(message, cname, fName){
        console.log(message);
      },

      init: function(){
        // Create the registry for OnlineTracking
        if (typeof onlineTrackingRegistry === "undefined") {
          window.onlineTrackingRegistry = {};
        }
        if (typeof onlineTrackingGarbage === "undefined") {
          window.onlineTrackingGarbage = {};
        }
        if (typeof onlineUserTracking === "undefined") {
          window.onlineUserTracking = {};
        }
        this.registry = window.onlineTrackingRegistry;
        this.garabageRegistry = window.onlineTrackingGarbage;
        this.userRegistry = window.onlineUserTracking;
        this.cometd.addListener("/meta/connect", this, this.metaConnectionCallback);
        this.logFunction("addListener to /meta/connect", "OnlineTrackingController", "init");
        this.cometd.addListener("/meta/handshake", this, this.metaHandshakeCallback);
        this.logFunction("addListener to /meta/handshake", "OnlineTrackingController", "init");
        this.cometd.addListener("/user/**", this, this.userEventCallback);
        this.logFunction("addListener to /user/**", "OnlineTrackingController", "init");
      },

      reset: function(){
        // Create the registry for OnlineTracking
        if (typeof onlineTrackingRegistry === "undefined") {
          window.onlineTrackingRegistry = {};
        }
        if (typeof onlineTrackingGarbage === "undefined") {
          window.onlineTrackingGarbage = {};
        }
        if (typeof onlineUserTracking === "undefined") {
          window.onlineUserTracking = {};
        }
        this.registry = window.onlineTrackingRegistry;
        this.garabageRegistry = window.onlineTrackingGarbage;
        this.userRegistry = window.onlineUserTracking;
      },

      register: function(id, userName, tracking){
        this.registry[id] = tracking;
        var userInfo = this.userRegistry[userName];
        var isOnline = false;
        if (typeof userInfo === "object") {
          userInfo.count++;
          isOnline = userInfo.isOnline;
        }
        else {
          this.userRegistry[userName] = {
            count: 1,
            isOnline: false
          };
        }
        // (userName === this.userName) => current user is always online, isn't he?
        return isOnline || (userName === this.userName);
      },

      unregister: function(id, userName){
        var trackingItem = this.registry[id];
        if (trackingItem != null) {
          this.garabageRegistry[id] = trackingItem;
          delete this.registry[id];
        }
        if (this.garabageCollectionTimeout == null) {
          var myself = this;
          this.garabageCollectionTimeout = window.setTimeout(function(){
            myself.garabageCollectionTimeout = null;
            myself.doGarabageCollect(true);
          }, 300000 /* 300 seconds */);
        }
      },

      doGarabageCollect: function(deactivate /* boolean */){
        if (this.garabageCollectionTimeout != null) {
          // In case this is called out of metaHandshakeCallback
          window.clearTimeout(this.garabageCollectionTimeout);
        }
        if (typeof deactivate !== "boolean") {
          deactivate = false;
        }
        var myself = this;
        if (deactivate) {
          this.cometd.batch(function(){
            myself.doGarabageCollectImpl(true);
          });
        }
        else {
          myself.doGarabageCollectImpl(false);
        }
      },

      doGarabageCollectImpl: function(deactivate){
        for (key in this.garabageRegistry) {
          var item = this.garabageRegistry[key];
          delete this.garabageRegistry[key];
          var userName = item.getUser();
          var userInfo = this.userRegistry[userName];
          if (typeof userInfo === "object") {
            userInfo.count--;
            if (userInfo.count === 0) {
              delete this.userRegistry[userName];
            }
          }
          // deactivate items
          if (deactivate) {
            try {
              item.deactivate();
            }
            catch (e) {
              deactivate = false;
            }
          }
        }
      },

      activate: function(update){
        if (typeof update === "undefined") {
          update = false;
        }
        this.logFunction("[" + update + "]", "OnlineTrackingController", "activate");
        // Anonymous function argument in batch requires locally scoped variables
        var cometd = this.cometd;
        var registry = this.registry; // local variable for closure
        var logFunction = this.logFunction; // local variable for closure
        this.cometd.batch(function(){
          var usersHash = {}; // use hash to ignore dublicate users
          for (key in registry) {
            var item = registry[key];
            if (item instanceof OnlineTracking) {
              // The dojox.cometd used by activate must be the same
              // and the call must not have an asynchronous parts.								
              var wasConnected = item.isConnected();
              item.activate(false /* batch service calls */);
              if (update || !wasConnected) {
                usersHash[item.getUser()] = true;
              }
            }
          }
          // Create an array instead
          var payload = [];
          for ( var userid in usersHash) {
            if (usersHash[userid] === true) {
              payload.push(userid);
            }
          }
          if (payload.length > 0) {
            // Publish a service request that triggers the broadcast events
            cometd.publish("/service/users", JSON.stringify(payload));
            logFunction("requests update using /service/users payload = " + JSON.stringify(payload) + ";",
              "OnlineTrackingController", "activate");
          }
        });
      },

      activateWithUpdate: function(){
        this.activateWithUpdate(true);
      },

      metaHandshakeCallback: function(message){
        this.activate(this.update);
        try {
          this.login();
        }
        catch (e) {
          console.error(e);
        }
        this.reset();
      },

      login: function(){
        if (this.userName == null) {
          if (this.userFunction != null) {
            this.userName = this.userFunction.call();
          }
        }
        if (this.userName != null) {
          // Somehow the publish does not properly work with an empty string as payload
          this.cometd.publish("/service/user/" + this.userName + "/online", this.userName /* payload? */);
          this.logFunction("publish on /service/user/" + this.userName + "/online", "OnlineTrackingController", "login");
        }
        else {
          // Could not retrieve the userid
          // TODO Check whether this works reliable or do we need to repeat this until it works??				
        }
      },

      logout: function(){
        if (this.garabageCollectionTimeout != null) {
          // No garbage collection is necessary any more, page is left immediately
          window.clearTimeout(this.garabageCollectionTimeout);
        }
        if (this.userName != null) {
          // Somehow the publish does not properly work with an empty string as payload
          this.cometd.publish("/service/user/" + this.userName + "/offline", this.userName /* payload? */);
          this.logFunction("publish on /service/user/" + this.userName + "/offline", "OnlineTrackingController", "logout");
        }
      },

      metaConnectionCallback: function(message){
        if (typeof message.successful === "boolean" && message.successful) {
          // cancel pending timeouts
          if (this.timeoutHandle != null) {
            window.clearTimeout(this.timeoutHandle);
            this.timeoutHandle = null;
          }
          if (!this.connected) {
            this.handleConnectedEvent();
          }
        }
        else if (typeof message.failure === "boolean" && message.failure && this.connected) {
          if (!this.timeoutHandle) {
            var myself = this;
            this.timeoutHandle = window.setTimeout(function(){
              myself.handleDisconnectedEvent.call(myself);
            }, this.timeout);
            this.logFunction("OnlineTrackingController => connectionBroken => Setting timeout to " + this.timeout,
              "OnlineTrackingController", "metaConnectionCallback");
          }
        }
      },

      handleConnectedEvent: function(){
        // TODO I think we need to call activate for OnlineTracking instances whose initial subscribe failed 
        // in case of network connection problem without removal of the server-side cometd session (i.e. without sweeping)
        this.connected = true;
        this.logFunction("", "OnlineTrackingController", "handleConnectedEvent");
        if (typeof onlineTrackingRegistry != "undefined") {
          var registry = onlineTrackingRegistry;
          var usersHash = {};
          for (key in registry) {
            var item = registry[key];
            if (item instanceof OnlineTracking) {
              // item.isConnected == true might mean that is just been activated using a handshake
              if (!item.isConnected()) {
                usersHash[item.getUser()] = true;
              }
              item.connectionEstablished();
            }
          }
          // Create an array instead
          var payload = [];
          for ( var userid in usersHash) {
            if (usersHash[userid] === true) {
              payload.push(userid);
            }
          }
          if (payload.length > 0) {
            // Publish a service request that triggers the broadcast events
            this.cometd.publish("/service/users", JSON.stringify(payload));
            this.logFunction("requests update using /service/users payload = " + JSON.stringify(payload) + ";",
              "OnlineTrackingController", "handleConnectedEvent");
          }
        }
        else {
          this.logFunction("[" + update + "]: onlineTrackingRegistry does not yet exist", "OnlineTrackingController",
            "handleConnectedEvent");
        }
      },

      handleDisconnectedEvent: function(){
        this.logFunction("", "OnlineTrackingController", "handleDisconnectedEvent");
        this.connected = false;
        this.timeoutHandle = null;
        if (typeof onlineUserTracking != "undefined") {
          var userRegistry = onlineUserTracking;
          for (key in userRegistry) {
            var item = userRegistry[key];
            if (typeof item.isOnline === "boolean") {
              item.isOnline = false;
            }
          }
        }
        if (typeof onlineTrackingRegistry != "undefined") {
          var registry = onlineTrackingRegistry;
          for (key in registry) {
            var item = registry[key];
            if (item instanceof OnlineTracking) {
              item.connectionBroken();
            }
          }
        }
      },

      userEventCallback: function(message){
        this.logFunction("", "OnlineTrackingController", "userEventCallback");
        if (typeof message.channel === "string") {
          var channel = message.channel;
          if (this.onlineChannelReg.test(channel)) {
            // online event
            var user = channel.replace(this.onlineChannelReg, "$1");
            this._userOnlineImpl(user);
            this.onUserOnline(user);
          }
          else if (this.offlineChannelReg.test(channel)) {
            // offline event
            var user = channel.replace(this.offlineChannelReg, "$1");
            this._userOfflineImpl(user);
            this.onUserOffline(user);
          }
        }
      },

      _userOnlineImpl: function(user){
        this.logFunction(user + " is online", "OnlineTrackingController", "_userOnlineImpl");
      },

      _userOfflineImpl: function(user){
        this.logFunction(user + " is offline", "OnlineTrackingController", "_userOfflineImpl");
      },

      onUserOnline: function(userName){
        var userInfo = this.userRegistry[userName];
        if (typeof userInfo === "object") {
          userInfo.isOnline = true;
        }
      },

      onUserOffline: function(userName){
        if (typeof userInfo === "object") {
          userInfo.isOnline = false;
        }
      }
    };
  }());