// DashboardController.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.portal.DashboardController");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.PageSettings");
dojo.require("dojo.cookie");
dojo.require("dijit.layout.ContentPane");


dojo.declare("com.ibm.bpm.social.portal.DashboardController", [dijit.layout.ContentPane], {

  dashboardURL: "", // the currently selected dashboard's URL

  dashboardTitle: "", // the title set for the dashboard's URL

  pageHandler: null,

  _currentPageSettings: null,

  _jspUrl: "/jsp/socialPortal/dashboard.jsp",

  _subscribedChannels: null,

  _lastDashboardParams: null,

  /**
   * postCreate
   * 
   * @type {void}
   */
  postCreate: function postCreate_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);


    this.loadingMessage = ""; // overwrite the message from ContentPane, since we show our own one

    this.pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();
    this._currentPageSettings = new com.ibm.bpm.social.util.common.PageSettings(); // just a default, will be populated later 
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

    // create the start page icon/button
    this.startPageButton = this.pageHandler.createStartPageIcon();
    dojo.addClass(this.startPageButton, "startPageBookmark-dashboards");
    this.connect(this.startPageButton, "onclick", this._onStartPageButtonClick);
    this.connect(this.startPageButton, "onkeypress", function(e){
      if (e.keyCode == dojo.keys.ENTER) {
        this._onStartPageButtonClick(e);
      }
    });

    this._subscribedChannels = [];
    this._lastDashboardParams = new Object();

    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, dojo.hitch(this,
      "_onDashboardSelected")));
    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.DASHBOARD_LOADED, dojo.hitch(this,
      "_dashboardLoaded")));
    this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD, dojo.hitch(this,
      "_forceNewDashboard")));

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Cleanup our resources
   * 
   * @type {void}
   */
  destroy: function destroy_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._lastDashboardParams = null;
    for ( var i = 0; i < this._subscribedChannels.length; i++) {
      if (this._subscribedChannels[i]) {
        dojo.unsubscribe(this._subscribedChannels[i]);
      }
    }

    this.inherited(arguments);

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

    /**
    * Event handler subscribed on topic com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD
    * 
    * @type {void}
    */
    _onDashboardSelected: function _onDashboardSelected_DashboardController(msg) {
    /**
    * The restartDashboardCookie keeps track of whether restartable dashboards are 
    * at the "restart" page.
    * 
    * The cookie is of the form:
    * {
    *   lastDashboardURL: string -> The last dashboard URL that was run (used in dashboard_restart.jsp)
    *   dashboards: array -> List of dashboards that have been accessed
    *       {
    *           dashboardURL: string -> The URL of this dashboard
    *           atRestartDashboard: boolean -> true if dashboard is at "restart" page
    *       }
    * }
    * Cookie expires in one day, and is deleted when the user logs out
    */
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (msg.runURL) {
            // remove the zForceNew param, since the URL is stored w/o that param
            var runURL = msg.runURL.replace("&zForceNew=true", "");
            var cookie = dojo.cookie("restartDashboardCookie");
            
            var url = runURL;
            if (url == 'RESTART_DASHBOARD') {
            	url = this.dashboardURL;
            	if (cookie) {
            		var cookieValues = dojo.fromJson(cookie);
            		url = cookieValues.lastDashboardURL;
            	}
            }
            var urlParamString = url.substring(url.indexOf('?')+1);
            var urlParamList = urlParamString.split('&');
            var urlParams = new Object();
            for (var p1=0; p1<urlParamList.length; p1++) {
            	var p2 = urlParamList[p1].indexOf('=');
            	if (p2 > -1) {
            		urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
            	}
            }
            if ((urlParams.processApp == undefined) && (runURL.indexOf('viewScoreboard.do') != -1)) {
            	// special case for scoreboard
            	urlParams.processApp = 'TWP';
            	urlParams.serviceName = 'Scoreboard';
            } 	
            
			var newHash = 'dashboards/'+urlParams.processApp+'/'+urlParams.serviceName;
			if (urlParams.snapshot) {
				newHash = newHash + '/'+urlParams.snapshot;
			}
			var newHashParams = "";
			for (var paramName in urlParams) {
				if ((paramName != 'processApp') && (paramName != 'serviceName') && (paramName != 'snapshot')) {
					if (newHashParams) {
						newHashParams = newHashParams + '&';
					}
					newHashParams = newHashParams + encodeURIComponent(paramName) + '=' + encodeURIComponent(urlParams[paramName]);
				}
			}
			var msgParamString = "";
			if (msg.params) {
				for (var msgParamName in msg.params) {
					if (msgParamString) {
						msgParamString = msgParamString + '&';
					}
					msgParamString = msgParamString + encodeURIComponent(msgParamName) + '=' + encodeURIComponent(msg.params[msgParamName]);
				}
			}
			if (newHashParams) {
				newHash = newHash + '?' + newHashParams;
				if (msgParamString) {
					newHash = newHash + '&' + msgParamString;
				}
			} else {
				if (msgParamString) {
					newHash = newHash + '?' + msgParamString;
				}
			}
			newHash = com.ibm.bpm.social.util.common.Util.removeParameter(newHash,'isWorkTab');
			newHash = com.ibm.bpm.social.util.common.Util.removeParameter(newHash,'zResumable');
			com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
			
			var tmpParamString = undefined;
			if (msgParamString) {
				tmpParamString =  com.ibm.bpm.social.util.common.Util.removeParameter('?'+msgParamString,'isWorkTab');
			}
			
			// if no snapshot specified, determine which snapshot to use
			if (!urlParams.snapshot) {
				urlParams.snapshot = "tip";
				var item = this.pageHandler._findDashboardFromURL(url);
				if (item) {
					var runParams = this.pageHandler._parseQueryString(item.params.runURL);
					if (runParams.snapshot) {
						urlParams.snapshot = runParams.snapshot;
					}
				}
			} 
			if (this._lastDashboardParams[urlParams.processApp] == undefined) {
				this._lastDashboardParams[urlParams.processApp] = new Object();
			}
			if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName] == undefined) {
				this._lastDashboardParams[urlParams.processApp][urlParams.serviceName] = new Object();
			}
			if (tmpParamString) {
				if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] != tmpParamString) {
					this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] = tmpParamString;
			        msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
				}
				msg.runURL = msg.runURL + '&' + msgParamString;
			} else {
				if (this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot]) {
					this._lastDashboardParams[urlParams.processApp][urlParams.serviceName][urlParams.snapshot] = '';
					msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
				}
			}
            
			if (dojo.indexOf(window.zResumable,urlParams.processApp) == -1) {
				msg.runURL = com.ibm.bpm.social.util.common.Util.removeParameter(msg.runURL,'zResumable');
			}

            if (runURL !== "RESTART_DASHBOARD") {
                var cookieValues;
                if (cookie) {
                    cookieValues = dojo.fromJson(cookie);
                    if(this._isUrlInCookie(cookieValues, runURL) === false) {
                        // Add the current dashboard to the cookie if it is not already included                        
                        cookieValues.dashboards.push({
                            atRestartDashboard: false,
                            dashboardURL: runURL
                        });
                    }
                } else {
                    // There is no cookie, so create one
                    var dashboardsArray = [{
                        atRestartDashboard: false,
                        dashboardURL: runURL
                    }];
                    cookieValues = { dashboards: dashboardsArray };
                }

                this.dashboardURL = msg.runURL;
                this.dashboardTitle = (msg.display ? msg.display : "dashboard");
                for (var i = 0; i < cookieValues.dashboards.length; i++) {
                    var cur = cookieValues.dashboards[i];
                    if (cur.dashboardURL === runURL && cur.atRestartDashboard === true) {
                        var newUrl = runURL + "&zForceNew=true";
                        cookieValues.dashboards[i].atRestartDashboard = false;
                        this.dashboardURL = newUrl;
                    }
                }

                // Lastly, update the lastDashboardURL to be the current URL
                cookieValues.lastDashboardURL = runURL;
                dojo.cookie("restartDashboardCookie", dojo.toJson(cookieValues), {expires: 1, path: '/'});
            } 
            else { // msg.runURL === "RESTART_DASHBOARD"
                if (cookie) {
                    var cookieValues = dojo.fromJson(cookie);
                    this.dashboardURL = cookieValues.lastDashboardURL + '&zForceNew=true';
                    for (var i = 0; i < cookieValues.dashboards.length; i++) {
                        var cur = cookieValues.dashboards[i];
                        if (cur.dashboardURL === cookieValues.lastDashboardURL) {
                            cur.atRestartDashboard = false;
                        }
                    }

                    dojo.cookie("restartDashboardCookie", dojo.toJson(cookieValues), {expires:1, path:'/'});
                } else {
                    this.dashboardURL += '&zForceNew=true';
                }
            }

            var content = ppConfig.portalContextRootJS + this._jspUrl;
            content += '?url=' + escape(this.dashboardURL) + '&title=' + this.dashboardTitle;

            // remove the startPageButton from the document, else IE will purge the inner span
            if (this.startPageButton.parentNode) {
                this.startPageButton.parentNode.removeChild(this.startPageButton);
            }

			// if this is the work tab, load it
			if (msg.params && msg.params.isWorkTab && msg.params.isWorkTab==true) {
				this.pageHandler.selectWorkTab();
			} else {
				// now load the dashboard by setting the ContentPane href attrib
				this.set("href", content);
				this.pageHandler.setSelectedTab('dashboardOrganizerTab');
			}
        }
        else {
            com.ibm.bpm.social.util.common.Log.logError(1, "missing parameter: runURL");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);

        return;
    },

	_forceNewDashboard: function _forceNewDashboard_DashboardController(processApp,serviceName,snapshot) {
		if (this._lastDashboardParams[processApp] == undefined) {
			this._lastDashboardParams[processApp] = new Object();
		}
		if (this._lastDashboardParams[processApp][serviceName] == undefined) {
			this._lastDashboardParams[processApp][serviceName] = new Object();
		}
		if (!snapshot) {
			snapshot = "tip";
			var item = this.pageHandler._findDashboardFromParams({
				processApp: processApp,
				serviceName: serviceName
			});
			if (item) {
				var runParams = this.pageHandler._parseQueryString(item.params.runURL);
				if (runParams.snapshot) {
					snapshot = runParams.snapshot;
				}
			}
		} 
		this._lastDashboardParams[processApp][serviceName][snapshot] = 'FORCE_NEW_DASHBOARD=true';
	},
	
    /**
    * Determine whether there is already an entry in the cookie for the 
    * current URL
    * 
    * Returns boolean
    */
    _isUrlInCookie: function _urlInCookie_DashboardController(cookieValues, compareURL) {
        var dashboardURLArray = cookieValues.dashboards;
        for (var i = 0; i < dashboardURLArray.length; i++) {
            if (dashboardURLArray[i].dashboardURL === compareURL) return true;
        }
        return false;
    },

  /**
   * 
   * @type {void}
   */
  getPageSettings: function getPageSettings_DashboardController(){
    com.ibm.bpm.social.util.common.Log.log(2);
    return this._currentPageSettings;
  },

  /**
   * Return page settings for the current dashboard
   * 
   * @type {void}
   */
  _createPageSettings: function _createPageSettings_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);


    // cut of the host:port part of teh dashboards URL  http(s)://host:port/portal/jsp/andSoOn
    var urlPart = this.dashboardURL.split('/'); // == ["http(s):", "", "host:port", "portal", "jsp", "andSoOn"]
    var path = this.dashboardURL;
    if (urlPart[0]) {
    	urlPart.splice(0, 3);
    	path = "/" + urlPart.join("/");
	}

    var pageSettings = new com.ibm.bpm.social.util.common.PageSettings({
      tab: this._currentPageSettings.tabPrefixes.DASHBOARDS,
      dashboardURL: path
    });

    this._currentPageSettings = pageSettings;

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return pageSettings;
  },

  /**
   * Overwrite of ContentPane. This is called when the jsp is loaded (however, the iframe may take longer)
   * 
   * @type {void}
   */
  onDownloadEnd: function onDownloadEnd_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    var startPageButtonDiv = dojo.byId("startPageButtonDashboards");
    startPageButtonDiv.innerHTML = this._nlsResources['task-list-loading'];
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * Update the start page icon after the content of the iframe is loaded
   * 
   * @type {void}
   */
  _dashboardLoaded: function _dashboardLoaded_DashboardController(dashboardInfo){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "url:" + dashboardInfo.url + ", title:" + dashboardInfo.title);

    var startPageButtonDiv = dojo.byId("startPageButtonDashboards");
    // this hides the loading message and shows the start page icon
    startPageButtonDiv.innerHTML = "";
    startPageButtonDiv.appendChild(this.startPageButton);
    this.pageHandler.checkAndUpdateStartPageIcon(this._createPageSettings(), this.startPageButton);
	// when a dashboard is loaded or reloaded, make sure navigation is enabled    
    dojo.publish(com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION,[]);
    this.startPageButton.focus();//a11y
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Internal event handler for the start page button
   * 
   * @type {void}
   */
  _onStartPageButtonClick: function _onStartPageButtonClick_DashboardController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var currentPageSettings = this._createPageSettings();
    this.pageHandler.setStartPage(currentPageSettings, this.startPageButton);

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  }

});