/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.DashboardOrganizer");
dojo.require("com.ibm.bpm.social.util.common.DashboardOrganizerSettings");
dojo.require("com.ibm.bpm.social.util.common.SystemRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");
dojo.require("com.ibm.bpm.social.widgets.ExposedItemsList");
dojo.require("com.ibm.bpm.social.widgets.SmartList");
dojo.require("dojo.dnd.Source");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

	// DashboardOrganizer is a dropdown list that contains the names of all dashboards, saved searches and scoreboards
	// that are exposed to the current user.  The user can drag and drop the dashbard names to reorder the way the dashboards will be displayed on the tab bar.
	// A separator visually distinguishes the displayed dashboards (above the separator) from the hidden dashboards (below).

	// The user can drag dashboard names both within a section and across sections (from hidden to visible and vice versa).  
	// The results are saved on a per-user basis as a user preference - the preference is saved when the dropdown is closed *if* changes were made.
	// When the dropdown is closed and there were changes made then the tabs on the Portal tab bar are reordered to reflect the order selected by the user.
	
dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
	
dojo.declare(
	"com.ibm.bpm.social.portal.DashboardOrganizer",
	[com.ibm.bpm.social.widgets.common._RestWidget, com.ibm.bpm.social.widgets.SmartList, dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "DashboardOrganizer",
	//MAX_DISPLAYED_DASHBOARDS: 3,
	LOG_LEVEL: 1,
	DASHB0ARD_SEPARATOR_ID: "dashboardSeparatorItem",
	dashboardOrganizerEventConnectors : [],
    widgetsToCleanUp : [],
	visibleTabsText: "",
	dragToReorderText: "",
	openOnCreate: false,
	dashboardOrganizerSnapshotIconDiv_id : "_id_dashboardOrganizerSnapshotIconDiv_",
	dashboardOrganizerSnapshotIcon_id : "_id_dashboardOrganizerSnapshotIcon_",
	dashboardOrganizerSnapshotIcon_avatar_id : "_id_dashboardOrganizerSnapshotIcon_avatar_",
	snapshotContainer_id : "_id_snapshotContainer_",
	snapshotElement_innerDiv_id : "_id_snapshotElement_innerDiv_",
	snapshotElementDate_id : "_id_snapshotElementDate_",
	snapshotElement_id : "_id_snapshotElement_",
	processIcon_id : "_id_processIcon_",
	innerLi_id : "_id_innerLi_",
	currentlySelectedTab : null,
	prefsDirty  : false,
	isProcessServer : false,
	keyboardMoveOperationActive: false,
	originalLocationOfItemBeingMoved: {},  // this variable should be used only if keyboardMoveOperationActive==true, otherwise its value is unpredictable.
											// also, when I use dojo.move() I intentionally don't use "first" and "last" but always use "before" and "after".
	
	showEmptyMessage: false,  // turn off display of message by parent class Smartlist
	
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.portal", "templates/DashboardOrganizer.html"),
	constructor: function constructor_DashboardOrganizer(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.visibleTabsText = this.messages["dashboardOrganizer-visibleTabs"];
		this.dragToReorderText = this.messages["dashboardOrganizer-dragToReorder"];
		this.saveDojoDndManager = dojo.dnd._manager;
		this.localDndManager = new com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndManager();
	},
	
    postCreate: function postCreate_DashboardOrganizer() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		// Read dashboard order prefs from user profile
		this._readFromUserProfile();

		this.systemRestHandler = com.ibm.bpm.social.util.common.SystemRestHandler.getInstance();
		// get exposed items
		this._getExposedItems();
		this.dashboardOrganizerSubscriptions=[];
		this.dashboardOrganizerSubscriptions.push(dojo.subscribe("dashboardOrganizer_hideSnapshotContainer", dojo.hitch(this, function(data) {
			this._closeSnapshotItems();
		})));
		if (!this.dojoConnections) {
			this.dojoConnections=[];
		}
		// make the top labels tabbable
		dojo.attr(this.topHeadersDiv, 'tabindex', 0);

		if (this.openOnCreate) {
			this.show();
		} else {
			this.hide(false);  // false=don't save dashboard order setting.
		}

		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_getSavedSearches: function _getSavedSearches_DashboardOrganizer(exposedItemsArray){
		var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var taskQueuesList = taskQueues.getCustomQueues();
		// create object in the same structure as exposed items from the exposed items REST sevice 
		for (var t=0; t<taskQueuesList.length; t++) {
			var tq = taskQueuesList[t];
			var item = new Object();
			item["type"]="queue";
			item["subtype"]="task";
			item["name"]=tq.name;
			item["itemID"]="TQ."+tq.name;
			item["ID"]=tq.savedSearchID;
			item["display"]=tq.displayName;
			item["snapshotID"]="";
			item["tip"]=false;  // important: must be false or Exposed Items List will mistakenly believe the env is Process Server
			exposedItemsArray.push(item);
		}	
        var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
        var processQueuesList = processQueues.getCustomQueues();
		// create object in the same structure as exposed items from the exposed items REST sevice 
		for (var p=0; p<processQueuesList.length; p++) {
			var pq = processQueuesList[p];
			var item = new Object();
			item["type"]="queue";
			item["subtype"]="process";
			item["name"]=pq.name;
			item["itemID"]="PQ."+pq.name;
			item["ID"]=pq.savedSearchID;
			item["display"]=pq.displayName;
			item["snapshotID"]="";
			item["tip"]=false;  // important: must be false or Exposed Items List will mistakenly believe the env is Process Server
			exposedItemsArray.push(item);
		}	
	},
	
	_getExposedItems: function _getExposedItems_DashboardOrganizer(){
		var FUNC_NAME = "_getExposedItems";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		// add saved search information to our list of exposed items
		this._getSavedSearches(exposedItemsArray);
		// this.orderedDashboardItems is filtered for the exposed dashboards and 
		//   the data has been manipulated (disambiguated names, snapshot grouping); it contains the dashboards in user preference order.  It is used
		//   to create the drag and drop node.
		if (!this.orderedDashboardItems) {
		    this.orderedDashboardItems = [];
	   		var itemListWidget = new com.ibm.bpm.social.widgets.ExposedItemsList({trackSnapshotsOnIPad:true});
			// itemListWidget.getData() takes all exposed items and returns only dashboard, scoreboard and 'queue' items, 
			//    with disambiguated names and snapshot groupings.
			// com.ibm.bpm.social.portal.dashboardItems is global - it is also used in PageHandler.
			com.ibm.bpm.social.portal.dashboardItems = itemListWidget.getData(exposedItemsArray,["dashboard","scoreboard","queue"]);
			
			// we'll need to know if this is Process Server
			this.isProcessServer = itemListWidget.isProcessServer();
			// sort the exposed items based on the user prefs
			var sortedDashboards=this._sortExposedItemsUsingUserPrefs(com.ibm.bpm.social.portal.dashboardItems, this._storeddashboardOrganizerSettings.dashboardOrderObj);
			// refresh  com.ibm.bpm.social.portal.dashboardItems
			com.ibm.bpm.social.portal.dashboardItems = sortedDashboards;
			
			//add new DOM elements using the given data
			dojo.forEach(sortedDashboards, dojo.hitch(this, function(item){
			  var newItem={};
			  if (item.baseID == "HIDDEN") {  // the HIDDEN separator will have been created in _sortExposedItemsUsingUserPrefs()
				// add menu separator
				newItem["name"]="";
				newItem["type"]=[];
				newItem["type"].push("separator");
				newItem["type"].push("uniqueSeparatorID");
				newItem["baseID"]="uniqueSeparatorID";
				newItem["params"]={"dndListItemType":"separator"};
			  } else {
				  newItem["name"]=item.title;  // this name has been disambiguated
				  newItem["originalTitle"]=item.params.originalTitle;  // this is the name of the dashboard directly from REST; used to compare with user prefs if necessary
				  newItem["type"]=[];
				  newItem["type"].push("dashboardItem");
				  newItem["baseID"]=item.params.baseID;
				  item.params["dndListItemType"]="dashboardItem";
				  newItem["params"]=item.params;
				  
				  // Identify what icon to show in the list.  These are the icon types:
				  // 1. Work - this is an out-of-the-box dashboard
				  // 2. Team Performance - this is an out-of-the-box dashboard
				  // 3. Process Performance - this is an out-of-the-box dashboard
				  // 4. Adhoc Reports- this is an out-of-the-box dashboard
				  // 5. Custom Dashboard (any dashboard other than the 3 above)
				  // 6. Scoreboards 
				  // 7. Saved Search (also known as 'queue')
				  
				  // First deal with the out-of-the-box dashboards; for now we can identify them only by name
				  if (newItem.params.preciseItemType && newItem.params.preciseItemType=="dashboard") {
					if (newItem["originalTitle"] == "Team Performance") {
						newItem.params.iconNamePrefix = "Header_team_perf";
					} else if (newItem["originalTitle"] == "Process Performance") {
						newItem.params.iconNamePrefix = "Header_process_perf";
					} else if (newItem["originalTitle"] == "BPM_WORK") {
						newItem.params.iconNamePrefix = "Header_work";
						newItem.params.isWorkTab=true;
						// the page handler will need to know the work tab title and other data.
						com.ibm.bpm.social.portal.PageHandler.getInstance().setWorkTabInfo(item);
					} else if (newItem["originalTitle"] == "Ad-Hoc Reports") {
						newItem.params.iconNamePrefix = "Header_adhoc_report";
					} else {
						newItem.params.iconNamePrefix = "Header_custom_dashboard";
					}
				  } else if (newItem.params.preciseItemType && newItem.params.preciseItemType=="scoreboard") {
						newItem.params.iconNamePrefix = "Header_scoreboard";					
				  } else if (newItem.params.preciseItemType && newItem.params.preciseItemType=="queue") {
						newItem.params.iconNamePrefix = "Header_saved_search";					
				  }
			  }
			  this.orderedDashboardItems.push(newItem);
			}));
			 
			// this.dashboardsList is the dnd object
			this.dashboardsList = this._makeDndNode(this.listElement, this.orderedDashboardItems, false);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_sortExposedItemsUsingUserPrefs: function _mergeExposedItemsAndUserPrefs_DashboardOrganizer(exposedDashboards, userPrefs){
		// Will return an array containing exposed items in the order they appear in user prefs;  a separator will separate the displayed and hidden dashboards.
		//    Find match for pref item within the exposed items:
		//         A. Compare userPrefs.baseIDSeg with the baseID in exposed dashboard or saved search item (on Process Center this is the only comparison needed).
		//             Note that if this is a saved search item then baseID is the item's savedSearchID.
		//         B. If there is no match for the baseIDSeg then we need to do another check if the env is Process Server:
		//            C. Compare the last segment of itemID in exposed dashboard item with userPrefs.itemIDSeg 
		//           	 If the itemIDs match, we accept the exposed dash as a match for the pref
		//    After loop, see if there are any more dashboards in exposed items that didn't get moved in first loop; move these into
		//    the return array at the top of the hidden group.
		//  
		
		retArray=[];
		hiddenSeparatorObject={"baseID":"HIDDEN", "title":"HIDDEN", "params":{"originalTitle":"HIDDEN"}};
		var hiddenIndex=-1;
		if (userPrefs && userPrefs.length>0 && exposedDashboards && exposedDashboards.length>0){
			dojo.forEach( userPrefs, dojo.hitch( this, function(prefItem){
				// try to find match in exposedDashboards
				var itemIndex=-1;
				for (var i=0; i<exposedDashboards.length; i++) {
					if (exposedDashboards[i].params && exposedDashboards[i].params.baseID) { 
						var baseID = exposedDashboards[i].params.baseID;
						if (prefItem.baseIDSeg == baseID) {
							itemIndex=i;
							break;
						} else {
							//   1. If a user installs a new snapshot of an app ON PROCESS SERVER then we won't find the *previous* newest snapshot.
							//   2. If a user sets a new snapshot to be the default version ON BOTH PC AND PS then we won't find the *previous* default version.
							//   So, we compare the pref item's itemID segment with the exposed dashboard's itemID segment,
							//      and if they match then we use it.
							//   *This check is never done for saved searches*
							if (exposedDashboards[i].params.itemID && exposedDashboards[i].params.itemID.length>0) {
								var itemIDArray = exposedDashboards[i].params.itemID.split('-');
								if (prefItem.itemIDSeg == itemIDArray[itemIDArray.length-1]) {
									itemIndex=i;
									break;
								}										
							}
						}
					}
				}
				// if match, move exposed item into retArray; 
				if (itemIndex>=0) {
					retArray.push(exposedDashboards[itemIndex]);
					exposedDashboards.splice(itemIndex,1);

				} else if (prefItem.baseIDSeg && prefItem.baseIDSeg == "HIDDEN") {
					retArray.push(hiddenSeparatorObject);	
					hiddenIndex=retArray.length-1;
				}
			}));
		}
		if (exposedDashboards && exposedDashboards.length>0) {
			// there are exposedDashboards that need to be moved to retArray
			// if there's no separator, just add dashboards to top of retArray.  Separator will be added below.
			for (var i=exposedDashboards.length-1; i>=0; i--){
				retArray.splice(hiddenIndex+1,0,exposedDashboards[i]);
			}
		}
		// if there's no separator add it at top
		if (hiddenIndex==-1) {
			retArray.splice(0,0,hiddenSeparatorObject);
			hiddenIndex=0;
		}
		// if there were no user prefs then put the 'default' dashboards above the separator.  These
		//   are Work, Team Performance and Process Performance
		
		if (!userPrefs || (userPrefs && userPrefs.length==0)) {
			var tArray = [];
			for (var i=0; i<retArray.length; i++){
				var tItem = retArray[i];

				if (tItem.params.originalTitle == "BPM_WORK") {
					tArray[0]=retArray.splice(i,1)[0];
					i--;
				}
				if (tItem.params.originalTitle == "Team Performance") {
						tArray[1]=retArray.splice(i,1)[0];
					i--;
				}
				if (tItem.params.originalTitle == "Process Performance") {
						tArray[2]=retArray.splice(i,1)[0];
					i--;
				}
			}
			
			for (var i=tArray.length-1; i>=0; i--) {
				if (tArray[i] && tArray[i].params.originalTitle) {
					retArray.splice(0,0,tArray[i]);
				}
			}
		}
		return retArray;
	},
	
	// creates a dojo/dnd/Source
	_makeDndNode: function _makeDndNode_DashboardOrganizer(node, data, selfAccept){
		// create the Source
		var dndObj = com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndSource(node, {	// CustomDndSource code is at the bottom of this file
			accept: [ "dashboardItem"],
			selfAccept: selfAccept === undefined ? true : selfAccept,
			copyOnly: false,
			creator: dojo.hitch(this, "_makeListItem")
		});

		// insert new nodes to the Source; the first argument indicates that they will not be highlighted (selected)
		// when inserted
		dndObj.insertNodes(false, data);

		return dndObj;
	},

	// this is called by the dnd framework  - it creates an item for the source it is attached to (see _makeDndNode)
	_makeListItem: function _makeListItem_DashboardOrganizer(item, hint){
		var type="";
		var node=null;
		if (hint != "avatar") {
			// addItem is in the parent class SmartList
			var params={};
			params['title']=item.name;
			params['iconClass'] = item.params.iconNamePrefix;
			
			var index = this.addItem(item.params, params);
			var li = dojo.query('li[smartListIndex="'+index+'"]', this.listElement);
			if(typeof(li)=='undefined' || li==null || typeof(li.length)=='undefined' || li.length==null || li.length<=0) {
				return null;
			}
			node = li[0];
		} else {
			// create avatar body
			// See if this item is the 'currently selected' item in the list; if it is then the text will need
			//   to be in bold
			var currentSelItem=false;
			if (item.params && item.params.baseID && this.currentlySelectedTab && (this.currentlySelectedTab == item.params.baseID)) {
				currentSelItem=true;
			}
			node=this._createAvatar(item.name, item.params, currentSelItem);
		}
		// set the returned type of the item
		type = item.type;
		return { node: node, data: item, type: type };
	},
	
	_createAvatar: function _createAvatar_DashboardOrganizer(itemName, params, currentSelItem) {
		var div = dojo.create("div", { 
				"class": "dojoDndItemOver ",
				"id": "dashboardOrganizerAvatar"
			});
		var selClas=" ";
		if (currentSelItem) {
			selClas = " listItemSelected ";
		}
        var innerDiv = dojo.create("div", {
          'class': "bpm-social-dashboardOrganizer-avatar "+selClas
        }, div);
		if (document.body.dir == 'rtl') {
		  this._createAvatarTextDiv(innerDiv, itemName, params, currentSelItem);
		  this._createAvatarIconDiv(innerDiv, itemName, params, currentSelItem);
		}
		else {				
		  this._createAvatarIconDiv(innerDiv, itemName, params, currentSelItem);
		  this._createAvatarTextDiv(innerDiv, itemName, params, currentSelItem);
		}

		return div;
	},

	_createAvatarIconDiv: function _createAvatarIconDiv_DashboardOrganizer(innerDiv, itemName, params, currentSelItem) {
        var icon = dojo.create("div", {
          'class': "bpm-social-dashboardOrganizer-avatar-process-icon " + params['iconNamePrefix'],
		  'id': this.processIcon_id+params.baseID,
          //'tabIndex': "0",  we don't want to tab here because it gets too busy
          //'aria-label': createParams['title'],  taken care of in text div
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, innerDiv);
	},
	_createAvatarTextDiv: function _createAvatarTextDiv_DashboardOrganizer(innerDiv, itemName, params, currentSelItem) {
		var textClas="bpm-social-dashboardOrganizer-avatar-text";
		if (currentSelItem) {
			textClas="bpm-social-dashboardOrganizer-avatar-text-selected";
		}
        var text = dojo.create("a", {
          'class': textClas,
          'aria-label': itemName
        }, innerDiv);
        var text2 = dojo.create("div", {
        }, text);
        text2.appendChild(document.createTextNode(itemName));	
        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
	},

      /* _createItem: Handles creating each item's HTML elements. params is provided when addItem is called in _makeListItem(). 
	      This function is called from addItem() in the parent class SmartList */
      _createItem: function _createItem_DashboardOrganizer(index, li, params, createParams){
		if (params && params.dndListItemType) {
			if (params.dndListItemType=="dashboardItem") {
				var innerLi = dojo.create("li", {
				  'class': "organizerInnerLi",
				  'id': this.innerLi_id+params.baseID
				}, li);
				if (document.body.dir == 'rtl') {
				  this._createItemNameElement(index, li, innerLi, params, createParams);
				  this._createItemIconElement(index, li, innerLi, params, createParams);
				}
				else {				
				  this._createItemIconElement(index, li, innerLi, params, createParams);
				  this._createItemNameElement(index, li, innerLi, params, createParams);
				}
			} else if (params.dndListItemType=="separator") {
				this._createSeparatorElement(index, li, params, createParams);
			}
		}
      },

      _createItemNameElement: function _createItemNameElement_DashboardOrganizer(index, li, innerLi, params, createParams){
        var title = "<i>Unknown</i>";
        if (createParams != null && typeof (createParams) != 'undefined') {
          title = createParams['title'];
        }
		if (dojo.isIE) {
			dojo.addClass(li, 'organizerOuterLi');
		}

        var text = dojo.create("a", {
          'class': "dashboardItem-text",
          //'aria-label': title //is reading text in attached div
        }, innerLi);
		text.tabIndex = 0;
        dijit.setWaiRole(text, "link");
        var text2 = dojo.create("div", {
          'class': "dashboardItem-title-div",
        }, text);
        //text2.tabIndex = 0;		
        text2.appendChild(document.createTextNode(title));			
        

        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
		
        // If there are snapshots for this item, add sections for them.
        if (params.snapshotItems && params.snapshotItems.length > 0) {
			  var menuIconDiv = dojo.create("div", {
				'class': "snapshotsContainerIconDiv",
				'border': "none",
				'id': this.dashboardOrganizerSnapshotIconDiv_id + params.baseID
			  }, innerLi);//attach to parent of text for proper aria role hierarchy
			  var menuIcon = dojo.create("img", {
				'class': "bpm-social-blue-twistie-down",
				'border': "none",
				'tabIndex': "0",
				'alt': "",
				'aria-label': this.messages.twistieJawsExpandText,
				'id': this.dashboardOrganizerSnapshotIcon_id + params.baseID
			  }, menuIconDiv);
			  var menuIconAltText = dojo.create("span", {
				'class': "hi-contrast-icon dropdown-text",
				'innerHTML': "v",
				'id': "_id_alttext_" + params.baseID

			  }, menuIconDiv);
			  menuIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
			  dijit.setWaiRole(menuIcon, "button");
			  
			  var snapshotsContainer = dojo.create("div", {
				'class': "snapshotsContainerHide",
				'id': this.snapshotContainer_id + params.baseID
			  }, li);
			  
			  var connectFunc = dojo.connect(menuIcon, 'onclick', this, function(evt) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);
			  
			  connectFunc = dojo.connect(menuIconDiv, 'onclick', this, function(evt) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);
			  
			  connectFunc = dojo.connect(menuIcon, 'keypress', this, function(evt){
				if (evt.keyCode == dojo.keys.ENTER || evt.keyCode==dojo.keys.RIGHT_ARROW || evt.keyCode==dojo.keys.LEFT_ARROW || evt.charCode==dojo.keys.SPACE ) {
				  if (snapshotsContainer) {
					this._hideShowSnapshotsContainer(evt, menuIcon, snapshotsContainer);
				  }
				}
			  });
			  this.dashboardOrganizerEventConnectors.push(connectFunc);

			  for ( var i = 0; i < params.snapshotItems.length; i++) {
				this._addSnapshotElement(snapshotsContainer, params.snapshotItems[i].params["snapshotName"], params.snapshotItems[i], (i==0)/*first item*/);
			  }
        }
        this.dashboardOrganizerEventConnectors.push(dojo.connect(text, 'keypress', this, "_handleTextKeyPressEvent"));
        this.dashboardOrganizerEventConnectors.push(dojo.connect(text, 'onclick', this, "_handleTextClickEvent"));
      },
	  
      _createItemIconElement: function _createItemIconElement_DashboardOrganizer(index, li, innerLi, params, createParams){
        var icon = dojo.create("div", {
          'class': "process-icon " + createParams['iconClass'],
		  'id': this.processIcon_id+params.baseID,
          //'tabIndex': "0",  we don't want to tab here because it gets too busy
          //'aria-label': createParams['title'],  taken care of in text div
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, innerLi);
        //dijit.setWaiRole(icon, "button");
        this.dashboardOrganizerEventConnectors.push(dojo.connect(icon, 'keypress', this, "_handleTextKeyPressEvent"));
        this.dashboardOrganizerEventConnectors.push(dojo.connect(icon, 'onclick', this, "_handleTextClickEvent"));
      },
		
	 _hideShowSnapshotsContainer: function _hideShowSnapshotsContainer_DashboardOrganizer(evt, menuIcon, container, onlyHide) {
		// This function toggles the state of snapshot containers, hiding if open or opening if hidden.
		// param 'onlyHide' means hide open snapshot containers but don't open hidden ones.
		if (evt) {
			dojo.stopEvent(evt);
		}
		if (menuIcon && container) {
			if (dojo.hasClass(container, "snapshotsContainerHide") && !onlyHide) {
				dojo.removeClass(container, "snapshotsContainerHide");
				dojo.addClass(container, "snapshotsContainerShow");
				dojo.addClass(menuIcon, "twistie-selected");
				dojo.attr(menuIcon, 'aria-label', this.messages.twistieJawsCollapseText);
                if (container.firstChild) container.firstChild.focus();
			} else if (dojo.hasClass(container, "snapshotsContainerShow")) {
				dojo.removeClass(container, "snapshotsContainerShow");
				dojo.addClass(container, "snapshotsContainerHide");
				dojo.attr(menuIcon, 'aria-label', this.messages.twistieJawsExpandText);
				if (dojo.hasClass(menuIcon, "twistie-selected")) {
					dojo.removeClass(menuIcon, "twistie-selected");
				}
			}
		}
	 },
      _createSeparatorElement: function _createSeparatorElement_DashboardOrganizer(index, li, params, createParams){
		dojo.attr(li, "id", this.DASHB0ARD_SEPARATOR_ID);
		dojo.addClass(li, "indicatorClassForSeparator");
        var textDiv = dojo.create("div", {'class':"hiddenTabsHeader"}, li);      
        textDiv.appendChild(document.createTextNode(this.messages["dashboardOrganizer-hiddenTabs"]));			
        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(textDiv);
        var sepLine = dojo.create("div", {'class': "dashboardSeparator"}, li);      
		dojo.attr(textDiv, "tabindex", 0);
		dojo.attr(textDiv, "aria-label", this.messages["dashboardOrganizer-hiddenTabs"]);
		
	},

      _addSnapshotElement: function _addSnapshotElement_DashboardOrganizer(snapshotsContainer, display, params, firstItem){
        // params is the snapshot information
        if (display && params) {
			//var btd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(display);
			var innerDiv=dojo.create("div", {
				'id':this.snapshotElement_innerDiv_id+params.params.baseID,
				'tabindex':0,
                'role': "link"                
			}, snapshotsContainer);
			if (firstItem) {
				dojo.addClass(innerDiv, "firstSnapshotItem");
			}
			// keep an array of snapshot information for easy lookup
			// when a menu item is selected
			if (!this.snapshotElements) {
				this.snapshotElements = {};
			}			
			this.snapshotElements[this.snapshotElement_innerDiv_id+params.params.baseID] = {
				"display": display,
				"params": params
			};
			var connectFunc = dojo.connect(innerDiv, 'onclick', this, function(evt) {
				if (evt) {
					this._dashboardSnapshotItemClick(evt);
				}
			});
			this.dashboardOrganizerEventConnectors.push(connectFunc);
			connectFunc = dojo.connect(innerDiv, 'keypress', this, function(evt) {
				if (evt) {
					this._dashboardSnapshotKeypress(evt);
				}
			});
			this.dashboardOrganizerEventConnectors.push(connectFunc);

			var innerDiv2=dojo.create("div", {
				'class':"dashboardItem-snapshot-inner-div"
			}, innerDiv);
			var text = dojo.create("a", {
			  'id':this.snapshotElement_id+params.params.baseID,
			  'aria-label': display,
			  'class': "dashboardItem-snapshot-main-text"
			}, innerDiv2);
	        var text3 = dojo.create("div", {
        	  'class': "dashboardItem-snapshot-title-div",
        	}, text);
        	text3.appendChild(document.createTextNode(display));			
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text3);
			dijit.setWaiRole(text, "link");
			var innerDiv3=dojo.create("div", {
				'class':"dashboardItem-snapshot-inner-div-date"
			}, innerDiv);
			
			str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(params.params.snapshotCreateDate);
	        date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
			var snapDate = com.ibm.bpm.social.util.common.Util.formatDateObject(date, {formatLength: "short", selector: "date"});		
			
			var text2 = dojo.create("a", {
			  'innerHTML': this.messages["snapshotLastUpdated"]+snapDate,
			  'id':this.snapshotElementDate_id+params.params.baseID,
			  //'aria-label': display,
			  'class': "dashboardItem-snapshot-date-text"
			}, innerDiv3);
			dijit.setWaiRole(text2, "link");
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);
        }
      },


      /**
       * Read the user profile setting "Portal Dashboard Display Order" and parse it into the members
       * 
       * @returns {com.ibm.bpm.social.util.common.DashboardOrganizerSettings}
       */
      _readFromUserProfile: function _readFromUserProfile_DashboardOrganizer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        var dashboardOrganizerSettings = new com.ibm.bpm.social.util.common.DashboardOrganizerSettings();
        var userProfileString;
        if (userProfile && userProfile.userPreferences &&
						userProfile.userPreferences[com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME]) {
		  
          userProfileString = userProfile.userPreferences[com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME];
		  if (!userProfileString) {
			userProfileString="";
		  }
          com.ibm.bpm.social.util.common.Log.log(2, "dashboard order preference string is: [" + userProfileString+"]");

          var newdashboardOrganizerSettings = dashboardOrganizerSettings.parseFromString(userProfileString, true);
          if (!newdashboardOrganizerSettings.isValid) {
            // String is invalid
            // the original dashboardOrganizerSettings object is unmodified and has default values
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is invalid, using default string: " +
              dashboardOrganizerSettings.getCurrentProfileString());
          }
          else {
            // the string is valid and the values are mixed into the original DashboardOrganizerSettings object
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is valid, using now string: " +
              dashboardOrganizerSettings.getCurrentProfileString());
          }
        }
        else {
          // No setting
          com.ibm.bpm.social.util.common.Log.log(2, "user attribute not set, using default string: " +
            dashboardOrganizerSettings.getCurrentProfileString());
        }


        if (com.ibm.bpm.social.util.common.Log.logLevel > 1) {
          com.ibm.bpm.social.util.common.Log.logObject(2, {
            DashboardOrderObj: dashboardOrganizerSettings.dashboardOrderObj,
			_currentProfileString: dashboardOrganizerSettings._currentProfileString
          }, "Got following settings from user profile: ");
        }

        // update the _storeddashboardOrganizerSettings to the newest settings, but use a separate copy
        this._storeddashboardOrganizerSettings = new com.ibm.bpm.social.util.common.DashboardOrganizerSettings(dashboardOrganizerSettings);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return dashboardOrganizerSettings;
      },


      /**
       * write the default start page settings into the user profile
       * 
       * @param {com.ibm.bpm.social.util.common.DashboardOrganizerSettings} DashboardOrganizerSettings if given, the settings to be
       *        written to the profile
       * 
       * @type {void}
       */
     _writeDashboardOrderToUserProfile: function _writeDashboardOrderToUserProfile_DashboardOrganizer(dashboardOrganizerSettings){
		com.ibm.bpm.social.util.common.Log.logEntry(2, "saving settings!");
        if (dashboardOrganizerSettings) {
          com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME,
            dashboardOrganizerSettings.getCurrentProfileString());
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
		this.prefsDirty = false;
        return;
      },
	  
	  
	  
     _dashboardSnapshotItemClick: function _dashboardSnapshotItemClick_DashboardOrganizer(evt){
		var itemData = this.snapshotElements[evt.currentTarget.id];
		this._onDashboardItemSelected(itemData.params.params);
     },

     _dashboardSnapshotKeypress: function _dashboardSnapshotKeypress_DashboardOrganizer(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
			var itemData = this.snapshotElements[evt.currentTarget.id];
			this._onDashboardItemSelected(itemData.params.params);
		}
     },

      /* _onDashboardTextClicked: Called when the text for an item is clicked. 
      	When clicked, display the dashboard. Same functionality 
      	as _onDashboardiconClicked. */
     _onDashboardTextClicked: function _onDashboardTextClicked_DashboardOrganizer(index, li, params){
		this._onDashboardItemSelected(params);
     },
	 
      /* _onDashboardTextSpaceBar: Called when the text for an item is spacebar'd.
      	This indicates that the item is being selected for dragging, or that the move operation should be completed.
		*/
     _onDashboardTextSpaceBar: function _onDashboardTextSpaceBar_DashboardOrganizer(index, li, params){
		var allItems=this._getOrderedItems(this.dashboardsList);
		// do nothing if this is the only item in the list
		this.originalLocationOfItemBeingMoved={};
		if (allItems && allItems.length>1) {
			if (this.keyboardMoveOperationActive==true) {
				// complete the operation
				this.keyboardMoveOperationActive=false;
				// close snapshot containers and untwist twistie if needed
				this._closeSnapshotItems();

				if (dojo.hasClass(li, "keyboardSelected")) {
					dojo.removeClass(li, "keyboardSelected");
				}
			} else {
				// We need to make it appear with the draggable style. 
				this.keyboardMoveOperationActive=true;
				this._closeSnapshotItems();
				if (!dojo.hasClass(li, "keyboardSelected")) {
					dojo.addClass(li, "keyboardSelected");
				}
				// Save the location of this item, so it can be replaced if move operation is cancelled
				var allItems=this._getOrderedItems(this.dashboardsList);
				// find where this item is in the list
				if (allItems && allItems.length>1) {
					for (var i=0; i<allItems.length; i++) {
						if (li.id && allItems[i].dojoUniqueId && (li.id == allItems[i].dojoUniqueId)) {
							var beforeOrAfter="";
							var toItem=null;
							if (i==allItems.length-1) {  // if it is the last item in the list, 
								beforeOrAfter="after";
								toItem=allItems[i-1].dojoUniqueId;
							} else {
								beforeOrAfter="before";
								toItem=allItems[i+1].dojoUniqueId;
							}
							this.originalLocationOfItemBeingMoved={"beforeOrAfter":beforeOrAfter, "toItem":toItem} ;
							break;
						}
					}
				}
			}
		}
     },
	 
      /* _onDashboardArrowMove: Called when an item is to be moved
      	*/
     _onDashboardArrowMove: function _onDashboardArrowMove_DashboardOrganizer(index, li, params, keyCode){
		if (this.keyboardMoveOperationActive==true) {
				var allItems=this._getOrderedItems(this.dashboardsList);
				// find where this item is in the list
				if (allItems && allItems.length>1) {
					var thisItemIndex=-1;
					for (var i=0; i<allItems.length; i++) {
						if (li.id && allItems[i].dojoUniqueId && (li.id == allItems[i].dojoUniqueId)) {
							thisItemIndex=i;
							break;
						}
					}
					if (thisItemIndex>=0) {
						var moveAccomplished=false;
						// which direction?
						if (keyCode==dojo.keys.UP_ARROW) {
							if (thisItemIndex>0) {
								var toItem=dojo.byId(allItems[thisItemIndex-1].dojoUniqueId);
								dojo.place(li, toItem, "before");
								moveAccomplished=true;
							}
						} else if (keyCode==dojo.keys.DOWN_ARROW) {
							if (thisItemIndex<allItems.length) {
								var toItem=dojo.byId(allItems[thisItemIndex+1].dojoUniqueId);
								dojo.place(li, toItem, "after");
								moveAccomplished=true;
							}
						}
						if (moveAccomplished) {
							this._focusOnTextElement(li);
						}
					}
				}
				/* var list = dojo.byId("list"),
					three = dojo.byId("three");
			 
				dojo.place(three, list, "first"); */
			
		}
     },
	 
      /* _onDashboardEscapeKey: 
	     If a move operation is not active, close the dropdown and save the dashboard order;
		 If a move operation is active, cancel the operation.
      	*/
     _onDashboardEscapeKey: function _onDashboardEscapeKey_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			// cancel the operation
			if (dojo.hasClass(li, "keyboardSelected")) {
				dojo.removeClass(li, "keyboardSelected");
			}
			this._resetDashboardOrder(index, li, params);
			this.keyboardMoveOperationActive=false;
			this.originalLocationOfItemBeingMoved= {};
		} else {
			this.hide(true);
		}
     },
	 
      /* _resetDashboardOrder: 
		 If a move operation is active, reset the order of dashboards to where it was before the move operation started.
      	*/
     _resetDashboardOrder: function _resetDashboardOrder_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			var allItems=this._getOrderedItems(this.dashboardsList);
			// find where this item is in the list
			if (allItems && allItems.length>1) { // this is just a sanity check
				var target=dojo.byId(this.originalLocationOfItemBeingMoved.toItem);
				if (target) {
					dojo.place(li, target, this.originalLocationOfItemBeingMoved.beforeOrAfter);
					this._focusOnTextElement(li);
				}
			}
		}
	},

	_focusOnTextElement: function _focusOnTextElement_DashboardOrganizer(li) {
		var a = dojo.query(".dashboardItem-text",li);
		var node=null;
		var doNotClose = this.doNotClose;
		if (a.length > 0) {
			this.doNotClose = true;
			node = a[0];
		} else {
			a = dojo.query(".dashboardItem-text-selected",li);
			if (a.length > 0) {
				this.doNotClose = true;
				node = a[0];
			}
		} 
		if (node!=null) {
			node.focus();
			this.doNotClose = doNotClose;
		}
	},
	
      /* _onDashboardTabKey: If a move operation is in progress, this
	     cancels it;
      	*/
     _onDashboardTabKey: function _onDashboardTabKey_DashboardOrganizer(index, li, params){
		if (this.keyboardMoveOperationActive==true) {
			this._onDashboardEscapeKey(index, li, params);
		}
     },

      _handleTextKeyPressEvent: function _handleTextKeyPressEvent_DashboardOrganizer(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user pressed on img, don't handle that here; it's handled in _hideShowSnapshotsContainer()
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
            if (info['index'] != null && info['li'] != null) {
			  if (evt.keyCode == dojo.keys.ENTER) {
				  // the item has been selected to be shown on the tab bar
				  this._onDashboardTextClicked(info['index'], info['li'], info['params']);
			  } else if (evt.charCode == dojo.keys.SPACE) {
				   // the item has been chosen to be moved to another location; if pressed while move
				   // operation is in progress then space completes the operation
				  this._onDashboardTextSpaceBar(info['index'], info['li'], info['params']);
			  } else if (evt.keyCode == dojo.keys.UP_ARROW || evt.keyCode == dojo.keys.DOWN_ARROW) {
				  // if a move operation is in progress, these keys will move the item up or down
				  this._onDashboardArrowMove(info['index'], info['li'], info['params'], evt.keyCode);
			  } else if (evt.keyCode == dojo.keys.ESCAPE) {
				  // if a move operation is in progress, escape will cancel it
				  this._onDashboardEscapeKey(info['index'], info['li'], info['params']);
			  } else if (evt.keyCode == dojo.keys.TAB) {
				  // if a move operation is in progress, tab will cancel it
				  this._onDashboardTabKey(info['index'], info['li'], info['params']);
			  }
		  }
        }
      },

      _handleTextClickEvent: function _handleTextClickEvent_DashboardOrganizer(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user clicked on img, don't handle that here
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
          if (info['index'] != null && info['li'] != null) {
            this._onDashboardTextClicked(info['index'], info['li'], info['params']);
          }
        }
      },

	 _onDashboardItemSelected: function _onDashboardItemSelected_DashboardOrganizer(params){
	    if (!com.ibm.bpm.social.portal.PageHandler.getInstance().isNavigationAllowed()) {
			this.hide(true);
	    	return;
	    }
		if (params.type == 'queue') {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
				dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
						savedSearchName: params.name
					}]);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
						dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
							savedSearchName: params.name
						}]);
					}
				}));
   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
		    }
		} else {
			var newparams={isWorkTab: params.isWorkTab?true:false};
			var pubObj={
				runURL: params.runURL,
				display: params.title,
				params: newparams
			};
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [pubObj]);
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(params)]);
		this.hide(true);
	},

      _cleanUpConnects: function _cleanUpConnects_DashboardOrganizer(){
        //disconnect all dojo event connectors
        if (this.dashboardOrganizerEventConnectors) {
          dojo.forEach(this.dashboardOrganizerEventConnectors, function(connector){
            dojo.disconnect(connector);
          });
        }
        this.dashboardOrganizerEventConnectors = [];
		for(var i=0; i<this.dashboardOrganizerSubscriptions.length; i++) {
			dojo.unsubscribe(this.dashboardOrganizerSubscriptions[i]);
		}
		this.dashboardOrganizerSubscriptions=[];
      },

      _cleanUpMenusAndWidgets: function _cleanUpMenusAndWidgets_DashboardOrganizer(){
        if (this.widgetsToCleanUp && this.widgetsToCleanUp.length > 0) {
          for ( var i = 0; i < this.widgetsToCleanUp.length; i++) {
            if (this.widgetsToCleanUp[i].destroyRecursive) {
              this.widgetsToCleanUp[i].destroyRecursive();
            }
            else if (this.widgetsToCleanUp[i].destroy) {
              this.widgetsToCleanUp[i].destroy();
            }
          }
        }
        this.widgetsToCleanUp = [];
        if (this.snapshotElements) {
          delete this.snapshotElements;
        }
        this._cleanUpConnects();
      },
	
	_onXhrErrors: function _onXhrErrors_DashboardOrganizer(data, xhrArgs){
		console.debug(["DashboardOrganizer experienced a data retrieval error.",
		   data,
		   xhrArgs]);
	},
	
	_updateDashboardOrder: function _updateDashboardOrder_DashboardOrganizer(displayedDashboards, hiddenDashboards){
		// displayed dashboards:
		var newObj=[];
		newObj = this._getDataForPrefs(displayedDashboards, newObj);
		newObj.push({"baseIDSeg":"HIDDEN","itemIDSeg":"HIDDEN"});
		this._getDataForPrefs(hiddenDashboards, newObj);
 
		// update the profile setting;  then compare the current with the new and persist the new only if it's different
		var saveCurrentStr = this._storeddashboardOrganizerSettings._currentProfileString;
		this._storeddashboardOrganizerSettings.dashboardOrderObj=newObj;
		this._storeddashboardOrganizerSettings.createProfileString();
		if (saveCurrentStr != this._storeddashboardOrganizerSettings._currentProfileString) {
			this.prefsDirty = true;
		}
	},
	_getDataForPrefs: function _getDataForPrefs_DashboardOrganizer(dataArray, newObj) {
		dojo.forEach(dataArray, dojo.hitch(this, function(item){
			var baseID = item.data.params.baseID;
			var dashItem={};
			if (baseID) {
				dashItem["baseIDSeg"]=baseID;
				dashItem["itemIDSeg"]="";
				if (item.data.params.type!="queue" && item.data.params.itemID && item.data.params.itemID.length>0 ) {
					var itemIDArray = item.data.params.itemID.split('-');
					if (itemIDArray && dojo.isArray(itemIDArray) && itemIDArray.length>0 ) {
						dashItem["itemIDSeg"]=itemIDArray[itemIDArray.length-1];
					}
				}
				newObj.push(dashItem);
			 } else {
				console.log("invalid exposed dashboard item, cannot save preference");
			 }
		 }));
		 return newObj;
	},
	
	show: function show_DashboardOrganizer(){
		if (this.dashboardsList) {
		  dojo.style(this.containerDiv, "display", "block");
		}
		
		// Add vertical scrolling if needed, and adjust widths as needed
		var processPortalFooter = dojo.byId("processPortalFooter");
		var footerHeight=0;
		if (processPortalFooter && processPortalFooter.offsetHeight) {
			footerHeight = processPortalFooter.offsetHeight;
		}
		var bannerArea = dojo.query('.processPortalBanner')[0];
		var bannerHeight=0;
		if (bannerArea && bannerArea.offsetHeight) {
			bannerHeight = bannerArea.offsetHeight;
		}
		var vs = dojo.window.getBox();
		var availableHeight=0;
		var totalH=0;
		var minHeight=375;
		// vs.h is height of the viewport
		if (vs && vs.h > 0) {
			// how may items are in this list?
			var itemCount = this._getNumberOfListItems();
			var y = itemCount*39 + 200;
			// take the greater of computed value and minHeight
			y = Math.max(y, minHeight);
			// take the smaller of viewport height and the computed height
			totalH = Math.min(vs.h, y);
			// subtract some constants to derive availableHeight;  further adjustments may be made below
			var allowedSpaceAboveBanner = 30;
			availableHeight = totalH - (footerHeight + bannerHeight + allowedSpaceAboveBanner);
			dojo.style(this.scrollingDiv,"height",availableHeight+"px");
		}
		
		dojo.style(this.scrollingDiv,"overflowY", "auto");
		var hasVerticalScrollbar= this.scrollingDiv.scrollHeight>this.scrollingDiv.clientHeight;
		
		// when there's no vertical scrollbar, we may need to add some height anyway - this is 
		//   because when a drag operation starts space is added above or below the drop target and
		//   that added space may cause scrollbars to appear.  If we add extra height then scrollbar won't
		//   appear.  
		if (!hasVerticalScrollbar) {
			var listElemOffsetHeight = this.listElement.offsetHeight;
			var scrollingDivOffsetHeight = this.scrollingDiv.offsetHeight;
			if (scrollingDivOffsetHeight > listElemOffsetHeight) {
				var y = scrollingDivOffsetHeight - listElemOffsetHeight;
				if (y < 99) {
					// here if we need to add some height...
					var z = 99-y;
					availableHeight+=z;
					// ... but add more height ONLY if it won't cause the dropdown to cover the footer
					if (totalH-availableHeight > 99) {
						dojo.style(this.scrollingDiv,"height",availableHeight+"px");					
					}
				} else if (y > 99) {
					// conversely there may be too much height
					var z = y - 99;;
					availableHeight-=z;
					dojo.style(this.scrollingDiv,"height",availableHeight+"px");					
				}
			}
		}
		
		// adjust when there's not enough space on left or right for the dropdown to be completely visible
		if (this.isLeftToRight()) {
			var tabIcon = dojo.query('.processPortalDashboardOrganizerTabIcon')[0];
			if (tabIcon.offsetLeft < 175) {
				// not enough space, so adjust
				dojo.style(this.containerDiv, "marginLeft", "-90px");
				dojo.style(this.topArrow, "marginLeft", "61px");
			} else { 
				dojo.style(this.containerDiv, "marginLeft", "");
				dojo.style(this.topArrow, "marginLeft", "");
			}
		} else {
			var banner = dojo.query('.processPortalBanner')[0];
			var tabIcon = dojo.query('.processPortalDashboardOrganizerTabIcon')[0];
			var x = 0;
			if (banner.offsetWidth > tabIcon.offsetLeft) {
				x = banner.offsetWidth - tabIcon.offsetLeft;
			}
			var leftVal = (tabIcon.offsetLeft - (this.containerDiv.offsetWidth/2));
			if (x<175) {
				// not enough space, so adjust; take scrollbar into account
				leftVal=leftVal-90;
				if (hasVerticalScrollbar) {
					leftVal=leftVal-30;
					dojo.style(this.topArrow, "marginRight", "46px");
				} else {
					dojo.style(this.topArrow, "marginRight", "59px");
				}
			} else {
				// take scrollbar into account in this case too
				if (hasVerticalScrollbar) {
					dojo.style(this.topArrow, "marginRight", "165px");
				} else {
					dojo.style(this.topArrow, "marginRight", "");
				}
			}
			if (!hasVerticalScrollbar) {
				leftVal=leftVal-16;
			}
			dojo.style(this.containerDiv, "marginLeft", "0px");
			dojo.style(this.containerDiv, "left", leftVal+"px");
		}
		// end adjustments
		
		// save the dnd manager so it can be replaced upon hide
		dojo.dnd._manager = this.localDndManager;
		
		this.dropdownOpen = true;
		
		// If there is a selected tab, highlight it.  Set focus to headers.
		var textNodeToHighlight = null;
		var liNodeToHighlight = null;
		if (this.currentlySelectedTab!=null) {
			// get all the items in the organizer.
			// if match found, find the node with the proper classname and highlight it.
			var map = this.dashboardsList.map;
			for (var elem in map) {
				if (map[elem].data && map[elem].data.baseID && map[elem].data.baseID==this.currentlySelectedTab) {
					textNodeToHighlight=dojo.byId(elem);
					liNodeToHighlight=dojo.byId(this.innerLi_id+map[elem].data.baseID);
				}
			}			
		}
		var node = null;
		if (textNodeToHighlight!=null) {
			node=textNodeToHighlight;
		} else {
			node = this.listElement.firstChild;
			while (node && dojo.hasClass(node,"indicatorClassForSeparator")) {
				node = node.nextSibling;
			}
		}
		var modClass=false;
		if (node && textNodeToHighlight) {
			var a = dojo.query(".dashboardItem-text",node);
			if (a.length > 0) {
				this.doNotClose = true;
				node = a[0];
				modClass=true;
			} else {
				node = null;
			} 
		}
		// This fixes a weird behavior that occurs when webkit is the useragent when there the temp tab has a snapshot name in it:  when the
		//   organizer is shown then the entire tab div moves upward.  Tweaking the height fixes it.
		if (dojo.isWebKit) {
			var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
			if (orgDiv) {
				if (dojo.hasClass(orgDiv,"hasSnapshot")) {
					dojo.style(orgDiv, "height", "1px");
				}
			}
		}
		window.setTimeout(dojo.hitch(this,function() {	
			if (!node) {
				node = this.listElement;
				this.listElement.tabIndex = 0;
			} else {
				this.listElement.tabIndex = -1;
			}

			this.topHeadersDiv.focus();
			
			//node.focus(); 
			if (modClass==true) {
				dojo.removeClass(node, "dashboardItem-text");
				dojo.addClass(node, "dashboardItem-text-selected");
                if (node.nextSibling) {
                    dojo.addClass(node.nextSibling, "dashboardItem-text-selected");//blue-twistie
                }
				if (liNodeToHighlight!=null) {
					dojo.addClass(liNodeToHighlight,"listItemSelected");
				}
			}
			this.doNotClose = false;
		}),500);
		dojo.query(".dashboardItem-title-div",this.containerDiv).forEach(function(node){
			if (node.scrollWidth > node.offsetWidth) {
				node.title = node.textContent;
				if (dojo.isIE < 9)
					node.title = node.innerText;
			}
		});
		
	},
	
	onBlur: function() {
		if (!this.doNotClose) {
			window.setTimeout(dojo.hitch(this,function() {
				if ((this.dropdownOpen == true) && !this.doNotClose) {
					this.hide(true);
				}
			}),250);
		}
	},

	_getNumberOfListItems: function _getNumberOfListItems_DashboardOrganizer () {
		var map = this.dashboardsList.map;
		var elementCount = 0;
		for (var elem in map) {
			if (map[elem].data) {
				elementCount++;
			}
		}			
		return elementCount;
	},
	
	hide: function hide_DashboardOrganizer(savePrefs){
		if (this.dashboardsList) {
		  dojo.style(this.containerDiv, "display", "none");
		}
		// close snapshot containers and untwist twistie if needed
		this._closeSnapshotItems();
		
		// remove selected style on text if needed
		this._removeSelectedStyleOnText();
		
		// remove selected style on list item if needed
		this._removeSelectedStyleOnLi();
		
		// remove hover state on item if needed (if keyboard spacebar was used to select an item for drag)
		this._removeHoverStateOnItem();
		
		this.dropdownOpen = false;

		dojo.dnd._manager = this.saveDojoDndManager;

		this.keyboardMoveOperationActive = false;
		this.originalLocationOfItemBeingMoved= {};
		// defect 134873: hide and show tab in order to remove visual artifact on IE
		if (dojo.isIE) {
			var tabBarWidget = dijit.byId("processPortalTabBar");
			if (tabBarWidget && tabBarWidget.organizerTab) {
				dojo.style(tabBarWidget.organizerTab,"display","none");
				dojo.style(tabBarWidget.organizerTab,"display","inline");
			}
		}
		
		if (savePrefs===true) {
			this._updateDashboardOrder(this.getDisplayedDashboards(), this.getHiddenDashboards());
			if (this.prefsDirty) {
				this._writeDashboardOrderToUserProfile(this._storeddashboardOrganizerSettings);
				dojo.publish(com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS,[]);
			}
		}
	},

	_closeSnapshotItems: function _closeSnapshotItems_DashboardOrganizer() {
		// get list of all items so we can close snapshot containers and untwist twistie if needed
		dojo.forEach(this.getItems(), dojo.hitch(this, function(item){
			if (item.params && item.params.snapshotItems && item.params.snapshotItems.length>0) {
				var twistie=dojo.byId(this.dashboardOrganizerSnapshotIcon_id + item.params.baseID);
				var container=dojo.byId(this.snapshotContainer_id + item.params.baseID);
				this._hideShowSnapshotsContainer(null, twistie, container, true);	 // last param, true, means hide open snapshot containers
			}
		}));
	},

	_removeSelectedStyleOnText: function _removeSelectedStyleOnText_DashboardOrganizer() {
		var a = dojo.query(".dashboardItem-text-selected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "dashboardItem-text-selected");
			dojo.addClass(a[0], "dashboardItem-text");
		}
	},

	_removeSelectedStyleOnLi: function _removeSelectedStyleOnLi_DashboardOrganizer() {
		var a = dojo.query(".listItemSelected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "listItemSelected");
		}
	},
	
	_removeHoverStateOnItem: function _removeHoverStateOnItem_DashboardOrganizer() {
		var a = dojo.query(".keyboardSelected", this.listElement);
		if (a && a.length>0) {
			dojo.removeClass(a[0], "keyboardSelected");
		}
	},

    // return a list of the displayed dashboards in the correct sorted order
	getDisplayedDashboards: function getDisplayedDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("displayed");
		}
		return null;
	},
  
    // return a list of the hidden dashboards in the correct sorted order (order doesn't really matter here)
	getHiddenDashboards: function getHiddenDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("hidden");
		}
		return null;
	},
  
    // return a list of all dashboards
	getAllDashboards: function getAllDashboards_DashboardOrganizer() {
		if (this.dashboardsList) {
			return this._getListOfDashboards("all");
		}
		return null;
	},
  
    // find a specific dashboard, including within snapshots
	getSpecificDashboard: function getSpecificDashboard_DashboardOrganizer(baseID) {
		var dashboardItem=null;
		if (this.dashboardsList) {
			var allDashboards = this._getListOfDashboards("all");
			if (allDashboards.length > 0) {
				var tItem = null;
				for (var i=0; i<allDashboards.length; i++) {
					tItem = allDashboards[i];
					if (tItem.data.params.baseID == baseID) {
						dashboardItem = tItem;
					} else {
						// check snapshots, if there are any
						if (tItem.data.params.snapshotItems && tItem.data.params.snapshotItems.length > 0) {
							for (var x=0; x < tItem.data.params.snapshotItems.length; x++) {
								var tItem2 = {};
								// the attribute "data" is expected by the caller of this function
								tItem2["data"] = tItem.data.params.snapshotItems[x];
								if (tItem2["data"].params.baseID == baseID) {
									dashboardItem = tItem2;
								}
							}
						}
					}
				}
			}
		}
		return dashboardItem;
	},
  
    // find a specific dashboard by itemID, NOT including within snapshots
	getSpecificDashboardByItemID: function getSpecificDashboardByItemID_DashboardOrganizer(itemID) {
		var dashboardItem=null;
		if (this.dashboardsList) {
			var allDashboards = this._getListOfDashboards("all");
			if (allDashboards.length > 0) {
				var tItem = null;
				for (var i=0; i<allDashboards.length; i++) {
					tItem = allDashboards[i];
					if (tItem.data.params.itemID == itemID) {
						dashboardItem = tItem;
					}
				}
			}
		}
		return dashboardItem;
	},
	
	_getListOfDashboards: function _getListOfDashboards_DashboardOrganizer(type) {
		var returnItems=[];
		var separatorFound=false;
		dojo.forEach( this._getOrderedItems(this.dashboardsList), dojo.hitch( this, 
			function(item){	
				if (!separatorFound) {
					separatorFound=dojo.indexOf(item.type,"separator")>=0?true:false;
				}
				if (dojo.indexOf(item.type,"dashboardItem")>=0 && type=="displayed" && !separatorFound) {
					returnItems.push(item);
				} else if (dojo.indexOf(item.type,"dashboardItem")>=0 &&  type=="hidden" && separatorFound) {
					returnItems.push(item);
				} else if (dojo.indexOf(item.type,"dashboardItem")>=0 &&  type=="all") {
					returnItems.push(item);
				}
        }));
		return returnItems;
	},
  
    // return the items in a Source object in sorted order
	_getOrderedItems: function _getOrderedItems_DashboardOrganizer(container){
	  var items=[];
	  container.getAllNodes().forEach(function(node){
		var id = node.id;
		var tempItem=container.getItem(id);
		tempItem["dojoUniqueId"]=id;
		items.push(tempItem);
	  });
	  return items;
	},
	
	setCurrentlySelectedTab: function setCurrentlySelectedTab_DashboardOrganizer(baseID){
		this.currentlySelectedTab=baseID;
	},

	setCurrentlySelectedTabByItemID: function setCurrentlySelectedTabByItemID_DashboardOrganizer(itemID){
		// find the top-level item (not snapshot) with this itemID, and get its baseID
		var item = this.getSpecificDashboardByItemID(itemID);
		if (item) {
			this.currentlySelectedTab=item.data.baseID;
		}
	},
	
	// Convenience function; the returned value is what was retrieved from the ExposedItemsList in the _getExposedItems() above.
	// Returns true if the current env is Process Server
	// *** The return value is reliable *if* there were >0 exposed items found in the ExposedItemsList.getData() function.
	// If there were 0 exposed items found the the value is unreliable but is also irrelevant -- 
	//    the returned value only matters when there are exposed items.
	
	getIsProcessServer: function isProcessServer_DashboardOrganizer() {
		return this.isProcessServer;
	},

	destroy: function destroy_DashboardOrganizer(){
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if (this.dashboardsList) {
		  if (this.dashboardsList.destroyRecursive) {
			this.dashboardsList.destroyRecursive();
		  }
		  else if (this.dashboardsList.destroy) {
			this.dashboardsList.destroy();
		  }
		}
		if (this.dojoConnections) {
			for (var i=0; i <this.dojoConnections.length; i++) {
				this.dojoConnections[i].destroy();
			}
			this.dojoConnections=null;
		}
		this._cleanUpMenusAndWidgets();

		if (this.saveDojoDndManager) {
			dojo.dnd._manager = this.saveDojoDndManager;
			if (this.localDndManager && this.localDndManager.destroy) {
				this.localDndManager.destroy();
			}
			this.localDndManager=null;
		}
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});


////////////////////////////////////////
// GLOBALS
////////////////////////////////////////

com.ibm.bpm.social.portal.DashboardOrganizer.DISPLAY_ORDER_PREFERENCE_NAME = "Portal Dashboard Display Order";



////////////////////////////////////////
// Extend dojo.dnd.Source in order to override checkAcceptance
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndSource", dojo.dnd.Source, {
    checkAcceptance : function(source, nodes) {
		// Don't allow the separator to be dragged.
		var attr = dojo.attr(nodes[0],"id");
		if (attr && attr==="dashboardSeparatorItem") {
			return false;
		}
		// since checkAcceptance is called when a drag operation starts, we can
		//   publish a message telling the Organizer to close the snapshot containers of 
		//   all items, as needed.
		dojo.publish("dashboardOrganizer_hideSnapshotContainer",[]);		
		/* // Get the ID that the Organizer will use to identify the snapshot container.
		if (nodes[0].id) {
			// source stores the baseID that we need
			if (source && source.map && source.map[nodes[0].id]) {
				var map = source.map[nodes[0].id];
				if (map.data && map.data.params && map.data.params.snapshotItems && map.data.params.snapshotItems.length>0) {
					// this is the ID we need:
					dojo.publish("dashboardOrganizer_hideSnapshotContainer",[{"baseID":map.data.params.baseID }]);
				}
			}
		} */
		
       return this.inherited(arguments);
    },
	onDndStart: function(source, nodes, copy){
		this.savedFocusNode = dijit.getFocus().node;
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		dijit.focus(organizer.domNode);
		this.inherited(arguments);
	},
	onDndCancel: function(){
		this.inherited(arguments);
		if (this.savedFocusNode) {
			dijit.focus(this.savedFocusNode);
			this.savedFocusNode = null;
		}
	},    
    onDropInternal: function(nodes, copy){
    	var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
    	organizer.doNotClose = true;
    	this.inherited(arguments);
   		window.setTimeout(function() {
   			organizer.doNotClose = false;
   		},250);
    },
	onMouseDown: function(e){
		// ON IE, need to add the dojoDndMove class *before* the drag starts.  Remove it in onMouseUp().
		// summary:
		//		event processor for onmousedown
		// e: Event
		//		mouse event
		if (dojo.isIE) {
			dojo.addClass(dojo.body(), "dojoDndMove");
		}
		this.inherited(arguments);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup
		// e: Event
		//		mouse event
		if (dojo.isIE) {
			dojo.removeClass(dojo.body(), "dojoDndMove");
		}
		this.inherited(arguments);
	}
});

////////////////////////////////////////
// Extend dojo.dnd.Manager in order to override avatar creation
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndManager", dojo.dnd.Manager, {
	// avatar's offset from the mouse
	OFFSET_X: 4,
	OFFSET_Y: 4,
    makeAvatar : function(source, nodes) {
		return new com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndAvatar(this);
    }
});
////////////////////////////////////////
// Extend dojo.dnd.Avatar in order to override avatar
////////////////////////////////////////
dojo.declare("com.ibm.bpm.social.portal.DashboardOrganizer.CustomDndAvatar", dojo.dnd.Avatar, {
	construct: function(){
		// summary:
		//		constructor function;
		//		it is separate so it can be (dynamically) overwritten in case of need
		this.isA11y = dojo.hasClass(dojo.body(),"dijit_a11y");
		var a = dojo.create("table", {
				"class": "dojoDndAvatar",
				style: {
					position: "absolute",
					zIndex:   "1999",
					margin:   "0px"
				}
			}),
			source = this.manager.source, node,
			b = dojo.create("tbody", null, a),
			k = Math.min(5, this.manager.nodes.length), i = 0;
		for(; i < k; ++i){
			if(source.creator){
				// create an avatar representation of the node
				node = source._normalizedCreator(source.getItem(this.manager.nodes[i].id).data, "avatar").node;
			}else{
				// or just clone the node and hope it works
				node = this.manager.nodes[i].cloneNode(true);
				if(node.tagName.toLowerCase() == "tr"){
					// insert extra table nodes
					var table = dojo.create("table"),
						tbody = dojo.create("tbody", null, table);
					tbody.appendChild(node);
					node = table;
				}
			}
			node.id = "";
			tr = dojo.create("tr", null, b);
			td = dojo.create("td", null, tr);
			if (this.isA11y) {
				icon = dojo.create("span", {id : "a11yIcon", innerHTML : "<"}, td);
				icon.style.display = 'none';
			}
			td.appendChild(node);
		}
		this.node = a;
	}
});