// PageHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.portal.PageHandler");

dojo.require("dojo.cookie");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.config.ViewConfig");
dojo.require("com.ibm.bpm.social.util.common.PageSettings");
dojo.require("com.ibm.bpm.social.util.common.SystemRestHandler");
dojo.require("com.ibm.bpm.social.portal.DashboardOrganizer");
dojo.require("com.ibm.bpm.social.portal.DashboardDropdown");
dojo.require("com.ibm.bpm.social.portal.ProcessPortalTabBar");
dojo.require("com.ibm.bpm.social.widgets.ExposedItemsList");

dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");

dojo
  .declare(
    "com.ibm.bpm.social.portal.PageHandler",
    null,
    {

      /**
       * PageHandler : a class handling setting and getting the default start page for the work tab
       * 
       * <p>
       * Mapping of pages to string:
       * 
       * <table width="100%" >
       * 
       * <tr>
       * <td><b> Page </b></td>
       * <td><b> Stored string </b></td>
       * </tr>
       * <tr>
       * <td> My Tasks </td>
       * <td> /tasks </td>
       * </tr>
       * <tr>
       * <td> Dashboards tab </td>
       * <td> /dashboards?dashboard=PATH_TO_DASHBOARD </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Open Tasks </td>
       * <td> /tasks/open?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Completed Tasks </td>
       * <td> /tasks/completed?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> </td>
       * <td> </td>
       * </tr>
       * <tr>
       * <td> Saved Search </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i> </td>
       * </tr>
       * <tr>
       * <td> Saved Search with search string </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i>&amp;searchFilter=<i>URL_encoded_search_string</i>
       * </td>
       * </tr>
       * <tr>
       * <td> Sorted Saved Search with search string </td>
       * <td> /tasks/queries<b>?query</b>=<i>URL_encoded_name_of_saved_search</i><b>&amp;searchFilter</b>=<i>URL_encoded_search_string</i><b>&amp;sortAttributes</b>=<i>sortAttr1,sortAttr2,..</i><b>&amp;sortOrder</b>=<i>order1,order2,...</i>
       * </td>
       * </tr>
       * </table>
       * 
       */


      _storedStartPageSettings: null,
      _currentPageSettings: null,

      _startPageButton: null, // DOM node of the icon/button we are maintaining

      // DOM connections tracker
      _connects: [],

      _activePageWidget: null,
      _workPageWidget: null,
      _dashboardPageWidget: null,
      _hashsub: null,
      _navsub1: null,
      _navsub2: null,
      _navsub3: null,
	  _workDashboardItem: null,
	  _pageHander: null,
	  _preventNavigation: null,


      /**
       * Constructor
       * 
       * @type {void}
       */
      constructor: function constructor_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Return a copy of PageSettings of this instance. <br>
       * Since we return a copy, we can protect our PageSettings object.
       * 
       * @type {void}
       */
      getStoredPageSettings: function getStoredPageSettings_PageHandler(){
        com.ibm.bpm.social.util.common.Log.log(3);
        return new com.ibm.bpm.social.util.common.PageSettings(this._storedStartPageSettings);
      },


      /**
       * Cleanup handler <br>
       * Note: since this is not a widget, needs to called manually
       * 
       * @type {void}
       */
      destroy: function destroy_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

		if (this._hashsub) {
			dojo.unsubscribe(this._hashsub);
			this._hashsub = null;
		}
		if (this._navsub1) {
			dojo.unsubscribe(this._navsub1);
			this._navsub1 = null;
		}
		if (this._navsub2) {
			dojo.unsubscribe(this._navsub2);
			this._navsub2 = null;
		}
		if (this._navsub3) {
			dojo.unsubscribe(this._navsub3);
			this._navsub3 = null;
		}
        while (this._connects.length > 0) {
          var c = this._connects.pop();
          try {
            dojo.disconnect(c);
          }
          catch (e) {
            // we are not really interested in errors while cleanup  
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Read the users profile setting "Portal default page" and parse it into the members
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings}
       */
      readFromUserProfile: function readFromUserProfile_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        var startPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        var userProfileString;

        if (userProfile && userProfile.userPreferences &&
          userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME]) {
          userProfileString = userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME];
          com.ibm.bpm.social.util.common.Log.log(2, "userProfile string is " + userProfileString);

          var newStartPageSettings = startPageSettings.parseFromString(userProfileString, true);
          if (!newStartPageSettings.isValid) {
            // String is invalid
            // the original startPageSettings object is unmodified and has default values
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is invalid, using default string: " +
              startPageSettings.getCurrentProfileString());
          }
          else {
            // the string is valid and the values are mixed into the original PageSettings object
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is valid, using now string: " +
              startPageSettings.getCurrentProfileString());
          }
        }
        else {
          // if the start page is not set, show My Tasks
          com.ibm.bpm.social.util.common.Log.log(2, "user attribute not set, using default string: " +
            startPageSettings.getCurrentProfileString());
        }


        if (com.ibm.bpm.social.util.common.Log.logLevel > 1) {
          com.ibm.bpm.social.util.common.Log.logObject(2, {
            Tab: startPageSettings.tab,
            Filter: startPageSettings.filterConfigId,
            Saved_Search: startPageSettings.savedSearchName,
            Search_Filter: startPageSettings.searchFilterString,
            Sort_Attributes: startPageSettings.sortAttributes,
            dashboardURL: startPageSettings.dashboardURL
          }, "Got following settings from user profile: ");
        }

        // update the _storedStartPageSettings to the newest settings, but use a separate copy
        this._storedStartPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageSettings;
      },


      /**
       * write the default start page settings into the user profile
       * 
       * @param {com.ibm.bpm.social.util.common.PageSettings} PageSettings if given, the settings to be
       *        written to the profile
       * 
       * @type {void}
       */
      _writeStartPageToUserProfile: function _writeStartPageToUserProfile_PageHandler(startPageSettings){
        com.ibm.bpm.social.util.common.Log.logEntry(2, startPageSettings.getCurrentProfileString);
        if (startPageSettings) {
          com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME,
            startPageSettings.getCurrentProfileString());
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * 
       * @type {void}
       */
      setStartPage: function setStartPage_PageHandler(startPageSettings, startPageButton){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var newPageSettings = startPageSettings;

        if (startPageButton.isSelected === true) {
          // we are already on the start page -> User wants to reset the start page to the default start page
          newPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        }

        this._writeStartPageToUserProfile(newPageSettings);
        this._storedStartPageSettings = newPageSettings;

        this._currentPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);
        this.checkAndUpdateStartPageIcon(this._currentPageSettings, startPageButton);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Compare the new content of the start page against the stored one and update the page icon if the
       * content changed
       * 
       * @type {void}
       */
      checkAndUpdateStartPageIcon: function checkAndUpdateStartPageIcon_PageHandler(startPageSettings, startPageButton /* domNode */){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler");

        if (startPageSettings.getCurrentProfileString() == this._storedStartPageSettings.getCurrentProfileString()) {
          // set the check mark icon
          dojo.removeClass(startPageButton, "startPageBookmark-unselected");
          dojo.addClass(startPageButton, "startPageBookmark-selected");
          startPageButton.isSelected = true;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "&#x2713;"; // a11y check mark sign
          if (this._storedStartPageSettings.getCurrentProfileString() == startPageSettings.defaultPageString) {
            // no reset to the default page since we are already there   
            startPageButton.title = this._nlsResources["page_handler_is_default_page"];
          }
          else {
            startPageButton.title = this._nlsResources["page_handler_is_default_page_with_reset"];
          }
        }
        else {
          // set the plus icon
          dojo.removeClass(startPageButton, "startPageBookmark-selected");
          dojo.addClass(startPageButton, "startPageBookmark-unselected");
          startPageButton.isSelected = false;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "+"; //a11y
          startPageButton.title = this._nlsResources["page_handler_set_start_page_title"];
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler");
        return;
      },


      /**
       * Return a div formatted as a start page button
       * 
       * @type {void}
       */
      createStartPageIcon: function createStartPageIcon_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // create the start page icon/button
        var startPageButton = document.createElement("div");

        startPageButton.tabIndex = "0";
        startPageButton.isSelected = true;
        dojo.addClass(startPageButton, "startPageBookmark-selected");

        var a11yNode = document.createElement("span"); //<span class="bpm-social-a11y">+</span>
        a11yNode.className = "bpm-social-a11y";
        a11yNode.innerHtml = "&#x2713;"; // a11y check mark sign 
        startPageButton.appendChild(a11yNode);

        dijit.setWaiRole(startPageButton, "button");
        dijit.setWaiState(startPageButton, "label", this._nlsResources["page_handler_set_start_page"]);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageButton;
      },


      /**
       * Initialize the dashboard organizer dropdown on the organize tab
       * 
       * @type {void}
       */
      createDashboardOrganizer: function createDashboardOrganizer_PageHandler(openOnCreate){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this.dashboardOrganizer) {
          var div = document.createElement("div");
          dojo.byId("processPortalDashboardOrganizerId").appendChild(div);
          this.dashboardOrganizer = new com.ibm.bpm.social.portal.DashboardOrganizer({"openOnCreate":openOnCreate}, div);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Return the dashboard organizer
       * 
       * @type {DashboardOrganizer}
       */
	  getDashboardOrganizer: function getDashboardOrganizer_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer;
	  },
	  
      /**
       * Return the displayed dashboards
       * 
       * @type {list}
       */
	  getDisplayedDashboards: function getDisplayedDashboards_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer.getDisplayedDashboards();
	  },
	  
      /**
       * Return the hidden dashboards
       * 
       * @type {list}
       */
	  getHiddenDashboards: function getHiddenDashboards_PageHandler () {
        if (!this.dashboardOrganizer) {
          this.createDashboardOrganizer(false);
        }
		return this.dashboardOrganizer.getHiddenDashboards();
	  },

      /**
       * This functions captures a click on the banner to check if the drop down needs to be closed. (because
       * onblur handler of the DashboardOrganizer doesn't work here (for any reason)
       * 
       * @param event
       * @type void
       */

      hideDashboardOrganizer: function hideDashboardOrganizer_PageHandler(event){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        var target = null;
        if (dojo.isIE == 8) {
          target = event.srcElement;
        }
        else {
          target = event.target;
        }

        if (target && target.id && target.id != "processPortalDashboardTabDropDownImg" && target.id != "processPortalDashboardOrganizerLink" 
			&& target.id.indexOf("id_dashboardOrganizerSnapshotIcon")<0 ) {
          if (this.dashboardOrganizer && this.dashboardOrganizer.dropdownOpen) {
            com.ibm.bpm.social.util.common.Log.log(2, "Caught click on banner and dashboard organizer is open. Hiding ...");
            this._hideDashboardOrganizer();
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(3);
        return false;
      },

      /**
       * handle a click on the dashboard organizer dropdown arrow
       * 
       * @type {void}
       */
      onDashboardOrganizerClick: function onDashboardOrganizerClick_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!this.dashboardOrganizer) {
          // creating the drop down also opens it
          this.createDashboardOrganizer(true);
        }
        else if (this.dashboardOrganizer.dropdownOpen) {
          //DashboardOrganizer is already open, close it);
		  this._hideDashboardOrganizer();
        }
        else {
          this.dashboardOrganizer.show();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },
	  
	  _hideDashboardOrganizer: function _hideDashboardOrganizer_PageHandler() {
          this.dashboardOrganizer.hide(true);  //true: save organizer user pref setting
	  },


      /**
       * Event handler for the onDashboardRestart event, delegate this to the dashboard controller
       * 
       * @type {void}
       */
      restartDashboard: function restartDashboard_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: "RESTART_DASHBOARD",
          display: "RESTART_DASHBOARD"
        }]);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
      
      _taskCompletion: function _taskCompletion_PageHandler(taskId) {
        var rc = false;
        if (taskId) {
         	var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
				'taskId': taskId
			});
			if (taskData) {
				if (taskData.errorInfo) {
					// 401 is unathorized, so the task exists and we
					// want to let the default error message show up,
					// otherwise assume the id is bad 
					if (taskData.errorInfo.ioArgs.xhr.status != 401) {
						if (this._workDashboardItem != null) {
							var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["task_not_available"]);
						}
						taskId = null;
					}
				}/*  else {
					// If the work dashboard isn't exposed we need to handle some messaging here.
					if (this._workDashboardItem == null && taskData.state && taskData.state == "STATE_FINISHED") {
						var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(nls["task_already_completed"]);
						taskId = null;
					}
				} */
			} else {
				if (this._workDashboardItem != null) {
					var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
					com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["task_not_available"]);
				}
				taskId = null;
			}
		}
		if (taskId) {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this.showTaskCompletion(taskId,0);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this.showTaskCompletion(taskId,0);
					}
				}));
				com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
			rc = true;
		} 
        return rc;
      },
      
      _processInstance: function _processInstance(piid) {
      	var rc = false;
		if (piid) {
			var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
				'header': true, 
				'instanceId': piid,
				'skipCache': true
			});
			if (instanceData) {
				if (instanceData.errorInfo) {
					// 401 is unathorized, so the process exists and
					// want to let the default error message show up,
					// otherwise assume the id is bad
					if (instanceData.errorInfo.ioArgs.xhr.status != 401) {
						var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["process_instance_not_available"]);
						piid = null;
					}
				}
			} else {
				var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["process_instance_not_available"]);
				piid = null;
			}
		}
		if (piid) {
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this.displayLegacyInstanceDetails(piid,0);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this.displayLegacyInstanceDetails(piid,0);
					}
				}));
				com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
			rc = true;
		}
		return rc;
      },
      
      _showSavedSearch: function _showSavedSearch_PageHandler(item,searchFilter) {
		com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
			savedSearchName: item.params.name,
			searchFilter: searchFilter
		}]);
	    dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(item.params)]);
      },
      
      _findSavedSearch: function _findSavedSearch_PageHandler(savedSearchName,searchFilter,errorMessageData,showSearchIfFound) {
       	var item;
		var found = false;
		for ( var i = 0; i < com.ibm.bpm.social.portal.dashboardItems.length; i++) {
			item = com.ibm.bpm.social.portal.dashboardItems[i];
			if (item.params.name == savedSearchName && item.params.type=="queue") {
				found = true;
				break;
			}
		}
		if (found && showSearchIfFound) {
			// If showErrorMessage is true then tell the user they are being redirected to their default start page
			if (errorMessageData && errorMessageData.showErrorMessage && errorMessageData.showErrorMessage==true) {			
				// If currentHash is the string "null" then this code has been reached during login and 
				//   there's no need to display any message.
				// If currentHash is defined but not the string "null", show error message indicating that the 
				//     requested URL is not available and the user's start page will be shown.  
				// If the current hash is available then supply it in the messasge.
				if (errorMessageData.currentHash && errorMessageData.currentHash=="null") {
					// do nothing
				} else {
					if (errorMessageData.currentHash) {
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
							dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [errorMessageData.currentHash]));
					} else {
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
					}
				}
			}
			if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
				this._showSavedSearch(item,searchFilter);
		    } else {
				this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
					if (this.viewReadySub) {
						dojo.unsubscribe(this.viewReadySub);
						this.viewReadySub = null;
						this._showSavedSearch(item,searchFilter);
					}
				}));
   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
			}
      	}
      	return found;
      },
      
      _showTaskList: function _showTaskList_PageHandler(state,searchFilter) {
		com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
			state: state,
			searchFilter: searchFilter
		}]);
		if (this._workDashboardItem != null) {
			this._workDashboardItem.params["isWorkTab"]=true;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);
		}
      },
      
      _parseQueryString: function _parseQueryString(url) {
		if (!url) url = "";
		var urlParamString = url.substring(url.indexOf('?')+1);
		var urlParamList = urlParamString.split('&');
		var urlParams = new Object();
		// We need to capture whether or not this is a scoreboard.  We do this
		//   by seeing if the url contains "viewScoreboard.do". If this is false, check again for "serviceName=Scoreboard".
		var isScoreboard = (url.indexOf("viewScoreboard.do")>-1)?true:false;
		if (isScoreboard == false) {
			isScoreboard = (url.indexOf("serviceName=Scoreboard")>-1)?true:false;
		}
		for (var p1=0; p1<urlParamList.length; p1++) {
			var p2 = urlParamList[p1].indexOf('=');
			if (p2 > -1) {
				urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
			}
		}
		urlParams["isScoreboard"] = isScoreboard;
		return urlParams;
      },
      
      
      _findDashboardFromURL: function _findDashboardFromURL_PageHandler(dashboardURL) {
        	var dashboardParams = this._parseQueryString(dashboardURL);
		return this._findDashboardFromParams(dashboardParams);
	 },

	 _compareParams: function _compareParams(param1,param2) {
		var match = false;
		if (param1 == param2) {
			match = true;
		} else if (encodeURIComponent(param1) == param2) {
			match = true;
		} else {
			try {
				var d = decodeURIComponent(param2);
				if (param1 == d) {
					match = true;
				}
			} catch (e) {}
		}
		return match;
	 },

      _findDashboardFromParams: function _findDashboardFromParams_PageHandler(dashboardParams) {
		// Note that a scoreboard URL does not contain a 'snapshot' request param even if the scoreboard
		//    is from a snapshot.   A scoreboard URL just has:      executeServiceByName?processApp=ZZZ&serviceName=Scoreboard&id=2015.71
	     var item = null;
     	   	var found = false;
		if ((dashboardParams.processApp == 'TWP') && (dashboardParams.serviceName == 'Scoreboard') && dashboardParams.id) {
			dashboardParams.processApp = undefined;
			dashboardParams.serviceName = undefined;
			dashboardParams.isScoreboard = true;
		}   
		for ( var i = 0; (i < com.ibm.bpm.social.portal.dashboardItems.length) && !found; i++) {
			item = com.ibm.bpm.social.portal.dashboardItems[i];
			if (item && item.params && item.params.runURL) {
				var runParams = this._parseQueryString(item.params.runURL);
				// if this is the work tab we don't care about the rest of the logic
				if (!item.params.isWorkTab) {
					// Make sure we compare items of the same type
					var typesMatch=(dashboardParams.isScoreboard===runParams.isScoreboard);
					if (typesMatch) {
						if ((dashboardParams.processApp == runParams.processApp) &&
							this._compareParams(dashboardParams.serviceName,runParams.serviceName)) {
							if (!dashboardParams.snapshot && !runParams.snapshot && !dashboardParams.isScoreboard) {
								// No snapshot specified, so use default;  NOTE the check for not scoreboard, since scoreboard URLs don't have a snapshot param
								//  This handles a sneaky case where the Tip is not the top-level item because one of the
								//  snapshots has been set as the default version.
								found = true;
							} else if ((dashboardParams.snapshot && runParams.snapshot) && 
								this._compareParams(dashboardParams.snapshot,runParams.snapshot)) {
								// Anything except a scoreboard, and the snapshot params match
								found = true;
							} else if (dashboardParams.isScoreboard===true && (dashboardParams.id == runParams.id)) {
								// Scoreboard, not in a snapshot, the ids must match
								found=true;
							} else if (item.params.snapshotItems && dojo.isArray(item.params.snapshotItems)) {
								//  This can be a dashboard or scoreboard (not Saved Search).
								//  See if the dashboard URL is found in the snapshots
								var snapItem = null;
								for (var j=0; (j<item.params.snapshotItems.length) && !found; j++) {
									snapItem=item.params.snapshotItems[j];
									if (snapItem && snapItem.params && snapItem.params.runURL) {
										var snapParams = this._parseQueryString(snapItem.params.runURL);
										// Here if this is not a scoreboard....
										if (!dashboardParams.isScoreboard) {
											if (this._compareParams(dashboardParams.snapshot,snapParams.snapshot)) {
												found = true;
											}
										} else {
											if (dashboardParams.id == snapParams.id) {
												found = true;
											}
										}
									}
								}
								if (found) {
									item = snapItem;
								}
							} 
							// defect 137099
							// if a snapshot was not specified and the logic did not find a
							// tip, the default should be used (process server)
							if (!found && !dashboardParams.snapshot && !dashboardParams.isScoreboard) {
								found = true;
							}
						}
					} 
				}
			}
		}
		if (!found) {
		    // Check exposed URLs
			item = null;
			for ( var i = 0; (i < com.ibm.bpm.social.portal.exposedURLs.length) && !found; i++) {
				item = com.ibm.bpm.social.portal.exposedURLs[i];
				if (item && item.params && item.params.runURL) {
					var runParams = this._parseQueryString(item.params.runURL);
					if ((dashboardParams.processApp == runParams.processApp) &&
						this._compareParams(dashboardParams.serviceName,runParams.serviceName)) {
						if (!dashboardParams.snapshot && !runParams.snapshot) {
							// No snapshot specified, so use default;
							found = true;
						} else if ((dashboardParams.snapshot && runParams.snapshot) && 
							this._compareParams(dashboardParams.snapshot,runParams.snapshot)) {
							// snapshot params match
							found = true;
						} else if (item.params.snapshotItems && dojo.isArray(item.params.snapshotItems)) {
							//  See if the URL is found in the snapshots
							var snapItem = null;
							for (var j=0; (j<item.params.snapshotItems.length) && !found; j++) {
								snapItem=item.params.snapshotItems[j];
								if (snapItem && snapItem.params && snapItem.params.runURL) {
									var snapParams = this._parseQueryString(snapItem.params.runURL);
									// Compare snapshots
									if (this._compareParams(dashboardParams.snapshot,snapParams.snapshot)) {
										found = true;
									}
								}
							}
							if (found) {
								item = snapItem;
							}
						}
						// defect 137099
						// if a snapshot was not specified and the logic did not find a
						// tip, the default should be used (process server)
						if (!found && !dashboardParams.snapshot) {
							found = true;
						}
					}
				}
			}
		}
		if (!found) {
			item = null;
		}
		return item;
	},
	
	_showDashboard: function _showDashboard_PageHandler(item,params) {
		if (item && item.params && item.params.isWorkTab && item.params.isWorkTab==true) {
			this.selectWorkTab();
		} else {
			// We need to remove the 'id' attribute in params if this is a scoreboard because it gets erroneously added to the url
			var isScoreboard = (item.params.runURL.indexOf("viewScoreboard.do")>-1)?true:false;
			if (isScoreboard && params && params.id) {
				delete params.id;
			}
			if (!this.selectDashboardsTab(item.params.runURL, item.title, undefined, params)) {			
				dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(item.params)]);
			}
		}
    },

	// This is called by the organizer if the work dashboard is exposed
	setWorkTabInfo: function setWorkTabInfo_PageHandler(workTabItem) {
		this._workDashboardItem = workTabItem;
	},
	
	checkUrlHash: function checkUrlHash_PageHandler(hash) {
	  
		var rc = false;
		if (hash == undefined) {
  			hash = com.ibm.bpm.social.util.common.Util.getCurrentHash(true);
  		}
		dojo.cookie("com_ibm_bpm_process_portal_hash", null, {path: "/"});
		if (hash) {
			com.ibm.bpm.social.util.common.Util.setCurrentHash('');
			if (hash.charAt(0) != '/') {
				hash = '/' + hash;
			}
			var paramString = null;
			var i = hash.indexOf('?');
			if (i != -1) {
				paramString = hash.substring(i+1);
				var j = paramString.indexOf('=');
				var k = paramString.indexOf('?');
				while ((j != -1) && (k != -1) && (j > k)) {
					i = i + k;
					paramString = paramString.substring(k);
					j = paramString.indexOf('=');
					k = paramString.indexOf('?');
				}
				if (j != -1) {
					hash = hash.substring(0,i);
				} else {
					paramString = null;
				}
			}
			hash = hash.split('/');

			// convert legacy urls to new form
			if ((hash.length == 3) && (hash[1] == 'tasks')) {
				var taskId = parseInt(hash[2]);
				hash[1] = 'dashboards';
				hash[2] = 'TWP';
				hash[3] = 'BPM_WORK';
				paramString = 'tw.local.view=taskcompletion&tw.local.taskid='+taskId;
			}
			if ((hash.length == 3) && (hash[1] == 'processes')) {
				var piid = parseInt(hash[2]);
				hash[1] = 'dashboards';
				hash[2] = 'TWP';
				hash[3] = 'Process+Performance';
				paramString = 'tw.local.selectedInstanceId='+piid;
			}
			
			if ((hash.length > 3) && (hash[1] == 'dashboards')) {
				var processApp = hash[2];
				var serviceName = hash[3];
				var snapshot = null;
				if (hash.length > 4) {
					snapshot = hash[4];
					for (var s=5; hash.length > s; s++) {
						snapshot = snapshot + '/' + hash[s];
					}
				}
				var params = new Object();
				if (paramString) {
					var paramList = paramString.split('&');
					for (var i=0; i<paramList.length; i++) {
						var j = paramList[i].indexOf('=');
						if (j > -1) {
							var paramName = paramList[i].substring(0,j);
							if ((processApp == 'TWP') && (serviceName == 'BPM_WORK')) {
								if ((paramName == 'view') || 
									(paramName == 'id') || 
									(paramName == 'taskid') || 
									(paramName == 'searchFilter') ||	
									(paramName == 'state')) {
									paramName = 'tw.local.'+paramName;
								}
							}
							params[paramName] = paramList[i].substring(j+1);
							// if the parameter is a searchFilter, check for escaped ampersand
							// within the string
							while ((paramName == 'tw.local.searchFilter') &&
								   (params[paramName].charAt(params[paramName].length-1) == '\\') &&
								   ((i+1)<paramList.length)) {
								i++;
								params[paramName] = params[paramName] + '&' + paramList[i];
							}
							// if the parameter is a saved search id, the only other allowed
							// parameter is searchFilter and any other ampersands must be part
							// of the saved search name instead of parameter delimiters
							if ((paramName == 'tw.local.id') && (params['tw.local.view'] == 'savedsearch')) {
								while (((i+1)<paramList.length) && 
									(paramList[i+1].indexOf('searchFilter') != 0) &&
									(paramList[i+1].indexOf('tw.local.searchFilter') != 0)) {
									i++;
									params[paramName] = params[paramName] + '&' + paramList[i];
								}
							}
						}
					}
				}
				if (((this._workDashboardItem != null)) && (processApp == 'TWP') && (serviceName == 'BPM_WORK')) {
					if (params['tw.local.view'] == 'taskcompletion') {
						var taskId = params['tw.local.taskid'];
						rc = this._taskCompletion(taskId);
						if (!rc) {
							// log message for bad task id
							com.ibm.bpm.social.util.common.Log.log(2, "The specified task id is invalid: "+params.taskid,"PageHandler");
						} 
					}
					else if (params['tw.local.view'] == 'tasks') {
						if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
							this._showTaskList(params['tw.local.state'],params['tw.local.searchFilter']);
					    } else {
							this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
								if (this.viewReadySub) {
									dojo.unsubscribe(this.viewReadySub);
									this.viewReadySub = null;
									this._showTaskList(params['tw.local.state'],params['tw.local.searchFilter']);
								}
							}));
			   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
						}
						rc = true;
					}
					else if (params['tw.local.view'] == 'savedsearch') {
						rc = this._findSavedSearch(params['tw.local.id'], params['tw.local.searchFilter'], null, true);
						if (!rc) {
							// log message for bad task id
							com.ibm.bpm.social.util.common.Log.log(2, "The specified saved search id is invalid: "+params['tw.local.id'],"PageHandler");
						} 
					}
					else {
						com.ibm.bpm.social.util.common.Log.log(2, "The specified work view is not supported: "+params['tw.local.view'],"PageHandler"); 
					}
				} else if ((dojo.isIE < 9) && (processApp == 'TWP') && ((serviceName == 'Process+Performance') || (serviceName == 'Process Performance')) && params['tw.local.selectedInstanceId']) {
					if (this._workDashboardItem == null) {
						// work tab must be exposed on IE8 for process instance view 
						rc = false; 
					} else {
						// process instance view on IE8
						this._processInstance(params['tw.local.selectedInstanceId']);
						rc = true;		
					}
				} else {
					// find item for dashboard and show it
		        	var dashboardParams = this._parseQueryString(paramString);
					dashboardParams.processApp = processApp;
					dashboardParams.serviceName = serviceName;
					if (snapshot) {
						dashboardParams.snapshot = snapshot;
					}
					var item = this._findDashboardFromParams(dashboardParams);
					if (item) {
						this._showDashboard(item,params);
						rc = true;
					}
				}				
			} else {
				// log message for unsupported fragment
				com.ibm.bpm.social.util.common.Log.log(2, "The specified URL fragment is not supported: #"+hash.join('/'),"PageHandler"); 
			}
		}	
		return rc;
	  },

      isNavigationAllowed: function isNavigationAllowed_PageHandler() {
      	if (this._preventNavigation) {
      		com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this._preventNavigation);
      		return false;
      	}
      	return true;
      },	

      /**
       * Read the start page settings from user profile and show the start page
       * 
       * @type {void}
       */
      showStartPage: function showStartPage_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this.navsub1) {
        	this.navsub1 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.NAVIGATION,dojo.hitch(this,function(msg) {
        		if (msg.indexOf('%') != -1) {
        			try {
        				msg = decodeURIComponent(msg);
        			} catch (e) {
        				// if we cannot decode, it must no be encoded and
        				// just happens to have a % character in the msg
        			}
        		}
        		if (this.isNavigationAllowed()) {
	        		if (msg) {
	        			var rc = this.checkUrlHash(msg);
	        			if (rc != true) {
	        				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
		        			this.showStartPage();
	        			}
	        		} else {
	        			this.showStartPage();
	        		}
        		}
        	}));
        }
        if (!this.navsub2) {
        	this.navsub2 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.PREVENT_NAVIGATION,dojo.hitch(this,function(msg) {
        		this._preventNavigation = msg;
        	}));
        }
        if (!this.navsub3) {
        	this.navsub3 = dojo.subscribe(com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION,dojo.hitch(this,function() {
        		this._preventNavigation = null;
        	}));
        }
		var message="";
        this._workPageWidget = dijit.byId("workPage");
        this._workPageDiv = dojo.byId("workPageDiv");
        this._dashboardPageWidget = dijit.byId("dashboardPage");
        this._dashboardPageDiv_Div = dojo.byId("dashboardPageDiv");
		// defect 130687
		this._dashboardPageDiv = dojo.byId("dashboardPage");
		if (this._dashboardPageDiv) {
			dojo.style(this._dashboardPageDiv, "overflow", "hidden");
		}
        if (!this.dashboardOrganizer) {
			this.createDashboardOrganizer(false);
            // ensure the drop down is closed
            if (this.dashboardOrganizer.dropdownOpen) {
				this.dashboardOrganizer.hide(false); // false: don't save dashboard order user pref
            }
         }
        if (!com.ibm.bpm.social.portal.exposedURLs) {
			this._getExposedURLList();
         }

		var currentHash = com.ibm.bpm.social.util.common.Util.getCurrentHash(true);
		var retVal = this.checkUrlHash();
		if (retVal != true) {
			var needToShowWorkTab=false;
			var dashboardURL = this._storedStartPageSettings.dashboardURL;

			if (this._storedStartPageSettings.tab === this._storedStartPageSettings.tabPrefixes.DASHBOARDS) {
	        	var dashboardParams = this._parseQueryString(dashboardURL);
				var item = null;
				if ((dojo.isIE < 9) && (dashboardParams.processApp == 'TWP') &&
					((dashboardParams.serviceName == 'Process+Performance') ||
					(dashboardParams.serviceName == 'Process Performance')) &&
					dashboardParams['tw.local.selectedInstanceId']) {
					// default start page is process instance view but the user is using
					// IE8 
					if (this._workDashboardItem != null) {
						item = "IE8_PROCESS_INSTANCE";
					}
				} else {
					item = this._findDashboardFromParams(dashboardParams);
				}
				if (item) {
					// If currentHash is the string "null" then this code has been reached during login and 
					//   there's no need to display any message.
					// If currentHash is defined but not the string "null", show error message indicating that the 
					//     requested URL is not available and the user's start page will be shown.  
					// If the current hash is available then supply it in the messasge.
					if (currentHash && currentHash=="null") {
						// do nothing
					} else {
						if (currentHash) {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
								dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [currentHash]));
						} else {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
						}
					}
					if (item == "IE8_PROCESS_INSTANCE") {
						this._processInstance(dashboardParams['tw.local.selectedInstanceId']);
					} else {
						// Show the dashboard!
						this._showDashboard(item,dashboardParams);
					}
				} else {
					com.ibm.bpm.social.util.common.Log.log(1, "Lookup of dashboard failed.");
					needToShowWorkTab=true;
          		}
	        } 
	        else if (this._storedStartPageSettings.savedSearchName) {
				// If the user's start page is a saved search, try to find it. If it is found then show it (_findSavedSearch() will show it).
				//  If it is not found, show the default start page.
				var errorMessageData={"showErrorMessage":true,"currentHash":currentHash?currentHash:null};
				if (!this._findSavedSearch(this._storedStartPageSettings.savedSearchName, this._storedStartPageSettings.searchFilterString, errorMessageData, true)) {
					needToShowWorkTab=true;
				}
	        }
			else if (this._storedStartPageSettings.tab === this._storedStartPageSettings.tabPrefixes.WORK) {
				// the default start page is the work page;  if it is exposed, then show message;
				//    if it is not exposed, situation will be handled below.
				if (this._workDashboardItem != null) {
					// If currentHash is the string "null" then this code has been reached during login and 
					//   there's no need to display any message.
					if (currentHash && currentHash=="null") {
					} else {
						if (currentHash) {
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"],
								dojo.string.substitute(this._nlsResources["page_url_not_valid_details"], [currentHash]));
						} else {
							// defect 127208
							// I do not know why we were showing an error here since there is no
							// hash being used and the work tab is available so I am commenting
							// out the error message for this case
							// com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_url_not_valid"]);
						}
					}
				}
				needToShowWorkTab=true; 
	        }
			else {
				// In all other cases switch to work page as default page
				needToShowWorkTab=true; 
	        }
			if (needToShowWorkTab==true) {
				// Make sure work tab is exposed
				if (this._workDashboardItem != null) {
					if (!this._storedStartPageSettings.savedSearchName) {
						// If the start page was actually the work tab, don't show any error
						if (this._storedStartPageSettings.tab != this._storedStartPageSettings.tabPrefixes.WORK) {
							// Show error message indicating that the user's start page is not available and the default start page will be shown.  
							if (dashboardURL && dashboardURL.length>0) {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_no_longer_available"],
									dojo.string.substitute(this._nlsResources["page_no_longer_available_details"], [dashboardURL]));
							} else {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["page_no_longer_available"]);
							}
						}
					}
					var state='open';
					if (this._storedStartPageSettings.filterConfigId == 'completedTasks') {
						state='close';
					}
					if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
						this._showTaskList(state,this._storedStartPageSettings.searchFilterString);
					} else {
						this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,dojo.hitch(this,function() {
							if (this.viewReadySub) {
								dojo.unsubscribe(this.viewReadySub);
								this.viewReadySub = null;
								this._showTaskList(state,this._storedStartPageSettings.searchFilterString);
							}
						}));
		   		       	com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
					}
				} else {
					com.ibm.bpm.social.util.common.Log.log(1, "Lookup of dashboard failed.");
					// Find the best possible alternative to the work page
					// Get the tab bar widget
					var tabBarWidget = dijit.byId("processPortalTabBar");
					// If there are any visible tabs, display the first in the list
					var fallbackPageFound=false;
					var itemToDisplay=null;
					if (tabBarWidget && tabBarWidget.tabList && tabBarWidget.tabList.length>0) {
						// _showDashboard is expecting an object with attributes called title and params.  So
						//     we'll make a temporary object from the tabList's params
						var itemToDisplay = {"title":tabBarWidget.tabList[0].params.title, "params":tabBarWidget.tabList[0].params};
						fallbackPageFound=true;
					} else {
						// Show first hidden tab, if there are any
						var hiddenDashboards = this.getHiddenDashboards();
						if (hiddenDashboards && hiddenDashboards.length>0) {
							var i=0;
							while( (i<hiddenDashboards.length) && !fallbackPageFound) {
								var hiddenItem=hiddenDashboards[i];
								if (hiddenItem && hiddenItem.data && hiddenItem.data.params) {
								    if (hiddenItem.data.params.runURL) {
										// dashboard or scoreboard
										itemToDisplay = this._findDashboardFromURL(hiddenItem.data.params.runURL);
										if (itemToDisplay) {
											fallbackPageFound=true;	
										}
									} else if (hiddenItem.data.params.type=="queue") {
										// saved search
										// try to find it, but don't show it yet
										var foundSavedSearch=this._findSavedSearch(hiddenItem.data.params.name, "", null, false);
										if (foundSavedSearch) {
											itemToDisplay = {"title":hiddenItem.data.params.name, "params":hiddenItem.data.params};
											fallbackPageFound=true;	
										}
									}
								}
								i++
							}
						}
					}
					// If we found a fallback!
					if (fallbackPageFound==true) {
						// Display message: Here if the work dashboard is not exposed.
						if (this._storedStartPageSettings.savedSearchName) {
							// If the page that wasn't found was a saved search:
							message = this._nlsResources["page_handler_saved_search_not_available"];
							details = this._nlsResources["page_handler_saved_search_not_available_details"];
							details = dojo.string.substitute(details, [this._storedStartPageSettings.savedSearchName]);
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
						} else {
							// Display message: Here if the page that wasn't found was a dashboard or scoreboard
							message = this._nlsResources["page_handler_page_not_available"];
							if (dashboardURL && dashboardURL.length>0) {
								details = this._nlsResources["page_handler_page_not_available_details"];
								details = dojo.string.substitute(details, [dashboardURL]);
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
							} else {
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message);
							}
						}
						if (itemToDisplay.params.type == "scoreboardOrDashboard") {
							// If the fallback page is a dashboard or scoreboard:
							this._showDashboard(itemToDisplay);
						} else if (itemToDisplay.params.type == "queue") {
							// If the fallback page is a saved search, find and show:
							this._findSavedSearch(itemToDisplay.params.name,"",null,true);
						}
					} else {
						message = this._nlsResources["page_handler_no_dashboards_available"];
						com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message);
					}
				}
			}
        }
        
		if (!window.history || !window.history.replaceState) {
			// if the current browser does not support the HTML5 history
			// API then we need to watch for hash changes
			this._hashsub = dojo.subscribe('/dojo/hashchange',this,'_hashChanged');
		}		

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
	  
	  _getExposedURLList: function _getExposedURLList_pageHandler() {
		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		// get exposed URLs
	   	var itemListWidget = new com.ibm.bpm.social.widgets.ExposedItemsList();
		com.ibm.bpm.social.portal.exposedURLs = itemListWidget.getData(exposedItemsArray,["url"]);
	  },
      
      _hashChanged: function _hashChanged_pageHandler() {
      	var h = com.ibm.bpm.social.util.common.Util.getCurrentHash();
      	if (h && (h != com.ibm.bpm.social.util.common.Util.lastHash) && 
      		(h != deocodeURIComponent(com.ibm.bpm.social.util.common.Util.lastHash))) {
      		if (this.isNavigationAllowed()) {
      			this.checkUrlHash(h);
      		} else {
      			// reset hash since navigation is being prevented
      			window.location.hash = '#'+encodeURIComponent(com.ibm.bpm.social.util.common.Util.lastHash);
      		}
      	}
      },


      /**
       * displayInstanceDetails: Used by the dashboards page for cross-site scripting purposes. Opens the
       * process details for a given instance.
       * 
       * @param bpdId
       */
      displayInstanceDetails: function displayInstanceDetails_PageHandler(bpdId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		// make sure work tab is exposed
		if (this._workDashboardItem != null) {
			this.selectWorkTab();
			if (delay) {
				delay = delay + 500;
			} else {
				delay = 500;
			}
			//setTimeout helps with iframe publish issues
			setTimeout(function(){
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
				'instanceId': bpdId
			  }]);
			}, delay);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * displayLegacyInstanceDetails: Opens the legacy process details for a given instance.
       * 
       * @param bpdId
       */
      displayLegacyInstanceDetails: function displayLegacyInstanceDetails_PageHandler(bpdId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		// make sure work tab is exposed
		if (this._workDashboardItem != null) {
			this.selectWorkTab();
			if (delay) {
				delay = delay + 500;
			} else {
				delay = 500;
			}
			//setTimeout helps with iframe publish issues
			setTimeout(function(){
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE, [{
				'instanceId': bpdId
			  }]);
			  if (this._workDashboardItem != null) {
			  	this._workDashboardItem.params["isWorkTab"]=true;
			  	dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);          
	  		  }
			}, delay);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },



      /**
       * 
       * @type {void}
       */
      showTaskCompletion: function showTaskCompletion_PageHandler(tkId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._workPageDiv.getAttribute('data-loaded') == "false" && this._workDashboardItem != null) {
          this._workPageWidget.set("href", ppConfig.portalContextRootJS + '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
        if (delay) {
        	delay = delay + 500;
        } else {
        	delay = 500;
        }
        //setTimeout helps with iframe publish issues
        setTimeout(dojo.hitch(this, function(){
        
        	// defect 124601
        	// save the hash history before selecting the work tab and then
        	// restore it afterward to prevent the work tab from moving to
        	// the top of the history
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
            "taskId": tkId
          }]);

        	var tmpHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	 		com.ibm.bpm.social.util.common.Util._portalHashHistory = new Array();
			com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
			com.ibm.bpm.social.util.common.Util._portalHashHistory = tmpHistory;
			tmpHistory = null;
			
		  if (this._workDashboardItem != null) {
			  this._workDashboardItem.params["isWorkTab"]=true;
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);          
		  }
        }, delay));
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

	  isWorkDataLoaded: function isWorkDataLoaded_PageHandler() {
		return (this._workPageDiv.getAttribute('data-loaded') == "true");
   	  },

	  loadWorkData: function loadWorkData_PageHandler() {
        if (this._workPageDiv.getAttribute('data-loaded') == "false" && this._workDashboardItem != null) {
          this._workPageWidget.set("href", ppConfig.portalContextRootJS+ '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
	  },
	
      /**
       * Open the work tab
       * 
       * @param tab
       */
      selectWorkTab: function selectWorkTab_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
		if (this._workDashboardItem != null) {
			this._workDashboardItem.params["isWorkTab"]=true;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this._workDashboardItem.params)]);
			this.loadWorkData();
			var wasRealSwitch = this.setSelectedTab('processPortalWorkTab');
			if (!wasRealSwitch) {
			  // Trigger a refresh if the user clicked again on the work tab
			  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
			}
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },

      /**
       * Open the dashboards tab
       * 
       * @param tab
       */
      selectDashboardsTab: function selectDashboardsTab_PageHandler(dashboardURL, dashboardTitle, tabId, params){
    	  
        if(typeof(dashboardURL)=='undefined' || dashboardURL==null) {
          return false;
        }

        if(typeof(tabId)=="undefined" || tabId==null){
          tabId = 'dashboardOrganizerTab';
        }
    	  
        if (this._dashboardPageDiv_Div.getAttribute('data-loaded') == "false") {
            this._dashboardPageWidget.startup();
            this._dashboardPageDiv_Div.setAttribute('data-loaded', "true");
        }
        
        var dashboardParams = null;
        if (dojo.isIE < 9) {
        	dashboardParams = this._parseQueryString(dashboardURL);
        }
        if (dashboardParams && (dashboardParams.processApp == 'TWP') &&
        	((dashboardParams.serviceName == 'Process+Performance') ||
        	(dashboardParams.serviceName == 'Process Performance')) &&
        	params['tw.local.selectedInstanceId']) { 

			var piid = parseInt(params['tw.local.selectedInstanceId']);
			this._processInstance(piid);
			return true;
        
        }
	     
	    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: dashboardURL,
          display: dashboardTitle,
          params: params
        }]);
        this.setSelectedTab(tabId); // , '/' + ppConfig.portalContextRoot + '/jsp/socialPortal/dashboard.jsp?url=' + escape(dashboardURL) + '&title=' + dashboardTitle);

        return false;
      },
      
      /**
       * Opens the Process Performance Dashboard
       * 
       * @param event (unused, but passed via <b>onclick</b>)
       */
      showProcessDashboard: function showProcessDashboard_PageHandler(event, params){
		var hash = '/dashboards/TWP/Process+Performance';
		var first = true;
		for (var p in params) {
			if (first) {
				hash = hash + '?' + p + '=' + params[p];
				first = false;
			} else {
				hash = hash + '&' + p + '=' + params[p];
			}
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[hash]);
      },

      /**
       * Opens the Team Performance Dashboard
       * 
       * @param event (unused, but passed via <b>onclick</b>)
       */
  	  showTeamDashboard: function showTeamDashboard_PageHandler(event){
		var hash = '/dashboards/TWP/Team+Performance';
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[hash]);
      },

      /**
       * handle a click on the portals primary tabs (Work/Dashboards)
       * 
       * @param id the id of the tab to switch to
       * @return {Boolean} false if previous and next page were the same, true otherwise
       */
      setSelectedTab: function setSelectedTab_PageHandler(id){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // Remove previously selected tab
        dojo.query(".processPortalTab").removeClass("processPortalSelectedTab");
		var node = dojo.byId(id);
		if (node) {
			// Select the new tab
			dojo.addClass(id, "processPortalSelectedTab");
		}
		var previousPage = dojo.query(".processPortalSelectedPage")[0];
		dojo.removeClass(previousPage, "processPortalSelectedPage");
		dojo.addClass(previousPage, "processPortalNonSelectedPage");
		targetPage=null;
		if (id=="processPortalWorkTab") {
			targetPage = dojo.byId("workPageDiv");
		} else {
			targetPage = dojo.byId(dojo.byId(id).getAttribute('data-page'));
		}
		dojo.removeClass(targetPage, "processPortalNonSelectedPage");
		dojo.addClass(targetPage, "processPortalSelectedPage");
		var realRefresh = (previousPage.id != targetPage.id);
		// Publish page switch event
		if (realRefresh) {
		  var target = targetPage.id.replace("Div", "");
		  var previous = previousPage.id.replace("Div", "");
		  dojo.publish("com.ibm.bpm.socialPortal.PageTabSwitchEvent", [{
			showing: target,
			hidding: previous
		  }]);
		}
        com.ibm.bpm.social.util.common.Log.logExit(2, "real refresh: " + realRefresh);
        // indicate that a real page switch was done 
        return (realRefresh);
      }
    });


////////////////////////////////////////
// GLOBALS
////////////////////////////////////////


com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME = "Portal Default Page";

/**
 * Singleton object representing the currently stored startPage and providing the icon
 */
com.ibm.bpm.social.portal.PageHandler.getInstance = function getInstance_PageHandler(){
  com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler", "getInstance");
  var handler;
  if (com.ibm.bpm.social.portal.PageHandler._instance) {
    handler = com.ibm.bpm.social.portal.PageHandler._instance;
  }
  else {
    handler = new com.ibm.bpm.social.portal.PageHandler();
    handler.readFromUserProfile();
    com.ibm.bpm.social.portal.PageHandler._instance = handler;
  }
  com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler", "getInstance");
  return handler;
};
