/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.PortalSidebar");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.declare(
	"com.ibm.bpm.social.portal.PortalSidebar",
	[dijit.layout._LayoutWidget, dijit._Templated],
	{
	
    templateString: "",
	CLASS_NAME: "PortalSidebar",
	LOG_LEVEL: 2,
  
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.portal", "templates/PortalSidebar.html"),
	constructor: function(){
		var FUNC_NAME = "constructor";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.tabs = null;
		this.selectedTabIndex = -1;		
		this.instantiatedDeclaratively = false;
		this.isDisplayingIcons = false;
		this.canDisplayIcons = true;
		this.showWidgetTopic = "defaultShowWidgetTopic";
		this.portalSidebarSubscriptions = [];
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},            	
	
    postCreate: function() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
				
		this.tabs = [];
		
		dijit.setWaiRole(this.tabList, "tablist");
		
		var childWidgets = dojo.query("div[tabName]", this.containerNode);
		if(childWidgets.length>0) {
			this.instantiatedDeclaratively = true;
			for(var i=0; i<childWidgets.length; i++) {
				var title = childWidgets[i].getAttribute("tabName");
				if(title=="") {
					title = "Tab "+(i+1);
				}
				this._createTab(title);
			}
		}
		
		this.portalSidebarSubscriptions.push(dojo.subscribe(this.showWidgetTopic, dojo.hitch(this, function(data) {
			window.setTimeout(dojo.hitch(this, '_fixSidebarTabWidth'), 150);
		})));
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    },
	
	addTab: function(title, widget, iconClass, iconCharacter, addSymbol) {
		var FUNC_NAME = "addTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.addChild(widget);
		dijit.setWaiRole(widget, "tabpanel");
		dojo.style(widget.domNode, "display", "none");
		var tab = this._createTab(title, widget, iconClass, iconCharacter, addSymbol);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		return tab;
	},
	
	setTabTag: function(index, tag, removeOnSelect, className) {
		if(typeof(removeOnSelect)=='undefined' || removeOnSelect==null) {
			removeOnSelect = true;
		}
		if(removeOnSelect && index==this.selectedTabIndex) {
			tag = null;
		}
		if(-1<index && index<this.tabs.length) {
			var tagEle = dojo.query(".tab-tag", this.tabs[index])[0];
			if(typeof(tagEle)!='undefined' && tagEle!=null) {
				dojo.removeClass(tagEle);
				dojo.addClass(tagEle, "tab-tag");
				if(typeof(tag)=='undefined' || tag==null || tag=="") {
					dojo.attr(tagEle, "innerHTML", "");
					dojo.style(tagEle, "display", "none");
					dojo.removeAttr(tagEle, "auto-remove");
				}
				else {
					if(typeof(className)!='undefined' && className!=null && className!="") {
						dojo.addClass(tagEle, className);
					}
					dojo.attr(tagEle, "auto-remove", ""+removeOnSelect);
					dojo.attr(tagEle, "innerHTML", tag);
					dojo.style(tagEle, "display", "");
				}
			}
		}
	},
	
	removeTabTag: function(index) {
		this.setTabTag(index, null);
	},
	
	_createTab: function(title, widget, iconClass, iconCharacter, addSymbol) {
		var FUNC_NAME = "_createTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(addSymbol) == 'undefined') {
			addSymbol = false;
		}
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Adding '"+title+"' tab to PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
		var li = dojo.create("li", {'tabIndex':"0",
			'tabnumber': this.tabs.length,
			'aria-controls':widget.id,
			'aria-selected':"false",
			'aria-label':title+". Unselected",
			'actual-title':title});
		
		// Defect 135222
		// Symbol to come before the text in rtl. I'll add it here as opposed to the resource bundle as I'm assuming the @ causes
		// an issues above in JAWS (added via 71295)
		if(addSymbol) {
            var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			if (userProfileObj.userPreferences['Locale'] == "he") { //Defect 136929 Hebrew only, not Bidi           
				title = title+' @';
			} else {            
				title = '@'+title;
            }
		}
		dijit.setWaiRole(li, "tab");
		var anchor = dojo.create("a", {'tabIndex':"-1"}, li);
		if(this.tabs.length==0){
			if(document.body.dir == 'rtl') { // if bidi
				dojo.addClass(li, "rightmost-tab");
				dojo.create("span", {'class':"tab-cap-right bpm-social-sidebar-nav-deselected-right"}, anchor);
			}
			else {
				dojo.addClass(li, "leftmost-tab");
				dojo.create("span", {'class':"tab-cap-left bpm-social-sidebar-nav-deselected-left"}, anchor);
			}
		}
		else {
			var prevTab = this.tabs[this.tabs.length-1];
			if(document.body.dir == 'rtl') { // if bidi
				dojo.removeClass(prevTab, "leftmost-tab");
				dojo.destroy(dojo.query("a .tab-cap-left", prevTab)[0]);
				dojo.create("span", {'class':"tab-divider bpm-social-sidebar-nav-divider"}, dojo.query("a", prevTab)[0]);
			}
			else {
				dojo.removeClass(prevTab, "rightmost-tab");
				dojo.destroy(dojo.query("a .tab-cap-right", prevTab)[0]);
				dojo.create("span", {'class':"tab-divider bpm-social-sidebar-nav-divider"}, dojo.query("a", prevTab)[0]);
			}
		}
		if(document.body.dir == 'rtl') { // if bidi
			dojo.addClass(li, "leftmost-tab");
			dojo.create("span", {'class':"tab-cap-left bpm-social-sidebar-nav-deselected-left"}, anchor);
		}
		else {
			dojo.addClass(li, "rightmost-tab");
			dojo.create("span", {'class':"tab-cap-right bpm-social-sidebar-nav-deselected-right"}, anchor);
		}
		if(typeof(iconClass) != "undefined") {
			if(this.isDisplayingIcons) {
				dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':""}, anchor);
				dojo.create("span", {'class':"tab-icon "+iconClass, 'style':"display: inline-block;", 'title':title}, anchor);
				dojo.create("span", {'class':"hi-contrast-icon", 'title':title, 'innerHTML': ""+iconCharacter}, anchor);
			}
			else {
				dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':title}, anchor);
				dojo.create("span", {'class':"tab-icon "+iconClass, 'style':"display: none;", 'title':title}, anchor);
				dojo.create("span", {'class':"hi-contrast-icon", 'style':"display: none;", 'title':title, 'innerHTML': ""+iconCharacter}, anchor);
			}
		}
		else {
			dojo.create("span", {'class':"tab-text bpm-social-sidebar-nav-deselected-center", 'innerHTML':title}, anchor);
			this.canDisplayIcons = false;
		}
		dojo.create("span", {'class':"tab-tag", 'style':"display: none;"}, anchor);
		com.ibm.bpm.social.util.common.Log.log(5, "Adding connectors to '"+title+"' tab of PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
		dojo.connect(anchor,'onclick',this,"_tabClickEvent");
		dojo.connect(li,'onkeypress',this,"_tabKeyPressEvent");
		if(document.body.dir == 'rtl') { // if bidi
			dojo.place(li, this.tabList, "first");
		}
		else {
			dojo.place(li, this.tabList);
		}
		
		this.tabs.push(li);


		//Round tab widths to 1 decimal point to ensure correct width calculation on chrome. Defect 124870
		var preferredWidth = (100/this.tabs.length).toFixed(1) +"%";

		com.ibm.bpm.social.util.common.Log.log(5, "Preferred width of '"+title+"' tab of PortalSidebar = "+preferredWidth, this.CLASS_NAME, FUNC_NAME);
		for(var i=0; i<this.tabs.length; i++) {
			com.ibm.bpm.social.util.common.Log.log(5, "Setting tab "+i+" of PortalSidebar to have width = "+preferredWidth, this.CLASS_NAME, FUNC_NAME);
			dojo.style(this.tabs[i], "width", preferredWidth);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		return this.tabs.length-1;
	},
	
	_tabClickEvent: function(evt) {
		var index = evt.target.parentNode.parentNode.getAttribute("tabnumber");
		this.selectTab(index);
	},
	
	_tabKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var index = evt.target.getAttribute("tabnumber");
			this.selectTab(index);
		}
	},
	
	selectTab: function(index) {
		var FUNC_NAME = "selectTab";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(-1<index && index<this.tabs.length) {
			var tag = dojo.query(".tab-tag", this.tabs[index])[0];
			if(dojo.attr(tag, "auto-remove")=="true") {
				this.removeTabTag(index);
			}
		}
		if(index!=this.selectedTabIndex) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Selecting tab "+index+" of PortalSidebar", this.CLASS_NAME, FUNC_NAME);
			var widgets = this.getChildren();
			if(-1<this.selectedTabIndex && this.selectedTabIndex<this.tabs.length){
				var tab = this.tabs[this.selectedTabIndex];
				var leftCap = dojo.query(".tab-cap-left", tab);
				if(leftCap.length > 0) {
					dojo.removeClass(leftCap[0], "bpm-social-sidebar-nav-selected-left");
					dojo.addClass(leftCap[0], "bpm-social-sidebar-nav-deselected-left");
				}
				var rightCap = dojo.query(".tab-cap-right", tab);
				if(rightCap.length > 0) {
					dojo.removeClass(rightCap[0], "bpm-social-sidebar-nav-selected-right");
					dojo.addClass(rightCap[0], "bpm-social-sidebar-nav-deselected-right");
				}
				var tabText = dojo.query(".tab-text", tab);
				if(tabText.length > 0) {
					dojo.removeClass(tabText[0], "bpm-social-sidebar-nav-selected-center");
					dojo.addClass(tabText[0], "bpm-social-sidebar-nav-deselected-center");
				}
				dojo.removeClass(tab, "selected");
				if(this.selectedTabIndex<widgets.length){
					dojo.style(widgets[this.selectedTabIndex].domNode, "display", "none");
				}
				dojo.attr(tab, 'aria-selected', 'false');
				dojo.attr(tab, 'aria-label', dojo.attr(tab, 'actual-title')+". Unselected");
			}
			if(-1<index && index<this.tabs.length){
				var tab = this.tabs[index];
				var leftCap = dojo.query(".tab-cap-left", tab);
				if(leftCap.length > 0) {
					dojo.removeClass(leftCap[0], "bpm-social-sidebar-nav-deselected-left");
					dojo.addClass(leftCap[0], "bpm-social-sidebar-nav-selected-left");
				}
				var rightCap = dojo.query(".tab-cap-right", tab);
				if(rightCap.length > 0) {
					dojo.removeClass(rightCap[0], "bpm-social-sidebar-nav-deselected-right");
					dojo.addClass(rightCap[0], "bpm-social-sidebar-nav-selected-right");
				}
				var tabText = dojo.query(".tab-text", tab);
				if(tabText.length > 0) {
					dojo.removeClass(tabText[0], "bpm-social-sidebar-nav-deselected-center");
					dojo.addClass(tabText[0], "bpm-social-sidebar-nav-selected-center");
				}
				dojo.addClass(tab, "selected");
				if(index<widgets.length) {
					dojo.style(widgets[index].domNode, "display", "block");
				}
				dojo.attr(tab, 'aria-selected', 'true');
				dojo.attr(tab, 'aria-label', dojo.attr(tab, 'actual-title')+". Selected");
			}
			this.selectedTabIndex = index;
			dojo.publish(com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED, [{}]);
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	layout: function() {},
	
	startup: function(selectedTabIndex) {
		var FUNC_NAME = "startup";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		var i;
		
		if(this.instantiatedDeclaratively) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Instantiating PortalSidebar declaratively! (It was created via HTML.)", this.CLASS_NAME, FUNC_NAME);
			var widgets = this.getChildren();
			for(i=0; i<widgets.length; i++) {
				dojo.style(widgets[i].domNode, "display", "none");
			}
		}
		if (selectedTabIndex) {
			this.selectTab(selectedTabIndex);
		} else {
			this.selectTab(0);
		}
		this._fixSidebarHeight();
		window.setTimeout(dojo.hitch(this, '_fixSidebarTabWidth'), 150);
		this.connect(window, 'onresize', '_fixSidebarHeight');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_fixSidebarHeight: function() {
        try {
    		var y = dojo.coords(this.domNode).y;
    		if (y > 0) {
    			var h = document.documentElement.clientHeight;
    			this.domNode.style.height = (h - y - 50) + 'px';
    			var yWidgets = dojo.coords(this.containerNode).y;
    			this.containerNode.style.height = Math.max(50, h - yWidgets - 40) + 'px';
                com.ibm.bpm.social.util.common.Log.log(2, "y: "+y+", h: "+h+", yWidgets: "+yWidgets, "PortalSideBar");
    		} else {
                com.ibm.bpm.social.util.common.Log.log(2, "y: "+y, "PortalSideBar");
    			//window.setTimeout(dojo.hitch(this,'_fixSidebarHeight'), 250);//defect 99422
    		}
    	} catch (error) {
    		// throwing error on log out
            com.ibm.bpm.social.util.common.Log.log(2, "caught error: "+error, "PortalSideBar");
    	}
	},
	
	_fixSidebarTabWidth: function() {
		// if the domNode height is not set, try again
		if (!this.domNode.style.height) {
			this._fixSidebarHeight();
		}
		for(var i=0; i<this.tabs.length; i++) {
			var tabTitle = dojo.query(".tab-text", this.tabs[i])[0];
			var titleText = dojo.attr(tabTitle, 'innerHTML');
			dojo.attr(tabTitle, 'innerHTML', titleText+"&nbsp;");
			dojo.style(tabTitle, 'fontWeight', 'bold');
			if(tabTitle != null) {
				if(tabTitle.scrollWidth <= 0 || tabTitle.clientWidth <= 0) {
					dojo.attr(tabTitle, 'innerHTML', titleText);
					dojo.style(tabTitle, 'fontWeight', '');
					return;
				}
				if(tabTitle.scrollWidth > tabTitle.clientWidth) {
					dojo.attr(tabTitle, 'innerHTML', titleText);
					dojo.style(tabTitle, 'fontWeight', '');
					this.displayAlternateIcons();
					break;
				}
				dojo.attr(tabTitle, 'innerHTML', titleText);
				dojo.style(tabTitle, 'fontWeight', '');
			}
		}
	},
	
	displayAlternateIcons: function() {
		var FUNC_NAME = "displayAlternateIcons";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(this.canDisplayIcons) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Displaying alternate icons for PortalSidebar tabs due to inadequate width.", this.CLASS_NAME, FUNC_NAME);
			this.isDisplayingIcons = true;
			for(var i=0; i<this.tabs.length; i++) {
				dojo.query(".tab-text", this.tabs[i])[0].innerHTML = "";
				if(dojo.hasClass(dojo.query("body")[0], "dijit_a11y")) {
					dojo.style(dojo.query(".hi-contrast-icon", this.tabs[i])[0], "display", "inline-block");
				}
				else {
					dojo.style(dojo.query(".tab-icon", this.tabs[i])[0], "display", "inline-block");
				}
			}
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
    
	destroy: function(finalize) {
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		for(var i=0; i<this.portalSidebarSubscriptions.length; i++) {
			dojo.unsubscribe(this.portalSidebarSubscriptions[i]);
		}
		this.inherited('destroy',arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
});


