/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.ProcessPortalTab");
dojo.require("dijit._Widget");

dojo.declare(
	"com.ibm.bpm.social.portal.ProcessPortalTab",
	dijit._Widget , 
	{
	
	CLASS_NAME: "ProcessPortalTab",
	
	LOG_LEVEL: 1,
	params: null,
	savedParams: null, // if this tab is used to display a snapshot, savedParams will store the original params 
	savedTextScrollWidth: null, 
	isWorkTab: false,
	workTabBaseID: null,
	workTabTitle: null,
	isProcessServer: false,
	ffscrollwidth: null,

    postCreate: function postCreate_ProcessPortalTab() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if (this.params && this.params.isWorkTab && this.params.isWorkTab==true) {
			this.isWorkTab=true;
			this.workTabBaseID=this.params.baseID;
			this.workTabTitle=this.params.title;
			this._createWorkTab();
		} else {
			this._createTab();
		}
		
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_createTab: function _createTab_ProcessPortalTab() {
		dojo.addClass(this.domNode,'processPortalTab');
		dojo.addClass(this.domNode,'dashboardTab');
		var icon = document.createElement('div');
		icon.className = this.params.iconNamePrefix+'_processPortalDashboardTabIcon';  // the icon is specific to the dashboard/scoreboard/saved-search type
		dojo.attr(icon, "id", this.params.baseID+"_iconID"); // so we can find this icon later
		this.domNode.appendChild(icon);
		var txt = document.createElement('div');
		txt.className = 'processPortalWorkTabText';
		this.domNode.appendChild(txt);
		var a = document.createElement('a');
		a.href = 'javascript:void(0);';
		txt.appendChild(a);
		this.connect(a,'onclick','_handleTabClick');
        var text2 = dojo.create("div", {
          	'class': "processPortalDashboardTabTextDiv",
        }, a);
        text2.appendChild(document.createTextNode(this.params.title));			        
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);				
        //a11y: fix no visual focus indicator by default or by css only in IE; chrome needs it also for white outline
        this.connect(a,'onfocus','_handleTabFocus');
        this.connect(a,'onblur','_handleTabBlur');
	},
	
	_createWorkTab: function _createWorkTab_ProcessPortalTab() {
		dojo.attr(this.domNode, 'id', "processPortalWorkTab");
		dojo.addClass(this.domNode,'processPortalTab');
		dojo.addClass(this.domNode,'dashboardTab');//??????????//
		dojo.attr(this.domNode,'data-page',"workPageDiv");
		dojo.attr(this.domNode,'dojoAttachPoint',"workTab");
		var icon = document.createElement('div');
		icon.className = 'processPortalWorkTabIcon';
		dojo.attr(icon, "id", this.params.baseID+"_iconID"); // so we can find this icon later
		this.domNode.appendChild(icon);
		var txt = document.createElement('div');
		txt.className = 'processPortalWorkTabText';
		this.domNode.appendChild(txt);
		var a = document.createElement('a');
		a.id  = 'processPortalWorkLink';
		a.href = 'javascript:void(0);';
		txt.appendChild(a);
		this.connect(a,'onclick','_handleTabClick');
        var text2 = dojo.create("div", {
          	'class': "processPortalDashboardTabTextDiv",
        }, a);
        text2.appendChild(document.createTextNode(this.params.title));			        
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text2);		
		this.domNode.title = this.domNode.textContent;
		if (dojo.isIE < 9)
			this.domNode.title = this.domNode.innerText;		
        this.connect(a,'onfocus','_handleTabFocus');//a11y
        this.connect(a,'onblur','_handleTabBlur');//a11y
	},
	
	getCurrentWidth: function getCurrentWidth_ProcessPortalTab() {
		var w = 13 + this.domNode.offsetWidth;
		return w;
	},

	getCurrentTextWidth: function getCurrentTextWidth_ProcessPortalTab() {
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		if (div) {
			return div.offsetWidth;
		}
	},

	getSnapshotTextWidth: function getSnapshotTextWidth_ProcessPortalTab() {
		var snapshotTextDiv = dojo.query(".processPortalDashboardTabTextDivSnapshot",this.domNode)[0];
		var offsetWidth=0;
		if (snapshotTextDiv) {
			offsetWidth = snapshotTextDiv.offsetWidth;
		}
		return offsetWidth;
	},

	setCurrentTextWidth: function setCurrentTextWidth_ProcessPortalTab(w) {
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		var snapWidth = this.getSnapshotTextWidth();
		if (div) {
			div.title = div.textContent;
			
			if (dojo.isIE < 9)
				div.title = div.innerText;
			
			if (w) {
				if (div.scrollWidth < w) {
					div.style.width = div.scrollWidth + 'px';
				} else {
					div.style.width = w + 'px';
				}
			} else {
				if (snapWidth > div.scrollWidth) {
					div.style.width = snapWidth + 'px';
				} else {
					//Fix for defect 133534, for ff on mac, need to used the scrollWidth+1 to avoid truncation
					if (dojo.isFF && navigator.userAgent.toLowerCase().indexOf("mac os")>=0) {
						if (div.style.width == "" || div.style.width == null || typeof(div.style.width) === undefined) {							
							div.style.width = (div.scrollWidth+1) + 'px';
							if (this.ffscrollwidth == null) {
								this.ffscrollwidth = div.scrollWidth;
							}
						} else {
							if (this.ffscrollwidth != null) {
								div.style.width = this.ffscrollwidth + 'px';
							} else {
								div.style.width = div.scrollWidth + 'px';
							}
						}					
					} else {
						div.style.width = div.scrollWidth + 'px';
					}
				}
			}
		}
	},

	hideIcon: function hideIcon_ProcessPortalTab() {
		var icon=null;
		icon = dojo.byId(this.params.baseID+"_iconID");
		if (icon!=null) {
			icon.style.display = 'none';
		}
	},

	showIcon: function showIcon_ProcessPortalTab() {
		var icon=null;
		icon = dojo.byId(this.params.baseID+"_iconID");
		if (icon!=null) {
			icon.style.display = 'block';
		}
	},

	_handleViewReady: function _handleViewReady_ProcessPortalTab() {
		if (this.viewReadySub) {
			dojo.unsubscribe(this.viewReadySub);
			this.viewReadySub = null;
			this._handleTabClick();
		}
	},
	
	_handleTabClick: function _handleTabClick_ProcessPortalTab(e) {
		if (e) {
			dojo.stopEvent(e);
		}
	    if (!com.ibm.bpm.social.portal.PageHandler.getInstance().isNavigationAllowed()) {
	    	return;
	    }
		if (this.params.type == 'queue') {
		   if (com.ibm.bpm.social.portal.PageHandler.getInstance().isWorkDataLoaded()) {
		    com.ibm.bpm.social.portal.PageHandler.getInstance().setSelectedTab('processPortalWorkTab');
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{
					savedSearchName: this.params.name
				}]);
		    } else {
			this.viewReadySub = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY,this,'_handleViewReady');
   		      com.ibm.bpm.social.portal.PageHandler.getInstance().loadWorkData();
		    }
		} else if (this.params.isWorkTab && this.params.isWorkTab==true) {
			com.ibm.bpm.social.portal.PageHandler.getInstance().selectWorkTab();		
		} else {
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
				runURL: this.params.runURL,
				display: this.params.title
			}]);
		}
		dojo.publish(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB, [com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB(this.params)]);
	},
	
	saveOriginalParams: function saveOriginalParams_ProcessPortalTab(params) {
		this.savedParams = params;
	},
	
	setNewParams: function setNewParams_ProcessPortalTab(newParams) {
		// Called when this tab will be used to display a snapshot; save the original tab params
		this.saveOriginalParams(this.params);
		this.params = newParams;
		// save the width of the tab text so it can be restored when the tab no longer displays the snapshot name
		var div = dojo.query('.processPortalDashboardTabTextDiv',this.domNode)[0];
		if (div) {
				this.savedTextScrollWidth = div.scrollWidth;
		}
		// if this is a snapshot, add the snapshot name under the dashboard name
		if (this.params.isSnapshot!=undefined && this.params.isSnapshot==true) {
			var textNode = dojo.query(".processPortalDashboardTabTextDiv", this.domNode);
			if (textNode[0]) {
				var subText = dojo.create("div", {
					'class': "processPortalDashboardTabTextDivSnapshot",
				}, textNode[0]);
				subText.appendChild(document.createTextNode(this.params.snapshotName));	
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(subText);	
				var parentWidth = textNode[0].scrollWidth;
				var widthToUse = Math.max(parentWidth, subText.scrollWidth);
				dojo.style(textNode[0], "width", "inherit");
				subText.style.width = 'inherit';
			}
		}
		
	},
	
	restoreOriginalParams: function restoreOriginalParams_ProcessPortalTab() {
		var textNode=null;
		if (this.savedTextScrollWidth!=null) {
			textNode = dojo.query(".processPortalDashboardTabTextDiv", this.domNode);
			if (textNode[0]) {
				textNode[0].style.width = this.savedTextScrollWidth + 'px';
			}
		}
		textNode = dojo.query(".processPortalDashboardTabTextDivSnapshot", this.domNode);
		if (textNode[0]) {
			dojo.destroy(textNode[0]);
		}
		this.params = this.savedParams;
		this.savedParams=null;
		this.savedTextScrollWidth=null;
	},

    _handleTabFocus: function _handleTabFocus_ProcessPortalTab(evt) {
         var node = evt.target.firstChild;
         if (node) { dojo.addClass(node,"focus"); }
    },

    _handleTabBlur: function _handleTabBlur_ProcessPortalTab(evt) {
         var node = evt.target.firstChild;
         if (node) { dojo.removeClass(node,"focus"); }
    },

	destroy: function destroy_ProcessPortalTab() {
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
	
});	