/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.ProcessPortalTabBar");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.portal.ProcessPortalTab");

dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");

dojo.declare(
	"com.ibm.bpm.social.portal.ProcessPortalTabBar",
	[dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "ProcessPortalTabBar",
	
	LOG_LEVEL: 1,
	
	tabList: null,
	tempTabParams: null,
	snapshotParams : null,
	snapshotTab : null,
	organizeTabsText: "",
	textZoom: 0,
	textZoomTimer: 0,
	isProcessServer: false,
	mumContextRoot: window.endpointURLObject.mumJS,
	
	
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.portal", "templates/ProcessPortalTabBar.html"),
	constructor: function constructor_ProcessPortalTabBar(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.organizeTabsText = this.messages["dashboardOrganizer-OrganizeTabs"];
	},	
    postCreate: function postCreate_ProcessPortalTabBar() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.tabList = new Array();
		this.updateDashboardTabs();
		this.processPortalTabBarSubscriptions=[];
		this.processPortalTabBarSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS,dojo.hitch(this,function(msg) {
        		this.updateDashboardTabs(msg);
        	})));
		this.processPortalTabBarSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB,dojo.hitch(this,function(msg) {
        		this.selectDashboardTab(msg);
        	})));
		this.connect(window,'onresize','_checkTabWidths');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.textZoomTimer = window.setTimeout(dojo.hitch(this,'_checkForTextZoomChange'),500);
	}, 
	
	updateDashboardTabs: function updateDashboardTabs_ProcessPortalTabBar() {
		var selectedParams = this._removeDashboardTabs();
		var pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();
		var dd = pageHandler.getDisplayedDashboards();
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		// Set variable that indicates if the environment is Process Server.  Note that if there are 0 exposed dashboards
		//  then the value of isProcessServer is both irrelevant and unreliable.
		this.isProcessServer = organizer.getIsProcessServer();
		if (dd) {
			for (var i=0; i<dd.length; i++) {
				var tab = new com.ibm.bpm.social.portal.ProcessPortalTab({params: dd[i].data.params, isProcessServer:this.isProcessServer});
				this.organizerTab.parentNode.insertBefore(tab.domNode,this.organizerTab);
				this.tabList.push(tab);
				// Was this tab showing a snapshot?  If so, show it again
				var snapWasShowing=false;
				if (this.snapshotTab != null && this.snapshotTab.savedParams!=null && this.snapshotTab.savedParams.baseID == tab.params.baseID) {
					snapWasShowing=true;
				}
				if (selectedParams && ((selectedParams.baseID == tab.params.baseID) || 
				   (snapWasShowing && selectedParams.baseID == this.snapshotTab.params.baseID))) {
					dojo.addClass(tab.domNode,"processPortalSelectedTab");
					// tell organizer what the selected tab is
					organizer.setCurrentlySelectedTab(tab.params.baseID);
					if (snapWasShowing && selectedParams.baseID == this.snapshotTab.params.baseID) {
						tab.setNewParams(this.snapshotTab.params);
					}
					selectedParams = null;
				}
				// two cases for removing temp tab
				if (this.tempTabParams) {
					if (this.tempTabParams.baseID == dd[i].data.params.baseID) {
						this._removeTemporaryTab();
						dojo.addClass(tab.domNode,"processPortalSelectedTab");
					}  else if (this.tempTabParams.itemID == dd[i].data.params.itemID) {
						var dashItem=organizer.getSpecificDashboard(this.tempTabParams.baseID);
						if (dashItem!=null) {
							tab.setNewParams(dashItem.data.params);
							this.snapshotParams = dashItem.data.params;
							this.snapshotTab = tab;
							this._removeTemporaryTab();
							dojo.addClass(tab.domNode,"processPortalSelectedTab");
						}
					}
				}
				
				
			}
		}
		if (selectedParams) {
			this._createTemporaryTab(selectedParams);
			dojo.addClass(this.organizerTab,"processPortalSelectedTab");
		}
		// if the organizer tab is the only tab, change its class so it has a bigger margin on left (or right in rtl)
		if (!dd || dd&&dd.length==0) {
			if (dojo.hasClass(this.organizerTab,"processPortalTab")){
				dojo.removeClass(this.organizerTab,"processPortalTab");
				dojo.addClass(this.organizerTab,"processPortalOrganizerTab");
			}
		} else {
			if (dojo.hasClass(this.organizerTab,"processPortalOrganizerTab")){
				dojo.removeClass(this.organizerTab,"processPortalOrganizerTab");
				dojo.addClass(this.organizerTab,"processPortalTab");
			}
		}
		this._checkTabWidths();
	},
	
	_checkForTextZoomChange: function _checkForTextZoomChange_ProcessPortalTabBar() {
		if (this.textZoomTimer) {
			var userName = dojo.byId('processPortalUserName');
			if (userName.offsetHeight != this.textZoom) {
				this._checkTabWidths();
				this.textZoom = userName.offsetHeight;
			}
			this.textZoomTimer = window.setTimeout(dojo.hitch(this,'_checkForTextZoomChange'),500);
		}
	},
	
	_checkTabWidths: function _checkTabWidths_ProcessPortalTabBar() {
		var availableWidth = this.domNode.offsetWidth - 34;
	    var organizerDropSpace = 175 - this.organizerTab.offsetWidth;
		var userDiv = dojo.byId('processPortalUser');
		if (userDiv) {
			availableWidth = availableWidth - userDiv.offsetWidth;
			organizerDropSpace = organizerDropSpace - userDiv.offsetWidth;
			var userName = dojo.byId('processPortalUserName');
			if (userName.offsetWidth == 0) {
				window.setTimeout(dojo.hitch(this,'_checkTabWidths'),250);
			}
		}
		if (organizerDropSpace > 0) {
			availableWidth = availableWidth - organizerDropSpace;
		}
		if (availableWidth > 0) {
			var w = 0;
			for (var i=0; i<this.tabList.length; i++) {
				this.tabList[i].showIcon();
				this.tabList[i].domNode.style.display = 'block';
				this.tabList[i].setCurrentTextWidth();
				w = w + this.tabList[i].getCurrentWidth();
			}
			if (dojo.isFF && navigator.userAgent.toLowerCase().indexOf("mac os")>=0) {
			    this.organizerTabTextDiv.style.width = (this.organizerTabTextDiv.scrollWidth+1) + 'px';
			} else {
				this.organizerTabTextDiv.style.width = this.organizerTabTextDiv.scrollWidth + 'px';
			}		
			w = w + 13 + this.organizerTab.offsetWidth;
			if (w > availableWidth) {
				w = 0;
				var maxTxtWidth = 0;
				for (var i=0; i<this.tabList.length; i++) {
					this.tabList[i].hideIcon();
					var txtWidth = this.tabList[i].getCurrentTextWidth();
					if (txtWidth > maxTxtWidth) {
						maxTxtWidth = txtWidth;
					}
					w = w + this.tabList[i].getCurrentWidth();
				}
				if (this.organizerTabTextDiv.offsetWidth > maxTxtWidth) {
					maxTxtWidth = this.organizerTabTextDiv.offsetWidth;
				}
				w = w + 13 + this.organizerTab.offsetWidth;
				while ((w > availableWidth) && (maxTxtWidth > 25)) {
					maxTxtWidth = maxTxtWidth - 5;
					w = 0;
					for (var i=0; i<this.tabList.length; i++) {
						this.tabList[i].setCurrentTextWidth(maxTxtWidth);
						w = w + this.tabList[i].getCurrentWidth();
					}
					if (this.organizerTabTextDiv.offsetWidth > maxTxtWidth) {
						this.organizerTabTextDiv.style.width = maxTxtWidth + 'px';
					}
					w = w + 13 + this.organizerTab.offsetWidth;
				}
				if (w > availableWidth) {
					var i=this.tabList.length; 
					while((i > 0) && (w > availableWidth)) {
						i--;
						var tab = this.tabList[i];
						if (!dojo.hasClass(tab.domNode,'processPortalSelectedTab')) {
							w = w - tab.getCurrentWidth();
							tab.domNode.style.display = 'none';
						}
					}
				}
			}
			// is there a snapshot version displayed?
			var snapshotTextDiv = dojo.query(".processPortalDashboardTabTextDivSnapshot");
			if (snapshotTextDiv && snapshotTextDiv.length>0) {
				var offsetWidth = snapshotTextDiv[0].offsetWidth;
				var pNode = snapshotTextDiv[0].parentNode;
				if (pNode && offsetWidth > pNode.offsetWidth) {
					pNode.style.width = offsetWidth + 'px';
				}
				dojo.style(snapshotTextDiv, "width", "inherit");
			}
		} else {
			// no width, so hide all tabs
			for (var i=0; i<this.tabList.length; i++) {
				this.tabList[i].domNode.style.display = 'none';
			}
		}
		// always hide the drop down when changing the tabs
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		if (organizer.dropdownOpen) {
			organizer.hide(true);
		}
	},
	
	_removeDashboardTabs: function removeDashboardTabs_ProcessPortalTabBar() {
		var selectedParams = null;
		while (this.tabList.length > 0) {
			var tab = this.tabList.pop();
			if (dojo.hasClass(tab.domNode,"processPortalSelectedTab")) {
				selectedParams = tab.params;
			}
			tab.destroy();
		}
		return selectedParams;
	},
	
	_createTemporaryTab: function createTemporaryTab_ProcessPortalTabBar(params) {
		// See Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB() for the structure of the params argument
		this._removeTemporaryTab();
		this.tempTabParams = params;
		this.organizerTabTextDiv.innerHTML = ""; 
		dojo.style(this.organizerTabTextDiv, "fontWeight", "bold");
		if (params.title) {
			this.organizerTabTextDiv.appendChild(document.createTextNode(params.title));			
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.organizerTabTextDiv);	
			var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
			if (orgDiv) {
				dojo.removeClass(orgDiv,"hasSnapshot");
			}
			dojo.removeClass(this.organizerTab,"hasSnapshot");
			if (params.isSnapshot!=undefined && params.isSnapshot==true) {
				var subText = dojo.create("div", {
					'class': "processPortalDashboardTabTextDivSnapshot",
				}, this.organizerTabTextDiv);
				var snapText = document.createTextNode(params.snapshotName);
				subText.appendChild(snapText);	
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(subText);
				if (orgDiv) {
					dojo.addClass(orgDiv,"hasSnapshot");
				}
			}
			this.organizerTabTextDiv.title = this.organizerTabTextDiv.textContent;
			if (dojo.isIE < 9)
				this.organizerTabTextDiv.title = this.organizerTabTextDiv.innerText;
		}
		
		this.organizerTabText.style.display = 'inline';
		dojo.addClass(this.organizerTab,"processPortalTemporaryTab");
		// tell the organizer the id of the selected tab
		var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
		// tell the organizer the id of the selected tab; 
		organizer.setCurrentlySelectedTabByItemID(params.itemID);
	},

	_removeTemporaryTab: function _removeTemporaryTab_ProcessPortalTabBar() {
		dojo.removeClass(this.organizerTab,"processPortalTemporaryTab");
		dojo.removeClass(this.organizerTab,"processPortalSelectedTab");
		var orgDiv=dojo.byId("processPortalDashboardOrganizerId");
		if (orgDiv) {
			dojo.removeClass(orgDiv,"hasSnapshot");
		}
		this.organizerTabText.style.display = 'none';
		this.organizerTabTextDiv.innerHTML = "";
		this.organizerTabTextDiv.style.width = "0";
		this.tempTabParams = null;
	},

	_resetSnapshotTab: function _resetSnapshotTab_ProcessPortalTabBar(baseID) {
		// find the tab to reset
		var tab=null;
		for (var i=0; i<this.tabList.length; i++) {
			if (this.tabList[i].params.baseID == baseID) {
				tab = this.tabList[i];
			}
		}		
		if (tab != null) {
			tab.restoreOriginalParams();
			this.snapshotParams = null;
			this.snapshotTab = null;
		}
	},
	
	selectDashboardTab: function selectDashboardTab_ProcessPortalTabBar(params) {
		// This is called when the topic com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB is published.
		// The structure of the params argument is defined in Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB()
		if (this.tempTabParams) {
			if ((params == undefined) || (this.tempTabParams.baseID != params.baseID)) {
				this._removeTemporaryTab();
			}
		}
		// If a tab is currently showing a snapshot, and the selected tab is not that tab, then reset the tab to its original state
		if (this.snapshotParams) {
			if ((params == undefined) || (this.snapshotParams.baseID != params.baseID)) {
				this._resetSnapshotTab(this.snapshotParams.baseID);
			}
		}
		dojo.query(".processPortalSelectedTab",this.domNode).forEach(function(node) {
			dojo.removeClass(node,"processPortalSelectedTab");
		});
		if (params) {
			var organizer = com.ibm.bpm.social.portal.PageHandler.getInstance().getDashboardOrganizer();
			var tab = null;
			for (var i=0; i<this.tabList.length; i++) {
				if (params.baseID && this.tabList[i].params.baseID == params.baseID) {
					tab = this.tabList[i];
				}
			}
			if (tab == null) {
				// See if a different version of this item is already shown in a tab; if so, display it there instead
				//   of creating a temp tab
				var matchIndex=-1;
				for (var i=0; i<this.tabList.length; i++) {
					if (this.tabList[i].params.itemID == params.itemID) {
						matchIndex=i;
						break;
					}
				}
				if (matchIndex>-1) {
					var dashItem=organizer.getSpecificDashboard(params.baseID);
					if (dashItem!=null) {
						tab = this.tabList[matchIndex];
						tab.setNewParams(dashItem.data.params);
						this.snapshotParams = dashItem.data.params;
						this.snapshotTab = tab;
					}
				} else {
					this._createTemporaryTab(params);
					dojo.addClass(this.organizerTab,"processPortalSelectedTab");
				}
			}

			if (tab!=null) {
				// tell the organizer the id of the selected tab; if tab is currently showing a snapshot, use the baseID of the original tab
				var baseID = tab.savedParams!=null ? tab.savedParams.baseID : tab.params.baseID;
				organizer.setCurrentlySelectedTab(baseID);
				dojo.addClass(tab.domNode,"processPortalSelectedTab");
			}
		}
		this._checkTabWidths();
	},
	
	destroy: function destroy_ProcessPortalTabBar() {
		/**
		 * This is also called from theme.html::customLogoutFunction
		 */
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.textZoomTimer = 0;
		if (this.processPortalTabBarSubscriptions && this.processPortalTabBarSubscriptions.length) {
			for(var i=0; i<this.processPortalTabBarSubscriptions.length; i++) {
				dojo.unsubscribe(this.processPortalTabBarSubscriptions[i]);
			}
			this.processPortalTabBarSubscriptions=[];
		}
		this._removeDashboardTabs();
		this._removeTemporaryTab();
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});