/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.UserDropdown");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");

dojo.require("com.ibm.bpm.social.widgets.dialogs.UserProfileDialog");
dojo.require("com.ibm.bpm.social.portal.DropdownMenu");

dojo.declare(
	"com.ibm.bpm.social.portal.UserDropdown",
	[dijit._Widget, dijit._Templated],
	{
		templateString: "<div class=\"user-dropdown processPortalDropdownMenu\"/>",
		dropdownMenu: null,
		dropdownOpen: false,
		blurBlock: true,
		userMenu: null,
		connects: null,
		
		postCreate: function() {
			//Sort out the localization
			dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
			this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
			com.ibm.bpm.social.portal.UserDropdown.id = this.domNode.id;
			
			//Create the menu
			this.userMenu = this._createMenu();
			/*TODO: Create the proper dom setup for this to work*/
			var node = dojo.byId("processPortalUserDropdownId");
			this.dropdownMenu = new com.ibm.bpm.social.portal.DropdownMenu(
				{dropDown: this.userMenu,
				 dropDownContainer: this.domNode,
				 _buttonNode: node,
				 _popupStateNode: node,
				 _aroundNode: node});
			this.dropdownMenu.startup();
			this.dropdownMenu.loadDropDown();
			this.userProfileDialog = null;
			
			//Setup Event Handling  We want the menu to close everytime focus is lost, but
			//the blur event gets processed before a click to the username link which opens
			//the menu. So in order to achieve the clicking behavior desired by design, a
			//state must be managed determining whether the mouse is over the open link 
			//when the blur event is fired.
			var self = this;
			
			if(!this.connects) {
				this.connects = new Array();
			}

			this.connects.push(dojo.query("#processPortalUserDropdownId > *").connect('mouseover',
			function(event) {
				self._blockBlur(true, self);
				}));
			
			this.connects.push(dojo.query("#processPortalUserDropdownId > *").connect('mouseout',
			function(event) {
				self._blockBlur(false, self);
				}));
			
			this.connects.push(dojo.connect(this.userMenu, 'onBlur', function(event) { 
				self._blurActions(self);
			}));
		},
		
		adjustMenuStyle: function() {
			var node = dojo.byId("processPortalUserDropdownId_dropdown");
			if(node) {
				dojo.addClass(node, "processPortalDropDownMenu");
			}
			
			dojo.style(this.userMenu.domNode, "padding-left", "15px");
			dojo.style(this.userMenu.domNode, "padding-right", "15px");
			
			var items = this.userMenu.getChildren();
			if(items) {
				dojo.forEach(items, function(item, index) {
					item.iconNode.style.display = "none";
				});
			}
		},
		
		_createMenu: function() {
			var menu = new dijit.Menu();
			var self = this;
			
			var userProfile = new  dijit.MenuItem({
				label: this.messages.userMenuProfile,
				onClick: function(event) {
					self._showUserProfile();
					self.dropdownOpen = false;
				},
				onKeyDown: this._menuKeyDown
			});
			menu.addChild(userProfile);
			
			var logout = new  dijit.MenuItem({
				label: this.messages.userMenuLogout,
				onClick: dojo.hitch(this,this._logoutUser),
				onKeyDown: this._menuKeyDown
			});
			menu.addChild(logout);
			
			return menu;
		},
	
		_menuKeyDown: function(evt) {
			var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
			if (keypressed == dojo.keys.ENTER || keypressed == dojo.keys.SPACE) {
				var target = evt.target;
				if(dojo.isIE) {
					target = evt.target.childNodes[1];
				} else {
					target = evt.target.childNodes[3];
				}
				
				var action = target.innerHTML;
				
				if (action == this.messages.userMenuProfile) {
					this._showUserProfile();
				} else if (action == this.messages.userMenuLogout) {
					this._logoutUser();
				} else {
					//shouldn't hit, something is messed up if it does.
				}
			}
		},
		
		_logoutUser: function() {
			try {				
				if (typeof disconnectFromBayeuxServer === "function") {
				  disconnectFromBayeuxServer();
				}
			} finally {
				// Otherwise the cometd calls are lost because the authorization is removed before
				// the cometd calls above are triggered.
				setTimeout(com.ibm.bpm.logout,500);				
			}			
		},
		
		toggle: function() {
			if(this.dropdownOpen) {
				this.hide();
			} else {
				this.show();
			}
		},
		
		show: function() {
			if (this.dropdownMenu) {
				this.dropdownMenu.loadDropDown();
				this.dropdownOpen = true;
			}
			this.adjustMenuStyle();
		},
		
		hide: function() {
			if (this.dropdownMenu) {
				this.dropdownMenu.closeDropDown(false);
				this.dropdownOpen = false;
			}
		},
		
		destroy: function() {
			if (this.dropdownMenu) {
				if(this.dropdownMenu.destroyRecursive) {
					this.dropdownMenu.destroyRecursive();
				} else if (this.dropdownMenu.destroy) {
					this.dropdownMenu.destroy();
				}
			}
			if(this.userProfileDialog) {
				this.userProfileDialog.destroy();
			}

			if(this.connects) {
				for(var i=0; i < this.connects.length; i++) {
					dojo.disconnect(this.connects[i]);
					this.connects[i] = null;
				}
				this.connects = null;
			}
			this.inherited(arguments);
		},
		
		_showUserProfile: function() {
			if (!this.userProfileDialog) {
				this.userProfileDialog = new com.ibm.bpm.social.widgets.dialogs.UserProfileDialog();
			}
			this.userProfileDialog.show();
		},
		
		_blockBlur: function(value, self) {
			self.blurBlock = value;
		},
		
		_blurActions: function(self) {
			if(self.dropdownOpen && !self.blurBlock) {
				self.toggle();
			}
		}
	}
);
		
			
			
		
