/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.WorkViewContainer");

dojo.require("dojo.fx");

dojo.require("dojo.number");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.portal.views.TaskListView");

dojo.require("com.ibm.bpm.social.portal.views.TaskCompletionView");

dojo.require("com.ibm.bpm.social.portal.views.InstanceDetailsView");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.portal.PageHandler");

dojo.declare(
	"com.ibm.bpm.social.portal.WorkViewContainer",
	[dijit._Widget , dijit._Templated],
	{

	templateString: "",
	
	CLASS_NAME: "WorkViewContainer",
	
	LOG_LEVEL: 1,

	templatePath: dojo.moduleUrl("com.ibm.bpm.social.portal", "templates/WorkViewContainer.html"),

    view_List: null,

    view_Completion: null,
    
    view_Instance: null,

    currentView: null,

    queuedViews: [],

    selectViewActive: false,

    _cometdConnectCounter: 0,

    _cometdConnectMax: 10,

    _userProfileChangeSubscribeHandle: null,

	_ignorePageTab: false,
	
    postCreate: function() {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Loading stylesheet...", this.CLASS_NAME, FUNC_NAME);
		this._loadStylesheet();

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Registering listeners...", this.CLASS_NAME, FUNC_NAME);
		this._registerListeners();

		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Registering work page listeners...", this.CLASS_NAME, FUNC_NAME);
		this.__registerWorkPageListener();

		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	__registerWorkPageListener: function(){
		var FUNC_NAME = "_registerWorkpageListener";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

		dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", dojo.hitch( this, function _onPageTabSwitchEvent(args){
			if(args!=null && args.showing=="dashboardPage"){
				dojo.addClass("processPortalContent", "dashboards");
				dojo.removeClass(dojo.body(), "processPortalWork");

				if(this.currentView){
					com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL,"Calling currentView.unload due to PageTabSwitchEvent", this.CLASS_NAME, FUNC_NAME);
					this.currentView.unload();
				}

			} else {
				dojo.removeClass("processPortalContent", "dashboards");
				dojo.addClass(dojo.body(), "processPortalWork");

				if(this.currentView){
					if ((this.currentView instanceof com.ibm.bpm.social.portal.views.TaskCompletionView)
						&& !this.currentView.taskId && !this.taskTemplateId) {
						// if there is no task, it is because the last view was for
						// a task that was completed.  switch to the task list
						this._loadTaskListView({});
					} else {
					    if (!this._ignorePageTab) {
							var obj = {
								taskId: this.currentView.taskId,
								taskTemplateId: this.currentView.taskTemplateId,
								instanceId: this.currentView.instanceId
							};
							com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Calling currentView.load due to PageTabSwitchEvent", this.CLASS_NAME, FUNC_NAME);
							this.currentView.load(obj);
						} 
					}
				}
			}
			
			this._ignorePageTab = false; //reset ignorePageTab no matter what happens
		}));

		if(dojo.hasClass(dojo.byId("workPageDiv"), "processPortalSelectedPage")){
			dojo.addClass(dojo.body(), "processPortalWork");
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_registerListeners: function(){

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, dojo.hitch(this, this._loadTaskCompletionView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, dojo.hitch(this, this._loadProcessInstanceView));
		
		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE, dojo.hitch(this, this._loadLegacyProcessInstanceView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, dojo.hitch(this, this._loadTaskListView));

		dojo.subscribe(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM, dojo.hitch(this, this._openProcessDiagram));

		if (!(dojo.isSafari && navigator.userAgent.match(/iPad/i))) { // If not Safari on iPad allow CometD
			this._registerCometdListeners();
		}
	},

	_registerCometdListeners: function(){
		this._cometdConnectCounter++;
		if(this._cometdConnectCounter >= this._cometdConnectMax){
			return;
		}

		if(! (window.connectedToBayeuxServer==true && window.connectionEstablished==true) ){
			setTimeout(dojo.hitch(this, this._registerCometdListeners), 500);
		}

		if(this._userProfileChangeSubscribeHandle == null){
			var channel = "/" + com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED.replace(/\./g, "/");
			this._userProfileChangeSubscribeHandle = dojox.cometd.subscribe( channel, dojo.hitch(this, this._handleUserProfileChange) );
		}
	},
	
	_loadProcessInstanceView: function(obj){
		var FUNC_NAME = "_loadProcessInstanceView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.instanceId)=="undefined"){
			return;
		}
		
		com.ibm.bpm.social.portal.PageHandler.getInstance().showProcessDashboard(null, { "tw.local.selectedInstanceId": "" + obj.instanceId });
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	_loadLegacyProcessInstanceView: function(obj){
		var FUNC_NAME = "_loadLegacyProcessInstanceView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.instanceId)=="undefined"){
			return;
		}
		if (!this.view_Instance) {
          this.view_Instance = new com.ibm.bpm.social.portal.views.InstanceDetailsView({
		    'top': this.domNode.offsetTop,
		    'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		this.selectView(this.view_Instance, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	

	_loadTaskCompletionView: function(obj){
		this._ignorePageTab = true;
		var FUNC_NAME = "_loadTaskCompletionView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(typeof(obj.taskId)=="undefined" && typeof(obj.taskTemplateId)=="undefined"){
			return;
		}

		//Close any open "Process Diagram" dialogs.  This is important for ad-hoc events that do not use dialogs
		dojo.query(".bpm-social-popup-participants").forEach(function(dialog){
			try{
				dijit.byNode(dialog).delayedDestroy();
			} catch (e){

			}
		});

 		if (!this.view_Completion) {
          this.view_Completion = new com.ibm.bpm.social.portal.views.TaskCompletionView({
            'top': this.domNode.offsetTop,
            'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		
		this.selectView(this.view_Completion, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_loadTaskListView: function(obj){
		var FUNC_NAME = "loadTaskListView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		if (!this.view_List) {
          this.view_List = new com.ibm.bpm.social.portal.views.TaskListView({
		    'top': this.domNode.offsetTop,
		    'left': this.domNode.offsetLeft
          }, dojo.create("div", {}, this.container));
        }
		
		this.selectView(this.view_List, obj);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_handleUserProfileChange: function(obj){
		if(obj.data && obj.data.username){
			com.ibm.bpm.social.util.common.Util.getUserAvatarURL(obj.data.username);
		}
	},

	_openProcessDiagram: function(obj){
		//console.warn("WorkViewContainer::_openProcessDiagram entering with obj:",obj);
		if(obj.instanceId && obj.criticalPathFlowIds){
			var instData = com.ibm.bpm.social.util.common.Util.getInstanceData({instanceId:obj.instanceId, header: true});

			var flows = obj.criticalPathFlowIds;
			var parsedFlows = [];
			var bit = null;

			for(var x = 0; x < flows.length; x++){
				if(flows[x]!=null){
					bit = flows[x].replace(/_colon_/g, ":").replace(/_hyphen_/g, "-");
					parsedFlows.push(bit);
				}
			}

			dojo.require("com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog");
			var pdd = new com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog({
				"instanceId": obj.instanceId,
				"title": instData.processTemplateName,
				"subTitle": instData.name,
				"criticalPathFlowIds": parsedFlows
			}, dojo.create("div"));
			pdd.show();
		}
	},

	_loadStylesheet: function(){

	},

	_loadSpecifiedStylesheet: function(path){
		if(path.indexOf("/")==0){
			if(dojo.query('link[href="'+path+'"]').length == 0){
	            var linkElement = document.createElement("link");
	            linkElement.rel = "stylesheet";
	            linkElement.type = "text/css";
            	linkElement.href = path;
	            var headElement= document.getElementsByTagName("head")[0];
	            headElement.appendChild(linkElement);
			}
		} else {
			if(dojo.query('link[href$="'+path+'"]').length == 0){
	            var linkElement = document.createElement("link");
	            linkElement.rel = "stylesheet";
	            linkElement.type = "text/css";
	            linkElement.href = dojo.moduleUrl("com.ibm.bpm.social", path);
	            var headElement= document.getElementsByTagName("head")[0];
	            headElement.appendChild(linkElement);
			}
		}
	},

	selectView: function(view, obj){
		var FUNC_NAME = "selectView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Switching to view: "+view, this.CLASS_NAME, FUNC_NAME);
		if(this.selectViewActive){
			this.queuedViews[0] = {'view': view, 'obj': obj}; // change to .push for infinite depth
			return;
		}

		this.selectViewActive = true;

		//Register callback for displaying view when ready
		var handle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.VIEW_READY, dojo.hitch(this, function(){

			if(this.currentView!=null){
				if(this.currentView.unload){
					this.currentView.unload();
				}
			/*
				dojo.addClass(this.currentView.domNode, 'workViewFadeOut');

				dojo.removeClass(this.currentView.domNode, 'workViewSelected');

				dojo.addClass(view.domNode, 'workViewFadeIn');

				var fInNode = view.domNode;
				var fOutNode = this.currentView.domNode;

				dojo.fadeOut({
					node: fOutNode,
					onEnd: dojo.hitch(this, function(){

						dojo.fadeIn({
                                        		node: fInNode,
                                        		onEnd: dojo.hitch(this, function(){

                                                		dojo.removeClass(fOutNode, 'workViewFadeOut');

                                                		dojo.addClass(fInNode, 'workViewSelected');

                                                		dojo.removeClass(fInNode, 'workViewFadeIn');
                                        		})
						}).play();
					})
				}).play();
			*/
				dojo.removeClass(this.currentView.domNode, 'workViewSelected');

				dojo.addClass(view.domNode, 'workViewSelected');

			} else {
				dojo.addClass(view.domNode, 'workViewSelected');
			}

			this.currentView = view;

			dojo.unsubscribe(handle);

			this.selectViewActive = false;

			if(this.queuedViews.length > 0){
				var local = this.queuedViews.pop();
				this.selectView(local.view, local.obj);
			}

		}));

		//Call view initialization
		try {
		  view.load(obj);
		} catch(e){
		  // in case the load has an exception, should not happen but if it does one is stuck in a view.
		  // TODO: one could show an error message. So far it only makes sure the user can still work.
		  com.ibm.bpm.social.util.common.Log.log(2, e,this.CLASS_NAME, FUNC_NAME);
		  this.currentView = view;
		  dojo.unsubscribe(handle);
		  this.selectViewActive = false;
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	destroy: function() {
		/**
		 * This is also called from theme.html::customLogoutFunction
		 */
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this._destroyView(this.view_List);
		this._destroyView(this.view_Completion);
		this._destroyView(this.view_Instance);
		this.inherited(arguments);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},

	_destroyView: function(view){
		var FUNC_NAME = "destroyView";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		if(view && view.destroy){
			view.destroy();
			view = null;
		}
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}

});