/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.InstanceDetailsView");

dojo.require("dijit._Templated");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bpm.social.widgets.details.InstanceDetails");

dojo.require("com.ibm.bpm.social.widgets.stream.Stream");

dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");

dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");


dojo.declare(
	"com.ibm.bpm.social.portal.views.InstanceDetailsView",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: dojo.cache("com.ibm.bpm.social.portal.views", "templates/InstanceDetailsView.html"),
	
	CLASS_NAME: "InstanceDetailsView",
	
	LOG_LEVEL: 1,
    
    active: false,
    
    sidebarWidth: 0,
    
    instanceId: 0,
    
    postCreate: function(arguments) {
		var FUNC_NAME = "postCreate";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		dojo.create("div", {
			'id': 'details_content_widget_target',
			'innerHTML': 'Details Container - Please clobber me with a widget!'
		}, this.detailsContent);

		dojo.create("div", {
			'id': 'details_stream_widget_target',
			'innerHTML': 'Stream Container - Please clobber me with a widget!'
		}, this.streamContent);
			
		this.initializeContentWidgets();
	},

	initializeContentWidgets: function(){
		var FUNC_NAME = "initializeContentWidgets";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		//*** Content teams place widget initialization here ***//
		this.details = new com.ibm.bpm.social.widgets.details.InstanceDetails({}, 'details_content_widget_target');
		
		this.stream = new com.ibm.bpm.social.widgets.stream.Stream({}, 'details_stream_widget_target');
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	load: function(arguments){
		var FUNC_NAME = "load";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		//*** Called from the WorkViewContainer every time immediately before the view is displayed ***//

		if(! (arguments.instanceId) ){
			return;
		}
		
		this.active = true;
		this.instanceId = arguments.instanceId;
		
		// disconnect from instance updates
		if (this.instanceUpdateHandle) {
			dojo.unsubscribe(this.instanceUpdateHandle);
			this.instanceUpdateHandle = null;
		}

		//TODO Need some way to invoke stream via InstanceId
		var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
			'header': true, 
			'instanceId': this.instanceId,
			'skipCache': true
		});
		
		if (typeof instanceData.errorInfo === "object") {
			//
			// Error occured
			//
			var errorInfo = instanceData.errorInfo;
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error occurred gettting instanceData", this.CLASS_NAME, FUNC_NAME);
			this._showErrorMessage(errorInfo);
		}else{
	  
			//TODO get the latest task ID in the list and use. only temporary until Stream service is instance-based
			var task = null;
			if(instanceData.tasks.length>0){
				task = instanceData.tasks[instanceData.tasks.length-1];
				com.ibm.bpm.social.util.common.Util.currentTaskData = task;
			}
			var tid = -1;
			if(task!=null) {
				tid = dojo.number.parse(task.tkiid);
			}
			
			
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Refreshing details...", this.CLASS_NAME, FUNC_NAME);
			this.details.refresh({
				instanceId: this.instanceId
			});
			
			
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Refreshing stream...", this.CLASS_NAME, FUNC_NAME);
			this.stream.refresh({
				taskid:tid,
			    instanceid:this.instanceId
			});

			// connect to instance updates
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Subscribing to instance due date updates...", this.CLASS_NAME, FUNC_NAME);
			this.instanceUpdateHandle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED + "." + this.instanceId, dojo.hitch(this,
				function(tkiid) {
					if(this.active) {
						this.details.refresh({
							instanceId: this.instanceId
						});
						this.stream.refresh({
							taskid: tkiid,
							instanceid: this.instanceId
						});
					}
				}, tid));
		
			var newHash = 'dashboards/TWP/Process+Performance?tw.local.selectedInstanceId='+this.instanceId;
			com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
		}
		
		dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{"viewType":"InstanceDetailsView"}]);
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	 _showErrorMessage: function(errorInfo){
		var errorMessage = errorInfo.data.message;
		var statusCode = errorInfo.ioArgs.xhr.status;
		var errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
		//var statusCode = errorInfo.ioArgs.xhr.status;
		if (statusCode == 401) {
		  // 401 Unauthorized
		  errorHandler.showWarningDialog("KEY_REST_UNAUTHORIZED", {
			1: "UNKOWN"
		  }, function(){
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
		  });
		}else{
			com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(errorMessage, null /* details */, function(){
				dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
			});
		}
		com.ibm.bpm.social.util.common.Log.logError(this.LOG_LEVEL, ""+this.CLASS_NAME+" experienced an error: "+errorMessage);
	  },
	
	unload: function(arguments){
		var FUNC_NAME = "unload";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		this.active = false;
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	},
	
	destroy: function(){
		var FUNC_NAME = "destroy";
		com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
		
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Destroying details...", this.CLASS_NAME, FUNC_NAME);
		if(this.details && this.details.destroy){
			this.details.destroy();
			this.details = null;
		}
		// disconnect from instance updates
		if (this.instanceUpdateHandle) {
			dojo.unsubscribe(this.instanceUpdateHandle);
			this.instanceUpdateHandle = null;
		}
		
		com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Destroying stream...", this.CLASS_NAME, FUNC_NAME);
		if(this.stream && this.stream.destroy){
			this.stream.destroy();
			this.stream = null;
		}
		
		this.inherited(arguments);
		
		com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	}
});