/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.TaskCompletionView");

dojo.require("dijit._Templated");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.portal.PortalSidebar");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.require("com.ibm.bpm.social.widgets.details.InstanceDetails");

dojo.require("com.ibm.bpm.social.widgets.stream.Stream");

dojo.require("com.ibm.bpm.social.widgets.experts.Experts");

dojo.require("com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget");

dojo.declare("com.ibm.bpm.social.portal.views.TaskCompletionView", [dijit._Widget,
                                                                    dijit._Templated], {

  templateString: dojo.cache("com.ibm.bpm.social.portal.views", "templates/TaskCompletionView.html"),
  CLASS_NAME: "TaskCompletionView",
  LOG_LEVEL: 1,

  active: false,
  taskId: 0,
  lastListOrInstanceTopic: null,
  lastInstancePayload: null,
  lastTaskAlreadyCompleted: "",

  postCreate: function(arguments){
	var FUNC_NAME = "postCreate";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
	  dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
	  this.taskMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

    dojo.create("div", {
      'id': 'completion_content_widget_target'
    }, this.completionContent);

    dojo.create("div", {
      'id': 'completion_sidebar_widget_target'
    }, this.sidebarContent);

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Subscribing to TaskCompletionView topics...", this.CLASS_NAME, FUNC_NAME);
    dojo.subscribe(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", dojo.hitch(this,
      function(args){
        dojo.addClass(this.domNode, "maximizedContent");
        dojo.addClass(dojo.body(), "socialPortalWorkMaximized");
        dojo.query(".workViewContainer").addClass("maximizedContent");
      }));

    dojo.subscribe(com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT + "-TaskCompletionView", dojo.hitch(this,
      function(args){
        dojo.removeClass(this.domNode, "maximizedContent");
        dojo.removeClass(dojo.body(), "socialPortalWorkMaximized");
        dojo.query(".workViewContainer.maximizedContent").removeClass("maximizedContent");
      }));

    this.initializeContentWidgets();

    //Determine width of our main content
    this._sizeContentContainer();

    // Add tabindex of -1 for accessibility so we can change focus to this tab through dojo
    dojo.query('#completion_content_widget_target').attr("tabindex", "-1");

    // Logic for triggering size adjustments.
    // 	Might be temporary depending on what the shell is going to provide.
    this.connect(dojo.isIE && dojo.isIE < 9 ? this.domNode : dojo.global, 'onresize', "_sizeContentContainer");
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _returnToLastView: function(){
	var FUNC_NAME = "_returnToLastView";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _sizeContentContainer: function(){
    // Browser automatically sizes the coachContainer due to using a layout with 100% 
    // Only the allocation of the available space need to be recalculated using JavaScript.
    if (this.coachContainer) {
      this.coachContainer.resize();
    }
  },

  _coachLoaded: function(tkiid /* string */){
  // Any logic that needs to be triggered upon loading of the coach can be added here.
  },

  _coachCanceled: function(){
    //
    // Stay on the page since the side bar still provides meaningful information.
    //
    this.coachContainer.hide();
    // Return to the last view
	this.taskId = null;
	this.taskTemplateId = null;
    this._returnToLastView();
  },

  _taskCompleted: function(nextTkiid /* auto flow task */){
	var FUNC_NAME = "_taskCompleted";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    // Switch back to task list.
    // Is there a smarter solution which switches back to where we came from?
    if (nextTkiid) {
    	// autoflow case
		// remove this task from the history list (since it finished)
		var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
		if (portalHashHistory && (portalHashHistory.length > 0)) {
			portalHashHistory.pop();
		}	
		// load next task and update sidebar
		this.load({
			"taskId": nextTkiid,
			"update": true
		});
    }
    else {
   		this.taskId = null;
	    this.taskTemplateId = null;

		// force process performance dashboard to use a new instance
		dojo.publish(com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD,['TWP','Process+Performance']);

		var src = null;
		var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
		if (portalHashHistory && (portalHashHistory.length > 1)) {
			// remove this task from the history list (since it finished)
			portalHashHistory.pop();
			// check if the previous window was the process performance dashboard
			var src = portalHashHistory.pop();
			var tmp = src;
			if (tmp.charAt(0) != '/') {
				tmp = '/' + tmp;
			}
			var i = tmp.indexOf('?');
			if (i != -1) {
				tmp = tmp.substring(0,i);
			}
			var tmp = tmp.split('/');
			if (tmp && (tmp.length > 3) && (tmp[2] == 'TWP') && ((tmp[3] == 'Process+Performance') || (tmp[3] == 'Process Performance'))) {
				// go to the task list instead of the process perfomance dashboard
				src = '/dashboards/TWP/BPM_WORK?tw.local.view=tasks';
			}
			dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[src]);
		} else {
			// let navigation logic figure out where to go
			com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
		}
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _updateInstance: function(){
    if (this.active && this.taskId) {
      // we need to update everything because in Critical path simply everything might be changed
      this.load({
        "taskId": this.taskId,
        "update": true
      });
    }
  },

  _updateTask: function(changes){
    if (this.active && this.taskId) {
      if (changes && changes.assignee) {
        // if the assignee is changed, one cannot work on the task anymore
        if (this.coachContainer) {
          this.coachContainer.onCompleted();
        }
      }
      else {
        this.load({
          "taskId": this.taskId,
          "update": true
        });
      }
    }
  },

  _taskAlreadyCompleted: function(piid /* string */){
	this.taskId = null;
    this.taskTemplateId = null;
    if (piid) {
      // Show instance view
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        instanceId: piid
      }]);
    }
    else {
      // Stay on page
      this.coachContainer.hide();
    }
  },
  _taskDoesNotExist: function(){
    // Show the task list again.
    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
  },
  initializeContentWidgets: function(){
	var FUNC_NAME = "initializeContentWidgets";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Content teams place widget initialization here ***//
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TaskCompletionWidget....", this.CLASS_NAME, FUNC_NAME);
    this.coachContainer = new com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget({
      dialog: false
    }, 'completion_content_widget_target');
    this.connect(this.coachContainer, "onLoaded", "_coachLoaded");
    this.connect(this.coachContainer, "onCanceled", "_coachCanceled");
    this.connect(this.coachContainer, "onCompleted", "_taskCompleted");
    this.connect(this.coachContainer, "onTaskAlreadyCompleted", "_taskAlreadyCompleted");
    this.connect(this.coachContainer, "onTaskDoesNotExist", "_taskDoesNotExist");

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the PortalSidebar....", this.CLASS_NAME, FUNC_NAME);
    this.sidebar = new com.ibm.bpm.social.portal.PortalSidebar({
      showWidgetTopic: com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION
    }, 'completion_sidebar_widget_target');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the InstanceDetails for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.details = new com.ibm.bpm.social.widgets.details.InstanceDetails({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.details, this.details, 'bpm-social-details-alt-icon', '&Dagger;');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the Stream for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.stream = new com.ibm.bpm.social.widgets.stream.Stream({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.stream, this.stream, 'bpm-social-stream-alt-icon', ':=');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the Experts for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.experts = new com.ibm.bpm.social.widgets.experts.Experts({
      'sidebar': true
    });
    this.sidebar.addTab(this.messages.experts, this.experts, 'bpm-social-experts-alt-icon', '?');

    this.sidebar.startup();
    var expertsTabButton = dojo.query("li[role=\"tab\"][actual-title=\""+this.messages.experts+"\"]")[0];
    //connect onclick and keydown handlers to refresh experts list
    dojo.connect(expertsTabButton, "onclick", dojo.hitch(this.experts, this.experts.showExpertsList));
    dojo.connect(expertsTabButton, "onkeydown", dojo.hitch(this, function(event) {
      if(event.keyCode == 13 /*Enter Key */) {
          this.experts.showExpertsList();
      }
    }));
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _loadTask: function(taskData, update){
  var taskAlreadyCompleted = false;
  if (taskData && taskData.state && taskData.state == "STATE_FINISHED") {
	if (this.lastTaskAlreadyCompleted != taskData.tkiid) {
		this.lastTaskAlreadyCompleted = taskData.tkiid;
	} else {
		this.lastTaskAlreadyCompleted = "";
		taskAlreadyCompleted = true;
	}
  }
  if (!taskAlreadyCompleted) {
		if (update) {
		  // Only update the meta data of the coach container
		  this.coachContainer.updateTask(taskData);
		  this._sizeContentContainer(); // Due to problems with IE
		}
		else {
		  // 
		  // Show the task completion ui.
		  //
		  this.coachContainer.loadCoach(taskData);
		  this._sizeContentContainer(); // Due to problems with IE
		}

		if (this.taskId) {
		  this.details.refresh({
			instanceId: taskData.piid
		  });

		  this.stream.refresh({
			taskid: this.taskId,
			instanceid: taskData.piid
		  });

		  this.experts.refresh({
			taskid: this.taskId
		  });
		  var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=taskcompletion&tw.local.taskid='+this.taskId;
		  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);

		  // connect to task updates
		  this.taskUpdateHandle =
			dojo
			  .subscribe(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this.taskId, dojo.hitch(this, this._updateTask));

		  // connect to instance updates
		  this.instanceUpdateHandle =
			dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED + "." + taskData.piid, dojo.hitch(this,
			  this._updateInstance));
		} 
	} else {
		  var newHash2 = 'dashboards/TWP/BPM_WORK?tw.local.view=tasks';
		  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash2);
		  var params={"isWorkTab":true};
		  var msg={"runURL":newHash2,"params":params};
		  dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [msg]);
	}
  },

  _loadTaskTemplate: function(taskTemplateData){
    // Prereqs are checked by the TaskCompletionWidget
    this.coachContainer.loadCoach4Template(taskTemplateData);
    this._sizeContentContainer(); // Due to problems with IE
  },

  _showErrorMessage: function(errorInfo){
    var errorMessage = errorInfo.data.message;
    var statusCode = errorInfo.ioArgs.xhr.status;
    //
    // Is it a well-known error code?
    //
	this.taskId = null;
	this.taskTemplateId = null;
    if (statusCode == 401) {
        // 401 Unauthorized
	      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.taskMessages["user_not_authorized_for_task"], null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });
    }
    else if (statusCode == 404) {
        // 404 Not Found
	      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.taskMessages["task_not_available"], null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });      
    }
    else {
      //
      // Otherwise just show the error message.
      //
      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(errorMessage, null /* details */, function(){
	    com.ibm.bpm.social.util.common.Util.gotoPreviousHash();
      });
    }
	com.ibm.bpm.social.util.common.Log.logError(this.LOG_LEVEL, ""+this.CLASS_NAME+" experienced an error: "+errorMessage);
  },

  load: function(arguments){
	var FUNC_NAME = "load";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Called from the WorkViewContainer every time immediately before the view is displayed ***//

    //
    // Either the taskId or the taskTemplateId should be provided
    //
    if (typeof arguments.taskId === "undefined" && typeof arguments.taskTemplateId === "undefined") {
	  com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "No taskId or taskTemplateId", this.CLASS_NAME, FUNC_NAME);
      return;
    }

    // disconnect from update handlers
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Disconnecting update handlers...", this.CLASS_NAME, FUNC_NAME);
    if (this.instanceUpdateHandle) {
      dojo.unsubscribe(this.instanceUpdateHandle);
      this.instanceUpdateHandle = null;
    }
    if (this.taskUpdateHandle) {
      dojo.unsubscribe(this.taskUpdateHandle);
      this.taskUpdateHandle = null;
    }

    this.active = true;

    //
    // Initially set all the ids to null.
    // Is it a task or a taskTemplate Id ??   If yes, call the REST api and retrieve the object.
    //
    this.taskId = null;
    this.taskTemplateId = null;
    var taskData = null;
    var taskTemplateData = null;
    var result = null;
    if (arguments.taskId) {
      this.taskId = arguments.taskId;
      this.taskTemplateId = null;
      // Always retrieve an updated task since the state could have changed in the meantime
      result = com.ibm.bpm.social.util.common.Util.getTaskData({
        "taskId": this.taskId,
        "skipCache": true
      });
      taskData = result;
    }
    else {
      this.taskId = null;
      this.taskTemplateId = arguments.taskTemplateId;
      result = com.ibm.bpm.social.util.common.Util.getTaskTemplateData({
        "taskTemplateId": this.taskTemplateId
      });
      taskTemplateData = result;
    }
    //
    // Processing of the result of either the getTaskData or the getTaskTemplateData call.
    //
    if (result) {
      //
      // Did an error occur?
      // Since task definitions are also tasks (which are startable), the same error message can be used.
      //
      if (typeof result.errorInfo === "object") {
        //
        // Error occurred
        //
        var errorInfo = result.errorInfo;
		if(this.taskId != null) {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error loading task "+this.taskId, this.CLASS_NAME, FUNC_NAME);
		}
		else {
			com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Error loading task "+this.taskTemplateId, this.CLASS_NAME, FUNC_NAME);
		}
        this._showErrorMessage(errorInfo);
      }
      else {
        //
        // The rest call was successful
        //
        if (taskData) {
          this._loadTask(taskData, typeof arguments.update === "boolean" && arguments.update);
        }
        else if (taskTemplateData) {
          this._loadTaskTemplate(taskTemplateData);
        }
      }
    }

    dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{
      "viewType": "TaskCompletionView"
    }]);
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  unload: function(arguments){
	var FUNC_NAME = "unload";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    this.active = false;
    if (this.coachContainer) {
      this.coachContainer.exitTaskCollaboration();
      this.coachContainer.unloadCoach();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  destroy: function(){
	var FUNC_NAME = "destroy";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    if (this.instanceUpdateHandle) {
      dojo.unsubscribe(this.instanceUpdateHandle);
      this.instanceUpdateHandle = null;
    }
    if (this.taskUpdateHandle) {
      dojo.unsubscribe(this.taskUpdateHandle);
      this.taskUpdateHandle = null;
    }
    if (this.coachContainer) {
      this.coachContainer.exitTaskCollaboration();
    }
    this.inherited(arguments);

	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  }
});