/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.views.TaskListView");

dojo.require("dijit._Templated");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.widgets.LaunchableProcessList");

dojo.require("com.ibm.bpm.social.widgets.FollowedProcessList");

dojo.require("com.ibm.bpm.social.widgets.TaggedProcessList");

dojo.require("com.ibm.bpm.social.portal.PortalSidebar");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bpm.social.widgets.task.TasksContainer");

dojo.require("dijit.layout.ContentPane");

dojo.declare("com.ibm.bpm.social.portal.views.TaskListView", [dijit._Widget,
                                                              dijit._Templated], {
  templateString: "",
  CLASS_NAME: "TaskListView",
  LOG_LEVEL: 1,

  templatePath: dojo.moduleUrl("com.ibm.bpm.social.portal.views", "templates/TaskListView.html"),

  postCreate: function(arguments){
	var FUNC_NAME = "postCreate";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    //Create placeholder divs to be clobbered
    dojo.create("div", {
      'id': 'tasklist_content_widget_target'
    }, this.taskListContent);

    dojo.create("div", {
      'id': 'tasklist_sidebar_widget_target'
    }, this.sidebarContent);

	this.dashboardBannerHeader.innerHTML = this.messages["workPage-dashboardBannerTitle"];
    this.initializeContentWidgets();
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },
  
  initializeContentWidgets: function(){
	var FUNC_NAME = "initializeContentWidgets";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Content teams place widget initialization here ***//
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TasksContainer...", this.CLASS_NAME, FUNC_NAME);
    this.listContainer = new com.ibm.bpm.social.widgets.task.TasksContainer({
      tabHeaderDiv: this.taskListNav
    }, 'tasklist_content_widget_target');
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
    this.sidebar = new com.ibm.bpm.social.portal.PortalSidebar({
		showWidgetTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST
	}, 'tasklist_sidebar_widget_target');

	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the LaunchableProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.launch = new com.ibm.bpm.social.widgets.LaunchableProcessList({
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		showTaskTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION
	});
		
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the FollowedProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.followed = new com.ibm.bpm.social.widgets.FollowedProcessList({
		refreshRate:60,
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		refreshTopic:com.ibm.bpm.social.portal.Topics.REFRESH_FOLLOW_LIST,
		profileChangedTopic:com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED
	}); 
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Creating the TaggedProcessList for the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.tagged = new com.ibm.bpm.social.widgets.TaggedProcessList({
		refreshRate: 60,
		showProcessTopic:com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE,
		showTaskTopic:com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION,
		refreshTopic:com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST
	});
	var taggedTabIndex;
	var selectedTabIndex=0;
	
	com.ibm.bpm.social.util.common.Log.log(this.LOG_LEVEL, "Adding tabs to the PortalSidebar...", this.CLASS_NAME, FUNC_NAME);
	this.sidebar.addTab(this.messages['launchableProcessList-tabName'], this.launch, 'bpm-social-launch-alt-icon', 'O');
	this.sidebar.addTab(this.messages['followedProcessList-tabName'], this.followed, 'bpm-social-followed-alt-icon', '&#9733;');
	taggedTabIndex = this.sidebar.addTab(this.messages['taggedProcessList-tabName'], this.tagged, 'bpm-social-mentions-alt-icon', '&#64;', true);
	this.tagged.claimControlOfTab(this.sidebar, taggedTabIndex);
	
    this.sidebar.startup(selectedTabIndex);
	this.mentionsVisible = false;
    this.connect(this.sidebar, "selectTab", "_sidebarTabSelected");
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  _sidebarTabSelected: function(index){
    if (index == 2) {
      // @Mentions tab selected
      this.tagged.onSelected();
      this.mentionsVisible = true;
    }
    else {
      this.tagged.onDeSelected();
      this.mentionsVisible = false;
    }
  },

  load: function(arguments){
	var FUNC_NAME = "load";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
    //*** Called from the WorkViewContainer every time immediately before the view is displayed ***//
    
    //Add individual widget refresh calls
    
	var queue = null;
	if (arguments.savedSearchName) {
		queue = this.listContainer._getQueue(arguments.savedSearchName,true);
	}
	if (queue) {
		if (arguments.searchFilter) {
			this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES).setSearchText(arguments.searchFilter);
		}
		this.listContainer._setCustomViewQueue(queue);
	} else {
		if (arguments.state) {
			var filter = null;
			if (arguments.state == 'open') {
				filter = com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById(com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS);
			}
			if (arguments.state == 'close') {
				filter = com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById(com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS);
			}
			if (filter) {
				this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS)._doFilter(filter);
			}			
		}
		if (arguments.searchFilter) {
			this.listContainer._getListController(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS).setSearchText(arguments.searchFilter);
		}
	    this.listContainer.refresh();
		var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS;
	    var tabConfig = this.listContainer._getTabConfig(id);
		this.listContainer.tabContainerWidget.selectTab(tabConfig, tabConfig.tab);
	}

	dojo.publish(com.ibm.bpm.social.portal.Topics.VIEW_READY, [{
      "viewType": "TaskListView"
    }]);
	if (this.mentionsVisible) {		
      this.tagged.onShow();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  unload: function(arguments){
	var FUNC_NAME = "unload";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
	if (this.mentionsVisible) {
      this.tagged.onHide();
    }
	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  },

  destroy: function(){
	var FUNC_NAME = "destroy";
	com.ibm.bpm.social.util.common.Log.logEntry(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);

    this.inherited(arguments);

	com.ibm.bpm.social.util.common.Log.logExit(this.LOG_LEVEL, "", this.CLASS_NAME, FUNC_NAME);
  }

});
