/*
 * IBM Confidential
 *
 * IBM Rational Focal Point
 *
 * Copyright IBM Corporation 2010. All Rights Reserved.
 */

// Input widget that addresses dynamic Bidi complex expressions
// (e.g. URIs, emails).
//
// @author bidi_bdl_lkemmel
//

//------------------------------------------------------------------------------
// BidiTextBox class and commonly used bidi utils
//------------------------------------------------------------------------------

//==============================================================================
// CONSTANTS
//==============================================================================

dojo.provide("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox = {};

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF = navigator.userAgent.toLowerCase().indexOf("firefox") >= 0;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE = !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF && navigator.appName.toLowerCase().indexOf("microsoft") >= 0;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants = {};

// Direction constants
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR = 1;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR = 2;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR = 3;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM = "\u200e";
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM = "\u200f";

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE = '\u202A';
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE = '\u202B';

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF = '\u202C';

// Regex for a left-to-right character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR = new RegExp("[a-zA-Z" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + "]");

// Keep these distinct, if we would need to support some language specific feature
// Regex for a Hebrew character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR = new RegExp("[\\u05be\\u05c0\\u05c3\\u05c6\\u05d0-\\u05f4"
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + "\\ufb1d\\ufb1f-\\ufb28\\ufb2a-\\ufb4f]");

// Regex for an Arabic character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR = new RegExp("[\\u061b\\u0621-\\u064a\\u066d-\\u066f\\u0671-\\u06d5"
        + "\\u06dd\\u06e5\\u06e6\\u06ee\\u06ef\\u06fa-\\u070d\\u0710\\u0712-\\u072f"
        + "\\u074d-\\u07a5\\u07b1\\ufb50-\\ufd3d\\ufd50-\\ufdfc\\ufe70-\\ufefc]");

// Regex for control character (only address those we utilize)
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR = new RegExp("[" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE + "]", "m");

// Delimiting strings to create Bidi segments
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;

//==============================================================================
// com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory namespace
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory = {

//==============================================================================
// Extends the given element with the properties of BidiTextBox.
//==============================================================================
    subclass: function(id, direction, delimiters) {
        this.subclassElement(document.getElementById(id), direction, delimiters);
    },

    subclassElement: function(element, direction, delimiters) {
        if (element && "BidiTextBox" != element.subclass) {
            element = new com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox(element, direction, delimiters);
        }
    }
};

//==============================================================================
// BidiTextBox class
//==============================================================================

//==============================================================================
// BidiTextBox constructor
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox = function(element, direction, delimiters) {
    for (var property in com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype) {
        try {
            if (!element[property]) {
                element[property] = this[property];
            }
        } catch(e) {
            return element;
        }
    }
    element.delimiters = delimiters;

    if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR) {
        element.dir = "";
        element.isContextualDir = true;
        element.init();
    } else {
        if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR) {
            element.dir = "rtl";
        } else if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR) {
            element.dir = "ltr";
        }
        if (delimiters) {
            element.init();
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.push(element);
        }
    }
    return element;
}

// Static members
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.subclass = "BidiTextBox";

//==============================================================================
// Init BidiTextBox object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.init = function() {
    this.connectListeners();

    var value = this.normalizeContent(false);
    this.formatContent(value, true);
};

if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {

//==============================================================================
// Connect event listeners for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        // this.attachEvent("onfocus", this.onFocusIn);
        // this.attachEvent("onkeyup", this.onKeyUp);
        // this.attachEvent("oncut", this.onCut);
        // this.attachEvent("onpaste", this.onPaste);
        // this.attachEvent("ondrop", this.onPaste);
        // window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        // if (this.delimiters) {
            // this.attachEvent("onkeydown", this.onKeyDown);
            // this.attachEvent("oncopy", this.onCopy);
            // this.attachEvent("onmouseup", this.onMouseUp);
            // this.attachEvent("oncontextmenu", this.onContextMenu);
            // this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        // }
		
		this.attachEvent("onfocus", dojo.hitch(this, this.onFocusIn));
        this.attachEvent("onkeyup", dojo.hitch(this, this.onKeyUp));
        this.attachEvent("oncut", dojo.hitch(this, this.onCut));
        this.attachEvent("onpaste", dojo.hitch(this, this.onPaste));
        this.attachEvent("ondrop", dojo.hitch(this, this.onPaste));
        window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        if (this.delimiters) {
            this.attachEvent("onkeydown", dojo.hitch(this, this.onKeyDown));
            this.attachEvent("oncopy", dojo.hitch(this, this.onCopy));
            this.attachEvent("onmouseup", dojo.hitch(this, this.onMouseUp));
            this.attachEvent("oncontextmenu", dojo.hitch(this, this.onContextMenu));
            this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        }
    };

//==============================================================================
// Copy event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();

        var range = document.selection.createRange();
        if (range && range.text) {
            window.clipboardData.setData('Text', range.text.replace(/[\u200e\u200f]/g, ''));
            event = event || window.event;
            event.cancelBubble = true;
            event.returnValue = false;
            return false;
        }
    };

//==============================================================================
// Cut event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCut = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onCopy(event);

        var range = document.selection.createRange();
        if (range) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.hideText();
            range.text = "";
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };

//==============================================================================
// Paste event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onPaste = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.update(false);"
                + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();", 0);
    };

//==============================================================================
// Mouseup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onMouseUp = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    };

//==============================================================================
// Contextmenu event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onContextMenu = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem;
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = setInterval(
                "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem.update(false)", 1000);
    };

//==============================================================================
// Keydown event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        if (!event.ctrlKey && !event.altKey) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();
        }
    };
//==============================================================================
// Keyup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyUp = function(event) {
        if (this.subclass != "BidiTextBox") {
            if(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == null || 
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == undefined) {
        	var inputs = document.getElementsByTagName("input");
        	for(var i=0; i<inputs.length; i++) {
        	  var input = inputs[i];
        	    if(input.name == "reportName") {
        	        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = input;
        	        break;
    		     }
        	}
            }
            return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onKeyUp(event);
        }
        if (!event) {
            event = window.event;
        }
        if (!event.altKey) {
            if (!event.ctrlKey) {
                // 'Home', left arrow and right arrow key code values
                NAVIGATION_KEYS = [36, 37, 39];
                i = NAVIGATION_KEYS.length;
                while (i-- > 0) {
                    if (event.keyCode == NAVIGATION_KEYS[i]) {
                        this.getSafePosition();
                        return;
                    }
                }
            }
            this.update(false);
        }
    };

//==============================================================================
// Validates caret position after a key or mouse event occurs.
// The caret will not be placed between a Bidi formatting code and the
// associated character. To accomplish this, the caret may be moved 1 position
// forward or backwards, depending on navigation direction.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSafePosition = function() {
        if (!this.selection) {
            return;
        }
        var value = this.getValue();
        if (!value) {
            return;
        }
        var startMove, endMove;
        var prevSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (!this.selection) {
            return;
        }
        startMove = this.selection[0] - prevSelection[0];
        endMove = this.selection[1] - prevSelection[1];
    
        // First do the end. We may need to move both end and start (when selection
        // is collapsed), so have the start be updated also.
        if (startMove) {
            if (!this.isMovingOverControl(value, this.selection[0], startMove)) {
                startMove = 0;
            } else {
                startMove = startMove > 0 ? 1 : -1;
                do {
                    this.selection[0] = Math.max(0, this.selection[0] + startMove);
                } while (this.isMovingOverControl(value, this.selection[0], startMove));
                startMove = this.selection[0] - prevSelection[0];
            }
        }
        if (endMove && this.isMovingOverControl(value, this.selection[1], endMove)) {
                endMove = endMove > 0 ? 1 : -1;
            var textLen = value.length;
            do {
                this.selection[1] = Math.min(textLen, this.selection[1] + endMove);
            } while (this.isMovingOverControl(value, this.selection[1], endMove));
    
            if (this.selection[0] > this.selection[1]) {
                this.selection[0] = this.selection[1];
            }
            this.setSelection();
    
        } else if (startMove) {
            if (this.selection[0] > this.selection[1]) {
                this.selection[1] = this.selection[0];
                this.setSelection();
            } else {
                this.moveSelectionStart(startMove);
            }
        }
    };

//==============================================================================
// Checks if the caret is moving over a control character.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.isMovingOverControl = function(value, position, direction) {
        if (direction < 0) {
            return position - 1 >= 0
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position));
        }
        if (direction > 0) {
            return position + 1 <= this.getValue().length
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position - 1));
        }
        return false;
    };

//==============================================================================
// Moves selection start point, leaving the end intact.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.moveSelectionStart = function(value) {
        if (this.selection) {
            var range = document.selection.createRange();
            if (range) {
                range.moveStart('character', value);
                range.select();
            }
        }
    };

} else {

//==============================================================================
// Connect event listeners for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        this.addEventListener("focus", this.onFocusIn, false);
        this.addEventListener("change", this.update, true);
        this.addEventListener("input", this.update, true);
        this.addEventListener("drop", this.update, false);
		
		//If a div element
		if(this.localName == "div") {
			this.addEventListener("keyup", this.update, true);
		}
		
        window.addEventListener("unload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload, true);
        if (this.delimiters) {
            this.addEventListener("keydown", this.onKeyDown, true);

            this.setAttribute("oncopy", ';');
            if (typeof this["oncopy"] == 'function') {
                this.removeAttribute("oncopy");
                this.addEventListener("copy", this.onCopy, true);
            }
            this.setAttribute("oncut", ';');
            if (typeof this["oncut"] == 'function') {
                this.removeAttribute("oncut");
                this.addEventListener("cut", this.onCopy, true);
            }
            window.addEventListener("submit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit, false);
        }
    };

//==============================================================================
// Copy and cut event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        this.getSelection();
        if (this.selection && this.selectionStart != this.selectionEnd) {
            this.normalizeContent(true);

            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };
    
//==============================================================================
// Keydown event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        // older Firefox of version < 3
        if (event.ctrlKey && !event.shiftKey && !event.altKey
                && (event.keyCode == event.DOM_VK_C && typeof this["oncopy"] != 'function'
                || event.keyCode == event.DOM_VK_X && typeof this["oncut"] != 'function')) {
            this.onCopy(event);
        } else {
            this.getSelection();
        }
    };
}

//==============================================================================
// Sets global Bidi element to this object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onFocusIn = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == this
        || document.activeElement && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == document.activeElement) {
        return;
    }
    if (this.subclass != "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
        var target = event.srcElement || event.target;
        if (target && target.subclass == "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = target;
        }
    } else {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    }
};

//==============================================================================
// Inserts controls characters after delimiters and adjusts locally saved
// selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.formatContent = function(value, applyValue) {
    if (!value) {
        value = this.getValue();
    }
    if (this.isContextualDir) {
        this.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(0, value);
    }
    if (!this.delimiters) {
        return value;
    }
    if (this.dir == "ltr" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(value)
        || this.dir == "rtl" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(value)) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var CONTROL_STR = this.dir == "rtl" ? com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING
        : com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING;
    CONTROL_STR_LEN = CONTROL_STR.length;

    var newStr = new String(value);
    var i, j, c;
    var start = 0, end = -1;
    var afterStrong = false;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length - 1; i++, j++) {
        c = value.charAt(i);
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
            afterStrong = this.dir == "ltr";
        } else if (c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR)) {
            afterStrong = this.dir == "rtl";
        } else if (afterStrong && this.delimiters.indexOf(c) >= 0) {
            newStr = newStr.substr(0, j + 1) + CONTROL_STR + newStr.substr(j + 1);
            afterStrong = false;
            if (this.selection) {
                if (this.selection[0] == i) {
                    ++start;
                } else if (this.selection[0] > i) {
                    start += CONTROL_STR_LEN;
                }
                if (this.selection[1] == i) {
                    ++end;
                } else if (this.selection[1] > i) {
                    end += CONTROL_STR_LEN;
                }
            }
            j += CONTROL_STR_LEN;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.setValue(newStr, true);
    }
    this.unhideText();
    return newStr;
};

//==============================================================================
// Gets and caches the current native selection.
// Returns integer array containing the selection start and end points.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSelection = function() {
    if (typeof this.selectionStart == "number") {
        this.selection = [this.selectionStart, this.selectionEnd];
    } else if (document.selection) {
        try {
            var start = 0;
            var range = document.selection.createRange().duplicate();
            var range2 = range.duplicate();
            var rangeLength = range.text.length;
            
            if (this.isMultiLine) {
                range2.moveToElementText(this);
            } else {
                range2.expand('textedit');
            }
            // Return value of range#moveStart doesn't seem reliable, so
            // also limit number of iterations.
            // Otherwise we can fall into infinite loop.
            var n = range2.text.length;
            while (start < n && range.compareEndPoints('StartToStart', range2) > 0) {
                if (range.text.charAt(0).match(/[\r\n]/)) {
                    if (!range.moveStart("word", -1)) {
                        break;
                    }
                } else if (!range.moveStart('character', -1)) {
                    break;
                }
                ++start;
            }
            this.selection = [start, start + rangeLength];
        } catch (e) {
            this.selection = null;
        }
    }
};

//==============================================================================
// Gets text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getValue = function() {
	return this.value || this.text || this.textContent || this.innerHTML || "";
};

//==============================================================================
// Form submit event handler.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.handleSubmit = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    }
    this.resetSelection();
    this.normalizeContent(true);
};

//==============================================================================
// Sets text color to the background color making that way the text invisible.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.hideText = function() {
    if (typeof this.foreground == 'undefined') {
        this.foreground = this.style.color;
    }
    this.style.color = this.style.backgroundColor || "white";
};

//==============================================================================
// Removes controls characters and adjusts locally saved selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.normalizeContent = function(applyValue) {
    var value = this.getValue();
    if (!value || !this.delimiters) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var newStr = new String(value);
    var i, j;
    var start = 0, end = -1;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length; i++) {
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(i))) {
            newStr = newStr.substr(0, j) + newStr.substr(j + 1);
            if (this.selection) {
                if (this.selection[0] > i) {
                    --start;
                }
                if (this.selection[1] > i) {
                    --end;
                }
            }
        } else {
            ++j;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.hideText();
        this.setValue(newStr, true);
    }
    return newStr;
};

//==============================================================================
// Resets our private selection object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.resetSelection = function() {
    this.selection = null;
};

//==============================================================================
// Sets the native selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setSelection = function() {
    if (!this.selection || this.readOnly) {
        return;
    }
    if (this.setSelectionRange) {
        if (this.selectionStart != this.selection[0] || this.selectionEnd != this.selection[1]) {
            this.focus();
            this.setSelectionRange(this.selection[0], this.selection[1]);
        }
    } else if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        var newSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (newSelection[0] != this.selection[0] || newSelection[1] != this.selection[1]) {
            var range = this.createTextRange() || document.body.createTextRange();
            if (range) {
                if (this.isMultiLine) {
                    range.moveToElementText(item);
                } else {
                    range.expand('textedit');
                }
                range.collapse(true);
                range.moveEnd('character', newSelection[1]);
                range.moveStart('character', newSelection[0]);
                range.select();
            }
        }
    }
};

//==============================================================================
// Sets the text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setValue = function(newValue, setSelection) {
    if (!newValue) {
        newValue = this.newValue;
    }
    if (newValue != this.getValue()) {
        if (this.text) {
            this.text = newValue;
        } else if (this.value) {
            this.value = newValue;
        }
        if (setSelection) {
            this.setSelection();
        }
    }
};

//==============================================================================
// Resets element's text color.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.unhideText = function() {
    this.style.color = this.foreground;
};

//==============================================================================
// Updates the text value though normalizing and subsequent formatting.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.update = function(delay) {
        // On load FF fires focus on body element, and as a result
        // BidiTextBox.gCurrentItem can be uninitialized.
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    this.getSelection();
	
    var origValue = this.getValue();
    this.newValue = this.formatContent(this.normalizeContent(false), false);
	
    if (this.newValue != origValue) {
        this.hideText();
        if (delay == true) {
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.setValue(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.unhideText();"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        } else {
            this.setValue(this.newValue, true);
            this.unhideText();
            this.resetSelection();
        }
    }
};

//==============================================================================
// Handle submit events in this window.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit = function(event) {
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.length; i-- > 0; ) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes[i].handleSubmit(event);
    }
}

//==============================================================================
// Handle window unload.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload = function(event) {
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
}

//==============================================================================
// Stops item update invocation.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate = function() {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval) {
        clearInterval(BidiTextBox.gInterval);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
    }
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR.test(text)
            || com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains LTR characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar = function(c) {
    return c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR);
}

//==============================================================================
// Returns direction resolved to LTR or RTL.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection = function(dir, value) {
    if (dir) {
        return dir;
    }
    if (value) {
        for (var i = 0, n = value.length; i < n; i++) {
            var c = value.charAt(i);
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
                return "rtl";
            }
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(c)) {
                return "ltr";
            }
        }
    }
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2);
}

//==============================================================================
// Extends text controls with Bidi properties and applies base text direction
// for both dynamic and static elements.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleBidi = function() {
    //var visibility = document.body.style.visibility;
    //document.body.style.visibility = 'hidden';
    var nodes = document.getElementsByTagName('input');
    for (i = nodes.length; i-- > 0;) {
        if (/^text/.test(nodes[i].type)) {
            if (/bidi_url/.test(nodes[i].className)) {
                if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
                    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(nodes[i],
                        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR, ".:@\\/");
                }
            } else {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
            }
        }
    }
    nodes = document.getElementsByTagName('textarea');
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
    }
    nodes = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements();
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(nodes[i]);
    }
    //document.body.style.visibility = visibility;
}

//==============================================================================
// Applies base text direction for dynamic element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection = function(node) {
    if (node && !node.dir) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir = getBidiDir(0);
        if (bidiTextDir) {
            node.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir;
        } else if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
        }
    }
}

//==============================================================================
// Applies base text direction for static element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection = function(node) {
    if (!node || /bidi_url/.test(node.className)) {
        return;
    }
    if (node.nodeType == '3' && /[^\s]+/.test(node.nodeValue)) {
        var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(0), node.nodeValue);
        if (dir == 'ltr') {
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'rtl') {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "ltrRightAlign");
            } else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM);
            }
        } else {
        	if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'ltr') {
        		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "rtlLeftAlign");
        	} else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM);
            }
        }
    } else {
        for (var i = node.childNodes.length; i-- > 0;) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(node.childNodes[i]);
        }
    }
}

// Code added to handle the case of English then Arabic then ( in LTR
// Or Arabic then English then ( in RTL
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses = function(node, markup) {
	node.nodeValue = node.nodeValue.replace(/\(/g, markup + '('); 
}

//==============================================================================
// Applies base text direction for a text node.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection = function(node, className) {
    if (node) {
        if (node.nodeValue) {
            node.nodeValue = node.nodeValue.replace(/[\u00a0]$/gm, "");
        }
        node = node.parentNode;
        if (node && node.tagName && node.tagName.toUpperCase() != 'SCRIPT') {
            // When there is no block-level element parented to this node,
            // use immediate parent anyway
            var parent = node;
            while (node && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle(node, 'display') == 'inline') {
                node = node.parentNode; 
            }
            if (node) {
                parent = node;
            }            
            if (!/ltrRightAlign|rtlLeftAlign/.test(parent.className)) {
                parent.className += (parent.className ? " " : "") + className;
            }
        }
    }
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle = function(node, style) {
    var gcs = "getComputedStyle" in window;
    return (gcs ? window.getComputedStyle(node, null) : node.currentStyle).getPropertyValue(style); 
}


com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES = {
	bidiText: 'bidiText'
 };
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES = ['TR', 'TD', 'a', 'h1'];

//==============================================================================
// Retreives document static elements enabled for bidi.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements = function() {
    var bidiNodes = [];
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES.length; i-- > 0;) {
        var nodes = document.getElementsByTagName(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES[i]);
        for (var j = nodes.length; j-- > 0;) {
            var classNames = nodes[j].className.split(' ');
            for (var k = classNames.length; k-- > 0;) {
                if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES[classNames[k]]) {
                    bidiNodes.push(nodes[j]);
                    break;
                }
            }
        }
    }
    return bidiNodes;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir = function(which) {
        
	// which = 0, to get the text direction explicitly defined by the user (LTR or RTL or default)
	// If user setting is contextual, we return nothing... And the JS will check for buffer content
	// If the text direction can not be determined from the buffer content
	// The JS will call which=2, to get the default direction for user language preference
	// 
	// As we'll always use contextual mode, we always return an empty string when which=0
	if (which == 0) return '';

			
	// which = 2, to return the default direction for user language preference
	// Currently we do not support RTL languages, so this will always return "ltr"
	if (which == 2) return 'ltr';
}

/**
 * ============================================================================================================================================
 * BELOW ARE ALL EDITS CONTRIBUTED AFTER THE PORT OF THIS FILE TO EVEREST
 * This includes pretty much all of the changes which we had to make to get the file working in 
 * our environment, as well as some small snippets which help consolidate actions.
 * ============================================================================================================================================
 */ 

/**
 * This method accepts text as a parameter and determines the proper BTD depending on content
 * and user setting
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD = function(text) {
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	
	if(btdpref != "ctx") {
		return btdpref;
	} else {
		if(typeof text != "undefined" && text != "") {
			var calcbtd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection("", text);
			if(calcbtd == "") {
				//something went wrong
				return "default";
			} else {
				return calcbtd;
			}
		} else {
			//should pass in a valid string, but if no string, obviously doesn't matter.
			return "default";
		}
	}
}

/**
 * This method takes in a node, and sets the appropriate BTD value to the "dir" attribute, depending
 * on the user preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD = function(node) {

	if(typeof node == "undefined") {
		return;
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(node.innerHTML);
	if(btdpref != "default") {
		node.setAttribute("dir", btdpref);
        if(btdpref == "rtl") {
            var text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(node.innerHTML);
			if (node.innerText) {
				node.innerText = text;
			} else {
				node.innerHTML = text;
			}
        }
	}
}

/**
 * This method takes in text, and returns the appropriate BTD
 * value for the "dir" attribute, depending on the user
 * preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD = function(text) {

    var dir = "";
	if(text == "") {
		return dir;
	}	
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
	if(btdpref != "default") {
		dir = btdpref;
	}
    return dir;
}

/**
 * This method abstracts the process of setting an input elements BTD based on the user preference
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD = function(node) {
	
	if(typeof node == "undefined" || node == null) {
		return;
	}
	
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	if(btdpref == "ctx") {
		//contextual requires bidiTextBox to be used or auto if WebKit browsers
		if(!com.ibm.bpm.social.util.common.Util.isChrome) {
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
		} else {
			//Webkit supports the "auto" dir attribute
			node.dir = "auto";
		}
	} else if (btdpref != "default") {
		//either ltr or rtl
		node.dir = btdpref;
	}
}

/**
 * This method inputs text and adds Unicode Control Characters
 * (UCC) based on the user preference.  Returns the text with
 * UCC added.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText = function(text) {
    var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
    if(dir != "default") {
        //set the appropriate control character
        var controlChar = (dir == "ltr") ? 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

        //wrap the html in the control characters
        text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;

        if (dir == "rtl") {
            text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(text);
        }
    }
    return text;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis = function(text) {
    var newTxt = text;
            //fixParenthesis, fix colon
    newTxt = newTxt.replace(/\(/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + '('); 
    newTxt = newTxt.replace(/\:/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + ':'); 

    return newTxt;
}

