// PageSettings.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.PageSettings");


dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.config.ViewConfig");


dojo.declare("com.ibm.bpm.social.util.common.PageSettings", null, {


  // copy constructor
  constructor: function constructor_PageSettings(obj){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.tab = this.tabPrefixes.WORK;
    this.defaultPageString = this.tabPrefixes.WORK + "/" + this.profileFilterIds["OPEN"];

    if (obj) {
      dojo.safeMixin(this, obj);
      this.createProfileString();
    }
    else {
      this._currentProfileString = this.defaultPageString;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * the selected/active tab, default /tasks
   */

  tab: "",


  /**
   * URL of the dashboard
   */
  dashboardURL: "",

  /**
   * filterConfigId: empty or one of
   * 
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS
   * com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS
   * 
   */
  filterConfigId: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,

  /**
   * searchString: empty or plain text representation of the search string as entered by the user
   */
  searchFilterString: "",

  /**
   * savedSearchName: name of the used search if page is saved searches
   */
  savedSearchName: "",


  /**
   * sortAttributes: Array of attributes the list is sorted
   */
  sortAttributes: [],

  /**
   * Cache for the last calculated profileString
   * 
   */
  _currentProfileString: "",

  /**
   * Indicate if we found the PageSettings valid
   */
  isValid: true,


  /**
   * Valid parameters in the user attributes string
   */
  profileParameters: {
    QUERY: "query",
    SEARCHFILTER: "searchFilter",
    SORTATTRIBUTES: "sortAttributes",
    SORTORDER: "sortOrder",
    DASHBOARD: "dashboard"

  },

  /**
   * URL-Representation of filters
   */
  profileFilterIds: {
    OPEN: "open",
    COMPLETED: "completed",
    QUERIES: "queries"
  },


  /**
   * Map describing which string in the user attribute maps to which filter <br>
   * !!! Needs to be kept aligned with profileFilterIds !!!
   * 
   */
  filterIdMapping: {
    "open": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,
    "completed": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS,
    "queries": com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS
  },


  /**
   * Available tabs
   */
  tabPrefixes: {
    WORK: "/tasks",
    DASHBOARDS: "/dashboards"
  },


  /**
   * default value for the start page, calculated in constructor
   */
  defaultPageString: "",

  /**
   * Reset the class to it default values
   * 
   * @type {void}
   */
  reset: function reset_PageSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.savedSearchName = "";
    this.searchFilterString = "";
    this.filterConfigId = com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS;
    this.sortAttributes = [];
    this.dashboardURL = "";

    // update the current profile string
    this.createProfileString();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },


  /**
   * Return the cached profile string
   * 
   * @type {void}
   */
  getCurrentProfileString: function getCurrentProfileString_PageSettings(){
    return this._currentProfileString;
  },

  /**
   * Take the string an parse it into a new PageSettings object<br>
   * Note: This function does not change the until the doMixin is set to true
   * 
   * @param {String} a string like from the user profile
   * @param {Boolean} if true and the string is found valid, copy the settings into the current object
   * @returns {com.ibm.bpm.social.util.common.PageSettings}
   */
  parseFromString: function parseFromString_PageSettings(profileString, doMixin){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var params = null;
    var param = "";
    var paramKey = "";
    var paramValue = "";
    var parts = null;
    var prefixPart = "";
    var desiredFilter = "";
    var sortAttributes = [];
    var sortOrders = [];
    var sortOrder = "";
    var valid = false;

    var newPageSettings = com.ibm.bpm.social.util.common.PageSettings();


    // Prefix check, string must start with anything from this.tabPrefixes
    for ( var i in this.tabPrefixes) {
      if (profileString.indexOf(this.tabPrefixes[i]) === 0) {
        valid = true;
        newPageSettings.tab = this.tabPrefixes[i];
        com.ibm.bpm.social.util.common.Log.log(3, "found valid prefix " + newPageSettings.tab);
        break;
      }
    }

    if (valid) {
      // check if there are more parts
      parts = profileString.split("?");
      prefixPart = parts[0] ? parts[0] : "";
      if (parts.length > 1) {
        params = parts[1].split("&");
      }
      else {
        params = null;
      }


      // if we got add. parameters appended with "?", read the params
      if (params) {
        for (i = 0; i < params.length; i++) {
          param = params[i].split("=");
          if (param.length == 2 && param[0] && param[1]) {
            paramKey = param[0];
            paramValue = param[1];

            if (paramKey == this.profileParameters.QUERY) {
              newPageSettings.savedSearchName = window.decodeURIComponent(paramValue);
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&lt;/g,'<');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&gt;/g,'>');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&quot;/g,'"');
              newPageSettings.savedSearchName = newPageSettings.savedSearchName.replace(/&amp;/g,'&');
              
            }
            else if (paramKey == this.profileParameters.SEARCHFILTER) {
              newPageSettings.searchFilterString = window.decodeURIComponent(paramValue);
            }
            else if (paramKey == this.profileParameters.SORTATTRIBUTES) {
              sortAttributes = paramValue.split(",");
            }
            else if (paramKey == this.profileParameters.SORTORDER) {
              sortOrders = paramValue.split(",");
            }
            else if (paramKey == this.profileParameters.DASHBOARD) {
              newPageSettings.dashboardURL = window.decodeURIComponent(paramValue);
            }
          }
        }

        // set the sortAttributes array from the strings
        newPageSettings.sortAttributes = [];
        for (i = 0; i < sortAttributes.length; i++) {
          if (sortAttributes[i]) {
            sortOrder = sortOrders[i];
            if (sortOrders[i] != "ASC" && sortOrders[i] != "DESC") {
              sortOrder = "ASC";
            }
            newPageSettings.sortAttributes.push({
              property: sortAttributes[i],
              sortDir: sortOrder
            });
          }
        }
      }


      // Prefix processing for the Work tab
      if (newPageSettings.tab == this.tabPrefixes.WORK) {
        // read filter config
        desiredFilter = prefixPart.substr(prefixPart.lastIndexOf("/") + 1); // "/tasks/open"  -> "open"
        newPageSettings.filterConfigId = this.filterIdMapping[desiredFilter];
        if (!newPageSettings.filterConfigId) {
          newPageSettings.filterConfigId = this.filterIdMapping[this.profileFilterIds.OPEN];
          com.ibm.bpm.social.util.common.Log.log(2, "Invalid filter from profile, using default filter: " +
            newPageSettings.filterConfigId);
        }

        // sanity check: no queries filter w/o savedSearchName
        if (desiredFilter == this.profileFilterIds.QUERIES && !newPageSettings.savedSearchName) {
          com.ibm.bpm.social.util.common.Log.log(2,
            "Invalid name for saved search from profile, switching to default MyTasks/Open Tasks");
          valid = false;
        }
      }


      // Prefix processing for the Dashboards tab
      else if (newPageSettings.tab == this.tabPrefixes.DASHBOARDS) {
        // Check if we got a valid URL
        if (newPageSettings.dashboardURL === "") {
          com.ibm.bpm.social.util.common.Log.log(2, "missing dashboard URL, switching to default MyTasks/Open Tasks");
          valid = false;
        }
      }
    }

    newPageSettings.isValid = valid;

    if (valid && doMixin) {
      // update the profile string presentation
      newPageSettings.createProfileString();
      dojo.safeMixin(this, newPageSettings);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return newPageSettings;
  },

  /**
   * Take all relevant settings and create a string that can be written into the users profile
   * 
   * @type {void}
   */
  createProfileString: function createProfileString_PageSettings(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var filterPart = "";
    var params = [];
    var result;

	if (this.dashboardURL != "") {
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'isWorkTab');
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'isScoreboard');
		// ensure that the processApp, serviceName and snapshot are not duplicated
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'processApp',1);
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'serviceName',1);
		this.dashboardURL = com.ibm.bpm.social.util.common.Util.removeParameter(this.dashboardURL,'snapshot',1);
	}

    if (this.tab == this.tabPrefixes.DASHBOARDS) {
      result = this.tabPrefixes.DASHBOARDS;
      params.push(this.profileParameters.DASHBOARD + "=" + window.encodeURIComponent(this.dashboardURL));
    }
    else {

      // saved searches
      if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {

        // saved search name
        filterPart = this.profileFilterIds.QUERIES;
        params.push(this.profileParameters.QUERY + "=" + window.encodeURIComponent(this.savedSearchName));

        // sort attributes
        // ASZ: Tempoariliy disabled to be consistent in reading and writing, because reading from profile doesn't work correctly for sort.
//      if (this.sortAttributes && this.sortAttributes.length > 0) {
//        var sortAttributes = this.profileParameters.SORTATTRIBUTES + "=";
//        var sortOrder = "&" + this.profileParameters.SORTORDER + "=";
//        for ( var i = 0; i < this.sortAttributes.length; i++) {
//          if (i > 0) {
//            sortAttributes += ",";
//            sortOrder += ",";
//          }
//          sortAttributes += this.sortAttributes[i].property;
//          sortOrder += this.sortAttributes[i].sortDir;
//        }
//        params.push(sortAttributes + sortOrder);
//      }
      }
      // my task view
      else {
        if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
          filterPart = this.profileFilterIds.OPEN;
        }
        else if (this.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
          filterPart = this.profileFilterIds.COMPLETED;
        }
      }

      if (this.searchFilterString) {
        params.push(this.profileParameters.SEARCHFILTER + "=" + window.encodeURIComponent(this.searchFilterString));
      }

      // compose String
      result = this.tabPrefixes.WORK + "/" + filterPart;
    }

    if (params.length > 0) {
      result += "?";
      while (params.length > 0) {
        result += params.shift();
        if (params.length > 0) {
          result += "&";
        }
      }
    }

    // cache the string for quick access
    this._currentProfileString = result;

    com.ibm.bpm.social.util.common.Log.logExit(2, "Calculated Start page string is: " + result);
    return result;
  }
});
