// ProcessRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.ProcessRestHandler");

dojo.require("dojo.string");

dojo.require("com.ibm.bpm.social.util.common.RestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.declare("com.ibm.bpm.social.util.common.ProcessRestHandler", com.ibm.bpm.social.util.common.RestHandler, {

  constructor: function constructor_ProcessRestHandler(params){},

  /**
   * @param {object}
   * 
   * @param {String} id
   * @param {function} load
   * @param {function} error
   * @param {boolean} async
   * @param {String} parts one of "all", "data", "none" - default is "all"
   * @param {String} systemID federation only
   */
  getProcess: function getProcess_ProcessRestHandler(id, load, error, async, parts, systemID){

    if (this.isApiTypeFederated()) {
      if (!systemID) {
        this.errorNoSystemID();
      }
    }

    var url = "/process";
    url += "/" + id;
    url = this.addSystemID(url, systemID);

    if (parts && parts != "all") {
      url += "?parts=" + parts;
    }

    var sync = true;
    if (async) {
      sync = false;
    }
    var myself = this;
    this.performRequest("get", {
      url: url,
      sync: sync,
      // load: load,
      load: function(process, ioArgs){
        process = myself.addSystemProperties(process, systemID);
        load(process, ioArgs);
      },
      error: error
    });
  },
  
  /**
   *
   * @param startURL
   * @param load
   * @param error
   * @param async
   * @param systemID
   * @returns nothing
   */
  launchProcess: function launchProcess_ProcessRestHandler(startURL, load, error, async, systemID) {
	var inx = startURL.indexOf("/process");
	var url=startURL.substring(inx);
	url = this.addSystemID(url, systemID);
	this.performRequest("post", {
		url: url,
		load: load,
		error: error,
		sync: !async
	});
  },  
  
  
  /**
   * @param {Array} items
   * @param {String} idKey e.g TKIID or tkiid or id
   * @param actionsFilter
   * @param {Function} load
   * @param {Function} error
   * @param async
   * @param {String} domain, optional for federated backend and more than one item
   * @returns {dojo.Deferred}
   */
  getActions: function getActions_TaskRestHandler(items, idKey, actionsFilter, load, error, async, domain){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    if (actionsFilter) {
      com.ibm.bpm.social.util.common.Log.logObject(3, actionsFilter, "actionsFilter");
    }

    var deferred = null;

    var self = this;
    var idString = "";
    dojo.forEach(items, function(item){
      if (idString !== "") {
        idString += ",";
      }
      idString += self.prefixID(item[idKey], domain, item.systemID);
    });

    var url = "/process/actions?";
    var newLoad = load;
    if (this.isApiTypeFederated()) {
      if (domain) {
        url += "domain=" + domain + "&";
      }
      else if (items.length === 1 && items[0].systemID) {
        url += "systemID=" + items[0].systemID + "&";
        // insert the system id, in federated case the system id is not returned if no domain is specified
        newLoad = function(data, ioArgs){
          if (data && data.items) {
            for ( var i = 0; i < data.items.length; i++) {
              data.items[i].systemID = items[0].systemID;
            }
          }
          load(data, ioArgs);
        };

      }
      else {
        this.errorNoDomainOrSystemID();
      }
    }
    url += "instanceIds=" + encodeURIComponent(idString);

    if (actionsFilter) {
      var actionsFilterString = "";
      for ( var i in actionsFilter) {
        if (actionsFilterString !== "") {
          actionsFilterString += ",";
        }
        actionsFilterString += actionsFilter[i];
      }
      url += "&actions=" + actionsFilterString;
    }

    deferred = this.performRequest("get", {
      url: url,
      sync: !async,
      load: load,
      error: error
    });

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return deferred;
  },


  /**
   * @param {object}
   * @param processID
   * @param {Function} load
   * @param {Function} error
   * @param async
   * @param systemID
   * @param domain
   * @returns {void} nothing
   */
  getActionsForProcess: function getActionsForProcess_ProcessRestHandler(processID, load, error, async, domain){

    var url = "/process/" + encodeURIComponent(processID) + "/actions?";

    var newLoad = load;
    if (this.isApiTypeFederated()) {
      if (domain) {
        url += "domain=" + domain + "&";
      }
//      else if (items.length === 1 && items[0].systemID) {
//        url += "systemID=" + items[0].systemID + "&";
//        // insert the system id, in federated case the system id is not returned if no domain is specified
//        newLoad = function(data, ioArgs){
//          if (data && data.items) {
//            for ( var i = 0; i < data.items.length; i++) {
//              data.items[i].systemID = items[0].systemID;
//            }
//          }
//          load(data, ioArgs);
//        };
//      }
      else {
        this.errorNoDomainOrSystemID();
      }
    }

    var deferred = this.performRequest("get", {
      url: url,
      sync: !async,
      load: load,
      error: error
    });

    return deferred;
  },


  /**
   * @param {object} params
   * 
   * @param id
   * @param load
   * @param error
   * @param priority e.g. Highest/10, High/20, Normal/30, Low/40 or Lowest/50
   * @param dueDate in ISO-8601 format (e.g. 2011-01-05T03:15:30.000-00:00)
   * @param systemID
   * @returns {void} nothing
   */
  updateProcessProperties: function updateProcessProperties_ProcessRestHandler(params){
    var id = params.processID;
    var load = params.load;
    var error = params.error;
    var priority = params.priority;
    var dueDate = params.dueDate;
    var systemID = params.systemID;

    var url = "/process";
    url += "/" + id + "?action=update";

    if (priority) {
      url += "&priority=" + priority;
    }

    if (dueDate) {
      url += "&dueDate=" + dueDate;
    }

    url = this.addSystemID(url, systemID);
    this.performRequest("put", {
      url: url,
      sync: true,
      load: load,
      error: error
    });
  },

  runAdHocAction: function runAdHocAction__ProcessRestHandler(params){
    var id = params.processID;
    var actionID = params.actionID;
    var load = params.load;
    var error = params.error;
    var systemID = params.systemID;

    var url = "/process";
    url += "/" + id + "?action=trigger" + "&id=" + actionID;


    url = this.addSystemID(url, systemID);
    this.performRequest("put", {
      url: url,
      sync: true,
      load: load,
      error: error
    });
  },
	
  /**
   * 
   * @param piid
   * @param comment
   * @param load
   * @param error
   * @param async
   * @param systemID
   * @returns {void} nothing
   */	
  postInstanceStreamComment: function postInstanceStreamComment_ProcessRestHandler(piid, comment, load, error, async, systemID) {
		var url = "/process/" + piid;
		
		var url = this.addSystemID(url, systemID);
		
		var content = {
			action: "comment",
			comment: comment
		};
		
		this.performRequest("post", {
			url: url,
			content: content,
			load: load,
			error: error,
			sync: !async
		});		
  },
  
  statics: {
    _instance: null,

    _createInstance: function _createInstance_ProcessRestHandler(params){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.util.common.ProcessRestHandler();
        com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
        // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
      }
      else {
        // programming error
        var error = new Error("RestHandler instance has already been initialized!");
        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
        throw error;
      }
      return this._instance;
    },

    _getInstance: function _getInstance_ProcessRestHandler(){
      if (this._instance === null) {
        // TODO for now return default handler
        this._instance = new com.ibm.bpm.social.util.common.ProcessRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.util.common.ProcessRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.ProcessRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.ProcessRestHandler.prototype.statics._getInstance();
};
