// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.RestHandler");

dojo.require("dojo.string");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.widgets.task.util.DataCache");
dojo.require("com.ibm.bpm.social.widgets.task.util.Systems");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");

dojo.declare("com.ibm.bpm.social.util.common.RestHandler", null, {
  prefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the bfm calls
  bfmPrefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the vmm calls
  vmmPrefix: "",

  // root for form generator service
  generatorServicePrefix: "",

  _errorHandler: null,

  error: null,

  version: "v1",

  threshold: 100,

  // 30 seconds timeout to account for some slowness and also to be consistent with Business Space/Mashups settings
  defaultTimeout: 30000,

  _apiType: null,

  _iWidget: null,

  FEDERATION_MODE: "federationMode=true",

  _systems_refresh_required: false,

  constructor: function constructor_RestHandler(params){
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
    // IV09357:
    // Check whether a timoeout has been set using the mashup maker configuration:
    //
    // In order to overwrite the timeout for the Human Workflow Widget, open the Administration Console.
    // Select "Mashups_ConfigService" under Resources -> Resource Environment -> Resource Environment Providers.
    // Then select "Custom properties". Create a new property "com.ibm.bpm.social.widgets.task.requestTimeout" and set its value to the wanted timeout in milliseconds. 
    // In addition to that select the com.ibm.mashups.properties.client and add ",com.ibm.bpm.social.widgets.task.requestTimeout" to 
    // the value of this property (It is a comma separated list of values therefore do not forget the comma).
    //
// TODO disabled for now
//    if (typeof ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"] !== "undefined") {
//      var timeout = ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"];
//      if (typeof timeout === "string") {
//        timeout = parseInt(timeout);
//      }
//      com.ibm.bpm.social.util.common.Log.log(2, "The default timeout for the Human Workflow Widgets is set to " + timeout +
//        " ms");
//      this.defaultTimeout = timeout;
//    }
  },

  getTimeout: function(){
    return this.defaultTimeout;
  },

  /**
   * required for handling of federation results
   */
  setIWidget: function setIWidget_RestHandler(iWidget){
    this._iWidget = iWidget;
  },

  getVmmPrefix: function getVmmPrefix_RestHandler(){
    return this.vmmPrefix;
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_RestHandler(){
    if (this._apiType === null) {
      var systems = this.getSystems();
      if (systems) {
        this._apiType = systems.getApiType();
      }
    }
    return this._apiType;
  },

  /**
   * Returns if the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns if the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns if the REST service endpoint is a ICM system
   * 
   * @returns {boolean} true if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns if the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  performRequest: function performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var deferred = null;

    if (request) {

      if (request.load && this.isApiTypeFederated() && !isBulk) {
        // show federation result errors as exception in BSpace message area
        // for bulk request the error is shown in the error popup
        var load = request.load;
        this._apiCaller = null;
        if (arguments && arguments.callee && arguments.callee.caller && arguments.callee.caller.name) {
          this._apiCaller = arguments.callee.caller.name;
        }
        var systems = this.getSystems();
        var _systemsSending401Error = null;
        if (systems && systems._systemsSending401Error) {
          _systemsSending401Error = systems._systemsSending401Error;
        }
        var myself = this;
        request.load =
          function(data, ioArgs){
            if (data && data.federationResult && data.federationResult.length > 0) {
              if (!myself._iWidget) {
                // federation result cannot be handled with iWidget.handleException!
                com.ibm.bpm.social.util.common.Log.logError(1,
                  "com.ibm.bpm.social.util.common.RestHandler.performRequest",
                  new Error("Property '_iWidget' is required to handle federation results!!!"));
              }
              myself._errorHandler.handleFederationResultErrors(data.federationResult, myself._iWidget, myself._apiCaller,
                _systemsSending401Error);
            }
            // continue with result processing
            load(data, ioArgs);
          };
      }
    }
    deferred = this._performRequest(kind, request, isURLComplete, isBulk);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return deferred;
  },

  /**
   * Note: This function should not call any function which requires system information, because it is also
   * used by the getSystems call. Put code requiring system information in performRequest above.
   * 
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  _performRequest: function _performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var deferred = null;

    request.headers = request.headers || {};

    if (!isURLComplete) {
      request.url = this.getRequestUrl(request.url);
    }
    // make sure the handleAs parameter is set. 
    // Also it is important to set the right request header for json, otherwise 
    // JAXRS will set the header to XML which is not correct for json.
    if (!request.handleAs) {
      request.handleAs = "json";
      request.headers.Accept = "application/json";
    }
    else if (request.handleAs == "json") {
      request.headers.Accept = "application/json";
    }

    if (!request.timeout) {
      request.timeout = this.defaultTimeout;
    }
    if (kind == "get" && (request.preventCache === undefined || request.preventCache === null)) {
      request.preventCache = true;
    }

    if (this.FEDERATION_MODE) {
    if (request.url.indexOf("?") > -1) {
      request.url += "&" + this.FEDERATION_MODE;
    }
    else {
      request.url += "?" + this.FEDERATION_MODE;
      }
    }

	//Set up a listener for 401 responses -- set default action to redirect to login page
	//If an error for some other reason, allow specified handler to take over
	var _error = request.error;
	var myself = this;
	request.error = function(data, ioArgs) {
		var req = ioArgs.xhr;
		if(typeof req != "undefined") {
			var reqStatus = req.status;
			if(typeof reqStatus != undefined) {
				if(reqStatus == 401) {
					//double check to make sure its an actual authentication timeout-
					//check for the authenticate header
					var respHeaders = req.getAllResponseHeaders();
					if(respHeaders.indexOf("WWW-Authenticate") >= 0) {
						//Perform REDIRECT
						com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
					}
				}
			}
		}
		if(_error) {
			_error(data, ioArgs);
		} else {
			//Generic error handler if none is specified in original request
			myself._errorHandler.showGenericError(data, ioArgs);
		}
	};
	
    // if (!request.error) {
      // // default error handler
      // var myself = this;
      // request.error = function(data, ioArgs){
        // myself._errorHandler.showGenericError(data, ioArgs);
      // };
    // }

    if (kind == "get" && request.url.length > com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH) {
      // do a POST request instead of a GET
      var iParams = request.url.indexOf("?") + 1;
      if (iParams > 0) {
        com.ibm.bpm.social.util.common.Log.log(3, "Do a POST instead of a get because url is > " +
          com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH);
        com.ibm.bpm.social.util.common.Log.log(3, "url: " + request.url);
        kind = "post";
        request.contentType = "application/x-www-form-urlencoded; charset=UTF-8";
        request.headers["X-Method-Override"] = "GET";
        request.postData = request.url.substring(iParams);
        request.url = request.url.substring(0, iParams - 1);
      }
    }

    // overwrite the browser locale with the users profile language
    var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var locale = (userProfile && userProfile.userPreferences) ? userProfile.userPreferences['Locale'] : null;

    if (locale) {
      request.headers["Accept-Language"] = locale;
    }

    // Defect 116603 - SiteMinder may redirect the request to an HTML
    // login form when the current user authentication times out.  We
    // detect this case by checking for exceptions during json parsing.
    // If the response cannot be parsed as json we check the response
    // content type to see if "text/html" content was returned.  If so,
    // we assume it is a login form and redirect to our normal logout
    // processing.
    if (!dojo.contentHandlers['jsonOriginal']) {
		dojo.contentHandlers['jsonOriginal'] = dojo.contentHandlers['json'];
		dojo.contentHandlers['json'] = function(xhr) {
			var ret = null;
			try {
				ret = dojo.contentHandlers['jsonOriginal'](xhr);
			} catch (e) {
				contentType = xhr.getResponseHeader("Content-Type");
				if  (contentType && (contentType.indexOf('text/html') != -1)) {
					com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
				} else {
					throw e;
				}
			}
			return ret;
		}
	}
	
    if (kind == "get") {
      // GET request do not need a contentType because they do not have an entity body.
      deferred = dojo.xhrGet(request);
    }
    else if (kind == "rawPost") {
      deferred = dojo.rawXhrPost(request);
    }
    else if (kind == "post") {
      deferred = dojo.xhrPost(request);
    }
    else if (kind == "rawPut") {
      ///////////////
      deferred = dojo.rawXhrPut(request);
    }
    else if (kind == "put") {
      ///////////////
      deferred = dojo.xhrPut(request);
    }
    else if (kind == "delete") {
      deferred = dojo.xhrDelete(request);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);

    return deferred;
  },

  /**
   * @param {String} url the url without prefix
   * @return {String} the complete url for the request
   */
  getRequestUrl: function getRequestUrl_RestHandler(url){
    return this.prefix + this.version + url;
  },

  setUrl: function setUrl_RestHandler(/*String*/urlName, /*String*/url){
    if (urlName) {
      this[urlName] = url;
      if ((url !== "") && url.slice(-1) != "/") {
        this[urlName] += "/";
      }
    }
    com.ibm.bpm.social.util.common.Log.log(3, "urlName=" + this[urlName]);
  },

  /**
   * Prefixes the ID with the systemID, if a domain and a systemID is specified.
   * 
   * @param {String} id
   * @param {String} domain optional, needed for a federated system
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the id, prefixed with the system id, if domain and a systemID are specified
   * @throws error if domain, but no system ID is specified for a federated backend
   */
  prefixID: function prefixID_RestHandler(id, domain, systemID){
    var result = id;
    if (domain) {
      if (systemID) {
        result = systemID + '$' + id;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return result;
  },


  /**
   * Federation only - Adds the systemID to the url
   * 
   * @param {String} url
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the url including the specified systemID
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemID: function addSystemID_RestHandler(url, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (url.indexOf("?") > 0) {
          url += "&";
        }
        else {
          url += "?";
        }
        url += "systemID=" + systemID;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return url;
  },

  /**
   * Adds a processAppName to the url (future use - not used yet)
   * 
   * @param {String} url
   * @param {String} processAppName optional, needed for an ICM system
   * @returns {String} the url including the specified processAppName
   * @returns the identical url if processAppName is null or undefined
   */
  _addProcessAppName: function _addProcessAppName_RestHandler(url, processAppName){
    if (processAppName) {
      if (url.indexOf("?") > 0) {
        url += "&";
      }
      else {
        url += "?";
      }
      url += "processAppName=" + processAppName;
    }
    return url;
  },

  /**
   * Federation only - Adds the properties systemID and systemType to the retrieved item
   * 
   * @param {Object} item a task, task template, process, process template, activity object
   * @param {String} systemID
   * @returns {Object} the updated item
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemProperties: function addSystemProperties_RestHandler(item, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        item.systemID = systemID;
        item.systemType = this.getSystems().getSystemType(item.systemID);
      }
      else {
        this.errorNoSystemID();
      }
    }
    return item;
  },

  /**
   * @returns {Object} a {@link com.ibm.bpm.social.widgets.task.util.Systems} object
   * @throws error if no configuration information retrieved
   */
  getSystems: function getSystems_RestHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var url = this.getRequestUrl("/systems");

    var systems = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
    if (systems === null) {
      var myself = this;
      var error = function(data, ioArgs){
        if (ioArgs.xhr.status == 404) {
          //myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorEndpointConfiguration");
          com.ibm.bpm.social.util.common.Util.restConfigurationException(data, ioArgs);
        }
        else {
          myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorSystemsMetadata");
        }
      };
      var errors = false;
      var i;
      var load = function(data, ioArgs){
        com.ibm.bpm.social.util.common.Log.logObject(3, data, "data");
        if (data && data.federationResult && data.federationResult.length > 0) {
          // graceful handling of 401 in the systems call: ignore but memorize
          var _systemsSending401Error = myself._errorHandler.getSystemsSending401Error(data.federationResult);
          // ... and then make sure the 401 errors do not raise an error message:
          errors = myself._errorHandler.showFederationResultErrors(data.federationResult, "KEY_ErrorSystemsMetadata", true);
        }

        systems = new com.ibm.bpm.social.widgets.task.util.Systems({
          systems: data,
          systemsSending401Error: _systemsSending401Error
        });
      };

      this._performRequest("get", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json",
        // allow browser cache to prevent double calls during initialization
        preventCache: this._systems_refresh_required
      }, true);

      if (systems !== null) {
        com.ibm.bpm.social.util.common.Log.log(1, "systems=" + systems);
        // cache the data
        var cache = 3600000; // 1 hour
        if (errors) {
          // we had errors so cache only for 5 minutes
          cache = 300000;
          this._systems_refresh_required = false;
        }
        this._systems_refresh_required = true;
        com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, systems, cache);
      }
      else {
        var exc = new Error("RestHandler.getSystems: No configuration information retrieved!");
        com.ibm.bpm.social.util.common.Log.consoleError(exc.message);
        throw exc;
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systems;
  },

  errorNoDomain: function errorNoDomain_RestHandler(){
    var error = new Error("No domain name specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoSystemID: function errorNoSystemID_RestHandler(){
    var error = new Error("No system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoDomainOrSystemID: function errorNoDomainOrSystemID_RestHandler(){
    var error = new Error("No domain name or system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  _errorNotSupportedForFederation: function _errorNotSupportedForFederation_RestHandler(url){
    var error = new Error("This call is not supported for federation: " + url);
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    //throw error;
  }
});


/**
 * Maximum length for a GET request, if exceeded POST will be used
 * <p>
 * some browser have a length limit for the url, e.g. for IE this is 2083
 */
com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH = 2000;
