// SecureInnerHtmlPreparation.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo
  .declare(
    "com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation",
    null,
    {

      /*static*/_cache: {},

      originalString: null,
      parsed: false,
      parsedString: null,

      constructor: function(originalString){
        com.ibm.bpm.social.util.common.Log.logEntry(2, null, "SecureInnerHtmlPreparation", "constructor");
        this.originalString = originalString;
        com.ibm.bpm.social.util.common.Log.logExit(2, null, "SecureInnerHtmlPreparation", "constructor");
      },

      /**
       * creates a secure inner HTML
       * 
       * @returns the secure inner HTML
       */
      getSecureString: function getSecureString_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.originalString === null) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning null.");
          return null;
        }
        else if (this._cache[this.originalString]) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning cached value: " + this._cache[this.originalString]);
          return this._cache[this.originalString];
        }
        else if (!this.parsed) {
          this._parse();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "Returning: " + this.parsedString);
        return this.parsedString;
      },

      _parse: function _parse_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.parsedString = "";

        // first step: tokenize the original string
        var tokens = [];

        var currentToken = null;
        var position = 0;

        while (position < this.originalString.length) {
          switch (this.originalString.charAt(position)) {
            case '<' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LT,
                "text": "<"
              };
              break;
            case '>' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_GT,
                "text": ">"
              };
              break;
            case '/' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SLASH,
                "text": "/"
              };
              break;
            case '\n' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case '\r' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK) {
                  tokens.push(currentToken);
                }
                else {
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case ' ' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_SPACE) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += " ";
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SPACE,
                "text": " "
              };
              break;
            default:
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_TEXT) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += this.originalString.charAt(position);
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_TEXT,
                "text": this.originalString.charAt(position)
              };
              break;
          }

          position++;
        }
        if (currentToken !== null) {
          tokens.push(currentToken);
        }

        // Generate output
        this.parsedString = "";
        var containsHTML = false;
        var openTags = [];
        var tagName;

        for ( var i = 0; i < tokens.length; i++) {
          var token = tokens[i];
          var nextToken1 = i + 1 < tokens.length ? tokens[i + 1] : null;
          var nextToken2 = i + 2 < tokens.length ? tokens[i + 2] : null;
          var nextToken3 = i + 3 < tokens.length ? tokens[i + 3] : null;

          if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT) {
            // is there an allowed tag starting
            if (nextToken1 &&
              nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_TEXT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken1.text.toLowerCase()) > -1 &&
              nextToken2 &&
              (nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SPACE ||
                nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SLASH || nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_GT) &&
              this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2) > -1) {

              tagName = nextToken1.text.toLowerCase();

              var nextSlash = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_SLASH, i + 2);
              var nextGt = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2);

              if (nextSlash == -1 || nextGt < nextSlash) {
                containsHTML = true;
                this.parsedString += "<" + tagName + ">";
                i = nextGt;

                if (dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS, tagName) == -1) {
                  openTags.push(tagName);
                }
              }
              else if (nextSlash + 1 == nextGt) {
                containsHTML = true;
                this.parsedString += "<" + tagName + "/>";
                i = nextGt;
              }
              else {
                this.parsedString += "&lt;";
              }
            }
            // is there an allowed tag that is open ending
            else if (nextToken1 && nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_SLASH && nextToken2 &&
              nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_TEXT && nextToken3 &&
              nextToken3.type == com.ibm.bpm.social.util.common.TOKEN_GT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken2.text.toLowerCase()) > -1 &&
              dojo.indexOf(openTags, nextToken2.text.toLowerCase()) > -1) {

              tagName = nextToken2.text.toLowerCase();

              // ensure to close all open tags that were opened after the corresponding opening tag to create valid HTML
              var closingTagName;
              do {
                closingTagName = openTags.pop();

                this.parsedString += "</" + closingTagName + ">";
              }
              while (closingTagName != tagName);

              i += 3;
            }
            else {
              this.parsedString += "&lt;";
            }
          }
          else if (token.type == com.ibm.bpm.social.util.common.TOKEN_GT) {
            this.parsedString += "&gt;";
          }
          else {
            this.parsedString += token.text;
          } // end if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT)
        } // end for ( var i = 0; i < tokens.length; i++)

        // close all opened tags that were not closed to create valid HTML
        while (openTags.length > 0) {
          tagName = openTags.pop();
          this.parsedString += "</" + tagName + ">";
        }

        // if there is no HTML at all replace \n with <br>
        if (!containsHTML) {
          this.parsedString = this.parsedString.replace(com.ibm.bpm.social.util.common.REGEXP_LINEBREAK, "<br>");
        }

        this.parsed = true;

        this._cache[this.originalString] = this.parsedString;

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _getNextTokenPosition: function(/*Token[]*/tokens, /*int*/type, /*int*/offset){
        for ( var i = offset; i < tokens.length; i++) {
          if (tokens[i].type == type) {
            return i;
          }
        }

        return -1;
      }
    });

com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS = ["abbr",
                                                   "acronym",
                                                   "b",
                                                   "big",
                                                   "br",
                                                   "cite",
                                                   "code",
                                                   "dfn",
                                                   "em",
                                                   "h1",
                                                   "h2",
                                                   "h3",
                                                   "h4",
                                                   "h5",
                                                   "h6",
                                                   "hr",
                                                   "i",
                                                   "kbd",
                                                   "li",
                                                   "ol",
                                                   "p",
                                                   "s",
                                                   "samp",
                                                   "small",
                                                   "strike",
                                                   "strong",
                                                   "sub",
                                                   "sup",
                                                   "tt",
                                                   "u",
                                                   "ul",
                                                   "var"];

com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS = ["br",
                                                          "hr"];

com.ibm.bpm.social.util.common.REGEXP_LINEBREAK = new RegExp("\n", "g");

com.ibm.bpm.social.util.common.TOKEN_GT = 0; // >
com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK = 1; // \n
com.ibm.bpm.social.util.common.TOKEN_LT = 2; // <
com.ibm.bpm.social.util.common.TOKEN_SLASH = 3; // /
com.ibm.bpm.social.util.common.TOKEN_SPACE = 4;
com.ibm.bpm.social.util.common.TOKEN_TEXT = 5;