// SocialRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.util.common.SocialRestHandler");

dojo.require("dojo.string");

dojo.require("com.ibm.bpm.social.util.common.RestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.declare("com.ibm.bpm.social.util.common.SocialRestHandler", com.ibm.bpm.social.util.common.RestHandler, {
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getCurrentUserAvatar: function getCurrentUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteUserAvatar: function deleteUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getMentions: function getMentions_SocialRestHandler(pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instances/mentions?page="+pagenum+"&pagesize="+pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param tkid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteMentions: function deleteMentions_SocialRestHandler(tkid, load, error, async, systemID) {
		var url = "/social/task/"+tkid+"/mentions";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteAllMentions: function deleteAllMentions_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/task/mentions/all";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getFollowedInstances: function getFollowedInstances_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/instances/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	isFollowingInstance: function isFollowingInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	followInstance: function followInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	unfollowInstance: function unfollowInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertsForTask: function getExpertsForTask_SocialRestHandler(tkiid, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/experts";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param user
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertUserData: function getExpertUserData_SocialRestHandler(tkiid, user, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/userdata?username=" + encodeURIComponent(user);
		
		var url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param comment
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	postTaskStreamComment: function postTaskStreamComment_SocialRestHandler(tkiid, comment, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/comment";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			postData: comment,
			contentType: "application/json",  //Need to override the default set for posts here.
			sync: !async
		});	
	},
	
	/**
	 * 
	 * @param piid
	 * @param since
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	updateStreamEvents: function updateStreamEvents_SocialRestHandler(piid, since, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		
		if(since) {
			url += "since=" + since;
		}
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getStreamEvents: function getStreamEvents_SocialRestHandler(piid, pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		url += "page=" + pagenum + "&pagesize=" + pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	statics: {
		_instance: null,

		_createInstance: function(params){
		  if (this._instance === null) {
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
			// TODO handle params, e.g. 
	//        this._instance.setIWidget(iWidget);
	//        this._instance.setUrl("prefix", this.getServiceURLRoot());
	//        this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
	//        this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
		  }
		  else {
			// programming error
			var error = new Error("RestHandler instance has already been initialized!");
			com.ibm.bpm.social.util.common.Log.consoleError(error.message);
			throw error;
		  }
		  return this._instance;
		},

		_getInstance: function(){
		  if (this._instance === null) {
			// TODO for now return default handler
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
	//        // programming error
	//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
	//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
	//        throw error;
		  }
		  return this._instance;
		}
	}
});

com.ibm.bpm.social.util.common.SocialRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.SocialRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._getInstance();
};