/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation");

dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("dojo.date");

dojo.require("dojox.date.hebrew");
dojo.require("dojox.date.hebrew.Date");
dojo.require("dojox.date.hebrew.locale");

dojo.require("dojox.date.islamic");
dojo.require("dojox.date.islamic.Date");
dojo.require("dojox.date.islamic.locale");

/**
 * creates a secure string which can be set using DOMElement.innerHTML
 *
 * @param innerHTML the original inner HTML string
 * @returns {String} the secure inner HTML string
 */
com.ibm.bpm.social.util.common.Util.createSecureInnerHtml = function(/*String*/ innerHTML) {
  if(typeof innerHTML == "string") {
    var secureInnerHtmlPreparation = new com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation(innerHTML);
    return secureInnerHtmlPreparation.getSecureString();
  } else {
    return innerHTML;
  }
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS.sssZ
 */
com.ibm.bpm.social.util.common.Util.getDateString = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getFullYear() + "-";
  if (d.getMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getMonth() + 1) + '-';
  if (d.getDate() < 10) {
    result = result + '0';
  }
  result = result + d.getDate() + "T";
  if (d.getHours() < 10) {
    result = result + '0';
  }
  result = result + d.getHours() + ":";
  if (d.getMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getMinutes() + ":";
  if (d.getSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getSeconds() + ".";
  if (d.getMilliseconds() < 100) {
    result = result + '0';
  }
  if (d.getMilliseconds() < 10) {
    result = result + '0';
  }
  result = result + d.getMilliseconds();
  var offset = d.getTimezoneOffset();
  var modifier = "";
  if (offset < 0) {
    modifier = "+";
  } else {
	modifier = "-";
  }
  var tzh = Math.floor(Math.abs(offset) / 60);
  var tzm = offset - (tzh * 60);
  if (tzh < 10) {
    tzh = '0' + tzh;
  }
  result = result + modifier + tzh + ":";
  if (tzm < 10) {
    result = result + '0';
  }
  result = result + tzm;
  return result;
};

com.ibm.bpm.social.util.common.Util.fixDateTimeZone = function(str){
	var i = str.indexOf('Z');
	if (i != -1) {
		str = str.substring(0,i);
		if (i <20) {
			str = str + '.000';
		}
		str = str + '+0000';
	}
	return str;
};

com.ibm.bpm.social.util.common.Util.formatDate = function(str){
	var date,d1,t1;
	
	str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
	date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
	
    d1 = com.ibm.bpm.social.util.common.Util.formatDateObject(date, {formatLength: "long", selector: "date"});
	t1 = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});
	
	return (d1+" "+t1);
};

com.ibm.bpm.social.util.common.Util.formatDateObject = function(d, options){
  var ret = "";
  var calPref = com.ibm.bpm.social.util.common.Util.getCalendarUserPreference();
  if (calPref == "hebrew") {
      var dateObj = new dojox.date.hebrew.Date(d);
      ret = dojox.date.hebrew.locale.format(dateObj, options);
  } else if (calPref == "islamic") {
      var dateObj = new dojox.date.islamic.Date(d);
      ret = dojox.date.islamic.locale.format(dateObj, options);
  } else {
      ret = dojo.date.locale.format(d, options);
  }
  return ret;
};

/*
 *  @param Date string
 *  @returns String in long date format, no time
 */
com.ibm.bpm.social.util.common.Util.formatDateLong = function(str){
  var date, d1;

  str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
  date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);

  d1 = dojo.date.locale.format(date, {
    formatLength: "long",
    selector: "date"
  });
  return d1;
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS
 */
com.ibm.bpm.social.util.common.Util.getTimestampUTC = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getUTCFullYear() + "-";
  if (d.getUTCMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getUTCMonth() + 1) + '-';
  if (d.getUTCDate() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCDate() + "T";
  if (d.getUTCHours() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCHours() + ":";
  if (d.getUTCMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCMinutes() + ":";
  if (d.getUTCSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCSeconds();
  // skip milliseconds and skip TIMEZONE identifier
  return result;
};

com.ibm.bpm.social.util.common.Util.getDateFromString = function(str){

	//Normalize "2011-12-19T10:36:56.060-05:00" to "2011-12-19T10:36:56.060-0500"
	if(str.lastIndexOf(":") > 23){
		var substr1 = str.substr(23,6);
		var substr2 = substr1.replace(":", "");
		str = str.replace(substr1, substr2);
	}
	
	var yr = parseInt(str.substr(0, 4));
	var tmp = str.substr(5, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var mnth = parseInt(tmp);
	tmp = str.substr(8, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var dy = parseInt(tmp);
	tmp = str.substr(11, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var hr = parseInt(tmp);
	tmp = str.substr(14, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var min = parseInt(tmp);
	tmp = str.substr(17, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var sec = parseInt(tmp);
	tmp = str.substr(20, 3);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var msec = parseInt(tmp);
	var result = new Date(yr, mnth - 1, dy, hr, min, sec, msec);
	tmp = str.substr(23, 3);
	var isNegative = (tmp.charAt(0) == '-');
	tmp = tmp.substr(1);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzh = parseInt(tmp);
	tmp = str.substr(26, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzm = parseInt(tmp);
	if (isNegative) {
		tzh = -tzh;
		tzm = -tzm;
	}
	var offset = result.getTimezoneOffset();
	offset = offset + (tzh * 60) + tzm;
	result = new Date(result.getTime() - (offset * 60000));
	return result;
};

com.ibm.bpm.social.util.common.Util.formatServerDateString = function(str){
  var dateStr;
  var d;

  if (dojo.isIE) {
    d = dojo.date.stamp.fromISOString(str);
  }
  else {
    d = new Date(str);
  }

  dateStr = d.toDateString() + " " + d.toLocaleTimeString();
  dateStr = dateStr.substring(4, dateStr.length);
  dateStr = dateStr.substring(0, 6) + "," + dateStr.substring(6, dateStr.length);

  return dateStr;
};

com.ibm.bpm.social.util.common.Util.portalPIIMessages = null;

com.ibm.bpm.social.util.common.Util.restConfigurationException = function(error, xhrArgs){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	if(com.ibm.bpm.social.util.common.Util.portalPIIMessages==null){
		//NOTE:  Locale 'en' is hard-coded here as the required message is only available in the English bundle for 8.0
		dojo.xhrGet({
			"url": ppConfig.portalContextRootJS+"/resources/PortalPIIMessages_en.json",
			"sync": true,
			"handleAs": "json",
			"load": function restConfigurationException_xhrGetLoad(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util.portalPIIMessages = data;
				com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
			},
			"error": function(error, xhrArgs){
				com.ibm.bpm.social.util.common.Log.logError(2, "PortalPIIMessages_en.json could not be retrieved.");
			}
		});
	} else {
		com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
	}
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.openRestConfigurationException = function(){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"]);
	
	var details = "<p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"];
	details += "</p><p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"];
	details += "</p>";
	
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"], details);
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.userData = {};

com.ibm.bpm.social.util.common.Util.getCurrentUser = function(){
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentUser) {
    com.ibm.bpm.social.util.common.Util.currentUser = {
      userName: ""
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/current?parts=none",
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentUser = data.data;
        com.ibm.bpm.social.util.common.Util.userData[data.data.userName] = data.data;
      },
      error: function(error, xhrArgs){
    	  com.ibm.bpm.social.util.common.Util.restConfigurationException(error, xhrArgs);
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentUser;
};

com.ibm.bpm.social.util.common.Util.getUser = function(username, ignoreCacheOnDirty){

  if(typeof(ignoreCacheOnDirty)=="undefined"){
	  ignoreCacheOnDirty = false;
  }
	
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (username == null) {
    return com.ibm.bpm.social.util.common.Util.getCurrentUser();
  }

  if (!com.ibm.bpm.social.util.common.Util.userData[username] ||
		  ((typeof(com.ibm.bpm.social.util.common.Util.userData[username].dirty)=="undefined"||
			com.ibm.bpm.social.util.common.Util.userData[username].dirty) && ignoreCacheOnDirty) ) {
    com.ibm.bpm.social.util.common.Util.userData[username] = {
      userName: username
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/" + encodeURIComponent(username) + "?parts=none&preventCache=" + new Date().getTime(),
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.userData[username] = data.data;
        com.ibm.bpm.social.util.common.Util.userData[username].dirty = false;
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.userData[username];
};

com.ibm.bpm.social.util.common.Util.primeUserCache = function(id, name, url){
	if(typeof(com.ibm.bpm.social.util.common.Util.userData[id])=="undefined"){
		var newUser = {
				"userName": id,
				"fullName": name,
				"userPreferences": {
					"Image": url
				},
				"dirty": true
		};
		com.ibm.bpm.social.util.common.Util.userData[id] = newUser;
	}
};

com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};

/**
 * Returns the URL for the user's avatar.
 * @param username the username (or user id) of the user you want to see the avatar for.
 * @param ignoreCache true if a new URL should be returned since the last time this method was called.
 */
com.ibm.bpm.social.util.common.Util.getUserAvatarURL = function(username, ignoreCache){
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL entering with username:{"+username+"}");
	if(typeof(ignoreCache)=="undefined"){
		ignoreCache = false;
	}
	
	if (com.ibm.bpm.social.util.common.Util.userAvatarURLs == null) {
		com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};
	}
	
	if (username == null) {
		return null;
	}

   	if (ignoreCache || !com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]) {
	   	com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = window.endpointURLObject.bpmRestJS +"/v1/avatar/image/" + encodeURIComponent(username) + "?preventCache=" + new Date().getTime();
		if(typeof(connectionsBridge)!="undefined" 
			&& connectionsBridge.bootstrap==true 
			&& connectionsBridge.isAvatarServiceAvailable()){ //Is Connections available and configured?
			
			var user = com.ibm.bpm.social.util.common.Util.getUser(username);
			if(typeof(user["emailAddress"])!="undefined" 
				&& user["emailAddress"]!=null 
				&& user["emailAddress"].length > 0){ //Does the user have a defined email address?
				
				var avatar = connectionsBridge.getAvatarUrl(user["emailAddress"]);
				if(avatar!=null){
					com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = avatar;
				}
			}
		}		
	}
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL returning {"+com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]+"}");
	return com.ibm.bpm.social.util.common.Util.userAvatarURLs[username];
};

com.ibm.bpm.social.util.common.Util._integrations = null;

com.ibm.bpm.social.util.common.Util.getSametimeURL = function(){
  var ints = com.ibm.bpm.social.util.common.Util._getIntegrations()
  return ints["SAMETIME_SERVER"];
};

com.ibm.bpm.social.util.common.Util.getConnectionsURL = function(){
	var ints = com.ibm.bpm.social.util.common.Util._getIntegrations();
	return ints["CONNECTIONS_SERVER"];
};

com.ibm.bpm.social.util.common.Util._getIntegrations = function(){
	if(!com.ibm.bpm.social.util.common.Util._integrations){
		dojo.xhrGet({
			//url: "/rest/bpm/wle/v1/system/env/variable?bpdShortname=TWP&name=SAMETIME_SERVER",
			url: window.endpointURLObject.bpmRestJS +"/v1/social/system/integrations",
			handleAs: "json",
			sync: true,
			headers: {
				"Content-Type": "application/json",
				"Accept": "application/json"
			},
			load: function(data, xhrArgs){
				if (data.data && data.data.collabSystemsList) {
					com.ibm.bpm.social.util.common.Util._integrations = {};
					
					for(var x = 0; x < data.data.collabSystemsList.length; x++){
						com.ibm.bpm.social.util.common.Util._integrations[data.data.collabSystemsList[x].name] = data.data.collabSystemsList[x].value;
					}
					
				} else {
					com.ibm.bpm.social.util.common.Util_integrations = {};
				}
			},
			error: function(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util._integrations = {};
			}
		});
	}
	return com.ibm.bpm.social.util.common.Util._integrations;
}


com.ibm.bpm.social.util.common.Util.getParameter = function(param){
  var result = null;
  if (window.location.search) {
    var search = window.location.search;
    if (search.indexOf("?") === 0) {
      search = search.substring(1); //Remove leading '?' from the query string
    }
    var list = search.split("&");
    for ( var i = 0; i < list.length; i++) {
      var key = "";
      var val = "";
      var j = list[i].indexOf('=');
      if (j == -1) {
        key = list[i];
      }
      else {
        key = list[i].substring(0, j);
        val = list[i].substring(j + 1);
      }
      if (key == param) {
        result = val;
        i = list.length;
      }
    }
  }
  return result;
};

com.ibm.bpm.social.util.common.Util.currentInstanceData = {};

com.ibm.bpm.social.util.common.Util.getInstanceData = function(paramObj){

  var iid = null;
  var params = "none";
  var skipCache = false;
  var updateTaskData = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["diagram"]) {
      p.push("diagram");
    }
    if (paramObj["header"]) {
      p.push("header");
    }
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["executionTree"]) {
      p.push("executionTree");
    }
    if (paramObj["instanceId"]) {
      iid = paramObj["instanceId"];
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
    if (paramObj["updateTaskData"]){
    	updateTaskData = true;
    }

    if (p.length > 0) {
      params = p.join("%7C");
    }
  }

  if(iid==null) return {}; //iid = com.ibm.bpm.social.util.common.Util.getInstanceId();

  if(com.ibm.bpm.social.util.common.Util.currentInstanceData == null) {
	  com.ibm.bpm.social.util.common.Util.currentInstanceData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = {
    		'piid': iid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/process/";
    url += iid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = data.data;

        if(updateTaskData==true){
        	for(var i = 0; i < data.data.tasks.length; i++){
        		var t = data.data.tasks[i];
        		com.ibm.bpm.social.util.common.Util.currentTaskData[t.tkiid] = t;
        	}
        }

      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentInstanceData[iid];
};

com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};

com.ibm.bpm.social.util.common.Util.getTaskTemplateData = function(paramObj) {

  var tktid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["taskTemplateId"]) {
      tktid = paramObj["taskTemplateId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
      skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskTemplateData == null) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = {
        'tktid': tktid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/taskTemplate/";
    url += tktid;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid];
};

com.ibm.bpm.social.util.common.Util.currentTaskData = {};

com.ibm.bpm.social.util.common.Util.getTaskData = function(paramObj){

  var tid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["taskId"]) {
      tid = paramObj["taskId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskData == null) {
	  com.ibm.bpm.social.util.common.Util.currentTaskData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskData[tid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = {
    		'tkiid': tid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/task/";
    url += tid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskData[tid];
};

com.ibm.bpm.social.util.common.Util.isCommentSupported = function(event){
  if (event.object && event.object.objectType) {
    if (event.object.objectType == 'ibm.bpm.comment') {
      return true;
    }
    if (event.object.objectType.indexOf('ibm.bpm.helprequest') == 0) {
      return true;
    }
  }
  return false;
};

com.ibm.bpm.social.util.common.Util.isIPad = navigator.userAgent.indexOf("iPad") > -1;

com.ibm.bpm.social.util.common.Util.isWebkit = navigator.userAgent.indexOf("WebKit") > -1;

/*
 * Simple method which directs the browser to the login page using the current hostname and port
 */
com.ibm.bpm.social.util.common.Util.redirectToLoginPage = function() {
	var location = window.location;
	var loginPage = 'login.jsp';
	// if the user has a custom logout page, we need to redirect to this
	// page instead of the login.jsp so that they can properly log out of
	// all resources
	if (portalConfigLogoutExitPage && (portalConfigLogoutExitPage != 'login.jsp')) {
		loginPage = 'customLogoutRedirect.jsp';
	}
	var redirUrl = window.endpointURLObject.processPortalJS +"/" +loginPage;
	window.location = redirUrl;
};

/*
 * Method to get the Base Text Direction User preference.  Needs to be used to properly align
 * necessary enabled fields with Base Text Direction.
 *
 * @return BTD 'ltr' || 'rtl' || 'ctx'
 */
com.ibm.bpm.social.util.common.Util.getBTDUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var baseTextDirPref = "ctx";
	
	if(typeof userProfileObj != "undefined") {
		baseTextDirPref = userProfileObj.userPreferences["Base Text Direction"];
		
		if(baseTextDirPref == "generalPreference.fields.baseTextDir.contextual" || baseTextDirPref == "") {
			baseTextDirPref = "ctx";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.ltr") {
			baseTextDirPref = "ltr";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.rtl") {
			baseTextDirPref = "rtl";
		} else {
			//unexpected, default to "ctx"
			baseTextDirPref = "ctx";
		}
	}
	
	return baseTextDirPref;
};

/*
 * Method to get the Calendar Type User preference.  Needs to be used to display
 * dates according to calendar types Gregorian, Hebrew or Islamic.
 *
 * @return calendarType 'gregorian' || 'islamic' || 'hebrew'
 */

com.ibm.bpm.social.util.common.Util.getCalendarUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var calPref = "gregorian";	
	if(typeof userProfileObj != "undefined") {
		var pref = userProfileObj.userPreferences["Calendar Type"];
        if (pref) {
            if (pref == "generalPreference.fields.calendarType.islamic") {
                calPref = "islamic";
            } else if (pref == "generalPreference.fields.calendarType.hebrew") {
                calPref = "hebrew";
            }
        }
    }
    return calPref;
};

/*
 * Method to get the hash portion of a URL
 */

com.ibm.bpm.social.util.common.Util.getCurrentHash = function(checkCookie) {
	var h = window.location.hash;
   	if (h && (h.indexOf('#dashboards%25') == 0)) {
   		h = '#' + decodeURI(h.substring(1));
   		window.location.hash = h;
   	} 
	if (checkCookie && !h) {
		h = dojo.cookie("com_ibm_bpm_process_portal_hash");
	}
	if (h && (h.charAt(0) == '#')) {
		h = h.substring(1);
	}
	if (h) {
		try {
			h = decodeURIComponent(h);
		} catch(e) {
			// if we cannot decode the text, assmume it is already decoded (FireFox)
		}
	}
	if (h) {
		while (h.length && (h.charAt(h.length-1) == ' ')) {
			h = h.substring(0,h.length-1);
		}
	}
	return h;
}

/*
 * Method to remove a parameter from a URL
 */
com.ibm.bpm.social.util.common.Util.removeParameter = function(url,param,skip) {
	var skipped = 0;
	var start = url.indexOf('?'+param+'=');
	if (start == -1) {
		start = url.indexOf('&'+param+'=');
	}
	while (start != -1) {
		var end = url.indexOf('&',start+1);
		var next = start;
		if (skip && (skip > skipped)) {
			skipped++;
			next++;
		}  else {
			if (end == -1) {
				url = url.substring(0,start);
			} else {
				url = url.substring(0,start+1)+url.substring(end+1);
			}
		}
		start = url.indexOf('?'+param+'=',next);
		if (start == -1) {
			start = url.indexOf('&'+param+'=',next);
		}
	}
	return url;
}

/*
 * Method to set the hash portion of a URL
 */
com.ibm.bpm.social.util.common.Util.setCurrentHash = function(h) {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	for (var i=portalHashHistory.length-1; i>-1; i--) {
		if (portalHashHistory[i] == h) {
			portalHashHistory.splice(i,1);
		}
	}
	if (h) {
		portalHashHistory.push(h);
	}
	if (portalHashHistory.length > 10) {
		portalHashHistory.splice(0,portalHashHistory.length-10);
	}
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS + '/' + h; 
	if (window.history && window.history.replaceState) {
		window.history.replaceState({},'',window.ppConfig.portalContextRootJS+'/'+h);
	} else {
		com.ibm.bpm.social.util.common.Util.lastHash = h;
		window.location.hash = '#'+encodeURIComponent(h);
	}
}

/*
 * Current window url information
 */
if (com.ibm.bpm.social.util.common.Util.urlData == undefined) {
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS;
}

/*
 * Show current window url information in dialog
 */
com.ibm.bpm.social.util.common.Util.showProcessPortalUrl = function(event) {
	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
  	var messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
	var msg = '<div id="showLinkDialogText">'+messages.showLinkDialog+'</div><br><br><div style="word-wrap:break-word" tabIndex="0" aria-labelledBy="showLinkDialogText" role="textbox" dir="ltr">'+
			  com.ibm.bpm.social.util.common.Util.urlData+
			  '</div>';
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(msg);
}

/*
 * Creates an object to publish with the topic com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB (Topics.js);
 * This topic is published in many places and the required elements in the object were hard to keep track of
 */
com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB = function(itemParams) {
	var newObj={};
	newObj.title="";
	newObj.baseID="";
	newObj.itemID="";
	newObj.isTip=false;
	newObj.isSnapshot=false;
	newObj.snapshotName="";
	newObj.isWorkTab=false;
	if (itemParams) {
		if (itemParams.title!=undefined) {
			newObj.title=itemParams.title;
		}
		if (itemParams.baseID!=undefined) {
			newObj.baseID=itemParams.baseID;
		}
		if (itemParams.itemID!=undefined) {
			newObj.itemID=itemParams.itemID;
		}
		if (itemParams.isTip!=undefined) {
			newObj.isTip=itemParams.isTip;
		}
		if (itemParams.isSnapshot!=undefined) {
			newObj.isSnapshot=itemParams.isSnapshot;
		}
		if (itemParams.snapshotName!=undefined) {
			newObj.snapshotName=itemParams.snapshotName;
		}
		if (itemParams.isWorkTab!=undefined) {
			newObj.isWorkTab=itemParams.isWorkTab;
		}
	}
	return newObj;
}
 
/*
 * Method to go to return to the previous hash location
 */
com.ibm.bpm.social.util.common.Util.gotoPreviousHash = function() {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	if (portalHashHistory.length > 0) {
		portalHashHistory.pop();
	}	
	if (portalHashHistory.length > 0) {
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[portalHashHistory.pop()]);
	} else {
		window.location.hash = '#';
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[""]);
	}
}
 
 /*
 * Method to get query parameters from a url
 */
 com.ibm.bpm.social.util.common.Util.getQueryParamters = function(url) {
 	var urlParams = new Object(); 
 	if (url && (url.indexOf('?') != -1)) {
		var urlParamString = url.substring(url.indexOf('?')+1);
		var urlParamList = urlParamString.split('&');
		for (var p1=0; p1<urlParamList.length; p1++) {
			var p2 = urlParamList[p1].indexOf('=');
			if (p2 > -1) {
				urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
			}
		}
	}
	return urlParams;
}

com.ibm.bpm.social.util.common.Util.escapeChannelSegment = function(channelSegment) {
  return channelSegment.replace(/~/g, "~007e").replace(/\//g, "~002f");
};
 