/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/   

dojo.provide("com.ibm.bpm.social.util.connections.ConnectionsBridge");

dojo.require("dojo.cookie");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.declare("com.ibm.bpm.social.util.connections.ConnectionsBridge", null, {
	
	connectionsServer: null,
	bootstrap: false,
	bizCardQueue: [],
	loadcount: 0,
	loadtotal: 1,
	objectCheckCounter: 0,
	objectCheckMax: 25,
	objectCheckIntervalId: null,
	avatars: {},
	avatarServiceAvailable: null,
	messages: null,
	
	onLoad: function _connectionsBridge_onLoad(connUrl) {
	    // We only need the scheme/host/port for Business Card support.
	    var qIdx = connUrl.indexOf("?");
	    if (qIdx != -1) {
	        connUrl = connUrl.substring(0, qIdx);
	    }	
		this.connectionsServer = connUrl;
		
		// Load Localization Bundles
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		var cookie = dojo.cookie("card.popup.slim");
		if(typeof(cookie)=="undefined"){
			dojo.cookie("card.popup.slim", "1");
		}
		
		this.loadConnectionsAPI();
		
		this.isAvatarServiceAvailable();	
		
		/**
		 * Link click override, to prevent unwanted navigation away from Process Portal
		 **/
		dojo.connect(dojo.body(), "onclick", function(e){
			try{
				var _connUrl = new dojo._Url(connUrl);
				var _tgtUrl = new dojo._Url(e.target.href);
	
				if(_connUrl.host==_tgtUrl.host && _connUrl.port==_tgtUrl.port){
					dojo.stopEvent(e);
					window.open(e.target.href);
				}	
			} catch (err) { }
		});
	},

	loadConnectionsAPI: function _connectionsBridge_loadConnectionsAPI() {
		/** As a result of this next line, a window level object named "SemTagSvc" is created for us to interact with the Connections API **/
		this._embedJavascript(this.connectionsServer+"/profiles/ibm_semanticTagServlet/javascript/semanticTagService.js", this.checkLoadCompletion);
	},

	/*
	_embedCSS: function(url){
		var linkElement=document.createElement("link");
		linkElement.rel="stylesheet";
		linkElement.type="text/css";
		linkElement.href=url;
		document.getElementsByTagName("head")[0].appendChild(linkElement);
	},
	*/
	
	_embedJavascript: function _connectionsBridge__embedJavascript(url, callback){
		//console.debug("_embedJavascript::entering with url:",url);
		var script=document.createElement("script");
		script.type="text/javascript";
		if (script.readyState){  //Internet Explorer
			script.onreadystatechange = dojo.hitch(this, function(){
				if (script.readyState == "loaded" || script.readyState == "complete"){
					 script.onreadystatechange = null;
					 dojo.hitch(this, callback)();
				}
			});
		} else {  //Everyone else
			script.onload = dojo.hitch(this, callback);
		}
		script.src=url;
		document.getElementsByTagName("head")[0].appendChild(script);
		//console.debug("_embedJavascript::exiting");
	},
	
	checkLoadCompletion: function _connectionsBridge_checkLoadCompletion() {
		this.loadcount++;
		if(this.loadcount==this.loadtotal){
			this.bootstrap = true;
			
			/** Wait for the Javascript object to be created **/
			this.objectCheckIntervalId = setInterval(dojo.hitch(this, function(){
				this.objectCheckCounter++;
				if(typeof(SemTagSvc)!="undefined"  && SemTagSvc!=null){
					/** Once defined, process all queued wrapper requests **/
					clearInterval(this.objectCheckIntervalId);
					this.objectCheckIntervalId = null;
					this.processQueuedBusinessCards();
					
					dojo.connect(LCSemTagMenu,"mouseover", com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
					
					dojo.publish(com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED, [{}]);
				}
				
				/** Only wait and check for a set period of time **/
				if(this.objectCheckCounter >= this.objectCheckMax && this.objectCheckIntervalId!=null){
					clearInterval(this.objectCheckIntervalId);
				}
			}), 500);
			
		}
	},
	
	/**
	 * Convert an element containing a standard user's display name into a Connections-enabled
	 * Business Card link.
	 */
	wrapElement: function _connectionsBridge_wrapElement(/*Node or ID*/ node, /*String*/ userName, /*String*/ userEmail) {
		//console.debug("ConnectionsBridge::wrapElement entering with userName:{"+userName+"} and userEmail:{"+userEmail+"}");
		
		if(typeof(node)=="undefined" || 
				(typeof(userName)=="undefined" || userName.length==0) ||
					(typeof(userEmail)=="undefined" || userEmail.length==0)){
			return false;
		}
		
		var success = false;
		
		if(this.bootstrap==true && typeof(SemTagSvc)!="undefined" && SemTagSvc!=null){
			
			if(typeof(node)=="string"){ /** node is a String containing the ID of the HTMLElement **/
				node = dojo.byId(node);
			}

			if(node.getAttribute("connectId")!=null){
				var connectId = node.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			
			node.innerHTML = "";
			
			var nameNode = dojo.create("a", null, node);
			dojo.addClass(nameNode, "fn");
			dojo.addClass(nameNode, "url");
			nameNode.href = "javascript:void(0);";
			nameNode.innerHTML = userName;
			
			var emailNode = dojo.create("span", null, node);
			dojo.addClass(emailNode, "email");
			dojo.style(emailNode, "display", "none");
			emailNode.innerHTML = userEmail;

			dojo.addClass(node, "vcard");
			
			setTimeout(dojo.hitch(this, function(){
				SemTagSvc.parseDom(null, node); //.parentNode
			}), 500);
			
			success = true;
		} else {
			this.bizCardQueue.push([node, userName, userEmail]);
			success = false;
		}
		
		return success;
	},
	
	/**
	 * Convert an element containing a standard user's avatar into a Connections-enabled
	 * Business Card link.
	 */
	wrapImage: function _connectionsBridge_wrapImage(/*Image Node or ID*/ node, /*String*/ userEmail) {
		//console.debug("ConnectionsBridge::wrapImage entering with userEmail:{"+userEmail+"}");
		
		if(typeof(node)=="undefined" ||
			(typeof(userEmail)=="undefined" || userEmail.length==0)){
			return false;
		}
		
		var success = false;
		
		if(this.bootstrap==true && typeof(SemTagSvc)!="undefined" && SemTagSvc!=null){

			if(typeof(node)=="string"){ /** node is a String containing the ID of the IMGElement **/
				node = dojo.byId(node);
			}
			
			if(node.getAttribute("connectId")!=null){
				var connectId = node.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			
			//Check to see if the hierarchy necessary is already in place
			var container = null;
			var nameNode = null;
			
			var bca = false;
			try{
				bca = dojo.hasClass(node.parentNode, "bpm-bizcard-anchor");
				nameNode = node.parentNode;
			} catch (e){
				bca = false;
				nameNode = null;
			}
			
			/*
			if(nameNode.getAttribute("connectId")!=null){
				var connectId = nameNode.getAttribute("connectId");
				try{
					dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
				} catch (e){
					//console.debug("Invalid disconnect handle attempted.");
				}
			}
			*/
			
			var bcc = false;
			try{
				bcc = dojo.hasClass(node.parentNode.parentNode, "bpm-bizcard-container");
				container = node.parentNode.parentNode;
			} catch (e){
				bcc = false;
				container = null;
			}
			
			if(! (bca && bcc) ){
				container = dojo.create("span", {"class": "bpm-bizcard-container"}, node, "before");

				nameNode = dojo.create("a", {"class": "bpm-bizcard-anchor"}, container);
				nameNode.href = "javascript:void(0);";
			
				dojo.place(nameNode, container, "only");
			}
			
			dojo.place(node, nameNode, "only");
			
			dojo.addClass(nameNode, "fn");
			dojo.addClass(nameNode, "url");
			
			var emailNode = dojo.create("span", null, container);
			dojo.addClass(emailNode, "email");
			dojo.style(emailNode, "display", "none");
			emailNode.innerHTML = userEmail;

			dojo.addClass(container, "vcard");
			
			setTimeout(dojo.hitch(this, function(){
				SemTagSvc.parseDom(null, container); //node.parentNode
			}), 500);
			
			success = true;
			
		} else {
			this.bizCardQueue.push([node, userEmail]);
			success = false;
		}
		
		return success;
	},
	
	processQueuedBusinessCards: function _connectionsBridge_processQueuedBusinessCards(){
		var x = 0;
		while(x = this.bizCardQueue.pop())
		{
			if(x.length==2){
				this.wrapImage(x[0], x[1]); /** node and userEmail **/
			} else if(x.length==3){
				this.wrapElement(x[0], x[1], x[2]); /** node, userName, and userEmail **/
			}
			
		}
	},
	
	isAvatarCached: function _connectionsBridge_isAvatarCached(email){
		return (typeof(this.avatars[email])!="undefined" && this.avatars[email]!=null);
	},
	
	clearAvatarCache: function _connectionsBridge_clearAvatarCache(email){
		if(typeof(email)=="undefined"){
			this.avatars = {};
		}
		
		this.avatars[email] = null;
	},
	
	getAvatarUrl: function _connectionsBridge_getAvatarUrl(email){
		if(!this.avatars[email] || this.avatars[email]==null){
		
			var url = (typeof(window.MASHUPS_PROXY_URL)!="undefined") ? window.MASHUPS_PROXY_URL : window.endpointURLObject.mumJS +"/proxy";
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += this.connectionsServer.replace("://", "/");
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += "profiles/json/profile.do?email=";		// ...profiles/atom/profile.do...???
			url += encodeURIComponent(email);
			
			//	{url} == {https://localhost:9080/mum/proxy/https/w3-connections.ibm.com/profiles/atom/profile.do?email=user@us.ibm.com}
			dojo.xhrGet({
				"url": url,
				"handleAs":"json",
				"sync": true,
				"load": dojo.hitch(this, function(data, xhrArgs){
					if(data["X_inDirectory"]=="true"){
						this.avatars[email] = data["photo"];
					} else {
						this.avatars[email] = null;
					}
				}),
				"error":dojo.hitch(this, function(data, xhrArgs){
					this.avatars[email] = null;
				}),
				"failOk": true,
				"preventCache": true
			});
		}	
		return this.avatars[email];
	},
	
	isAvatarServiceAvailable: function _connectionsBridge_isAvatarServiceAvailable(){
		if(this.avatarServiceAvailable==null){
			this.avatarServiceAvailable = false;
			
			var url = (typeof(window.MASHUPS_PROXY_URL)!="undefined") ? window.MASHUPS_PROXY_URL : "/mum/proxy";
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += this.connectionsServer.replace("://", "/");
			url += (url.substring(url.length-1)=="/")?"":"/";
			url += "profiles/html/searchProfiles.do";
			
			var self = this;
			dojo.xhrGet({
				"url": url,
				"handleAs":"text",
				"sync": true,
				"handle": function(data, xhrArgs){
					self.avatarServiceAvailable = (xhrArgs.xhr.status==200)
					if(!self.avatarServiceAvailable){
						console.error(self.messages.connectionsProxyError.replace("{0}", self.connectionsServer));
					}
				},
				"failOk": true,
				"preventCache": true
			});
		}
		//console.warn("returning this.avatarServiceAvailable:{"+this.avatarServiceAvailable+"}");
		return this.avatarServiceAvailable;
	}
	

});