/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/   

dojo.provide("com.ibm.bpm.social.util.sametime.SametimeBridge");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.declare("com.ibm.bpm.social.util.sametime.SametimeBridge", null, {
	
	bootstrap: false,
	liveNameQueue: [],
	loadcount: 0,
	loadtotal: 2,
	loginAttempts: 0,
	loginAttemptsMax: 5,
	deferredLogin: false,
	
	onLoad: function() {
		if(typeof(window.stproxyConfig)!=="undefined"){
			this.loadSTProxy();
		}
	},

	loadSTProxy: function() {
		console.debug("loadSTProxy::entering...");
		
		this._embedCSS(window.stproxyConfig.server+"/stwebclient/dojo.blue/sametime/themes/WebClientAll.css");

		this._embedJavascript(window.stproxyConfig.server+"/stbaseapi/baseComps.js", this.checkLoadCompletion);
		
		this._embedJavascript(window.stproxyConfig.server+"/stwebclient/widgetsLight.js", this.checkLoadCompletion);

		console.debug("loadSTProxy::exiting...");
	},
	
	_embedCSS: function(url){
		var linkElement=document.createElement("link");
		linkElement.rel="stylesheet";
		linkElement.type="text/css";
		linkElement.href=url;
		document.getElementsByTagName("head")[0].appendChild(linkElement);
	},

	_embedJavascript: function(url, callback){
		console.debug("_embedJavascript::entering... with url:",url);
		var script=document.createElement("script");
		script.type="text/javascript";
		if (script.readyState){  //Internet Explorer
			script.onreadystatechange = dojo.hitch(this, function(){
				if (script.readyState == "loaded" || script.readyState == "complete"){
					 script.onreadystatechange = null;
					 dojo.hitch(this, callback)();
				}
			});
		} else {  //Everyone else
			script.onload = dojo.hitch(this, callback);
		}
		script.src=url;
		document.getElementsByTagName("head")[0].appendChild(script);
		console.debug("_embedJavascript::exiting...");
	},
	
	processQueuedLiveNames: function() {
		console.debug("processQueuedLiveNames::entering...");
		var x = 0;
		while(x = this.liveNameQueue.pop())
		{
			this.createLiveName(x.shortName,x.domNode);
		}
		console.debug("processQueuedLiveNames::exiting...");
	},
	
	checkLoadCompletion: function() {
		console.debug("checkLoadCompletion::entering...");
		this.loadcount++;
		if(this.loadcount==this.loadtotal){
			this.bootstrap = true;

			if(typeof(window.stproxy)!=="undefined" && this.deferredLogin!==false){
				this.loginByShortname(this.deferredLogin);
			}
		}
		console.debug("checkLoadCompletion::exiting...");
	},
	
	loginCurrentUser: function(){
		
		var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
		this.loginByShortname(user.userName);
		/*
		dojo.xhrGet({
			url: "/rest/bpm/wle/v1/user/current?parts=none",
			handleAs: "json",
			sync: true,
			headers: {
				"Content-Type": "application/json", 
				"Accept":"application/json"
			},
			load: dojo.hitch(this, function(data, xhrArgs){
				this.loginByShortname(data.data.userName);
			})
		});
		*/
	},
	
	loginByShortname : function (shortName) {
		console.debug("loginByShortname::entering...", ["shortName", shortName]);
		if(this.bootstrap==false){
			console.debug("loginByShortname deferring login");
			this.deferredLogin = shortName;
			return;
		}
		
		this.userName = shortName;
		this.loginAttempts++;
		if(!window.stproxy && this.loginAttempts <= this.loginAttemptsMax)
		{
			console.debug("ST not ready, returning early...");
			setTimeout(dojo.hitch(this, function(){
				this.loginByShortname(shortName);
			}), 3000);
			return;
		}
				
		/* 
		window.stproxy.login.loginByPassword(
			user,
			password,
			window.stproxy.awareness.AVAILABLE,
			'I am available',
			dojo.hitch(this, this.loginOK), 
			dojo.hitch(this.loginFailed)
		);
		*/
		
		/* */
		window.stproxy.login.loginByToken(
			shortName,
			window.stproxy.awareness.AVAILABLE,
			"",
			dojo.hitch(this, this.loginOK), 
			dojo.hitch(this, this.loginFailed)
		);
		
		console.debug("loginByShortname::exiting...");
	},
		
	loginOK: function(){
		console.debug("Login succeeded...");
		this.processQueuedLiveNames();
	},
		
	loginFailed: function(){
		console.error("Login failed...");
	},

	createLiveName : function (shortName,domNode) {
		if(this.bootstrap == false) {
			this.liveNameQueue.push({'shortName' : shortName, 'domNode': domNode});
			return;
		}

		//Production code would perform shortName to fullName User Cache lookup
		var userData = com.ibm.bpm.social.util.common.Util.getUser(shortName);
		
		if(typeof(domNode) == 'string') {
			//var ln = new sametime.LiveName({'userId': this.userHash[shortName] },dojo.byId(domNode));
			var ln = new sametime.LiveName({'userId': userData.emailAddress },dojo.byId(domNode));
			dojo.style(ln.domNode , {display : "inline",color : "#B9C1ED", cursor : "pointer"});
			ln.activate();
		} else {
			//var ln = new sametime.LiveName({'userId': this.userHash[shortName] },domNode);
			var ln = new sametime.LiveName({'userId':userData.emailAddress },domNode);
			dojo.style(ln.domNode , {display : "inline", color: "#B9C1ED", cursor : "pointer"});
			ln.activate();
		}
	}
});