// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.ui.Avatar");


dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dojox.cometd");

dojo.require("com.ibm.bpm.social.widgets.common.BizCard");
dojo.require("com.ibm.bpm.social.util.common.Log");


dojo.declare("com.ibm.bpm.social.util.ui.Avatar", [dijit._Widget,
                                                   dijit._Templated], {

  templateString: "",
  templatePath: dojo.moduleUrl("com.ibm.bpm.social.util.ui", "templates/Avatar.html"),

  _onlineTracking: null,
  _userName: null,
  _useBizCard: false,

  /**
   * Initialize the object. Set the user object.
   * 
   * @param {String} userName : username of the user object
   */
  constructor: function constructor_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  postCreate: function postCreate_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //localization
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
       
    // Blank image must be set to support background image on the <img> tag
    this.img.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    
    // Accessibility   
    dijit.setWaiRole(this.domNode, "img");
    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the user name.
   * 
   * @param {String} userName : username of the user object
   * @param {String} cssClass: css class for the image, as there are different sizes for example
   *        "bpm-social-default-avatar-medium"
   * @param {Boolean} useBizCard: not all avatars might use the Business Card functionality
   */
  setUser: function setUser_Avatar(user, cssClass, useBizCard){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
   
    dojo.addClass(this.img, cssClass);
    this._user = user;
    this._useBizCard = useBizCard;
    this._cssClass = cssClass;

    this._setUser();   

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  setCSSClass: function setCSSClass_Avatar(CSSClass){
    dojo.addClass(this.img, CSSClass);
  },


  /**
   * Remove the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  unsetCSSClass: function unsetCSSClass_Avatar(CSSClass){
    dojo.removeClass(this.img, CSSClass);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  _setUser: function _setUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // set User
    if (this._user !== null) {
      this.img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this._user["userName"]);
      this.img.alt = this._user["fullName"];
      
      // Accessibility         
      dijit.setWaiState(this.domNode, "label", this._user["fullName"]);

      //
      // Support for Online Awareness
      //      

      if (typeof OnlineTracking === "function") {
        this._deregisterUser();
        this._onlineTrackingConnects = [];
        var myself = this;
        this._onlineTracking = new OnlineTracking(this._user["userName"]);
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOnline", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.addClass(myself.onlineIcon, "online");
          myself.onlineIcon.title=myself._nlsResources["onlinetrackingTooltip"];    
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"] + ". " + myself._nlsResources["onlinetrackingTooltip"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOffline", function(){
          dojo.removeClass(myself.onlineIcon, "online");
          dojo.addClass(myself.onlineIcon, "offline");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onConnectionBroken", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.removeClass(myself.onlineIcon, "online");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        dojox.cometd.batch(function(){
          // Subscribe to appropriate channels
          myself._onlineTracking.activate();
        });

      }
      if (this._useBizCard) { 
    	com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.img, this._user);
    	
    	// Accessibility    	
    	dijit.setWaiState(this.img, "labelledby", this.id);  		
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy all connect from online tracking and also the online tracking object. After that a new
   * 
   * @returns {void} nothing
   */
  _deregisterUser: function _deregisterUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (this._onlineTrackingConnects) {
      for ( var i = 0; i < this._onlineTrackingConnects.length; i++) {
        dojo.disconnect(this._onlineTrackingConnects[i]);
      }
      this._onlineTrackingConnects = null;
    }
    if (this._onlineTracking != null) {
      this._onlineTracking.unregister();
      this._onlineTracking = null;
    }
    if (this._useBizCard) {
      com.ibm.bpm.social.widgets.common.BizCard.unwrapImage(this.img);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy class object
   * 
   * @returns {void} nothing
   */
  destroy: function destroy_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._deregisterUser();
    this.inherited(arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }

});
