/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect");

//
// -- Class EventQueue
//
dojo.declare("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect", null,
{
  _blankPageURL: dojo.moduleUrl("com.ibm.bpm.social.util.ui", "templates/blankPage.html"),   
  
  _isInDialog: false,
  
  constructor: function(/*string*/ iframePrefix, isInDialog)
  {
    this._lastShimmeredDomNode = null;
    this._isInDialog = isInDialog;
    
    this.IFramePool_size = 3;
    this.IFramePool_idPrefix = iframePrefix;
    this.IFramePool_freeIFrameName = iframePrefix + "FreeOne";
  },

  
  /**
   * Reapplies the background effect on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the effect reapply if the previous size was the same.
   */
  reapplyEffect: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
	  this.reapplyShimmersOn(domNode, optimize);
  },
  
  /** 
   * creates the background effect
   */
  createEffect: function(domNode)
  {
	  this.createShimmer(domNode);
  },
  
  /** 
   * removes the background effect
   */  
  removeEffect: function(domNode) 
  {
	  this.removeShimmer(domNode);
  },  
  
  /**
   * Reapplies a shimmer on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the shimmer reapply if the previous size was the same.
   */
  reapplyShimmersOn: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
    // To optimize, compare with latest size.
    if (!optimize || !this._lastShimmeredDomNode || 
        domNode.offsetWidth != this._lastShimmeredDomNode.width || domNode.offsetHeight != this._lastShimmeredDomNode.height)
    {
      // Remove possible previous shimmer and add a new one with the new size.
      this.removeShimmer(domNode);
      this.createShimmer(domNode);
      
      // Remember the current this.resultsDiv size.
      this._lastShimmeredDomNode = {width:domNode.offsetWidth, height:domNode.offsetHeight};
    }
  },
  
  // @deprecated - use createEffect
  createShimmer: function(domNode)
  {
    this._log("createShimmer() - id: " + domNode.id);
    this._createShimmer_0(domNode);
    //this._createShimmerDashboard(domNode);
    //this._createGenericObjShimmer({domNode:domNode});
  },
  
  // @deprecated - use removeEffect  
  removeShimmer: function(domNode) 
  {
    this._log("removeShimmer() - id: " + domNode.id);
    this._removeShimmer_0(domNode);
    //this._removeShimmerDashboard(domNode);
    //this._removeGenericObjShimmer({domNode:domNode});
  },
  
  
  //////////////////////////////////////////////////////////////////////////////////////////////////
  // IMPLEMENTATION: Hand made version. The iframe appears OVER the div.
  //////////////////////////////////////////////////////////////////////////////////////////////////
  
  _createShimmer_0: function(domNode)
  {
    // LL
    var objParent = domNode.parentNode;    
    if(objParent){  
      try{
        objParent.removeChild(domNode);
        objParent.appendChild(domNode);
      }catch(err){
      }
    }  
    
    // Look for existing shimmer.
    var shimmerId = this._getShimmerId(domNode);
    var shimmer = document.getElementById(shimmerId);
    if(shimmer)
    {
      this.removeShimmer(domNode);
    }
    
    // Create new shimmer (need to use iframe, if use DIV does not work on Lotus Forms)
    shimmer = document.createElement('iframe');
    shimmer.id = shimmerId;
    shimmer.src = this._blankPageURL;
    //shimmer.src = "javascript:'<html></html>';"
    
//    var opacityVal = 0;//0 [0..10]
//    shimmer.style.opacity = opacityVal/10;
//    shimmer.style.filter = 'alpha(opacity=' + opacityVal*10 + ')';
    
    // Set sizes
    var inWidth = domNode.offsetWidth;
    var inHeight = domNode.offsetHeight;
    var inTop = this._getPositionTop(domNode);
    var inLeft = this._getPositionLeft(domNode);
    
    shimmer.style.position='absolute';
    shimmer.style.width=inWidth+"px";
    shimmer.style.height=inHeight+"px";
    shimmer.style.top=inTop+"px";
    shimmer.style.left=inLeft+"px";

    // defect 108900; account specifically for IE8
	//if (dojo.isIE == 8) {
	//	shimmer.style.zIndex='1'; //Defect 109956 Setting this to 1 caused the users to be hidden in IE8
	//							  // in the dropdown, had to backout this fix for this defect.
	//} else if (dojo.isIE && dojo.isIE < 8) {
    if (dojo.isIE) {
		shimmer.style.zIndex='0';
	} else {
		shimmer.style.zIndex='-1';
	}
    
    shimmer.setAttribute('frameborder','0');

    // Trick: Add the shimmer at the end or beginning, depending if this shimmer is used inside or not of a dialog.
    if (this._isInDialog)
    {
      document.body.appendChild(shimmer);
    }
    else
    {
      document.body.insertBefore(shimmer, document.body.firstChild);
    }

  
    this._log("Shimmer added to document body.");
  },
  
  
  /**
   * Local implementation.
   */
  _removeShimmer_0: function(domNode)
  {
    var shimmerId = this._getShimmerId(domNode);
    try 
    {
      var shimmer = document.getElementById(shimmerId);
      if (shimmer) 
      {
        document.body.removeChild(shimmer);
        this._log("Shimmer removed from document body.");
      }
    } 
    catch(e) 
    {
      this._log(e);
    }
  },
  
  
  _getShimmerId: function(domNode)
  {
    var shimmerId = (domNode.id) ? (domNode.id + "_shimmerutil_shimmer_") : "_unnamed_shimmerutil_shimmer_";
    return shimmerId;
  },
  
  
  _getPositionLeft: function(node)
  {
      var curleft = 0;
      if (node.offsetParent) 
      {
        while (1) 
        {
          curleft += node.offsetLeft;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
      } 
      else if (node.x) 
      {
          curleft += node.x;
      }
      return curleft;
  },
  
  
  _getPositionTop: function(node)
  {
    var curtop = 0;
    if (node.offsetParent) 
    {
        while (1) 
        {
          curtop += node.offsetTop;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
    } 
    else if (node.y) 
    {
        curtop += node.y;
    }
    return curtop;
  },
  
  _log: function( message )
  {
    //console.log("ShimmerUtil (" + this.IFramePool_idPrefix + "): " + message);
  }
}
);

