dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil");

//
// -- Collection utilities.
//

/**
 * Sorts the specified array of objects using the objects field named as specified by sortField.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy = function( array, sortField ) /*void*/
{
  // Define sorter object.
  var sorter = {
    sortBy: function( a,b)
    {
      if ( a[sortField] == b[sortField] )
        return 0;
      else
        return (a[sortField] < b[sortField]) ? -1 : 1;
    }
  };

  // Sort.
  array.sort( sorter.sortBy );
};



/**
 * Filters the specified array.
 *
 * Creates a new array containing the elements in the original array that evaluates to true when applied the filter.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.filterArray = function( /*Array*/ array, booleanFilter ) /*Array*/
{
  var filtered = new Array();
  for (var i = 0; i<array.length; i++)
  {
    var context = this;
    if ( booleanFilter( array[i], context ) )
    {
      filtered.push( array[i] );
    }
  }

  return filtered;
};


/**
 * Partitions the list based on a specified field.
 * Each time the field value changes from the previous item in the list, the groupHandler is called.
 * For each element in the group, the elementHandler is called.
 *
 * The parameter array should be previously sorted by the specified field.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayInGroupsBy = function( array, field, groupHandler, unassignedGroupMessage )
{
  var currentGroupName = null;
  var currentGroup = null;
  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize group if necessary.
    if ( currentGroupName == null || currentGroupName != fieldValue )
    {
      if ( currentGroup != null )
      {
        // Notify of finished group.
        groupHandler( currentGroupName, currentGroup );
      }
      currentGroupName = fieldValue;
      currentGroup = new Array();
    }

    // Process element.
    currentGroup.push( elem );
  } // for

  // Process last group, if any.
  if ( currentGroup != null )
  {
    groupHandler( currentGroupName, currentGroup );
  }
};


/**
 * Creates a map from an array.
 * The returned map is JSON object with the keys being the values of the specified field
 * in the mapped elements.
 *
 * The parameter array will be sorted as a side effect.
 *
 * @return A not nullable "map", from string to objects originally in array.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayBy = function( array, field, unassignedGroupMessage )
{
  com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy( array, field );
  var map = new String();

  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize bucket if necessary.
    var bucket = map[fieldValue];
    if ( bucket == null )
    {
      // No bucket yet. Create it.
      bucket = new Array();
      map[fieldValue] = bucket;
    }

    // Add element to bucket.
    bucket.push( elem );
  } // for

  return map;
};




/** 
 * Makes all elements in this array unique, i.e., removes all duplicate elements from array.
 * Assumes array is already sorted. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.unique = function(/*Array*/ array, compare)
{
  if (typeof(compare) == 'undefined') 
  {
    compare = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend;
  }
  var dst = 0;  // Destination for elements
  var src = 0;  // Source of elements
  var limit = array.length - 1;
  while (src < limit) 
  {
    while (compare(array[src], array[src + 1]) == 0) 
    {
      if (++src == limit) 
      {
        break;
      }
    }
    array[dst++] = array[src++];
  }
  if (src == limit) 
  {
    array[dst++] = array[src];
  }
  array.length = dst;
}


/** 
 * Adds all the elements in the specified arrays to destArray. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.addAll = function(/*Array*/ destArray, /*Array[]*/ arrays) 
{
  for (var a = 0;  a < arrays.length;  a++) 
  {
    arr = arrays[a];
    for (var i = 0;  i < arr.length;  i++) 
    {
      destArray.push(arr[i]);
    }
  }
}


/** 
 * Compares two objects using built-in JavaScript operators. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend = function(a, b) 
{
  if (a < b)
    return -1;
  else if (a > b)
    return 1;
  return 0;
}



/**
 * Will search the array until it matches the search critera.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf = function( /*Array*/ array, element /*, from*/ ) /*Number*/
{
  var length = array.length;

  var from = Number(arguments[2]) || 0;
  from = (from < 0) ? Math.ceil(from) : Math.floor(from);
  if (from < 0)
    from += length;

  for (; from < length; from++)
  {
    if (array[from] == element)
      return from;
  }
  return -1;
};
