/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.RestClient");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.util.RestClient", null,
{
  REASON_ADMINISTRATOR : 7,
  bpmServiceRoot: window.endpointURLObject.bpmRestJS,
  restgatewayerror: "Error 404: REST Service Gateway:",
  _queryIndex:0,

  constructor: function(bpmServiceRoot, iContext)
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    // Normalize service urls.
    if (bpmServiceRoot) {
        this.bpmServiceRoot = bpmServiceRoot;
    }
//PKL -- TODO bug
    if (iContext)
    {
      this.iContext= iContext;
    }

    this.bpmServiceRoot = this._normalizeUrl( this.bpmServiceRoot );
    this.bpmServiceRoot +="/v1";

    com.ibm.bpm.social.util.common.Log.log(2, "Using bpmServiceRoot=" + this.bpmServiceRoot ,"RestClient");

    this._bpmHeaders = {};
    this._bpmHeaders["accept"] = "application/json"; /*  "text/xml"; */
    if (!this.bpmServiceRoot) {
        this._bpmHeaders["content-type"] = "charset=utf-8";
    } else {
        this._bpmHeaders["content-type"] = "application/json";
    }
	
    dojo.requireLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
    
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");

  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByFilter: function( /*String*/ name, /*int*/ offset, /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes ) 
  {
   //return this._searchBPMUsers( "filter=" + name, offset, size, attributes );
     return this._searchBPMUsers( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, attributes );
  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByName: function( /*string*/ name, /*int*/ offset, /*int*/ size, /*boolean*/ includeGroups, /*boolean*/ includeUsers, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly ) 
  {
    var searchGroups = includeGroups ? (includeGroups == true) : false;	// Default false
    var searchUsers = (includeUsers != null && includeUsers != undefined) ? (includeUsers == true) : true;	// Default true
    var cnsAll = [];
    
	this.errormsg = false;
	if (searchUsers)
	{
	    cnsAll = this.searchBPMByFilter( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly );	
    }
	this.errormsg = false;
	
    // Cut any extra not requested elements.
    if ( cnsAll.length > size )
    {
      cnsAll.length = size;
    }

    return cnsAll;
  },

  /**
   * Generic BPM query. Searchs BPM for users matching the search
   * attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPMUsers: function( /*String*/ searchAttributes, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    var includeAttributes = "";
    var extractor = "";
    return this._searchBPM( "users", searchAttributes, includeAttributes, extractor, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, loadCallback );
  },

  /**
   * Generic BPM query. Searchs BPM for users/groups matching the
   * search attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPM: function( /*String*/ entity, /*String*/ searchAttributes, /*String*/ attributes, /*function*/ extractor, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "Querying BPM for " + entity + ":'" + "filter=" +searchAttributes + "', maxresult="+size+", assignTaskidFilter="+assignTaskidFilter+" , collabTaskidFilter="+collabTaskidFilter+" ,namesonly: "+namesonly+" , using encodeURIComponent on "+searchAttributes ,"RestClient");

    var assignTaskString = "";
    if (assignTaskidFilter != null) {
        assignTaskString = "&assignTaskidFilter="+assignTaskidFilter;
    }
    var collabTaskString = "";
    if (collabTaskidFilter != null) {
        collabTaskString = "&collabTaskidFilter="+collabTaskidFilter;
    }
    var namesonlyString = "&namesonly=true";//performance
    if (namesonly != 'undefined' && (namesonly == false || namesonly == "false")) {
        namesonlyString = "";
    }
    //encode special character of userids in searchAttributes
    var url = this.bpmServiceRoot + "/" + entity + "?" + "filter=" +
              (window.encodeURIComponent(searchAttributes)) +
              "&maxresult=" + size +
              assignTaskString +
              collabTaskString +
              namesonlyString +
              "&parts=none";

    var results = []; // default value
    var self = this;
    var okHandler = function( /*String*/data, ioArgs )
    {
      com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM in okHandler, result is " + ioArgs.xhr.status,"RestClient");
      if (data)
      {
        // Collect desired information from results
        //console.log("_searchBPM data: "+dojo.fromJson(data));
        var d = dojo.fromJson(data);
        if (d.data && d.data.users) {        
            for (var i in d.data.users) {
                var d2 = d.data.users[i];
                if (!d2.isDisabled) {
                    com.ibm.bpm.social.util.common.Log.log(2,"...fullName: "+d2.fullName+", userName: " + d2.userName + ", userID: " + d2.userID,"RestClient");
                    results.push( 
                        {"cn":"",
                        "userID":d2.userID,
                        "fullName":d2.fullName,
                        "userName":d2.userName} 
                    );
                }
                //cn,userID,fullName,userName
            }
        }
      }
      //console.log("_searchBPM loadCallback: "+loadCallback); == undefined
      if (loadCallback) {
          loadCallback(results, ioArgs);
      }
    };
    var errorHandler = function( /*String*/data, ioArgs )
    {
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM In Error Handler, status is " + ioArgs.xhr.status,"RestClient");
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM responsetext " + ioArgs.xhr.responseText,"RestClient");
       var msg; 
       if ((ioArgs.xhr.status != 404) || ((ioArgs.xhr.status == 404) && (ioArgs.xhr.responseText.indexOf(self.restgatewayerror) >= 0)))  {
          if (!self.errormsg) {		     
             self.errormsg = true;
             var msg, msg2=null;
             if (ioArgs.xhr.responseText != null) {                 
                 var r1 = ioArgs.xhr.responseText.split(",");
                 for (var j=0; j<r1.length; j++) {
                      var r2 = r1[j].split(':');
                      if (r2[0] == '"errorMessage"') {
                          msg2 = r1[j].substr(16,r1[j].length-17);
                          break;
                      }
                 }
                 if (msg2 == null) {
                     msg2 = ioArgs.xhr.responseText;//responseText did not contain errorMessage, cannot format it
                 }
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:msg2} );
             } else {
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:ioArgs.xhr.status} );
             }
             dojo.publish("com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred");
             com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM publish errorOccurred topic","RestClient");
             dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");
             com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(msg);             
          } 
        }
    };

    this._executeGet( url, okHandler, errorHandler, (loadCallback == null), this._bpmHeaders );

    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_searchBPM(" + searchAttributes + "," + offset + "," + size + ")=" + results.length ,"RestClient");
    com.ibm.bpm.social.util.common.Log.logEntry(2, "results: " + dojo.toJson( results ) ,"RestClient");
    return results;
  },

  //
  // -------- BPC
  //

  /**
   * Calculates the potential owners of the specified task.
   * <p>
   * Synchronic call.
   */
  queryTaskPotentialOwners: function( /*String*/ taskId, systemId, okHandler, errorHandler ) /*void*/
  {    
    var url = this.bpcServiceRoot + "/task/" + taskId + "/role/REASON_POTENTIAL_OWNER";
    if (systemId) {
        url+= "?systemID="+systemId;
    }
    this._executeGet( url, okHandler, errorHandler, true );
  },

  //
  // -------- Helpers
  //


  _executeGet: function( resourcePath, okHandler, errorHandler, sync, headers )
  {
  	com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    this._queryIndex++;

    //var encodedUrl = encodeURI( resourcePath );
    var encodedUrl = resourcePath;
    
    // Rewrite only if needed.
    var needsRewrite = (encodedUrl.indexOf("http") == 0);

    var url = (needsRewrite && this.iContext) ? this.iContext.io.rewriteURI( encodedUrl ) : encodedUrl;
    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_executeGet(" + url + ")" ,"RestClient");

    if ( !sync )
    {
      sync = false;
    }
    
    var request =
    {
      url: url,
      load: okHandler,
      error: errorHandler,
	  timeout: 5000,
      sync: sync
    };

    if (headers)
    {
      request.headers = headers;
    }

    dojo.xhrGet(request);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");
  },


  _getOperationIdString: function()
  {
    return "[" + this._queryIndex + "]";
  },


  /**
   * If url is not null, tries to remove the trailing '/' character and trim spaces.
   * If null, returns null.
   */
  _normalizeUrl: function( /*String*/ url ) /*String*/
  {
    var newUrl = url;
    if ( url )
    {
      url = dojo.trim( url );
      if( url.lastIndexOf("/") == url.length - 1 )
      {
        newUrl = url.substring(0, url.length - 1);
      }
    }

    return newUrl;
  },


  _log: function( message )
  {
    com.ibm.bpm.social.util.common.Log.log(2, message ,"RestClient");
  }
}
);

