/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.ActionDropDownHelper");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.ProcessRestHandler");

dojo.require("com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog");
dojo.require("com.ibm.bpm.social.widgets.ActionDropDownMenu");
dojo.require("com.ibm.bpm.social.portal.Topics");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.ActionDropDownHelper",
    [com.ibm.bpm.social.widgets.common._RestWidget],
    {

      processActionsOrder: ["ACTION_CHANGE_INSTANCE_DUEDATE",
                            "ACTION_VIEW_PROCESS_AUDIT",
                            "ACTION_CHANGE_CRITICAL_PATH",
                            "ACTION_VIEW_PROCESS_DIAGRAM"],

      constructor: function(params){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
      },

      _calculateActions: function(task /* object */, availableActions /* object */, callbacks /* callbacks */,
        withCollaborate /* boolean */){
        var result = {
          taskAction: [],
          processAction: [],
          piid: ""
        };
        //
        // Which task actions qualify?
        //
        var ifReassignBackToGroupOK = dojo.indexOf(availableActions, "ACTION_CANCELCLAIM") != -1;
        var ifTransferToUserOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOUSER") != -1;
        var ifModifyPriorityOK = dojo.indexOf(availableActions, "ACTION_UPDATEPRIORITY") != -1;
        var ifModifyDueDateOK = dojo.indexOf(availableActions, "ACTION_UPDATEDUEDATE") != -1;
        var ifCollaborate = dojo.indexOf(availableActions, "ACTION_INVITE") != -1;
        //
        // MODIFY_TASK
        //
        if (ifTransferToUserOK || ifModifyPriorityOK || ifModifyDueDateOK) {
          if (typeof callbacks.modifyTask === "function") {
            result.taskAction.push("MODIFY_TASK");
          }
        }
        //
        //  REASSIGN_BACK_TO_GROUP
        //
        if (ifReassignBackToGroupOK) {
          if (typeof callbacks.reassignBackToGroup === "function") {
            result.taskAction.push("REASSIGN_BACK_TO_GROUP");
          }
        }
        //
        // COLLABORATE
        //
        if (withCollaborate && ifCollaborate) {
          if (typeof callbacks.collaborate === "function") {
            result.taskAction.push("COLLABORATE");
          }
        }
        //
        // VIEW_INSTANCE
        //
        var piid = null;
        if (typeof task["piid"] !== "undefined") {
          piid = task.piid;
        }
        else if (typeof task["PI_PIID"] !== "undefined") {
          piid = task["PI_PIID"];
        }
        if (piid != null) {
          result.piid = piid;
          result.processAction.push("VIEW_INSTANCE");
        }
        return result;
      },

      _addModifyTaskEntry: function(menu /* dijit.Menu */, callback /* function */){
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["task_menu_modify_task"],
          iconClass: "bpm-social-modify-task",
          onClick: callback
        }));
      },

      _addReassignBackToGroupEntry: function(menu /* dijit.Menu */, callback /* function */){
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["task_menu_cancel"],
          iconClass: "bpm-social-reassign-back-to-group",
          onClick: callback
        }));
      },

      _addCollaborationEntry: function(menu /* dijit.Menu */, callback /* function */){
          if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) { // Prevent collaboration menu option if user is using Safari on iPad but allow for Chrome on iPad
            return
          }

          menu.addChild(new dijit.MenuItem({
            label: this._nlsResources["task_menu_collaborate"],
            iconClass: "bpm-social-collaborate",
            onClick: callback
         }));
      },

      _addViewInstanceEntry: function(menu /* dijit.Menu */, piid /* string */, callback /* function */){
        if (typeof callback !== "function") {
          callback = function(){
            dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
              instanceId: piid
            }]);
          };
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_view_instance"],
          iconClass: "bpm-social-view-instance",
          onClick: callback
        }));
      },

      _addModifyInstanceEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var showF = function(process){
            var cddd = new com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog({
              "instanceId": process.piid,
              "dueDate": process.dueDate,
              "callback": function(data){}
            }, dojo.create("div"));
            cddd.show();
          };
          var myself = this;
          if (processObject.dueDate != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              //var xhr = myself._createProcessXHR(piid);
              //xhr.addCallback(showF);

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        //
        // Different labels for side bar and task completion ui, task row.
        // If processObject is entire process object => side bar. 
        //
        var label =
          processObject.executionState != null ? this._nlsResources["process_menu_change_due_date_instance"]
            : this._nlsResources["process_menu_modify_instance"];
        menu.addChild(new dijit.MenuItem({
          label: label,
          iconClass: "bpm-social-modify-instance",
          onClick: callback
        }));
      },

      _addViewAuditHistoryEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            var ahDialog = new com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog({
              'instanceId': process.piid,
              'templateName': process.processTemplateName,
              'instanceName': process.name,
              'proxyUrl': myself.getProxyUrl()
            });
            ahDialog.show();
          };
          if (processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              // var xhr = myself._createProcessXHR(piid);
              // xhr.addCallback(showF);

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["prcesss_menu_view_audit_history"],
          iconClass: "bpm-social-view-audit-history",
          onClick: callback
        }));
      },

      _addCriticalPathEntry: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            var cpmDialog = new com.ibm.bpm.social.widgets.dialogs.CriticalPathDialog({
              'instanceId': process.piid,
              'templateName': process.processTemplateName,
              'instanceName': process.name,
              'proxyUrl': myself.getProxyUrl()
            });
            cpmDialog.show();
          };
          if (processObject.piid != null && processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred
              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }
        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_critical_path_management"],
          iconClass: "bpm-social-critical-path-management",
          onClick: callback
        }));
      },

      _addShowProcessDiagram: function(menu /* dijit.Menu*/, processObject /* object */, callback /* function */){
        var piid = processObject.piid;
        if (typeof callback !== "function") {
          var myself = this;
          var showF = function(process){
            //
            // Load required libraries
            //
            /** ** BEGIN Loading Process Diagram-specific modules *** */
            /* NO LONGER REQUIRED AS OF 70752 - loading moved inside ProcessDiagramDialog widget
              try {
                dojo.registerModulePath('ibm_ilog.diagram', '/ProcessPortal/layers/' + socialversion + '/ibm_ilog/diagram');
                dojo["require"]("ibm_ilog.diagram.ibm_ilog_diagram_base"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('ibm_ilog.diagram', '/WebViewer/widgets/ibm_ilog/diagram');
              }

              try {
                dojo.registerModulePath('com.ibm.bpm.wpd', '/ProcessPortal/layers/' + socialversion + '/com/ibm/bpm/wpd'); //Needed, in addition to /com/ibm/bpm, for img & css files
                dojo["require"]("com.ibm.bpm.wpd_base"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('com.ibm.bpm.wpd', '/WebViewer/widgets/com/ibm/bpm/wpd');
              }

              try {
                dojo["require"]("com.ibm.bpm.social_diagram"); //Alternate syntax used to avoid dojo build detection
              }
              catch (e) {
                console.warn(e);
                dojo.registerModulePath('com.ibm.bpm.social', '/ProcessPortal/com/ibm/bpm/social');
              }
              */
            /** ** END Loading Process Diagram-specific modules *** */
            //
            // Open process diagram
            //
            dojo.require("com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog");
            var pdd = new com.ibm.bpm.social.widgets.dialogs.ProcessDiagramDialog({
              "instanceId": process.piid,
              "title": process.processTemplateName,
              "subTitle": process.name
            }, dojo.create("div"));
            pdd.show();
          };
          if (processObject.processTemplateName != null && processObject.name != null) {
            // All properties are already available => process instance need not be retrieved
            callback = function(){
              showF(processObject);
            };
          }
          else {
            callback = function(){
              // Chaining of two xhr calls using deferred

              myself.processRestHandler.getProcess(piid, showF, myself._onXhrErrors, true, "header");
            };
          }

        }
        menu.addChild(new dijit.MenuItem({
          label: this._nlsResources["process_menu_show_process_diagram"],
          iconClass: "bpm-social-view-process-diagram",
          onClick: callback
        }));
      },

      _addLoadingEntry: function(menu /* dijit.Menu */){
        var item = new dijit.MenuItem({
          label: this._nlsResources["KEY_Loading"]
        });
        menu.addChild(item);
        menu._loadingEntry = item;
      },

      _removeLoadingEntry: function(menu /* dijit.Menu */){
        if (typeof menu._loadingEntry === "object") {
          //menu.removeChild(menu._loadingEntry);
          //menu._loadingEntry = null;
          dojo.style(menu._loadingEntry.domNode, "display", "none");

        }
      },

      _addSeparatorEntry: function(menu /* dijit.Menu */){
        if (menu.containerNode.children.length) {
          if (!dojo.hasClass(menu.containerNode, "actionDropDown")) {
            dojo.addClass(menu.containerNode, "actionDropDown");
          }
          menu.addChild(new dijit.MenuSeparator());
        }
      },

      _addAdHocEntry: function _addAdHocEntry_ActionDropDownHelper(menu, process, action, dialog, onCompletedCallback,
        onCanceledCallback, onMessageClosedCallback){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var piid = process.piid;
        var myself = this;
        if (dialog === undefined) {
          // Default
          dialog = false;
        }
        var actionId = action["ID"];
        var displayName = action["displayName"];
        menu.addChild(new dijit.MenuItem({
          label: action.displayName,
          iconClass: "bpm-social-run-ad-hoc-event",
          onClick: function(){

            var params =
              {
                processID: piid,
                actionID: actionId,
                load: function(response){
                  if (response.data && response.data.tkiid && response.data.tkiid.length > 0) {
                    if (dialog) {
                      com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog.showCoach(response.data.tkiid[0],
                        onCompletedCallback, onCanceledCallback);
                    }
                    else {
                      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
                        taskId: response.data.tkiid[0]
                      }]);
                    }
                  }
                  else {
                    var message = myself._nlsResources["process_ad_hoc_event_started"];
                    message = dojo.string.substitute(message, [displayName]);
                    com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, null, onMessageClosedCallback);
                  }
                },
                error: function(){
                  var message = myself._nlsResources["process_ad_hoc_event_failed"];
                  message = dojo.string.substitute(message, [displayName]);
                  com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message);
                }
              };
            myself.processRestHandler.runAdHocAction(params);
          }
        }));
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createDropDownForTask: function _createDropDownForTask_ActionDropDownHelper(anchor /* DomNode */,
        task /* object */, availableActions /* object */, callbacks /* object */, withCollaborate /* boolean */,
        omitViewInstance /* boolean */){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //
        // Calculate the actions
        // 
        var actions = this._calculateActions(task, availableActions, callbacks, withCollaborate);
        if ((actions.taskAction.length + actions.processAction.length === 0) && !com.ibm.bpm.social.util.common.Util.isIPad) {
          // In case there is a process instance associated VIEW_INSTANCE should be available
          //However, on the iPad, we use the context menu to display the link for read-only Activity Details which all users have access to.
          //We account for this case by extending the logic to validate when we are on the iPad and in the Diagram view.
          return null;
        }
        // There are actions for the drop down
        var menu = new com.ibm.bpm.social.widgets.ActionDropDownMenu({
          leftClickToOpen: true
        });

        if (com.ibm.bpm.social.util.common.Util.isIPad && omitViewInstance == true) {
          menu.addChild(new dijit.MenuItem({
            label: this._nlsResources["process_diagram_activity_details"],
            iconClass: "bpm-social-cancel-enrollment",
            onClick: callbacks.viewActivityDetails
          }));
        }

        for ( var i = 0; i < actions.taskAction.length; i++) {
          switch (actions.taskAction[i]) {
            case "MODIFY_TASK" :
              // Create Modify Task entry       
              this._addModifyTaskEntry(menu, callbacks.modifyTask);
              break;
            case "REASSIGN_BACK_TO_GROUP" :
              // Create Reassign back to group entry
              this._addReassignBackToGroupEntry(menu, callbacks.reassignBackToGroup);
              break;
            case "COLLABORATE" :
              // Create Collaboration entry
              this._addCollaborationEntry(menu, callbacks.collaborate);
              break;
            default:
              com.ibm.bpm.social.util.common.Log.log(1, "Unexpected action: " + actions.taskAction[i]);
              break;
          }
        }

        if (omitViewInstance === undefined) {
          //Default is to include 'View Instance' link
          omitViewInstance = false;
        }

        if (actions.piid && omitViewInstance === false) {
          // The "View Instance" entry is always possible if there is an associated instance, unless we are viewing the Process Diagram
          this._addSeparatorEntry(menu);
          this._addViewInstanceEntry(menu, actions.piid, callbacks.viewInstance);
        }
        menu.bindDomNode(anchor);

        // a11y; dijit.Menu behaves like a context menu, which means it is opened using Shift+F10, our context menu behaves like a button and should react on Enter
        this._connectEnter(menu, anchor);

        com.ibm.bpm.social.util.common.Log.logExit(2);

        return menu;
      },

      /**
       * connects the ENTER key to open the Menu, this normally does not happen, as a dijit.Menu only reacts
       * on Shift+F10
       * 
       * @param menu The menu that should be opened on ENTER.
       * @param anchor The DOM node that should be used as anchor
       */
      _connectEnter: function _connectEnter_ActionDropDownHelper(/*dijit.Menu*/menu, /*DOMElement*/anchor){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (menu && menu._bindings && menu._bindings[0] && menu._bindings[0].connects) {
          menu._bindings[0].connects.push(dojo.connect(anchor, "onkeydown", menu, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              this._scheduleOpen(evt.target, menu._bindings[0].iframe);
            }
          }));
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1, "Cannot connect ENTER key to Menu anchor.");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Adds the process related entries.
       * 
       * @param menu The menu to which the entries are added.
       * @param result The data object of the process/<process id>/actions call.
       * @param process The process object (having at least a piid property).
       * @param dialog Whether a dialog for ad hoc event is necessary.
       * @param callbacks A callback object that is used in case special callback handler are needed.
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       */
      _addProcessEntries: function _addProcessEntries_ActionDropDownHelper(menu /* menu */, result /* object */,
        process /* object */, dialog /* boolean */, callbacks /* object */, omitViewInstance /*boolean*/,
        omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
            omitCriticalPath = false;
        }

        var actions = result.actions;
        var standardActions = result.standardActions;
        var i;
        if (standardActions &&
          standardActions.length > 0 &&
          (dojo.indexOf(standardActions, "ACTION_CHANGE_INSTANCE_DUEDATE") > -1 ||
            dojo.indexOf(standardActions, "ACTION_VIEW_PROCESS_AUDIT") > -1 ||
            dojo.indexOf(standardActions, "ACTION_CHANGE_CRITICAL_PATH") > -1 || dojo.indexOf(standardActions,
            "ACTION_VIEW_PROCESS_DIAGRAM") > -1)) {

          // Separator not needed as in case of task actions there is already the View Instance entry for processes
          // If we only have one MenuItem, it's the "Loading..." MenuItem that is hidden.  We don't need an initial separator in this case.
          if (omitViewInstance === true && menu.getChildren().length > 1) {
            this._addSeparatorEntry(menu);
          }
          for (i = 0; i < this.processActionsOrder.length; i++) {
            var standardAction = this.processActionsOrder[i];
            if (dojo.indexOf(standardActions, standardAction) != -1) {
              switch (standardAction) {
                case "ACTION_CHANGE_INSTANCE_DUEDATE" :
                  // Create Modify Instance entry
                  var modifyInstanceCallback = callbacks.modifyInstance || null;
                  this._addModifyInstanceEntry(menu, process, modifyInstanceCallback);
                  break;
                case "ACTION_VIEW_PROCESS_AUDIT" :
                  // Create View Audit History entry
                  this._addViewAuditHistoryEntry(menu, process, callbacks.viewAuditHistory);
                  break;
                case "ACTION_CHANGE_CRITICAL_PATH" :
                  // Create Critical Path Management entry
                  if (omitCriticalPath !== true) {
                  	this._addCriticalPathEntry(menu, process, callbacks.criticalPathManagement);
                  }
                  break;
                case "ACTION_VIEW_PROCESS_DIAGRAM" :
                  // Process Diagrqamm entry
                  if (omitProcessDiagram !== true) {
                    this._addShowProcessDiagram(menu, process, callbacks.viewProcessDiagram);
                  }
                  break;
                default:
                  com.ibm.bpm.social.util.common.Log.log(1, "Unexpected action: " + standardAction);
                  break;
              }
            }
          }
        }
        if (actions && actions.length > 0) {
          var onCompletedCallback = null;
          if (typeof callbacks.onCompletedCallback === "function") {
            onCompletedCallback = callbacks.onCompletedCallback;
          }
          var onCanceledCallback = null;
          if (typeof callbacks.onCanceledCallback === "function") {
            onCanceledCallback = callbacks.onCanceledCallback;
          }
          var onClosedCallback = null;
          if (typeof callbacks.onMessageClosedCallback === "function") {
            onClosedCallback = callbacks.onMessageClosedCallback;
          }
          this._addSeparatorEntry(menu);
          for (i = 0; i < actions.length; i++) {
            var action = actions[i];
            this._addAdHocEntry(menu, process, action, dialog, onCompletedCallback /* function */,
              onCanceledCallback /* function */, onClosedCallback /* function */);
          }
        }
        //This case should only be encountered on the Process Diagram.  All others will have at least "View Instance" option.
        if (menu.getChildren().length == 1) {
          menu._loadingEntry.set('label', this._nlsResources["no_actions_available"]);
          menu._loadingEntry.set('disabled', true);
          dojo.style(menu._loadingEntry.domNode, "display", "block");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Creates a drop down menu for the action menu. Which can be used for the latter calls.
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @return The dijit.Menu instance which need to be destroyed by the calling class.
       */
      createDropDown: function createDropDown_ActionDropDownHelper(anchor /* DomNode */){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // There are actions for the drop down
        var menu = new com.ibm.bpm.social.widgets.ActionDropDownMenu({
          leftClickToOpen: true
        });

        menu.bindDomNode(anchor);

        // a11y; dijit.Menu behaves like a context menu, which means it is opened using Shift+F10, our context menu behaves like a button and should react on Enter
        this._connectEnter(menu, anchor);

        com.ibm.bpm.social.util.common.Log.logExit(2);

        return menu;
      },

      /**
       * Creates a dijit.Menu for the Task Row.
       * 
       * The following callbacks are excepted:
       * <ul>
       * <li>modifyTask</li>
       * <li>reassignBackToGroup</li>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <ul>
       * 
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @param task The associated task.
       * @param availableActions The availableActions of the task.
       * @param callbacks The callbacks for the actions.
       * @return The dijit.Menu instance which need to be destroyed by the calling class.
       * 
       */
      createDropDownForTaskRow: function(anchor /* DomNode */, task /* object */, availableActions /* object */,
        callbacks /* object */){
        return this._createDropDownForTask(anchor, task, availableActions, callbacks, false /* no collaboration */);
      },

      /**
       * Creates a dijit.Menu for the Task Completion UI.
       * 
       * The following callbacks are excepted:
       * <ul>
       * <li>modifyTask</li>
       * <li>reassignBackToGroup</li>
       * <li>collaborate</li>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <ul>
       * 
       * @param anchor The DomNode to which the menu will be attached.
       * @param task The associated task.
       * @param availableActions The availableActions of the task.
       * @param callbacks The callbacks for the actions.
       * @returns The dijit.Menu instance which need to be destroyed by the calling class.
       */
      createDropDownForTaskCompletion: function(anchor /* DomNode */, task /* object */, availableActions /* object */,
        callbacks /* object */){
        return this._createDropDownForTask(anchor, task, availableActions, callbacks, true /* including collaboration */);
      },

      /**
       * Make sure that the process actions are added if the menu gets focus.
       * 
       * @param menu
       * @param process A process object which need to have at least the piid property
       * @param dialog Whether a dialog for ad hoc event is necessary.
       * @param callbacks {void} nothing
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       * 
       * @return A deferred that provides the result of the process action call on success.
       */
      addProcessActionsOnFocus: function addProcessActionsOnFocus_ActionDropDownHelper(menu /* Menu */,
        process /* object */, callbacks /* object */, omitViewInstance /*boolean*/, omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
          omitCriticalPath = false;
        }
        var myself = this;
        // Provide a Deferred that allow other interessted parties to take part in using the result.
        // Requires that all callback return the result to keep the callback chain.
        var deferred = new dojo.Deferred();
        var piid = process.piid;
        menu._processActionsPending = true;
        menu._loadingEntryAdded = false;
        //
        // Define the logic for processing the result of process/{piid}/action as callback on 
        // a deferred to make it independent of the trigger. 
        // (onFocus or some other circumenstances outside this module)
        //
        deferred.then(function(result){
          // A process action call has been conducted, we do not need another one onFocus.
          menu._processActionsPending = false;
          if (menu._loadingEntryAdded) {
            myself._removeLoadingEntry(menu);
          }
          myself._addProcessEntries(menu, result, process, false, callbacks, omitViewInstance, omitProcessDiagram, omitCriticalPath);
          return result;
        });
        menu.connect(menu, "onFocus", function(){
          // Only call the process/{piid}/actions if it has not been done already.
          if (menu._processActionsPending) {
            menu._loadingEntryAdded = true;
            myself._addLoadingEntry(menu);

            // var xhr = myself._createProcessActionXHR(piid);
            // xhr.then(function(result){
            // // Trigger all required actions              
            // deferred.resolve(result);
            // });

            myself.processRestHandler.getActionsForProcess(piid, function(result){
              deferred.resolve(result);
            }, myself._onXhrErrors, true);

          }
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return deferred;
      },

      /**
       * In contrast to the OnFocus method the process actions are immediately retrieved.
       * 
       * Which is used in the TaskCompletionUI where only one task is under consideration. In the task list
       * are potentially hundreds of items shown for which the process action call cannot be done in advance
       * without degrading the performance. The following callback entries can be used to define callbacks for
       * completion of ad hoc event dialog and for cancelation of an ad hoc event dialog.
       * 
       * <ul>
       * <li>onCompletedCallback</li>
       * <li>onCanceledCallback</li>
       * </ul>
       * 
       * In addition to that the following entries are also used
       * 
       * <ul>
       * <li>modifyInstance</li>
       * <li>viewAuditHistory</li>
       * <li>criticalPathManagement</li>
       * <li>viewProcessDiagram</li>
       * <ul>
       * 
       * 
       * @param menu
       * @param process A process object which need to have at least the piid property
       * @param callbacks
       * @param dialog Whether the coaches for ad hoc event should be shown in a dialog. {void} nothing
       * @param omitViewInstance Defaults to false. Determine whether to include the 'View Instance' menu item
       * @param omitProcessDiagram Defaults to false. Determine whether to include the 'View Process Diagram'
       *        menu item or link
       * @param omitCriticalPath Defaults to false. Determine whether to include the 'Critical Path' menu item
       */
      addProcessActions: function addProcessActions_ActionDropDownHelper(menu /* Menu */, process /* object */,
        callbacks /* object */, dialog /* boolean */, omitViewInstance /*boolean*/, omitProcessDiagram /*boolean*/, omitCriticalPath /*boolean*/){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var piid = process.piid;
        if (dialog === undefined) {
          // Default is no dialog
          dialog = false;
        }
        if (omitViewInstance === undefined) {
          //Default is to include View Instance link
          omitViewInstance = false;
        }
        if (omitProcessDiagram === undefined) {
          //Default is to include Process Diagram link
          omitProcessDiagram = false;
        }
        if (omitCriticalPath === undefined) {
            //Default is to include Critical Path link
          omitCriticalPath = false;
        }
        var myself = this;
        myself._addLoadingEntry(menu);
        // Immediately retrieve the process actions

        var async = true;
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          async = false;
        }
        myself.processRestHandler.getActionsForProcess(piid,
          function(result){
            if ((result.standardActions && result.standardActions.length > 0) ||
              (result.actions && result.actions.length > 0)) {
              dojo.publish(com.ibm.bpm.social.portal.Topics.PROCESS_ACTIONS_AVAILABLE, [result]);
            }
            myself._removeLoadingEntry(menu);
            myself._addProcessEntries(menu, result, process, dialog, callbacks, omitViewInstance, omitProcessDiagram, omitCriticalPath);
            return result;
          }, myself._onXhrErrors, async);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _onXhrErrors: function _onXhrErrors_ActionDropDownHelper(data, xhrArgs){
        console.debug(["ActionDropDownHelper experienced an error.",
                       data,
                       xhrArgs]);
      },

      statics: {
        _instance: null,
        _getInstance: function(){
          if (this._instance === null) {
            this._instance = new com.ibm.bpm.social.widgets.ActionDropDownHelper();
          }
          return this._instance;
        }
      }
    });


com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance = function(){
  return com.ibm.bpm.social.widgets.ActionDropDownHelper.prototype.statics._getInstance();
};