/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.ExposedItemsList");

dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");

dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");

/* ExposedItemList 
*/
dojo.declare(
    "com.ibm.bpm.social.widgets.ExposedItemsList",
    [com.ibm.bpm.social.widgets.common._RestWidget],
    {
      constructor: function(){
        this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		this.hasTips = false;  // this will be set during processing. false means this is process server;  true means this is process center
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.trackSnapshotsOnIPad=false;
		this.scoreboardMessageKeyMapping= {
			"scoreboard.MyPerf.name": "pmmyperformance",
			"scoreboard.MyTeamPerf.name": "pmmyteamperformance",
			"scoreboard.ProcPerf.name": "pmprocessperformance",
			"scoreboard.SLAView.name": "pmslaoverview",
			"Ad Hoc Reports": "adhocreports"
		  };
		this.workDashIdentifier="BPM_WORK";
      },

      getData: function(exposedItemsList/*response from exposed items REST query: data.data.exposedItemsList*/, typeArg/*array of strings*/){
        var exposedItems = exposedItemsList;

        var items = [];
        var snapshotItems = {};

		// This function supports the Portal Launch List and the Dashboard Organizer (also called the Tab Organizer).
		// It collects all instances of an item (e.g. a dashboard) and groups them together, with a 'top level' item and 'snapshots' beneath it.
		// Note that a top level items is defined this way, using a dashboard as an example:
		// 1.  If a default version of the PD in which the dashboard is found is set as the default (in Process Admin) then
		//        that instance of the dashboard is the top level item.
		//  2. If there is no default version but there is a tip (Process Center only) then that is the top level item.
		//  3. If there is no default or tip then the snapshot with the most recent create date is the top level item.
		// The other instances of the dashboard are grouped at a single level beneath the top item. On Process Center it is
		//   possible that the tip is beneath the top level item if a different version of the app is set as the default.
		// This function also disambiguates the displayed name of a top level item.  This is necessary when, for example,
		//    > 1 Process Apps have a dashboard with the same name. 
		//
        // For iPad: display only the top level item -- do not show any submenu **NOTE**:  if this.trackSnapshotsOnIPad then we'll ignore this restriction.

        var isIPad = false;
        if (dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
          isIPad = true;
        }
        var duplicateItemCount = {};
        var tipOrDefault = {};
		var toolkitItems = {};
        var filteredItems = [];
		// If there are no tips at all present, we assume that the environment is Process Server, which involves a slightly
		//    different piece of logic when associating snapshots later on.
		
		// Loop over the items and retain those that match the requested item type(s)
        dojo.forEach( exposedItems, dojo.hitch( this, function(item){
		  /* select items based on typeArg*/
		  var itemMatchesType = false;
		  if (item.type) {
			  if (dojo.indexOf(typeArg,"scoreboard")>=0 && item["type"] == "scoreboard") {
				itemMatchesType = true;
				item.itemType = "scoreboardOrDashboard";
				item.preciseItemType = "scoreboard";
			  }
			  else if (dojo.indexOf(typeArg,"dashboard")>=0  && (item["type"] == "service" && item["subtype"] == "dashboard")) {
				itemMatchesType = true;
				item.itemType = "scoreboardOrDashboard";
				item.preciseItemType = "dashboard";
			  }
			  else if (dojo.indexOf(typeArg,"queue")>=0 && item["type"] == "queue") {
				itemMatchesType = true;
				item.itemType = "queue";
				item.preciseItemType = "queue";
			  }
			  else if (dojo.indexOf(typeArg,"url")>=0  && (item["type"] == "service" && item["subtype"] == "url")) {
				itemMatchesType = true;
				item.itemType = "url";
				item.preciseItemType = "url";
			  }
			  else if (dojo.indexOf(typeArg,"processOrService")>=0) {
				  // keep two types of data: processes and startable services; for processes, keep only those for which there is a 'startURL'.
				  var startURLExists = ((typeof item.startURL != 'undefined') && item.startURL);
				  if (item.type == "process" && startURLExists) {
						item.itemType = "exposedprocess";
						item.preciseItemType = "exposedprocess";
						itemMatchesType = true;
				  } else if (item.type == "service" && item.subtype && item.subtype == "startable_service") {
						item.itemType = "tasktemplate";
						item.preciseItemType = "tasktemplate";
						itemMatchesType = true;
				  }
			  } 
		  }
		  if (itemMatchesType) {  // process only items of requested type
			
			var baseID = item.ID;
			item["baseID"] = baseID;  // add ID to item under attr 'baseID' because 'ID' is too common
			// save it!
			filteredItems.push(item);
		  }
		}));  // end dojo.forEach

		
		// Loop over the items and track dashboards and scoreboards that are defined in toolkits
        dojo.forEach( filteredItems, dojo.hitch( this, function(item){
			var itemID = item.itemID;
			if (typeof (toolkitItems[itemID]) == 'undefined') {
				// initialize...
				toolkitItems[itemID] = {
					'definedInAToolkit':false,
					'usedInProcessApp':false
				};
			}
			if (item.itemType == "scoreboardOrDashboard") {
				// Track dashboards and scoreboards that are defined in toolkits.
				//   The rules for a dashboard or scoreboard defined in a toolkit are:
				//   1. It must be used in a process app in order for it to be displayed
				//   2. It can be displayed only once even if it is used in many process apps
				// if item.topLevelToolkitAcronym is defined then this dashboard is defined in a toolkit
				if (item.topLevelToolkitAcronym) {
					toolkitItems[itemID].definedInAToolkit = true;
				} else {
					// It is not defined in a toolkit but it is used in a process app
					toolkitItems[itemID].usedInProcessApp = true;
				}
			}
		}));  // end dojo.forEach
		
		
		// Remove toolkit dashboards or scoreboards that don't meet the rules:
		//   1. It must be used in a process app in order for it to be displayed
		//   2. It can be displayed only once even if it is used in many process apps
        var tempItemsArray = [];
        dojo.forEach(filteredItems, dojo.hitch(this, function(item){
			var itemID = item.itemID;
			var keepItem = true;
			// If this dashboard or scoreboard was defined in a toolkit and is used in one or more process apps, then
			//   keep only the toolkit item.  If it is defined in a toolkit but not used in
			//   any process app then don't keep it.	
			if (item.itemType == "scoreboardOrDashboard") {
				if (toolkitItems[itemID].definedInAToolkit == true) {
					if (toolkitItems[itemID].usedInProcessApp == true) {
						if (!item.topLevelToolkitAcronym) {
							// not the toolkit item
							keepItem = false;
						}
						
					} else {
						// not used in a process app, so don't keep it
						keepItem = false;
					}
				}
			}
			// Be sure to keep items that aren't dashboards or scoreboards!  That's why keepItem is true by default.
			if (keepItem) {
				tempItemsArray.push(item);
			}
        }));
		
		// clone tempItemsArray into filteredItems for more processing
		filteredItems = dojo.clone(tempItemsArray);
		
		
		// We will track this information:
		// 1. duplicate items by title
		// 2. duplicate items by itemID
		// 3. whether or not a tip exists for an app
		// 4. latest or default Version for an app
        dojo.forEach( filteredItems, dojo.hitch( this, function(item){
			var itemID = item.itemID;
			var snapName = (item.tip ? this.messages["tipDisplayName"] : item.snapshotName);
			var baseID = item.ID;
			var snapshotCreateDate = item.snapshotCreatedOn;
			var title = item.display;
			
			// Use 'title' field if it exists
			if (typeof item.title != "undefined" && item.title) {
			  title = item.title;
			}
			
			// Track items by itemID, so we can keep track of tips and snapshots
			if (typeof (duplicateItemCount[itemID]) == 'undefined') {
			  duplicateItemCount[itemID] = {'count': 0};  // initialize duplicate count to 0 since this item is seen for the first time.
			}
			
			// On Process Center the tip and the latest snapshot have the same itemID and snapshot name
			if (typeof (duplicateItemCount[itemID][snapName]) == 'undefined') {
			  duplicateItemCount[itemID].count++;
			}
			
			// Also track items by display name, so we can know if we need to disambiguate the display name;
			// DO NOT disambiguate saved search names.  Don't add saved search names to duplicateItemCount[title].
			if (item.itemType != "queue") {
				if (typeof (duplicateItemCount[title]) == 'undefined') {
				  duplicateItemCount[title] = {'itemsArray': [item]}; //  this is the first time this 'title' has been seen; we may see more so keep an array of them
				}
				else {
				  // the title was seen before; add current item to array of items with this title
					duplicateItemCount[title].itemsArray.push(item);
				}
			}
			
			// track tips, default Versions and most recent Versions for this itemID.
			if (typeof (tipOrDefault[itemID]) == 'undefined') {
				// initialize...
				tipOrDefault[itemID] = {
					'tipExists':false,
					'baseID':"",
					'tipBaseID': "",
					'defaultBaseID': "",
					'snapshotCreateDate': ""
				};
			}
			if (toolkitItems[itemID].definedInAToolkit == false || (toolkitItems[itemID].definedInAToolkit == true && item.topLevelToolkitAcronym)){
				if (item.tip) {
					tipOrDefault[itemID].tipExists = true;
					tipOrDefault[itemID].tipBaseID = baseID;
					tipOrDefault[itemID].baseID = "";
					tipOrDefault[itemID].snapshotCreateDate = snapshotCreateDate;				
					this.hasTips = true;
					//  We need to remove the &snapshot query param from the runURL of a Tip.
					if (item["runURL"]) {
						item["runURL"] = this.removeSnapshotParamFromRunURL(item["runURL"]);
					}	
				} else {
					if (true==item.isDefault) {
						tipOrDefault[itemID].defaultBaseID = baseID;		
						// Set the baseID (latest version) to ""
						tipOrDefault[itemID].baseID = "";
					} else if ((tipOrDefault[itemID].defaultBaseID=="") && (tipOrDefault[itemID].tipExists==false) && tipOrDefault[itemID].snapshotCreateDate < snapshotCreateDate) {
						// Note that if the default or tip was already found, we don't want to track latest version, hence the checks in this if stmt
						tipOrDefault[itemID].baseID = baseID;
						tipOrDefault[itemID].snapshotCreateDate = snapshotCreateDate;
					}
				}
			}

		}));  // end dojo.forEach
		
				
		// So far we've tracked this information:
		// 1. duplicate items by title
		// 2. duplicate items by itemID
		// 3. whether or not a tip exists for the app
		// 4. latest or default Version
		// 5. toolkit information
		//
		// We need to do these things:
        // 1. For each tip (or default Version) create an array of objects containing snapshots (if any).
        // 		The top-level list will contain either tips or default Versions (on Process Center) 
		//           or default or latest Version (on Process Server);
		//           snapshots will be in an array that is the child of the top-level item.
		//      On Process Center -- if there is a snapshot that is identified as the default then we will not store the tip
		// 2. Disambiguate names.  
		// 3. For a dashboard that is defined in a toolkit:  Show it only if it is used in a Process App. Also,
		//     show such a dashboard only once regardless of how many Process Apps it is used in.
        dojo.forEach(filteredItems, dojo.hitch(this, function(item){
          var itemID = item.itemID;
          var title = item.display;
		  // use 'title' field if it exists
		  if (typeof item.title != "undefined" && item.title) {
			  title = item.title;
		  }
		  // originalTitle is the non-nls name of the item.  originalTitle is used in other places 
		  //   to identify specific dashboards, like the Work and Process Performance dashboards.
		  var originalTitle=item.display;
          var snapName = (item.tip ? this.messages["tipDisplayName"] : item.snapshotName);
          var appId = item.processAppID;
          var baseID = item.baseID;
          var snapshotCreateDate = item.snapshotCreatedOn;
		  var isTip_defaultSnapshotDoesNotExist= (item.tip && ""==tipOrDefault[itemID].defaultBaseID);
		  var isTip_defaultSnapshotExists= (item.tip && ""!=tipOrDefault[itemID].defaultBaseID);
		  var isDefault = item.isDefault;
		  
		  // Determine if the item is a snapshot.
		  var thisIsASnapshot = true;
		  if (duplicateItemCount[itemID].count > 1) {  // there can't be snapshots unless there are >1 items
			  if (isTip_defaultSnapshotDoesNotExist) {
				  thisIsASnapshot = false;
			  } else if (isTip_defaultSnapshotExists) {  // just putting this case here for clarity
				  thisIsASnapshot = true;		  
			  } else if (isDefault) {  // just putting this case here for clarity
				  thisIsASnapshot = false;		  
			  } else if (baseID==tipOrDefault[itemID].defaultBaseID) {  // this is the default Version
				  thisIsASnapshot = false;
			  } else if (baseID==tipOrDefault[itemID].baseID) {  // this is the latest Version
				  thisIsASnapshot = false;
			  }
		  }  else {
			  thisIsASnapshot = false;
		  }
		  //  If this is the Work dashboard (title is "BPM_WORK") then we want to keep only 1 item even
		  //     if we see numerous snapshots.  We will keep the one that has been identified as the
		  //     top-level item.   In addition, if the runURL has a snapshot request param, we need to
		  //     remove it.  This is because the Work dashboard will be recognized in Portal only *without* snapshot info.
		  var continueProcessing=!(this.workDashIdentifier==originalTitle && thisIsASnapshot);
		  if (continueProcessing) {
			  if (this.workDashIdentifier==originalTitle) {
				  if (item["runURL"]) {
					  item["runURL"] = this.removeSnapshotParamFromRunURL(item["runURL"]);
				  }
			  }
		  
			  //decide whether any additional identifying information should be displayed (disambiguation)
			  if (duplicateItemCount[title] && duplicateItemCount[title].itemsArray.length > 1 && (!thisIsASnapshot)) {
				//    Loop over items with the same title. If the items are in the same Process App/Toolkit but in a different track (also called branch or workspace)
				//    AND they're both tips, then add the branch name
				var useBranchName = false;
				var getTaskTemplate = false;
				for ( var x = 0; x < duplicateItemCount[title].itemsArray.length; x++) {
				  var arrayItem = duplicateItemCount[title].itemsArray[x];
				  if (baseID != arrayItem.baseID) { // don't compare an item to itself!
					if (itemID == arrayItem.itemID && appId == arrayItem.processAppID && item.tip == true && arrayItem.tip == true) {
					  useBranchName = true;
					}
					else if (itemID != arrayItem.itemID || appId != arrayItem.processAppID) {
					  getTaskTemplate = true;
					}
				  }
				}
				// Don't disambiguate saved search names
				if (item.itemType!="queue" ) {
					if (useBranchName) {
					  // disambiguate using branchName (if branchName doesn't exist - i.e., isn't even in the object -  or if it is null, use snapshotName)
					  var branchName =
						(typeof item.branchName != "undefined" && item.branchName) ? item.branchName : item.snapshotName;
					  title = title + " (" + branchName + ")";
					}
					else if (getTaskTemplate) {
					  var name1 = "";
					  // this might be a process app or a toolkit
					  name1 = (item.topLevelToolkitName) ? item.topLevelToolkitName: item.processAppName ;
					  title = title + " (" + name1 + ")";
					}
				}
			  }
			  //  For scoreboards - see if the title is actually a key in a message bundle:
			  //  NOTE that we did not want to disambiguate the NLS title of a scoreboard --
			  //     if a scoreboard and a item of another type, such as dashboard, have the same name their
			  //      types will be indicated by icons in displayed lists, not by disambiguated names.
			  if (item["type"] == "scoreboard") {
				  title = this.getNLSDisplayName(title);
				  // replace the title in the object
				  item.title=title;
			  }
			  // create object for the list
			  var runURL = (item["runURL"] ? item["runURL"] : null);  // used for dashboards
			  var startURL = (item["startURL"] ? item["startURL"] : null);  // used for exposed processes
			  var itemObj =
				this.createItemObject(title, itemID, item.itemType, item.preciseItemType, baseID, appId, item.snapshotID, item.tip, thisIsASnapshot,
					snapshotCreateDate, snapName, item.branchID, runURL, startURL, originalTitle, item.name, item.itemReference);
					
			  if (!isIPad || this.trackSnapshotsOnIPad==true) {
				if (thisIsASnapshot == true) {
				  // store this snapshot object - it will be added to the parent tip object later.
				  if (!(itemID in snapshotItems)) {  // create array and add item to it
					var snapArray = [];
					snapArray.push(itemObj);
					snapshotItems[itemID] = snapArray;
				  }
				  else {  // add item to existing array
					snapshotItems[itemID].push(itemObj);
				  }
				}
				else {  // this is not a snapshot
				  items.push(itemObj);
				}
			  } else { // iPad case
				if (thisIsASnapshot == false) {
				  items.push(itemObj);
				}
			  }
		  }
        }));

        //sort items alphabetically
        items.sort(function(a, b){
          var ret = 0;
          if (a.title > b.title) {
            ret = 1;
          }
          else if (a.title < b.title) {
            ret = -1;
          }
          return ret;
        });

        //sort array of snapshots from newest to oldest
        for ( var sItem in snapshotItems) {
          var snapItems = snapshotItems[sItem];
          snapItems.sort(function(a, b){
            var ret = 0;
            if (a.params.snapshotCreateDate < b.params.snapshotCreateDate) {
              ret = 1;
            }
            else if (a.params.snapshotCreateDate > b.params.snapshotCreateDate) {
              ret = -1;
            }
            return ret;
          });
        }

        // for each list item, associate snapshots, if any
        for ( var i = 0; i < items.length; i++) {
          var itemID = items[i]["itemID"];
		  var filteredSnapItems = [];
          var snapItems = snapshotItems[itemID];
          if (snapItems) {
			// Double check the branchID to make sure snapshot items are on same
			//     track, otherwise remove from the snapshot list.
			//     This check is only valid if the environment is Process Center.
			if(this.hasTips) {
				for(var j = 0; j < snapItems.length; j++) {
					var curItem = snapItems[j];
					if(typeof curItem.params != "undefined" && 
						typeof curItem.params.branchID != "undefined" &&
						typeof items[i]["params"] != "undefined" &&
						typeof items[i]["params"]["branchID"] != "undefined") {
						if(curItem.params.branchID == items[i]["params"]["branchID"]) {
							//if the IDs match, they should be on the same
							//branch, otherwise not
							filteredSnapItems.push(curItem);
						}
					} else {
						//by default, just add it
						filteredSnapItems.push(curItem);
					}	
				}
			} else {  // Process Server case:
				// Make sure the itemReference is the same, otherwise remove from the snapshot list.
				//     This check is only valid if the environment is Process Server.
				for(var j = 0; j < snapItems.length; j++) {
					var curItem = snapItems[j];
					if(typeof curItem.params != "undefined" && 
						typeof curItem.params.itemReference != "undefined" &&
						typeof items[i]["params"] != "undefined" &&
						typeof items[i]["params"]["itemReference"] != "undefined") {
						if(curItem.params.itemReference == items[i]["params"]["itemReference"]) {
							//if the IDs match, they should be on the same
							//branch, otherwise not
							filteredSnapItems.push(curItem);
						}
					} else {
						//by default, just add it
						filteredSnapItems.push(curItem);
					}	
				}
			}
			
			if(filteredSnapItems.length > 0) {
				// Now that we have associated a snapshot with a top-level item, we need
				// to change the snapshot 'title' to match the possibly disambiguated title
				// of the top-level item
				for (var x=0; x<filteredSnapItems.length; x++) {
					filteredSnapItems[x].title=items[i].title;
					filteredSnapItems[x].params.title=items[i].title;
				}
				// Add snapshot items:
				items[i].params["snapshotItems"] = filteredSnapItems;
			}
          }
        }

		return items;

      },

	  removeSnapshotParamFromRunURL: function (runURLArg) {
		var retRunURL=runURLArg;

		// remove snapshot info if present; this isn't fancy uri parsing, just text location and removal
		var inx = retRunURL.indexOf("&snapshot=");
		if (inx>0) {
			var parts1 = retRunURL.split("?");
			if (parts1.length==2) {
				var parts2 = parts1[1].split("&");
				var newRunURL = parts1[0]+"?";
				var count = 0;
				if (parts2.length>1) {
					for (var i=0; i<parts2.length; i++) {
						if (parts2[i].indexOf("snapshot=") <0) {
							if (count > 0) {
								newRunURL = newRunURL+"&";										
							}
							count++;
							newRunURL = newRunURL+parts2[i];
						}
					}
				}
				// was anything done?
				if (count>1) {
					retRunURL = newRunURL;
				}
			}
		}		
		return retRunURL;
	  },
	  
      createItemObject: function(title, itemID, itemType, preciseItemType, baseID,  appId, snapshotID, tip, isSnapshot, snapshotCreateDate,
        snapName, branchID, runURL, startURL, originalTitle, name, itemReference){
        var itemObj = {
          'title': title,
          'itemID': itemID,
          'params': {
            'type': itemType,
			'preciseItemType': preciseItemType,
            'baseID': baseID,
			'title' : title, // yes it is stored again
            'itemID': itemID, // yes it is stored again
            'processID': appId,
            'snapshotID': snapshotID,
            'isTip': tip,
            'isSnapshot': isSnapshot,
            'snapshotCreateDate': snapshotCreateDate,
            'snapshotName': snapName,
			'branchID': branchID,
			'runURL' : runURL,
			'startURL' : startURL,
			'originalTitle': originalTitle,
			'name': name,
			'itemReference': itemReference
          }
        };
        return itemObj;
      },

	  getNLSDisplayName: function(displayName){
		var msgKey = this.scoreboardMessageKeyMapping[displayName];
		if (msgKey && this.messages[msgKey]) {
		  displayName = this.messages[msgKey];
		}
		return displayName;
	  },

	  isProcessServer: function() {
		// Returns true if the current env is Process Server
		// *** The return value is reliable *if* there were >0 exposed items found in the getData() function.
		// If there were 0 exposed items found the the value is unreliable but is also irrelevant -- 
		//    the returned value only matters when there are exposed items.
		return (this.hasTips==false) ? true:false;
	  },
	  
      destroy: function(finalize){
		this.inherited(arguments);
      }

    });
