/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.FollowedProcessList");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");

dojo.require("com.ibm.bpm.social.widgets.ProcessList");
dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");
dojo.require("com.ibm.bpm.social.widgets.dialogs.UnfollowDialog");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

/* FollowedProcessList - Extends ProcessList. Displays a list of processes 
	that the user has marked as following. Clicking an item will take the user 
	to the task details for that process. Clicking an icon will remove it from 
	the list.
	
	Updated: 2/6/2012
	Experts: Ben Myers @ Raleigh, Scott Johnson @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.FollowedProcessList",
	[com.ibm.bpm.social.widgets.ProcessList, com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets", "templates/ProcessList.html"),
	constructor: function() {
		this.sortBy = "instanceDueDate"; //database column to order the list by
		
		//the topic that will be published to when a task is clicked - the 
		// expected behavior is this will take the user to the task details
		this.showProcessTopic = "defaultProcessListTopic";
		
		//the topic to subscribe to for refreshes
		this.refreshTopic = "defaultRefreshTopic";
		
		//the topic to subscribe to when the user profile preferences change
		this.profileChangedTopic = "defaultProfileChangedTopic";
		this.profilePreferences = null;
		
		this.followedProcessListSubscriptions = [];
		
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
	},            	
	
    postCreate: function() {
		this.inherited(arguments);
		
		//localization
    	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//set a message to appear when the list is empty
		this.setEmptyMessage(this.messages['followedProcessList-empty']);
		
		//FollowedProcessList uses the parent's template, but adds a class so 
		// styles can be applied specifically to it (such as for stars)
		dojo.addClass(this.domNode, "bpm-social-followedprocesslist");
		
		//get the user profile and preferences
		this.profilePreferences = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if(this.profilePreferences != null) {
			this.profilePreferences = this.profilePreferences.userPreferences;
		}
		
		//subscribe to refresh channel and refresh when received
		this.followedProcessListSubscriptions.push(dojo.subscribe(this.refreshTopic, dojo.hitch(this, function(data) {
			this.refresh();
		})));
		
		//subscribe to profile changes and update profile when received
		this.followedProcessListSubscriptions.push(dojo.subscribe(this.profileChangedTopic, dojo.hitch(this, function(data) {
			this.profilePreferences = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			if(this.profilePreferences != null) {
				this.profilePreferences = this.profilePreferences.userPreferences;
			}
		})));
		
		//create the unfollow dialog
		this.dialog = new com.ibm.bpm.social.widgets.dialogs.UnfollowDialog();
	},
    
	/* _createItem: Inherited from ProcessList. Adds a dojo tooltip to the 
		list's stars. */
	_createItem: function(index, li, params, createParams) {
		this.inherited(arguments);
		//add tooltip
		var icon = dojo.query(".process-icon", li)[0];
		dojo.attr(icon, "title", this.messages["followedProcessList-unfollowHover"]);
		dojo.attr(icon, "aria-label", "Stop following process "+createParams['title']);
		dojo.attr(icon, 'innerHTML', '<span class="hi-contrast-icon">&#9733;</span>');
		var text = dojo.query(".process-text", li)[0];
		dojo.attr(text, "tabindex", "0");
		dojo.attr(text, "aria-label", "Go to process "+createParams['title']);
		
		//configure base text direction here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text);
	},
	
	/* refresh: Inherited from SmartList. Updates the information displayed in 
		the widget by requesting a new set of data, clearing the list, and 
		repopulating it. */
    refresh: function() {
		
		this.socialRestHandler.getFollowedInstances(dojo.hitch(this, this._doRefresh),
			dojo.hitch(this, this._onXhrErrors), true);
    },
    _doRefresh: function(data, xhrArgs) {
    	
		//clear all the DOM elements from the ProcessList
		this.clearList();
		
		//add new DOM elements using the given data
		var items = data.data.data;
    	dojo.forEach(items, dojo.hitch(this, function(item) {
			if(item.instanceName == null || item.instanceName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
				this.addProcessItem(this.messages["followedProcessList-defaultProcess"].replace("${id}", item.instanceId), {'instanceId': item.instanceId});
			}
			else {
				this.addProcessItem(item.instanceName, {'instanceId': item.instanceId});
			}
		}));
		
		//restore focus to the item the keyboard-only user was focused on
		//this._restoreFocusedItem();
		
		//dojo.attr(this.listElement, "aria-busy", "false");
    },
	
	/* addProcessItem: Adds an item to the ProcessList. Inherited from 
		ProcessList. */
	addProcessItem: function(title, params) {
		params['iconClass'] = 'bpm-social-followed-star';
		params['iconClassHighlight'] = 'bpm-social-followed-star';
		this.inherited(arguments);
	},
	
	/* _onProcessIconClicked: Called when a star is clicked. When clicked, 
		unfollow the item. */
	_onProcessIconClicked: function(index, li, params) {
		var instanceId = params['instanceId'];
		this.unfollow(instanceId);
	},
	
	/* _onProcessTextClicked: Called when the text for an item is clicked. 
		When clicked, take the user to the details for that process. */
	_onProcessTextClicked: function(index, li, params) {
		//publish a dojo topic to display the details
		if(params!=null && params['instanceId']!=null) {
			dojo.publish(this.showProcessTopic, [{'instanceId': params['instanceId']}]);
		}
	},
	
	/* unfollow: Called when a star is clicked. Remove the item from the list 
		and mark it as no longer being followed. */
	unfollow: function(instanceId) {
		if(instanceId!=null) {
			//make a rest call to unfollow the process
			
			this.socialRestHandler.unfollowInstance(instanceId, 
				dojo.hitch(this, this.refresh),
				dojo.hitch(this, function(data, xhrArgs) {
					this.refresh(); 
					this._onXhrErrors(data, xhrArgs);
				}),
				true);
		}
    },
    
    _onXhrErrors: function(data, xhrArgs) {
		if(this.listElement) {
			dojo.attr(this.listElement, "aria-busy", "false");
		}
    	console.debug(["FollowedProcessList experienced an error.", data, xhrArgs]);
    },
	
	destroy: function(finalize) {
		for(var i=0; i<this.followedProcessListSubscriptions.length; i++) {
			dojo.unsubscribe(this.followedProcessListSubscriptions[i]);
		}
		this.dialog.destroy();
		this.inherited(arguments);
	}
    
});


