/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.LaunchableProcessList");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");

dojo.require("com.ibm.bpm.social.util.common.SystemRestHandler");
dojo.require("com.ibm.bpm.social.util.common.ProcessRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.require("com.ibm.bpm.social.widgets.ExposedItemsList");

dojo.require("com.ibm.bpm.social.widgets.ProcessList");
dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");

dojo.require("com.ibm.bpm.social.Aether");

/* LaunchableProcessList - Extends ProcessList. Displays a list of processes 
	that have been exposed to the user. Clicking an item will launch a new 
	instance of the process.
	
	Created: November 2011
*/
dojo
  .declare(
    "com.ibm.bpm.social.widgets.LaunchableProcessList",
    [com.ibm.bpm.social.widgets.ProcessList,
     com.ibm.bpm.social.widgets.common._RestWidget,
     dijit._Templated],
    {

      templateString: "",
      templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets", "templates/ProcessList.html"),
      constructor: function(){
        this.showProcessTopic = "defaultShowProcessTopic";
        this.showTaskTopic = "defaultShowTaskTopic";
        this.widgetsToCleanUp = [];
        this.processListEventConnectors = [];
        this.menuHasBeenHovered = false;

        this.systemRestHandler = com.ibm.bpm.social.util.common.SystemRestHandler.getInstance();
        this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
      },

      postCreate: function(){
        this.inherited(arguments);

        dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
        this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

        //set a message to appear when the list is empty
        this.setEmptyMessage(this.messages['launchableProcessList-empty']);

        //LaunchableProcessList uses the parent's template, but adds a class 
        // so styles can be applied specifically to it (such as for cogs)
        dojo.addClass(this.domNode, "bpm-social-launchableprocesslist");

        //get the user profile and preferences
        this.currentUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
      },

      /* refresh: Inherited from SmartList. Updates the information displayed in 
      	the widget by requesting a new set of data, clearing the list, and 
      	repopulating it. */
      refresh: function(){
        dojo.attr(this.listElement, "aria-busy", "true");

		// get the exposed items from REST service
		// com.ibm.bpm.social.portal.exposedItems is global; it's the unmodified response from exposed items REST service;
		//  we don't modify it, we clone it and modify the clone.
		if (!com.ibm.bpm.social.portal.exposedItems) {
			com.ibm.bpm.social.portal.exposedItems = "";
			this.systemRestHandler.getExposedItems(dojo.hitch(this, function(data, xhrArgs){
				com.ibm.bpm.social.portal.exposedItems = dojo.clone(data.data.exposedItemsList);
		  }), dojo.hitch(this, this._onXhrErrors), false);  // arg 'false' means do xhr *synchronously*
		}
		  
		// clone the exposed items:
		var exposedItemsArray = dojo.clone(com.ibm.bpm.social.portal.exposedItems);
		this._doRefresh(exposedItemsArray);
      },
      _doRefresh: function(data){
		this.itemList = new com.ibm.bpm.social.widgets.ExposedItemsList();

        this.cleanUpMenusAndWidgets();
		var items = this.itemList.getData(data,["processOrService"]);

        //add new DOM elements using the given data
        dojo.forEach(items, dojo.hitch(this, function(item){
          this.addProcessItem(item.title, item.params);
        }));

        //restore focus to the item the keyboard-only user was focused on
        this._restoreFocusedItem();

        dojo.attr(this.listElement, "aria-busy", "false");
      },

      /* addProcessItem: Adds an item to the ProcessList. Inherited from 
      	ProcessList. */
      addProcessItem: function(title, params){
        params['iconClass'] = 'bpm-social-gray-cog';
        params['iconClassHighlight'] = 'bpm-social-white-cog';
        this.inherited(arguments);
      },

      /* _createItem: Handles creating each process item's HTML elements. Params 
      	is provided when addProcessItem is called. */
      _createItem: function(index, li, params, createParams){
        //create the HTML elements
        if (document.body.dir == 'rtl') {
          this._createItemNameElement(index, li, params, createParams);
          this._createLaunchIconElement(index, li, params, createParams);
        }
        else {
          this._createLaunchIconElement(index, li, params, createParams);
          this._createItemNameElement(index, li, params, createParams);
        }
      },

      _createLaunchIconElement: function(index, li, params, createParams){
        var icon = dojo.create("div", {
          'class': "process-icon " + params['iconClass'],
          'tabIndex': "0",
          'aria-label': "Launch process " + createParams['title'],
          'innerHTML': '<span class="hi-contrast-icon">&gt;</span>' // Seen in high contrast, not read by JAWS
        }, li);
        dijit.setWaiRole(icon, "button");
        this.processListEventConnectors.push(dojo.connect(icon, 'keypress', this, "_processIconKeyPressEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onclick', this, "_processIconClickEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onfocus', this, "_processIconFocusEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onblur', this, "_processIconBlurEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onmouseenter', this, "_processIconMouseoverEvent"));
        this.processListEventConnectors.push(dojo.connect(icon, 'onmouseleave', this, "_processIconMouseoutEvent"));
      },

      _createItemNameElement: function(index, li, params, createParams){
        var title = "<i>Unknown</i>";
        if (createParams != null && typeof (createParams) != 'undefined') {
          title = createParams['title'];
        }
        // If there are snapshots for this item, the tip name will be inline
        var text = dojo.create("a", {
          'class': "process-text",
          'innerHTML': title,
          'aria-label': "Choose a snapshot to launch for process " + title + "."
        }, li);
        dijit.setWaiRole(text, "link");

        //Configure Base Text Direction Here:
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(text);
		
        // If there are snapshots for this item, create a dropdown menu to contain them
        if (params.snapshotItems && params.snapshotItems.length > 0) {
          var menuIcon = dojo.create("img", {
            'class': "bpm-social-white-drop-arrow",
            'border': "none",
            'tabIndex': "0",
            'alt': "",
            'id': "id_" + params.baseID
          }, text);
          var menuIconAltText = dojo.create("span", {
            'class': "hi-contrast-icon dropdown-text",
            'innerHTML': "v",
            'id': "id_alttext_" + params.baseID

          }, text);
          menuIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
          dijit.setWaiRole(menuIcon, "button");
		  var menuDropdown = new dijit.Menu({
			"style": "display:none",
            leftClickToOpen: true
          });
          menuDropdown.bindDomNode(menuIcon);		  
          params["menuDropdownWidget"] = menuDropdown;
          this.widgetsToCleanUp.push(menuDropdown);
          dojo.addClass(menuDropdown.domNode, "processPortalDropDownMenu");

          var connectFunc = dojo.connect(menuDropdown.domNode, "onmouseenter", this, function(evt){
            this._onMenuDropdownHovered(menuDropdown, evt);
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuDropdown.domNode, "onmouseleave", this, function(evt){
            this._onMenuDropdownUnhovered(menuDropdown, evt);
          });
          this.processListEventConnectors.push(connectFunc);

          connectFunc = dojo.connect(menuIcon, 'onmouseover', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIcon, 'onclick', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIcon, 'keypress', this, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              menuDropdown._scheduleOpen(evt.target, null);
              this.menuHasBeenHovered = false;
            }
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'onmouseover', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'onclick', this, function(evt){
            dojo.stopEvent(evt);
            menuDropdown._scheduleOpen(evt.target, null, {
              x: evt.pageX,
              y: evt.pageY
            });
            this.menuHasBeenHovered = false;
          });
          this.processListEventConnectors.push(connectFunc);
          connectFunc = dojo.connect(menuIconAltText, 'keypress', this, function(evt){
            if (evt.keyCode == dojo.keys.ENTER) {
              dojo.stopEvent(evt);
              menuDropdown._scheduleOpen(evt.target, null);
              this.menuHasBeenHovered = false;
            }
          });
          this.processListEventConnectors.push(connectFunc);

          for ( var i = 0; i < params.snapshotItems.length; i++) {
            this._addSnapshotMenuItem(menuDropdown, params.snapshotItems[i].params["snapshotName"], params.snapshotItems[i]);
          }
          menuDropdown.startup();
        }
        this.processListEventConnectors.push(dojo.connect(text, 'keypress', this, "_processTextKeyPressEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onclick', this, "_processTextClickEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onfocus', this, "_processTextFocusEvent"));
        this.processListEventConnectors.push(dojo.connect(text, 'onblur', this, "_processTextBlurEvent"));
      },

      _onMenuDropdownHovered: function(menuDropdown, evt){
        this.menuHasBeenHovered = true;
      },
      _onMenuDropdownUnhovered: function(menuDropdown, evt){
        this.menuHasBeenHovered = false;
        dijit.popup.close(menuDropdown);
      },
      _onItemMouseOut: function(index, li, params){
        this.inherited(arguments);
        if (params["menuDropdownWidget"]) {
          var menu = params["menuDropdownWidget"];
          window.setTimeout(dojo.hitch(this, function(){
            if (!this.menuHasBeenHovered) dijit.popup.close(menu);
          }), 750);
        }
      },

      _addSnapshotMenuItem: function(menuDropdown, display, params){
        // params is the snapshot information
        if (display && params) {
          var self = this;
		  
		  var btd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(display);

          var item = new dijit.MenuItem({
            label: display,
            onClick: dojo.hitch(this, "_menuItemClick"),
            dir: btd
          });
          this.widgetsToCleanUp.push(item);
          menuDropdown.addChild(item);
          // for simplicity and because the snapshots are launched from a menu and not from
          // the launchable items list, keep an array of snapshot information for easy lookup
          // when a menu item is selected
          if (!this.snapshotMenuItems) {
            this.snapshotMenuItems = {};
          }

          this.snapshotMenuItems[item.id] = {
            "display": display,
            "params": params
          };

        }
      },

      _menuItemClick: function(evt){
        var itemData = this.snapshotMenuItems[evt.currentTarget.id];
        this._onProcessItemSelected(itemData.params.params);
      },

      /* _onProcessIconClicked: Called when a cog is clicked. When clicked, 
      	launch a new instance of the process, or publish message for task template. Same functionality as 
      	_onProcessTextClicked.  */
      _onProcessIconClicked: function(index, li, params){
        this._onProcessItemSelected(params);
      },

      /* _onProcessTextClicked: Called when the text for an item is clicked. 
      	When clicked, launch a new instance of the process. Same functionality 
      	as _onProcessiconClicked. */
      _onProcessTextClicked: function(index, li, params){
        this._onProcessItemSelected(params);
      },

      _processTextKeyPressEvent: function(evt){
        // if user pressed on img, don't handle that here
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user pressed on img, don't handle that here
        if (theTag != "img") {
          if (evt.keyCode == dojo.keys.ENTER) {
            var info = this._extractItemInfoFromEvent(evt);
            if (info['index'] != null && info['li'] != null) {
              this._onProcessTextClicked(info['index'], info['li'], info['params']);
            }
          }
        }
      },

      _processTextClickEvent: function(evt){
        var theTag = null;
        if (evt.target && evt.target.tagName) {
          theTag = evt.target.tagName;
          theTag = theTag.toLowerCase();
        }
        // if user clicked on img, don't handle that here
        if (theTag != "img") {
          var info = this._extractItemInfoFromEvent(evt);
          if (info['index'] != null && info['li'] != null) {
            this._onProcessTextClicked(info['index'], info['li'], info['params']);
          }
        }
      },

      _onProcessItemSelected: function(params){
        if (params.type == "exposedprocess") {
          this._launchProcess(params["startURL"]);
        }
        else if (params.type == "tasktemplate") {
          this._openTaskTemplateCoach(params["baseID"]);
        }
      },

      /* _launchProcess: Called when an item is clicked. Launch a new instance 
      	of the process. Once the process is launched the user will either be 
      	taken to a coach or will receive a notification. */
      _launchProcess: function(startURL){
        //make a rest call to launch the process
        this.processRestHandler.launchProcess(startURL, dojo.hitch(this, this._doLaunchProcess), dojo.hitch(this, this._onLaunchProcessError), true, null);
      },
      _doLaunchProcess: function(data, xhrArgs){
        if (typeof (data) == 'undefined' || data == null || typeof (data.data) == 'undefined' || data.data == null ||
          typeof (data.data.tasks) == 'undefined' || data.data.tasks == null) {
          return;
        }

        var tasks = data.data.tasks;

        //check to see if the task is assigned to the last user in the lane
        if (this.currentUser != null) {
          for ( var i = 0; i < tasks.length; i++) {
            if (tasks[i].owner == this.currentUser.userName) {
              //if so, open it and let the user fill out the coach
              this._openProcessCoach(tasks[i]["tkiid"]);
              return;
            }
          }
        }

        //otherwise just display a notification that the process started
        this._displayProcessNotification(data.data.piid, data.data.name, tasks.length == 0);
      },
      _onLaunchProcessError: function(data, xhrArgs){
        var errMsg = "";
        if (xhrArgs && xhrArgs.xhr && typeof (xhrArgs.xhr.responseText) != 'undefined') {
          var responseTextJson = dojo.fromJson(xhrArgs.xhr.responseText);
          var errMsg = "<div>" + responseTextJson.errorMessage + "</div>";
        }
        var message = new com.ibm.bpm.social.Aether({
          'message': "<div>" + this.messages["launchableprocesslist-launchProcessFailed"] + "</div>" + errMsg,
          'containerDivId': "messageNotificationDiv",
          'position': 'top-right',
          'fade': true,
          'parent': this,
          'displayCloseIcon': true
        });
      },

      /* _openProcessCoach: Called when a process is launched and the user needs 
      	to fill out a coach. */
      _openProcessCoach: function(taskId){
        dojo.publish(this.showTaskTopic, [{
          "taskId": taskId
        }]);
      },

      /* _openTaskTemplateCoach: Called when a human service is launched and the 
      	user needs to fill out a coach. */
      _openTaskTemplateCoach: function(baseID){
        dojo.publish(this.showTaskTopic, [{
          "taskTemplateId": baseID
        }]);
      },

      /* _displayProcessNotification: Called when a process is launched and the 
      	user isn't responsible for filling it out. */
      _displayProcessNotification: function(processID, processName, hideLink){
        if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) { // Prevent notification if user is using Safari on iPad but allow if Chrome on iPad
          return
        }  
        
        if (typeof (hideLink) == 'undefined') {
          hideLink = false;
        }
        var link = processName;
        if (link == null || link.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
          link = this.messages["launchableprocesslist-defaultProcess"].replace("${id}", processID);
        }
        link = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(link);
        if (!hideLink) {
          link = "<a href='#' dojoattachevent='onclick: _handleActionClick'>" + link + "</a>";
        }
        var msg = this.messages['launchableProcessList-started'].replace("${process}", link);

        var message = new com.ibm.bpm.social.Aether({
          'message': "<div>" + msg + "</div>",
          'containerDivId': "messageNotificationDiv",
          'position': 'top-right',
          'fade': true,
          'parent': this,
          'fireEvent': dojo.hitch(this, function(processID){
            dojo.publish(this.showProcessTopic, [{
              "instanceId": processID
            }]);
          }, processID),
          'displayCloseIcon': true
        });
      },

      _onXhrErrors: function(data, xhrArgs){
        if (this.listElement) {
          dojo.attr(this.listElement, "aria-busy", "false");
        }
        console.debug(["LaunchableProcessList experienced an error.",
                       data,
                       xhrArgs]);
      },

      /* clearList: Make sure to clear extra event connectors. */
      clearList: function(){
        this.inherited(arguments);
        this.cleanUpConnects();
      },

      cleanUpConnects: function(){
        //disconnect all dojo event connectors
        if (this.processListEventConnectors) {
          dojo.forEach(this.processListEventConnectors, function(connector){
            dojo.disconnect(connector);
          });
        }
        this.processListEventConnectors = [];
      },

      cleanUpMenusAndWidgets: function(){
        if (this.widgetsToCleanUp && this.widgetsToCleanUp.length > 0) {
          for ( var i = 0; i < this.widgetsToCleanUp.length; i++) {
            if (this.widgetsToCleanUp[i].destroyRecursive) {
              this.widgetsToCleanUp[i].destroyRecursive();
            }
            else if (this.widgetsToCleanUp[i].destroy) {
              this.widgetsToCleanUp[i].destroy();
            }
          }
        }
        this.widgetsToCleanUp = [];
        if (this.snapshotMenuItems) {
          delete this.snapshotMenuItems;
        }
        this.cleanUpConnects();
      },

      destroy: function(finalize){
        this.cleanUpMenusAndWidgets();
      }

    });
