/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.common.BizCard");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip");

com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;

/**
 * Use the method to determine whether or not default actions should be used.
 * The purpose of this method is to abstract the different possible integrations
 * from widget-level code and centralize it in one place.
 */
com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard = function(){
	
	//TODO Possibly allow for parameters to check for only specific integrations...
	
	return typeof(connectionsBridge)=="undefined";
	
}

com.ibm.bpm.social.widgets.common.BizCard._connections = {};

com.ibm.bpm.social.widgets.common.BizCard.wrapElement = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapElement entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections(node, user);
		} 
		if(success) return;
	}
	
	node.innerHTML = "";
	
	dojo.addClass(node, "vcard");
	
	var nameNode = dojo.create("a", null, node);
	nameNode.href = "javascript:void(0);";
	nameNode.innerHTML = (typeof(user["fullName"])!="undefined") ? user["fullName"] : user["userName"];
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.wrapImage = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapImage entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections(node, user);
		}
		if(success) return;
	}
	
	var container = dojo.create("span",{"class": "bpm-bizcard-container"}, node, "before");
	
	var nameNode = dojo.create("a", {"class": "bpm-bizcard-anchor"}, container);
	nameNode.href = "javascript:void(0);";
	
	dojo.place(nameNode, container, "only");
	
	dojo.place(node, nameNode, "only");
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}		
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.unwrapImage = function(node) {
	if(typeof(node)=="undefined" || node == null){
		return;
	}
	
	var connectId = node.getAttribute("connectId");
	if(typeof(connectId) != "undefined" && connectId != null) {
		node.setAttribute("connectId", null);
		dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
	}
	
	if(node.parentNode != null && typeof(node.parentNode) != "undefined" && dojo.hasClass(node.parentNode, "bpm-bizcard-anchor")) {
		var bizCard = node.parentNode.parentNode;
		if(bizCard != null && typeof(bizCard) != "undefined" && dojo.hasClass(bizCard, "bpm-bizcard-container vcard")) {
			if(bizCard.parentNode != null && typeof(bizCard.parentNode) != "undefined") {
				dojo.place(node, bizCard.parentNode);
				dojo.destroy(bizCard);
			}
		}
	}
};

com.ibm.bpm.social.widgets.common.BizCard._hideTooltip = function(){
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
};
/** Listeners to close the default business card **/
dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM,com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
/** End listeners to close the default business card **/

com.ibm.bpm.social.widgets.common.BizCard._showTooltip = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_showTooltip entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
	
	var data = {
			'fullName': user['fullName'],
			'userName': user['userName'], 
			'displayName': user['fullName'],
			'title': typeof(user['userPreferences']['Title'])!='undefined' ? user['userPreferences']['Title'] : '',
			'taskEmailAddress': typeof(user['userPreferences']['Task Email Address'])!='undefined' ? user['userPreferences']['Task Email Address'] : '',
			'phoneNumber': typeof(user['userPreferences']['Phone Number'])!='undefined' ? user['userPreferences']['Phone Number'] : '',
			'includeTaskDetails': false
	};
	
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
	
	com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = new dijit.TooltipDialog({
        content: com.ibm.bpm.social.widgets.common.BizCard._activityOwner.domNode,
        onMouseLeave: function(){
			dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        },
		onMouseEnter: function() {
			//If we get this event it means we definitely arent using an IPAD, so I can
			//cancel the timer going
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
        },
		//handled by bizCardConnect (defect 124992)
        //onClick: function(){
		//    console.log("in onclick");
        //	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        //},
		onKeyDown: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				node.focus();
			}
		},
		onKeyUp: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			//don't start the close timer just because of a key up (defect 124992)
			//com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();
			if(dojo.isIE || com.ibm.bpm.social.util.common.Util.isChrome) {
				var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
				if(keypressed == dojo.keys.TAB) {
					if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
						dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
						node.focus();
					}
				}
			}
		}
    });
	
	//Need to close when user clicks outside of dialog, since we close when they click on the dialog
	//might as well go all the way with this.  Should clear up most issues with IPAD
	var _bizCardConnect = dojo.connect(window, "onmousedown", function(event) {
		try{
		    //only close the window when user clicks outside of the bizcard popup
			//Also use event.target and not event.originalTarget,which
			//is only supported by Firefox.
			//(defect 124992)
			if (dojo.isDescendant(event.target, com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.domNode) == false) {
			//if(event.originalTarget.getAttribute("userName")==null){
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				dojo.disconnect(_bizCardConnect);
			}
		} catch (e) {
			
		}
	});
	com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();

	/** Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var _stack = [];
	while(dijit.popup._stack.length > 0){
		_stack.push(dijit.popup._stack.pop());
	}
	/** End Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	
	dijit.popup.open({
		popup: com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog,
		around: dojo.byId(node)
	});
	
	//Configure fix for accessibility defect 88867
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.setAttribute("tabindex", 0);
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.focus();
	
	
	node.setAttribute("tabindex", 0);
	/** Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var tmp = dijit.popup._stack.pop();
	while(_stack.length > 0){
		dijit.popup._stack.push(_stack.pop());
	}
	dijit.popup._stack.push(tmp);
	/** End Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/

};

/**
 * These timers are primarily used for the ipad, however it also affects desktop browser
 * behavior.  The issue is that if a user opens a dialog, but never enters it, then the dialog
 * just sticks around forever.  This will close the dialog after a certain amount of time
 * if the user doesn't "enter" the dialog
 */
com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer = function() {
	com.ibm.bpm.social.widgets.common.BizCard._closeTimer = 
		window.setTimeout(com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer, 3000);
};	

com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer = function() {
	if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
		window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
	}
	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
};
/**
 * Product-specific integrations below
 * TODO: This could be refactored to allow for dynamic evaluation of available integrations
 */

/**
 * IBM Connections
 */

com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapElement_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["fullName"])!="undefined" 
			&& typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
				&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapElement(node, user["fullName"], user["emailAddress"]);
		}
	}
	return success;
};

com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapImage_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
			&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapImage(node, user["emailAddress"]);
		}
	}
	return success;
};


