/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.debug.LoggingWidget");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.declare(
	"com.ibm.bpm.social.widgets.debug.LoggingWidget",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.debug", "templates/LoggingWidget.html"),
	constructor: function() {
		this.loggingWidgetConnectors = [];
		if(com && com.ibm && com.ibm.bpm && com.ibm.bpm.social && com.ibm.bpm.social.debug) {
			this.loggingTopic = com.ibm.bpm.social.debug.MESSAGE_LOGGED_TOPIC;
		}
		else {
			this.loggingTopic = "defaltLoggingTopic";
		}
		this.list = null;
		this.loggingSubscription = null;
		
		this.descriptionText = "Logging...";
		this.selectAllText = "Select All";
		this.minimizeText = "Minimize";
		this.maximizeText = "Maximize";
	},
	
    postCreate: function() {
		this.inherited(arguments);
		
		//load all existing logs
		if(com && com.ibm && com.ibm.bpm && com.ibm.bpm.social && com.ibm.bpm.social.debug) {
			this.loggingTopic = com.ibm.bpm.social.debug.MESSAGE_LOGGED_TOPIC;
			var existingLogs = com.ibm.bpm.social.debug.getLogs();
			if(existingLogs != null) {
				for(var i=0; i<existingLogs.length; i++) {
					this.addLog(existingLogs[i].type, existingLogs[i].message);
				}
			}
		}
		
		//subscribe to updates (whenever a message is logged)
		this.loggingSubscription = dojo.subscribe(this.loggingTopic, this, function(msgObj) {
			if(msgObj != null) {
				this.addLog(msgObj.type, msgObj.message);
			}
		});
		
		//add text and listeners
		if(this.selectAllButton != null) {
			dojo.attr(this.selectAllButton, "value", this.selectAllText);
			this.loggingWidgetConnectors.push(dojo.connect(this.selectAllButton, "click", dojo.hitch(this, this.highlightAllText)));
		}
		if(this.changeSizeButton != null) {
			dojo.attr(this.changeSizeButton, "value", this.minimizeText);
			this.loggingWidgetConnectors.push(dojo.connect(this.changeSizeButton, "click", dojo.hitch(this, this.togglePanel)));
		}
		if(this.descriptionNode != null) {
			dojo.attr(this.descriptionNode, "innerHTML", this.descriptionText);
		}
		
		//hide the select all button on ipad
		if(com.ibm.bpm.social.util.common.Util.isIPad) {
			dojo.style(this.selectAllButton, "display", "none");
		}
	},
	
	highlightAllText: function() {
		if(this.list != null && !com.ibm.bpm.social.util.common.Util.isIPad) {
			//remove old highlighting
			if(document.selection) {
				document.selection.empty();
			}
			else if(window.getSelection) {
				window.getSelection().removeAllRanges();
			}
			
			//highlight text
			if(document.selection) {
				var range = document.body.createTextRange();
				range.moveToElementText(this.list);
				range.select();
			}
			else if(window.getSelection) {
				var range = document.createRange();
				range.selectNode(this.list);
				window.getSelection().addRange(range);
			}
		}
	},
	
	isScrolledToBottom: function() {
		if(this.logList == null) {
			return false;
		}
		return this.logList.scrollTop + this.logList.offsetHeight + 5 >= this.logList.scrollHeight;
	},
	
	scrollToBottom: function() {
		if(this.logList != null) {
			this.logList.scrollTop = this.logList.scrollHeight;
		}
	},
	
	isMinimized: function() {
		return this.domNode != null && dojo.hasClass(this.domNode, "minimized");
	},
	
	togglePanel: function() {
		if(this.domNode != null) {
			if(this.isMinimized()) {
				this.maxmimizePanel();
			}
			else {
				this.minimizePanel();
			}
		}
	},
	
	minimizePanel: function() {
		if(this.domNode != null) {
			dojo.addClass(this.domNode, "minimized");
			if(this.changeSizeButton != null) {
				dojo.attr(this.changeSizeButton, "value", this.maximizeText);
			}
		}
	},
	
	maxmimizePanel: function() {
		if(this.domNode != null) {
			dojo.removeClass(this.domNode, "minimized");
			if(this.changeSizeButton != null) {
				dojo.attr(this.changeSizeButton, "value", this.minimizeText);
			}
			this.scrollToBottom();
		}
	},
	
	addLog: function(type, msg) {
		if(this.list == null) {
			this.list = dojo.create("ol", null, this.logList);
		}
		var wasScrolledToBottom = this.isScrolledToBottom();
		var li = dojo.create("li", {'class': "bpm-process-portal-logging-widget-"+type+"-message"}, this.list);
		var msgSpace = dojo.create("span", {'class': "bpm-process-portal-logging-widget-message", 'innerHTML': ""+msg}, li);
		
		//maintain scroll to bottom
		if(wasScrolledToBottom) {
			this.scrollToBottom();
		}
	},
    
	destroy: function(finalize) {
		dojo.forEach(this.loggingWidgetConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		if(this.loggingSubscription != null) {
			dojo.unsubscribe(this.loggingSubscription);
		}
		this.inherited(arguments);
	}
});