/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.details.TaskDetailsListItem");

dojo.require("dijit._Templated");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");

dojo.require("com.ibm.bpm.social.widgets.common.BizCard");

dojo.require("dojo.date.locale");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.declare("com.ibm.bpm.social.widgets.details.TaskDetailsListItem", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                                        dijit._Templated], {

  templateString: "",
  templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.details", "templates/TaskDetailsListItem.html"),

  taskId: null,
  taskName: null,
  taskAssignee: null,
  taskAssigneeDisplayName: null,
  taskOwner: null,
  taskOwnerDisplayName: null,
  taskStarted: null,
  taskCompleted: null,
  taskDue: null,
  taskStatus: null,
  taskPriority: null,

  sidebar: false,

  _userProfileChangeSubscribeHandle: null,
  _taskDetailsChangeHandle: null,
  _onlineTracking: null,

  postCreate: function(){
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    this.taskIconImg.src = dojo.moduleUrl("dojo", "resources/blank.gif");

    this._applyTaskData();

    this._registerListeners();
  },

  _applyTaskData: function(){

    if (this.taskStatus != null) {
      dojo.addClass(this.domNode, this.taskStatus.toLowerCase());

      if (this.taskStatus.toLowerCase() == "closed") {
        dojo.addClass(this.taskIconImg, "bpm-social-task-complete-avatar");
        dojo.removeClass(this.taskIconImg, "bpm-social-unfinished-task-avatar");
      }

      this.taskIconImg.setAttribute('title', this.taskStatus);
      this.taskIconImg.setAttribute('alt', this.taskStatus);
    }

    this.taskNameField.innerHTML = this.taskDisplayName;
    this.taskNameField.title = this.taskDisplayName;

    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskNameField);

    this.updateTaskParticipants();

    var str, d, ds, dt;
    if (this.taskStarted != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskStarted);
      this.taskStartedField.innerHTML = this.messages.created.replace("${date}", dateStr);
    }

    if (this.taskCompleted != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskCompleted);
      this.taskCompletedField.innerHTML = this.messages.completed.replace("${date}", dateStr);
      dojo.style(this.taskDueField, "display", "none");
      dojo.style(this.taskCompletedField, "display", "block");
    }
    else if (this.taskDue != null) {
      var dateStr = com.ibm.bpm.social.util.common.Util.formatDate(this.taskDue);
      this.taskDueField.innerHTML = this.messages.due.replace("${date}", dateStr);
      dojo.style(this.taskCompletedField, "display", "none");
      dojo.style(this.taskDueField, "display", "block");
    }

  },

  updateTaskParticipants: function(){

    var user = null;

    if (this.taskOwner != null) {

      user = com.ibm.bpm.social.util.common.Util.getUser(this.taskOwner);
      this.taskOwnerDisplayName = user.fullName;

      this.taskOwnerField_1.innerHTML = this.taskOwnerDisplayName;
      this.taskOwnerField_2.innerHTML = this.taskOwnerDisplayName;
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskOwnerField_1);
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.taskOwnerField_2);
    }
    else if (this.taskAssignee != null) {

      user = com.ibm.bpm.social.util.common.Util.getUser(this.taskAssignee);

      this.taskOwnerField_1.innerHTML = this.taskAssigneeDisplayName;
      this.taskOwnerField_2.innerHTML = this.taskAssigneeDisplayName;

   }

    if (user != null && typeof (user["userID"]) != "undefined") {
      /** Business Card integration * */
      com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.taskOwnerField_1, user);
      com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.taskOwnerField_2, user);
      this._avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContainer);
      this._avatar.setUser(user, " bpm-social-default-avatar-small", true);
      /** End Business Card integration * */
    }

  },

  getOnlineTracking: function(){
    if (this._onlineTracking != null) {
      return this._onlineTracking;
    }
  },

  _onTaskSubjectClicked: function(e){

    this._checkCollaborationSessionAvailablity(this.taskId);

    /*dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
    	"taskId": this.taskId,
    	"params": []
    }]);
    */
  },

  _onTaskSubjectKeyPress: function(e){
    if (e.keyCode == dojo.keys.ENTER || e.charOrCode == " ") {
      this._onTaskSubjectClicked(e);
    }
  },

  /* _checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
  _checkCollaborationSessionAvailablity: function(taskID){
    // Response will look something like this:
    // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
    if (taskID) {
      /* /ProcessPortal/notification/roster/task/ */
      dojo.xhrGet({
        url: this.getProxyUrl() +window.endpointURLObject.processPortalNotificationJS +"/roster/task/" + taskID,
        handleAs: "json",
        preventCache: true,
        sync: true,
        headers: {
          "Content-Type": "application/json",
          "Accept": "application/json"
        },
        load: dojo.hitch(this, function(data, xhrArgs){
          this._checkNumberOfParticipants(data, xhrArgs, taskID);
        }),
        error: dojo.hitch(this, this._onXhrErrors)
      });
    }
  },


  _onXhrErrors: function(data, xhrArgs){

    com.ibm.bpm.social.util.common.Log.logEntry(1, "Error communicating with the server.");
    //console.debug([data, xhrArgs]);
  },

  /* _checkNumberOfParticipants: See if there aren't too many participants in this task already. */
  _checkNumberOfParticipants: function(data, xhrArgs, taskID){
    // data will look something like this:
    // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
    // or this:
    // {"message":"NO_SESSION_FOR_TASK"}

    if (!data) {
      return;
    }
    // If there's no collab session, then data won't contain an element called 'participants',
    // and in that case we go ahead and publish the show task topic.
    var doPublishShowTaskCompletionTopic = false;
    if (data.participants) {
      var participants = data.participants;
      var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
      var currentUser = null;
      if (userProfileObj) {
        currentUser = userProfileObj.userName;
      }
      if (participants && (participants.length < 2 || this._currentUserIsAlreadyInSession(currentUser, participants))) {
        // publish the show task topic
        doPublishShowTaskCompletionTopic = true;
      }
      else {
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.messages.collaborationSessionFull,
          null /* details */, function(){
            return;
          });
      }
    }
    else {
      doPublishShowTaskCompletionTopic = true;
    }
    if (doPublishShowTaskCompletionTopic) {
      // publish the show task topic
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
        "taskId": taskID
      }]);
    }
  },

  _currentUserIsAlreadyInSession: function(currentUser, participants){
    if (currentUser && participants) {
      for ( var i in participants) {
        if (participants[i].userName == currentUser) {
          return true;
        }
      }

    }
    return false;
  },

  _registerListeners: function(){
    if (this._userProfileChangeSubscribeHandle == null) {
      this._userProfileChangeSubscribeHandle =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, dojo
          .hitch(this, this._handleUserProfileChange));
    }

    if (this._taskDetailsChangeHandle == null) {
      this._taskDetailsChangeHandle =
        dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this.taskId, dojo.hitch(this,
          this._handleTaskDetailsChange));
    }
  },

  _handleUserProfileChange: function(obj){
    /**
     * Will only trigger off of local changes (based on subscription) by the current user, hence no check for
     * IsCurrentUser *
     */
    if (obj.username && obj.username == this.taskOwner) {
      var user = com.ibm.bpm.social.util.common.Util.getUser(obj.username);
      this._avatar.setUser(user, "bpm-social-default-avatar-medium", true);
    }
  },

  _handleTaskDetailsChange: function(obj){

    var skipCache = true;
    if (typeof (obj.skipCache) != "undefined") {
      skipCache = obj.skipCache;
    }

    var task = com.ibm.bpm.social.util.common.Util.getTaskData({
      taskId: this.taskId,
      skipCache: skipCache
    });

    this.taskName = task.name;
    this.taskDisplayName = task.displayName;
    this.taskAssignee = task.assignedToDisplayName;
    this.taskOwner = task.owner;
    this.taskStarted = task.startTime;
    this.taskCompleted = task.completionTime;
    this.taskDue = task.dueTime;
    this.taskStatus = task.status;
    this.taskPriority = task.priority;

    this._applyTaskData();
  },

  destroy: function(){
    if (this._userProfileChangeSubscribeHandle != null) {
      dojo.unsubscribe(this._userProfileChangeSubscribeHandle);
    }

    if (this._taskDetailsChangeHandle != null) {
      dojo.unsubscribe(this._taskDetailsChangeHandle);
    }

    if (this._avatar) {
      this._avatar.destroy();
      this._avatar = null;
    }
    this.inherited(arguments);
  }

});
