/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");

dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");

dojo.declare("com.ibm.bpm.social.widgets.dialogs.ErrorDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString: "<div><div class='bpm-social-dialog-box'>"
    + "<div dojoAttachPoint='message' class='bpm-social-dialog-message'></div>"
    + "<div dojoAttachPoint='detailsTitle' style='display:inline'></div>"
    + "<div dojoAttachPoint='detailsContent' class='bpm-social-dialog-details-content'></div>" + "</div>"
    + "<div dojoAttachPoint='buttonContainer' class='bspace-dialog-buttonContainer'></div>"
    + "<div style='clear: both'></div>" + "</div>",

  _message: "",
  _label: null,
  _details: null,

  /**
   * Creates a popup which will show a message.
   * 
   * @param message The message to be shown
   * @param details The details to be shown
   * @param label The label of the confirmation button
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    com.ibm.bpm.social.util.common.Log.logEntry();
    if (typeof params.message === 'string') {
      this._message = params.message;
    }
    if (typeof params.label === 'string') {
      this._label = params.label;
    }
    if (typeof params.details === 'string') {
      this._details = params.details;
    }
    com.ibm.bpm.social.util.common.Log.logExit();
  },

  postCreate: function postCreate_ErrorDialog(){
    this.message.innerHTML = this._message;
    var self = this;
    if (this._label != null) {
      this.addButton(this.id + "_confirmationButton", this._label, function(){
        self.hide();
      });
    }

    // If details exist, create a title pane for it.
    if (this._details) {
      var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

      var detailsTitlePic = document.createElement("div");
      detailsTitlePic.innerHTML = "+";
      dojo.addClass(detailsTitlePic, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitlePic);

      var detailsTitleLabel = document.createElement("div");
      detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
      dojo.addClass(detailsTitleLabel, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitleLabel);

      this.detailsTitle.tabIndex = "0";

      dojo.style(this.detailsContent, "display", "none");
      this.detailsContent.innerHTML = this._details;

      var detailsHidden = true;
      this.connect(this.detailsTitle, "ondijitclick", function(e){
        if (detailsHidden) {
          dojo.style(this.detailsContent, "display", "block");
          detailsHidden = false;
          detailsTitlePic.innerHTML = "-";
          detailsTitleLabel.innerHTML = bundle["dialog_details_hide"];
        }
        else {
          dojo.style(this.detailsContent, "display", "none");
          detailsHidden = true;
          detailsTitlePic.innerHTML = "+";
          detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
        }
      });
    }
    else {
      dojo.style(this.detailsTitle, "display", "none");
      dojo.style(this.detailsContent, "display", "none");
    }

    this.connect(this.popup, 'onHide', "_finish");
    this.inherited("postCreate", arguments);
  },

  _finish: function _finish_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    this.onClose();
    // Basically this is also a dijit._Widget with a meaningful destroy method.#
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
    com.ibm.bpm.social.util.common.Log.logEntry();
  },

  /**
   * Event handling hook.
   */
  onClose: function onClose_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    com.ibm.bpm.social.util.common.Log.logExit();
  }

});

/**
 * Shows an info popup. For example in case an ad hoc event has been started but no immediate task is
 * available.
 * 
 * I.e. dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional} If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["info_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-info"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };


/**
 * Creates a modal warning popup with the provided message. <br />
 * I.e. dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["warning_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-warning"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };

/**
 * Creates a modal error popup with the provided message. <br />
 * I.e. dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["error_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-error"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };
