/*
Licensed Materials - Property of IBM
5724-M23, 5724-M24, 5724-L01, 5724-R31
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("dojo.cache");
dojo.registerModulePath("com.ibm.socialPortal.templates", window.endpointURLObject.processPortalJS +"/com/ibm/bpm/social/widgets/dialogs/templates");
dojo.declare("com.ibm.bpm.social.widgets.dialogs.GoToSpacesDialog", com.ibm.bspace.common.util.ui.Dialog, {
	templateString: dojo.cache("com.ibm.socialPortal.templates", "GoToSpacesDialog.html"),

     postCreate: function() {
         this.connections = new Array();

		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.goToSpacesMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");		
	
		// Update Bspace Go to Spaces styling elements for social portal
		this.cancelButton.value = this.goToSpacesMessages.cancel;
		this.cancelButton.title = this.goToSpacesMessages.cancel;					
		this.filterText.innerHTML = this.goToSpacesMessages.goToSpaces_navigateMessage;

 		dojo.require("com.ibm.bspace.common.util.ui.FilterField");
 		dojo.require('com.ibm.bspace.common.util.ui.Helper');

 		com.ibm.bspace.common.util.ui.Helper.loadStyleSheet();
		
 		this.divFilterHolder = document.createElement("div");
 		if(document.body.dir=='rtl'){
 			this.divFilterHolder.className = "goToSpacesFilterField_rtl";
 		}
 		this.goToSpacesFilterField = new com.ibm.bspace.common.util.ui.FilterField({hint: this.goToSpacesMessages.goToSpaces_filter, title: this.goToSpacesMessages.goToSpaces_filterTitle, width: 200}, this.divFilterHolder);
 		
 		this.goToSpacesFilterField.setOnFilterCallback(this, this.applyFilter);
 		this.filter.appendChild(this.goToSpacesFilterField.domNode);

         this.inherited(arguments);
     },
 	applyFilter: function(filterValue) {
 		var f = filterValue.toLowerCase();
 		if (this.spaceList) {
 			for (var i=0; i<this.spaceList.length; i++) {
 				var show = 'block';
 				if (f) {
 					if (this.spaceList[i].title.toLowerCase().indexOf(f) == -1) {
 						show = 'none';
 					}
 				}
 				this.spaceList[i].div.style.display = show;
 			}
 		}
 	},
	_handleKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			var focus = dijit._curFocus;
			var next = null;
			if (this.isPopup) {
				next = this.goToSpacesFilterField.filterBox;
			}
			if (focus == this.goToSpacesFilterField.filterBox) {
				var div = this.spacesList.firstChild;
				while (div && div.style.display == 'none') {
					div = div.nextSibling;
				}
				if (e.shiftKey) {
					if (this.isPopup) {
						div = this.spacesList.lastChild;
						while (div && div.style.display == 'none') {
							div = div.previousSibling;
						}
					} else {
						div = null;
					}
				}
				if (div) {
					var id = div.id.substring(14);
					next = document.getElementById(id);
				}
			} else {
				var focus = document.getElementById('spaceEntryDiv_'+focus.id);
				if (focus) {
					var div = focus.nextSibling;
					while (div && div.style.display == 'none') {
						div = div.nextSibling;
					}
					if (e.shiftKey) {
						div = focus.previousSibling;
						while (div && div.style.display == 'none') {
							div = div.previousSibling;
						}
					}
					if (div) {
						var id = div.id.substring(14);
						next = document.getElementById(id);
					}
				}
			}
			if (next) {
				next.focus();
				dojo.stopEvent(e);
			}
		}
	},
	
	refresh: function() {
		while (this.connections.length > 0) {
			dojo.disconnect(this.connections.pop());
		}
		this.connections.push(dojo.connect(this.domNode,'onkeypress',this,'_handleKeyPress'));
		this.connections.push(dojo.connect(this.cancelButton,'onclick',this,'hide'));

		this.spacesList.innerHTML = "";
		this.getSpaces();

		this.dirty = false;
	},
	
	_compareSpaces: function(space1, space2) {
		var rc = 0;
		if (space1.title < space2.title) {
			rc = -1;
		}
		if (space1.title > space2.title) {
			rc = 1;
		}
		return rc;
	},
	
	_addSpaceDiv: function(data, messages) {
		
		var isLTR = document.body.dir != 'rtl';
		var spaceEntryDiv = document.createElement("div");
		spaceEntryDiv.id = 'spaceEntryDiv_'+data.id;
		spaceEntryDiv.style.clear = "both";
		spaceEntryDiv.style.padding = "4px 0px 4px 0px";
		spaceEntryDiv.style.width = "100%";					
		
		var spaceIconDiv = document.createElement("div");
		if (document.body.dir == 'rtl') {
			spaceIconDiv.innerHTML="<img src=\""+data.icon+"\" alt=\"\" style=\"padding-right:3px; height:16px; width:16px;\"></img>&nbsp;";
		}
		else {
			spaceIconDiv.innerHTML="<img src=\""+data.icon+"\" alt=\"\" style=\"padding-left:3px; height:16px; width:16px;\"></img>&nbsp;";
		}
		dojo.style(spaceIconDiv, "cssFloat", isLTR ? "left" : "right");	
		spaceEntryDiv.appendChild(spaceIconDiv);
							
		var spaceLinkDiv = document.createElement("div");
		dojo.style(spaceLinkDiv, "cssFloat", isLTR ? "left" : "right");
		dojo.attr(spaceLinkDiv,"role","listitem");
		dojo.attr(spaceLinkDiv,"aria-labelledby", data.id+"_bsGoToSpacesSpaceOwnerDiv");
							
		var a = document.createElement('a');
		a.id = data.id;
		a.href = "#";
		a.innerHTML = data.title;
		dojo.attr(a,"role","link");
		
		///// Defect # 100590 and 100846
		var titleStr = data.description + " - " + dojo.string.substitute(messages.goToSpaces_ownerLabel, [data.owner]);
		a.title = titleStr;
		
		spaceLinkDiv.appendChild(a);
		this.connections.push(dojo.connect(a,'onclick',this,'_changeSpace'));
		spaceEntryDiv.appendChild(spaceLinkDiv);
		
		//accessibility for JAWS									
		var spaceOwnerLabelledByDiv = document.createElement("span");
		var str = "";
		if (data.description) str = data.description + ".  ";
		spaceOwnerLabelledByDiv.innerHTML = str + dojo.string.substitute(messages.goToSpaces_ownedBy, [data.owner]);
		spaceOwnerLabelledByDiv.id = data.id+"_bsGoToSpacesSpaceOwnerDiv";
		dojo.attr(spaceOwnerLabelledByDiv,"style",{"display":"none"});
		spaceEntryDiv.appendChild(spaceOwnerLabelledByDiv);																
							
		var spaceOwnerDiv = document.createElement("div");
		spaceOwnerDiv.className = "goToSpacesListOwner";
		spaceOwnerDiv.innerHTML = data.owner;
		spaceOwnerDiv.style.color = "#999999";
		dojo.style(spaceOwnerDiv, "cssFloat", isLTR ? "right" : "left");
		if (document.body.dir == 'rtl') {
			spaceOwnerDiv.style.textAlign = "left";
			spaceOwnerDiv.style.paddingLeft = "10px";
		}
		else {
			spaceOwnerDiv.style.textAlign = "right";
			spaceOwnerDiv.style.paddingRight = "10px";
		}					
		spaceEntryDiv.appendChild(spaceOwnerDiv);
		data.div = spaceEntryDiv;
		
		this.spacesList.appendChild(spaceEntryDiv);
	},
	
	_changeSpace: function(e) {
		var pageId = e.target.id;
		location.href = window.endpointURLObject.mumJS +"/enabler#pid=" + pageId;
    },
    
    getSpaces: function() {
		var self = this;
		var xhrArgs = {
			url: window.endpointURLObject.mumJS +"/mycontenthandler?uri=space%3Acollection&num=20&start=0&aspect=ac&nocache=true",
			handleAs: "xml",
			preventCache: true,
			load: function(data){
				var list = new Array();
				var docNode = data.documentElement;

			   // Write text content into the display.
			   for(var i = 0; i < docNode.childNodes.length; i++){
				 var feedElement = docNode.childNodes[i];
			     if (feedElement.nodeName == "atom:entry") {

			    	 var spaceName = "Space";
			    	 var spaceId = "";
			    	 var spaceIcon = "";
			    	 var spaceOwner = "";
			    	 var isHidden = false;
			    	 var canEdit = false;
					 var isLegacyProcessPortalSpace = false;
			    	 var spaceElements = feedElement.childNodes;
			    	 for (var j=0; j<spaceElements.length; j++) {
			    		 
			    		 // Ignore the space ID (found in the atom:id element), and instead get the navigation
			    		 // model ID which is the ID of the first page in the space.  This will be used to
			    		 // form the URL which will take us directly to the space.
			    		 if (spaceElements[j].nodeName == "atom:link" && spaceElements[j].getAttribute("ca:rel") && spaceElements[j].getAttribute("ca:rel") == "navigation") {
			    			 // The ID is stored in the format "nm:id:PAGEID"
			    			 var navigationId = spaceElements[j].getAttribute("ext:uri");
			    			 if (navigationId) {
			    				 var idTokens = navigationId.split(":");
				    			 spaceId = idTokens[idTokens.length-1];
			    			 }
			    		 }
			    		 
			    		 // Look for space title to display. This is a fallback in case the
			    		 // localized title in the atom:content doesn't exist.
			    		 if (spaceElements[j].nodeName == "atom:title") {
			    			 spaceName = spaceElements[j].firstChild.nodeValue;
			    		 }
			    		 
			    		 // Look for space content which contains the metadata for the space.
			    		 if (spaceElements[j].nodeName == "atom:content") {
			    			 
			    			 var metadataElements = spaceElements[j].firstChild.childNodes;
			    			 for (var k=0; k<metadataElements.length; k++) {
			    				 // Look for space icon metadata.
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}iconUrl") {
			    					 spaceIcon = metadataElements[k].firstChild.nodeValue;
			    				 }
			    				 
			    				 // Look for space owner display name.
			    				 if (metadataElements[k].nodeName == "ca:owner") {
			    					 spaceOwner = metadataElements[k].getAttribute("ca:display-name");

			    				 }
			    				 
			    				 // If found, overwrite the default title with the localized one.
			    				 if (metadataElements[k].nodeName == "ca:title") {
			    					 var locale = metadataElements[k].getAttribute("xml:lang");
			    					 if (locale == dojo.locale) {
			    						spaceName =  metadataElements[k].firstChild.nodeValue;
			    					 }
			    				 }
								 
								 // Look for flag indicating this is a legacy BPM 8.0 Process Portal space
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && 
			    						 metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.bspace.bpm.socialSpace") {
			    					 isLegacyProcessPortalSpace = (metadataElements[k].firstChild.nodeValue === 'true');
			    				 }
			    				 
			    				 // Look for the hidden flag (absense of flag means space is not hidden).
			    				 //<ca:meta-data ca:name="{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.mashups.space.hidden">true</ca:meta-data>
			    				 if (metadataElements[k].nodeName == "ca:meta-data" && metadataElements[k].getAttribute("ca:name") && 
			    						 metadataElements[k].getAttribute("ca:name") == "{http://www.ibm.com/xmlns/prod/lotus/mashups/v1.0/extensions}com.ibm.mashups.space.hidden") {
			    					 isHidden = (metadataElements[k].firstChild.nodeValue === 'true');
			    				 }
			    				 
			    				 // If the user has either the EDITOR or MANAGER role, then they are allowed to edit the space.
			    				 if (metadataElements[k].nodeName == "ac:allowed-access" && metadataElements[k].getAttribute("xmlns:ac") && 
			    						 metadataElements[k].getAttribute("xmlns:ac") == "http://www.ibm.com/xmlns/prod/lotus/access-control/v1.0") {
					    			 var accessLevelElements = metadataElements[k].childNodes;
					    			 for (var m=0; m<accessLevelElements.length; m++) {
					    				 var accessType = accessLevelElements[m].getAttribute("ac:type");
					    				 if (accessType && (accessType == "EDITOR" || accessType == "MANAGER")) {
					    					 canEdit = true;
											 break;
					    				 }
					    			 }
			    				 }
			    			 }
			    		 }
			    	 }
			    	 
			    	 
					// Only include the space if it is not hidden or if the user is an owner/editor of the space.
					// If the space is hidden and the user is only a viwer then it should not display.
					if (!isLegacyProcessPortalSpace) {
						if (!isHidden || canEdit) {
							list.push({
									id: spaceId,
									title: spaceName,
									owner: spaceOwner,
									description: spaceName,
									icon: spaceIcon
							});
						}
					}
			     }
			   }
			   
				list.sort(function(space1, space2) {
					var rc = 0;
					if (space1.title < space2.title) {
						rc = -1;
					}
					if (space1.title > space2.title) {
						rc = 1;
					}
					return rc;
				});

				for (var i=0; i<list.length; i++) {
					self._addSpaceDiv(list[i], self.goToSpacesMessages);
				}
				self.spaceList = list;
				if (self.goToSpacesFilterField) {
					self.goToSpacesFilterField.reset();
				}
				
			},
			 error: function(error, ioargs){
				var errorDialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({message: "" + error });
				errorDialog.show();
			}
		};

		// Call the asynchronous xhrGet
		var deferred = dojo.xhrGet(xhrArgs);    	
    },
	
	show: function() {
		this.inherited(arguments);
		this.refresh();
	}
});