/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012,2013. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.InvitationDialog");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.UserChooser");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.models.BPMModel");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.widgets.experts.DialogExpertTile");
dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("com.ibm.bpm.social.util.ui.Avatar");
dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

/**
 * Invitation dialog used when inviting people to a group task.
 * 
 */
dojo.declare("com.ibm.bpm.social.widgets.dialogs.InvitationDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.dialogs", "templates/InvitationDialog.html"),

  constructor: function(parameters){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Life cycle management:
    // As dijit._Widget this.connect should be used in order have everything freed upon destruction.
    // The destroy method will be called in event handler to onHide.
    // Might not be used as the com.ibm.bspace.common.util.ui.Dialog.postCreate already calls Helper.wrapElement that 
    // wraps the elements out of the template and should take care of them when destroying.
    //
    this._wrappedElements = [];

    // TODO There could be a configurable limit. So far it is 5. If it changes we need to ask for that limit.
    this.COLLAB_LIMIT = 5;
    //
    // Looking up helper
    //
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Processing of parameters
    //
    var task = parameters.task;

    //
    // labels used in the template
    //    
    this.labelForInvitedUsers = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTitle"];
    this.labelForInvitedUsersSubTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle"];    
    this.labelForInvitedUsersSubTitleWithoutSection =
    this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle_WithoutSection"];
    this.labelForInvitedUsersTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTooltip"];
    this.labelForMessageSection = this._nlsResources["task_dialog_startCollaboration_messageSectionTitle"];
    this.labelForMessageSectionPlaceholder = this._nlsResources["task_dialog_startCollaboration_messageSectionPlaceholder"];
    // this.labelForMessageSectionTitle = this._nlsResources["task_dialog_startCollaboration_messageSectionTooltip"];
    this.labelForInviteButton = this._nlsResources["task_dialog_startCollaboration_saveButton"];
    this.labelForInviteButtonTitle = this._nlsResources["task_dialog_startCollaboration_saveButtonTooltip"];
    this.labelForSuggestedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_suggestedCollaborators"];
    this.labelForInvitedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_invitedCollaborators"];
    this.labelForSuggestedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_suggestedCollaboratorsTooltip"];
    this.labelForInvitedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_invitedCollaboratorsTooltip"];
    this.labelForMore = this._nlsResources["task_dialog_startCollaboration_more"];
    this.labelForBack = this._nlsResources["task_dialog_startCollaboration_back"];
    //
    //
    // Normalizing properties because the properties used by query are different from
    // the properties returned by GET /task
    // Although there is currently no use case where the provided task is retrieved using a query.
    //
    if (typeof task.TKIID === "string") {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping;
    }
    else {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping;
    }
    this._availableActions = parameters.availableActions;
    this._tkiid = task[this.mapping["TKIID"]];
    this._taskName = task[this.mapping["NAME"]];
    this._taskDisplayName = task[this.mapping["DISPLAY_NAME"]];
    this._processInstanceName = task[this.mapping["PI_NAME"]];
    this._piid = task[this.mapping["PIID"]];
    // Remember whether a user is expert or frequent collaborator
    this._userCategory = {};
    //
    // Assumption: task has already an owner. Otherwise inviting does not make sense.
    // 
    this._currentOwner = {
      value: task[this.mapping["OWNER"]],
      type: "user"
    };
    this.placeHolderState = {
      modified: true
    };
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  postCreate: function postCreate_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calling com.ibm.bspace.common.util.ui.Dialog.postCreate and thereby wrapping the radio buttons and buttons.
    // Because the Dialog.postCreate method calls com.ibm.bspace.common.util.ui.Helper.wrapElement which
    // finds all markup that needs to be wrapped.
    //
    this.inherited("postCreate", arguments);
    this.connect(this.popup, 'onHide', function(){
      // Basically this is also a dijit._Widget with a meaningful destroy method.
      // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
      setTimeout(dojo.hitch(this, this.destroy), 1000);
    });
    
    // Accessibility
    dijit.setWaiRole(this.suggestedCollaborators, "listbox");
    var suggestionsTitleId = this.id + "_suggestionsTitle";
    this.suggestionsTitle.id = suggestionsTitleId;
    dijit.setWaiState(this.suggestedCollaborators, "labelledby", suggestionsTitleId);
    dijit.setWaiRole(this.showMore, "button");     
    dijit.setWaiRole(this.showBack, "button");
    
    //
    // Custom Logic Starts Here
    //    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  showDialog: function showDialog_InvitationDialog(user){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var expertsBoxDim = this._calculateSize();
    // get the number of experts per line before the invited users are inserted
    this._calculateNumberOfExperts();
    this._setupUserChooser(user);
    if (user == null) {
      // user is not already known
      this._setupSuggestedCollaborators();
    }
    // insert the invited users in the dialog, that can be done independent of the experts callback
    // this._setupInvitedUsers();
    this.connect(this.inviteButton, "onclick", "onInvite");
    // if there is a limit of allowed collaborators then we do not need to create the whole dialog. 
    // In that case do nothing.     
    this._setupMessageBox();
    // button is disabled 
    com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, true);
    // Remove the tooltip for the dialog
    this.domNode.removeAttribute("title");
    this.show();
    // Set the width of the dialog to assure that there is no white space,
    // if the title is too long.
    dojo.style(this.popup.domNode, "width", expertsBoxDim.w + "px");
    this._connectKey = dojo.connect(this.domNode, "onkeydown", this, "_onKeyDown");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  destroy: function destroy_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    //
    // Assumption: Connections established with this.connect are disconnected in the destroy method of dijij._Widget
    //
    if (this._connectKey) {
      dojo.disconnect(this._connectKey);
      this._connectKey = null;
    }

    if (this._wrappedElements.length > 0) {
      for ( var i = 0; i < this._wrappedElements.length; i++) {
        com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements);
      }
    }
    if (this._expertsWidgets != null) {
      for ( var j = 0; j < this._expertsWidgets.length; j++) {
        var widget = this._expertsWidgets[j];
        widget.destroy();
      }
      this._expertsWidgets = null;
    }
    if (this._invitedWidgets != null) {
      for ( var j = 0; j < this._invitedWidgets.length; j++) {
        var widget = this._invitedWidgets[j];
        widget.destroy();
      }
      this._invitedWidgets = null;
    }
    this._restHandler = null;
    this._errorHandler = null;
    this._messageField = null;
    this.inherited("destroy", arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _onKeyDown: function _onKeyDown_InvitationDialog(e){
    var key = e.keyCode;
    if (key == dojo.keys.ESCAPE) {
      this.hide();
    }
  },
 
  _calculateSize: function _calculateSize_InvitationDialog(){
    // calculate the size of the area necessary for the experts/invitee box
    var size = dojo.window.getBox();
    var width = size.w;
    var height = size.h;
    // this is the size we should use. Otherwise the dialog is smaller and has scrollbars
    var expertsBoxHeight = 300;
    var expertsBoxWidth = 675;
    // the width of the window plus 32 for the rest of the dialog need to be there
    if (width - 32 < expertsBoxWidth) {
      expertsBoxWidth = (width - 32) > 0 ? width - 32 : width;
    }
    // the height of the window plus 250 for the rest of the dialog needs to be there 
    if (height - 250 < expertsBoxHeight) {
      expertsBoxHeight = (height - 250) > 0 ? height - 250 : height;
    }
    // make the dummy element that size and make it visible till the dialog is shown
    dojo.style(this.content, "width", expertsBoxWidth + "px");
    dojo.style(this.collaboratorsSectionDummy, "height", expertsBoxHeight + "px");
    dojo.style(this.collaboratorsSectionDummy, "display", "block");
    
    return {"h": expertsBoxHeight, "w": expertsBoxWidth};
  },

  _calculateNumberOfExperts: function _calculateNumberOfExperts_InvitationDialog(){
    var expertsBoxWidth = 630;
    if (this.content && this.content.offsetWidth) {
      // the scrollbar also needs some px this reduce 15
      expertsBoxWidth = this.content.offsetWidth - 25;
    }
    // for 3 experts tiles we need 600
    if (expertsBoxWidth >= 630) {
      this._numberExpertsPerLine = 3;
    }
    else {
      // otherwise calculate how many fit
      var mod = expertsBoxWidth % 210;
      this._numberExpertsPerLine = ((expertsBoxWidth - mod) / 210);
    }
    // now make room for the real experts or just make it small
    dojo.style(this.collaboratorsSectionDummy, "height", 0 + "px");
  },

  _checkButtonEnablement: function _checkButtonEnablement_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var valid = true;
    valid = (this.userChooser.chooserInput.value.length > 0 && this.userChooser.getChoice() !== null);
    if (this.inviteButton) {
      com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, !valid);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
  },

  _setupUserChooser: function _setupUserChooser_InvitationDialog(prepopulatedUseridString){
    //
    // TODO Make sure that (de-) selecting suggested collaborators plus entering users in the user chooser is synchronized.
    //
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Create and configure the user chooser
    //
    var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
      includeGroups: false,
      includeUsers: true
    });
    var myself = this;
    myself._ucClickedFirstTime = false;
    var args = {
      searchMinInputLength: 0,
      ucModel: ucModel,
      _onUCFocus: function(e){
        if (!myself._ucClickedFirstTime) {
          myself._ucClickedFirstTime = true;
          dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
          myself.userChooser.chooserInput.value = "";
        }
        com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);

        // myself.userChooser.chooserInput.value = "";
        myself._checkButtonEnablement();
        for ( var j = 0; j < myself._expertsWidgets.length; j++) {
          var widget = myself._expertsWidgets[j];
          if (widget.isAdded) {
            widget.isAdded = false;
            dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
            widget.domNodeA11y.innerHTML = "";
          }
        }
      },
      _onUCBlur: function(e){
        var choice = myself.userChooser.getChoice();
        var valid = (myself.userChooser.chooserInput.value.length < 0 || choice !== null);
        if (!valid) {
          myself.userChooser.chooserInput.value = "";
          myself.userChooser._hideResults();
          myself._checkButtonEnablement();
        }
        else {
          // make sure only the user specified in the user chooser is marked for invite
          for ( var j = 0; j < myself._expertsWidgets.length; j++) {
            var widget = myself._expertsWidgets[j];
            if (widget.userName == choice.userName) {
              dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "&#10004;";
              widget.isAdded = true;
            }
            else if (widget.isAdded) {
              widget.isAdded = false;
              dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "";
            }
          }
        }
        dojo.addClass(myself.userChooser.noUsersFoundDiv, "hidden");
      }
    };
    this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
    this.connect(this.userChooser, "onChangeHandler", "_checkButtonEnablement");
    //
    // initially show a help text in the input field
    //      
    this.userChooser.chooserInput.value = this._nlsResources["task_dialog_startCollaboration_invitedUseridFieldHelp"];
    dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    //
    // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
    //
    this.userChooser.setCollabTaskidFilter(this._tkiid);

    // allow to set a string in the UserChooser field, e.g. for calling this dialog
    // from the Experts List, thereby pre-populating the selected user from the Experts list.
    if (prepopulatedUseridString) {
      if (prepopulatedUseridString.fullName) {
        this.userChooser._chooseItemValue(prepopulatedUseridString);
      }
      else {
        this.userChooser.chooserInput.value = prepopulatedUseridString;
      }
      myself._ucClickedFirstTime = true;
      dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    }

    // A11y: add label for user chooser
    var ucLabel = document.createElement("label");
    ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
    ucLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_invitedUseridField_label"];
    ucLabel.style.display = "none";

    this.invitedUserSection.appendChild(ucLabel);
    this.invitedUserSection.appendChild(this.userChooser.domNode);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setupMessageBox: function _setupMessageBox_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calculate the default invitation message
    //
    var taskLabel = this._taskDisplayName;
    if (!taskLabel || taskLabel === "") {
      taskLabel = this._taskName;
    }
    var instanceName = this._processInstanceName;
    if (!instanceName || instanceName === "") {
      instanceName = this._piid;
    }
    // Set id and for appropriately
    this.messageBox.id = this.popup.id + "_MessageBox";
    this.messageBoxLabel.htmlFor = this.popup.id + "_MessageBox";
    // Use placeholder instead
    /*
    var msg = dojo.string.substitute(this._nlsResources["task_dialog_startCollaboration_messageDefault"], {
        1: taskLabel,
        2: instanceName
      });
      this.messageBox.appendChild(document.createTextNode(msg));
    */
    //configure base text direction here:
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.messageBox);
    if (dojo.isIE) {
      this._addPlaceholder(this.messageBox, this.labelForMessageSectionPlaceholder, this.placeHolderState);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _addPlaceholder: function(textarea, placeHolderText, placeHolderState){
    var fontStyle = "normal";
    if (textarea.style.fontStyle != null) {
      fontStyle = textarea.style.fontStyle;
    }
    var setPlaceHolderText = function(){
      textarea.value = placeHolderText;      
      textarea.style.fontStyle = "italic";
    };
    var removePlaceHolderText = function(){
      textarea.value = "";      
      textarea.style.fontStyle = fontStyle;
    };
    // Initial Setup 
    placeHolderState.modified = false;
    setPlaceHolderText();
    // Focus received
    dojo.connect(textarea, "onfocus", function(){
      if (!placeHolderState.modified) {
        removePlaceHolderText();
      }
    });
    // Focus lost
    dojo.connect(textarea, "onblur", function(){
      if (textarea.value != "") {
        placeHolderState.modified = true;
      }
      else {
        placeHolderState.modified = false;
        setPlaceHolderText();
      }
    });

  },

  _setupSuggestedCollaborators: function _setupSuggestedCollaborators_InvitationDialog(){
    this._socialRestHandler.getExpertsForTask(this._tkiid, dojo.hitch(this, this._expertsCallback), function(error){
    // TODO Error Handling for experts call
    }, false /* not async */);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _removeCurrentUser: function _removeCurrentUser_InvitationDialog(group){
    if (group && this._currentUser) {
      for ( var i = 0; i < group.length; i++) {
        if (group[i].userName == this._currentUser) {
          group.splice(i, 1);
          i--;
        }
      }
    }
  },

  _removeDuplicateUsers: function _removeDuplicateUsers_InvitationDialog(systemIdentified, expertParticipant){
    // calculate the invited users to delete them form the experts lists
    // they are displayed separately.	
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    if (expertParticipant && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < expertParticipant.length; i++) {
        var userName = expertParticipant[i].userName;
        /*
            var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);        
            if (filter > -1) {
              expertParticipant.splice(i, 1); // splicing the current item shift the items to the left
              i--;
            }
        */
        this._userCategory[userName] = categoryName;
      }
    }
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    if (systemIdentified && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < systemIdentified.length; i++) {
        var userName = systemIdentified[i].userName;
        /*
        var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);
        */
        if (typeof this._userCategory[userName] === "string") {
          // Already an expert 
          systemIdentified.splice(i, 1); // splicing the current item shift the items to the left     
          i--;
        }
        if (typeof this._userCategory[userName] === "undefined") {
          // Rememeber the type which is not already "Expert"
          this._userCategory[userName] = categoryName;
        }
      }
    }
  },

  _expertsCallback: function _expertsCallback_InvitationDialog(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // Establish user's name
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    if (userProfileObj) {
      this._currentUser = userProfileObj.userName;
    }

    var anker = this.suggestedCollaborators;
    var taskId = this._tkiid;
    var counter = 0;
    var systemIdentified = data["systemIdentifiedUsers"];
    var expertParticipantGroup = data["expertParticipantGroupUsers"];
    var numberExpertsPerLine = this._numberExpertsPerLine;
    var displayedUsers = {};
    // Keep track of ExpertTile objects in order to destroy them later
    var expertsWidgets = this._expertsWidgets = [];
    // TODO Check whether the categories (Frequent Collaborator, Experts) can also be user defined in the plugable member service
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    // remove the current user from the lists
    this._removeCurrentUser(systemIdentified);
    this._removeCurrentUser(expertParticipantGroup);
    // remove the user which are already invited
    // insert system suggested users        
    this._removeDuplicateUsers(systemIdentified, expertParticipantGroup);
    var tileLabel = this._nlsResources["task_dialog_startCollaborationSuggested_title"];
    var myself = this;
    dojo.forEach(systemIdentified, function(item){     
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    // insert modeled experts
    dojo.forEach(expertParticipantGroup, function(item){      
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
        
    for ( var j = 0; j < this._expertsWidgets.length; j++) {
      var widget = this._expertsWidgets[j];
      this.connect(widget, "onClick", "_expertClicked");
    }
    
    if (counter == 0) {
      dojo.style(this.suggestedCollaboratorsSection, "display", "none");
      this.userChooserLabel.innerHTML=this.labelForInvitedUsersSubTitleWithoutSection;      
      this.userChooserLabel.setAttribute("title",this.labelForInvitedUsersSubTitleWithoutSection);
      com.ibm.bpm.social.util.common.Log.logExit(2);
      return;
    }
    else {
      dojo.style(this.suggestedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMore, "display", "none");
    }
    else {
      dojo.style(this.showMore, "display", "block");
    }
    
    this.connect(this.showMore, "onclick", function(){
    	this._showMore(anker);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });
    this.connect(this.showMore, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showMore(anker);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });
    
    this.connect(this.showBack, "onclick", function(){
    	this._showLess(anker, numberExpertsPerLine);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });   
    this.connect(this.showBack, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showLess(anker, numberExpertsPerLine);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },
  
  _showMore: function _showMore_InvitationDialog(anker) {
	  dojo.forEach(anker.childNodes, function(node){
	        dojo.style(node, "display", "inline-block");
	  });
	  dojo.style(this.showBack, "display", "block");
	  dojo.style(this.showMore, "display", "none"); 
  },
  
  _showLess: function _showLess_InvitationDialog(anker, numberExpertsPerLine) {
	  var i = 0;
      dojo.forEach(anker.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBack, "display", "none");
      dojo.style(this.showMore, "display", "block");
  },
  
  _createAccForDialogExpertTile: function _setWaiForDialogExpertTile_InvitationDialog(widget, pos) {
	  dijit.setWaiRole(widget.domNode,"option");
	  dijit.setWaiState(widget.domNode, "selected", "false");	   
	  dijit.setWaiState(widget.domNode, "labelledby", widget.avatar.id + " " + widget.taskDetails.id);
      if(pos === 0) {
        dojo.attr(widget.domNode, 'tabindex', "0");
      } else {
        dojo.attr(widget.domNode, 'tabindex', "-1");    	  
      } 
    
      // Keyboard handling for list navigation
      // Key down: next expert
      // Key up: previous expert 
      // Key space: select      
      this.connect(widget, "onKeyDown", function(e) {
    	  var numberOfExperts = this._expertsWidgets.length;
    	  if(e.keyCode === dojo.keys.DOWN_ARROW && pos < numberOfExperts - 1) {
            this._expertsWidgets[pos + 1].domNode.focus(); 
    	  } else if(e.keyCode === dojo.keys.UP_ARROW && pos > 0) {
    	    this._expertsWidgets[pos - 1].domNode.focus(); 
      	  } else if(e.keyCode === dojo.keys.SPACE) {
      		 this._selectWidget(widget, false); 
      	  }
      });
  },

  _setupInvitedUsers: function _invitedUsers_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // show invited users in a box
    var anker2 = this.invitedCollaborators;
    var taskId = this._tkiid;
    // Keep track of ExpertTile objects in order to destroy them later
    var invitedWidgets = this._invitedWidgets = [];
    // create a list of the invited users
    var invitedUsers = [];
    if (this.task.collaboration.currentUsers && this.task.collaboration.currentUsers.length > 0) {
      for ( var i = 0; i < this.task.collaboration.currentUsers.length; i++) {
        if (this.task.collaboration.currentUsers[i] != this._currentUser) {
          var userName = this.task.collaboration.currentUsers[i];
          var user = {
            "userName": userName
          };
          if (typeof this._userCategory[userName] === "string") {
            user["categoryName"] = this._userCategory[userName];
          }
          invitedUsers.push(user);
        }
      }
    }
    var numberExpertsPerLine = this._numberExpertsPerLine;
    // insert invited users
    var myself = this;
    var counter = 0;
    if (invitedUsers.length > 0) {
      //anker2.style.display = "block";
      // if the box is already full then do not show these entries
      if (counter > numberExpertsPerLine) {
        this.invitedCollaboratorsSection.style.display = "none";
      }
      else {
        this.invitedCollaboratorsSection.style.display = "block";
      }
      dojo.forEach(invitedUsers, function(item){
        counter++;
        item['taskId'] = taskId;
        item['includeTaskDetails'] = false;
        item['sidebar'] = false;
        item['collaborate'] = false;
        var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
        ee.domNode.title = myself._nlsResources["task_dialog_startCollaborationInvited_title"];
        invitedWidgets.push(ee);
        anker2.appendChild(ee.domNode);
        dojo.addClass(ee.domNode, "bpm-social-experts-expert-selected");
        ee.domNodeA11y.innerHTML = "&#10004;";
        if (counter > numberExpertsPerLine) {
          dojo.style(ee.domNode, "display", "none");
        }
      });
      for ( var j = 0; j < invitedWidgets.length; j++) {
        var widget = invitedWidgets[j];
        this.connect(widget, "onClick", "_invitedUserClicked");
      }
    }
    if (counter == 0) {
      dojo.style(this.invitedCollaboratorsSection, "display", "none");
    }
    else {
      dojo.style(this.invitedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMoreInvited, "display", "none");
    }
    else {
      dojo.style(this.showMoreInvited, "display", "block");
    }
    
    // show more functions for invited 
    this.connect(this.showMoreInvited, "onclick", function(){
      dojo.forEach(anker2.childNodes, function(node){
        dojo.style(node, "display", "inline-block");
      });
      dojo.style(this.showBackInvited, "display", "block");
      dojo.style(this.showMoreInvited, "display", "none");
    });
    this.connect(this.showBackInvited, "onclick", function(){
      var i = 0;
      dojo.forEach(anker2.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBackInvited, "display", "none");
      dojo.style(this.showMoreInvited, "display", "block");
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _expertClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, false);
  },

  _invitedUserClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, true);
  },

  _userClicked: function _userClicked_InvitationDialog(event, alreadyInviteed){
    var widget = dijit.getEnclosingWidget(event.target);
    if (widget instanceof com.ibm.bpm.social.util.ui.Avatar) {
      widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
    }
    this._selectWidget(widget, alreadyInviteed);
  },
  
  _selectWidget: function _selectWidget_InvitationDialog(widget, alreadyInviteed){
    if (widget != null) {
      if (!alreadyInviteed && widget.isAdded) {
        // Remove
        widget.isAdded = false;
        dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "";
        dijit.setWaiState(widget.domNode, "selected", "false");
        // Is there a clear function on the user chooser
        this.userChooser.chooserInput.value = "";
        this._checkButtonEnablement();       
      }
      else {
        // only one invitee can be selected. Therefore deselect all the others
        // TODO : grey out the already invited users. They should be display only. 	
        for ( var j = 0; j < this._expertsWidgets.length; j++) {
          var w = this._expertsWidgets[j];
          if (w.userName != widget.userName) {
            w.isAdded = false;
            dojo.removeClass(w.domNode, "bpm-social-experts-expert-selected");
            dijit.setWaiState(w.domNode, "selected", "false");
            w.domNodeA11y.innerHTML = "";
          }
        }
        widget.isAdded = true;
        dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "&#10004;";
        dijit.setWaiState(widget.domNode, "selected", "true");
        var user = {
          "userName": widget.userName,
          "fullName": widget.fullName
        };
        //
        // TODO Validate logic for setting a user on the user chooser
        //
        if (user != null) {
          if (user.fullName != null) {
            this.userChooser._chooseItemValue(user);
          }
          else {
            // Does this make sense given that user is an object !              
            this.userChooser.chooserInput.value = user;
          }
        }
      }      
    }
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getMessageText: function _getMessageText_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var messageText = "[no message specified]"; // User should never see this - no NLS necessary
    if (this.messageBox) {
      if (this.placeHolderState.modified) {
        messageText = this.messageBox.value;
      }
      else {
        messageText = "";
      }
    }
    if (dojo.trim(messageText) === "") {
      messageText = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return messageText;
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getInvitedUserid: function _getInvitedUserid_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // TODO Check implications of selecting/entering groups instead of single user
    var choice = null;
    if (this.userChooser) {
      choice = this.userChooser.getChoice();
      if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
        return {
          value: choice.userName,
          type: "user"
        };
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return choice;
  },

  onInvite: function onInvite_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    var myself = this;

    // if nothing can be don in the dialog it is an cancel button

    var invitedUser = this._getInvitedUserid();
    var userid = "[not selected yet]";
    if (invitedUser) {
      userid = invitedUser.value;
    }
    var message = this._getMessageText();
    var tkiid = this._tkiid; // Need for error handler below
    // 
    // Rest call to backend 
    //    
    this._restHandler.inviteUserToCollaborate(this._tkiid, function(response){
      // Good case => not action necessary
      // insert the invited user in task object.
      if (myself.task.collaboration && myself.task.collaboration.currentUsers) {
        if (dojo.indexOf(myself.task.collaboration.currentUsers, userid) === -1) {
          myself.task.collaboration.currentUsers.push(userid);
        }
      }
    }, function(data, ioArgs){
      var messageParameters = {
        1: tkiid
      };
      // the errorCodeMap will overrule the statusCodeMap
      var errorCodeMap = {
        "CWTBG0007E": {
          key: "KEY_TaskInvitedUserDoesNotExist",
          parameters: {
            1: userid
          }
        },
        "CWTBG0561E": {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        "CWTBG0018E": {
          key: "KEY_TaskInviteTaskNotClaimed"
        },
        "CWTBG0048E": {
          key: "KEY_TaskInviteTaskNotFound"
        },
        "CWTBG0540E": {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      // for error codes not found in the errorCodeMapm use the statusCodeMap
      var statusCodeMap = {
        404: {
          key: "KEY_TaskInviteNotAllowedForTask"
        },
        403: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        401: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        400: {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      var errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
      errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
    }, userid, message);

    this.hide();
    com.ibm.bpm.social.util.common.Log.logExit(1);
  }

});


com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping = {
  "TKIID": "TKIID",
  "NAME": "NAME",
  "DISPLAY_NAME": "DISPLAY_NAME",
  "PI_NAME": "PI_NAME",
  "PIID": "PIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping = {
  "TKIID": "tkiid",
  "NAME": "name",
  "DISPLAY_NAME": "displayName",
  "PI_NAME": "processInstanceName",
  "PIID": "piid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};


/**
 * Opens the InvitationDialog
 * 
 * @param task
 * @param user An object with a userName and fullName property
 * @param user {void} nothing
 */
com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog = function(task, user){
  var nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
  var dialog = new com.ibm.bpm.social.widgets.dialogs.InvitationDialog({
    "title": nlsResources["task_dialog_startCollaboration_title"],
    //"style": "min-width: 50%",
    //"style": "min-height: 50%",
    "task": task
  });
  dialog.showDialog(user);
};
