/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog");
dojo.require("com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("com.ibm.bspace.common.util.ui.FullScreenDialog");

dojo.declare("com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString: "<div dojoAttachPoint='content'><div dojoAttachPoint='completionContent'></div></div>",

  _task: null,
  _onCompletedCalled: false,

  /**
   * Creates a popup for working on coaches of a task.
   * 
   * @param taskId The task id of the task to be shown. *
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    // There is no reuse of the dialog, therefore store the task object internally.
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._task = params.task;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  postCreate: function postCreate_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited("postCreate", arguments);
    //
    // Create the TaskCompletionWidget and associate it with the div "completionContent" in the template.
    // Establish all necessary callbacks.
    //        
    this.coachContainer = new com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget({
      dialog: true
    }, this.completionContent);
    this.connect(this.coachContainer, "onLoaded", "_coachLoaded");
    this.connect(this.coachContainer, "onCanceled", "_coachCanceled");
    this.connect(this.coachContainer, "onCompleted", "_taskCompleted");
    this._sizeContent();
    this.coachContainer.resize();
    this.coachContainer.loadCoach(this._task);
    //
    // In case the dialog is just closed, call the _finish method.
    // Which in turn calls onCanceled and makes sure everything is destroyed.
    //
    this.connect(this.popup, 'onHide', "_finish");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _finish: function _finish_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    // Either call onCompleted or onCanceled
    if (!this._onCompletedCalled) {
      this.onCanceled();
    }
    // Basically this is also a dijit._Widget with a meaningful destroy method.
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  destroy: function destroy_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited("destroy", arguments);
    if (this.coachContainer != null) {
      this.coachContainer.destroy();
      this.coachContainer = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Event handling hook for the case that the dialog has been simply closed.
   */
  onCanceled: function onCanceled_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * 
   * Event handling hook for the case that the coaches interaction has been finished.
   */
  onCompleted: function onCompleted_TaskCompletionDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._onCompletedCalled = true;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _coachLoaded: function _coachLoaded_TaskCompletionDialog(tkiid /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, tkiid);
    // Any logic that needs to be triggered upon loading of the coach can be added here.
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _coachCanceled: function _coachCanceled_TaskCompletionDialog(message /* string */, action /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Close the dialog and return
    //
    this.hide();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _taskCompleted: function(nextTkiid /* auto flow task */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, nextTkiid);
    // Switch back to task list.
    // Is there a smarter solution which switches back to where we came from?
    if (nextTkiid !== null && nextTkiid !== undefined) {
      //
      // Update header
      //
      console.log("Update header");
    }
    else {
      // Make sure that hide does not trigger onCanceled.
      this._onCompletedCalled = true;
      this.hide();
      this.onCompleted();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, nextTkiid);
  },

  show: function(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    // The size of the content has already been set.
    this.connect(dojo.isIE && dojo.isIE < 9 ? dojo.body() : dojo.global, 'onresize', "_sizeContent");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _sizeContent: function(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var viewport = dojo.window.getBox();
    // Minimal width of 300px and minimal height of 200px.
    // Take 66% of the viewport.
    var width = Math.max(viewport.w * 2 / 3, 300);
    var height = Math.max(viewport.h * 2 / 3, 200);
    this.coachContainer.domNode.style.width = width + "px";
    this.coachContainer.domNode.style.height = height + "px";
    if (this.coachContainer != null) {
      // The size of the iframe need to be adjusted.
      this.coachContainer.resize();
    }
    /*
    dojo.query(".dijitDialogPaneContent", this.popup.domNode).style("height", "400px").style("width", "500px").style(
      "padding", "0"); //35px header height + 15px padding
    */
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }

});

/**
 * Retrieves the task object and a dialog with the coach of the task.
 * 
 * When the coach(es) is(are) properly completed, the onCompletedCallback is called. When the dialog is closed
 * the onCanceledCallback is called.
 * 
 * @param taskId
 * @param onCompletedCallback Called upon completion of the coaches interaction.
 * @param onCanceledCallback Called if dialog is closed.
 */
com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog.showCoach =
  function(taskId /* string */, onCompletedCallback /* function */, onCanceledCallback /* function */){
    var restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    restHandler.getTask(taskId, function(task, ioArgs){
      var dialog = new com.ibm.bpm.social.widgets.dialogs.TaskCompletionDialog({
        "task": task,
        "title": task.displayName
      });
      if (typeof onCompletedCallback === "function") {
        dialog.connect(dialog, "onCompleted", onCompletedCallback);
      }
      if (typeof onCanceledCallback === "function") {
        dialog.connect(dialog, "onCanceled", onCanceledCallback);
      }
      dialog.show();
    }, function(errorDetails, ioArgs){

    }, false);
  };
