/* 
	 Licensed Materials - Property of IBM
	 5725-C96
	 Copyright IBM Corporation 2012. All Rights Reserved.
	 US Government Users Restricted Rights- Use, duplication or disclosure
	 restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.dialogs.UnfollowDialog");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");

dojo.declare(
	"com.ibm.bpm.social.widgets.dialogs.UnfollowDialog", 
	[com.ibm.bspace.common.util.ui.Dialog], {
	
	templateString: '<div dojoAttachPoint="container"></div>',
	
	preamble: function(params) {
    	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.unfollowDialogMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		if(typeof(params) == 'undefined' || params == null) {
			params = {};
		}
		params.title = this.unfollowDialogMessages["followedProcessList-unfollowDialogTitle"];
		return [params];
	},
	
	constructor: function() {
		this.currInstanceId = null;
		this.parentWidget = null;
	},            	
	
    postCreate: function() {
		this.inherited(arguments);
		
		this.container.className = "bpm-task-dialog-container";
		this.container.title = ""; // Unset the title, else we see the tooltip from the dialog on all controls that don't have a title
		
		this.description = document.createElement("div");
		this.description.innerHTML = this.unfollowDialogMessages["followedProcessList-unfollowDialog"];
		this.description.className = "bpm-task-dialog-description";
		this.container.appendChild(this.description);
		
		var t = document.createElement("table");
		var tb = document.createElement("tbody");
		var tr = document.createElement("tr");
		var td1 = document.createElement("td");
		var td2 = document.createElement("td");
		
		t.appendChild(tb);
		tb.appendChild(tr);
		tr.appendChild(td1);
		tr.appendChild(td2);
		
		this.container.appendChild(t);
		this.dontShowAgainCheckbox = document.createElement("input");
		this.dontShowAgainCheckbox.type = "checkbox";
		td1.appendChild(this.dontShowAgainCheckbox);
		
		this.dontShowAgainMessage = document.createElement("span");
		this.dontShowAgainMessage.innerHTML = this.unfollowDialogMessages["followedProcessList-dontShowAgain"];
		
		td2.appendChild(this.dontShowAgainMessage);
		
		this.unfollowButtonDiv = document.createElement("div");
		this.unfollowButtonDiv.className = "bpm-task-dialog-claimButtonDiv";
		this.container.appendChild(this.unfollowButtonDiv);
		
		this.unfollowButton = document.createElement("button");
		this.unfollowButton.innerHTML = this.unfollowDialogMessages["followedProcessList-unfollowButton"];
		this.unfollowButton.title = this.unfollowDialogMessages["followedProcessList-unfollowHover"];
		this.unfollowButtonDiv.appendChild(this.unfollowButton);
		
		
		this.connect(this.unfollowButton, "onclick", "_onUnfollowConfirmed");
		
		com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this.dontShowAgainCheckbox);
		com.ibm.bspace.common.util.ui.Button.wrapElement(this.unfollowButton);
		
		this.set("content", this.container);
	},
	
	showConfirmationDialog: function showConfirmationDialog_ClaimDialog(instanceId, widget) {
		this.currInstanceId = null;
		this.parentWidget = null;
		if(typeof(widget) != "undefined" && widget != null && typeof(instanceId) != "undefined" && instanceId != null) {
			this.currInstanceId = instanceId;
			this.parentWidget = widget;
			// Reset the combo box to unchecked before displaying the dialog.
			com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.dontShowAgainCheckbox, false);
			this.show();
		}
	},
	
	_onUnfollowConfirmed: function _onClaimConfirmed_TasksListController() {
		// Hide the dialog
		this.hide();
		
		// Set global user preferences if necessary
		if(this.dontShowAgainCheckbox.checked === true) {
			com.ibm.bpm.socialportal.GlobalAssets.setUserPreference('Show Unfollow Messages', "false");
		}
		
		if(this.parentWidget != null && this.currInstanceId != null) {
			this.parentWidget.unfollow(this.currInstanceId);
		}
	},
	
	_unfollow: function _claim_TasksListController() {
		/*com.ibm.bpm.social.util.common.Log.logEntry(2);
		var claimed = false;
		var self = this;
		this._restHandler.claimTask(this.taskId, function(responseObject, ioArgs){
			claimed = true;
		}, function(data, ioArgs){
			if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status === 409 && ioArgs.xhr.responseText) {
				var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
				if (response.errorNumber === "CWTBG0014E") {
					try {
						// Check whether the task has been claimed since last list refresh
						// Check if user is allowed to complete
						// Refresh available actions for this item synchronously					
						var availableActions = [];
						self._restHandler.getActionsForTask(self.taskId, ["ACTION_COMPLETE"], function(responseObject, ioArgs){
							var data = response;
							if (data.tasks.length === 1) {
								availableActions = data.tasks[0].actions;
							}
						}, null, self.systemId);
						if (availableActions) {
							if (dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1) {
								// continue normal processing
								claimed = true;
							}
						}
					}
					catch (e) {
						// This is just logic for checking whether an error condition can be ignored.
						// Therefore ignore any errors either.
					}
				}
			}
			if (!claimed) {
				// handle errors
				var name = self.displayName;
				var statusCodeMap = {
					409: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					401: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					403: {
						key: "KEY_ClaimTask",
						parameters: {
							1: name
						}
					},
					404: {
						key: "KEY_ClaimNotExistingTask",
						parameters: {
							1: name
						}
					}
				};
				var errorCodeMap = {
					"CWTKA0072E": {
						key: "KEY_ClaimSuspendedTask",
						parameters: {
							1: name
						}
					}
				};
				
				self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
			}
		}, this.systemId ? this.systemId : null);
		com.ibm.bpm.social.util.common.Log.logExit(2, claimed);
		if (claimed) {
			this.onClaimed(this.taskId);
		}
		else {
			this.onFailed(this.taskId);
		}*/
	},
	
	destroy: function(){
	    if (this._confirmClaimDialog) {
		    if (this._confirmClaimDialog.dontShowAgainCheckbox) {
				com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._confirmClaimDialog.dontShowAgainCheckbox);
			}
			if (this._confirmClaimDialog.unfollowButton) {
				com.ibm.bspace.common.util.ui.Button.unwrapElement(this._confirmClaimDialog.unfollowButton);
			}
		}	
		this.inherited("destroy", arguments);
	}
	
});
