/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.DialogExpertTile");

dojo.require("dijit._Templated");

dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");
dojo.require("com.ibm.bpm.social.widgets.dialogs.InvitationDialog");
dojo.require("com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip");
dojo.require("com.ibm.bpm.social.widgets.common.BizCard");
dojo.require("com.ibm.bpm.social.util.ui.Avatar");

/**
 * TODO This class needs to be consolidate with the ExpertTile.
 */
dojo.declare("com.ibm.bpm.social.widgets.experts.DialogExpertTile", [dijit._Widget,
                                                                     dijit._Templated], {

  templateString: "",
  templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.experts", "templates/DialogExpertTile.html"),

  /**
   * Input provided in the constructor based on the experts call.
   */
  avgHours: 0,
  completed: 0,
  email: null,
  fullName: null,
  workload: 0,
  userName: null,

  /**
   * Added information for fine tuning available functions also provided using the constructor.
   */
  categoryName: null, // If set, will be shown in the taskDetails section
  collaborate: false, // If true, collaboration action is shown  
  includeTaskDetails: false, // If true, shows the task details i.e. the statistics (avgHours, completed...)
  sidebar: false, // If true, assumes that it is rendered in the sidebar.
  taskId: null, // Added to allow for collaboration... required if includeTaskDetails == true


  livename: null, // Set by sametime integration logic
  imageUrl: null, // ??
  phoneNumber: null, // ??
  title: "", // ?
  refresh: null, // Used for timeout  handle
  
  /**
   * 
   */
  postCreate: function(){
    this.inherited(arguments);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    
    //
    // extends of template
    //
    var div = document.createElement("div");
    dojo.addClass(div, "bpm-social-experts-expert-icon");
    this.expert.appendChild(div);
    var divA11y = document.createElement("div");
    dojo.addClass(divA11y, "bpm-social-experts-expert-icon-a11y");
    this.domNodeA11y = divA11y;
    div.appendChild(divA11y);
       
    //
    // Basic Setup
    //
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Create a unique id to be able to refer to the name field
    // It is dijit._Widget that has a unique id.
    var nameId = this.id + "_name";
    dojo.attr(this.expertname, "id", nameId);
    this.domNode.title = "";
    //
    // Retrieve preferences of current user -> this.userData
    // Retrieve user avatar and add it the div[#expert-avatar]
    //
    this._getUserData();
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    if (!this.fullName && user && user.fullName) {
      this.fullName = user.fullName;
    }
    this.expertname.innerHTML = this.fullName;

    if (window.sametime && this.fullName) {
      // the text-overflow:ellipsis style does not function properly
      // when there are span elements within the div, so clip the
      // text instead if we are using sametime
      this.expertname.style.textOverflow = 'clip';
      // Overwrite name and add the live name
      this.expertname.innerHTML = "";
      this.livename = window.sametime.LiveName({
        'userId': this.userName,
        'disableClicks': true,
        'disableHoverBizCard': true
      }, dojo.create("span", null, this.expertname));
      this.livename.activate();

      this._updateStatus();
      this.refresh = window.setInterval(dojo.hitch(this, this._updateStatus), 2000);
    }
    
    this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
    this.avatar.setUser(user, "bpm-social-default-avatar-medium", false);
    
    /** Business Card integration * */
    /** Default * */
    var node = this.expertname;
    if (this.livename != null) {
      var nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
      if (nodes.length == 1) {
        node = nodes[0];
        var img = dojo.create("img", {
          "src": dojo.moduleUrl("dojo", "resources/blank.gif"),
          "class": "connectionsOverlayImage"
        }, dojo.query(".stproxy_general", this.domNode)[0], "first");
        com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
        var divA11y = document.createElement("div");
        divA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
        divA11y.addClass("bpm-social-experts-expert-name-a11y");
        this.onlineTrackingDomNodeA11y = divA11y;
        img.appendChild(divA11y);
      }
    }

    /*
    // Do not show the BizCard as clicking on the tile selects the user and 
    // should not show the BizCard
    com.ibm.bpm.social.widgets.common.BizCard.wrapElement(node, user);
    com.ibm.bpm.social.widgets.common.BizCard.wrapImage(dojo.query("img", this.img)[0], user);
    */
    /** End Business Card integration * */


    //
    // Adjusting functionality based on configuration of instance.
    // Processing according to display in rtl direction
    //
    if (this.categoryName) {
      this.taskDetails.appendChild(document.createTextNode(this.categoryName));
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    else if (this.includeTaskDetails) {
      // If includeTaskDetails add content to dojoAttachPoint taskDetails
      var data;
      var firstName = this.fullName.split(" ")[0];
      this._updateUserData();
      if (this.completed == 0) {
        this.taskDetails.innerHTML = this.messages.expertNoTaskData.replace("${user}", firstName);
      }
      else {
        data = {
          'userName': firstName,
          'average': this.avgHours,
          'completed': this.completed
        };
        //defect 149810 Fix Expert tooltip wording
        this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip_short, data, /\$\{([^\}]+)\}/g);
      }
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    // 
    // Add actions for mention | collaborate | chat
    // Remove mention if not in sidebar
    // 
    this.mention.innerHTML = "@" + this.messages.expertsMention;
    this.gethelp.innerHTML = this.messages.expertsCollaborate;
    this.startchat.innerHTML = this.messages.expertsChat;
    // Refering to the name field
    dijit.setWaiState(this.mention, "describedby", nameId);
    dijit.setWaiState(this.gethelp, "describedby", nameId);
    dijit.setWaiState(this.startchat, "describedby", nameId);
    this.actionSeparator1.innerHTML = this.messages.expertsSeparator;
    this.actionSeparator2.innerHTML = this.messages.expertsSeparator;
    // Downgrade actions if not in sidebar
    if (!this.sidebar) {
      this.mention.style.display = 'none';
      this.actionSeparator1.style.display = 'none';
    }
    //
    // The collaborate constructor parameter allows to configure whether collaboration action is supported.
    //
    if (!this.collaborate) {
      this.actionSeparator2.style.display = 'none';
      this.helpDiv.style.display = 'none';
    }
    //
    // Remove chat action
    //
    if (!(window.sameTime && this.fullName)) {
      dojo.style(this.chatDiv, "display", "none");
    }
  },

  _handleMention: function(e){
    var taskCompletionView = this._getTaskCompletionView();
    if (taskCompletionView) {
      // switch to the stream tab
      taskCompletionView.sidebar.selectTab(1);
      // set focus to input box
      taskCompletionView.stream._handlePostInitialFocus();
      // tag user
      taskCompletionView.stream.tagUser(this.userName, this.fullName);
    }
    dojo.stopEvent(e);
  },

  _handleGetHelp: function(e){
    var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
      "taskId": this.taskId
    });
    com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(taskData, {
      userName: this.userName,
      fullName: this.fullName
    });
    dojo.stopEvent(e);
  },

  _handleChat: function(e){
    if (this.livename && this.presenceStatus == "online") {
      this.livename.startChat();
    }
    dojo.stopEvent(e);
  },

  _showTooltip: function(){
    return;
  },

  _hideTooltip: function(){
    if (this.myTooltipDialog) {
      this.myTooltipDialog = null;
    }

  },

  _updateUserData: function(){
    var self = this;
    this.socialRestHandler.getExpertUserData(this.taskId, this.userName, function(data, xhrArgs){
      self.avgHours = data.avgHours;
      self.completed = data.completed;
    }, "", false);
  },

  _getUserData: function(){
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    /*if(user && user.userPreferences && user.userPreferences.Image){
    	this.img.innerHTML = "<img alt='"+this.fullName+"' src='"+user.userPreferences.Image+"' />";
    }*/
    this.userData = user.userPreferences;
    if (!this.userData) {
      this.userData = new Object();
    }
  },

  _updateStatus: function(){
    dojo.removeClass(this.domNode, "online");
    dojo.removeClass(this.domNode, "offline");

    if (this.livename) {
      switch (this.livename.currentTextStyle) {
        case "stproxy_liveNameAvailableText" :
        case "stproxy_liveNameAwayText" :
        case "stproxy_liveNameInMeetingText" :
          dojo.addClass(this.domNode, "online");
          this.presenceStatus = "online";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-online"];
          }
          break;
        case "stproxy_liveNameOfflineText" :
        case "stproxy_liveNameDndText" :
        default:
          dojo.addClass(this.domNode, "offline");
          this.presenceStatus = "offline";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
          }
          break;
      }
    }
    else {
      dojo.style(this.chatDiv, "display", "none");
    }

    /** Business Card integration * */
    if (!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard() && this.livename != null) {
      /** Account for Sametime DOM structure, which can be delayed in being created due to network latency * */
      var nodes = dojo.query(".stproxy_liveNamePadding .hasHover", this.expertname);
      if (nodes.length == 0) {
        nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
        if (nodes.length == 1) {
          var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
          if (!this.fullName) {
            this.fullName = user.fullName;
          }
          com.ibm.bpm.social.widgets.common.BizCard.wrapElement(nodes[0], user);
        }
      }
    }
    /** End Business Card integration * */

  },


  _getTaskCompletionView: function(){
    var node = this.domNode;
    while (node && !dojo.hasClass(node, 'workViewCompletion')) {
      node = node.parentNode;
    }
    if (node == null) {
      node = dojo.query('.workViewCompletion')[0];
    }
    var taskCompletionView = dijit.byId(node.id);
    return taskCompletionView;
  },


  _doMouseEnter: function(e){
    dojo.addClass(this.expert, 'hover');
  },

  _doMouseLeave: function(e){
    dojo.removeClass(this.expert, 'hover');
  },

  _onFocus: function(){
    dojo.addClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onBlur: function(){
    dojo.removeClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onKeyDown: function(e){
    if (e.keyCode === dojo.keys.RIGHT_ARROW && this.livename) {
      this.startchat.focus();
    } 
  },
 
  destroy: function(){
    if (this.refresh) {
      window.clearInterval(this.refresh);
    }
    
    this._hideTooltip();

    if (this.avatar) {
        this.avatar.destroy();
        this.avatar = null;
    }

    this.inherited(arguments);
  }

});
