/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.Experts");

dojo.require("dijit._Templated");

dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");

dojo.require("dijit.Dialog");

dojo.require("com.ibm.bpm.social.widgets.experts.ExpertTile");

dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");

dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");

dojo.declare(
	"com.ibm.bpm.social.widgets.experts.Experts",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.experts", "templates/Experts.html"),
	popup: null,
	userid: null,
	taskid: null,
	AUTO_EXPERTS: "auto",
	MANUAL_EXPERTS: "manual",
	expertTiles: null,
	sidebar: false,
	expertsUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/experts",
	_taskClaimedHandle: null,
	_refreshInProgress: false,
	_refreshNeeded: false,
	_currentUser: null,
		
    postCreate: function() {
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		dijit.setWaiState(this.expertsWidgetContainer, "live", "polite");
		dijit.setWaiState(this.expertsWidgetContainer, "atomic", "true");
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.taskMessages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
		
		this.autoSubscript.innerHTML = this.messages.expertsAutoHeader;
		this.manualSubscript.innerHTML = this.messages.expertsManualHeader;
		this.autoMore.innerHTML = '+ '+this.messages.showAll;
		this.manualMore.innerHTML = '+ '+this.messages.showAll;
		//this.helpLink.innerHTML = this.messages.expertsHelpText;
		//this.helpLink.href = "/BPMHelp/index.jsp?topic=/com.ibm.wbpm.wle.widget.doc/portalhelp/topic/experts.html";
		
		if(this.sidebar){
			dojo.addClass(this.domNode, 'sidebar');
		}
		
		var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if (userProfileObj) {
			this._currentUser = userProfileObj.userName;
		} 

		this.expertTiles = new Array();
		if(this.taskid!=null){
			this.refresh({taskid:this.taskid});
		}
	},
	
	showExpertsList: function() {
		this.refresh({taskid: this.taskid});
	},
	refresh: function(obj){
		if(! (obj.taskid) ){
			return;
		}
		this.taskid = obj.taskid;
		if(this.expertsWidgetContainer.style.display == "none") {
			return;
		}
		if (this._refreshInProgress) {
			this._refreshNeeded = true;
		} else {
			this._refreshInProgress = true;
			this._clearExperts();
			this._showLoading();
			this._getExperts();
			this._subscribeToTaskClaimed();
		}
	},
	
	_clearExperts: function(){
		
		dojo.forEach(this.expertTiles, function(ee){
			if(ee.destroy){
				ee.destroy();
			}
		});
		// Do not destroy the tiles several times.
		this.expertTiles = new Array();
		
		dojo.empty(this.autoExperts);
		
		dojo.empty(this.manualExperts);
		
	},
	
	_getExperts: function() {
		
		this.socialRestHandler.getExpertsForTask(this.taskid, 
			dojo.hitch(this, "_getExpertsResponse"), dojo.hitch(this, "_onXhrErrors"), true);
	},
	
	_isCollaborationSupported: function() {
		var col = false;
		this.taskRestHandler.getActionsForTask(this.taskid, null, function(data) {
			var actions = data.tasks[0].actions;
			for (var i=0; i<actions.length; i++) {
				if (actions[i] == 'ACTION_INVITE') {
					col = true;
				}
			}
		}, function() {}, false, null, null);
		return col;
	},
	
	_removeCurrentUser: function(group) {
		if(group && this._currentUser) {
			for (var i=0; i<group.length; i++) {
				if (group[i].userName == this._currentUser) {
					group.splice(i,1);
					i--;
				}
			}
		}
	},
	
	_getExpertsResponse: function(data,xhrArgs) {
		this._hideLoading();
		
		var ee = null;
		var cnt = 0;
		var tmpUserCache = {}; //Hashmap of automatic users to easily track task data if needed for manual users

		var col = this._isCollaborationSupported();
		
		this.autoSubscript.innerHTML = this.messages.expertsAutoHeader;
		dojo.removeClass(this.autoExperts,'bpm-social-experts-more');
		this.autoMore.style.display = 'none';
		
		var systemIdentified = data["systemIdentifiedUsers"];
		this._removeCurrentUser(systemIdentified);
		if(!systemIdentified || !systemIdentified.length){
			this.autoSubscript.innerHTML = this.messages.expertsAutoHeader + " (0)";
			dojo.addClass(this.autoExperts, 'no-experts');
			this.autoExperts.innerHTML = this.messages.noExperts_AutoHeader;
		} else {
			cnt = 0;
			dojo.forEach(systemIdentified, dojo.hitch(this, function(item){
				if(cnt < 6) { //max of 6 system-identified experts (fix for RTC defect 100772)
					item['taskId'] = this.taskid;
					item['includeTaskDetails'] = true;
					item['sidebar'] = this.sidebar;
					item['collaborate'] = col;
					tmpUserCache[item['userName']] = item;
					ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(item);
					this.autoExperts.appendChild(ee.domNode);
					this.expertTiles.push(ee);
					cnt++;
					if (cnt >5) {
						dojo.addClass(ee.domNode,'bpm-social-experts-all');
						dojo.addClass(this.autoExperts,'bpm-social-experts-more');
						this.autoMore.style.display = 'block';
					}
				}
			}));
			this.autoSubscript.innerHTML = this.messages.expertsAutoHeader + " ("+cnt+")"
		}
		
		this.manualSubscript.innerHTML = this.messages.expertsManualHeader;
		dojo.removeClass(this.manualExperts,'bpm-social-experts-more');
		this.manualMore.style.display = 'none';

		var expertParticipantGroup = data["expertParticipantGroupUsers"];
		this._removeCurrentUser(expertParticipantGroup);
		if(!expertParticipantGroup || !expertParticipantGroup.length){
			this.manualSubscript.innerHTML = this.messages.expertsManualHeader+ " (0)";
			dojo.addClass(this.manualExperts, 'no-experts');
			this.manualExperts.innerHTML = this.messages.noExperts_ManualHeader;
		} else {
			this.manualSubscript.innerHTML = this.messages.expertsManualHeader + " ("+expertParticipantGroup.length+")";
			cnt = 0;
			dojo.forEach(expertParticipantGroup, dojo.hitch(this, function(item){
				var tmp;
				if(tmpUserCache[item['userName']]){
					tmp = tmpUserCache[item['userName']];
				} else {
					item['taskId'] = this.taskid;
					item['includeTaskDetails'] = true;
					item['sidebar'] = this.sidebar;
					item['collaborate'] = col;
					tmp = item;
				}
				ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(tmp);
				this.manualExperts.appendChild(ee.domNode);
				this.expertTiles.push(ee);
				cnt++;
				if (cnt >5) {
					dojo.addClass(ee.domNode,'bpm-social-experts-all');
					dojo.addClass(this.manualExperts,'bpm-social-experts-more');
					this.manualMore.style.display = 'block';
				}
			}));
		}
				
		this._refreshInProgress = false;
		if (this._refreshNeeded) {
			this._refreshNeeded = false;
			this.refresh({taskid:this.taskid});
		}
	},
	
	_handleAutoMore: function() {
		dojo.removeClass(this.autoExperts,'bpm-social-experts-more');
		this.autoMore.style.display = 'none';
	},

	_handleManualMore: function() {
		dojo.removeClass(this.manualExperts,'bpm-social-experts-more');
		this.manualMore.style.display = 'none';
	},

	_onXhrErrors: function(data,xhrArgs) {
		// TODO: how do we show errors?
		this._refreshInProgress = false;
		if (this._refreshNeeded) {
			this._refreshNeeded = false;
			this.refresh({taskid:this.taskid});
		}
	},

	_subscribeToTaskClaimed: function() {
		if (this._taskClaimedHandle) {
			dojo.unsubscribe(this._taskClaimedHandle);
			this._taskClaimedHandle = null;
		}
		this._taskClaimedHandle = dojo.subscribe(com.ibm.bpm.social.portal.Topics.TASK_CLAIMED+"."+this.taskid, dojo.hitch(this, '_taskClaimed'));
	},
	
	_taskClaimed: function() {
		this.refresh({taskid:this.taskid});
	},
	
	_showLoading: function() {
		this.autoExperts.innerHTML = this.taskMessages["KEY_Loading"];
		this.manualExperts.innerHTML = this.taskMessages["KEY_Loading"];
	},

	_hideLoading: function() {
		this.autoExperts.innerHTML = "";
		this.manualExperts.innerHTML = "";
	},
	destroy: function() {
	    this._clearExperts();
		if (this._taskClaimedHandle) {
			dojo.unsubscribe(this._taskClaimedHandle);
			this._taskClaimedHandle = null;
		}
		this.inherited(arguments);
	}
	
});