/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.Stream");

dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");
dojo.require("com.ibm.bpm.social.widgets.stream.StreamEvent");
dojo.require("com.ibm.bpm.social.widgets.stream.StreamInputBox");
dojo.require("com.ibm.bpm.social.util.common.ProcessRestHandler");
dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.declare(
	"com.ibm.bpm.social.widgets.stream.Stream",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.stream", "templates/Stream.html"),
	showSelection: 'all',	
	events: null,
	fetchsize: 50,
	more: null,
	moreConnect: null,
	userid: null,
	taskid: null,
	instanceid: null,
	lastStreamEvent: null,
	sub: null,
	timer: null,
	sidebar: false,
	//streamUrl: "/ProcessPortal/api/rest/task/{tid}/stream?preventDefault={preventCache}", //IBPM
	streamUrl: window.endpointURLObject.bpmRestJS +"/v1/social/instance/{iid}/stream?preventDefault={preventCache}", //IBPM
	commentUrl: window.endpointURLObject.bpmRestJS +"/v1/process/{iid}",
	streamInputBox: undefined,
    _userProfileChangeSubscribeHandle: null,
    _fixStreamHeightTimer: null,
	
	constructor: function() {
		this.progressCount = 0;
	},
			
    postCreate: function() {

    	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		this.processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    	this.postInitial.value = this.messages.post_ellipse;
    	this.postButton.innerHTML = this.messages.post;
    	this.attachFileSpan.innerHTML = this.messages.file;
    	this.attachFileIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    	this.attachFileIcon.alt = "";
    	this.attachLinkSpan.innerHTML = this.messages.link;
    	this.attachLinkIcon.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    	this.attachLinkIcon.alt = "";
		this.attachFileLabel.innerHTML = this.messages.file_attach;
		this.attachLinkLabel.innerHTML = this.messages.link_attach;
		//this.cancelFileImg.alt = this.messages.cancelFile;
		//this.cancelLinkImg.alt = this.messages.cancelLink;
		this.browseButton.innerHTML = this.messages.browse;
		//this.cancelFileImg.src = dojo.moduleUrl('dojo','resources/blank.gif'); 
		//this.cancelLinkImg.src = dojo.moduleUrl('dojo','resources/blank.gif');
		this.streamTitle.innerHTML = this.messages.stream;
		
		dijit.setWaiState(this.postInitial, "label", this.messages.post);
		dijit.setWaiState(this.attachFileText, "label", this.messages.file_attach);
		dijit.setWaiState(this.attachFileInput, "label", this.messages.file_attach);
		dijit.setWaiState(this.attachLinkInput, "label", this.messages.link_attach);
		dijit.setWaiState(this.cancelFileButton, "label", this.messages.cancelFile);
		dijit.setWaiState(this.cancelLinkButton, "label", this.messages.cancelLink);
		
		this.events = new Array();

		if(this.sidebar){
			dojo.addClass(this.domNode, 'sidebar');
			//dojo.style(this.streamTitle, 'display', 'none');
		}
		
    	this.streamInputBox = new com.ibm.bpm.social.widgets.stream.StreamInputBox({},this.postInput);
    	dijit.setWaiState(this.streamInputBox.postInput, "label", this.messages.post);
    	
		if(this.instanceid!=null){
			this._clearEvents();
			this._getEvents();
			this.sub = dojo.subscribe("/com/ibm/bpm/social/updateStream/"+this.instanceid,dojo.hitch(this,'_getNewEvents'));
		}
	
		// subscribe to user profile changes
		if(this._userProfileChangeSubscribeHandle == null){
        	dojo.subscribe(com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED, dojo
          	.hitch(this, this._handleUserProfileChange));
        }

		// do not show the "Attachment" option for iPad
		if (dojo.isSafari && navigator.userAgent.indexOf('iPad') != -1) {
			this.attachFile.style.display = 'none';
		}
		
		// use timer to adjust the stream height to fill available space
		this._fixStreamHeightTimer = setInterval(dojo.hitch(this, '_fixStreamHeight'), 250);
		
		//ensure that the file attachement form is always ltr aligned.
		this.attachFileText.dir = "ltr";
	},
	
	_fixStreamHeight: function() {
		if (this.eventDiv.parentNode.offsetHeight) {
			var h = this.eventDiv.parentNode.offsetHeight - this.header.offsetHeight;
			if (this.streamTitle.offsetHeight) {
				h = h - this.header.offsetTop;
			}
			if (h < 0) h = 0;
			this.eventDiv.style.height = h + 'px';
		}
	},
	
	_createHiddenIFrame: function() {
		var name = 'frame'+(new Date().getTime());
		var iframe = document.createElement('iframe');
		iframe.name=name;
		iframe.style.display="none";
		iframe.title=name;
		iframe.id=name;
		this.hiddenIFramesDiv.appendChild(iframe);
		var fn = dojo.hitch(this,function() {
			if (iframe.contentWindow.location.href != 'about:blank') {
				this._getNewEvents();
				this._hideProgressIndicator();
				if (iframe.attachEvent) {
					iframe.detachEvent('onload',fn);
				} else {
					iframe.removeEventListener('onload',fn,false);
				}
				if (iframe.contentDocument &&
					iframe.contentDocument.documentElement &&
					(iframe.contentDocument.documentElement.textContent || 
					iframe.contentDocument.documentElement.innerText)) {
					var txt = iframe.contentDocument.documentElement.textContent;
					if (!txt) {
						txt = iframe.contentDocument.documentElement.innerText;
					}
					var result = dojo.fromJson(txt);
					if (result && result.status) {
						if (result.status == "error") {
							// Display error!
							dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
							com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(result.Data.errorMessage);
						}
					}
				}
				this.hiddenIFramesDiv.removeChild(iframe);
				delete iframe;
			}
		});
		if (iframe.attachEvent) {
			iframe.attachEvent('onload',fn);
		} else {
			iframe.addEventListener('load',fn,false);
		}
		return iframe;
	},
	
	_handleUserProfileChange: function(obj){
		var currUser = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile().userName;
		var avatarURL = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(currUser);
		var se = null;
		for (var i=0; i<this.events.length; i++) {
			var e = this.events[i];
			if (e.streamEventObject && (e.streamEventObject != se)) {
				se = e.streamEventObject;
				se.updateAvatarUrlForUserid(avatarURL,currUser);
			}
		}
	},

	refresh: function(obj){
		if(! (obj.instanceid && obj.taskid) ){
			return;
		}
		
		dojo.unsubscribe(this.sub);
		
		this.instanceid = obj.instanceid;
		
		this.taskid = obj.taskid;
		
		this._clearEvents();
		
		this._getEvents();
		
		this.streamInputBox.clearInput(false);
		this._toggleHeaderIfCommentsProhibited();
		this._cancelFile();
		this._cancelLink();
		
		this.sub = dojo.subscribe("/com/ibm/bpm/social/updateStream/"+this.instanceid,dojo.hitch(this,'_getNewEvents'));
	},
	
	_toggleHeaderIfCommentsProhibited: function() {
		if(this.instanceid != null) {
			this._restHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
			this._restHandler.getActionsForProcess(this.instanceid, dojo.hitch(this, function(result) {
				if(result && result.standardActions && result.standardActions.indexOf("ACTION_ADD_COMMENT") >= 0) {
					this._showPostCommentHeader();
				}
				else if(result && result.standardActions) {
					this._hidePostCommentHeader();
				}
			}),function(){}, true, null, null);
		}
	},
	
	_showPostCommentHeader: function() {
		dojo.style(this.header, "display", "");
	},
	
	_hidePostCommentHeader: function() {
		dojo.style(this.header, "display", "none");
	},
	
	_showProgressIndicator: function() {
		this.progressCount++;
		this.progressIndicator.style.display = 'block';	
	},
	
	_hideProgressIndicator: function() {
		if (this.progressCount > 0) {
			this.progressCount--;
			if (this.progressCount == 0) {
				this.progressIndicator.style.display = 'none';
			}
		}
	},
	
	_clearEvents: function() {
		//destroy the events
		for(var i=0; i<this.events.length; i++) {
			if(this.events[i] && this.events[i].streamEventObject) {
				this.events[i].streamEventObject.destroy();
			}
		}
		//destroy the array
		delete this.events;
		this.events = new Array();
		if (this.more) {
			dojo.disconnect(this.moreConnect);
			this.more = null;
		}
		while (this.eventDiv.firstChild) {
			var c = this.eventDiv.firstChild;
			var se = dijit.byId(c.id);
			if (se && se.destroy) {
				se.destroy();
			} else {
				this.eventDiv.removeChild(c);
				delete c;
			}
		}
		this.lastStreamEvent = null;
	},
	
	_fixActorAvatar: function(actor) {
		if (actor && actor.objectType) {
			if ((actor.objectType.toLowerCase() == 'person') && actor.image) {
				actor.image.url = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(actor.id);
			}
		}
	},
	
	_fixAvatars: function(data) {
		for (var i=0; i<data.items.length; i++) {
			var event = data.items[i];
			this._fixActorAvatar(event.actor);
			if (event.replies && event.replies.items) {
				for (var j=0; j<event.replies.items.length; j++) {
					this._fixActorAvatar(event.replies.items[j].author);
				}
			}
		}
	},
	
	_getNewEvents: function() {
		if (this.timer) {
			window.clearTimeout(this.timer);
			this.timer = null;
			this._showProgressIndicator();
			var url = this.getProxyUrl() + this.streamUrl.replace("{iid}", this.instanceid).replace("{preventCache}",(new Date().getTime()) );
			if (this.events.length > 0) {
				url = url + '&since=' + this.events[this.events.length-1].published;
			}
			// read data
			dojo.xhrGet({
				url: url,
				handleAs: "json",
				headers: {
					"Content-Type": "application/json", 
					"Accept":"application/json"
				},
				load: dojo.hitch(this,'_loadNewEvents'), 
				error: dojo.hitch(this,'_errorEvents')
			});
		}
	},
	
	_getEvents: function(last) {
		if (this.taskid) {
			// read data
			this._showProgressIndicator();
			
			this.socialRestHandler.getStreamEvents(this.instanceid, 1, this.fetchsize, 
				dojo.hitch(this, '_loadEvents'), dojo.hitch(this, '_errorEvents'), true);
		}
	},
	
	_loadEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			for (var i=0; i<data.items.length; i++) {
				events[i] = data.items[i];
			}
		}
		var len = events.length;
		this._addEvents(events);
		if (len == this.fetchsize) {
			this.more = document.createElement('div');
			this.more.className = 'bpm-social-stream-event-data-div bpm-social-stream-more';
			this.eventDiv.appendChild(this.more);
			var a = document.createElement('a');
			a.href = 'javascript:void(0)';
			this.more.appendChild(a);
			a.innerHTML = this.messages.more;
			this.moreConnect = dojo.connect(a,'onclick',this,'_getMoreEvents');
		}
	},
	
	_loadMoreEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			for (var i=0; i<data.items.length; i++) {
				events[i] = data.items[i];
			}
		}
		var len = events.length;
		if (len) {
			var testDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[0].published);
			if (testDate) {
				var testTime = testDate.getTime();
				var d = com.ibm.bpm.social.util.common.Util.getDateFromString(events[0].published);
				while (d && (d.getTime() >= testTime)) {
					events.splice(0,1);
					if (events.length > 0) {
						d = com.ibm.bpm.social.util.common.Util.getDateFromString(events[0].published);
					} else {
						d = null;
					}
				}
			} 	
		}
		this._addEvents(events,true);
		if (len == this.fetchsize) {
			this.more = document.createElement('div');
			this.more.className = 'bpm-social-stream-event-data-div bpm-social-stream-more';
			this.eventDiv.appendChild(this.more);
			var a = document.createElement('a');
			a.href = 'javascript:void(0)';
			this.more.appendChild(a);
			a.innerHTML = this.messages.more;
			this.moreConnect = dojo.connect(a,'onclick',this,'_getMoreEvents');
		}
	},
	
	_checkForEventUpdate: function(event) {
		var found = false;
		for (var i=0; (i<this.events.length) && !found; i++) {
			// use timestamp to find match (may need better logic)
			if (this.events[i].published == event.published) {
				var se = this.events[i].streamEventObject;
				if (se) {
					se.updateEvent(event);
				}
			}
		}
	},
	
	_loadNewEvents: function(data,xhrArgs) {
		var events = new Array();
		if (data && data.items) {
			this._fixAvatars(data);
			var lastDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[this.events.length-1].published);
			for (var i=0; i<data.items.length; i++) {
				if (data.items[i].published) {
					var testDate = com.ibm.bpm.social.util.common.Util.getDateFromString(data.items[i].published);
					if (lastDate && testDate && (lastDate.getTime() < testDate.getTime())) {				
						events[events.length] = data.items[i];
					} else {
						this._checkForEventUpdate(data.items[i]);
					}
				}
			}
		}
		this._addEvents(events);
	},
	
	_errorEvents: function(data,xhrArgs) {
		// TODO: how do we show errors?
		this._hideProgressIndicator();
	},
	
	_isEventCollapsable: function(e, se) {
		var collapse = false;
		if (se) {
			collapse = se.isEventCollapsable(e);
		}
		return collapse;
	},
	
	_addEvents: function(events,more) {
		var size = events.length;
		var se = this.lastStreamEvent;
		var moreindex = 0;
		if (more) {
			se = null;
		}
		while (events.length > 0) {
			var e = events.pop();
			if (this._isEventCollapsable(e,se)) {
				se.addEvent(e);
			} else {
				var node = this.eventDiv.firstChild;
				if (more) {
					node = null;
					if (se) {
						node = se.domNode;
					}
				} 
				se = new com.ibm.bpm.social.widgets.stream.StreamEvent({event:e, taskid:this.taskid, instanceid:this.instanceid});
				this.eventDiv.insertBefore(se.domNode,node);
			}
			
			if(e.actor && e.actor.id && e.actor.displayName && e.actor.image && e.actor.image.url){
				com.ibm.bpm.social.util.common.Util.primeUserCache(e.actor.id, e.actor.displayName, e.actor.image.url);
			}
			
			e.streamEventObject = se;
			if (more) {
				this.events.splice(moreindex,0,e);
				moreindex++;
			} else {
				this.events.push(e);
			}
		}
		if (more) {
			// if the event immediately after the last inserted event
			// can be collapsed, we need to remove the old markup and
			// collpase the events together
			if (this._isEventCollapsable(this.events[moreindex],se)) {
				var se2 = this.events[moreindex].streamEventObject;
				var tmp = se2.events;
				se2.destroy();
				while (tmp.length > 0) {
					var e = tmp.pop();
					se.addEvent(e);
					e.streamEventObject = se;
				}
			}
		}
		this._hideProgressIndicator();
		if (!more) {
			this.lastStreamEvent = se;
			this.timer = window.setTimeout(dojo.hitch(this,'_getNewEvents'),300000);
		}
	},
	
	_getMoreEvents: function() {
		dojo.disconnect(this.moreConnect);
		this.eventDiv.removeChild(this.more);
		delete this.more;
		this.more = null;
		var page = Math.floor(this.events.length/this.fetchsize) + 1;
		this._showProgressIndicator();
		
		this.socialRestHandler.getStreamEvents(this.instanceid, page, this.fetchsize,
			dojo.hitch(this, '_loadMoreEvents'), dojo.hitch(this, '_errorEvents'));
	},
	
	
	_fixAttachFileSize: function() {
		this.attachFileTable.style.width = this.header.offsetWidth + 'px';
		this.attachFileText.size = 1;
		this.attachFileInput.size = 30;
		var size = 30;
		if (dojo.isChrome) {
			var inputSize = this.header.offsetWidth;
			this.attachFileInput.style.width = inputSize+'px'; 
			while ((inputSize > 100) && (this.attachFileTable.offsetWidth > this.header.offsetWidth)) {
				inputSize = inputSize - 5;
				this.attachFileInput.style.width = inputSize+'px'; 
			}
		} else {
			while ((size > 5) && (this.attachFileTable.offsetWidth > this.header.offsetWidth)) {
				size--;
				this.attachFileInput.size = size; 
			}
		}
		this.attachFileText.size = size;
		while ((size > 5) && ((this.attachFileText.offsetWidth + this.attachFileButton.offsetWidth + 5) > this.attachFileInput.offsetWidth)) {
			size--;
			this.attachFileText.size = size; 
		}
		if (dojo.hasClass(document.documentElement,'dijitRtl')) {
			dojo.attr(this.attachFileDiv, 'dir', 'ltr');
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.browseButton);
		} else {
			if (dojo.isIE) {
				this.attachFileButton.style.left = this.attachFileText.offsetWidth + 'px';
			} else {
				this.attachFileButton.style.right = '-' +  (this.attachFileButton.offsetWidth+5) + 'px';
			}
		}
	},
	
	_fixAttachLinkSize: function() {
		this.attachLinkTable.style.width = this.header.offsetWidth + 'px';
		this.attachLinkInput.size = 40;
		var size = 40;
		while (this.attachLinkTable.offsetWidth > this.header.offsetWidth) {
			size--;
			this.attachLinkInput.size = size; 
		}
	},
	
	_attachFile: function() {
		this.attachFileDiv.style.display = 'block';
		if (!dojo.isSafari)	this.attachFileInput.style.width = "auto";
		this._fixAttachFileSize();
		this.attachFileText.style.width = "auto";
		this.attachFile.disabled = true;
		dojo.addClass(this.attachFile,'bpm-social-link-disabled');
		if (this.attachLinkDiv.style.display == 'block') {
			this._fixAttachLinkSize();
		}
		window.setTimeout(dojo.hitch(this, function(){
			this.attachFileInput.focus();
			this._fixAttachFileSize();
		}),100);
	},
	
	
	_attachLink: function() {
		this.attachLinkDiv.style.display = 'block';
		this.attachLinkInput.style.width = "auto";
		this._fixAttachLinkSize();
		this.attachLink.disabled = true;
		dojo.addClass(this.attachLink,'bpm-social-link-disabled');
		if (this.attachFileDiv.style.display == 'block') {
			this._fixAttachFileSize();
		}
		window.setTimeout(dojo.hitch(this, function(){this.attachLinkInput.focus();}),100);
	},
	
	_handlePost: function() {
		var txt = this.streamInputBox.getPostInput();
		if (txt) {
			this._showProgressIndicator();
			
			this.processRestHandler.postInstanceStreamComment(this.instanceid, txt,
				dojo.hitch(this, function(data, xhrArgs) {
					if (data.status == '200') {
						this._getNewEvents();
						this.streamInputBox.clearInput(true);
					} else {
						//TODO: error handling
					}
					this._hideProgressIndicator();
				}), dojo.hitch(this, function(data, xhrArgs) {
					this._hideProgressIndicator();
				}), false);  /*false means XHR is synchronous; this fixes problem where stream text and attached file or URL might end up in different stream posts.*/
		}
		if (this.attachFileInput.value && this.attachFileText.value) {
			this._showProgressIndicator();
			name = this.attachFileInput.value;
			name = name.substring(name.lastIndexOf('/')+1);
			name = name.substring(name.lastIndexOf('\\')+1);
			this.attachFileName.value = name;
			this.attachFileForm.action = window.endpointURLObject.bpmRestJS +'/v1/process/'+this.instanceid+'?action=addDocument&docType=file&parts=all&accept=application/json&override-content-type=text/plain&name='+encodeURIComponent(name);
			this.attachFileInstanceId.value = this.instanceid;
			this.attachFileForm.target = this._createHiddenIFrame().name;
			this.attachFileForm.submit();
			this.attachFileText.value = '';
			if (this.attachFileInput.form.reset) {
				try {
					this.attachFileInput.form.reset();
				} catch(e) {
				}
			}
		}
		if (this.attachLinkInput.value) {
			this._showProgressIndicator();
			name = this.attachLinkInput.value;
			if (name.indexOf('://') == -1) {
				this.attachLinkInput.value = 'http://'+name;
				name = this.attachLinkInput.value;
			}
			if (name.length > 50) {
				name = name.substring(0,47)+"...";
			}
			this.attachLinkName.value = name;
			this.attachLinkForm.action = window.endpointURLObject.bpmRestJS +'/v1/process/'+this.instanceid+'?action=addDocument&docType=url&parts=all&accept=application/json&override-content-type=text/plain&name='+encodeURIComponent(name)+'&docUrl='+encodeURIComponent(this.attachLinkInput.value);
			this.attachLinkInstanceId.value = this.instanceid;
			this.attachLinkForm.target = this._createHiddenIFrame().name;
			this.attachLinkForm.submit();
			this.attachLinkInput.value = '';
		}
		this.attachFileDiv.style.display = 'none';

		this.attachFile.disabled = false;
		dojo.removeClass(this.attachFile,'bpm-social-link-disabled');
		this.attachLinkDiv.style.display = 'none';
		this.attachLink.disabled = false;
		dojo.removeClass(this.attachLink,'bpm-social-link-disabled');
	},
	
	_cancelFile: function(e) {
		this.attachFileText.value = '';
		// this.attachFileInput.form.reset(); used to be called here, but it blew up on IE9 -totally crashed the browser.
		this.attachFileDiv.style.display = 'none';
		this.attachFile.disabled = false;
		dojo.removeClass(this.attachFile,'bpm-social-link-disabled');
		if (e) {
			dojo.stopEvent(e);
		}
		return false;
	},
	
	_cancelLink: function(e) {
		this.attachLinkInput.value = '';
		this.attachLinkDiv.style.display = 'none';
		this.attachLink.disabled = false;
		dojo.removeClass(this.attachLink,'bpm-social-link-disabled');
		if (e) {
			dojo.stopEvent(e);
		}
		return false;
	},
	
	_handlePostInputKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			this._handlePost();
			dojo.stopEvent(e);
		}
	},
	
	_handleLinkKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			dojo.stopEvent(e);
		}
	},

	_handleBrowseFileKeyPress: function(e) {
		// In IE, enter key tries to submit the form by default, so let's prevent this
		// (Space bar opens the file browser)
		if (dojo.isIE) {
			if (e.keyCode == dojo.keys.ENTER) {
				if(e.preventDefault) {
					// IE 9
					e.preventDefault();
				} else {
					// IE < 9
					e.cancelBubble = true;
					e.returnValue = false;
				}
			}			
		}
	},
	
	_handleKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ESCAPE) {
			// Accessibility 
			// If user presses escape, change focus out to main content
			try { // prevent default behavior - in Firefox, ESC stops the page
				e.preventDefault(); // Non-IE
			} catch(x) {
				e.returnValue = false; // IE
			}
			
			if (dojo.style(dojo.query(".workViewCompletion")[0], "display") === "none") {
				var node = dojo.query(".detailsContent .instanceName")[0];
				dojo.attr(node, "tabindex", "-1");
				var blur = dojo.connect(node, "onblur", function() {
					dojo.attr(this, "tabindex", "0");
					dojo.disconnect(this, blur);
				});
				node.focus();
			} else {
				dojo.query(".processPortalSelectedPage .bpm-task-header-subject .bpm-social-drop-down")[0].focus();
			}
		}
	},
	
	_handlePostInitialFocus: function() {
		dojo.removeClass(this.header,'bpm-social-stream-header-init');
		this.streamInputBox.clearInput(true);
	},
	
	_fileInputChanged: function() {
		// PMR 138549 - this will be called via dojo when the onchange event is fired; and also when the input gets focus - see _fileInputOnFocus() below.
		// This covers browsers that don't fire onchange when the same file is picked twice in a row in the file dialog.
		name = this.attachFileInput.value;
		name = name.substring(name.lastIndexOf('/')+1);
		name = name.substring(name.lastIndexOf('\\')+1);
		this.attachFileText.value = name;
	},
	
	_fileInputOnFocus: function() {
		// PMR 138549 - using onfocus enables us to know when the file dialog is closed.
		this._fileInputChanged();
	},
	
	tagUser: function(userName,fullName) {
		this.streamInputBox.clearInput();
		this.streamInputBox.tagUser(userName,fullName);
	},
	
    _setPostFocusStyle: function(e) {
         dojo.addClass("bpm_social_button_left","bpm-social-button-focus");

    },

    _removePostFocusStyle: function(e) {
         dojo.removeClass("bpm_social_button_left","bpm-social-button-focus");
    },

    _setBrowseFocusStyle: function(e) {
         dojo.addClass("attach_file_button","bpm-social-button-focus");
		this._fileInputOnFocus();

    },

    _removeBrowseFocusStyle: function(e) {
         dojo.removeClass("attach_file_button","bpm-social-button-focus");
    },

	destroy: function() {
		this.streamInputBox.destroy();
		if (this._fixStreamHeightTimer) {
			window.clearInterval(this._fixStreamHeightTimer);
			this._fixStreamHeightTimer = null;
		}
		if (this.timer) {
			window.clearTimeout(this.timer);
			this.timer = null;
		}
		if (this.sub) {
			dojo.unsubscribe(this.sub);
			this.sub = null;
		}
		this._clearEvents();
		if (this.more) {
			dojo.disconnect(this.moreConnect);
			this.more = null;
		}
		if (this._userProfileChangeSubscribeHandle) {
			dojo.unsubscribe(this._userProfileChangeSubscribeHandle);
			this._userProfileChangeSubscribeHandle = null;
		}
		this.inherited('destroy',arguments);
	}
	
});

