/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.StreamEvent");

dojo.require("com.ibm.bpm.social.widgets.stream.StreamInputBox");
dojo.require("dijit._Templated");

dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");
dojo.require("com.ibm.bpm.social.widgets.common.BizCard");

dojo.require("com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.require("com.ibm.bpm.social.util.ui.Avatar");

dojo.declare("com.ibm.bpm.social.widgets.stream.StreamEvent", [com.ibm.bpm.social.widgets.common._RestWidget,
                                                               dijit._Templated], {

  templateString: "",
  templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.stream", "templates/StreamEvent.html"),
  events: null,
  overIcon: null,
  overIconTimer: null,
  overIconDelay: 1000,
  overIconTooltip: null,
  taskid: null,
  instanceid: null,
  //commentUrl: "/ProcessPortal/api/rest/task/{tid}/comment",
  commentUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/comment",
  auditTrailUrl: window.endpointURLObject.processPortalSupport +"/jsp/processAudit.do?bpdInstanceId=${iid}",
  streamInputBox: undefined,

  constructor: function(){
    this.events = new Array();
  },

  postCreate: function(){
    dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();

    dojo.addClass(this.domNode, 'bpm-social-stream-grouped-event-expanded');

    this.comment.innerHTML = this.messages.comment;
    this.showDiscussion.innerHTML = this.messages.showComments;
    this.hideDiscussion.innerHTML = this.messages.hideComments;
    this.postButton.innerHTML = this.messages.post;

    this.showDiscussion.style.display = 'none';
    this.hideDiscussion.style.display = 'none';
    this.comments.style.display = 'none';
    this.newComment.style.display = 'none';
    this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
    this._avatar=[];

    if (this.event) {
      this.events.push(this.event);
    }

    this._showActor();
    for ( var i = 0; i < this.events.length; i++) {
      this._showEvent(this.events[i]);
    }

    this.streamInputBox = new com.ibm.bpm.social.widgets.stream.StreamInputBox({}, this.postInput);
    dijit.setWaiState(this.streamInputBox.postInput, "label", this.messages.comment);

    this.text.tabIndex = '0';
    this.connect(this.text, 'onfocus', '_handleTextFocus');
  },

  _handleTextFocus: function(e){
    if (e.target == this.text) {
      this._screenReaderReadText(this.text.innerHTML);
    }
  },

  getActorId: function(e){
    return this.events[0].actor.id;
  },

  updateAvatarUrlForUserid: function(url, userid){
    // update actor avatar
    if (this.events[0].actor.id == userid) {
      if (!this.events[0].actor.image) {
        this.events[0].actor.image = new Object();
      }
      this.events[0].actor.image.url;

      /** Business Card integration * */
      if (this.events[0].actor.id != 'ibm.bpm.system') {
        var user = com.ibm.bpm.social.util.common.Util.getUser(userid);
        if (typeof (user["userID"]) != "undefined") {
          this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);
        }
      }
      /** End Business Card integration * */
    }

    // update discussion avatars
    var updateDiscussion = false;
    for ( var i = 0; i < this.events.length; i++) {
      var e = this.events[i];
      if (e.replies && e.replies.items && e.replies.items.length) {
        for ( var j = 0; j < e.replies.items.length; j++) {
          var r = e.replies.items[j];
          if (r.author && (r.author.id == userid)) {
            if (!r.author.image) {
              r.author.image = new Object();
            }
            //r.author.image.url = url;
            r.author.image.url = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(r.author.id);
            updateDiscussion = true;
          }
        }
      }
    }
    if (updateDiscussion) {
      this.updateComments();
    }
  },

  isEventCollapsable: function(e){
    var rc = false;
    // check if the same actor
    if (this.getActorId() == e.actor.id) {
      // check if file or link
      if (e.object && e.object.objectType && (e.object.objectType.toLowerCase() == 'file')) {
        // check if attached within 10 seconds of the previous event
        var lastDate = com.ibm.bpm.social.util.common.Util.getDateFromString(this.events[this.events.length - 1].published);
        var newDate = com.ibm.bpm.social.util.common.Util.getDateFromString(e.published);
        if ((newDate.getTime() - lastDate.getTime()) < 10000) {
          rc = true;
        }
      }
    }
    return rc;
  },

  addEvent: function(e){
    // add attachment event 
    this._showAttachmentWithLastEvent(e);
    this.events.push(e);
  },

  _showActor: function(){
    var url = dojo.moduleUrl("dojo", "resources/blank.gif");
    var alt = '';
    if (this.events[0].actor.objectType.toLowerCase() == 'service') {
      if (this.events[0].object.objectType.toLowerCase().indexOf('overdue') != -1) {
        this.avatar.setCSSClass('bpm-social-overdue-avatar');
      }
      else {
        this.avatar.setCSSClass('bpm-social-generic-system-avatar');
      }
    }
    else if (this.events[0].actor.objectType.toLowerCase() == 'person') {
      if (this.events[0].actor.image && this.events[0].actor.image.url) {
        url = this.events[0].actor.image.url;
      }
      else {
        this.avatar.setCSSClass('bpm-social-default-avatar-medium');
      }
      alt = this.events[0].actor.displayName;
    }
    else if (this.events[0].actor.objectType.toLowerCase() == 'group') {
      // no group avatar yet, so use default for now
      this.avatar.setCSSClass('bpm-social-default-avatar-medium');
    }

    this.actor.innerHTML = "";
    if (this.events[0].actor.displayName && (this.events[0].actor.id != 'ibm.bpm.system')) {
      this.actor.innerHTML =
        "<span class='bpm-social-user' userId='" + this.events[0].actor.id + "'>" + this.events[0].actor.displayName +
          "</span> ";

      /** Business Card integration * */
      var user = com.ibm.bpm.social.util.common.Util.getUser(this.events[0].actor.id);
      if (typeof (user["fullName"]) == "undefined" && typeof (this.events[0].actor.displayName) != "undefined") {
        user["fullName"] = this.events[0].actor.displayName;
      }

      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user", this.actor)[0], user);
        this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);
      }
      /** End Business Card integration * */
    }

  },

  _showAttachmentWithLastEvent: function(e){
    var text = e.content;

    var tmp = e.object.url;
    var i = text.indexOf(tmp);
    if (i == -1) {
      tmp = e.object.displayName;
      i = text.indexOf(tmp);
    }
    if (i != -1) {
      text =
        text.substring(0, i) + '<a href="' + e.object.url + '" target="_blank">' + tmp + '</a>' +
          text.substring(i + tmp.length);
    }

    var div = this.text.firstChild;
    div.innerHTML = div.innerHTML + '<br>' + text;
  },

  _showEvent: function(e){
    if (e.content) {
      //create the dom nodes
      var div = document.createElement('div');
      div.className = 'bpm-social-stream-event-data-text';
      var textNode = document.createElement("div");
      dojo.addClass(textNode, "bpm-social-stream-event-text-token");

      var text = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(e.content);
      text = text.replace(/<user/g, "<span class='bpm-social-user'");
      text = text.replace(/<\/user>/g, "</span>");

      //default to this text, however further code may override this with modified text
      textNode.innerHTML = text;

      //TODO Temporarily pre-pend user to beginning of comment
      //This will be updated in US-55939
      if (e.object && e.object.objectType.toLowerCase() == 'ibm.bpm.comment') {

        //Adjust here for mirroring make a user node if one is needed
        if (e.actor && e.actor.displayName) {
          var userNode = dojo.create("span", {
            //style: {
            //  "margin-right": "4px"
            //},
            innerHTML: e.actor.displayName + ":"
          });
          dojo.addClass(userNode,
            "bpm-social-user proper-noun bpm-social-stream-event-comment-author bpm-social-stream-event-text-token");
          //Append the user node and the text and apply btd separately for each
          div.appendChild(userNode);
          com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(userNode);
        }

		// Defect 132580
        textNode.innerHTML = dojo.string.substitute(
					this.messages["taggedProcessList-quotedText"], {
					'text': text
				});
        //For User Comments we need to apply base text direction to the content node
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(textNode);
      }

      var auditLink = null;
      if (e.object && e.object.objectType.toLowerCase() == 'ibm.bpm.task') {
        if (e.content.indexOf('completed the task') > -1) {
          //TODO:  Need a better way of signifying specific TASK & INSTANCE based status / actions
          var iid = this.instanceid;

          auditLink = dojo.create("a", {
            'onclick': dojo.hitch(this, function(){

              var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
                "instanceId": iid,
                "header": true
              });

              var ahDialog = new com.ibm.bpm.social.widgets.dialogs.AuditHistoryDialog({
                'instanceId': instanceData.piid,
                'templateName': instanceData.processTemplateName,
                'instanceName': instanceData.name,
                'proxyUrl': this.getProxyUrl()
              });
              ahDialog.show();

            }),
            'innerHTML': this.messages.showAudit,
            'style': 'cursor: pointer; margin-left: 10px;'
          });

        }
        //make a node to hold the comment text
        textNode.innerHTML = text;
      }

      if (e.object && e.object.objectType && (e.object.objectType.toLowerCase() == 'file')) {
        var tmp = e.object.url;
        var i = text.indexOf(tmp);
        if (i == -1) {
          tmp = e.object.displayName;
          i = text.indexOf(tmp);
        }
        if (i != -1) {
          text =
            text.substring(0, i) + '<a href="' + e.object.url + '" target="_blank">' + tmp + '</a>' +
              text.substring(i + tmp.length);
        }
        //make a node to hold the comment text, will be btd'ed
        textNode.innerHTML = text;
      }

      div.appendChild(textNode);

      //add a div to fix floating issue where dates creep up onto same line as comments
      var noFloatDiv = dojo.create("div");
      dojo.addClass(noFloatDiv, "nofloat");
      div.appendChild(noFloatDiv);

      var user = null;

      /** Business Card integration * */
      var properNouns = dojo.query(".proper-noun", div);
      for ( var x = 0; x < properNouns.length; x++) {

        //Handle BTD for each of the proper nouns (Apply UCC) 
        //Do this before BizCard wrap so we don't lose dojo connects
        this._applyUCCproperNoun(properNouns[x]);

        var objId = properNouns[x].getAttribute("objId");
        if (typeof (objId) != "undefined" && objId != null && objId != "") {
          user = com.ibm.bpm.social.util.common.Util.getUser(objId);
          if (typeof (user["userID"]) != "undefined") {
            com.ibm.bpm.social.widgets.common.BizCard.wrapElement(properNouns[x], user);
          }
        }
      }

      if (e.actor.id != "ibm.bpm.system") {
        user = com.ibm.bpm.social.util.common.Util.getUser(e.actor.id);
        if (typeof (user["fullName"]) == "undefined" && typeof (e.actor.displayName) != "undefined") {
          user["fullName"] = e.actor.displayName;
        }
        if (typeof (user["userID"]) != "undefined") {
          com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user.proper-noun", div)[0], user);
        }
      }

      var bpmUserTags = dojo.query(".bpm-user-tag", div);
      for ( var x = 0; x < bpmUserTags.length; x++) {
        var objId = bpmUserTags[x].getAttribute("objId");
        if (typeof (objId) != "undefined" && objId != null && objId != "") {
          user = com.ibm.bpm.social.util.common.Util.getUser(objId);
          if (typeof (user["userID"]) != "undefined") {
            com.ibm.bpm.social.widgets.common.BizCard.wrapElement(bpmUserTags[x], user);
          }
        }
      }
      /** End Business Card integration * */

      if (auditLink != null) {
        dojo.place(auditLink, div, "last");
      }

      if (e.published) {
        var timestamp = document.createElement('div');
        timestamp.className = 'bpm-social-stream-event-timestamp';
        timestamp.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(e.published);
        this.text.insertBefore(timestamp, this.text.firstChild);
      }
      this.text.insertBefore(div, this.text.firstChild);
    }
    if (e.replies && e.replies.items && e.replies.items.length) {
      //this._hideDiscussion(undefined,e);
      this._showDiscussion(undefined, e);
    }
    if (com.ibm.bpm.social.util.common.Util.isCommentSupported(e)) {
      this.comment.style.display = 'inline';
      dojo.addClass(this.domNode, 'bpm-social-stream-event-show-subscript');
    }
    var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
    if (e.target && (e.target.id == user.userName)) {
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.created') {
        dojo.addClass(this.domNode, 'bpm-social-help-request');
        this.comment.style.display = 'none';
      }
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.replied') {
        dojo.addClass(this.domNode, 'bpm-social-help-request-completed');
      }
      if (e.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.ignored') {
        dojo.addClass(this.domNode, 'bpm-social-help-request-ignored');
      }
    }
    if (e.object.objectType.toLowerCase() == 'ibm.bpm.instance.overdue') {
      dojo.addClass(this.domNode, 'bpm-social-stream-event-overdue');
    }
  },

  _applyUCCproperNoun: function(node){
    //ensure span passed in
    if (typeof node == "undefined" || node == "" || node == null) {
      return;
    }

    //check if link inside of span (will happen if username bizcard wrapped)
    var html = node.innerHTML;
    if (html == "") {
      return;
    }

    var endElemInd = -1;
    var closeElemStartInd = -1;
    var isLink = false;
    var htmlText = html;

    if (html.indexOf("<a") != -1) {
      var endElemInd = html.indexOf(">");
      if (endElemInd == -1) {
        return;
      }
      var closeElemStartInd = html.indexOf("<", endElemInd);
      if (closeElemStartInd == -1 || closeElemStartInd < endElemInd) {
        return;
      }

      //determine the necessary direction (may differ based on user pref)
      isLink = true;
      htmlText = html.substring(endElemInd + 1, closeElemStartInd);
    }

    //Apply The UCC characters and fix the parenthesis to the text
    htmlText = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(htmlText);

    //Add the link html back, if there was any
    if (isLink) {
      htmlText = html.substring(0, endElemInd + 1) + htmlText + html.substring(closeElemStartInd, html.length + 1);
    }

    //Apply the updated HTML
    node.innerHTML = htmlText;
  },

  _showComment: function(event, i){
    var div = document.createElement('div');
    div.className = 'bpm-social-stream-event';
    var reply = event.replies.items[i];

    //Determine the btd and generate the html attribute string for the userid of the comment
    var userbtdstring = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(reply.author.displayName);
    if (userbtdstring != "default") {
      userbtdstring = "dir=\"" + userbtdstring + "\"";
    }
    else {
      userbtdstring = "";
    }

    //Determine the btd and generate the html attribute string for the content of the comment
    var replyContent = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(reply.content);
    var contbtdstring = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(replyContent);
    if (contbtdstring != "default") {
      contbtdstring = "dir=\"" + contbtdstring + "\"";
    }
    else {
      contbtdstring = "";
    }

    div.innerHTML =
      "<div class='bpm-social-stream-event-img-div'tabIndex='0'></div>" +
      "<div class='bpm-social-stream-event-data-div' tabIndex='0'>" +
         "<div class='bpm-social-stream-event-comment-text'>" +
           "<span class='bpm-social-user'" + userbtdstring + " userId='" + reply.author.id + "'>" + 
               reply.author.displayName +
           "</span><br><span " + contbtdstring + ">" + replyContent + "</span></div>" +
          "<span class='bpm-social-stream-event-timestamp'>" +
           com.ibm.bpm.social.util.common.Util.formatDate(reply.published) + 
        "</span></div>";
    var avatar= new com.ibm.bpm.social.util.ui.Avatar();
    this._avatar.push(avatar);
    div.firstChild.appendChild(avatar.domNode);
    this.comments.insertBefore(div, this.newComment);
    

    /** Business Card integration * */
    if (reply.author.id != "ibm.bpm.system") {
      var user = com.ibm.bpm.social.util.common.Util.getUser(reply.author.id);
      if (typeof (user["fullName"]) == "undefined" && typeof (reply.author.displayName) != "undefined") {
        user["fullName"] = reply.author.displayName;
      }

      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query(".bpm-social-user", div)[0], user);
        avatar.setUser(user, "bpm-social-default-avatar-small", true);
      }
    }
    /** End Business Card integration * */
  },

  updateComments: function(){
    if (this.showDiscussion.style.display == 'none') {
      while (this.comments.firstChild != this.newComment) {
        var c = this.comments.firstChild;
        this.comments.removeChild(c);
        delete c;
      }
      this._showDiscussion();
    }
    else {
      while (this.discussionIcons.firstChild) {
        var c = this.discussionIcons.firstChild;
        this.discussionIcons.removeChild(c);
        delete c;
      }
      this._hideDiscussion();
    }
  },

  _showDiscussion: function(mouseEvent, e){
    this.commentDivider.style.display = 'inline';
    this.showDiscussion.style.display = 'none';
    this.hideDiscussion.style.display = 'inline';
    while (this.discussionIcons.firstChild) {
      var c = this.discussionIcons.firstChild;
      this.discussionIcons.removeChild(c);
      delete c;
    }
    this.discussionCount.innerHTML = "";
    for ( var i = 0; i < this.events.length; i++) {
      if (this.events[i].replies && this.events[i].replies.items && this.events[i].replies.items.length) {
        for ( var j = 0; j < this.events[i].replies.items.length; j++) {
          if ((e == undefined) || (e == this.events[i])) {
            this._showComment(this.events[i], j);
          }
        }
      }
    }
    this.comments.style.display = 'block';
  },

  _addDiscussionIcon: function(eventIndex, commentIndex){
    var event = this.events[eventIndex];
    var img = document.createElement('img');
    img.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_18x18.png");
    var author = this.events[eventIndex].replies.items[commentIndex].author;
    if (author && author.image && author.image.url) {
      img.src = author.image.url;
    }
    img.id = eventIndex + '_' + commentIndex + '_' + this.domNode.id;
    img.alt = "";
    img.style.width = '16px';
    img.style.height = '16px';
    this.discussionIcons.appendChild(img);

    /** Business Card integration * */
    if (author.id != "ibm.bpm.system") {
      img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(author.id);
      var user = com.ibm.bpm.social.util.common.Util.getUser(author.id);
      if (typeof (user["userID"]) != "undefined") {
        com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
      }
    }
    /** End Business Card integration * */

  },

  _mouseOver: function(e){
    if (e.target.src && !this.overIcon) {
      this.overIcon = e.target;
      this.overIconTimer = window.setTimeout(dojo.hitch(this, '_showDiscussionIconTooltip'), this.overIconDelay);
    }
  },

  _mouseMove: function(e){
    if (this.overIconTimer) {
      window.clearTimeout(this.overIconTimer);
      this.overIconTimer = window.setTimeout(dojo.hitch(this, '_showDiscussionIconTooltip'), this.overIconDelay);
    }
  },

  _mouseOut: function(e){
    if (this.overIconTooltip) {
      this.overIconTooltip.parentNode.removeChild(this.overIconTooltip);
      delete this.overIconTooltip;
      this.overIconTooltip = null;
      dojo.removeClass(this.overIcon, 'bpm-social-event-comment-active');
    }
    if (this.overIconTimer) {
      window.clearTimeout(this.overIconTimer);
      this.overIconTimer = null;
    }
    this.overIcon = null;
  },

  _showDiscussionIconTooltip: function(){
    this.overIconTimer = null;
    this.overIconTooltip = document.createElement('div');
    this.overIconTooltip.className = 'bpm-social-stream-event-tooltip';
    var div = document.createElement('div');
    div.className = 'bpm-social-stream-event-img-div';
    this.overIconTooltip.appendChild(div);
    var img = document.createElement('img');
    img.className = 'bpm-social-stream-event-img';
    img.alt = '';
    img.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
    div.appendChild(img);
    div = document.createElement('div');
    div.className = 'bpm-social-stream-event-data-div';
    this.overIconTooltip.appendChild(div);
    var tmp = this.overIcon.id.split('_');
    var i = eval(tmp[0]);
    var j = eval(tmp[1]);
    var reply = this.events[i].replies.items[j];
    if (reply.author && reply.author.image && reply.author.image.url) {
      //img.src = reply.author.image.url;
      img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(reply.author.id);
    }
    var replyContent = com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(reply.content);
    div.innerHTML =
      "<span class='bpm-social-user' userId='" + reply.author.id + "'>" + reply.author.displayName + "</span><br>" +
        replyContent + "<br>" + "<span class='bpm-social-stream-event-timestamp'>" +
        com.ibm.bpm.social.util.common.Util.getDateFromString(reply.published).toLocaleString() + "</span>";
    document.body.appendChild(this.overIconTooltip);
    var coords = dojo.coords(this.overIcon, true);
    var x = coords.x - 50;
    var y = coords.y + 24;
    this.overIconTooltip.style.left = '0px';
    this.overIconTooltip.style.top = y + 'px';
    this.overIconTooltip.style.visibility = 'hidden';
    document.body.appendChild(this.overIconTooltip);
    if ((x + this.overIconTooltip.offsetWidth + 5) > document.body.offsetWidth) {
      x = document.body.offsetWidth - this.overIconTooltip.offsetWidth - 5;
    }
    this.overIconTooltip.style.left = x + 'px';
    this.overIconTooltip.style.visibility = 'visible';
    dojo.addClass(this.overIcon, 'bpm-social-event-comment-active');
  },

  _hideDiscussion: function(mouseEvent, e){
    this.commentDivider.style.display = 'inline';
    this.showDiscussion.style.display = 'inline';
    this.hideDiscussion.style.display = 'none';
    while (this.comments.firstChild != this.newComment) {
      var c = this.comments.firstChild;
      this.comments.removeChild(c);
      delete c;
    }
    if (this.newComment.style.display == 'none') {
      this.comments.style.display = 'none';
    }
    var len = 0;
    for ( var i = 0; i < this.events.length; i++) {
      if (this.events[i].replies && this.events[i].replies.items) {
        for ( var j = 0; j < this.events[i].replies.items.length; j++) {
          if ((e == undefined) || (e == this.events[i])) {
            if (this.discussionIcons.children.length == 5) {
              this.discussionIcons.appendChild(document.createTextNode('...'));
            }
            if (this.discussionIcons.children.length < 5) {
              this._addDiscussionIcon(i, j);
            }
          }
          len++;
        }
      }
    }
    this.discussionCount.innerHTML = "(" + len + ")";
  },

  _handleComment: function(e){
    if (this.newComment.style.display == 'none') {
      this.comments.style.display = 'block';
      this.newComment.style.display = 'block';
      this.streamInputBox._setPostInputFocus();
    }
    else {
      this.newComment.style.display = 'none';
      if (this.hideDiscussion.style.display == 'none') {
        this.comments.style.display = 'none';
      }
    }
    if (e) {
    	dojo.stopEvent(e);
    }
  },

  _handleShowAudit: function(){
    this.audit.style.display = 'block';
    this.hideAuditButton.style.display = 'inline';
    this.showAuditButton.style.display = 'none';
    this.hideAuditButton.focus();
  },

  _handleHideAudit: function(){
    this.audit.style.display = 'none';
    this.hideAuditButton.style.display = 'none';
    this.showAuditButton.style.display = 'inline';
    this.showAuditButton.focus();
  },

  _handlePostInputKeyPress: function(e){
    if (e.keyCode == dojo.keys.ENTER) {
      this._handlePost();
      dojo.stopEvent(e);
    }
  },

  _getEventId: function(){
    var id = null;
    for ( var i = 0; (id == null) && (i < this.events.length); i++) {
      if (com.ibm.bpm.social.util.common.Util.isCommentSupported(this.events[i])) {
        id = this.events[i].object.id;
        if (id.indexOf('.') != -1) {
          id = id.substring(id.indexOf('.') + 1);
        }
      }
    }
    return id;
  },

  _getEventTargetName: function(){
    var name = null;
    for ( var i = 0; (name == null) && (i < this.events.length); i++) {
      if (com.ibm.bpm.social.util.common.Util.isCommentSupported(this.events[i])) {
        name = this.events[i].target.displayName;
        if (!name) {
          name = this.events[i].target.id;
        }
      }
    }
    return name;
  },

  _handlePost: function(){
    var txt = this.streamInputBox.getPostInput();
    if (txt) {
      var data = {
        message: txt
      };
      var id = this._getEventId();
      dojo.addClass(this.domNode, 'bpm-social-event-post-in-progress');

      this.socialRestHandler.postTaskStreamComment(id, dojo.toJson(data), dojo.hitch(this, function(data, xhrArgs){
        if (data.posted) {
          this.getEventUpdates();
        }
        else {
          dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
        }
      }), dojo.hitch(this, function(data, xhrArgs){
        dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
      }), true);

    }
    this.streamInputBox.clearInput();
    this._handleComment();
  },

  getEventUpdates: function(){
    dojo.publish("/com/ibm/bpm/social/updateStream/" + this.instanceid, [{}]);
    dojo.removeClass(this.domNode, 'bpm-social-event-post-in-progress');
  },

  updateEvent: function(event){
    for ( var j = 0; j < this.events.length; j++) {
      if (event.published && (event.published == this.events[j].published)) {
        if (this.events[j].object && (this.events[j].object.objectType == 'ibm.bpm.helprequest.created')) {
          if (event.object && (event.object.objectType != 'ibm.bpm.helprequest.created')) {
            this.events[j].object.objectType = event.object.objectType;
            dojo.removeClass(this.domNode, 'bpm-social-help-request');
            dojo.removeClass(this.domNode, 'bpm-social-help-request-reply-in-progress');
            this.comment.style.display = 'inline';
            if (event.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.replied') {
              dojo.addClass(this.domNode, 'bpm-social-help-request-completed');
            }
            if (event.object.objectType.toLowerCase() == 'ibm.bpm.helprequest.ignored') {
              dojo.addClass(this.domNode, 'bpm-social-help-request-ignored');
            }
          }
        }
        if (event.replies && event.replies.items && event.replies.items.length) {
          var needsUpdate = true;
          if (this.events[j].replies && this.events[j].replies.items &&
            (this.events[j].replies.items.length == event.replies.items.length)) {
            needsUpdate = false;
          }
          if (needsUpdate) {
            this.events[j].replies = event.replies;
            this.updateComments();
          }
        }
      }
    }
  },

  _screenReaderReadText: function(txt){
    var div = dojo.byId('screenReaderDiv');
    if (!div) {
      div = document.createElement('div');
      div.id = 'screenReaderDiv';
      div.style.position = 'absolute';
      div.style.top = '-10px';
      div.style.width = '1px';
      div.style.height = '1px';
      div.style.overflow = 'hidden';
      dijit.setWaiRole(div, 'note');
      dijit.setWaiState(div, 'live', 'polite');
      document.body.appendChild(div);
    }
    div.innerHTML = '';
    div.innerHTML = txt;
  },

  _handleOnMouseOverText: function(e){
    var node = e.target;
    // check if the mouse is over a user tag
    if (node && dojo.hasClass(node, 'bpm-user-tag')) {
      // if the user name is clipped, provide hover help
      if (node.offsetWidth > node.parentNode.offsetWidth) {
        node.title = node.innerHTML;
      }
    }
  },

  destroy: function(){
    this._mouseOut();
    this.events = null;
    this.streamInputBox.destroy();
    if (this.avatar){
       this.avatar.destroy();
       this.avatar=null;
    }
    if (this._avatar){
      for (var i=0; i<this._avatar.length; i++){
        this._avatar[i].destroy();
      }
      this._avatar=null;
    }
    this.inherited('destroy', arguments);
  }

});
