/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.stream.StreamInputBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.util.RestClient");

dojo.declare(
	"com.ibm.bpm.social.widgets.stream.StreamInputBox",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.stream", "templates/StreamInputBox.html"),
	userSearchList: undefined,
	userSearchConnections: null,
	userSearchValue: undefined,
	userSearchTimer: undefined,
	userSearchSelected: undefined,
	restClient: undefined,
	dropDownMaxSize: (dojo.isSafari && (navigator.userAgent.indexOf('iPad') != -1))? 5:10,
			
	constructor: function() {
		this.userSearchConnections = new Array();
	},
			
    postCreate: function() {
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.restClient = new com.ibm.bpm.social.util.ui.userchooser.util.RestClient();
		
		//Try to configure BTD here?
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.postInput);
	},
	
	getPostInput: function () {
		var text = '';
		var child = this.postInput.firstChild;
		while (child) {
			if ((child.nodeName.toLowerCase() == 'span') && dojo.hasClass(child,'bpm-user-tag')) {
				text = text + '<taggedUser>'+child.id+'</taggedUser>';
			} else if (child.textContent) {
				text = text + child.textContent;
			} else if (child.innerText) {
				text = text + child.innerText;
			} else if (child.innerHTML) {
				text = text + child.innerHTML;
			} else if (child.nodeValue) {
				text = text + child.nodeValue;
			}
			child = child.nextSibling;
		}
		return text;
	},

	clearInput: function(setFocus) {
		this._cancelUserSearch();
		this.postInput.innerHTML = '';
		if (setFocus) {
			this.postInput.focus();
		}
		
	},
	
	_getNameForUser: function(user) {
		var name = user.fullName;
		if (!name) {
			name = user.cn;
		}
		if (!name) {
			name = user.userName;
		}
		return name;
	},
	
	_getNodeText: function(node) {
		var text = '';
		var child = node.firstChild;
		while (child) {
			if ((child.nodeName.toLowerCase() == 'span') && dojo.hasClass(child,'bpm-user-tag')) {
				text = text + '<span id="'+child.id+'" class="bpm-user-tag">'+child.innerHTML+'</span>';
			} else if (child.nodeName.toLowerCase() == 'span') {
				text = text + this._getNodeText(child);
			} else if (child.textContent) {
				text = text + dojox.html.entities.encode(child.textContent);
			} else if (child.innerHTML){
				text = text + child.innerHTML;
			} else if (child.nodeValue) {
				text = text + dojox.html.entities.encode(child.nodeValue);
			}
			child = child.nextSibling;
		}
		return text;
	},
	
	_getPostInput: function() {
		return this._getNodeText(this.postInput);
	},
	
	tagUser: function(userName,fullName) {
		var span = document.createElement('span');
		span.id = userName;
		span.className = 'bpm-user-tag';
		span.contentEditable = 'false';
		span.innerHTML = fullName;
		this.postInput.appendChild(span);
		span = document.createElement('span');
		span.innerHTML = '&nbsp;'
		this.postInput.appendChild(span);
		if (window.getSelection) {
			window.getSelection().collapse(span,1);
		}
	},
	
	_selectUserDiv: function(userDiv) {
		if (this.userSearchSelected) {
			dojo.removeClass(this.userSearchSelected,'bpm-social-stream-user-selected');
		}
		this.userSearchSelected = userDiv;
		if (this.userSearchSelected) {
			dojo.addClass(this.userSearchSelected,'bpm-social-stream-user-selected');
    		var user = this.userSearchList[eval(this.userSearchSelected.id)];
    		var name = this._getNameForUser(user);
    		this._screenReaderReadText(name);
		}
	},
	
	_getIndexOfTagChar: function(txt) {
		// need to ignore any existing user tags in the string
		var tmp = txt;
		var tagStart = tmp.indexOf('<span');
		while (tagStart != -1) { 
			var tagStop = tmp.indexOf('</span>',tagStart);
			var prefix = tmp.substring(0,tagStart);
			while (prefix.length < tagStop) {
				prefix = prefix + 'x';
			}
			tmp = prefix + tmp.substring(tagStop);
			tagStart = tmp.indexOf('<span',tagStop);
		}
		return tmp.indexOf('@');
	},
	
	_handleEnterKey: function(e) {
		if (this.userSearchList) {
			if (this.userSearchSelected) {
				var user = this.userSearchList[eval(this.userSearchSelected.id)];
				var text = this._getPostInput();
				var index = this._getIndexOfTagChar(text);
				if (index != -1) {
					this.postInput.innerHTML = text.substring(0,index);
					this.tagUser(user.userName,this._getNameForUser(user));
					this._cancelUserSearch();
					dojo.stopEvent(e);
				}
			}
		}
	},
	
	_handlePostInputKeyPress: function(e) {
		if (e.keyCode == dojo.keys.ENTER) {
			this._handleEnterKey(e);
		}
		if (this.userSearchList) {
			if (e.keyCode == dojo.keys.UP_ARROW) {
				if (this.userSearchSelected.previousSibling) {
					this._selectUserDiv(this.userSearchSelected.previousSibling);
				}
				dojo.stopEvent(e);
			}
			if (e.keyCode == dojo.keys.DOWN_ARROW) {
				if (this.userSearchSelected.nextSibling) {
					this._selectUserDiv(this.userSearchSelected.nextSibling);
				}
				dojo.stopEvent(e);
			}
		}
		if (e.keyCode == dojo.keys.BACKSPACE) {
			var sel = null;
			if (window.getSelection) {
				sel = window.getSelection();
				if (sel && sel.focusNode && (sel.focusOffset == 0)) {
					var node = sel.focusNode;
					while (node && (node.parentNode != this.postInput)) {
						node = node.parentNode;
					}
					if (node && node.previousSibling && dojo.hasClass(node.previousSibling,'bpm-user-tag')) {
						var span = node.previousSibling;
						this.postInput.removeChild(span);
						delete span;
						dojo.stopEvent(e);
					} 
				} else if (sel && sel.focusNode && sel.focusNode.parentNode && dojo.hasClass(sel.focusNode.parentNode,'bpm-user-tag')) {
					var span = sel.focusNode.parentNode;
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					if (!dojo.isIE) {
						window.getSelection().collapse(span,0);
					}
					dojo.stopEvent(e);
				}
			} else if (document.selection) {
	            var range = document.selection.createRange();
	            if (dojo.hasClass(range.parentElement(),'bpm-user-tag')) {
					var span = range.parentElement();
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					dojo.stopEvent(e);
	            }
			}
		}
		if (e.keyCode == dojo.keys.DELETE) {
			if (window.getSelection) {
				var sel = window.getSelection();
				if (sel && sel.focusNode && (sel.focusNode.textContent != undefined) && (sel.focusOffset == sel.focusNode.textContent.length)) {
					var node = sel.focusNode;
					while (node && (node.parentNode != this.postInput)) {
						node = node.parentNode;
					}
					if (node && node.nextSibling && dojo.hasClass(node.nextSibling,'bpm-user-tag')) {
						var span = node.nextSibling;
						this.postInput.removeChild(span);
						delete span;
						dojo.stopEvent(e);
					} 
				} else if (sel && sel.focusNode && sel.focusNode.parentNode && dojo.hasClass(sel.focusNode.parentNode,'bpm-user-tag')) {
					var span = sel.focusNode.parentNode;
					span.id = '';
					span.className = '';
					span.innerHTML = '';
					window.getSelection().collapse(span,0);
					dojo.stopEvent(e);
				}
			}
		}
		else if (e.charCode > 0) {
			setTimeout(dojo.hitch(this, "_startUserSearch", 10));
		}
		else if (document.selection) {
            var range = document.selection.createRange();
            if (dojo.hasClass(range.parentElement(),'bpm-user-tag')) {
				var span = range.parentElement();
				span.id = '';
				span.className = '';
				span.innerHTML = '';
				dojo.stopEvent(e);
            }
		}
	},
	
	_handlePostInputKeyUp: function(e) {
		if(e.keyCode == dojo.keys.BACKSPACE || e.keyCode == dojo.keys.CLEAR || e.keyCode == dojo.keys.DELETE) {
			this._startUserSearch();
		}
	},
	
	_startUserSearch: function() {
		var txt = this._getPostInput();
		this.userSearchIndex = this._getIndexOfTagChar(txt);
		if (this.userSearchIndex == -1) {
			this._cancelUserSearch();
		} else {
			var newUserSearchValue = txt.substring(this.userSearchIndex+1);
			if (this.userSearchValue != newUserSearchValue) {
				this._cancelUserSearch();
				this.userSearchValue = newUserSearchValue;
				this.userSearchTimer = window.setTimeout(dojo.hitch(this,'_doUserSearch'),1000);
			}
		}
	},
	
	_cancelUserSearch: function() {
		this.userSearchValue = undefined;
		if (this.userSearchTimer) {
			window.clearTimeout(this.userSearchTimer);
		}
		if (this.userSearchDiv) {
			this._hideUserSearchDiv();
		}
		if (this.userSearchConnections) {
			while(this.userSearchConnections.length) {
				dojo.disconnect(this.userSearchConnections.pop());
			}
		}
		this.userSearchList = undefined;
		this.userSearchSelected = undefined;
	},
	
	_highlightText: function(fullText,highlightText) {
		var f = fullText.toLowerCase();
		var h = highlightText.toLowerCase();
		var result = '';
		var start = 0;
		var index = f.indexOf(h);
		while (index != -1) {
			result = result + fullText.substring(start,index) +
					'<span class="bpm-social-stream-search-text">' + 
					fullText.substring(index,index+h.length) + '</span>';
			start = index + h.length;
			index = f.indexOf(h,start);
		}
		result = result + fullText.substring(start);
		return result;
	},
	
	_isInvalidWildcard: function(searchString) {
		//check for wildcard character in the middle of the search string
		var wildcard = searchString.indexOf('*');
		while (wildcard != -1) {
			if ((wildcard > 0) && (wildcard < (searchString.length-1))) {
				// we do not support wildcards in the middle of the string
				return true;
			} else {
				wildcard = searchString.indexOf('*',wildcard+1);
			}
		}
		return false;
	},
	
	_doUserSearch: function() {
		this.userSearchTimer = undefined;
		if (this.userSearchValue) {
			var tmp = this.userSearchValue;
			// convert % to * for consistency
		    var wildcard = tmp.indexOf('%');
		    while (wildcard != -1) {
		    	tmp = tmp.substring(0,wildcard)+'*'+tmp.substring(wildcard+1);
		        wildcard = tmp.indexOf('%',wildcard+1);
		    }
		    // check for invalid wildcard usage
		    if (this._isInvalidWildcard(tmp)) {
		    	// clear previous results before showing error
				while(this.userSearchConnections.length) {
					dojo.disconnect(this.userSearchConnections.pop());
				}
				this._selectUserDiv(null);
				while(this.userSearchDiv.firstChild) {
					var child = this.userSearchDiv.firstChild;
					this.userSearchDiv.removeChild(child);
					delete child;
				}
				// use message already in the user chooser
			    dojo.requireLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
			    var ucMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
				this.userSearchDiv.innerHTML = '<span class="bpm-social-stream-no-users-msg">'+ucMessages.invalidWildcardMessage+'</span>';
				this.userSearchSelected = undefined;
				this._showUserSearchDiv();
				return;
		    } 
			// check for mashups wild card setting
			var useWildCards = true;
			try {
				var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
				if (cs != null) {
				   var autoWildCard = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.AUTO_WILDCARD_DISABLED);
				   if (autoWildCard == true) 
				   {
					   useWildCards = false;
				   }
				}
			} catch (err) {
				// if error, then assume wildcards are ok
			}
		    if (useWildCards) {
				if (tmp.length > 2 && tmp.charAt(0) == '"' && tmp.charAt(tmp.length-1) == '"') {
					tmp = tmp.substring(1, tmp.length-1);
				}
				else {
					if (tmp.charAt(0) != '*') tmp = '*' + tmp;
					if (tmp.charAt(tmp.length-1) != '*') tmp = tmp + '*';
				}
		    }
		    
			var users = this.restClient.searchBPMByName(tmp, 0, this.dropDownMaxSize, false, true );
			while(this.userSearchConnections.length) {
				dojo.disconnect(this.userSearchConnections.pop());
			}
			this._selectUserDiv(null);
			while(this.userSearchDiv.firstChild) {
				var child = this.userSearchDiv.firstChild;
				this.userSearchDiv.removeChild(child);
				delete child;
			}
			if (users.length == 0) {
				this.userSearchDiv.innerHTML = '<span class="bpm-social-stream-no-users-msg">'+this.messages.noUsersFound+'</span>';
				this.userSearchSelected = undefined;
			} else {
				this.userSearchList = users;
				for (i=0; i<users.length; i++) {
					var name = this._getNameForUser(users[i]);
					var child = document.createElement('div');
					dojo.addClass(child,'bpm-social-stream-user');
					child.innerHTML = this._highlightText(name,this.userSearchValue) +
						'<br><span class="bpm-social-stream-user-name">'+this._highlightText(users[i].userName,this.userSearchValue)+'</span>';					
					child.id = ''+i;
					this.userSearchDiv.appendChild(child);
					if (this.userSearchDiv.firstChild == child) {
						this._selectUserDiv(child);
					}
					this.userSearchConnections.push(dojo.connect(child,'onmouseover',this,'_handleUserMouseOver'));
					this.userSearchConnections.push(dojo.connect(child,'onmousedown',this,'_handleUserMouseDown'));
				}
			}
			this._showUserSearchDiv();
		}
	},
	
	_getUserDiv: function(e) {
		var div = e.target;
		while (div && !dojo.hasClass(div,'bpm-social-stream-user')) {
			div = div.parentNode;
		}
		return div;
	},

	_handleUserMouseOver: function(e) {
		this._selectUserDiv(this._getUserDiv(e));
		if (this.userSearchSelected && !this.userSearchSelected.title && (this.userSearchSelected.scrollWidth > this.userSearchSelected.offsetWidth)) { 
    		var user = this.userSearchList[eval(this.userSearchSelected.id)];
    		var name = this._getNameForUser(user);
			this.userSearchSelected.title = name;
		}
	},

	_handleUserMouseDown: function(e) {
		window.setTimeout(dojo.hitch(this,'_setPostInputFocus'),100);
		this.userSearchSelected = this._getUserDiv(e);
		this._handleEnterKey(e);
	},
	
	_setPostInputFocus: function() {
		this.postInput.focus();
	},
	
	_handlePostInputBlur: function() {
		window.setTimeout(dojo.hitch(this,'_cancelUserSearch'),100);
	},
	
	_showUserSearchDiv: function() {
		var div = this.userSearchDiv.parentNode;
		div.style.display = 'block';
		if (!div.parentNode) {
			this.domNode.appendChild(div);
		}
		var coords = dojo.coords(div);
		div.parentNode.removeChild(div);
		div.style.width = this.domNode.offsetWidth + 'px';
		div.style.left = (coords.x + document.body.scrollLeft) + 'px';
		div.style.top = (coords.y + document.body.scrollTop) + 'px';
		div.style.position = 'absolute'; 
		document.body.appendChild(div);
	},
	
	_hideUserSearchDiv: function() {
		var div = this.userSearchDiv.parentNode;
		div.style.display = 'none';
		if (div.parentNode) {
			div.parentNode.removeChild(div);
		}
		div.style.left = '0px';
		div.style.top = '0px';
		div.style.position = 'relative'; 
		this.domNode.appendChild(div);
	},
	
	_screenReaderReadText: function(txt) {
		var div = dojo.byId('screenReaderDiv');
		if (!div) {
			div = document.createElement('div');
			div.id = 'screenReaderDiv';
			div.style.position = 'absolute';
			div.style.top = '-10px';
			div.style.width = '1px';
			div.style.height = '1px';
			div.style.overflow = 'hidden';
			dijit.setWaiRole(div,'note');
			dijit.setWaiState(div,'live','polite');
			document.body.appendChild(div);
		}
		div.innerHTML = '';
		div.innerHTML = txt;
	},

	destroy: function() {
	    try {
			this._cancelUserSearch();
		} catch (error) {
			//throwing error on log out
		}
		this.inherited('destroy',arguments);
	}
	
});

