// TasksContainer.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.TasksContainer");

dojo.require("com.ibm.bpm.social.widgets.task.list.TasksListController");
dojo.require("com.ibm.bpm.social.widgets.task.config.ControllerConfig");
dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.config.ViewConfig");
dojo.require("com.ibm.bpm.social.widgets.task.util.ProcessQueues");
dojo.require("com.ibm.bpm.social.widgets.task.util.TaskQueues");
dojo.require("com.ibm.bpm.social.widgets.task.util.TabConfiguration");
dojo.require("com.ibm.bpm.social.widgets.task.util.TabContainer");
dojo.require("com.ibm.bpm.social.widgets.task.util.Constants");
dojo.require("com.ibm.bspace.common.util.ui.Helper");

dojo.require("com.ibm.bpm.social.portal.PageHandler");
dojo.require("com.ibm.bpm.social.util.common.PageSettings");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.widgets.ActionDropDownMenu");
dojo.require("dijit.MenuItem");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.TasksContainer",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: '<div class="bpm-task-list-container" dojoType="dijit.layout.ContentPane">\
    	  <div role="button" dojoAttachPoint="startPageButton"></div>\
        <div dojoAttachPoint="tabContainer"></div>\
        </div>',

      widgetsInTemplate: true,

      _pageHandler: null,

      _listControllers: null,

      constructor: function constructor_TasksContainer(params){
        if (params) {
          if (params.tabHeaderDiv) {
            this._tabHeaderDiv = params.tabHeaderDiv;
          }
        }
        this._subscribedChannels = [];
      },

      postCreate: function postCreate_TasksContainer(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //com.ibm.bpm.social.util.common.Util.loadStylesheet("widgets/task/themes/TasksContainer.css");

        this._pageHandler = com.ibm.bpm.social.portal.PageHandler.getInstance();

        var newStartPageButton = this._pageHandler.createStartPageIcon();
        this.startPageButton.parentNode.replaceChild(newStartPageButton, this.startPageButton);
        this.startPageButton = newStartPageButton;

        this.connect(this.startPageButton, "onclick", this._onStartPageButtonClick);
        this.connect(this.startPageButton, "onkeypress", function(e){
          if (e.keyCode == dojo.keys.ENTER) {
            this._onStartPageButtonClick(e);
          }
        });

        this.containerNode = this.domNode;
        // array for the tabs
        this.tabs = [];
        this._connectsMenu = [];

        // an array for the created listController widgets
        this._listControllers = [];

        // when the Critical Path Management updates something, we need to refresh
        this._subscribedChannels.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED, dojo.hitch(
          this, this.reloadActiveTab)));

        // suspend automatic refresh if dashboards are shown
        this._subscribedChannels.push(dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", dojo.hitch(this,
          function(args){
            if (args !== null && args.showing !== "workPage") {
              // notify actual tab that it will be hidden
              this._onHideTab(this.tabContainerWidget.lastTab.id);
            }
            // no notification needed if workPage is shown again because the tab will be refreshed
          })));


        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Refresh/Reload all tabs of the TasksContainer
       */
      refresh: function refresh_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        try {
          if (!this.tabs || this.tabs.length === 0) {
            this._createContent();
          }
          this.tabContainerWidget.reloadTabs();
        }
        catch (e) {
          com.ibm.bpm.social.util.common.Log.logError(0, "refresh_TasksContainer", e);
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(e.message, /* details */e.stack, /* callback */null);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },


      /**
       * Create the lightweight tab container and it's tabs using Tab Configuration objects
       * 
       * @type void
       */
      _createContent: function _createContent_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // start page handling
        var storedStartPageConfig = this._pageHandler.getStoredPageSettings();

        var startPageId = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS;
        if (storedStartPageConfig.filterConfigId == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
          startPageId = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        }


        // Create My Tasks and Saved Searches tab  (lightweight tabs)
        // first prepare for a TabConfiguration-object for every shown tab - the information
        // of this objects are used for creating the tabs
        // new com.ibm.bpm.social.widgets.task.util.TabConfiguration(tabName, id, isActive, context, loadFunction, tooltip, widgetID, custom)

        // My Tasks Tab 
        var myTasksControllerConfig = com.ibm.bpm.social.widgets.task.config.ControllerConfig.getMyTasksConfig();
        var queue = this._getQueue(myTasksControllerConfig.viewConfig.queueName);
        if (queue !== null) {
          myTasksControllerConfig.queue = queue;
          this.tabs.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS,
            title: this._nlsResources[myTasksControllerConfig.viewConfig.titleKey],
            tooltip: this._nlsResources[myTasksControllerConfig.viewConfig.tooltipKey],
            controllerConfig: myTasksControllerConfig
          });
          var listController = new com.ibm.bpm.social.widgets.task.list.TasksListController({
            typeId: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS
          });
          if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS) {
            //listController.setSearchText(storedStartPageConfig.searchFilterString);
            myTasksControllerConfig.selectedFilter = storedStartPageConfig.filterConfigId;
          }
          // track the state of the controller's default page config
          this.connect(listController, "onLoadItemsSuccess", this._checkAndUpdateStartPageIcon);
          this.connect(listController, "onFocusNeeded", this._setFocusToActiveTab);
          this._listControllers.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS,
            controller: listController
          });
        }
        else {
          var message = this._nlsResources["task_error_saved_search_does_not_exists"];
          message = dojo.string.substitute(message, [myTasksControllerConfig.viewConfig.queueName]);
          var details = this._nlsResources["task_error_saved_search_does_not_exists_myTasks_details"];
          details = dojo.string.substitute(details, [myTasksControllerConfig.viewConfig.queueName]);
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, /* details */details, /* callback */null);
          com.ibm.bpm.social.util.common.Log.consoleError(message);
        }


        // Saved Searches Tab Configuration

        this._dropDown = document.createElement("span");
        var queueNotFound = false;

        // check first, if we need custom views
        var showSavedSearchesTab = this._createQueueDropDown(this._dropDown);
        if (showSavedSearchesTab) {
          var customViewControllerConfig = com.ibm.bpm.social.widgets.task.config.ControllerConfig.getCustomViewsConfig();
          // set the default queue
          queue = null;
          if (storedStartPageConfig.savedSearchName) {
            queue = this._getQueue(storedStartPageConfig.savedSearchName);
          }
          if (queue) {
            customViewControllerConfig.queue = queue;
            // TODO ASZ: Map  SortAttribs from user profile to the queue
            // DOES NOT WORK currently
//            if (defaultPage.sortAttributes.length > 0) {
//              queue.sortAttributes = [];
//              for ( var i = 0; i < defaultPage.sortAttributes.length; i++) {
//                queue.sortAttributes.push({
//                  name: defaultPage.sortAttributes[i].property,
//                  sortOrder: defaultPage.sortAttributes[i].sortDir
//                });
//              }
//            }

          }
          else {
            queueNotFound = true;
            customViewControllerConfig.queue = this._queues[0];
          }


          this.tabs.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES,
            title: this._nlsResources[customViewControllerConfig.viewConfig.titleKey],
            tooltip: this._nlsResources[customViewControllerConfig.viewConfig.tooltipKey],
            controllerConfig: customViewControllerConfig
          });
          var tableController = new com.ibm.bpm.social.widgets.task.list.TasksListController({
            typeId: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES
          });
          if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES) {
            // tableController.setSearchText(storedStartPageConfig.searchFilterString);
          }
          // track the state of the controller's default page config
          this.connect(tableController, "onLoadItemsSuccess", this._checkAndUpdateStartPageIcon);
          this.connect(listController, "onFocusNeeded", this._setFocusToActiveTab);
          this._listControllers.push({
            id: com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES,
            controller: tableController
          });
        }
        this._tabHeaderDiv.style.display = "none";

        //create an array containing the TabConfiguration-objects
        var tabs = new Array();

        for ( var i = 0; i < this.tabs.length; i++) {
          var tab = this.tabs[i];
          var tabConfig =
            new com.ibm.bpm.social.widgets.task.util.TabConfiguration(tab.title /* tabName */,
              tab.id /* tab id */,
              true /* isActive */,
              this /* context */,
              "_initTab" /* loadFunction */,
              tab.tooltip /* tab tooltip */,
              this.id /* widget id */,
              tab.controllerConfig /* our user defined properties stored in TabConfiguration.custom */);

          tabs.push(tabConfig);
          if (tab.id == startPageId) {
            tabConfig.setContentIsShownAfterInit();
          }
        }

        //create the lightweight tab container
        this.tabContainerWidget = new com.ibm.bpm.social.widgets.task.util.TabContainer({
          tabHeaderDiv: this._tabHeaderDiv
        }, this.tabContainer);
        this.tabContainerWidget.setTabConfiguration(tabs);


        // due to errors we can't emsure that there is a tab at all ( both myTasks and SavedSearches might not be there)
        var savedSearchesTab = this._getTabConfig(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES);
        var myTasksTab = this._getTabConfig(com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_MY_TASKS);

        if (startPageId == com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES) {
          // cases:
          // 1. showSavedSearchesTab && !queueNotFound    -> all ok, show savedSearchesTab, no message
          // 2. showSavedSearchesTab && queueNotFound     -> show queueNotFound message,  show myTasksTab
          // 3. !showSavedSearchesTab                     -> show queueNotFound message,  show myTasksTab
          var showQueueNotFoundMessage = !showSavedSearchesTab || queueNotFound;

          if (showQueueNotFoundMessage) {
            message = this._nlsResources["task_info_saved_search_not_available"];
            message = dojo.string.substitute(message, [storedStartPageConfig.savedSearchName]);
            details = this._nlsResources["task_info_saved_search_not_available_details"];
            details = dojo.string.substitute(details, [storedStartPageConfig.savedSearchName]);
            com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, /* details */details, /* callback */null);
            com.ibm.bpm.social.util.common.Log.consoleError(message);
          }

          if (showSavedSearchesTab && !queueNotFound) {
            if (myTasksTab) {
              myTasksTab.setContentIsShownAfterInit(false);
            }
            if (savedSearchesTab) {
              savedSearchesTab.setContentIsShownAfterInit();
            }
          }
          else {
            // show myTasksTab
            if (myTasksTab) {
              myTasksTab.setContentIsShownAfterInit();
            }
            if (savedSearchesTab) {
              savedSearchesTab.setContentIsShownAfterInit(false);
            }
          }

        }


        //after creating the lightweight tab container the content for the first tab is also created and shown
        this.tabContainerWidget.createTabs();
        this.connect(this.tabContainerWidget, "showTab", this._onShowTab);

        // insert the new drop down if there are saved searches
        if (showSavedSearchesTab) {
          var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
          var ssTabConfig = this._getTabConfig(id);
          if (ssTabConfig) {
            ssTabConfig.tab.appendChild(this._dropDown);
          }
          dojo.addClass(this._dropDown, "bpm-social-drop-down");
          dojo.addClass(this._dropDown, "bpm-social-drop-down-right");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Update the default page icon if the page content changed
       * 
       * @type {void}
       */
      _checkAndUpdateStartPageIcon: function _checkAndUpdateStartPageIcon_TasksContainer(ccPageConfig){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksContainer");
        // we can't use the ccPageConfig delivered by controller, because we can't identify the controller from it 
        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        var newPageConfig = listController.getPageSettings();
        this._pageHandler.checkAndUpdateStartPageIcon(newPageConfig, this.startPageButton);

        com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksContainer");
        return;
      },

      _setFocusToActiveTab: function _setFocusToActiveTab_TasksContainer(){
        if (this._tabHeaderDiv.style.display !== "none") {
          this.tabContainerWidget.focusActiveTab();
        }
        else {
          // no saved searches, so tab header is hidden, set focus to Work link
          var workLink = dojo.byId("processPortalWorkLink");
          if (workLink) {
            workLink.focus();
          }
        }
      },

      /**
       * Function getting invoked by TabContainer <br>
       * Returning false means that we don't reload all tabs on showTab
       * 
       * @returns {Boolean}
       */
      reloadAllTabs: function reloadAllTabs_TasksContainer(){
        return false;
      },


      /**
       * Reload the currently shown tab
       */
      reloadActiveTab: function reloadActiveTab_TasksContainer(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        listController.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _initTab: function _initTab_TasksContainer(div, tabConfiguration){

        var listController = this._getListController(tabConfiguration.id);
        if (listController) {
          listController.setConfig(tabConfiguration.custom);
          listController.createContent();
          div.appendChild(listController.domNode);
        }
      },

      _onShowTab: function _onShowTab_TasksContainer(tabConfiguration){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "New tab selected: " + tabConfiguration.tabName);
        if (this.tabContainerWidget.lastTab.id !== tabConfiguration.id) {
          // notify actual tab that it will be hidden
          this._onHideTab(this.tabContainerWidget.lastTab.id);
        }
        // refresh new tab
        var listController = this._getListController(tabConfiguration.id);
        listController.refresh();
        var newContentDiv = dojo.byId("tab_" + tabConfiguration.id + "__initTab"); // tab_SavedSearches__initTab
        if (newContentDiv) {
          dijit.setWaiState(newContentDiv, "labelledby", listController.listTitle.id);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _onHideTab: function _onHideTab_TasksContainer(tabId){
        com.ibm.bpm.social.util.common.Log.log(3, "tabId=" + tabId);
        if (tabId) {
          var listController = this._getListController(tabId);
          if (listController !== null) {
            listController.onHide();
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _getListController: function _getListController_TasksContainer(id){
        var listController = null;
        var filtered = dojo.filter(this._listControllers, function(item){
          return item.id === id;
        });
        if (filtered.length === 1) {
          listController = filtered[0].controller;
        }
        return listController;
      },

      /**
       * Check for available queues/saved searches and put them into a drop down
       * 
       * @param {domNode} anker Where to place the drop down
       * @returns {Boolean} false if no queues found, true otherwise
       */
      _createQueueDropDown: function _createQueueDropDown_TasksContainer(anker){
        this._queueDropDown = new com.ibm.bpm.social.widgets.ActionDropDownMenu({//UX: allow positioning of popup below the anchor in same place each time.
          leftClickToOpen: true
        });
        dojo.addClass(this._queueDropDown.domNode, "processPortalDropDownMenu");//UX: override icon height & width on .dijitMenuItemIcon
        dojo.addClass(this._queueDropDown.domNode, "tundra");//so the menu separator shows up
        dojo.style(this._queueDropDown.domNode, {
          "paddingLeft": "15px",
          "paddingRight": "15px"
        });//UX
        var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var taskQueuesList = taskQueues.getCustomQueues();
        var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
        var processQueuesList = processQueues.getCustomQueues();
        this._queues = [];
        var elementsExist = false;
        if (taskQueuesList && taskQueuesList.length > 0) {
          for ( var k = 0; k < taskQueuesList.length; k++) {
            var queue = taskQueuesList[k];
            queue.queueHandler = taskQueues;
            var menuItem = new dijit.MenuItem({
              label: queue.getDisplayName()
            });
            this._queueDropDown.addChild(menuItem);
            menuItem.containerNode._queueId = k;
            this._connectsMenu.push(this.connect(menuItem, "onClick", "_onQueueDropdownClick"));
            this._queues.push(queue);
          }
          elementsExist = true;
        }
        // insert a separator, of there tasks and instances saved searches
        var numOfTaskQueues = 0;
        if (taskQueuesList && taskQueuesList.length > 0 && processQueuesList && processQueuesList.length > 0) {
          this._queueDropDown.addChild(new dijit.MenuSeparator());
          numOfTaskQueues = taskQueuesList.length;
        }
        if (processQueuesList && processQueuesList.length > 0) {
          for (k = 0; k < processQueuesList.length; k++) {
            queue = processQueuesList[k];
            queue.queueHandler = processQueues;
            menuItem = new dijit.MenuItem({
              label: queue.getDisplayName()
            });
            this._queueDropDown.addChild(menuItem);
            menuItem.containerNode._queueId = k + numOfTaskQueues;
            this.connect(menuItem, "onClick", "_onQueueDropdownClick");
            this._queues.push(queue);
          }
          elementsExist = true;
        }

        // insert the drop down only, if it exists
        if (elementsExist) {
          // <div dojoAttachPoint="menuButton" class="bpm-social-drop-down" tabIndex="0" style="display:none" waiRole="button"><div class="bpm-social-drop-down-text">&#9660;</div></div>
          anker.setAttribute("role", "button");
          anker.setAttribute("tabindex", 0);
          var span = document.createElement("span");
          dojo.addClass(span, "bpm-social-drop-down-text");
          span.innerHTML = "&#9660;";
          anker.appendChild(span);
          this.connect(anker, "onkeypress", function(e){
            if (e.keyCode == dojo.keys.ENTER) {
              this._queueDropDown._scheduleOpen(e.target);
            }
          });
          this._queueDropDown.bindDomNode(anker);
        }
        return elementsExist;
      },


      /**
       * Store the Work tab as default start page
       */
      _onStartPageButtonClick: function _onStartPageButtonClick_TasksContainer(evt){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var listController = this._getListController(this.tabContainerWidget.lastTab.id);
        var newPageConfig = listController.getPageSettings();
        this._pageHandler.setStartPage(newPageConfig, this.startPageButton);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Get the TabConfig with id
       * 
       * @return {TabConfiguration}
       */
      _getTabConfig: function _getTabConfig_TasksContainer(id){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var tabInfo = null;
        if (this.tabContainerWidget && this.tabContainerWidget.tabInformations) {
          var tabInfos = dojo.filter(this.tabContainerWidget.tabInformations, function(item){
            return item.id === id;
          });
          if (tabInfos && tabInfos[0]) {
            tabInfo = tabInfos[0];
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return tabInfo;
      },


      /**
       * Callback when user selected a saved search from the drop down
       * 
       * @type {void}
       */
      _onQueueDropdownClick: function _onQueueDropdownClick_TasksContainer(domEvent){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        var tabConfig = this._getTabConfig(id);
        if (tabConfig) {

          var target = null;
          if (dojo.isIE == 8) {
            target = domEvent.srcElement;
          }
          else {
            target = domEvent.target;
          }

          // for keyboard access, we get the the row of the html table instead of the menu item label
          if (dojo.hasClass(target, "dijitMenuItem")) {
            target = dojo.query(".dijitMenuItemLabel", target)[0];
          }

          var queue = this._queues[target._queueId]; // == menuItem.containerNode._queueId
          this._setCustomViewQueue(queue);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

	_setCustomViewQueue: function _setCustomViewQueue(queue) {
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var id = com.ibm.bpm.social.widgets.task.config.ViewConfig.ID_SAVED_SEARCHES;
        var tabConfig = this._getTabConfig(id);
		var controllerConfig = tabConfig.custom;
		controllerConfig.queue = queue;
		// reinit filter
		controllerConfig.filterConfigs =
			[com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews()];
		var controller = this._getListController(id);
		controller.setConfig(controllerConfig);
		//check if same tab is selected, if yes , refresh is done in _onShowTab via showTab callback
		if (tabConfig.id != this.tabContainerWidget.lastTab.id) {
			this.tabContainerWidget.selectTab(tabConfig, tabConfig.tab);
		} else {
			controller.refresh();
		}
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _getQueue: function _getQueue_TasksContainer(queueName, includeProcessQueues){
        // retrieve all (Saved Searches)
        var taskQueues = com.ibm.bpm.social.widgets.task.util.TaskQueues.getInstance();
        var q = taskQueues.getQueue(queueName); // returns null on queue not found
        var queue = q;

        if (q) {
          // create a new queue based on the queue returned from taskQueues
          queue = new com.ibm.bpm.social.widgets.task.util.Queue(q);
          queue.queueHandler = taskQueues;
        }
        else {
          var processQueues = com.ibm.bpm.social.widgets.task.util.ProcessQueues.getInstance();
          q = processQueues.getQueue(queueName); // returns null on queue not found
          if (q) {
            // create a new queue based on the queue returned from processQueues
            queue = new com.ibm.bpm.social.widgets.task.util.Queue(q);
            queue.queueHandler = processQueues;
          }
        }
        return queue;
      },

      _cleanupMenu: function _cleanupMenu_TasksContainer(){
        if (this._connectsMenu) {
          for ( var k = 0; k < this._connectsMenu.length; k++) {
            dojo.disconnect(this._connectsMenu[k]);
          }
          this._connectsMenu = [];
        }
        if (this._queueDropDown) {
          this._queueDropDown.destroyRecursive();
          this._queueDropDown.destroy();
        }
      },

      destroy: function destroy_TasksController(){

        for ( var i = 0; i < this._subscribedChannels.length; i++) {
          if (this._subscribedChannels[i]) {
            dojo.unsubscribe(this._subscribedChannels[i]);
          }
        }

        this._cleanupMenu();
        if (this._listControllers) {
          for ( var k = 0; k < this._listControllers.length; k++) {
            try {
              this._listControllers[k].controller.destroy();
            }
            catch (e) {
              com.ibm.bpm.social.util.common.Log.logError(2, "destroy_TasksController", e);
            }
          }
          this._listControllers = null;
        }
        this.inherited(arguments);
      }


    });
