// FilterConfig.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.config.FilterConfig");

dojo.require("com.ibm.bpm.social.widgets.task.list.Category");

/**
 * Filter Config: Settings about which items and attributes to display or filter out and
 */
dojo.declare("com.ibm.bpm.social.widgets.task.config.FilterConfig", null, {
  id: null,
  titleKey: null,
  interactionFilter: null,
  queryFilter: null,
  categories: null,
  displayedProperties: null,

  /**
   * a list of sort properties as expected by the entity query as properties in queue.item.properties
   * {property: "PROPERTY_KEY", isSort: true/false, sortDir: "ASC/DESC"}
   */
  sortProperties: null,

  constructor: function constructor_FilterConfig(params){
    if (params) {
      dojo.safeMixin(this, params);
    }
  }
});

com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS = "customViews";
com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS = "completedTasks";
com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS = "openTasks";


com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigOpenTasks = function(){

  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS,
    titleKey: "task-list-filter-open",
    interactionFilter: "ASSESS_AND_WORK_ON",
    queryFilter: null,
    categories: [com.ibm.bpm.social.widgets.task.list.Category.OVERDUE,
                 com.ibm.bpm.social.widgets.task.list.Category.AT_RISK,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK,
                 com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER],
    /* Caution: if sort properties are changed the sort algorithm for delta refresh may need to be changed!!! */
    sortProperties: [{
                       property: "IS_AT_RISK",
                       isSort: true,
                       sortDir: "DESC"
                     },
                     {
                       property: "DUE",
                       isSort: true,
                       sortDir: "ASC"
                     },
                     {
                       property: "TKIID",
                       isSort: true,
                       sortDir: "ASC"
                     }],

    selectedProperties: ["TAD_DISPLAY_NAME",
                         "PRIORITY",
                         "DUE",
                         "PI_DISPLAY_NAME",
                         "ASSIGNED_TO_ROLE_DISPLAY_NAME",
                         "NAME",
                         "PI_PIID",
                         "PI_NAME",
                         "OWNER",
                         "STATE",
                         "IS_AT_RISK",
                         "TKIID"],

    displayedProperties: ["TAD_DISPLAY_NAME",
                          "PRIORITY",
                          "DUE",
                          "PI_DISPLAY_NAME",
                          "ASSIGNED_TO_ROLE_DISPLAY_NAME"]
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCompletedTasks = function(){
  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS,
    titleKey: "task-list-filter-completed",
    interactionFilter: "CHECK_COMPLETED",
    queryFilter: null, // TODO
    categories: null,
    /* Caution: if sort properties are changed the sort algorithm for delta refresh may need to be changed!!! */
    sortProperties: [{
      property: "COMPLETED",
      isSort: true,
      sortDir: "DESC"
    }],
    selectedProperties: ["TAD_DISPLAY_NAME",
                         "PI_DISPLAY_NAME",
                         "COMPLETED",
                         "NAME",
                         "PI_PIID",
                         "PI_NAME",
                         "OWNER",
                         "STATE",
                         "TKIID"],

    displayedProperties: ["TAD_DISPLAY_NAME",
                          "PI_DISPLAY_NAME",
                          "COMPLETED"]
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews = function(){
  return new com.ibm.bpm.social.widgets.task.config.FilterConfig({
    id: com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS,
    titleKey: "",
    interactionFilter: null,
    queryFilter: null,
    categories: null,
    sortProperties: [],
    displayedProperties: []
  });
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigById = function(id){
  if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCustomViews();
  }
  else if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigCompletedTasks();
  }
  else if (id == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
    return com.ibm.bpm.social.widgets.task.config.FilterConfig.getFilterConfigOpenTasks();
  }
};

com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING =
// maps query columns to details properties
[{
   column: "TKIID",
   property: "tkiid"
 },
 {
   column: "TASK.TKIID",
   property: "tkiid"
 },
 {
   column: "NAME",
   property: "name"
 },
 {
   column: "TAD_DISPLAY_NAME",
   property: "displayName"
 },
 {
   column: "PRIORITY",
   property: "priority"
 },
 {
   column: "IS_AT_RISK",
   property: "isAtRisk"
 },
 {
   column: "DUE",
   property: "dueTime"
 },
 {
   column: "COMPLETED",
   property: "completionTime"
 },
 {
   column: "OWNER",
   property: "owner"
 },
 {
   column: "STATE",
   property: "state"
 },
 {
   column: "PI_PIID",
   property: "piid"
 },
 {
   column: "PI_NAME",
   property: "processInstanceName"
 },
 {
   column: "PI_DISPLAY_NAME",
   property: "processInstanceName"
 }];

/**
 * 
 * @param items an array of task details as returned from a push event
 * @returns an aray of task details as returned from a query
 */
com.ibm.bpm.social.widgets.task.config.FilterConfig.mapTasks = function(items){
  var mappedItems = new Array();
  if (items) {
    for ( var i = 0; i < items.length; i++) {
      var item = items[i];
      if (item) {
        var mappedItem = {};
        for ( var m = 0; m < com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING.length; m++) {
          var property = com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING[m].property;
          if (item[property] !== undefined) {
            var column = com.ibm.bpm.social.widgets.task.config.FilterConfig.TASK_PROPERTY_MAPPING[m].column;
            mappedItem[column] = item[property];
          }
        }
        if (item.assignedToType == "user" && item.assignedTo) {
          mappedItem.OWNER = item.assignedTo;
        }
        else if (item.assignedToType == "group") {
          if (item.assignedToDisplayName) {
            mappedItem.ASSIGNED_TO_ROLE_DISPLAY_NAME = item.assignedToDisplayName;
          }
          if (item.assignedTo) {
            mappedItem.ASSIGNED_TO_ROLE = item.assignedTo;
          }
        }
        mappedItems.push(mappedItem);
      }
    }
  }
  return mappedItems;
};
