// CoachRenderer.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.form.CoachRenderer");

dojo.require("com.ibm.bpm.social.widgets.task.form.Renderer");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("dijit.form.Button");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

/**
 * All logic that is specific to the iWidget implementation or the Process Portal implementation is part of
 * this class which is added as mixin (by adding it to the constructor of the CoachRenderer).
 * 
 * This should help to keep the implementation of
 * 
 * com.ibm.bpc.widget.editor.form.CoachRenderer
 * 
 * and
 * 
 * com.ibm.bpm.social.widgets.task.form.CoachRenderer
 * 
 * in sync.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.form.CoachRendererProcessPortalHelper", null, {

  MODE_INIT: com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT,
  MODE_EDIT: com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT,

  constructor: function(parameters){
    // Helper for logging.
    this.logHelper = com.ibm.bpm.social.util.common.Log;
    // Is there already a callback object?        
    if (typeof window["BPM_CoachCallback"] != "object") {
      //
      // The used correlation token is either the taskId or generated correlationToken.
      // 
      window["BPM_CoachCallback"] = com.ibm.bpm.social.widgets.task.form.RendererPool.getInstance();
    }
  },
  setupRestHandler: function(){
    // Set the rest urls
    this.restHandler = new com.ibm.bpm.social.util.common.TaskRestHandler();
  },

  _getTaskState: function(){
    var state = null;
    var taskObject = null;
    this.restHandler.getTask(this.task.tkiid, function(responseObject, ioArgs){
      taskObject = responseObject;
    }, function(data, ioArgs){
    // Problems retrieving the task...
    }, false, null, this.task.systemID);
    if (taskObject !== null) {
      state = taskObject.state;
    }
    return state;
  },

  _log: function log_CoachRenderer(level, message){
    this.logHelper.log(level, message);
  },

  /**
   * calls exitCollaboration on coach
   * 
   * @returns {void} nothing
   */
  exitTaskCollaboration: function exitTaskCollaboration_CoachRenderer(){
    try {
      //exit task collaboration
      if (this.frame && this.frame.contentWindow && this.frame.contentWindow["exitCollaboration"]) {
        this.frame.contentWindow["exitCollaboration"](true);
      }
    }
    catch (e) {
      // In case there are any security violations due to different domains
    }
  }

});

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.form.CoachRenderer",
    [dijit._Widget,
     dijit._Templated,
     com.ibm.bpm.social.widgets.task.form.Renderer,
     com.ibm.bpm.social.widgets.task.form.CoachRendererProcessPortalHelper],
    {

      templateString: '<div dojoAttachPoint="root">'
        + '<iframe dojoAttachPoint="frame" style="border:0px;width:100%;height:100%;min-width:0px; min-height:0px;overflow: auto" frameborder="0"></iframe>'
        + '</div>',

      isNGCoach: false,

      /**
       * Constructor that sets all the parameter that are required according to the interface description.
       * 
       * @param parameters
       * @returns {void} nothing
       */
      constructor: function(parameters){
        this.iContext = parameters.iContext;
        this.task = parameters.task;
        this.taskTemplate = parameters.taskTemplate;
        this.mode = parameters.context;
        this.clientType = parameters.clientType;
        this.clientSettings = parameters.clientSettings;
        this.parentTKIID = parameters.parentTKIID;
        this.setupRestHandler();
        this._isTryCoachReload = false;
        this.objectId = this.task ? this.task.tkiid : this.taskTemplate.tktid;
        this._isDialog = parameters.dialog;
      },

      _createCallbackAdapter: function _createCallbackAdapter_CoachRenderer(){
        this.logHelper.logEntry(2);
        var callbackObject = window["BPM_CoachCallback"];
        this.connect(callbackObject, "onInitialized", "_onInitialized");
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.connect(callbackObject, "onInitialized", "_onInitialized4Ipad");
        }
        this.connect(callbackObject, "onCompleted", "_onCompleted");
        this.connect(callbackObject, "onError", "_onError");
        this.connect(callbackObject, "onSuspended", "_onSuspended");

        // this is a workaround for IE to handle the iframe's onload event correctly

        if (this.frame.attachEvent) {
          this._onResourceLoadedImplFn = dojo.hitch(this, "_onResourceLoadedImpl");
          this.frame.attachEvent("onload", this._onResourceLoadedImplFn);
        }
        else {
          this.connect(this.frame, "onload", "_onResourceLoadedImpl");
        }

        this.logHelper.logExit(2);
      },

      _onResourceLoadedImpl: function _onResourceLoadedImpl_CoachRenderer(){
        var source = "";
        try {
          source = this.frame.contentWindow.location.pathname;
        }
        catch (e) {
          // Security violance due to Same Origin Policy for iFrame should not 
          // happen as for Process Portal, the server and port should be the same.
          // For Business Space a proxy is used.
          source = "";
        }
        this.onResourceLoaded(source);
      },

      /**
       * Which resource has been loaded is unknown. {void} nothing
       */
      onResourceLoaded: function onResourceLoaded_CoachRenderer(url){},

      _onInitialized: function _onInitialized_CoachRenderer(frame, taskID, isOldCoach){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onInitialized Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          var wasOldCoach = !this.isNGCoach;
          this.isNGCoach = !isOldCoach;
          if (typeof taskID === "string") {
            // Only heritage coaches (=old coaches) do not notify about auto flow to next task. 
            if (taskID.substr(0, 1) == "t" && wasOldCoach) {
              // New taskID encountered => autoflow for old-style coaches
              // Remove the prefix
              taskID = taskID.substring(1);
              if (taskID != this.objectId) {
                var originalID = this.objectId;
                this.objectId = taskID;
                this.onSubmitted(originalID, taskID);
              }
            }
            else if (taskID.substr(0, 1) == "p") {
              this._log(2, "Initiating task " + taskID + " encountered");
            }
          }
          this.onLoaded(this.objectId, !isOldCoach);
        }
        this._log(2, "CoachRenderer._onInitialized Exit " + this.objectId);
      },

      _onCompleted: function _onCompleted_CoachRenderer(frame, nextTkiid){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onCompleted Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          
          // Make sure that the keyboard is closed after the coach was completed.
          if (com.ibm.bpm.social.util.common.Util.isIPad) {
        	frame.document.activeElement.blur();
          }
          
          if (typeof nextTkiid == "string") {
            if (nextTkiid.substr(0, 1) == "t") {
              // Remove the prefix
              nextTkiid = nextTkiid.substring(1);
            }
            var originalID = this.objectId;
            this.objectId = nextTkiid;
            this.onSubmitted(originalID, nextTkiid);
          }
          else {
            if (this.mode === this.MODE_INIT) {
              // A human service was started, as the human service does not get persisted and 
              // we do not know anything about the implementation we just want to close 
              // the tab without firing any event
              this.onCanceled(this.objectId);
            }
            else {
              // Do we need to check the state of the task?
              // Is it possible to cancel a coach using a submit button of the form?
              this.onSubmitted(this.objectId);
            }
          }
        }
        this._log(2, "CoachRenderer._onCompleted Exit " + this.objectId);
      },

      _onError: function _onError_CoachRenderer(frame, message){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onError Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          if (message === "tm_task_closed") {
            this.onErrorOccurred("task-row-details-error-not-available");
          }
          else if (message === "tm_task_not_owned") {
            this.onErrorOccurred("user_not_authorized_for_task");
          }
          else {
            this.onErrorOccurred(message);
          }
        }
        this._log(2, "CoachRenderer._onError Exit " + this.objectId);
      },

      _onSuspended: function _onSuspended_CoachRenderer(frame){
        // Do not use logEntry since it logs the arguments automatically
        this._log(2, "CoachRenderer._onSuspended Entry " + this.objectId);
        if (frame == this.frame.contentWindow) {
          this._log(2, "Iframe matches");
          this.onCanceled();
        }
        this._log(2, "CoachRenderer._onSuspended Exit " + this.objectId);
      },

      /*
       * Because there are problems with bounded iFrames on iPad, the coach will 
       * be displayed in full size and the viewport will be changed so that the user
       * can scroll the entire page to see the complete coach.
       */
      _onInitialized4Ipad: function _onInitialized4Ipad_CoachRenderer(frame, taskID, isOldCoach){
        if (frame == this.frame.contentWindow) {
          var viewport = document.querySelector("meta[name=viewport]");
        	 
      	  // In case of a coach flow, this method is called for each coach step. To assure that the 
          // previous viewport calculation does not influence the following, the original settings are 
          // restored.
          if (this.processPortalVPSettings) {
   	 		// Make sure all calculated width and height settings are removed...
   	 		viewport.setAttribute('content', 'width = device-width, height = device-height');
   	 		//... and then return to the old settings
   	 		viewport.setAttribute('content', this.processPortalVPSettings);
   	 	  } else {
   	 		// Store the process portal viewport settings to restore it sometimes 
            this.processPortalVPSettings = viewport.getAttribute("content");  			  
   		  }	
          	
          var frameWidth = frame.document.body.clientWidth;
          var frameHeight = frame.document.body.clientHeight;          

  		  var newWidth = "device-width";
          // 394 = 314 (sidebar) + 2x35 (margins) + 10 (random pixels)
  		  var frameWidthDelta = 394;
  		  if (frameWidth > window.innerWidth - frameWidthDelta) {
  			  // pixels are hardcoded, because out of some reasons dynamic calculation didn't work
  			  // properly here
              newWidth = frameWidth + frameWidthDelta;
            }

            var newHeight = "device-height";
            // 219 = 56 (top bar) + 55 (coach header) + 28 (bottom bar) + 2x35 (margins) + 10 (random pixels)
            var frameHeightDelta = 219;
            if (frameHeight > window.innerHeight - frameHeightDelta) {
			  // pixels are hardcoded, because out of some reasons dynamic calculation didn't work
			  // properly here
              newHeight = frameHeight + frameHeightDelta;
            }

            viewport.setAttribute('content', 'width = ' + newWidth + ', height = ' + newHeight +
              ', initial-scale = 1.0, minimum-scale = 1.0');

            // Scroll to the right border, if the width was changed in rtl mode.
            if (newWidth !== "device-width" && document.body.dir === "rtl") {
              window.scrollTo(newWidth, 0);
            }
        }
      },

      /**
       * Creates the form. Beware that the elem is not part of the document yet.
       * 
       * @param elem - anchor element where the form is added.
       * 
       * @returns {void} nothing
       */
      load: function(elem /* DOM element */){
        elem.appendChild(this.domNode);
        this._createCallbackAdapter();
        // Retrieve the url and set it on the iframe
        var url = this.clientSettings.url;
        // Get the coach debug trace settings from the cookie and set them on the url
        var coachDebug = dojo.cookie("com_ibm_bpm_coach_debug_logger");
        if (!!coachDebug) {
          url += coachDebug;
        }

        // A11y: set the title of the iFrame
        if (this.task) {
          this.frame.title = this.task.displayName;
        }
        else if (this.taskTemplate) {
          this.frame.title = this.taskTemplate.displayName;
        }

        this.frame.src = url;
      },

      /**
       * Resizes the form in order to allow sophisticated scrolling.
       * 
       * @param dim
       * @returns {void} nothing
       */
      resize: function(dim /*{width:1,height:1} */){
        //Blocked due to mashups defect 18110:
        //TODO: handle case when widget is maximized
        //TODO: handle case where Coach input requires more height than the default
        //
        // Calculate width, height. Math.max makes sure that the numbers are positive.
        //

        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          // Because the viewport matches, auto width is ok here.
          this.root.style.width = "auto";
        }
        else {
          var frameWidth = "100%";
          if (typeof dim.width == "number" && isNaN(dim.width) !== true) {
            frameWidth = Math.max(0, dim.width) + "px";
          }
          this.frame.style.width = frameWidth;
        }

        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          // Because the viewport matches, auto height is ok here.
          this.root.style.height = "auto";
        }
        else {
          var frameHeight = "100%";
          if (typeof dim.height == "number" && isNaN(dim.height) !== true) {
            frameHeight = Math.max(0, dim.height) + "px";
          }
          this.frame.style.height = frameHeight;
        }
      },

      /**
       * Destroys the widget.
       * 
       * 
       * @returns {void} nothing
       */
      unload: function unload_CoachRenderer(){
        if (com.ibm.bpm.social.util.common.Util.isIPad && this.processPortalVPSettings) {
          var viewport = document.querySelector("meta[name=viewport]");
          // Make sure all calculated width and height settings are removed...
          viewport.setAttribute('content', 'width = device-width, height = device-height');
          //... and then return to the old settings
          viewport.setAttribute('content', this.processPortalVPSettings);
        }

        this.destroyRecursive();
      },

      /**
       * Shows the form.
       * 
       * @returns {void} nothing
       */
      show: function(){
        this.root.style.visibility = "visible";
      },

      /**
       * Hides the form.
       * 
       * @returns {void} nothing
       */
      hide: function(){
        this.root.style.visibility = "hidden";
      },

      /**
       * Returns whether the form is valid.
       * 
       * @returns boolean - true if valid.
       */
      isValid: function(){
        return true;
      },

      /**
       * Disables any controls.
       * 
       * @param message
       * @param details
       * @param action
       * @returns {void} nothing
       */
      onErrorOccurred: function(message, description, details, action){

      },

      /**
       * Save is not supported
       * 
       * @returns {boolean} false
       */
      isSaveSupported: function(){
        return false;
      },

      /**
       * Implementation of save in "edit" mode.
       * 
       * @returns {void} nothing
       */
      save: function(){
      // Not supported
      },


      /**
       * Called by the Task Information Widget if the task can be started in case of init mode or if the task
       * can be completed in case of edit mode.
       * 
       * @returns {void} nothing
       */
      enableActions: function(){

      },

      /**
       * Called by the Task Information Widget if the task cannot be started in case of init mode or if the
       * task cannot be completed in case of edit mode. This might be the case if the task is suspended or the
       * waiting for subtasks.
       * 
       * @returns {void} nothing
       */
      disableActions: function(){

      },

      /**
       * ************************** Technical methods that are not part of the interface
       * ************************
       */

      /**
       * Destroys all the widgets.
       * 
       * @returns {void} nothing
       */
      destroy: function(){
        this.exitTaskCollaboration();
        this.inherited("destroy", arguments);
      }

    });

// 
// Can be removed in the context of the pending tech-debt items.
//
dojo.declare("com.ibm.bpm.social.widgets.task.form.RendererPool", null, {
  unkownToken: 0,
  knownToken: {},
  handle: null,

  constructor: function(){
    // Establish listener for onMessage events for the HTML5 postMessage events.
	// *****  When cross document messaging is turned off this connect won't work. In that
	//        situation, the _dispatch function will be called directly by com.ibm.bpm.socialportal.onMessage() which
	//        is defined in modules\bpmsocial\src\main\webapp\jsp\index.jsp  ******
    this.handle = dojo.connect(window, "message", this, "_dispatch");
  },

  destroy: function(){
    if (this.handle) {
      dojo.disconnect(this.handle);
    }
  },

  _dispatch: function _dispatch_RendererPool(event){
    var payloadLiteral = event.data;
    try {
      //
      // Only take the event seriously if the payload is json and
      // there is a property "name" of type string.
      // Otherwise this is probably some other event posted using postMessage in 
      // this instance of the browser.
      //
      var payload = dojo.fromJson(payloadLiteral);
      if (typeof payload.name == "string") {
        var eventType = payload["name"];
        var taskID = null;
        if (typeof payload["taskID"] === "string") {
          // taskID is optional
          taskID = payload["taskID"];
        }
        var broadcastingWindow = event.source;
        var firstParameter = null;
        var secondParameter = null;
        if (payload.parameters && payload.parameters.length > 0 && typeof payload.parameters[0] === "string") {
          if (payload.parameters[0].length > 0) {
            firstParameter = payload.parameters[0];
          }
          if (payload.parameters[1] && typeof payload.parameters[1] === "string") {
            secondParameter = payload.parameters[1];
          }
        }
        switch (eventType) {
          case "onInitialized" :
            if (firstParameter == "oldCoach") {
              this.onInitialized(broadcastingWindow, taskID, true /* old Coach */);
            }
            else {
              this.onInitialized(broadcastingWindow, taskID, false /* new Coach */);
            }
            break;
          case "onCompleted" :
            if (firstParameter) {
              // autoflow to next tkiid
              var nextTkiid = firstParameter;
              this.onCompleted(broadcastingWindow, nextTkiid);
            }
            else {
              this.onCompleted(broadcastingWindow, null);
            }
            break;
          case "onError" :
            if (secondParameter !== null) {
              this.onError(broadcastingWindow, secondParameter);
            }
            else {
              this.onError(broadcastingWindow, firstParameter);
            }
            break;
          case "onSuspended" :
            this.onSuspended(broadcastingWindow);
            break;
          default:
            this._log(2, "Ignoring unknown event type: " + eventType);
            break;
        }
      }
    }
    catch (e) {
      // Ignore the event. There might be other events using postMessage.
      this._log(2, "Ignoring non-JSON event: " + payloadLiteral);
    }
  },

  onInitialized: function(/* window */broadcastingWindow,/* string */taskID, /* boolean */isOldCoach){},

  onCompleted: function(/* window */broadcastingWindow,/* string */newTkiid){},

  onError: function(/* window */broadcastingWindow,/* string */message){},

  onSuspended: function(/* window */broadcastingWindow){},

  _log: function(level, message){
    com.ibm.bpm.social.util.common.Log.log(level, message);
  },

  statics: {
    _instance: null,
    _getInstance: function(){
      if (this._instance === null) {
        this._instance = new com.ibm.bpm.social.widgets.task.form.RendererPool();
      }
      return this._instance;
    }
  }
});

com.ibm.bpm.social.widgets.task.form.RendererPool.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.form.RendererPool.prototype.statics._getInstance();
};
