/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2011, 2012. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget");

dojo.require("com.ibm.bpm.social.widgets.ActionDropDownHelper");
dojo.require("com.ibm.bpm.social.widgets.task.util.ClaimDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");
dojo.require("com.ibm.bpm.social.widgets.dialogs.InvitationDialog");
dojo.require("com.ibm.bpm.social.widgets.task.form.CoachRenderer");
dojo.require("com.ibm.bpm.social.widgets.task.util.PropertyFormatter");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.portal.Topics");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date.locale");


dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: dojo.cache("com.ibm.bpm.social.widgets.task.form", "templates/TaskCompletionWidget.html"),
      widgetsInTemplate: true,

      //
      // dojoAttachPoints out of template:
      //
      // 0) The header area.
      // 2) The full size control
      // 3) Subject area
      // 4) Menu anker
      // 5) The due date section
      // 6) The actual due date control
      // 7) Anker for coach container
      header: null,
      fullSizeControl: null,
      subjectArea: null,
      menuAnker: null,
      dueDateSection: null,
      dueDateArea: null,
      coachContainerAnker: null,
	  fullSize: false,
      //
      // Properties referenced in the template:
      // (Which are set in the constructor)
      //
      due_date_label: "",
      back_link_to_work: "",
      maximize_label: "",
      restore_label: "",
      loading_label: "",

      // State of the fullSizeControl 
      // For template fullSize is always true therefore the user preference (for this session) need to be stored separately
      fullSize: false,
      fullSizeUserSelection: false,

      // The coachRenderer that listens to the coach.     
      coachRenderer: null,

      // Utility for making calls against the rest api.
      restHandler: null,
      // If the header has been updated already, it stays there during an error message.
      headerUpdated: false,

      //
      // Set of properties which need to be kept between the asynchronous calls to 
      // a) fetch the clientSetttings
      // b) fetch the actions of the task (Authorization verification)
      // which are cleared with the _clear method.
      //
      _id: "", // Used for tracing purposes
      _task: null,
      _taskTemplate: null,
      _clientType: null,
      _clientSettings: null,
      _actions: null,
      _ngCoach: false,

      // For resource allocation
      _modifyTaskHandler: null,

      //
      // Is the widget displayed in dialog mode?
      // Default is no dialog.
      //
      _isDialog: false,
      /**
       * The TaskCompletionWidget can be used for several coaches. (Any state object are reseted at the end)
       * Assumption: The task or task template could be retrieved with HTTP code 200 (Otherwise
       * TaskCompletionView does the error handling)
       * 
       * <ol>
       * <li>loadCoach(task) or loadCoach4Template(taskDefinition) => triggers the rest calls for
       * clientSettings and actions</li>
       * <li>onLoaded() => Coach is loaded </li>
       * <li>onCompleted(newTkiid) => If newTkiid == null => next page, newTkiid != null => updateTask
       * (nextTkiid in case of autoflow) </li>
       * </ol>
       * 
       * Alternate Flow and Error case:
       * <ol>
       * <li>task is already completed => onTaskAlreadyCompleted </li>
       * <li>an error occurred => for old-style coaches => onCompleted for next generation coaches =>
       * onCanceled stay on page</li>
       * </ol>
       * 
       * Remark: The different behaviour for old-style and next-gen coaches is due to the error message in the
       * jsps for old-style coaches which mention that the task list is shown afterwards.
       * 
       * The following error situations can happen in course of the processing:
       * <ul>
       * <li>REST call does not return 200 (OK) => general error message
       * (server_not_available,server_not_available_action) </li>
       * <li>REST calls ok, but _isTaskAndUserFit == false => missing authorization =>
       * user_not_authorized_for_task </li>
       * <li>(Might be catched before) Old style coach is assigned to different person =>
       * _closeCoachDueToError called => user_not_authorized_for_task </li>
       * </ul>
       * 
       */
      constructor: function constructor_TaskCompletionWidget(params){
        // Are we used in dialog mode?
        if (typeof params.dialog === "boolean") {
          // If yes, the header will have display:hidden and no additional popups are shown.
          // Neither will there be any action drop down.
          this._isDialog = params.dialog;
        }
        // PII is loaded before they are used in _Widget.buildRendering.
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        //
        // Set labels used in the template.
        //
        this.due_date_label = this._nlsResources["task_header_due"];
        this.back_link_to_work = this._nlsResources["back_link_to_work"];
        this.maximize_label = this._nlsResources["task_completion_maximize_coach"];
        this.restore_label = this._nlsResources["task_completion_restore_coach"];
        this.loading_label = this._nlsResources["KEY_Loading"];
        //
        // Lookup helper for dropDown menu and rest handler.
        //
        this._dropDownHelper = new com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();
        this.restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
      },

      postCreate: function(){

        if (this._isDialog) {
          // Do not show the header
          this.header.style.display = "none";
          // Set the correct style
          dojo.removeClass(this.domNode, "bpm-task-coach-widget");
          dojo.addClass(this.domNode, "bpm-task-coach-popup");
        }

        // Do not show the maximize button, if device is an iPad, because in this case the
        // coach is always shown in full size (due to problems with bounded iFrames).
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.fullSizeControl.style.display = "none";
        }

        // Default is fullSize == false
        this.fullSizeControl.title = this._nlsResources["fullsize-icon-collapsed"];
        this.fullSizeControlA11y.innerHTML = this.maximize_label;

        // retrieve full size controls from banner
        this.fullSizeRestore = dojo.byId("processPortalFullScreenCoachIcon");
        this.fullSizeRestore.title = this._nlsResources["fullsize-icon-expanded"];
        this.fullSizeRestoreA11y = dojo.byId("processPortalFullScreenCoachIconA11y");
        this.fullSizeRestoreA11y.innerHTML = this.restore_label;
        dijit.setWaiRole(this.fullSizeRestoreA11y, "button");
        dijit.setWaiState(this.fullSizeRestoreA11y, "label", this.restore_label);
        this.fullSizeSubject = dojo.byId("processPortalFullScreenCoachSubject");
        this.fullSizeDue = dojo.byId("processPortalFullScreenCoachDue");
        this.fullSizeDueLabel = dojo.byId("processPortalFullScreenCoachDueLabel");
        this.fullSizeDueLabel.innerHTML = this.due_date_label + " ";
        this.fullSizeDueDate = dojo.byId("processPortalFullScreenCoachDueDate");

		// defect 135784
		dojo.removeClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');
		
        //
        // Add listener to the full size icon
        //
        this.connect(this.fullSizeControl, "click", function(){
          this.expandToFullSize();
          this.fullSizeUserSelection = this.fullSize;
        });
        this.connect(this.fullSizeControl, "keypress", function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this.expandToFullSize();
            this.fullSizeUserSelection = this.fullSize;
          }
        });
        //
        // Add listener to the restore icon
        //
        this.connect(this.fullSizeRestore, "click", function(){
          this.collapseToNormalSize();
          this.fullSizeUserSelection = this.fullSize;
        });
        this.connect(this.fullSizeRestore, "keypress", function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this.collapseToNormalSize();
            this.fullSizeUserSelection = this.fullSize;
          }
        });
      },

      _showLoadingSection: function _showLoadingSection_TaskCompletionWidget(){
        if (this.loadingTimeOut != null) {
          clearTimeout(this.loadingTimeOut);
          this.loadingTimeOut = null;
        }
        // Only show after some time the loading indicator because it might irritate the user
        // if the coach loads quickly anyway.
        this.loadingTimeOut = setTimeout(dojo.hitch(this, this._showLoadingSectionImpl), 1000);
      },

      _showLoadingSectionImpl: function _showLoadingSectionImpl_TaskCompletionWidget(){
        this.loadingSection.style.display = "block";
        this.loadingTimeOut = null;
      },

      _clearLoadingSection: function _clearLoadingSection_TaskCompletionWidget(){
        if (this._task) {
          this._createActionDropDown(this._actions);
        }
        if (this.loadingTimeOut != null) {
          clearTimeout(this.loadingTimeOut);
          this.loadingTimeOut = null;
        }
        this.loadingSection.style.display = "none";
      },

      show: function(){
        this.domNode.style.visibility = "";
      },

      hide: function(){
        this.domNode.style.visibility = "hidden";
      },

      expandToFullSize: function(){
        // Publish a maximize request which the controller is listing to.
        dojo.publish(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", [{}]);
        // hide header
        this.header.style.display = "none";

        this._modifyBanner();

        this.fullSize = true;
        this.resize();

		// defect 135784
		dojo.addClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');

      },

      collapseToNormalSize: function(){
        // Publish a restore request which the controller is listing to.
        // Do not show a side bar for task templates.
        if (!this._taskTemplate) {
          dojo.publish(com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT + "-TaskCompletionView", [{}]);
        }
        // show header
        this.header.style.display = "block";

        this._restoreBanner();
        this.fullSize = false;
        this.resize();

		// defect 135784
		dojo.removeClass(dojo.byId('processPortalCollaborationBanner'),'fullSize');
      },

      _modifyBanner: function _modifyBanner_TaskCompletionWidget(){
        dojo.style(dojo.byId("processPortalTabBar"), "display", "none");
        dojo.style(dojo.byId("processPortalUser"), "display", "none");
        dojo.style(dojo.byId("processPortalFullScreenCoach"), "display", "block");
      },

      _restoreBanner: function _restoreBanner_TaskCompletionWidget(){
        dojo.style(dojo.byId("processPortalFullScreenCoach"), "display", "none");
        dojo.style(dojo.byId("processPortalTabBar"), "display", "block");
        dojo.style(dojo.byId("processPortalUser"), "display", "block");
      },

      _initFullSizeMode: function _initFullSizeControl(isTemplate /* boolean */){
        if (this._isDialog || com.ibm.bpm.social.util.common.Util.isIPad) {
          // Assumption the dialog contains its own instance of this class which is not 
          // used in fullscreen mode.
          // => no manipulations of banner are necessary
          return;
        }
        // Reset to "" in case of task
        // this.fullSizeControl.style.display = isTemplate ? "none" : "";
        this.fullSizeControl.style.display = "";
        this.fullSizeRestore.style.display = this.fullSizeControl.style.display;

        if (isTemplate && !this.fullSizeUserSelection) {
          // In order to make sure that the publish for the MAXIMIZE_CONTENT topic occurs.
          dojo.publish(com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT + "-TaskCompletionView", [{}]);
          this.header.style.position = "static";
        }
        else {
        	// We have different behaviour depending on where this is launched (My Tasks or Launch pane). Defect 131867 removed the
        	// follwoing style on fixed it's issue when opening from the right hand pane, however broke it in the case of opening
        	// via my task. It seems like the MAXIMIZE_CONTENT in the above short circuits the maximize coming from the maximize button
        	// For now if we launch via the my task add the styling below, remove it otherwise
            this.header.style.position = "absolute";
            this.header.style.top = 0;
            this.header.style.left = 0;
        }
        // Check if last opened coach was a startable human service which is always full size but user wants normal size
        if (!this.fullSizeUserSelection) {
          // Always collapse in order to make sure that the publish for the RESTORE_CONTENT topic occurs.            
          this.collapseToNormalSize();
        }
        else {
          // Always expand in order to make sure that the publish for the MAXIMIZE_CONTENT topic occurs.
          this.expandToFullSize();
        }
      },

      _clear: function(updateMode /* boolean */){
        if (this.coachRenderer && !updateMode) {
          this.unloadCoach();
        }
        // Clear cached objects    
        this._id = "";
        this._task = null;
        this._taskTemplate = null;
        this._clientType = null;
        this._clientSettings = null;
        this._actions = null;
        this._ngCoach = null;

        if (this._menu) {
          // Unwrap and remove the element out of the DOM. Otherwise it does not disappear.     
          com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this.menuAnker);
          this.menuAnker.style.display = "none";
          this._menu.destroyRecursive();
          this._menu = null;
        }
        // clear header
        this.headerUpdated = false;
        this.subjectArea.innerHTML = "";
        this.dueDateArea.innerHTML = "";
        this.fullSizeSubject.innerHTML = "";
        this.fullSizeDueDate.innerHTML = "";
        if (this._modifyTaskHandler) {
          this._modifyTaskHandler.destroy();
          this._modifyTaskHandler = null;
        }
      },

      unloadCoach: function unloadCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        // Coach need to be removed before loading the next one
        if (this.coachRenderer) {
          this.coachRenderer.unload();
          this.coachRenderer = null;
        }
        com.ibm.bpm.social.util.common.Log.logExit(2, this._id);
      },

      /**
       * Check authorization for working on the task by retrieving the actions and the clientSettings for the
       * task.
       * 
       * In case of error, the error is shown and onCancel is called.
       * 
       * @param {object} The task object.
       */
      loadCoach: function loadCoach_TaskCompletionWidget(/* object */task){
        com.ibm.bpm.social.util.common.Log.logEntry(2, task.tkiid);
        var myself = this;
        this._clear(false /* no update */);
        this._task = task;
        var message;
        // 
        // Check prerequisites.
        //
        if (!task || typeof task.tkiid != "string" || typeof task.state != "string") {
          com.ibm.bpm.social.util.common.Log.log(2, "The task not available");
          message = this._nlsResources["task_not_available"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onTaskDoesNotExist();
          });
          return;
        }
        this._initFullSizeMode(false);
        this._id = this._task.tkiid; // for tracing purposes
        var taskId = task.tkiid;
        this._updateHeader(this._task);

        //
        // Is task already completed
        //
        if (this._task.state == "STATE_FINISHED") {
          // In case the TaskCompletionView stays on this page.
          // fullSizeUserSelection will not be changed
          this.collapseToNormalSize();
          var piid = null;
          if (this._task.piid) {
            piid = this._task.piid;
          }
          com.ibm.bpm.social.util.common.Log.log(2, "The task " + taskId + " with piid " + piid + " is already completed");
          message = this._nlsResources["task_already_completed"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, null /* details */, function(){
            // TaskCompletionView can decided based on (piid != null) which view to show.
            myself.onTaskAlreadyCompleted(piid);
          });
          return;
        }
        // The client type is always the same. (both for heritage coaches and for next generation coaches)            
        this._clientType = "IBM_WLE_Coach";
        // Make the asynchronous calls to
        // 1) Retrieve the clientSettings
        // 2) Retrieve authorization information        
        this.restHandler.getClientSetting(null, taskId, this._clientType, function(response){
          myself._setClientSettings(response);
        }, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, true, task.systemID);
        this._retrieveActions(taskId, false, true /* async */);
        //
        // The header can already be shown
        //
        this._showLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Retrieve client settings and display the coach.
       * 
       * @param {object} The task object.
       */
      loadCoach4Template: function loadCoach4Template_TaskCompletionWidget(/* object */taskTemplate){
        com.ibm.bpm.social.util.common.Log.logEntry(2, taskTemplate.tktid);
        var myself = this;
        this._clear(false /* no update */);
        this._taskTemplate = taskTemplate;
        // 
        // Check prerequisites.
        //
        if (!taskTemplate || typeof taskTemplate.tktid != "string" || typeof taskTemplate.state != "string") {
          com.ibm.bpm.social.util.common.Log.log(2, "The task template not available");
          var message = this._nlsResources["task_template_not_available"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onTaskDoesNotExist();
          });
          return;
        }
        this._initFullSizeMode(true /* isTemplate */);
        this._id = this._taskTemplate.tktid; // For tracing purposes        
        this._updateHeader4Template(this._taskTemplate);
        var taskTemplateId = taskTemplate.tktid;
        // The client type is always the same. (both for heritage coaches and for next generation coaches)                      
        this._clientType = "IBM_WLE_Coach";
        // Make the asynchronous calls to
        // 1) Retrieve the clientSettings
        // 2) Retrieve authorization information        
        this.restHandler.getClientSetting(taskTemplateId, null, this._clientType, function(response){
          myself._setClientSettings(response);
        }, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, true, taskTemplate.systemID);
        //
        // The header can already be shown
        //
        this._showLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _handleTaskDetailsChange: function handleTaskDetailsChange_TaskCompletionWidget(task){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._task.dueTime = task.dueDateTime;
        this._updateHeader(this._task);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },
      /**
       * Updates the meta data of the coach container. Used in case of autoflow. The coach itself is kept.
       * 
       * @param task
       * @returns {updateTask_TaskCompletionWidget} {void} nothing
       */
      updateTask: function updateTask_TaskCompletionWidget(task /* object */){
        com.ibm.bpm.social.util.common.Log.logEntry(2, task.tkiid);
        this._clear(true /* update */);
        this._task = task;
        this._id = this._task.tkiid; // for tracing purposes        
        this._updateHeader(this._task);
        var taskId = task.tkiid;
        // 1) Retrieve authorization information
        // retrive actions synchronously because they are required in the next step
        this._retrieveActions(taskId, false, false /* async */);
        this._clearLoadingSection();
        this.show();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _updateHeader: function(task){
        // Set the due date the subject of the activity.
        this.headerUpdated = true;
        this.subjectArea.innerHTML = task.displayName;
        this.fullSizeSubject.innerHTML = task.displayName;

        //apply btd to the subject area field
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subjectArea);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.fullSizeSubject);

        this.dueDateSection.style.visibility = "";
        this.dueDateArea.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(task.dueTime);
        this.fullSizeDueDate.innerHTML = this.dueDateArea.innerHTML;
      },

      _updateHeader4Template: function(taskTemplate){
        // Set the due date the subject of the activity.
        this.headerUpdated = true;
        this.subjectArea.innerHTML = taskTemplate.displayName;
        this.fullSizeSubject.innerHTML = taskTemplate.displayName;
        
        //apply btd to the subject area field
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subjectArea);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.fullSizeSubject);
        
        this.dueDateSection.style.visibility = "hidden";
        this.fullSizeDue.style.visibility = "hidden";
      },

      _retrieveActions: function _retrieveActions_TaskCompletionWidget(taskId, recall /* boolean */, async){
        var myself = this;
        this.actions = null;
        this.restHandler.getActionsForTask(taskId, com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget.TASK_ACTIONS,
          function(response){
            var data = response;
            if (data.tasks.length === 1) {
              var actions = data.tasks[0].actions || [];
              myself._setActions(actions, false);
            }
            else {
              myself._showErrorAndCancel(myself._nlsResources["user_not_authorized_for_task"]);
            }
          }, function(error, ioArgs){
            myself._showErrorAndCancel();
          }, async, this._task.systemID, this._task.domain);
      },

      _setClientSettings: function _setClientSettings_TaskCompletionWidget(clientSettings){
        com.ibm.bpm.social.util.common.Log.log(2, "Client Settings received.");
        // APAR JR45854 : show an error message if there is no coach available for a task. For example the 
        // task is implemented by an external activity.               
        if (!clientSettings || !clientSettings.url || clientSettings.url==""){
          var message= this._nlsResources["task_completion_no_coach_available"];
          this._clientSettings=null;
          this._showErrorAndCancel(message);          
        } 
        else {        
          this._clientSettings = clientSettings;
           // If we are dealing with a task template or the actions are also available, the 
           // necessary information are retrieved and the coach can be shown.
          if (this._taskTemplate || this._actions) {
             this._showCoach();
          }
        }
      },

      _setActions: function _setActions_TaskCompletionWidget(/* object */actions,/* boolean */recall){
        com.ibm.bpm.social.util.common.Log.log(2, "Task Actions received.");
        if (actions && dojo.indexOf(actions, "ACTION_CLAIM") != -1) {
          // If this is already a recall, continue and rely on the isUserAndTaskFit check later
          if (!recall) {
            // Not authorized yet
            this._actions = null;
            var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
            if (userProfile && userProfile.userPreferences &&
              userProfile.userPreferences['Alert On Assign And Run'] === "true") {
              com.ibm.bpm.social.util.common.Log.log(2, "Show confirmation dialog for claim");
              if (!this._confirmClaimDialog) {
                this._createClaimDialog();
              }
              this._confirmClaimDialog.showConfirmationDialog(this._task);
            }
            else {
              // synchronous call
              this._autoClaimTask();
            }
          }
        }
        else {
          this._actions = actions;
        }
        // Do we have both results ??
        if (this._clientSettings && this._actions) {
          this._showCoach();
        }
      },

      _autoClaimTask: function _autoClaimTask_TaskCompletionWidget(){
        var myself = this;
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        // Synchronous calls because we need to first claim before checking the permission again.
        this.restHandler.claimTask(this._task.tkiid, null, function(error, ioArgs){
          myself._showErrorAndCancel();
        }, this._task.systemID);
        // Retrieve the actions again.
        this._retrieveActions(this._task.tkiid, true /* recall */, true /* async */);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createActionDropDown: function _createActionDropDown_TaskCompletionWidget(availableActions /* object */){
        if (this._isDialog || this._menu) {
          // No header => no action drop down
          return;
        }
        var modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler({
          task: this._task,
          availableActions: availableActions
        });
        this._modifyTaskHandler = modifyTaskHandler; // In order to destroy in case it is a dijit._Widget
        this.connect(modifyTaskHandler, "onTaskModified", function(dueDateTime, priority, assignee){
          dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_UPDATED + "." + this._id, [{
            "dueDateTime": dueDateTime,
            "priority": priority,
            "assignee": assignee
          }]);
        });
        //
        // Create the menu and add the task actions
        //
        this._menu = this._dropDownHelper.createDropDownForTaskCompletion(this.menuAnker, this._task, availableActions, {
          modifyTask: function(){
            modifyTaskHandler.showDialog();
          },
          reassignBackToGroup: function(){
            modifyTaskHandler.reassignBackToGroup();
          },
          collaborate: dojo.hitch(this, "_showInvitationDialog")
        });
        //
        // Add the process actions if necessary
        //
        if (this._menu && typeof this._task.piid == "string") {
          this._dropDownHelper.addProcessActions(this._menu, {
            "piid": this._task.piid
          }, {
            onCompletedCallback: function(){
            // No action necessary
            },
            onCanceledCallback: function(){
            // No action necessary
            }
          }, true, false, false, true /* omitCriticalPath */);
        }
        if (this._menu) {
          com.ibm.bspace.common.util.ui.Helper.wrapElement(this.menuAnker);
          dojo.style(this.menuAnker, "display", "inline-block");
        }
      },

      _showInvitationDialog: function _showInvitationDialog_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(this._task);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _createClaimDialog: function _createClaimDialog_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        this._confirmClaimDialog = new com.ibm.bpm.social.widgets.task.util.ClaimDialog({
          "restHandler": this.restHandler
        });
        this.connect(this._confirmClaimDialog, "onClaimed", function(){
          com.ibm.bpm.social.util.common.Log.logEntry(2);
          myself._retrieveActions(myself._task.tkiid, true /* recall */, true /* async */);
        });
        this.connect(this._confirmClaimDialog, "onFailed", function(){
          myself.onCanceled();
        });
        this.connect(this._confirmClaimDialog, "onCancelled", function(){
          myself.onCanceled();
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _isTaskAndUserFit: function _isTaskAndUserFit_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        //
        // How to deal with collaboration invitations??    
        // Assumption the authorization is transparently handled by the actions call (taking into account collaboration invitations).
        // ACTION_SETOUTPUTMESSAGE is used for invited user.
        // ACTION_COMPLETE is used for owner. 
        // We need to distinguish in this way, because only the owner is allowed to actually complete the task.
        // 
        if (this._actions &&
          (dojo.indexOf(this._actions, "ACTION_COMPLETE") !== -1 || dojo.indexOf(this._actions, "ACTION_SETOUTPUTMESSAGE") !== -1)) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "true");
          return true;
        }
        else {
          com.ibm.bpm.social.util.common.Log.logExit(2, "false");
          return false;
        }
      },

      _showCoach: function _showCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        var myself = this;
        // Check preconditions only for tasks
        if (this._task && !this._isTaskAndUserFit(this._task)) {
          var message = this._nlsResources["user_not_authorized_for_task"];
          com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, null /* details */, function(){
            myself.onCanceled();
          });
          com.ibm.bpm.social.util.common.Log.logExit(2);
          return false;
        }
        // Delegate the rendering to the CoachRenderer which is also used in Business Space.        
        var mode = com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT;
        mode =
          this._task && this._task.piid ? com.ibm.bpm.social.widgets.task.form.Renderer.MODE_EDIT
            : com.ibm.bpm.social.widgets.task.form.Renderer.MODE_INIT;

        var parameters = {
          serviceURL: window.endpointURLObject.bpmRest +"/v1/",
          mode: mode,
          clientType: this._clientType,
          clientSettings: this._clientSettings,
          resizeHeightAutomatically: false,
          dialog: this._isDialog
        };
        if (this._task) {
          parameters["task"] = this._task;
        }
        if (this._taskTemplate) {
          parameters["taskTemplate"] = this._taskTemplate;
        }
        this.coachRenderer = new com.ibm.bpm.social.widgets.task.form.CoachRenderer(parameters);
        // Establish callbacks...              
        // Connect using the dijit._Widget whose lifecycle is bound to the methods. Thereby the connects get automatically disconnected.
        this.coachRenderer.connect(this.coachRenderer, "onSubmitted", function(tkiid, nextTkiid){
          com.ibm.bpm.social.util.common.Log.log("onSubmitted(" + tkiid + "," + nextTkiid + ")");
          if (nextTkiid) {
            myself.onCompleted(nextTkiid);
          }
          else {
            myself._closeCoach();
          }
        });
        this.coachRenderer.connect(this.coachRenderer, "onCanceled", function(){
          myself._closeCoach();
        });
        this.coachRenderer.connect(this.coachRenderer, "onLoaded", function onLoadedCallback_TaskCompletionWidget(
          tkiid /* string */, newGeneration /* boolean */){
          // In case the coach has been hidden before
          myself.coachRenderer.show();
          if (newGeneration) {
            com.ibm.bpm.social.util.common.Log.log(2, myself._id + " uses next generation coach.");
            myself._ngCoach = true;
            myself.onLoaded();
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(2, myself._id + " uses heritage coach.");
            myself._ngCoach = false;
            myself.onLoaded();
          }
        });
        this.coachRenderer.connect(this.coachRenderer, "onErrorOccurred", function(message, description, details, action){
          myself._closeCoachDueToError(message);
        });
        this.coachRenderer.connect(this.coachRenderer, "onResourceLoaded", function(url){
          myself._clearLoadingSection();
        });
        this._applyHeight();
        this.coachRenderer.load(this.coachContainerAnker);
        this.coachRenderer.enableActions();
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return true;
      },

      _closeCoachDueToError: function _closeCoachDueToError_TaskCompletionWidget(message){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.unloadCoach();
        var errorMessage = this._nlsResources[message];
        if (!errorMessage) {
          errorMessage = message;
        }
        this._showErrorAndCancel(errorMessage);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _closeCoach: function _closeCoach_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.coachRenderer) {
          this.unloadCoach();
          // blue sky scenario: The task has been properly completed.
          this.onCompleted();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      exitTaskCollaboration: function exitTaskCollaboration_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.coachRenderer) {
          this.coachRenderer.exitTaskCollaboration();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _applyHeight: function _applyHeight_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        //
        // The coachRenderer should take all the space available below the header.
        // Shall we set some minimal sizes?
        //
        var availableSpace = dojo._getContentBox(this.domNode).h - 35 - 35; //defect 121150
        
        // For some reason in expanded mode the size drops to the bottom of the windows (maybe not taking into account the page footer).
        // 124647
        //if (this.fullSize) {  // remove this condition to factor in the page footer in all the calculations (133663)
       		availableSpace = availableSpace - 25;
        //}
        
        if (!this.fullSize) {
          var headerHeight = dojo._getBorderBox(this.header).h;
          availableSpace = availableSpace - headerHeight;
        }
        if (this.coachRenderer) {
          this.coachRenderer.resize({
            height: availableSpace
          });
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      resize: function resize_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._applyHeight();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _showErrorAndCancel: function _showErrorAndCancel_TaskCompletionWidget(message, action){
        com.ibm.bpm.social.util.common.Log.logEntry(2, this._id);
        var myself = this;
        if (message === null) {
          // default message => server not available
          message = this._nlsResources["server_not_available"];
        }
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, null /* details */, function(){
          myself.onCanceled();
        });
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * The coach for task with id tkiid has been successfully loaded.
       * 
       * @param {string} The tkiid of the task.
       */
      onLoaded: function onLoaded_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.log(2, "called for " + this._id);
        this._clearLoadingSection();
      },

      /**
       * The interaction with the coaches has been finished.
       */
      onCompleted: function onCompleted_TaskCompletionWidget(/* string */newTkiid){
        if (this.fullSize) {
          this._restoreBanner();
        }
        if (newTkiid) {
          com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted(" + newTkiid + ") for " + this._id);
        }
        else {
          if (this._task) {
            com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted() for " + this._id);
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCompleted() for template " + this._id);
          }
        }
      },

      /**
       * Loading of the coach was stopped
       */
      onCanceled: function onCanceled_TaskCompletionWidget(){
        if (this.fullSize) {
          this._restoreBanner();
        }
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onCanceled()");
      },

      /**
       * onTaskAlreadyCompleted used for triggering the instance view. {void} nothing
       */
      onTaskAlreadyCompleted: function onShowInstance_TaskCompletionWidget(piid){
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onTaskAlreadyCompleted(" + piid + ")");
      },

      /**
       * Task does not exist. {void} nothing
       */
      onTaskDoesNotExist: function onTaskDoesNotExist_TaskCompletionWidget(){
        com.ibm.bpm.social.util.common.Log.log(2, "TaskCompletionWidget.onTaskDoesNotExist()");
      },

      destroy: function destroy_TaskCompletionWidget(){
        // Set references to null
        this._clear(false /* no update */);
        // Process _Widget.destroy which unconnects all connects.
        this.inherited(arguments);
      }

    });

com.ibm.bpm.social.widgets.task.form.TaskCompletionWidget.TASK_ACTIONS = ["ACTION_CLAIM",
                                                                          "ACTION_COMPLETE",
                                                                          "ACTION_CANCELCLAIM",
                                                                          "ACTION_INVITE",
                                                                          "ACTION_REASSIGNTOUSER",
                                                                          "ACTION_REASSIGNTOGROUP",
                                                                          "ACTION_SETOUTPUTMESSAGE",
                                                                          "ACTION_UPDATEPRIORITY",
                                                                          "ACTION_UPDATEDUEDATE"];