// Category.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.list.Category");

dojo.require("dijit.TitlePane");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.list.Category", dijit.TitlePane, {

  _displayed: false,
  _moreAvailable: null,

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_Category(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    if (!this.id) {
      // TODO check if one of the valid categories
      this.id = com.ibm.bpm.social.widgets.task.list.Category.OVERDUE;
    }
    this._moreAvailable = false;
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * postCreate
   * 
   */
  postCreate: function postCreate_Category(){
    this.inherited(arguments);
    dojo.addClass(this.domNode, "bpm-task-list-category");
    dijit.setWaiRole(this.domNode, "group");
    dijit.setWaiRole(this.containerNode, "presentation");

    // when the Category is hidden at the beginning, RPT complains about missing label:
    // Elements having a WAI-ARIA group role must have a label specified with aria-label or aria-labelledby
    if (this.id) {
      var nlsKey = "task_category_" + this.id + "_collapsed";
      dijit.setWaiState(this.domNode, "label", this._nlsResources[nlsKey]);
    }


  },

  _setTitle: function setTitle_Category(){
    var count = "0";
    // currently, "_collapsed" and "_more_available" suffix do exclude each other
    // (so only 3 versions of the message yet). Could be four in the future though. 
    var moreAvailableKeySuffix = ""; // by default, no suffix
    var sectionCollapsedSuffix = ""; // by default, no suffix
    if (this.count) {
      count = this.count;
      if (this.open) {
        // this category is expanded
        if (this._moreAvailable === true) {
          moreAvailableKeySuffix = "_more_available";
        }
      }
      else {
        // collapsed
        sectionCollapsedSuffix = "_collapsed";
      }
    }
    var title = "";
    if (this.id) {
      var nlsKey = "task_category_" + this.id + sectionCollapsedSuffix + moreAvailableKeySuffix;
      title = dojo.string.substitute(this._nlsResources[nlsKey], {
        1: count
      });
    }
    this.set("title", title);
    dijit.setWaiState(this.domNode, "label", title);
  },

  setCount: function setCount_Category(count, moreAvailable){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "count " + count + ", moreAvailable " + moreAvailable);
    if (moreAvailable !== undefined) {
      this._moreAvailable = moreAvailable;
    }
    if (count) {
      this.count = count;
    }
    if (this.count) {
      this._setTitle();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  getCount: function getCount_Category(){
    if (this.count && this.count > 0) {
      if (this._moreAvailable) {
        return this.count - 1;
      }
      else {
        return this.count;
      }
    }
    else {
      return 0;
    }
  },

  isIncomplete: function isIncomplete_Category(){
    if (this._moreAvailable) {
      return true;
    }
    else {
      return false;
    }
  },

  isDisplayed: function isDisplayed_Category(){
    return this._displayed;
  },

  isExpanded: function isExpanded_Category(){
    if (this.open) {
      return true;
    }
    else {
      return false;
    }
  },

  toggle: function toggle_Category(silent){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);
    if (silent !== true) {
      if (this.open) {
        this.onExpanded(this.count);
      }
      else {
        this.onCollapsed(this.count);
      }
    }
    com.ibm.bpm.social.util.common.Log.logEntry(2);
  },

  onCollapsed: function onCollapsed_Category(count){
    // event
    dojo.addClass(this.titleBarNode, "bpm-task-list-category-collapsed");
    this._setTitle();
  },

  onExpanded: function onExpanded_Category(count){
    // event
    dojo.removeClass(this.titleBarNode, "bpm-task-list-category-collapsed");
    this._setTitle();
  },

  show: function show_Category(){
    this._displayed = true;
    dojo.style(this.domNode, "display", "block");
  },

  hide: function hide_Category(){
    this._displayed = false;
    dojo.style(this.domNode, "display", "none");
  }

});

/*
 *  @param String categoryId
 *  @returns Date
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate = function(categoryId){
  var resultDate = null;
  if (categoryId) {
    var now = new Date();
    var endOfToday = new Date(now);
    endOfToday.setHours(23, 59, 59, 0);
    var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

    // getting the end of this week needs some more computation: depends on current weekday and locale
    // Note: there are no start and end due dates for the "atRisk" category - this is indicated by an extra flag 
    var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
    // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
    firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
    var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
    var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);

    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      resultDate = null;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      resultDate = now;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      resultDate = endOfToday;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      resultDate = endOfTomorrow;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      resultDate = endOfThisWeek;
    }
  }
  return resultDate;
};

/*
 *  @param String categoryId
 *  @returns Date
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate = function(categoryId){
  var resultDate = null;
  if (categoryId) {
    var now = new Date();
    var endOfToday = new Date(now);
    endOfToday.setHours(23, 59, 59, 0);
    var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

    // getting the end of this week needs some more computation: depends on current weekday and locale
    // Note: there are no start and end due dates for the "atRisk" category - this is indicated by an extra flag 
    var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
    // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
    firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
    var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
    var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);

    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      resultDate = now;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      resultDate = endOfToday;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      resultDate = endOfTomorrow;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      resultDate = endOfThisWeek;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      resultDate = null; // null, not empty string
    }
  }
  return resultDate;
};


/*
 *  This method makes categories comparable, so we can use < or > in order to find out
 *  whether an item is in a lower (later) or in a higher category.
 *  This is used, for example, to decide whether we have trailing collapsed categories (which should be suppressed).
 *  As numeric values we simply use multiples of 100, so if later on some category is introduced in the middle, we can give it
 *  a new value without having to shift the other values (e.g. give a new category a value of 350).
 *  
 *  @param String categoryId
 *  @returns int   some number that reflects the sort order, not necessarily adjacent
 *                 or -1 if the category id is not a known id 
 */
com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal = function(categoryId){
  var result = -1;
  if (categoryId) {
    if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
      result = 100;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
      result = 200;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) {
      result = 300;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) {
      result = 400;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) {
      result = 500;
    }
    else if (categoryId === com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER) {
      result = 600;
    }
  }
  return result;
};


com.ibm.bpm.social.widgets.task.list.Category.OVERDUE = "overdue";
com.ibm.bpm.social.widgets.task.list.Category.AT_RISK = "atRisk";
com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY = "dueToday";
com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW = "dueTomorrow";
com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK = "dueThisWeek";
com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER = "dueLater";
