// List.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.list.List");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.list.Category");
dojo.require("com.ibm.bpm.social.widgets.task.list.TaskRow");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");
dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.list.List", [dijit._Widget,
                                                           dijit._Templated], {

  FADING_DURATION: 2000,

  templateString: '<div class="bpm-task-list" dojoAttachPoint="itemList"></div>',

  widgetsInTemplate: false,

  model: null,

  modelEffectiveSize: 0,

  pageSize: 0,

  _categoryWidgets: null,

  _rowWidgets: null,

  _rowCache: [],

  _connectRowHandlers: null,

  _connectCategoryHandlers: null,

  _lastSeenCategoryNumVal: null,

  _animations: [],

  _rowState: {}, // keeps allExpanded or an array of expanded/collapsed tkiids for each filter id (openTasks, completedTasks)

  _initializingRows: false,

  _focus: {}, // keeps last focus info for each filter id (openTasks, completedTasks)

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_List(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._categoryWidgets = [];
    this._rowWidgets = {};
    this._connectRowHandlers = [];
    this._connectCategoryHandlers = [];

    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the Layout config of the table Setting the config purges the content
   * 
   * @param {Object} config
   */
  setConfig: function setConfig_List(config){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this.config = config;

    this.setKeepSavedFocus(false);

    // clear all data in the categories
    this._clearList();
    // also hide all categories now
    this._hideCategories();

    // initialize expanded row settings
    this._initRowState();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Resize the table (Not necessarily implemented by children)
   * 
   * @param {Object} dimensions JSON object {"h" : number, "w" : number }
   */
  resize: function(args){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (args && args.h > 0) {
      this.domNode.style.height = args.h + "px";
    }

    if (args && args.w > 0) {
      this.domNode.style.width = args.w + "px";
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * destroy the view, Not necessarily implemented by children
   */
  destroy: function destroy_List(config){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    var i;
    if (this._connectRowHandlers) {
      for (i = 0; i < this._connectRowHandlers.length; i++) {
        dojo.disconnect(this._connectRowHandlers[i]);
      }
      this._connectRowHandlers = null;
    }
    if (this._rowWidgets) {
      for ( var categoryId in this._rowWidgets) {
        for (i = 0; i < this._rowWidgets[categoryId].length; i++) {
          this._rowWidgets[categoryId][i].destroy();
        }
      }
      this._rowWidgets = null;
    }
    if (this._connectCategoryHandlers) {
      for (i = 0; i < this._connectCategoryHandlers.length; i++) {
        dojo.disconnect(this._connectCategoryHandlers[i]);
      }
      this._connectCategoryHandlers = null;
    }
    if (this._categoryWidgets) {
      for (i = 0; i < this._categoryWidgets.length; i++) {
        this._categoryWidgets[i].destroyDescendants();
        this._categoryWidgets[i].destroy();
      }
      this._categoryWidgets = null;
    }

    this.inherited('destroy', arguments);
    com.ibm.bpm.social.util.common.Log.logExit(1);
  },


  /**
   * postCreate
   */
  postCreate: function(){

    this._rowCache = new Array();

    this.containerNode = this.domNode;
    dijit.setWaiRole(this.domNode, "list");

    // initialize expanded row settings
    this._initRowState();

  },


  onDetailsRequested: function(item){},

  onCategoryCollapsed: function(categoryId, count){},

  onCategoryExpanded: function(categoryId, count){},

  onFilter: function(){},

  /**
   * Set the page size which is later used to decide whether a trailing section should be displayed
   * 
   * @param {int} actual page size requested by user, initially 25, doubled by "Show more"
   * @type {void}
   */
  setPageSize: function setPageSize_List(pageSize){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.pageSize = pageSize;
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the model(data) to be displayed
   * 
   * @param {Array} items Array of items
   * @type {void}
   */
  setModel: function setModel_List(model){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    this._initializingRows = true;

    // PERFORMANCE TEST
    // PERFORMANCE TEST this._rowsFromCache = 0;
    // PERFORMANCE TEST this._newRows = 0;

    // Stop all current animations, otherwise there might be fading effects on the 
    // full reloaded list.
    var item, k;
    for (k = 0; k < this._animations.length; k++) {
      this._animations[k].stop(false);
    }
    this._animations = [];

    this.setKeepSavedFocus(false);

    this.model = model;
    this.modelEffectiveSize = 0;

    // clear all data in the categories
    // TODO: check, if we can do the clean and reload better for expand and collapse, 
    // right now it is a bit slow. 
    this._clearList();

    var myself = this;
    if (this.model) {
      if (this.config && this.config.filter.categories) {

        // add the categories to the model
        this._addCategoriesToModel(model);

        dojo.attr(this.domNode, "aria-busy", "true");

        // find category with highest index.
        // This will be needed later to identify and suppress trailing categories
        myself._lastSeenCategoryNumVal = -1;
        for (k = 0; k < this.model.length; k++) {
          item = this.model[k];
          // TODO   Optimization possible: we get items in sorted order, so rather than doing this loop,
          // it would be ok just to check the category of the last non-extra item.
          // Loop over 25 (or few hundred) iterations is quite cheap though, so until we are sure that all backends sort correctly
          // we can use this defensive way.   
          if (item && item.isExtra === true) {
            // skip this item
            com.ibm.bpm.social.util.common.Log.log(3, "Extra item: " + item.TKIID);
          }
          else {
            myself.modelEffectiveSize++;
            // maintain highest due date seen: memorize highest one
            // TODO  Optimization: memorize the cat id (string) first, transforming to a number only on mismatch
            // In most cases, the string will already match. 
            var actualCatNumVal = com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal(item.categoryId);
            if (actualCatNumVal > myself._lastSeenCategoryNumVal) {
              myself._lastSeenCategoryNumVal = actualCatNumVal;
            }
          }
          //});

        }
        // create new content       
        if (this.model) {
          dojo.forEach(this.config.filter.categories, function(categoryId){
            var tasksInCategory = dojo.filter(myself.model, function(item){
              return item.categoryId === categoryId;
            });
            var taskContainer = document.createElement("div");

            if (!myself._rowWidgets[categoryId]) {
              myself._rowWidgets[categoryId] = [];
            }

            for (k = 0; k < tasksInCategory.length; k++) {
              var item = tasksInCategory[k];
              if (item && item.isExtra !== true) {
                // the row destroy is called through the category destroyDescendants call               
                var itemWidget = myself._getNewRowWidget(item, categoryId);
                taskContainer.appendChild(itemWidget.domNode);
              }
            }

            var categoryWidget = myself._getCategoryWidget(categoryId);

            // compute effective length; this may be different than tasksInCategory.length 
            // because we have tried to fetch one extra item 
            var moreAvailable = false;
            var effectiveLength = tasksInCategory.length;
            if (tasksInCategory.length > 0 && tasksInCategory[tasksInCategory.length - 1] &&
              tasksInCategory[tasksInCategory.length - 1].isExtra === true) {
              moreAvailable = true;
              effectiveLength = tasksInCategory.length - 1;
            }
            // show or hide the widget
            // trailing categories (i.e. those who have a numerically "higher" category than the last task fetched from the backend)
            // are not shown, neither collapsed nor in expanded state.
            // Exception to this: when the page is not full (less items retrieved from the server page size)
            var categoryIsTrailing = false;
            if (categoryWidget.id && categoryWidget.id !== com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) {
              // Overdue is never trailing
              if (myself._lastSeenCategoryNumVal && myself._lastSeenCategoryNumVal > 0 && categoryWidget.id) {
                // Transform current category name into a comparable numeric value:
                var categoryNumVal = com.ibm.bpm.social.widgets.task.list.Category.getCategoryNumVal(categoryWidget.id);
                if (categoryNumVal > myself._lastSeenCategoryNumVal) {
                  categoryIsTrailing = true;
                  if (!categoryWidget.isExpanded()) {
                    // Here, we can auto-expand collapsed categories that are beyond the screen.
                    // auto-expand invisible categories; otherwise, there may be strange effects,
                    // for example, if we expand a category which has 0 items in the meantime
                    // in which case the "expand" operation would make the category disappear - very confusing.
                    // This auto-expand is currently disabled here:
                    // categoryWidget.toggle(true); // This "true" means "silent", i.e. do not cause another fetch from backend  
                  }
                }
              }
            }

            // find out whether there are more items on the server available than the current page size
            // here, the where clause (that was built to reflect the expanded sections) will affect the items available on the server  
            // Because we retrieve one extra record, this is visible in the fact whether that extra item was retrieved 
            // (current page size is initially 25, then potentially doubled by "show more" requests)
            var pageIsFull = false;
            if (myself.modelEffectiveSize && myself.pageSize && myself.modelEffectiveSize === myself.pageSize) {
              pageIsFull = true;
            }

            // Hide categories if one (or both) of these conditions are true:
            // 1. an expanded category has 0 items (note that collapsed categories have always 0 items due to the where clause we use)
            // 2. trailing categories (collapsed or expanded) in case that they would start beyond the page size because page is full  
            if ((effectiveLength === 0 && categoryWidget.isExpanded()) || (categoryIsTrailing && pageIsFull === true)) {
              categoryWidget.hide();
              com.ibm.bpm.social.util.common.Log.log(1, "Hiding category '" + categoryWidget.id + "', eff size: " +
                myself.modelEffectiveSize + ", pageSize: " + myself.pageSize, "List", "setModel");
            }
            else {
              categoryWidget.set("content", taskContainer);
              categoryWidget.setCount(effectiveLength, moreAvailable);
              categoryWidget.show();
            }
          });
        }
      }
      else {
        if (this.model) {
          // delete old content

          this.pageSize = 0;
          this._lastSeenCategoryNumVal = null;

          // hide all categories
          this._hideCategories();

          // no category, so use pseudo category 1
          if (!this._rowWidgets[1]) {
            this._rowWidgets[1] = [];
          }
          // Display a plain list of items
          for (k = 0; k < this.model.length; k++) {
            item = this.model[k];
            if (item.isExtra !== true) {
              var itemWidget = myself._getNewRowWidget(item, 1);
              myself.domNode.appendChild(itemWidget.domNode);
            }
            else {
              com.ibm.bpm.social.util.common.Log.log(3, "Extra item: " + item.TKIID);
            }
          }
        }
      }

      // maintain expand/collapse state of rows
      var filterId = this._getFilterId();
      if (this._rowState[filterId]) {
        if (this._rowState[filterId].allExpanded) {
          this.expandAll(true);
          this.onShowCollapseAllButton();
        }
        else {
          this.collapseAll(true);
        }
      }

      dojo.attr(this.domNode, "aria-busy", "false");

    }
    else {
      com.ibm.bpm.social.util.common.Log.log(2, "no items");
    }


    this._initializingRows = false;

    // PERFORMANCE TEST console.debug("from cache: " + this._rowsFromCache + ", new rows: " + this._newRows);

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _getNewRowWidget: function _getNewRowWidget_List(item, categoryId){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "tkiid=" + item.TKIID + ", category=" + categoryId);
    // get the row from the cache if possible
    var itemWidget = null;
    var newRow = false;
    if (this._rowCache.length > 0) {
      itemWidget = this._rowCache.pop();
      com.ibm.bpm.social.util.common.Log.log(3, "Using row from cache.");
      // PERFORMANCE TEST
      // PERFORMANCE TEST this._rowsFromCache++;
    }
    else {
      var params = {
        model: item,
        open: false,
        controller: this.config.controller,
        displayedProperties: this.config.filter.displayedProperties
      };
      if (this.config.type == com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
        itemWidget = new com.ibm.bpm.social.widgets.task.list.TaskRow(params);
      }
      else {
        var e = new Error("Unexpected config type: " + this.config.type);
        com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected config type.", e);
        throw e;
      }
      newRow = true;
      // PERFORMANCE TEST
      // PERFORMANCE TEST this._newRows++;
    }

    itemWidget.initRow({
      model: item,
      open: false,
      controller: this.config.controller,
      displayedProperties: this.config.filter.displayedProperties
    });

    // store the row widget for clean up and update
    if (!this._rowWidgets[categoryId]) {
      this._rowWidgets[categoryId] = new Array();
    }
    this._rowWidgets[categoryId].push(itemWidget);

    // create connects for new rows 
    if (newRow) {
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onDetailsRequested", dojo.hitch(this, function(rowModel){
        this.onDetailsRequested(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onWorkOn", dojo.hitch(this, function(rowModel){
        this.onWorkOn(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onRefreshNeeded", this, "onRefreshNeeded"));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onExpanded", this, dojo.hitch(this, function(rowModel){
        this._onTaskRowExpanded(rowModel);
      })));

      this._connectRowHandlers.push(dojo.connect(itemWidget, "onCollapsed", this, dojo.hitch(this, function(rowModel){
        this._onTaskRowCollapsed(rowModel);
      })));
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onModelUpdate", this, dojo.hitch(this, function(model){
        // Row update triggered by "Modify Task"
        this.updateModel([model], true);
      })));
      this._connectRowHandlers.push(dojo.connect(itemWidget, "onDeleteRowNeeded", this, dojo.hitch(this, function(tkiid){
        // get task row widget
        var row = this._getTaskWidget(tkiid);
        // Remove row from the list if not already removed by notification message
        if (row !== null) {
          this._deleteRowWidget(row, false);
        }
      })));
    }

    // now make it visible
    dojo.style(itemWidget.domNode, "display", "block");

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return itemWidget;
  },

  /**
   * Update the list model and task row
   * 
   * @param items updated tasks
   * @param manualUpdateMode row change has been triggered by the user
   * @param searchFilterApplied the task list is filtered with a search string
   * @param notNewAssignment
   * @returns {void} nothing
   */
  updateModel: function updateModel_List(items, manualUpdateMode, searchFilterApplied, notNewAssignment){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    if (items) {
      com.ibm.bpm.social.util.common.Log.log(3, "Updates for " + items.length + " task(s).");

      var filterId = this._getFilterId();

      var itemsToAdd = new Array();
      var itemIdsToUpdate = new Array();
      var itemsToUpdate = {}; // tkiid: {"item": item, "rowWidget": rowWidget, "actionsNeeded": true|false}
      var itemsNeedingActions = new Array();
      var i;
      var item;
      var extraItem = this._getExtraItem();

      for (i = 0; i < items.length; i++) {
        item = items[i];
        if (item) {

          com.ibm.bpm.social.util.common.Log.log(2, "Update for task " + item.TKIID + ": " + dojo.toJson(item));

          var isExtraItem = (extraItem !== null && extraItem.TKIID === item.TKIID);
          var fullRefresh = false;

          // get task row widget
          var row = this._getTaskWidget(item["TKIID"]);

          if (searchFilterApplied && row === null) {
            // ignore new tasks if search filter has been applied
            com.ibm.bpm.social.util.common.Log.log(2, "Search filter applied: Ignoring new task " + item.TKIID);
            continue;
          }

          //
          //check if row is visible on this list
          var inFilter = true;
          var inScope = true;

          if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
            if (item.STATE && item.STATE !== "STATE_READY" && item.STATE !== "STATE_CLAIMED") {
              // item does no longer match to state filter
              if (isExtraItem) {
                com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                  " is extra item, state does no longer apply to filter " + filterId);
                fullRefresh = true;
              }
              else {
                com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
                inFilter = false;
              }
            }
            else if (item.OWNER && item.OWNER !== com.ibm.bpm.social.util.common.Util.getCurrentUser().userName) {
              // item does not match filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (extraItem !== null && item.DUE) {
              var itemDueDate = dojo.date.stamp.fromISOString(item.DUE);
              var extraItemDueDate = dojo.date.stamp.fromISOString(extraItem.DUE);
              if (isExtraItem) {
                if (dojo.date.compare(itemDueDate, extraItemDueDate) !== 0) {
                  // due date changed for extra item
                  com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                    " is extra item, due date changed for filter " + filterId);
                  fullRefresh = true;
                }
              }
              else {
                // behind extra item?
                if ((dojo.date.compare(itemDueDate, extraItemDueDate) === 0 && (item.TKIID > extraItem.TKIID)) ||
                  dojo.date.compare(itemDueDate, extraItemDueDate) > 0) {
                  com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " is behind extra item for filter " +
                    filterId);
                  inScope = false;
                }
              }
            }
          }
          else if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
            if (item.COMPLETED === null) {
              // item does not match to completed filter
              inFilter = false;
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
            }
            else if (item.STATE && item.STATE !== "STATE_FINISHED") {
              // item does not match to completed filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (!item.OWNER || item.OWNER !== com.ibm.bpm.social.util.common.Util.getCurrentUser().userName) {
              // item does not match to completed filter
              com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " does not apply to filter " + filterId);
              inFilter = false;
            }
            else if (extraItem !== null && item.COMPLETED) {
              var itemCompletedDate = dojo.date.stamp.fromISOString(item.COMPLETED);
              var extraItemCompletedDate = dojo.date.stamp.fromISOString(extraItem.COMPLETED);
              if (isExtraItem) {
                if (dojo.date.compare(itemCompletedDate, extraItemCompletedDate) !== 0) {
                  // completed changed for extra item
                  com.ibm.bpm.social.util.common.Log.log(3, "Full refresh: task " + item.TKIID +
                    " is extra item, completed changed for " + filterId);
                  fullRefresh = true;
                }
              }
              else {
                // behind extra item?
                if ((dojo.date.compare(itemCompletedDate, extraItemCompletedDate) === 0 && (item.TKIID > extraItem.TKIID)) ||
                  dojo.date.compare(itemCompletedDate, extraItemCompletedDate) < 0) {
                  com.ibm.bpm.social.util.common.Log.log(3, "Task " + item.TKIID + " is behind extra item for filter " +
                    filterId);
                  inScope = false;
                }
              }
            }
          }

          if (fullRefresh) {
            // trigger full refresh because extra item was modified
            this.onFullRefreshNeeded();
            com.ibm.bpm.social.util.common.Log.logExit(2, "Full Refresh");
            return;
          }

          if (row === null && manualUpdateMode && !item.OWNER) {
            // manual update may update properties of a task which has already been deleted
            com.ibm.bpm.social.util.common.Log.logExit(2, "Task " + item.TKIID +
              " is already deleted, properties cannot be updated.");
            return;
          }

          if (row === null) {
            //
            // new items
            if (inScope && inFilter) {
              // add new row
              itemsToAdd.push(item);
              itemsNeedingActions.push(item);
              if (!this.model) {
                this.model = new Array();
              }
              // add new item to model - keep eventual extra item at the end
              this.model.splice(0, 0, item);
            }

            // TODO: rework special logic for completed tasks to increase global counter.
            if (inFilter &&
              (!notNewAssignment || (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS))) {
              this.onIncreaseGlobalCount();
            }
          }
          else {
            //
            // existing rows
            if (inScope && inFilter) {
              // update existing task row
              if (itemsToUpdate[item.TKIID] === undefined) {
                // add only once but update data if another message is received for the same tkiid
                itemIdsToUpdate.push(item.TKIID);
              }
              itemsToUpdate[item.TKIID] = {
                "item": item,
                "row": row,
                "actionsNeeded": false
              };
              if (row.actionsNeeded(item)) {
                itemsNeedingActions.push(item);
                itemsToUpdate[item.TKIID].actionsNeeded = true;
              }
            }
            else {
              // Remove row from the list            
              this._deleteRowWidget(row, inFilter && (manualUpdateMode || !inScope));
            }
          }
        }
      }

      // trigger actions call - this needs to be done before creating/updating the row, 
      // because the deferred callback can only be registered after the actions call has been done
      if (itemsNeedingActions.length > 0) {
        // retrieve actions async
        this.config.controller.retrieveAvailableActionsForItems(itemsNeedingActions);
      }

      // add items
      if (itemsToAdd.length > 0) {
        if (this.config && this.config.filter.categories) {
          // add the categories to the model
          this._addCategoriesToModel(itemsToAdd);
        }
        for (i = 0; i < itemsToAdd.length; i++) {
          item = itemsToAdd[i];
          if (item) {
            var categoryId = item.categoryId ? item.categoryId : 1;
            var categoryWidget = this._getCategoryWidget(categoryId);

            // Do not synchronize for a task in a collapsed group.
            if (!categoryWidget || categoryWidget.isExpanded()) {
              row = this._getNewRowWidget(item, categoryId, true);
              this._synchronize(row, true);
            }
          }
        }
        if (this._rowState[filterId].allExpanded) {
          // expand new rows
          this._initializingRows = true;
          this.expandAll(true);
          this._initializingRows = false;
        }
      }

      //
      // update items
      for (i = 0; i < itemIdsToUpdate.length; i++) {
        var tkiid = itemIdsToUpdate[i];
        item = itemsToUpdate[tkiid].item;
        if (item) {
          var rowDeleted = false;
          var rowToUpdate = itemsToUpdate[tkiid].row;
          var actionsNeeded = itemsToUpdate[tkiid].actionsNeeded;
          if (rowToUpdate.updateRow(item, actionsNeeded)) {
            this._synchronize(rowToUpdate, false);
          }
        }
      }

    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Updates the UI (row position) and the model.
   * 
   * @param row task row
   * @param addRow a new row has been added
   * @returns {void} nothing
   */
  _synchronize: function _synchronize_List(row, addRow){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    // The following algorithm is applied to the task rows:
    // -> show task reordered in current category:    
    //      Find current category
    //      If category not visible -> Create category 
    //      Move task to it and sort

    var currentCategoryId = row.model.categoryId ? row.model.categoryId : 1; // 1 is pseudo category for completedTasks
    var oldPosition = dojo.indexOf(this._rowWidgets[currentCategoryId], row);
    if (!addRow) {
      com.ibm.bpm.social.util.common.Log.log(3, "Move row for task " + row.model.TKIID);
      // Find current category
      var oldCategoryId = row.model.categoryId;
      com.ibm.bpm.social.util.common.Log.log(3, "Old category = " + oldCategoryId + ", old position = " * oldPosition);
      if (this.config && this.config.filter.categories) {
        this._addCategoriesToModel([row.model]);
        currentCategoryId = row.model.categoryId;

        // If categories are different, remove from old category storage and add to new 
        // category storage.          
        if (oldCategoryId !== currentCategoryId) {
          var tasksInOldCategory = this._rowWidgets[oldCategoryId];
          var indexOfTask = dojo.indexOf(tasksInOldCategory, row);
          tasksInOldCategory.splice(indexOfTask, 1);
          this._rowWidgets[currentCategoryId].push(row);
        }
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.log(3, "Add row for task " + row.model.TKIID);
    }

    // used filter openTasks or completedTasks
    var filterId = this._getFilterId();

    // Change order in _rowWidgets cache
    var itemADate, itemBDate;
    var tasksInCurrentCategory = this._rowWidgets[currentCategoryId];
    tasksInCurrentCategory.sort(function(itemA, itemB){
      if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
        itemADate = dojo.date.stamp.fromISOString(itemA.model.DUE);
        itemBDate = dojo.date.stamp.fromISOString(itemB.model.DUE);
        if (dojo.date.compare(itemADate, itemBDate) === 0) {
          return itemA.model.TKIID > itemB.model.TKIID;
        }
        return dojo.date.compare(itemADate, itemBDate);
      }
      else if (filterId === com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
        itemADate = dojo.date.stamp.fromISOString(itemA.model.COMPLETED);
        itemBDate = dojo.date.stamp.fromISOString(itemB.model.COMPLETED);
        if (dojo.date.compare(itemADate, itemBDate) === 0) {
          return itemA.model.TKIID > itemB.model.TKIID;
        }
        return dojo.date.compare(itemBDate, itemADate);

      }
    });
    var newPosition = dojo.indexOf(tasksInCurrentCategory, row);
    com.ibm.bpm.social.util.common.Log.log(3, "New category = " + currentCategoryId + ", new position = " + newPosition);

    //
    // Now change order in dom tree
    //
    if (addRow || oldPosition !== newPosition || currentCategoryId !== oldCategoryId) {
      var myself = this;

      var currentCategoryWidget = this._getCategoryWidget(currentCategoryId);

      var fadeOutArgs = null;
      if (!addRow) {
        fadeOutArgs =
          {
            node: row.domNode,
            duration: this.FADING_DURATION,
            onStop: function(){
              com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeOut_synchronize");
              // Remove opacity settings, because this row might be reused.       
              myself._removeOpacitySettings(row.domNode);
              com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeOut_synchronize");
            },
            onEnd: function(){
              com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onEndFadeOut_synchronize");

              if (!this._keepSavedFocus) {
                myself.saveFocus();
              }

              if (currentCategoryWidget !== null) {
                // openTasks
                // handle categories
                row.setStatus(currentCategoryId);

                var oldCategoryWidget = null;

                // Move row to new position
                if (currentCategoryId !== oldCategoryId) {
                  oldCategoryWidget = myself._getCategoryWidget(oldCategoryId);
                  oldCategoryWidget.containerNode.childNodes[0].removeChild(row.domNode);

                  // If old category is empty, hide it.
                  if (tasksInOldCategory.length === 0) {
                    com.ibm.bpm.social.util.common.Log.log(3, "Hide old category " + oldCategoryId);
                    oldCategoryWidget.hide();
                  }
                  else {
                    // Decrease category count
                    oldCategoryWidget.setCount(oldCategoryWidget.count - 1);
                  }

                  // If current category is not visible, make it visible.
                  if (!currentCategoryWidget.isDisplayed()) {
                    com.ibm.bpm.social.util.common.Log.log(3, "Show new category " + currentCategoryId);
                    currentCategoryWidget.setCount(1, false);
                    currentCategoryWidget.show();
                    currentCategoryWidget.set('content', document.createElement("div"));
                  }
                  else {
                    // Increment the count
                    currentCategoryWidget.setCount(currentCategoryWidget.count + 1);
                  }
                }
                else {
                  // same category, new position
                  currentCategoryWidget.containerNode.childNodes[0].removeChild(row.domNode);
                }
              }
              else {
                // completedTasks
                myself.domNode.removeChild(row.domNode);
              }

              //
              // Insert the row to the new position
              var rowParentNode = null;
              if (currentCategoryWidget !== null) {
                // openTasks
                rowParentNode = currentCategoryWidget.containerNode.childNodes[0];
              }
              else {
                // completedTasks
                rowParentNode = myself.domNode;
              }

              if (tasksInCurrentCategory.length - 1 === newPosition) {
                rowParentNode.appendChild(row.domNode);
              }
              else {
                // IE 9 requires the nextSibling to be null!
                var nextSibling =
                  newPosition < rowParentNode.childNodes.length ? rowParentNode.childNodes[newPosition] : null;
                rowParentNode.insertBefore(row.domNode, nextSibling);
              }

              if (currentCategoryWidget.isExpanded()) {
                if (oldCategoryWidget !== null && !oldCategoryWidget.isExpanded()) {
                  // increment local count
                  myself.onIncreaseLocalCount();
                }
              }
              else {
                if (oldCategoryWidget !== null && oldCategoryWidget.isExpanded()) {
                  // Decrement local count. 
                  myself.onDecreaseLocalCount();
                }
              }

              myself._restoreFocusIfNeeded(row, currentCategoryWidget);

              // Remove animation from internal storage
              myself._animations.splice(dojo.indexOf(myself._animations, this), 1);
              com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onEndFadeOut_synchronize");
            }
          };
      }
      var fadeInArgs = {
        node: row.domNode,
        duration: this.FADING_DURATION,
        onStop: function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeIn_synchronize");
          myself._removeOpacitySettings(row.domNode);
          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeIn_synchronize");
        }
      };
      if (addRow) {
        fadeInArgs.beforeBegin = function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
          if (currentCategoryWidget !== null) {
            // If current category is not visible, make it visible.
            if (!currentCategoryWidget.isDisplayed()) {
              com.ibm.bpm.social.util.common.Log.log(3, "Show new category " + currentCategoryId);
              currentCategoryWidget.setCount(1, false);
              currentCategoryWidget.show();
              currentCategoryWidget.set('content', document.createElement("div"));
            }
            else {
              // Increment the count
              currentCategoryWidget.setCount(currentCategoryWidget.count + 1);
            }
          }
          // move the row to the correct position and then fade in
          myself._addOpacitySettings(row.domNode);
          var rowParentNode = null;
          if (currentCategoryWidget !== null) {
            // openTasks
            rowParentNode = currentCategoryWidget.containerNode.childNodes[0];
          }
          else {
            // completedTasks
            rowParentNode = myself.domNode;
          }
          // IE 9 requires the nextSibling to be null!
          var nextSibling = newPosition < rowParentNode.childNodes.length ? rowParentNode.childNodes[newPosition] : null;
          rowParentNode.insertBefore(row.domNode, nextSibling);

          if (currentCategoryWidget === null || currentCategoryWidget.isExpanded) {
            // increment local count
            myself.onIncreaseLocalCount();
          }

          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "beforeBeginFadeIn");
        };
      }
      else {
        fadeInArgs.beforeBegin = function(){
          com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
          myself._addOpacitySettings(row.domNode);
          com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "beforeBeginFadeIn_synchronize");
        };
      }

      var animation = null;
      // In IE 8 the opacity settings for child elements with position 
      // absolute or relative will be ignored, so all such elements will 
      // be listed explicitly to fade out and in.
      if (dojo.isIE < 9) {
        var fadeOutElements = null;
        if (!addRow) {
          fadeOutElements = [dojo.fadeOut(fadeOutArgs),
                             dojo.fadeOut({
                               node: row.imgPriority,
                               duration: this.FADING_DURATION
                             }),
                             dojo.fadeOut({
                               node: row.imgStatus,
                               duration: this.FADING_DURATION
                             }),
                             dojo.fadeOut({
                               node: row.imgGroup,
                               duration: this.FADING_DURATION
                             })];
        }
        var fadeInElements = [dojo.fadeIn(fadeInArgs),
                              dojo.fadeIn({
                                node: row.imgPriority,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgPriority, "filter", null);
                                }

                              }),
                              dojo.fadeIn({
                                node: row.imgStatus,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgStatus, "filter", null);
                                }

                              }),
                              dojo.fadeIn({
                                node: row.imgGroup,
                                duration: this.FADING_DURATION,
                                onEnd: function(){
                                  // Remove opacity setting again, because otherwise the 
                                  // element might look screwed up.
                                  dojo.style(row.imgGroup, "filter", null);
                                }
                              })];

        if (addRow) {
          animation = dojo.fx.combine(fadeInElements);

        }
        else {
          animation = dojo.fx.chain([dojo.fx.combine(fadeOutElements),
                                     dojo.fx.combine(fadeInElements)]);
        }
      }
      else {
        if (addRow) {
          animation = dojo.fadeIn(fadeInArgs);

        }
        else {
          animation = dojo.fx.chain([dojo.fadeOut(fadeOutArgs),
                                     dojo.fadeIn(fadeInArgs)]);
        }
      }
      this._animations.push(animation);
      animation.play();
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * delete a row widget
   * 
   * @param row
   * @param keepGlobal no event for decrement of the global count, e.g. if task is in filter but not in list
   *        scope
   * @returns {void} nothing
   */
  _deleteRowWidget: function _deleteRowWidget_List(row, keepGlobal){
    com.ibm.bpm.social.util.common.Log.logEntry(2, row.model.TKIID);

    // Remove the task from the category and push it to the cache
    // This needs to be done immediately to make sure that for manual updates 
    // the delete is not triggered again for the push event
    var tasksInSameCategory = this._rowWidgets[row.model.categoryId];
    var indexOfTask = dojo.indexOf(tasksInSameCategory, row);
    tasksInSameCategory.splice(indexOfTask, 1);

    var myself = this;
    var fadeArgs = {
      node: row.domNode,
      duration: this.FADING_DURATION,
      onStop: function(){
        com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onStopFadeOut_deleteRowWidget");
        // Remove opacity settings, because this row might be reused.
        myself._removeOpacitySettings(row.domNode);
        com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onStopFadeOut_deleteRowWidget");
      },
      onEnd: function(){
        com.ibm.bpm.social.util.common.Log.logEntry(3, row.model.TKIID, "List", "onEndFadeOut_deleteRowWidget");

        if (!this._keepSavedFocus) {
          myself.saveFocus();
        }

        var categoryWidget = myself._getCategoryWidget(row.model.categoryId);
        categoryWidget.containerNode.childNodes[0].removeChild(row.domNode);

        // Remove opacity settings, because this row might be reused.       
        myself._removeOpacitySettings(row.domNode);

        var widget = null;
        // If the category is empty, hide it
        if (tasksInSameCategory.length === 0) {
          categoryWidget.hide();
        }
        else {
          // Decrease category count
          categoryWidget.setCount(categoryWidget.count - 1);
        }

        if (row.open) {
          // row is expanded -> collapse
          row.toggle();
        }

        myself._rowCache.push(row);

        // Synchronize model     
        myself.model.splice(dojo.indexOf(myself.model, row.model), 1);

        if (categoryWidget.isExpanded()) {
          // decrement local count
          myself.onDecreaseLocalCount();
        }
        if (!keepGlobal) {
          // decrement global count
          myself.onDecreaseGlobalCount();
        }

        myself._restoreFocusIfNeeded(row, categoryWidget);

        // Remove animation from internal storage
        myself._animations.splice(dojo.indexOf(myself._animations, this), 1);
        com.ibm.bpm.social.util.common.Log.logExit(3, row.model.TKIID, "List", "onEndFadeOut_deleteRowWidget");
    }
    };

    var animation = null;
    // In IE 8 the opacity settings for child elements with position 
    // absolute or relative will be ignored, so all such elements will 
    // be listed explicitly to fade out.
    if (dojo.isIE < 9) {
      var fadeOutElements = [dojo.fadeOut(fadeArgs),
                             dojo.fadeOut({
                               node: row.imgPriority,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgPriority, "filter", null);
                               }
                             }),
                             dojo.fadeOut({
                               node: row.imgStatus,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgStatus, "filter", null);
                               }

                             }),
                             dojo.fadeOut({
                               node: row.imgGroup,
                               duration: this.FADING_DURATION,
                               onEnd: function(){
                                 // Remove opacity setting again, because this row might be reused.
                                 dojo.style(row.imgGroup, "filter", null);
                               }
                             })];


      animation = dojo.fx.combine(fadeOutElements);
    }
    else {
      animation = dojo.fadeOut(fadeArgs);
    }
    this._animations.push(animation);
    animation.play();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  collapseAll: function collapseAll_List(keepRowState){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var filterId = this._getFilterId();
    if (!keepRowState) {
      // set all collapsed row state
      if (this._rowState[filterId]) {
        this._rowState[filterId].allExpanded = false;
        this._rowState[filterId].expandedRows = new Array();
        this._rowState[filterId].collapsedRows = null;
      }
    }

    // collapse rows
    var self = this;
    var tkiidList = [];
    // get the tkiids for list of shown tasks for the bulk api call we do later on.
    if (this.config.filter.categories) {
      dojo.forEach(this._categoryWidgets, function(categoryWidget){
        dojo.forEach(categoryWidget.getChildren(), function(taskWidget){
          if (taskWidget.open) {
            // row is expanded -> collapse
            taskWidget.toggle();

          }
          else if (dojo.indexOf(self._rowState[filterId].expandedRows, taskWidget.model.TKIID) !== -1) {
            // row is collapsed but part of expandedRows -> expand
            tkiidList.push(taskWidget.model.TKIID);
            taskWidget.toggle();
          }
        });
      });
    }
    else {
      dojo.forEach(this._rowWidgets[1], function(taskWidget){
        if (taskWidget.open) {
          // row is expanded -> collapse
          taskWidget.toggle();
        }
        else if (dojo.indexOf(self._rowState[filterId].expandedRows, taskWidget.model.TKIID) !== -1) {
          // row is collapsed but part of expandedRows -> expand
          tkiidList.push(taskWidget.model.TKIID);
          taskWidget.toggle();
        }
      });
    }

    // Now trigger the bulk api call to receive the task details
    if (tkiidList.length > 0) {
      this.onTaskDetailsNeeded(tkiidList);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  getExpandedCategoriesCount: function getExpandedCategoriesCount_List(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var count = 0;

    if (this._categoryWidgets) {
      for ( var i = 0; i < this._categoryWidgets.length; i++) {
        if (this._categoryWidgets[i].isExpanded()) {
          count++;
        }
      }
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, "count=" + count);
    return count;
  },

  expandAll: function expandAll_List(keepRowState){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var filterId = this._getFilterId();
    if (!keepRowState) {
      // set all expanded row state
      if (this._rowState[filterId]) {
        this._rowState[filterId].allExpanded = true;
        this._rowState[filterId].expandedRows = null;
        this._rowState[filterId].collapsedRows = new Array();
      }
    }

    var self = this;
    var tkiidList = [];
    // get the tkiids for list of shown tasks for the bulk api call we do later on.
    // But get only the collapsed ones, as otherwise there may be lost data:
    // this is in the case that one (expanded) inline task already has (unsaved) data filled in, refresh would loose those
    // TODO: Possible optimitation is to do more parallelism, using this sequence
    //  1) get the list of tkiids,
    //  2) trigger the bulk call (async)
    //  3) to the toggle
    // thus 2 and 3 could run in parallel.
    if (this.config.filter.categories) {
      dojo.forEach(this._categoryWidgets, function(categoryWidget){
        if (categoryWidget.open) {
          dojo.forEach(categoryWidget.getChildren(), function(taskWidget){
            if (!taskWidget.open && dojo.indexOf(self._rowState[filterId].collapsedRows, taskWidget.model.TKIID) === -1) {
              // row is collapsed and not part of collapsedRows
              tkiidList.push(taskWidget.model.TKIID);
              taskWidget.toggle();
            }
          });
        }
      });
    }
    else {
      dojo.forEach(this._rowWidgets[1], function(taskWidget){
        if (!taskWidget.open && dojo.indexOf(self._rowState[filterId].collapsedRows, taskWidget.model.TKIID) === -1) {
          // row is collapsed and not part of collapsedRows
          tkiidList.push(taskWidget.model.TKIID);
          taskWidget.toggle();
        }
      });
    }

    if (tkiidList.length > 0) {
      // Now trigger the bulk api call to receive the task details
      this.onTaskDetailsNeeded(tkiidList);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  //
  // Render details + simple approval input form and controls.
  //
  setDetailsSimpleApproval: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleApproval(taskId, details);
  },
  //
  // Render details + simple completion input form and controls.
  //
  setDetailsSimpleCompletion: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleCompletion(taskId, details);
  },
  //
  // Render details + simple choice input form and controls.
  //
  setDetailsSimpleChoice: function(taskId, details){
    this._getTaskWidget(taskId).setDetailsSimpleChoice(taskId, details);
  },
  setDetails: function setDetails_List(taskId, details){
    this._getTaskWidget(taskId).setDetails(details);
  },

  // remove the rows from all categories and cache them
  _clearList: function _clearList_List(){
    if (this._rowWidgets) {
      for ( var categoryId in this._rowWidgets) {
        for ( var i = 0; i < this._rowWidgets[categoryId].length; i++) {
          // put row into cache and remove from parent in DOM tree
          var row = this._rowWidgets[categoryId][i];
          dojo.style(row.domNode, "display", "none");
          row.domNode.parentNode.removeChild(row.domNode);
          this._rowCache.push(row);
        }
        this._rowWidgets[categoryId] = [];
      }
    }
  },

  _hideCategories: function _hideCategories_List(){
    if (this._categoryWidgets) {
      for ( var i = 0; i < this._categoryWidgets.length; i++) {
        this._categoryWidgets[i].hide();
      }
    }
  },

  _getTaskWidget: function _getTaskWidget_List(taskId){
    var taskWidget = null;
    for ( var categoryId in this._rowWidgets) {
      if (this._rowWidgets[categoryId] && !taskWidget) {
        for ( var i = 0; i < this._rowWidgets[categoryId].length; i++) {
          if (this._rowWidgets[categoryId][i].model.TKIID === taskId) {
            taskWidget = this._rowWidgets[categoryId][i];
            break;
          }
        }
      }
    }
    return taskWidget;
  },

  /*
   * Returns existing category widget or creates new one.
   */
  _getCategoryWidget: function _getCategoryWidget_List(categoryId){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var categoryWidget = null;
    if (this.config && this.config.filter.categories) {
      var existingCategoryWidgets = dojo.filter(this._categoryWidgets, function(category){
        return (category.id === categoryId);
      });

      if (existingCategoryWidgets.length > 0) {
        categoryWidget = existingCategoryWidgets[0];
      }
      // create a new widget, no wipe in for resize calculation!
      else {
        categoryWidget = new com.ibm.bpm.social.widgets.task.list.Category({
          id: categoryId,
          duration: 0
        });

        categoryWidget.set("content", document.createElement("div"));

        // store the connections per category, as they get deleted by category
        this._connectCategoryHandlers.push(dojo.connect(categoryWidget, "onCollapsed", this, function(count){
          this.saveFocus();
          this.onCategoryCollapsed(categoryId, count);
        }));

        this._connectCategoryHandlers.push(dojo.connect(categoryWidget, "onExpanded", this, function(count){
          this.saveFocus();
          this.onCategoryExpanded(categoryId, count);
        }));

        this._categoryWidgets.push(categoryWidget);
        this.domNode.appendChild(categoryWidget.domNode);
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return categoryWidget;
  },

  _getExtraItem: function _getExtraItem_List(){
    var item = null;
    if (this.model && this.model.length > 1) {
      // get last item
      item = this.model[this.model.length - 1];
      if (!item.isExtra) {
        item = null;
      }
    }
    return item;
  },

  // To comply with table view, the taskID must be the second parameter
  showMessage: function showMessage_List(message, taskId){
    this._getTaskWidget(taskId).showMessage(message);
  },


  /**
   * Group tasks by: Overdue, At Risk(later), Due Today, Due Tomorrow, Due This Week, Due Later
   * 
   * @param model
   */
  _addCategoriesToModel: function _addCategoriesToModel_List(model){
    var categories = null;
    if (this.config && this.config.filter.categories) {
      categories = this.config.filter.categories;
    }
    if (categories !== null) {
      var now = new Date();
      var endOfToday = new Date(now);
      endOfToday.setHours(23, 59, 59, 0);
      var endOfTomorrow = dojo.date.add(endOfToday, "day", 1);

      // getting the end of this week needs some more computation: depends on current weekday and locale
      var firstDayOfWeek = dojo.cldr.supplemental.getFirstDayOfWeek(dojo.locale);
      // TODO  The above line assigns 0 for "en-us" which is not desired, patched in the next line
      firstDayOfWeek = 1; // TODO  Find a way to identify the first working day of the week
      var addDaysToToday = (6 + firstDayOfWeek - now.getDay()) % 7;
      var endOfThisWeek = dojo.date.add(endOfToday, "day", addDaysToToday);
      var myself = this;
      dojo.forEach(model, function(item){
        var due = dojo.date.stamp.fromISOString(item.DUE);

        if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.OVERDUE) && dojo.date.compare(now, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.OVERDUE;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) && item.IS_AT_RISK === true) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.AT_RISK;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY) &&
          dojo.date.compare(endOfToday, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_TODAY;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW) &&
          dojo.date.compare(endOfTomorrow, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_TOMORROW;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK) &&
          dojo.date.compare(endOfThisWeek, due) >= 0) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_THIS_WEEK;
        }
        else if (myself._hasCategory(com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER)) {
          item.categoryId = com.ibm.bpm.social.widgets.task.list.Category.DUE_LATER;
        }
      });
    }
  },

  /**
   * Indicates whether a particular category should be rendered according to the current configuration
   * 
   * @param id String the category name which is to be looked up in the configuration
   */
  _hasCategory: function _hasCategory_List(id){
    var filteredArr = dojo.filter(this.config.filter.categories, function(categoryId){
      return categoryId == id;
    });
    return filteredArr.length > 0;
  },

  _getFilterId: function _getFilterId_List(){
    if (this.config && this.config.filter && this.config.filter.id) {
      return this.config.filter.id;
    }
    return null;
  },

  _initRowState: function _initRowState_List(){
    var filterId = this._getFilterId();
    if (filterId !== null) {
      if (this._rowState[filterId] === undefined) {
        this._rowState[filterId] = {
          allExpanded: false,
          expandedRows: new Array(),
          collapsedRows: null
        };
      }
    }
  },

  _onTaskRowExpanded: function _onTaskRowExpanded_List(item){
    this.onTaskRowExpanded(item);
    if (!this._initializingRows) {
      // maintain row state
      var filterId = this._getFilterId();
      if (filterId !== null && this._rowState[filterId]) {
        if (this._rowState[filterId].allExpanded) {
          // all expanded
          var index = dojo.indexOf(this._rowState[filterId].collapsedRows, item.TKIID);
          if (index !== -1) {
            // remove from collapsedRows
            this._rowState[filterId].collapsedRows.splice(index, 1);
          }

        }
        else if (dojo.indexOf(this._rowState[filterId].expandedRows, item.TKIID) === -1) {
          // not all expanded, add to expandedRows
          this._rowState[filterId].expandedRows.push(item.TKIID);
        }
      }
    }
  },

  _onTaskRowCollapsed: function _onTaskRowCollapsed_List(item){
    this.onTaskRowCollapsed(item);
    if (!this._initializingRows) {
      // maintain row state
      var filterId = this._getFilterId();
      if (filterId !== null && this._rowState[filterId]) {
        if (!this._rowState[filterId].allExpanded) {
          // not all expanded
          var index = dojo.indexOf(this._rowState[filterId].expandedRows, item.TKIID);
          if (index !== -1) {
            // remove from expandedRows
            this._rowState[filterId].expandedRows.splice(index, 1);
          }
        }
        else if (dojo.indexOf(this._rowState[filterId].collapsedRows, item.TKIID) === -1) {
          // all expanded, add to collapsedRows
          this._rowState[filterId].collapsedRows.push(item.TKIID);
        }
      }
    }
  },

  onTaskDetailsNeeded: function onTaskDetailsNeeded_List(tkiidList){},

  onWorkOn: function onWorkOn_List(item){},

  onRefreshNeeded: function onRefreshNeeded_List(){},

  onTaskRowExpanded: function onTaskRowExpanded_List(item){},

  onTaskRowCollapsed: function onTaskRowCollapsed_List(item){},

  /**
   * event triggered if a task is removed from the list (global count)
   */
  onDecreaseGlobalCount: function onDecreaseGlobalCount_List(){},

  /**
   * event triggered if a task is added to the list (global count)
   */
  onIncreaseGlobalCount: function onIncreaseGlobalCount_List(){},

  /**
   * event triggered if a task is moved from an expanded to a collapsed category (local count)
   */
  onDecreaseLocalCount: function onDecreaseLocalCount_List(){},

  /**
   * event triggered if a task is moved from a collapsed to an expanded category (local count)
   */
  onIncreaseLocalCount: function onIncreaseLocalCount_List(){},

  /**
   * event triggered if the task list is in all rows expanded state to show the button in the correct state
   */
  onShowCollapseAllButton: function onShowCollapseAllButton_List(){},

  /**
   * event triggered if the extra item is removed or the due date or state of the extra item is changed
   */
  onFullRefreshNeeded: function onFullRefreshNeeded_List(){},

  onBlur: function onBlur_List(){
    if (!this._keepSavedFocus) {
      this._clearSavedFocus();
    }
  },

  /*
   * event triggered if list has no more visible element to set the focus to
   */
  onFocusNeeded: function onFocusNeeded_List(){},

  /**
   * Save the information about the focus, reset saved focus if focus is not on a task row or category
   */
  saveFocus: function saveFocus_List(){
    com.ibm.bpm.social.util.common.Log.logExit(2);
    var focus = null;
    var foc = dijit.getFocus();
    var focusedNode = null;
    if (foc.node) {
      focusedNode = foc.node;
    }
    else if (foc.bookmark && foc.bookmark.mark && foc.bookmark.mark.startContainer) {
      focusedNode = foc.bookmark.mark.startContainer;
    }
    if (focusedNode) {
      focus = {};
      var widget = dijit.getEnclosingWidget(focusedNode);
      if (widget) {
        if (widget instanceof com.ibm.bpm.social.widgets.task.list.TaskRow && widget.model && widget.model.TKIID) {
          // task row widget
          focus.TKIID = widget.model.TKIID;
          focus.categoryId = widget.model.categoryId;
          var previousNode = widget.domNode.previousSibling;
          var nextNode = widget.domNode.nextSibling;
          if (previousNode) {
            widget = dijit.getEnclosingWidget(previousNode);
            if (widget && widget.model && widget.model.TKIID) {
              focus.previousTKIID = widget.model.TKIID;
            }
          }
          else {
            focus.previousTKIID = null;
          }
          if (nextNode) {
            widget = dijit.getEnclosingWidget(nextNode);
            if (widget && widget.model && widget.model.TKIID) {
              focus.nextTKIID = widget.model.TKIID;
            }
          }
          else {
            focus.nextTKIID = null;
          }
          com.ibm.bpm.social.util.common.Log.log(3, "Focused item is task " + focus.TKIID);
        }
        else if (widget instanceof com.ibm.bpm.social.widgets.task.list.Category) {
          // category Widget
          focus.categoryId = widget.id;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused item is category " + focus.categoryId);
        }
      }
    }

    // save the focus settings
    if (!focus || (!focus.TKIID && !focus.categoryId)) {
      focus = null;
    }
    var filterId = this._getFilterId();
    this._focus[filterId] = focus;

    com.ibm.bpm.social.util.common.Log.logExit(2, filterId + ": " + dojo.toJson(this._focus[filterId]));
  },

  /**
   * Restore the focus only if row was the focused row or if the focused group has been hidden
   * 
   * @param row the row widget which has been moved or removed
   * @param categoryWidget the group widget in which the row was before move or remove
   * @returns {Boolean} true if focus has been restored
   */
  _restoreFocusIfNeeded: function _restoreFocusIfNeeded_List(row, categoryWidget){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var focusRestored = false;
    var savedFocus = this._getSavedFocus();
    if (savedFocus) {
      if ((row && savedFocus.TKIID === row.model.TKIID) ||
        (categoryWidget && savedFocus.categoryId === categoryWidget.id && !categoryWidget.isDisplayed())) {
        focusRestored = this.restoreFocus(true);
      }
      else {
        this._clearSavedFocus();
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, focusRestored);
    return focusRestored;
  },

  /**
   * Restore the focus if possible and scroll the focused item into the view
   * <p>
   * Same task row if it still exists <br>
   * Next task row in same group if it (still) exists <br>
   * Previous task row in same group if it (still) exists <br>
   * Next group header if available <br>
   * Previous group header if available
   * <p>
   * saved focus is updated if changed<br>
   * saved focus is cleared if it cannot be restored or if not in autoRefresh mode
   * 
   * @param {Boolean} autoRefresh The saved focus is cleared if we are not in autoRefresh mode, e.g. on return
   *        from Task Completion View, Instance View
   * @returns {Boolean} true if focus has been restored
   */
  restoreFocus: function restoreFocus_List(autoRefresh){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var focusRestored = false;
    var focus = this._getSavedFocus();
    var saveNewFocus = true;
    if (focus) {
      com.ibm.bpm.social.util.common.Log.log(3, dojo.toJson(focus));
      var focusNode = null;
      if (focus.TKIID) {
        var row = this._getTaskWidget(focus.TKIID);
        if (row) {
          // same row
          focusNode = row.focusNode;
          saveNewFocus = false;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is same task " + focus.TKIID);
        }
      }
      if (!focusNode && focus.nextTKIID) {
        row = this._getTaskWidget(focus.nextTKIID);
        if (row) {
          // next row
          focusNode = row.focusNode;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is next task " + focus.nextTKIID);
        }
      }
      if (!focusNode && focus.previousTKIID) {
        row = this._getTaskWidget(focus.previousTKIID);
        if (row) {
          // previous row
          focusNode = row.focusNode;
          com.ibm.bpm.social.util.common.Log.log(3, "Focused node is previous task " + focus.previousTKIID);
        }
      }
      if (!focusNode && focus.categoryId) {
        // find category widget
        var categoryWidget = this._getCategoryWidget(focus.categoryId);
        if (categoryWidget != null) {
          if (categoryWidget.isDisplayed()) {
            focusNode = categoryWidget.focusNode;
            saveNewFocus = (focus.TKIID !== undefined);
            com.ibm.bpm.social.util.common.Log.log(3, "Focused node is actual group " + focus.categoryId);
          }
          else {
            // find next displayed category
            var widget = null;
            var node = categoryWidget.domNode.nextSibling;
            while (node) {
              widget = dijit.getEnclosingWidget(node);
              if (widget.isDisplayed()) {
                focusNode = widget.focusNode;
                com.ibm.bpm.social.util.common.Log.log(3, "Focused node is next displayed group " + widget.id);
                break;
              }
              node = node.nextSibling;
            }
            if (node === null) {
              // find previous displayed category
              node = categoryWidget.domNode.previousSibling;
              while (node) {
                widget = dijit.getEnclosingWidget(node);
                if (widget.isDisplayed()) {
                  focusNode = widget.focusNode;
                  com.ibm.bpm.social.util.common.Log.log(3, "Focused node is previous displayed group " + widget.id);
                  break;
                }
                node = node.previousSibling;
              }
            }
          }
        }
      }
      if (focusNode !== null) {
        // set focus to node
        dojo.window.scrollIntoView(focusNode);
		// Defect 135428 - IE 8 unable to focus on nodes that are hidden, display = none
		try {
			focusNode.focus();
			focusRestored = true;
		} catch(e) {
			com.ibm.bpm.social.util.common.Log.log(3, "Unable to focus on node");
			// focus could not be restored, let the container set the focus
			this.onFocusNeeded();
		}
      }
      else {
        // focus could not be restored, let the container set the focus
        com.ibm.bpm.social.util.common.Log.log(3, "No node found to focus!");
        this.onFocusNeeded();
      }
    }
    else {
      // no saved focus, focus was somewhere else in the list - do not change the list focus
      com.ibm.bpm.social.util.common.Log.log(3, "No saved list focus - keep the actual focus");
    }
    // update saved focus
    if (!focusRestored || !autoRefresh) {
      this._clearSavedFocus();
    }
    else if (saveNewFocus) {
      this.saveFocus();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, focusRestored);
    return focusRestored;
  },

  _getSavedFocus: function _getSavedFocus_List(){
    var filterId = this._getFilterId();
    com.ibm.bpm.social.util.common.Log.log(2, filterId);
    return this._focus[filterId];
  },

  _clearSavedFocus: function _clearSavedFocus_List(){
    var filterId = this._getFilterId();
    com.ibm.bpm.social.util.common.Log.log(2, filterId);
    this._focus[filterId] = null;
  },

  setKeepSavedFocus: function setKeepSavedFocus_List(keep){
    com.ibm.bpm.social.util.common.Log.log(2, keep);
    this._keepSavedFocus = keep;
  },

  _addOpacitySettings: function _addOpacitySettings_List(node){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (dojo.isIE < 9) {
      dojo.style(node, "filter", "0");
    }
    else {
      dojo.style(node, "opacity", "0");
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _removeOpacitySettings: function _removeOpacitySettings_List(node){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (dojo.isIE < 9) {
      dojo.style(node, "filter", null);
    }
    else {
      dojo.style(node, "opacity", null);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }
});