// SearchBar.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.SearchBar");

dojo.require("dijit.form.TextBox");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.list.SearchBar",
    [dijit._Widget,
     dijit._Templated],
    {
      widgetsInTemplate: true,
      templateString: '<div class="bpm-task-list-content-hide" dojoAttachPoint="searchBar"  tabindex="-1">'
        + '<div class="bpm-task-list-header-search" dojoAttachPoint="searchBarTable">'
        + '<span class="bpm-social-search-tf-left">'
        + '<div dojoAttachPoint="submitButton" class="bpm-social-magnify" role="button" tabindex="-1">'
        + '<span class="bpm-social-a11y">&#8629;</span></div>'
        + '</span>'
        + '<span class="bpm-social-search-tf-center">'
        + '<label for="${id}_input" style="display: none" dojoAttachPoint="textBoxLabel"></label>'
        + '<input id="${id}_input" name="search" dojoAttachPoint="textBox" class="bpm-social-search-tf-center" type="text" />'
        + '</span>'
        + '<span class="bpm-social-search-tf-right">'
        + '<div dojoAttachPoint="clearButton" class="bpm-social-close-x bpm-task-list-content-hide"  role="button" tabindex="0">'
        + '<span class="bpm-social-a11y">x</span></div>' + '</span>' + '</div>' + '<div class="nofloat">' + '</div>'
        + '</div>',

      templatePath: null,

      /**
       * postCreate lifecycle function
       * 
       */
      postCreate: function postCreate_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this.clearButton.disabled = true;
        dijit.setWaiState(this.clearButton, 'disabled', 'true');
        this._enableClear = false;
        this._visible = false;
        this.textBoxLabel.innerHTML = this._nlsResources["task-list-search-bar-input-label"];
        dijit.setWaiState(this.textBox, "label", this._nlsResources["task-list-search-bar-input-label"]);
        this.submitButton.title = this._nlsResources["task-list-search-bar-submit-tooltip"];
        //com.ibm.bspace.common.util.ui.TextField.wrapElement(this.textBox);
        // TODO: send search request
        this._connectHandlers = [];

        //configure BTD here
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.textBox);

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * show the search bar
       * 
       */
      show: function show_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this._visible) {
          return;
        }
        this._visible = true;

        this._connectHandlers = [];

        this._connectHandlers.push(dojo.connect(this.submitButton, "onclick", this, function(){
          this._submit();
        }));
        this._connectHandlers.push(dojo.connect(this.submitButton, "onkeypress", this, function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this._submit();
          }
        }));

        this._connectHandlers.push(dojo.connect(this.clearButton, "onclick", this, function(){
          this._clear();
        }));
        this._connectHandlers.push(dojo.connect(this.clearButton, "onkeypress", this, function(evt){
          if (evt.keyCode == dojo.keys.ENTER) {
            this._clear();
          }
        }));

        this._connectHandlers.push(dojo.connect(this.textBox, "onkeydown", this, function(evt){
          this._handleKeyEvent(evt);
        }));

        dojo.removeClass(this.searchBar, "bpm-task-list-content-hide");
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Do not show the search bar, if not supported
       * 
       * @type (void)
       */
      hide: function hide_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        // TODO : how to remove ? Only hide
//        if (this.domNode) {
//          this.domNode.parentNode.removeChild(this.domNode);
//        }
        this._visible = false;
        dojo.addClass(this.searchBar, "bpm-task-list-content-hide");
        if (this._connectHandlers) {
          for ( var i = 0; i < this._connectHandlers.length; i++) {
            dojo.disconnect(this._connectHandlers[i]);
          }
        }
        this._connectHandlers = null;
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Callback to send the submit
       * 
       * @type (void)
       */
      onSubmit: function onSubmit_SearchBar(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.log(2, "search text: " + text);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback to send the clear
       * 
       * @type (void)
       */
      onClear: function onClear_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * destroy the widgets {void} nothing
       */
      destroy: function destroy_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this._connectHandlers) {
          for ( var i = 0; i < this._connectHandlers.length; i++) {
            try {
              dojo.disconnect(this._connectHandlers[i]);
            }
            catch (e) {
              com.ibm.bpm.social.util.common.Log.logError(2, "destroy_SearchBar", e);
            }
          }
          this._connectHandlers = null;
        }
        //com.ibm.bspace.common.util.ui.TextField.unwrapElement(this.textBox);
        this.inherited('destroy', arguments);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Handle the key event for the input field. For key pressed enter send the submit event.
       * 
       * @param {Object} event
       */
      _handleKeyEvent: function _handleKeyEvent_SearchBar(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
          dojo.stopEvent(evt);
          this._submit();
        }
      },

      /**
       * clear the entry in the input field and send a refresh event
       */
      _clear: function _clear_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.textBox.value = "";
        this.textBox.focus();
        this._lastSearchText = null;
        this.onClear();

        this.clearButton.title = "";
        this.clearButton.disabled = true;
        dijit.setWaiState(this.clearButton, "disabled", "true");
        dojo.addClass(this.clearButton, "bpm-task-list-content-hide");
        this._enableClear = false;
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Show the clear button if we have a text in the text field
       * 
       * @type {void}
       */
      _enableClearButton: function _enableClearButton_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.clearButton.title = this._nlsResources["task-list-search-bar-clear-tooltip"];
        this.clearButton.disabled = false;
        dijit.setWaiState(this.clearButton, 'disabled', 'false');
        dojo.removeClass(this.clearButton, "bpm-task-list-content-hide");
        this._enableClear = true;
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Get the data from the text input field and send the search event
       * 
       */
      _submit: function _submit_SearchBar(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._isValid()) {
          this._lastSearchText = this._reworkText(this.textBox.value);
          this.onSubmit(this._lastSearchText);
          if (!this._enableClear) {
            this._enableClearButton();
          }
        }
        else if (this._lastSearchText) {
          // search is not valid if the input field is empty, clear the search if there was one performed
          this._clear();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Set the text from outside
       * 
       * @param {String} text the sreach text to display
       * @type {void}
       */
      setSearchText: function setSearchText_SearchBar(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "text to display: " + text);
        this.textBox.value = text;
        if (!this._enableClear) {
          this._enableClearButton();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Check for the text field to be valid
       */
      _isValid: function _isValid_SearchBar(){
        if (this.textBox && this.textBox.value !== "") {
          return true;
        }
        return false;
      },

      /**
       * Make sure no single quotes are in the search string, then the last one is escaped
       * 
       * @param text
       * @returns {string} text
       */
      _reworkText: function _reworkText_SearchBar(text){
        // do not fix the search string but provide an error message now
        var newText = text;
        if (text) {
          newText = dojo.string.trim(text);
        }

        return newText;
      }

    });
