// Table.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 
dojo.provide("com.ibm.bpm.social.widgets.task.list.Table");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bspace.common.util.ui.CheckBox");
dojo.require("com.ibm.bspace.common.util.ui.Helper");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.list.Table", [dijit._Widget,
                                                            dijit._Templated],
  {

    templateString: dojo.cache("com.ibm.bpm.social.widgets.task", "list/templates/Table.html"),

    MAX_CELL_WIDTH: 150,
    CELL_PADDING: 13,

    /**
     * constructor
     * 
     * @param {Object} parameters members: config, model
     */

    constructor: function constructor_Table(parameters){


      this._sizeCache = {
        ScrollbarSize: 0,
        RowHeight: 0,
        HeaderHeight: 0,
        LeftWidth: 0,
        RightWidth: 0,
        ScrollerWidth: 0
      };

      this.scrollbarSize = this.getScrollbarSize();
      // private members

      //this._groupHeight = 0;
      // table DOM elements
      this._dnTableHeader = null;
      this._columns = []; // store this._config.displayedProperties
//        this._selectAllCheckbox = null;
//        this._showSelectAllCheckbox = true;
      this._filterOnProperties = {}; // keep track of filtered properties
      this._config = null; // Object specifying our layout and supportede DND types
      this._model = []; // List of JSON objects, each forming a row
      this._configurationApplied = false; // indicate if the current config has been set and rendered
      this._currentSelection = -1;
      //this._groupHeaders = []; // keep track of pseudo rows (=Group headers)
      this._lastPublishedSelection = [];
      this._lastChecked = null;
      this._lastOver = null;

      this._availableTableSpace = null; // set from the outside, is the max size the table can fill with data
      this._modelRepresentation = []; // List of JSON objects specifying the elements that belong to one row.
      // One row in a model is represented by multiple elements in the table. Here they are connected to access
      // easily all elements of a specific row.
      //  {dataRow:    <tr DOM node in data table>
      //   checkboxRow: <tr DOM node in checkbox table>
      //   checkbox:  <checkbox for that row>
      //   actionRow: <tr DOM node in that row>
      //   actions:  <list of inline actions in that row>
      //   isGroupHeader: <flag for group header>,
      //   offset:    <group header offset> // group headers are in the local model,
      //                                    // but not in the list, this is the offset
      //                    // of the accumulated group headers   }
      this._tableConnects = new Array();
      this._tableContentConnects = new Array();
      //this._groupHeaderOffset = -1;
      this.focusedRow = -1;
      this.focusedColumn = 0;
      this.focusNode = null;

      this.maxCellWidth = this.MAX_CELL_WIDTH;

      // cache the config & model if we got one
      if (parameters) {
        if (parameters.config) {
          this._config = parameters.config;
        }
        if (parameters.model) {
          this._model = parameters.model;
        }
      }
      this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    },


    /**
     * postCreate
     */
    postCreate: function postCreate_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (dojo.isIE && (dojo.isIE == 7)) {
        this.Center.style.width = "10000px";
      }


      this._dnTableHeader = document.createElement('tr');
      dijit.setWaiRole(this._dnTableHeader, "row"); // a11y
      this._dnHeaderBody.appendChild(this._dnTableHeader);
//        if (!this._showSelectAllCheckbox) {
//          dojo.addClass(this.CheckboxTableBody, 'bSpaceTableNoCheckBox');
//        }

      // perform actions on the objects we got in the constructor
      if (this._config) {
        this.setConfig(this._config);
        if (this._model && this._model.length > 0) {
          this.setModel(this._model);
        }
      }

      this._sizeCache.TableHeight = 300; // ensure non-zero defaults
      this._sizeCache.TableWidth = 300; // ensure non-zero defaults

// will be done in cleanup        
//        if (!com.ibm.bpm.social.util.common.Util.isIPad) {
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseout", this, "_onTableMouseOut"));
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseover", this, "_onTableMouseOver"));
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClick"));
//        }
//        else {
//          this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClickDelayed"));
//        }
//        this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mousedown", this, "_onTableMouseDown"));
//        this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onkeydown", this, "_onRowKeyEvent"));


//        this._dndConnects = new Array();
//        this._dndConnects.push(dojo.subscribe("/dnd/drop", dojo.hitch(this, this._onDnDNotification)));
//        this._dndConnects.push(dojo.subscribe("/dnd/start", dojo.hitch(this, this._onDnDStart)));

      this._lastOver = null;
      this.HeaderRightFocusNode.innerHTML = this._nlsResources["task_actions_title"];
      this.HeaderRightFocusNode.title = this._nlsResources["task_actions_title"];
      dijit.setWaiState(this.domNode, "label", this._nlsResources["task_saved_searches_table_description"]);

      // maintain focus in table content when focus gets outside of content but inside of table
      this._tableConnects.push(dojo.connect(this.Table, "onfocus", this, "_onOuterTableFocus"));
      this._tableConnects.push(dojo.connect(this.VScrollerContainer, "onfocus", this, "_onOuterTableFocus"));
      this._tableConnects.push(dojo.connect(this.Center, "onfocus", this, "_onOuterTableFocus"));

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    ///////////////////////////////////////
    // public methods / Renderer interface
    ///////////////////////////////////////

    /**
     * Set the model(data) to be displayed and triggers a refresh
     * 
     * @param {Array} items Array of items
     * @param {Boolean} callRenderTable implicitly call renderTable
     * @type {void}
     */
    setModel: function setModel_Table(items, callRenderTable){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");
      try {
        this._model = items;

        this._cleanupTableContent();

        var i;
        var wasDirty = false;

        if (this._model.length > 0) {
          // check if we must render the header also
          if (this._config) {
            if (this._configurationApplied === false) {
              this._setColumns(this._config.displayedProperties);
              this._configurationApplied = true;
              wasDirty = true;
            }

            //this._model = this._prepareModelForGroupBy(items);

//            if (this._selectAllCheckbox) {
//              com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//            }
            //this._initializeGroupFormatter();

//            var itemsCreatedByDND = false;

//            if (!itemsCreatedByDND) {
            for (i = 0; i < this._model.length; i++) {
              this._renderRow(i);
            }
//            }

            // synchronize the row heights
            var rows = this._modelRepresentation;
            if (rows.length > 0) {
              // take largest row height
              this._sizeCache.RowHeight = Math.max(rows[0].dataRow.offsetHeight, rows[0].actionRow.offsetHeight);
              com.ibm.bpm.social.util.common.Log.log(1, "this._sizeCache.RowHeight=" + this._sizeCache.RowHeight + "px");
//            if (dojo.isFF || (dojo.isIE) || dojo.isChrome) {
              for ( var i = 0; i < rows.length; i++) {
                var row = rows[i];
                var height = 0;
//                    if (row.isGroupHeader) {
//                      height = this.getGroupHeaderHeight();
//                    }
//                    else {
                height = this._sizeCache.RowHeight;
                row.dataRow.style.height = height + 'px';
//                    }
//                    if (row.checkboxRow) {
//                      row.checkboxRow.style.height = height + 'px';
//                    }
                row.actionRow.style.height = height + 'px';

              }
            }
//                else if (dojo.isIE && dojo.isIE <= 7) {
//                  // this is a bit complicated in IE since the table has not much interest in the height settings
//                  for (i = 0; i < rows.length; i++) {
//                    row = rows[i];
//                    height = -1;
////                    if (row.isGroupHeader) {
////                      if (i < rows.length - 1) {
////                        height = this.getGroupHeaderHeight(); //  + 1;
////                      }
////                    }
////                    else {
//                      height = this._sizeCache.RowHeight;
//                      height -= 3;
////                    }
//                    if (height >= 0) {
//                      row.dataRow.style.height = height + 'px';
//                      if (row.checkboxRow) {
//                        row.checkboxRow.style.height = height + 'px';
//                      }
//                      row.actionRow.style.height = height + 'px';
//                    }
//                  }
//                }
//                else {
//                  // webkit goes here
//                  for (i = 0; i < rows.length; i++) {
//                    row = rows[i];
//                    height = 0;
//                    //if (dojo.hasClass(rows[i], "groupBanner")) {
//                    if (row.isGroupHeader) {
//                      height = this.getGroupHeaderHeight();
//                      if (row.checkboxRow) {
//                        row.checkboxRow.style.height = height + 'px';
//                      }
//                      row.actionRow.style.height = height + 'px';
//                    }
//                }
//          }

            // Reset sort if setModel w/o setConfig is called
            if (wasDirty === false) {
              for (i = 0; i < this._config.displayedProperties.length; i++) {
                var prop = this._config.displayedProperties[i];
                prop.isSort = false;
                for ( var j = 0; j < this._config.sortProperties.length; j++) {
                  if (this._config.sortProperties[j].property == prop.id) {
                    prop.isSort = true;
                  }
                }
              }
            }
            this._setSortIndicators();

            // if search called in the meantime, display it now
            for (i in this._filterOnProperties) {
              if (i) {
                this.setSearchActive(i, true);
              }
            }

//            this.setLastFocus()
            if (this._refreshedBySort === true) {
              this._setFocusInHeader(true);
              this._refreshedBySort = false;

            }
            else {
              // set initial focus
              this._setFocusInHeader(false);
            }
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(1, "called with empty config -> noop!");
          }

//          if (this._config.maintainSelection) {
//            this._checkSelectAllStatus();
//            this._lastPublishedSelection = this.getCheckedData();
//          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(2, "no items");
        }

        if (callRenderTable) {
          this._afterRendering(true);
        }
      }
      catch (xcp) {
        com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
        throw (xcp);
      }
      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },


//    setLastFocus: function setLastFocus_Table(){
//      // set focus to filter icon in header of the last filter property
//      // otherwise we would lose the focus because the header is destroyed when refreshing
//      if (this.lastFilterProperty) {
//        var column = this._getColumnForProperty(this.lastFilterProperty);
//        if (column) {
//          var gif = dojo.query(".magnifier_hidden", column);
//          if (gif && gif.length > 0) {
//            window.setTimeout(function(){
//              gif[0].focus();
//            }, 100);
//          }
//        }
//        this.lastFilterProperty = null;
//      }
//    },


    /**
     * 
     * @type {void}
     */
    setSortProperties: function setSortProperties_Table(props){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (props) {
        this._config.sortProperties = props;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2);
      return;
    },

    /**
     * Set the Layout config of the table Setting the config purges the content
     * 
     * @param {Object} config
     */

    setConfig: function setConfig_Table(config){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");
      try {
        if (config) {
          this._config = config;

//            // ASZ@MoveToSocial prevent DND
//            this._config.supportedDndTypes = "";
//            this._config.publishedDndTypes = "";

          // ASZ@MoveToSocial prevent Grouping
          //this._config.activeGroupByProperty = null;

          if (!this._config.sortProperties) {
            this._config.sortProperties = [];
          }

          this._filterOnProperties = {};
          this._configurationApplied = false; // render header on setModel to prevent dirty display
          // ensure we have a dndSource but create only a new DnDSource if something has changed
//            if (!this._isPrototypTable) {
//              this._createDndTarget();
//              this._createDndSource();
//            }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1, "empty config received!");
        }
      }
      catch (xcp) {
        com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
        throw (xcp);
      }

      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },

    /**
     * Resize the table
     * 
     * @param {Object} dimensons JSON object {"h" : number, "w" : number }
     */
    resize: function resize_Table(dimensons){
      com.ibm.bpm.social.util.common.Log.logEntry(1, "", "Table");

      if (!dimensons) {
        return;
      }
      // initially we get a resize, but table sizes are not set.
      if (!this._sizeCache.LeftWidth || !this._sizeCache.RightWidth || !this._sizeCache.ScrollerWidth) {
        this._setTableSize();
      }
      if (dimensons.w && (dimensons.w > -1)) {
        var centerWidth =
          dimensons.w - this._sizeCache.LeftWidth - this._sizeCache.RightWidth - this._sizeCache.ScrollerWidth;
        if (centerWidth > this._sizeCache.CenterWidth) {
          // enlarge outer div first 
          this.domNode.style.width = dimensons.w + "px";
          this.Center.style.width = centerWidth + 'px';
        }
        else {
          // reduce inner div first
          this.Center.style.width = centerWidth + 'px';
          this.domNode.style.width = dimensons.w + "px";
        }

        this._sizeCache.CenterWidth = centerWidth;
        this._sizeCache.TableWidth = dimensons.w;
        com.ibm.bpm.social.util.common.Log.log(1, "New width " + dimensons.w + "px");
      }

      if (dimensons.h && (dimensons.h > -1)) {
        this.domNode.style.height = dimensons.h + "px";
        this._sizeCache.TableHeight = dimensons.h;
        com.ibm.bpm.social.util.common.Log.log(1, "New height " + dimensons.h + "px");
      }
      com.ibm.bpm.social.util.common.Log.logExit(1, "", "Table");
    },

    getScrollbarSize: function getScrollbarSize_Table(){

      var size;

      if ((size = this._sizeCache.ScrollbarSize)) {
        return size;
      }

      var div = document.createElement("div");
      div.style.visibility = "hidden";
      div.style.overflow = "scroll";
      div.style.position = "absolute";
      div.style.height = 100 + 'px';
      div.style.width = 100 + 'px';
      div.style.top = "-1000px";
      div.style.left = "-1000px";
      document.body.appendChild(div);
      size = div.offsetWidth - div.clientWidth;
      document.body.removeChild(div);
      this._sizeCache.ScrollbarSize = size + 1;
      // need to add 1px, otherwise it does not work
      // in some environments
      return size;
    },

//      /**
//       * Return the item displayed at index rowIndex (count starting at 0)
//       * 
//       * @type {void}
//       */
//      deselectAll: function deselectAll_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // check if we need to do anything
//        if (this.getCheckedData().length > 0) {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//          this._selectAllCheckboxClicked();
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//
//      },


    /**
     * activate / highlight rows (different to highlight from mouseclick) <br>
     * NOTE: should only be called with the indices from the original model that does not include any group
     * headers.
     * 
     * @param {Array} indices
     * @param {Boolean} switchOff if present, remove highlight instead of adding.
     */
    highlightItems: function highlightItems_Table(indices, switchOff){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var row = null;

      if (this._model.length > 0) {
        if (indices) {
          if (!(indices instanceof Array)) {
            indices = [indices];
          }

          for ( var i = 0; i < indices.length; i++) {
            // the index comes from the list and does not include the group headers.
            // thus we have to add an offset for potential group headers
            var index = (this._modelRepresentation[indices[i]].offset) + (indices[i]);
            if (index < this._modelRepresentation.length) {
              row = this._modelRepresentation[index];
              // For group headers we need to move one further
              //while (this._modelRepresentation.length - 1 > index && row.isGroupHeader) {
              while (this._modelRepresentation.length - 1 > index) {
                index++;
                row = this._modelRepresentation[index];
              }
              if (this._modelRepresentation.length > index && row !== null) {
                if (!switchOff) {
                  dojo.addClass(row.dataRow, "taskActive");
//                    if (this._config.isSelectable !== false) {
//                      dojo.addClass(row.checkboxRow, "taskActive");
//                    }
                  dojo.addClass(row.actionRow, "taskActive");
                }
                else {
                  dojo.removeClass(row.dataRow, "taskActive");
//                    if (this._config.isSelectable !== false) {
//                      dojo.removeClass(row.checkboxRow, "taskActive");
//                    }
                  dojo.removeClass(row.actionRow, "taskActive");
                }
              }
            }
          }
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /*
     * Delete all rows in the table
     */
    _cleanupTableContent: function _cleanupTableContent_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      // remove all event handlers on the table content
      this._lastOver = null;

      //this._groupHeaderOffset = -1;

      if (this._tableContentConnects !== null) {
        for ( var k = 0; k < this._tableContentConnects.length; k++) {
          if (this._tableContentConnects[k][0] != this._dnTableBody) {
            // remove all connections except the body which we can't purge because auf DnD
            dojo.disconnect(this._tableContentConnects[k]);
          }
        }
      }

      // Uncheck the row, otherwise some visual check indicator might remain.
      //        for(var i=0; i<this._modelRepresentation.length; i++){
      //          if (this._modelRepresentation.checkbox){
      //             com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._modelRepresentation.checkbox, false);
      //            }
      //        }
      // TODO : AB make sure all widgets are deleted. The grouping elements include dojo widgets.!!!!
      // remove the representation layer
      if (this._modelRepresentation) {
        delete this._modelRepresentation;
        this._modelRepresentation = [];
      }

      // delete all the data from the table
      // but we must keep the table body, because the DnDSource is bound to it!
      this.DataTable.removeChild(this._dnTableBody);
      this._dnTableBody = document.createElement('tbody');
      this.DataTable.appendChild(this._dnTableBody);

//        if (this.dndSource) {
//          var nodes = this.dndSource.getAllNodes();
//          for ( var i = 0; i < nodes.length; i++) {
//            this._dnTableBody.removeChild(nodes[i]);
//          }
//          this.dndSource.clearItems();
//        }

//        // cleanup any items not listed as DnD items
      while (this._dnTableBody.hasChildNodes()) {
        this._dnTableBody.removeChild(this._dnTableBody.firstChild);
      }

      this.ActionTable.removeChild(this.ActionTableBody);
      this.ActionTableBody = document.createElement('tbody');
      this.ActionTable.appendChild(this.ActionTableBody);

//        var tbody = this.CheckboxTable.removeChild(this.CheckboxTableBody);
//        var checkboxes = dojo.query("input", tbody);
//        if (checkboxes) {
//          for (i = 0; i < checkboxes.length; i++) {
//            com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(checkboxes[i]);
//          }
//        }


//        this.CheckboxTableBody = document.createElement('tbody');
//        this.CheckboxTable.appendChild(this.CheckboxTableBody);

      // create the event handlers again for the table content
      if (!com.ibm.bpm.social.util.common.Util.isIPad) {
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "mouseover", this, "_onTableMouseOver"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "mouseover", this, "_onTableMouseOver"));
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "mouseout", this, "_onTableMouseOut"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "mouseout", this, "_onTableMouseOut"));
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onclick", this, "_onTableClick"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onclick", this, "_onTableClick"));
      }
      else {
        this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onclick", this, "_onTableClickDelayed"));
//          this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onclick", this, "_onTableClickDelayed"));
      }
/////////////////////////////////////////////////
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseout", this, "_onTableMouseOut"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "mouseover", this, "_onTableMouseOver"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onclick", this, "_onTableClick"));
      this._tableContentConnects.push(dojo.connect(this._dnTableBody, "onkeydown", this, "_onKeyEvent"));
/////////////////////////////////////////////////

      this._tableContentConnects.push(dojo.connect(this.ActionTableBody, "onkeydown", this, "_onKeyEvent"));
//        this._tableContentConnects.push(dojo.connect(this.CheckboxTableBody, "onkeydown", this, "_onRowKeyEvent"));

      // event handlers for header rows
      this._tableContentConnects.push(dojo.connect(this.HeaderTable, "onkeydown", this, "_onKeyEvent"));
      this._tableContentConnects.push(dojo.connect(this.HeaderRight, "onkeydown", this, "_onKeyEvent"));
      this._tableContentConnects.push(dojo.connect(this.HeaderTable, "onclick", this, "_onHeaderClick"));
      this._tableContentConnects.push(dojo.connect(this.HeaderRight, "onclick", this, "_onHeaderClick"));

      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    /**
     * this function is not available for the table, a message should be displayed for a task
     */
    showMessage: function showMessage_Table(message, taskID){
      com.ibm.bpm.social.util.common.Log.logEntry(1);

      com.ibm.bpm.social.util.common.Log.logError(1, "This function is not available for the Table.");

      com.ibm.bpm.social.util.common.Log.logExit(1);
    },

    _getModelIndex: function _getModelIndex_Table(itemIndex){
      var numItems = Number(itemIndex) + 1;
      var count = 0;
      var modelIndex = 0;

      while (count < numItems) {
//          if (!(this._model[modelIndex] instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader)) {
//            count++;
//          }
        if (count < numItems) {
          modelIndex++;
        }
      }

      return modelIndex;
    },

    ///////////////////////////////////////
    // public events
    // (to connect to using dojo.connect)
    ///////////////////////////////////////

    /**
     * Sort triggered by user
     * 
     * @param {String} property column / property used as sort criteria
     * @param {String} direction one of {ASC|DESC}
     */
    onSort: function onSort_Table(property, direction){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. Sort on '" + property + "', " + direction);
    },


    /**
     * User dropped one (ToDo: or more?) item onto the table
     * 
     * @param {Array} objects list of objects that are dropped
     * @param {String} type the type of the drop
     */
    onDrop: function onDrop_Table(objects, type){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. dropped " + objects.length + " of type " + type);
    },


    /**
     * User clicked on an action icon
     * 
     * @param {String} action the id of the action clicked
     * @param {Object} item the item
     */
    onActionRequested: function onActionClicked_Table(action, item){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired. Action : " + action);

    },


    /**
     * User selected one or more items <br>
     * Note: use the wrapper _onSelectionChanged to prevent double event emission.
     * 
     * @param {Array} items an array of 0...n items
     */
    onSelectionChanged: function onSelectionChanged_Table(items){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired, selected items:" + items.length);
    },


    /**
     * onFocusChanged
     * 
     * @param {Object} item
     */
    onFocusChanged: function onFocusChanged_Table(item){
      com.ibm.bpm.social.util.common.Log.log(1, "event fired");
      com.ibm.bpm.social.util.common.Log.logObject(3, item);
    },


    /**
     * onFilter
     * 
     * @param {Object} property
     * @param {Object} node the dom node
     */
    onFilter: function onFilter_Table(property, node){
      this.lastFilterProperty = property;
      com.ibm.bpm.social.util.common.Log.log(1, "event fired: property=" + property);
    },

    ////////////////////////////////
    /* End of Renderer interface  */
    ////////////////////////////////
    /**
     * Activate the magnifier icon on a column
     * 
     * @param {Object} property
     * @param {Object} active optional, if missing, set search active for property
     */
    setSearchActive: function setSearchActive_Table(property, active){
      com.ibm.bpm.social.util.common.Log.log(1, "called: property=" + property + ", active=" + active);

      var column = null;
      var gifs = null;

      if (!property) {
        com.ibm.bpm.social.util.common.Log.log(1, "Error, no property received !");
        return;
      }

      if (active === null) {
        active = false;
      }

      column = this._getColumnForProperty(property);
      if (column) {
        gifs = dojo.query(".magnifier_hidden", column);
      }
      else {
        com.ibm.bpm.social.util.common.Log.log(1, "Error, no column found for property !");
      }

      // class for the magnifier icon
      if (gifs && gifs[0]) {
        if (active) {
          dojo.addClass(gifs[0], 'magnifier_active');
          this._filterOnProperties[property] = true;
        }
        else {
          dojo.removeClass(gifs[0], 'magnifier_active');
          delete (this._filterOnProperties[property]);
        }
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    /* Helpers for the controller */

    hasHScrollbar: function hasHScrollbar_Table(plannedWidth){
      var width = this._sizeCache.TableWidth;
      if (plannedWidth) {
        width = plannedWidth;
      }
      var leftWidth = 26;
      var rightWidth = 26;
      var hasHScroller = (this.DataTable.offsetWidth > width - leftWidth - rightWidth - this.scrollbarSize);

      var hScroll = hasHScroller;
      com.ibm.bpm.social.util.common.Log.log(1, "returns " + hScroll);
      return hScroll;

    },

    hasVScrollbar: function hasVScrollbar_Table(){
      var height = this._sizeCache.TableHeight;
      var width = this._sizeCache.TableWidth;

      var headerHeight = this._sizeCache.HeaderHeight;
      if (!this._sizeCache.HeaderHeight) {
        this._sizeCache.HeaderHeight = headerHeight;
      }

      var leftWidth = 26;
      var rightWidth = 26;
      var hasHScroller = (this.DataTable.offsetWidth > width - leftWidth - rightWidth - this.scrollbarSize);
      var scrollerHeight = hasHScroller ? this.scrollbarSize : 0;
      var hasVScroller = (this.DataTable.offsetHeight > height - headerHeight - scrollerHeight);

      var vScroll = hasVScroller;
      com.ibm.bpm.social.util.common.Log.log(1, "returns " + vScroll);
      return vScroll;

    },

    getTableHeights: function getTableHeights_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");


      var rowHeight = this.getRowHeight();
      com.ibm.bpm.social.util.common.Log.log(1, "rowHeight=" + rowHeight);

      var headHeight = this.getHeaderHeight();
      com.ibm.bpm.social.util.common.Log.log(1, "headHeight=" + headHeight);

      //var groupHeight = this.getGroupHeaderHeight();
      //com.ibm.bpm.social.util.common.Log.log(1, "groupHeight=" + groupHeight);


      var _tableHeights = {
        headHeight: headHeight,
        //groupHeight: groupHeight,
        rowHeight: rowHeight
      };


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return _tableHeights;
    },


    getHeaderHeight: function getHeaderHeight_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var height;
      if (this._sizeCache.HeaderHeight) {
        height = this._sizeCache.HeaderHeight;
      }
      else {
        if (!this._configurationApplied) {
          this._prototypeTable();
        }
        height = this.HeaderTable.offsetHeight;
      }
      if (!height || height < 10) {
        height = 26;
      }
      this._sizeCache.HeaderHeight = height;

      com.ibm.bpm.social.util.common.Log.logExit(2, "HeaderHeight is " + height);
      return height;
    },


    getRowHeight: function getRowHeight_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var height = null;

      if (this._sizeCache.RowHeight) {
        height = this._sizeCache.RowHeight;
      }
      else {
        if (this._modelRepresentation && this._modelRepresentation.length > 0) {
          height = this._modelRepresentation[0].dataRow.offsetHeight;
        }
        else {
          this._prototypeTable();
          if (this._modelRepresentation && this._modelRepresentation.length > 0) {
            height = this._modelRepresentation[0].dataRow.offsetHeight;
          }
          // remove any prototype rows
          //AB: A complete cleanup is necessary
          this._cleanupTableContent();
        }
      }
      if (!height || height < 10) {
        // ensure a number, just a default
        height = 25;
      }

      this._sizeCache.RowHeight = height;
      com.ibm.bpm.social.util.common.Log.logExit(2, "RowHeight is :" + height + "px");
      return height;
    },

//      getGroupHeaderHeight: function getGroupHeaderHeight_Table(){
//
//        if (this._config && this._config.groupFormatter) {
//          this._groupHeight = this._config.groupFormatter.getHeaderHeight();
//        }
//        return this._groupHeight;
//      },

    /**
     * calculate how many rows will fit in the given height
     * 
     * @param {Number} height
     */
    getMaxRowCount: function getMaxRowCount_Table(height){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      // calculate the maximal number of rows new, because available space changed.
      // As long as the height does not change, the max row count will not change.
      if (!this._availableTableSpace || height != this._availableTableSpace) {
        var tableHeights = this.getTableHeights();
        var headHeight = this._sizeCache.HeaderHeight;

        if (!this._sizeCache.HeaderHeight) {
          this._sizeCache.HeaderHeight = this.HeaderTable.offsetHeight;
        }
        var rowHeight = tableHeights.rowHeight;
//          var groupHeight = 0;
//          if (this._config && this._config.activeGroupByProperty) {
//            // we have at least one group header
//            groupHeight = tableHeights.groupHeight;
//          }
        com.ibm.bpm.social.util.common.Log.log(1, "headHeight=" + headHeight + "; rowHeight=" + rowHeight);

//          var availableRowSpace = height - headHeight - groupHeight - 1; // - 1 since the first row has an additional 1px border-top
        var availableRowSpace = height - headHeight - 1; // - 1 since the first row has an additional 1px border-top
        com.ibm.bpm.social.util.common.Log.log(1, "availableRowSpace (after subtracting headHeight)=" + availableRowSpace);

        // FLM: if we have a HScroller we will most likely have one every page
        var scrollHeight = this.hasHScrollbar() ? this.scrollbarSize : 0;
        this._maxRowCount = Math.floor((availableRowSpace - scrollHeight) / (rowHeight));
        // we need to calculate the max row count new, when we do not have actual data right now.
        // because there can be an horizontal scrollbar later
        if (this._modelRepresentation.length > 0 && !this._isPrototypTable) {
          this._availableTableSpace = height;
        }
      }
      com.ibm.bpm.social.util.common.Log.log(1, "exit, maxRowCount=" + this._maxRowCount);
      return this._maxRowCount;
    },

    /*
     *  Calculate how many rows fit the space really. Max Row Count is a guess, when the
     *  model is not set. But with the model the real number can be calculated. Because
     *  for grouping the number of fitting rows can be less then max row count.
     */
    getRowCount: function getRowCount_Table(height, removeFromTop){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var rowCount = 0;
      // if we have grouping and not a prototype table and an actual model representation
//        if (this._config && this._config.activeGroupByProperty && !this._isPrototypTable &&
      if (this._config && !this._isPrototypTable && this._modelRepresentation.length > 0) {
        var tableHeights = this.getTableHeights();
        var headHeight = this._sizeCache.HeaderHeight;
        if (!this._sizeCache.HeaderHeight) {
          this._sizeCache.HeaderHeight = this.HeaderTable.offsetHeight;
        }
        var rowHeight = tableHeights.rowHeight;
        // FLM: if we have a HScroller we will most likely have one every page
        var scrollHeight = this.hasHScrollbar() ? this.scrollbarSize : 0;
        var availableRowSpace = height - scrollHeight - headHeight - 1; // - 1 since the first row has an additional 1px border-top
        // calculate the group header height
        //var groupHeight = tableHeights.groupHeight;

        // we have at least one group header and not a prototype table
        // now start calculating the number of rows.
        if (!removeFromTop) {
          // Start at the top of the list to check, how many elements fit into the
          // table. The elements at the end of the list that don't fit are removed.
          var i = 0;
          while (availableRowSpace > 0 && i < this._modelRepresentation.length) {
//              if (this._modelRepresentation[i].isGroupHeader) {
//                availableRowSpace = availableRowSpace - groupHeight;
//              }
//              else {
            availableRowSpace = availableRowSpace - rowHeight;
            rowCount++;
//              }
            i++;
          }
          // If there is not enough space for the last row, then reduce the row count.
          // A group header does not count, but we have to make sure to call the remove later in the entityList
          // Thus the posRow is used instead for the correct position.
//            if (availableRowSpace < 0 && rowCount > 0 && !this._modelRepresentation[i - 1].isGroupHeader) {
          if (availableRowSpace < 0 && rowCount > 0) {
            rowCount--;
          }
          // now remove the elements between i and modelRepresentation.length, thus store the pos, we can not
          // remove here, because that is called from entity list.
          // We can delete this direction quite easy, but the other one we can not. So this
          // direction will be deleted by the table removeLastRows and the other by a new setModel call.
          i--;
          this._posRemove = i;
        }
        else {
          // start at the bottom of the list and check how many items fit into the table
          // everything above that are removed.
          i = this._modelRepresentation.length - 1;
          while (i > 0 && availableRowSpace > 0) {
//              if (this._modelRepresentation[i].isGroupHeader) {
//                if (availableRowSpace - groupHeight > 0) {
//                  availableRowSpace = availableRowSpace - groupHeight;
//                }
//                else {
//                  // should never happen
//                  if (rowCount > 0) {
//                    rowCount--;
//                  }
//                  availableRowSpace = 0;
//                }
//              }
//              else {
            if (availableRowSpace - rowHeight > 0) {
//                if (availableRowSpace - groupHeight - rowHeight > 0) {
              availableRowSpace = availableRowSpace - rowHeight;
              rowCount++;
            }
            else {
              // if the predecessor is a group header, then we can just stop here
              availableRowSpace = 0;
//                  if (rowCount > 0 && !this._modelRepresentation[i + 1].isGroupHeader) {
              if (rowCount > 0) {
                rowCount--;
              }
            }
//              }
            i--;
          }
        }
      }
      else {
        rowCount = this.getMaxRowCount(height);
      }
      com.ibm.bpm.social.util.common.Log.log(1, "exit, rowCount=" + rowCount + "  pos " + i);
      return rowCount;

    },


    /*
     *  Remove the specified number of rows.
     */
    removeLastRows: function(count){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var len = this._modelRepresentation.length;
      var pos = this._posRemove;
      var i, k;
      this._posRemove = null;
      // if we have stored the exact position from where to delete, then use this one. Because then
      // we do not need to look for the last element.
      if (pos > 0) {
        // delete the rows but not the group header, if there are more elements still for this group
        for (k = pos; k < len; k++) {
          var row = this._modelRepresentation.pop();
          this._model.pop();
//            if (row.isGroupHeader) {
//              // remove the last element from the array
//              this._groupHeaders.pop();
//            }
          this._dnTableBody.removeChild(row.dataRow);
          this.ActionTableBody.removeChild(row.actionRow);
//            this.CheckboxTableBody.removeChild(row.checkboxRow);
        }
      }
      else {
        // use the count to delete the rows
        k = 0; // count the model elements
        i = 0; // count removed rows
        while (i < count && k < len) {
          row = this._modelRepresentation.pop();
          this._model.pop();
//            if (row.isGroupHeader) {
//              // remove the last element from the array
//              this._groupHeaders.pop();
//            }
//            else {
          k++;
//            }
          i++;
          this._dnTableBody.removeChild(row.dataRow);
          this.ActionTableBody.removeChild(row.actionRow);
//            this.CheckboxTableBody.removeChild(row.checkboxRow);
        }
      }

//        // check if the last element in the table is now a group header
//        var last = this._modelRepresentation.length - 1;
//        if (last >= 0 && this._modelRepresentation[last].isGroupHeader) {
//          row = this._modelRepresentation.pop();
//          this._model.pop();
//          // remove the last element from the array
//          this._groupHeaders.pop();
//          this._dnTableBody.removeChild(row.dataRow);
//          this.ActionTableBody.removeChild(row.actionRow);
//          this.CheckboxTableBody.removeChild(row.checkboxRow);
//        }


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },
    ///////////////////////////////////////
    // dojo lifecycle functions
    ///////////////////////////////////////
    /**
     * Call this method to trigger rendering after model has been set. Call is also after the table has been
     * modified.
     */
    renderTable: function renderTable_Table(delayedExecution){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (delayedExecution === null || delayedExecution === undefined) {
        delayedExecution = true;
      }
      this._afterRendering(delayedExecution);
      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    _afterRendering: function _afterRendering_Table(delayedExecution){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var self = this;


      if (delayedExecution) {
        // do calculation in new thread.
        // this gives the browser a chance to render the table before measurement takes place.
        // neede for iPad support but does not seem to harm other platforms
        window.setTimeout(function(){
          self._afterRendering(false);
        }, 1);
        com.ibm.bpm.social.util.common.Log.logExit(2, "delayedExecution, exiting w/o actions", "Table");
        return;
      }


      if (!this._tableHeights) {
        this.getTableHeights();
      }

      this.maxCellWidth =
        Math.max(this.MAX_CELL_WIDTH, Math.floor(this._sizeCache.CenterWidth / this._config.displayedProperties.length));
      com.ibm.bpm.social.util.common.Log.log(2, "maxCellWidth is now " + this.maxCellWidth);


      if (this._modelRepresentation && this._modelRepresentation.length > 0) {
        this._handleCellWidth();
      }

      this._setTableSize();

      if (this._modelRepresentation && this._modelRepresentation.length > 0) {
        //if (rows.length > 0) {
        this._handleLargeCells();
      }

      if (!this.vScrollEvent) {
        if (com.ibm.bpm.social.util.common.Util.isIPad) {
          this.BodyCenter.style.overflowY = "auto";
          this.vScrollEvent = dojo.connect(this.BodyCenter, 'onscroll', this, "_moveVerticalTouch");
        }
        else {
          this.vScrollEvent = dojo.connect(this.VScrollerContainer, 'onscroll', this, "_moveVertical");
        }
      }

      if (!this.hScrollEvent) {
        this._lastHScrollLeft = this.Center.scrollLeft;
        this.hScrollEvent = dojo.connect(this.Center, 'onscroll', this, "_moveHorizontal");
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _moveVertical: function _moveVertical_Table(e){
      var y = this.VScrollerContainer.scrollTop;
      // check focus if view is scrolled
      com.ibm.bpm.social.util.common.Log.log(3, "scrollTop " + y + ", DataTable.offsetTop " + this.DataTable.offsetTop);
      var checkFocus = this.DataTable.offsetTop !== -y;
      this.DataTable.style.top = -y + 'px';
      this.ActionTable.style.top = -y + 'px';
//        this.CheckboxTable.style.top = -y + 'px';
      if (checkFocus) {
        this._focusNextVisibleRow();
      }
    },

    _moveVerticalTouch: function _moveVerticalTouch_Table(e){
      var y = this.BodyCenter.scrollTop;
      com.ibm.bpm.social.util.common.Log.log(3, "scrollTop " + y + ", ActionTable.offsetTop " + this.ActionTable.offsetTop);
      var checkFocus = this.ActionTable.offsetTop !== -y;
      //this.DataTable.style.top = -y + 'px';
      this.ActionTable.style.top = -y + 'px';
//        this.CheckboxTable.style.top = -y + 'px';
      if (checkFocus) {
        this._focusNextVisibleRow();
      }
    },

//    /**
//     * Move the viewport so that the last item is shown
//     * 
//     * @type {void}
//     */
//    scrollToLastItem: function scrollToLastItem_Table(){
//      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//      var delta = this.VScrollerContainer.scrollHeight - this.VScrollerContainer.clientHeight;
//      this.DataTable.style.top = -delta + 'px';
//      this.ActionTable.style.top = -delta + 'px';
////        this.CheckboxTable.style.top = -delta + 'px';
//      this.VScroller.style.top = -delta + 'px';
//      this.VScrollerContainer.style.top = -delta + 'px';
//      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      return;
//    },

    /**
     * Change the focus if the focused row scrolls out of the visible area
     */
    _focusNextVisibleRow: function _focusNextVisibleRow_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusedRow !== -1) {
        com.ibm.bpm.social.util.common.Log.log(3, "focused row " + this.focusedRow);
        var index = this.focusedRow;
        var scrollHeight = this.Table.offsetHeight - this.HeaderTable.offsetHeight;
        var focusedRow = this._modelRepresentation[index].dataRow;
        var focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
        var focusedRowBottom = focusedRowTop + focusedRow.offsetHeight;
        com.ibm.bpm.social.util.common.Log.log(3, "scrollHeight " + scrollHeight + ", focusedRowTop " + focusedRowTop +
          ", focusedRowBottom " + focusedRowBottom);
        if (focusedRowBottom > scrollHeight) {
          // set focus to previous visible row
          while (focusedRowBottom > scrollHeight) {
            index--;
            focusedRow = this._modelRepresentation[index].dataRow;
            focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
            focusedRowBottom = focusedRowTop + focusedRow.offsetHeight;
          }
          this._setFocusedRow(this.focusedRow, index);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to row " + index);
        }
        else if (focusedRowTop <= 0) {
          // set focus to next visible row
          while (focusedRowTop < 0) {
            index++;
            focusedRow = this._modelRepresentation[index].dataRow;
            focusedRowTop = focusedRow.offsetTop + this.ActionTable.offsetTop;
          }
          this._setFocusedRow(this.focusedRow, index);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to row " + index);
        }
        else {
          this._setFocus(false);
        }
      }
      else {
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _moveHorizontal: function _moveHorizontal_Table(e){
      // check focus if view is scrolled
      com.ibm.bpm.social.util.common.Log.log(3, "lastHScrollLeft " + this._lastHScrollLeft + " scrollLeft " +
        this.Center.scrollLeft);
      if (this._lastHScrollLeft !== this.Center.scrollLeft) {
        this._focusNextVisibleColumn();
        this._lastHScrollLeft = this.Center.scrollLeft;
      }
    },

    /**
     * Change the focus if the focused column scrolls out of the visible area
     */
    _focusNextVisibleColumn: function _focusNextVisibleColumn_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "focused row " + this.focusedRow + " focusedColumn " + this.focusedColumn);
      if (this.focusedColumn < this._getChildElementCount(this.HeaderTable.rows[0])) {
        // focused column is in scrollable view
        var focusedRow = null;
        if (this.focusedRow === -1) {
          focusedRow = this.HeaderTable.rows[0];
        }
        else {
          focusedRow = this._modelRepresentation[this.focusedRow].dataRow;
        }
        var index = this.focusedColumn;
        var focusColumn = focusedRow.childNodes[index];
        var scrollLeft = this.Center.scrollLeft;
        var scrollRight = scrollLeft + this.Center.offsetWidth;
        var focusColumnLeft = focusColumn.offsetLeft;
        var focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
        com.ibm.bpm.social.util.common.Log.log(3, "scrollLeft " + scrollLeft + " scrollRight " + scrollRight +
          " focusColumnLeft " + focusColumnLeft + " focusColumnRight " + focusColumnRight);
        if (this._lastHScrollLeft < scrollLeft && focusColumnLeft < scrollLeft) {
          // left of visible area - move focus to the right
          while (focusColumnLeft < scrollLeft) {
            index++;
            focusColumn = focusedRow.childNodes[index];
            focusColumnLeft = focusColumn.offsetLeft;
            //focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
          }
          this.focusedColumn = index;
          this._setFocusToNode(focusColumn.focusNode, true);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to column " + index);
        }
        else if (this._lastHScrollLeft > scrollLeft && focusColumnRight > scrollRight) {
          // right of visible area - move focus to the left            
          while (focusColumnRight > scrollLeft + this.Center.offsetWidth) {
            index--;
            focusColumn = focusedRow.childNodes[index];
            //focusColumnLeft = focusColumn.offsetLeft;
            focusColumnRight = focusColumn.offsetLeft + focusColumn.offsetWidth;
          }
          this.focusedColumn = index;
          this._setFocusToNode(focusColumn.focusNode, true);
          com.ibm.bpm.social.util.common.Log.log(2, "focus moved to column " + index);
        }
        else {
          this._setFocus(false);
        }
      }
      else {
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _setTableSize: function _setTableSize_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var width = this._sizeCache.TableWidth;
      var height = this._sizeCache.TableHeight;

//        for ( var i = 0; i < this._groupHeaders.length; i++) {
//          // hide group Headers since they disrupt measuring
//          this._groupHeaders[i].style.display = "none";
//        }


      this.BodyCenter.style.width = "";
      this.BodyCenter.style.height = "";
      this.DataTable.style.display = "inline-table";
      this.HeaderTable.style.display = "inline-table";
      this.DataTable.style.width = "";
      // for some reason this does not work in FF
      if ((dojo.isIE && (dojo.isIE < 8))) {
        this.DataTable.style.height = "";
      }
      this.HeaderTable.style.width = "";
      this.HeaderTable.style.height = "";

      // start measuring
      var headerHeight = this.HeaderTable.offsetHeight;
      this._sizeCache.HeaderHeight = headerHeight;

      var dataWidth = this.DataTable.firstChild.offsetWidth;
      //var dataHeight = this.DataTable.firstChild.offsetHeight + this._groupHeaders.length * this._groupHeight;
      var dataHeight = this.DataTable.firstChild.offsetHeight;
      var headerWidth = this.HeaderTable.offsetWidth;
      var leftWidth = 0; // this.HeaderLeft.offsetWidth;
      var rightWidth = this.ActionTable.offsetWidth;

      com.ibm.bpm.social.util.common.Log.log(2, "Measures: dataHeight:" + dataHeight + "px, dataWidth:" + dataWidth +
        "px, headerWidth:" + headerWidth + "px, headerHeight:" + headerHeight + "px, leftWidth:" + leftWidth +
        "px, rightWidth:" + rightWidth + "px");

      // end measuring


      // some sanity checks
      if (rightWidth > 100) {
        // to wide, make no sense
        rightWidth = 28;
      }
      else if (rightWidth < 20) {
        // to small, no inline action at all 
        rightWidth = 0;
      }

//        // left contains only checkboxes, they cant be bigger than 30 (?) px
//        if (leftWidth > 30) {
//          leftWidth = 24;
//        }
//
//        // if we have no checkboxes at all, don't show the left side at all
//        if (this._config.isSelectable === false) {
      leftWidth = 0;
//        }

//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          if (headerHeight > 25) {
//            headerHeight = 20;
//          }
//        }
      // }

      this.DataTable.style.display = "block";
      this.HeaderTable.style.display = "block";


      // if the "no items" message is shown
      dataWidth = Math.max(dataWidth, headerWidth);
      headerWidth = dataWidth;
      this._sizeCache.ContentWidth = dataWidth;

      this.DataTable.style.width = dataWidth + 'px';

//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          this.DataTable.style.height = dataHeight + 'px';
//          this.ActionTable.style.height = dataHeight + 'px';
////          this.CheckboxTable.style.height = dataHeight + 'px';
//        }


      this.HeaderTable.style.width = dataWidth + 'px';

      this.BodyCenter.style.width = dataWidth + 'px';


      if (this._config.fitToContent) {
        width = dataWidth + leftWidth + rightWidth + 2;
        height = dataHeight + headerHeight + 2;
        this.domNode.style.width = width + 'px';
        this.domNode.style.height = height + 'px';
        this._sizeCache.TableHeight = height;
        this._sizeCache.TableWidth = width;
      }

      var hasHScroller = (dataWidth > width - leftWidth - rightWidth - this.scrollbarSize);
      var scrollerHeight = hasHScroller ? this.scrollbarSize : 0;

      var hasVScroller = (dataHeight > height - headerHeight - scrollerHeight);
      var scrollerWidth = hasVScroller ? this.scrollbarSize : 0;

      if (this._config.fitToContent) {
        scrollerHeight = 0;
        scrollerWidth = 0;
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "Sizes before calculating centerWidth: width:" + width +
        "px, leftWidth:" + leftWidth + "px, rightWidth:" + rightWidth + "px, scrollerWidth:" + scrollerWidth + "px");
      var centerWidth = width - leftWidth - rightWidth - scrollerWidth;

      this._sizeCache.CenterWidth = centerWidth;

      if (!hasVScroller) {
        this.VeryRight.style.display = "none";
        this.DataTable.style.top = '0px';
        this.ActionTable.style.top = '0px';
//          this.CheckboxTable.style.top = '0px';
      }
      else {
        this.VeryRight.style.display = "block";
      }

      this.Center.style.height = height + 'px';
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        this.BodyCenter.style.height = (height - headerHeight) + 'px';
      }
      // this.Left.style.width = leftWidth + 'px';
      this.Right.style.width = rightWidth + 'px';
      this.Center.style.width = centerWidth + 'px';
      this.VeryRight.style.width = scrollerWidth + 'px';
      this.VScroller.style.width = scrollerWidth + 'px';
      this.VScrollerContainer.style.width = (scrollerWidth + 1) + 'px';
      this.VScroller.style.height = dataHeight + 'px';
      this._appendOrResizeVScrollerImage(dataHeight, scrollerWidth);

//        if (leftWidth > 2 && headerHeight > 2) {
//          this.HeaderLeft.style.width = (leftWidth - 2) + 'px'; // -2 is the border
//          this.HeaderLeft.style.height = (headerHeight - 2) + 'px';
//        }
//        else {
//          this.HeaderLeft.style.width = '0px';
//          this.HeaderLeft.style.height = '0px';
//        }

      if (rightWidth > 2 && headerHeight > 2) {
        this.HeaderRight.style.width = (rightWidth - 2) + 'px';
        // this.HeaderRight.style.height = (headerHeight - 2) + 'px';
        if (dojo.isFF) {
          this.HeaderRight.style.height = headerHeight + 'px'; // bug: using "display: table-cell" reduces height in FF   
        }
        else {
          this.HeaderRight.style.height = (headerHeight - 2) + 'px';
        }

      }
      else {
        this.HeaderRight.style.width = '0px';
        this.HeaderRight.style.height = '0px';
      }

      this.HeaderVeryRight.style.width = (scrollerWidth > 2 ? scrollerWidth - 2 : scrollerWidth) + 'px'; // 1 px padding left and right
      this.HeaderVeryRight.style.height = (headerHeight > 2 ? headerHeight - 2 : headerHeight) + 'px'; // 1 px padding top and bottom
      // this.BodyLeft.style.width = leftWidth + 'px';
      this.BodyRight.style.width = rightWidth + 'px';

      var newHeight = height - headerHeight - scrollerHeight;
      if (newHeight < 0) {
        newHeight = 0;
      }
      // this.BodyLeft.style.height = newHeight + 'px';
      this.BodyRight.style.height = newHeight + 'px';
      if (newHeight > 2) {
        // HeaderVeryRight placeholder has 2px padding
        this.VScrollerContainer.style.height = (newHeight - 2) + 'px';
      }
      else {
        this.VScrollerContainer.style.height = newHeight + 'px';
      }

      this._sizeCache.LeftWidth = leftWidth;
      this._sizeCache.CenterWidth = centerWidth;
      this._sizeCache.RightWidth = rightWidth;
      this._sizeCache.ScrollerWidth = scrollerWidth;


      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return true;
    },

    // get first row, that is not a group header
    _getFirstDataRow: function(rows){
      var k = 0;
//        while (rows[k].isGroupHeader === true && k < rows.length) {
//          k++;
//        }
//        if (k >= rows.length) {
//          return null;
//        }
//        else {
      return rows[k].dataRow;
//        }
    },

    _handleCellWidth: function _handleCellWidth_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var widthMap = [];
      this.actualWidthMap = [];
      var cell, node, headerCells, headerText, t, i, row;

      var rows = this._modelRepresentation;
      var headerRow = this._dnHeaderBody.firstChild;
      // determine size
      // Each column in the data table has now the size to fit the content, 
      // so we need to measure only the first rows cells


      var padding = this.CELL_PADDING;

      if (rows.length > 0) {
        // find the first row with real data in it
        var dataRow = this._getFirstDataRow(rows);
        if (dataRow !== null) {

          // iterate over all columns of the first row
          for (t = 0; t < dataRow.childNodes.length; t++) {

            var cellWidth = dataRow.childNodes[t].offsetWidth;
            if (cellWidth < padding) {
              // avoid problems with negative values
              cellWidth = padding;
            }

            if (cellWidth - padding > this.maxCellWidth) {
              // the column is wider than max size, we need to backup its original size
              // for use in _handleLargeCells later
              for (i = 0; i < rows.length; i++) {
                row = rows[i];
//                  if (!row.isGroupHeader) {
                cell = row.dataRow.childNodes[t];
                node = cell.firstChild;
                if (node) {
                  var width = cellWidth - padding;
                  if (dataRow !== row) {
                    width = node.offsetWidth; // this saves one set of offsetWidth
                  }
                  cell.originalWidth = width;
                  if (!widthMap[t] || width > widthMap[t]) {
                    widthMap[t] = width;
                  }
                  if (widthMap[t] > this.maxCellWidth) {
                    widthMap[t] = this.maxCellWidth;
                  }
                }
//                  }
              }
            }
            else {
              if (!widthMap[t] || cellWidth - padding > widthMap[t]) {
                widthMap[t] = cellWidth - padding;
              }
            }
          }
        }
      } // end if (rows.length > 0)

      // determine header width
      for (t = 0; t < headerRow.childNodes.length; t++) {
        cell = headerRow.childNodes[t];
        node = cell.firstChild;
        if (node.tagName == "TABLE") {
          // in webkit only the table body has a width, not the table itself
          node = node.firstChild;
        }
        if (node) {
          width = node.offsetWidth - 2;
          if (width < 0) {
            // avoid problems with negative sizes
            width = 0;
          }
          cell.originalWidth = width;
          if (!widthMap[t] || width > widthMap[t]) {
            widthMap[t] = width;
          }
        }
      }

      // set size
      // header
      for (t = 0; t < headerRow.childNodes.length; t++) {
        cell = headerRow.childNodes[t];
        node = cell.firstChild;
        if (node && dojo.hasClass(cell, "tableCell")) {
          width = widthMap[t];
          // save the actual size for restoring after refresh
          this.actualWidthMap[t] = width;
          node.style.width = (width + 2) + 'px';
          node.style.display = "block";

          // handle large text
          headerCells = node.getElementsByTagName("td");
          if (headerCells && headerCells.length > 2) {
            headerText = headerCells[1].firstChild;
            if (headerText) {
              if (width < 32) {
                // avoid problems with negative sizes
                width = 32;
              }
              if (this._config.isFilterable) {
                // cell width - sort div - filter div
                headerText.style.width = (width - 32) + "px";
              }
              else {
                //  filter div is hidden
                headerText.style.width = (width - 16) + "px";
              }
            }
          }
        }
      }

      // data table
      for (i = 0; i < rows.length; i++) {
        row = rows[i];
//          if (!row.isGroupHeader) {
        for (t = 0; t < row.dataRow.childNodes.length; t++) {
          cell = row.dataRow.childNodes[t];
          node = cell.firstChild;
          if (node && dojo.hasClass(cell, "tableData")) {
            width = widthMap[t];
            if (width > 0) {
              node.style.width = width + 'px';
              node.style.display = "block";
              if (dojo.hasClass(cell, "bSpaceBlankImageHolder")) {
                node.style.height = '1px';
              }
            }
          }
//            }
        }
      }

      this.widthMap = widthMap;
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * If we have free horizontal space give it to all cells who need it. If the cell is still to large, make
     * an ellipsis.
     */
    _handleLargeCells: function _handleLargeCells_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      // do we have free space?
      var freeSpace = this._sizeCache.CenterWidth - this.DataTable.offsetWidth;
      var initialFreeSpace = freeSpace;
      var widthMap = this.widthMap;
      var monsterCells = [];
      var monsterHeaders = [];

      // handle too large cells
      var headerRow = this._dnHeaderBody.firstChild;
      var rows = this._modelRepresentation;

      for ( var t = 0; t < headerRow.childNodes.length; t++) {
        var addWidth = 0;

        if (widthMap[t] > this.maxCellWidth) {
          addWidth = Math.min(widthMap[t] - this.maxCellWidth, freeSpace);
          if (addWidth < 0) {
            addWidth = 0;
          }
          if (addWidth < Math.min(widthMap[t] - this.maxCellWidth)) {
            // check for monstercells
            // check in header
            var cell = headerRow.childNodes[t];
            var node = cell.firstChild;
            if (cell.originalWidth > this.maxCellWidth + addWidth) {
              node.maxWidth = widthMap[t] + this.CELL_PADDING;
              monsterCells.push(node);
            }
            // check in data cells
            for ( var i = 0; i < rows.length; i++) {
//                if (!rows[i].isGroupHeader) {
              //if (!dojo.hasClass(rows[i], "groupBanner")) {
              cell = rows[i].dataRow.childNodes[t];
              //var cell = rows[i].childNodes[t];
              node = cell.firstChild;
              if (cell.originalWidth > this.maxCellWidth + addWidth) {
                node.maxWidth = widthMap[t] + this.CELL_PADDING;
                monsterCells.push(node);
              }
//                }
            }
          }
          freeSpace = freeSpace - addWidth;
        }
        if (freeSpace > 0) {
          if (t == headerRow.childNodes.length - 1) {
            addWidth += freeSpace;
            freeSpace = 0;
          }
        }
        if (addWidth > 0) {
          // add free space to header
          cell = headerRow.childNodes[t];
          node = cell.firstChild;
          var width = Math.min(widthMap[t], this.maxCellWidth) + addWidth;
          this.actualWidthMap[t] = width;
          node.style.width = (width + 2) + 'px';

          // handle large text
          var headerCells = node.getElementsByTagName("td");
          if (headerCells && headerCells.length > 2) {
            var headerText = headerCells[1].firstChild;
            if (headerText) {
              if (width < 32) {
                // avoid problems with negative sizes
                width = 32;
              }
              if (this._config.isFilterable) {
                // cell width - sort div - filter div
                headerText.style.width = (width - 32) + "px";
              }
              else {
                //  filter div is hidden
                headerText.style.width = (width - 16) + "px";
              }
            }
          }

          // add free space to data cell
          for (i = 0; i < rows.length; i++) {
//              if (!rows[i].isGroupHeader) {
            cell = rows[i].dataRow.childNodes[t];
            node = cell.firstChild;
            width = Math.min(widthMap[t], this.maxCellWidth) + addWidth;
            node.style.width = width + 'px';
          }
//            }
        }
      }

      // add ellipsis
      for ( var k = 0; k < monsterCells.length; k++) {
        cell = monsterCells[k];
        this._addEllipsis(cell);
      }

      // size headers
//        if (rows.length > 0 && this._config.activeGroupByProperty) {
//          var groupHeaders = this._groupHeaders;
//          var style = (dojo.isIE ? "block" : "table-row");
//          for (k = 0; k < groupHeaders.length; k++) {
//            groupHeaders[k].style.display = style;
//            var gWidth = Math.max(this._sizeCache.CenterWidth, this._sizeCache.ContentWidth);
//            this._config.groupFormatter.resizeWidth(groupHeaders[k].firstChild.firstChild, gWidth);
//          }
//        }
      if (initialFreeSpace > 0) {
        this.DataTable.style.width = this._sizeCache.CenterWidth + 'px';
        this.BodyCenter.style.width = this._sizeCache.CenterWidth + 'px';
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _addEllipsis: function _addEllipsis_Table(cell){
      var cellWidth = cell.offsetWidth;
      var parentTd = cell.parentNode;
      // Typically, ellipsis is done via css. Only older brwosers don't support this
      if (dojo.isFF && (dojo.isFF < 7) && dojo.hasClass(parentTd, "tableData")) {
        // don't do it twice
        if (cell.tagName != "TABLE") {
          // Set display back to block, otherwise the width setting would not be used.
          var inlineTable = document.createElement("table");
          dijit.setWaiRole(inlineTable, "presentation"); //a11y
          inlineTable.style.tableLayout = "fixed";
          inlineTable.style.display = "block";
          inlineTable.style.width = cellWidth + "px";
          inlineTable.style.borderCollapse = "collapse";
          inlineTable.style.borderWidth = 0;
          parentTd.replaceChild(inlineTable, cell);
          var row = inlineTable.insertRow(0); //a11y: none
          var content = row.insertCell(0); //a11y: none
          content.vAlign = "top";
          content.style.padding = 0;
          cell.style.width = (cellWidth - 10) + "px";
          content.appendChild(cell);
          var ellipsis = row.insertCell(1);
          ellipsis.vAlign = "top";
          ellipsis.style.padding = 0;
          ellipsis.innerHTML = "...";
        }
        else {
          cell.style.width = cellWidth + "px";
          content = cell.firstChild.firstChild.firstChild.firstChild;
          content.style.width = (cellWidth - 10) + "px";
        }
      }
    },

    /**
     * destroy
     */
    destroy: function destroy_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      this._lastOver = -1;
      this._currentSelection = -1;

//        if (this._dndConnects) {
//          for ( var i = 0; i < this._dndConnects.length; i++) {
//            dojo.unsubscribe(this._dndConnects[i]);
//          }
//          delete this._dndConnects;
//          this._dndConnects = null;
//        }

      if (this.vScrollEvent) {
        dojo.disconnect(this.vScrollEvent);
        this.vScrollEvent = null;
      }
      if (this.hScrollEvent) {
        dojo.disconnect(this.hScrollEvent);
        this.hScrollEvent = null;
      }
      if (this._tableConnects !== null) {
        for ( var k = 0; k < this._tableConnects.length; k++) {
          dojo.disconnect(this._tableConnects[k]);
        }
        delete this._tableConnects;
        this._tableConnects = null;
      }

      if (this._tableContentConnects !== null) {
        for ( var k = 0; k < this._tableContentConnects.length; k++) {
          dojo.disconnect(this._tableContentConnects[k]);
        }
        delete this._tableContentConnects;
        this._tableContentConnects = null;
      }

      // TODO : AB make sure all widgets are deleted. The grouping elements include dojo widgets.!!!!
      // remove the representation layer
      if (this._modelRepresentation) {
        delete this._modelRepresentation;
        // AB: do not set it to null, because maybe there are still events accessing the array
        this._modelRepresentation = [];
      }
      if (this._columns) {
        delete this._columns;
        this._columns = null;
      }
//        if (this._selectAllCheckbox) {
//          com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._selectAllCheckbox);
//          delete this._selectAllCheckbox;
//          this._selectAllCheckbox = null;
//        }
      if (this._model) {
        delete this._model;
        this._model = null;
      }

//        // Remove dnd objects.
//        if (this.dndSource) {
//          this.dndSource.destroy();
//          this.dndSource = null;
//        }
//        if (this.dndTarget) {
//          this.dndTarget.destroy();
//          this.dndTarget = null;
//        }

      this.inherited('destroy', arguments);
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");

    },

    //////////////////////////////////////////
    // public interface
    //////////////////////////////////////////


    /**
     * return array of data items of selected rows
     */
//      getCheckedData: function getCheckedData_Table(){
//        var checked = [];
//        var rows = this._modelRepresentation;
//        for ( var i = 0; i < rows.length; i++) {
//          var row = rows[i];
////          if (!row.isGroupHeader) {
//          if (row.checkbox && row.checkbox.checked) {
//            checked.push(this._model[i]);
//          }
////          }
//        }
//        return checked;
//      },
//      /**
//       * enable / disable select all checkbox
//       */
//      setShowSelectAllCheckbox: function setShowSelectAllCheckbox_Table(bool){
//        this._showSelectAllCheckbox = bool;
//        if (!this._showSelectAllCheckbox) {
//          dojo.addClass(this.Table, 'bSpaceTableNoCheckBox');
//        }
//        else {
//          dojo.removeClass(this.Table, 'bSpaceTableNoCheckBox');
//        }
//        if ((dojo.isIE && (dojo.isIE < 8))) {
//          // rerender
//          this._configurationApplied = false;
//          this.setModel(this._model);
//        }
//      },
    // row specifics
    /**
     * return the model/data/rows the table displays
     */
    getModel: function getModel_Table(){
      return this._model;
    },

    // some functions to move rows up/down

    rowMoveUp: function rowMoveUp_Table(rowIndex, animate){
      if (rowIndex > 0) {
        return this.swapRows(rowIndex - 1, rowIndex, animate);
      }
      else {
        return false;
      }
    },

    rowMoveDown: function rowMoveUp_Table(rowIndex, animate){
      if (rowIndex < this._model.length - 1) {
        return this.swapRows(rowIndex, rowIndex + 1, animate);
      }
      else {
        return false;
      }
    },

    /**
     * Connect to this method for notification if the row swap animation is finished. {void} nothing
     */
    onSwapFinished: function(){

    },

    swapRows: function swapRows_Table(rowIndex1, rowIndex2, animate){
//        if (rowIndex1 < 0 || rowIndex2 < 0 || rowIndex1 == rowIndex2 || rowIndex1 > this._model.length - 1 ||
//          rowIndex2 > this._model.length - 1 || this.animationInAction ||
//          this._modelRepresentation[rowIndex1].isGroupHeader || this._modelRepresentation[rowIndex2].isGroupHeader) {
      if (rowIndex1 < 0 || rowIndex2 < 0 || rowIndex1 == rowIndex2 || rowIndex1 > this._model.length - 1 ||
        rowIndex2 > this._model.length - 1 || this.animationInAction) {
        return false;
      }

      this.animationInAction = true;

      if (rowIndex1 > rowIndex2) {
        var temp = rowIndex2;
        rowIndex2 = rowIndex1;
        rowIndex1 = temp;
      }

      // unhover rows if necessary
      this._unsetHoverForRow(this._modelRepresentation[rowIndex1], rowIndex1);
      this._unsetHoverForRow(this._modelRepresentation[rowIndex2], rowIndex2);

      var actionNodes = this._modelRepresentation[rowIndex1].actions;
      var actions = this._model[rowIndex1].inlineActions;
      if (actionNodes) {
        for ( var k = 0; k < actionNodes.length; k++) {
          dojo.removeClass(actionNodes[k], actions[k].hoverClass);
        }
      }

      this._lastOver = null;

      // swap model
      var tempModelItem = this._model[rowIndex1];
      this._model[rowIndex1] = this._model[rowIndex2];
      this._model[rowIndex2] = tempModelItem;

      // swap representation
      var tempRepItem = this._modelRepresentation[rowIndex1];
      this._modelRepresentation[rowIndex1] = this._modelRepresentation[rowIndex2];
      this._modelRepresentation[rowIndex2] = tempRepItem;


      tempRepItem = this._modelRepresentation[rowIndex1];
      if (tempRepItem.actionRow) {
        this._setRowId(tempRepItem.actionRow, rowIndex1, tempRepItem.actionRow._parentId);
      }
      if (tempRepItem.dataRow) {
        this._setRowId(tempRepItem.dataRow, rowIndex1, tempRepItem.dataRow._parentId);
      }
//        if (tempRepItem.checkboxRow) {
//          this._setRowId(tempRepItem.checkboxRow, rowIndex1, tempRepItem.checkboxRow._parentId);
//        }

      tempRepItem = this._modelRepresentation[rowIndex2];
      if (tempRepItem.actionRow) {
        this._setRowId(tempRepItem.actionRow, rowIndex2, tempRepItem.actionRow._parentId);
      }
      if (tempRepItem.dataRow) {
        this._setRowId(tempRepItem.dataRow, rowIndex2, tempRepItem.dataRow._parentId);
      }
//        if (tempRepItem.checkboxRow) {
//          this._setRowId(tempRepItem.checkboxRow, rowIndex2, tempRepItem.checkboxRow._parentId);
//        }

      if (!animate) {
        this._swapRowsDOMQuick(rowIndex1, rowIndex2);
      }
      else {
        this._swapRowsDOMAnimated(rowIndex1, rowIndex2);
      }

      return true;
    },

    _swapRowsDOMQuick: function(rowIndex1, rowIndex2){
      var row1 = this._modelRepresentation[rowIndex1];
      var row2 = this._modelRepresentation[rowIndex2];

//        if (row2.checkboxRow) {
//          this.CheckboxTableBody.removeChild(row1.checkboxRow);
//          dojo.place(row1.checkboxRow, row2.checkboxRow, "replace");
//          dojo.place(row2.checkboxRow, this.CheckboxTableBody, rowIndex2);
//        }
      if (row2.dataRow) {
        this._dnTableBody.removeChild(row1.dataRow);
        dojo.place(row1.dataRow, row2.dataRow, "replace");
        dojo.place(row2.dataRow, this._dnTableBody, rowIndex2);
      }
      if (row2.actionRow) {
        this.ActionTableBody.removeChild(row1.actionRow);
        dojo.place(row1.actionRow, row2.actionRow, "replace");
        dojo.place(row2.actionRow, this.ActionTableBody, rowIndex2);
      }
      this.animationInAction = false;
      this.onSwapFinished();
    },

    _swapRowsDOMAnimated: function(rowIndex1, rowIndex2, endAt, current){
      var row1 = this._modelRepresentation[rowIndex1];
      var row2 = this._modelRepresentation[rowIndex2];
      if (!endAt) {
        endAt = row1.dataRow.offsetTop - row2.dataRow.offsetTop;
        current = 1;
      }

//        if (row1.checkboxRow) {
//          this._positionRowContent(row1.checkboxRow, -current);
//        }
      if (row1.dataRow) {
        this._positionRowContent(row1.dataRow, -current);
      }
      if (row1.actionRow) {
        this._positionRowContent(row1.actionRow, -current);
      }

      if (row2.checkboxRow) {
        this._positionRowContent(row2.checkboxRow, current);
      }
      if (row2.dataRow) {
        this._positionRowContent(row2.dataRow, current);
      }
      if (row2.actionRow) {
        this._positionRowContent(row2.actionRow, current);
      }

      if (current === 0) {
        this.animationTimer = null;
        this._swapRowsDOMQuick(rowIndex1, rowIndex2);
        return;
      }

      current = current + Math.ceil(endAt / 10);
      var self = this;
      this.animationTimer = window.setTimeout(function(){
        if (endAt - current < 0) {
          current = 0;
          // loop stops if current = 0
        }
        self._swapRowsDOMAnimated(rowIndex1, rowIndex2, endAt, current);
      }, 50);
    },

    _positionRowContent: function(row, pos){
      if (row) {
        var tds = row.childNodes;
        if (tds) {
          for ( var i = 0; i < tds.length; i++) {
            var node = tds[i].firstChild;
            if (node) {
              if (pos !== 0) {
                node.style.position = "relative";
                node.style.top = pos + "px";
              }
              else {
                node.style.position = "";
                node.style.top = "";

              }
            }
          }
        }
      }
    },

    ///////////////////////////////////////
    // internals
    ///////////////////////////////////////

    _setColumns: function _setColumns_Table(columns){
      this._columns = columns;
      while (this._dnTableHeader.firstChild) {
        this._dnTableHeader.removeChild(this._dnTableHeader.firstChild);
      }
//        while (this.HeaderLeft.firstChild) {
//          this.HeaderLeft.removeChild(this.HeaderLeft.firstChild);
//        }
      this._renderHeaderRow();
    },


    /**
     * display the sort indicator
     */
    _setSortIndicators: function _setSortIndicators_Table(){

      com.ibm.bpm.social.util.common.Log.logEntry(2);

      // sort props are given in this._config.sortProperties
      // remove sort indicators from columns
      var cols = dojo.query("div", this._dnTableHeader);
      for ( var i = 0; i < cols.length; i++) {
        dojo.removeClass(cols[i], "sortAscPrimary");
        dojo.removeClass(cols[i], "sortAscSecondary");
        dojo.removeClass(cols[i], "sortDescSecondary");
        dojo.removeClass(cols[i], "sortDescPrimary");
      }

      var column = null;
      var sortProp, div;

      if (this._config.sortProperties && this._config.sortProperties.length > 0) {

        // primary sort
        sortProp = this._config.sortProperties[0];
        if (sortProp.property) {
          column = this._getColumnForProperty(sortProp.property);
          if (column) {
            // show sort icons
            div = dojo.query(".sortHidden", column)[0];
            if (sortProp.sortDir == 'ASC') {
              div.parentNode.title = this._nlsResources["KEY_SortDirectionDesc"];
              dojo.addClass(div, 'sortAscPrimary');
            }
            else {
              div.parentNode.title = this._nlsResources["KEY_SortDirectionAsc"];
              dojo.addClass(div, 'sortDescPrimary');
            }
          }
        }

        // secondary sort
        if (this._config.sortProperties.length > 1) {
          sortProp = this._config.sortProperties[1];
          if (sortProp.property) {
            column = this._getColumnForProperty(sortProp.property);
            if (column) {
              // show sort icons
              div = dojo.query(".sortHidden", column)[0];
              if (sortProp.sortDir == 'ASC') {
                div.parentNode.title = this._nlsResources["KEY_SortDirectionDesc"];
                dojo.addClass(div, 'sortAscSecondary');
              }
              else {
                div.parentNode.title = this._nlsResources["KEY_SortDirectionAsc"];
                dojo.addClass(div, 'sortDescSecondary');
              }
            }
          }
        }
      }
    },


    /**
     * Render all cells in the header row of the table, including the checkbox row (first row)
     */
    _renderHeaderRow: function _renderHeaderRow_Table(){

      if (this._config.isSelectable !== false) {
        // first row is selectAll checkbox
        // dijit.setWaiRole(this.HeaderLeft, "gridcell"); // a11y

//          this._selectAllCheckbox = document.createElement('input');
//          dijit.setWaiRole(this._selectAllCheckbox, "checkbox"); // a11y
//          this._selectAllCheckbox.type = 'checkbox';
//          this.HeaderLeft.appendChild(this._selectAllCheckbox);

//          com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this._selectAllCheckbox);
//          this.connect(this._selectAllCheckbox, "click", "_selectAllCheckboxClicked");

        // keyboard navigation is done on the bSpace Checkbox button, which is next sibling to the real xbox
//          if (this._selectAllCheckbox.nextSibling) {
//            var button = this._selectAllCheckbox.nextSibling;
//            button.title = this._nlsResources["KEY_selectAllItems"];
//            this.focusNode = button; // focus node of the whole table
//            this.connect(button, "onfocus", "_onFocus");
//            button.tabIndex = "0"; // enable tab stop. Note: This is the only tabstop .
//          }
      }

      for ( var i = 0; i < this._columns.length; i++) {
        this._renderHeaderForColumn(this._columns[i]);
        this._dnTableHeader.lastChild._focusColumnId = i;

      }
    },

    /**
     * Reactivate last focus or set initial focus when table gets focus
     */
    onFocus: function onFocus_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusNode !== null) {
        com.ibm.bpm.social.util.common.Log.log(3, "row " + this.focusedRow + "  column " + this.focusedColumn);
        // reactivate focus node
        this._setFocusToNode(this.focusNode);
        if (this.focusedRow != -1) {
          // highlight focused row
          var dataRow = this._modelRepresentation[this.focusedRow].dataRow;
          this._onTableMouseOver({
            target: dataRow
          });
        }
      }
      else {
        // set focus node
        if (this.focusedRow >= 0) {
          this._setFocusInRow();
        }
        else {
          this._setFocusInHeader();
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

//    /**
//     * highlite a cell when navigating with keyboard
//     * 
//     * @param {Object} node
//     */
//    _focusOnNode: function _focusOnNode_Table(node){
//      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//      if (node) {
//        // changing className in FF changes focus
//        var activeDoc = document.activeElement;
//        dojo.addClass(node, "bSpaceTableOver");
//        activeDoc.focus();
//      }
//      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//    },

    /**
     * Sets the focus to the node
     * 
     * @param node new focus node
     * @param delayed delayed execution of focus setting
     */
    _setFocusToNode: function _setFocusToNode_Table(node, delayed){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (node) {
        this._setFocusNode(node);
        if (delayed) {
          window.setTimeout(dojo.hitch(this, function(){
            this._setFocus(true);
          }, 100));
        }
        else {
          this._setFocus(true);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * The focus node is the only node within the table with a tabIndex of 0. The tabIndex of the last focus
     * node must be reset before setting focus to another node.
     */
    _setFocusNode: function _setFocusNode_Table(node){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "focusedRow " + this.focusedRow + "  focusedColumn " + this.focusedColumn);
      if (node) {
        if (this.focusNode) {
          this.focusNode.tabIndex = -1;
        }
        node.tabIndex = 0;
        this.focusNode = node;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    _setFocus: function _setFocus_Table(scrollIntoView){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      if (this.focusNode) {
        if (scrollIntoView) {
          this._scrollIntoView();
        }
        this.focusNode.focus();
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * Scrolls the table cell of the focused node into the view
     */
    _scrollIntoView: function _scrollIntoView_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var focusedRow = null;
      if (this.focusedRow === -1) {
        focusedRow = this.HeaderTable.rows[0];
      }
      else {
        focusedRow = this._modelRepresentation[this.focusedRow].dataRow;
      }
      if (focusedRow) {
        var focusColumn = focusedRow.childNodes[this.focusedColumn];
        if (focusColumn) {
          dojo.window.scrollIntoView(focusColumn);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

//      /**
//       * select/deselect all row checkboxes (w/o) sending event
//       * 
//       * @param {bool} select select or unselect
//       */
//      _selectAllRowCheckboxes: function _selectAllRowCheckboxes_Table(select){
//
//        for ( var i = 0; i < this._modelRepresentation.length; i++) {
//          this._checkRow(i, select);
//        }
//        if (select === false && this._config.maintainSelection) {
//          // for maintainSelection mode, we must lowlight manually
//          this._lowLightAll();
//        }
//
//        this._currentSelection = -1;
//      },


//      /**
//       * handle the select all event
//       * 
//       * @param {Object} e not used
//       */
//      _selectAllCheckboxClicked: function _selectAllCheckboxClicked_Table(e){
//        this._selectAllRowCheckboxes(this._selectAllCheckbox.checked);
//        // fire onSelectionChanged event
//        this._onSelectionChanged(this.getCheckedData());
//      },


//      /**
//       * handle the select group event
//       * 
//       * @param {Object} index not used
//       */
//      _selectGroupCheckboxClicked: function _selectGroupCheckboxClicked_Table(index){
//        var rows = this._modelRepresentation;
//        var checkbox = rows[index].checkbox;
//        var select = checkbox.checked;
//
//        for ( var i = Number(index) + 1; i < rows.length; i++) {
//          if (rows[i].isGroupHeader) {
//            break;
//          }
//          this._checkRow(i, select);
//        }
//        this._onSelectionChanged(this.getCheckedData());
//        this._checkSelectAllStatus();
//      },

    /**
     * Set focus in table content if focus gets outside of table content, e.g. Scrollers
     */
    _onOuterTableFocus: function _onOuterTableFocus_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      com.ibm.bpm.social.util.common.Log.log(3, "activeElement.className " + document.activeElement.className);
      if (document.activeElement !== this.focusNode) {
        com.ibm.bpm.social.util.common.Log.log(3, "Setting focus to focusNode");
        this._setFocus(false);
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Set focus node in header if header is clicked
     */
    _onHeaderClick: function _onHeaderClick(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      this.focusedRow = -1;
      var headerNode = this._getParentWithClass(event.target, 'bSpaceTableHeader');
      if (!headerNode) {
        this.focusedColumn = this._getChildElementCount(this.HeaderTable.rows[0]);
        this._setFocusToNode(event.target);
      }
      else if (headerNode._focusColumnId !== undefined) {
        this.focusedColumn = headerNode._focusColumnId;
        this._setFocusToNode(headerNode.focusNode, true);
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /*
     * For iPad we combine mouseOver and onClick. We have to start a new thread to get the
     * mouseover effect rendered. Otherwise the inline action would highlight after it is processed
     * (which can be quite a while on the iPad).
     */
    _onTableClickDelayed: function _onTableClickDelayed_Table(event){
      this._onTableMouseOver(event);
      var self = this;
      setTimeout(function(){
        self._onTableClick(event);
        setTimeout(function(){
          self._onTableMouseOut(event);
        }, 1000);
      }, 100);
    },

    /**
     * Handle click event in table area. Select a row or send an action request. Sets focus to clicked cell.
     */
    _onTableClick: function _onTableClick_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no active table data");
        return;
      }

      // check for the necessary information of the event to process any further
      var node = event.target;
      if (!node) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no event target or no rowId");
        return;
      }
      if (node._focusColumnId !== undefined) {
        this.focusedColumn = node._focusColumnId;
      }
      var rowId = node["_rowId"];
      if (rowId !== undefined && rowId >= 0) {
        var row = this._modelRepresentation[rowId];

//          if (com.ibm.bpm.social.util.common.Util.isIPad) {
//            // necessary for highlighting since we disabled the over/out stuff for iPad
//            if (this._config.isSelectable !== false && dojo.hasClass(event.target, "bSpaceTableCheckBox")) {
//              // click into the checkbox cell, but checkbox is still display=none
//              node = row.checkbox;
//              com.ibm.bspace.common.util.ui.CheckBox.setChecked(row.checkbox, !row.checkbox.checked);
//            }
//          }


        // if the click was on an action, then return afterwards. There is no more to do
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          var actions = this._model[rowId].inlineActions;
          var action = actions[actionId];

          // fire only if enabled
          if (action.enabled) {
            this.onActionRequested(action.id, this._model[rowId]);
            return;
          }
        }
        // if there is no check box, then set only the focus to the row
        if (this._config.isSelectable === false) {
          this._setFocusedRow(this.focusedRow, rowId);
          return;
        }
//          // group header check box was clicked, thus mark the group members
//          if (row.isGroupHeader === true) {
//            if (row.checkbox === node) {
//              this._selectGroupCheckboxClicked(rowId);
//            }
//          }
//          else {
        if ((event.ctrlKey) && (this._selectionPending != rowId)) {
//          if ((event.ctrlKey || (node.type && node.type == "checkbox")) && (this._selectionPending != rowId)) {

//            // for the click event on the checkbox the current value only has to be
//            // set to make sure the selection is set correct. The checkbox status is
//            // set by the widget itself. For the ctrl key the status has to toggle.
//            // toggle the check box when ctrl key is pressed
//            var value = node.type == "checkbox" ? row.checkbox.checked : !row.checkbox.checked;
//            this._checkRow(rowId, value);
//            // remember last selected element for shift selection
//            if (row.checkbox.checked) {
//              // set the selection for the checkbox here, as it is not handled in mouse down.
//              // But that is ok as by clicking the checkbox the DnD is also not possible
//              // and therefore setting the selection and sending on focus changed can 
//              // be done here.
          this._currentSelection = Number(rowId);
          this._setFocusedRow(this.focusedRow, rowId);
//            }
//            else {
//              this._currentSelection = -1;
//            }
        }
//          else if (!this._config.maintainSelection && this._deselectionPending >= 0) {
//            // deselect all otheres when the deselectionPending is set
//            for ( var i = 0; i < this._modelRepresentation.length; i++) {
//              if (i != this._deselectionPending) {
//                this._checkRow(i, false);
//              }
//            }
//          }
//          }
        this._selectionPending = -1;
        this._deselectionPending = -1;
//          // check, if the select all checkbox needs to change
//  //        this._checkSelectAllStatus();
//          // send the new selection state
//          //defect 704588, focus changed at mouse down not click
        this.onFocusChanged(this._model[rowId]);
////          this._onSelectionChanged(this.getCheckedData());
//
//          // if we are in selection maintaining mode, where a checkbox may be 
//          // selected from model, we must highlight the one the user clicked on and 
//          // lolight the previous
        if (this._config.maintainSelection) {
          this._lowLightAll(rowId);
          this._setRowSelected(rowId);
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Helper to set a row to selected
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _setRowSelected: function _setRowSelected_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var row = this._modelRepresentation[rowId];
      row.checked = true;
      dojo.removeClass(row.dataRow, 'bSpaceTableOver');
      dojo.removeClass(row.actionRow, 'bSpaceTableOver');
      dojo.addClass(row.dataRow, 'bSpaceTableSelected');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.addClass(nodes[i], 'bSpaceTableSelected');
            dojo.removeClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      
      dojo.addClass(row.actionRow, 'bSpaceTableSelected');
      this._lastChecked = rowId;

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },
    /**
     * Helper to set a row to unselected
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _setRowUnselected: function _setRowUnselected_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      var row = this._modelRepresentation[rowId];
      row.checked = false;
      dojo.addClass(row.dataRow, 'bSpaceTableOver');
      dojo.addClass(row.actionRow, 'bSpaceTableOver');

      dojo.removeClass(row.dataRow, 'bSpaceTableSelected');
      dojo.removeClass(row.actionRow, 'bSpaceTableSelected');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.removeClass(nodes[i], 'bSpaceTableSelected');
            dojo.addClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      this._lastChecked = null;

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },

    /*
      * Handle all mouse over event for the table area.
      */
    _onTableMouseOver: function _onTableMouseOver_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(4);
      var node = event.target;
      if (!node || this.animationInAction) {
        return;
      }
      var rowId = node["_rowId"];
      // first check if the node has a row id
      if (rowId !== undefined && rowId >= 0) {
        //if it is a real event
        if (event.screenX) {
          dojo.stopEvent(event);
        }
        // the row id points to an entry in modelRepresentation array, which defines a row in the table
        var row = this._modelRepresentation[rowId];
        // if the current node is an action node, then the hover class for the action needs to be set.
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          var actionNodes = row.actions;
          var actions = this._model[rowId].inlineActions;
          if (actions[actionId].enabled) {
            dojo.addClass(actionNodes[actionId], actions[actionId].hoverClass);
          }
        }
        // the row is not already marked with hover, then set the necessary classes
        if (this._lastOver !== rowId) {
          // if there was another row marked before, then the old one has to be reset
          this._setHoverForRow(row, rowId);
          if (this._lastOver !== null) {
            var oldRow = this._modelRepresentation[this._lastOver];
            this._unsetHoverForRow(oldRow, this._lastOver);
          }
          this._lastOver = rowId;
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(4);
    },

    _onTableMouseOut: function _onTableMouseOut_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(4);
      var node = event.target;
      if (!node || this.animationInAction) {
        return;
      }
      // when the mouse is outside the table the mouse over marking has to be removed
      var outside = !this._insideTable(event);
      if (this._lastChecked && outside) {
        this._setRowUnselected(this._lastChecked);
      }
      var rowId = node["_rowId"];
      if (rowId !== undefined && rowId >= 0) {
        dojo.stopEvent(event);
        var row = this._modelRepresentation[rowId];
        // if the target was an action node, then disable the action again
        var actionId = node["_actionId"];
        if (actionId !== undefined && actionId >= 0) {
          // remove the hover marking from the current action
          var actions = this._model[rowId].inlineActions;
          var actionNodes = row.actions;
          dojo.removeClass(actionNodes[actionId], actions[actionId].hoverClass);
        }
        if (this._lastOver !== null && outside) {
          var oldRow = this._modelRepresentation[this._lastOver];
          this._unsetHoverForRow(oldRow, this._lastOver);
          this._lastOver = null;
        }
      }
      com.ibm.bpm.social.util.common.Log.logExit(4);
    },


//      /**
//       * Handle the shift key pressed and mouse click
//       * 
//       * @param {Number} rowId
//       * @type {void}
//       */
//      _handleShiftKeyList: function(rowId){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        var allItems = this._modelRepresentation;
//        // only if we have an selection that indicates where to start
//        // marking
//        if (this._currentSelection >= 0) {
//          var startIndex = this._currentSelection;
//          var stopIndex = rowId;
//          if (startIndex > stopIndex) {
//            startIndex = rowId;
//            stopIndex = this._currentSelection;
//          }
//          for ( var i = 0; i < allItems.length; i++) {
//            var aItem = allItems[i];
//            if (i < startIndex || i > stopIndex) {
//              this._checkRow(i, false);
//            }
//            else if (!aItem.checkbox.checked) {
//              this._checkRow(i, true);
//            }
//          }
//        }
//        else {
//          this._checkRow(rowId, true);
//          this._currentSelection = Number(rowId);
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },


    _setHoverForRow: function(row, rowId){
      // when it is a group header, only the checkbox gets displayed
//        if (this._config.isSelectable !== false) {
//          dojo.addClass(row.checkboxRow, 'bSpaceTableOver');
//        }
//        if (!row.isGroupHeader) {
      dojo.addClass(row.dataRow, 'bSpaceTableOver');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.addClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
      dojo.addClass(row.actionRow, 'bSpaceTableOver');
//        }
    },

    _unsetHoverForRow: function(row, rowId){
//        if (this._config.isSelectable !== false) {
//          dojo.removeClass(row.checkboxRow, 'bSpaceTableOver');
//        }
      // for group headers only the checkbox classes have to be set
//        if (!row.isGroupHeader) {
      dojo.removeClass(row.actionRow, 'bSpaceTableOver');
      dojo.removeClass(row.dataRow, 'bSpaceTableOver');
      // there is a problem in IE10 with styles for tables. They are not set correctly on the tr node.
      if (dojo.isIE>9){
        var nodes= dojo.query("td", row.dataRow);
        if (nodes){
          for (var i=0; i<nodes.length;i++){
            dojo.removeClass(nodes[i], 'bSpaceTableOver');
          }
        }
      }
//        if (this._config.isSelectable !== false) {
//          dojo.removeClass(row.checkboxRow, 'bSpaceTableOver');
//          // Only if not highlighted, otherwise, it won't be display=none
//          if (!row.checkbox.checked) {
//            com.ibm.bspace.common.util.ui.CheckBox.setChecked(row.checkbox, false);
//          }
//        }
//        }
    },

    /*
     * Check for an event if it happened inside the table content or outside.
     */
    _insideTable: function(event){
      // for IE it is toTarget, for FF relatedTarget
      var related = event.relatedTarget ? event.relatedTarget : event.toTarget;
      var node = event.target;
      if (related) {
        try {
          if (related["_parentId"] !== undefined && related["_rowId"] !== undefined &&
            related["_parentId"] === node["_parentId"]) {
            return true;
          }
          else {
            return false;
          }
        }
        catch (err) {
          // FF throws an exception if the event is outside the browser.
          // Permission denied to access property 'tagName' from a non-chrome context
          return false;
        }
      }
      return false;
    },

    /*
     * If onmousedown occurs, dnd will have higher priority than the selection handling.
     * dnd needs the item to be checked, because we want the behaviour that all checked items
     * + the current item will be dragged. After onmousedown happened, the selection handling happens
     * in the _onRowClick method. For ctrlKey, we remember the previous checked state.
     */
    _onTableMouseDown: function _onTableMouseDown_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: got no active table data");
        return;
      }

      var node = event.target;
      if (!node) {
        return;
      }
      var rowId = node["_rowId"];

//        if (rowId !== undefined && rowId >= 0 && this._config.isSelectable !== false) {
//          var row = this._modelRepresentation[rowId];
//          // shiftKey implementation: select all items in between first selected item
//          // (item with oldest timestamp) and current selected item.
//          // not supported in maintainSelection mode
//          if (event.shiftKey && !this._config.maintainSelection) {
//            // Find most recent checked item
//            this._handleShiftKeyList(rowId);
//
//            // stopping the event to prevent the user selecting text in the table when doing multiselection
//            dojo.stopEvent(event);
//          }
//          else {
      // no check box and not ctrl
      if (!(event.ctrlKey) && this._config.isSelectable !== false) {
//        if (!(event.ctrlKey || (node.type && node.type == "checkbox"))) {

        // Select this item if not already checked
        // if already checked, the selection does not change
        // and the deselection is done in the onclick event
//              if (!row.checkbox.checked) {
//                if (!this._config.maintainSelection) {
//                  this._checkRow(rowId, true);
//                  // deselect all otheres when row clicked
//                  for ( var i = 0; i < this._modelRepresentation.length; i++) {
//                    if (i != rowId) {
//                      this._checkRow(i, false);
//                    }
//                  }
//                  this._onSelectionChanged(this.getCheckedData());
//                }
//                else {
//                  // deselect all otheres
        this._lowLightAll(rowId);
//                }
//                // set timestamp of selection for shift key handling
//                this._currentSelection = Number(rowId);
//              }
//              else {
//                // deselect all besides this item in the onclick event.
//                // The deselection can only be done here in case
//                // an unselected item was selected.
//                this._deselectionPending = rowId;
//              }

        // Fire onFocusChanged for teamlist integration.
        this._setFocusedRow(this.focusedRow, rowId);
      }
      // if we are in selection maintaining mode, where a checkbox may be 
      // selected from model, we must highlight the one the user clicked on and 
      // lolight the previous
      if (this._config.maintainSelection) {
        this._lowLightAll(rowId);
        this._setRowSelected(rowId);
      }
//          }
//        }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },

    /**
     * Helper function to remove the orange highlight from all rows, except the one given in rowId
     * 
     * @param {Number} rowId
     * @type {void}
     */
    _lowLightAll: function _lowLightAll_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      for ( var i = 0; i < this._modelRepresentation.length; i++) {
        //if (i != rowId) {
        dojo.removeClass(this._modelRepresentation[i].dataRow, 'bSpaceTableSelected');
//          dojo.removeClass(this._modelRepresentation[i].checkboxRow, 'bSpaceTableSelected');
        dojo.removeClass(this._modelRepresentation[i].actionRow, 'bSpaceTableSelected');
        //  }
      }

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },


    /**
     * create a sort indicator td
     * 
     * @type {TD} a td containing the soirt indicator
     */
    _createSortIndicator: function _createSortIndicator_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(3);

      var td_sort = document.createElement('td'); // a11y: no role
      td_sort.style.cursor = "pointer";
      td_sort.title = this._nlsResources["KEY_SortDirectionAsc"]; // default is ASC
      dijit.setWaiRole(td_sort, "columnheader"); // a11y

      // TODO : a11y
      var sortDiv = document.createElement('div');
      sortDiv.className = "sortHidden";
      td_sort.appendChild(sortDiv);

      // a11y stuff (high contrast) for sortDesc
      var sortAscText = document.createElement("span");
      sortAscText.innerHTML = "&#9650;";
      dojo.addClass(sortAscText, "a11yButtonText");
      sortDiv.appendChild(sortAscText);

      com.ibm.bpm.social.util.common.Log.logExit(3);
      return td_sort;
    },

    /**
     * Render the top cell of a specific column
     * 
     * @param {Object} column object describing the column
     */
    _renderHeaderForColumn: function _renderHeaderForColumn_Table(column){

      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

//        // no row for activeGroupByProperty
//        if (this._config.activeGroupByProperty == column.id) {
//          com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//          return;
//        }

      var mainHeaderCell = document.createElement('td');
      // a11y
      // TODO: shouldn't this be on the sub-table in the label TD?
      dijit.setWaiRole(mainHeaderCell, "presentation");

      mainHeaderCell.displayedProperty = column;
      mainHeaderCell.className = 'bSpaceTableHeader tableCell';
      mainHeaderCell.id = this.id + "_" + column.id;

      if (column.width) {
        mainHeaderCell.style.width = column.width;
      }

      this._dnTableHeader.appendChild(mainHeaderCell);

      // add a column to the table header row (with border) and trailing separator
      // this._appendVerticalSeparator(this._dnTableHeader);

      // to conform to the UI guidelines/style proposals, render table in the table
      // to get a frame around the cell ?
      var td_table = document.createElement('table');
      dijit.setWaiRole(td_table, "presentation"); // a11y
      td_table.cellSpacing = 0;
      td_table.cellPadding = 0;
      mainHeaderCell.appendChild(td_table);

      var td_tbody = document.createElement('tbody'); // a11y: no role
      td_table.appendChild(td_tbody);
      var td_tr = document.createElement('tr'); // a11y: no role
      td_tbody.appendChild(td_tr);

//        //
//        // append image for the search indicator
//        //
//        if (this._config.isFilterable !== false && column.isFilterable !== false) {
//          var td_search = document.createElement('td'); // a11y: no role
//          td_search.title = this._nlsResources["KEY_ColumnFilter"];
//          td_search.style.cursor = "pointer";
//          td_tr.appendChild(td_search);
//          var div = document.createElement('div');
//          dijit.setWaiRole(div, "button"); // a11y
//
//          // Create title for the search button.
//          if (dojo.isIE) {
//            // a11y for IE only
//            div.title = this._nlsResources["KEY_ColumnFilter"];
//          }
//          else {
//            // a11y for FF only
//            var genId = "id_" + new Date().getTime();
//            div.setAttribute("aria-labelledby", genId);
//            var hiddenTitle = document.createElement("span");
//            hiddenTitle.innerHTML = this._nlsResources["KEY_ColumnFilter"];
//            hiddenTitle.id = genId;
//            hiddenTitle.style.display = "none";
//            td_search.appendChild(hiddenTitle);
//          }
//
//          // For keyboard handling: search button is accessible via tab.
//          div.tabIndex = 0;
//
//          // Make it visible, if it is in focus.
//          this.connect(div, "onfocus", function(){
//            dojo.addClass(div, 'magnifier_inactive');
//          });
//          this.connect(div, "onblur", function(){
//            dojo.removeClass(div, 'magnifier_inactive');
//          });
//
//          div.className = 'magnifier_hidden';
//          td_search.appendChild(div);
//
//          // a11y stuff (high contrast) for sortDesc
//          var serachText = document.createElement("span");
//          serachText.innerHTML = "&#9660;";
//          dojo.addClass(serachText, "a11yButtonText");
//          div.appendChild(serachText);
//
//          this.connect(td_search, "onclick", '_doSearch');
//          this.connect(td_search, "onkeypress", function(e){
//            if (e.keyCode == dojo.keys.ENTER) {
//              this._doSearch(e);
//            }
//          });
//
//          // highlite indicators on mouse over
//          this.connect(td_search, "mouseover", function(){
//            if (this._modelRepresentation && this._modelRepresentation.length > 0) {
//              dojo.addClass(td_search, 'bSpaceTableSelected');
//            }
//          });
//          this.connect(td_search, "mouseout", function(){
//            dojo.removeClass(td_search, 'bSpaceTableSelected');
//          });
//        }
//        // if filtering is not enabled for this property
//        else {
      var td_search = document.createElement('td'); // a11y: no role          
      td_tr.appendChild(td_search);
      var div = document.createElement('div');
      div.className = 'filterhidden';
      td_search.appendChild(div);
//        }

      //
      // append text cell / header title
      //
      var td_text = document.createElement('td'); // a11y: no role
      dijit.setWaiRole(td_text, "presentation");
      td_tr.appendChild(td_text);
      // td_text.noWrap = true;
      td_text.className = 'bspace-style bspace-heading5 headerCell';

      var innerDiv = document.createElement('div');
      innerDiv.className = "headerInnerText";
      td_text.appendChild(innerDiv);
      innerDiv.innerHTML = column.label;
      dijit.setWaiRole(innerDiv, "presentation");
      innerDiv.id = this.id + "_label_" + column.id;


//        var th = document.createElement('th');
//        th.innerHTML = column.label;
//        this._dataTableHeaderRow.appendChild(th);

      //
      // containing div for the sort indicators
      //
      if (this._config.isSortable !== false && column.isSortable !== false) {
        td_text.tabIndex = -1; // enable focus setting by code
        mainHeaderCell.focusNode = td_text;
        var td_sort = this._createSortIndicator();
        td_tr.appendChild(td_sort);
        dijit.setWaiRole(innerDiv, "presentation");
        dijit.setWaiState(td_sort, 'labelledby', innerDiv.id);
        // innerDiv.title = td_sort.title;
        innerDiv.style.cursor = "pointer";
      }
      else {
        // if sortin is not enabled for this property
        td_sort = document.createElement('td'); // a11y: no role          
        td_tr.appendChild(td_sort);
        div = document.createElement('div');
        div.className = 'sortHidden';
        td_sort.appendChild(div);
        dijit.setWaiState(td_sort, 'hidden', 'true');
        dijit.setWaiRole(innerDiv, "columnheader");
      }

      // cell callback for mouse events
      // KJ: There is no need for a hover or click feedback, if there are no actions placed
      // in the header.
      if ((this._config.isSortable !== false && column.isSortable !== false) ||
        (this._config.isFilterable !== false && column.isFilterable !== false)) {
        this.connect(mainHeaderCell, "onmouseover", '_onHeaderCellMouseOver');
        this.connect(mainHeaderCell, "onmouseout", '_onHeaderCellMouseOut');
        this.connect(mainHeaderCell, "onclick", '_doSort');
        this.connect(mainHeaderCell, "onkeypress", function(e){
          if (e.keyCode == dojo.keys.ENTER) {
            this._doSort(e);
          }
        });
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },


//    /**
//     * handle a click on the "magnifier" icon
//     * 
//     * @param {Object} e onClick event object
//     */
//    _doSearch: function _doSearch_Table(e){
//      var column = this._getParentWithClass(e.target, 'bSpaceTableHeader');
//      var property = column.displayedProperty.id;
//
//      // fire the event
//      this.onFilter(property, column);
//
//    },


    /**
     * handle a click on a column header = do sort
     * 
     * @param {Object} e onClick event object
     */
    _doSort: function _doSort_Table(e){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var column = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var property = column.displayedProperty;
      var isSort;
      if (property.isSort === true) {
        // switch direction
        property.sortDir = (property.sortDir == "ASC" ? "DESC" : "ASC");
      }

      // remember focused header
      var headerNode = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      this.focusedColumn = headerNode._focusColumnId;
      this._refreshedBySort = true;

      this.onSort(property.id, property.sortDir);

      com.ibm.bpm.social.util.common.Log.logExit(2);
    },


    _createActionTd: function _createActionTd_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");

      var td = null;
      td = document.createElement('td'); // a11y : no role
      td.className = 'tableData actions';

      // ensure image is added left
      td.align = 'right';
      td.noWrap = true;
      if (document.body.dir == 'rtl') {
        td.align = 'left';
      }

      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
      return td;
    },

    /**
     * After the available actions call, we get notified here to update the state of all inline actions
     * 
     * @type void
     */
    updateActions: function updateActions_Table(){
      com.ibm.bpm.social.util.common.Log.logEntry(1);

      for ( var j = 0; j < this._model.length; j++) {
        var item = this._model[j];

        if (item.inlineActions && item.inlineActions.length > 0) {
          var actionRepresentation = this._modelRepresentation[j].actions;

          for ( var i = 0; i < actionRepresentation.length; i++) {
            var actionDiv = actionRepresentation[i];

            var action = item.inlineActions[actionDiv._actionId]; //  actionDiv._actionId == index of the action

            // we can't ensure that each action has only one link, User might have it more times in his saved search
            var linkedCells = dojo.filter(this._modelRepresentation[j].cellsWithLinks, function(cell){
              return cell._actionName == action.id;
            });

            if (action.enabled === false) {
              dojo.addClass(actionDiv, action.disabledClass);
              for ( var c = 0; c < linkedCells.length; c++) {
                dojo.removeClass(linkedCells[c], "bpm-task-clickable-link");
                dijit.removeWaiRole(linkedCells[c], "link");
              }
            }
            else {
              dojo.removeClass(actionDiv, action.disabledClass);
              for (c = 0; c < linkedCells.length; c++) {
                dojo.addClass(linkedCells[c], "bpm-task-clickable-link");
                dijit.setWaiRole(linkedCells[c], "link");
              }
              actionDiv.tabIndex = -1; // enable focus setting by code
            }
          }

        }
      }


      com.ibm.bpm.social.util.common.Log.logExit(1);
    },

    /**
     * set the action icons (at the end of the row)
     * 
     * @param {Object} tr tr to append the actions
     * @param {Object} td td to append the actions
     * @param {Number} index index of the item in the model
     */
    _setActions: function _setActions_Table(tr, td, index){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      var actions = this._model[index].inlineActions;

      if (actions) {
        this._modelRepresentation[index].actions = [];
        var actionRepresentation = this._modelRepresentation[index].actions;
        for ( var i = 0; i < actions.length; i++) {
          var action = actions[i];
          var actionDiv = document.createElement("span");
          dojo.addClass(actionDiv, "itemAction");

          if (action.description) {
            actionDiv.title = action.description; // =tooltip;
          }
          else if (action.tooltip) {
            actionDiv.title = action.tooltip; // =tooltip;
          }

          dojo.addClass(actionDiv, action.iconClass);
          if (action.enabled === false) {
            dojo.addClass(actionDiv, action.disabledClass);
          }
          // AB: enable keyboard navigation that way. Only problem actions and 
          // data rows are scrolling differently.
          //actionDiv.tabIndex = 0;

          dojo.addClass(actionDiv, "a11yFilledBox");
          if (dojo.hasClass(dojo.body(), "dijit_a11y")) {
            actionDiv.innerHTML = "&#x25EF;";
          }

          td.appendChild(actionDiv);
          // store the actions id in the node and the node on the representation array
          actionRepresentation[i] = actionDiv;
          // index of the action in the specific row
          actionDiv._actionId = i;
          actionDiv._focusColumnId = this._maxFocusColumns;
          this._maxFocusColumns++;
        }
        // We need the reference to the actions for the key handling.
        tr.actions = actions;
      }
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },

    /**
     * Render a specific row identified by <index>
     * 
     * @param {Object} index
     */
    _renderRow: function _renderRow_Table(index){
      com.ibm.bpm.social.util.common.Log.logEntry(3, "", "Table");
      var tr = this._createRow(index);
      this._dnTableBody.appendChild(tr);
      com.ibm.bpm.social.util.common.Log.logExit(3, "", "Table");
    },


    /**
     * Refresh a row
     * 
     * @param {Number} rowId
     * 
     * @type {void}
     */
    _refreshRow: function _refreshRow_Table(rowId){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");

      var trData = this._getParentWithType(this._modelRepresentation[rowId].dataRow, "tr");
//        var trCb = this._getParentWithType(this._modelRepresentation[rowId].checkboxRow, "tr");
      var trIA = this._getParentWithType(this._modelRepresentation[rowId].actionRow, "tr");
      var trNew = this._createRow(rowId);
      trNew.className = trData.className;
      this._dnTableBody.replaceChild(trNew, trData);
//        this._modelRepresentation[rowId].checkboxRow.className = trCb.className;
      this._modelRepresentation[rowId].actionRow.className = trIA.className;
//        this.CheckboxTableBody.replaceChild(this._modelRepresentation[rowId].checkboxRow, trCb);
      this.ActionTableBody.replaceChild(this._modelRepresentation[rowId].actionRow, trIA);

      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
      return;
    },

    /**
     * Set the row id for all elements under the current node.
     * 
     * @param {DOMElement} node
     * @param {Number} rowId
     * @param {DOMElement} parentId
     * @type {void}
     */
    _setRowId: function(node, rowId, parentId){
      if (!node) {
        return;
      }
      node._rowId = rowId;
      node._parentId = parentId;
      var children = node.getElementsByTagName("*");
      if (children) {
        for ( var k = 0; k < children.length; k++) {
          children[k]._rowId = rowId;
          children[k]._parentId = parentId;
        }
      }
    },


    /**
     * Row creator for dojo DND
     * 
     * @param {Number} index
     * @param {String} hint If the method is used for avatar creation, hint is "avatar", otherwise it is null.
     */
    _createRow: function _createRow_Table(index, hint){
      var tr;
      var data = this._model[index];

//        if (this._groupHeaderOffset < 0) {
//          // we need to now the offset per model entry for group headers
//          // they are inserted in the modelRepresentation to calculate
//          // the position correct for calls from outside the table with indices
//          this._groupHeaderOffset = 0;
//        }

      // insert the representation for this row. Add the specific DOM nodes later.
      if (hint !== "avatar") {
        this._modelRepresentation[index] = {
          dataRow: null,
          focusNode: null,
//            checkboxRow: null,
//            checkbox: null,
          actionRow: null,
          actions: null,
          cellsWithLinks: [],
//            isGroupHeader: false,
//            offset: this._groupHeaderOffset
          offset: 0
        };
      }
//        if (data instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) {
//          this._groupHeaderOffset++;
//          this._modelRepresentation[index].offset = this._groupHeaderOffset;
//          tr = this._createGroupBanner(index, data);
//        }
//        else {
      tr = this._createDataRow(index, data, hint);
//        }

//        // IE7 DnD source won't work with tr, will need a table instead.
//        if (hint == "avatar" && dojo.isIE < 8) {
//          var ta = document.createElement("table");
//          var tb = document.createElement("tbody");
//          ta.appendChild(tb);
//          tb.appendChild(tr);
//          return ta;
//        }
      return tr;
    },


    /**
     * Create a regular data row.
     * 
     * @param {Number} index
     * @param {Object} data a single row from the model
     * @param {String} hint If the method is used for avatar creation, hint is "avatar", otherwise it is null.
     *        If the creation is for the avatar, there is no need for the checkbox and the inline action.
     */
    _createDataRow: function _createDataRow_Table(index, data, hint){
      com.ibm.bpm.social.util.common.Log.logEntry(2);

      this._maxFocusColumns = 0;

      var td;
      var img;
      var tr = document.createElement('tr');
      dijit.setWaiRole(tr, "row"); // a11y
      if (hint !== "avatar") {
        this._modelRepresentation[index].dataRow = tr;
      }

      // The first data row is in the tab order. From there you have to navigate with arrow
      // down/up through the table.
      if (index === 0) {
//          if (index == 0 && this._config.isSelectable !== false) {
        // tr.tabIndex = 0;
//          // set an event handler for focus in complete table
//          this._tableContentConnects.push(dojo.connect(tr, "onfocus", this, "_onTableMouseOver"));
      }
      else {
//            // Need this, otherwise we cannot focus on row.
        tr.tabIndex = -1;
      }

      var tr_class = 'taskRow bSpaceTableRow';
      if (index === 0) {
        tr_class += ' firstRow';
      }
      if (this._isPrototypTable) {
        tr_class += ' tablePrototype';
      }
      tr.className = tr_class;
//        // create the checkbox
//        if (this._config.isSelectable !== false && hint !== "avatar") {
//          var cbtr = document.createElement('tr');
//          dijit.setWaiRole(cbtr, "row"); // a11y
//
//          // Need this, otherwise we cannot focus on row.
//          cbtr.tabIndex = -1;
//
//          if (this._config.maintainSelection) {
//            cbtr.className = "taskRow";
//          }
//          else {
//            cbtr.className = tr_class;
//          }
//
//          td = document.createElement('td');
//          dijit.setWaiRole(td, "gridcell"); // a11y
//          dojo.addClass(td, 'tableData bSpaceTableCheckBox');
//          var checkbox = document.createElement('input');
//          dijit.setWaiRole(checkbox, "checkbox"); // a11y
//          checkbox.type = 'checkbox';
//          // Checkbboxes in list must not be accessible via tab key, otherwise
//          // you would have to tab a lot to pass the list (in case that there
//          // are a lot of tasks).
//          checkbox.tabIndex = -1;
//          td.appendChild(checkbox);
//          // convert to common UI checkbox
//          com.ibm.bspace.common.util.ui.CheckBox.wrapElement(checkbox);
//          // disable checkbox, only enabled when mouse over or highlight
//          // UI Checkbox renders with a overlay button
//          var lineNo = Number(index) + 1;
//          checkbox.nextSibling.title = this._nlsResources["KEY_selectItemInRow"].replace("${1}", lineNo);
//
//          // no tabStop on line checkboxes
//          checkbox.tabIndex = -1;
//          cbtr.appendChild(td);
//
//          // if selected in model, also set the checkbox on
//          if (data.enabled) {
//            com.ibm.bspace.common.util.ui.CheckBox.setChecked(checkbox, data.enabled);
//          }
//          this.CheckboxTableBody.appendChild(cbtr);
//
//          // store the node in model representation and the index in the node
//          this._modelRepresentation[index].checkbox = checkbox;
//          this._modelRepresentation[index].checkboxRow = cbtr;
//          // set row id for all elements of the value
//          this._setRowId(cbtr, index, this.id);
      //}
      // iterate through the cells of the row
      var dnColumn = this._dnTableHeader.firstChild;

      while (dnColumn) {

        td = document.createElement('td');
        if (dojo.hasClass(dnColumn, 'bSpaceTableHeader')) { // check if the current column is a real col (not only separator)

          td.className = 'tableData';
          dijit.setWaiRole(td, "columnheader"); // a11y
          dijit.setWaiState(td, "describedby", dnColumn.id); // a11y

          var value = null;
          var formatter = null;
          var valueDiv = null;
          var linkToAction = null;

          // find corresponding column definition
          for ( var i = 0; i < this._columns.length; i++) {
            if (dnColumn.displayedProperty.id == this._columns[i].id) {
              formatter = this._columns[i].formatter;
              linkToAction = this._columns[i].linkToAction;
              break;
            }
          }

          value = data[dnColumn.displayedProperty.id];
          if (value != undefined) {
            if (dojo.isFunction(formatter)) {
              // the formatter returns either a completely formatted div or null
              valueDiv = formatter(value);
            }
            if (!valueDiv) {
              valueDiv = document.createElement('div'); //a11y: none
              valueDiv.innerHTML = value;
              valueDiv.title = value; // =tooltip;
              valueDiv.tabIndex = -1;
            }

            // some fields can be links which do the same as inline actions
            if (linkToAction) {
              dojo.addClass(valueDiv, "bpm-task-clickable-link");
              // store the id for later use
              for (i = 0; i < data.inlineActions.length; i++) {
                if (data.inlineActions[i].id == linkToAction) {
                  valueDiv._actionId = i;
                  valueDiv._actionName = linkToAction;
                  // AB: enable keyboard navigation that way. Only problem actions and 
                  // data rows are scrolling differently.
                  //valueDiv.tabIndex = 0;
                }
              }
              this._modelRepresentation[index].cellsWithLinks.push(valueDiv);
            }

            // do not allow to wrap lines
            dojo.addClass(valueDiv, "tableCell");
            // KJ: If display is block, the width of the div will adapt to the cell
            // width. But we need the real size to do the Firefox maximize handling in
            // _setMaxWidth().
            if (dojo.isFF || (dojo.isIE && dojo.isIE > 7)) {
              valueDiv.style.display = "inline";
            }
            td.appendChild(valueDiv);
            td.focusNode = valueDiv;
            valueDiv._focusColumnId = this._maxFocusColumns;
            this._maxFocusColumns++;
          }
        } // end of if (dojo.hasClass(dnColumn, 'bSpaceTableHeader'))

        // add empty gif if we have no values to display
        if (!td.firstChild) {
          dojo.addClass(td, 'bSpaceBlankImageHolder');
          img = document.createElement('img'); //a11y: none
          img.src = dojo.moduleUrl("com.ibm.bspace.common.util.ui", "img/blank.gif");
          img.alt = this._nlsResources["task_saved_searches_empty_cell"];
          td.appendChild(img);
          td.focusNode = img;
          img._focusColumnId = this._maxFocusColumns;
          this._maxFocusColumns++;
        }

        if (dnColumn.style.width) {
          td.style.width = dnColumn.style.width;
        }

        tr.appendChild(td);
        // set row id for all elements of row
        this._setRowId(tr, index, this.id);

        dnColumn = dnColumn.nextSibling;

      } // end of while

      if (hint !== "avatar") {
        // append buttons at the end
        var atr = document.createElement('tr');
        dijit.setWaiRole(atr, "presentation"); // a11y
        // Need this, otherwise we cannot focus on row.
        //AB: we can not tab throguth actions
        //atr.tabIndex = -1;

        this._modelRepresentation[index].actionRow = atr;
        // keep links to the other parts of the row
        atr.className = tr_class;

        var actionsTd = this._createActionTd();
        actionsTd._rowId = index;
        atr.appendChild(actionsTd);
        this._setActions(atr, actionsTd, index);
        this.ActionTableBody.appendChild(atr);
        // set row id for all elements of the row
        this._setRowId(atr, index, this.id);
      }

      com.ibm.bpm.social.util.common.Log.logExit(2);
      return tr;
    },


//      /**
//       * 
//       * Create group banner ( a row visually emphasized )
//       * 
//       * @param {Object} index
//       * @param {Object} data
//       */
//      _createGroupBanner: function _createGroupBanner_Table(index, data){
//        var tr = document.createElement('tr');
//
//        this._modelRepresentation[index].isGroupHeader = true;
//        var tr_class = 'groupBanner bSpaceTableCheckBox bSpaceTableRow';
//
//        // The first row is in the tab order. From there you have to navigate with arrow
//        // down/up through the table.
//        tr.tabIndex = -1;
//        if (index === 0) {
//          tr_class += ' firstRow';
//          if (this._config.isSelectable !== false) {
//            tr.tabIndex = 0;
//            // set an event handler for focus in complete table
//            this._tableContentConnects.push(dojo.connect(tr, "onfocus", this, "_onTableMouseOver"));
//          }
//        }
//        if (this._isPrototypTable) {
//          tr_class += ' tablePrototype';
//        }
//        tr.className = tr_class;
//
//        this._modelRepresentation[index].dataRow = tr;
//
//        // create checkbox
//        var cbtr = document.createElement('tr');
//        cbtr.className = tr_class;
//        dijit.setWaiRole(cbtr, "row"); // a11y
//        // Need this, otherwise we cannot focus on row.
//        cbtr.tabIndex = -1;
//
//        var td = document.createElement('td');
//        cbtr.appendChild(td);
//        this.CheckboxTableBody.appendChild(cbtr);
//        td.align = 'center';
//        dojo.addClass(td, 'bSpaceTableCheckBox');
//
//        var selectGroupCheckbox = document.createElement('input');
//        selectGroupCheckbox._rowId = index;
//        dijit.setWaiRole(selectGroupCheckbox, "checkbox"); // a11y
//        selectGroupCheckbox.type = 'checkbox';
//        selectGroupCheckbox.tabIndex = -1;
//        td.appendChild(selectGroupCheckbox);
//        this._modelRepresentation[index].checkbox = selectGroupCheckbox;
//        this._modelRepresentation[index].checkboxRow = cbtr;
//
//        com.ibm.bspace.common.util.ui.CheckBox.wrapElement(selectGroupCheckbox);
//        selectGroupCheckbox.nextSibling.title =
//          data.groupName ? dojo.string.substitute(this._nlsResources["KEY_selectGroupItems"], {
//            1: data.groupName
//          }) : this._nlsResources["KEY_selectGroupItemsUncategorized"];
//        // set row id for all elements of the value
//        this._setRowId(cbtr, index, this.id);
//
//        // action row
//        var atr = document.createElement('tr');
//        this._modelRepresentation[index].actionRow = atr;
//        dijit.setWaiRole(atr, "row"); // a11y
//
//        // Need this, otherwise we cannot focus on row.
//        atr.tabIndex = -1;
//
//        atr.className = tr_class;
//
//        var actionsTd = this._createActionTd();
//        atr.appendChild(actionsTd);
//        this.ActionTableBody.appendChild(atr);
//        // set row id for all elements of the row
//        this._setRowId(atr, index, this.id);
//
//
//        // data row
//        td = document.createElement('td');
//        td.style.padding = "0";
//        td.colSpan = this._dnTableHeader.cells.length;
//        td.appendChild(this._config.groupFormatter.renderGroup(data.groupName));
//        // set the index for all children, so that a mouse event works properly
//        this._setRowId(tr, index, this.id);
//
//        tr.appendChild(td);
//
//        this._groupHeaders.push(tr);
//
//        return tr;
//      },

    /**
     * handle mouse over row event
     * 
     * @param {Object} e
     */
    _onHeaderCellMouseOver: function _onHeaderCellMouseOver_Table(e){
      if (!this._modelRepresentation || this._modelRepresentation.length === 0) {
        com.ibm.bpm.social.util.common.Log.log(1, "exit: no action since message is shown");
        return;
      }

      var td = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var gif = null;
      if (td) {
        dojo.addClass(td, 'bSpaceTableOver');

        // show magnifier icon
        gif = dojo.query(".magnifier_hidden", td)[0];
        if (gif) {
          dojo.addClass(gif, 'magnifier_inactive');
        }

        // show sort icon if sortable and not already sorted
        if (td.displayedProperty.isSortable && !td.displayedProperty.isSort) {
          gif = dojo.query(".sortHidden", td)[0];
          if (gif) {
            dojo.addClass(gif, 'sortAscPrimary');
          }
        }
      }
    },

    _onHeaderCellMouseOut: function _onHeaderCellMouseOut_Table(e){


      var td = this._getParentWithClass(e.target, 'bSpaceTableHeader');
      var gif = null;
      if (td) {
        dojo.removeClass(td, 'bSpaceTableOver');

        gif = dojo.query(".magnifier_inactive", td)[0];
        // hide the magnifier icon
        if (gif) {
          dojo.removeClass(gif, 'magnifier_inactive');
        }

        // hide sort icons if sortable and not already sorted
        if (td.displayedProperty.isSortable && !td.displayedProperty.isSort) {
          gif = dojo.query(".sortHidden", td)[0];
          if (gif) {
            dojo.removeClass(gif, 'sortAscPrimary');
          }
        }
      }
    },


    /**
     * Check a checkbox of a row of or off
     * 
     * @param {Number} rowId index of the row to be checked on/off
     * @param {Boolean} check check the checkbox on / off
     * @type {void}
     */
    checkRow: function checkRow_Table(rowId, check){
      this._checkRow(rowId, check);
    },


    /**
     * Check the checkbox in the specified mode
     * 
     * @param {Number} rowId index of the row to be checked on/off
     * @param {Boolean} check check the checkbox on / off
     * @type {void}
     */
    _checkRow: function _checkRow_Table(rowId, check){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
      // highlight row and remember selection
      if (check) {
        this._setRowSelected(rowId);
        this._enableNextAction(rowId, 0);
      }
      else {
        this._setRowUnselected(rowId);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
    },


    /**
     * create a simple table to do some tests
     */
    _prototypeTable: function _prototypeTable_Table(sampleModel){
      var lconfig = this._config; // save old config
      this._isPrototypTable = true;

      //   ToDo: if we have a config but no sample model, try to generate a model from config

      // if we also got a model, we can test under real circumstances
      if (this._config && sampleModel && sampleModel.length > 0) {

        // no grouping here
//          this._config.activeGroupByProperty = null;
        this.setModel([sampleModel[0]]);
//          this._config.activeGroupByProperty = lconfig.activeGroupByProperty;
        // ok, all done.
        this._isPrototypTable = false;
      }

      else {

        var simpleProperty = [{
          id: "LOADING",
          label: "",
          formatter: null,
          description: null,
          inline: null
        }];

        var simpleConfig = {
          actions: [],
          displayedProperties: simpleProperty
        };

        var simpleModel = [{
          LOADING: "Loading ..."
        }];

        this.setConfig(simpleConfig);
        this.setModel(simpleModel);

        // restore old config, due to lazy rendering table will remain until next setModel
        // setConfig respects _isPrototypTable, thus we must unset it first
        this._isPrototypTable = false;

        if (lconfig) {
          this.setConfig(lconfig);
        }
      }
    },


    /**
     * Step up the DOM tree from domNode unless we find a node with the named class
     * 
     * @param {Object} node
     * @param {Object} className
     */
    _getParentWithClass: function _getParentWithClass_Table(node, className){
      var newNode = node;
      while (newNode && !dojo.hasClass(newNode, className)) {
        newNode = newNode.parentNode;
      }
      return newNode;
    },

    /**
     * Step up the DOM tree from domNode unless we find a node with the named class
     * 
     * @param {Object} node
     * @param {Object} type
     */
    _getParentWithType: function _getParentWithType_Table(node, type){
      var newNode = node;
      while (newNode && newNode.nodeName != type.toUpperCase()) {
        newNode = newNode.parentNode;
      }
      return newNode;
    },

    /**
     * Find the header cell for the given property
     * 
     * @param {String} property
     */

    _getColumnForProperty: function _getColumnForProperty_Table(property){
      var td = null;
      var columns = dojo.query("td.bSpaceTableHeader", this._dnTableHeader);
      property = property.toUpperCase();
      for ( var i = 0; i < columns.length; i = i + 1) {
        if (columns[i].displayedProperty.id == property) {
          td = columns[i];
          break;
        }
      }
      return td;
    },

    /**
     * Keyboard navigation on table works as follows
     * <ul>
     * <li>arrow down: focus next visible row</li>
     * <li>arrow up: focus previous visible row or header</li>
     * <li>left arrow: focus previous column (in header or table row)</li>
     * <li>right arrow: focus next column (in header or table row)</li>
     * <li>enter: execute action if defined for current column</li>
     * <li>tab: unset hover before table is left</li>
     * </ul>
     */
    _onKeyEvent: function _onKeyEvent_Table(event){
      com.ibm.bpm.social.util.common.Log.logEntry(2, "  key " + event.keyCode + "  row " + this.focusedRow +
        "  focusedColumn " + this.focusedColumn, "Table");
      // var tr = event.currentTarget;
      var node = event.target;

      var rowId = this.focusedRow;

      var k = dojo.keys;
      switch (event.keyCode) {
        case k.DOWN_ARROW :
//            if (this._lastChecked !== null) {
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus onheader, focus next visible row
            this.focusedRow = 0;
            this._focusNextVisibleRow();
          }
          else if (rowId + 1 < this._modelRepresentation.length) {
            // Set focus on next row
            this._setFocusedRow(rowId, rowId + 1);
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.UP_ARROW :
//            if (this._lastChecked !== null) {
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId > 0) {
            // focus previous row
            this._setFocusedRow(rowId, rowId - 1);
          }
          else {
            // focus header
            this._setFocusInHeader(true);
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.LEFT_ARROW :
//            if (this._lastChecked !== null) {
//              this._enableNextAction(rowId, 0);
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus previous or first header item
            if (this.focusedColumn > 0) {
              var columns = this._getChildElementCount(this.HeaderTable.rows[0]);
              if (this.focusedColumn >= columns) {
                // focus on Actions, move to last table header
                this.focusedColumn = columns - 1;
              }
              else {
                this.focusedColumn--;
              }
              this._setFocusInHeader(true);
            }
          }
          else {
            // focus previous or first row item
            if (this.focusedColumn > 0) {
              this.focusedColumn--;
              this._setFocusInRow();
            }
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
        case k.RIGHT_ARROW :
//            if (this._lastChecked !== null) {
//              this._enableNextAction(rowId, 0);
//              dojo.stopEvent(event);
//              return;
//            }
          if (rowId === -1) {
            // focus next or last header item
            if (this.focusedColumn < this._getChildElementCount(this.HeaderTable.rows[0])) {
              this.focusedColumn++;
              this._setFocusInHeader(true);
            }
          }
          else {
            // focus next or last row item
            if (this.focusedColumn < this._maxFocusColumns - 1) {
              this.focusedColumn++;
              this._setFocusInRow();
            }
          }
          // Avoid that this key triggers any scrollbar action.
          dojo.stopEvent(event);
          break;
//          case k.SPACE :
//            if (rowId !== -1) {
//              row = this._modelRepresentation[rowId];
//              this._checkRow(rowId, !row.checked);
//            }
//        break;
        case k.TAB :
          // Remove hover indicator in this case, because table area will be left.
          if (this._lastOver !== null) {
            var row = this._modelRepresentation[this._lastOver];
            this._unsetHoverForRow(row, this._lastOver);
            this._lastOver = null;
          }
//          if (this._lastChecked !== null) {
//            this._setRowUnselected(this._lastChecked);
//          }
          break;

        case k.ENTER :
          // enter was on an action?
          var actionId = node["_actionId"];
          if (actionId !== undefined && actionId >= 0) {
            var actions = this._model[rowId].inlineActions;
            var action = actions[actionId];
            // fire only if enabled
            if (action.enabled) {
              this.onActionRequested(action.id, this._model[rowId]);
            }
          }
          break;
        default:
          break;
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "  key " + event.keyCode + "  row " + this.focusedRow +
        "  focusedColumn " + this.focusedColumn, "Table");
    },

    /**
     * Sets the focus to a new row. Sets the focus to the focusNode of the current focus column and highlights
     * the focused row. Scrolls the view if the new focused row is not (completely) visible.
     * 
     * @param currentIndex
     * @param newIndex
     * @returns {void} nothing
     */
    _setFocusedRow: function _setFocusedRow_Table(currentIndex, newIndex){
      this.focusedRow = newIndex;
      var focusNode = null;
      var newDataRow = this._modelRepresentation[newIndex].dataRow;
      var y;
      if (currentIndex) {
        if (newIndex > currentIndex) {
          // scroll down ?
          var newDataRowBottom = newDataRow.offsetTop + newDataRow.offsetHeight;
          // bottom of row is outside visible view?
          if ((this.DataTable.offsetTop + newDataRowBottom) > this.VScrollerContainer.offsetHeight) {
            y = this.DataTable.offsetTop - (newDataRow.offsetHeight * (newIndex - currentIndex));
            if (com.ibm.bpm.social.util.common.Util.isIPad) {
              this.BodyCenter.scrollTop = -y;
            }
            else {
              this.VScrollerContainer.scrollTop = -y;
            }
          }
        }
        else if (newIndex < currentIndex) {
          // scroll up ?
          // top of row is outside visible view?
          if (newDataRow.offsetTop < this.VScrollerContainer.scrollTop) {
            y = newDataRow.offsetTop;
            if (com.ibm.bpm.social.util.common.Util.isIPad) {
              this.BodyCenter.scrollTop = y;
            }
            else {
              this.VScrollerContainer.scrollTop = y;
            }
          }
        }
      }
      // set focus
      this._setFocusInRow();
    },

    /**
     * Sets the focus to the focusNode of the current focus column in the focused row.
     */
    _setFocusInRow: function _setFocusInRow_Table(){
      if (this.focusedRow != -1) {
        var focusNode = null;
        var dataRow = this._modelRepresentation[this.focusedRow].dataRow;
        var columns = this._getChildElementCount(dataRow);
        if (this.focusedColumn < columns) {
          focusNode = dataRow.childNodes[this.focusedColumn].focusNode;
        }
        else {
          var actionsIndex = this.focusedColumn - columns;
          var actions = this._modelRepresentation[this.focusedRow].actions;
          if (actionsIndex < actions.length) {
            focusNode = actions[actionsIndex];
          }
        }
        if (focusNode) {
          this._setFocusToNode(focusNode, true);
          this.onFocusChanged(this._model[this.focusedRow]);
          this._onTableMouseOver({
            target: dataRow
          });
        }
      }
    },

    /**
     * Sets the focus to the focusNode of the current focus column in the header row.
     */
    _setFocusInHeader: function _setFocusInHeader_Table(setFocus){
      this.focusedRow = -1;
      var focusNode = null;
      var headerRow = this.HeaderTable.rows[0];
      if (this.focusedColumn < this._getChildElementCount(headerRow)) {
        focusNode = headerRow.childNodes[this.focusedColumn].focusNode;
      }
      else {
        focusNode = this.HeaderRightFocusNode;
      }
      if (focusNode) {
        if (setFocus) {
          this._setFocusToNode(focusNode, true);
        }
        else {
          this._setFocusNode(focusNode);
        }
      }
    },

    _enableNextAction: function _enableNextAction_Table(rowId, index, newIndex){
//        var actions = this._modelRepresentation[rowId].actions;
//        if (actions && actions.length > 0) {
      // If the first action has no key or ENTER defined, trigger it on enter.
      var inlineActions = this._model[rowId].inlineActions;
      if (inlineActions && inlineActions.length > 0) {
        var actionId = index;
        dojo.removeClass(inlineActions[actionId], inlineActions[actionId].hoverClass);
        if (newIndex > index) {
          while (!inlineActions[newIndex].enabled && newIndex !== index) {
            newIndex++;
            if (newIndex >= inlineActions.length) {
              newIndex = 0;
            }
          }
        }
        else {
          while (!inlineActions[newIndex].enabled && newIndex !== index) {
            newIndex--;
            if (newIndex < 0) {
              newIndex = inlineActions.length - 1;
            }

          }
        }
        if (inlineActions[newIndex].enabled) {
          dojo.addClass(inlineActions[newIndex], inlineActions[newIndex].hoverClass);
          this._currentActionIndex = newIndex;
        }
        else {
          this._currentActionIndex = null;
        }
      }
      else {
        this._currentActionIndex = null;
      }
    },

//      /**
//       * create a target for droppable tasks
//       */
//      _createDndTarget: function _createDndTarget_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // create a target for droppable tasks
//        if (this.dndAccept != this._config.supportedDndTypes) {
//          this.dndAccept = this._config.supportedDndTypes || [];
//          // Create a target for drop actions in the empty area below the rows.
//          if (!this.dndTarget) {
//            this.dndTarget = new com.ibm.bpc.widget.util.DnDSource(this.Center, {
//              accept: this.dndAccept,
//              horizontal: false,
//              copyOnly: false
//            });
//            // this is just for completeness.
//            // typically dndSource is created after dndTarget, so this code
//            // normally shouldn't be necessary. Nevertheless, be defensive.
//            if (this.dndSource) {
//              this.dndSource.accept = dojo.clone(this.dndTarget.accept);
//            }
//          }
//          else {
//            // To update just the accepted types, recreate the dndTarget.accept object
//            // dndTarget.accept is an object mapping each accept to 1:
//            // Object { bspace_htm_TaskList=1}
//            var accept = {};
//            for ( var i = 0; i < this.dndAccept.length; i++) {
//              accept[this.dndAccept[i]] = 1;
//            }
//            this.dndTarget.accept = accept;
//            // if we have a DnDSource, update the accepted types
//            if (this.dndSource) {
//              this.dndSource.accept = accept;
//            }
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//      /**
//       * create a DnD source on the table body
//       */
//      _createDndSource: function _createDndSource_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // create a source for the rows
//        if (this.dndType != this._config.publishedDndTypes) {
//          this.dndType = this._config.publishedDndTypes || [];
//          if (!this.dndSource && this.dndType.length > 0) {
//            var args = {
//              creator: dojo.hitch(this, this._dndCreator),
//              accept: this.dndAccept,
//              horizontal: false,
//              copyOnly: false
//            };
//            this.dndSource = new com.ibm.bpc.widget.util.DnDSource(this._dnTableBody, args);
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//
//      /*
//       * Creator callback for DnD
//       */
//      _dndCreator: function _dndCreator_Table(data, hint){
//        // Add extra row as separator below
//        var types = [];
//        // check for a specific dnd type on the item
//        var item = data["dnd.ref"]._model[data["dnd.index"]];
//        if (item.dndType) {
//          types = item.dndType;
//        }
//        else {
//          // otherwise use the class dnd type
////          types =
////            (data["dnd.ref"]._model[data["dnd.index"]] instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) ? ""
////              : data["dnd.ref"].dndType;
//          types = data["dnd.ref"].dndType;
//        }
//        var node = data["dnd.ref"]._createRow(data["dnd.index"], hint);
//        return {
//          node: node,
//          data: data,
//          type: types
//        };
//      },
//
//
//      /**
//       * Check if row is selected. Insert the node id with the marker1. This function is called from the
//       * DnDSource. The table is connected to the DnDSource via the synchronizeSource function.
//       */
//      getDnDSelection: function(){
//        var selection = {};
//        if (this._modelRepresentation) {
//          for ( var i = 0; i < this._modelRepresentation.length; i++) {
//            var row = this._modelRepresentation[i];
////            if (!row.isGroupHeader) {
//            //if (row.dataRow["_rowId"] == this._lastOver || row.checkbox.checked) {
//            if (row.dataRow["_rowId"] == this._lastOver) {
//              selection[row.dataRow.id] = 1;
//            }
////            }
//          }
//        }
//        return selection;
//      },
//
//      /*******************************************************************************************************
//       * Callback for the DnD Subscription
//       * 
//       * @param {Object} source
//       * @param {Object} nodes
//       * @param {Object} iscopy
//       * @param {Object} target
//       */
//
//      _onDnDNotification: function _onDnDNotification_Table(source, nodes, iscopy, target){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        // accept only events if we are the target
//        if ((this.dndTarget && (this.dndTarget.node == target.node)) ||
//          (this.dndSource && this.dndSource.node == target.node)) {
//          var typeToObjectsMap = {};
//          dojo.forEach(nodes, function(aNode){
//            var data = source.getItem(aNode.id);
//            dojo.forEach(data.type, function(type){
//              if (!typeToObjectsMap[type]) {
//                typeToObjectsMap[type] = [];
//              }
//              typeToObjectsMap[type].push(data.data);
//            });
//          });
//          for ( var k in typeToObjectsMap) {
//            this.onDrop(typeToObjectsMap[k], k);
//          }
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },
//
//      /*
//       *  When a drag starts, the current node should be selected. The ctrl key
//       *  selection is triggered by the onclick not the on mouse down, but for
//       *  the DnD it has to be selected before the onclick event.
//       *  Thus select the current node, if not already done.
//       */
//      _onDnDStart: function _onDnDStart_Table(source, nodes, copy){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//        if (source && source.anchor) {
//          var rowId = source.anchor["_rowId"];
//          this._selectionPending = -1;
//          if (rowId !== undefined && rowId >= 0) {
//            var row = this._modelRepresentation[rowId];
//            // if the current node is the lastOver
//            if (this._lastOver == rowId) {
//              // set the checkbox, because for ctrl it is not set already
////              if (!row.checkbox.checked) {
//              this._currentSelection = Number(rowId);
//              this._checkRow(rowId, true);
//              this.focusedRow = rowId;
//              this.onFocusChanged(this._model[rowId]);
//              this._onSelectionChanged(this.getCheckedData());
//              this._selectionPending = rowId;
////              }
//            }
//          }
//          // flag that a DnD is started, that is necessary to finish
//          // the correct selection in the onclick event. As a mousedown
//          // on an unselected row deselects all others and selects the current.
//          // But if the current node is already selected and a DnD is started
//          // the deselection should not happen. Thus the deselection happens in
//          // the onclick in that case.
//          this._deselectionPending = -1;
//        }
//        com.ibm.bpm.social.util.common.Log.logExit(2, "", "Table");
//      },

//      /**
//       * _prepareModelForGroupBy - insert the group headers into the model - determine the groups and build a
//       * list of them
//       * 
//       * @param {Array} model the model we got from outside
//       */
//      _prepareModelForGroupBy: function _prepareModelForGroupBy_Table(model){
//        var preparedItems = [];
//        var item;
//        var groupByProperty = this._config.activeGroupByProperty;
//
//        if (groupByProperty) {
//          var lastGroup = {}; // Set value to something that won't match a group property value
//
//          // determine the groups, extract them into new list
//
//          for ( var i = 0; i < model.length; i++) {
//            item = model[i];
//            var groupName = item[groupByProperty];
//
//            if (groupName != lastGroup) {
//              lastGroup = groupName;
//              // insert the group header as new row.
//              preparedItems.push(new com.ibm.bpm.social.widgets.task.list._GroupHeader(groupName));
//
//
//            }
//            preparedItems.push(item);
//          }
//        }
//        else {
//          // no grouping
//          preparedItems = model;
//        }
//
//        return preparedItems;
//      },
//
//      _initializeGroupFormatter: function _initializeGroupFormatter_Table(){
//        if (this._config.groupFormatter) {
//          this._cleanupGroupFormatter();
//          if (this._config.activeGroupByProperty && this._model.length > 1) {
//            var groupsList = this._getListOfGroups(this._model);
//            this._config.groupFormatter.initialize(groupsList);
//          }
//        }
//        this._groupHeaders = [];
//
//      },
//
//      _cleanupGroupFormatter: function _cleanupGroupFormatter_Table(){
//        if (this._config.groupFormatter) {
//          this._config.groupFormatter.cleanup();
//        }
//      },
//
//
//      /**
//       * get a list of groups and the count of members in the groups
//       * 
//       * @param {Array} items (the model)
//       * @return com.ibm.widget.util.Group[]
//       */
//
//      _getListOfGroups: function _getListOfGroups_Table(items){
//        var groupsList = [];
//        var groupEntry = null;
//        var item = null;
//
//        for ( var i = 0; i < items.length; i++) {
//          item = items[i];
//          if (item instanceof com.ibm.bpm.social.widgets.task.list._GroupHeader) {
//            if (groupEntry) {
//              groupsList.push(groupEntry);
//            }
//
//            groupEntry = new com.ibm.widget.util.Group(item.groupName);
//
//          }
//          else {
//            if (groupEntry) {
//              groupEntry.countVisible++;
//            }
//          }
//        }
//
//        if (groupEntry) {
//          groupsList.push(groupEntry);
//        }
//
//        return groupsList;
//      },


//      /**
//       * Checks selectAll checkbox, if all items are selected. Unchecks selectAll checkbox, if at least one
//       * item is not selected.
//       */
//      _checkSelectAllStatus: function _checkSelectAllStatus_Table(){
//        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "Table");
//
//        // global check all checkbox
//        var rows = this._modelRepresentation;
//        var allChecked = true;
////        var allInGroupChecked = true;
//        for ( var i = (rows.length) - 1; i >= 0; i--) {
//          var checkbox = rows[i].checkbox;
////          if (rows[i].isGroupHeader === true && i !== ((rows.length) - 1)) {
////            com.ibm.bspace.common.util.ui.CheckBox.setChecked(checkbox, allInGroupChecked);
////            allInGroupChecked = true;
////          }
////          else {
//          if (!checkbox.checked) {
//            //            allInGroupChecked = false;
//            allChecked = false;
//          }
////          }
//        }
//        if (allChecked) {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, true);
//        }
//        else {
//          com.ibm.bspace.common.util.ui.CheckBox.setChecked(this._selectAllCheckbox, false);
//        }
//
//      },

    _appendOrResizeVScrollerImage: function _appendOrResizeVScrollerImage_Table(imageHeight, scrollerWidth){

      if (dojo.isIE) {
        // IE8 bug: if the scrollbar container has the same width as it's content, the scrollbar cannot scrolled
        // with the arrow keys
        if (!this.VScrollImage) {
          dojo.addClass(this.VScroller, 'bSpaceBlankImageHolder');
          var scrollImage = document.createElement('img'); //a11y: none
          scrollImage.src = dojo.moduleUrl("com.ibm.bspace.common.util.ui", "img/blank.gif");
          this.VScrollImage = this.VScroller.appendChild(scrollImage);
        }

        this.VScrollImage.height = imageHeight;
        // if the width of the  scrollContainer is not 3 pixel wider, the scrollbar cannot scrolled with the arrow keys
        if (dojo.isIE == 8) {
          this.VScrollerContainer.style.width = (scrollerWidth + 3) + 'px';
        }
        else {
          this.VScrollerContainer.style.width = (scrollerWidth + 1) + 'px';
        }
      }
    },

    /**
     * wrapper around onSelectionChanged to prevent double events
     * 
     * @param {Array} items an array of 0...n items
     */
    _onSelectionChanged: function(/* Array */items){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      var changed = false;
      items.sort();

      if (items.length == this._lastPublishedSelection.length) {
        for ( var i = 0; i < items.length; i++) {
          if (items[i] != this._lastPublishedSelection[i]) {
            changed = true;
            break;
          }
        }
      }
      else {
        // selection changed definitely as size changed
        changed = true;
      }

      if (changed) {
        this._lastPublishedSelection = items.slice(0);
        this.onSelectionChanged(items);
      }
      com.ibm.bpm.social.util.common.Log.logExit(2, "items changed :" + (changed ? "true" : " false"));
    },

    /**
     * Set the page size which is later used to decide whether a trailing section should be displayed
     * 
     * @param {int} actual page size requested by user, initially 25, doubled by "Show more"
     * @type {void}
     */
    setPageSize: function setPageSize_Table(pageSize){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      this.pageSize = pageSize;
      com.ibm.bpm.social.util.common.Log.logExit(2);
    },

    _getChildElementCount: function _getChildElementCount_Table(node){
      if (node.childElementCount) {
        return node.childElementCount;
      }
      if (node.children) {
        return node.children.length;
      }
      return 0;
    }

  });


//dojo.declare("com.ibm.bpm.social.widgets.task.list._GroupHeader", null, {
//  constructor: function(groupName){
//    this.groupName = groupName;
//  }
//});
