// TaskRow.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TaskRow");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.Menu");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.util.Constants");
dojo.require("com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.PropertyFormatter");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.widgets.ActionDropDownHelper");
dojo.require("com.ibm.bpm.social.widgets.task.list.Category");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.require("com.ibm.bspace.common.util.ui.Helper");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.list.TaskRow", [dijit._Widget,
                                                              dijit._Templated], {

  templateString: dojo.cache("com.ibm.bpm.social.widgets.task", "list/templates/TaskRow.html"),

  open: false,

  _processActionsReceived: false,
  _processActionsDeferred: null,
  _postInstanceComment: false,

  /**
   * 
   * @type {void}
   */
  constructor: function constructor_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    //
    // Lookup helper for dropDown menu.
    //  
    this._dropDownHelper = com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * postCreate
   * 
   */
  postCreate: function postCreate_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this.inherited(arguments);

    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * make sure all data is set with defaults or the incoming values, thus all old values in a cached row
   * widget are replaced
   */
  initRow: function initRow_TaskRow(param){
    com.ibm.bpm.social.util.common.Log.logEntry(2, param.model.TKIID);

    this.model = param.model;
    this.controller = param.controller;
    this.displayedProperties = param.displayedProperties;

    // Status
    this.setStatus(this.model.categoryId);

    // Subject
    this.subject.innerHTML = "";
    if (this._isDisplayedProperty("TAD_DISPLAY_NAME")) {
      this.subject.innerHTML =
        com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this.model.TAD_DISPLAY_NAME);

      //configure BTD for Subject
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.subject);

      this.headerLabel.innerHTML = this._nlsResources["task-row-task-subject-label"];
      this.headerLabel.setAttribute("id", "taskrowsubject" + this.id);
      dijit.setWaiState(this.subject, "labelledby", this.headerLabel.id);
    }
    else {
      // set default values
      this.subject.innerHTML = "";
      this.headerLabel.innerHTML = "";
    }
    dijit.setWaiRole(this.subject, "button");

    if (param.controller.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_COMPLETED_TASKS) {
      this.subject.title = this._nlsResources["task_action_view_instance_details_description"];
    }
    else {
      this.subject.title = this._nlsResources["task_action_run_task_description"];
    }

    dijit.setWaiState(this.menuButton, "label", this._nlsResources["task_actions_title"]);

    // Priority
    if (this._isDisplayedProperty("PRIORITY")) {
      this._setPriority(this.model.PRIORITY);
    }
    else {
      // set default values
      this._setPriority(null);
    }

    // Due
    if (this._isDisplayedProperty("DUE")) {
      this._setDue(this.model.DUE);
    }
    else {
      // set default values
      this._setDue(null);
    }

    // Instance
    this.instance.innerHTML = "";
    if (this._isDisplayedProperty("PI_DISPLAY_NAME")) {
      var text = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatText(this.model.PI_DISPLAY_NAME);
      //configure BTD for the instance
      text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(text);
      this.instance.innerHTML = text;
      this.instanceLabel.setAttribute("id", "taskrowinstancelabel" + this.id);
      dijit.setWaiState(this.instance, "labelledby", this.instanceLabel.id);
      this.instanceLabel.innerHTML = this._nlsResources["task-row-instance-name-label"];
    }
    else {
      // set default values
      this.instance.innerHTML = "";
      this.instanceLabel.innerHTML = "";
    }

    // assigned to group
    if (this._isDisplayedProperty("ASSIGNED_TO_ROLE_DISPLAY_NAME")) {
      this._setAssignee(this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME);
    }
    else {
      // set default values
      this._setAssignee(null);
    }
    // complete date
    if (this._isDisplayedProperty("COMPLETED")) {
      this._setCompleteDate(this.model.COMPLETED);
    }
    else {
      // set default values
      this._setCompleteDate(null);
    }

    // Clean up expanded area
    this._clearDetailsArea();

    // is the Modify Task dialog open?
    if (this._modifyTaskHandler && this._modifyTaskHandler.dialog && this._modifyTaskHandler.dialog.popup &&
      this._modifyTaskHandler.dialog.popup.open) {
      // use a new ModifyTaskHandler 
      // otherwise the properties of the existing ModifyTaskHandler will be updated and modifications will be done to the new task
      // the existing one will be destroyed when the dialog is hidden
      this._modifyTaskHandler = null;
    }

    // actions drop down
    this._clearMenuButton();
    this._callback = dojo.hitch(this, "_handleAvailableActions");
    this.controller.setActionsCallbackSuccess(this.model, this._callback);

    // task details
    if (param.open) {
      this._expand(true); // retrieve task details
    }
    else {
      this.open = false; // initial default state is collapsed
      this._collapse();
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * update model and widget with new data
   * <p>
   * TODO can the task subject and the process instance name change?
   * 
   * @param model updated model
   * @param actionsNeeded callback for actions needs to be established
   * @returns {boolean} synchronizationNeeded
   */
  updateRow: function updateRow_TaskRow(model, actionsNeeded){
    com.ibm.bpm.social.util.common.Log.logEntry(2, model.TKIID);

    var synchronizationNeeded = false;
    var deleteRow = false;

    // assigned to another user
    if (model.OWNER !== undefined) {
      if (this.model.OWNER !== model.OWNER) {
        this.model.OWNER = model.OWNER;
        this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME = null;
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        if (this.model.OWNER !== user.userName) {
          // remove row
          deleteRow = true;
          this.onDeleteRowNeeded(this.model.TKIID);
        }
        else {
          this._setAssignee(null);
        }
      }
    }

    if (!deleteRow) {
      // Priority
      if (model.PRIORITY !== undefined) {
        if (this._isDisplayedProperty("PRIORITY") && this.model.PRIORITY !== model.PRIORITY) {
          this.model.PRIORITY = model.PRIORITY;
          this._setPriority(this.model.PRIORITY);
        }
      }

      // Due
      if (model.DUE !== undefined) {
        if (this._isDisplayedProperty("DUE") && this.model.DUE !== model.DUE) {
          this.model.DUE = model.DUE;
          this._setDue(this.model.DUE);

          // If the task is not in the atRisk group, set the atRisk flag to false 
          // and wait for any notification to fix this.
          if (this.model.categoryId !== com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
            this.model.IS_AT_RISK = false;
          }

          synchronizationNeeded = true;
        }
      }

      // category
      if (model.categoryId !== undefined) {
        if (this.model.categoryId !== model.categoryId) {
          this.model.categoryId = model.categoryId;
          synchronizationNeeded = true;
        }
      }

      // Status
      if (model.DUE !== undefined && model.IS_AT_RISK !== undefined) {
        if (this.model.DUE !== model.DUE || this.model.IS_AT_RISK !== model.IS_AT_RISK) {
          this.model.IS_AT_RISK = model.IS_AT_RISK;
          this.setStatus(this.model.categoryId);
          synchronizationNeeded = true;
        }
      }

      // assigned to group
      if (model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== undefined) {
        if (this._isDisplayedProperty("ASSIGNED_TO_ROLE_DISPLAY_NAME") &&
          this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== model.ASSIGNED_TO_ROLE_DISPLAY_NAME) {
          this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME = model.ASSIGNED_TO_ROLE_DISPLAY_NAME;
          this.model.OWNER = null;
          this._setAssignee(this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME);
        }
      }

      // complete date
      if (model.COMPLETED !== undefined) {
        if (this._isDisplayedProperty("COMPLETED") && this.model.COMPLETED !== model.COMPLETED) {
          this.model.COMPLETED = model.COMPLETED;
          this._setCompleteDate(this.model.COMPLETED);
          synchronizationNeeded = true;
        }
      }

      // actions drop down 
      if (actionsNeeded) {
        this._clearMenuButton();
        this._callback = dojo.hitch(this, "_handleUpdateAvailableActions");
        this.controller.setActionsCallbackSuccess(this.model, this._callback);
      }
    }

    // KJ: Removed, because the flickering was quite confusing in the
    // Modify Task Dialog case PLUS if the details are not refreshed,
    // the in-line completion task content will not be lost! 
    // Anyway: still have to discuss this...
    /*
    // TODO task details
    if (this.open) {
      this._expand(true); // retrieve task details ?
    }*/

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return synchronizationNeeded;
  },

  /**
   * checks if the new model requires new actions
   * <p>
   * This is the case if owner or state changed
   * 
   * @param model
   * @returns {boolean} actionsNeeded
   */
  actionsNeeded: function actionsNeeded_TaskRow(model){
    com.ibm.bpm.social.util.common.Log.logEntry(2, model.TKIID);

    var actionsNeeded = false;

    if (model.STATE !== undefined) {
      actionsNeeded = (this.model.STATE !== model.STATE);
    }

    // assigned to group
    if (model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== undefined &&
      this.model.ASSIGNED_TO_ROLE_DISPLAY_NAME !== model.ASSIGNED_TO_ROLE_DISPLAY_NAME) {
      actionsNeeded = true;
    }

    // assigned to another user
    if (model.OWNER !== undefined) {
      if (this.model.OWNER !== model.OWNER) {
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        if (model.OWNER !== user.userName) {
          // task row will be removed
          actionsNeeded = false;
        }
        else {
          actionsNeeded = true;
        }
      }
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, actionsNeeded);
    return actionsNeeded;
  },

  _isDisplayedProperty: function _isDisplayedProperty_TaskRow(key){
    if (this.displayedProperties && dojo.indexOf(this.displayedProperties, key) !== -1 && this.model[key] !== undefined) {
      return true;
    }
    else if (!this.displayedProperties && this.model[key] !== undefined) {
      return true;
    }
    return false;
  },

  destroy: function destroy_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    if (this._modifyTaskHandler) {
      this._modifyTaskHandler.destroy();
      this._modifyTaskHandler = null;
    }
    if (this._menu) {
      this._clearMenuButton();
      com.ibm.bspace.common.util.ui.Helper.unwrapElementList([this.menuButton]);
    }
    this._clearDetailsArea();
    if (this._connectsCallback) {
      for ( var i = 0; i < this._connectsCallback.length; i++) {
        dojo.disconnect(this._connectsCallback[i]);
      }
      this._connectsCallback = null;
    }
    this._dropDownHelper = null;
    this._nlsResources = null;

    this.controller = null;
    this.model = null;
    this._callback = null;
    this.inherited('destroy', arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setDue: function _setDue_TaskRow(dueDate){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + dueDate);
    if (dueDate) {
      this.dueLabel.innerHTML = this._nlsResources["task_row_due"];
      this.dueDate.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(dueDate);
      this.dueDate.setAttribute("id", "taskrowduedate" + this.id);
      this.dueLabel.setAttribute("id", "taskrowduelabel" + this.id);
      this.dueDate.setAttribute("aria-labelledby", "taskrowduelabel" + this.id);
    }
    else {
      this.dueLabel.innerHTML = "";
      this.dueDate.innerHTML = "";
    }
  },

  _setCompleteDate: function _setCompleteDate_TaskRow(completeDate){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + completeDate);
    if (completeDate) {
      this.completeLabel.innerHTML = this._nlsResources["task_row_completed"];
      this.completeDate.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(completeDate);
      this.completeDate.setAttribute("id", "taskrowcompletedate" + this.id);
      this.completeLabel.setAttribute("id", "taskrowcompletelabel" + this.id);
      this.completeDate.setAttribute("aria-labelledby", "taskrowcompletelabel" + this.id);
    }
    else {
      this.completeLabel.innerHTML = "";
      this.completeDate.innerHTML = "";
    }
  },

  _setPriority: function _setPriority_TaskRow(priority){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + priority);
    if (priority) {
      // get the new tooltip and css class
      var priorityText = com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(priority);
      var priorityClass = null;
      var priorityTooltip = null;
      var priorityLabel = null;
      var priorityA11y = null;
      switch (priorityText) {
        case "VeryHigh" :
          priorityClass = "bpm-social-very-high";
          priorityTooltip = "task-row-priority-very-high-tooltip";
          priorityLabel = "KEY_PrioVeryHighTooltip";
          priorityA11y = "KEY_PrioVeryHigh";
          break;
        case "High" :
          priorityClass = "bpm-social-high";
          priorityTooltip = "task-row-priority-high-tooltip";
          priorityLabel = "KEY_PrioHighTooltip";
          priorityA11y = "KEY_PrioHigh";
          break;
        case "Medium" :
          priorityClass = "bpm-social-medium";
          priorityTooltip = "task-row-priority-normal-tooltip";
          priorityLabel = "KEY_PrioMediumTooltip";
          priorityA11y = "KEY_PrioMedium";
          break;
        case "Low" :
          priorityClass = "bpm-social-low";
          priorityTooltip = "task-row-priority-low-tooltip";
          priorityLabel = "KEY_PrioLowTooltip";
          priorityA11y = "KEY_PrioLow";
          break;
        case "VeryLow" :
          priorityClass = "bpm-social-very-low";
          priorityTooltip = "task-row-priority-very-low-tooltip";
          priorityLabel = "KEY_PrioVeryLowTooltip";
          priorityA11y = "KEY_PrioVeryLow";
          break;
        default:
          break;

      }
      //// remove the old values
      // remove the old class and set the new one
      dojo.replaceClass(this.imgPriority, priorityClass, ["bpm-social-very-low",
                                                          "bpm-social-low",
                                                          "bpm-social-medium",
                                                          "bpm-social-high",
                                                          "bpm-social-very-high"]);
      // normal mode 
      this.imgPriority.title = this._nlsResources[priorityTooltip];
      this.imgPriorityLabel.innerHTML = this._nlsResources[priorityLabel];
      this.imgPriorityLabel.setAttribute("id", "taskrowpriority" + this.id);
      dijit.setWaiState(this.imgPriority, "labelledby", this.imgPriorityLabel.id);


      // high contrast mode
      this.priorityA11yLabel.innerHTML = this._nlsResources["task-row-priority-label"] + " : ";
      this.priorityA11yLabel.setAttribute("id", "taskrowprioritya11y_label" + this.id);

      this.priorityA11y.innerHTML = this._nlsResources[priorityA11y];
      dijit.setWaiState(this.priorityA11y, "labelledby", this.priorityA11yLabel.id);

    }
    else {
      //// set default values
      this.imgPriority.title = "";
      this.imgPriorityLabel.innerHTML = "";
      // high contrast mode
      this.priorityA11y.innerHTML = "";
      this.priorityA11yLabel.innerHTML = "";
      // remove the old class
      dojo.removeClass(this.imgPriority, ["bpm-social-very-low",
                                          "bpm-social-low",
                                          "bpm-social-medium",
                                          "bpm-social-high",
                                          "bpm-social-very-high"]);
    }

  },

  _setAssignee: function _setAssignee_TaskRow(assignedToRole){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + assignedToRole);
    this.group.innerHTML = "";
    if (assignedToRole) {
      // From Process Portal: determine group display name
      // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
      // So we are just using the last two '_'s to pull the PG name
      if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
        var roleName = assignedToRole;
        var displayName = roleName.substring(0, roleName.lastIndexOf("_"));
        displayName = displayName.substring(0, displayName.lastIndexOf("_"));
        assignedToRole = displayName;
      }
      this.groupLabel.setAttribute("id", "taskrowgrouplabel" + this.id);
      this.groupLabel.innerHTML = this._nlsResources["task_row_owner-label"];
      this.group.innerHTML = assignedToRole;
      dijit.setWaiState(this.group, "labelledby", this.groupLabel.id);
      dojo.addClass(this.imgGroup, "bpm-social-group");
    }
    else {
      //// set default values
      this.group.innerHTML = "";
      this.groupLabel.innerHTML = "";
      dojo.removeClass(this.imgGroup, "bpm-social-group");
    }
  },

  setStatus: function setStatus_TaskRow(categoryId){
    com.ibm.bpm.social.util.common.Log.log(2, this.model.TKIID + " - " + categoryId);
    //// remove the old one
    if (this._statusClass) {
      dojo.removeClass(this.imgStatus, this._statusClass);
    }
    var statusTooltip = null;
    switch (categoryId) {
      case com.ibm.bpm.social.widgets.task.list.Category.OVERDUE :
        this._statusClass = "bpm-social-overdue";
        statusTooltip = "task-row-status-overdue-tooltip";
        break;
      case com.ibm.bpm.social.widgets.task.list.Category.AT_RISK :
        this._statusClass = "bpm-social-at-risk";
        statusTooltip = "task-row-status-at-risk-tooltip";
        break;
      default:
        this._statusClass = "bpm-social-normal-task";
        statusTooltip = "task-row-status-normal-tooltip";
        break;
    }
    //// set the new one
    dojo.addClass(this.imgStatus, this._statusClass);
    // normal mode
    this.imgStatus.title = this._nlsResources[statusTooltip];
    this.imgStatus.setAttribute("id", "imgStatus" + this.id);
    this.imgStatus.setAttribute("alt", this.imgStatus.title);
    dijit.setWaiState(this.imgStatus, "label", this._nlsResources["task-row-status-label"]);
  },

  _workOn: function _workOn_TaskRow(e){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID, "TaskRow");

    dojo.stopEvent(e);
    this.onWorkOn(this.model);

    com.ibm.bpm.social.util.common.Log.logExit(2, this.model.TKIID, "TaskRow");
  },

  onWorkOn: function onWorkOn_TaskRow(model){},

  /**
   * Sets details for task and append the simple approval user input form if the user has the necessary rights
   * to complete the task
   * 
   */
  setDetailsSimpleApproval: function setDetailsSimpleApproval_TaskRow(taskId, details){
    var myself = this;
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details
        myself.setDetails(details);
      }
      else {
        myself._setDetailsSimpleApprovalImpl(taskId, details, false /* choiceFallback */);
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  setDetailsSimpleCompletion: function setDetailsSimpleCompletion_TaskRow(taskId, details){
    var myself = this;
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details        
        myself.setDetails(details);
      }
      else {
        myself._setDetailsSimpleCompletionImpl(taskId, details);
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  setDetailsSimpleChoice: function setDetailsSimpleChoice_TaskRow(taskId, details){
    var myself = this;
    this._triggerProcessActionsDeferred(this.model["PI_PIID"], null /* systemID */);
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.controller.setActionsCallbackSuccess(this.model, function(){
      // Is user entitled to claim or complete the task??
      var availableActions = myself.model.availableActions;
      if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 && dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
        // Not entitled !! => show simple details
        myself.setDetails(details);
      }
      else {
        if (details.variables && details.variables.choices && details.variables.choices.items &&
          details.variables.choices.items.length > 0) {
          myself._setDetailsSimpleChoiceImpl(taskId, details);
        }
        else {
          myself._setDetailsSimpleApprovalImpl(taskId, details, true /* choiceFallback */);
        }
      }
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _handleErrorForInlineCompletion: function _handleErrorForInlineCompletion_TaskRow(taskId, displayName){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    var myself = this;
    var handled = false;
    myself._restHandler.getActionsForTask(taskId, ["ACTION_CLAIM",
                                                   "ACTION_COMPLETE"],
      function(data){
        if (data.tasks.length === 1) {
          var availableActions = data.tasks[0].actions || [];
          if (dojo.indexOf(availableActions, "ACTION_CLAIM") == -1 &&
            dojo.indexOf(availableActions, "ACTION_COMPLETE") == -1) {
            handled = true;
            myself._errorHandler.showWarningDialog("KEY_TaskInlineAlreadyAssigned", {
              1: displayName
            }, null);
          }
        }
      }, function(){
      // Ignore
      }, false /* synchronous call */);
    if (!handled) {
      this._errorHandler.showErrorDialog("KEY_TaskInlineCouldNotBeCompleted", {
        1: displayName
      }, null);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _createInlineForm: function _createInlineForm_TaskRow(styleClass /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var inlineForm = dojo.create("div", {}, this.expanded, "last");
    dojo.addClass(inlineForm, styleClass);
    //
    // Make sure that the task details are not closed when clicking in the inline input form
    // or while entering values.
    // 
    // Clicking on the details area would close the details. Prevent this for the controls.    
    //
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(inlineForm, "onclick", function(event){
      dojo.stopEvent(event);
      event.preventDefault();
    }));
    this._expandedConnects.push(dojo.connect(inlineForm, "onkeydown", function(event){
      event.stopPropagation();
    }));
    this._expandedConnects.push(dojo.connect(inlineForm, "onkeyup", function(event){
      event.stopPropagation();
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return inlineForm;
  },

  _createCommentField: function _createCommentField_TaskRow(inlineForm /* DOM Element*/){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // label for accessibility
    var label = dojo.create("label", {
      "for": this.id + "_InlineCommentField",
      "style": "display: none"
    }, inlineForm, "last");
    label.innerHTML = this._nlsResources["task-inline-comment-placeholder"];
    var commentField = dojo.create("textarea", {
      id: this.id + "_InlineCommentField",
      rows: "2",
      placeholder: this._nlsResources["task-inline-comment-placeholder"],
      title: this._nlsResources["task-inline-comment-title"]
    }, inlineForm, "last");
    dojo.addClass(commentField, "bpm-task-inline-comment");
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return commentField;
  },

  _setDetailsSimpleApprovalImpl: function _setDetailsSimpleApprovalImpl_TaskRow(taskId, details, choicesFallback){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a reject and approval button.
    // 
    // a) div for container => bpm-task-approval-form
    // b) input[type=text] for comment => bpm-task-approval-form-comment, bpm-task-inline-comment
    // c) input[type=button] for reject button => bpm-task-approval-form-reject, bpm-task-inline-red-button
    // d) input[type=button] for approve button => bpm-task-approval-form-approve, bpm-task-inline-green-button
    // e) technical div for closing floating area => bpm-task-approval-form-closing
    var inlineForm = this._createInlineForm("bpm-task-approval-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-approval-form-comment");
    var rejectButton = dojo.create("input", {
      type: "button",
      name: "SimpleApproval_Reject",
      disabled: "disabled",
      title: this._nlsResources["task-inline-reject-title-disabled"],
      value: this._nlsResources["task-inline-reject"]
    }, inlineForm, "last");
    rejectButton.dataTitleNoComment = this._nlsResources["task-inline-reject-title-disabled"];
    rejectButton.dataTitleWithComment = this._nlsResources["task-inline-reject-title-enabled"];
    dojo.addClass(rejectButton, "bpm-task-approval-form-reject bpm-task-inline-red-button-disabled");
    var approveButton = dojo.create("input", {
      type: "button",
      name: "SimpleApproval_Accept",
      title: this._nlsResources["task-inline-approve-title"],
      value: this._nlsResources["task-inline-approve"]
    }, inlineForm, "last");
    dojo.addClass(approveButton, "bpm-task-approval-form-approve bpm-task-inline-green-button");
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-approval-form-closing");
    //
    // 2) Establish logic for reject button handling. (I.e. rejecting requires comment according to design)
    // 
    // Clicking on the details area would close the details. Prevent this for the controls.
    // Reject is only enabled if a comment has been entered. This logic is implemented in private function updateCommentField.
    // 
    var updateCommentField = function(){
      if (commentField.value.length > 0) {
        rejectButton.disabled = false;
        rejectButton.title = rejectButton.dataTitleWithComment;
        dojo.replaceClass(rejectButton, "bpm-task-inline-red-button", "bpm-task-inline-red-button-disabled");
      }
      else {
        rejectButton.disabled = true;
        rejectButton.title = rejectButton.dataTitleNoComment;
        dojo.replaceClass(rejectButton, "bpm-task-inline-red-button-disabled", "bpm-task-inline-red-button");
      }
    };
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(commentField, "onclick", function(event){
      updateCommentField();
    }));
    // Make sure that the button enablement/disablement happen at each key stroke.
    this._expandedConnects.push(dojo.connect(commentField, "onkeyup", function(event){
      updateCommentField();
    }));
    this._expandedConnects.push(dojo.connect(commentField, "onchange", function(event){
      updateCommentField();
    }));
    //
    // 3) Finally trigger the completion of the Simple Approval Huamn Service.
    //  
    var myself = this;
    var property = choicesFallback ? "decision" : "approved";
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    this._expandedConnects.push(dojo.connect(approveButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value || "";
      myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, property, choicesFallback ? "true" : true, comment,
        function(response){
          var streamMessage =
            comment.length > 0 ? myself._nlsResources["task-inline-simple-approval-comment"]
              : myself._nlsResources["task-inline-simple-approval"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
    }));
    this._expandedConnects.push(dojo.connect(rejectButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value ? commentField.value : "";
      myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, property, choicesFallback ? "false" : false, comment,
        function(response){
          var streamMessage = myself._nlsResources["task-inline-simple-approval-rejection-comment"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Sets details for task and append the simple completion user input form if the user has the necessary
   * rights to complete the task
   */
  _setDetailsSimpleCompletionImpl: function _setDetailsSimpleCompletionImpl_TaskRow(taskId, details, choiceFallback){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a complete button.
    // 
    // a) div for container => bpm-task-completion-form
    // b) input[type=text] for comment => bpm-task-completion-form-comment, bpm-task-inline-comment
    // c) input[type=button] for complete button => bpm-task-completion-form-complete, bpm-task-inline-blue-button
    // d) technical div for closing floating area => bpm-task-completion-form-closing
    var inlineForm = this._createInlineForm("bpm-task-completion-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-completion-form-comment");
    var completeButton = dojo.create("input", {
      type: "button",
      name: "SimpleCompletion_Accept",
      title: this._nlsResources["task-inline-complete-title"],
      value: this._nlsResources["task-inline-complete"]
    }, inlineForm, "last");
    dojo.addClass(completeButton, "bpm-task-completion-form-complete bpm-task-inline-blue-button");
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-completion-form-closing");
    //
    // 2) Finally trigger the completion of the Simple Completion Human Service    
    //
    var myself = this;
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    if (!this._expandedConnects) {
      this._expandedConnects = [];
    }
    this._expandedConnects.push(dojo.connect(completeButton, "onclick", function(){
      // Provide the data as escaped json to REST call
      var comment = commentField.value || "";
      myself._restHandler.completeSimpleCompletionTask(taskId, unclaimed, comment, function(response){
        var streamMessage =
          comment.length > 0 ? myself._nlsResources["task-inline-simple-completion-comment"]
            : myself._nlsResources["task-inline-simple-completion"];
        var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
        var fullName = user.fullName;
        var parameters = [fullName,
                          myself.model["TAD_DISPLAY_NAME"],
                          comment];
        myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
        myself.onDeleteRowNeeded(taskId);
      }, function(error, ioArgs){
        myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
        myself.onRefreshNeeded();
      });
    }));
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  /**
   * Sets details for task and append the simple choice user input form if the user has the necessary rights
   * to complete the task.
   */
  _setDetailsSimpleChoiceImpl: function _setDetailsSimpleChoiceImpl_TaskRow(taskId, details){
    com.ibm.bpm.social.util.common.Log.logEntry(2, taskId);
    var myself = this;
    this.setDetails(details);
    //
    // 1) Create dom elements for input form. Bascically a text field for comments and a complete button.
    // 
    // a) div for container => bpm-task-choice-form
    // b) input[type=text] for comment => bpm-task-choice-form-comment, bpm-task-inline-comment
    // c) input[type=button] for each option button => bpm-task-choice-form-complete, bpm-task-inline-blue-button
    // d) select for the available options => bpm-task-choice-form-options
    // e) technical div for closing floating area => bpm-task-choice-form-closing
    var inlineForm = this._createInlineForm("bpm-task-choice-form");
    var commentField = this._createCommentField(inlineForm);
    dojo.addClass(commentField, "bpm-task-choice-form-comment");
    var submitButtons = [];
    var numberOfChoices = details.variables.choices.items.length;
    for ( var i = numberOfChoices - 1; i >= 0; i--) {
      var choice = details.variables.choices.items[i];
      var completeButton = dojo.create("input", {
        type: "button",
        name: "SimpleChoice_Accept",
        title: myself._nlsResources["task-inline-complete-title"],
        value: choice
      }, inlineForm, "last");
      dojo.addClass(completeButton, "bpm-task-choice-form-complete bpm-task-inline-blue-button");
      submitButtons.push(completeButton);
    }
    var closingDiv = dojo.create("div", {}, inlineForm, "last");
    dojo.addClass(closingDiv, "bpm-task-choice-form-closing");
    //
    // 2) Finally trigger the completion of the Simple Choice Human Service    
    //           
    var unclaimed = dojo.indexOf(this.model.availableActions, "ACTION_CLAIM") !== -1;
    var clickHandler =
      function(decision){
        // Provide the data as escaped json to REST call
        var comment = commentField.value || "";
        myself._restHandler.completeSimpleInlineTask(taskId, unclaimed, "decision", decision, comment, function(){
          var streamMessage =
            comment.length > 0 ? myself._nlsResources["task-inline-simple-choice-comment"]
              : myself._nlsResources["task-inline-simple-choice"];
          var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
          var fullName = user.fullName;
          var parameters = [fullName,
                            myself.model["TAD_DISPLAY_NAME"],
                            decision,
                            comment];
          myself._postMessage(myself.model["PI_PIID"], streamMessage, parameters);
          myself.onDeleteRowNeeded(taskId);
        }, function(error, ioArgs){
          myself._handleErrorForInlineCompletion(taskId, myself.model["TAD_DISPLAY_NAME"]);
          myself.onRefreshNeeded();
        });
      };
    dojo.forEach(submitButtons, function(completeButton){
      myself.connect(completeButton, "onclick", function(){
        clickHandler(completeButton.value);
      });
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _postMessage: function _postMessage_TaskRow(piid, streamMessage, parameters){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // Current user is cached by singleton function.
    streamMessage = dojo.replace(streamMessage, parameters);
    var restHandler = this._restHandler;
    // Make sure the process actions have already been retrieved
    var myself = this;
    if (this._processActionsDeferred != null) {
      this._processActionsDeferred.then(function(){
        if (myself._postInstanceComment) {
          restHandler.publishOnInstanceStream(piid, streamMessage, function(){}, function(){});
        }
      });
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * sets detailed information for specific task as description and specific key value pairs
   * 
   * Expected details format: {narrative: "...", businessData: [{label: "myLabel", value: "myValue", type:
   * String|Integer|Boolean|Date|Time|Decimal, name: "myVariableName"}, ...]}
   */
  setDetails: function setDetails_TaskRow(details){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    // Clean up expanded area
    this._clearDetailsArea();

    var isDetailsFound = false;

    if (details) {

      if (!(details.narrative === null || details.narrative === "")) {
        var narrativeContainer = document.createElement("div");
        narrativeContainer.innerHTML = com.ibm.bpm.social.util.common.Util.createSecureInnerHtml(details.narrative);
        dijit.setWaiState(narrativeContainer, "label", this._nlsResources["task-row-task-definition-label"]);

        //Apply base text direction to the narrative container
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(narrativeContainer);

        narrativeContainer.setAttribute("id", "narrativecontainer" + this.id);

        dojo.style(narrativeContainer, "marginBottom", "10px");
        dojo.addClass(narrativeContainer, "bpm-task-businessData");
        this.expanded.appendChild(narrativeContainer);
        isDetailsFound = true;
      }

      if (details.businessData) {
        var self = this;
        var businessDataContainer = document.createElement("div");
        dijit.setWaiState(businessDataContainer, "label", this._nlsResources["task-row-task-details-label"]);
        businessDataContainer.setAttribute("id", "businessdata" + this.id);
        //dojo.style(businessDataContainer, "position", "relative");
        dojo.addClass(businessDataContainer, "bpm-task-businessDataContainer");
        var labelString = this._nlsResources["task-row-task-details-business-data-label"];
        dojo.forEach(details.businessData, function(businessData, index){
          // Labels of business data with null or empty string value will 
          // not be displayed.
          if (typeof businessData.value != "undefined" && businessData.value !== null && businessData.value !== "") {
            var labelAndValue = document.createElement("div");
            dojo.addClass(labelAndValue, "bpm-task-businessDataContainer-entry");
            var label = document.createElement("span");
            label.innerHTML = dojo.string.substitute(labelString, [businessData.label]) + "&nbsp;";
            dojo.addClass(label, "bpm-task-businessDataLabel bpm-task-businessData bpm-task-businessData-string");
            if (businessData.alias) {
              label.title = businessData.alias.toLowerCase();
            }
            labelAndValue.appendChild(label);
            var formattedValue = self.formatBusinessData(businessData.type, businessData.value);
            labelAndValue.appendChild(formattedValue);
            //appropriately tag all string business data elements for base text direction application
            if (businessData.type.toLowerCase() == "string") {
              dojo.addClass(formattedValue, "bpm-task-businessData-string");
            }
            businessDataContainer.appendChild(labelAndValue);
            isDetailsFound = true;
          }
        });

        //query all string business data elements if pref is not "Default"
        var btdpref = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
        if (btdpref != "default") {
          dojo.query(".bpm-task-businessData-string", businessDataContainer).forEach(function(node, index, arr){
            //configure BTD for each element
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(node);
          });
        }
        var clearDiv = document.createElement("div");
        dojo.style(clearDiv, "clear", "both");
        businessDataContainer.appendChild(clearDiv);

        this.expanded.appendChild(businessDataContainer);
      }
    }

    if (!isDetailsFound) {
      this.showMessage(this._nlsResources["task-row-details-not-found"]);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  formatBusinessData: function formatBusinessData_TaskRow(type, value){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var valueContainer = document.createElement("span");
    dojo.addClass(valueContainer, "bpm-task-businessDataValue bpm-task-businessData");

    switch (type) {
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DATE_TYPE :
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_TIME_TYPE :
        valueContainer.innerHTML = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatDate(value);
        break;
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DECIMAL_TYPE :
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_INTEGER_TYPE :
        valueContainer.innerHTML = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatNumber(value);
        break;
      case com.ibm.bpm.social.widgets.task.list.TaskRow.BD_BOOLEAN_TYPE :
        valueContainer.innerHTML =
          (value ? this._nlsResources["task-row-details-boolean-true"]
            : this._nlsResources["task-row-details-boolean-false"]);
        break;
      default:
        var valueParts = value.toString().split("\n");
        for ( var i = 0; i < valueParts.length; i++) {
          valueContainer.appendChild(document.createTextNode(valueParts[i]));
          if (i + 1 != valueParts.length) {
            valueContainer.appendChild(document.createElement("br"));
          }
        }
        break;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return valueContainer;
  },

  _expand: function _expand_TaskRow(evt){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    dojo.style(this.expanded, "display", "block");
    dojo.replaceClass(this.taskBoxAttach, "bpm-social-task-box-selected", ["bpm-social-task-box"]);
    this.open = true;
    if (evt) {
      this.onDetailsRequested(this.model);
    }
    this.onExpanded(this.model);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  onDetailsRequested: function onDetailsRequested_taskRow(model){},

  _collapse: function _collapse_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    dojo.style(this.expanded, "display", "none");
    dojo.replaceClass(this.taskBoxAttach, "bpm-social-task-box", ["bpm-social-task-box-selected"]);
    if (this.open) {
      this.open = false;
      this.onCollapsed(this.model);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  toggle: function toggle_TaskRow(evt){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    if (this.open) {
      // collapse
      this._collapse();
    }
    else {
      // expand
      this._expand(evt);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _clearDetailsArea: function _clearDetailsArea_TaskRow(){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    if (this._expandedConnects) {
      for ( var i = 0; i < this._expandedConnects.length; i++) {
        dojo.disconnect(this._expandedConnects[i]);
      }
      this._expandedConnects = null;
    }
    if (this.expanded.hasChildNodes()) {
      while (this.expanded.childNodes.length >= 1) {
        //this.expanded.firstChild.destroyRecursive(true);
        dojo.destroy(this.expanded.firstChild);
        //this.expanded.removeChild(this.expanded.firstChild);
      }
    }
    dojo.style(this.expanded, null);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  showMessage: function showMessage_TaskRow(message){
    this._clearDetailsArea();
    this.expanded.innerHTML = message;
  },

  _createMenuButton: function _createMenuButton_TaskRow(){
    var myself = this;
    if (this.model && this.model.availableActions) {

      var params = {
        task: this.model,
        availableActions: this.model.availableActions,
        taskDisplayName: this.model.TAD_DISPLAY_NAME
      };
      if (this._modifyTaskHandler) {
        this._modifyTaskHandler.init(params);
      }
      else {
        this._modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler(params);

        this.connect(this._modifyTaskHandler, "onTaskModified", this._handleTaskModifyUpdates);

        this.connect(this._modifyTaskHandler, "onError", this.onRefreshNeeded);

        this.connect(this._modifyTaskHandler, "onHide", dojo.hitch(this, function(modifyTaskHandler){
          // destroy this handler if task row has been reused while dialog was open
          if (this._modifyTaskHandler != modifyTaskHandler) {
            modifyTaskHandler.destroy();
          }
        }));
      }

      if (this._menu) {
        this._clearMenuButton();
      }

      this._menu = this._dropDownHelper.createDropDownForTaskRow(this.menuButton, this.model, this.model.availableActions, {
        modifyTask: function(){
          myself._modifyTaskHandler.showDialog();
        },
        reassignBackToGroup: function(){
          myself._modifyTaskHandler.reassignBackToGroup();
        }
      });

      if (this._menu) {        
        this._menu.connect(this._menu, "onOpen", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);
        }));
        this._menu.connect(this._menu, "onClose", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
        }));

        // task is associated with process
        if (typeof this.model["PI_PIID"] == "string") {
          var piid = this.model["PI_PIID"];
          this._processActionsDeferred = this._dropDownHelper.addProcessActionsOnFocus(this._menu, {
            "piid": piid
          }, {}, false, false, true /*omitCriticalPath*/);
          this._setupProcessActionsDeferred(this._processActionsDeferred);
        }
        com.ibm.bspace.common.util.ui.Helper.wrapElement(this.menuButton);
        dojo.style(this.menuButton, "display", "block");
      }
    }
  },

  _clearMenuButton: function _clearMenuButton_TaskRow(){
    if (this._menu) {
      this._menu.destroyRecursive();
      this._menu = null;
    }
  },

  onRefreshNeeded: function onRefreshNeeded_TaskRow(){},

  /**
   * @param model updated model
   */
  onModelUpdate: function onModelUpdate_TaskRow(model){},

  /**
   * 
   */
  onDeleteRowNeeded: function onDeleteRowNeeded_TaskRow(tkiid){},

  /*
   * Callback function to be executed when the update of the available actions 
   * for the item is finished successfully.
   * 
   * @param data response data
   * @returns {void} nothing
   */
  _handleUpdateAvailableActions: function _handleUpdateAvailableActions_TaskRow(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    var myself = this;
    var tasksForId = dojo.filter(data.tasks, function(task){
      return task.tkiid === myself.model.TKIID;
    });
    if (tasksForId.length > 0) {
      this.model.availableActions = tasksForId[0].actions;
      this._handleAvailableActions(data);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * callback function to be executed when the retrieval of available actions for the item is finished
   * successfully
   * 
   * @param data response data, ignored here because available actions are available on
   *        this.model.availableActions
   * @returns {void} nothing
   */
  _handleAvailableActions: function _handleAvailableActions_TaskRow(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);

    // enable menu
    this._createMenuButton();

    // enable onclick and onkeypress Enter on subject
    if (!this._connectsCallback) {
      this._connectsCallback = [];
      this._connectsCallback.push(dojo.connect(this.subject, "onclick", this, this._workOn));
      this._connectsCallback.push(dojo.connect(this.subject, "onkeydown", this, function(evt){
        if (evt.keyCode == dojo.keys.ENTER) {
          this._workOn(evt);
        }
      }));
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setupProcessActionsDeferred: function _setupProcessActionsDeferred(deferred){
    if (deferred != null) {
      deferred.then(dojo.hitch(this, "_handleAvailableProcessActions"));
    }
  },

  _triggerProcessActionsDeferred: function _triggerProcessActionsDeferred(piid /* string */, systemId){
    if (this._processActionsDeferred != null) {
      if (!this._processActionsReceived) {
        var deferred = this._processActionsDeferred;
        this._restHandler.getProcessActions(piid, function(result){
          deferred.resolve(result);
        }, function(){}, true /* async */, null /* parts */, systemId);
      }
    }
  },

  /**
   * Callback function to be executed when the retrieval of the available process actions of the itm
   * 
   * @param result
   * @returns {_handleAvailableProcessActions_TaskRow} {void} nothing
   */
  _handleAvailableProcessActions: function _handleAvailableProcessActions_TaskRow(result){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    this._processActionsReceived = true;
    if (result.standardActions != null) {
      if (dojo.indexOf(result.standardActions, "ACTION_ADD_COMMENT") != -1) {
        this._postInstanceComment = true;
      }
      else {
        this._postInstanceComment = false;
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return result;
  },

  onExpanded: function onExpanded_TaskRow(model){},

  onCollapsed: function onCollapsed_TaskRow(model){},

  _handleTaskModifyUpdates: function _handleTaskModifyUpdates_TaskRow(dueDateTime, priority, assigneeChanged, newAssignee){
    com.ibm.bpm.social.util.common.Log.logEntry(2, this.model.TKIID);
    var updatedValues = {
      "TASK.TKIID": this.model.TKIID,
      "TKIID": this.model.TKIID
    };
    if (dueDateTime) {
      updatedValues["DUE"] = dueDateTime;
    }
    if (priority) {
      updatedValues["PRIORITY"] = priority;
    }

    if (newAssignee) {
      updatedValues["OWNER"] = newAssignee;
      this.onModelUpdate(updatedValues, true);
    }
    else if (assigneeChanged) {
      // will be updated after push event is received
      this.group.innerHTML = this._nlsResources["task-row-details-loading"];
    }
    else {
      this.onModelUpdate(updatedValues, true);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }
});

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_STRING_TYPE = "String";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_INTEGER_TYPE = "Integer";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DECIMAL_TYPE = "Decimal";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_DATE_TYPE = "Date";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_TIME_TYPE = "Time";

com.ibm.bpm.social.widgets.task.list.TaskRow.BD_BOOLEAN_TYPE = "Boolean";
