// TasksListController.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.widgets.task.list.TasksListController");

dojo.require("com.ibm.bpm.social.portal.Topics");
dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.widgets.task.list.List");
dojo.require("com.ibm.bpm.social.widgets.task.list.Table");
dojo.require("com.ibm.bpm.social.widgets.task.list.SearchBar");
dojo.require("com.ibm.bpm.social.widgets.task.list.PaginationBar");
dojo.require("com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.EntityQuery");
dojo.require("com.ibm.bpm.social.widgets.task.util.ClaimDialog");
dojo.require("com.ibm.bpm.social.util.common.ProcessRestHandler");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.Action");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog");
dojo.require("com.ibm.bpm.social.util.common.PageSettings");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojo.cldr.supplemental");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");


dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.list.TasksListController",
    [dijit._Widget,
     dijit._Templated],
    {

      /**
       * Need 2 containers for calculation of list height. Outer container (class="bpm-task-list-view")
       * contains the paddings in css. Inner container ("listView") is needed to calculate the bottom
       * margin/padding
       */
      templateString: '\
        <div class="bpm-task-list-view">\
          <div dojoAttachPoint="listView">\
              <div class="bpm-task-list-header" dojoAttachPoint="listHeader">\
                <div dojoAttachPoint="listTitle" id="${id}_heading" class="bpm-task-list-header-title" ></div>\
                <div dojoAttachPoint="listRefresh" class="bpm-task-list-header-refresh" style="display:none;" onclick="com.ibm.bpm.social.portal.PageHandler.getInstance().selectWorkTab();" href="javascript:void(0);"></div>\
                <div dojoAttachPoint="listSearch" class="bpm-task-list-header-search"></div>\
				<div class="bpm-task-list-header-break"></div>\
                  <div dojoAttachPoint="collapseExpandAllButton" class="bpm-social-show-hide-button bpm-social-show-all" tabindex="0">\
                    <div dojoAttachPoint="collapseExpandAllButtonText" class="bpm-social-show-hide-button-text">&#9660;</div>\
                  </div>\
              </div>\
              <div dojoAttachPoint="listAction" class="bpm-task-list-actions">\
                <div class="bpm-task-list-title" dojoAttachPoint="listActionBarTitle" style="display:none;"></div>\
                <div class="bpm-task-list-action-right" dojoAttachPoint="listActionRight" role="tablist">\
                  <div dojoAttachPoint="collapseExpandAllButtonOldLocation"></div>\
                </div>\
              </div>\
              <div dojoAttachPoint="list" role="presentation">\
                  <div dojoAttachPoint="listMessage" class="bpm-task-list-content bpm-task-list-message-hide" ></div>\
                  <div dojoAttachPoint="listContent" class="bpm-task-list-content" tabindex="-1" >\
                    <div dojoAttachPoint="itemList"></div>\
                  </div>\
              </div>\
              <div dojoAttachPoint="pagination"></div>\
           </div>\
        </div>',

      widgetsInTemplate: true,

      listWidget: null,

      selectedFilterConfig: null,

      _isCEButtonInCollapsedState: true,

      _isLastCategoryInCollapsedState: false,

      _numberOfCollapsedCategories: 0,

      _moreItemsAvailable: true,

      _deferredForActions: null,

      _deferredForQuery: null,

      _totalCountCache: null,

      _totalCountLimitExceeded: false,

      _restHandler: null,

      _errorHandler: null,

      _refreshHandler: null,

      _subscribedTaskCompletion: null,

      _subscribedInstanceView: null,

      _subscribedShowDialog: null,

      _subscribedHideDialog: null,

      _scrollTop: 0,

      _startPageSettings: null,

      _searchFilterApplied: false,

      _searchText: null,

      /**
       * 
       * @type {void}
       */
      constructor: function constructor_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController");

        // keep track of created widgets
        this._childWidgets = [];

        // flag whether last category is collapsed, which requires special behavior
        this._isLastCategoryInCollapsedState = false;

        // flag whether any visible category is collapsed, which requires special behavior
        this._numberOfCollapsedCategories = 0;
        // make sure the count is set to null initially, as non-null will use the cached version  
        this._totalCountCache = null;
        this._totalCountLimitExceeded = false;

        // flag whether more items than requested are available (we do fetch one more)
        this._moreItemsAvailable = true;

        // object containing to dojo.subscribe handle if we are waiting for claim dialog response
        this._waitingForClaimDialogChannel = null;

        // track settings for the start page 
        this._startPageSettings = new com.ibm.bpm.social.util.common.PageSettings();

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      destroyChildren: function destroyChildren_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // destroy all children
        while (this._childWidgets.length > 0) {
          var widget = this._childWidgets.pop();
          if (widget) {
            widget.destroy();
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      destroy: function destroy_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        for ( var i = 0; i < this._connectHandlers.length; i++) {
          try {
            dojo.disconnect(this._connectHandlers[i]);
          }
          catch (e) {
            com.ibm.bpm.social.util.common.Log.logError(2, "destroy_TasksListController", e);
          }

        }
        this._connectHandlers = null;

        if (this._confirmClaimDialog) {
          this._confirmClaimDialog.destroy();
        }

        if (this._refreshHandler) {
          this._refreshHandler.destroy();
        }

        this._unSubscribeViewChanges();

        // destroy child widgets
        this.destroyChildren();


        com.ibm.bpm.social.util.common.Log.logExit(2);
        this.inherited(arguments);
      },

      /**
       * postCreate
       */
      postCreate: function postCreate_TasksListController(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        //com.ibm.bpm.social.util.common.Util.loadStylesheet("css/css_sprite.css");
        // Including the entire Portal.css stylesheet causes problems for the page layout. (Assertion by Portal team)
        // Styles should be copied to other stylesheets on an as needed basis.
        // com.ibm.bpm.social.util.common.Util.loadStylesheet("css/Portal.css"); 

        this.containerNode = this.domNode;

        // TODO find a place for creation with params, e.g. endpoint
        this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
        this._processRestHandler = com.ibm.bpm.social.util.common.ProcessRestHandler.getInstance();
        this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
        this._connectHandlers = new Array();

        // connect onresize    
        this._connectHandlers.push(this.connect(window, "onresize", "_onResize"));

        //
        // collapse/expand all button
        this.collapseExpandAllButton.title = this._nlsResources["task-list-expand-all"];

        var self = this;
        // onclick 
        this.connect(this.collapseExpandAllButton, "ondijitclick", function(){
          if (self._isCEButtonInCollapsedState) {
            self._showCollapseAllButton();
            self.listWidget.expandAll();
          }
          else {
            self._showExpandAllButton();
            self._collapsingAll = true;
            self.listWidget.collapseAll();
            self._collapsingAll = false;
            self._onResize();
          }
        });
        // hover
        this.connect(this.collapseExpandAllButton, "onmouseover", function(){
          if (self._isCEButtonInCollapsedState) {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-show-all");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-show-all-hover");
          }
          else {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-hide-all");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-hide-all-hover");
          }
        });
        // mouse out
        this.connect(this.collapseExpandAllButton, "onmouseout", function(){
          if (self._isCEButtonInCollapsedState) {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-show-all-hover");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-show-all");
          }
          else {
            dojo.removeClass(self.collapseExpandAllButton, "bpm-social-hide-all-hover");
            dojo.addClass(self.collapseExpandAllButton, "bpm-social-hide-all");
          }
        });
		setTimeout(function() {
			if(self !== undefined)
				self._onResize();
		}, 300);
      },

      _createTableConfigObject: function _createTableConfigObject_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var activeQueue = this.config.queue;
        this.config.priorities = {
          type: "defaultPriority",
          showImage: false
        };
        var formatterFactory = new com.ibm.bpm.social.widgets.task.util.PropertyFormatter({
          config: this.config,
          queueConfig: activeQueue,
          widgetId: this.id
        });

        var displayedProperties = [];
        var property = null;


        // we need to map the properties received from the backend to map to the table config object
        for ( var i = 0; i < activeQueue.item.properties.length; i++) {
          property = activeQueue.item.properties[i];
          if (property.displayed) {
            var linkToAction = null;
            if (property.property == "TAD_DISPLAY_NAME") {
              linkToAction = "runTask";
            }
            else if (property.property == "PI_NAME") { // "PI_DISPLAY_NAME" is filtered out in queue._setAtrributes 
              linkToAction = "viewInstance";
            }

            // choose formatter
            var formatterType = null;
            if (property.property == "ASSIGNED_TO_ROLE") {
              formatterType = "ASSIGNED_TO_ROLE";
            }
            else if (property.property == "ASSIGNED_TO_PARTICIPANT") {
              formatterType = "ASSIGNED_TO_ROLE";
            }
            else if (property.property == "PRIORITY") {
              formatterType = "defaultPriority";
            }
            else {
              formatterType = property.type;
            }
            var formatter = formatterFactory.getFormatter(formatterType, false, property.title);

            displayedProperties.push({
              id: property.property,
              label: property.title,
              formatter: formatter,
              description: property.description,
              inline: property.inline,
              labelDisplayed: property.labelDisplayed,
              isSort: property.isSort,
              isSortable: property.isSortable,
              isFilterable: property.isFilterable,
              sortDir: property.sortDir,
              linkToAction: linkToAction,
              type: property.type
            });
          }
        }

        var config = { // table config
          displayedProperties: displayedProperties,
          labelDisplayed: true,
          supportedDndTypes: null,
          publishedDndTypes: null,
          parentProperty: null, // TODO ASZ this.config.parentProperty
          noItemsFoundMessage: this._nlsResources["task-list-no-tasks"],
          activeGroupByProperty: null, // TODO ASZ  this.config.activeGroupByProperty,
          groupFormatter: null,
          isSelectable: false,
          fitToContent: false,
          isFilterable: false,
          sortProperties: this.selectedFilterConfig.sortProperties
        };

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return config;
      },

      /**
       * 
       * @type {void}
       */
      _createInlineActions: function _createInlineActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var actions = [];

        // Note: Order matters (defines the order in the actions table)

        var runTask = new com.ibm.bpm.social.widgets.task.util.Action({
          id: "runTask",
          titleKey: this._nlsResources["task_action_run_task"],
          tooltip: this._nlsResources["task_action_run_task_description"],
          enabled: false,
          hoverClass: "bpm-social-run-hover",
          iconClass: "bpm-social-run",
          disabledClass: "bpm-social-run-disabled action_disabled"
        });
        actions.push(runTask);

        var viewInstance = new com.ibm.bpm.social.widgets.task.util.Action({
          id: "viewInstance",
          titleKey: this._nlsResources["task_action_view_instance_details"],
          tooltip: this._nlsResources["task_action_view_instance_details_description"],
          enabled: false,
          hoverClass: "bpm-social-details-hover",
          iconClass: "bpm-social-details",
          disabledClass: "bpm-social-details-disabled action_disabled"
        });

        actions.push(viewInstance);

        if (this.config.queue.entityTypeName != "PROCESS_INSTANCE") {
          var modifyTask = new com.ibm.bpm.social.widgets.task.util.Action({
            // this is a meta-action that includes the actions Assign, Reassign back to group, change due date, change priority 
            id: "modifyTask",
            titleKey: this._nlsResources["task_menu_modify_task"],
            tooltip: this._nlsResources["task_dialog_modifyTask_description"],
            enabled: false,
            hoverClass: "bpm-social-edit-hover",
            iconClass: "bpm-social-edit",
            disabledClass: "bpm-social-edit-disabled action_disabled"
          });
          actions.push(modifyTask);
        }
        else {
          var modifyInstance = new com.ibm.bpm.social.widgets.task.util.Action({
            id: "modifyInstance",
            titleKey: this._nlsResources["process_menu_modify_instance"],
            tooltip: this._nlsResources["process_menu_modify_instance_description"],
            enabled: false,
            hoverClass: "bpm-social-edit-hover",
            iconClass: "bpm-social-edit",
            disabledClass: "bpm-social-edit-disabled action_disabled"
          });
          actions.push(modifyInstance);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return actions;
      },

      /**
       * Set the Controller config
       * 
       * @param {com.ibm.bpm.social.widgets.task.config.ControllerConfig} config
       * 
       */
      setConfig: function setConfig_TasksListController(config){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.config = config;
        this._isDirty = true;

        // initialize refresh handler
        if (this._refreshHandler === null) {
          var deltaRefresh = true;
          if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "table") {
            deltaRefresh = false;
          }
          this._refreshHandler =
            new com.ibm.bpm.social.widgets.task.list.TasksListRefreshHandler(dojo.hitch(this,
              this._refreshHandlerFullRefresh),
              dojo.hitch(this, this._setModel),
              dojo.hitch(this, this._updateModel),
              this.id,
              deltaRefresh);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * calculate the height for the list content. return list content height in px
       */
      _getListContentHeight: function _getListContentHeight_TasksListController(){
        var viewport = dojo.window.getBox();
        var posNode = dojo.position(this.domNode);
        var posListView = dojo.position(this.listView);
        var bottom = posNode.y + posNode.h - posListView.y - posListView.h;
        var posList = dojo.position(this.list);
        var posPagination = dojo.position(this.pagination);
        var result = viewport.h - posList.y - posPagination.h - bottom;
        // no min height
        if (result < 0) {
          result = 0;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "height: " + result + " px");
        return result;
      },

      /**
       * return list content width in px
       */
      _getListContentWidth: function _getListContentWidth_TasksListController(){
        // TODO clientWidth is too small if we are actually scrolling, 
        // however using offsetWidth results in the table having outer scroll bars
        var result = this.listContent.clientWidth;

        // container size will be limited to 900, so this should not happen
        if (result < 400) {
          result = 400;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "width:" + result + " px");
        return result;
      },

      /**
       * create the visible parts
       */

      createContent: function createContent_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.config) {

          var self = this;

          // delete old content

          if (this._childWidgets.length > 0) {
            this.destroyChildren();
          }

          // create new content

          // create filter area
          if (this.config && this.config.filterConfigs && this.config.filterConfigs.length > 1) {
            dojo.forEach(this.config.filterConfigs, function(filter, index){
              var filterSpan = document.createElement("div");
              dijit.setWaiRole(filterSpan, "tab");
              filterSpan.tabIndex = "0";
              dojo.style(filterSpan, "display", "inline-block");
              dojo.addClass(filterSpan, "bpm-task-list-filter");
              filterSpan.innerHTML = self._nlsResources[filter.titleKey];
              self.listActionRight.insertBefore(filterSpan, self.collapseExpandAllButtonOldLocation);
              self[filter.id + "_filter"] = filterSpan;
              self.connect(filterSpan, "ondijitclick", function(e){
                if (filter.id !== self.config.selectedFilter) {
                  self._doFilter(filter);
                }
              });

              if (index > 0) {
                var sepSpan = document.createElement("div");
                dojo.style(sepSpan, "display", "inline-block");
                sepSpan.innerHTML = "|";
                dijit.setWaiState(sepSpan, "hidden", "true");
                self.listActionRight.insertBefore(sepSpan, filterSpan);
              }
            });
          }

          var selectedFilter = this.config.selectedFilter;
          if (selectedFilter) {
            var selectedFilterFindings = dojo.filter(this.config.filterConfigs, function(filter){
              return filter.id === selectedFilter;
            });
            if (selectedFilterFindings.length > 0) {
              this._setFilterConfig(selectedFilterFindings[0]);
              if (this.config.filterConfigs.length > 1) {
                this._decorateSelectedFilter(selectedFilterFindings[0]);
              }
            }

            // set title
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
              this.listTitle.innerHTML = "";
              this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
            }
            else {
              if (this.config.viewConfig.titleKey) {
                this.listTitle.innerHTML = this._nlsResources[this.config.viewConfig.titleKey];
              }
              else {
	              this.listTitle.innerHTML = "";
    	          this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
              }
            }

            // If user is using Safari on iPad, display refresh icon for work view
            if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) {
              dojo.style(this.listRefresh, "display", "block");
            }

            // determine categories
            var categories = null;
            if (this.selectedFilterConfig && this.selectedFilterConfig.categories) {
              if (typeof this.selectedFilterConfig.categories == "function") {
                this.selectedFilterConfig.categories();
              }
              categories = this.selectedFilterConfig.categories;
            }

            // create query
            if (!this._query) {
              var restHandler = com.ibm.bpm.social.util.common.QueryRestHandler.getInstance();

              this._query = new com.ibm.bpm.social.widgets.task.util.EntityQuery({
                restHandler: restHandler,
                //          D2DFilter: parameters.D2DFilter,             
                widgetId: this.config.id,
                systems: this._restHandler.getSystems(),
                userId: com.ibm.bpm.social.util.common.Util.getCurrentUser().userName
              });
            }

            // check if collapse/expand is possible
            if (!this.config.viewConfig.allowCollapseExpand) {
              this.listActionRight.style.display = "none"; // no expand/collapse action on table views
            }

            // create the view widget
            if (this.config.viewConfig.viewType == "table") {
              // table has its own scroll bars
              this.listContent.style.overflow = "hidden";

              dijit.setWaiRole(this.listActionRight, "presentation");
              this.listWidget = new com.ibm.bpm.social.widgets.task.list.Table({}, this.itemList);
              this._isDirty = true;

              this._connectHandlers.push(this.connect(this.listWidget, "onActionRequested", "_onActionRequested"));
              this._connectHandlers.push(this.connect(this.listWidget, "onSort", "_onSort"));

            }
            else { //  default is List view 
              this.listWidget = new com.ibm.bpm.social.widgets.task.list.List({
                config: {
                  controller: this,
                  filter: this.selectedFilterConfig,
                  type: this.config.type
                }
              }, this.itemList);

              this._connectHandlers.push(this.connect(this.listWidget, "onCategoryCollapsed", "_onCategoryCollapsed"));
              this._connectHandlers.push(this.connect(this.listWidget, "onCategoryExpanded", "_onCategoryExpanded"));
              this._connectHandlers.push(this.connect(this.listWidget, "onWorkOn", "_workOn"));
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskDetailsNeeded", "_getBulkTaskDetails"));
              this._connectHandlers.push(this.connect(this.listWidget, "onRefreshNeeded", "_refresh"));
              this._connectHandlers.push(this.connect(this.listWidget, "onDecreaseLocalCount",
                function(){
                  if (this._paginationWidget.localCount > 1) {
                    this._paginationWidget.setLocalCount(this._paginationWidget.localCount - 1,
                      this._totalCountLimitExceeded);
                  }
                  else {
                    this._refresh(true);
                  }
                }));
              this._connectHandlers.push(this.connect(this.listWidget, "onDecreaseGlobalCount",
                function(){
                  this._totalCountCache--;
                  this._paginationWidget.setGlobalCount(this._paginationWidget.globalCount - 1,
                    this._totalCountLimitExceeded);
                }));
              this._connectHandlers.push(this.connect(this.listWidget, "onIncreaseLocalCount", function(){
                if (this._paginationWidget.localCount === 0) {
                  this._removeMessage();
                  dojo.style(this.collapseExpandAllButton, "display", "block");
                }
                this._paginationWidget.setLocalCount(this._paginationWidget.localCount + 1, this._totalCountLimitExceeded);
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onIncreaseGlobalCount",
                function(){
                  this._totalCountCache++;
                  this._paginationWidget.setGlobalCount(this._paginationWidget.globalCount + 1,
                    this._totalCountLimitExceeded);
                }));

              // no refresh on notifications while a dialog is open or any task row is expanded
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskRowExpanded", function(item){
                if (this.selectedFilterConfig.id === "openTasks") {
                  var idString = item.systemID ? item.systemID + "_" + item["TASK.TKIID"] : item["TASK.TKIID"];
                  this._refreshHandler.suspendFullRefresh("taskRowExpanded_" + idString);
                }
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onTaskRowCollapsed", function(item){
                if (!this._collapsingAll) {
                  this._onResize();
                }
                if (this.selectedFilterConfig.id === "openTasks") {
                  var idString = item.systemID ? item.systemID + "_" + item["TASK.TKIID"] : item["TASK.TKIID"];
                  this._refreshHandler.resumeFullRefresh("taskRowExpanded_" + idString);
                }
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onFullRefreshNeeded", function(){
                this._refreshHandler.doFullRefresh();
              }));
              this._connectHandlers.push(this.connect(this.listWidget, "onFocusNeeded", function(){
                com.ibm.bpm.social.util.common.Log.log(3, "Focus active task list tab header");
                this.onFocusNeeded();
              }));
            }

            this._connectHandlers.push(dojo.connect(this.listWidget, "onDetailsRequested", this, function(item){
              var taskId = item["TASK.TKIID"];
              self.listWidget.showMessage(self._nlsResources["task-row-details-loading"], taskId);

              // Fetch data from the server
              self._restHandler.getTask(taskId, function(task){
                self._setTaskDetails(task);
                self._onResize();
              }, function(data, ioArgs){
                // task is assigned to somebody else or deleted
                if (ioArgs.xhr.status == 401 || ioArgs.xhr.status == 404) {
                  self.listWidget.showMessage(self._nlsResources["task-row-details-error-not-available"], taskId);
                }
                else {
                  self.listWidget.showMessage(data.message, taskId);
                }
              }, true, null, item.systemID);
            }));

            this._connectHandlers.push(this.connect(this.listWidget, "onShowCollapseAllButton", function(){
              if (self._isCEButtonInCollapsedState) {
                self._showCollapseAllButton();
              }
            }));

            // pagination bar
            this._paginationWidget = new com.ibm.bpm.social.widgets.task.list.PaginationBar({}, this.pagination);
            this.pagination = this._paginationWidget.domNode;
            this._connectHandlers.push(this.connect(this._paginationWidget, "onShowMore", "_refresh"));

            // Search bar          
            this._searchWidget = new com.ibm.bpm.social.widgets.task.list.SearchBar({}, this.listSearch);
            if (this._searchText) {
              this._searchWidget.setSearchText(this._searchText);
            }
            //this.listSearch = this._searchWidget.domNode;
            this.connect(this._searchWidget, "onSubmit", "_search");
            this.connect(this._searchWidget, "onClear", "_clearSearch");
            this._setSearch();
            // TODO filter

          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Handler for sort requests (currently only from table)
       * 
       * @param {String} property column / property used as sort criteria
       * @param {String} direction one of {ASC|DESC}
       * @type {void}
       */
      _onSort: function _onSort_TasksListController(property, direction){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var newSortProp = {
          isSort: true,
          property: property,
          sortDir: direction
        };

        // ensure no doubble sort
        this.selectedFilterConfig.sortProperties = dojo.filter(this.selectedFilterConfig.sortProperties, function(sortProp){
          return sortProp.property != newSortProp.property;
        });

        this.selectedFilterConfig.sortProperties.unshift(newSortProp);


        // remove older sort properties ( 2 are allowed )
        while (this.selectedFilterConfig.sortProperties.length > 2) {
          this.selectedFilterConfig.sortProperties.pop();
        }


        this._setFilterConfig(this.selectedFilterConfig);
        this.listWidget.setSortProperties(this.selectedFilterConfig.sortProperties);

        this.refresh();


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Generic handler on all actions send out from the listWidget
       * 
       * Note: this is only used for the table
       * 
       * @param {String} actionId
       * @param {Object} data a object specific to the action, e.g. a task or other item
       * @type {void}
       */
      _onActionRequested: function _onActionRequested_TasksListController(actionId, data){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        if (actionId) {
          if (actionId == "runTask") {
            this._workOn(data);
          }
          else if (actionId == "modifyTask") {

            // Fetch complete task from the server since the saved search may not contain all necessary fields
            var task = null;
            this._restHandler.getTask(data["TASK.TKIID"], function(result){
              task = result;
            }, function(response, ioArgs){
              var name = data.TAD_DISPLAY_NAME || data.NAME || data["TASK.TKIID"];
              var statusCodeMap = {
                401: {
                  key: "KEY_TaskModifyNotAuthorized",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                },
                404: {
                  key: "KEY_UpdateNotExistingTask",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                },
                409: {
                  key: "KEY_TaskModifyNotAvailable",
                  hideDetails: false,
                  parameters: {
                    1: name
                  }
                }
              };
              myself._errorHandler.showError(response, ioArgs, statusCodeMap, null, null);
            }, false, null, data.systemID);
            if (task) {
              task.availableActions = data.availableActions;
              this._modifyTask(task);
            }
            else {
              this.refresh();
            }
          }
          else if (actionId == "modifyInstance") {
            this._modifyInstance(data);
          }
          else if (actionId == "viewInstance") {
            this._viewInstance(data);
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * 
       * @type {void}
       */
      _modifyTask: function _modifyTask_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!this._modifyTaskHandler) {
          this._modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler({
            task: item,
            availableActions: item.availableActions
          });
          this._childWidgets.push(this._modifyTaskHandler);
          this._connectHandlers.push(this.connect(this._modifyTaskHandler, "onTaskModified", this.refresh));
        }
        else {
          this._modifyTaskHandler.init({
            task: item,
            availableActions: item.availableActions
          });
        }

        this._modifyTaskHandler.showDialog();

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },
      /**
       * 
       * @type {void}
       */
      _modifyInstance: function _modifyInstance_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        item.piid = item.piid || item["PROCESS_INSTANCE.PIID"];

        var process = null;
        var myself = this;
        this._processRestHandler.getProcess(item.piid, function(responseObject, ioArgs){
          process = responseObject;
        }, function(data, ioArgs){
          com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_modifyInstance_error");
          com.ibm.bpm.social.util.common.Log.logError(2, data, ioArgs);
          myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListData" +
            com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(myself.config.type), null);
          com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_modifyInstance_error");

        }, false, null, item.systemID);

        if (process) {

          this._modifyInstanceDialog = new com.ibm.bpm.social.widgets.dialogs.ChangeDueDateDialog({
            "instanceId": process.piid,
            "dueDate": process.dueDate,
            "callback": dojo.hitch(this, this._onInstanceModified)
          }, dojo.create("div"));
          this._modifyInstanceDialog.show();

          // Note: ChangeDueDateDialog destroys itself on hide. Just ensure that it is cleaned otherwise 
          this._childWidgets.push(this._modifyInstanceDialog);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * 
       * @param {Object} item an object having a piid set
       * @type {void}
       */
      _viewInstance: function _viewInstance_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (item) {
          item.piid = item.piid || item["PROCESS_INSTANCE.PIID"];
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
            'instanceId': item.piid
          }]);
        }
        else {
          com.ibm.bpm.social.util.common.Log.logError(1, "Error: Received item without piid set.");
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Callback for the ChangeDueDateDialog (_modifyInstanceDialog)
       */
      _onInstanceModified: function _onInstanceModified_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      _onCategoryCollapsed: function _onCategoryCollapsed_TasksListController(categoryId, count){
        // TODO streamline strategy how many more items we fetch
        com.ibm.bpm.social.util.common.Log.log(1, "Category '" + categoryId + "' collapsed, possibly needing " + count +
          " more items.");
        this._numberOfCollapsedCategories++;
        if (this._numberOfCollapsedCategories > 6) {
          com.ibm.bpm.social.util.common.Log.log(1, "Warning: count for collapsed items greater than 6");
          this._numberOfCollapsedCategories = 6;
        }
        this._refresh(true);
      },

      _onCategoryExpanded: function _onCategoryExpanded_TasksListController(categoryId, count){
        com.ibm.bpm.social.util.common.Log.log(1, "Category '" + categoryId + "' expanded, showing " + count +
          " more items.");
        this._numberOfCollapsedCategories--;
        if (this._numberOfCollapsedCategories < 0) {
          com.ibm.bpm.social.util.common.Log.log(1, "Warning: count for collapsed items less than 0");
          this._numberOfCollapsedCategories = 0;
        }
        this._refresh(true);
      },

      refresh: function refresh_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        try {

          // refresh from outside, show loading message instead of previous content
          this._showMessage("task-list-loading");

          this._refresh(false);
        }
        catch (xcp) {
          com.ibm.bpm.social.util.common.Log.logError(1, "", xcp);
          throw (xcp);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _refreshHandlerFullRefresh: function _refreshHandlerFullRefresh_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._saveListFocus();
        this._refresh(true);
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _refresh: function _refresh_TasksListController(autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "autoRefresh=" + autoRefresh);

        // TODO: set collapse-all/expand-all button to default
        // For this, encapsulate the button stuff as reusable method (currently all done in postCreate)

        // save scroll position
        this._scrollTop = 0;
        if (this.listContent) {
          com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
            this.listContent.scrollHeight + ", scrollTop: " + this.listContent.scrollTop);
          this._scrollTop = this.listContent.scrollTop;
        }

        // load items 
        var size = this._paginationWidget.getPageSize();
        this._refreshItems(true, autoRefresh, size);

        // Reset expandCollapseAll button
        this._showExpandAllButton();

        // indicate that we had a refresh
        this.onRefreshDone();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * Send out event to TasksContainer that we did a refresh
       * 
       * @type {void}
       */
      onRefreshDone: function onRefreshDone_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        com.ibm.bpm.social.util.common.Log.logExit(3);
      },

      /*
       * Callback method for API call to fetch items from server.
       * @param {Object} autoRefresh indicates whether this is an auto-refresh-run or not
       */
      _setModel: function _setModel_TasksListController(autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // this method might be called by an async XHR response
        if (this._destroyed) {
          com.ibm.bpm.social.util.common.Log.log(1, "List is already destroyed.");
          return;
        }


        // TODO autorefresh?
        var count = 0;
        var model = this.items;

        // compute whether to show the pagination button or not before calling this._onResize() later on
        if (this._paginationWidget) {
          if (this._moreItemsAvailable === false) {
            // hide Show more button 
            this._paginationWidget.showPaginationButton(false);
          }
          else {
            // show Show more button 
            this._paginationWidget.showPaginationButton(true);
          }
        }

        // special behaviour: when there are categories collapsed, then we will show the collapsed categories
        // with no items on the screen (rather than the "no items found" message)
        if (!model || (model.length === 0 && this._numberOfCollapsedCategories === 0)) {
          com.ibm.bpm.social.util.common.Log.log(1, "No Tasks.");
          var noItemsMessage = null;
          if (this.config.queue.entityTypeName == "TASK") {
            noItemsMessage = "task-list-no-tasks";
          }
          else if (this.config.queue.entityTypeName == "PROCESS_INSTANCE") {
            noItemsMessage = "task-list-no-instances";
          }
          else {
            var e = new Error("Unexpected entityTypeName '" + this.config.queue.entityTypeName + "'");
            com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected entityTypeName '" + this.config.queue.entityTypeName +
              "'", e);
            throw e;
          }

          this._showMessage(noItemsMessage);
          dojo.style(this.collapseExpandAllButton, "display", "none");

          // set list widget to correct state - no items
          this.listWidget.setModel(model);

          // Update pagination bar
          this._paginationWidget.setLocalCount(0);
          this._paginationWidget.setGlobalCount(0);

        }
        else {
          // retrieve available actions async
          this._retrieveAvailableActionsForQueryResult(this.items);

          // count items; thereby ignore the last one if it is the "extra item"
          var effectiveLength = 0;
          var i;
          var removeExtra = false;
          for (i = 0; i < model.length; i++) {
            if (model[i].isExtra !== true) {
              effectiveLength++;
            }
            else {
              removeExtra = true;
            }
          }
          count = effectiveLength;

          // categorize into groups (do this here, the we can reuse maybe for table)

          // update List widget


          if (this.config.viewConfig.viewType == "table") {
            if (this._isDirty) {
              var tableConfig = this._createTableConfigObject();
              this.listWidget.setConfig(tableConfig);
              this._isDirty = false;
            }
            if (model.length > 0 && removeExtra) {
              // table doesn't handle the extra item we fetched for pagination 
              model.pop();
            }
          }


          // set page size first, because this helps determine which sections will be hidden 
          this.listWidget.setPageSize(this._paginationWidget.getPageSize());
          // this is a more expensive operation

          // PERFORMANCE TEST var ts = new Date();
          this.listWidget.setModel(model);
          // PERFORMANCE TEST var te = new Date();
          // PERFORMANCE TEST console.debug("Tasks: " + count + " time: " + dojo.date.difference(te, ts, "millisecond"));

          // Setting counters

          // Note that we do caching for the total count, as this will in many cases not change over time, 
          // thus we avoid to go to the backend for the total count whenever we can.
          // However there may be situations where the total count changes over time (e.g.  by the fact that we operate
          // in a multi-user environment). 
          // As this could lead to an error message like "Showing 43 of approximately 41 results", we will at least
          // avoid this case: thus, the minimal heuristic is
          //   "if the local count is found greater that the global count, increase the global count without asking the backend".
          if (this._totalCountCache && this._totalCountCache < count) {
            // update cache value 
            this._totalCountCache = count;
            // We always request one extra item to decide whether we should render the Show more button.
            // Thus, we could possibly have received more items than our cached totalCount value.
            // In this case, we know that the cached value is too small;
            // to avoid the extra call out to the backend, we show the (known to be wrong) cached value for the total count anyway
            // (at least we have corrected it to be not smaller than what we show)
            // In addition, in this case we switch to the alternate message "showing x of more than y".
            // Flag will be cached as well, and will be reset later when once we should get an accurate count again.
            // Note: the count will be accurate only when all categories are expanded.
            if (this._moreItemsAvailable === true) {
              this._totalCountLimitExceeded = true;
            }
          }
          // Update pagination bar
          this._paginationWidget.setLocalCount(count, this._totalCountLimitExceeded);

          if (this._totalCountCache != this._paginationWidget.getGlobalCount()) {
            // update value rendered in the pagination widget
            this._paginationWidget.setGlobalCount(this._totalCountCache, this._totalCountLimitExceeded);
          }

          dojo.style(this.collapseExpandAllButton, "display", "block");

          this._onResize();

          this._removeMessage();
        }

        var focusRestored = false;
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          focusRestored = this.listWidget.restoreFocus(autoRefresh);
        }

        if (!focusRestored) {
          // restore scroll position approximately
          if (this.listContent && this._scrollTop) {
            com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
              this.listContent.scrollHeight + ", scrollTop: " + this.listContent.scrollTop);
            this.listContent.scrollTop =
              Math.min(this._scrollTop, this.listContent.scrollHeight - this.listContent.clientHeight);
          }
        }

        this._refreshHandler.refreshFinished();

        // listen for view changes to suspend/resume automatic refresh
        this._subscribeViewChanges();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _updateModel: function _updateModel_TasksListController(tasks, notNewAssignment){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (this.listWidget && this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          // map event properties to query properties
          var items = com.ibm.bpm.social.widgets.task.config.FilterConfig.mapTasks(tasks);
          this.listWidget.updateModel(items, false, this._searchFilterApplied, notNewAssignment);
        }
        else {
          throw new Error("Programming error: No delta refresh (updateModel) for com.ibm.bpm.social.widgets.task.list.Table!");
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /*
       * Refresh the list content.
       * @param {boolean} async indicates whether the list shall be loaded asynchronous or synchronous
       * @param {Object} autoRefresh indicates whether this is an auto-refresh-run or not
       */
      _refreshItems: function _refreshItems_TasksListController(async, autoRefresh, size){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        if (this._deferredForQuery) {
          // another query is still in progress, cancel it
          this._deferredForQuery.cancel();
        }

        this._refreshHandler.refreshStarted(autoRefresh);

        // Set the default if parameters have not be specified
        if (this._flagInitConfiguration) {
          // ASZ: There is no _initConfiguration, thus removing
          // this._initConfiguration(this.config);
          this._flagInitConfiguration = false;
        }

        try {
          // If async is not defined, the list is loaded asynchronously.
          if (!async) {
            // Default is asyn because otherwise the GUI would stall.
            // If pagination triggers the refresh, no arguments are provided and the default apply.
            async = true;
          }
          // If autoRefresh is not defined, it is assumed that this is not an auto-refresh-run.
          if (autoRefresh === null || autoRefresh !== true) {
            autoRefresh = false;
          }
          // Reset selectedItems, because the new set of items will most likely differ from the old one.
          // Basically this also enforces that only visible items can be selected.
          // aschoen: Are there situation when the refresh is triggered as callback from the pagination and the selection
          // aschoen: need to be kept?
          // aschoen: For example because the user enters a item number which has been shown before.
          // aschoen: But I doubt this because even if the same window (:= (first,last)) is shown, the data may have changed.
          // aschoen: If more logic is required, we need to move this to the loadCallback method. There we could check whether
          // aschoen: the already selected items are visible.
          this.selectedItems = [];

          // Reset the toolbar actions.
          // TODO
//          var allowedActions = this._getAllowedActions();
//          this.toolbarWidget.setAvailableActions(allowedActions);

          this._loadItems(async, autoRefresh, 0, size);
        }
        catch (e) {
          if (e && e.type == "list") {
            this._showMessage(e.messageKey, e.severity, e.parameters);
          }
          else {
            throw e;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _loadItems: function _loadItems_TasksListController(async, autoRefresh, offset, pageSize){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // clear the query object 
        this._query.clear();

        // Set defaults
        if (offset === null || offset === undefined || (offset < 0)) {
          offset = 0;
        }
        if (pageSize === null || pageSize === undefined || (pageSize <= 0)) {
          pageSize = this.config["pageSize"];
        }

        com.ibm.bpm.social.util.common.Log.log(1, " offset: " + offset);
        com.ibm.bpm.social.util.common.Log.log(1, " pageSize: " + pageSize);

        // if there was a problem detected with the current queue
        // throw the exception and don't process the data
        if (this.config.queue.exception) {
          this._flagErrorCaseRefreshLoadItems = false;
          throw this.config.queue.exception;
        }
        else {
          // update the queue title
          if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS) {
              this.listTitle.innerHTML = "";
              this.listTitle.appendChild(document.createTextNode(this.config.queue.getDisplayName()));
              var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=savedsearch&tw.local.id='+encodeURIComponent(this.config.queue.name);
              if (this._searchText) {
              	newHash = newHash + "&tw.local.searchFilter=" + encodeURIComponent(this._searchText);
              }
              com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
          } else {
	          // update URL hash
	          var newHash = 'dashboards/TWP/BPM_WORK?tw.local.view=tasks';
	          if (this.config.selectedFilter == 'openTasks') {
	          	newHash = newHash + '&tw.local.state=open';
	          }
	          if (this.config.selectedFilter == 'completedTasks') {
	          	newHash = newHash + '&tw.local.state=close';
	          }
	          if (this._searchText) {
	          	newHash = newHash + '&tw.local.searchFilter=' + encodeURIComponent(this._searchText);
	          }
	       	  com.ibm.bpm.social.util.common.Util.setCurrentHash(newHash);
          }
          
          // choose type
          this._query.setType(this.config.queue);
          // set search capability
          this._checkSearchCapability();
          this._setSearch();

          if (offset === 0) {
            // reinitialize list of dojo.Deferred for retrieved tasks
            this._deferredForActions = null;
          }
          var self = this;
          // We fetch one object more from the backend, to decide whether we should render a "more available" indicator 
          this._updateQuery(async, pageSize + 1, offset);
          // only for list view type we have categories
          if (this.config.selectedFilter != com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_CUSTOM_VIEWS &&
            this.listWidget) {
            if (this.selectedFilterConfig && this.selectedFilterConfig.selectedProperties) {
              this._query.setSelectedAttributes(this.selectedFilterConfig.selectedProperties);
            }
            if (this.selectedFilterConfig && this.selectedFilterConfig.categories && this.listWidget._categoryWidgets) {
              if (this.listWidget._categoryWidgets.length > 0 && this.listWidget.getExpandedCategoriesCount() === 0) {
                this._paginationWidget.setLocalCount(0, this._totalCountLimitExceeded);
                // when we come here, then all are collapsed.
                // Make sure that all categories are shown on the screen: 
                this.listWidget.setModel([]);
                com.ibm.bpm.social.util.common.Log.logExit(1,
                  "Exiting because no category is expanded -> no need to load something");
                return;
              }

              var tmpWhereClause = null;
              tmpWhereClause = this._computeWhereClauseForExpandedSections(this.listWidget._categoryWidgets);
              if (tmpWhereClause) {
                var tmpFilter = {
                  whereClause: tmpWhereClause
                };
                this._query.setCustomFilter(tmpFilter);
              }
            }
          }
          // set the search text
          this._query.setSearchFilter(this._searchText);

          // create the inline actions
          this._inlineActions = this._createInlineActions();

          this._refreshHandler.waitingForQueryResult();

          this._deferredForQuery =
            this._query.performContentRequest(function _loadItems_success_TasksListController(responseObject){
              com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_loadItems_success");
              self._deferredForQuery = null;
              self._flagErrorCaseRefreshLoadItems = false;
              self.itemIdentifier = responseObject.identifier;
              //self.items = responseObject.items;
              // we have fetched more items than requested;
              // if we really got more than requested, then mark the extra items
              var objectsToBeReturned = pageSize;
              var objectsReceived = responseObject.items.length;
              if (objectsReceived > objectsToBeReturned) {
                self._moreItemsAvailable = true;
              }
              else {
                self._moreItemsAvailable = false;
                objectsToBeReturned = objectsReceived; // may be pageSize or less than pageSize
              }
              var i, item;
              self.items = [];

              for (i = 0; i < objectsReceived; i++) {
                item = responseObject.items[i];
                self.items.push(item);
                if (i == objectsToBeReturned && self._moreItemsAvailable === true) {
                  // extra item, mark it
                  item.isExtra = true;
                }
                else {
                  // mark the others as well
                  item.isExtra = false;
                }

                // add inlineActions for table view 
                if (self.config.viewConfig.viewType == "table" && self._inlineActions) {
                  item.inlineActions = [];
                  for ( var j = 0; j < self._inlineActions.length; j++) {
                    // each item gets a own copy of the inlineActions
                    item.inlineActions.push(dojo.clone(self._inlineActions[j]));
                  }
                }

                // map task status to translated strings
                if (item["STATUS"]) {
                  item.STATUS = self._nlsResources["task_status_" + item.STATUS];
                }
                if (item["PI_STATUS"]) {
                  item.PI_STATUS = self._nlsResources["instance_status_" + item.PI_STATUS];
                }


              }
              if (objectsReceived < pageSize + 1 && self._numberOfCollapsedCategories === 0) {
                // update the total count
                self._totalCountCache = objectsReceived;
              }

              // if we already have fetched the total count from the server, then we do not do it again;
              // this introduces the risk that the total count becomes inaccurate over time,
              // but we do intentionally take this risk (which is why we say "... of approximately yyy").

              // However: due to an enhancement in the API, we get the totalCount for free, even when we only ask for e.g. 25 items.
              // Note though that this totalCount value is not always what we need, but only when no DUE date filtering is done.
              // Otherwise (when we filter away collapsed categories by due date expressions), the totalCount we get for free is as well reduced,
              // because the totalCount delivered by the API uses those filters as well - thus we have to check this first.
              // Note that this is not a bug by the API but rather a limitation: if the totalCount would ignore the dueDate, then
              // the totalCount attribute would no longer come for free, but rather would cause a performance penalty
              // (due to the fact that a second query would be needed internally in the backend).
              var dueDateUsed = false;
              if (self._query._getWhereClause) {
                var whereClauseObject = self._query._getWhereClause();
                if (whereClauseObject && whereClauseObject.whereClause && whereClauseObject.whereClause !== "") {
                  dueDateUsed = true;
                }
              }
              if (dueDateUsed === false && responseObject.totalCount) {
                // we can use the count data that we got for free
                var count = responseObject.totalCount;
                var countLimitExceeded = false;
                if (responseObject.countLimitExceeded && responseObject.countLimitExceeded === true) {
                  countLimitExceeded = true;
                }
                // Set the count on the paginationWidget
                if (self._paginationWidget) {
                  self._paginationWidget.setGlobalCount(count, countLimitExceeded);
                }
                // Set the count cache and the flag whether this did hit the truncation limit
                self._totalCountCache = count;
                self._totalCountLimitExceeded = countLimitExceeded;
                // TODO: exploit the countLimitExceeded attribute in the result object; if true, switch to a slightly different message.
                // However, for this first find out whether the regular count call as triggered by self._countItems() may be truncated as well.
                // This is relevant for switching back to the regular message.              
              }
              else {
                // we can NOT use the count data that we got for free, as its accuracy may be affected by the DUE date filter.
                // If possible, use the cache; if cache not set yet, use an extra count call
                if (self._totalCountCache === null) {
                  // count items
                  self._countItems(true, false, true);
                }
              }
              self._searchFilterApplied = (self._searchText !== null);
              self._refreshHandler.queryFinished(true);

              // create a new start page config object
              self._startPageSettings = self.createPageSettings();
              self.onLoadItemsSuccess(self._startPageSettings);

              com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_loadItems_success");

            }, function _loadItems_error_TasksListController(data, ioArgs){
              com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksListController", "_loadItems_error");
              com.ibm.bpm.social.util.common.Log.logObject(2, data, "request data");
              com.ibm.bpm.social.util.common.Log.logObject(2, ioArgs, "request ioArgs");
              self._deferredForQuery = null;
              if (data.dojoType == 'cancel') {
                // request has been cancelled
                com.ibm.bpm.social.util.common.Log.logError(2, "Request has been cancelled.");
                self._refreshHandler.queryFinished(false);
              }
              else {
                if (data.dojoType == 'timeout') {
                  // request timed out
                  com.ibm.bpm.social.util.common.Log.logError(2, "Request timed out.");
                  // data, ioArgs, statusCodeMap, errorCodeMap, callback
                  self._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListDataTimeout", null, false, null, null);
                }
                else {
                  // handle error
                  var statusCodeMap =
                    {
                      400: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      },
                      404: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      },
                      0: {
                        key: "KEY_LoadListData" +
                          com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type),
                        hideDetails: false
                      }
                    };
                  var errorCodeMap = {
                    "CWTBG0564E": {
                      key: "KEY_Search_Error"
                    },
                    "CWTBG0567E": {
                      key: "KEY_Search_To_Large_Error"
                    },
                    hideDetails: false
                  };
                  // data, ioArgs, statusCodeMap, errorCodeMap, callback
                  self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, null);
                }
                self.items = [];
                self._refreshHandler.queryFinished(false);
                // show empty message in the list 
                var noItemsMessage = null;
                if (self.config.queue.entityTypeName != "PROCESS_INSTANCE") {
                  noItemsMessage = "task-list-no-tasks";
                }
                else {
                  noItemsMessage = "task-list-no-instances";
                }
                self._showMessage(noItemsMessage);
              }
              com.ibm.bpm.social.util.common.Log.logExit(2, "", "TasksListController", "_loadItems_error");
            });
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },


      /**
       * Post an event that settings relevant to the page settings have changed
       * 
       * @type {void}
       */
      onLoadItemsSuccess: function onLoadItemsSuccess_TasksListController(PageSettings){
        com.ibm.bpm.social.util.common.Log.logExit(2, "Event published ");
      },

      _countItems: function _countItems_TasksListController(async, autoRefresh){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // if there was a problem detected with the current queue
        // throw the expection and don't process the data
        if (this.config.queue.exception) {
          this._flagErrorCaseRefreshCountItems = false;
          throw this.config.queue.exception;
        }
        else {
          var self = this;
          this._query.clear();
          // TODO set the type correct
          this._query.setType(this.config.queue);
          this._updateQuery(async);
          // we used the filter for excluding the collapsed categories - reset as for the count we want them all 
          var tmpFilter = null;
          this._query.setCustomFilter(tmpFilter);
          this._query.performCountRequest(function(responseObject){
            self._flagErrorCaseRefreshCountItems = false;
            self._countCallback(async, autoRefresh, responseObject.count);
          }, function(data, ioArgs){
            // TODO remove
//                // Not every user is a business category or work basket administrator:
//                if (ioArgs.xhr.status == 401 &&
//                  (self.config.type == com.ibm.bpm.social.widgets.task.util.Constants.WORK_BASKET_TYPE || self.config.type == com.ibm.bpm.social.widgets.task.util.Constants.BUSINESS_CATEGORY_TYPE)) {
//                  // Disable all actions in this case.
//                  self.toolbarWidget.setAvailableActions();
//                  // Display message in view widget.
//                  var messageKey =
//                    "KEY_LoadListDataNotAuthorized" +
//                      com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type);
//                  self._showMessage(messageKey, "WARNING");
//                }
//                else
            // TODO Do wee need this?
            if (ioArgs.xhr.status == 400 && !self._flagErrorCaseRefreshCountItems) {
              self._flagInitConfiguration = true;
              self._flagErrorCaseRefreshCountItems = true;
              self._refreshItems(true);
            }
//            else {
            // TODO do not show error dialog, because performContentRequest may also show one
            // TODO ? log error somewhere?
//              self._errorHandler.showErrorMessage(data, ioArgs, "KEY_LoadListData" +
//                com.ibm.bpm.social.widgets.task.util.Constants.getEntityPostFix(self.config.type), null);
//            }
          });
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _updateQuery: function _updateQuery_TasksListController(async, pageSize, offset, ids){

        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._query.setPageSize(pageSize);
        this._query.setOffset(offset);
        this._query.setAsync(async);
        this._query.setQueue(this.config.queue);

        // set the right filter config
        var selectedFilter = this.config.selectedFilter;
        if (selectedFilter) {
          var selectedFilterFindings = dojo.filter(this.config.filterConfigs, function(filter){
            return filter.id === selectedFilter;
          });
          if (selectedFilterFindings.length > 0) {
            this._setFilterConfig(selectedFilterFindings[0]);
//            if (this.config.viewConfig.viewType == "list") {
//              this._decorateSelectedFilter(selectedFilterFindings[0]);
//            }

          }
        }
        this._query.setUserRoleFilter(this.selectedFilterConfig);


        // TODO
//        this._query.setIdFilter({
//          "property": this.config.parentProperty,
//          "ids": ids
//        });
        // TODO ???
//        this._query.setPrioritiesConfig(this.config[com.ibm.bpm.social.widgets.task.util.Constants.PRIORITIES_CONFIG]);
//        this._query.setGroupingProperties({
//          "property": this.config.activeGroupByProperty,
//          "order": this.config.activeGroupBySortOrder
//        });
        // TODO for now
//        if (this.selectedFilterConfig.groupingProperties) {
//          this._query.setGroupingProperties(this.selectedFilterConfig.groupingProperties);
//        }

        // TODO
//        var selectedAttributes = this._getSelectedAttributes(this.config.queue, this.config.activeGroupByProperty);
//        this._query.setSelectedAttributes(selectedAttributes);
        // TODO modify ???
        if (this.config.viewConfig.selectedAttributes) {
          var selectedAttributes = this.config.viewConfig.selectedAttributes;
          this._query.setSelectedAttributes(selectedAttributes);
        }

        // TODO
//        var customFilter = this.config.activeFilter;
//        this._sortFilterCriteria(customFilter);
//        this._query.setCustomFilter(customFilter);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      getQuery: function getQuery_TasksListController(){
        return this._query;
      },

      _countCallback: function _countCallback_TasksListController(async, autoRefresh, count){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // this method might be called by an async XHR response
        if (this._destroyed) {
          com.ibm.bpm.social.util.common.Log.log(1, "List is already destroyed.");
          return;
        }

        //
        // Set the count cache and the flag whether this did hit the truncation limit
        //
        this._totalCountCache = count;
        // assumption: this count call does not truncate the count.
        this._totalCountLimitExceeded = false;

        //
        // Set the count on the paginationWidget
        //
        if (this._paginationWidget) {
          this._paginationWidget.setGlobalCount(count, this._totalCountLimitExceeded);
        }

        // TODO remove
//        //
//        // Check if navigation state information needs to be applied.
//        //
//        if (this.pendingOffset != null) {
//          this.paginationWidget.setFirstVisibleItem(this.pendingOffset);
//          this.pendingOffset = null;
//        }
//
//        var offset = this.paginationWidget.getFirstVisibleItem() - 1;
//        var pageSize = this.paginationWidget.getItemsPerPage();
//        var lastItem = this.paginationWidget.getLastVisibleItem();
//
//        // Ge: We still need recalculate the allowed actions without selection
//        //     no matter what the count and selection is
//        var allowedActions = this._getAllowedActions();
//        this.toolbarWidget.setAvailableActions(allowedActions);
//
//        if (count > 0) {
//          this._loadItems(async, autoRefresh, offset, pageSize);
//        }
//        else {
//          // no items in the list, therefore reset the item array
//          this.items = [];
//          this._showNoItemMessage();
//
//          if ((dojo.isIE && (dojo.isIE < 8))) {
//            // for some reasons showing a message extends the container to app. 1000px
//            // readjust here
//            this.viewContainer.style.height = this.viewContainerHeight + "px";
//          }
//
//          // count == 0
//          // Do not refresh, if the dom node does not exist anymore, e.g. if the widget
//          // was already destroyed and an async list refresh callback happens.
//          // + Do not refresh the list, if it is an auto-refresh and tasks are selected.
//          if (this.domNode && (!autoRefresh || (autoRefresh && this.getSelectedItems().length === 0))) {
//            // Start a new refresh cycle, if wanted
//            this._createRefreshTimeout();
//          }
//
//          this._updateFilterDescription();
//
//        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /*
       * Shows a message in the list body.
       */
      _showMessage: function _showMessage_TasksListController(messageKey, severity, parameters){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // Check, if there is a message key defined.
        if (messageKey) {
          var message = this._nlsResources[messageKey];
          if (message) {
            // if there are parameters defined for the message
            if (parameters) {
              message = dojo.string.substitute(message, parameters);
            }
            var viewMessage = document.createElement("div");
            viewMessage.style.padding = "4px";
            viewMessage.style.position = "relative";

            // Handle message
            var text = document.createElement("div");
            text.innerHTML = message;

            // Handle severity
            if (severity == "WARNING") {
              var warning = document.createElement("div");
              dojo.addClass(warning, "smallWarning");
              warning.style.position = "absolute";
              if (this.isLeftToRight()) {
                warning.style.left = "12px";
                warning.style.right = "auto";
                text.style.marginLeft = "25px";
                text.style.marginRight = "0";
              }
              else {
                warning.style.right = "12px";
                warning.style.left = "auto";
                text.style.marginRight = "25px";
                text.style.marginLeft = "0";
              }
              viewMessage.appendChild(warning);
            }

            viewMessage.appendChild(text);

            // Remove a previous message, e.g. if the query table does not exist and
            // the user refreshes the list.
            if (this._viewMessage) {
              this.listMessage.removeChild(this._viewMessage);
            }
            this._viewMessage = viewMessage;
            this.listMessage.appendChild(viewMessage);
            // switch the css classes

            dojo.removeClass(this.listMessage, "bpm-task-list-message-hide");
            dojo.addClass(this.listContent, "bpm-task-list-content-hide");
            dojo.addClass(this.pagination, "bpm-task-list-content-hide");
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(1, "Warning: TasksListController._showMessage received an empty message");
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log.log(1,
            "Warning: TasksListController._showMessage received an empty message key");
        }

        com.ibm.bpm.social.util.common.Log.log(1, "TasksListController._showMessage exit");
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _removeMessage: function _removeMessage_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._viewMessage) {
          this.listMessage.removeChild(this._viewMessage);
          // switch the css classes
          dojo.removeClass(this.listContent, "bpm-task-list-content-hide");
          dojo.removeClass(this.pagination, "bpm-task-list-content-hide");
          dojo.addClass(this.listMessage, "bpm-task-list-message-hide");
          this._viewMessage = null;
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Return the due date where clause, one pair for each expanded section
       * 
       * @param id object[] the array of categories
       */
      _computeWhereClauseForExpandedSections: function _computeWhereClauseForExpandedSections_TasksListController(categories){
        var whereClause = null;
        var partialClause = null;
        if (categories) {
          // Optimization for the frequent special case that all categories are expanded: no where clause at all
          // Thus, do the quick check whether no categories expanded. This would allow a quick return
          // and, more important, the backend would not have to look at the due date for filtering. 
          var allCategoriesAreExpanded = true;
          dojo.forEach(categories, function(category){
            if (category && category.id && (category.isExpanded() === false)) {
              allCategoriesAreExpanded = false;
            }
          });
          if (allCategoriesAreExpanded === true) {
            // quick return
            return null;
          }

          // assumption: categories are shown in ascending order
          // Loop over the categories and add the start due date and the end due date to the where clause.
          // Expanded sections only.
          // TODO  Optimization: Simplify WHERE clause by combining adjacent expanded sections
          dojo.forEach(categories, function(category){
            if (category && category.id && category.isExpanded()) {
              partialClause = null;
              // there may be invisible categories - allow those as well, there could have items shown up
              var startDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate(category.id);
              var endDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate(category.id);
              // special case first category: no start due date
              if (startDueDate === null && endDueDate !== null) {
                // only end due date, e.g for OVERDUE
                partialClause = "(DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(endDueDate) + "'))";
              }
              else if (startDueDate !== null && endDueDate === null) {
                // only start due date, e.g. for DUE LATER;
                // AT RISK has as well only a start date, but that one is handled separately, combined with IS_AT_RISK flag.
                partialClause = "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startDueDate) + "'))";
              }
              else if (startDueDate !== null && endDueDate !== null) {
                partialClause =
                  "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startDueDate) + "')" + " AND " +
                    " DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(endDueDate) + "'))";
              }
              // defensive: there may be situations where neither start nor end is set - do only append anything in that case
              if (partialClause !== null) {
                if (whereClause === null) {
                  whereClause = partialClause;
                }
                else {
                  whereClause = whereClause + " OR " + partialClause;
                }
              }
            }
          });
          // Special case for atRisk:
          // This category is not determined by the due date;
          // thus, both the expanded case and the collapsed case have to be reflected in the where clause 
          dojo.forEach(categories, function(category){
            if (category && category.id && category.id === com.ibm.bpm.social.widgets.task.list.Category.AT_RISK) {
              // Compute the atRisk condition; note that overdue tasks are always atRisk, this overdue have to be excluded
              partialClause = null;
              var startAtRiskDate =
                com.ibm.bpm.social.widgets.task.list.Category
                  .getCategoryEndDueDate(com.ibm.bpm.social.widgets.task.list.Category.OVERDUE);
              if (category && category.id && category.isExpanded()) {
                // if the atRisk category is EXPANDED, then this is the contribution to the WHERE clause:
                partialClause =
                  "(DUE>TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startAtRiskDate) + "')" + " AND " +
                    " IS_AT_RISK = true) ";
                if (partialClause !== null) {
                  if (whereClause === null) {
                    whereClause = partialClause;
                  }
                  else {
                    whereClause = whereClause + " OR " + partialClause;
                  }
                }
              }
              else {
                // if the atRisk category is COLLAPSED, then this is the contribution to the WHERE clause:
                // limitation: "NOT (...)" not accepted by condition parser; thus use De Morgan rules
                partialClause =
                  "(DUE<=TS('" + com.ibm.bpm.social.util.common.Util.getTimestampUTC(startAtRiskDate) + "')" + " OR " +
                    " IS_AT_RISK <> true) ";
                if (partialClause !== null) {
                  if (whereClause === null) {
                    // no tasks outside AT_RISK do qualify, no tasks inside AT_RISK category do qualify either -> enforce the empty list:
                    whereClause = "(IS_AT_RISK=true AND IS_AT_RISK=false)";
                  }
                  else {
                    whereClause = "(" + whereClause + ") AND " + partialClause;
                  }
                }
              }
              // there may be invisible categories - allow those as well, there could have items shown up
              var startDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryStartDueDate(category.id);
              var endDueDate = com.ibm.bpm.social.widgets.task.list.Category.getCategoryEndDueDate(category.id);
              // special case first category: no start due date
              var partialClause = null;

            }
          });
        }
        return whereClause;
      },

      /**
       * Set the Filter config and copy the settings to the current queue
       * 
       * @param {com.ibm.bpm.social.widgets.task.config.FilterConfig} filterConfig
       */
      _setFilterConfig: function _setFilterConfig_TasksListController(filterConfig){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.selectedFilterConfig = filterConfig;
        var property, i, j, k, index, filterProperty, queueProperty;
        var item = this.config.queue.getAttributes();
        if (item && item.properties) {
          // if we have a filter config, it overrules the sort order from the queue
          // ignore sortProperties received with the attributes call
          if (filterConfig.sortProperties && filterConfig.sortProperties.length > 0) {
            var queueProperties = item.properties;
            var filterProperties = filterConfig.sortProperties;
            // reset old filter settings
            for (k = 0; k < queueProperties.length; k++) {
              queueProperties[k].isSort = false;
            }

            for (i = 0; i < filterConfig.sortProperties.length; i++) {
              filterProperty = filterProperties[i];
              for (j = 0; j < queueProperties.length; j++) {
                // mix in the filter config for this property
                queueProperty = queueProperties[j];

                if (filterProperty.property == queueProperty.property) {
                  // mixin the filter sort attributes 
                  dojo.mixin(queueProperty, filterProperty);
                }
              }
            }
          }
          else if (item && item.sortProperties) {
            filterConfig.sortProperties = item.sortProperties;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      retrieveAvailableActionsForItems: function retrieveAvailableActionsForItems_TasksListController(items){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        // retrieve available actions async
        this._retrieveAvailableActionsForQueryResult(items);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Retrieve the available actions for the actual query result.
       * <p>
       * Maintains the map of dojo.Deferred objects for each task.
       * <p>
       * The list of available actions is added as property availableActions to the task item.
       * 
       * @param {Array} items a list of task items as returned by the query
       * @returns {void} nothing
       */
      _retrieveAvailableActionsForQueryResult: function _retrieveAvailableActionsForQueryResult_TasksListController(items){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var actions = null;
        var deferredTaskActions =
          this._retrieveAvailableTaskActions(items, com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS,
            true);
        if (this.config.viewConfig.viewType == "table") {
          // only necessary for saved searches not for task list
          var deferredProcessActions = this._retrieveAvailableProcessActions(items, actions, true);
          if (deferredTaskActions) {
            deferredTaskActions.then(dojo.hitch(this, "_updateTaskActions"), null);
          }
          if (deferredProcessActions) {
            deferredProcessActions.then(dojo.hitch(this, "_updateProcessActions"), null);
          }
        }
        else if (deferredTaskActions) {
          this._addDeferredForActions(deferredTaskActions, items);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback invoked after the available actions are retrieved and appended to the model (this.items)
       * (see _retrieveAvailableTaskActions). Updates the inline actions in this.items.inlineActions from
       * this.items.availableActions, then notifies the table to update the actions <br>
       * Note: only called if using table view (custom searches)
       * 
       * @returns {void} nothing
       */
      _updateTaskActions: function _updateTaskActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var item, action, inlineActions, availableActions, piid, canModifyTask, canRunTask, canViewInstanceDetails, canComplete, canClaim;

        // iterate over the model and determine available actions
        for ( var i = 0; i < this.items.length; i++) {
          item = this.items[i];
          availableActions = dojo.isArray(item.availableActions) ? item.availableActions : [];
          inlineActions = dojo.isArray(item.inlineActions) ? item.inlineActions : [];

          if (this.config.queue.entityTypeName != "PROCESS_INSTANCE") {
            // Which task actions qualify?
            //
            var ifReassignBackToGroupOK = dojo.indexOf(availableActions, "ACTION_CANCELCLAIM") != -1;
            var ifTransferToUserOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOUSER") != -1;
            var ifTransferToGroupOK = dojo.indexOf(availableActions, "ACTION_REASSIGNTOGROUP") != -1;
            var ifModifyPriorityOK = dojo.indexOf(availableActions, "ACTION_UPDATEPRIORITY") != -1;
            var ifModifyDueDateOK = dojo.indexOf(availableActions, "ACTION_UPDATEDUEDATE") != -1;

            piid = null;
            if (typeof item["PROCESS_INSTANCE.PIID"] == "string") {
              piid = item["PROCESS_INSTANCE.PIID"];
              item.piid = piid; // shortcut for our _viewInstance method 
            }
            else if (typeof item.piid == "string") {
              piid = item.piid;
            }

            canModifyTask =
              (ifReassignBackToGroupOK || ifTransferToUserOK || ifTransferToGroupOK || ifModifyPriorityOK || ifModifyDueDateOK);

            canViewInstanceDetails = (piid !== null);
          }
          else {
            canModifyTask = false; // modifyTask is not shown in processes list
            canViewInstanceDetails = true; // if we got the process instance, we are allowed to see the details
          }

          canClaim = dojo.indexOf(availableActions, "ACTION_CLAIM") != -1;
          canComplete = dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1;
          canRunTask = (canComplete || canClaim);

          for ( var j = 0; j < inlineActions.length; j++) {
            action = inlineActions[j];
            if (action.id == "modifyTask") {
              action.enabled = canModifyTask;
            }
            else if (action.id == "runTask") {
              action.enabled = canRunTask;
            }
            else if (action.id == "viewInstance") {
              action.enabled = canViewInstanceDetails;
            }
          }
        }

        // notify the table that the actions may have changed
        this.listWidget.updateActions();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Callback invoked after the available actions are retrieved and appended to the model (this.items)
       * (see _retrieveAvailableProcessActions). Updates the inline actions in this.items.inlineActions from
       * this.items.availableActions, then notifies the table to update the actions <br>
       * Note: only called if using table view (custom searches)
       * 
       * @returns {void} nothing
       */
      _updateProcessActions: function _updateProcessActions_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var item, action, inlineActions, availableActions, piid, canViewInstanceDetails, canModifyInstance;

        // iterate over the model and determine available actions
        for ( var i = 0; i < this.items.length; i++) {
          item = this.items[i];
          availableActions = dojo.isArray(item.availableProcessActions) ? item.availableProcessActions : [];
          inlineActions = dojo.isArray(item.inlineActions) ? item.inlineActions : [];

          canViewInstanceDetails = true; // if we got the process instance, we are allowed to see the details
          canModifyInstance = dojo.indexOf(availableActions, "ACTION_CHANGE_INSTANCE_DUEDATE") != -1;

          for ( var j = 0; j < inlineActions.length; j++) {
            action = inlineActions[j];
            if (action.id == "modifyInstance") {
              action.enabled = canModifyInstance;
            }
            else if (action.id == "viewInstance") {
              action.enabled = canViewInstanceDetails;
            }
          }
        }

        // notify the table that the actions may have changed
        this.listWidget.updateActions();

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Retrieve the available actions for a list of processes.
       * <p>
       * The list of available actions is added as property availableActions to the processes item.
       * 
       * @param {Array} items a list of process items as returned by the query
       * @param {Array} actions a list of actions to retrieve (currently not supported)
       * @param {Boolean} async an asynchronous request
       * @returns {dojo.Deferred} an object which allows to add callback handlers for dojo.xhr request
       */
      _retrieveAvailableProcessActions: function _retrieveAvailableProcessActions_TasksListController(items, actions, async){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var deferred = null;

        if (items && items.length > 0) {
          var domain = items && items.length > 0 && items[0].domain ? items[0].domain : null;
          var self = this;
          deferred =
            this._processRestHandler.getActions(items, "PROCESS_INSTANCE.PIID", actions,
              function _retrieveAvailableProcessActions_success__TasksListController(responseObject, ioArgs){

                var result = responseObject;
                // dispatch getActions result to the individual item
                if (result && result.process) {
                  dojo.forEach(result.process, function(processResult){
                    var filteredArr = dojo.filter(items, function(item){
                      return item["PROCESS_INSTANCE.PIID"] === processResult.piid;
                    });
                    if (filteredArr.length === 1) {
                      filteredArr[0].availableProcessActions = processResult.actions || [];
                      com.ibm.bpm.social.util.common.Log.log(2, "Actions for " + processResult.piid + ": " +
                        (processResult.actions !== null ? processResult.actions.toString() : ""), "TasksListController",
                        "_retrieveAvailableProcessActions");
                    }
                  });
                }
              }, function _retrieveAvailableProcessActions_error__TasksListController(data, ioArgs){
                // TODO ??? notification?
                self._errorHandler.showGenericError(data, ioArgs);
              }, async ? async : false, domain);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return deferred;
      },

      /**
       * Retrieve the available actions for a list of tasks.
       * <p>
       * The list of available actions is added as property availableActions to the task item.
       * 
       * @param {Array} items a list of task items as returned by the query
       * @param {Array} actions a list of actions to retrieve
       * @param {Boolean} async an asynchronous request
       * @returns {dojo.Deferred} an object which allows to add callback handlers for dojo.xhr request
       */
      _retrieveAvailableTaskActions: function _retrieveAvailableTaskActions_TasksListController(items, actions, async){
        com.ibm.bpm.social.util.common.Log.logEntry(3);

        var deferred = null;

        if (items && items.length > 0) {
          var domain = items && items.length > 0 && items[0].domain ? items[0].domain : null;
          var self = this;
          deferred =
            this._restHandler.getActions(items, "TASK.TKIID", actions, function(responseObject, ioArgs){
              var result = responseObject;
              // dispatch getActions result to the individual item
              if (result && result.tasks) {
                dojo.forEach(result.tasks, function(taskResult){
                  var filteredArr = dojo.filter(items, function(item){
                    return item["TASK.TKIID"] === taskResult.tkiid;
                  });
                  if (filteredArr.length === 1) {
                    filteredArr[0].availableActions = taskResult.actions || [];
                    com.ibm.bpm.social.util.common.Log.log(3, "Actions for " + taskResult.tkiid + ": " +
                      (taskResult.actions !== null ? taskResult.actions.toString() : ""), "TasksListController",
                      "_retrieveAvailableTaskActions");
                  }
                });
              }
            }, function(data, ioArgs){
              // TODO ??? notification?
              self._errorHandler.showGenericError(data, ioArgs);
            }, async ? async : false, domain);
        }
        com.ibm.bpm.social.util.common.Log.logExit(3);
        return deferred;
      },

      /**
       * Handles a map of task ids and the dojo.Deferred object corresponding to the available actions call in
       * order to register a callback for processing when the actions are available.
       * 
       * @param {dojo.Deferred} deferred an object which allows to add callback handlers for dojo.xhr request
       * @param {Array} items a list of task items as returned by the query
       * @returns {void} nothing
       */
      _addDeferredForActions: function _addDeferredForActions_TasksListController(deferred, items){
        var self = this;
        dojo.forEach(items, function(item){
          var entry = self._getDeferredForActions(item);
          if (entry !== null) {
            // replace entry
            entry.deferred = deferred;
          }
          else {
            // add new entry
            if (self._deferredForActions === null) {
              self._deferredForActions = new Array();
            }
            self._deferredForActions.push({
              id: self._restHandler.prefixID(item["TASK.TKIID"], item.domain, item.systemID),
              deferred: deferred
            });
          }
        });
      },

      /**
       * @param {Object} item a task item as returned by the Query
       * @returns {dojo.Deferred} an object which allows to add callback handlers for the dojo.xhr request
       *          which retrieves the actions for the task
       */
      _getDeferredForActions: function _getDeferredForActions_TasksListController(item){
        var entry = null;
        if (this._deferredForActions !== null) {
          var id = this._restHandler.prefixID(item["TASK.TKIID"], item.domain, item.systemID);
          var filteredArr = dojo.filter(this._deferredForActions, function(entry){
            return entry.id == id;
          });
          if (filteredArr.length > 0) {
            entry = filteredArr[0];
          }
        }
        return entry;
      },

      /**
       * @param item a task item
       * @param {Function(data)} callback a callback function to be executed when the retrieval of available
       *        actions for the item is finished successfully
       * @returns {Boolean} true if a dojo.Deferred object is found for the task
       */
      setActionsCallbackSuccess: function setActionsCallbackSuccess_TasksListController(item, callback){
        var result = false;
        var entry = this._getDeferredForActions(item);
        if (entry !== null) {
          entry.deferred.addCallback(callback);
          result = true;
        }
        else {
          // programming error
          throw new Error(item.TKIID + " - No deferred found to add callback!");
        }
        return result;
      },

      /**
       * @param item a task item containing TKIID and optional domain, systemID
       * @param {Function(data)} callback a callback function to be executed when the retrieval of available
       *        actions for the item is finished and failed
       * @returns {Boolean} true if a dojo.Deferred object is found for the task
       */
      setActionsCallbackError: function setActionsCallbackError_TasksListController(item, callback){
        var result = false;
        var entry = this._getDeferredForActions(item);
        if (entry !== null) {
          entry.deferred.addErrback(callback);
          result = true;
        }
        return result;
      },

      _getBulkTaskDetails: function _getBulkTaskDetails_TasksListController(tkiidList){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var myself = this;

        // Fetch data from the server
        myself._restHandler.getTasksInBulkMode(tkiidList, function(bulkResult){
          // Note: In bulk mode, a successful status (HTTP 200) can still mean errors.
          // Thus, error handling will be done in _setTaskDetailsBulk as well.
          myself._setTaskDetailsBulk(bulkResult);
        }, function(data, ioArgs){
          for ( var k = 0; k < tkiidList.length; k++) {
            var taskId = tkiidList[k];
            // task is assigned to somebody else or deleted
            if (ioArgs.xhr.status == 401 || ioArgs.xhr.status == 404) {
              myself.listWidget.showMessage(myself._nlsResources["task-row-details-error-not-available"], taskId);
            }
            else {
              myself.listWidget.showMessage(data.message, taskId);
            }
          }
        }, false, null);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _workOn: function _workOn_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // save focus here because claim dialog may get focus before TCUI is shown
        this._saveListFocus();

        var workOn = true;
        if (item.availableActions) {
          // try to claim Task if it can be claimed
          var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
          var confirm =
            (userProfile && userProfile.userPreferences && userProfile.userPreferences['Alert On Assign And Run'] === "true");

          if (confirm) {
            // ensure up-to-date actions for this task
            this
              ._retrieveAvailableTaskActions([item], com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS);
          }

          if (dojo.indexOf(item.availableActions, "ACTION_CLAIM") != -1) {
            // we can claim, check if we need a dialog
            if (confirm) {
              // show confirmation dialog, if we can claim the task
              com.ibm.bpm.social.util.common.Log.log(2, "show confirm dialog for claim");
              if (!this._confirmClaimDialog) {
                this._createClaimDialog();
              }
              this._confirmClaimDialog.showConfirmationDialog(item);
              workOn = false;
            }
            else {
              workOn = this._claim(item);
            }
          }
        }
        else {
          // item may be a process instance
          if (item.TKIID || item["TASK.TKIID"]) {
            item["TASK.TKIID"] = item.TKIID || item["TASK.TKIID"];
            workOn = true;
            this._waitingForClaimDialogChannel =
              dojo.subscribe(com.ibm.bpm.social.portal.Topics.COACH_CANCELED, dojo.hitch(this, this._claimDialogCancelled));
          }
        }

        if (workOn) {
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
            "taskId": item["TASK.TKIID"]
          }]);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _claim: function _claim_TasksListController(item){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var claimed = false;
        var self = this;
        this._restHandler.claimTask(item["TASK.TKIID"], function(responseObject, ioArgs){
          claimed = true;

        }, function(data, ioArgs){
          if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status == 409 && ioArgs.xhr.responseText) {
            var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
            if (response.errorNumber == "CWTBG0014E") {
              // task has been claimed since last list refresh
              // check if user is allowed to complete
              // refresh available actions for this item synchronously
              self._retrieveAvailableTaskActions([item],
                com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS);
              if (item.availableActions) {
                if (dojo.indexOf(item.availableActions, "ACTION_COMPLETE") != -1) {
                  // continue normal processing
                  claimed = true;
                }
              }
            }
          }

          if (!claimed) {
            // handle errors
            var name = item.TAD_DISPLAY_NAME;
            var statusCodeMap = {
              409: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              401: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              403: {
                key: "KEY_ClaimTask",
                parameters: {
                  1: name
                }
              },
              404: {
                key: "KEY_ClaimNotExistingTask",
                parameters: {
                  1: name
                }
              }
            };
            var errorCodeMap = {
              "CWTKA0072E": {
                key: "KEY_ClaimSuspendedTask",
                parameters: {
                  1: name
                }
              }
            };

            self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
            // trigger list refresh
            self._refresh(true);
          }
        }, item.system ? item.system : null);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return claimed;
      },

      /**
       * 
       * @type {void}
       */
      _createClaimDialog: function _createClaimDialog_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._confirmClaimDialog = new com.ibm.bpm.social.widgets.task.util.ClaimDialog({
          "restHandler": this._restHandler
        });

        this._childWidgets.push(this._confirmClaimDialog);

        this.connect(this._confirmClaimDialog, "onClaimed", "_claimDialogSuccessful");
        this.connect(this._confirmClaimDialog, "onFailed", "_claimDialogFailed");
        this.connect(this._confirmClaimDialog, "onCancelled", "_claimDialogCancelled");

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _claimDialogSuccessful: function _claimDialogSuccessful_TasksListController(taskId){
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
          "taskId": taskId
        }]);
      },

      _claimDialogFailed: function _claimDialogFailed_TasksListController(taskId){
        this._refresh(true);
      },

      _claimDialogCancelled: function _claimDialogCancelled_TasksListController(taskId){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (this._waitingForClaimDialogChannel) {
          // currently the TaskCompletionWidget might be shown, show myself again.
          dojo.unsubscribe(this._waitingForClaimDialogChannel);
          this._waitingForClaimDialogChannel = null;
          // triggers refresh
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },


      /**
       * set the search text for the next refresh
       * 
       * @type {void}
       */
      setSearchText: function setSearchText_TasksListController(text){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (text === "") {
          // clear on empty string 
          text = null;
        }
        this._searchText = text;
        if (this._searchWidget) {
          this._searchWidget.setSearchText(text);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _search: function _search_TasksListController(text){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._searchText = text;

        this._showMessage("task-list-loading");
        var size = this._paginationWidget.getPageSize();
        // We maintain now collapse/expand state of categories, so this should be respected here also, 
        // even if it is ugly that a category disappears on expand.
//        if (this.selectedFilterConfig && this.selectedFilterConfig.categories) {
//          // expand all categories prior to search. This avoids the ugly case:
//          //  - user collapses "Overdue" category
//          //  - user searches for "blabla", nothing found, "Overdue" category stays visible but collapsed 
//          //  - user expands "Overdue"
//          // ==> "Overdue" category disappears ... which would be confusing
//          var myself = this;
//          dojo.forEach(this.listWidget._categoryWidgets, function(category){
//            if (category && category.isExpanded && (category.isExpanded() === false)) {
//              category.toggle(true); // set silent flag to true, to avoid reload on each category expansion
//              myself._numberOfCollapsedCategories--; // used to decide what message we show 
//            }
//          });
//        }
        this._refreshItems(true, false, size);
        //this.refresh();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _clearSearch: function _clearSearch_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._searchText = null;
        this.refresh();
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _setSearch: function _setSearch_TasksListController(){
        // TODO : check if system has text search index , check if organized by tasks 
        if (this._searchEnabled) {
          this._searchWidget.show();
        }
        else {
          this._searchWidget.hide();
        }
      },

      _checkSearchCapability: function _checkSearchCapability_TasksListController(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var systems = this._restHandler.getSystems();
        this._searchEnabled = false;
        if (this._query._type === com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE) {
          this._searchEnabled = true;
        }
        // as we do only have one system right now, it is the first one
        if (this._searchEnabled && systems && systems._systemIDs) {
          var systemId = systems._systemIDs[0];
          if (systemId) {
            this._searchEnabled = systems.isSystemSearchable(systemId);
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(1, "searchEnabled : this._searchEnabled");
      },

      _doFilter: function(filter){
        this.config.selectedFilter = filter.id;

        this._setFilterConfig(filter);

        this.listWidget.setConfig({
          controller: this,
          filter: filter,
          type: this.config.type
        });

        // remove previous filter settings
        this._query.setCustomFilter(null);

        // clear total count cache
        this._totalCountCache = null;

        this.refresh();

        this._decorateSelectedFilter(filter);
      },

      _decorateSelectedFilter: function _decorateSelectedFilter_TasksListController(filter){
        var self = this;
        dojo.forEach(this.config.filterConfigs, function(currentFilter){
          var currentFilterSpan = self[currentFilter.id + "_filter"];
          dojo.removeClass(currentFilterSpan, "bpm-task-list-filter-active");
          dojo.addClass(currentFilterSpan, "bpm-task-list-filter-inactive");
          dijit.setWaiState(currentFilterSpan, "selected", "false");
        });
        var filterSpan = this[filter.id + "_filter"];
        dojo.removeClass(filterSpan, "bpm-task-list-filter-inactive");
        dojo.addClass(filterSpan, "bpm-task-list-filter-active");
        dijit.setWaiState(filterSpan, "selected", "true");
      },

      _setTaskDetailsBulk: function _setTaskDetailsBulk_TasksListController(bulkResult){
        if (bulkResult) {
          var k;
          // handle those tasks for which we received a successful response  
          if (bulkResult.tasks) {
            for (k = 0; k < bulkResult.tasks.length; k++) {
              this._setTaskDetails(bulkResult.tasks[k]);
            }
          }
          // as well, handle those tasks for which we received an errorneous response  
          if (bulkResult.failedOperations) {
            for (k = 0; k < bulkResult.failedOperations.length; k++) {
              var currentError = bulkResult.failedOperations[k];
              var currentErrMsg = currentError.errorMessage;
              // Special case: For tasks that were deleted in the meantime,
              // show the easy to understand error message (same as non-bulk mode)
              if (currentErrMsg && currentErrMsg.substr(0, 11) == "CWTBG0048E:") {
                currentErrMsg = this._nlsResources["task-row-details-error-not-available"];
              }
              this.listWidget.showMessage(currentErrMsg, currentError.tkiid);
            }
          }
          this._onResize();
        }
      },

      _setTaskDetails: function _setTaskDetails_TasksListController(task){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var taskId = task.tkiid;

        // Process business data
        var businessDataForTR = [];
        if (task.processData && task.processData.businessData) {
          for ( var k in task.processData.businessData) {
            var label = null;
            if (task.processData.businessData[k].label) {
              label = task.processData.businessData[k].label;
            }
            else {
              label = task.processData.businessData[k].name;
            }
            var newBusinessDataEntryForTR = {
              label: label,
              value: task.processData.businessData[k].value,
              type: task.processData.businessData[k].type,
              name: task.processData.businessData[k].name,
              alias: task.processData.businessData[k].alias
            };
            businessDataForTR.push(newBusinessDataEntryForTR);
          }
        }

        var details = {
          narrative: task.description,
          businessData: businessDataForTR
        };
        //
        // Add variables to details if available in order to show choices for SimpleChoice Human Service.
        //
        if (task.data && task.data.variables) {
          details.variables = task.data.variables;
        }

        //
        // Check for know Human Service that require special handling.
        // These services are part of the system toolkit.
        var serviceID = task.serviceID;
        var isFinished = task.state == "STATE_FINISHED";

        if (serviceID == "1.71d16b41-1693-4f57-9227-a149d8bf3c1b") {
          // Simple Approval Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleApproval(taskId, details);
          }
        }
        else if (serviceID == "1.46a5689a-0864-4a33-a4c6-040ab815d75a") {
          // Simple Completion Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleCompletion(taskId, details);
          }
        }
        else if (serviceID == "1.63e23d5c-9a1e-4104-b4a7-cef70df65289") {
          // Simple Choice Human Service from system toolkit                
          if (isFinished) {
            this.listWidget.setDetails(taskId, details);
            if (this.config.selectedFilter == com.ibm.bpm.social.widgets.task.config.FilterConfig.ID_OPEN_TASKS) {
              this._showInlineTaskFinishedWarning(task.displayName);
            }
          }
          else {
            this.listWidget.setDetailsSimpleChoice(taskId, details);
          }
        }
        else {
          // Default case => no special handling, just show the details
          this.listWidget.setDetails(taskId, details);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Show a warning if we are opening a inline task in the open tasks view that is already finished in the
       * backend
       * 
       * @type {void}
       */
      _showInlineTaskFinishedWarning: function _showInlineTaskFinishedWarning_TasksListController(displayName){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._errorHandler.showWarningDialog("KEY_TaskInlineAlreadyCompleted", {
          1: displayName
        }, function(){
          this._refresh(true);
        });

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      _showExpandAllButton: function _showExpandAllButton_TasksListController(){
        this._isCEButtonInCollapsedState = true;
        this.collapseExpandAllButton.title = this._nlsResources["task-list-expand-all"];
        if (dojo.hasClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover")) {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-show-all-hover");
        }
        else {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-hide-all");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-show-all");
        }
        this.collapseExpandAllButtonText.innerHTML = "&#9660;";
      },

      _showCollapseAllButton: function _showCollapseAllButton_TasksListController(){
        this._isCEButtonInCollapsedState = false;
        this.collapseExpandAllButton.title = this._nlsResources["task-list-collapse-all"];
        if (dojo.hasClass(this.collapseExpandAllButton, "bpm-social-show-all-hover")) {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-show-all-hover");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-hide-all-hover");
        }
        else {
          dojo.removeClass(this.collapseExpandAllButton, "bpm-social-show-all");
          dojo.addClass(this.collapseExpandAllButton, "bpm-social-hide-all");
        }
        this.collapseExpandAllButtonText.innerHTML = "&#9650;";
      },

      _onResize: function _onResize_TasksListController(){
        var contentHeight = this._getListContentHeight();
        dojo.style(this.listContent, "height", contentHeight + "px");

        if (this.listWidget) {
          if (this.config && this.config.viewConfig.viewType == "table") {
            // table must always be limited
            this.listWidget.resize({
              h: contentHeight,
              w: this._getListContentWidth()
            });
            this.listWidget.renderTable(false);
          }
          else {
            var padding = "";
            com.ibm.bpm.social.util.common.Log.log(3, "clientHeight: " + this.listContent.clientHeight + ", scrollHeight: " +
              this.listContent.scrollHeight);
            if (this.listContent.clientHeight < this.listContent.scrollHeight) {
              // we have a scrollbar, add padding between list and scrollbar
              padding = "10px";
            }
            if (document.body.dir == 'rtl') {
              dojo.style(this.listWidget.domNode, "paddingLeft", padding);
            }
            else {
              dojo.style(this.listWidget.domNode, "paddingRight", padding);
            }
          }
        }
      },

      /**
       * listen for view changes to suspend/resume automatic refresh
       */
      _subscribeViewChanges: function _subscribeViewChanges_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);

        if (this._subscribedTaskCompletion === null) {
          // disable automatic refresh if the task completion view is shown, list is always refreshed on return
          this._subscribedTaskCompletion =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, dojo.hitch(this, this._disableRefresh));
        }

        if (this._subscribedInstanceView === null) {
          // disable automatic refresh if the instance view is shown, list is always refreshed on return
          this._subscribedInstanceView =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, dojo.hitch(this, this._disableRefresh));
        }

        if (this._subscribedShowDialog === null) {
          // suspend automatic full refresh if a dialog is shown, list may not be refreshed on return
          this._subscribedShowDialog =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, dojo.hitch(this, this._onShowDialog));
        }

        if (this._subscribedHideDialog === null) {
          // resume automatic full refresh if a dialog is hidden
          this._subscribedHideDialog =
            dojo.subscribe(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, dojo.hitch(this, this._onHideDialog));
        }

      },

      _unSubscribeViewChanges: function _unSubscribeViewChanges_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);

        if (this._subscribedTaskCompletion !== null) {
          dojo.unsubscribe(this._subscribedTaskCompletion);
          this._subscribedTaskCompletion = null;
        }

        if (this._subscribedInstanceView !== null) {
          dojo.unsubscribe(this._subscribedInstanceView);
          this._subscribedInstanceView = null;
        }

        if (this._subscribedShowDialog !== null) {
          dojo.unsubscribe(this._subscribedShowDialog);
          this._subscribedShowDialog = null;
        }

        if (this._subscribedHideDialog !== null) {
          dojo.unsubscribe(this._subscribedHideDialog);
          this._subscribedHideDialog = null;
        }
      },

      onHide: function onHide_TasksListController(){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        this._refreshHandler.disable();
        this._unSubscribeViewChanges();
      },


      _disableRefresh: function _disableRefresh_TasksListController(data){
        var details = "";
        if (data && data.taskId) {
          details = " taskId=" + data.taskId;
        }
        else if (data && data.instanceId) {
          details = " instanceId=" + data.instanceId;
          this._saveListFocus();
        }
        com.ibm.bpm.social.util.common.Log.log(3, this.id + details);

        this._refreshHandler.disable();
        this._unSubscribeViewChanges();
      },

      _onShowDialog: function _onShowDialog_TasksListController(dialogId){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        this._saveListFocus();
        this._refreshHandler.suspendFullRefresh(dialogId);
      },

      _onHideDialog: function _onHideDialog_TasksListController(dialogId){
        com.ibm.bpm.social.util.common.Log.log(3, this.id);
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          this.listWidget.setKeepSavedFocus(false);
        }
        this._refreshHandler.resumeFullRefresh(dialogId);
      },

      _saveListFocus: function _saveListFocus_TasksListController(){
        if (this.config && this.config.viewConfig && this.config.viewConfig.viewType == "list") {
          this.listWidget.saveFocus();
          this.listWidget.setKeepSavedFocus(true);
        }
      },

      /**
       * Return settings that describe the current content
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings} object with query information
       */
      getPageSettings: function getPageSettings_TasksListController(){
        return this._startPageSettings;
      },

      /**
       * collect the controller settings for the start page.
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings} object with query information
       */
      createPageSettings: function createPageSettings_TasksListController(){
        var result = new com.ibm.bpm.social.util.common.PageSettings({
          savedSearchName: this._query._queue.name,
          searchFilterString: this._query._searchFilter,
          sortAttributes: this.selectedFilterConfig.sortProperties,
          filterConfigId: this.selectedFilterConfig.id
        });
        return result;
      },

      onFocusNeeded: function onFocusNeeded_TasksListController(){}

    });

com.ibm.bpm.social.widgets.task.list.TasksListController.TASK_ACTIONS = ["ACTION_CLAIM",
                                                                         "ACTION_COMPLETE",
                                                                         "ACTION_CANCELCLAIM",
                                                                         "ACTION_REASSIGNTOUSER",
                                                                         "ACTION_REASSIGNTOGROUP",
                                                                         "ACTION_UPDATEPRIORITY",
                                                                         "ACTION_UPDATEDUEDATE",
                                                                         "ACTION_ADDCOMMENT"];