/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.widgets.task.util.ClaimDialog");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.util.ClaimDialog", com.ibm.bspace.common.util.ui.Dialog, {

  templateString: '<div dojoAttachPoint="container"></div>',

  _tkiid: null,

  preamble: function(params){
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this.title = this._nlsResources["task_dialog_confirm_claim_title"];
    params.title = this.title;
    return [params];
  },

  /**
   * Prerequisites:
   * <ol>
   * <li> restHandler a rest handler to claim the task </li>
   * </ol>
   * 
   * @param params {void} nothing
   */
  constructor: function(params){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "", "ClaimDialog");
    this._restHandler = params.restHandler;
    //
    // Instanciate an error handler.
    //
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Assemble the dialog. {void} nothing
   */
  postCreate: function postCreate_ClaimDialog(){
    this.inherited("postCreate", arguments);
    this.container.className = "bpm-task-dialog-container";
    this.container.title = ""; // Unset the title, else we see the tooltip from the dialog on all controlls that don't have a title

    this.description = document.createElement("div");
    this.description.innerHTML = this._nlsResources["task_dialog_confirm_claim_description"];
    this.description.className = "bpm-task-dialog-description";
    this.container.appendChild(this.description);

    var t = document.createElement("table");
    var tb = document.createElement("tbody");
    var tr = document.createElement("tr");
    var td1 = document.createElement("td");
    var td2 = document.createElement("td");

    t.appendChild(tb);
    tb.appendChild(tr);
    tr.appendChild(td1);
    tr.appendChild(td2);

    this.container.appendChild(t);
    this.dontShowAgainCheckbox = document.createElement("input");
    this.dontShowAgainCheckbox.type = "checkbox";
    td1.appendChild(this.dontShowAgainCheckbox);

    this.dontShowAgainMessage = document.createElement("span");
    this.dontShowAgainMessage.innerHTML = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];
    this.dontShowAgainMessage.title = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];

    td2.appendChild(this.dontShowAgainMessage);

    this.claimButtonDiv = document.createElement("div");
    this.claimButtonDiv.className = "bpm-task-dialog-claimButtonDiv";
    this.container.appendChild(this.claimButtonDiv);

    this.claimButton = document.createElement("button");
    this.claimButton.innerHTML = this._nlsResources["task_dialog_confirm_claim_button"];
    this.claimButton.title = this._nlsResources["task_dialog_confirm_claim_button_tooltip"];
    this.claimButtonDiv.appendChild(this.claimButton);


    this.connect(this.claimButton, "onclick", "_onClaimConfirmed");

    com.ibm.bspace.common.util.ui.CheckBox.wrapElement(this.dontShowAgainCheckbox);
    // ASZ: Title is not shown if set before wrap
    this.dontShowAgainCheckbox.nextSibling.title = this._nlsResources["task_dialog_confirm_claim_dont_show_again"];
    com.ibm.bspace.common.util.ui.Button.wrapElement(this.claimButton);

    this.set("content", this.container);
    this.connect(this.popup, "onCancel", "_dialogCancel");
    this.connect(this.popup, "onHide", "_onHide");

  },

  _dialogCancel: function _dialogCancel_ClaimDialog(){
    if (this.taskId) {
      this.onCancelled(this.taskId);
    }
    else {
      this.onCancelled();
    }
  },

  _onHide: function _onHide_ClaimDialog(){
    dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, [this.id]);
  },

  /**
   * Requires a task object either one with a TKIID and TAD_DISPLAY_NAME property or one with a tkiid and
   * displayName property.
   * 
   * @param task The task object.
   * @param systemId The system id.
   * @returns {showConfirmationDialog} {void} nothing
   */
  showConfirmationDialog: function showConfirmationDialog_ClaimDialog(task /* object */, systemId){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, [this.id]);

    this.taskId = task.tkiid || task["TASK.TKIID"];
    this.displayName = task.displayName || task["TAD_DISPLAY_NAME"] || task["NAME"];

    this.systemId = systemId;
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog About to show dialog for " + this.displayName + " (" +
      this.taskId + ")");
    // Reset the combo box to unchecked before displaying the dialog.
    com.ibm.bspace.common.util.ui.CheckBox.setChecked(this.dontShowAgainCheckbox, false);
    this.show();
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _onClaimConfirmed: function _onClaimConfirmed_TasksListController(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    //
    // Hide the dialog
    //
    this.hide();
    //
    // Set global user preferences if necessary
    //
    if (this.dontShowAgainCheckbox.checked === true) {
      com.ibm.bpm.socialportal.GlobalAssets.setUserPreference('Alert On Assign And Run', "false");
    }
    this._claim();
    com.ibm.bpm.social.util.common.Log.logExit(1);
    return;
  },

  _claim: function _claim_TasksListController(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var claimed = false;
    var self = this;
    this._restHandler.claimTask(this.taskId, function(responseObject, ioArgs){
      claimed = true;
    }, function(data, ioArgs){
      if (ioArgs && ioArgs.xhr && ioArgs.xhr.status && ioArgs.xhr.status === 409 && ioArgs.xhr.responseText) {
        var response = self._errorHandler.getResponseObject(ioArgs.xhr.responseText);
        if (response.errorNumber === "CWTBG0014E") {
          try {
            // Check whether the task has been claimed since last list refresh
            // Check if user is allowed to complete
            // Refresh available actions for this item synchronously          
            var availableActions = [];
            self._restHandler.getActionsForTask(self.taskId, ["ACTION_COMPLETE"], function(responseObject){
              if (responseObject.tasks.length === 1) {
                availableActions = responseObject.tasks[0].actions || [];
              }
            }, null /* no error handling */, self.systemId);
            if (dojo.indexOf(availableActions, "ACTION_COMPLETE") != -1) {
              // continue normal processing
              claimed = true;
            }
          }
          catch (e) {
            // This is just logic for checking whether an error condition can be ignored.
            // Therefore ignore any errors either.
          }
        }
      }
      if (!claimed) {
        // handle errors
        var name = self.displayName;
        var statusCodeMap = {
          409: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          401: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          403: {
            key: "KEY_ClaimTask",
            parameters: {
              1: name
            }
          },
          404: {
            key: "KEY_ClaimNotExistingTask",
            parameters: {
              1: name
            }
          }
        };
        var errorCodeMap = {
          "CWTKA0072E": {
            key: "KEY_ClaimSuspendedTask",
            parameters: {
              1: name
            }
          }
        };

        self._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
      }
    }, this.systemId ? this.systemId : null);
    com.ibm.bpm.social.util.common.Log.logExit(2, claimed);
    if (claimed) {
      this.onClaimed(this.taskId);
    }
    else {
      this.onFailed(this.taskId);
    }
  },

  onClaimed: function onClaimed_ClaimDialog(taskId /* string */){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onClaimed " + this.taskId);
    dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_CLAIMED + "." + this.taskId, [{}]);
  },


  onCancelled: function onCancelled_ClaimDialog(taskId){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onCancelled " + this.taskId);
  },

  onFailed: function onFailed_ClaimDialog(taskId /* string */){
    com.ibm.bpm.social.util.common.Log.log(2, "ClaimDialog.onFailed " + this.taskId);
  },

  destroy: function(){
    com.ibm.bspace.common.util.ui.CheckBox.unwrapElement(this._confirmClaimDialog.dontShowAgainCheckbox);
    com.ibm.bspace.common.util.ui.Button.unwrapElement(this._confirmClaimDialog.claimButton);
    this.inherited("destroy", arguments);
  }

});
