// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

//**************************************
//
// Version: 1.9 11/04/13 13:31:35
//              
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.DataCache");

dojo.require("com.ibm.bpm.social.util.common.Log");

dojo.declare("com.ibm.bpm.social.widgets.task.util.DataCache", null, {
  _data: {},
  _expirationDate: {},
  _usageCount: {},

  constructor: function(){},

  startCleanupProcess: function(period){
    var myself = this;
    window.setInterval(function(){
      myself._removeOutdatedEntries();
    }, period);
  },

  /**
   * Stores the data and the current date under the key.
   * 
   * @param {Object} key
   * @param {Object} data
   */
  set: function set_DataCache(key /* string */, data /* object */, expirationPeriod /* number (milliseconds) */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    this._data[key] = data;
    this._usageCount[key] = 0;
    if (expirationPeriod > 0) {
      this._expirationDate[key] = (new Date()).getTime() + expirationPeriod;
    }
    else {
      // default expiration to keep cache from increasing indefinitely
      // 1 hour = 3600000 milli seconds
      this._expirationDate[key] = (new Date()).getTime() + 3600000;
    }
    com.ibm.bpm.social.util.common.Log.logExit(3, "data:" + data);
  },

  /**
   * If there has been data cached for the url and it has not been expired, serve it.
   * 
   * @param {Object} key
   */
  get: function get_DataCache(key /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    var data = this._data[key];
    if (data) {
      var expirationDate = this._expirationDate[key];
      var now = (new Date()).getTime();
      if (expirationDate > now) {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> data available");
        this._usageCount[key]++;
        return data;
      }
      else {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> d=> data outdated");
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.logExit(3, "=> data not available");
    }
    return null;
  },

  _removeOutdatedEntries: function _removeOutdatedEntries_DataCache(){
    com.ibm.bpm.social.util.common.Log.logEntry(3);
    var now = (new Date()).getTime();
    for ( var key in this._expirationDate) {
      // check also for usage ?
      if (this._expirationDate[key] < now) {
        com.ibm.bpm.social.util.common.Log.log(2, "key:" + key);
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(3);
  }

});

com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE = new com.ibm.bpm.social.widgets.task.util.DataCache();
com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.startCleanupProcess(60000);