// DateTime.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.util.DateTime");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");
dojo.require("dojo.i18n");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.util._DateTimeDateTextBox", dijit.form.DateTextBox, {
  _onDropDownMouseDown: function(/*Event*/e){
    // on iPad when clicking
    // - the text box do not show the drop down but the keyboard
    // - the drop down do not show the keyboard but the calendar
    if (!com.ibm.bpm.social.util.common.Util.isIPad || e.target == this._popupStateNode ||
      e.target.parentNode == this._popupStateNode) {
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        // if the input field is readonly the keyboard is not opened
        this.textbox.setAttribute("readOnly", "readonly");
        window.setTimeout(dojo.hitch(this, function(){
          this.inherited("_onDropDownMouseDown", [e]);
          this.textbox.removeAttribute("readOnly");
        }));
      }
      else {
        this.inherited(arguments);
      }
    }
  },

  closeDropDown: function(/*Boolean*/focus){
    // when closing with iPad focus always needs to be false
    focus = focus && !com.ibm.bpm.social.util.common.Util.isIPad;
    this.inherited("closeDropDown", [focus]);
  }
});

dojo.declare("com.ibm.bpm.social.widgets.task.util._DateTimeTimeTextBox", dijit.form.TimeTextBox, {
  _onDropDownMouseDown: function(/*Event*/e){
    // on iPad when clicking
    // - the text box do not show the drop down but the keyboard
    // - the drop down do not show the keyboard but the calendar
    if (!com.ibm.bpm.social.util.common.Util.isIPad || e.target == this._popupStateNode ||
      e.target.parentNode == this._popupStateNode) {
      if (com.ibm.bpm.social.util.common.Util.isIPad) {
        // if the input field is readonly the keyboard is not opened
        this.textbox.setAttribute("readOnly", "readonly");
        window.setTimeout(dojo.hitch(this, function(){
          this.inherited("_onDropDownMouseDown", [e]);
          this.textbox.removeAttribute("readOnly");
        }));
      }
      else {
        this.inherited(arguments);
      }
    }
  },

  closeDropDown: function(/*Boolean*/focus){
    // when closing with iPad focus always needs to be false
    focus = focus && !com.ibm.bpm.social.util.common.Util.isIPad;
    this.inherited("closeDropDown", [focus]);
  }
});

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.DateTime",
    [dijit._Widget,
     dijit._Templated],
    {

      templateString: '<div dojoattachpoint="dateTime" class="bpm-task-modify-task-dialog-due">'
        + '<table border="0" style="border-collapse: collapse"><tbody><tr>'
        + ' <td dojoAttachPoint="_dateLabelTd"><label dojoAttachPoint="_dateTime_labelDate"></label></td>'
        + ' <td dojoAttachPoint="_dateTd"><div dojoAttachPoint="_dateTime_date"></div></td>'
        + ' <td dojoAttachPoint="_timeLabelTd"><label dojoAttachPoint="_dateTime_labelTime"></label></td>'
        + ' <td dojoAttachPoint="_timeTd"><div dojoAttachPoint="_dateTime_time"></div></td>' + '</tr></tbody></table></div>',

      _dateTime: null,
      _date: null,
      _time: null,

      _isOnChangeDate: false,
      _isOnChangeTime: false,

      _ignoreNextOnChangeDate: false,
      _ignoreNextOnChangeTime: false,

      /**
       * @param param {
       *        <dl>
       *        <dt>dateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now", default is "now"</dd>
       *        <dt>minDateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now" </dd>
       *        <dt>maxDateTime:</dt>
       *        <dd>(optional) a {Date] object or string "now" </dd>
       *        <dt>datePromptMessage:</dt>
       *        <dd>(optional) property promptMessage for <i>DateTextBox</i> widget</dd>
       *        <dt>dateConstraints:</dt>
       *        <dd>(optional) constraints for <i>DateTextBox</i> widget</dd>
       *        <dt>timePromptMessage:</dt>
       *        <dd>(optional) property promptMessage for <i>TimeTextBox</i> widget</dd>
       *        <dt>timeConstraints:</dt>
       *        <dd>(optional) constraints for <i>TimeTextBox</i> widget</dd>
       *        </dl> }
       * @return {void} nothing
       */
      constructor: function constructor_DateTime(param){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");

        this._showNoLabels = param.showNoLabels;

        this._date = {
          properties: {
            required: true,
            intermediateChanges: true
          },
          constraints: {
            formatLength: "short"
          },
          textBox: null,
          isValid: true
        };

        this._time = {
          properties: {
            required: true,
            intermediateChanges: true
          },
          constraints: {
            formatLength: "short",
            visibleIncrement: "T00:15:00",
            clickableIncrement: "T00:15:00",
            visibleRange: "T03:00:00"
          },
          textBox: null,
          isValid: true
        };

        if (param.dateTime) {
          // may be "now"
          this._dateTime = this._parseDateTime(param.dateTime);
        }

        if (param.datePromptMessage) {
          this._date.properties.promptMessage = param.datePromptMessage;
        }
        else {
          this._date.properties.promptMessage = this._nlsResources.KEY_DatePrompt;
        }
        if (param.dateConstraints) {
          dojo.mixin(this._date.constraints, param.dateConstraints);
        }

        if (param.timePromptMessage) {
          this._time.properties.promptMessage = param.timePromptMessage;
        }
        else {
          this._time.properties.promptMessage = this._nlsResources.KEY_TimePrompt;
        }
        if (param.timeConstraints) {
          dojo.mixin(this._time.constraints, param.timeConstraints);
        }

        // initialize min/max constraints
        if (param.minDateTime) {
          this._minDateTime = param.minDateTime;
        }
        if (param.maxDateTime) {
          this._maxDateTime = param.maxDateTime;
        }
        if (this._minDateTime || this.maxDateTime) {
          var constraints = this._getMinMaxConstraints(this._dateTime, this._time.constraints.formatLength);
          if (constraints.minDate) {
            this._date.constraints.min = constraints.minDate;
          }
          if (constraints.maxDate) {
            this._date.constraints.max = constraints.maxDate;
          }
          if (constraints.minTime) {
            this._time.constraints.min = constraints.minTime;
          }
          if (constraints.maxTime) {
            this._time.constraints.max = constraints.maxTime;
          }
        }

        var calType = com.ibm.bpm.social.util.common.Util.getCalendarUserPreference();
        this._datePackage = null;
        if (calType === "hebrew" || calType === "islamic") {
          this._datePackage = "dojox.date." + calType;
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      postCreate: function postCreate_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // create content
        //

        // date
        var id = this.id + "_date"; // unique id
        this._dateTime_date.setAttribute("id", id);
        this._dateTime_labelDate.setAttribute("for", id);
        this._dateTime_labelDate.innerHTML = this._nlsResources.task_dialog_modifyTask_DueSectionTitle;

        var date = null;
        if (this._dateTime !== null) {
          date = dojo.clone(this._dateTime);
          date.setHours(0, 0, 0, 0);
        }

        this._date.textBox = new com.ibm.bpm.social.widgets.task.util._DateTimeDateTextBox({
          value: dojo.date.stamp.toISOString(date, {
            selector: 'date'
          }),
          datePackage: this._datePackage,
          constraints: this._date.constraints
        }, this._dateTime_date);

        dojo.mixin(this._date.textBox, this._date.properties);
        dijit.setWaiState(this._date.textBox.textbox, "label", this._nlsResources.task_dialog_modifyTask_DueSectionTitle);

        if (!this._showNoLabels) {
          dojo.addClass(this._dateLabelTd, "bpm-task-modify-task-dialog-first-td");
        }
        else {
          dojo.addClass(this._dateTd, "bpm-task-modify-task-dialog-first-td");
          this._dateLabelTd.style.display = "none";

        }

        // time
        id = this.id + "_time"; // unique id
        this._dateTime_time.setAttribute("id", id);
        this._dateTime_labelTime.setAttribute("for", id);
        this._dateTime_labelTime.innerHTML = this._nlsResources["task_dialog_modifyTask_DueTime-label"];

        var time = null;
        if (this._dateTime !== null) {
          time = dojo.clone(this._dateTime);
        }
        this._time.textBox = new com.ibm.bpm.social.widgets.task.util._DateTimeTimeTextBox({
          value: time,
          constraints: this._time.constraints
        }, this._dateTime_time);
        dojo.mixin(this._time.textBox, this._time.properties);

        if (this._showNoLabels) {
          this._timeLabelTd.style.display = "none";

        }
        dijit.setWaiState(this._time.textBox.textbox, "label", this._nlsResources["task_dialog_modifyTask_DueTime-label"]);

        // initialize properties
        //
        this._date.isValid = this._date.textBox.validate();
        if (this._date.isValid) {
          this._time.isValid = this._time.textBox.validate();
        }
        else {
          this._time.isValid = true;
        }

        // set handlers
        //
        this._setHandlers();

        // Add tundra style to time and date picker, otherwise the styling is
        // not complete.

        this.connect(this._date.textBox, "openDropDown", function(){
          dojo.addClass(this._date.textBox.dropDown.domNode.parentNode, "tundra");
        });
        this.connect(this._time.textBox, "openDropDown", function(){
          dojo.addClass(this._time.textBox.dropDown.domNode.parentNode, "tundra");
        });

        // Do common UI styling stuff
        dojo.addClass(this._date.textBox.domNode, 'bspace-border');
        dojo.addClass(this._date.textBox.domNode, "bspace-combo-box");
        dojo.addClass(this._time.textBox.domNode, 'bspace-border');
        dojo.addClass(this._time.textBox.domNode, "bspace-combo-box");

        if (!this.isValid()) {
          dojo.addClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
          dojo.addClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
          dojo.addClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
          dojo.addClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
        }

        this.connect(this, "onChange", function(){
          if (this.isValid()) {
            dojo.removeClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.removeClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.removeClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
            dojo.removeClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
          }
          else {
            dojo.addClass(this._date.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.addClass(this._time.textBox.textbox.parentNode, 'bspace-border-incorrect');
            dojo.addClass(this._date.textBox.domNode, 'bspace-text-field-incorrect');
            dojo.addClass(this._time.textBox.domNode, 'bspace-text-field-incorrect');
          }
        });

        this.connect(this._date.textBox.domNode, "mouseover", function(){
          dojo.addClass(this._date.textBox.domNode, 'bspace-border-over');
          dojo.addClass(this._date.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._date.textBox.domNode, "mouseout", function(){
          dojo.removeClass(this._date.textBox.domNode, 'bspace-border-over');
          dojo.removeClass(this._date.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._date.textBox.textbox, "focus", function(){
          dojo.addClass(this._date.textBox.domNode, 'bspace-border-active');
          dojo.addClass(this._date.textBox.textbox, 'bspace-text-field-active');
          dojo.addClass(this._date.textBox.domNode, 'bspace-combo-box-active');
        });
        this.connect(this._date.textBox.textbox, "blur", function(){
          dojo.removeClass(this._date.textBox.domNode, 'bspace-border-active');
          dojo.removeClass(this._date.textBox.textbox, 'bspace-text-field-active');
          dojo.removeClass(this._date.textBox.domNode, 'bspace-combo-box-active');
        });

        this.connect(this._time.textBox.domNode, "mouseover", function(){
          dojo.addClass(this._time.textBox.domNode, 'bspace-border-over');
          dojo.addClass(this._time.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._time.textBox.domNode, "mouseout", function(){
          dojo.removeClass(this._time.textBox.domNode, 'bspace-border-over');
          dojo.removeClass(this._time.textBox.domNode, 'bspace-combo-box-over');
        });
        this.connect(this._time.textBox.textbox, "focus", function(){
          dojo.addClass(this._time.textBox.domNode, 'bspace-border-active');
          dojo.addClass(this._time.textBox.textbox, 'bspace-text-field-active');
          dojo.addClass(this._time.textBox.domNode, 'bspace-combo-box-active');
        });
        this.connect(this._time.textBox.textbox, "blur", function(){
          dojo.removeClass(this._time.textBox.domNode, 'bspace-border-active');
          dojo.removeClass(this._time.textBox.textbox, 'bspace-text-field-active');
          dojo.removeClass(this._time.textBox.domNode, 'bspace-combo-box-active');
        });

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      startup: function startup_DateTime(){

      },

      /**
       * @return {Date} dateTime
       */
      getDateTime: function getDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var dateTime = null;
        if (this.isValid()) {
          dateTime = this._getDateTime();
        }
        com.ibm.bpm.social.util.common.Log.log(3, "com.ibm.bpm.social.widgets.task.util.DateTime.getDateTime ... dateTime=" +
          dateTime);
        com.ibm.bpm.social.util.common.Log.logExit(1);
        return dateTime;
      },

      /**
       * @type {void}
       */
      isValid: function isValid_DateTime(){
        return this._date.isValid && this._time.isValid;
      },

      /**
       * @type {void}
       */
      isDirty: function isDirty_DateTime(){
        if (this.isValid()) {
          return this._isDateTimeChanged();
        }
        return false;
      },

      onChange: function onChange_DateTime(newValue){
      // event fired if value changes
      },

      setDisabled: function setDisabled_DateTime(disabled){
        if (this._date.textBox !== null) {
          this._date.textBox.set('disabled', disabled);
        }
        if (this._time.textBox !== null) {
          this._time.textBox.set('disabled', disabled);
        }
      },

      _setHandlers: function _setHandlers_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        this._disconnectHandlers();

        if (this._date.textBox !== null) {
          this._eventConnections.push(dojo.connect(this._date.textBox, "onChange", this, this._onChangeDate));
        }
        if (this._time.textBox !== null) {
          this._eventConnections.push(dojo.connect(this._time.textBox, "onChange", this, this._onChangeTime));
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
        return;
      },

      _onChangeDate: function _onChangeDate_DateTime(newValue){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._onChangeDate ... newValue=" + newValue);
        if (!this._isOnChangeDate && !this._ignoreNextOnChangeDate) {
          this._isOnChangeDate = !this._isOnChangeTime;
          if (newValue === undefined || (this._date.textBox.required && newValue === null)) {
            this._date.isValid = false;
          }
          else {
            this._date.isValid = true;
            if (!this._isOnChangeTime) {
              // auto fill empty time field
              if (this._time.textBox !== null) {
                if (this._getTime() === null) {
                  // set time to midnight if no time is set
                  var midnight = new Date();
                  midnight.setHours(0, 0, 0, 0);
                  this._setTime(midnight);
                  this._ignoreNextOnChangeTime = true; // ignore next onChange because validation is done here
                }
              }

              // set new min/max values
              this._setMinMaxConstraints();

              // validate date first
              if (this._date.textBox !== null) {
                this._date.isValid = this._date.textBox.validate(true);
              }
              if (this._time.textBox !== null) {
                this._time.isValid = this._time.textBox.validate();
              }
            }
          }

          this.onChange(this.getDateTime());
          this._isOnChangeDate = false;
        }

        this._ignoreNextOnChangeDate = false;

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _onChangeTime: function _onChangeTime_DateTime(newValue){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._onChangeTime ... newValue=" + newValue);
        if (!this._isOnChangeTime && !this._ignoreNextOnChangeTime) {
          this._isOnChangeTime = !this._isOnChangeDate;
          if (newValue === undefined || (this._time.textBox.required && newValue === null)) {
            this._time.isValid = false;
          }
          else {
            this._time.isValid = true;
            if (!this._isOnChangeDate) {
              // auto fill empty date field
              if (this._date.textBox !== null) {
                if (this._getDate() === null) {
                  // set date today or tomorrow depending on current time if no date is set
                  var date = new Date();
                  date.setHours(0, 0, 0, 0);
                  if (dojo.date.compare(this._getTime(), new Date(), "time") < 0) {
                    date = dojo.date.add(date, "day", 1); // tomorrow
                  }
                  this._setDate(date);
                  this._ignoreNextOnChangeDate = true; // ignore next onChange because validation is done here
                }
              }

              // set new min/max values
              this._setMinMaxConstraints();

              // validate time first
              if (this._time.textBox !== null) {
                this._time.isValid = this._time.textBox.validate(true);
              }
              if (this._date.textBox !== null) {
                this._date.isValid = this._date.textBox.validate();
              }
            }
          }

          this.onChange(this.getDateTime());
          this._isOnChangeTime = false;
        }

        this._ignoreNextOnChangeTime = false;

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * @return {Boolean} isChanged
       */
      _isDateTimeChanged: function _isDateTimeChanged_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var isChanged = false;
        var newDateTime = this._getDateTime();
        if (this._dateTime !== null && newDateTime !== null) {
          isChanged = dojo.date.compare(newDateTime, this._dateTime, "datetime") !== 0;
        }
        else {
          isChanged = !(this._dateTime === null && newDateTime === null);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return isChanged;
      },

      /**
       * @return {Date} dateTime
       */
      _getDateTime: function _getDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var dateTime = null;
        dateTime = this._getDateTimeFromDateAndTime(this._getDate(), this._getTime());

        return dateTime;
      },

      /**
       * @return {Date} date
       */
      _getDate: function _getDate_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var date = null;
        if (this._date.textBox !== null) {
          date = this._date.textBox.get('value');
          if (this._datePackage) {
            date = date.toGregorian();
          }
        }
        return date;
      },

      /**
       * @param {Date} date
       */
      _setDate: function _setDate_DateTime(date){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._date.textBox !== null) {
          this._date.textBox.set('value', date);
        }
      },

      /**
       * @return {Date} time
       */
      _getTime: function _getTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var time = null;
        if (this._time.textBox !== null) {
          time = this._time.textBox.get('value');
        }
        return time;
      },

      /**
       * @param {Date} time
       */
      _setTime: function _setTime_DateTime(time){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._time.textBox !== null) {
          this._time.textBox.set('value', time);
        }
      },

      /**
       * @return {Date} dateTime
       */
      _getDateTimeFromDateAndTime: function _getDateTimeFromDateAndTime_DateTime(date, time){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... date=" + date + ", time=" + time);
        var dateTime = null;

        if (date !== undefined && time !== undefined) {
          if (date !== null && time === null) {
            // date only
            dateTime = date;
          }
          else if (time !== null) {
            if (date === null) {
              // time only means today
              date = new Date();
            }
            // combine date and time
            dateTime =
              new Date(date.getFullYear(), date.getMonth(), date.getDate(), time.getHours(), time.getMinutes(), time
                .getSeconds(), time.getMilliseconds());
          }
        }
        else {
          com.ibm.bpm.social.util.common.Log
            .log(3,
              "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... date and/or time is undefined/invalid");
        }

        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getDateTimeFromDateAndTime ... dateTime=" + dateTime);
        com.ibm.bpm.social.util.common.Log.logExit(2);

        return dateTime;
      },

      /**
       * @return {Date} dateTime
       */
      _parseDateTime: function _parseDateTime_DateTime(dateTime){
        var result = dateTime;
        if (result !== null) {
          if (typeof result === "string" && result === "now") {
            result = new Date();
          }
        }
        return result;
      },

      /**
       * Sets the min and max constraints on the textboxes
       * 
       * @returns {void} nothing
       */
      _setMinMaxConstraints: function _setMinMaxConstraints_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var constraints = this._getMinMaxConstraints(this._getDateTime(), this._time.textBox.constraints.formatLength);
        if (this._date.textBox !== null) {
          if (constraints.minDate) {
            this._date.textBox.constraints.min = constraints.minDate;
          }
          if (constraints.maxDate) {
            this._date.textBox.constraints.max = constraints.maxDate;
          }
        }
        if (this._time.textBox !== null) {
          if (constraints.minTime) {
            this._time.textBox.constraints.min = constraints.minTime;
          }
          if (constraints.maxTime) {
            this._time.textBox.constraints.max = constraints.maxTime;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      /**
       * Calculates the new min and max constraints depending on the actual values
       * 
       * @param {Date} dateTime
       * @param {String} timeFormatLength the formatLength property of the time constraints "short", "medium",
       *        "long" or "full", default is "short"
       * @returns {Object} {minDate, maxDate, minTime, maxTime}
       */
      _getMinMaxConstraints: function _getMinMaxConstraints_DateTime(dateTime, timeFormatLength){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var result = {};
        var compareDate = null;
        var compareTime = null;

        // min date, time
        var minDateTime = this._getMinDateTime();
        if (minDateTime !== null) {
          if (!timeFormatLength || timeFormatLength === "short" || timeFormatLength === "medium") {
            minDateTime.setSeconds(0, 0);
          }
          com.ibm.bpm.social.util.common.Log.log(3,
            "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... minDateTime=" + minDateTime);
          // minDate is date limit date, minTime is date limit time
          var minDate = dojo.clone(minDateTime);
          minDate.setHours(0, 0, 0, 0);
          var minTime = dojo.clone(minDateTime);
          if (dateTime) {
            compareDate = dojo.date.compare(dateTime, minDateTime, "date");
            compareTime = dojo.date.compare(dateTime, minDateTime, "time");
            // default constraints: minDate and minTime
            if (compareDate > 0) {
              // no time constraint
              minTime = -Infinity;
            }
            if (compareTime < 0) {
              // minDate is next day, no time constraint
              minDate = dojo.date.add(minDate, "day", 1);
              minTime = -Infinity;
            }
          }
          else {
            // no date time set yet
            // minDate is next day, no time constraint
            minDate = dojo.date.add(minDate, "day", 1);
            minTime = -Infinity;
          }
          result.minDate = minDate;
          result.minTime = minTime;
        }

        // max date, time
        var maxDateTime = this._getMaxDateTime();
        if (maxDateTime !== null) {
          if (!timeFormatLength || timeFormatLength === "short" || timeFormatLength === "medium") {
            maxDateTime.setSeconds(0, 0);
          }
          com.ibm.bpm.social.util.common.Log.log(3,
            "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... maxDateTime=" + maxDateTime);
          // maxDate is date limit date, maxTime is date limit time
          var maxDate = dojo.clone(maxDateTime);
          maxDate.setHours(0, 0, 0, 0);
          var maxTime = dojo.clone(maxDateTime);
          if (dateTime) {
            compareDate = dojo.date.compare(dateTime, maxDateTime, "date");
            compareTime = dojo.date.compare(dateTime, maxDateTime, "time");
            // default constraints: maxDate and maxTime
            if (compareDate < 0) {
              // no time constraint
              maxTime = Infinity;
            }
            if (compareTime > 0) {
              // maxDate is previous day, no time constraint
              maxDate = dojo.date.add(maxDate, "day", -1);
              maxTime = Infinity;
            }
          }
          else {
            // no date time set yet
            // maxDate is previous day, no time constraint
            maxDate = dojo.date.add(maxDate, "day", -1);
            maxTime = Infinity;
          }
          result.maxDate = maxDate;
          result.maxTime = maxTime;
        }

        com.ibm.bpm.social.util.common.Log.logObject(3, result,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMinMaxConstraints ... result");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _getMinDateTime: function _getMinDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this._minDateTime) {
          var minDate = null;
          if (this._date.textBox !== null) {
            minDate = this._date.textBox.constraints.min ? this._date.textBox.constraints.min : null;
          }
          var minTime = null;
          if (this._time.textBox !== null) {
            minTime = this._time.textBox.constraints.max ? this._time.textBox.constraints.min : null;
          }
          this._minDateTime = this._getDateTimeFromDateAndTime(minDate, minTime);
        }
        var result = this._parseDateTime(this._minDateTime);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMinDateTime ... minDateTime=" + result);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _getMaxDateTime: function _getMaxDateTime_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this._maxDateTime) {
          var maxDate = null;
          if (this._date.textBox !== null) {
            maxDate = this._date.textBox.constraints.max ? this._date.textBox.constraints.max : null;
          }
          var maxTime = null;
          if (this._time.textBox !== null) {
            maxTime = this._time.textBox.constraints.max ? this._time.textBox.constraints.max : null;
          }
          this._maxDateTime = this._getDateTimeFromDateAndTime(maxDate, maxTime);
        }
        var result = this._parseDateTime(this._maxDateTime);
        com.ibm.bpm.social.util.common.Log.log(3,
          "com.ibm.bpm.social.widgets.task.util.DateTime._getMaxDateTime ... maxDateTime=" + result);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return result;
      },

      _disconnectHandlers: function _disconnectHandlers_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._eventConnections) {
          for ( var i = 0; i < this._eventConnections.length; i++) {
            dojo.disconnect(this._eventConnections[i]);
          }
        }
        this._eventConnections = [];
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

      destroy: function destroy_DateTime(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        this._disconnectHandlers();

        if (this._date && this._date.textBox) {
          this._date.textBox.destroy();
          this._date.textBox = null;
        }
        if (this._time && this._time.textBox) {
          this._time.textBox.destroy();
          this._time.textBox = null;
        }
        this.inherited("destroy", arguments);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      }

    });
