// EntityQuery.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

// **************************************
//
// File Version: 1.5 11/10/03 16:29:49
//
// **************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.EntityQuery");

dojo.require("com.ibm.bpm.social.util.common.Log");

/*
 * Creates and executes a list query based on various parameters.
 */
dojo.declare("com.ibm.bpm.social.widgets.task.util.EntityQuery", null, {
  _customFilter: null,
  _userRoleFilter: null,
  _queue: null,
  _prioritiesConfig: null,
  _idFilter: null,
  _groupingProperties: null,
  _sortProperties: null,
  _async: true,
  _offset: 0,
  _pageSize: null,
  _selectedAttributes: null,
  _type: null,

  constructor: function constructor_EntityQuery(parameters){
    this.restHandler = parameters.restHandler;
    this._D2DFilter = parameters.D2DFilter;
    this._userId = parameters.userId;
    this._widgetId = parameters.widgetId;
    this._systems = parameters.systems;
  },

  /**
   * returns the type of the query, can be com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE or
   * com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE.
   * 
   * @return the type of the query
   */
  getType: function getType_EntityQuery(){
    return this._type;
  },

  setType: function setType_EntityQuery(queue){
    if (queue.entityTypeName == "TASK") {
      this._type = com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE;
    }
    else if (queue.entityTypeName == "PROCESS_INSTANCE") {
      this._type = com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE;
    }
    else {
      var e = new Error("Unexpected entityTypeName '" + queue.entityTypeName + "'.");
      com.ibm.bpm.social.util.common.Log.logError(1, "Unexpected entityTypeName '" + queue.entityTypeName + "'.");
      throw e;
    }
  },
  setAsync: function setAsync_EntityQuery(async){
    this._async = async;
  },

  setOffset: function setOffset_EntityQuery(offset){
    this._offset = offset;
  },

  setPageSize: function setPageSize_EntityQuery(pageSize){
    this._pageSize = pageSize;
  },

  setSelectedAttributes: function setSelectedAttributes_EntityQuery(selectedAttributes){
    this._selectedAttributes = selectedAttributes;
  },

  setCustomFilter: function setCustomFilter_EntityQuery(customFilter){
    this._customFilter = customFilter;
  },

  clear: function clear_EntityQuery(){
    this._type = null;
    this._customFilter = null;
    this._selectedAttributes = null;
    this._offset = null;
    this._pageSize = null;
    this._async = null;
    this._userRoleFilter = null;
    this._queue = null;
    this._prioritiesConfig = null;
    this._idFilter = null;
    this._sortProperties = null;
  },

  /*
   * Checks if the filter syntax is valid by executing a query.
   */
  testCustomFilter: function testCustomFilter_EntityQuery(customFilter, error){
    // Store current query state in local variables
    var oldFilter = this._customFilter;
    this.setCustomFilter(customFilter);

    var oldQueue = this._queue;
    this.setQueue(customFilter.queue);

    var oldAsync = this._async;
    this.setAsync(false);

    var oldSelectedAttributes = this._selectedAttributes;
    this.setSelectedAttributes(null);

    var oldOffset = this._offset;
    this.setOffset(0);

    var oldPageSize = this._pageSize;
    this.setPageSize(1);

    var ok = false;
    this._loadItems(function(responseObject, ioArgs){
      ok = true;
    }, error);

    // Reset the query state
    this.setCustomFilter(oldFilter);
    this.setQueue(oldQueue);
    this.setAsync(oldAsync);
    this.setSelectedAttributes(oldSelectedAttributes);
    this.setOffset(oldOffset);
    this.setPageSize(oldPageSize);

    return ok;
  },

  setQueue: function setQueue_EntityQuery(queue){
    this._queue = queue;
  },

  setUserRoleFilter: function setUserRoleFilter_EntityQueue(userRoleFilter){
    this._userRoleFilter = userRoleFilter;
  },

  setIdFilter: function setIdFilter_EntityQueue(idFilter){
    this._idFilter = idFilter;
  },

  setPrioritiesConfig: function setPrioritiesConfig_EntityQueue(prioritiesConfig){
    this._prioritiesConfig = prioritiesConfig;
  },

  setGroupingProperties: function setGroupingProperties_EntityQueue(groupingProperties){
    this._groupingProperties = groupingProperties;
  },

  setSearchFilter: function setSearchFilter_EntityQueue(text){
    this._searchFilter = text;
  },

  /**
   * @param load
   * @param error {void} nothing
   * @returns {dojo.Deferred} an object which allows to cancel the dojo.xhr request
   */
  performContentRequest: function performContentRequest_EntityQuery(load, error){
    return this._loadItems(load, error);
  },

  performCountRequest: function performCountRequest_EntityQuery(load, error){
    this._countItems(load, error);
  },

  /*
   * Returns whereClause and/or interactionFilter.
   */
  toObject: function toObject_EntityQuery(){
    return this._getWhereClause();
  },

  _loadItems: function _loadItems_EntityQuery(load, error){

    var params = {};
    // set the order by statement
    var sortBy = null;
    var sortProperties = (this._userRoleFilter ? this._userRoleFilter.sortProperties : null);
    // sort order 
    if (sortProperties && sortProperties.length > 0) {
      for ( var i = 0; i < sortProperties.length; i++) {
        if (!sortBy) {
          sortBy = sortProperties[i].property + " " + sortProperties[i].sortDir;
        }
        else {
          sortBy += "," + sortProperties[i].property + " " + sortProperties[i].sortDir;
        }
      }
    }

    // the following is old mimic and only there for compat reasons
    // if we have no sortProperties from the _userRoleFilter, get the sort from the items

    // check for common sort settings 
    if (!sortProperties && this._queue.item && this._queue.item.properties) { // TODO ?
      var properties = this._queue.item.properties;
      for (i = 0; i < properties.length; i++) {
        if (properties[i].isSort === true) {
          if (!sortBy) {
            sortBy =
              properties[i].property + " " + this._replaceOrderParameter(properties[i].property, properties[i].sortDir);
          }
          else {
            // don't set the same property twice
            if (!this._groupingProperties || this._groupingProperties.property != properties[i].property) {
              sortBy =
                sortBy + "," + properties[i].property + " " +
                  this._replaceOrderParameter(properties[i].property, properties[i].sortDir);
            }
          }
          // TODO what is the API limit for sortBy ? 
          // break;
        }
      }
    }

    params.sortAttributes = sortBy;

    var currentQuery = this._getWhereClause();
    var whereClause = "";
    if (currentQuery.whereClause) {
      whereClause = currentQuery.whereClause;
    }

    // for process based queries include the task data
    if (this._queue.entityTypeName == "PROCESS_INSTANCE") {
      params.includeTaskData = true;
    }

    params.queryFilter = whereClause;
    params.interactionFilter = currentQuery.interactionFilter;
    params.queryTable = this._queue.getPlainQueueName();
    params.domain = this._queue.getDomain();
    params.processAppName = this._queue.processAppName; // may be null, or undefined, or a valid String
    params.selectedAttributes = this._selectedAttributes;
    params.type = this._type;
    params.offset = this._offset;
    params.size = this._pageSize;
    params.searchFilter = this._searchFilter;
    params.async = this._async;
    params.load = function(responseObject, ioArgs){
      load(responseObject);
    };
    params.error = function(data, ioArgs){
      error(data, ioArgs);
    };
    // Fetch data from server    
    return this.restHandler.getQueryTableContent(params);
  },

  _countItems: function _countItems_EntityQuery(load, error){
    var params = {};

    var currentQuery = this._getWhereClause();

    params.whereClause = currentQuery.whereClause;
    params.interactionFilter = currentQuery.interactionFilter;
    params.queryTable = this._queue.getPlainQueueName();
    params.domain = this._queue.getDomain();
    params.processAppName = this._queue.processAppName; // may be null, or undefined, or a valid String
    params.selectedAttributes = this._selectedAttributes;
    params.type = this._type;
    params.offset = this._offset;
    params.size = this._pageSize;
    params.searchFilter = this._searchFilter;
    params.async = this._async;
    params.load = function(responseObject, ioArgs){
      load(responseObject);
    };
    params.error = function(data, ioArgs){
      error(data, ioArgs);
    };

    this.restHandler.getQueryTableCount(params);
  },

  _getWhereClause: function _getWhereClause_EntityQuery(){
    // get the properties map for the current queue
    var propertiesMap = {};
    if (this._queue[com.ibm.bpm.social.widgets.task.util.Constants.QUEUE_PROPERTIES_MAP]) {
      propertiesMap = this._queue[com.ibm.bpm.social.widgets.task.util.Constants.QUEUE_PROPERTIES_MAP];
    }
    var whereClause = null;
    var interactionFilter = null;
    var whereClauseOptionalProperties = null;
    if (this._userRoleFilter) {
      whereClause = this._userRoleFilter.templateWhereClause;
      interactionFilter = this._userRoleFilter.interactionFilter;
      whereClauseOptionalProperties = this._userRoleFilter.templateWhereClauseOptional;
    }
    var whereClauseOptional = "";
    if (!interactionFilter || whereClause) {
      // ensure empty or undefined interactionFilter to be mapped to null
      // or, if there is a whereclause, then we do not use the interaction filter but the where clause
      interactionFilter = null;
    }

    // optional whereClauses are added only if the optional property is available
    if (whereClauseOptionalProperties && this.restHandler.isApiTypeWps()) {
      dojo.forEach(whereClauseOptionalProperties, function(optionalItem, index){
        if (propertiesMap[optionalItem.property] !== null) {
          var clause = optionalItem.clause;
          if (optionalItem.clauses) {
            // more than one possible clause
            // last clause with all properties available is used
            for ( var i = 0; i < optionalItem.clauses.length; i++) {
              var validClause = true;
              var item = optionalItem.clauses[i];
              for ( var p = 0; p < item.additionalProperties.length; p++) {
                if (propertiesMap[item.additionalProperties[p]] === null) {
                  validClause = false;
                  break;
                }
              }
              if (validClause) {
                clause = item.clause;
              }
            }
          }
          if (clause) {
            if (index > 0) {
              whereClauseOptional += " AND ";
            }
            whereClauseOptional += clause;
          }
        }
      });

      if (whereClauseOptional !== "") {
        if (whereClause === null || whereClause === undefined || whereClause === "") {
          whereClause = whereClauseOptional;
        }
        else {
          whereClause += " AND " + whereClauseOptional;
        }
      }
    }

    if (this._customFilter) {
      if (!whereClause) {
        whereClause = this._createFilterWhereClause();
      }
      else {
        whereClause = whereClause + " AND (" + this._createFilterWhereClause() + ")";
      }
    }
    // Add D2D filter, if there is one.
    if (this._D2DFilter) {
      var applicNames = this._D2DFilter.split(",");
      if (applicNames) {
        // use the replacement variable for the application
        var D2DWhereClause = "${TASKAPPLICNAME}" + " IN (";
        for ( var k in applicNames) {
          if (k > 0) {
            D2DWhereClause += ",";
          }
          D2DWhereClause +=
            "'" + com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(applicNames[k]) + "'";
        }
        D2DWhereClause += ")";

        var applicationAttribute = propertiesMap["TASKAPPLICNAME"];
        // but only if the task applic name column is in the table
        if (applicationAttribute) {
          whereClause += " AND (" + D2DWhereClause + ")";
        }
      }
    }
    whereClause = this._replaceWhereParameters(whereClause, propertiesMap);

    if (this._idFilter && this._idFilter.property) {
      // Get attribute name for the parent property.
      var parentPropertyAttribute = propertiesMap[this._idFilter.property];

      if (this._idFilter.ids && this._idFilter.ids.length > 0) {
        var idWhereClause = null;
        if (this._idFilter.ids.length == 1) {
          idWhereClause = parentPropertyAttribute + " = ID('" + this._idFilter.ids[0] + "')";
        }
        else if (this._idFilter.ids.length > 1) {
          idWhereClause = parentPropertyAttribute + " IN (";
          dojo.forEach(this._idFilter.ids, function(id, index){
            if (index > 0) {
              idWhereClause += ", ";
            }
            idWhereClause += "ID('" + id + "')";
          });
          idWhereClause += ")";
        }

        if (whereClause === "") {
          whereClause = idWhereClause;
        }
        else {
          whereClause += " AND " + idWhereClause;
        }
      }
      else {
        if (whereClause === "") {
          whereClause = "(" + parentPropertyAttribute + " IS NULL";
        }
        else {
          whereClause += " AND (" + parentPropertyAttribute + " IS NULL";
        }

        // Get expanded tree nodes from cookie.
        var cookieForExpandedIds = dojo.cookie(this._widgetId + "_lazyLoadingTreeSaveStateCookie");
        var expandedIds = [];
        if (cookieForExpandedIds) {
          expandedIds = cookieForExpandedIds.split(",");
        }

        // Remove "null" elements from the array. I'm not quite sure where they come from, I
        // suppose from the DOJO tree implementation.
        expandedIds = dojo.filter(expandedIds, function(id){
          return (id != "null" && id != "undefined" && id !== "");
        });

        // Now create the where clause by using the expanded id's.
        if (expandedIds.length == 1) {
          whereClause += " OR " + parentPropertyAttribute + " = ID('" + expandedIds[0] + "')";
        }
        else if (expandedIds.length > 1) {
          whereClause += " OR " + parentPropertyAttribute + " IN (";
          dojo.forEach(expandedIds, function(id, index){
            if (index > 0) {
              whereClause += ", ";
            }
            whereClause += "ID('" + id + "')";
          });
          whereClause += ")";
        }

        whereClause += ")";
      }
    }

    return {
      whereClause: whereClause,
      interactionFilter: interactionFilter
    };
  },

  _replaceOrderParameter: function _replaceOrderParameter_EntityQuery(propertyName, order){
    var properties = this._queue.item.properties;
    var property = dojo.filter(properties, function(elem){
      return (elem.property == propertyName && elem.type == "priority");
    });
    var result = order;
    if (property.length > 0) {
      var prios = this._prioritiesConfig;
      if (prios) {
        // get the settings for high priority
        var high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
        var low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
        // specify the sort order depending on the high and low range values
        if (typeof high == "number" && typeof low == "number") {
          if (high < low) {
            if (order == "ASC") {
              // ascending means high values last
              result = "DESC";
            }
            else {
              result = "ASC";
            }
          }
        }
      }
    }
    return result;
  },

  _createFilterWhereClause: function _createFilterWhereClause_EntityQuery(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var whereClause = "";
    if (this._customFilter) {
      // if the filter has a user defined where clause
      if (this._customFilter.whereClause) {
        whereClause = "(" + this._customFilter.whereClause + ")";
      }
      // go over all filter criterias
      if (this._customFilter.criteria && this._customFilter.criteria.length > 0) {
        var criterias = this._customFilter.criteria;
        var lastPropertyName = null;
        for ( var k in criterias) {
          // operator only if there are multiple filter criteria
          var f = criterias[k];
          if (k > 0) {
            if (lastPropertyName == f.property) {
              whereClause = whereClause + " OR ";
            }
            else {
              whereClause = whereClause + ") AND (";
            }
          }
          else {
            // if there was a fixed whereclause already
            if (whereClause !== "") {
              whereClause += " AND ";
            }
            // otherwise just start the whereclause here
            whereClause = whereClause + "(";
          }
          lastPropertyName = f.property;
          var type = f.type;
          var dateUntil;
          var date;
          var date24;
          switch (type) {
            case com.ibm.bpm.social.widgets.task.util.Constants.PROPERTY_TYPE_ID_CONFIG :
              //case "id" :
              var v, domain;

              if (f.properties) {
                // ID filtering with multiple properties
                if (this._systems.isApiTypeFederated()) {
                  // we need to prefix the ID in Federation
                  domain = f.domain || this._queue.domain;
                  if (f.systemID) {
                    // there is already a systemID (usually coming from a Filter event)
                    var prefixedID = this.restHandler.prefixID(f.value, domain, f.systemID);
                    whereClause = whereClause + " " + f.property + "=ID('" + prefixedID + "')";
                    for (v = 0; v < f.properties.length; v++) {
                      whereClause = whereClause + " OR " + f.properties[v] + "=ID('" + prefixedID + "')";
                    }
                  }
                  else {
                    // there is no systemID (usually coming from an adhoc filter), we need to search in all systems of the matching type
                    var systemIDs = this._systems.getSystems(domain);

                    // ID can only be WLE or WPS because it is an equal compare no wild card search
                    var isWleID =
                      com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE.test(f.value) ||
                        com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE.test(f.value);
                    var isWpsID = com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS.test(f.value);

                    var prefixedIDs = [];

                    for (v = 0; v < systemIDs.length; v++) {
                      if ((isWleID && this._systems.isSystemWle(systemIDs[v])) ||
                        (isWpsID && this._systems.isSystemWps(systemIDs[v]))) {
                        prefixedIDs.push(this.restHandler.prefixID(f.value, domain, systemIDs[v]));
                      }
                    }
                    // if a filter event includes a wrong ID then we can not set the prefix 
                    if (prefixedIDs.length === 0) {
                      throw new Error("prefixedIDs.length==0");
                    }

                    whereClause = whereClause + " " + f.property + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                    for (v = 0; v < f.properties.length; v++) {
                      whereClause =
                        whereClause + " OR " + f.properties[v] + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                    }
                  }
                }
                else {
                  // in non-federation we do not need to prefix the ID
                  whereClause = whereClause + " " + f.property + "=ID('" + f.value + "')";
                  for (v = 0; v < f.properties.length; v++) {
                    whereClause = whereClause + " OR " + f.properties[v] + "=ID('" + f.value + "')";
                  }
                }
              }
              else {
                // ID filtering with one property
                if (this._systems.isApiTypeFederated()) {
                  // we need to prefix the ID in Federation
                  domain = f.domain || this._queue.domain;
                  if (f.systemID) {
                    // there is already a systemID (usually coming from a Filter event)
                    prefixedID = this.restHandler.prefixID(f.value, domain, f.systemID);
                    whereClause = whereClause + " " + f.property + "=ID('" + prefixedID + "')";
                  }
                  else {
                    // there is no systemID (usually coming from an adhoc filter), we need to search in all systems
                    systemIDs = this._systems.getSystems(domain);

                    isWleID =
                      com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE.test(f.value) ||
                        com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE.test(f.value);
                    isWpsID = com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS.test(f.value);

                    prefixedIDs = [];

                    for (v = 0; v < systemIDs.length; v++) {
                      if ((isWleID && this._systems.isSystemWle(systemIDs[v])) ||
                        (isWpsID && this._systems.isSystemWps(systemIDs[v]))) {
                        prefixedIDs.push(this.restHandler.prefixID(f.value, domain, systemIDs[v]));
                      }
                    }
                    // if a filter event includes a wrong ID then we can not set the prefix 
                    if (prefixedIDs.length === 0) {
                      throw new Error("prefixedIDs.length==0");
                    }

                    whereClause = whereClause + " " + f.property + " IN (ID('" + prefixedIDs.join("'),ID('") + "'))";
                  }
                }
                else {
                  // in non-federation we do not need to prefix the ID
                  whereClause = whereClause + " " + f.property + "=ID('" + f.value + "')";
                }
              }

              break;
            case "text" :
            case "user" :
              whereClause =
                whereClause + " " + f.property + " LIKE '%" +
                  com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(f.value) + "%'";
              break;
            case "date" :
              date = new Date(f.value);
              date.setHours(0, 0, 0);
              date24 = dojo.date.add(date, "day", 1);
              date24.setHours(0, 0, 0);
              whereClause =
                whereClause + " " + f.property + " >= TS('" +
                  com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "') AND " + f.property + " < TS('" +
                  com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              break;
            case "dateRange" :
              date = null;
              if (f.fromValue && !f.untilValue) {
                date = new Date(f.fromValue);
                date.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (!f.fromValue && f.untilValue) {
                date = new Date(f.untilValue);
                date = dojo.date.add(date, "day", 1);
                date.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " < TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (f.fromValue && f.untilValue) {
                var dateFrom = new Date(f.fromValue);
                dateUntil = new Date(f.untilValue);

                if (dojo.date.compare(dateFrom, dateUntil) > 0) {
                  var temp = f.fromValue;
                  f.fromValue = f.untilValue;
                  f.untilValue = temp;
                  dateFrom = new Date(f.fromValue);
                  dateUntil = new Date(f.untilValue);
                }

                dateFrom.setHours(0, 0, 0);
                dateUntil = dojo.date.add(dateUntil, "day", 1);
                dateUntil.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(dateFrom) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(dateUntil) + "')";
              }
              break;
            case "simpleDate" :
              var value = f.value;
              date = new Date();
              if (value == "UntilNow") {
                whereClause =
                  whereClause + " " + f.property + " <= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              else if (value == "Yesterday") {
                var date0 = dojo.date.add(date, "day", -1);
                date0.setHours(0, 0, 0);
                date24 = date;
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date0) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "Today") {
                date.setHours(0, 0, 0);
                date24 = dojo.date.add(date, "day", 1);
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "Tomorrow") {
                date0 = dojo.date.add(date, "day", +1);
                date0.setHours(0, 0, 0);
                date24 = dojo.date.add(date0, "day", +1);
                date24.setHours(0, 0, 0);
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date0) + "') AND " + f.property +
                    " < TS('" + com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date24) + "')";
              }
              else if (value == "FromNow") {
                whereClause =
                  whereClause + " " + f.property + " >= TS('" +
                    com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date) + "')";
              }
              break;
            case "boolean" :
              whereClause = whereClause + " " + f.property + " = " + f.value;
              break;
            case "number" :
            case "float" :
            case "numericPriority" :
              var operatorMap = {
                equal: "=",
                less: "<",
                lessEqual: "<=",
                greater: ">",
                greaterEqual: ">="
              };
              whereClause = whereClause + " " + f.property + " " + operatorMap[f.operator] + " " + f.value;
              break;
            case "priority" :
              var prios = this._prioritiesConfig;
              var high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
              var low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
              if (f.value == "High") {
                if (high < low) {
                  whereClause = whereClause + " " + f.property + " <= " + high;
                }
                else {
                  whereClause = whereClause + " " + f.property + " >= " + high;
                }
              }
              else if (f.value == "Medium") {
                if (high < low) {
                  whereClause = whereClause + " (" + f.property + " > " + high + " AND " + f.property + " < " + low + ") ";
                }
                else {
                  whereClause = whereClause + " (" + f.property + " > " + low + " AND " + f.property + " < " + high + ") ";
                }
              }
              else {
                if (high < low) {
                  whereClause = whereClause + " " + f.property + " >= " + low;
                }
                else {
                  whereClause = whereClause + " " + f.property + " <= " + low;
                }
              }
              break;
            case "defaultPriority" :
              var range = com.ibm.bpm.social.widgets.task.util.Constants.propertyValueMapper(f.value);
              if (range) {
                if (!range.low && range.high) {
                  whereClause = whereClause + " " + f.property + " < " + range.high;
                }
                else if (range.low && range.high) {
                  whereClause =
                    whereClause + " (" + f.property + " < " + range.high + " AND " + f.property + " >= " + range.low + ") ";
                }
                else if (range.low && !range.high) {
                  whereClause = whereClause + " " + f.property + " >= " + range.low;
                }
              }
              break;
            case "TASK.STATE" :
              if (f.value == "STATE_RUNNING") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "STATE_RUNNING" + " OR " + f.property + "=" + "STATE_CLAIMED" +
                    ") ";
              }
              else if (f.value == "STATE_FAILED") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "STATE_FAILED" + " OR " + f.property + "=" + "STATE_EXPIRED" +
                    ") ";
              }
              else {
                whereClause = whereClause + " " + f.property + "=" + f.value;
              }
              break;
            case "PROCESS_INSTANCE.STATE" :
              // Unsupported values in WLE and mixed ["STATE_TERMINATING", "STATE_COMPENSATED", "STATE_COMPENSATING", "STATE_FAILING", "STATE_COMPENSATION_FAILED"]
              var isWpsOnly = false;
              if (this._systems) {
                isWpsOnly = this._systems.isWpsOnly(this._queue.domain);
              }
              if (f.value == "STATE_FAILED") {
                whereClause += " (";
                whereClause += f.property + "=" + "STATE_FAILED";
                if (isWpsOnly) {
                  whereClause += " OR " + f.property + "=" + "STATE_FAILING";
                }
                whereClause += " OR " + f.property + "=" + "STATE_TERMINATED";
                if (isWpsOnly) {
                  whereClause += " OR " + f.property + "=" + "STATE_TERMINATING";
                }
                whereClause += ") ";
              }
              else if (f.value == "STATE_COMPENSATING") {
                if (isWpsOnly) {
                  whereClause =
                    whereClause + " (" + f.property + "=" + "STATE_COMPENSATING" + " OR " + f.property + "=" +
                      "STATE_COMPENSATED" + ") ";
                }
              }
              else {
                if (isWpsOnly || f.value != "STATE_COMPENSATION_FAILED") {
                  whereClause = whereClause + " " + f.property + "=" + f.value;
                }
              }
              break;
            case "ESCALATION.ACTIVATION_STATE" :
              if (f.value == "ACTIVATION_STATE_RUNNING") {
                whereClause =
                  whereClause + " (" + f.property + "=" + "ACTIVATION_STATE_RUNNING" + " OR " + f.property + "=" +
                    "ACTIVATION_STATE_CLAIMED" + ") ";
              }
              else {
                whereClause = whereClause + " " + f.property + "=" + f.value;
              }
              break;
            case "usergroup" :
              var input = com.ibm.bpm.social.widgets.task.util.PropertyFormatter.formatTextForSQL(f.value);
              whereClause =
                whereClause + " (" + f.property + " LIKE 'cn=%" + input + "%,o=%') OR (" + f.property +
                  " NOT LIKE 'cn=%' AND " + f.property + " LIKE '%" + input + "%')";
              break;
            default:
              whereClause = whereClause + " " + f.property + "=" + f.value;
              break;
          }
        }
        whereClause = whereClause + ")";
      }
    }
    com.ibm.bpm.social.util.common.Log.log(2,
      "com.ibm.bpm.social.widgets.task.util.EntityQuery._createFilterWhereClause whereclause :" + whereClause);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return whereClause;
  },

  _replaceWhereParameters: function _replaceWhereParameters_EntityQuery(whereClause, propertiesMap){
    // replace date values if necessary
    if (!whereClause) {
      return "";
    }
    var todayDate = new Date();
    var date;

    var today = null;
    if (whereClause.indexOf("{today}") >= 0) {
      today = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(todayDate);
      propertiesMap["today"] = today;
    }
    // create the iso date string for yesterday date
    // yyyy-mm-ddT23:59:59+GMThh:mm
    var yesterday = null;
    if (whereClause.indexOf("{yesterday}") >= 0) {
      date = dojo.date.add(todayDate, "day", -1);
      date.setHours(23, 59, 59);
      yesterday = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date);
      propertiesMap["yesterday"] = yesterday;
    }
    // create the iso date string for tomorrows date
    // yyyy-mm-ddT00:00:00+GMThh:mm
    var tomorrow = null;
    if (whereClause.indexOf("{tomorrow}") >= 0) {
      date = dojo.date.add(todayDate, "day", 1);
      date.setHours(0, 0, 0);
      tomorrow = com.ibm.bpm.social.widgets.task.util.Constants.getGMTISOString(date);
      propertiesMap["tomorrow"] = tomorrow;
    }
    // get the high prio value
    var high = null;
    var highCompare = null;
    var low = null;
    if (whereClause.indexOf("{high}") >= 0) {
      var prios = this._prioritiesConfig;
      highCompare = "<=";
      high = 0;
      if (prios) {
        // get the settings for high priority
        high = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOHIGH_CONFIG];
        low = prios[com.ibm.bpm.social.widgets.task.util.Constants.PRIOLOW_CONFIG];
        // specify the compare sign depending on the high and low range
        if (typeof high == "number" && typeof low == "number") {
          if (high > low) {
            // all values greate equal high have high priority
            highCompare = ">=";
          }
        }
        propertiesMap["high"] = high;
        propertiesMap["highCompare"] = highCompare;
      }
    }
    var user = null;
    if (whereClause.indexOf("{user}") >= 0) {
      // Escape the userId to support user id's with special characters
      user = window.encodeURIComponent(this._userId);
      propertiesMap["user"] = user;
    }
    // insert the variables in the where clause
    //        whereClause = dojo.string.substitute(whereClause, {
    //            today: today,
    //            tomorrow: tomorrow,
    //            yesterday: yesterday,
    //            user: user,
    //            high: high,
    //            highCompare: highCompare
    //        });

    whereClause = dojo.string.substitute(whereClause, propertiesMap);
    return whereClause;
  }
});