// ModifyTaskHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler");

dojo.require("dijit.Dialog");
dojo.require("dijit.form.FilteringSelect");

dojo.require("com.ibm.bpm.social.portal.Topics");

dojo.require("com.ibm.bpm.social.widgets.task.util.Constants");
dojo.require("com.ibm.bpm.social.widgets.task.util.DateTime");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.UserChooser");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.models.BPMModel");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.Util");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("com.ibm.bspace.common.util.ui.RadioButton");
dojo.require("com.ibm.bspace.common.util.ui.Button");
dojo.require("com.ibm.bspace.common.util.ui.CheckBox");
dojo.require("com.ibm.bspace.common.util.ui.Helper");
dojo.require("com.ibm.bspace.common.util.ui.FilteringSelect");

dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler",
    null,
    {

      constructor: function(parameters){
        this._connects = [];
        this._wrappedElements = [];
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
        this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
        this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();

        this.init(parameters);
      },

      init: function init_ModifyTaskHandler(parameters){

        var task = parameters.task;

        if (typeof task.TKIID === "string") {
          this.mapping = com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.queryMapping;
        }
        else {
          this.mapping = com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.itemMapping;
        }
        this._dueDateTime = task[this.mapping["DUE"]];
        this._priority = task[this.mapping["PRIORITY"]];
        this._availableActions = parameters.availableActions;
        this._tkiid = task[this.mapping["TKIID"]];

        if (parameters.taskDisplayName) {
          this._taskDisplayName = parameters.taskDisplayName;
        }
        else {
          this._taskDisplayName = this._tkiid;
        }

        if (this._availableActions) {
          this.ifReassignBackToGroupOK = dojo.indexOf(this._availableActions, "ACTION_CANCELCLAIM") != -1;
          this.ifTransferToUserOK = dojo.indexOf(this._availableActions, "ACTION_REASSIGNTOUSER") != -1;
          this.ifTransferToGroupOK = dojo.indexOf(this._availableActions, "ACTION_REASSIGNTOGROUP") != -1;
          this.ifModifyPriorityOK = dojo.indexOf(this._availableActions, "ACTION_UPDATEPRIORITY") != -1;
          this.ifModifyDueDateOK = dojo.indexOf(this._availableActions, "ACTION_UPDATEDUEDATE") != -1;
        }
        else {
          this.ifReassignBackToGroupOK = true;
          this.ifTransferToUserOK = true;
          this.ifTransferToGroupOK = true;
          this.ifModifyPriorityOK = true;
          this.ifModifyDueDateOK = true;
        }

        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        this.ifOverallModifyOK =
          this.ifReassignBackToGroupOK || this.ifTransferToUserOK || this.ifModifyPriorityOK ||
            this.ifModifyDueDateOK;

        this.reassignBackToGroupSwitch = null;

        this._destroyUserChooser();
        this._currentOwner = null;
        if (task[this.mapping["OWNER"]]) {
          this._currentOwner = {
            value: task[this.mapping["OWNER"]],
            type: "user"
          };
        }
        else if (task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]]) {
          this._currentOwner = {
            value: task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]],
            type: "group"
          };
        }
        else if (task[this.mapping["ASSIGNED_TO_ROLE"]]) {
          // From Process Portal: determine group display name
          // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
          // So we are just using the last two '_'s to pull the PG name
          var assignedToRole = task[this.mapping["ASSIGNED_TO_ROLE"]];
          if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
            var roleName = assignedToRole;
            var displayName = roleName.substring(0, roleName.lastIndexOf("_"));
            displayName = displayName.substring(0, displayName.lastIndexOf("_"));
            assignedToRole = displayName;
          }
          this._currentOwner = {
            value: assignedToRole,
            type: "group"
          };
        }
        else {
          var error = new Error("Task does not have an owner or a group. ID: " + task[this.mapping["TKIID"]]);
          com.ibm.bpm.social.util.common.Log.consoleError(error.message);
          throw error;
        }
      },

      /**
       * a value in a field has changed, check validation to enable/disable Save button
       */
      _onValueChanged: function _onValueChanged_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var valid = true;

        if (this.ifModifyPriorityOK && this._priorityFilteringSelect) {
          // !! because isValid() returns false or selected option ?!? (Dojo 1.6.1)
          valid = valid && !!this._priorityFilteringSelect.isValid();
        }
        if (this.ifModifyDueDateOK && this.dateTimePicker) {
          valid = valid && this.dateTimePicker.isValid();
        }
        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        if (this.ifTransferToUserOK && this.userChooser &&
          !this.userChooser.chooserInput.disabled) {
          valid =
            valid &&
              (this.userChooser.chooserInput.value.length === 0 || this.userChooser.getChoice() !== null || dojo.hasClass(
                this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help"));
        }

        com.ibm.bspace.common.util.ui.Button.setDisabled(this._saveButton, !valid);

        com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
      },

      destroy: function destroy_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this._destroyDialog();
        this._restHandler = null;
        this._errorHandler = null;
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Destroys the controls in the dialog and the dialog itself.
       */
      _destroyDialog: function _destroyDialog_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        while (this._connects && this._connects.length > 1) {
          dojo.disconnect(this._connects.pop());
        }

        if (this._wrappedElements.length > 0) {
          com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements);
          this._wrappedElements = [];
        }

        if (this._priorityFilteringSelect) {
          this._priorityFilteringSelect.destroy();
          this._priorityFilteringSelect = null;
        }

        this._destroyUserChooser();

        if (this.dateTimePicker) {
          this.dateTimePicker.destroy();
          this.dateTimePicker = null;
        }

        if (this.dialog) {
          this.dialog.destroyRecursive();
          this.dialog = null;
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _destroyUserChooser: function _destroyUserChooser_ModifyTaskHandler(){
        if (this.userChooser) {
          this.userChooser.destroy();
          this.userChooser = null;
        }
      },

      /**
       * Assigns the task back to the group, this is the public method that does fire the onTaskModified event
       */
      reassignBackToGroup: function reassignBackToGroup_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        var success = this._reassignBackToGroup();
        if (success) {
          this.onTaskModified(null, null, true);
        }

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Assigns the task back to the group, this is the internal method that does not fire the onTaskModified
       * event
       * 
       * @returns {Boolean} true if the task was successfully reassigned
       */
      _reassignBackToGroup: function reassignBackToGroup_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.cancelTask(this._tkiid, function(){
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            403: {
              key: "KEY_TaskModifyCancelNotAvailable",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            409: {
              key: "KEY_TaskModifyCancelNotAvailable",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            404: {
              key: "KEY_TaskModifyCancelNotExisting",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          myself._errorHandler.showError(data, ioArgs, statusCodeMap, null, function(){
            myself.onError();
          });
        });

        com.ibm.bpm.social.util.common.Log.logExit(2, "result=" + success);

        return success;
      },

      /**
       * Assign task to a specific group
       * 
       * @param group {void} group name to assign the task to
       */
      _assignTaskToGroup: function _assignTaskToGroup_ModifyTaskHandler(group){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.assignTaskToGroup(this._tkiid, function(response){
          myself._currentOwner = {
            value: group,
            type: "group"
          };
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            400: {
              key: "KEY_transferGroupDoesNotExist",
              parameters: {
                1: group
              }
            },
            404: {
              key: "KEY_TransferTask"
            }
          };
          var errorCodeMap = {
            "CWTBG0065E": {
              key: "KEY_transferGroupDoesNotExist"
            },
            "CWTBG0048E": {
              key: "KEY_delegateNotExistingTask"
            }
          };

          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        }, group);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return success;
      },

      /**
       * Assign the task to a specific user
       * 
       * @param user {void} user id to the assign the task to
       */
      _assignTaskToUser: function _assignTaskToUser_ModifyTaskHandler(user){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var myself = this;
        var success = false;
        this._restHandler.assignTaskToUser(this._tkiid, function(response){
          myself._currentOwner = {
            value: user,
            type: "user"
          };
          success = true;
        }, function(data, ioArgs){
          var statusCodeMap = {
            400: {
              key: "KEY_TransferUserDoesNotExist",
              parameters: {
                1: user
              }
            },
            404: {
              key: "KEY_delegateNotExistingTask"
            }
          };
          var errorCodeMap = {
            "CWTBG0065E": {
              key: "KEY_TransferUserDoesNotExist",
              parameters: {
                1: user
              }
            },
            "CWTKA0049E": {
              key: "KEY_delegationNotSupported"
            },
            "CWTKA0110E": {
              key: "KEY_delegationForUnauthUser"
            }
          };

          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        }, user);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return success;
      },

      _updateTaskProperties: function _updateTaskProperties_ModifyTaskHandler(newPriority, newDueDateTime){
        var myself = this;
        var hasError = false;

        var errorFunction = function(data, ioArgs){
          hasError = true;

          var statusCodeMap = {
            400: {
              key: "KEY_UpdateTaskError",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            401: {
              key: "KEY_UpdateTaskNotAuthorized",
              parameters: {
                1: myself._taskDisplayName
              }
            },
            404: {
              key: "KEY_UpdateNotExistingTask",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          var errorCodeMap = {
            "CWTBG0073E": {
              key: "KEY_UpdateTaskError",
              parameters: {
                1: myself._taskDisplayName
              }
            }
          };
          myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap, function(){
            myself.onError();
          });
        };

        // the API does not allow to update both with one API call

        // update due date
        if (newDueDateTime) {
          this._restHandler.updateTaskProperties(this._tkiid, function(response){
            // TODO I do not understand why we need that. The Class will be destroyed afterwards
            if (newDueDateTime) {
              com.ibm.bpm.social.util.common.Log.log(1, "newDueDateTime: " + newDueDateTime, "ModifyTaskHandler",
                "_updateTaskProperties");
              myself._dueDateTime = newDueDateTime;

            }
          }, errorFunction, null, newDueDateTime);
        }

        if (!hasError && newPriority) {
          this._restHandler.updateTaskProperties(this._tkiid, function(response){
            // TODO I do not understand why we need that. The Class will be destroyed afterwards
            if (newPriority) {
              com.ibm.bpm.social.util.common.Log.log(1, "newPriority: " + newPriority, "ModifyTaskHandler",
                "_updateTaskProperties");
              myself._priority = newPriority;
            }
          }, errorFunction, newPriority, null);
        }
        return !hasError;
      },

      showDialog: function showDialog_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        // Make sure that the old dialog was destroyed
        this._destroyDialog();

        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DIALOG, ["modifyTask"]);

        this.dialog = new com.ibm.bspace.common.util.ui.Dialog({
          templateString: '<div dojoAttachPoint="content" ></div>',
          title: this._nlsResources["task_dialog_modifyTask_title"]
        });

        var hideHandle = dojo.connect(this.dialog.popup, "onHide", dojo.hitch(this, function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.HIDE_DIALOG, ["modifyTask"]);
          dojo.disconnect(hideHandle);
          this.onHide(this);
        }));

        dojo.addClass(this.dialog.content, "bpm-task-modify-task-dialog");

        var content = this.dialog.content;

        // Description
        var dSection = this._createDescriptionSection(content);

        var formSections = document.createElement("div");
        content.appendChild(formSections);
        dijit.setWaiRole(formSections, "form");
        dijit.setWaiState(formSections, "labelledby", dSection.id);

        // Assignee section
        this._createAssigneeSection(formSections);

        // Priority section
        this._createPrioritySection(formSections);

        // Due date and time section
        this._createDueDateTimeSection(formSections);

        // Create save button
        var saveButton = document.createElement("button");
        saveButton.innerHTML = this._nlsResources["task_dialog_modifyTask_saveButton"]; // Save";
        saveButton.title = this._nlsResources["task_dialog_modifyTask_saveButtonTooltip"];
        if (document.body.dir == 'rtl') {
          dojo.style(saveButton, "float", "left");
        }
        else {
          dojo.style(saveButton, "float", "right");
        }

        saveButton.onclick = dojo.hitch(this, this.onSave);
        var buttonSection = document.createElement("div");
        dojo.style(buttonSection, "background", "white");
        buttonSection.appendChild(saveButton);
        this._saveButton = com.ibm.bspace.common.util.ui.Button.wrapElement(saveButton);
        this._wrappedElements.push(this._saveButton);
        formSections.appendChild(buttonSection);

        // check if the dialog defaults are valid. Else we must disable the save button
        this._onValueChanged();

        this.dialog.show();

        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      _createDescriptionSection: function _createDescriptionSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var descriptionSection = document.createElement("div");
        this._styleSection(descriptionSection);
        dojo.style(descriptionSection, "maxWidth", "35em");
        if (this._taskDisplayName === this._tkiid) {
          descriptionSection.innerHTML = this._nlsResources["task_dialog_modifyTask_description"];
        }
        else {
          descriptionSection.innerHTML =
            dojo.string.substitute(this._nlsResources["task_dialog_modifyTask_description_ext"], {
              1: "<b>" + this._taskDisplayName + "</b>"
            });
        }
        descriptionSection.title = " ";
        descriptionSection.id = this.dialog.id + "_description";
        dijit.setWaiRole(descriptionSection, "heading");

        parentNode.appendChild(descriptionSection);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return descriptionSection;
      },

      /*
       * Returns null, if assignee hasn't changed.
       */
      _getNewAssignee: function _getNewAssignee_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.userChooser) {
          var choice = this.userChooser.getChoice();
          if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
            return choice.userName;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createAssigneeSection: function _createAssigneeSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var self = this;

        // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
        // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
        // support the assignment to a specific group but only the reassignment to the last group. Therefore 
        // do not use this.ifTransferToGroupOK.
        if (this.ifReassignBackToGroupOK || this.ifTransferToUserOK) {

          var reassignBackToGroup, reassignLabel;

          var assigneeSection = document.createElement("div");
          this._styleSection(assigneeSection);

          var assigneeLabel = document.createElement("div");
          assigneeLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_assigneeSectionTitle"]; // "Task Assignment";
          assigneeLabel.title = this._nlsResources["task_dialog_modifyTask_assigneeSectionTooltip"];
          assigneeLabel.id = this.dialog.id + "_assigneeLabel";
          this._styleSectionTitle(assigneeLabel);
          assigneeSection.appendChild(assigneeLabel);

          // APAR JR45837: assign back to group is only handled via ACTION_CANCELCLAIM not ACTION_REASSIGNTOGROUP.
          // If only ACTION_REASSIGNTOGROUP would be true then the user chooser would open empty as we do not 
          // support the assignment to a specific group but only the reassignment to the last group. Therefore 
          // do not use this.ifTransferToGroupOK.
          if (this.ifReassignBackToGroupOK && !this.ifTransferToUserOK) {
            // Create checkbox pattern: Just one Checkbox "Reassign back to group"
            // -> No UserChooser 

            reassignBackToGroup = document.createElement("input");
            reassignBackToGroup.name = "ModifyTaskHandler_Assignee";
            reassignBackToGroup.id = this.dialog.id + "_checkbox1";
            reassignBackToGroup.type = "checkbox";
            reassignBackToGroup.tabIndex = "0";
            reassignBackToGroup.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
            assigneeSection.appendChild(reassignBackToGroup);
            this._wrappedElements.push(com.ibm.bspace.common.util.ui.CheckBox.wrapElement(reassignBackToGroup));

            reassignLabel = document.createElement("label");
            reassignLabel.id = this.dialog.id + "_checkbox1_label";
            reassignLabel.htmlFor = reassignBackToGroup.nextSibling.id;
            reassignLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_reassignBackToGroup"];
            reassignLabel.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
            assigneeSection.appendChild(reassignLabel);
            this.reassignBackToGroupSwitch = reassignBackToGroup;
            dijit.setWaiState(reassignBackToGroup, "labelledby", reassignLabel.id);
          }
          else {

            // transfer is  allowed, create a user chooser
            var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
              // includeGroups: this.ifTransferToGroupOK,
              includeGroups: false,
              includeUsers: this.ifTransferToUserOK
            });

            // Create UserChooser 
            self._ucClickedFirstTime = false;
            var args = {
              searchMinInputLength: 0,
              ucModel: ucModel,
              _onUCFocus: function(e){
                if (!self._ucClickedFirstTime) {
                  self._ucClickedFirstTime = true;
                  dojo.removeClass(self.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
                  self.userChooser.chooserInput.value = "";
                }
                com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);
              },
              _onUCBlur: function(e){
                var valid = (self.userChooser.chooserInput.value.length < 0 || self.userChooser.getChoice() !== null);
                if (!valid) {
                  self.userChooser.chooserInput.value = "";
                  self.userChooser._hideResults();
                }
                dojo.addClass(self.userChooser.noUsersFoundDiv, "hidden");
              },
              // defect 103438 
              // performance problem. The user list is loaded twice. 
              // The list will be loaded with the first click.
              preloadInitialSuggestions: false
            };
            this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
            this.userChooser.onChangeHandler = dojo.hitch(this, this._onValueChanged);

            dijit.setWaiState(self.userChooser.chooserInput, "label",
              this._nlsResources["task_dialog_modifyTask_assigneeSectionTitle"]);

            this.userChooser.chooserInput.title = this._nlsResources["task_dialog_modifyTask_assignToTooltip"];
            this.userChooser.chooserInput.tabIndex = "0";
            dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");

            // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
            this.userChooser.setAssignTaskidFilter(this._tkiid);

            if (this.ifReassignBackToGroupOK) {
              // need option pattern: 2 Radio buttons, one for the UserChooser and one for assign_back_to_group
              var table = document.createElement("table");
              dijit.setWaiRole(table, "presentation");
              dijit.setWaiState(table, "describedby", assigneeLabel.id);
              dojo.style(table, "borderCollapse", "collapse");

              var row = table.insertRow(0);
              dijit.setWaiRole(row, "radiogroup");

              // First radio button
              var td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");

              var transferTo = document.createElement("input");
              transferTo.name = "ModifyTaskHandler_Assignee";
              transferTo.type = "radio";
              transferTo.id = this.dialog.id + "_radio1";

              // Enable userChooser when radio button was clicked
              this._connects.push(dojo.connect(transferTo, "onclick", function(){
                if (transferTo.checked) {
                  self.userChooser.chooserInput.disabled = false;
                  dijit.setWaiState(self.userChooser.chooserInput, "disabled", "false");
                }
                self._onValueChanged();
              }));

              td.appendChild(transferTo);
              this._wrappedElements.push(com.ibm.bspace.common.util.ui.RadioButton.wrapElement(transferTo));
              com.ibm.bspace.common.util.ui.RadioButton.setChecked(transferTo, true);
              dijit.setWaiRole(transferTo.nextSibling, "radio");
              dijit.setWaiState(transferTo.nextSibling, "label",
                this._nlsResources["task_dialog_modifyTask_assignUser-label"]);
              dijit.setWaiState(transferTo.nextSibling, "controls", this.userChooser.chooserInput.id);
              transferTo.nextSibling.tabIndex = "0";


//              var radio1Label = document.createElement("label");
//              radio1Label.htmlFor = transferTo.nextSibling.id;
//              radio1Label.innerHTML = this._nlsResources["task_dialog_modifyTask_assignUser-label"];
//              radio1Label.style.display = "none";
//              td.appendChild(radio1Label);

              // User Chooser
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              dijit.setWaiState(self.userChooser.chooserInput, "label",
                this._nlsResources["task_dialog_modifyTask_assignUser-label"]);

              td.appendChild(this.userChooser.domNode);

//              var ucLabel = document.createElement("label");
//              ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
//              ucLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_User-label"];
//              ucLabel.style.display = "none";
//              td.appendChild(ucLabel);

              reassignBackToGroup = document.createElement("input");
              reassignBackToGroup.name = "ModifyTaskHandler_Assignee";
              reassignBackToGroup.type = "radio";
              reassignBackToGroup.id = this.dialog.id + "_reassignBackToGroup";
              reassignBackToGroup.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
              dijit.setWaiRole(transferTo, "radio");

              // Disable userChooser when radio button was clicked
              this._connects.push(dojo.connect(reassignBackToGroup, "onclick", function(){
                if (reassignBackToGroup.checked) {
                  self.userChooser.chooserInput.disabled = true;
                  dijit.setWaiState(self.userChooser.chooserInput, "disabled", "true");
                }
                self._onValueChanged();
              }));

              // Radio button 2
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              td.appendChild(reassignBackToGroup);
              this._wrappedElements.push(com.ibm.bspace.common.util.ui.RadioButton.wrapElement(reassignBackToGroup));
              reassignBackToGroup.nextSibling.tabIndex = "0";
              this.reassignBackToGroupSwitch = reassignBackToGroup;

              // Label for Radio 2
              td = row.insertCell(-1);
              dijit.setWaiRole(td, "presentation");
              reassignLabel = document.createElement("label");
              reassignLabel.id = this.dialog.id + "_checkbox2_label";
              reassignLabel.htmlFor = reassignBackToGroup.nextSibling.id;
              reassignLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_reassignBackToGroup"];
              reassignLabel.title = this._nlsResources["task_dialog_modifyTask_reassignBackToGroupTooltip"];
              td.appendChild(reassignLabel);
              dijit.setWaiState(reassignBackToGroup, "labelledby", reassignLabel.id);

              assigneeSection.appendChild(table);
            }
            else {
              assigneeSection.appendChild(this.userChooser.domNode);
            }
          }

          parentNode.appendChild(assigneeSection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Returns null, if priority hasn't changed.
       */
      _getNewPriority: function _getNewPriority_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.prioritySelection) {
          var selectedPriority =
            (com.ibm.bpm.social.util.common.Util.isIPad ? this.prioritySelection.value : this._priorityFilteringSelect
              .get("value"));
          if (com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(selectedPriority) !== com.ibm.bpm.social.widgets.task.util.Constants
            .propertyKeyMapper(this._priority)) {
            return selectedPriority;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createPrioritySection: function _createPrioritySection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.ifModifyPriorityOK) {
          var prioritySection = document.createElement("div");
          prioritySection.title = this._nlsResources["task_dialog_modifyTask_prioritySectionTooltip"];
          this._styleSection(prioritySection);

          var currentPriority = com.ibm.bpm.social.widgets.task.util.Constants.propertyKeyMapper(this._priority);

          var priorityLabel = document.createElement("div");
          priorityLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_prioritySectionTitle"];
          this._styleSectionTitle(priorityLabel);
          priorityLabel.id = this.dialog.id + "_priority";
          prioritySection.appendChild(priorityLabel);

          this.prioritySelection = document.createElement("select");

          var veryLow = document.createElement("option");
          veryLow.value = "50";
          veryLow.innerHTML = this._nlsResources["KEY_PrioVeryLow"]; // "Very Low";
          if (currentPriority === "VeryLow") {
            veryLow.selected = "selected";
          }
          this.prioritySelection.appendChild(veryLow);

          var low = document.createElement("option");
          low.value = "40";
          low.innerHTML = this._nlsResources["KEY_PrioLow"]; // "Low";
          if (currentPriority === "Low") {
            low.selected = "selected";
          }
          this.prioritySelection.appendChild(low);

          var medium = document.createElement("option");
          medium.value = "30";
          medium.innerHTML = this._nlsResources["KEY_PrioMedium"]; // "Medium";
          if (currentPriority === "Medium") {
            medium.selected = "selected";
          }
          this.prioritySelection.appendChild(medium);

          var high = document.createElement("option");
          high.value = "20";
          high.innerHTML = this._nlsResources["KEY_PrioHigh"]; // "High";
          if (currentPriority === "High") {
            high.selected = "selected";
          }
          this.prioritySelection.appendChild(high);

          var veryHigh = document.createElement("option");
          veryHigh.value = "10";
          veryHigh.innerHTML = this._nlsResources["KEY_PrioVeryHigh"]; // "Very High";
          if (currentPriority === "VeryHigh") {
            veryHigh.selected = "selected";
          }
          this.prioritySelection.appendChild(veryHigh);

          if (com.ibm.bpm.social.util.common.Util.isIPad) {
            // on iPad use the native control
            prioritySection.appendChild(this.prioritySelection);
            this._connects.push(dojo.connect(this.prioritySelection, "onchange", this, this._onValueChanged));
          }
          else {
            this._priorityFilteringSelect = new com.ibm.bspace.common.util.ui.FilteringSelect({
              "intermediateChanges": true
            }, this.prioritySelection);
            this._connects.push(dojo.connect(this._priorityFilteringSelect, "onChange", this, this._onValueChanged));
            prioritySection.appendChild(this._priorityFilteringSelect.domNode);
            this._priorityFilteringSelect.textbox.title =
              this._nlsResources["task_dialog_modifyTask_prioritySectionTooltip"];
            dijit.setWaiState(this._priorityFilteringSelect.textbox, "label",
              this._nlsResources["task_dialog_modifyTask_prioritySectionTitle"]);
          }

          parentNode.appendChild(prioritySection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /*
       * Returns null, if dueDateTime hasn't changed.
       */
      _getNewDueDateTime: function _getNewDueDateTime_ModifyTaskHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.dateTimePicker && this.dateTimePicker.isValid()) {
          // Ignore seconds, because you cannot define seconds via dateTimePicker.
          var differenceInSeconds =
            dojo.date.difference(this.dateTimePicker.getDateTime(), dojo.date.stamp.fromISOString(this._dueDateTime),
              "second");
          if (Math.abs(differenceInSeconds) >= 60) {
            var dateString = dojo.date.stamp.toISOString(this.dateTimePicker.getDateTime(), {
              zulu: true
            });
            return dateString;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return null;
      },

      _createDueDateTimeSection: function _createDueDateTimeSection_ModifyTaskHandler(parentNode){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.ifModifyDueDateOK) {
          var dueDateSection = document.createElement("div");
          dueDateSection.title = this._nlsResources["task_dialog_modifyTask_DueSectionTooltip"]; // "Due Date";
          this._styleSection(dueDateSection);

          var dueDateLabel = document.createElement("div");
          dueDateLabel.innerHTML = this._nlsResources["task_dialog_modifyTask_DueSectionTitle"]; // "Due Date";
          this._styleSectionTitle(dueDateLabel);

          dueDateSection.appendChild(dueDateLabel);

          this.dateTimePicker = new com.ibm.bpm.social.widgets.task.util.DateTime({
            dateTime: dojo.date.stamp.fromISOString(this._dueDateTime),
            showNoLabels: true
          });

          this._connects.push(dojo.connect(this.dateTimePicker, "onChange", this, this._onValueChanged));

          dueDateSection.appendChild(this.dateTimePicker.domNode);

          parentNode.appendChild(dueDateSection);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      onTaskModified: function onTaskModified_ModifyTaskHandler(dueDateTime, priority, assigneeChanged, newAssignee){
        com.ibm.bpm.social.util.common.Log.log(1, "Event onTaskModified fired: dueDateTime=" + dueDateTime + ", priority=" +
          priority + ", assigneeChanged=" + assigneeChanged + ", newAssignee=" + newAssignee);
      },

      /**
       * 
       * @type {void}
       */
      onSave: function onSave_ModifyTaskHandler(e){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var newDueDateTime = this._getNewDueDateTime();
        var newPriority = this._getNewPriority();
        var newAssignee = this._getNewAssignee();

        var updatePropertiesSuccess = false;
        var updateAssigneeSuccess = false;

        // assignee change needs to be handled first to avoid adding a task to the list which does no longer apply to the list
        if (this.reassignBackToGroupSwitch && this.reassignBackToGroupSwitch.checked) {
          com.ibm.bpm.social.util.common.Log.log(1, "assigning back to group");
          updateAssigneeSuccess = this._reassignBackToGroup();
        }
        else {
          if (newAssignee) {
            com.ibm.bpm.social.util.common.Log.log(1, "newAssignee: " + newAssignee);
            updateAssigneeSuccess = this._assignTaskToUser(newAssignee);
            if (!updateAssigneeSuccess) {
              newAssignee = null;
            }
          }
        }

        if (newDueDateTime || newPriority) {
          updatePropertiesSuccess = this._updateTaskProperties(newPriority, newDueDateTime);
          if (!updatePropertiesSuccess) {
            newDueDateTime = null;
            newPriority = null;
          }
        }

        if (updatePropertiesSuccess || updateAssigneeSuccess) {
          this.onTaskModified(newDueDateTime, newPriority, updateAssigneeSuccess, newAssignee);
        }

        this.dialog.hide();

        com.ibm.bpm.social.util.common.Log.logExit(1);
        return;
      },

      _styleSection: function _styleSection_ModifyTaskHandler(node){
        dojo.style(node, "padding", "10px 0");
        dojo.style(node, "background", "white");
      },

      _styleSectionTitle: function _styleSectionTitle_ModifyTaskHandler(node){
        dojo.style(node, "fontWeight", "bold");
        dojo.style(node, "padding", "0 0 10px 0");
      },

      onError: function onError_ModifyTaskHandler(){},

      onHide: function onHide_ModifyTaskHandler(modifyTaskHandler){}

    });


com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.queryMapping = {
  "DUE": "DUE",
  "PRIORITY": "PRIORITY",
  "TKIID": "TKIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler.itemMapping = {
  "DUE": "dueTime",
  "PRIORITY": "priority",
  "TKIID": "tkiid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};
