// QueueLister.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

dojo.provide("com.ibm.bpm.social.widgets.task.util.QueueLister");

dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.common.QueryRestHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.Queue");
dojo.require("com.ibm.bpm.social.widgets.task.util.Constants");


/**
 * Virtual base class for listing queues <br>
 * Children must at least set the type to e.g. com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE
 */

dojo.declare("com.ibm.bpm.social.widgets.task.util.QueueLister", null, {

  _queues: null,

  _restHandler: null,

  type: null,

  queuesToIgnore: {
    "portal.savedsearch.inbox": true,
    "portal.savedsearch.help_reqs": true,
    "portal.savedsearch.history": true,
    "IBM.PI_PROCESSLIST_ALL": true,
    "IBM.DEFAULTALLPROCESSLIST_75": true,
    "portal.savedsearch.alerts": true,
    "IBM.DEFAULTALLTASKSLIST_75": true,
    "IBM.PI_TASKLIST_ALL": true
  },


  constructor: function constructor_QueueLister(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);

    if (!this.type) {
      com.ibm.bpm.social.util.common.Log.logError(1, "Error: QueueLister requires type set!");
    }

    // TODO find a place for creation with params, e.g. endpoint
    this._restHandler = com.ibm.bpm.social.util.common.QueryRestHandler.getInstance();

    this._queues = this._getAllQueues();

    com.ibm.bpm.social.util.common.Log.logExit(2);
    return;
  },

  /**
   * 
   * @param name name of the queue
   * @returns {object} queue object with queue meta data from all query table call
   */

  getQueue: function getQueue_QueueLister(name){
    var queue = null;
    if (!this._queues) {
      this._queues = this._getAllQueues();
    }
    var filtered = dojo.filter(this._queues, function(item){
      return item.name === name;
    });
    if (filtered.length === 1) {
      queue = filtered[0];
    }
    return queue;
  },

//  /**
//   * 
//   * @param queue queue object
//   * @returns {object} queue object where the attributes are set
//   */
//  setQueueAttributes: function setQueueAttributes(queue){
//    if (queue) {
//      // If not already done, load attributes for Queue, 
//      // and pass them into the queue (invoking setAttributes as callback)
//      if (queue.attributesLoaded !== true) {
//        this._restHandler.getAttributesForQueryTable(this.type, queue.getPlainQueueName(), queue.processAppName, function(data){
//          queue.setAttributes(data);
//        }, dojo.hitch(this, this._getAttributesForQueueErrorCallback), true, queue.domain);
//      }
//    }
//    return queue;
//  },

  /**
   * 
   * @param queue queue object
   * @returns {object} queue attributes where the attributes are set
   */
  getAttributes: function getAttributes_QueueLister(queue){
    var attributes = null;
    if (queue) {
      // If not already done, load attributes for Queue, 
      // and pass them into the queue (invoking setAttributes as callback)

      if (queue.attributesLoaded !== true) {
        var load = function(data){
          attributes = data;
        };
        this._restHandler.getAttributesForQueryTable(this.type, queue.getPlainQueueName(), queue.processAppName,
          this.type == com.ibm.bpm.social.widgets.task.util.Constants.PROCESS_TYPE, load, dojo.hitch(this,
            this._getAttributesForQueueErrorCallback), true, queue.domain);
      }
    }
    return attributes;
  },


  /**
   * Load all task queues and return only the custom queues. Thus filter out the list of bSpace and WLE
   * internal queues.
   * 
   * {object} List of queues
   */
  getCustomQueues: function getCustomQueues_QueueLister(){
    var allQueues = this._queues;
    if (!allQueues) {
      allQueues = this._getAllTaskQueues();
    }
    var queues = [];
    if (allQueues) {
      for ( var k in allQueues) {
        var queue = allQueues[k];
        if (queue && queue.kind == "SAVED_SEARCH" && this.queuesToIgnore[queue.name] !== true) {
          queues.push(queue);
        }
      }
    }
    return queues;
  },

  /**
   * Return all QTs available
   * 
   * @returns {Queues[]} if no qt is found, returns null otherwise. If null is returned, details may be found
   *          in
   *          <li> this._errorObject.data </li>
   *          <li> this._errorObject.ioArgs </li>
   */

  _getAllQueues: function _getAllQueues_QueueLister(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var i, j, qtTitle;
    var allQueryTables = null;
    this._retrievedQTs = null;
    var systems = this._restHandler.getSystems();
    var domainNames = systems.getDomainNames();

    // cases:
    // 1) domains.length=0 || isApiTypeFederated==false 
    //   - Single server setup (one WLE/WPS) : query w/o domain name
    //   - No domain to be displayed 
    // 
    // 2) domains.length=1:
    //   - domain setup with only one system: query with domain name but 
    //   - No domain to be displayed
    // 
    // 3) domains.length=n
    //  - domain setup with multiple systems: query with domain name and
    //  - domain to be displayed
    // standard handling
    // retrieve all QTs 
    if (!systems.isApiTypeFederated()) {
      this._restHandler.getAllQueryTables(this.type, null, dojo.hitch(this, this._getAllQTLoadCallback), dojo.hitch(this,
        this._getAllQTErrorCallback));

      // this._retrievedQTs is null in case an error occurred at the REST API call
      if (this._retrievedQTs) {
        allQueryTables = this._retrievedQTs;
      }
    }
    else {
      // federated TODO ???
      allQueryTables = [];

      for (i = 0; i < domainNames.length; i++) {
        var domainName = domainNames[i];

        // only retrieve queries if
        // (
        // no process application is set and the domain does not include any icm systems
        // OR
        // a process application is set and the domain contains at least one system with that process application
        // ) AND
        // widget type is com.ibm.bpm.social.widgets.task.util.Constants.TASK_TYPE or domain does not contain ICM
        if (systems.isDomainWleOnly(domainName)) {
          // federation only for WLE only domains
          // do not send the attributes for WLE or mixed domains as they are not necessary in that case
          this._restHandler.getAllQueryTables(this.type, null, dojo.hitch(this, this._getAllQTLoadCallback), dojo.hitch(
            this, this._getAllQTErrorCallback), domainName);
        }

        if (this._errorObject) {
          // TODO SaS: as long as all=true is not yet fully available and not used by us we need to see 404 as a valid response
          // (happens if an application name is not known in all systems of the domain)
          if (this._errorObject.ioArgs.xhr.status == 404) {
            this._errorObject = null;
          }
          else {
            // an unexected error occurred
            allQueryTables = null;
            break;
          }
        }
        else {
          // we must update the queue be identified with the domain and process application
          for (j = 0; j < this._retrievedQTs.length; j++) {
            this._retrievedQTs[j].domain = domainName;
          }
          this._retrievedQTs = this._checkEligibleQTs(this._retrievedQTs, domainName);

          allQueryTables = allQueryTables.concat(this._retrievedQTs);
        }
      }
    }

    if (allQueryTables) {
      for (j = 0; j < allQueryTables.length; j++) {
        // convert raw objects to Queues 
        allQueryTables[j] = new com.ibm.bpm.social.widgets.task.util.Queue(allQueryTables[j], systems);
      }

      // Sort on name
      if (allQueryTables.length > 1) {
        // sort on name
        allQueryTables.sort(function(qt1, qt2){
          var qt1Name = qt1.getQualifiedDisplayName();
          var qt2Name = qt2.getQualifiedDisplayName();
          if (qt1Name < qt2Name) //sort string ascending
          {
            return -1;
          }
          if (qt1Name > qt2Name) {
            return 1;
          }
          return 0;
        });

      }
    }

    var found_queues = [];
    for (j = 0; j < allQueryTables.length; j++) {
      found_queues.push(allQueryTables[j].getQualifiedDisplayName());
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, "found : " + found_queues.join(", "));
    return allQueryTables;
  },

  /**
   * XHR helper for _getAllQueues<br>
   * Returns a list of QTs in _allQueryTables
   * 
   * @type {void}
   */
  _getAllQTLoadCallback: function _getAllQTLoadCallback_QueueLister(responseObject, ioArgs){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (responseObject) {
      this._retrievedQTs = responseObject.items;
      this._errorObject = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * XHR helper for _getAllQueues
   * 
   * @type {void}
   */
  _getAllQTErrorCallback: function _getAllQTErrorCallback_QueueLister(data, ioArgs){
    this._errorObject = {};
    this._errorObject.data = data;
    this._errorObject.ioArgs = ioArgs;
    this._retrievedQTs = null;
    com.ibm.bpm.social.util.common.Log.logError(1, "", data);
  },

  /**
   * XHR helper for _getAttributesForQueue
   * 
   * @type {void}
   */
  _getAttributesForQueueErrorCallback: function _getAttributesForQueueErrorCallback_QueueLister(data, ioArgs){
    this._errorObject = {};
    this._errorObject.data = data;
    this._errorObject.ioArgs = ioArgs;
    com.ibm.bpm.social.util.common.Log.logError(1, "", data);
  }

});
