/*
	Licensed Materials - Property of IBM
	5725-C96
	Copyright IBM Corp. 2008, 2011, 2012 All Rights Reserved.
	US Government Users Restricted Rights - Use, duplication or
	disclosure restricted by GSA ADP Schedule Contract with
	IBM Corp.
*/
dojo.provide("com.ibm.bpm.social_diagram");
if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");


dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						var widget = hash[widgetId];
						if(widget){	// may be null on page w/multiple dojo's loaded
							outAry.push(widget);
						}
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body.contentEditable == 'true' || (body.firstChild && body.firstChild.contentEditable == 'true');
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};
		function radioName(node) {
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE <= 9 && child.scopeName !== "HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(dojo.attr(child, "checked") && rn) {
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		function rs(node) {
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}
		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dojo.Stateful"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.Stateful"] = true;
dojo.provide("dojo.Stateful");


dojo.declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			dojo.mixin(this, mixin);
		}
	},
	
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3
		
		return this[name];
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this;
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.
		
		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
                        propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							try{
								propertyCallbacks[i].call(self, name, oldValue, value);
							}catch(e){
								console.error(e);
							}
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(dojo.indexOf(propertyCallbacks, callback), 1);
			}
		};
	}
	
});

}

if(!dojo._hasResource["dijit._WidgetBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._WidgetBase"] = true;
dojo.provide("dijit._WidgetBase");




(function(){

dojo.declare("dijit._WidgetBase", dojo.Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// mix in our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo._mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(this._getSetterAttributes(), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	_getSetterAttributes: function(){
		// summary:
		//		Returns list of attributes with custom setters for this widget
		var ctor = this.constructor;
		if(!ctor._setterAttrs){
			var r = (ctor._setterAttrs = []),
				attrs,
				proto = ctor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
		}
		return ctor._setterAttrs;	// String[]
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || dojo.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.replaceClass(mapNode, value, this["class"]);
		this._set("class", value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.replaceClass(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			this._set(name, value);
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handles = [dojo._connect(obj, event, this, method)];
		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var handle = dojo.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	},

	defer: function(fcn, delay){
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(dojo.hitch(this, 
			function(){ 
				timer = null;
				if(!this._destroyed){ 
					dojo.hitch(this, fcn)(); 
	}
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

})();

}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");

dojo.getObject("window", true, dojo);

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view using minimal movement, if it is not already.
	
	// Don't rely on node.scrollIntoView working just because the function is there since
	// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
	// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
	// and when there's a fixed position scrollable element

	node = dojo.byId(node);
	var body, doc = node.ownerDocument || dojo.doc;

	// has() functions but without has() support
	if(!("rtl_adjust_position_for_verticalScrollBar" in dojo.window)){
		body = dojo.body();
		var	scrollable = dojo.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = dojo.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last");
		dojo.window.rtl_adjust_position_for_verticalScrollBar = dojo.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
	}

	if(!("position_fixed_support" in dojo.window)){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		body = dojo.body();
		var	outer = dojo.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = dojo.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last");
		dojo.window.position_fixed_support = dojo.position(inner).x != dojo.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
	}

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		body = doc.body || doc.getElementsByTagName("body")[0];
		var	html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE,
			isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if(node == body || node == html){ return; }
		if(!(dojo.isMozilla || isIE || isWK || dojo.isOpera) && ("scrollIntoView" in node)){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
			rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
			scrollRoot = (isWK || backCompat) ? body : html,
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return (isIE <= 6 || (isIE == 7 && backCompat))
					? false
					: (dojo.window.position_fixed_support && (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do
		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el),
				rtl = dojo.getComputedStyle(el).direction.toLowerCase() == "rtl";
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE || isIE >= 9){ elPos.x = 0; } // older IE can have values > 0
				if(elPos.y < 0 || !isIE || isIE >= 9){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					if(rtl && dojo.window.rtl_adjust_position_for_verticalScrollBar){
						elPos.x += scrollBarSize;
					}
					elPos.w = clientSize;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				t = nodePos.y - elPos.y, // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			var s, old;
			if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
				s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				old = el.scrollLeft;
				el.scrollLeft += s;
				s = el.scrollLeft - old;
				nodePos.x -= s;
			}
			if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
				s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
				old = el.scrollTop;
				el.scrollTop += s;
				s = el.scrollTop - old;
				nodePos.y -= s;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");




// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
					if(sel.rangeCount){
						bm.mark = sel.getRangeAt(0).cloneRange();
					}
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					targetWindow.document.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.body.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.body.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget.set("focused", false);
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				widget.set("focused", true);
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");


dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
};

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary:
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");




dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, choices, layoutNode, /*Object*/ aroundNodeCoords){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//		It also passes in the available size for the popup, which is useful for tooltips to
	//		tell them that their width is limited to a certain amount.   layoutNode() may return a value expressing
	//		how much the popup had to be modified to fit into the available space.   This is used to determine
	//		what the best placement is.
	// aroundNodeCoords: Object
	//		Size of aroundNode, ex: {w: 200, h: 50}

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;
		var overflow = 0;

		// calculate amount of space available given specified position of node
		var spaceAvailable = {
			w: corner.charAt(1) == 'L' ? (view.l + view.w) - pos.x : pos.x - view.l,
			h: corner.charAt(1) == 'T' ? (view.t + view.h) - pos.y : pos.y - view.t
		};

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltip's size changes based on position, due to triangle)
		if(layoutNode){
			var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
			overflow = typeof res == "undefined" ? 0 : res;
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY;

		overflow += (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow,
				spaceAvailable: spaceAvailable
			};
		}
		
		return !overflow;
	});

	// In case the best position is not the last one we checked, need to call
	// layoutNode() again.
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
	}

	// And then position the node.   Do this last, after the layoutNode() above
	// has sized the node, due to browser quirks when the viewport is scrolled
	// (specifically that a Tooltip will shrink to fit as though the window was
	// scrolled to the left).
	//
	// In RTL mode, set style.right rather than style.left so in the common case,
	// window resizes move the popup along with the aroundNode.
	var l = dojo._isBodyLtr(),
		s = node.style;
	s.top = best.y + "px";
	s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
	s[l ? "right" : "left"] = "auto";       // needed for FF or else tooltip goes to far left

	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var aroundNodePos = dojo.position(aroundNode, true);

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode, {w: width, h: height});
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below-alt":
				leftToRight = !leftToRight;
				// fall through
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above-alt":
				leftToRight = !leftToRight;
				// fall through
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");



dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 1000,

	_idGen: 1,

	_createWrapper: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups.
		//		Puts widget inside a wrapper DIV (if not already in one),
		//		and returns pointer to that wrapper DIV.

		var node = widget.domNode || widget,
			wrapper = widget.declaredClass ? widget._popupWrapper :
				node.parentNode && dojo.hasClass(node.parentNode, "dijitPopup") ? node.parentNode : null;

		if(!wrapper){
			// Create wrapper <div> for when this widget [in the future] will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{ display: "none"},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
		
		return wrapper;	// DOMNode
	},

	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Moves the popup widget off-screen.
		//		Do not use this method to hide popups when not in use, because
		//		that will create an accessibility issue: the offscreen popup is
		//		still in the tabbing order.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, {
			visibility: "hidden",
			top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
			display: ""
		});
	},

	hide: function(/*dijit._Widget*/ widget){
		// summary:
		//		Hide this popup widget (until it is ready to be shown).
		//		Initialization for widgets that will be used as popups
		//
		// 		Also puts widget inside a wrapper DIV (if not already in one)
		//
		//		If popup widget needs to layout it should
		//		do so when it is made visible, and popup._onShow() is called.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, "display", "none");
	},
		
	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

		// If we are opening a new popup that isn't a child of a currently opened popup, then
		// close currently opened popup(s).   This should happen automatically when the old popups
		// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
		while(stack.length && (!args.parent || !dojo.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
			dijit.popup.close(stack[stack.length-1].widget);
		}

		// Get pointer to popup wrapper, and create wrapper if it doesn't exist
		var wrapper = this._createWrapper(widget);


		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.display = "";
		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget?*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented.
		//		If no popup is specified, closes all popups.

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while((popup && dojo.some(stack, function(elem){return elem.widget == popup;})) ||
			(!popup && stack.length)){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.hide(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE < 9){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/*DomNode*/ node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = (this.iframe = dijit._frames.pop());
		node.appendChild(iframe);
		if(dojo.isIE<7 || dojo.isQuirks){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		Resize the iframe so it's the same size as node.
		//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
		if(this.iframe){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");



dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");


(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_quirks: d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = d.isQuirks;
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");



// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");


dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE < 9){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");


dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
		// summary:
		//		Determines if an element has a particular role.
		// returns:
		//		True if elem has the specific role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has a role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the role for an element (which should be a wai role).
		// returns:
		//		The role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.

			dojo.attr(elem, "role", role);
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");





////////////////// DEFERRED CONNECTS ///////////////////

// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

////////////////// ONDIJITCLICK SUPPORT ///////////////////

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

dojo.declare("dijit._Widget", dijit._WidgetBase, {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- deferred connections
	//				A call like dojo.connect(myWidget, "onMouseMove", func)
	//				will essentially do a dojo.connect(myWidget.domNode, "onMouseMove", func)
	//			- ondijitclick
	//				Support new dojoAttachEvent="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)
	

	////////////////// DEFERRED CONNECTS ///////////////////

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		this.inherited(arguments);

		if(this.domNode){
			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	////////////////// FOCUS RELATED ///////////////////
	// _onFocus() and _onBlur() are called by the focus manager

	// focused: [readonly] Boolean
	//		This widget or a widget it contains has focus, or is "active" because
	//		it was recently clicked.
	focused: false,

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	////////////////// ONDIJITCLICK SUPPORT ///////////////////

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = this.inherited(arguments, [obj, event == "ondijitclick" ? "onclick" : event, method]);

		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(d.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							
							// Stop event to prevent scrolling on space key in IE.
							// But don't do this for _HasDropDown because it surpresses the onkeypress
							// event needed to open the drop down when the user presses the SPACE key.
							if(!("openDropDown" in this && obj == this._buttonNode)){
								e.preventDefault();
							}
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target == dijit._lastKeyDownNode &&	// === breaks greasemonkey
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
		}

		return handles;		// _Widget.Handle
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");



new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".	 interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }").
					//	Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var dummyClass = {}, instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	// TODO: remove this in 2.0, when we stop caching parameters.
	d.connect(d, "extend", function(){
		instanceClasses = {};
	});

	function getProtoInfo(cls, params){
		// cls: A prototype
		//		The prototype of the class to check props on
		// params: Object
		//		The parameters object to mix found parameters onto.
		for(var name in cls){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
			params[name] = val2type(cls[name]);
		}
		return params;
	}

	function getClassInfo(/*String*/ className, /*Boolean*/ skipParamsLookup){
		// summary:
		//		Maps a widget name string like "dijit.form.Button" to the widget constructor itself,
		//		and a list of that widget's parameters and their types
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{
		//				cls: dijit.Button,
		//				params: { label: "string", disabled: "boolean"}
		//			}

		var c = instanceClasses[className];
		if(!c){
			// get pointer to widget class
			var cls = d.getObject(className), params = null;
			if(!cls){ return null; }		// class not defined [yet]
			if(!skipParamsLookup){ // from fastpath, we don't need to lookup the attrs on the proto because they are explicit
				params = getProtoInfo(cls.prototype, {})
			}
			c = { cls: cls, params: params };
			
		}else if(!skipParamsLookup && !c.params){
			// if we're calling getClassInfo and have a cls proto, but no params info, scan that cls for params now
			// and update the pointer in instanceClasses[className]. This happens when a widget appears in another
			// widget's template which still uses dojoType, but an instance of the widget appears prior with a data-dojo-type,
			// skipping this lookup the first time.
			c.params = getProtoInfo(c.cls.prototype, {});
		}
		
		return c;
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.

		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var attrName = (args.scope || d._scopeName) + "Type",	// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.
			var node, type, clsInfo, clazz, scripts, fastpath;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				fastpath = obj.fastpath;
				clsInfo = obj.clsInfo || (type && getClassInfo(type, fastpath));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes. no fastpath/data-dojo-type support here.
				node = obj;
				type = attrName in mixin ? mixin[attrName] : node.getAttribute(attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] :
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};
				
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				d._mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				d._mixin(params, obj.inherited);
			}
			
			// mix things found in data-dojo-props into the params
			if(fastpath){
				var extra = node.getAttribute(attrData + "props");
				if(extra && extra.length){
					try{
						extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
						d._mixin(params, extra);
					}catch(e){
						// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
						throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
					}
				}

				// For the benefit of _Templated, check if node has data-dojo-attach-point/data-dojo-attach-event
				// and mix those in as though they were parameters
				var attachPoint = node.getAttribute(attrData + "attach-point");
				if(attachPoint){
					params.dojoAttachPoint = attachPoint;
				}
				var attachEvent = node.getAttribute(attrData + "attach-event");
				if(attachEvent){
					params.dojoAttachEvent = attachEvent;
				}
				dojo.mixin(params, mixin);
			}else{
				// FIXME: we need something like "deprecateOnce()" to throw dojo.deprecation for something.
				// remove this logic in 2.0
				// read parameters (ie, attributes) specified on DOMNode

				var attributes = node.attributes;

				// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
				for(var name in clsInfo.params){
					var item = name in mixin ? { value:mixin[name], specified:true } : attributes.getNamedItem(name);
					if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
					var value = item.value;
					// Deal with IE quirks for 'class' and 'style'
					switch(name){
					case "class":
						value = "className" in mixin ? mixin.className : node.className;
						break;
					case "style":
						value = "style" in mixin ? mixin.style : (node.style && node.style.cssText); // FIXME: Opera?
					}
					var _type = clsInfo.params[name];
					if(typeof value == "string"){
						params[name] = str2obj(value, _type);
					}else{
						params[name] = value;
					}
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
				var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script, attrData);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			// FIXME: in 2.0, drop jsId support. use data-dojo-id instead
			var jsname = (node.getAttribute(attrData + "id") || node.getAttribute("jsId"));
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(rootNode, args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantitate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a
		//		`rootNode` member, that is used.
		//
		// args: Object
		//		a kwArgs object passed along to instantiate()
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._Templated`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dojo.byId(root) : dojo.body();
		args = args || {};

		var attrName = (args.scope || d._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode,			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				// TODO: what if this is a widget and dir/lang are declared in data-dojo-props?
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = (!parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser) || (args && args.template);

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					// FIXME: desupport dojoType in 2.0. use data-dojo-type instead
					var type, html5 = recurse && child.getAttribute(attrData + "type");
					if(html5){
						type = html5;
					}else{
						// fallback to backward compatible mode, using dojoType. remove in 2.0
						type = recurse && child.getAttribute(attrName);
					}
					
					var fastpath = html5 == type;

					if(type){
						// if dojoType/data-dojo-type specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							fastpath: fastpath,
							clsInfo: getClassInfo(type, fastpath), // note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Ignore bogus entries in inherited hash like {dir: ""}
		var inherited = {};
		if(args && args.inherited){
			for(var key in args.inherited){
				if(args.inherited[key]){ inherited[key] = args.inherited[key]; }
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root,
			inherited: inherited
		}, list);

		// go build the object instances
		var mixin = args && args.template ? {template: true} : null;
		return this.instantiate(list, mixin, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){
		if(dojo.config.parseOnLoad){
			dojo.parser.parse();
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.getObject("dijit.wai.onload") === dojo._loaders[0]){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");


/*=====
dojo.cache = {
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary:
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with dojoAttachEvent=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to attributeMap
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process dojoAttachPoint
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");;
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this._attachEvents.push(this.connect(baseNode, event, thisFunc));
						}
					}
				}

				// waiRole, waiState
				// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directicly
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			dojo.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];
			
			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.common._RestWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.common._RestWidget"] = true;
dojo.provide("com.ibm.bpm.social.widgets.common._RestWidget");



dojo.declare("com.ibm.bpm.social.widgets.common._RestWidget", dijit._Widget, {

	proxyUrl: "",
	
	getProxyUrl: function(){
		var url = this.proxyUrl;
		if(url!=""){
			if(url.lastIndexOf("/")==(url.length-1)){
				url = url.substring(0, url.length-1);
			}
		}
		//TODO Should this be guaranteed to return something in the format '/prefix',
		//or is eliminating a trailing slash good enough?
		return url;
	}
	
});

}

if(!dojo._hasResource["dijit._Contained"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Contained"] = true;
dojo.provide("dijit._Contained");


dojo.declare("dijit._Contained",
		null,
		{
			// summary:
			//		Mixin for widgets that are children of a container widget
			//
			// example:
			// | 	// make a basic custom widget that knows about it's parents
			// |	dojo.declare("my.customClass",[dijit._Widget,dijit._Contained],{});

			getParent: function(){
				// summary:
				//		Returns the parent widget of this widget, assuming the parent
				//		specifies isContainer
				var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
				return parent && parent.isContainer ? parent : null;
			},

			_getSibling: function(/*String*/ which){
				// summary:
				//      Returns next or previous sibling
				// which:
				//      Either "next" or "previous"
				// tags:
				//      private
				var node = this.domNode;
				do{
					node = node[which+"Sibling"];
				}while(node && node.nodeType != 1);
				return node && dijit.byNode(node);	// dijit._Widget
			},

			getPreviousSibling: function(){
				// summary:
				//		Returns null if this is the first child of the parent,
				//		otherwise returns the next element sibling to the "left".

				return this._getSibling("previous"); // dijit._Widget
			},

			getNextSibling: function(){
				// summary:
				//		Returns null if this is the last child of the parent,
				//		otherwise returns the next element sibling to the "right".

				return this._getSibling("next"); // dijit._Widget
			},

			getIndexInParent: function(){
				// summary:
				//		Returns the index of this widget within its container parent.
				//		It returns -1 if the parent does not exist, or if the parent
				//		is not a dijit._Container

				var p = this.getParent();
				if(!p || !p.getIndexOfChild){
					return -1; // int
				}
				return p.getIndexOfChild(this); // int
			}
		}
	);

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");


dojo.declare("dijit._Container",
	null,
	{
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		// isContainer: [protected] Boolean
		//		Indicates that this widget acts as a "parent" to the descendant widgets.
		//		When the parent is started it will call startup() on the child widgets.
		//		See also `isLayoutContainer`.
		isContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget or int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number"){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//      Destroys all the widgets inside this.containerNode,
			//      but not this widget itself
			dojo.forEach(this.getChildren(), function(child){ child.destroyRecursive(preserveDom); });
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node && dijit.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return dojo.indexOf(this.getChildren(), child);	// int
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Startup all children of this widget
			dojo.forEach(this.getChildren(), function(child){ child.startup(); });

			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");





dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent()
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when my size changes so that I can re-layout.
				// For browsers where I can't directly monitor when my size changes,
				// monitor when the viewport changes size, which *may* indicate a size change for me.
				this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments to resize.
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				dojo.marginBox(node, changeSize);

				// set offset of the node
				if(changeSize.t){ node.style.top = changeSize.t + "px"; }
				if(changeSize.l){ node.style.left = changeSize.l + "px"; }
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = dojo.mixin(dojo.marginBox(node), mb);	// just use dojo.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling dojo.contentBox() since that may fail if size was recently set)
			var cs = dojo.getComputedStyle(node);
			var me = dojo._getMarginExtents(node, cs);
			var be = dojo._getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = dojo._getPadExtents(node, cs);
			this._contentBox = {
				l: dojo._toPixelValue(node, cs.paddingLeft),
				t: dojo._toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			var cls = this.baseClass + "-child "
				+ (child.baseClass ? this.baseClass + "-" + child.baseClass : "");
			dojo.addClass(child.domNode, cls);
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			var cls = this.baseClass + "-child"
					+ (child.baseClass ?
						" " + this.baseClass + "-" + child.baseClass : "");
			dojo.removeClass(child.domNode, cls);
			
			this.inherited(arguments);
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return its content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me = dojo._getMarginExtents(node, cs);
	var pb = dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		var newSize = widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size
		if(newSize){
			// if the child returned it's new size then use that
			dojo.mixin(widget, newSize);
		}else{
			// otherwise, call marginBox(), but favor our own numbers when we have them.
			// the browser lies sometimes
			dojo.mixin(widget, dojo.marginBox(widget.domNode));
			dojo.mixin(widget, dim);
		}
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Widget[]*/ children,
			/*String?*/ changedRegionId, /*Number?*/ changedRegionSize){
		// summary
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array of Widgets or at least objects containing:
		//			* domNode: pointer to DOM node to position
		//			* region or layoutAlign: position to place DOM node
		//			* resize(): (optional) method to set size of node
		//			* id: (optional) Id of widgets, referenced from resize object, below.
		// changedRegionId:
		//		If specified, the slider for the region with the specified id has been dragged, and thus
		//		the region's height or width should be adjusted according to changedRegionSize
		// changedRegionSize:
		//		See changedRegionId.

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.    TODO: move these lines to LayoutContainer?   Unneeded other places I think.
		children = dojo.filter(children, function(item){ return item.region != "center" && item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.region == "center" || item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = (child.region || child.layoutAlign);

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.position = "absolute";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// Size adjustments to make to this child widget
			var sizeSetting = {};

			// Check for optional size adjustment due to splitter drag (height adjustment for top/bottom align
			// panes and width adjustment for left/right align panes.
			if(changedRegionId && changedRegionId == child.id){
				sizeSetting[child.region == "top" || child.region == "bottom" ? "h" : "w"] = changedRegionSize;
			}

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				sizeSetting.w = dim.w;
				size(child, sizeSetting);
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				sizeSetting.h = dim.h;
				size(child, sizeSetting);
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client" || pos == "center"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit.layout._ContentPaneResizeMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._ContentPaneResizeMixin"] = true;
dojo.provide("dijit.layout._ContentPaneResizeMixin");




dojo.declare("dijit.layout._ContentPaneResizeMixin", null, {
	// summary:
	//		Resize() functionality of ContentPane.   If there's a single layout widget
	//		child then it will call resize() with the same dimensions as the ContentPane.
	//		Otherwise just calls resize on each child.
	//
	//		Also implements basic startup() functionality, where starting the parent
	//		will start the children

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// isContainer: [protected] Boolean
	//		Indicates that this widget acts as a "parent" to the descendant widgets.
	//		When the parent is started it will call startup() on the child widgets.
	//		See also `isLayoutContainer`.
	isContainer: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	_startChildren: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		dojo.forEach(this.getChildren(), function(child){
			child.startup();
			child._started = true;
		});
	},

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		if(this._started){ return; }

		var parent = dijit._Contained.prototype.getParent.call(this);
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		this.inherited(arguments);

		this._startChildren();

		if(this._isShown()){
			this._onShow();
		}

		if(!this._childOfLayoutWidget){
			// If my parent isn't a layout container, since my style *may be* width=height=100%
			// or something similar (either set directly or via a CSS class),
			// monitor when my size changes so that I can re-layout.
			// For browsers where I can't directly monitor when my size changes,
			// monitor when the viewport changes size, which *may* indicate a size change for me.
			this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
				// Using function(){} closure to ensure no arguments to resize.
				this._needLayout = !this._childOfLayoutWidget;
				this.resize();
			});
		}
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propagate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = dojo.query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return dojo.hasAttr(node, "data-dojo-type") || dojo.hasAttr(node, "dojoType") || dojo.hasAttr(node, "widgetId");
			}),
			candidateWidgets = dojo.filter(childWidgetNodes.map(dijit.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		dojo.toggleClass(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download (see [20099]).
		// However, don't load href for closed TitlePanes.
		if(!this._wasShown && this.open !== false){
			this._onShow();
		}

		this._resizeCalled = true;

		this._scheduleLayout(changeSize, resultSize);
	},

	_scheduleLayout: function(changeSize, resultSize){
		// summary:
		//		Resize myself, and call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layout(changeSize, resultSize);
		}else{
			this._needLayout = true;
			this._changeSize = changeSize;
			this._resultSize = resultSize;
		}
	},

	_layout: function(changeSize, resultSize){
		// summary:
		//		Resize myself according to optional changeSize/resultSize parameters, like a layout widget.
		//		Also, since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		//
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from set('content', ...))... but deferred until
		//		the ContentPane is visible

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			dojo.marginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = dojo.mixin(dojo.marginBox(cn), mb); // just use dojo.marginBox() to fill in missing values
			}
			this._contentBox = dijit.layout.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = dojo.contentBox(cn);
		}

		this._layoutChildren();

		delete this._needLayout;
	},
	
	_layoutChildren: function(){
		// Call _checkIfSingleChild() again in case app has manually mucked w/the content
		// of the ContentPane (rather than changing it through the set("content", ...) API.
		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || dojo.contentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			dojo.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			var node = this.domNode, parent = this.domNode.parentNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !dojo.hasClass(node, "dijitHidden") &&
					parent && parent.style && (parent.style.display != 'none');
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does layout/resize of child widget(s)

		if(this._needLayout){
			// If a layout has been scheduled for when we become visible, do it now
			this._layout(this._changeSize, this._resultSize);
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	}
});

}

if(!dojo._hasResource["dojo.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.html"] = true;
dojo.provide("dojo.html");


dojo.getObject("html", true, dojo);

// the parser might be needed..
(function(){ // private scope, sort of a namespace

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0,
		d = dojo;
	
	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		//
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = dojo.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element.
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
		
		// always empty
		d.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = d._toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && d.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					d.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				d.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	dojo.declare("dojo.html._ContentSetter", null,
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",
			
			// id: String?
			//		Usually only used internally, and auto-generated with each instance
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document,
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,
			
			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,

			// parserScope: String
			//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
			//		will search for data-dojo-type (or dojoType).  For backwards compatibility
			//		reasons defaults to dojo._scopeName (which is "dojo" except when
			//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
			parserScope: dojo._scopeName,

			// startup: Boolean
			//		Start the child widgets after parsing them.   Only obeyed if parseContent is true.
			startup: true,
			
			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..
 
				// the original params are mixed directly into the instance "this"
				dojo.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dojo.byId( this.node || node );
	
				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "",
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node

				var node = this.node;
				if(!node) {
				    // can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
	
					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e);
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},
			
			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					dojo.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},
	
			onBegin: function(){
				// summary
				//		Called after instantiation, but before set();
				//		It allows modification of any of the object properties
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to
				//		optionally pre-process html string content
				var cont = this.content;
	
				if(dojo.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}
  
					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();
				
				this.content = cont;
				return this.node; /* DomNode */
			},
	
			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},
	
			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically.
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults;
				delete this.node;
				delete this.content;
			},
  
			onContentError: function(err){
				return "Error occured setting content: " + err;
			},
			
			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key];
				}
			},
			_parse: function(){
				// summary:
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					var inherited = {};
					dojo.forEach(["dir", "lang", "textDir"], function(name){
						if(this[name]){
							inherited[name] = this[name];
						}
					}, this);
					this.parseResults = dojo.parser.parse({
						rootNode: rootNode,
						noStart: !this.startup,
						inherited: inherited,
						scope: this.parserScope
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},
  
			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element.
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params:
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage:
			//		dojo.html.set(node, "some string");
			//		dojo.html.set(node, contentNode, {options});
			//		dojo.html.set(node, myNode.childNodes, {options});
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(dojo.mixin(
					params,
					{ content: cont, node: node }
			));
			return op.set();
		}
	};
})();

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");







dojo.declare(
	"dijit.layout.ContentPane", [dijit._Widget, dijit.layout._ContentPaneResizeMixin],
{
	// summary:
	//		A widget containing an HTML fragment, specified inline
	//		or by uri.  Fragment may include widgets.
	//
	// description:
	//		This widget embeds a document fragment in the page, specified
	//		either by uri, javascript generated markup or DOM reference.
	//		Any widgets within this content are instantiated and managed,
	//		but laid out according to the HTML structure.  Unlike IFRAME,
	//		ContentPane embeds a document fragment as would be found
	//		inside the BODY tag of a full HTML document.  It should not
	//		contain the HTML, HEAD, or BODY tags.
	//		For more advanced functionality with scripts and
	//		stylesheets, see dojox.layout.ContentPane.  This widget may be
	//		used stand alone or as a base class for other widgets.
	//		ContentPane is useful as a child of other layout containers
	//		such as BorderContainer or TabContainer, but note that those
	//		widgets can contain any widget as a child.
	//
	// example:
	//		Some quick samples:
	//		To change the innerHTML: cp.set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList: cp.set('content', dojo.query('div [class=selected]', userSelection))
	//
	//		To do an ajax update: cp.set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

/*=====
	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to set("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",
=====*/

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// parserScope: String
	//		Flag passed to parser.  Root for attribute names to search for.   If scopeName is dojo,
	//		will search for data-dojo-type (or dojoType).  For backwards compatibility
	//		reasons defaults to dojo._scopeName (which is "dojo" except when
	//		multi-version support is used, when it will be something like dojo16, dojo20, etc.)
	parserScope: dojo._scopeName,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via set('content', ...) / set('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div dojoType="dijit.layout.ContentPane" href="./bar" ioArgs="{timeout: 500}">
	ioArgs: {},

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by set('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior set('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href
	//		or content is loaded.
	onLoadDeferred: null,

	// Override _Widget's attributeMap because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		title: []
	}),

	// Flag to parser that I'll parse my contents, so it shouldn't.
	stopParser: true,

	// template: [private] Boolean
	//		Flag from the parser that this ContentPane is inside a template
	//		so the contents are pre-parsed.
	// (TODO: this declaration can be commented out in 2.0)
	template: false,

	create: function(params, srcNodeRef){
		// Convert a srcNodeRef argument into a content parameter, so that the original contents are
		// processed in the same way as contents set via set("content", ...), calling the parser etc.
		// Avoid modifying original params object since that breaks NodeList instantiation, see #11906.
		if((!params || !params.template) && srcNodeRef && !("href" in params) && !("content" in params)){
			var df = dojo.doc.createDocumentFragment();
			srcNodeRef = dojo.byId(srcNodeRef)
			while(srcNodeRef.firstChild){
				df.appendChild(srcNodeRef.firstChild);
			}
			params = dojo.delegate(params, {content: df});
		}
		this.inherited(arguments, [params, srcNodeRef]);
	},

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);
	},

	buildRendering: function(){
		this.inherited(arguments);

		// Since we have no template we need to set this.containerNode ourselves, to make getChildren() work.
		// For subclasses of ContentPane that do have a template, does nothing.
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}

		// remove the title attribute so it doesn't show up when hovering
		// over a node  (TODO: remove in 2.0, no longer needed after #11490)
		this.domNode.title = "";

		if(!dojo.attr(this.domNode,"role")){
			dijit.setWaiRole(this.domNode, "group");
		}
	},

	_startChildren: function(){
		// summary:
		//		Call startup() on all children including non _Widget ones like dojo.dnd.Source objects

		// This starts all the widgets
		this.inherited(arguments);

		// And this catches stuff like dojo.dnd.Source
		if(this._contentSetter){
			dojo.forEach(this._contentSetter.parseResults, function(obj){
				if(!obj._started && !obj._destroyed && dojo.isFunction(obj.startup)){
					obj.startup();
					obj._started = true;
				}
			}, this);
		}
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so set("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (a set('href', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));

		this._set("href", href);

		// _setHrefAttr() is called during creation and by the user, after creation.
		// Assuming preload == false, only in the second case do we actually load the URL;
		// otherwise it's done in startup(), and only if this widget is shown.
		if(this.preload || (this._created && this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// dojo.Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make set("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this._set("href", "");

		// Cancel any in-flight requests (a set('content', ...) will cancel any in-flight set('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		if(this._created){
			// For back-compat reasons, call onLoad() for set('content', ...)
			// calls but not for content specified in srcNodeRef (ie: <div dojoType=ContentPane>...</div>)
			// or as initialization parameter (ie: new ContentPane({content: ...})
			this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));
		}

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a set('content') not a set('href')

		return this.onLoadDeferred; 	// dojo.Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make get("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		this.inherited(arguments);

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				return this.refresh();	// If child has an href, promise that fires when the load is complete
			}
		}
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this.onLoadDeferred.addCallback(dojo.hitch(this, "onLoad"));
		this._load();
		return this.onLoadDeferred;		// If child has an href, promise that fires when refresh is complete
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this._set("isLoaded", true);
		try{
			this.onLoadDeferred.callback(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this._set("isLoaded", false);
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive();
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			dojo.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == dojo.body()){
					widget.destroyRecursive();
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		dojo.html._emptyNode(this.containerNode);

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(/*String|DocumentFragment*/ cont, /*Boolean*/ isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// dojo.html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof dojo.html._ContentSetter)){
			setter = this._contentSetter = new dojo.html._ContentSetter({
				node: this.containerNode,
				_onError: dojo.hitch(this, this._onError),
				onContentError: dojo.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		};

		var setterParams = dojo.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: this.parseOnLoad,
			parserScope: this.parserScope,
			startup: false,
			dir: this.dir,
			lang: this.lang
		}, this._contentSetterParams || {});

		setter.set( (dojo.isObject(cont) && cont.domNode) ? cont.domNode : cont, setterParams );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(!isFakeContent){
			if(this._started){
				// Startup each top level child widget (and they will start their children, recursively)
				this._startChildren();
	
				// Call resize() on each of my child layout widgets,
				// or resize() on my single child layout widget...
				// either now (if I'm currently visible) or when I become visible
				this._scheduleLayout();
			}

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// override on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(data){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

}

if(!dojo._hasResource["dijit.form._FormMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormMixin"] = true;
dojo.provide("dijit.form._FormMixin");



dojo.declare("dijit.form._FormMixin", null, {
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)
	// description:
	//		Can extract all the form widgets
	//		values and combine them into a single javascript object, or alternately
	//		take such an object and set the values for all the contained
	//		form widgets

/*=====
	// value: Object
	//		Name/value hash for each child widget with a name and value.
	//		Child widgets without names are not part of the hash.
	//
	//		If there are multiple child widgets w/the same name, value is an array,
	//		unless they are radio buttons in which case value is a scalar (since only
	//		one radio button can be checked at a time).
	//
	//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
	//
	//		Example:
	//	|	{ name: "John Smith", interests: ["sports", "movies"] }
=====*/

	// state: [readonly] String
	//		Will be "Error" if one or more of the child widgets has an invalid value,
	//		"Incomplete" if not all of the required child widgets are filled in.  Otherwise, "",
	//		which indicates that the form is ready to be submitted.
	state: "",

	//	TODO:
	//	* Repeater
	//	* better handling for arrays.  Often form elements have names with [] like
	//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
	//
	//

		reset: function(){
			dojo.forEach(this.getDescendants(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//		provides a few additional (ui-specific) features.
			//		1 - it will highlight any sub-widgets that are not
			//			valid
			//		2 - it will call focus() on the first invalid
			//			sub-widget
			var didFocus = false;
			return dojo.every(dojo.map(this.getDescendants(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					dojo.window.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			dojo.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*Object*/ obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by get('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.set('value', dojo.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			dojo.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call set('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
			
			// Note: no need to call this._set("value", ...) as the child updates will trigger onChange events
			// which I am monitoring.
		},

		getValues: function(){
			dojo.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.   See description of `value` for details.
			// description:

			// The value is updated into this.value every time a child has an onChange event,
			// so in the common case this function could just return this.value.   However,
			// that wouldn't work when:
			//
			// 1. User presses return key to submit a form.  That doesn't fire an onchange event,
			// and even if it did it would come too late due to the setTimout(..., 0) in _handleOnChange()
			//
			// 2. app for some reason calls this.get("value") while the user is typing into a
			// form field.   Not sure if that case needs to be supported or not.

			// get widget values
			var obj = { };
			dojo.forEach(this.getDescendants(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget)
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							dojo.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = dojo.getObject(name, false, obj);
							if(value === undefined){
								dojo.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=dojo.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(dojo.isArray(prev)){
							prev.push(value);
						}else{
							dojo.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						dojo.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					} else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					} else{
						// can not set value when there is no name
					}
				} else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid.
			//		Deprecated, will be removed in 2.0.  Use get("state") instead.

			return this.state == "";
		},

		onValidStateChange: function(isValid){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
			//
			//		Deprecated.  Will be removed in 2.0.  Use watch("state", ...) instead.
		},

		_getState: function(){
			// summary:
			//		Compute what this.state should be based on state of children
			var states = dojo.map(this._descendants, function(w){
				return w.get("state") || "";
			});

			return dojo.indexOf(states, "Error") >= 0 ? "Error" :
				dojo.indexOf(states, "Incomplete") >= 0 ? "Incomplete" : "";
		},

		disconnectChildren: function(){
			// summary:
			//		Remove connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			dojo.forEach(this._childConnections || [], dojo.hitch(this, "disconnect"));
			dojo.forEach(this._childWatches || [], function(w){ w.unwatch(); });
		},

		connectChildren: function(/*Boolean*/ inStartup){
			// summary:
			//		Setup connections to monitor changes to children's value, error state, and disabled state,
			//		in order to update Form.value and Form.state.
			//
			//		You can call this function directly, ex. in the event that you
			//		programmatically add a widget to the form *after* the form has been
			//		initialized.

			var _this = this;

			// Remove old connections, if any
			this.disconnectChildren();

			this._descendants = this.getDescendants();

			// (Re)set this.value and this.state.   Send watch() notifications but not on startup.
			var set = inStartup ? function(name, val){ _this[name] = val; } : dojo.hitch(this, "_set");
			set("value", this.get("value"));
			set("state", this._getState());

			// Monitor changes to error state and disabled state in order to update
			// Form.state
			var conns = (this._childConnections = []),
				watches = (this._childWatches = []);
			dojo.forEach(dojo.filter(this._descendants,
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget changes validity state - or
				// whenever the disabled attribute on that widget is changed.
				dojo.forEach(["state", "disabled"], function(attr){
					watches.push(widget.watch(attr, function(attr, oldVal, newVal){
						_this.set("state", _this._getState());
					}));
				});
			});

			// And monitor calls to child.onChange so we can update this.value
			var onChange = function(){
				// summary:
				//		Called when child's value or disabled state changes
				
				// Use setTimeout() to collapse value changes in multiple children into a single
				// update to my value.   Multiple updates will occur on:
				//	1. Form.set()
				//	2. Form.reset()
				//	3. user selecting a radio button (which will de-select another radio button,
				//		 causing two onChange events)
				if(_this._onChangeDelayTimer){
					clearTimeout(_this._onChangeDelayTimer);
				}
				_this._onChangeDelayTimer = setTimeout(function(){
					delete _this._onChangeDelayTimer;
					_this._set("value", _this.get("value"));
				}, 10);
			};
			dojo.forEach(
				dojo.filter(this._descendants, function(item){ return item.onChange; } ),
				function(widget){
					// When a child widget's value changes,
					// the efficient thing to do is to just update that one attribute in this.value,
					// but that gets a little complicated when a checkbox is checked/unchecked
					// since this.value["checkboxName"] contains an array of all the checkboxes w/the same name.
					// Doing simple thing for now.
					conns.push(_this.connect(widget, "onChange", onChange));

					// Disabling/enabling a child widget should remove it's value from this.value.
					// Again, this code could be more efficient, doing simple thing for now.
					watches.push(widget.watch("disabled", onChange));
				}
			);
		},

		startup: function(){
			this.inherited(arguments);

			// Initialize value and valid/invalid state tracking.  Needs to be done in startup()
			// so that children are initialized.
			this.connectChildren(true);

			// Make state change call onValidStateChange(), will be removed in 2.0
			this.watch("state", function(attr, oldVal, newVal){ this.onValidStateChange(newVal == ""); });
		},

		destroy: function(){
			this.disconnectChildren();
			this.inherited(arguments);
		}

	});

}

if(!dojo._hasResource["dijit._DialogMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._DialogMixin"] = true;
dojo.provide("dijit._DialogMixin");



dojo.declare("dijit._DialogMixin", null,
	{
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		attributeMap: dijit._Widget.prototype.attributeMap,

		execute: function(/*Object*/ formContents){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(){
			// summary:
			//		Finds focusable items in dialog,
			//		and sets this._firstFocusItem and this._lastFocusItem
			// tags:
			//		protected

			var elems = dijit._getTabNavigable(this.containerNode);
			this._firstFocusItem = elems.lowest || elems.first || this.closeButtonNode || this.domNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
		}
	}
);

}

if(!dojo._hasResource["dijit.TooltipDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TooltipDialog"] = true;
dojo.provide("dijit.TooltipDialog");






dojo.declare(
		"dijit.TooltipDialog",
		[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin],
		{
			// summary:
			//		Pops up a dialog that appears like a Tooltip

			// title: String
			// 		Description of tooltip dialog (required for a11y)
			title: "",

			// doLayout: [protected] Boolean
			//		Don't change this parameter from the default value.
			//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
			//		is never a child of a layout container, nor can you specify the size of
			//		TooltipDialog in order to control the size of an inner widget.
			doLayout: false,

			// autofocus: Boolean
			// 		A Toggle to modify the default focus behavior of a Dialog, which
			// 		is to focus on the first dialog element after opening the dialog.
			//		False will disable autofocusing. Default: true
			autofocus: true,

			// baseClass: [protected] String
			//		The root className to use for the various states of this widget
			baseClass: "dijitTooltipDialog",

			// _firstFocusItem: [private] [readonly] DomNode
			//		The pointer to the first focusable node in the dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_firstFocusItem: null,

			// _lastFocusItem: [private] [readonly] DomNode
			//		The pointer to which node has focus prior to our dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_lastFocusItem: null,

			templateString: dojo.cache("dijit", "templates/TooltipDialog.html", "<div role=\"presentation\" tabIndex=\"-1\">\n\t<div class=\"dijitTooltipContainer\" role=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" role=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" role=\"presentation\"></div>\n</div>\n"),

			_setTitleAttr: function(/*String*/ title){
				this.containerNode.title = title;
				this._set("title", title)
			},

			postCreate: function(){
				this.inherited(arguments);
				this.connect(this.containerNode, "onkeypress", "_onKey");
			},

			orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
				// summary:
				//		Configure widget to be displayed in given position relative to the button.
				//		This is called from the dijit.popup code, and should not be called
				//		directly.
				// tags:
				//		protected
				var newC = "dijitTooltipAB" + (corner.charAt(1) == 'L' ? "Left" : "Right")
						+ " dijitTooltip"
						+ (corner.charAt(0) == 'T' ? "Below" : "Above");
				
				dojo.replaceClass(this.domNode, newC, this._currentOrientClass || "");
				this._currentOrientClass = newC;
			},

			focus: function(){
				// summary:
				//		Focus on first field
				this._getFocusItems(this.containerNode);
				dijit.focus(this._firstFocusItem);
			},

			onOpen: function(/*Object*/ pos){
				// summary:
				//		Called when dialog is displayed.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected

				this.orient(this.domNode,pos.aroundCorner, pos.corner);
				this._onShow(); // lazy load trigger
			},

			onClose: function(){
				// summary:
				//		Called when dialog is hidden.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected
				this.onHide();
			},

			_onKey: function(/*Event*/ evt){
				// summary:
				//		Handler for keyboard events
				// description:
				//		Keep keyboard focus in dialog; close dialog on escape key
				// tags:
				//		private

				var node = evt.target;
				var dk = dojo.keys;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.containerNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				if(evt.charOrCode == dk.ESCAPE){
					// Use setTimeout to avoid crash on IE, see #10396.
					setTimeout(dojo.hitch(this, "onCancel"), 0);
					dojo.stopEvent(evt);
				}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else if(evt.charOrCode === dk.TAB){
					// we want the browser's default tab handling to move focus
					// but we don't want the tab to propagate upwards
					evt.stopPropagation();
				}
			}
		}
	);

}

if(!dojo._hasResource["dijit._CssStateMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._CssStateMixin"] = true;
dojo.provide("dijit._CssStateMixin");


dojo.declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	// hovering: [readonly] Boolean
	//		True if cursor is over this widget
	hovering: false,
	
	// active: [readonly] Boolean
	//		True if mouse was pressed while over this widget, and hasn't been released yet
	active: false,

	_applyAttributes: function(){
		// This code would typically be in postCreate(), but putting in _applyAttributes() for
		// performance: so the class changes happen before DOM is inserted into the document.
		// Change back to postCreate() in 2.0.  See #11635.

		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		dojo.forEach(["onmouseenter", "onmouseleave", "onmousedown"], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);
		
		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		dojo.forEach(["disabled", "readOnly", "checked", "selected", "focused", "state", "hovering", "active"], function(attr){
			this.watch(attr, dojo.hitch(this, "_setStateClass"));
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly/checked/selected so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets hovering and active properties depending on mouse state,
		//	which triggers _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._set("hovering", true);
					this._set("active", this._mouseDown);
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._set("hovering", false);
					this._set("active", false);
					break;

				case "mousedown" :
					this._set("active", true);
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						this._mouseDown = false;
						this._set("active", false);
						this.disconnect(mouseUpConnector);
					});
					break;
			}
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Incomplete - ValidationTextBox sets this.state to "Incomplete" if the current input value is not finished yet
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this.active, this.hovering) and from focus manager (this.focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this.active){
				multiply("Active");
			}else if(this.hovering){
				multiply("Hover");
			}
		}

		if(this._focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		dojo.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			dojo.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because dojo.toggleClass() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = dojo.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			dojo.toggleClass(node, clazz+"Hover", hovering && !active && !disabled);
			dojo.toggleClass(node, clazz+"Active", active && !disabled);
			dojo.toggleClass(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn("onmousedown", function(){
			active = true;
			setClass();
		});
		cn("onmouseup", function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.watch("disabled", setClass);
		this.watch("readOnly", setClass);
	}
});

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");






dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: [const] String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		value: "focusNode",
		id: "focusNode",
		tabIndex: "focusNode",
		alt: "focusNode",
		title: "focusNode"
	}),

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "_onMouseDown");
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this._set("disabled", value);
		dojo.attr(this.focusNode, 'disabled', value);
		if(this.valueNode){
			dojo.attr(this.valueNode, 'disabled', value);
		}
		dijit.setWaiState(this.focusNode, "disabled", value);

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._set("hovering", false);
			this._set("active", false);

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex : "focusNode";
			dojo.forEach(dojo.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(dojo.isWebKit || dijit.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');
				}
			}, this);
		}else{
			if(this.tabIndex != ""){
				this.focusNode.setAttribute('tabIndex', this.tabIndex);
			}
		}
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.  Use set('disabled', ...) instead.
		dojo.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	_onFocus: function(e){
		if(this.scrollOnFocus){
			dojo.window.scrollIntoView(this.domNode);
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.  Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled){
			dijit.focus(this.focusNode);
		}
	},

	compare: function(/*anything*/ val1, /*anything*/ val2){
		// summary:
		//		Compare 2 values (as returned by get('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(newValue){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==false,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		this._pendingOnChange = this._pendingOnChange
			|| (typeof newValue != typeof this._lastValueReported)
			|| (this.compare(newValue, this._lastValueReported) != 0);
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) && this._pendingOnChange){
			this._lastValueReported = newValue;
			this._pendingOnChange = false;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					clearTimeout(this._onChangeHandle);
				}
				// setTimout allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = setTimeout(dojo.hitch(this,
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}), 0); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			clearTimeout(this._onChangeHandle);
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.  Use set('value', ...) instead.
		dojo.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.  Use get('value') instead.
		dojo.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},
	
	_onMouseDown: function(e){
		// If user clicks on the button, even if the mouse is released outside of it,
		// this button should get focus (to mimics native browser buttons).
		// This is also needed on chrome because otherwise buttons won't get focus at all,
		// which leads to bizarre focus restore on Dialog close etc.
		if(!e.ctrlKey && dojo.mouseButtons.isLeft(e) && this.isFocusable()){ // !e.ctrlKey to ignore right-click on mac
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
				if (this.isFocusable()) {
					this.focus();
				}
				this.disconnect(mouseUpConnector);
			});
		}
	}
});

dojo.declare("dijit.form._FormValueWidget", dijit.form._FormWidget,
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.  See #8484, #8660.
	// TODO: unclear what that {value: ""} is for; FormWidget.attributeMap copies value to focusNode,
	// so maybe {value: ""} is so the value *doesn't* get copied to focusNode?
	// Seems like we really want value removed from attributeMap altogether
	// (although there's no easy way to do that now)

	// readOnly: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "readOnly".
	//		Similar to disabled except readOnly form values are submitted.
	readOnly: false,

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "",
		readOnly: "focusNode"
	}),

	_setReadOnlyAttr: function(/*Boolean*/ value){
		dojo.attr(this.focusNode, 'readOnly', value);
		dijit.setWaiState(this.focusNode, "readonly", value);
		this._set("readOnly", value);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(dojo.isIE < 9 || (dojo.isIE && dojo.isQuirks)){ // IE won't stop the event with keypress
			this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
		}
		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._lastValueReported = this._resetValue = this.value;
		}
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Hook so set('value', value) works.
		// description:
		//		Sets the value of the widget.
		//		If the value has changed, then fire onChange event, unless priorityChange
		//		is specified as null (or false?)
		this._handleOnChange(newValue, priorityChange);
	},

	_handleOnChange: function(/*anything*/ newValue, /*Boolean?*/ priorityChange){
		// summary:
		//		Called when the value of the widget has changed.  Saves the new value in this.value,
		//		and calls onChange() if appropriate.   See _FormWidget._handleOnChange() for details.
		this._set("value", newValue);
		this.inherited(arguments);
	},

	undo: function(){
		// summary:
		//		Restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time
		this._hasBeenBlurred = false;
		this._setValueAttr(this._resetValue, true);
	},

	_onKeyDown: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
			var te;
			if(dojo.isIE){
				e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
				te = document.createEventObject();
				te.keyCode = dojo.keys.ESCAPE;
				te.shiftKey = e.shiftKey;
				e.srcElement.fireEvent('onkeypress', te);
			}
		}
	},

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(dojo.isIE == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(e){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

}

if(!dojo._hasResource["dijit._HasDropDown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._HasDropDown"] = true;
dojo.provide("dijit._HasDropDown");



dojo.declare("dijit._HasDropDown",
	null,
	{
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the popupActive class on.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.
		//		Any dropdown taller than this will have scrollbars.
		//		Set to 0 for no max height, or -1 to limit height to available space in viewport
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subwidget
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon

			if(this.disabled || this.readOnly){ return; }

			// Prevent default to stop things like text selection, but don't stop propogation, so that:
			//		1. TimeTextBox etc. can focusthe <input> on mousedown
			//		2. dropDownButtonActive class applied by _CssStateMixin (on button depress)
			//		3. user defined onMouseDown handler fires
			e.preventDefault();

			this._docHandler = this.connect(dojo.doc, "onmouseup", "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		dropDown node.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser <select> nodes:
			//			1. mouse down on the select node (probably on the arrow)
			//			2. move mouse to a menu item while holding down the mouse button
			//			3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = dojo.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(dojo.hasClass(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = dijit.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened && dropDown.focus && dropDown.autoFocus !== false){
				// Focus the dropdown widget - do it on a delay so that we
				// don't steal our own focus.
				window.setTimeout(dojo.hitch(dropDown, "focus"), 1);
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// the drop down was already opened on mousedown/keydown; just need to call stopEvent()
			if(this._stopClickEvents){
				dojo.stopEvent(e);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);

			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			dojo.addClass(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			// summary:
			//		set up nodes and connect our mouse and keypress events

			this.inherited(arguments);

			this.connect(this._buttonNode, "onmousedown", "_onDropDownMouseDown");
			this.connect(this._buttonNode, "onclick", "_onDropDownClick");
			this.connect(this.focusNode, "onkeypress", "_onKey");
			this.connect(this.focusNode, "onkeyup", "_onKeyUp");
		},

		destroy: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }

			var d = this.dropDown, target = e.target;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){
					/* false return code means that the drop down handled the key */
					dojo.stopEvent(e);
					return;
				}
			}
			if(d && this._opened && e.charOrCode == dojo.keys.ESCAPE){
				this.closeDropDown();
				dojo.stopEvent(e);
			}else if(!this._opened &&
					(e.charOrCode == dojo.keys.DOWN_ARROW ||
						( (e.charOrCode == dojo.keys.ENTER || e.charOrCode == " ") &&
						  //ignore enter and space if the event is for a text input
						  ((target.tagName || "").toLowerCase() !== 'input' ||
						     (target.type && target.type.toLowerCase() !== 'text'))))){
				// Toggle the drop down, but wait until keyup so that the drop down doesn't
				// get a stray keyup event, or in the case of key-repeat (because user held
				// down key for too long), stray keydown events
				this._toggleOnKeyUp = true;
				dojo.stopEvent(e);
			}
		},

		_onKeyUp: function(){
			if(this._toggleOnKeyUp){
				delete this._toggleOnKeyUp;
				this.toggleDropDown();
				var d = this.dropDown;	// drop down may not exist until toggleDropDown() call
				if(d && d.focus){
					setTimeout(dojo.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			// Don't focus on button if the user has explicitly focused on something else (happens
			// when user clicks another control causing the current popup to close)..
			// But if focus is inside of the drop down then reset focus to me, because IE doesn't like
			// it when you display:none a node with focus.
			var focusMe = dijit._curFocus && this.dropDown && dojo.isDescendant(dijit._curFocus, this.dropDown.domNode);

			this.closeDropDown(focusMe);

			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns whether or not the dropdown is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/* Function */ loadCallback){
			// summary:
			//		Loads the data for the dropdown, and at some point, calls
			//		the given callback.   This is basically a callback when the
			//		user presses the down arrow button to open the drop down.
			// tags:
			//		protected

			loadCallback();
		},

		toggleDropDown: function(){
			// summary:
			//		Callback when the user presses the down arrow button or presses
			//		the down arrow key to open/close the drop down.
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			if(!this._opened){
				// If we aren't loaded, load it first so there isn't a flicker
				if(!this.isLoaded()){
					this.loadDropDown(dojo.hitch(this, "openDropDown"));
					return;
				}else{
					this.openDropDown();
				}
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit.popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit.popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				this._preparedNode = true;
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				dojo.style(ddNode, myStyle);
				
				// Figure out maximum height allowed (if there is a height restriction)
				var maxHeight = this.maxHeight;
				if(maxHeight == -1){
					// limit height to space available in viewport either above or below my domNode
					// (whichever side has more room)
					var viewport = dojo.window.getBox(),
						position = dojo.position(aroundNode, false);
					maxHeight = Math.floor(Math.max(position.y, viewport.h - (position.y + position.h)));
				}

				// Attach dropDown to DOM and make make visibility:hidden rather than display:none
				// so we call startup() and also get the size
				if(dropDown.startup && !dropDown._started){
					dropDown.startup();
				}

				dijit.popup.moveOffScreen(dropDown);
				// Get size of drop down, and determine if vertical scroll bar needed
				var mb = dojo._getMarginSize(ddNode);
				var overHeight = (maxHeight && mb.h > maxHeight);
				dojo.style(ddNode, {
					overflowX: "hidden",
					overflowY: overHeight ? "auto" : "hidden"
				});
				if(overHeight){
					mb.h = maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = aroundNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, aroundNode.offsetWidth);
				}else{
					delete mb.w;
				}
				
				// And finally, resize the dropdown to calculated height and width
				if(dojo.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					dojo.marginBox(ddNode, mb);
				}
			}

			var retVal = dijit.popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: dijit.getPopupAroundAlignment((this.dropDownPosition && this.dropDownPosition.length) ? this.dropDownPosition : ["below"],this.isLeftToRight()),
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					dojo.attr(self._popupStateNode, "popupActive", false);
					dojo.removeClass(self._popupStateNode, "dijitHasDropDownOpen");
					self._opened = false;
				}
			});
			dojo.attr(this._popupStateNode, "popupActive", "true");
			dojo.addClass(self._popupStateNode, "dijitHasDropDownOpen");
			this._opened=true;

			// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// focus:
			//		If true, refocuses the button widget
			// tags:
			//		protected

			if(this._opened){
				if(focus){ this.focus(); }
				dijit.popup.close(this.dropDown);
				this._opened = false;
			}
		}

	}
);

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");





dojo.declare("dijit.form.Button",
	dijit.form._FormWidget,
	{
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button dojoType="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Text to display in button.
	//		If the label is hidden (showLabel=false) then and no title has
	//		been specified, then label is also set as title attribute of icon.
	label: "",

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "",

	// type: String
	//		Defines the type of button.  "button", "submit", or "reset".
	type: "button",

	baseClass: "dijitButton",

	templateString: dojo.cache("dijit.form", "templates/Button.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode,focusNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdojoAttachPoint=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "valueNode"
	}),

	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			return false;
		}
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary:
		//		Handler when the user activates the button portion.
		if(this._onClick(e) === false){ // returning nothing is same as true
			e.preventDefault(); // needed for checkbox
		}else if(this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a nonform widget needs to be signalled
			for(var node=this.domNode; node.parentNode/*#5935*/; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					break;
				}
			}
		}else if(this.valueNode){
			this.valueNode.click();
			e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
		}
	},

	buildRendering: function(){
		this.inherited(arguments);
		dojo.setSelectable(this.focusNode, false);
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		// TODO: remove the method in 2.0, parser will do it all for me
		if(source && (!this.params || !("label" in this.params))){
			this.set('label', source.innerHTML);
		}
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			dojo.toggleClass(this.containerNode, "dijitDisplayNone", !val);
		}
		this._set("showLabel", val);
	},

	onClick: function(/*Event*/ e){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_clicked: function(/*Event*/ e){
		// summary:
		//		Internal overridable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		dojo.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for set('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this._set("label", content);
		this.containerNode.innerHTML = content;
		if(this.showLabel == false && !this.params.title){
			this.titleNode.title = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	},

	_setIconClassAttr: function(/*String*/ val){
		// Custom method so that icon node is hidden when not in use, to avoid excess padding/margin
		// appearing around it (even if it's a 0x0 sized <img> node)

		var oldVal = this.iconClass || "dijitNoIcon",
			newVal = val || "dijitNoIcon";
		dojo.replaceClass(this.iconNode, newVal, oldVal);
		this._set("iconClass", val);
	}
});


dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container, dijit._HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
	// |		<div dojotype="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	dojo.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: dojo.cache("dijit.form", "templates/DropDownButton.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\" dojoAttachPoint=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"focusNode,titleNode,_arrowWrapperNode\"\n\t\t\trole=\"button\" aria-haspopup=\"true\" aria-labelledby=\"${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdojoAttachPoint=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdojoAttachPoint=\"containerNode,_popupStateNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\" tabIndex=\"-1\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown && this.dropDownContainer){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		if(this.dropDown){
			dijit.popup.hide(this.dropDown);
		}

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!!dropDown && (!dropDown.href || dropDown.isLoaded));
	},

	loadDropDown: function(){
		// Loads our dropdown
		var dropDown = this.dropDown;
		if(!dropDown){ return; }
		if(!this.isLoaded()){
			var handler = dojo.connect(dropDown, "onLoad", this, function(){
				dojo.disconnect(handler);
				this.openDropDown();
			});
			dropDown.refresh();
		}else{
			this.openDropDown();
		}
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button dojoType="dijit.form.ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div dojoType="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: dojo.cache("dijit.form", "templates/ComboButton.html", "<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' role=\"presentation\"\n\t><tbody role=\"presentation\"><tr role=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" dojoAttachPoint=\"buttonNode\" dojoAttachEvent=\"ondijitclick:_onButtonClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode\"\n\t\t\trole=\"button\" aria-labelledby=\"${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\" role=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdojoAttachPoint=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdojoAttachEvent=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\trole=\"button\" aria-haspopup=\"true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" role=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" role=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" dojoAttachPoint=\"valueNode\"\n\t\t/></td></tr></tbody\n></table>\n"),

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		id: "",
		tabIndex: ["focusNode", "titleNode"],
		title: "titleNode"
	}),

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			dijit.focus(this._popupStateNode);
			dojo.stopEvent(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			dijit.focus(this.titleNode);
			dojo.stopEvent(evt);
		}
	},
	
	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			dijit.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary:
	//		A button that can be in two states (checked or not).
	//		Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		checked:"focusNode"
	}),

	_clicked: function(/*Event*/ evt){
		this.set('checked', !this.checked);
	},

	_setCheckedAttr: function(/*Boolean*/ value, /*Boolean?*/ priorityChange){
		this._set("checked", value);
		dojo.attr(this.focusNode || this.domNode, "checked", value);
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", value);
		this._handleOnChange(value, priorityChange);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary:
		//		Deprecated.  Use set('checked', true/false) instead.
		dojo.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
		this.set('checked', checked);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

}

if(!dojo._hasResource["dijit.form.DropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.DropDownButton"] = true;
dojo.provide("dijit.form.DropDownButton");




}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");



/*=====
dojo.__cookieProps = function(){
	//	expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes. << FIXME: 0 seems to disappear right away? FF3.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
	this.expires = expires;
	this.path = path;
	this.domain = domain;
	this.secure = secure;
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary:
	//		Get or set a cookie.
	//	description:
	// 		If one argument is passed, returns the value of the cookie
	// 		For two or more arguments, acts as a setter.
	//	name:
	//		Name of the cookie
	//	value:
	//		Value for the cookie
	//	props:
	//		Properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null, {expires: -1});
	var c = document.cookie;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + dojo.regexp.escapeString(name) + "=([^;]*)"));
		return matches ? decodeURIComponent(matches[1]) : undefined; // String or undefined
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
};

dojo.cookie.isSupported = function(){
	//	summary:
	//		Use to determine if the current browser supports cookies or not.
	//
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

}

if(!dojo._hasResource["dijit.layout.BorderContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.BorderContainer"] = true;
dojo.provide("dijit.layout.BorderContainer");





dojo.declare(
	"dijit.layout.BorderContainer",
	dijit.layout._LayoutWidget,
{
	// summary:
	//		Provides layout in up to 5 regions, a mandatory center with optional borders along its 4 sides.
	//
	// description:
	//		A BorderContainer is a box with a specified size, such as style="width: 500px; height: 500px;",
	//		that contains a child widget marked region="center" and optionally children widgets marked
	//		region equal to "top", "bottom", "leading", "trailing", "left" or "right".
	//		Children along the edges will be laid out according to width or height dimensions and may
	//		include optional splitters (splitter="true") to make them resizable by the user.  The remaining
	//		space is designated for the center region.
	//
	//		The outer size must be specified on the BorderContainer node.  Width must be specified for the sides
	//		and height for the top and bottom, respectively.  No dimensions should be specified on the center;
	//		it will fill the remaining space.  Regions named "leading" and "trailing" may be used just like
	//		"left" and "right" except that they will be reversed in right-to-left environments.
	//
	//		For complex layouts, multiple children can be specified for a single region.   In this case, the
	//		layoutPriority flag on the children determines which child is closer to the edge (low layoutPriority)
	//		and which child is closer to the center (high layoutPriority).   layoutPriority can also be used
	//		instead of the design attribute to conrol layout precedence of horizontal vs. vertical panes.
	// example:
	// |	<div dojoType="dijit.layout.BorderContainer" design="sidebar" gutters="false"
	// |            style="width: 400px; height: 300px;">
	// |		<div dojoType="dijit.layout.ContentPane" region="top">header text</div>
	// |		<div dojoType="dijit.layout.ContentPane" region="right" splitter="true" style="width: 200px;">table of contents</div>
	// |		<div dojoType="dijit.layout.ContentPane" region="center">client area</div>
	// |	</div>

	// design: String
	//		Which design is used for the layout:
	//			- "headline" (default) where the top and bottom extend
	//				the full width of the container
	//			- "sidebar" where the left and right sides extend from top to bottom.
	design: "headline",

	// gutters: [const] Boolean
	//		Give each pane a border and margin.
	//		Margin determined by domNode.paddingLeft.
	//		When false, only resizable panes have a gutter (i.e. draggable splitter) for resizing.
	gutters: true,

	// liveSplitters: [const] Boolean
	//		Specifies whether splitters resize as you drag (true) or only upon mouseup (false)
	liveSplitters: true,

	// persist: Boolean
	//		Save splitter positions in a cookie.
	persist: false,

	baseClass: "dijitBorderContainer",

	// _splitterClass: String
	// 		Optional hook to override the default Splitter widget used by BorderContainer
	_splitterClass: "dijit.layout._Splitter",

	postMixInProperties: function(){
		// change class name to indicate that BorderContainer is being used purely for
		// layout (like LayoutContainer) rather than for pretty formatting.
		if(!this.gutters){
			this.baseClass += "NoGutter";
		}
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }
		dojo.forEach(this.getChildren(), this._setupChild, this);
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget._setupChild().

		var region = child.region;
		if(region){
			this.inherited(arguments);

			dojo.addClass(child.domNode, this.baseClass+"Pane");

			var ltr = this.isLeftToRight();
			if(region == "leading"){ region = ltr ? "left" : "right"; }
			if(region == "trailing"){ region = ltr ? "right" : "left"; }

			// Create draggable splitter for resizing pane,
			// or alternately if splitter=false but BorderContainer.gutters=true then
			// insert dummy div just for spacing
			if(region != "center" && (child.splitter || this.gutters) && !child._splitterWidget){
				var _Splitter = dojo.getObject(child.splitter ? this._splitterClass : "dijit.layout._Gutter");
				var splitter = new _Splitter({
					id: child.id + "_splitter",
					container: this,
					child: child,
					region: region,
					live: this.liveSplitters
				});
				splitter.isSplitter = true;
				child._splitterWidget = splitter;

				dojo.place(splitter.domNode, child.domNode, "after");

				// Splitters aren't added as Contained children, so we need to call startup explicitly
				splitter.startup();
			}
			child.region = region;	// TODO: technically wrong since it overwrites "trailing" with "left" etc.
		}
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		this._layoutChildren();
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Override _LayoutWidget.addChild().
		this.inherited(arguments);
		if(this._started){
			this.layout(); //OPT
		}
	},

	removeChild: function(/*dijit._Widget*/ child){
		// Override _LayoutWidget.removeChild().

		var region = child.region;
		var splitter = child._splitterWidget
		if(splitter){
			splitter.destroy();
			delete child._splitterWidget;
		}
		this.inherited(arguments);
		
		if(this._started){
			this._layoutChildren();
		}
		// Clean up whatever style changes we made to the child pane.
		// Unclear how height and width should be handled.
		dojo.removeClass(child.domNode, this.baseClass+"Pane");
		dojo.style(child.domNode, {
			top: "auto",
			bottom: "auto",
			left: "auto",
			right: "auto",
			position: "static"
		});
		dojo.style(child.domNode, region == "top" || region == "bottom" ? "width" : "height", "auto");
	},

	getChildren: function(){
		// Override _LayoutWidget.getChildren() to only return real children, not the splitters.
		return dojo.filter(this.inherited(arguments), function(widget){
			return !widget.isSplitter;
		});
	},

	// TODO: remove in 2.0
	getSplitter: function(/*String*/region){
		// summary:
		//		Returns the widget responsible for rendering the splitter associated with region
		// tags:
		//		deprecated
		return dojo.filter(this.getChildren(), function(child){
			return child.region == region;
		})[0]._splitterWidget;
	},

	resize: function(newSize, currentSize){
		// Overrides _LayoutWidget.resize().

		// resetting potential padding to 0px to provide support for 100% width/height + padding
		// TODO: this hack doesn't respect the box model and is a temporary fix
		if(!this.cs || !this.pe){
			var node = this.domNode;
			this.cs = dojo.getComputedStyle(node);
			this.pe = dojo._getPadExtents(node, this.cs);
			this.pe.r = dojo._toPixelValue(node, this.cs.paddingRight);
			this.pe.b = dojo._toPixelValue(node, this.cs.paddingBottom);

			dojo.style(node, "padding", "0px");
		}

		this.inherited(arguments);
	},

	_layoutChildren: function(/*String?*/ changedChildId, /*Number?*/ changedChildSize){
		// summary:
		//		This is the main routine for setting size/position of each child.
		// description:
		//		With no arguments, measures the height of top/bottom panes, the width
		//		of left/right panes, and then sizes all panes accordingly.
		//
		//		With changedRegion specified (as "left", "top", "bottom", or "right"),
		//		it changes that region's width/height to changedRegionSize and
		//		then resizes other regions that were affected.
		// changedChildId:
		//		Id of the child which should be resized because splitter was dragged.
		// changedChildSize:
		//		The new width/height (in pixels) to make specified child

		if(!this._borderBox || !this._borderBox.h){
			// We are currently hidden, or we haven't been sized by our parent yet.
			// Abort.   Someone will resize us later.
			return;
		}

		// Generate list of wrappers of my children in the order that I want layoutChildren()
		// to process them (i.e. from the outside to the inside)
		var wrappers = dojo.map(this.getChildren(), function(child, idx){
			return {
				pane: child,
				weight: [
					child.region == "center" ? Infinity : 0,
					child.layoutPriority,
					(this.design == "sidebar" ? 1 : -1) * (/top|bottom/.test(child.region) ? 1 : -1),
					idx
				]
			};
		}, this);
		wrappers.sort(function(a, b){
			var aw = a.weight, bw = b.weight;
			for(var i=0; i<aw.length; i++){
				if(aw[i] != bw[i]){
					return aw[i] - bw[i];
				}
			}
			return 0;
		});

		// Make new list, combining the externally specified children with splitters and gutters
		var childrenAndSplitters = [];
		dojo.forEach(wrappers, function(wrapper){
			var pane = wrapper.pane;
			childrenAndSplitters.push(pane);
			if(pane._splitterWidget){
				childrenAndSplitters.push(pane._splitterWidget);
			}
		});

		// Compute the box in which to lay out my children
		var dim = {
			l: this.pe.l,
			t: this.pe.t,
			w: this._borderBox.w - this.pe.w,
			h: this._borderBox.h - this.pe.h
		};

		// Layout the children, possibly changing size due to a splitter drag
		dijit.layout.layoutChildren(this.domNode, dim, childrenAndSplitters,
			changedChildId, changedChildSize);
	},

	destroyRecursive: function(){
		// Destroy splitters first, while getChildren() still works
		dojo.forEach(this.getChildren(), function(child){
			var splitter = child._splitterWidget;
			if(splitter){
				splitter.destroy();
			}
			delete child._splitterWidget;
		});

		// Then destroy the real children, and myself
		this.inherited(arguments);
	}
});

// This argument can be specified for the children of a BorderContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// region: [const] String
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Values: "top", "bottom", "leading", "trailing", "left", "right", "center".
	//		See the `dijit.layout.BorderContainer` description for details.
	region: '',

	// layoutPriority: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Children with a higher layoutPriority will be placed closer to the BorderContainer center,
	//		between children with a lower layoutPriority.
	layoutPriority: 0,

	// splitter: [const] Boolean
	//		Parameter for child of `dijit.layout.BorderContainer` where region != "center".
	//		If true, enables user to resize the widget by putting a draggable splitter between
	//		this widget and the region=center widget.
	splitter: false,

	// minSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a minimum size (in pixels) for this widget when resized by a splitter.
	minSize: 0,

	// maxSize: [const] Number
	//		Parameter for children of `dijit.layout.BorderContainer`.
	//		Specifies a maximum size (in pixels) for this widget when resized by a splitter.
	maxSize: Infinity
});

dojo.declare("dijit.layout._Splitter", [ dijit._Widget, dijit._Templated ],
{
	// summary:
	//		A draggable spacer between two items in a `dijit.layout.BorderContainer`.
	// description:
	//		This is instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create it directly.
	// tags:
	//		private

/*=====
 	// container: [const] dijit.layout.BorderContainer
 	//		Pointer to the parent BorderContainer
	container: null,

	// child: [const] dijit.layout._LayoutWidget
	//		Pointer to the pane associated with this splitter
	child: null,

	// region: [const] String
	//		Region of pane associated with this splitter.
	//		"top", "bottom", "left", "right".
	region: null,
=====*/

	// live: [const] Boolean
	//		If true, the child's size changes and the child widget is redrawn as you drag the splitter;
	//		otherwise, the size doesn't change until you drop the splitter (by mouse-up)
	live: true,

	templateString: '<div class="dijitSplitter" dojoAttachEvent="onkeypress:_onKeyPress,onmousedown:_startDrag,onmouseenter:_onMouse,onmouseleave:_onMouse" tabIndex="0" role="separator"><div class="dijitSplitterThumb"></div></div>',

	postMixInProperties: function(){
		this.inherited(arguments);

		this.horizontal = /top|bottom/.test(this.region);
		this._factor = /top|left/.test(this.region) ? 1 : -1;
		this._cookieName = this.container.id + "_" + this.region;
	},

	buildRendering: function(){
		this.inherited(arguments);

		dojo.addClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V"));

		if(this.container.persist){
			// restore old size
			var persistSize = dojo.cookie(this._cookieName);
			if(persistSize){
				this.child.domNode.style[this.horizontal ? "height" : "width"] = persistSize;
			}
		}
	},

	_computeMaxSize: function(){
		// summary:
		//		Return the maximum size that my corresponding pane can be set to

		var dim = this.horizontal ? 'h' : 'w',
			childSize = dojo.marginBox(this.child.domNode)[dim],
			center = dojo.filter(this.container.getChildren(), function(child){ return child.region == "center";})[0],
			spaceAvailable = dojo.marginBox(center.domNode)[dim];	// can expand until center is crushed to 0

		return Math.min(this.child.maxSize, childSize + spaceAvailable);
	},

	_startDrag: function(e){
		if(!this.cover){
			this.cover = dojo.doc.createElement('div');
			dojo.addClass(this.cover, "dijitSplitterCover");
			dojo.place(this.cover, this.child.domNode, "after");
		}
		dojo.addClass(this.cover, "dijitSplitterCoverActive");

		// Safeguard in case the stop event was missed.  Shouldn't be necessary if we always get the mouse up.
		if(this.fake){ dojo.destroy(this.fake); }
		if(!(this._resize = this.live)){ //TODO: disable live for IE6?
			// create fake splitter to display at old position while we drag
			(this.fake = this.domNode.cloneNode(true)).removeAttribute("id");
			dojo.addClass(this.domNode, "dijitSplitterShadow");
			dojo.place(this.fake, this.domNode, "after");
		}
		dojo.addClass(this.domNode, "dijitSplitterActive dijitSplitter" + (this.horizontal ? "H" : "V") + "Active");
		if(this.fake){
			dojo.removeClass(this.fake, "dijitSplitterHover dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover");
		}

		//Performance: load data info local vars for onmousevent function closure
		var factor = this._factor,
			isHorizontal = this.horizontal,
			axis = isHorizontal ? "pageY" : "pageX",
			pageStart = e[axis],
			splitterStyle = this.domNode.style,
			dim = isHorizontal ? 'h' : 'w',
			childStart = dojo.marginBox(this.child.domNode)[dim],
			max = this._computeMaxSize(),
			min = this.child.minSize || 20,
			region = this.region,
			splitterAttr = region == "top" || region == "bottom" ? "top" : "left",	// style attribute of splitter to adjust
			splitterStart = parseInt(splitterStyle[splitterAttr], 10),
			resize = this._resize,
			layoutFunc = dojo.hitch(this.container, "_layoutChildren", this.child.id),
			de = dojo.doc;

		this._handlers = (this._handlers || []).concat([
			dojo.connect(de, "onmousemove", this._drag = function(e, forceResize){
				var delta = e[axis] - pageStart,
					childSize = factor * delta + childStart,
					boundChildSize = Math.max(Math.min(childSize, max), min);

				if(resize || forceResize){
					layoutFunc(boundChildSize);
				}
				// TODO: setting style directly (usually) sets content box size, need to set margin box size
				splitterStyle[splitterAttr] = delta + splitterStart + factor*(boundChildSize - childSize) + "px";
			}),
			dojo.connect(de, "ondragstart", dojo.stopEvent),
			dojo.connect(dojo.body(), "onselectstart", dojo.stopEvent),
			dojo.connect(de, "onmouseup", this, "_stopDrag")
		]);
		dojo.stopEvent(e);
	},

	_onMouse: function(e){
		var o = (e.type == "mouseover" || e.type == "mouseenter");
		dojo.toggleClass(this.domNode, "dijitSplitterHover", o);
		dojo.toggleClass(this.domNode, "dijitSplitter" + (this.horizontal ? "H" : "V") + "Hover", o);
	},

	_stopDrag: function(e){
		try{
			if(this.cover){
				dojo.removeClass(this.cover, "dijitSplitterCoverActive");
			}
			if(this.fake){ dojo.destroy(this.fake); }
			dojo.removeClass(this.domNode, "dijitSplitterActive dijitSplitter"
				+ (this.horizontal ? "H" : "V") + "Active dijitSplitterShadow");
			this._drag(e); //TODO: redundant with onmousemove?
			this._drag(e, true);
		}finally{
			this._cleanupHandlers();
			delete this._drag;
		}

		if(this.container.persist){
			dojo.cookie(this._cookieName, this.child.domNode.style[this.horizontal ? "height" : "width"], {expires:365});
		}
	},

	_cleanupHandlers: function(){
		dojo.forEach(this._handlers, dojo.disconnect);
		delete this._handlers;
	},

	_onKeyPress: function(/*Event*/ e){
		// should we apply typematic to this?
		this._resize = true;
		var horizontal = this.horizontal;
		var tick = 1;
		var dk = dojo.keys;
		switch(e.charOrCode){
			case horizontal ? dk.UP_ARROW : dk.LEFT_ARROW:
				tick *= -1;
//				break;
			case horizontal ? dk.DOWN_ARROW : dk.RIGHT_ARROW:
				break;
			default:
//				this.inherited(arguments);
				return;
		}
		var childSize = dojo._getMarginSize(this.child.domNode)[ horizontal ? 'h' : 'w' ] + this._factor * tick;
		this.container._layoutChildren(this.child.id, Math.max(Math.min(childSize, this._computeMaxSize()), this.child.minSize));
		dojo.stopEvent(e);
	},

	destroy: function(){
		this._cleanupHandlers();
		delete this.child;
		delete this.container;
		delete this.cover;
		delete this.fake;
		this.inherited(arguments);
	}
});

dojo.declare("dijit.layout._Gutter", [dijit._Widget, dijit._Templated],
{
	// summary:
	// 		Just a spacer div to separate side pane from center pane.
	//		Basically a trick to lookup the gutter/splitter width from the theme.
	// description:
	//		Instantiated by `dijit.layout.BorderContainer`.  Users should not
	//		create directly.
	// tags:
	//		private

	templateString: '<div class="dijitGutter" role="presentation"></div>',

	postMixInProperties: function(){
		this.inherited(arguments);
		this.horizontal = /top|bottom/.test(this.region);
	},

	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "dijitGutter" + (this.horizontal ? "H" : "V"));
	}
});

}

if(!dojo._hasResource["com.ibm.bpm.social.util.baseTextDirection.BidiTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.baseTextDirection.BidiTextBox"] = true;
/*
 * IBM Confidential
 *
 * IBM Rational Focal Point
 *
 * Copyright IBM Corporation 2010. All Rights Reserved.
 */

// Input widget that addresses dynamic Bidi complex expressions
// (e.g. URIs, emails).
//
// @author bidi_bdl_lkemmel
//

//------------------------------------------------------------------------------
// BidiTextBox class and commonly used bidi utils
//------------------------------------------------------------------------------

//==============================================================================
// CONSTANTS
//==============================================================================

dojo.provide("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox = {};

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF = navigator.userAgent.toLowerCase().indexOf("firefox") >= 0;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE = !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsFF && navigator.appName.toLowerCase().indexOf("microsoft") >= 0;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants = {};

// Direction constants
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR = 1;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR = 2;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR = 3;

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM = "\u200e";
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM = "\u200f";

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE = '\u202A';
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE = '\u202B';

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF = '\u202C';

// Regex for a left-to-right character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR = new RegExp("[a-zA-Z" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + "]");

// Keep these distinct, if we would need to support some language specific feature
// Regex for a Hebrew character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR = new RegExp("[\\u05be\\u05c0\\u05c3\\u05c6\\u05d0-\\u05f4"
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + "\\ufb1d\\ufb1f-\\ufb28\\ufb2a-\\ufb4f]");

// Regex for an Arabic character
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR = new RegExp("[\\u061b\\u0621-\\u064a\\u066d-\\u066f\\u0671-\\u06d5"
        + "\\u06dd\\u06e5\\u06e6\\u06ee\\u06ef\\u06fa-\\u070d\\u0710\\u0712-\\u072f"
        + "\\u074d-\\u07a5\\u07b1\\ufb50-\\ufd3d\\ufd50-\\ufdfc\\ufe70-\\ufefc]");

// Regex for control character (only address those we utilize)
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR = new RegExp("[" + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM
        + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE + "]", "m");

// Delimiting strings to create Bidi segments
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;

//==============================================================================
// com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory namespace
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory = {

//==============================================================================
// Extends the given element with the properties of BidiTextBox.
//==============================================================================
    subclass: function(id, direction, delimiters) {
        this.subclassElement(document.getElementById(id), direction, delimiters);
    },

    subclassElement: function(element, direction, delimiters) {
        if (element && "BidiTextBox" != element.subclass) {
            element = new com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox(element, direction, delimiters);
        }
    }
};

//==============================================================================
// BidiTextBox class
//==============================================================================

//==============================================================================
// BidiTextBox constructor
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox = function(element, direction, delimiters) {
    for (var property in com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype) {
        try {
            if (!element[property]) {
                element[property] = this[property];
            }
        } catch(e) {
            return element;
        }
    }
    element.delimiters = delimiters;

    if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR) {
        element.dir = "";
        element.isContextualDir = true;
        element.init();
    } else {
        if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RTL_DIR) {
            element.dir = "rtl";
        } else if (direction == com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR) {
            element.dir = "ltr";
        }
        if (delimiters) {
            element.init();
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.push(element);
        }
    }
    return element;
}

// Static members
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.subclass = "BidiTextBox";

//==============================================================================
// Init BidiTextBox object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.init = function() {
    this.connectListeners();

    var value = this.normalizeContent(false);
    this.formatContent(value, true);
};

if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {

//==============================================================================
// Connect event listeners for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        // this.attachEvent("onfocus", this.onFocusIn);
        // this.attachEvent("onkeyup", this.onKeyUp);
        // this.attachEvent("oncut", this.onCut);
        // this.attachEvent("onpaste", this.onPaste);
        // this.attachEvent("ondrop", this.onPaste);
        // window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        // if (this.delimiters) {
            // this.attachEvent("onkeydown", this.onKeyDown);
            // this.attachEvent("oncopy", this.onCopy);
            // this.attachEvent("onmouseup", this.onMouseUp);
            // this.attachEvent("oncontextmenu", this.onContextMenu);
            // this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        // }
		
		this.attachEvent("onfocus", dojo.hitch(this, this.onFocusIn));
        this.attachEvent("onkeyup", dojo.hitch(this, this.onKeyUp));
        this.attachEvent("oncut", dojo.hitch(this, this.onCut));
        this.attachEvent("onpaste", dojo.hitch(this, this.onPaste));
        this.attachEvent("ondrop", dojo.hitch(this, this.onPaste));
        window.attachEvent("onunload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload);
        if (this.delimiters) {
            this.attachEvent("onkeydown", dojo.hitch(this, this.onKeyDown));
            this.attachEvent("oncopy", dojo.hitch(this, this.onCopy));
            this.attachEvent("onmouseup", dojo.hitch(this, this.onMouseUp));
            this.attachEvent("oncontextmenu", dojo.hitch(this, this.onContextMenu));
            this.attachEvent("onsubmit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit);
        }
    };

//==============================================================================
// Copy event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();

        var range = document.selection.createRange();
        if (range && range.text) {
            window.clipboardData.setData('Text', range.text.replace(/[\u200e\u200f]/g, ''));
            event = event || window.event;
            event.cancelBubble = true;
            event.returnValue = false;
            return false;
        }
    };

//==============================================================================
// Cut event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCut = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onCopy(event);

        var range = document.selection.createRange();
        if (range) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.hideText();
            range.text = "";
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };

//==============================================================================
// Paste event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onPaste = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.update(false);"
                + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();", 0);
    };

//==============================================================================
// Mouseup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onMouseUp = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    };

//==============================================================================
// Contextmenu event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onContextMenu = function(event) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();

        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem;
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = setInterval(
                "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem.update(false)", 1000);
    };

//==============================================================================
// Keydown event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
        if (!event.ctrlKey && !event.altKey) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.getSelection();
        }
    };
//==============================================================================
// Keyup event handler for IE.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyUp = function(event) {
        if (this.subclass != "BidiTextBox") {
            if(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == null || 
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == undefined) {
        	var inputs = document.getElementsByTagName("input");
        	for(var i=0; i<inputs.length; i++) {
        	  var input = inputs[i];
        	    if(input.name == "reportName") {
        	        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = input;
        	        break;
    		     }
        	}
            }
            return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.onKeyUp(event);
        }
        if (!event) {
            event = window.event;
        }
        if (!event.altKey) {
            if (!event.ctrlKey) {
                // 'Home', left arrow and right arrow key code values
                NAVIGATION_KEYS = [36, 37, 39];
                i = NAVIGATION_KEYS.length;
                while (i-- > 0) {
                    if (event.keyCode == NAVIGATION_KEYS[i]) {
                        this.getSafePosition();
                        return;
                    }
                }
            }
            this.update(false);
        }
    };

//==============================================================================
// Validates caret position after a key or mouse event occurs.
// The caret will not be placed between a Bidi formatting code and the
// associated character. To accomplish this, the caret may be moved 1 position
// forward or backwards, depending on navigation direction.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSafePosition = function() {
        if (!this.selection) {
            return;
        }
        var value = this.getValue();
        if (!value) {
            return;
        }
        var startMove, endMove;
        var prevSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (!this.selection) {
            return;
        }
        startMove = this.selection[0] - prevSelection[0];
        endMove = this.selection[1] - prevSelection[1];
    
        // First do the end. We may need to move both end and start (when selection
        // is collapsed), so have the start be updated also.
        if (startMove) {
            if (!this.isMovingOverControl(value, this.selection[0], startMove)) {
                startMove = 0;
            } else {
                startMove = startMove > 0 ? 1 : -1;
                do {
                    this.selection[0] = Math.max(0, this.selection[0] + startMove);
                } while (this.isMovingOverControl(value, this.selection[0], startMove));
                startMove = this.selection[0] - prevSelection[0];
            }
        }
        if (endMove && this.isMovingOverControl(value, this.selection[1], endMove)) {
                endMove = endMove > 0 ? 1 : -1;
            var textLen = value.length;
            do {
                this.selection[1] = Math.min(textLen, this.selection[1] + endMove);
            } while (this.isMovingOverControl(value, this.selection[1], endMove));
    
            if (this.selection[0] > this.selection[1]) {
                this.selection[0] = this.selection[1];
            }
            this.setSelection();
    
        } else if (startMove) {
            if (this.selection[0] > this.selection[1]) {
                this.selection[1] = this.selection[0];
                this.setSelection();
            } else {
                this.moveSelectionStart(startMove);
            }
        }
    };

//==============================================================================
// Checks if the caret is moving over a control character.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.isMovingOverControl = function(value, position, direction) {
        if (direction < 0) {
            return position - 1 >= 0
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position));
        }
        if (direction > 0) {
            return position + 1 <= this.getValue().length
                && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(position - 1));
        }
        return false;
    };

//==============================================================================
// Moves selection start point, leaving the end intact.
// MSIE-specific.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.moveSelectionStart = function(value) {
        if (this.selection) {
            var range = document.selection.createRange();
            if (range) {
                range.moveStart('character', value);
                range.select();
            }
        }
    };

} else {

//==============================================================================
// Connect event listeners for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.connectListeners = function() {
        this.addEventListener("focus", this.onFocusIn, false);
        this.addEventListener("change", this.update, true);
        this.addEventListener("input", this.update, true);
        this.addEventListener("drop", this.update, false);
		
		//If a div element
		if(this.localName == "div") {
			this.addEventListener("keyup", this.update, true);
		}
		
        window.addEventListener("unload", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload, true);
        if (this.delimiters) {
            this.addEventListener("keydown", this.onKeyDown, true);

            this.setAttribute("oncopy", ';');
            if (typeof this["oncopy"] == 'function') {
                this.removeAttribute("oncopy");
                this.addEventListener("copy", this.onCopy, true);
            }
            this.setAttribute("oncut", ';');
            if (typeof this["oncut"] == 'function') {
                this.removeAttribute("oncut");
                this.addEventListener("cut", this.onCopy, true);
            }
            window.addEventListener("submit", com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit, false);
        }
    };

//==============================================================================
// Copy and cut event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onCopy = function(event) {
        this.getSelection();
        if (this.selection && this.selectionStart != this.selectionEnd) {
            this.normalizeContent(true);

            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };
    
//==============================================================================
// Keydown event handler for Firefox.
//==============================================================================
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        // older Firefox of version < 3
        if (event.ctrlKey && !event.shiftKey && !event.altKey
                && (event.keyCode == event.DOM_VK_C && typeof this["oncopy"] != 'function'
                || event.keyCode == event.DOM_VK_X && typeof this["oncut"] != 'function')) {
            this.onCopy(event);
        } else {
            this.getSelection();
        }
    };
}

//==============================================================================
// Sets global Bidi element to this object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.onFocusIn = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == this
        || document.activeElement && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem == document.activeElement) {
        return;
    }
    if (this.subclass != "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
        var target = event.srcElement || event.target;
        if (target && target.subclass == "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox") {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = target;
        }
    } else {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    }
};

//==============================================================================
// Inserts controls characters after delimiters and adjusts locally saved
// selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.formatContent = function(value, applyValue) {
    if (!value) {
        value = this.getValue();
    }
    if (this.isContextualDir) {
        this.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(0, value);
    }
    if (!this.delimiters) {
        return value;
    }
    if (this.dir == "ltr" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(value)
        || this.dir == "rtl" && !com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(value)) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var CONTROL_STR = this.dir == "rtl" ? com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM_STRING
        : com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM_STRING;
    CONTROL_STR_LEN = CONTROL_STR.length;

    var newStr = new String(value);
    var i, j, c;
    var start = 0, end = -1;
    var afterStrong = false;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length - 1; i++, j++) {
        c = value.charAt(i);
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
            afterStrong = this.dir == "ltr";
        } else if (c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR)) {
            afterStrong = this.dir == "rtl";
        } else if (afterStrong && this.delimiters.indexOf(c) >= 0) {
            newStr = newStr.substr(0, j + 1) + CONTROL_STR + newStr.substr(j + 1);
            afterStrong = false;
            if (this.selection) {
                if (this.selection[0] == i) {
                    ++start;
                } else if (this.selection[0] > i) {
                    start += CONTROL_STR_LEN;
                }
                if (this.selection[1] == i) {
                    ++end;
                } else if (this.selection[1] > i) {
                    end += CONTROL_STR_LEN;
                }
            }
            j += CONTROL_STR_LEN;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.setValue(newStr, true);
    }
    this.unhideText();
    return newStr;
};

//==============================================================================
// Gets and caches the current native selection.
// Returns integer array containing the selection start and end points.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getSelection = function() {
    if (typeof this.selectionStart == "number") {
        this.selection = [this.selectionStart, this.selectionEnd];
    } else if (document.selection) {
        try {
            var start = 0;
            var range = document.selection.createRange().duplicate();
            var range2 = range.duplicate();
            var rangeLength = range.text.length;
            
            if (this.isMultiLine) {
                range2.moveToElementText(this);
            } else {
                range2.expand('textedit');
            }
            // Return value of range#moveStart doesn't seem reliable, so
            // also limit number of iterations.
            // Otherwise we can fall into infinite loop.
            var n = range2.text.length;
            while (start < n && range.compareEndPoints('StartToStart', range2) > 0) {
                if (range.text.charAt(0).match(/[\r\n]/)) {
                    if (!range.moveStart("word", -1)) {
                        break;
                    }
                } else if (!range.moveStart('character', -1)) {
                    break;
                }
                ++start;
            }
            this.selection = [start, start + rangeLength];
        } catch (e) {
            this.selection = null;
        }
    }
};

//==============================================================================
// Gets text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.getValue = function() {
	return this.value || this.text || this.textContent || this.innerHTML || "";
};

//==============================================================================
// Form submit event handler.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.handleSubmit = function(event) {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate();
    }
    this.resetSelection();
    this.normalizeContent(true);
};

//==============================================================================
// Sets text color to the background color making that way the text invisible.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.hideText = function() {
    if (typeof this.foreground == 'undefined') {
        this.foreground = this.style.color;
    }
    this.style.color = this.style.backgroundColor || "white";
};

//==============================================================================
// Removes controls characters and adjusts locally saved selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.normalizeContent = function(applyValue) {
    var value = this.getValue();
    if (!value || !this.delimiters) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var newStr = new String(value);
    var i, j;
    var start = 0, end = -1;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length; i++) {
        if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar(value.charAt(i))) {
            newStr = newStr.substr(0, j) + newStr.substr(j + 1);
            if (this.selection) {
                if (this.selection[0] > i) {
                    --start;
                }
                if (this.selection[1] > i) {
                    --end;
                }
            }
        } else {
            ++j;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.hideText();
        this.setValue(newStr, true);
    }
    return newStr;
};

//==============================================================================
// Resets our private selection object.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.resetSelection = function() {
    this.selection = null;
};

//==============================================================================
// Sets the native selection.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setSelection = function() {
    if (!this.selection || this.readOnly) {
        return;
    }
    if (this.setSelectionRange) {
        if (this.selectionStart != this.selection[0] || this.selectionEnd != this.selection[1]) {
            this.focus();
            this.setSelectionRange(this.selection[0], this.selection[1]);
        }
    } else if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.gIsIE) {
        var newSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (newSelection[0] != this.selection[0] || newSelection[1] != this.selection[1]) {
            var range = this.createTextRange() || document.body.createTextRange();
            if (range) {
                if (this.isMultiLine) {
                    range.moveToElementText(item);
                } else {
                    range.expand('textedit');
                }
                range.collapse(true);
                range.moveEnd('character', newSelection[1]);
                range.moveStart('character', newSelection[0]);
                range.select();
            }
        }
    }
};

//==============================================================================
// Sets the text value.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.setValue = function(newValue, setSelection) {
    if (!newValue) {
        newValue = this.newValue;
    }
    if (newValue != this.getValue()) {
        if (this.text) {
            this.text = newValue;
        } else if (this.value) {
            this.value = newValue;
        }
        if (setSelection) {
            this.setSelection();
        }
    }
};

//==============================================================================
// Resets element's text color.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.unhideText = function() {
    this.style.color = this.foreground;
};

//==============================================================================
// Updates the text value though normalizing and subsequent formatting.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.prototype.update = function(delay) {
        // On load FF fires focus on body element, and as a result
        // BidiTextBox.gCurrentItem can be uninitialized.
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = this;
    this.getSelection();
	
    var origValue = this.getValue();
    this.newValue = this.formatContent(this.normalizeContent(false), false);
	
    if (this.newValue != origValue) {
        this.hideText();
        if (delay == true) {
            setTimeout("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.setValue(null, true);"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.unhideText();"
                    + "com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem.resetSelection();", 0);
        } else {
            this.setValue(this.newValue, true);
            this.unhideText();
            this.resetSelection();
        }
    }
};

//==============================================================================
// Handle submit events in this window.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleSubmit = function(event) {
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes.length; i-- > 0; ) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes[i].handleSubmit(event);
    }
}

//==============================================================================
// Handle window unload.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleUnload = function(event) {
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gBidiNodes = [];
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gCurrentItem = null;
}

//==============================================================================
// Stops item update invocation.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.stopUpdate = function() {
    if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval) {
        clearInterval(BidiTextBox.gInterval);
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gInterval = null;
    }
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiTextBox.gUpdatedItem = null;
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.HEBREW_CHAR.test(text)
            || com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.ARABIC_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains LTR characters.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText = function(text) {
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiControlChar = function(c) {
    return c.match(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTROL_CHAR);
}

//==============================================================================
// Returns direction resolved to LTR or RTL.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection = function(dir, value) {
    if (dir) {
        return dir;
    }
    if (value) {
        for (var i = 0, n = value.length; i < n; i++) {
            var c = value.charAt(i);
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.isBidiText(c)) {
                return "rtl";
            }
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.hasLtrText(c)) {
                return "ltr";
            }
        }
    }
    return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2);
}

//==============================================================================
// Extends text controls with Bidi properties and applies base text direction
// for both dynamic and static elements.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.handleBidi = function() {
    //var visibility = document.body.style.visibility;
    //document.body.style.visibility = 'hidden';
    var nodes = document.getElementsByTagName('input');
    for (i = nodes.length; i-- > 0;) {
        if (/^text/.test(nodes[i].type)) {
            if (/bidi_url/.test(nodes[i].className)) {
                if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
                    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(nodes[i],
                        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LTR_DIR, ".:@\\/");
                }
            } else {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
            }
        }
    }
    nodes = document.getElementsByTagName('textarea');
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection(nodes[i]);
    }
    nodes = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements();
    for (i = nodes.length; i-- > 0;) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(nodes[i]);
    }
    //document.body.style.visibility = visibility;
}

//==============================================================================
// Applies base text direction for dynamic element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setDynamicElementDirection = function(node) {
    if (node && !node.dir) {
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir = getBidiDir(0);
        if (bidiTextDir) {
            node.dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.bidiTextDir;
        } else if (typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory == 'object') {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
        }
    }
}

//==============================================================================
// Applies base text direction for static element.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection = function(node) {
    if (!node || /bidi_url/.test(node.className)) {
        return;
    }
    if (node.nodeType == '3' && /[^\s]+/.test(node.nodeValue)) {
        var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection(
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(0), node.nodeValue);
        if (dir == 'ltr') {
            if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'rtl') {
                com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "ltrRightAlign");
            } else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM);
            }
        } else {
        	if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir(2) == 'ltr') {
        		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection(node, "rtlLeftAlign");
        	} else {
            	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses(node, 
					com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM);
            }
        }
    } else {
        for (var i = node.childNodes.length; i-- > 0;) {
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setStaticElementDirection(node.childNodes[i]);
        }
    }
}

// Code added to handle the case of English then Arabic then ( in LTR
// Or Arabic then English then ( in RTL
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixParentheses = function(node, markup) {
	node.nodeValue = node.nodeValue.replace(/\(/g, markup + '('); 
}

//==============================================================================
// Applies base text direction for a text node.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setTextNodeDirection = function(node, className) {
    if (node) {
        if (node.nodeValue) {
            node.nodeValue = node.nodeValue.replace(/[\u00a0]$/gm, "");
        }
        node = node.parentNode;
        if (node && node.tagName && node.tagName.toUpperCase() != 'SCRIPT') {
            // When there is no block-level element parented to this node,
            // use immediate parent anyway
            var parent = node;
            while (node && com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle(node, 'display') == 'inline') {
                node = node.parentNode; 
            }
            if (node) {
                parent = node;
            }            
            if (!/ltrRightAlign|rtlLeftAlign/.test(parent.className)) {
                parent.className += (parent.className ? " " : "") + className;
            }
        }
    }
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveStyle = function(node, style) {
    var gcs = "getComputedStyle" in window;
    return (gcs ? window.getComputedStyle(node, null) : node.currentStyle).getPropertyValue(style); 
}


com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES = {
	bidiText: 'bidiText'
 };
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES = ['TR', 'TD', 'a', 'h1'];

//==============================================================================
// Retreives document static elements enabled for bidi.
//==============================================================================
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getStaticBidiElements = function() {
    var bidiNodes = [];
    for (var i = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES.length; i-- > 0;) {
        var nodes = document.getElementsByTagName(com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.TAG_NAMES[i]);
        for (var j = nodes.length; j-- > 0;) {
            var classNames = nodes[j].className.split(' ');
            for (var k = classNames.length; k-- > 0;) {
                if (com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.CLASS_NAMES[classNames[k]]) {
                    bidiNodes.push(nodes[j]);
                    break;
                }
            }
        }
    }
    return bidiNodes;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getBidiDir = function(which) {
        
	// which = 0, to get the text direction explicitly defined by the user (LTR or RTL or default)
	// If user setting is contextual, we return nothing... And the JS will check for buffer content
	// If the text direction can not be determined from the buffer content
	// The JS will call which=2, to get the default direction for user language preference
	// 
	// As we'll always use contextual mode, we always return an empty string when which=0
	if (which == 0) return '';

			
	// which = 2, to return the default direction for user language preference
	// Currently we do not support RTL languages, so this will always return "ltr"
	if (which == 2) return 'ltr';
}

/**
 * ============================================================================================================================================
 * BELOW ARE ALL EDITS CONTRIBUTED AFTER THE PORT OF THIS FILE TO EVEREST
 * This includes pretty much all of the changes which we had to make to get the file working in 
 * our environment, as well as some small snippets which help consolidate actions.
 * ============================================================================================================================================
 */ 

/**
 * This method accepts text as a parameter and determines the proper BTD depending on content
 * and user setting
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD = function(text) {
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	
	if(btdpref != "ctx") {
		return btdpref;
	} else {
		if(typeof text != "undefined" && text != "") {
			var calcbtd = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveTextDirection("", text);
			if(calcbtd == "") {
				//something went wrong
				return "default";
			} else {
				return calcbtd;
			}
		} else {
			//should pass in a valid string, but if no string, obviously doesn't matter.
			return "default";
		}
	}
}

/**
 * This method takes in a node, and sets the appropriate BTD value to the "dir" attribute, depending
 * on the user preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD = function(node) {

	if(typeof node == "undefined") {
		return;
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(node.innerHTML);
	if(btdpref != "default") {
		node.setAttribute("dir", btdpref);
        if(btdpref == "rtl") {
            var text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(node.innerHTML);
			if (node.innerText) {
				node.innerText = text;
			} else {
				node.innerHTML = text;
			}
        }
	}
}

/**
 * This method takes in text, and returns the appropriate BTD
 * value for the "dir" attribute, depending on the user
 * preference, and the innerHTML of the element.
 *
 * This method is intended to be used on elements which contain textual content as their innerHTML.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD = function(text) {

    var dir = "";
	if(text == "") {
		return dir;
	}	
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
	if(btdpref != "default") {
		dir = btdpref;
	}
    return dir;
}

/**
 * This method abstracts the process of setting an input elements BTD based on the user preference
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD = function(node) {
	
	if(typeof node == "undefined" || node == null) {
		return;
	}
	
	//Since we are guaranteed a page refresh before user expects BTD user setting to take effect,
	//cache this, because it's an expensive lookup.
	if(typeof com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD == "undefined") {
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD = 
			com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
	}
	var btdpref = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.userPrefBTD;
	if(btdpref == "ctx") {
		//contextual requires bidiTextBox to be used or auto if WebKit browsers
		if(!com.ibm.bpm.social.util.common.Util.isChrome) {
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiElemFactory.subclassElement(node,
				com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.CONTEXTUAL_DIR, "");
		} else {
			//Webkit supports the "auto" dir attribute
			node.dir = "auto";
		}
	} else if (btdpref != "default") {
		//either ltr or rtl
		node.dir = btdpref;
	}
}

/**
 * This method inputs text and adds Unicode Control Characters
 * (UCC) based on the user preference.  Returns the text with
 * UCC added.
 */
com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText = function(text) {
    var dir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.resolveProperBTD(text);
    if(dir != "default") {
        //set the appropriate control character
        var controlChar = (dir == "ltr") ? 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
            com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

        //wrap the html in the control characters
        text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;

        if (dir == "rtl") {
            text = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis(text);
        }
    }
    return text;
}

com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.fixRTLParenthesis = function(text) {
    var newTxt = text;
            //fixParenthesis, fix colon
    newTxt = newTxt.replace(/\(/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + '('); 
    newTxt = newTxt.replace(/\:/g, com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM + ':'); 

    return newTxt;
}


}

if(!dojo._hasResource["com.ibm.bpm.social.Aether"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.Aether"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.Aether");






dojo.declare(
	"com.ibm.bpm.social.Aether",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	
    templateString:"<div class='social-portal-notification-instance' dojoAttachEvent='onmouseover:_doMouseOver, onmouseout:_doMouseOut, onclick:_doClick'>\n\t<div class = 'social-portal-notification-window' dojoAttachPoint ='window'>\n\t\t<div class='social-portal-notification-close' dojoAttachPoint='closeIcon' dojoAttachEvent='onmouseover:_doIconMouseOver,onmouseout:_doIconMouseOut,onclick:_doIconClick'>&times;</div>\n\t\t<table>\n\t\t\t<tr>\n\t\t\t\t<td class='social-portal-notification-icon-align'>\n\t\t\t\t\t<div class='social-portal-notification-icon' dojoAttachPoint='icon'></div>\n\t\t\t\t</td>\n\t\t\t\t<td>\n\t\t\t\t\t<div class='social-portal-notification-message' dojoAttachPoint='message'>${message}</div>\n\t\t\t\t</td>\n\t\t\t</tr>\n\t\t</table>\n\t\t\n\t</div>\n</div>\n",
	
	containerDivId : "aether",
	
    message: null,
	
	img_src: dojo.moduleUrl("dojo", "resources/blank.gif"),

	eventParam: null,

    timeout: 5000,
	
    timer: null,
		
	_container: null,
	
	positionOptions: ['top-left', 'top-center', 'top-right', 'bottom-left', 'bottom-center', 'bottom-right'],
	
	position: null,
	
	width: null,
	
	fade: true,
	
	displayCloseIcon: false,
	
	constructor: function(params){
		
		dojo.mixin(this, params);
		
		if(!this.position) this.position = 'center';
		
		this.createContainer();
			
	},
	
	createContainer: function(){
		if(dojo.byId(this.containerDivId)){
			this.container = dojo.byId(this.containerDivId);
			this.container.className = 'social-portal-notification '+this.position;
		} else {
			this.container = dojo.create('div', {
				'id': this.containerDivId,
				'class': 'social-portal-notification '+this.position
			}, dojo.body());
		}
	},
	
	createIconImage:function(){
		var image = document.createElement('img');
		image.src = this.img_src;
		dojo.addClass(image, "bpm-social-generic-system-avatar");
		this.icon.appendChild(image);
		
		if(this.eventParam){
			try{
				if(this.eventParam.process.instance.comment.author.id){
					dojo.addClass(image, "bpm-social-default-avatar-medium");
					dojo.removeClass(image, "bpm-social-generic-system-avatar");
					image.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this.eventParam.process.instance.comment.author.id);
				}
			} catch(e){}

		}

	},
	
    postCreate: function() {
		dojo.style(this.domNode, 'opacity', 0);
		dojo.place(this.domNode, this.container);
		dojo.anim(this.domNode, {opacity: 1}, 250);
		
		if(this.width){
			dojo.style(this.domNode, 'width', this.width + 'px');
		}

		if(this.displayCloseIcon===false){
			dojo.style(this.closeIcon, 'display', 'none');
		}
		
		if(this.img_src)	{
			this.createIconImage();
		}
		
		if(this.fade===true){
			this.setTimer();
		}
	},
	
	setTimer: function() {
		this.timer = setTimeout(dojo.hitch(this, 'close'), this.timeout);
	},
	
	_doMouseOver: function(e){
		clearInterval(this.timer);
		dojo.addClass(this.domNode, 'hover');
	}, 
	
	_doMouseOut:function(e){
		dojo.removeClass(this.domNode, 'hover');
		this.setTimer();
	},
	
	_doClick: function(e){
		/*
		if(this.displayCloseIcon===false){
			//Only close the notification on a message click if the close icon is not displayed
			this.close();
		}
		*/
	},

	_doIconMouseOver: function(e){
		dojo.addClass(this.closeIcon, 'hover');		
	},
	
	_doIconMouseOut: function(e){
		dojo.removeClass(this.closeIcon, 'hover');
	},
	
	_doIconClick: function(e){
		this.close();
	},
	
	fireEvent:function(){
		//	parent defined
		console.debug("This function needs to be implemented by the parent.");
	},

	_handleActionClick:function(event){		
		this.fireEvent(event.target.id,this.eventParam);
	},

	close: function(){
		dojo.anim(this.domNode,{opacity: 0}, 500, null, dojo.hitch(this, 'remove'));
	},
	
	remove: function(){
		dojo.anim(this.domNode,{height: 0, margin: 0}, 250, null, dojo.partial(dojo.destroy, this.domNode));
		//TODO Look at destroying container if last notification
	}
	
});

}

if(!dojo._hasResource["com.ibm.bpm.social.portal.Topics"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.portal.Topics"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.portal.Topics");


com.ibm.bpm.social.portal.Topics.VIEW_READY = "com.ibm.bpm.social.portal.view.viewReady";

com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE = "com.ibm.bpm.social.portal.showProcessInstance";

com.ibm.bpm.social.portal.Topics.SHOW_LEGACY_PROCESS_INSTANCE = "com.ibm.bpm.social.portal.showLegacyProcessInstance";

com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION = "com.ibm.bpm.social.portal.showTaskCompletion";

com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST = "com.ibm.bpm.social.portal.showTasks";

com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD = "com.ibm.bpm.social.portal.showDashboard";

com.ibm.bpm.social.portal.Topics.DASHBOARD_LOADED = "com.ibm.bpm.social.portal.dashboardLoaded";

com.ibm.bpm.social.portal.Topics.MAXIMIZE_CONTENT = "com.ibm.bpm.social.portal.view.maximizeContent";

com.ibm.bpm.social.portal.Topics.RESTORE_CONTENT = "com.ibm.bpm.social.portal.view.restoreContent";

com.ibm.bpm.social.portal.Topics.REFRESH_FOLLOW_LIST = "com.ibm.bpm.social.portal.refreshFollowList";

com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST = "com.ibm.bpm.social.portal.refreshMentionsList";

com.ibm.bpm.social.portal.Topics.USER_PROFILE_CHANGED = "com.ibm.bpm.socialPortal.UserProfileChangedEvent"; //sic

com.ibm.bpm.social.portal.Topics.PROCESS_ACTIONS_AVAILABLE = "com.ibm.bpm.social.portal.processActionsAvailable";

com.ibm.bpm.social.portal.Topics.COACH_CANCELED = "com.ibm.bpm.social.portal.coachCanceled";

com.ibm.bpm.social.portal.Topics.CLOSE_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.closeProcessDiagram";

com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.refreshProcessDiagram";

com.ibm.bpm.social.portal.Topics.INSTANCE_DUE_DATE_UPDATED = "com.ibm.bpm.social.portal.instanceDueDateUpdated";

com.ibm.bpm.social.portal.Topics.TASK_UPDATED = "com.ibm.bpm.social.portal.taskUpdated";

com.ibm.bpm.social.portal.Topics.TASK_CLAIMED = "com.ibm.bpm.social.portal.taskClaimed";

com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM = "com.ibm.bpm.social.portal.openProcessDiagram";

com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED = "com.ibm.bpm.social.portal.taskListUpdated";

com.ibm.bpm.social.portal.Topics.SHOW_DIALOG = "com.ibm.bpm.social.portal.showDialog";

com.ibm.bpm.social.portal.Topics.HIDE_DIALOG = "com.ibm.bpm.social.portal.hideDialog";

com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED = "com.ibm.bpm.social.portal.sidebarWidgetChanged";

com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED = "com.ibm.bpm.social.portal.connectionsLoaded";

com.ibm.bpm.social.portal.Topics.UPDATE_DASHBOARD_TABS = "com.ibm.bpm.social.portal.updateDashboardTabs";

//See Util.js, com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB() for the structure of the object to be published with this topic

com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB = "com.ibm.bpm.social.portal.selectDashboardTab";

com.ibm.bpm.social.portal.Topics.NAVIGATION = "com.ibm.bpm.social.portal.navigation";

com.ibm.bpm.social.portal.Topics.PREVENT_NAVIGATION = "com.ibm.bpm.social.portal.preventNavigation";

com.ibm.bpm.social.portal.Topics.ALLOW_NAVIGATION = "com.ibm.bpm.social.portal.allowNavigation";

com.ibm.bpm.social.portal.Topics.FORCE_NEW_DASHBOARD = "com.ibm.bpm.social.portal.forceNewDashboard";



}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.Log"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.Log"] = true;
// Log.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

/**
 * These functions provide a logging facility that may determine the calling function
 * 
 * Usage:
 * 
 * To log/trace a string/message:
 * <li> com.ibm.bpm.social.util.common.Log.log(1,"x is larger ...")
 * 
 * To trace the entry of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logEntry(1)
 * 
 * To trace the exit of a function, add this as very first line. The function name should be determined
 * automatically
 * <li> com.ibm.bpm.social.util.common.Log.logExit(1)
 * 
 * To log objects:
 * <li> com.ibm.bpm.social.util.common.Log.logObject(integer pLogLevel, Object pObject, [String pTitle]);
 * 
 * Note: Third parameter is optional for example: com.ibm.bpm.social.util.common.Log.logObject(2, domNode,
 * "This is node hugo:");
 * 
 * 
 * Using the logError function, you may log an error to the console, and let the Log class determine the call
 * stack automatically
 * 
 * 
 * Output of the log may be adjusted/filtered using
 * <ul>
 * <li> com.ibm.bpm.social.util.common.Log.logLevel
 * <li> com.ibm.bpm.social.util.common.Log.includeClasses
 * <li> com.ibm.bpm.social.util.common.Log.excludeClasses
 * <li> com.ibm.bpm.social.util.common.Log.logArguments
 * 
 * 
 * Note about auto-resolving class names <br>
 * -------------------------------------
 * 
 * If you see the message "Couldn't resolve class name, might be within a constructor or invoked by
 * dojo.hitch" you should use the optional parameters cname while invoking the log functions. Example:
 * 
 * com.ibm.bpm.social.util.common.Log.logEntry(2, "", "TasksContainer");
 * 
 * 
 * Cases where you need this: - while a class is constructed (logging within constructor, postCreate) - while
 * logging in functions that were called using apply, call, dojo.hitch
 * 
 */


dojo.provide("com.ibm.bpm.social.util.common.Log");



/**
 * Logging level to provide some granularity/filtering Values:
 * <ul>
 * <li> 0 - No logging
 * <li> 1 - Log function entries and exits
 * <li> 2 - Additional data
 * <li> 3,.. - Much more data
 * </ul>
 */
// set 0 to just to initialize, will be set later according to the various settings, 
// see com.ibm.bpm.social.util.common.Log.determinLogContent()
com.ibm.bpm.social.util.common.Log.logLevel = 0;

/**
 * You may include only a subset in the output using includeClasses or excludeClasses
 * <p> - classes may be specified as regExp string
 * <p> - example using includeClasses : includeClasses = ["Selector", "com.ibm.bpm.social.util.common"]
 * <p> - to include all, just use ["*"]
 * <p> - exclude overrules include
 * <p> - to unset exclude or include use [] <p
 */

com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
com.ibm.bpm.social.util.common.Log.excludeClasses = [];


/**
 * Using this flag you can enable/disable the logEntry function to print out the arguments of the function
 */
com.ibm.bpm.social.util.common.Log.logArguments = true;


/**
 * Use this flag to also get info about the time when the log was written
 */
com.ibm.bpm.social.util.common.Log.logTime = false;

/**
 * Use this flag to also log also the date when the log was written
 */

com.ibm.bpm.social.util.common.Log.logDate = false;

/**
 * Use this flag to indent on each function call (when logEntry is called) and un-indent when leaving the
 * function (when logExit is called)
 * <p>
 * This works only if browser supports the console.group functionality
 */
/*@ignore@*/
com.ibm.bpm.social.util.common.Log.indentFunctions = (typeof console.group == "function" && !dojo.isIE);
/*@end@*/

com.ibm.bpm.social.util.common.Log.logEntriesAndExits = false;

/**
 * Determine the details of what and how to log
 * <ul>
 * <li> logLevel : (0 - 10) how much will be logged
 * <li> indentFunctions : (true|false) shall we track function calls ?
 * <li> logTime: (true|false) prefix log statements with a timestamp
 * <li> logDate: (true|false) prefix log statements with a timestamp including date
 * <li> logArguments : (true|false) print out the arguments of the function
 * <li> includeClasses : comma-separated string of class names to be included in the log
 * <li> excludeClasses : comma-separated string of class names to be excluded from the log
 * </ul>
 * 
 * @type {void}
 */
com.ibm.bpm.social.util.common.Log.determinLogContent =
  function(){

    if (com.ibm.bpm.social.debug.isLogging) { // allowClientSideLogging under the Mashups_ConfigService == true
      // logging to the Portal log console requested, provide as much details as possible
      com.ibm.bpm.social.util.common.Log.logLevel = 10;
      com.ibm.bpm.social.util.common.Log.logArguments = true;
      com.ibm.bpm.social.util.common.Log.logTime = true;
      com.ibm.bpm.social.util.common.Log.logDate = true;
      com.ibm.bpm.social.util.common.Log.includeClasses = ["*"];
      com.ibm.bpm.social.util.common.Log.excludeClasses = [];
      com.ibm.bpm.social.util.common.Log.indentFunctions = false;
    }
    else if ((typeof dojoConfig != "undefined") && dojoConfig.isDebug || (typeof djConfig != "undefined") &&
      djConfig.isDebug) {
      // typical developers setup, just use defaults
      com.ibm.bpm.social.util.common.Log.logLevel = 2;
      com.ibm.bpm.social.util.common.Log.logEntriesAndExits = true;
    }

    // check for query string settings 
    var paramString = window.location.search.replace("?", "");
    var paramKey = "";
    var paramValue = "";
    var value = "";
    var params = paramString.split("&");

    for ( var i = 0; i < params.length; i++) {
      paramKey = params[i].split("=")[0];
      paramValue = params[i].split("=")[1];

      if (paramKey && paramValue) {

        // logLevel
        if (paramKey == "logLevel") {
          value = Number(paramValue);
          if (!isNaN(value)) {
            com.ibm.bpm.social.util.common.Log.logLevel = value;
          }
        }
        // logArgument
        else if (paramKey == "logArguments") {
          com.ibm.bpm.social.util.common.Log.logArguments = (paramValue == "true");
        }
        // logDate
        else if (paramKey == "logDate") {
          com.ibm.bpm.social.util.common.Log.logTime = (paramValue == "true");
        }
        // logTime
        else if (paramKey == "logTime") {
          com.ibm.bpm.social.util.common.Log.logDate = (paramValue == "true");
        }
        // indentFunctions
        else if (paramKey == "indentFunctions") {
          com.ibm.bpm.social.util.common.Log.indentFunctions = (paramValue == "true");
        }
        // includeClasses
        else if (paramKey == "includeClasses") {
          com.ibm.bpm.social.util.common.Log.includeClasses = paramValue.split(",");
        }
        // excludeClasses
        else if (paramKey == "excludeClasses") {
          com.ibm.bpm.social.util.common.Log.excludeClasses = paramValue.split(",");
        }
        else if (paramKey == "logEntriesAndExits") {
          com.ibm.bpm.social.util.common.Log.logEntriesAndExits = (paramValue == "true");
        }
      }
    }


    // disable logging at all if no one requested it
    if (!com.ibm.bpm.social.debug.isLogging && !dojoConfig.isDebug && !djConfig.isDebug &&
      com.ibm.bpm.social.util.common.Log.logLevel === 0) {
      var emptyFunction = function(){};
      com.ibm.bpm.social.util.common.Log.log = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logEntry = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logExit = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logError = emptyFunction;
      com.ibm.bpm.social.util.common.Log.logObject = emptyFunction;
    }

    return;
  };


/**
 * Write a message to the console
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.log =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
      if (!pMessage) {
        pMessage = "";
      }

      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      com.ibm.bpm.social.util.common.Log.consoleLog(scope + " " + pMessage);
    }
  };

/**
 * Trace the entry of a method/function <br>
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */

com.ibm.bpm.social.util.common.Log.logEntry =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope =
        com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName, com.ibm.bpm.social.util.common.Log.logArguments);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroup(scope + " entry " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " entry " + pMessage);
      }
    }
  };


/**
 * Trace the exit of a method/function
 * 
 * @param {Number} pLogLevel the level of the message
 * @param {String} pMessage the message to write, may be null,
 * @param {String} cname - optional, use this if the class name of the invoking method can't be distinguished
 *        (e.g. use this in constructors)
 * @param {String} fktName - optional, use this if the function name of the invoking method can't be
 *        distinguished
 */
com.ibm.bpm.social.util.common.Log.logExit =
  function(/*integer*/pLogLevel, /*String */pMessage, /*String*/cname, /*String*/fktName){
    if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel && com.ibm.bpm.social.util.common.Log.logEntriesAndExits) {
      if (!pMessage) {
        pMessage = "";
      }
      var scope = com.ibm.bpm.social.util.common.Log._shouldTrace(cname, fktName);
      if (!scope) {
        return;
      }
      if (com.ibm.bpm.social.util.common.Log.indentFunctions) {
        com.ibm.bpm.social.util.common.Log.consoleGroupEnd(scope + " exit " + pMessage);
      }
      else {
        com.ibm.bpm.social.util.common.Log.consoleLog(scope + " exit " + pMessage);
      }
    }
  };


/**
 * Helper to determine if the message should be traced
 * 
 * @param {String} cname the class name, may be null
 * @param {String} fktName the function of the class, may be null
 * @param {String} includeArgs indicate if the args should be traced
 * @return boolean
 */
com.ibm.bpm.social.util.common.Log._shouldTrace =
  function(cname, fktName, includeArgs){

    var originFunction = arguments.callee.caller.caller;

    if ((cname == null || typeof (cname) == "undefined") || (fktName == null || typeof (fktName) == "undefined")) {
      // Note: the following cannot extracted to a utility function, since the arguments object will change
      var cls = null;

      if (originFunction._containingClass) {
        cname = originFunction._containingClass;
      }

      // works not for constructors
      if (originFunction.ctor && originFunction.ctor.prototype) {
        cls = originFunction.ctor.prototype.declaredClass;
      }
      if (cls) {
        var packages = cls.split(".");
        cname = packages[packages.length - 1];
      }
      else if (originFunction.name && (originFunction.name.lastIndexOf("_") > 0)) {
        var l = originFunction.name.split("_");
        // fktName = l.slice(0,l.length-1).join(_);
        cname = l[l.length - 1];
      }

      cname = cname || "<undefined>";

      if (originFunction.nom) {
        fktName = originFunction.nom;
      }
      else if (originFunction.name) {
        fktName = originFunction.name;
        fktName = fktName.replace("_" + cname, "");
      }

      fktName = fktName || "<undefined>";

      if (cname == "<undefined>" && fktName != "<undefined>" && typeof originFunction.toString == "function") {
        // in IE this is the only known way to get the class name
        // returns the whole function
        var originFunctionToString = originFunction.toString();
        // look if the function name is contained (if it was declared as function methodeName_className() it is)
        var iFktName = originFunctionToString.indexOf(fktName);
        if (iFktName != -1) {
          // find the first ( - that's directly after the class name
          var iFirstOpeningBracket = originFunctionToString.indexOf("(");
          // check that the function name does not come after the the first (, would mean it is contained anywhere in the function body
          if (iFirstOpeningBracket != -1 && iFirstOpeningBracket > iFktName) {
            // extract the class name
            cname = originFunctionToString.substring(iFktName + fktName.length + 1, iFirstOpeningBracket);
          }
        }
      }

      if (cname == "<undefined>") {
        com.ibm.bpm.social.util.common.Log
          .consoleWarn("Couldn't resolve classname, might be within a constructor or invoked by dojo.hitch. See Log.js to learn how to fix.");
      }
      else {
        originFunction._containingClass = cname;
      }
    }

    var include = com.ibm.bpm.social.util.common.Log.includeClasses;
    var exclude = com.ibm.bpm.social.util.common.Log.excludeClasses;
    var rc = cname + "." + fktName + "(";
    if (includeArgs === true && originFunction.arguments) {
      rc += com.ibm.bpm.social.util.common.Log._argumentsToString(originFunction.arguments);
    }
    rc += ")";


    if (include.length === 0 && exclude.length === 0) {
      return null;
    }

    for ( var i = 0; i < exclude.length; i++) {
      if (cname.search(exclude[i]) > -1) {
        return null;
      }
    }

    if (include[0] == "*") {
      return rc;
    }

    for (i = 0; i < include.length; i++) {
      if (cname.search(include[i]) > -1) {
        return rc;
      }
    }

    return false;
  };

/**
 * converts an array of arguments to a string
 * 
 * @param {String} args the arguments
 * @return {String} the formatted arguments
 * 
 */
com.ibm.bpm.social.util.common.Log._argumentsToString = function(/*Array*/args, /*int*/counter){
  var stringArgs = [];
  counter = counter || 0;
  if (args) {
    for ( var i = 0; i < args.length; i++) {
      if (args[i] === null) {
        stringArgs.push("null");
      }
      else if (typeof args[i] == "undefined") {
        stringArgs.push("undefined");
      }
      else if (typeof args[i] == "function") {
        stringArgs.push("[function]");
      }
      else if (typeof args[i] == "string" || typeof args[i] == "number" || typeof args[i] == "boolean") {
        // dojo.toJson automatically replaces line breaks with \n in strings and converts numbers and booleans to a string
        stringArgs.push(dojo.toJson(args[i]));
      }
      else if (typeof args[i] == "object") {
        if (dojo.isArrayLike(args[i])) {
          // to prevent recursion as arrays might reference themselve or contain cross references we have an counter with an exit condition
          if (counter < 3) {
            stringArgs.push("[" + com.ibm.bpm.social.util.common.Log._argumentsToString(args[i], counter + 1) + "]");
          }
          else {
            stringArgs.push("[array]");
          }
        }
        else if (typeof args[i].toString == "function") {
          stringArgs.push(args[i].toString());
        }
        else {
          stringArgs.push("[object]");
        }
      }
      else {
        stringArgs.push("[unknown]");
      }
    }
  }
  return stringArgs.join(", ");
};

/**
 * Log an error to the console, and let the Log class determine the call stack
 * 
 * @param {Number} pLogLevel
 * @param {String} pMethod Method or Message
 * @param {Object} xcp optional exception object
 */

com.ibm.bpm.social.util.common.Log.logError = function(/*integer*/pLogLevel, /*String*/pMethod, /*Object*/xcp){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length < 1) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logError"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logError"]));
    }

    var pMessage = pMethod ? pMethod : "";

    if (xcp) {
      pMessage = pMessage + " " + xcp.name + ":" + xcp.message;
      if (xcp.stack) {
        pMessage = pMessage + "\nStack:";
        var stack = xcp.stack.split("\n");
        var out = "";
        for ( var i = 0; i < stack.length; i++) {
          out = stack[i];
          if (out.length > 200) {
            // this is a eval'ed file instead of a function, which we don't wan to see in the console
            var file1 = out.match(/(\w*\.js)/) ? out.match(/(\w*\.js)/)[0] : "";
            var file2 = out.match(/@http.*$/) ? out.match(/@http.*$/)[0] : "";

            out = (file1 ? file1 : "");
            out += (file2 ? ":" + file2 : "");
            if (!out) {
              out = "undefined";
            }
          }
          pMessage = pMessage + "\n" + out;
        }
      }
    }
    com.ibm.bpm.social.util.common.Log.consoleError(pMessage);
  }
};

com.ibm.bpm.social.util.common.Log.logObject = function(/* integer */pLogLevel, /* Object */pObject, /* String */pTitle){
  if (com.ibm.bpm.social.util.common.Log.logLevel >= pLogLevel) {
    // check parameters
    if (arguments.length != 2 && arguments.length != 3) {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR, ["logObject"]));
    }
    if (typeof (arguments[0]) != "number") {
      throw new SyntaxError(dojo.string.substitute(com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR, ["first",
                                                                                                             "logObject"]));
    }

    if (pTitle) {
      com.ibm.bpm.social.util.common.Log.consoleDebug(pTitle);
    }
    /*@ignore@*/
    console.dir(pObject);
    /*@end@*/
  }
};

com.ibm.bpm.social.util.common.Log._getTS = function(){
  var d = new Date();
  var rc = "";
  var millis = d.getMilliseconds();
  millis = (millis.length == 2 ? "0" + millis : millis);
  if (com.ibm.bpm.social.util.common.Log.logDate) {
    rc = d.getFullYear() + "-" + ((d.getMonth() < 9) ? "0" : "") + (d.getMonth() + 1) + "-";
    rc = rc + ((d.getDate() < 9) ? "0" : "") + d.getDate() + " ";
  }
  rc = rc + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds() + "." + d.getMilliseconds() + " ";
  return rc;
};

// wrapper which might be used to handle browser issues

com.ibm.bpm.social.util.common.Log.consoleLog = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroup = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.group) {
    console.group(message);
  }
  else {
    console.log(message);
  }
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleGroupEnd = function(/* String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  if (console.groupEnd) {
    console.groupEnd();
  }
  console.log(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleDebug = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.debug(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleError = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.error(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleWarn = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.warn(message);
  /*@end@*/
};

com.ibm.bpm.social.util.common.Log.consoleInfo = function(/*String */message){
  if (com.ibm.bpm.social.util.common.Log.logTime) {
    message = com.ibm.bpm.social.util.common.Log._getTS() + message;
  }
  /*@ignore@*/
  console.info(message);
  /*@end@*/
};


/*
 * Do not translate
 */
com.ibm.bpm.social.util.common.Log.INVALID_NUMBER_ERROR =
  "Invalid number of parameters for com.ibm.bpm.social.util.common.Log.${0}";
com.ibm.bpm.social.util.common.Log.PARM_NO_NUMBER_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a number";
com.ibm.bpm.social.util.common.Log.PARM_NO_STRING_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a string";
com.ibm.bpm.social.util.common.Log.PARM_NO_OBJECT_ERROR =
  "Invalid type of ${0} parameter for com.ibm.bpm.social.util.common.Log.${1}. Must be a object";

// determine the log content to be displayed
com.ibm.bpm.social.util.common.Log.determinLogContent();

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation"] = true;
// SecureInnerHtmlPreparation.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation");



dojo
  .declare(
    "com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation",
    null,
    {

      /*static*/_cache: {},

      originalString: null,
      parsed: false,
      parsedString: null,

      constructor: function(originalString){
        com.ibm.bpm.social.util.common.Log.logEntry(2, null, "SecureInnerHtmlPreparation", "constructor");
        this.originalString = originalString;
        com.ibm.bpm.social.util.common.Log.logExit(2, null, "SecureInnerHtmlPreparation", "constructor");
      },

      /**
       * creates a secure inner HTML
       * 
       * @returns the secure inner HTML
       */
      getSecureString: function getSecureString_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this.originalString === null) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning null.");
          return null;
        }
        else if (this._cache[this.originalString]) {
          com.ibm.bpm.social.util.common.Log.logExit(2, "Returning cached value: " + this._cache[this.originalString]);
          return this._cache[this.originalString];
        }
        else if (!this.parsed) {
          this._parse();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "Returning: " + this.parsedString);
        return this.parsedString;
      },

      _parse: function _parse_SecureInnerHtmlPreparation(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.parsedString = "";

        // first step: tokenize the original string
        var tokens = [];

        var currentToken = null;
        var position = 0;

        while (position < this.originalString.length) {
          switch (this.originalString.charAt(position)) {
            case '<' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LT,
                "text": "<"
              };
              break;
            case '>' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_GT,
                "text": ">"
              };
              break;
            case '/' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SLASH,
                "text": "/"
              };
              break;
            case '\n' :
              if (currentToken !== null) {
                tokens.push(currentToken);
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case '\r' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK) {
                  tokens.push(currentToken);
                }
                else {
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK,
                "text": "\n"
              };
              break;
            case ' ' :
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_SPACE) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += " ";
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_SPACE,
                "text": " "
              };
              break;
            default:
              if (currentToken !== null) {
                if (currentToken.type != com.ibm.bpm.social.util.common.TOKEN_TEXT) {
                  tokens.push(currentToken);
                }
                else {
                  currentToken.text += this.originalString.charAt(position);
                  break;
                }
              }
              currentToken = {
                "type": com.ibm.bpm.social.util.common.TOKEN_TEXT,
                "text": this.originalString.charAt(position)
              };
              break;
          }

          position++;
        }
        if (currentToken !== null) {
          tokens.push(currentToken);
        }

        // Generate output
        this.parsedString = "";
        var containsHTML = false;
        var openTags = [];
        var tagName;

        for ( var i = 0; i < tokens.length; i++) {
          var token = tokens[i];
          var nextToken1 = i + 1 < tokens.length ? tokens[i + 1] : null;
          var nextToken2 = i + 2 < tokens.length ? tokens[i + 2] : null;
          var nextToken3 = i + 3 < tokens.length ? tokens[i + 3] : null;

          if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT) {
            // is there an allowed tag starting
            if (nextToken1 &&
              nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_TEXT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken1.text.toLowerCase()) > -1 &&
              nextToken2 &&
              (nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SPACE ||
                nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_SLASH || nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_GT) &&
              this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2) > -1) {

              tagName = nextToken1.text.toLowerCase();

              var nextSlash = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_SLASH, i + 2);
              var nextGt = this._getNextTokenPosition(tokens, com.ibm.bpm.social.util.common.TOKEN_GT, i + 2);

              if (nextSlash == -1 || nextGt < nextSlash) {
                containsHTML = true;
                this.parsedString += "<" + tagName + ">";
                i = nextGt;

                if (dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS, tagName) == -1) {
                  openTags.push(tagName);
                }
              }
              else if (nextSlash + 1 == nextGt) {
                containsHTML = true;
                this.parsedString += "<" + tagName + "/>";
                i = nextGt;
              }
              else {
                this.parsedString += "&lt;";
              }
            }
            // is there an allowed tag that is open ending
            else if (nextToken1 && nextToken1.type == com.ibm.bpm.social.util.common.TOKEN_SLASH && nextToken2 &&
              nextToken2.type == com.ibm.bpm.social.util.common.TOKEN_TEXT && nextToken3 &&
              nextToken3.type == com.ibm.bpm.social.util.common.TOKEN_GT &&
              dojo.indexOf(com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS, nextToken2.text.toLowerCase()) > -1 &&
              dojo.indexOf(openTags, nextToken2.text.toLowerCase()) > -1) {

              tagName = nextToken2.text.toLowerCase();

              // ensure to close all open tags that were opened after the corresponding opening tag to create valid HTML
              var closingTagName;
              do {
                closingTagName = openTags.pop();

                this.parsedString += "</" + closingTagName + ">";
              }
              while (closingTagName != tagName);

              i += 3;
            }
            else {
              this.parsedString += "&lt;";
            }
          }
          else if (token.type == com.ibm.bpm.social.util.common.TOKEN_GT) {
            this.parsedString += "&gt;";
          }
          else {
            this.parsedString += token.text;
          } // end if (token.type == com.ibm.bpm.social.util.common.TOKEN_LT)
        } // end for ( var i = 0; i < tokens.length; i++)

        // close all opened tags that were not closed to create valid HTML
        while (openTags.length > 0) {
          tagName = openTags.pop();
          this.parsedString += "</" + tagName + ">";
        }

        // if there is no HTML at all replace \n with <br>
        if (!containsHTML) {
          this.parsedString = this.parsedString.replace(com.ibm.bpm.social.util.common.REGEXP_LINEBREAK, "<br>");
        }

        this.parsed = true;

        this._cache[this.originalString] = this.parsedString;

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      _getNextTokenPosition: function(/*Token[]*/tokens, /*int*/type, /*int*/offset){
        for ( var i = offset; i < tokens.length; i++) {
          if (tokens[i].type == type) {
            return i;
          }
        }

        return -1;
      }
    });

com.ibm.bpm.social.util.common.ALLOWED_ELEMENTS = ["abbr",
                                                   "acronym",
                                                   "b",
                                                   "big",
                                                   "br",
                                                   "cite",
                                                   "code",
                                                   "dfn",
                                                   "em",
                                                   "h1",
                                                   "h2",
                                                   "h3",
                                                   "h4",
                                                   "h5",
                                                   "h6",
                                                   "hr",
                                                   "i",
                                                   "kbd",
                                                   "li",
                                                   "ol",
                                                   "p",
                                                   "s",
                                                   "samp",
                                                   "small",
                                                   "strike",
                                                   "strong",
                                                   "sub",
                                                   "sup",
                                                   "tt",
                                                   "u",
                                                   "ul",
                                                   "var"];

com.ibm.bpm.social.util.common.ALLOWED_SINGLE_ELEMENTS = ["br",
                                                          "hr"];

com.ibm.bpm.social.util.common.REGEXP_LINEBREAK = new RegExp("\n", "g");

com.ibm.bpm.social.util.common.TOKEN_GT = 0; // >
com.ibm.bpm.social.util.common.TOKEN_LINE_BREAK = 1; // \n
com.ibm.bpm.social.util.common.TOKEN_LT = 2; // <
com.ibm.bpm.social.util.common.TOKEN_SLASH = 3; // /
com.ibm.bpm.social.util.common.TOKEN_SPACE = 4;
com.ibm.bpm.social.util.common.TOKEN_TEXT = 5;

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ErrorDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.ErrorDialog"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");






dojo.declare("com.ibm.bpm.social.widgets.dialogs.ErrorDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString: "<div><div class='bpm-social-dialog-box'>"
    + "<div dojoAttachPoint='message' class='bpm-social-dialog-message'></div>"
    + "<div dojoAttachPoint='detailsTitle' style='display:inline'></div>"
    + "<div dojoAttachPoint='detailsContent' class='bpm-social-dialog-details-content'></div>" + "</div>"
    + "<div dojoAttachPoint='buttonContainer' class='bspace-dialog-buttonContainer'></div>"
    + "<div style='clear: both'></div>" + "</div>",

  _message: "",
  _label: null,
  _details: null,

  /**
   * Creates a popup which will show a message.
   * 
   * @param message The message to be shown
   * @param details The details to be shown
   * @param label The label of the confirmation button
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    com.ibm.bpm.social.util.common.Log.logEntry();
    if (typeof params.message === 'string') {
      this._message = params.message;
    }
    if (typeof params.label === 'string') {
      this._label = params.label;
    }
    if (typeof params.details === 'string') {
      this._details = params.details;
    }
    com.ibm.bpm.social.util.common.Log.logExit();
  },

  postCreate: function postCreate_ErrorDialog(){
    this.message.innerHTML = this._message;
    var self = this;
    if (this._label != null) {
      this.addButton(this.id + "_confirmationButton", this._label, function(){
        self.hide();
      });
    }

    // If details exist, create a title pane for it.
    if (this._details) {
      var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

      var detailsTitlePic = document.createElement("div");
      detailsTitlePic.innerHTML = "+";
      dojo.addClass(detailsTitlePic, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitlePic);

      var detailsTitleLabel = document.createElement("div");
      detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
      dojo.addClass(detailsTitleLabel, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitleLabel);

      this.detailsTitle.tabIndex = "0";

      dojo.style(this.detailsContent, "display", "none");
      this.detailsContent.innerHTML = this._details;

      var detailsHidden = true;
      this.connect(this.detailsTitle, "ondijitclick", function(e){
        if (detailsHidden) {
          dojo.style(this.detailsContent, "display", "block");
          detailsHidden = false;
          detailsTitlePic.innerHTML = "-";
          detailsTitleLabel.innerHTML = bundle["dialog_details_hide"];
        }
        else {
          dojo.style(this.detailsContent, "display", "none");
          detailsHidden = true;
          detailsTitlePic.innerHTML = "+";
          detailsTitleLabel.innerHTML = bundle["dialog_details_more"];
        }
      });
    }
    else {
      dojo.style(this.detailsTitle, "display", "none");
      dojo.style(this.detailsContent, "display", "none");
    }

    this.connect(this.popup, 'onHide', "_finish");
    this.inherited("postCreate", arguments);
  },

  _finish: function _finish_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    this.onClose();
    // Basically this is also a dijit._Widget with a meaningful destroy method.#
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
    com.ibm.bpm.social.util.common.Log.logEntry();
  },

  /**
   * Event handling hook.
   */
  onClose: function onClose_ErrorDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry();
    com.ibm.bpm.social.util.common.Log.logExit();
  }

});

/**
 * Shows an info popup. For example in case an ad hoc event has been started but no immediate task is
 * available.
 * 
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional} If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["info_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-info"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };


/**
 * Creates a modal warning popup with the provided message. <br />
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["warning_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-warning"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };

/**
 * Creates a modal error popup with the provided message. <br />
 * I.e. 
 * com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, function() {...});
 * 
 * @param message The message to be shown
 * @param details The details to be shown
 * @param callback The callback method which is called when the dialog is closed.
 * @param heritage {optional) If yes a Close button will be used instead of OK as many error message for
 *        heritage coaches refer to such a button.
 */
com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError =
  function(message /* string */, details /* string */, callback /* function */, heritage /* boolean */){
    var bundle = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    var label = bundle["dialog_button_ok"];
    if (typeof heritage === "boolean" && heritage) {
      label = bundle["dialog_heritage_button_close"];
    }
    var dialog = new com.ibm.bpm.social.widgets.dialogs.ErrorDialog({
      "title": bundle["error_dialog_title"],
      "label": label,
      "message": message,
      "details": details,
      "class": "bpm-social-dialog-error"
    });
    if (typeof callback === "function") {
      dialog.connect(dialog, "onClose", callback);
    }
    dialog.show();
  };

}

if(!dojo._hasResource["dojox.date.hebrew.numerals"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.numerals"] = true;
dojo.provide("dojox.date.hebrew.numerals");

//Conversion from "Hindi" numerals to Hebrew numerals and vice versa

(function(){

	var DIG="אבגדהוזחט";
	var	TEN="יכלמנסעפצ";
	var	HUN="קרשת";

	var transformChars = function(str, nogrsh){
		str = str.replace("יה", "טו").replace("יו", "טז");

		if(!nogrsh){
			var len = str.length;
			if(len > 1){
				str = str.substr(0, len - 1) + '"' + str.charAt(len - 1);
			}else{
				str += "\u05F3"; // 05F3:geresh
			}
		}
		return str; // String
	};
	 
	var parseStrToNumber = function(str){
		var num = 0;
		dojo.forEach(str, function(ch){
			var i;
			if((i = DIG.indexOf(ch)) != -1){
				num += ++i;
			}else if((i = TEN.indexOf(ch)) != -1){
				num += 10 * ++i;
			}else if((i = HUN.indexOf(ch)) != -1){
				num += 100 * ++i;
			}
		});
		return num; //Number
	};
	 
	var convertNumberToStr = function(num){
		var str  = "", n = 4, j = 9;
  		while(num){
			if(num >= n*100){
				str += HUN.charAt(n-1);
				num -= n*100;
				continue;
			}else if(n > 1){
				n--;
				continue;
			}else if(num >= j*10){
				str += TEN.charAt(j-1);
				num -= j*10;
			}else if(j > 1){
				j--;
				continue;
			}else if(num > 0){
				str += DIG.charAt(num-1);
				num = 0;
			}
		}
		return str; //String
	};

	dojox.date.hebrew.numerals.getYearHebrewLetters = function(/*Number */ year){
		// summary: converts the year from an integer to Hebrew numerals.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getYearHebrewLetters(date1.getFullYear());
		
		var rem = year % 1000;
		//FIXME: tests include dates outside this range and seem to pass.
		//	    	if((year - rem) / 1000 != 5){ throw new Error("Hebrew year "+year+" is not in range 5001-5999");}
		return transformChars(convertNumberToStr(rem)); // String
	};
	
	dojox.date.hebrew.numerals.parseYearHebrewLetters  = function(/*String hebrew year*/ year){
		// summary: converts the year written in Hebrew numerals to an integer
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |        	date.setFullYear(dojox.date.hebrew.numerals.parseYearHebrewLetters('\u05ea\u05e9\u05e1\u05f4\u05d7'));

		return parseStrToNumber(year) + 5000; // int
	};
	
	dojox.date.hebrew.numerals.getDayHebrewLetters =  function(day, /*boolean?*/ nogrsh){
		// summary: converts an integer to a String representing the number in Hebrew numerals.   Can be formatted with or without geresh &#x05f3;
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getDayHebrewLetters(date1.getDay());

		return transformChars(convertNumberToStr(day), nogrsh); // String
	};
	
	dojox.date.hebrew.numerals.parseDayHebrewLetters =  function(/*String hebrew*/ day){
		// summary: converts the string containing a Hebrew numeral to an integer
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		date1.setDate(dojox.date.hebrew.numerals.parseDayHebrewLetters('\u05d0')); // ALEPH
		return parseStrToNumber(day); // int
	};

	dojox.date.hebrew.numerals.getMonthHebrewLetters =  function(/*int*/month){
		// summary: converts an integer representing a  month to a String written in Hebrew numerals
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		document.writeln(dojox.date.hebrew.numerals.getMonthHebrewLetters(date1.getMonth());

		return transformChars(convertNumberToStr(month+1)); // String
	};

	dojox.date.hebrew.numerals.parseMonthHebrewLetters = function(/*String*/monthStr){
		// summary: converts a Hebrew numeral string representing
		//	a month to an integer.  The returned value
		//	is indexed in the month name array.  To use it for
		//	setMonth, do correction for leap year
		//
		// example:
		// |		var date = new dojox.date.hebrew.Date();
		// |            var number = dojox.date.hebrew.numerals.parseMonthHebrewLetters("\u05ea\u05de\u05d5\u05d6"); // Tammuz
		// |		date.setMonth(number);
			
		//month number from 0 to 12
		var monnum = dojox.date.hebrew.numerals.parseDayHebrewLetters(monthStr) - 1;

		if(monnum == -1 || monnum > 12){
			throw new Error("The month name is incorrect , month = " + monnum);
		}
		return monnum;
	};
})();

}

if(!dojo._hasResource["dojox.date.hebrew.Date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.Date"] = true;
dojo.provide("dojox.date.hebrew.Date");



dojo.declare("dojox.date.hebrew.Date", null, {
	// summary: A Date-like object which implements the Hebrew calendar
	//
	// description:
	//	A Date-like object which implements the Hebrew Calendar.  Because this object
	//	implements many of the same methods as the native JavaScript Date object, which
	//	implements the Gregorian calendar, it can often be used its place.  Note that
	//	this object does not extend Date or use its prototype.
	//
	// example:
	// |	
	// |
	// |	var date = new dojox.date.hebrew.Date();
	// |	console.log(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());

	// Hebrew date calculations are performed in terms of days, hours, and
	// "parts" (or halakim), which are 1/1080 of an hour, or 3 1/3 seconds.
	//_HOUR_PARTS: 1080,
	//_DAY_PARTS: 24*1080,
   
	// An approximate value for the length of a lunar month.
	// It is used to calculate the approximate year and month of a given
	// absolute date.
	//_MONTH_FRACT: 12*1080 + 793,
	//_MONTH_PARTS: 29*24*1080 + 12*1080 + 793,
	    
	// The time of the new moon (in parts) on 1 Tishri, year 1 (the epoch)
	// counting from noon on the day before.  BAHARAD is an abbreviation of
	// Bet (Monday), Hey (5 hours from sunset), Resh-Daled (204).
	//_BAHARAD: 11*1080 + 204,

	// The Julian day of the Gregorian epoch, that is, January 1, 1 on the
	// Gregorian calendar.
	//_JAN_1_1_JULIAN_DAY: 1721426,

	/**
	* The lengths of the Hebrew months.  This is complicated, because there
	* are three different types of years, or six if you count leap years.
	* Due to the rules for postponing the start of the year to avoid having
	* certain holidays fall on the sabbath, the year can end up being three
	* different lengths, called "deficient", "normal", and "complete".
	*/
	
	//"Absolute" indexes of  months:  Tishri -  0, Heshvan - 1, Kislev - 2, Tevet - 3, Shevat -  4, Adar I (leap years only) - 5, Adar - 6, Nisan - 7, Iyar - 8, Sivan - 9,   Tammuz-10, Av - 11, Elul - 12.

	_MONTH_LENGTH:  [
		// Deficient  Normal     Complete
		[   30,	    30,	    30	],		 //Tishri    0
		[   29,	    29,	    30	],		 //Heshvan   1
		[   29,	    30,	    30	],		 //Kislev   2
		[   29,	    29,	    29	],		 //Tevet  3
		[   30,	    30,	    30	],		 //Shevat  4
		[   30,	    30,	    30	],		 //Adar I (leap years only)  5
		[   29,	    29,	    29	],		 //Adar  6
		[   30,	    30,	    30	],		 //Nisan  7
		[   29,	    29,	    29	],		 //Iyar  8
		[   30,	    30,	    30	],		 //Sivan  9
		[   29,	    29,	    29	],		 //Tammuz  10
		[   30,	    30,	    30	],		 //Av  11
		[   29,	    29,	    29	]		 //Elul  12
	],

	/**
	* The cumulative # of days to the end of each month in a non-leap year
	* Although this can be calculated from the MONTH_LENGTH table,
	* keeping it around separately makes some calculations a lot faster
	*/
	_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  147,	   148,	   149  ],		// (Adar I placeholder)
		[  176,	   177,	   178  ],		// Adar
		[  206,	   207,	   208  ],		// Nisan
		[  235,	   236,	   237  ],		// Iyar
		[  265,	   266,	   267  ],		// Sivan
		[  294,	   295,	   296  ],		// Tammuz
		[  324,	   325,	   326  ],		// Av
		[  353,	   354,	   355  ]		// Elul
	],

	/**
	* The cumulative # of days to the end of each month in a leap year
	*/
	_LEAP_MONTH_START:  [
		// Deficient  Normal	Complete
		[    0,		0,		0  ],		// (placeholder)
		[   30,	    30,	    30  ],		// Tishri
		[   59,	    59,	    60  ],		// Heshvan
		[   88,	    89,	    90  ],		// Kislev
		[  117,	   118,	   119  ],		// Tevet
		[  147,	   148,	   149  ],		// Shevat
		[  177,	   178,	   179  ],		// Adar I
		[  206,	   207,	   208  ],		// Adar II
		[  236,	   237,	   238  ],		// Nisan
		[  265,	   266,	   267  ],		// Iyar
		[  295,	   296,	   297  ],		// Sivan
		[  324,	   325,	   326  ],		// Tammuz
		[  354,	   355,	   356  ],		// Av
		[  383,	   384,	   385  ]		// Elul
	],
	
	_GREGORIAN_MONTH_COUNT:  [
		//len len2   st  st2
		[  31,  31,   0,   0 ], // Jan
		[  28,  29,  31,  31 ], // Feb
		[  31,  31,  59,  60 ], // Mar
		[  30,  30,  90,  91 ], // Apr
		[  31,  31, 120, 121 ], // May
		[  30,  30, 151, 152 ], // Jun
		[  31,  31, 181, 182 ], // Jul
		[  31,  31, 212, 213 ], // Aug
		[  30,  30, 243, 244 ], // Sep
		[  31,  31, 273, 274 ], // Oct
		[  30,  30, 304, 305 ], // Nov
		[  31,  31, 334, 335 ] // Dec
		// len  length of month
		// len2 length of month in a leap year
		// st   days in year before start of month
		// st2  days in year before month in leap year
	],

    _date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,

 	constructor: function(){
		// summary: initialize the date object value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		var date2 = new dojox.date.hebrew.Date(date1);
		// |
		// |		var date3 = new dojox.date.hebrew.Date(5768,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._year = this._month = this._date = this._hours = this._minutes = this._seconds = this._milliseconds = NaN;
			}else{  // this is hebrew.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >= 3){
			// YYYY, MM, DD arguments passed, month is from 0-12,  "absolute" index of month
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			
			if(this._month > 12){
				console.warn("the month is incorrect , set 0  " + this._month + "   " + this._year );
				this._month = 0;
			}
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
  
		this._setDay();
	},
	
	getDate: function(){
		// summary: returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return this._date; // int
	},

	getDateLocalized: function(/*String?*/locale){
		// summary: returns the date value as hebrew numerals for the Hebrew locale,
		//		a number for all others.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDate());

		return (locale || dojo.locale).match(/^he(?:-.+)?$/) ?
			dojox.date.hebrew.numerals.getDayHebrewLetters(this._date) : this.getDate();
	},

	getMonth: function(){
		// summary: returns the month value (0 - 12)
		//
		// description: the result is the index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		// For non leap years, for months after Shevat, the actual position of
		// the month in the year (used for short format) is less than
		// the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getMonth()+1);
		// |		>> 7

		return this._month;
	},

	getFullYear: function(){
		// summary: returns the Year value
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |
		// |		console.log(date1.getFullYear());
		// |		>> 5769
		return this._year;
	},
			
	getHours: function(){
 		//summary: returns the hour value
		return this._hours;
	},
		
	getMinutes: function(){
		//summary: returns the minutes value

		return this._minutes;
	},

	getSeconds: function(){
		//summary: returns the seconds value
		return this._seconds;
	},

	getMilliseconds: function(){
		//summary: returns the milliseconds value

		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: sets the date number for a given month
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		date1.setDate(2);

		date = +date;
		var mdays;
		if(date>0){
			while (date > (mdays = this.getDaysInHebrewMonth(this._month, this._year))){
				date -= mdays;
				this._month++;
				if(this._month >= 13){this._year++; this._month -= 13;}
			}
		}else{
			while(date<=0){
				mdays = this.getDaysInHebrewMonth((this._month-1)>=0 ? (this._month-1) : 12, ((this._month-1)>=0)? this._year : this._year-1);
				this._month--;
				if(this._month < 0){this._year--; this._month += 13;}
				date += mdays;
			}
		}
		this._date = date;
		this._setDay();
		return this;
	},
	

	setFullYear: function(/*number*/year, /*number?*/month, /*number?*/ date){
		// summary: set the year
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setFullYear(5768);
		// |		date1.setFullYear(5768, 1, 1);
		
		this._year = year = +year;
		if(!this.isLeapYear(year) && this._month==5){  //incorrect month number for non leap year
			this._month++;
		}
		
		if(month !== undefined){this.setMonth(month);}
		if(date !== undefined){this.setDate(date);}
		
		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setMonth: function(/*number*/month){
		// summary: sets the month.  You should use "absolute" index in the month array:
		//	0. Tishri
		//	1. Heshvan
		//	2. Kislev
		//	3. Tevet
		//	4. Shevat
		//	5. Adar I (leap years only)
		//	6. Adar
		//	7. Nisan
		//	8. Iyar
		//	9. Sivan
		//	10. Tammuz
		//	11.	Av
		//	12. Elul - 12
		//  For non leap years, for months after Shevat, the actual position of
		//	the month in the year (used for short format) is less than
		//	the "absolute" index by 1.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setMonth(0); //first month

		month = +month; // coerce to a Number
		if(!this.isLeapYear(this._year) && month == 5){month++;}
	
		if(month>=0){
			while(month >12){
				this._year++;
				month -= 13;
				if (!this.isLeapYear(this._year) && month >= 5){month++;}
			}
		}else{
			while(month<0){
				this._year--;
				month += (!this.isLeapYear(this._year)  &&  month < -7) ? 12 : 13;
			}
		}
		
		this._month = month;

		var dnum = this.getDaysInHebrewMonth(this._month, this._year);
		if(dnum < this._date){
			this._date = dnum;
		} // if the date in this month more than number of the days in this month
		
		this._setDay();
		return this;
	},

	setHours: function(){
		//	summary: sets the hour
		//
		//	description: Sets the hour and optionally minutes, seconds, milliseconds also.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |		date1.setHours(12, 30, 0, 0);

		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours += +arguments[0];
		}

		if(hours_arg_no >= 2){
			this._minutes += +arguments[1];
		}

		if(hours_arg_no >= 3){
			this._seconds += +arguments[2];
		}

		if(hours_arg_no == 4){
			this._milliseconds += +arguments[3];
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInHebrewMonth(this._month, this._year);
			if(this._date > mdays)
			{
				this._month++;
				if(!this.isLeapYear(this._year) && this._month==5){ this._month++; }
				if(this._month >= 13){this._year++; this._month -= 13;}
				this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
		this._setDay();
		return this;
	},

	setMinutes: function(/*Number*/minutes){
		//summary: sets the minutes (0-59)
		minutes = +minutes;
		this._minutes = minutes % 60;
		this.setHours(parseInt(minutes / 60));
		this._setDay();
		return this;
	},

	setSeconds: function(/*Number*/seconds){
		//summary: sets the seconds (0-59)

		seconds = +seconds;
		this._seconds = seconds % 60;
		this.setMinutes(parseInt(seconds / 60));
		this._setDay();
		return this;
	},

	setMilliseconds: function(/*Number*/milliseconds){
		//summary: sets the milliseconds

		milliseconds = +milliseconds;
		this._milliseconds = milliseconds % 1000;
		this.setSeconds(parseInt(milliseconds / 1000));
		this._setDay();
		return this;
	},

	_setDay: function(){
		var day = this._startOfYear(this._year);
		if(this._month != 0){
			day += (this.isLeapYear(this._year) ? this._LEAP_MONTH_START : this._MONTH_START)[this._month || 0][this._yearType(this._year)];
		}
		day += this._date - 1;
		this._day = (day+1) % 7;
	},

	toString: function(){
		// summary: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format
		//
		// description: returns a string representation of the date in "dd, MM, yyyy HH:mm:ss" format (all numeric)
		//	For user presentation, use dojox.date.hebrew.locale.format which will present in the appropriate language
		//  and format.  toString() language- and culturally-specific conventions to keep this module free of
		//	dependencies on dojox.date.locale and dojo.cldr.
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date(5769, 6, 1);
		// |		console.log(date1.toString());
		// |		>>> "1, 6, 5769 0:0:0"
		return this._date + ", " + this._month + ", " + this._year + "  " + this._hours + ":" + this._minutes + ":" + this._seconds; // String
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInHebrewMonth: function(/*Number*/month, /*Number*/ year){
		// summary: returns the number of days in the given month and year

		// Aside from the leap month, these two months can vary: 1=HESHVAN, 2=KISLEV
		// The rest are a fixed length
		var yearType = (month == 1 || month == 2) ? this._yearType(year) : 0;
		return (!this.isLeapYear(this._year) && month == 5) ? 0 : this._MONTH_LENGTH[month][yearType];
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearType: function(/*Number*/year){
		var yearLength = this._handleGetYearLength(Number(year));
		if(yearLength > 380){
			yearLength -= 30;        // Subtract length of leap month.
		}

		var yearType = yearLength - 353;
		if (yearType < 0 || yearType > 2){
			throw new Error("Illegal year length " + yearLength + " in year " + year);
		}
		return yearType;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_handleGetYearLength: function(/*number*/eyear){
		return this._startOfYear(eyear+1) - this._startOfYear(eyear);
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_startOfYear: function(/*number*/year){
		var months = Math.floor((235 * year - 234) / 19),	// # of months before year
			frac = months * (12*1080 + 793) + 11*1080 + 204/*BAHARAD*/,	// Fractional part of day #
			day  = months * 29 + Math.floor(frac / (24*1080));	// Whole # part of calculation
		frac %= 24*1080;	// Time of day

		var wd = day % 7;	// Day of week (0 == Monday)

		if(wd == 2 || wd == 4 || wd == 6){
			// If the 1st is on Sun, Wed, or Fri, postpone to the next day
			day += 1;
			wd = day % 7;
		}
		if(wd == 1 && frac > 15 * 1080 + 204 && !this.isLeapYear(year)){
			// If the new moon falls after 3:11:20am (15h204p from the previous noon)
			// on a Tuesday and it is not a leap year, postpone by 2 days.
			// This prevents 356-day years.
			day += 2;
		}else if(wd == 0 && frac > 21 * 1080 + 589 && this.isLeapYear(year-1)){
			// If the new moon falls after 9:32:43 1/3am (21h589p from yesterday noon)
			// on a Monday and *last* year was a leap year, postpone by 1 day.
			// Prevents 382-day years.
			day += 1;
		}

		return day;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	isLeapYear: function(/*Number*/year){
	//	summary:
	//		Determines if the year (argument) is a leap year
	//	description: The Leap year contains additional month adar sheni
	//
		//return (year * 12 + 17) % 19 >= 12;
		var x = (year*12 + 17) % 19;
		return x >= ((x < 0) ? -7 : 12);
	},

	
	fromGregorian: function(/*Date*/gdate){
		// summary: This function sets this Date to the Hebrew Date corresponding to the Gregorian Date
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateHebrew.fromGregorian(dateGregorian);

		var result = (!isNaN(gdate)) ? this._computeHebrewFields(gdate) : NaN;
		this._year = (!isNaN(gdate)) ? result[0] : NaN;
		this._month = (!isNaN(gdate))? result[1] : NaN;
		this._date = (!isNaN(gdate)) ? result[2] : NaN;
		this._hours = gdate.getHours();
		this._milliseconds = gdate.getMilliseconds();
		this._minutes = gdate.getMinutes();
		this._seconds = gdate.getSeconds();
		if (!isNaN(gdate)) this._setDay();
		return this;
	},

	// ported from the Java class com.ibm.icu.util.HebrewCalendar.handleComputeFields from ICU4J v3.6.1 at http://www.icu-project.org/
	_computeHebrewFields: function(/*Date*/gdate){
		var julianDay = this._getJulianDayFromGregorianDate(gdate),
			d = julianDay - 347997,
			m = Math.floor((d * 24*1080) / (29*24*1080 + 12*1080 + 793)),       // Months (approx)
			year = Math.floor((19 * m + 234) / 235) + 1,  // Years (approx)
			ys  = this._startOfYear(year),                 // 1st day of year
			dayOfYear = (d - ys);
		// Because of the postponement rules, it's possible to guess wrong.  Fix it.
		while(dayOfYear < 1){
			year--;
			ys  = this._startOfYear(year);
			dayOfYear = d - ys;
		}

		// Now figure out which month we're in, and the date within that month

		var typeofYear = this._yearType(year),
			monthStart = this.isLeapYear(year) ? this._LEAP_MONTH_START : this._MONTH_START,
			month = 0;

		while(dayOfYear > monthStart[month][typeofYear]){
			month++;
		}
		month--;
		var dayOfMonth = dayOfYear - monthStart[month][typeofYear];
		return [year, month, dayOfMonth];
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianFields from ICU4J v3.6.1 at http://www.icu-project.org/
	toGregorian: function(){
		// summary: returns the equivalent Grogorian date value as a native Date object
		// example:
		// |		var dateHebrew = new dojox.date.hebrew.Date(5768,11,20);
		// |		var dateGregorian = dateHebrew.toGregorian();
		
		var hYear = this._year || 0,
			hMonth = this._month || 0,
			hDate = this._date || 0,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		var julianDay =  (hDate + day + 347997),
		// The Gregorian epoch day is zero for Monday January 1, year 1.
			gregorianEpochDay = julianDay - 1721426;

		// Here we convert from the day number to the multiple radix
		// representation.  We use 400-year, 100-year, and 4-year cycles.
		// For example, the 4-year cycle has 4 years + 1 leap day; giving
		// 1461 == 365*4 + 1 days.
		var rem = [];
		var n400 = this._floorDivide(gregorianEpochDay , 146097, rem), // 400-year cycle length
			n100 = this._floorDivide(rem[0] , 36524, rem), // 100-year cycle length
			n4 = this._floorDivide(rem[0] , 1461, rem), // 4-year cycle length
			n1 = this._floorDivide(rem[0] , 365, rem),
			year = 400*n400 + 100*n100 + 4*n4 + n1,
			dayOfYear = rem[0]; // zero-based day of year

		if(n100 == 4 || n1 == 4){
			dayOfYear = 365; // Dec 31 at end of 4- or 400-yr cycle
		}else{
			++year;
		}

		var isLeap = !(year%4) && // equiv. to (year%4 == 0)
				(year%100 || !(year%400)),
			correction = 0,
			march1 = isLeap ? 60 : 59; // zero-based DOY for March 1
		if(dayOfYear >= march1){ correction = isLeap ? 1 : 2; }
		var month = Math.floor((12 * (dayOfYear + correction) + 6) / 367); // zero-based month
		var dayOfMonth = dayOfYear -
				this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2] + 1; // one-based DOM

		return new Date(year, month, dayOfMonth, this._hours, this._minutes, this._seconds, this._milliseconds); // Date
	},
	_floorDivide: function(numerator, denominator, remainder){
		if(numerator >= 0){
			remainder[0] = (numerator % denominator);
			return Math.floor(numerator / denominator);
		}
		var quotient = Math.floor(numerator / denominator);
		remainder[0] = numerator - (quotient * denominator);
		return quotient;
	},

	getDay: function(){
		// summary: returns weekday value (0 - 6)
		//
		// example:
		// |		var date1 = new dojox.date.hebrew.Date();
		// |
		// |		console.log(date1.getDay());

		var hYear = this._year,
			hMonth = this._month,
			hDate = this._date,
			day = this._startOfYear(hYear);

		if(hMonth != 0){
			day += (this.isLeapYear(hYear) ? this._LEAP_MONTH_START : this._MONTH_START)[hMonth][this._yearType(hYear)];
		}

		day += hDate - 1;
		return (day+1) % 7;
	},

	// ported from the Java class com.ibm.icu.util.Calendar.computeGregorianMonthStart from ICU4J v3.6.1 at http://www.icu-project.org/
	_getJulianDayFromGregorianDate: function(gdate){
		//summary: returns the Julian day of a Gregorian date

		var year = gdate.getFullYear(),
			month = gdate.getMonth(),
			d = gdate.getDate(),
			isLeap = !(year%4) && (year%100 || !(year%400)), //TODO: dup
			y = year - 1;
		// This computation is actually ... + (_JAN_1_1_JULIAN_DAY - 3) + 2.
		// Add 2 because Gregorian calendar starts 2 days after Julian
		// calendar.
		var julianDay = 365*y + Math.floor(y/4) - Math.floor(y/100) +
			Math.floor(y/400) + 1721426 - 1;
		// At this point julianDay indicates the day BEFORE the first day
		// of January 1, <eyear> of the Gregorian calendar.
		if(month > 0) {
			julianDay += this._GREGORIAN_MONTH_COUNT[month][isLeap ? 3 : 2];
		}
		
		julianDay += d;
		return julianDay;
	}
});

dojox.date.hebrew.Date.prototype.valueOf = function(){
	return this.toGregorian().valueOf();
};

}

if(!dojo._hasResource["dojox.date.hebrew"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew"] = true;
dojo.provide("dojox.date.hebrew");


 // for compare
	
// Utility methods to do arithmetic calculations with hebrew.Dates

	// added for compat to date
dojox.date.hebrew.getDaysInMonth = function(/*hebrew.Date*/month){
	return month.getDaysInHebrewMonth(month.getMonth(), month.getFullYear());
};

//TODO: define hebrew.isLeapYear?  Or should it be invalid, since it has different meaning?

dojox.date.hebrew.compare = function(/*hebrew.Date*/dateheb1, /*hebrew.Date*/dateheb2, /*String?*/portion){
	//	summary:
	//		Compare two hebrew date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		hebrew.Date object
	//	date2:
	//		hebrew.Date object.  If not specified, the current hebrew.Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	if(dateheb1 instanceof dojox.date.hebrew.Date){
		dateheb1 = dateheb1.toGregorian();
	}
	if(dateheb2 instanceof dojox.date.hebrew.Date){
		dateheb2 = dateheb2.toGregorian();
	}
	
	return dojo.date.compare.apply(null, arguments);
};


dojox.date.hebrew.add = function(/*dojox.date.hebrew.Date*/date, /*String*/interval, /*int*/amount){
	//	based on and similar to dojo.date.add
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: hebrew.Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var newHebrDate = new dojox.date.hebrew.Date(date);

	switch(interval){
		case "day":
			newHebrDate.setDate(date.getDate() + amount);
			break;
		case "weekday":
			var day = date.getDay();
			var remdays = 0;
			if(amount < 0 && day == 6){ day = 5; remdays = -1; }
			
			if((day + amount) < 5 && (day + amount) >= 0){ //in the same week
				 newHebrDate.setDate(date.getDate() + amount + remdays);
			}else{
				var add = (amount > 0) ? 5 : -1;
				var adddays = (amount > 0) ? 2 : -2 ; /*first weekend */
				if  (amount > 0 && (day == 5 || day == 6)) { remdays =  4 - day; day = 4;}
				var newamount  =  day + amount - add;
				var weeks = parseInt(newamount / 5);
				var newday = newamount%5;
				newHebrDate.setDate(date.getDate() - day+ adddays + weeks * 7 + remdays + newday + add);
			}
			break;
		case "year":
			newHebrDate.setFullYear(date.getFullYear() + amount);
			break;
		case "week":
			amount *= 7;
			newHebrDate.setDate(date.getDate() + amount);
			break;
		case "month":
			var month = date.getMonth();
			var add = month + amount;
			if(!date.isLeapYear(date.getFullYear())){
				if(month < 5 && add >= 5){ add++;}
				else if (month > 5 && add <= 5){ add--;}
			}
			newHebrDate.setMonth(add);
			break;
		case "hour":
			newHebrDate.setHours(date.getHours() + amount);
			break;
		case "minute":
			newHebrDate.setMinutes(date.getMinutes() + amount);
			break;
		case "second":
			newHebrDate.setSeconds(date.getSeconds() + amount);
			break;
		case "millisecond":
			newHebrDate.setMilliseconds(date.getMilliseconds() + amount);
			break;
	}

	return newHebrDate; // dojox.date.hebrew.Date
};

dojox.date.hebrew.difference = function(/*dojox.date.hebrew.Date*/date1, /*dojox.date.hebrew.Date?*/date2, /*String?*/interval){
	//	based on and similar to dojo.date.difference
	//	summary:
	//        date1 - date2
	//	 date2 is hebrew.Date object.  If not specified, the current hebrew.Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond",  "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new dojox.date.hebrew.Date();
	interval = interval || "day";
	var yearDiff = date1.getFullYear() - date2.getFullYear();
	var delta = 1; // Integer return value
	switch(interval){
		case "weekday":
			var days = Math.round(dojox.date.hebrew.difference(date1, date2, "day"));
			var weeks = parseInt(dojox.date.hebrew.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date2.getDay();
				var bDay = date1.getDay();
	
				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new dojox.date.hebrew.Date(date2);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();
	
				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = -1;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = -1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = 0;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 1;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = 2;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			var startdate =  (date1.toGregorian() > date2.toGregorian()) ? date1 : date2; // more
			var enddate = (date1.toGregorian() > date2.toGregorian()) ? date2 : date1;
			
			var month1 = startdate.getMonth();
			var month2 = enddate.getMonth();
			
			if(yearDiff == 0){
				delta = ( !date1.isLeapYear(date1.getFullYear())  && startdate.getMonth() > 5 && enddate.getMonth() <=5) ? (startdate.getMonth() - enddate.getMonth() - 1) :
						(startdate.getMonth() - enddate.getMonth() );
			}else{
				delta = (!enddate.isLeapYear(enddate.getFullYear()) &&  month2 < 6) ? (13-month2-1) : (13-month2);
				delta +=  (!startdate.isLeapYear(startdate.getFullYear()) &&  month1 > 5) ? (month1 -1): month1;
				var i = enddate.getFullYear()  + 1;
				var e = startdate.getFullYear();
				for (i;   i < e;  i++){
					delta += enddate.isLeapYear(i) ? 13 : 12;
				}
			}
			if(date1.toGregorian() < date2.toGregorian()){
				delta = -delta;
			}
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojox.date.hebrew.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date1.toGregorian().getTime()- date2.toGregorian().getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojox.date.hebrew.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.hebrew.locale"] = true;
dojo.provide("dojox.date.hebrew.locale");









//Load the bundles containing localization information for
// names and formats


(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, locale, fullYear,  pattern){

		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad;
			var c = match.charAt(0);
			var l = match.length;
			var widthList = ["abbr", "wide", "narrow"];
			
			switch(c){
				case 'y':
					if(locale.match(/^he(?:-.+)?$/)){
						s = dojox.date.hebrew.numerals.getYearHebrewLetters(dateObject.getFullYear());
					}else{
						s = String(dateObject.getFullYear());
					}
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						if(!dateObject.isLeapYear(dateObject.getFullYear()) && m>5){m--;}
						if(locale.match(/^he(?:-.+)?$/)){
							s = dojox.date.hebrew.numerals.getMonthHebrewLetters(m);
						}else{
							s = m+1; pad = true;
						}
					}else{
						var monthNames = dojox.date.hebrew.locale.getNames('months',widthList[l-3], 'format', locale, dateObject);
						s = monthNames[m];
					}
					break;
				case 'd':
					if(locale.match(/^he(?:-.+)?$/)){
						s =  dateObject.getDateLocalized(locale);
					}else{
						s = dateObject.getDate(); pad = true;
					}
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'z':
					s = "";
					break;
				default:
					throw new Error("dojox.date.hebrew.locale.formatPattern: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}
	
dojox.date.hebrew.locale.format = function(/*hebrew.Date*/dateObject, /*object?*/options){
	// based on and similar to dojo.date.locale.format
	//summary:
	//		Format a Date object as a String, using  settings.
	//
	// description:
	//		Create a string from a hebrew.Date object using a known pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Default formatting lengths is 'short'
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojox.date.hebrew.locale._getHebrewBundle(locale);
	var str = [];

	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle, locale, options.fullYear);
	if(options.selector == "year"){
		var year = dateObject.getFullYear();
		return locale.match(/^he(?:-.+)?$/) ?
			dojox.date.hebrew.numerals.getYearHebrewLetters(year) : year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time

	return result; // String
};

dojox.date.hebrew.locale.regexp = function(/*object?*/options){
	//	based on and similar to dojo.date.locale.regexp
	// summary:
	//		Builds the regular needed to parse a hebrew.Date

	return dojox.date.hebrew.locale._parseInfo(options).regexp; // String
};

dojox.date.hebrew.locale._parseInfo = function(/*oblect?*/options){
/* based on and similar to dojo.date.locale._parseInfo */

	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojox.date.hebrew.locale._getHebrewBundle(locale);

	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];

	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = (timePattern === undefined) ? datePattern : datePattern + ' ' + timePattern; //hebrew resource file does not contain time patterns - a bug?
	}

	var tokens = [];
	
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};




dojox.date.hebrew.locale.parse= function(/*String*/value, /*object?*/options){
		// based on and similar to dojo.date.locale.parse
		// summary: This function parse string date value according to options
		// example:
		// |		var dateHebrew = dojox.date.hebrew.locale.parse('11/10/5740', {datePattern:'dd/MM/yy', selector:'date'});
		// |		in Hebrew locale string for parsing contains Hebrew Numerals
		// |
		// |  options = {datePattern:'dd MMMM yy', selector:'date'};
		// |
		// |   y - year
		// |   M, MM  - short month
		// |  MMM, MMMM - long month
		// |  d - date
		// |  a - am, pm
		// |   E, EE, EEE, EEEE  - week day
		// |
		// |    h, H, k, K, m, s, S,  -  time format
		
	value =  value.replace(/[\u200E\u200F\u202A-\u202E]/g, ""); //remove special chars

	if(!options){options={};}
	var info = dojox.date.hebrew.locale._parseInfo(options);
	
	var tokens = info.tokens, bundle = info.bundle;
	var re = new RegExp("^" + info.regexp + "$");
	
	var match = re.exec(value);

	var locale = dojo.i18n.normalizeLocale(options.locale);

	if(!match){ return null; } // null
	
	var date, date1;
	
	//var result = [1970,0,1,0,0,0,0]; //
	var result = [5730,3,23,0,0,0,0];  // hebrew date for [1970,0,1,0,0,0,0] used in gregorian locale
	var amPm = "";
	var mLength = 0;
	var widthList = ["abbr", "wide", "narrow"];
	var valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(locale.match(/^he(?:-.+)?$/)){
					result[0] = dojox.date.hebrew.numerals.parseYearHebrewLetters(v);
				}else{
					result[0] = Number(v);
				}
				break;
			case 'M':
				//if  it is short format, month is one letter or two letter with "geresh"
				if(l>2){
					//we do not know here if the year is leap or not
					var months = dojox.date.hebrew.locale.getNames('months', widthList[l-3], 'format', locale, new dojox.date.hebrew.Date(5769, 1, 1)),
						leapmonths = dojox.date.hebrew.locale.getNames('months', widthList[l-3], 'format', locale, new dojox.date.hebrew.Date(5768, 1, 1));
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
						leapmonths = dojo.map(leapmonths, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
					}
					var monthName = v;
					v = dojo.indexOf(months, monthName);
					if(v == -1){
						v = dojo.indexOf(leapmonths, monthName);
						if(v == -1){
							//console.debug("dojox.date.hebrew.locale.parse: Could not parse month name:  second   " + v +"'.");
							return false;
						}
					}
					mLength = l;
				}else{
					if(locale.match(/^he(?:-.+)?$/)){
						v = dojox.date.hebrew.numerals.parseMonthHebrewLetters(v);
					}else{
						v--;
					}
				}
				result[1] = Number(v);
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
				if(locale.match(/^he(?:-.+)?$/)){
					result[2] = dojox.date.hebrew.numerals.parseDayHebrewLetters(v);
				}else{
					result[2] = Number(v);
				}
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = Number(v);
				break;
			case 'm': //minutes
				result[4] = Number(v);
				break;
			case 's': //seconds
				result[5] = Number(v);
				break;
			case 'S': //milliseconds
				result[6] = Number(v);
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}
	var dateObject = new dojox.date.hebrew.Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]); // hebrew.Date
	//for non leap year, the index of the short month start from adar should be increased by 1
	if(mLength < 3 && result[1] >= 5 && !dateObject.isLeapYear(dateObject.getFullYear())){
		dateObject.setMonth(result[1]+1);
	}
	return dateObject; // hebrew.Date
};


function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g);
	var literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE  (tokens, bundle, options, pattern){
		// based on and similar to dojo.date.locale._buildDateTimeRE
		//
	
	pattern = dojo.regexp.escapeString(pattern);
	var locale = dojo.i18n.normalizeLocale(options.locale);
	
	return pattern.replace(/([a-z])\1*/ig, function(match){

			// Build a simple regexp.  Avoid captures, which would ruin the tokens list
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var p2 = '', p3 = '';
			if(options.strict){
				if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
				if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
			}else{
				p2 = '0?'; p3 = '0{0,2}';
			}
			switch(c){
				case 'y':
					s = '\\S+';
					break;
				case 'M':
					if(locale.match('^he(?:-.+)?$')){
						s = (l>2) ? '\\S+ ?\\S+' : '\\S{1,4}';
					}else{
						s = (l>2) ?  '\\S+ ?\\S+' : p2+'[1-9]|1[0-2]';
					}
					break;
				case 'd':
					if(locale.match('^he(?:-.+)?$')){
						s = '\\S[\'\"\'\u05F3]{1,2}\\S?';
					}else{
						s = '[12]\\d|'+p2+'[1-9]|30';
					}
					break;
				case 'E':
					if(locale.match('^he(?:-.+)?$')){
						s = (l>3) ? '\\S+ ?\\S+' : '\\S';
					}else{
						s = '\\S+';
					}
					break;
				case 'h': //hour (1-12)
					s = p2+'[1-9]|1[0-2]';
					break;
				case 'k': //hour (0-11)
					s = p2+'\\d|1[01]';
					break;
				case 'H': //hour (0-23)
					s = p2+'\\d|1\\d|2[0-3]';
					break;
				case 'K': //hour (1-24)
					s = p2+'[1-9]|1\\d|2[0-4]';
					break;
				case 'm':
				case 's':
					s = p2+'\\d|[0-5]\\d';
					break;
				case 'S':
					s = '\\d{'+l+'}';
					break;
				case 'a':
					var am = options.am || bundle['dayPeriods-format-wide-am'],
						pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					if(options.strict){
						s = am + '|' + pm;
					}else{
						s = am + '|' + pm;
						if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
						if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					}
					break;
				default:
					s = ".*";
			}
			if(tokens){ tokens.push(match); }
			return "(" + s + ")"; // add capture
		}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE. */
}
})();



(function(){
var _customFormats = [];
dojox.date.hebrew.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo.cldr: `dateFormat-xxxx` / `timeFormat-xxxx`
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.locale.format() for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojox.date.hebrew.locale._getHebrewBundle = function(/*String*/locale){
	var hebrew = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		hebrew = dojo.mixin(hebrew, bundle);
	}, this);
	return hebrew; /*Object*/
};
})();

dojox.date.hebrew.locale.addCustomFormats("dojo.cldr","hebrew");

dojox.date.hebrew.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale, /*dojox.date.hebrew.Date?*/date){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item:
	//	'months' || 'days'
	// type:
	//	'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// use:
	//	'standAlone' || 'format' (default)
	// locale:
	//	override locale used to find the names
	// date:
	//	required for item=months to determine leap month name
	//
	// using  var monthNames = dojox.date.hebrew.locale.getNames('months', 'wide', 'format', 'he', new dojox.date.hebrew.Date(5768, 2, 12));

	var label,
		lookup = dojox.date.hebrew.locale._getHebrewBundle(locale),
		props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';
	
	// return by copy so changes won't be made accidentally to the in-memory model
	var result = (label || lookup[props.join('-')]).concat();

	if(item == "months"){
		if(date.isLeapYear(date.getFullYear())){
			// Adar I (6th position in the array) will be used.
			// Substitute the leap month Adar II for the regular Adar (7th position)
			props.push("leap");
			result[6] = lookup[props.join('-')];
		}else{
			// Remove Adar I but leave an empty position in the array
			delete result[5];
		}
	}

	return result; /*Array*/
};

}

if(!dojo._hasResource["dojox.date.islamic.Date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic.Date"] = true;
dojo.provide("dojox.date.islamic.Date");




dojo.declare("dojox.date.islamic.Date", null, {
	// summary: The component defines the Islamic (Hijri) Calendar Object
	//
	// description:
	//	This module is similar to the Date() object provided by JavaScript
	//
	// example:
	// |	
	// |
	// |	var date = new dojox.date.islamic.Date();
	// |	document.writeln(date.getFullYear()+'\'+date.getMonth()+'\'+date.getDate());


	_date: 0,
	_month: 0,
	_year: 0,
	_hours: 0,
	_minutes: 0,
	_seconds: 0,
	_milliseconds: 0,
	_day: 0,
	_GREGORIAN_EPOCH : 1721425.5,
	_ISLAMIC_EPOCH : 1948439.5,

	constructor: function(){
		// summary: This is the constructor
		// description:
		//	This function initialize the date object values
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		var date2 = new dojox.date.islamic.Date("12\2\1429");
		// |
		// |		var date3 = new dojox.date.islamic.Date(date2);
		// |
		// |		var date4 = new dojox.date.islamic.Date(1429,2,12);

		var len = arguments.length;
		if(!len){// use the current date value, added "" to the similarity to date
			this.fromGregorian(new Date());
		}else if(len == 1){
			var arg0 = arguments[0];
			if(typeof arg0 == "number"){ // this is time "valueof"
				arg0 = new Date(arg0);
			}

			if(arg0 instanceof Date){
				this.fromGregorian(arg0);
			}else if(arg0 == ""){
				// date should be invalid.  Dijit relies on this behavior.
				this._date = new Date(""); //TODO: should this be NaN?  _date is not a Date object
			}else{  // this is Islamic.Date object
				this._year = arg0._year;
				this._month =  arg0._month;
				this._date = arg0._date;
				this._hours = arg0._hours;
				this._minutes = arg0._minutes;
				this._seconds = arg0._seconds;
				this._milliseconds = arg0._milliseconds;
			}
		}else if(len >=3){
			// YYYY MM DD arguments passed, month is from 0-12
			this._year += arguments[0];
			this._month += arguments[1];
			this._date += arguments[2];
			this._hours += arguments[3] || 0;
			this._minutes += arguments[4] || 0;
			this._seconds += arguments[5] || 0;
			this._milliseconds += arguments[6] || 0;
		}
	},

	getDate:function(){
		// summary: This function returns the date value (1 - 30)
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDate);
		return this._date;
	},
	
	getMonth:function(){
		// summary: This function return the month value ( 0 - 11 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getMonth()+1);

		return this._month;
	},

	getFullYear:function(){
		// summary: This function return the Year value
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getFullYear());

		return this._year;
	},
		
	getDay:function(){
		// summary: This function return Week Day value ( 0 - 6 )
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |
		// |		document.writeln(date1.getDay());

		return this.toGregorian().getDay();
	},
		
	getHours:function(){
		//summary: returns the Hour value
		return this._hours;
	},
	
	getMinutes:function(){
		//summary: returns the Minuites value
		return this._minutes;
	},

	getSeconds:function(){
		//summary: returns the seconde value
		return this._seconds;
	},

	getMilliseconds:function(){
		//summary: returns the Milliseconds value
		return this._milliseconds;
	},

	setDate: function(/*number*/date){
		// summary: This function sets the Date
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setDate(2);

		date = parseInt(date);

		if(date > 0 && date <= this.getDaysInIslamicMonth(this._month, this._year)){
			this._date = date;
		}else{
			var mdays;
			if(date>0){
				for(mdays = this.getDaysInIslamicMonth(this._month, this._year);
					date > mdays;
						date -= mdays,mdays =this.getDaysInIslamicMonth(this._month, this._year)){
					this._month++;
					if(this._month >= 12){this._year++; this._month -= 12;}
				}

				this._date = date;
			}else{
				for(mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ?(this._month-1) :11 ,((this._month-1)>=0)? this._year: this._year-1);
						date <= 0;
							mdays = this.getDaysInIslamicMonth((this._month-1)>=0 ? (this._month-1) :11,((this._month-1)>=0)? this._year: this._year-1)){
					this._month--;
					if(this._month < 0){this._year--; this._month += 12;}

					date+=mdays;
				}
				this._date = date;
			}
		}
		return this;
	},

	setFullYear:function(/*number*/year){
		// summary: This function set Year
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setYear(1429);

		this._year = +year;
	},

	setMonth: function(/*number*/month) {
		// summary: This function set Month
		//
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		date1.setMonth(2);

		this._year += Math.floor(month / 12);
		if(month > 0){
			this._month = Math.floor(month % 12);
		}else{
			this._month = Math.floor(((month % 12) + 12) % 12);
		}
	},

	setHours:function(){
		//summary: set the Hours
		var hours_arg_no = arguments.length;
		var hours = 0;
		if(hours_arg_no >= 1){
			hours = parseInt(arguments[0]);
		}

		if(hours_arg_no >= 2){
			this._minutes = parseInt(arguments[1]);
		}

		if(hours_arg_no >= 3){
			this._seconds = parseInt(arguments[2]);
		}

		if(hours_arg_no == 4){
			this._milliseconds = parseInt(arguments[3]);
		}

		while(hours >= 24){
			this._date++;
			var mdays = this.getDaysInIslamicMonth(this._month, this._year);
			if(this._date > mdays){
					this._month ++;
					if(this._month >= 12){this._year++; this._month -= 12;}
					this._date -= mdays;
			}
			hours -= 24;
		}
		this._hours = hours;
	},

	setMinutes:function(/*number*/minutes){
		//summary: set the Minutes

		while(minutes >= 60){
			this._hours++;
			if(this._hours >= 24){
				this._date++;
				this._hours -= 24;
				var mdays = this.getDaysInIslamicMonth(this._month, this._year);
				if(this._date > mdays){
						this._month ++;
						if(this._month >= 12){this._year++; this._month -= 12;}
						this._date -= mdays;
				}
			}
			minutes -= 60;
		}
		this._minutes = minutes;
	},
		
		
	setSeconds:function(/*number*/seconds){
		//summary: set Seconds
		while(seconds >= 60){
			this._minutes++;
			if(this._minutes >= 60){
				this._hours++;
				this._minutes -= 60;
				if(this._hours >= 24){
					this._date++;
					this._hours -= 24;
					var mdays = this.getDaysInIslamicMonth(this._month, this._year);
					if(this._date > mdays){
						this._month ++;
						if(this._month >= 12){this._year++; this._month -= 12;}
						this._date -= mdays;
					}
				}
			}
			seconds -= 60;
		}
		this._seconds = seconds;
	},
		
	setMilliseconds:function(/*number*/milliseconds){
		//summary: set the Millisconds
		while(milliseconds >= 1000){
			this.setSeconds++;
			if(this.setSeconds >= 60){
				this._minutes++;
				this.setSeconds -= 60;
				if(this._minutes >= 60){
					this._hours++;
					this._minutes -= 60;
					if(this._hours >= 24){
						this._date++;
						this._hours -= 24;
						var mdays = this.getDaysInIslamicMonth(this._month, this._year);
				if(this._date > mdays){
					this._month ++;
					if(this._month >= 12){this._year++; this._month -= 12;}
					this._date -= mdays;
					}
				}
			}
		}
			milliseconds -= 1000;
		}
		this._milliseconds = milliseconds;
	},
		
		
	toString:function(){
		// summary: This returns a string representation of the date in "DDDD MMMM DD YYYY HH:MM:SS" format
		// example:
		// |		var date1 = new dojox.date.islamic.Date();
		// |		document.writeln(date1.toString());

		//FIXME: TZ/DST issues?
		var x = new Date();
		x.setHours(this._hours);
		x.setMinutes(this._minutes);
		x.setSeconds(this._seconds);
		x.setMilliseconds(this._milliseconds);
		return this._month+" "+ this._date + " " + this._year + " " + x.toTimeString();
	},
		
		
	toGregorian:function(){
		// summary: This returns the equevalent Grogorian date value in Date object
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date(1429,11,20);
		// |		var dateGregorian = dateIslamic.toGregorian();

		var hYear = this._year;
		var hMonth = this._month;
		var hDate = this._date;
		var julianDay = hDate + Math.ceil(29.5 * hMonth) + (hYear - 1) * 354
						+ Math.floor((3 + (11 * hYear)) / 30) + this._ISLAMIC_EPOCH - 1;

		var wjd = Math.floor(julianDay - 0.5) + 0.5,
			depoch = wjd - this._GREGORIAN_EPOCH,
			quadricent = Math.floor(depoch / 146097),
			dqc = this._mod(depoch, 146097),
			cent = Math.floor(dqc / 36524),
			dcent = this._mod(dqc, 36524),
			quad = Math.floor(dcent / 1461),
			dquad = this._mod(dcent, 1461),
			yindex = Math.floor(dquad / 365),
			year = (quadricent * 400) + (cent * 100) + (quad * 4) + yindex;
		if(!(cent == 4 || yindex == 4)){
			year++;
		}
		
		var gYearStart = this._GREGORIAN_EPOCH + (365 * (year - 1)) + Math.floor((year - 1) / 4)
						- ( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400);
						
		var yearday = wjd - gYearStart;
		
		var tjd = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1)) + Math.floor((year - 1) / 4)
				-( Math.floor((year - 1) / 100)) + Math.floor((year - 1) / 400) + Math.floor( (739 / 12)
				+ ( (dojo.date.isLeapYear(new Date(year,3,1)) ? -1 : -2)) + 1);
			
		var leapadj = ((wjd < tjd ) ? 0 : (dojo.date.isLeapYear(new Date(year,3,1)) ? 1 : 2));
					
		var month = Math.floor((((yearday + leapadj) * 12) + 373) / 367);
		var tjd2 = (this._GREGORIAN_EPOCH - 1) + (365 * (year - 1))
					+ Math.floor((year - 1) / 4) - (Math.floor((year - 1) / 100))
					+ Math.floor((year - 1) / 400) + Math.floor((((367 * month) - 362) / 12)
					+ ((month <= 2) ? 0 : (dojo.date.isLeapYear(new Date(year,month,1)) ? -1 : -2)) + 1);
					
		var day = (wjd - tjd2) + 1;

		var gdate = new Date(year, (month - 1), day, this._hours, this._minutes, this._seconds, this._milliseconds);

		return gdate;
	},

	//TODO: would it make more sense to make this a constructor option? or a static?
	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	fromGregorian:function(/*Date*/gdate){
		// summary: This function returns the equivalent Islamic Date value for the Gregorian Date
		// example:
		// |		var dateIslamic = new dojox.date.islamic.Date();
		// |		var dateGregorian = new Date(2008,10,12);
		// |		dateIslamic.fromGregorian(dateGregorian);

		var date = new Date(gdate);
		var gYear = date.getFullYear(),
			gMonth = date.getMonth(),
			gDay = date.getDate();
		
		var julianDay = (this._GREGORIAN_EPOCH - 1) + (365 * (gYear - 1)) + Math.floor((gYear - 1) / 4)
					+ (-Math.floor((gYear - 1) / 100)) + Math.floor((gYear - 1) / 400)
					+ Math.floor((((367 * (gMonth+1)) - 362) / 12)
					+ (((gMonth+1) <= 2) ? 0 : (dojo.date.isLeapYear(date) ? -1 : -2)) + gDay);
		julianDay = Math.floor(julianDay) + 0.5;

		var days = julianDay - this._ISLAMIC_EPOCH;
		var hYear  = Math.floor( (30 * days + 10646) / 10631.0 );
		var hMonth = Math.ceil((days - 29 - this._yearStart(hYear)) / 29.5 );
		hMonth = Math.min(hMonth, 11);
		var hDay = Math.ceil(days - this._monthStart(hYear, hMonth)) + 1;

		this._date = hDay;
		this._month = hMonth;
		this._year = hYear;
		this._hours = date.getHours();
		this._minutes = date.getMinutes();
		this._seconds = date.getSeconds();
		this._milliseconds = date.getMilliseconds();
		this._day = date.getDay();
		return this;
	},
	
	valueOf:function(){
		// summary: This function returns The stored time value in milliseconds
		// since midnight, January 1, 1970 UTC

		return this.toGregorian().valueOf();
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_yearStart:function(/*Number*/year){
		//summary: return start of Islamic year
		return (year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_monthStart:function(/*Number*/year, /*Number*/month){
		//summary: return the start of Islamic Month
		return Math.ceil(29.5*month) +
			(year-1)*354 + Math.floor((3+11*year)/30.0);
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	_civilLeapYear:function(/*Number*/year){
		//summary: return Boolean value if Islamic leap year
		return (14 + 11 * year) % 30 < 11;
	},

	// ported from the Java class com.ibm.icu.util.IslamicCalendar from ICU4J v3.6.1 at http://www.icu-project.org/
	getDaysInIslamicMonth:function(/*Number*/month, /*Number*/ year){
		//summary: returns the number of days in the given Islamic Month
		var length = 0;
		length = 29 + ((month+1) % 2);
		if(month == 11 && this._civilLeapYear(year)){
			length++;
		}
		return length;
	},

	_mod:function(a, b){
		return a - (b * Math.floor(a / b));
	}
});

//TODOC
dojox.date.islamic.Date.getDaysInIslamicMonth = function(/*dojox.date.islamic.Date*/month){
	return new dojox.date.islamic.Date().getDaysInIslamicMonth(month.getMonth(),month.getFullYear()); // dojox.date.islamic.Date
};

}

if(!dojo._hasResource["dojox.date.islamic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic"] = true;
dojo.provide("dojox.date.islamic");


 // for compare
	
// Utility methods to do arithmetic calculations with islamic.Dates

	// added for compat to date
dojox.date.islamic.getDaysInMonth = function(/*islamic.Date*/month){
	return month.getDaysInIslamicMonth(month.getMonth(), month.getFullYear());
};

//TODO: define islamic.isLeapYear?  Or should it be invalid, since it has different meaning?

dojox.date.islamic.compare = function(/*islamic.Date*/date1, /*islamic.Date*/date2, /*String?*/portion){
	//	summary:
	//		Compare two islamic date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		islamic.Date object
	//	date2:
	//		islamic.Date object.  If not specified, the current islamic.Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	if(date1 instanceof dojox.date.islamic.Date){
		date1 = date1.toGregorian();
	}
	if(date2 instanceof dojox.date.islamic.Date){
		date2 = date2.toGregorian();
	}
	
	return dojo.date.compare.apply(null, arguments);
};

dojox.date.islamic.add = function(/*dojox.date.islamic.Date*/date, /*String*/interval, /*int*/amount){
	//	based on and similar to dojo.date.add
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: islamic.Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var newIslamDate = new dojox.date.islamic.Date(date);

	switch(interval){
		case "day":
			newIslamDate.setDate(date.getDate() + amount);
			break;
		case "weekday":
			var day = date.getDay();
			if(((day + amount) < 5) && ((day + amount) > 0)){
				 newIslamDate.setDate(date.getDate() + amount);
			}else{
				var adddays = 0, /*weekend */
					remdays = 0;
				if(day == 5){//friday
					day = 4;
					remdays = (amount > 0) ?  -1 : 1;
				}else if(day == 6){ //shabat
					day = 4;
					remdays = (amount > 0) ? -2 : 2;
				}
				var add = (amount > 0) ? (5 - day - 1) : -day
				var amountdif = amount - add;
				var div = parseInt(amountdif / 5);
				if(amountdif % 5 != 0){
					adddays = (amount > 0)  ? 2 : -2;
				}
				adddays = adddays + div * 7 + amountdif % 5 + add;
				newIslamDate.setDate(date.getDate() + adddays +  remdays);
			}
			break;
		case "year":
			newIslamDate.setFullYear(date.getFullYear() + amount);
			break;
		case "week":
			amount *= 7;
			newIslamDate.setDate(date.getDate() + amount);
			break;
		case "month":
			var month = date.getMonth();
			newIslamDate.setMonth(month + amount);
			break;
		case "hour":
			newIslamDate.setHours(date.getHours() + amount);
			break;
		case "minute":
			newIslamDate.setMinutes(date.getMinutes() + amount);
			break;
		case "second":
			newIslamDate.setSeconds(date.getSeconds() + amount);
			break;
		case "millisecond":
			newIslamDate.setMilliseconds(date.getMilliseconds() + amount);
			break;
	}

	return newIslamDate; // dojox.date.islamic.Date
};

dojox.date.islamic.difference = function(/*dojox.date.islamic.Date*/date1, /*dojox.date.islamic.Date?*/date2, /*String?*/interval){
	//	based on and similar to dojo.date.difference
	//	summary:
	//        date1 - date2
	//	 date2 is islamic.Date object.  If not specified, the current islamic.Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond",  "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new dojox.date.islamic.Date();
	interval = interval || "day";
	var yearDiff = date1.getFullYear() - date2.getFullYear();
	var delta = 1; // Integer return value
	switch(interval){
		case "weekday":
			var days = Math.round(dojox.date.islamic.difference(date1, date2, "day"));
			var weeks = parseInt(dojox.date.islamic.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date2.getDay();
				var bDay = date1.getDay();
	
				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new dojox.date.islamic.Date(date2);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();
	
				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = -1;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = -1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Fri
						case aDay == 5:
							adj = 0;
							break;
						// Range starts on Sat
						case aDay == 6:
							adj = 1;
							break;
						// Range ends on Fri
						case bDay == 5:
							adj = 2;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			var startdate =  (date1.toGregorian() > date2.toGregorian()) ? date1 : date2; // more
			var enddate = (date1.toGregorian() > date2.toGregorian()) ? date2 : date1;
			
			var month1 = startdate.getMonth();
			var month2 = enddate.getMonth();
			
			if (yearDiff == 0){
				delta = startdate.getMonth() - enddate.getMonth() ;
			}else{
				delta = 12-month2;
				delta +=  month1;
				var i = enddate.getFullYear()+1;
				var e = startdate.getFullYear();
				for (i;   i < e;  i++){
					delta += 12;
				}
			}
			if (date1.toGregorian() < date2.toGregorian()){
				delta = -delta;
			}
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojox.date.islamic.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date1.toGregorian().getTime()- date2.toGregorian().getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojox.date.islamic.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.date.islamic.locale"] = true;
dojo.provide("dojox.date.islamic.locale");









(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, locale, fullYear,  pattern){

		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s, pad;
			var c = match.charAt(0);
			var l = match.length;
			var widthList = ["abbr", "wide", "narrow"];
			
			switch(c){
				case 'G':
					s = bundle["eraAbbr"][0];
					break;
				case 'y':
					s = String(dateObject.getFullYear());
					break;
				case 'M':
					var m = dateObject.getMonth();
					if(l<3){
						s = m+1; pad = true;
					}else{
						var propM = ["months", "format", widthList[l-3]].join("-");
						s = bundle[propM][m];
					}
					break;
				case 'd':
					s = dateObject.getDate(true); pad = true;
					break;
				case 'E':
					var d = dateObject.getDay();
					if(l<3){
						s = d+1; pad = true;
					}else{
						var propD = ["days", "format", widthList[l-3]].join("-");
						s = bundle[propD][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle['dayPeriods-format-wide-' + timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					switch (c){
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.getTimezoneName(dateObject.toGregorian());
					if(s){ break; }
					l = 4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dateObject.toGregorian().getTimezoneOffset();
					var tz = [
						(offset <= 0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset) / 60), 2),
						dojo.string.pad(Math.abs(offset) % 60, 2)
					];
					if(l == 4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
				default:
					throw new Error("dojox.date.islamic.locale.formatPattern: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}
	
// based on and similar to dojo.date.locale.format
dojox.date.islamic.locale.format = function(/*islamic.Date*/dateObject, /*Object?*/options){
	// summary:
	//		Format a Date object as a String, using  settings.
	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojox.date.islamic.locale._getIslamicBundle(locale);
	var str = [];

	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle, locale, options.fullYear);
	if(options.selector == "year"){
		var year = dateObject.getFullYear();
		return year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time

	return result; // String
};

dojox.date.islamic.locale.regexp = function(/*object?*/options){
	//	based on and similar to dojo.date.locale.regexp
	// summary:
	//		Builds the regular needed to parse a islamic.Date
	return dojox.date.islamic.locale._parseInfo(options).regexp; // String
};

dojox.date.islamic.locale._parseInfo = function(/*oblect?*/options){
/* based on and similar to dojo.date.locale._parseInfo */

	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojox.date.islamic.locale._getIslamicBundle(locale);
	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];

	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = (typeof (timePattern) == "undefined") ? datePattern : datePattern + ' ' + timePattern;
	}

	var tokens = [];
	
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};




dojox.date.islamic.locale.parse= function(/*String*/value, /*Object?*/options){
	// based on and similar to dojo.date.locale.parse
	// summary: This function parse string date value according to options
	
	value =  value.replace(/[\u200E\u200F\u202A\u202E]/g, ""); //remove bidi non-printing chars

	if(!options){ options={}; }
	var info = dojox.date.islamic.locale._parseInfo(options);

	var tokens = info.tokens, bundle = info.bundle;
	var regexp = info.regexp.replace(/[\u200E\u200F\u202A\u202E]/g, ""); //remove bidi non-printing chars from the pattern
	var re = new RegExp("^" + regexp + "$");

	var match = re.exec(value);

	var locale = dojo.i18n.normalizeLocale(options.locale);

	if(!match){ return null; } // null
	
	var date, date1;
	
	var result = [1389,0,1,0,0,0,0];  //FIXME: islamic date for [1970,0,1,0,0,0,0] used in gregorian locale
	var amPm = "";
	var mLength = 0;
	var widthList = ["abbr", "wide", "narrow"];
	var valid = dojo.every(match, function(v, i){
		if(!i){return true;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				result[0] = Number(v);
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s ? s.replace(".","").toLowerCase() : s; } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
						return false;
					}
					mLength = l;
				}else{
					v--;
				}
				result[1] = Number(v);
				break;
			case 'D':
				result[1] = 0;
				// fallthrough...
			case 'd':
					result[2] =  Number(v);
				break;
			case 'a': //am/pm
				var am = options.am || bundle['dayPeriods-format-wide-am'],
					pm = options.pm || bundle['dayPeriods-format-wide-pm'];
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
					return false;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v == 24){ v = 0; }
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result[3] = Number(v);
				break;
			case 'm': //minutes
				result[4] = Number(v);
				break;
			case 's': //seconds
				result[5] = Number(v);
				break;
			case 'S': //milliseconds
				result[6] = Number(v);
		}
		return true;
	});

	var hours = +result[3];
	if(amPm === 'p' && hours < 12){
		result[3] = hours + 12; //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result[3] = 0; //12am -> 0
	}
	var dateObject = new dojox.date.islamic.Date(result[0], result[1], result[2], result[3], result[4], result[5], result[6]);
	return dateObject;
};


function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings)
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g);
	var literal = pattern.charAt(0) == "'";

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE  (tokens, bundle, options, pattern){
		// based on and similar to dojo.date.locale._buildDateTimeRE
		//
	
	pattern = dojo.regexp.escapeString(pattern);
	var locale = dojo.i18n.normalizeLocale(options.locale);
	
	return pattern.replace(/([a-z])\1*/ig, function(match){

			// Build a simple regexp.  Avoid captures, which would ruin the tokens list
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var p2 = '', p3 = '';
			if(options.strict){
				if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
				if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
			}else{
				p2 = '0?'; p3 = '0{0,2}';
			}
			switch(c){
				case 'y':
					s = '\\d+';
					break;
				case 'M':
					s = (l>2) ?  '\\S+ ?\\S+' : p2+'[1-9]|1[0-2]';
					break;
				case 'd':
					s = '[12]\\d|'+p2+'[1-9]|3[01]';
					break;
				case 'E':
					s = '\\S+';
					break;
				case 'h': //hour (1-12)
					s = p2+'[1-9]|1[0-2]';
					break;
				case 'k': //hour (0-11)
					s = p2+'\\d|1[01]';
					break;
				case 'H': //hour (0-23)
					s = p2+'\\d|1\\d|2[0-3]';
					break;
				case 'K': //hour (1-24)
					s = p2+'[1-9]|1\\d|2[0-4]';
					break;
				case 'm':
				case 's':
					s = p2+'\\d|[0-5]\\d';
					break;
				case 'S':
					s = '\\d{'+l+'}';
					break;
				case 'a':
					var am = options.am || bundle['dayPeriods-format-wide-am'],
						pm = options.pm || bundle['dayPeriods-format-wide-pm'];
					if(options.strict){
						s = am + '|' + pm;
					}else{
						s = am + '|' + pm;
						if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
						if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
					}
					break;
				default:
					s = ".*";
			}
			if(tokens){ tokens.push(match); }
			return "(" + s + ")"; // add capture
		}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE. */
}
})();



(function(){
var _customFormats = [];
dojox.date.islamic.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	_customFormats.push({pkg:packageName,name:bundleName});
};

dojox.date.islamic.locale._getIslamicBundle = function(/*String*/locale){
	var islamic = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		islamic = dojo.mixin(islamic, bundle);
	}, this);
	return islamic; /*Object*/
};
})();

dojox.date.islamic.locale.addCustomFormats("dojo.cldr","islamic");

dojox.date.islamic.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/context, /*String?*/locale, /*islamic Date Object?*/date){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	var label;
	var lookup = dojox.date.islamic.locale._getIslamicBundle(locale);
	var props = [item, context, type];
	if(context == 'standAlone'){
		var key = props.join('-');
		label = lookup[key];
		// Fall back to 'format' flavor of name
		if(label[0] == 1){ label = undefined; } // kludge, in the absence of real aliasing support in dojo.cldr
	}
	props[1] = 'format';
	
	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};


dojox.date.islamic.locale.weekDays = dojox.date.islamic.locale.getNames('days', 'wide', 'format');

dojox.date.islamic.locale.months = dojox.date.islamic.locale.getNames('months', 'wide', 'format');

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.Util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.Util"] = true;
/*
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.util.common.Util");

















/**
 * creates a secure string which can be set using DOMElement.innerHTML
 *
 * @param innerHTML the original inner HTML string
 * @returns {String} the secure inner HTML string
 */
com.ibm.bpm.social.util.common.Util.createSecureInnerHtml = function(/*String*/ innerHTML) {
  if(typeof innerHTML == "string") {
    var secureInnerHtmlPreparation = new com.ibm.bpm.social.util.common.SecureInnerHtmlPreparation(innerHTML);
    return secureInnerHtmlPreparation.getSecureString();
  } else {
    return innerHTML;
  }
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS.sssZ
 */
com.ibm.bpm.social.util.common.Util.getDateString = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getFullYear() + "-";
  if (d.getMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getMonth() + 1) + '-';
  if (d.getDate() < 10) {
    result = result + '0';
  }
  result = result + d.getDate() + "T";
  if (d.getHours() < 10) {
    result = result + '0';
  }
  result = result + d.getHours() + ":";
  if (d.getMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getMinutes() + ":";
  if (d.getSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getSeconds() + ".";
  if (d.getMilliseconds() < 100) {
    result = result + '0';
  }
  if (d.getMilliseconds() < 10) {
    result = result + '0';
  }
  result = result + d.getMilliseconds();
  var offset = d.getTimezoneOffset();
  var modifier = "";
  if (offset < 0) {
    modifier = "+";
  } else {
	modifier = "-";
  }
  var tzh = Math.floor(Math.abs(offset) / 60);
  var tzm = offset - (tzh * 60);
  if (tzh < 10) {
    tzh = '0' + tzh;
  }
  result = result + modifier + tzh + ":";
  if (tzm < 10) {
    result = result + '0';
  }
  result = result + tzm;
  return result;
};

com.ibm.bpm.social.util.common.Util.fixDateTimeZone = function(str){
	var i = str.indexOf('Z');
	if (i != -1) {
		str = str.substring(0,i);
		if (i <20) {
			str = str + '.000';
		}
		str = str + '+0000';
	}
	return str;
};

com.ibm.bpm.social.util.common.Util.formatDate = function(str){
	var date,d1,t1;
	
	str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
	date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);
	
    d1 = com.ibm.bpm.social.util.common.Util.formatDateObject(date, {formatLength: "long", selector: "date"});
	t1 = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});
	
	return (d1+" "+t1);
};

com.ibm.bpm.social.util.common.Util.formatDateObject = function(d, options){
  var ret = "";
  var calPref = com.ibm.bpm.social.util.common.Util.getCalendarUserPreference();
  if (calPref == "hebrew") {
      var dateObj = new dojox.date.hebrew.Date(d);
      ret = dojox.date.hebrew.locale.format(dateObj, options);
  } else if (calPref == "islamic") {
      var dateObj = new dojox.date.islamic.Date(d);
      ret = dojox.date.islamic.locale.format(dateObj, options);
  } else {
      ret = dojo.date.locale.format(d, options);
  }
  return ret;
};

/*
 *  @param Date string
 *  @returns String in long date format, no time
 */
com.ibm.bpm.social.util.common.Util.formatDateLong = function(str){
  var date, d1;

  str = com.ibm.bpm.social.util.common.Util.fixDateTimeZone(str);
  date = com.ibm.bpm.social.util.common.Util.getDateFromString(str);

  d1 = dojo.date.locale.format(date, {
    formatLength: "long",
    selector: "date"
  });
  return d1;
};

/*
 *  @param Date object
 *  @returns String in format of YYYY-MM-DDTHH:MM:SS
 */
com.ibm.bpm.social.util.common.Util.getTimestampUTC = function(d){
  var result = "";
  if (!d) {
    d = new Date();
  }
  result = result + d.getUTCFullYear() + "-";
  if (d.getUTCMonth() < 9) {
    result = result + '0';
  }
  result = result + (d.getUTCMonth() + 1) + '-';
  if (d.getUTCDate() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCDate() + "T";
  if (d.getUTCHours() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCHours() + ":";
  if (d.getUTCMinutes() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCMinutes() + ":";
  if (d.getUTCSeconds() < 10) {
    result = result + '0';
  }
  result = result + d.getUTCSeconds();
  // skip milliseconds and skip TIMEZONE identifier
  return result;
};

com.ibm.bpm.social.util.common.Util.getDateFromString = function(str){

	//Normalize "2011-12-19T10:36:56.060-05:00" to "2011-12-19T10:36:56.060-0500"
	if(str.lastIndexOf(":") > 23){
		var substr1 = str.substr(23,6);
		var substr2 = substr1.replace(":", "");
		str = str.replace(substr1, substr2);
	}
	
	var yr = parseInt(str.substr(0, 4));
	var tmp = str.substr(5, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var mnth = parseInt(tmp);
	tmp = str.substr(8, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var dy = parseInt(tmp);
	tmp = str.substr(11, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var hr = parseInt(tmp);
	tmp = str.substr(14, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var min = parseInt(tmp);
	tmp = str.substr(17, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var sec = parseInt(tmp);
	tmp = str.substr(20, 3);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var msec = parseInt(tmp);
	var result = new Date(yr, mnth - 1, dy, hr, min, sec, msec);
	tmp = str.substr(23, 3);
	var isNegative = (tmp.charAt(0) == '-');
	tmp = tmp.substr(1);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzh = parseInt(tmp);
	tmp = str.substr(26, 2);
	if (tmp.charAt(0) == '0') tmp = tmp.substr(1);
	var tzm = parseInt(tmp);
	if (isNegative) {
		tzh = -tzh;
		tzm = -tzm;
	}
	var offset = result.getTimezoneOffset();
	offset = offset + (tzh * 60) + tzm;
	result = new Date(result.getTime() - (offset * 60000));
	return result;
};

com.ibm.bpm.social.util.common.Util.formatServerDateString = function(str){
  var dateStr;
  var d;

  if (dojo.isIE) {
    d = dojo.date.stamp.fromISOString(str);
  }
  else {
    d = new Date(str);
  }

  dateStr = d.toDateString() + " " + d.toLocaleTimeString();
  dateStr = dateStr.substring(4, dateStr.length);
  dateStr = dateStr.substring(0, 6) + "," + dateStr.substring(6, dateStr.length);

  return dateStr;
};

com.ibm.bpm.social.util.common.Util.portalPIIMessages = null;

com.ibm.bpm.social.util.common.Util.restConfigurationException = function(error, xhrArgs){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	if(com.ibm.bpm.social.util.common.Util.portalPIIMessages==null){
		//NOTE:  Locale 'en' is hard-coded here as the required message is only available in the English bundle for 8.0
		dojo.xhrGet({
			"url": ppConfig.portalContextRootJS+"/resources/PortalPIIMessages_en.json",
			"sync": true,
			"handleAs": "json",
			"load": function restConfigurationException_xhrGetLoad(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util.portalPIIMessages = data;
				com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
			},
			"error": function(error, xhrArgs){
				com.ibm.bpm.social.util.common.Log.logError(2, "PortalPIIMessages_en.json could not be retrieved.");
			}
		});
	} else {
		com.ibm.bpm.social.util.common.Util.openRestConfigurationException();
	}
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.openRestConfigurationException = function(){
	com.ibm.bpm.social.util.common.Log.logEntry(1);
	
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"]);
	com.ibm.bpm.social.util.common.Log.logError(2, com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"]);
	
	var details = "<p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.explanation"];
	details += "</p><p>";
	details += com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE.useraction"];
	details += "</p>";
	
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(com.ibm.bpm.social.util.common.Util.portalPIIMessages["REST_APP_UNAVAIABLE"], details);
	com.ibm.bpm.social.util.common.Log.logExit(1);
};

com.ibm.bpm.social.util.common.Util.userData = {};

com.ibm.bpm.social.util.common.Util.getCurrentUser = function(){
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentUser) {
    com.ibm.bpm.social.util.common.Util.currentUser = {
      userName: ""
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/current?parts=none",
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentUser = data.data;
        com.ibm.bpm.social.util.common.Util.userData[data.data.userName] = data.data;
      },
      error: function(error, xhrArgs){
    	  com.ibm.bpm.social.util.common.Util.restConfigurationException(error, xhrArgs);
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentUser;
};

com.ibm.bpm.social.util.common.Util.getUser = function(username, ignoreCacheOnDirty){

  if(typeof(ignoreCacheOnDirty)=="undefined"){
	  ignoreCacheOnDirty = false;
  }
	
  if (com.ibm.bpm.social.util.common.Util.userData == null) {
    com.ibm.bpm.social.util.common.Util.userData = {};
  }

  if (username == null) {
    return com.ibm.bpm.social.util.common.Util.getCurrentUser();
  }

  if (!com.ibm.bpm.social.util.common.Util.userData[username] ||
		  ((typeof(com.ibm.bpm.social.util.common.Util.userData[username].dirty)=="undefined"||
			com.ibm.bpm.social.util.common.Util.userData[username].dirty) && ignoreCacheOnDirty) ) {
    com.ibm.bpm.social.util.common.Util.userData[username] = {
      userName: username
    };
    dojo.xhrGet({
      url: window.endpointURLObject.bpmRestJS +"/v1/user/" + encodeURIComponent(username) + "?parts=none&preventCache=" + new Date().getTime(),
      handleAs: "json",
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.userData[username] = data.data;
        com.ibm.bpm.social.util.common.Util.userData[username].dirty = false;
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.userData[username];
};

com.ibm.bpm.social.util.common.Util.primeUserCache = function(id, name, url){
	if(typeof(com.ibm.bpm.social.util.common.Util.userData[id])=="undefined"){
		var newUser = {
				"userName": id,
				"fullName": name,
				"userPreferences": {
					"Image": url
				},
				"dirty": true
		};
		com.ibm.bpm.social.util.common.Util.userData[id] = newUser;
	}
};

com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};

/**
 * Returns the URL for the user's avatar.
 * @param username the username (or user id) of the user you want to see the avatar for.
 * @param ignoreCache true if a new URL should be returned since the last time this method was called.
 */
com.ibm.bpm.social.util.common.Util.getUserAvatarURL = function(username, ignoreCache){
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL entering with username:{"+username+"}");
	if(typeof(ignoreCache)=="undefined"){
		ignoreCache = false;
	}
	
	if (com.ibm.bpm.social.util.common.Util.userAvatarURLs == null) {
		com.ibm.bpm.social.util.common.Util.userAvatarURLs = {};
	}
	
	if (username == null) {
		return null;
	}

   	if (ignoreCache || !com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]) {
	   	com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = window.endpointURLObject.bpmRestJS +"/v1/avatar/image/" + encodeURIComponent(username) + "?preventCache=" + new Date().getTime();
		if(typeof(connectionsBridge)!="undefined" 
			&& connectionsBridge.bootstrap==true 
			&& connectionsBridge.isAvatarServiceAvailable()){ //Is Connections available and configured?
			
			var user = com.ibm.bpm.social.util.common.Util.getUser(username);
			if(typeof(user["emailAddress"])!="undefined" 
				&& user["emailAddress"]!=null 
				&& user["emailAddress"].length > 0){ //Does the user have a defined email address?
				
				var avatar = connectionsBridge.getAvatarUrl(user["emailAddress"]);
				if(avatar!=null){
					com.ibm.bpm.social.util.common.Util.userAvatarURLs[username] = avatar;
				}
			}
		}		
	}
	//console.warn("com.ibm.bpm.social.util.common.Util.getUserAvatarURL returning {"+com.ibm.bpm.social.util.common.Util.userAvatarURLs[username]+"}");
	return com.ibm.bpm.social.util.common.Util.userAvatarURLs[username];
};

com.ibm.bpm.social.util.common.Util._integrations = null;

com.ibm.bpm.social.util.common.Util.getSametimeURL = function(){
  var ints = com.ibm.bpm.social.util.common.Util._getIntegrations()
  return ints["SAMETIME_SERVER"];
};

com.ibm.bpm.social.util.common.Util.getConnectionsURL = function(){
	var ints = com.ibm.bpm.social.util.common.Util._getIntegrations();
	return ints["CONNECTIONS_SERVER"];
};

com.ibm.bpm.social.util.common.Util._getIntegrations = function(){
	if(!com.ibm.bpm.social.util.common.Util._integrations){
		dojo.xhrGet({
			//url: "/rest/bpm/wle/v1/system/env/variable?bpdShortname=TWP&name=SAMETIME_SERVER",
			url: window.endpointURLObject.bpmRestJS +"/v1/social/system/integrations",
			handleAs: "json",
			sync: true,
			headers: {
				"Content-Type": "application/json",
				"Accept": "application/json"
			},
			load: function(data, xhrArgs){
				if (data.data && data.data.collabSystemsList) {
					com.ibm.bpm.social.util.common.Util._integrations = {};
					
					for(var x = 0; x < data.data.collabSystemsList.length; x++){
						com.ibm.bpm.social.util.common.Util._integrations[data.data.collabSystemsList[x].name] = data.data.collabSystemsList[x].value;
					}
					
				} else {
					com.ibm.bpm.social.util.common.Util_integrations = {};
				}
			},
			error: function(data, xhrArgs){
				com.ibm.bpm.social.util.common.Util._integrations = {};
			}
		});
	}
	return com.ibm.bpm.social.util.common.Util._integrations;
}


com.ibm.bpm.social.util.common.Util.getParameter = function(param){
  var result = null;
  if (window.location.search) {
    var search = window.location.search;
    if (search.indexOf("?") === 0) {
      search = search.substring(1); //Remove leading '?' from the query string
    }
    var list = search.split("&");
    for ( var i = 0; i < list.length; i++) {
      var key = "";
      var val = "";
      var j = list[i].indexOf('=');
      if (j == -1) {
        key = list[i];
      }
      else {
        key = list[i].substring(0, j);
        val = list[i].substring(j + 1);
      }
      if (key == param) {
        result = val;
        i = list.length;
      }
    }
  }
  return result;
};

com.ibm.bpm.social.util.common.Util.currentInstanceData = {};

com.ibm.bpm.social.util.common.Util.getInstanceData = function(paramObj){

  var iid = null;
  var params = "none";
  var skipCache = false;
  var updateTaskData = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["diagram"]) {
      p.push("diagram");
    }
    if (paramObj["header"]) {
      p.push("header");
    }
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["executionTree"]) {
      p.push("executionTree");
    }
    if (paramObj["instanceId"]) {
      iid = paramObj["instanceId"];
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
    if (paramObj["updateTaskData"]){
    	updateTaskData = true;
    }

    if (p.length > 0) {
      params = p.join("%7C");
    }
  }

  if(iid==null) return {}; //iid = com.ibm.bpm.social.util.common.Util.getInstanceId();

  if(com.ibm.bpm.social.util.common.Util.currentInstanceData == null) {
	  com.ibm.bpm.social.util.common.Util.currentInstanceData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = {
    		'piid': iid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/process/";
    url += iid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid] = data.data;

        if(updateTaskData==true){
        	for(var i = 0; i < data.data.tasks.length; i++){
        		var t = data.data.tasks[i];
        		com.ibm.bpm.social.util.common.Util.currentTaskData[t.tkiid] = t;
        	}
        }

      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentInstanceData[iid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentInstanceData[iid];
};

com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};

com.ibm.bpm.social.util.common.Util.getTaskTemplateData = function(paramObj) {

  var tktid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["taskTemplateId"]) {
      tktid = paramObj["taskTemplateId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
      skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskTemplateData == null) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = {
        'tktid': tktid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/taskTemplate/";
    url += tktid;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskTemplateData[tktid];
};

com.ibm.bpm.social.util.common.Util.currentTaskData = {};

com.ibm.bpm.social.util.common.Util.getTaskData = function(paramObj){

  var tid = null;
  var params = "none";
  var skipCache = false;
  if (paramObj != null) {
    var p = [];
    if (paramObj["data"]) {
      p.push("data");
    }
    if (paramObj["taskId"]) {
      tid = paramObj["taskId"];
    }
    if (p.length > 0) {
      params = p.join("%7C");
    }
    if (paramObj["skipCache"]){
    	skipCache = true;
    }
  }

  if(com.ibm.bpm.social.util.common.Util.currentTaskData == null) {
	  com.ibm.bpm.social.util.common.Util.currentTaskData = {};
  }

  if (!com.ibm.bpm.social.util.common.Util.currentTaskData[tid] || skipCache) {
    com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = {
    		'tkiid': tid
    };

    var url = window.endpointURLObject.bpmRestJS +"/v1/task/";
    url += tid;
    url += "?parts=";
    url += params;

    dojo.xhrGet({
      url: url,
      handleAs: "json",
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      sync: true,
      load: function(data, xhrArgs){
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid] = data.data;
      },
      error: function(data, ioArgs) {
        com.ibm.bpm.social.util.common.Util.currentTaskData[tid].errorInfo = { data : data, ioArgs: ioArgs};
      }
    });
  }
  return com.ibm.bpm.social.util.common.Util.currentTaskData[tid];
};

com.ibm.bpm.social.util.common.Util.isCommentSupported = function(event){
  if (event.object && event.object.objectType) {
    if (event.object.objectType == 'ibm.bpm.comment') {
      return true;
    }
    if (event.object.objectType.indexOf('ibm.bpm.helprequest') == 0) {
      return true;
    }
  }
  return false;
};

com.ibm.bpm.social.util.common.Util.isIPad = navigator.userAgent.indexOf("iPad") > -1;

com.ibm.bpm.social.util.common.Util.isWebkit = navigator.userAgent.indexOf("WebKit") > -1;

/*
 * Simple method which directs the browser to the login page using the current hostname and port
 */
com.ibm.bpm.social.util.common.Util.redirectToLoginPage = function() {
	var location = window.location;
	var loginPage = 'login.jsp';
	// if the user has a custom logout page, we need to redirect to this
	// page instead of the login.jsp so that they can properly log out of
	// all resources
	if (portalConfigLogoutExitPage && (portalConfigLogoutExitPage != 'login.jsp')) {
		loginPage = 'customLogoutRedirect.jsp';
	}
	var redirUrl = window.endpointURLObject.processPortalJS +"/" +loginPage;
	window.location = redirUrl;
};

/*
 * Method to get the Base Text Direction User preference.  Needs to be used to properly align
 * necessary enabled fields with Base Text Direction.
 *
 * @return BTD 'ltr' || 'rtl' || 'ctx'
 */
com.ibm.bpm.social.util.common.Util.getBTDUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var baseTextDirPref = "ctx";
	
	if(typeof userProfileObj != "undefined") {
		baseTextDirPref = userProfileObj.userPreferences["Base Text Direction"];
		
		if(baseTextDirPref == "generalPreference.fields.baseTextDir.contextual" || baseTextDirPref == "") {
			baseTextDirPref = "ctx";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.ltr") {
			baseTextDirPref = "ltr";
		} else if(baseTextDirPref == "generalPreference.fields.baseTextDir.rtl") {
			baseTextDirPref = "rtl";
		} else {
			//unexpected, default to "ctx"
			baseTextDirPref = "ctx";
		}
	}
	
	return baseTextDirPref;
};

/*
 * Method to get the Calendar Type User preference.  Needs to be used to display
 * dates according to calendar types Gregorian, Hebrew or Islamic.
 *
 * @return calendarType 'gregorian' || 'islamic' || 'hebrew'
 */

com.ibm.bpm.social.util.common.Util.getCalendarUserPreference = function() {
	var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
	var calPref = "gregorian";	
	if(typeof userProfileObj != "undefined") {
		var pref = userProfileObj.userPreferences["Calendar Type"];
        if (pref) {
            if (pref == "generalPreference.fields.calendarType.islamic") {
                calPref = "islamic";
            } else if (pref == "generalPreference.fields.calendarType.hebrew") {
                calPref = "hebrew";
            }
        }
    }
    return calPref;
};

/*
 * Method to get the hash portion of a URL
 */

com.ibm.bpm.social.util.common.Util.getCurrentHash = function(checkCookie) {
	var h = window.location.hash;
   	if (h && (h.indexOf('#dashboards%25') == 0)) {
   		h = '#' + decodeURI(h.substring(1));
   		window.location.hash = h;
   	} 
	if (checkCookie && !h) {
		h = dojo.cookie("com_ibm_bpm_process_portal_hash");
	}
	if (h && (h.charAt(0) == '#')) {
		h = h.substring(1);
	}
	if (h) {
		try {
			h = decodeURIComponent(h);
		} catch(e) {
			// if we cannot decode the text, assmume it is already decoded (FireFox)
		}
	}
	if (h) {
		while (h.length && (h.charAt(h.length-1) == ' ')) {
			h = h.substring(0,h.length-1);
		}
	}
	return h;
}

/*
 * Method to remove a parameter from a URL
 */
com.ibm.bpm.social.util.common.Util.removeParameter = function(url,param,skip) {
	var skipped = 0;
	var start = url.indexOf('?'+param+'=');
	if (start == -1) {
		start = url.indexOf('&'+param+'=');
	}
	while (start != -1) {
		var end = url.indexOf('&',start+1);
		var next = start;
		if (skip && (skip > skipped)) {
			skipped++;
			next++;
		}  else {
			if (end == -1) {
				url = url.substring(0,start);
			} else {
				url = url.substring(0,start+1)+url.substring(end+1);
			}
		}
		start = url.indexOf('?'+param+'=',next);
		if (start == -1) {
			start = url.indexOf('&'+param+'=',next);
		}
	}
	return url;
}

/*
 * Method to set the hash portion of a URL
 */
com.ibm.bpm.social.util.common.Util.setCurrentHash = function(h) {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	for (var i=portalHashHistory.length-1; i>-1; i--) {
		if (portalHashHistory[i] == h) {
			portalHashHistory.splice(i,1);
		}
	}
	if (h) {
		portalHashHistory.push(h);
	}
	if (portalHashHistory.length > 10) {
		portalHashHistory.splice(0,portalHashHistory.length-10);
	}
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS + '/' + h; 
	if (window.history && window.history.replaceState) {
		window.history.replaceState({},'',window.ppConfig.portalContextRootJS+'/'+h);
	} else {
		com.ibm.bpm.social.util.common.Util.lastHash = h;
		window.location.hash = '#'+encodeURIComponent(h);
	}
}

/*
 * Current window url information
 */
if (com.ibm.bpm.social.util.common.Util.urlData == undefined) {
	com.ibm.bpm.social.util.common.Util.urlData = window.ppConfig.portalContextRootJS;
}

/*
 * Show current window url information in dialog
 */
com.ibm.bpm.social.util.common.Util.showProcessPortalUrl = function(event) {
	
  	var messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
	var msg = '<div id="showLinkDialogText">'+messages.showLinkDialog+'</div><br><br><div style="word-wrap:break-word" tabIndex="0" aria-labelledBy="showLinkDialogText" role="textbox" dir="ltr">'+
			  com.ibm.bpm.social.util.common.Util.urlData+
			  '</div>';
	com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(msg);
}

/*
 * Creates an object to publish with the topic com.ibm.bpm.social.portal.Topics.SELECT_DASHBOARD_TAB (Topics.js);
 * This topic is published in many places and the required elements in the object were hard to keep track of
 */
com.ibm.bpm.social.util.common.Util.createObjectForSELECT_DASHBOARD_TAB = function(itemParams) {
	var newObj={};
	newObj.title="";
	newObj.baseID="";
	newObj.itemID="";
	newObj.isTip=false;
	newObj.isSnapshot=false;
	newObj.snapshotName="";
	newObj.isWorkTab=false;
	if (itemParams) {
		if (itemParams.title!=undefined) {
			newObj.title=itemParams.title;
		}
		if (itemParams.baseID!=undefined) {
			newObj.baseID=itemParams.baseID;
		}
		if (itemParams.itemID!=undefined) {
			newObj.itemID=itemParams.itemID;
		}
		if (itemParams.isTip!=undefined) {
			newObj.isTip=itemParams.isTip;
		}
		if (itemParams.isSnapshot!=undefined) {
			newObj.isSnapshot=itemParams.isSnapshot;
		}
		if (itemParams.snapshotName!=undefined) {
			newObj.snapshotName=itemParams.snapshotName;
		}
		if (itemParams.isWorkTab!=undefined) {
			newObj.isWorkTab=itemParams.isWorkTab;
		}
	}
	return newObj;
}
 
/*
 * Method to go to return to the previous hash location
 */
com.ibm.bpm.social.util.common.Util.gotoPreviousHash = function() {
	var portalHashHistory = com.ibm.bpm.social.util.common.Util._portalHashHistory;
	if (!portalHashHistory) {
		portalHashHistory = new Array();
		com.ibm.bpm.social.util.common.Util._portalHashHistory = portalHashHistory;
	}
	if (portalHashHistory.length > 0) {
		portalHashHistory.pop();
	}	
	if (portalHashHistory.length > 0) {
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[portalHashHistory.pop()]);
	} else {
		window.location.hash = '#';
		dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION,[""]);
	}
}
 
 /*
 * Method to get query parameters from a url
 */
 com.ibm.bpm.social.util.common.Util.getQueryParamters = function(url) {
 	var urlParams = new Object(); 
 	if (url && (url.indexOf('?') != -1)) {
		var urlParamString = url.substring(url.indexOf('?')+1);
		var urlParamList = urlParamString.split('&');
		for (var p1=0; p1<urlParamList.length; p1++) {
			var p2 = urlParamList[p1].indexOf('=');
			if (p2 > -1) {
				urlParams[urlParamList[p1].substring(0,p2)] = urlParamList[p1].substring(p2+1);
			}
		}
	}
	return urlParams;
}

com.ibm.bpm.social.util.common.Util.escapeChannelSegment = function(channelSegment) {
  return channelSegment.replace(/~/g, "~007e").replace(/\//g, "~002f");
};
 

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DataCache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.DataCache"] = true;
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT 

//**************************************
//
// Version: 1.9 11/04/13 13:31:35
//              
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.DataCache");



dojo.declare("com.ibm.bpm.social.widgets.task.util.DataCache", null, {
  _data: {},
  _expirationDate: {},
  _usageCount: {},

  constructor: function(){},

  startCleanupProcess: function(period){
    var myself = this;
    window.setInterval(function(){
      myself._removeOutdatedEntries();
    }, period);
  },

  /**
   * Stores the data and the current date under the key.
   * 
   * @param {Object} key
   * @param {Object} data
   */
  set: function set_DataCache(key /* string */, data /* object */, expirationPeriod /* number (milliseconds) */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    this._data[key] = data;
    this._usageCount[key] = 0;
    if (expirationPeriod > 0) {
      this._expirationDate[key] = (new Date()).getTime() + expirationPeriod;
    }
    else {
      // default expiration to keep cache from increasing indefinitely
      // 1 hour = 3600000 milli seconds
      this._expirationDate[key] = (new Date()).getTime() + 3600000;
    }
    com.ibm.bpm.social.util.common.Log.logExit(3, "data:" + data);
  },

  /**
   * If there has been data cached for the url and it has not been expired, serve it.
   * 
   * @param {Object} key
   */
  get: function get_DataCache(key /* string */){
    com.ibm.bpm.social.util.common.Log.logEntry(3, "key: " + key);
    var data = this._data[key];
    if (data) {
      var expirationDate = this._expirationDate[key];
      var now = (new Date()).getTime();
      if (expirationDate > now) {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> data available");
        this._usageCount[key]++;
        return data;
      }
      else {
        com.ibm.bpm.social.util.common.Log.logExit(3, "=> d=> data outdated");
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    else {
      com.ibm.bpm.social.util.common.Log.logExit(3, "=> data not available");
    }
    return null;
  },

  _removeOutdatedEntries: function _removeOutdatedEntries_DataCache(){
    com.ibm.bpm.social.util.common.Log.logEntry(3);
    var now = (new Date()).getTime();
    for ( var key in this._expirationDate) {
      // check also for usage ?
      if (this._expirationDate[key] < now) {
        com.ibm.bpm.social.util.common.Log.log(2, "key:" + key);
        delete this._data[key];
        delete this._expirationDate[key];
        delete this._usageCount[key];
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(3);
  }

});

com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE = new com.ibm.bpm.social.widgets.task.util.DataCache();
com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.startCleanupProcess(60000);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Systems"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.Systems"] = true;
// Systems.js
// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// Copyright IBM Corporation 2010, 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
// END COPYRIGHT 


//**************************************
//
// Version: 1.42 11/10/14 16:06:11
//
//**************************************

dojo.provide("com.ibm.bpm.social.widgets.task.util.Systems");



/**
 * Wraps the result of the <i>/systems</i> REST API call <br>
 * TODO add more getters
 * 
 * @author admard
 * 
 */

/*
 * "systems":
      [
         {  "systemID": {"type": "string", 
               "description": "ID of the system that is described by this object."
            },
            "systemType": {"type": "string",
               "description": "Type of the system that is described by this object.",
               "enum":
               [
                  "WLE",
                  "WPS",
                  "ICM"
               ]
            },
            "version": {"type": "string",
               "description": "Version of the system that is described by this object.",
            },
            "groupWorkItemsEnabled": {"type": "boolean",
               "description": "States whether group workitems can be used."
            },
            "substitutionEnabled": {"type": "boolean",
               "description": "States whether substitution is enabled."
            },
            "substitutionManagementRestrictedToAdministrators": {"type": "boolean",
               "description": "States whether substitution management is restricted to administrators."
            },
            "taskHistoryEnabled": {"type": "boolean",
               "description": "States whether the collection of task history events is enabled."
            },
            "workBasketsEnabled": {"type": "boolean",
               "description": "States whether work baskets are enabled."
            },
            "businessCategoriesEnabled": {"type": "boolean",
               "description": "States whether business categories are enabled."
            }
         }
      ],
   "domains": 
      [
         {  "name": {"type": "string",
               "description": "Name of the domain."
            },
            "displayName": {"type": "string",
               "description": "Display name of the domain."
            },
            "systems": {"type": ["string"],
               "description": "List of IDs for the systems in this domain."
            }
         }
      ], "optional": true
 */

dojo.declare("com.ibm.bpm.social.widgets.task.util.Systems", null, {

  /**
   * The api type of the REST service endpoint.
   * <ul>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   * <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   * <li>TODO temporarily "old" if no system</li>
   * </ul>
   * 
   * @type {String}
   */
  _apiType: null,

  /**
   * Federated endpoint: maps system ids to a system object as returned by the <i>/systems</i> API for all
   * systems
   * 
   * @type {Map}
   */
  _systems: null,

  /**
   * Federated endpoint: an array of all system ids that are available to the federated endpoint systems
   * 
   * @type {Map}
   */
  _systemIDs: null,

  /**
   * Configuration information for single system endpoint
   * 
   * @type {Object}
   */
  _system: null,

  /**
   * Federated endpoint: maps domain names to a domain object as returned by the <i>/systems</i> API for all
   * domains
   * 
   * @type {Map}
   */
  _domains: null,

  /**
   * List of all available domains
   * 
   * @type {String[]}
   */
  _domainNames: null,

  _systemsSending401Error: null,

  /**
   * @constructor
   * @param {Object} params param.systems a systems object as returned by the <i>/systems</i> API</li>
   * @throws Error if the expected parameters are not specified
   */
  constructor: function constructor_Systems(params){

    this._systemsSending401Error = null;
    if (params && params.systemsSending401Error) {
      this._systemsSending401Error = params.systemsSending401Error;
    }

    this._systemIDs = [];

    if (params && params.systems) {

      var systems = params.systems.systems;
      var domains = params.systems.domains;

      if (systems) {
        if (systems.length > 0) {
          // domains
          if (domains && domains.length > 0) {
            // save map of domains
            this._domainNames = [];
            if (domains.length > 0) {
              this._domains = {};
              for (i = 0; i < domains.length; i++) {
                var domain = domains[i];
                this._domains[domain.name] = domain;
                this._domainNames.push(domain.name);
                // TODO: SA REST API workaround - domains.systems may contain null values
                dojo.forEach(domain.systems, function(system, index){
                  if (system === null) {
                    domain.systems.splice(index, 1);
                  }
                });
              }
            }

            // federated endpoint
            this._apiType = com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;

            // save systems
            if (systems.length) {
              this._systems = {};
            }
            for ( var i = 0; i < systems.length; i++) {
              if (systems[i].systemID) {
                this._systems[systems[i].systemID] = systems[i];
                this._systemIDs.push(systems[i].systemID);
              }
            }
          }
          else {
            // single system endpoint
            this._apiType = systems[0].systemType;
            // save system
            this._system = systems[0];
            this._systemIDs.push(systems[0].systemID);
          }
        }
        else {
          // log if no system information was returned
          // the error popup with the federation result should show errors
          com.ibm.bpm.social.util.common.Log.log(1, "Warning!!! No system meta information available! systems=" +
            params.systems);
        }
      }
      else {
        // TODO temp old API - no systems object
        this._apiType = "old";
        this._system = {
          version: "7.0.0.0",
          systemType: com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
          systemID: "oldWPS"
        };
      }
      com.ibm.bpm.social.util.common.Log.log(1, "Systems.constructor: " + this);
    }
    else {
      // ASZ: throw prevents cloning via dojo.clone  
      // throw new Error("Systems.constructor: The parameter 'param.systems' is required.");
      com.ibm.bpm.social.util.common.Log.log(1,
        "Warning!!! The parameter 'param.systems' should be set (Ignore if appears during dojo.clone)");
    }
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_Systems(){
    return this._apiType;
  },

  /**
   * Returns whether the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if and only if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * Returns whether the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if and only if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns whether the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if and only if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns whether the REST service endpoint is an ICM system
   * 
   * @returns {boolean} true if and only if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_Systems(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns the system properties of the REST service endpoint for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system for a federated REST service endpoint
   * @returns {Object} the system properties of a system as returned by the <i>/systems</i> API
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystem: function getSystem_Systems(systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (this._systems[systemID]) {
          return this._systems[systemID];
        }
        else {
          //throw new Error("Systems.getSystem: No system found for systemID=" + systemID);
          com.ibm.bpm.social.util.common.Log.log(1, "Systems.getSystem: No system found for systemID=" + systemID);
          return null;
        }
      }
      else {
        // programming error
        throw new Error("Systems.getSystem: A systemID is required for a federated system!");
      }
    }
    else {
      return this._system;
    }
  },

  /**
   * Returns the systems object
   * 
   * @returns {String[]} an array of all systems ids in all domains
   */
  getSystemIDs: function getSystemIDs_Systems(){
    return this._systemIDs;
  },

  /**
   * Returns the system type for the specified system id.
   * 
   * @param {String} systemID optional, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType for the specified system id or the systemType of the single system if no
   *          systemID was specified
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          </ul>
   * @throws Error if no systemID is specified for federated backend.
   */
  getSystemType: function getSystemType_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType;
    }
    return null;
  },

  /**
   * Returns the estimated system type for the specified system id.
   * 
   * @param {String} systemID mandatory, the id of a backend system of a federated REST service endpoint
   * @returns {String} the systemType derived from the specified system id
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM</li>
   *          <li>null</li>
   *          </ul>
   */
  deriveSystemTypeFromSystemId: function deriveSystemTypeFromSystemId_Systems(systemID){
    if (systemID && systemID.length > 0) {
      // WPS systemID looks like this: "_PK:90230132.462033e2.cdc167f6.795e0002"
      if (systemID.indexOf("_PK:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
      }
      // ICM systemID looks like this: "_ICM:fmtc4235:P8ConnPt1:ICMPersonal"
      if (systemID.indexOf("_ICM:") === 0) {
        return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      // WLE systemID looks like this: "3b1ce0ca-f622-44e4-9ec5-5288ca724856"
      // Since only these 3 types exist and 2 of them can be recognized by a prefix substring,
      //  we can conclude the catch-all case as WLE
      return com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    // garbage-in - garbage-out case: return null if param 1 was null or the empty string
    return null;
  },

  /**
   * Returns whether the system is a WLE system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wle system
   */
  isSystemWle: function isSystemWle_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
    }
    return false;
  },

  /**
   * Returns whether the system is a WPS system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if wps system
   */
  isSystemWps: function isSystemWps_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
    }
    return false;
  },

  /**
   * Returns whether the system is an ICM system
   * 
   * @param systemID The system id of the system.
   * @returns {boolean} true if icm system
   */
  isSystemIcm: function isSystemIcm_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system && system.systemType) {
      return system.systemType == com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
    }
    return false;
  },

  /**
   * Returns the setting for search enabled on the system
   * 
   * @param systemID
   * @returns {boolean} true if search is enabled on system
   */
  isSystemSearchable: function isSystemSearchable_Systems(systemID){
    var system = this.getSystem(systemID);
    if (system) {
      return system.taskSearchEnabled === true;
    }
    return false;

  },


  /**
   * @returns {String} the system id if we have a single system backend
   * @throws Error if the backend is not a single system
   */
  getSystemID: function getSystemID_Systems(){
    if (!this.isApiTypeFederated()) {
      return this._system.systemID;
    }
    else {
      // programming error
      throw new Error("Systems.getSystemID: Method is not supported for Federation!");
    }
  },

  /**
   * Returns the map of all domains for a federated REST service endpoint
   * 
   * @returns {Map} the map of all domains, null if it is not a federated REST service endpoint.
   *          <dl>
   *          <dt>key</dt>
   *          <dd>domainName</dd>
   *          <dt>value</dt>
   *          <dd>Object of domain properties of the domain as returned by the <i>/systems</i> API</dd>
   *          </dl>
   * 
   */
  getDomains: function getDomains_Systems(){
    return this._domains;
  },

  /**
   * Returns a list of all available domain names
   * 
   * @returns {String[]} the list of all domains, null if it is not a federated REST service endpoint.
   */
  getDomainNames: function getDomainNames_Systems(){
    return this._domainNames;
  },


  /**
   * Returns the domain properties of the REST service endpoint for the specified domain name.
   * 
   * @param {String} domainName the name of a backend domain for a federated REST service endpoint
   * @returns {Object} the domain properties of the domain as returned by the <i>/systems</i> API
   * @throws Error if no domain name is specified.
   */
  getDomain: function getDomain_Systems(domainName){
    if (domainName) {
      if (this._domains[domainName]) {
        return this._domains[domainName];
      }
      else {
        // throw new Error("Systems.getDomain: No domain found for domainName=" + domainName);
        com.ibm.bpm.social.util.common.Log.log(1, "Systems.getDomain: No domain found for domainName=" + domainName);
        return null;
      }
    }
    else {
      // programming error
      throw new Error("Systems.getDomain: A domainName is required!");
    }
  },

  /**
   * Returns if the domain contains only WPS systems
   * 
   * @returns {boolean} true if the domain contains only WPS systems
   * @throws Error if no domain name is specified.
   */
  isDomainWpsOnly: function isDomainWpsOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
  },

  /**
   * Returns if the domain contains only WLE systems
   * 
   * @returns {boolean} true if the domain contains only WLE systems
   * @throws Error if no domain name is specified.
   */
  isDomainWleOnly: function isDomainWleOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
  },

  /**
   * Returns if the domain contains only ICM systems
   * 
   * @returns {boolean} true if the domain contains only ICM systems
   * @throws Error if no domain name is specified.
   */
  isDomainIcmOnly: function isDomainIcmOnly_Systems(domainName){
    return this.getDomainType(domainName) === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
  },

  /**
   * Are the systems in the domain wps only, icm only, wle only, or mixed:=more that one (any two or three)
   * 
   * @param domainName
   * @returns {String} the systemType if only one type, or mixed otherwise
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED</li>
   *          </ul>
   * @throws Error if no domain name is specified.
   */
  getDomainType: function getDomainType_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._domainType === undefined) {
      var wps = false;
      var wle = false;
      var icm = false;
      for ( var s = 0; s < domain.systems.length; s++) {
        var systemType = this.getSystemType(domain.systems[s]);
        wps = (wps || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS);
        wle = (wle || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE);
        icm = (icm || systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM);
      }
      if (wps && !wle && !icm) {
        // wps only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS;
      }
      else if (wle && !wps && !icm) {
        // wle only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE;
      }
      else if (icm && !wps && !wle) {
        // icm only
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM;
      }
      else {
        domain._domainType = com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED;
      }
    }
    return domain._domainType;
  },
  /**
   * Check for the domain, if it has a WPS system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWpsSystem: function getWpsSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WPS system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getWleSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not a WLE system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Check for the domain, if it has an ICM system, then return that, otherwise return null.
   * 
   * @param domainName
   * @returns {String} systemId
   * @throws Error if no domain name is specified.
   */
  getIcmSystem: function getIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systemId = null;
    var systems = domain.systems;
    var i = 0;
    while (i < systems.length && !systemId) {
      systemId = systems[i];
      // if it is not an ICM system, we have to search again
      // note that for 401 or 500 errors inside the federated systems call result (status "incomplete") this._systems[systemId] may not be set
      if (!this._systems[systemId] ||
        this._systems[systemId].systemType != com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
        systemId = null;
      }
      i++;
    }
    return systemId;
  },

  /**
   * Returns systems for domain name
   * 
   * @param domainName
   * @returns {object} systemIds
   * @throws Error if no domain name is specified.
   */
  getSystems: function getSystems_Systems(domainName){
    var domain = this.getDomain(domainName);
    var systems = domain.systems;
    return systems;
  },

  /**
   * Return true if a domain of kind WLE_ONLY or WPS_ONLY or ICM_ONLY has multiple systems in it.
   * 
   * @param {String} domainName
   * 
   * @type {Boolean}
   */
  isMultiSystemDomain: function isMultiSystemDomain_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2, "domainName");
    var domain = this.getDomain(domainName);
    var domainType = this.getDomainType(domainName);
    var isMulti = true;
    // if we erroneously got a MIXED domain, then is must be a multiSys domain 
    if (domainType != com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED) {
      isMulti = (domain.systems.length > 1);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "isMultiSystemDomain==" + (isMulti ? "true" : "false"));
    return isMulti;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS, federated with WPS et al., or federated WPS only
   */
  isWps: function isWps_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWpsSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE, federated with WLE et al., or federated WLE only
   */
  isWle: function isWle_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getWleSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM, federated with ICM et al., or federated ICM only
   */
  isIcm: function isIcm_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null; // will be set for single backend
    var systemFound = null; // will be set for federated backend
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      systemFound = this.getIcmSystem(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systemFound !== null || backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WPS or federated WPS only
   */
  isWpsOnly: function isWpsOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single WLE or federated WLE only
   */
  isWleOnly: function isWleOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE;
  },

  /**
   * @param domainName optional
   * @returns {boolean} true if the backend is single ICM or federated ICM only
   */
  isIcmOnly: function isIcmOnly_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = this._getBackendType(domainName);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return backendType === com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM ||
      backendType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
  },


  /**
   * Is there at leaset one icm systems in the domain?
   * 
   * @param domainName
   * @returns {boolean} tru if domain includes ICM system, else false
   * @throws Error if no domain name is specified.
   */
  hasIcmSystem: function hasIcmSystem_Systems(domainName){
    var domain = this.getDomain(domainName);
    if (domain._hasIcm === undefined) {
      var icm = false;
      var s = 0;
      while (s < domain.systems.length && !icm) {
        var systemType = this.getSystemType(domain.systems[s]);
        s++;
        icm = systemType === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM;
      }
      domain._hasIcm = icm;
    }
    return domain._hasIcm;
  },

  /**
   * @param domainName optional
   * @return {String} backendType com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM,
   *         com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM
   */
  _getBackendType: function _getBackendType_Systems(domainName){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var backendType = null;
    if (domainName) {
      // federated backend, may be wps/wle/icm only or mixed
      backendType = this.getDomainType(domainName);
    }
    else {
      // single backend
      backendType = this.getSystemType();
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, backendType);
    return backendType;
  }


});

////////////////////////////
// constants for systems
////////////////////////////

/**
 * API type WPS
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * API type WLE
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * API type ICM
 * 
 * @type {String}
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM = "SYSTEM_TYPE_ICM";

/**
 * API type WPS federated
 */
com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED = "federated";

/**
 * System type WPS tbd. if we need HTM/BFM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WPS = "SYSTEM_TYPE_WPS";
/**
 * System type WLE
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_WLE = "SYSTEM_TYPE_WLE";
/**
 * System type ICM
 */
com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM = "SYSTEM_TYPE_ICM";


/**
 * Domain type for WPS only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WPS = "WPS";

/**
 * Domain type for WLE only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_WLE = "WLE";

/**
 * Domain type for ICM only domains
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_ICM = "ICM";

/**
 * Domain type for mixed domains (any two of {WLE,WPS,ICM} or all three)
 */
com.ibm.bpm.social.widgets.task.util.Systems.DOMAIN_TYPE_MIXED = "MIXED";


/**
 * Define for the default domain name
 */
com.ibm.bpm.social.widgets.task.util.Systems.DEFAULT_DOMAIN_NAME = "default";


/**
 * Regular expression for a WLE TASK ID.
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
// integer number
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING = "^([0-9]+)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WLE_STRING);

/**
 * Regular expression for a WLE TEMPLATE ID.
 * 
 * If this changes, then update as well REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// UUID 8-4-4-4-12 (32 hexadecimal digit in 5 groups)
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING =
  "^([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12})$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PRECESS_TEMPLATE_ID_WLE =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_WLE_STRING);

/**
 * Regular expression for a WPS ID
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below and
 * REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING below
 */
// tkiid or tktid or piid or ptid
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_WPS_STRING);

/**
 * Regular expression for an ICM TKIID as received by Hong Dong Zhao (12. July 2011 12:12)
 * 
 * If this changes, then update as well REGEXP_ID_FEDERATED_STRING below
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING =
  "^([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*)$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_TKIID_ICM_STRING);


/**
 * Now specify the merged patterns, combined with OR (either pattern will be accepted)
 * 
 * We could have done as well the 3 two-out-of-three combinations, to e.g. reject an ICM id when no ICM is
 * involved. But the newly added ICM pattern is quite restrictive. And this pattern is only used for a first
 * syntactical check - semantical checks will follow anyway. So the damage by letting 3 types pass when only
 * two types of systems are involved is rather small.
 */
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+)|([0-9A-Fa-f]{32}:[A-Za-z]+\\w*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*:[A-Za-z]+(\\s*\\w)*))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_ID_FEDERATED_STRING);

com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING =
  "^((_[A-Za-z]+:[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8}\.[0-9a-fA-F]{1,8})|([0-9]+\.[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}))$";
com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED =
  new RegExp(com.ibm.bpm.social.widgets.task.util.Systems.REGEXP_PROCESS_TEMPLATE_ID_FEDERATED_STRING);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ErrorHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.task.util.ErrorHandler"] = true;
// ErrorHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT


// **************************************
//
// Version: 1.34 11/10/27 02:28:39
//
// **************************************
dojo.provide("com.ibm.bpm.social.widgets.task.util.ErrorHandler");








dojo
  .declare(
    "com.ibm.bpm.social.widgets.task.util.ErrorHandler",
    null,
    {

      /*
       * Error Handling

      For errors recognized during processing of a REST request, an appropriate HTTP status code is returned to the calling client (see the individual operations for details). The following HTTP status codes are returned by IBM Business Process Manager REST methods:

        * Successful completion of the request:
              o "200 OK" - successful completion
              o "201 Created" - successful completion, new resource created
              o "204 No Content" - successful completion, no content available
        * Expected error situations - additional error information is provided depending on the error type:
              o "400 Bad Request" - parameters are not valid or they are missing
              o "401 Unauthorized" - caller is not authorized for this request
              o "403 Forbidden" - caller is not allowed to complete this request
              o "404 Not Found" - resource does not exist
              o "406 Not Acceptable" - unsupported content type or content encoding requested
              o "409 Conflict" - conflict exists with the current state of the resource
              o "415 Unsupported Media Type" - unsupported content type or content encoding of the request
        * Unexpected error:
              o "500 Internal Server Error" - severe problem - programmer's details provided
              o "501 Not Implemented" - used for certain federated requests
              o "503 Service Unavailable" - federated requests could not be delivered to individual federation targets
              o "504 Gateway Timeout" - federated response has partial content due to missing individual responses
       */
      _genericStatusCodeMap: {
        "400": {
          key: "KEY_REST_BAD_REQUEST"
        },
        "401": {
          key: "KEY_REST_UNAUTHORIZED"
        },
        "403": {
          key: "KEY_REST_FORBIDDEN"
        },
        "404": {
          key: "KEY_REST_NOT_FOUND"
        },
        "406": {
          key: "KEY_REST_NOT_ACCEPTABLE"
        },
        "409": {
          key: "KEY_REST_CONFLICT"
        },
        "415": {
          key: "KEY_REST_UNSUPPORTED_MEDIA_TYPE"
        },
        "500": {
          key: "KEY_REST_INTERNAL_SERVER_ERROR"
        },
        "501": {
          key: "KEY_REST_NOT_IMPLEMENTED"
        },
        "503": {
          key: "KEY_REST_SERVICE_UNAVAILABLE"
        },
        "504": {
          key: "KEY_REST_GATEWAY_TIMEOUT"
        }
      },

      _errorCodeMapBC: {
        "CWTKB0001E": {
          key: "KEY_BC_NOT_FOUND"
        },
        "CWTKB0002E": {
          key: "KEY_BC_AUTHORIZATION_READ"
        },
        "CWTKB0003E": {
          key: "KEY_BC_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_BC_PARAMETER_TO_LONG_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKB0005E": {
          key: "KEY_BC_DELETE_WITH_TASKS"
        },
        "CWTKA0013E": {
          key: "KEY_BC_PARAMETER_NOT_SET_BUSINESSCATEGORY"
        // parameter from original message
        },
        "CWTKA0151E": {
          key: "KEY_BC_NOT_ENABLED"
        }
      },

      _errorCodeMapWB: {
        "CWTKW0001E": {
          key: "KEY_WB_NOT_FOUND"
        },
        "CWTKW0002E": {
          key: "KEY_WB_AUTHORIZATION_READ"
        },
        "CWTKW0003E": {
          key: "KEY_WB_AUTHORIZATION_CREATE"
        },
        "CWTKA0005E": {
          key: "KEY_WB_PARAMETER_TO_LONG_WORKBASKET"
        },
        "CWTKW0006E": {
          key: "KEY_DELETE_WORKBASKET_CONTAINS_TASKS"
        },
        "CWTKW0007E": {
          key: "KEY_DELETE_WORKBASKET_HAS_DISTRIBUTION_SOURCES"
        },
        "CWTKA0013E": {
          key: "KEY_WB_PARAMETER_NOT_SET_WORKBASKET"
        },
        // "CWTKA0149E": {    // error handled in WBInfoWidget
        "CWTKA0152E": {
          key: "KEY_WB_NOT_ENABLED"
        }
      },

      constructor: function constructor_ErrorHandler(){
        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social.widgets.task.util", "ErrorMessage");
      },

      /**
       * Displays a generic error message with details.
       * 
       * @see #_genericStatusCodeMap for handled status codes
       * 
       * @param data
       * @param ioArgs {void} nothing
       * @param {function} callback to connect to close on error dialog
       */
      showGenericError: function showGenericError_ErrorHandler(data, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var clientMessage = this._nlsResources["KEY_InternalError"];
        if (ioArgs && ioArgs.xhr) {
          var statusCode = ioArgs.xhr.status;
          var mapEntry = this._genericStatusCodeMap[statusCode];
          if (mapEntry && mapEntry !== null) {
            var key = mapEntry.key;
            if (key !== null) {
              clientMessage = this._nlsResources[key];
            }
          }
        }
        this._displayError(clientMessage, this._getDetailsMessage(data, ioArgs), null, ioArgs, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message mapped using the errorCodeMap and the statusCodeMap.
       * 
       * The errorCodeMap has precedence over the statusCodeMap. If no parameters are passed with the error
       * code the parameters are taken out of the rest call (errorMessage.errorMessageParameters) and used for
       * the specified message in business space.
       * 
       * Otherwise a generic message will be shown.
       * 
       * @param data
       * @param ioArgs
       * @param statusCodeMap e.g. {404: {key: "KEY_MissingForm", parameters: {1: htmlURL}, hideDetails:
       *        true}}
       * @param errorCodeMap e.g. {CWTKB0001E: {key: "KEY_Task_API_BusinessCategoryDoesNotExist", parameters:
       *        {1: name}, hideDetails: true}}, <br>
       *        in addition to support older code simple mapping like {CWTKB0001E:
       *        "KEY_Task_API_BusinessCategoryDoesNotExist"} is also supported
       * 
       * @return {void} nothing
       * 
       */
      showError: function showError_ErrorHandler(data, ioArgs, statusCodeMap, errorCodeMap, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if ((ioArgs && ioArgs.xhr) || errorCodeMap) {
          var statusCode = ioArgs.xhr.status;
          var mappedMessageProperties =
            this._getMappedMessageProperties(statusCode, statusCodeMap, data.responseText, errorCodeMap);
          this.showErrorMessage(data, ioArgs, mappedMessageProperties.key, mappedMessageProperties.parameters,
            mappedMessageProperties.hideDetails, null, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Show the error message with the error details.
       * 
       * @param data
       * @param ioArgs
       * @param key the message key
       * @param parameters parameters for the message, if no parameters specified, the parameters are
       *        retrieved from the original message
       * @param hideDetails don't show details
       * @param message default message if no message is found for key or no details
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showErrorMessage: function showErrorMessage_ErrorHandler(data, ioArgs, key, parameters, hideDetails, message, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (key && key !== null) {
          var mappedMessage = this._getMappedMessage(key, parameters);
          var detailsMessage = this._getDetailsMessage(data, ioArgs, hideDetails);

          if (mappedMessage === null) {
            if (message) {
              mappedMessage = message;
            }
            else {
              // no message at all: fall back to HTTP error message
              if (ioArgs && ioArgs.xhr && ioArgs.xhr.status) {
                var mapEntry = this._genericStatusCodeMap[ioArgs.xhr.status];
                if (mapEntry && mapEntry !== null) {
                  var newKey = mapEntry.key;
                  if (newKey) {
                    mappedMessage = this._getMappedMessage(newKey, null);
                  }
                }
              }
            }
          }
          else if (detailsMessage === null) {
            detailsMessage = message;
          }

          this._displayError(mappedMessage, detailsMessage, null, ioArgs, callback);
        }
        else {
          this.showGenericError(data, ioArgs, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * @param statusCode
       * @param statusCodeMap
       * @param response data.responseText or responseObject.failedOperations[i] for Bulk APIs
       * @param errorCodeMap
       * @return {key: "key", parameters: {parameters...}, hideDetails: true|false, fromErrorCode: true|false}
       */
      _getMappedMessageProperties: function _getMappedMessageProperties_ErrorHandler(statusCode, statusCodeMap, response,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var key = null;
        var parameters = null;
        var hideDetails = false;
        var fromErrorCode = false;
        if (errorCodeMap && response && response !== null) {
          try {
            var responseObject = this.getResponseObject(response);
            if (responseObject.errorNumber && typeof responseObject.errorNumber == "string" &&
              errorCodeMap[responseObject.errorNumber] !== null) {
              var mappedEntry = errorCodeMap[responseObject.errorNumber];
              if (mappedEntry) {
                if (typeof mappedEntry == "string") {
                  // simple error code map
                  key = mappedEntry;
                }
                else {
                  key = mappedEntry.key;
                  hideDetails = mappedEntry.hideDetails;
                  parameters = mappedEntry.parameters;
                }
                if (!parameters || parameters === null) {
                  if (responseObject.errorMessageParameters) {
                    parameters = {};
                    for ( var i = 0; i < responseObject.errorMessageParameters.length; i++) {
                      parameters[(i + 1).toString()] = responseObject.errorMessageParameters[i];
                    }
                  }
                }
                fromErrorCode = true;
              }
            }
          }
          catch (err) {
            // continue with the state map
          }
        }
        if (!fromErrorCode && statusCode !== undefined && statusCode !== null) {
          var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
          if (mapEntry === null) {
            mapEntry = this._mapStatusCode(statusCode, this._genericStatusCodeMap);
          }
          if (mapEntry !== null) {
            key = mapEntry.key;
            parameters = mapEntry.parameters;
            if (key) {
              hideDetails = mapEntry.hideDetails;
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return {
          key: key,
          parameters: parameters,
          hideDetails: hideDetails,
          fromErrorCode: fromErrorCode
        };
      },

      _getMessageForStatusCode: function _getMessageForStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        var mapEntry = this._mapStatusCode(statusCode, statusCodeMap);
        if (mapEntry && mapEntry !== null) {
          var key = mapEntry.key;
          if (key !== null) {
            result = this._nlsResources[key];
          }
        }
        return result;
      },

      _mapStatusCode: function _mapStatusCode_ErrorHandler(statusCode, statusCodeMap){
        var result = null;
        if (statusCode !== undefined && statusCode !== null) {
          if (statusCodeMap && statusCodeMap !== null && statusCodeMap[statusCode]) {
            result = statusCodeMap[statusCode];
          }
        }
        return result;
      },

      _getMappedMessage: function _getMappedMessage_ErrorHandler(key, parameters){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "key=" + key);
        var mappedMessage = null;

        var i;
        if (key !== null && this._nlsResources[key]) {
          mappedMessage = this._nlsResources[key];
          if (mappedMessage !== null && parameters !== null && parameters !== undefined) {
            // if the parameter is an empty object, the parameters are false
            var parameterOk = false;
            // check if the parameters include valid values
            for (i in parameters) {
              var value = parameters[i];
              if (value === null) {
                parameterOk = false;
                break;
              } else {
                parameterOk = true;
              }
            }
            if (parameterOk) {
              com.ibm.bpm.social.util.common.Log.logEntry(2," parameters=" + parameters);
              mappedMessage = dojo.string.substitute(mappedMessage, parameters);
            }
          }
        }
        // In case parameter where unknown
        if (/UNKOWN/.test(mappedMessage)) {
          mappedMessage = mappedMessage.replace(/'UNKOWN'/g, "");
          mappedMessage = mappedMessage.replace(/UNKOWN/g, "");
        }
        com.ibm.bpm.social.util.common.Log.logExit(2, mappedMessage);
        return mappedMessage;
      },

      _getDetailsMessage: function _getDetailsMessage_ErrorHandler(data, ioArgs, hideDetails){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var detailsMessage = null;
        if (data && !data.responseText && data.message !== undefined && data.name !== undefined) {
          // this is a JavaScriptError instead of a back-end error
          detailsMessage = data.name + ":" + data.message + " (JavaScript Error)<br>" + "at " + data.stack;
        }
        else if (ioArgs && ioArgs.xhr) {
          if (ioArgs && !hideDetails) {
            var details = this.getResponseObject(ioArgs.xhr.responseText);
            if (details) {
              if (details.errorMessage) {
                detailsMessage = details.errorMessage;
              }
              else if (dojo.isString(details)) {
                detailsMessage = details;
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return detailsMessage;
      },

      _displayError: function _displayError_ErrorHandler(message, details, messageDescription, ioArgs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        // add federation result errors
        if (ioArgs && ioArgs.xhr && ioArgs.xhr.responseText) {
          var responseObject = this.getResponseObject(ioArgs.xhr.responseText);
          if (responseObject.federationResult) {
            var errors = this._getFederationResultErrors(responseObject.federationResult);
            if (errors !== null) {
              // TODO SA: should we show them using BspaceGeneralHelper.handleException?
              // TODO SA: add ioArgs.url?
              messageDescription = this._getMessageDetailsList(errors);
            }
          }
        }

        com.ibm.bpm.social.util.common.Log.log(2, "Message: " + message);
        com.ibm.bpm.social.util.common.Log.log(2, "Details: " + details);

        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(message, details, callback);

        com.ibm.bpm.social.util.common.Log.logExit(2);
      },

      /**
       * Shows the bulk API failure in an error dialog if only one item has been selected for bulk processing.
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperation the response for the failing operation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Array} federationResults optional, the complete federation result
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIError: function showBulkAPIError_ErrorHandler(failedOperation, statusCodeMap, errorCodeMap,
        federationResults, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        var response = null;
        var statusCode = null;
        var statusCodes = statusCodeMap;
        if (failedOperation) {
          response = failedOperation;
          statusCode = failedOperation.derivedHttpStatusCode;
        }
        else if (federationResults & federationResults.length > 0) {
          // in federation the error may be returned as federation result, if the backend has a problem
          response = federationResults[0];
          statusCode = response.statusCode;
          // use generic mapping for status codes because federation result is related to errors with backend systems
          statusCodes = this._genericStatusCodeMap;
        }
        var mappedMessageProperties = this._getMappedMessageProperties(statusCode, statusCodes, response, errorCodeMap);
        this.showErrorMessage(null, null, mappedMessageProperties.key, mappedMessageProperties.parameters,
          mappedMessageProperties.hideDetails, response.errorMessage, callback);
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the bulk API failures in an error dialog for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param {Array} failedOperations the failedOperations as returned in the responseObject of the bulk
       *        API
       * @param {String} messageKey the general message key for the error
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @param {Object} idToNameMap maps ids to names for replacement of ids in error messages
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {String} nameKey the message key for the entity name
       * @param {Array} federationResults optional, the complete federation result
       * @param {Object} failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {function} callback to connect to close on error dialog
       * @return {void} nothing
       */
      showBulkAPIErrors: function showBulkAPIErrors_ErrorHandler(failedOperations, messageKey, statusCodeMap, errorCodeMap,
        idToNameMap, idKey, nameKey, federationResults, failedNamesForSystemIDs, callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1);

        var errors = new Array();

        // get federation errors
        if (failedNamesForSystemIDs && federationResults && federationResults.length > 0) {
          errors = errors.concat(this._getFederationResultErrors(federationResults, failedNamesForSystemIDs));
        }

        // get errors for failed operations
        errors = errors.concat(this._getBulkAPIErrors(failedOperations, idToNameMap, idKey, statusCodeMap, errorCodeMap));

        if (errors !== null && errors.length > 0) {
          // show list of errors
          this._displayError(this._nlsResources[messageKey], null, null, this._getMessageDetailsList(errors, nameKey,
            failedNamesForSystemIDs), null, callback);
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * Shows the errors in the federation result as a popup
       * 
       * @param {Array} federationResults the complete federation result
       * @param {String} messageKey the message key for the error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @param {function} callback to connect to close on error dialog
       * @returns {boolean} whether there were errors shown
       */
      showFederationResultErrors: function showFederationResultErrors_ErrorHandler(federationResults, messageKey, ignore401,
        callback){
        com.ibm.bpm.social.util.common.Log.logEntry(1, "messageKey=" + messageKey);
        var errorsShown = false;
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults, null, ignore401);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            var message = this._nlsResources[messageKey];
            var messageDescription = this._getMessageDetailsList(errors);
            this._displayError(message, null, messageDescription, null, callback);
            errorsShown = true;
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
        return errorsShown;
      },

      /**
       * Shows the errors in the federation result as global exceptions in BSpace
       * 
       * uses BSpace exception handler to show error on top of space
       * 
       * @param federationResults the complete federation result
       * @param iWidget
       * @param method
       * @returns {void} nothing
       */
      handleFederationResultErrors: function handleFederationResultErrors_ErrorHandler(federationResults, iWidget, method,
        systemsSending401Error){
        com.ibm.bpm.social.util.common.Log.logEntry(1);
        if (federationResults && federationResults.length > 0) {
          var errors = this._getFederationResultErrors(federationResults);
          if (errors !== null && errors.length > 0) {
            // errors in federation result
            for ( var i = 0; i < errors.length; i++) {
              var error = errors[i];
              var message = "";

              // message for status code
              if (error.statusCode !== undefined) {
                var statusMessage = this._getMessageForStatusCode(error.statusCode, this._genericStatusCodeMap);
                if (statusMessage) {
                  message += statusMessage;
                }
                else {
                  message += dojo.string.substitute(this._nlsResources.Key_HTTPStatusCode, [error.statusCode]);
                }
              }

              // system ID
              if (error.systemID) {
                message += " " + dojo.string.substitute(this._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
              }

              // error number, and not part of error message
              if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
                // WPS error message contains error number, WLE message not
                message += " " + error.number + ":";
              }

              // error message
              if (error.message) {
                message += " " + error.message;
              }

              com.ibm.bpm.social.util.common.Log.logError(1, "com.ibm.bpm.social.widgets.task.util.ErrorHandler" + "." +
                method, new Error(message));

              // Do not show "CWTBF0005E: The 'queryFilter' parameter is invalid.",
              // issued if a filter attribute is not supported by one of the backends

              // Do not show "CWTBF0020E: Request not propagated by the federation layer because a previous systems metadata request indicated that the system does not support '...' resources.
              // issued if a target system does not support that REST resource (e. g. WLE does not support escalations)

              // Do not show "CWTBF0021E: Request not propagated by the federation layer because a previous query metadata request indicated that the system does not support this resource.
              // issued if a target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0002E: The following query name is unknown: '...'.
              // issued if a WLE target system does not know a query, can happen because we use all=true

              // Do not show "CWWBQ0001E: The following query table name is unknown: '...'.
              // issued if a WPS target system does not know a query, can happen because we use all=true

              // Do not show "CWTBG0530E": The following process application name is unknown: '...'
              // issued if a WLE target system does not know a process application, can happen because we use all=true

              // Do not show "CWTKA0162E": The following process application name is unknown: '...'
              // issued if a WPS target system does not know a process application, can happen because we use all=true

              var handleException =
                !error.number ||
                  (error.number != "CWTBF0005E" && error.number != "CWTBF0020E" && error.number != "CWTBF0021E" &&
                    error.number != "CWTBG0002E" && error.number != "CWTBG0530E" && error.number != "CWWBQ0001E" && error.number != "CWTKA0162E");

              // for the systems that had returned a 401 in the initial systems call, ignore 401 errors now because that was already covered
              // But do not generally exclude all 401 errors - could be a different reason for those systems that had not returned a 401 ealier
              if (handleException && systemsSending401Error && error.statusCode == 401 &&
                (dojo.indexOf(systemsSending401Error, error.systemID) != -1)) {
                // overwrite the handleException flag
                handleException = false;
              }

              // Special handling for ICM 404 errors:
              // For version 7.5.1.0, the ICM rest api could not provide details for the 404 error cases.
              // However, the only relevant case that produce 404 errors from ICM systems in federated mode is
              // that a processAppName was specified that does not exist on the ICM system.
              // We must gracefully handle that case and ignore those errors, botp 719802
              // Currently, it is ok to ignore ALL 404 errors from ICM that enter this method
              // (only entered in federation mode); 404 in the /systems call are handles separately.  
              // NEEDSWORK: As soon as the ICM rest api provides details, then rather do a positive list
              // of error ICM details that should be ignored. (Not NLS specific, please).
              if (handleException && error.statusCode == 404 && error.systemID) {
                if (iWidget && iWidget._systems && iWidget._systems.getSystemType) {
                  if (iWidget._systems.getSystemType(error.systemID) === com.ibm.bpm.social.widgets.task.util.Systems.SYSTEM_TYPE_ICM) {
                    // overwrite the handleException flag
                    handleException = false;
                  }
                }
              }

              if (handleException && iWidget) {
                iWidget.handleException("com.ibm.bpm.social.widgets.task.util.ErrorHandler", method, message, null);
              }
            }
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(1);
      },

      /**
       * 
       * @param errors [{name: "name or array of names", systemID: "systemID", statusCode: "statusCode",
       *        number: "Number", message: "Message", details: "Details"}]
       * @param nameKey optional, only for bulk API errors, NLS key of text for error.name, e.g. "Task"
       * @returns {Object} a div containing the message details list
       */
      _getMessageDetailsList: function _getMessageDetailsList_ErrorHandler(errors, nameKey){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var messageDetails = null;

        if (errors !== null) {

          //***********************************
          // message details list
          // systemID, name and reason/details for each failing entry
          // TODO add hostName for system???

          var messageDetailsListDiv = document.createElement("div");
          var detailsDiv = document.createElement("div");
          dojo.addClass(detailsDiv, "errorList");
          messageDetailsListDiv.appendChild(detailsDiv);

          var myself = this;
          dojo.forEach(errors, function(error){

            var ulItem = document.createElement("ul");
            detailsDiv.appendChild(ulItem);

            //
            // item name, may constist of one or more names and optional a systemID
            //
            var itemName = "";
            // name
            if (error.name) {
              if (!dojo.isArray(error.name)) {
                itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name]);
              }
              else {
                for ( var i = 0; i < error.name.length; i++) {
                  if (itemName.length > 0) {
                    itemName += "<br/>";
                  }
                  itemName += dojo.string.substitute(myself._nlsResources[nameKey], [error.name[i]]);
                }
              }
            }

            // system ID
            if (error.systemID) {
              if (itemName.length > 0) {
                itemName += "<br/>";
              }
              itemName += dojo.string.substitute(myself._nlsResources["KEY_ErrorSystemID"], [error.systemID]);
            }

            if (itemName.length > 0) {
              var liItem = document.createElement("li");
              dojo.addClass(liItem, "errorListItem");
              liItem.innerHTML = itemName;
              ulItem.appendChild(liItem);
            }

            //
            // message, may consist of several parts
            //
            var ulMessage = document.createElement("ul");
            dojo.addClass(ulMessage, "errorListMessage");
            ulItem.appendChild(ulMessage);

            // message for status code
            if (error.statusCode !== undefined) {
              var statusMessage = myself._getMessageForStatusCode(error.statusCode, myself._genericStatusCodeMap);
              if (statusMessage && statusMessage !== null) {
                var liStatus = document.createElement("li");
                liStatus.innerHTML = statusMessage;
                ulMessage.appendChild(liStatus);
              }
            }

            // error number, and not part of error message
            if (error.number && (!error.message || error.message.indexOf(error.number) === -1)) {
              var liNumber = document.createElement("li");
              liNumber.innerHTML = error.number;
              ulMessage.appendChild(liNumber);
            }

            // error message
            if (error.message) {
              var liMessage = document.createElement("li");
              liMessage.innerHTML = error.message;
              ulMessage.appendChild(liMessage);
            }

            //
            // message details
            //
            if (error.details) {
              var ulMessageDetails = document.createElement("ul");
              dojo.addClass(ulMessageDetails, "errorListMessageDetails");
              ulMessage.appendChild(ulMessageDetails);

              var liDetails = document.createElement("li");
              liDetails.innerHTML = error.details;
              ulMessageDetails.appendChild(liDetails);
            }
          });

          messageDetails = messageDetailsListDiv.innerHTML;

        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return messageDetails;
      },

      /**
       * Retrieves the bulk API failures for each failing entry
       * 
       * Regarding statusCodeMap/errorCodeMap see #showError
       * 
       * @param failedOperations the failedOperations as returned in the responseObject of the bulk API
       * @param idToNameMap maps ids to (display) names
       * @param {String} idKey the attribute name for the id in failedOperation
       * @param {Object} statusCodeMap maps status codes to messages for failedOperations, e.g. 401, ...
       * @param {Object} errorCodeMap maps error codes to messages for failedOperations, eg. CWTKA0072E, ...
       * @returns {Array} an array of objects containing error information <br>
       *          [{name: "name", systemID: "systemID", message: "Message", details: "Details"}]
       */
      _getBulkAPIErrors: function _getBulkAPIErrors_ErrorHandler(failedOperations, idToNameMap, idKey, statusCodeMap,
        errorCodeMap){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var errors = new Array();

        if (failedOperations && failedOperations.length > 0) {
          var self = this;

          dojo.forEach(failedOperations, function(failedOperation){
            var error = {};

            if (failedOperation.systemID) {
              error.systemID = failedOperation.systemID;
            }
            var name = idToNameMap[failedOperation[idKey]];
            if (name !== null) {
              error.name = name;
            }

            var mappedMessageProperties =
              self._getMappedMessageProperties(failedOperation.derivedHttpStatusCode, statusCodeMap, failedOperation,
                errorCodeMap);
            var mappedMessage = self._getMappedMessage(mappedMessageProperties.key, mappedMessageProperties.parameters);
            var errorMessage = null;
            if (mappedMessage === null) {
              mappedMessage = failedOperation.errorMessage;
            }
            else if (!mappedMessageProperties.fromErrorCode && !mappedMessageProperties.hideDetails) {
              // if not mapped from error code add detailed error message
              errorMessage = failedOperation.errorMessage;
            }
            error.message = mappedMessage;

            if (errorMessage && errorMessage !== null) {
              error.details = errorMessage;
            }
            errors.push(error);
          });

        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      _getFederationResultErrors: function _getFederationResultErrors_ErrorHandler(federationResults,
        failedNamesForSystemIDs, ignore401){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errors = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errors = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (ignore401 && ignore401 === true && federationResult.statusCode === "401") {
                com.ibm.bpm.social.util.common.Log.log(2,
                  "HTTP error 401 skipped, will not be displayed as an error message");
              }
              else {
                // error returned from a backend system
                var error = {};
                error.statusCode = federationResult.statusCode;
                error.systemID = federationResult.systemID;
                error.number = federationResult.errorNumber;
                error.message = federationResult.errorMessage;
                if (!error.message) {
                  // TODO WLE: for 404 we get only statusCode and body - is this a temporary bug?
                  if (federationResult.body) {
                    error.message = federationResult.body;
                  }
                }
                if (failedNamesForSystemIDs && failedNamesForSystemIDs[error.systemID]) {
                  error.name = failedNamesForSystemIDs[error.systemID];
                }
                errors.push(error);
              }
            }
          });
          if (errors.length === 0) {
            // all responses have been ok
            errors = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errors, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errors;
      },

      /**
       * @param federationResults the complete federation result
       * @param failedNamesForSystemIDs optional, names for federated systems which returned an error
       * @param {boolean} ignore401 gracefuly skip 401 errors if true
       * @returns {Array} null if federationResults contains only statusCode 200 <br>
       *          or an array of objects containing error information <br>
       *          [{name: "array of names", systemID: "systemID", statusCode: "statusCode", number: "Number",
       *          message: "Message"}]
       */
      getSystemsSending401Error: function getSystemsSending401Error_ErrorHandler(federationResults){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        var errorSystems = null;
        if (federationResults && federationResults.length > 0) {
          var self = this;
          errorSystems = new Array();

          dojo.forEach(federationResults, function(federationResult){
            if (federationResult.statusCode !== undefined && federationResult.statusCode !== "200") {
              // graceful handling: ignore 401 errors when already covered
              if (federationResult.statusCode === "401") {
                errorSystems.push(federationResult.systemID);
              }
            }
          });
          if (errorSystems.length === 0) {
            // all responses have been ok or other than 401
            errorSystems = null;
          }
        }
        com.ibm.bpm.social.util.common.Log.logObject(3, errorSystems, "errors");
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return errorSystems;
      },

      getResponseObject: function getResponseObject(response){
        var responseObject = response;
        if (dojo.isString(response)) {
          try {
            responseObject = dojo.fromJson(response);
            // There may be an extra layer ".Data"; if this does exist, use it
            // but do it defensive after some extra checks
            if (responseObject.Data && responseObject.Data.errorNumber && !responseObject.errorNumber) {
              // all checks passed, skip the extra layer
              responseObject = responseObject.Data;
            }
          }
          catch (err) {
            // keep it just as string
          }
        }
        return responseObject;
      },

      getErrorCodeMapBC: function getErrorCodeMapBC_ErrorHandler(){
        return this._errorCodeMapBC;
      },

      getErrorCodeMapWB: function getErrorCodeMapWB_ErrorHandler(){
        return this._errorCodeMapWB;
      },

      showErrorDialog: function showErrorDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(mappedMessage, null /* details */, callback);
      },

      showWarningDialog: function showWarningDialog_ErrorHandler(key, parameters, callback){
        var mappedMessage = this._getMappedMessage(key, parameters);
        com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(mappedMessage, null /* details */, callback);
      },

      destroy: function(){
        // TODO temp error dialog
        if (this.popup && this.popup.destroy) {
          this.popup.destroy();
          this.popup = null;
        }
        this.inherited(arguments);
      },

      statics: {
        _instance: null,
        _getInstance: function(){
          if (this._instance === null) {
            this._instance = new com.ibm.bpm.social.widgets.task.util.ErrorHandler();
          }
          return this._instance;
        }
      }
    });

com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance = function(){
  return com.ibm.bpm.social.widgets.task.util.ErrorHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.RestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.RestHandler"] = true;
// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.RestHandler");










dojo.declare("com.ibm.bpm.social.util.common.RestHandler", null, {
  prefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the bfm calls
  bfmPrefix: window.endpointURLObject.bpmRestJS +"/",

  //root url for the vmm calls
  vmmPrefix: "",

  // root for form generator service
  generatorServicePrefix: "",

  _errorHandler: null,

  error: null,

  version: "v1",

  threshold: 100,

  // 30 seconds timeout to account for some slowness and also to be consistent with Business Space/Mashups settings
  defaultTimeout: 30000,

  _apiType: null,

  _iWidget: null,

  FEDERATION_MODE: "federationMode=true",

  _systems_refresh_required: false,

  constructor: function constructor_RestHandler(params){
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
    // IV09357:
    // Check whether a timoeout has been set using the mashup maker configuration:
    //
    // In order to overwrite the timeout for the Human Workflow Widget, open the Administration Console.
    // Select "Mashups_ConfigService" under Resources -> Resource Environment -> Resource Environment Providers.
    // Then select "Custom properties". Create a new property "com.ibm.bpm.social.widgets.task.requestTimeout" and set its value to the wanted timeout in milliseconds. 
    // In addition to that select the com.ibm.mashups.properties.client and add ",com.ibm.bpm.social.widgets.task.requestTimeout" to 
    // the value of this property (It is a comma separated list of values therefore do not forget the comma).
    //
// TODO disabled for now
//    if (typeof ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"] !== "undefined") {
//      var timeout = ibmConfig["com.ibm.bpm.social.widgets.task.requestTimeout"];
//      if (typeof timeout === "string") {
//        timeout = parseInt(timeout);
//      }
//      com.ibm.bpm.social.util.common.Log.log(2, "The default timeout for the Human Workflow Widgets is set to " + timeout +
//        " ms");
//      this.defaultTimeout = timeout;
//    }
  },

  getTimeout: function(){
    return this.defaultTimeout;
  },

  /**
   * required for handling of federation results
   */
  setIWidget: function setIWidget_RestHandler(iWidget){
    this._iWidget = iWidget;
  },

  getVmmPrefix: function getVmmPrefix_RestHandler(){
    return this.vmmPrefix;
  },

  /**
   * Returns the apiType of the REST service endpoint.
   * 
   * @returns {String} the apiType of the REST service endpoint
   *          <ul>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE</li>
   *          <li>com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED</li>
   *          <li>TODO temporarily "old" if no system</li>
   *          </ul>
   */
  getApiType: function getApiType_RestHandler(){
    if (this._apiType === null) {
      var systems = this.getSystems();
      if (systems) {
        this._apiType = systems.getApiType();
      }
    }
    return this._apiType;
  },

  /**
   * Returns if the REST service endpoint is a WPS system
   * 
   * @returns {boolean} true if endpoint is WPS
   */
  isApiTypeWps: function isApiTypeWps_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WPS;
  },

  /**
   * Returns if the REST service endpoint is a WLE system
   * 
   * @returns {boolean} true if endpoint is WLE
   */
  isApiTypeWle: function isApiTypeWle_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_WLE;
  },

  /**
   * Returns if the REST service endpoint is a ICM system
   * 
   * @returns {boolean} true if endpoint is ICM
   */
  isApiTypeIcm: function isApiTypeIcm_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_ICM;
  },

  /**
   * Returns if the REST service endpoint is a federated system
   * 
   * @returns {boolean} true if endpoint is Federated
   */
  isApiTypeFederated: function isApiTypeFederated_RestHandler(){
    return this.getApiType() === com.ibm.bpm.social.widgets.task.util.Systems.API_TYPE_FEDERATED;
  },

  /**
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  performRequest: function performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var deferred = null;

    if (request) {

      if (request.load && this.isApiTypeFederated() && !isBulk) {
        // show federation result errors as exception in BSpace message area
        // for bulk request the error is shown in the error popup
        var load = request.load;
        this._apiCaller = null;
        if (arguments && arguments.callee && arguments.callee.caller && arguments.callee.caller.name) {
          this._apiCaller = arguments.callee.caller.name;
        }
        var systems = this.getSystems();
        var _systemsSending401Error = null;
        if (systems && systems._systemsSending401Error) {
          _systemsSending401Error = systems._systemsSending401Error;
        }
        var myself = this;
        request.load =
          function(data, ioArgs){
            if (data && data.federationResult && data.federationResult.length > 0) {
              if (!myself._iWidget) {
                // federation result cannot be handled with iWidget.handleException!
                com.ibm.bpm.social.util.common.Log.logError(1,
                  "com.ibm.bpm.social.util.common.RestHandler.performRequest",
                  new Error("Property '_iWidget' is required to handle federation results!!!"));
              }
              myself._errorHandler.handleFederationResultErrors(data.federationResult, myself._iWidget, myself._apiCaller,
                _systemsSending401Error);
            }
            // continue with result processing
            load(data, ioArgs);
          };
      }
    }
    deferred = this._performRequest(kind, request, isURLComplete, isBulk);
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return deferred;
  },

  /**
   * Note: This function should not call any function which requires system information, because it is also
   * used by the getSystems call. Put code requiring system information in performRequest above.
   * 
   * @param kind
   * @param request
   * @param isURLComplete
   * @param isBulk
   * @returns {dojo.Deferred}
   */
  _performRequest: function _performRequest_RestHandler(kind, request, isURLComplete, isBulk){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var deferred = null;

    request.headers = request.headers || {};

    if (!isURLComplete) {
      request.url = this.getRequestUrl(request.url);
    }
    // make sure the handleAs parameter is set. 
    // Also it is important to set the right request header for json, otherwise 
    // JAXRS will set the header to XML which is not correct for json.
    if (!request.handleAs) {
      request.handleAs = "json";
      request.headers.Accept = "application/json";
    }
    else if (request.handleAs == "json") {
      request.headers.Accept = "application/json";
    }

    if (!request.timeout) {
      request.timeout = this.defaultTimeout;
    }
    if (kind == "get" && (request.preventCache === undefined || request.preventCache === null)) {
      request.preventCache = true;
    }

    if (this.FEDERATION_MODE) {
    if (request.url.indexOf("?") > -1) {
      request.url += "&" + this.FEDERATION_MODE;
    }
    else {
      request.url += "?" + this.FEDERATION_MODE;
      }
    }

	//Set up a listener for 401 responses -- set default action to redirect to login page
	//If an error for some other reason, allow specified handler to take over
	var _error = request.error;
	var myself = this;
	request.error = function(data, ioArgs) {
		var req = ioArgs.xhr;
		if(typeof req != "undefined") {
			var reqStatus = req.status;
			if(typeof reqStatus != undefined) {
				if(reqStatus == 401) {
					//double check to make sure its an actual authentication timeout-
					//check for the authenticate header
					var respHeaders = req.getAllResponseHeaders();
					if(respHeaders.indexOf("WWW-Authenticate") >= 0) {
						//Perform REDIRECT
						com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
					}
				}
			}
		}
		if(_error) {
			_error(data, ioArgs);
		} else {
			//Generic error handler if none is specified in original request
			myself._errorHandler.showGenericError(data, ioArgs);
		}
	};
	
    // if (!request.error) {
      // // default error handler
      // var myself = this;
      // request.error = function(data, ioArgs){
        // myself._errorHandler.showGenericError(data, ioArgs);
      // };
    // }

    if (kind == "get" && request.url.length > com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH) {
      // do a POST request instead of a GET
      var iParams = request.url.indexOf("?") + 1;
      if (iParams > 0) {
        com.ibm.bpm.social.util.common.Log.log(3, "Do a POST instead of a get because url is > " +
          com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH);
        com.ibm.bpm.social.util.common.Log.log(3, "url: " + request.url);
        kind = "post";
        request.contentType = "application/x-www-form-urlencoded; charset=UTF-8";
        request.headers["X-Method-Override"] = "GET";
        request.postData = request.url.substring(iParams);
        request.url = request.url.substring(0, iParams - 1);
      }
    }

    // overwrite the browser locale with the users profile language
    var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var locale = (userProfile && userProfile.userPreferences) ? userProfile.userPreferences['Locale'] : null;

    if (locale) {
      request.headers["Accept-Language"] = locale;
    }

    // Defect 116603 - SiteMinder may redirect the request to an HTML
    // login form when the current user authentication times out.  We
    // detect this case by checking for exceptions during json parsing.
    // If the response cannot be parsed as json we check the response
    // content type to see if "text/html" content was returned.  If so,
    // we assume it is a login form and redirect to our normal logout
    // processing.
    if (!dojo.contentHandlers['jsonOriginal']) {
		dojo.contentHandlers['jsonOriginal'] = dojo.contentHandlers['json'];
		dojo.contentHandlers['json'] = function(xhr) {
			var ret = null;
			try {
				ret = dojo.contentHandlers['jsonOriginal'](xhr);
			} catch (e) {
				contentType = xhr.getResponseHeader("Content-Type");
				if  (contentType && (contentType.indexOf('text/html') != -1)) {
					com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
				} else {
					throw e;
				}
			}
			return ret;
		}
	}
	
    if (kind == "get") {
      // GET request do not need a contentType because they do not have an entity body.
      deferred = dojo.xhrGet(request);
    }
    else if (kind == "rawPost") {
      deferred = dojo.rawXhrPost(request);
    }
    else if (kind == "post") {
      deferred = dojo.xhrPost(request);
    }
    else if (kind == "rawPut") {
      ///////////////
      deferred = dojo.rawXhrPut(request);
    }
    else if (kind == "put") {
      ///////////////
      deferred = dojo.xhrPut(request);
    }
    else if (kind == "delete") {
      deferred = dojo.xhrDelete(request);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2);

    return deferred;
  },

  /**
   * @param {String} url the url without prefix
   * @return {String} the complete url for the request
   */
  getRequestUrl: function getRequestUrl_RestHandler(url){
    return this.prefix + this.version + url;
  },

  setUrl: function setUrl_RestHandler(/*String*/urlName, /*String*/url){
    if (urlName) {
      this[urlName] = url;
      if ((url !== "") && url.slice(-1) != "/") {
        this[urlName] += "/";
      }
    }
    com.ibm.bpm.social.util.common.Log.log(3, "urlName=" + this[urlName]);
  },

  /**
   * Prefixes the ID with the systemID, if a domain and a systemID is specified.
   * 
   * @param {String} id
   * @param {String} domain optional, needed for a federated system
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the id, prefixed with the system id, if domain and a systemID are specified
   * @throws error if domain, but no system ID is specified for a federated backend
   */
  prefixID: function prefixID_RestHandler(id, domain, systemID){
    var result = id;
    if (domain) {
      if (systemID) {
        result = systemID + '$' + id;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return result;
  },


  /**
   * Federation only - Adds the systemID to the url
   * 
   * @param {String} url
   * @param {String} systemID optional, needed for a federated system
   * @returns {String} the url including the specified systemID
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemID: function addSystemID_RestHandler(url, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        if (url.indexOf("?") > 0) {
          url += "&";
        }
        else {
          url += "?";
        }
        url += "systemID=" + systemID;
      }
      else {
        this.errorNoSystemID();
      }
    }
    return url;
  },

  /**
   * Adds a processAppName to the url (future use - not used yet)
   * 
   * @param {String} url
   * @param {String} processAppName optional, needed for an ICM system
   * @returns {String} the url including the specified processAppName
   * @returns the identical url if processAppName is null or undefined
   */
  _addProcessAppName: function _addProcessAppName_RestHandler(url, processAppName){
    if (processAppName) {
      if (url.indexOf("?") > 0) {
        url += "&";
      }
      else {
        url += "?";
      }
      url += "processAppName=" + processAppName;
    }
    return url;
  },

  /**
   * Federation only - Adds the properties systemID and systemType to the retrieved item
   * 
   * @param {Object} item a task, task template, process, process template, activity object
   * @param {String} systemID
   * @returns {Object} the updated item
   * @throws error if no system ID is specified for a federated backend
   */
  addSystemProperties: function addSystemProperties_RestHandler(item, systemID){
    if (this.isApiTypeFederated()) {
      if (systemID) {
        item.systemID = systemID;
        item.systemType = this.getSystems().getSystemType(item.systemID);
      }
      else {
        this.errorNoSystemID();
      }
    }
    return item;
  },

  /**
   * @returns {Object} a {@link com.ibm.bpm.social.widgets.task.util.Systems} object
   * @throws error if no configuration information retrieved
   */
  getSystems: function getSystems_RestHandler(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    var url = this.getRequestUrl("/systems");

    var systems = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(url);
    if (systems === null) {
      var myself = this;
      var error = function(data, ioArgs){
        if (ioArgs.xhr.status == 404) {
          //myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorEndpointConfiguration");
          com.ibm.bpm.social.util.common.Util.restConfigurationException(data, ioArgs);
        }
        else {
          myself._errorHandler.showErrorMessage(data, ioArgs, "KEY_ErrorSystemsMetadata");
        }
      };
      var errors = false;
      var i;
      var load = function(data, ioArgs){
        com.ibm.bpm.social.util.common.Log.logObject(3, data, "data");
        if (data && data.federationResult && data.federationResult.length > 0) {
          // graceful handling of 401 in the systems call: ignore but memorize
          var _systemsSending401Error = myself._errorHandler.getSystemsSending401Error(data.federationResult);
          // ... and then make sure the 401 errors do not raise an error message:
          errors = myself._errorHandler.showFederationResultErrors(data.federationResult, "KEY_ErrorSystemsMetadata", true);
        }

        systems = new com.ibm.bpm.social.widgets.task.util.Systems({
          systems: data,
          systemsSending401Error: _systemsSending401Error
        });
      };

      this._performRequest("get", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json",
        // allow browser cache to prevent double calls during initialization
        preventCache: this._systems_refresh_required
      }, true);

      if (systems !== null) {
        com.ibm.bpm.social.util.common.Log.log(1, "systems=" + systems);
        // cache the data
        var cache = 3600000; // 1 hour
        if (errors) {
          // we had errors so cache only for 5 minutes
          cache = 300000;
          this._systems_refresh_required = false;
        }
        this._systems_refresh_required = true;
        com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(url, systems, cache);
      }
      else {
        var exc = new Error("RestHandler.getSystems: No configuration information retrieved!");
        com.ibm.bpm.social.util.common.Log.consoleError(exc.message);
        throw exc;
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return systems;
  },

  errorNoDomain: function errorNoDomain_RestHandler(){
    var error = new Error("No domain name specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoSystemID: function errorNoSystemID_RestHandler(){
    var error = new Error("No system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  errorNoDomainOrSystemID: function errorNoDomainOrSystemID_RestHandler(){
    var error = new Error("No domain name or system ID specified, this is required for a federated backend!");
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    throw error;
  },

  _errorNotSupportedForFederation: function _errorNotSupportedForFederation_RestHandler(url){
    var error = new Error("This call is not supported for federation: " + url);
    com.ibm.bpm.social.util.common.Log.consoleError(error.message);
    //throw error;
  }
});


/**
 * Maximum length for a GET request, if exceeded POST will be used
 * <p>
 * some browser have a length limit for the url, e.g. for IE this is 2083
 */
com.ibm.bpm.social.util.common.RestHandler.HTTP_GET_URL_MAX_LENGTH = 2000;

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SocialRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SocialRestHandler"] = true;
// SocialRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT
dojo.provide("com.ibm.bpm.social.util.common.SocialRestHandler");






dojo.declare("com.ibm.bpm.social.util.common.SocialRestHandler", com.ibm.bpm.social.util.common.RestHandler, {
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getCurrentUserAvatar: function getCurrentUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteUserAvatar: function deleteUserAvatar_SocialRestHandler(load, error, async, systemID) {
		var url = "/avatar/current";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	getMentions: function getMentions_SocialRestHandler(pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instances/mentions?page="+pagenum+"&pagesize="+pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param tkid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteMentions: function deleteMentions_SocialRestHandler(tkid, load, error, async, systemID) {
		var url = "/social/task/"+tkid+"/mentions";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 *
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 */
	deleteAllMentions: function deleteAllMentions_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/task/mentions/all";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getFollowedInstances: function getFollowedInstances_SocialRestHandler(load, error, async, systemID) {
		var url = "/social/instances/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	isFollowingInstance: function isFollowingInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	followInstance: function followInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	unfollowInstance: function unfollowInstance_SocialRestHandler(piid, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/following";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("delete", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertsForTask: function getExpertsForTask_SocialRestHandler(tkiid, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/experts";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param user
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	getExpertUserData: function getExpertUserData_SocialRestHandler(tkiid, user, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/userdata?username=" + encodeURIComponent(user);
		
		var url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param tkiid
	 * @param comment
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */	
	postTaskStreamComment: function postTaskStreamComment_SocialRestHandler(tkiid, comment, load, error, async, systemID) {
		var url = "/social/task/" + tkiid + "/comment";
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("post", {
			url: url,
			load: load,
			error: error,
			postData: comment,
			contentType: "application/json",  //Need to override the default set for posts here.
			sync: !async
		});	
	},
	
	/**
	 * 
	 * @param piid
	 * @param since
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	updateStreamEvents: function updateStreamEvents_SocialRestHandler(piid, since, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		
		if(since) {
			url += "since=" + since;
		}
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	/**
	 * 
	 * @param piid
	 * @param pagenum
	 * @param pagesize
	 * @param load
	 * @param error
	 * @param async
	 * @param systemID
	 * @returns {void} nothing
	 */
	getStreamEvents: function getStreamEvents_SocialRestHandler(piid, pagenum, pagesize, load, error, async, systemID) {
		var url = "/social/instance/" + piid + "/stream?";
		url += "page=" + pagenum + "&pagesize=" + pagesize;
		
		url = this.addSystemID(url, systemID);
		
		this.performRequest("get", {
			url: url,
			load: load,
			error: error,
			sync: !async
		});
	},
	
	statics: {
		_instance: null,

		_createInstance: function(params){
		  if (this._instance === null) {
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
			// TODO handle params, e.g. 
	//        this._instance.setIWidget(iWidget);
	//        this._instance.setUrl("prefix", this.getServiceURLRoot());
	//        this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
	//        this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
		  }
		  else {
			// programming error
			var error = new Error("RestHandler instance has already been initialized!");
			com.ibm.bpm.social.util.common.Log.consoleError(error.message);
			throw error;
		  }
		  return this._instance;
		},

		_getInstance: function(){
		  if (this._instance === null) {
			// TODO for now return default handler
			this._instance = new com.ibm.bpm.social.util.common.SocialRestHandler();
	//        // programming error
	//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
	//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
	//        throw error;
		  }
		  return this._instance;
		}
	}
});

com.ibm.bpm.social.util.common.SocialRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.SocialRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.SocialRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.SimpleMarkupUtils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.SimpleMarkupUtils"] = true;
// SimpleMarkupUtils.js
//BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C94
// (C) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT
dojo.provide("com.ibm.bpm.social.util.common.SimpleMarkupUtils");

com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags = ["span","b"];

com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup = function(markup){
	var result = '';
	if(markup !== undefined && markup !== null) {
		var temp = '';
		var quote = '';
		var tag = false;
		var i = 0;
		while (i < markup.length) {
			var c = markup.charAt(i);
			switch (c) {
			case '<':
				if (quote) {
					temp = temp + c;
				} else {
					result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(temp);
					temp = c;
					tag = true;
				}
				break;
			case '>':
				temp = temp + c;
				if (tag && !quote) {
					result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._checkTag(temp);
					temp = '';
					tag = false;
				}
				break;
			case '\'':
			case '\"':
				temp = temp + c;
				if (quote && (quote == c)) {
					quote = '';
				} else if (tag && !quote) {
					quote = c;
				}
				break;
			case '\\':
				temp = temp + c;
				i++;
				if (i < markup.length) {
					c = markup.charAt(i);
					temp = temp + c;
				}
				break;
			default:
				temp = temp + c;
			break;
			}
			i++;
		}
		if (temp) {
			result = result + com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(temp); 
		}
	}
	return result;
};

com.ibm.bpm.social.util.common.SimpleMarkupUtils._checkTag = function(tag) {
	var supported = false;
	var lc = tag.toLowerCase();
	for (i=0; i<com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags.length; i++) {
		var test = com.ibm.bpm.social.util.common.SimpleMarkupUtils._supportedTags[i];
		if (lc.indexOf('<'+test+' ') == 0) {
			supported = true;
			var tmp = document.createElement('div');
			tmp.innerHTML = tag + '</'+test+'>';
			tag = ('<'+test+' ');
			if (tmp.firstChild.className) {
				tag = tag + 'class="'+tmp.firstChild.className+'" ' + ' ';
			}
			if (tmp.firstChild.objid) {
				tag = tag + 'objid="'+tmp.firstChild.objid+'" ' + ' ';
			}
			tag = tag + '>';			
			delete tmp;
		}
		else if (lc.indexOf('<'+test+'>') == 0) {
			supported = true;
		}
		else if (lc.indexOf('<'+test+'/>') == 0) {
			supported = true;
		}
		else if (lc.indexOf('</'+test+' ') == 0) {
			supported = true;
		}
		else if (lc.indexOf('</'+test+'>') == 0) {
			supported = true;
		}
	}
	if (!supported) {
		tag = com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize(tag);
	}	
	return tag;
}

com.ibm.bpm.social.util.common.SimpleMarkupUtils._sanitize = function(markup) {
	var div = document.createElement('div');
	var textNode = document.createTextNode(markup);
	div.appendChild(textNode);
	return div.innerHTML;
}

}

if(!dojo._hasResource['org.cometd']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['org.cometd'] = true;
/*
 * Copyright (c) 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

if (typeof dojo !== 'undefined')
{
    dojo.provide('org.cometd');
}
else
{
    // Namespaces for the cometd implementation
    this.org = this.org || {};
    org.cometd = {};
}

org.cometd.JSON = {};
org.cometd.JSON.toJSON = org.cometd.JSON.fromJSON = function(object)
{
    throw 'Abstract';
};

org.cometd.Utils = {};

org.cometd.Utils.isString = function(value)
{
    if (value === undefined || value === null)
    {
        return false;
    }
    return typeof value === 'string' ||  value instanceof String;
};

org.cometd.Utils.isArray = function(value)
{
    if (value === undefined || value === null)
    {
        return false;
    }
    return value instanceof Array;
};

/**
 * Returns whether the given element is contained into the given array.
 * @param element the element to check presence for
 * @param array the array to check for the element presence
 * @return the index of the element, if present, or a negative index if the element is not present
 */
org.cometd.Utils.inArray = function(element, array)
{
    for (var i = 0; i < array.length; ++i)
    {
        if (element === array[i])
        {
            return i;
        }
    }
    return -1;
};

org.cometd.Utils.setTimeout = function(cometd, funktion, delay)
{
    return window.setTimeout(function()
    {
        try
        {
            funktion();
        }
        catch (x)
        {
            cometd._debug('Exception invoking timed function', funktion, x);
        }
    }, delay);
};

org.cometd.Utils.clearTimeout = function(timeoutHandle)
{
    window.clearTimeout(timeoutHandle);
};

/**
 * A registry for transports used by the Cometd object.
 */
org.cometd.TransportRegistry = function()
{
    var _types = [];
    var _transports = {};

    this.getTransportTypes = function()
    {
        return _types.slice(0);
    };

    this.findTransportTypes = function(version, crossDomain, url)
    {
        var result = [];
        for (var i = 0; i < _types.length; ++i)
        {
            var type = _types[i];
            if (_transports[type].accept(version, crossDomain, url) === true)
            {
                result.push(type);
            }
        }
        return result;
    };

    this.negotiateTransport = function(types, version, crossDomain, url)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            var type = _types[i];
            for (var j = 0; j < types.length; ++j)
            {
                if (type === types[j])
                {
                    var transport = _transports[type];
                    if (transport.accept(version, crossDomain, url) === true)
                    {
                        return transport;
                    }
                }
            }
        }
        return null;
    };

    this.add = function(type, transport, index)
    {
        var existing = false;
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                existing = true;
                break;
            }
        }

        if (!existing)
        {
            if (typeof index !== 'number')
            {
                _types.push(type);
            }
            else
            {
                _types.splice(index, 0, type);
            }
            _transports[type] = transport;
        }

        return !existing;
    };

    this.find = function(type)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                return _transports[type];
            }
        }
        return null;
    };

    this.remove = function(type)
    {
        for (var i = 0; i < _types.length; ++i)
        {
            if (_types[i] === type)
            {
                _types.splice(i, 1);
                var transport = _transports[type];
                delete _transports[type];
                return transport;
            }
        }
        return null;
    };

    this.clear = function()
    {
        _types = [];
        _transports = {};
    };

    this.reset = function()
    {
        for (var i = 0; i < _types.length; ++i)
        {
            _transports[_types[i]].reset();
        }
    };
};

/**
 * Base object with the common functionality for transports.
 */
org.cometd.Transport = function()
{
    var _type;
    var _cometd;

    /**
     * Function invoked just after a transport has been successfully registered.
     * @param type the type of transport (for example 'long-polling')
     * @param cometd the cometd object this transport has been registered to
     * @see #unregistered()
     */
    this.registered = function(type, cometd)
    {
        _type = type;
        _cometd = cometd;
    };

    /**
     * Function invoked just after a transport has been successfully unregistered.
     * @see #registered(type, cometd)
     */
    this.unregistered = function()
    {
        _type = null;
        _cometd = null;
    };

    this._debug = function()
    {
        _cometd._debug.apply(_cometd, arguments);
    };

    this._mixin = function()
    {
        return _cometd._mixin.apply(_cometd, arguments);
    };

    this.getConfiguration = function()
    {
        return _cometd.getConfiguration();
    };

    this.getAdvice = function()
    {
        return _cometd.getAdvice();
    };

    this.setTimeout = function(funktion, delay)
    {
        return org.cometd.Utils.setTimeout(_cometd, funktion, delay);
    };

    this.clearTimeout = function(handle)
    {
        org.cometd.Utils.clearTimeout(handle);
    };

    /**
     * Converts the given response into an array of bayeux messages
     * @param response the response to convert
     * @return an array of bayeux messages obtained by converting the response
     */
    this.convertToMessages = function (response)
    {
        if (org.cometd.Utils.isString(response))
        {
            try
            {
                return org.cometd.JSON.fromJSON(response);
            }
            catch(x)
            {
                this._debug('Could not convert to JSON the following string', '"' + response + '"');
                throw x;
            }
        }
        if (org.cometd.Utils.isArray(response))
        {
            return response;
        }
        if (response === undefined || response === null)
        {
            return [];
        }
        if (response instanceof Object)
        {
            return [response];
        }
        throw 'Conversion Error ' + response + ', typeof ' + (typeof response);
    };

    /**
     * Returns whether this transport can work for the given version and cross domain communication case.
     * @param version a string indicating the transport version
     * @param crossDomain a boolean indicating whether the communication is cross domain
     * @return true if this transport can work for the given version and cross domain communication case,
     * false otherwise
     */
    this.accept = function(version, crossDomain, url)
    {
        throw 'Abstract';
    };

    /**
     * Returns the type of this transport.
     * @see #registered(type, cometd)
     */
    this.getType = function()
    {
        return _type;
    };

    this.send = function(envelope, metaConnect)
    {
        throw 'Abstract';
    };

    this.reset = function()
    {
        this._debug('Transport', _type, 'reset');
    };

    this.abort = function()
    {
        this._debug('Transport', _type, 'aborted');
    };

    this.toString = function()
    {
        return this.getType();
    };
};

org.cometd.Transport.derive = function(baseObject)
{
    function F() {}
    F.prototype = baseObject;
    return new F();
};

/**
 * Base object with the common functionality for transports based on requests.
 * The key responsibility is to allow at most 2 outstanding requests to the server,
 * to avoid that requests are sent behind a long poll.
 * To achieve this, we have one reserved request for the long poll, and all other
 * requests are serialized one after the other.
 */
org.cometd.RequestTransport = function()
{
    var _super = new org.cometd.Transport();
    var _self = org.cometd.Transport.derive(_super);
    var _requestIds = 0;
    var _metaConnectRequest = null;
    var _requests = [];
    var _envelopes = [];

    function _coalesceEnvelopes(envelope)
    {
        while (_envelopes.length > 0)
        {
            var envelopeAndRequest = _envelopes[0];
            var newEnvelope = envelopeAndRequest[0];
            var newRequest = envelopeAndRequest[1];
            if (newEnvelope.url === envelope.url &&
                    newEnvelope.sync === envelope.sync)
            {
                _envelopes.shift();
                envelope.messages = envelope.messages.concat(newEnvelope.messages);
                this._debug('Coalesced', newEnvelope.messages.length, 'messages from request', newRequest.id);
                continue;
            }
            break;
        }
    }

    function _transportSend(envelope, request)
    {
        this.transportSend(envelope, request);
        request.expired = false;

        if (!envelope.sync)
        {
            var maxDelay = this.getConfiguration().maxNetworkDelay;
            var delay = maxDelay;
            if (request.metaConnect === true)
            {
                delay += this.getAdvice().timeout;
            }

            this._debug('Transport', this.getType(), 'waiting at most', delay, 'ms for the response, maxNetworkDelay', maxDelay);

            var self = this;
            request.timeout = this.setTimeout(function()
            {
                request.expired = true;
                if (request.xhr)
                {
                    request.xhr.abort();
                }
                var errorMessage = 'Request ' + request.id + ' of transport ' + self.getType() + ' exceeded ' + delay + ' ms max network delay';
                self._debug(errorMessage);
                self.complete(request, false, request.metaConnect);
                envelope.onFailure(request.xhr, envelope.messages, 'timeout', errorMessage);
            }, delay);
        }
    }

    function _queueSend(envelope)
    {
        var requestId = ++_requestIds;
        var request = {
            id: requestId,
            metaConnect: false
        };

        // Consider the metaConnect requests which should always be present
        if (_requests.length < this.getConfiguration().maxConnections - 1)
        {
            _requests.push(request);
            _transportSend.call(this, envelope, request);
        }
        else
        {
            this._debug('Transport', this.getType(), 'queueing request', requestId, 'envelope', envelope);
            _envelopes.push([envelope, request]);
        }
    }

    function _metaConnectComplete(request)
    {
        var requestId = request.id;
        this._debug('Transport', this.getType(), 'metaConnect complete, request', requestId);
        if (_metaConnectRequest !== null && _metaConnectRequest.id !== requestId)
        {
            throw 'Longpoll request mismatch, completing request ' + requestId;
        }

        // Reset metaConnect request
        _metaConnectRequest = null;
    }

    function _complete(request, success)
    {
        var index = org.cometd.Utils.inArray(request, _requests);
        // The index can be negative if the request has been aborted
        if (index >= 0)
        {
            _requests.splice(index, 1);
        }

        if (_envelopes.length > 0)
        {
            var envelopeAndRequest = _envelopes.shift();
            var nextEnvelope = envelopeAndRequest[0];
            var nextRequest = envelopeAndRequest[1];
            this._debug('Transport dequeued request', nextRequest.id);
            if (success)
            {
                if (this.getConfiguration().autoBatch)
                {
                    _coalesceEnvelopes.call(this, nextEnvelope);
                }
                _queueSend.call(this, nextEnvelope);
                this._debug('Transport completed request', request.id, nextEnvelope);
            }
            else
            {
                // Keep the semantic of calling response callbacks asynchronously after the request
                var self = this;
                this.setTimeout(function()
                {
                    self.complete(nextRequest, false, nextRequest.metaConnect);
                    nextEnvelope.onFailure(nextRequest.xhr, nextEnvelope.messages, 'error', 'Previous request failed');
                }, 0);
            }
        }
    }

    _self.complete = function(request, success, metaConnect)
    {
        if (metaConnect)
        {
            _metaConnectComplete.call(this, request);
        }
        else
        {
            _complete.call(this, request, success);
        }
    };

    /**
     * Performs the actual send depending on the transport type details.
     * @param envelope the envelope to send
     * @param request the request information
     */
    _self.transportSend = function(envelope, request)
    {
        throw 'Abstract';
    };

    _self.transportSuccess = function(envelope, request, responses)
    {
        if (!request.expired)
        {
            this.clearTimeout(request.timeout);
            this.complete(request, true, request.metaConnect);
            if (responses && responses.length > 0)
            {
                envelope.onSuccess(responses);
            }
            else
            {
                envelope.onFailure(request.xhr, envelope.messages, 'Empty HTTP response');
            }
        }
    };

    _self.transportFailure = function(envelope, request, reason, exception)
    {
        if (!request.expired)
        {
            this.clearTimeout(request.timeout);
            this.complete(request, false, request.metaConnect);
            envelope.onFailure(request.xhr, envelope.messages, reason, exception);
        }
    };

    function _metaConnectSend(envelope)
    {
        if (_metaConnectRequest !== null)
        {
            throw 'Concurrent metaConnect requests not allowed, request id=' + _metaConnectRequest.id + ' not yet completed';
        }

        var requestId = ++_requestIds;
        this._debug('Transport', this.getType(), 'metaConnect send, request', requestId, 'envelope', envelope);
        var request = {
            id: requestId,
            metaConnect: true
        };
        _transportSend.call(this, envelope, request);
        _metaConnectRequest = request;
    }

    _self.send = function(envelope, metaConnect)
    {
        if (metaConnect)
        {
            _metaConnectSend.call(this, envelope);
        }
        else
        {
            _queueSend.call(this, envelope);
        }
    };

    _self.abort = function()
    {
        _super.abort();
        for (var i = 0; i < _requests.length; ++i)
        {
            var request = _requests[i];
            this._debug('Aborting request', request);
            if (request.xhr)
            {
                request.xhr.abort();
            }
        }
        if (_metaConnectRequest)
        {
            this._debug('Aborting metaConnect request', _metaConnectRequest);
            if (_metaConnectRequest.xhr)
            {
                _metaConnectRequest.xhr.abort();
            }
        }
        this.reset();
    };

    _self.reset = function()
    {
        _super.reset();
        _metaConnectRequest = null;
        _requests = [];
        _envelopes = [];
    };

    return _self;
};

org.cometd.LongPollingTransport = function()
{
    var _super = new org.cometd.RequestTransport();
    var _self = org.cometd.Transport.derive(_super);
    // By default, support cross domain
    var _supportsCrossDomain = true;

    _self.accept = function(version, crossDomain, url)
    {
        return _supportsCrossDomain || !crossDomain;
    };

    _self.xhrSend = function(packet)
    {
        throw 'Abstract';
    };

    _self.transportSend = function(envelope, request)
    {
        this._debug('Transport', this.getType(), 'sending request', request.id, 'envelope', envelope);

        var self = this;
        try
        {
            var sameStack = true;
            request.xhr = this.xhrSend({
                transport: this,
                url: envelope.url,
                sync: envelope.sync,
                headers: this.getConfiguration().requestHeaders,
                body: org.cometd.JSON.toJSON(envelope.messages),
                onSuccess: function(response)
                {
                    self._debug('Transport', self.getType(), 'received response', response);
                    var success = false;
                    try
                    {
                        var received = self.convertToMessages(response);
                        if (received.length === 0)
                        {
                            _supportsCrossDomain = false;
                            self.transportFailure(envelope, request, 'no response', null);
                        }
                        else
                        {
                            success = true;
                            self.transportSuccess(envelope, request, received);
                        }
                    }
                    catch(x)
                    {
                        self._debug(x);
                        if (!success)
                        {
                            _supportsCrossDomain = false;
                            self.transportFailure(envelope, request, 'bad response', x);
                        }
                    }
                },
                onError: function(reason, exception)
                {
                    _supportsCrossDomain = false;
                    if (sameStack)
                    {
                        // Keep the semantic of calling response callbacks asynchronously after the request
                        self.setTimeout(function()
                        {
                            self.transportFailure(envelope, request, reason, exception);
                        }, 0);
                    }
                    else
                    {
                        self.transportFailure(envelope, request, reason, exception);
                    }
                }
            });
            sameStack = false;
        }
        catch (x)
        {
            _supportsCrossDomain = false;
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                self.transportFailure(envelope, request, 'error', x);
            }, 0);
        }
    };

    _self.reset = function()
    {
        _super.reset();
        _supportsCrossDomain = true;
    };

    return _self;
};

org.cometd.CallbackPollingTransport = function()
{
    var _super = new org.cometd.RequestTransport();
    var _self = org.cometd.Transport.derive(_super);
    var _maxLength = 2000;

    _self.accept = function(version, crossDomain, url)
    {
        return true;
    };

    _self.jsonpSend = function(packet)
    {
        throw 'Abstract';
    };

    _self.transportSend = function(envelope, request)
    {
        var self = this;

        // Microsoft Internet Explorer has a 2083 URL max length
        // We must ensure that we stay within that length
        var start = 0;
        var length = envelope.messages.length;
        var lengths = [];
        while (length > 0)
        {
            // Encode the messages because all brackets, quotes, commas, colons, etc
            // present in the JSON will be URL encoded, taking many more characters
            var json = org.cometd.JSON.toJSON(envelope.messages.slice(start, start + length));
            var urlLength = envelope.url.length + encodeURI(json).length;

            // Let's stay on the safe side and use 2000 instead of 2083
            // also because we did not count few characters among which
            // the parameter name 'message' and the parameter 'jsonp',
            // which sum up to about 50 chars
            if (urlLength > _maxLength)
            {
                if (length === 1)
                {
                    var x = 'Bayeux message too big (' + urlLength + ' bytes, max is ' + _maxLength + ') ' +
                            'for transport ' + this.getType();
                    // Keep the semantic of calling response callbacks asynchronously after the request
                    this.setTimeout(function()
                    {
                        self.transportFailure(envelope, request, 'error', x);
                    }, 0);
                    return;
                }

                --length;
                continue;
            }

            lengths.push(length);
            start += length;
            length = envelope.messages.length - start;
        }

        // Here we are sure that the messages can be sent within the URL limit

        var envelopeToSend = envelope;
        if (lengths.length > 1)
        {
            var begin = 0;
            var end = lengths[0];
            this._debug('Transport', this.getType(), 'split', envelope.messages.length, 'messages into', lengths.join(' + '));
            envelopeToSend = this._mixin(false, {}, envelope);
            envelopeToSend.messages = envelope.messages.slice(begin, end);
            envelopeToSend.onSuccess = envelope.onSuccess;
            envelopeToSend.onFailure = envelope.onFailure;

            for (var i = 1; i < lengths.length; ++i)
            {
                var nextEnvelope = this._mixin(false, {}, envelope);
                begin = end;
                end += lengths[i];
                nextEnvelope.messages = envelope.messages.slice(begin, end);
                nextEnvelope.onSuccess = envelope.onSuccess;
                nextEnvelope.onFailure = envelope.onFailure;
                this.send(nextEnvelope, request.metaConnect);
            }
        }

        this._debug('Transport', this.getType(), 'sending request', request.id, 'envelope', envelopeToSend);

        try
        {
            var sameStack = true;
            this.jsonpSend({
                transport: this,
                url: envelopeToSend.url,
                sync: envelopeToSend.sync,
                headers: this.getConfiguration().requestHeaders,
                body: org.cometd.JSON.toJSON(envelopeToSend.messages),
                onSuccess: function(responses)
                {
                    var success = false;
                    try
                    {
                        var received = self.convertToMessages(responses);
                        if (received.length === 0)
                        {
                            self.transportFailure(envelopeToSend, request, 'no response');
                        }
                        else
                        {
                            success=true;
                            self.transportSuccess(envelopeToSend, request, received);
                        }
                    }
                    catch (x)
                    {
                        self._debug(x);
                        if (!success)
                        {
                            self.transportFailure(envelopeToSend, request, 'bad response', x);
                        }
                    }
                },
                onError: function(reason, exception)
                {
                    if (sameStack)
                    {
                        // Keep the semantic of calling response callbacks asynchronously after the request
                        self.setTimeout(function()
                        {
                            self.transportFailure(envelopeToSend, request, reason, exception);
                        }, 0);
                    }
                    else
                    {
                        self.transportFailure(envelopeToSend, request, reason, exception);
                    }
                }
            });
            sameStack = false;
        }
        catch (xx)
        {
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                self.transportFailure(envelopeToSend, request, 'error', xx);
            }, 0);
        }
    };

    return _self;
};

org.cometd.WebSocketTransport = function()
{
    var _super = new org.cometd.Transport();
    var _self = org.cometd.Transport.derive(_super);
    var _cometd;
    // By default, support WebSocket
    var _supportsWebSocket = true;
    // Whether we were able to establish a WebSocket connection
    var _webSocketSupported = false;
    // Envelopes that have been sent
    var _envelopes = {};
    // Timeouts for messages that have been sent
    var _timeouts = {};
    var _webSocket = null;
    var _opened = false;
    var _connected = false;
    var _successCallback;

    function _websocketConnect()
    {
        // Mangle the URL, changing the scheme from 'http' to 'ws'
        var url = _cometd.getURL().replace(/^http/, 'ws');
        this._debug('Transport', this.getType(), 'connecting to URL', url);

        var webSocket = new org.cometd.WebSocket(url);
        var self = this;
        webSocket.onopen = function()
        {
            self._debug('WebSocket opened', webSocket);
            if (webSocket !== _webSocket)
            {
                // It's possible that the onopen callback is invoked
                // with a delay so that we have already reconnected
                self._debug('Ignoring open event, WebSocket', _webSocket);
                return;
            }
            self.onOpen();
        };
        webSocket.onclose = function(event)
        {
            var code = event ? event.code : 1000;
            var reason = event ? event.reason : undefined;
            self._debug('WebSocket closed', code, '/', reason, webSocket);
            if (webSocket !== _webSocket)
            {
                // The onclose callback may be invoked when the server sends
                // the close message reply, but after we have already reconnected
                self._debug('Ignoring close event, WebSocket', _webSocket);
                return;
            }
            self.onClose(code, reason);
        };
        webSocket.onerror = function()
        {
            webSocket.onclose({ code: 1002 });
        };
        webSocket.onmessage = function(message)
        {
            self._debug('WebSocket message', message, webSocket);
            if (webSocket !== _webSocket)
            {
                self._debug('Ignoring message event, WebSocket', _webSocket);
                return;
            }
            self.onMessage(message);
        };

        _webSocket = webSocket;
        this._debug('Transport', this.getType(), 'configured callbacks on', webSocket);
    }

    function _webSocketSend(envelope, metaConnect)
    {
        var json = org.cometd.JSON.toJSON(envelope.messages);

        _webSocket.send(json);
        this._debug('Transport', this.getType(), 'sent', envelope, 'metaConnect =', metaConnect);

        // Manage the timeout waiting for the response
        var maxDelay = this.getConfiguration().maxNetworkDelay;
        var delay = maxDelay;
        if (metaConnect)
        {
            delay += this.getAdvice().timeout;
            _connected = true;
        }

        var messageIds = [];
        for (var i = 0; i < envelope.messages.length; ++i)
        {
            var message = envelope.messages[i];
            if (message.id)
            {
                messageIds.push(message.id);
                var self = this;
                _timeouts[message.id] = this.setTimeout(function()
                {
                    if (_webSocket)
                    {
                        _webSocket.close(1000, 'Timeout');
                    }
                }, delay);
            }
        }

        this._debug('Transport', this.getType(), 'waiting at most', delay, 'ms for messages', messageIds, 'maxNetworkDelay', maxDelay, ', timeouts:', _timeouts);
    }

    function _send(envelope, metaConnect)
    {
        try
        {
            if (_webSocket === null)
            {
                _websocketConnect.call(this);
            }
            // We may have a non-null _webSocket, but not be open yet so
            // to avoid out of order deliveries, we check if we are open
            else if (_opened)
            {
                _webSocketSend.call(this, envelope, metaConnect);
            }
        }
        catch (x)
        {
            // Keep the semantic of calling response callbacks asynchronously after the request
            this.setTimeout(function()
            {
                envelope.onFailure(_webSocket, envelope.messages, 'error', x);
            }, 0);
        }
    }

    _self.onOpen = function()
    {
        this._debug('Transport', this.getType(), 'opened', _webSocket);
        _opened = true;
        _webSocketSupported = true;

        this._debug('Sending pending messages', _envelopes);
        for (var key in _envelopes)
        {
            var element = _envelopes[key];
            var envelope = element[0];
            var metaConnect = element[1];
            // Store the success callback, which is independent from the envelope,
            // so that it can be used to notify arrival of messages.
            _successCallback = envelope.onSuccess;
            _webSocketSend.call(this, envelope, metaConnect);
        }
    };

    _self.onMessage = function(wsMessage)
    {
        this._debug('Transport', this.getType(), 'received websocket message', wsMessage, _webSocket);

        var close = false;
        var messages = this.convertToMessages(wsMessage.data);
        var messageIds = [];
        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];

            // Detect if the message is a response to a request we made.
            // If it's a meta message, for sure it's a response;
            // otherwise it's a publish message and publish responses lack the data field
            if (/^\/meta\//.test(message.channel) || message.data === undefined)
            {
                if (message.id)
                {
                    messageIds.push(message.id);

                    var timeout = _timeouts[message.id];
                    if (timeout)
                    {
                        this.clearTimeout(timeout);
                        delete _timeouts[message.id];
                        this._debug('Transport', this.getType(), 'removed timeout for message', message.id, ', timeouts', _timeouts);
                    }
                }
            }

            if ('/meta/connect' === message.channel)
            {
                _connected = false;
            }
            if ('/meta/disconnect' === message.channel && !_connected)
            {
                close = true;
            }
        }

        // Remove the envelope corresponding to the messages
        var removed = false;
        for (var j = 0; j < messageIds.length; ++j)
        {
            var id = messageIds[j];
            for (var key in _envelopes)
            {
                var ids = key.split(',');
                var index = org.cometd.Utils.inArray(id, ids);
                if (index >= 0)
                {
                    removed = true;
                    ids.splice(index, 1);
                    var envelope = _envelopes[key][0];
                    var metaConnect = _envelopes[key][1];
                    delete _envelopes[key];
                    if (ids.length > 0)
                    {
                        _envelopes[ids.join(',')] = [envelope, metaConnect];
                    }
                    break;
                }
            }
        }
        if (removed)
        {
            this._debug('Transport', this.getType(), 'removed envelope, envelopes', _envelopes);
        }

        _successCallback.call(this, messages);

        if (close)
        {
            _webSocket.close(1000, 'Disconnect');
        }
    };

    _self.onClose = function(code, reason)
    {
        this._debug('Transport', this.getType(), 'closed', code, reason, _webSocket);

        // Remember if we were able to connect
        // This close event could be due to server shutdown, and if it restarts we want to try websocket again
        _supportsWebSocket = _webSocketSupported;

        for (var id in _timeouts)
        {
            this.clearTimeout(_timeouts[id]);
        }
        _timeouts = {};

        for (var key in _envelopes)
        {
            var envelope = _envelopes[key][0];
            var metaConnect = _envelopes[key][1];
            if (metaConnect)
            {
                _connected = false;
            }
            envelope.onFailure(_webSocket, envelope.messages, 'closed ' + code + '/' + reason);
        }
        _envelopes = {};

        if (_webSocket !== null && _opened)
        {
            _webSocket.close(1000, 'Close');
        }
        _opened = false;
        _webSocket = null;
    };

    _self.registered = function(type, cometd)
    {
        _super.registered(type, cometd);
        _cometd = cometd;
    };

    _self.accept = function(version, crossDomain, url)
    {
        // Using !! to return a boolean (and not the WebSocket object)
        return _supportsWebSocket && !!org.cometd.WebSocket && _cometd.websocketEnabled !== false;
    };

    _self.send = function(envelope, metaConnect)
    {
        this._debug('Transport', this.getType(), 'sending', envelope, 'metaConnect =', metaConnect);

        // Store the envelope in any case; if the websocket cannot be opened, we fail it in close()
        var messageIds = [];
        for (var i = 0; i < envelope.messages.length; ++i)
        {
            var message = envelope.messages[i];
            if (message.id)
            {
                messageIds.push(message.id);
            }
        }
        _envelopes[messageIds.join(',')] = [envelope, metaConnect];
        this._debug('Transport', this.getType(), 'stored envelope, envelopes', _envelopes);

        _send.call(this, envelope, metaConnect);
    };

    _self.reset = function()
    {
        _super.reset();
        if (_webSocket !== null && _opened)
        {
            _webSocket.close(1000, 'Reset');
        }
        _supportsWebSocket = true;
        _webSocketSupported = false;
        _timeouts = {};
        _envelopes = {};
        _webSocket = null;
        _opened = false;
        _successCallback = null;
    };

    return _self;
};

/**
 * The constructor for a Cometd object, identified by an optional name.
 * The default name is the string 'default'.
 * In the rare case a page needs more than one Bayeux conversation,
 * a new instance can be created via:
 * <pre>
 * var bayeuxUrl2 = ...;
 *
 * // Dojo style
 * var cometd2 = new dojox.Cometd('another_optional_name');
 *
 * // jQuery style
 * var cometd2 = new $.Cometd('another_optional_name');
 *
 * cometd2.init({url: bayeuxUrl2});
 * </pre>
 * @param name the optional name of this cometd object
 */
// IMPLEMENTATION NOTES:
// Be very careful in not changing the function order and pass this file every time through JSLint (http://jslint.com)
// The only implied globals must be "dojo", "org" and "window", and check that there are no "unused" warnings
// Failing to pass JSLint may result in shrinkers/minifiers to create an unusable file.
org.cometd.Cometd = function(name)
{
    var _cometd = this;
    var _name = name || 'default';
    var _crossDomain = false;
    var _transports = new org.cometd.TransportRegistry();
    var _transport;
    var _status = 'disconnected';
    var _messageId = 0;
    var _clientId = null;
    var _batch = 0;
    var _messageQueue = [];
    var _internalBatch = false;
    var _listeners = {};
    var _backoff = 0;
    var _scheduledSend = null;
    var _extensions = [];
    var _advice = {};
    var _handshakeProps;
    var _reestablish = false;
    var _connected = false;
    var _config = {
        maxConnections: 2,
        backoffIncrement: 1000,
        maxBackoff: 60000,
        logLevel: 'info',
        reverseIncomingExtensions: true,
        maxNetworkDelay: 10000,
        requestHeaders: {},
        appendMessageTypeToURL: true,
        autoBatch: false,
        advice: {
            timeout: 60000,
            interval: 0,
            reconnect: 'retry'
        }
    };

    /**
     * Mixes in the given objects into the target object by copying the properties.
     * @param deep if the copy must be deep
     * @param target the target object
     * @param objects the objects whose properties are copied into the target
     */
    this._mixin = function(deep, target, objects)
    {
        var result = target || {};

        // Skip first 2 parameters (deep and target), and loop over the others
        for (var i = 2; i < arguments.length; ++i)
        {
            var object = arguments[i];

            if (object === undefined || object === null)
            {
                continue;
            }

            for (var propName in object)
            {
                var prop = object[propName];
                var targ = result[propName];

                // Avoid infinite loops
                if (prop === target)
                {
                    continue;
                }
                // Do not mixin undefined values
                if (prop === undefined)
                {
                    continue;
                }

                if (deep && typeof prop === 'object' && prop !== null)
                {
                    if (prop instanceof Array)
                    {
                        result[propName] = this._mixin(deep, targ instanceof Array ? targ : [], prop);
                    }
                    else
                    {
                        var source = typeof targ === 'object' && !(targ instanceof Array) ? targ : {};
                        result[propName] = this._mixin(deep, source, prop);
                    }
                }
                else
                {
                    result[propName] = prop;
                }
            }
        }

        return result;
    };

    function _isString(value)
    {
        return org.cometd.Utils.isString(value);
    }

    function _isFunction(value)
    {
        if (value === undefined || value === null)
        {
            return false;
        }
        return typeof value === 'function';
    }

    function _log(level, args)
    {
        if (window.console)
        {
            var logger = window.console[level];
            if (_isFunction(logger))
            {
                logger.apply(window.console, args);
            }
        }
    }

    this._warn = function()
    {
        _log('warn', arguments);
    };

    this._info = function()
    {
        if (_config.logLevel !== 'warn')
        {
            _log('info', arguments);
        }
    };

    this._debug = function()
    {
        if (_config.logLevel === 'debug')
        {
            _log('debug', arguments);
        }
    };

    /**
     * Returns whether the given hostAndPort is cross domain.
     * The default implementation checks against window.location.host
     * but this function can be overridden to make it work in non-browser
     * environments.
     *
     * @param hostAndPort the host and port in format host:port
     * @return whether the given hostAndPort is cross domain
     */
    this._isCrossDomain = function(hostAndPort)
    {
        return hostAndPort && hostAndPort !== window.location.host;
    };

    function _configure(configuration)
    {
        _cometd._debug('Configuring cometd object with', configuration);
        // Support old style param, where only the Bayeux server URL was passed
        if (_isString(configuration))
        {
            configuration = { url: configuration };
        }
        if (!configuration)
        {
            configuration = {};
        }

        _config = _cometd._mixin(false, _config, configuration);

        if (!_config.url)
        {
            throw 'Missing required configuration parameter \'url\' specifying the Bayeux server URL';
        }

        // Check if we're cross domain
        // [1] = protocol://, [2] = host:port, [3] = host, [4] = IPv6_host, [5] = IPv4_host, [6] = :port, [7] = port, [8] = uri, [9] = rest
        var urlParts = /(^https?:\/\/)?(((\[[^\]]+\])|([^:\/\?#]+))(:(\d+))?)?([^\?#]*)(.*)?/.exec(_config.url);
        var hostAndPort = urlParts[2];
        var uri = urlParts[8];
        var afterURI = urlParts[9];
        _crossDomain = _cometd._isCrossDomain(hostAndPort);

        // Check if appending extra path is supported
        if (_config.appendMessageTypeToURL)
        {
            if (afterURI !== undefined && afterURI.length > 0)
            {
                _cometd._info('Appending message type to URI ' + uri + afterURI + ' is not supported, disabling \'appendMessageTypeToURL\' configuration');
                _config.appendMessageTypeToURL = false;
            }
            else
            {
                var uriSegments = uri.split('/');
                var lastSegmentIndex = uriSegments.length - 1;
                if (uri.match(/\/$/))
                {
                    lastSegmentIndex -= 1;
                }
                if (uriSegments[lastSegmentIndex].indexOf('.') >= 0)
                {
                    // Very likely the CometD servlet's URL pattern is mapped to an extension, such as *.cometd
                    // It will be difficult to add the extra path in this case
                    _cometd._info('Appending message type to URI ' + uri + ' is not supported, disabling \'appendMessageTypeToURL\' configuration');
                    _config.appendMessageTypeToURL = false;
                }
            }
        }
    }

    function _clearSubscriptions()
    {
        for (var channel in _listeners)
        {
            var subscriptions = _listeners[channel];
            for (var i = 0; i < subscriptions.length; ++i)
            {
                var subscription = subscriptions[i];
                if (subscription && !subscription.listener)
                {
                    delete subscriptions[i];
                    _cometd._debug('Removed subscription', subscription, 'for channel', channel);
                }
            }
        }
    }

    function _setStatus(newStatus)
    {
        if (_status !== newStatus)
        {
            _cometd._debug('Status', _status, '->', newStatus);
            _status = newStatus;
        }
    }

    function _isDisconnected()
    {
        return _status === 'disconnecting' || _status === 'disconnected';
    }

    function _nextMessageId()
    {
        return ++_messageId;
    }

    function _applyExtension(scope, callback, name, message, outgoing)
    {
        try
        {
            return callback.call(scope, message);
        }
        catch (x)
        {
            _cometd._debug('Exception during execution of extension', name, x);
            var exceptionCallback = _cometd.onExtensionException;
            if (_isFunction(exceptionCallback))
            {
                _cometd._debug('Invoking extension exception callback', name, x);
                try
                {
                    exceptionCallback.call(_cometd, x, name, outgoing, message);
                }
                catch(xx)
                {
                    _cometd._info('Exception during execution of exception callback in extension', name, xx);
                }
            }
            return message;
        }
    }

    function _applyIncomingExtensions(message)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            if (message === undefined || message === null)
            {
                break;
            }

            var index = _config.reverseIncomingExtensions ? _extensions.length - 1 - i : i;
            var extension = _extensions[index];
            var callback = extension.extension.incoming;
            if (_isFunction(callback))
            {
                var result = _applyExtension(extension.extension, callback, extension.name, message, false);
                message = result === undefined ? message : result;
            }
        }
        return message;
    }

    function _applyOutgoingExtensions(message)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            if (message === undefined || message === null)
            {
                break;
            }

            var extension = _extensions[i];
            var callback = extension.extension.outgoing;
            if (_isFunction(callback))
            {
                var result = _applyExtension(extension.extension, callback, extension.name, message, true);
                message = result === undefined ? message : result;
            }
        }
        return message;
    }

    function _notify(channel, message)
    {
        var subscriptions = _listeners[channel];
        if (subscriptions && subscriptions.length > 0)
        {
            for (var i = 0; i < subscriptions.length; ++i)
            {
                var subscription = subscriptions[i];
                // Subscriptions may come and go, so the array may have 'holes'
                if (subscription)
                {
                    try
                    {
                        subscription.callback.call(subscription.scope, message);
                    }
                    catch (x)
                    {
                        _cometd._debug('Exception during notification', subscription, message, x);
                        var listenerCallback = _cometd.onListenerException;
                        if (_isFunction(listenerCallback))
                        {
                            _cometd._debug('Invoking listener exception callback', subscription, x);
                            try
                            {
                                listenerCallback.call(_cometd, x, subscription.handle, subscription.listener, message);
                            }
                            catch (xx)
                            {
                                _cometd._info('Exception during execution of listener callback', subscription, xx);
                            }
                        }
                    }
                }
            }
        }
    }

    function _notifyListeners(channel, message)
    {
        // Notify direct listeners
        _notify(channel, message);

        // Notify the globbing listeners
        var channelParts = channel.split('/');
        var last = channelParts.length - 1;
        for (var i = last; i > 0; --i)
        {
            var channelPart = channelParts.slice(0, i).join('/') + '/*';
            // We don't want to notify /foo/* if the channel is /foo/bar/baz,
            // so we stop at the first non recursive globbing
            if (i === last)
            {
                _notify(channelPart, message);
            }
            // Add the recursive globber and notify
            channelPart += '*';
            _notify(channelPart, message);
        }
    }

    function _cancelDelayedSend()
    {
        if (_scheduledSend !== null)
        {
            org.cometd.Utils.clearTimeout(_scheduledSend);
        }
        _scheduledSend = null;
    }

    function _delayedSend(operation)
    {
        _cancelDelayedSend();
        var delay = _advice.interval + _backoff;
        _cometd._debug('Function scheduled in', delay, 'ms, interval =', _advice.interval, 'backoff =', _backoff, operation);
        _scheduledSend = org.cometd.Utils.setTimeout(_cometd, operation, delay);
    }

    // Needed to break cyclic dependencies between function definitions
    var _handleMessages;
    var _handleFailure;

    /**
     * Delivers the messages to the CometD server
     * @param messages the array of messages to send
     * @param longpoll true if this send is a long poll
     */
    function _send(sync, messages, longpoll, extraPath)
    {
        // We must be sure that the messages have a clientId.
        // This is not guaranteed since the handshake may take time to return
        // (and hence the clientId is not known yet) and the application
        // may create other messages.
        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];
            message.id = '' + _nextMessageId();
            if (_clientId)
            {
                message.clientId = _clientId;
            }
            message = _applyOutgoingExtensions(message);
            if (message !== undefined && message !== null)
            {
                messages[i] = message;
            }
            else
            {
                messages.splice(i--, 1);
            }
        }
        if (messages.length === 0)
        {
            return;
        }

        var url = _config.url;
        if (_config.appendMessageTypeToURL)
        {
            // If url does not end with '/', then append it
            if (!url.match(/\/$/))
            {
                url = url + '/';
            }
            if (extraPath)
            {
                url = url + extraPath;
            }
        }

        var envelope = {
            url: url,
            sync: sync,
            messages: messages,
            onSuccess: function(rcvdMessages)
            {
                try
                {
                    _handleMessages.call(_cometd, rcvdMessages);
                }
                catch (x)
                {
                    _cometd._debug('Exception during handling of messages', x);
                }
            },
            onFailure: function(conduit, messages, reason, exception)
            {
                try
                {
                    _handleFailure.call(_cometd, conduit, messages, reason, exception);
                }
                catch (x)
                {
                    _cometd._debug('Exception during handling of failure', x);
                }
            }
        };
        _cometd._debug('Send', envelope);
        _transport.send(envelope, longpoll);
    }

    function _queueSend(message)
    {
        if (_batch > 0 || _internalBatch === true)
        {
            _messageQueue.push(message);
        }
        else
        {
            _send(false, [message], false);
        }
    }

    /**
     * Sends a complete bayeux message.
     * This method is exposed as a public so that extensions may use it
     * to send bayeux message directly, for example in case of re-sending
     * messages that have already been sent but that for some reason must
     * be resent.
     */
    this.send = _queueSend;

    function _resetBackoff()
    {
        _backoff = 0;
    }

    function _increaseBackoff()
    {
        if (_backoff < _config.maxBackoff)
        {
            _backoff += _config.backoffIncrement;
        }
    }

    /**
     * Starts a the batch of messages to be sent in a single request.
     * @see #_endBatch(sendMessages)
     */
    function _startBatch()
    {
        ++_batch;
    }

    function _flushBatch()
    {
        var messages = _messageQueue;
        _messageQueue = [];
        if (messages.length > 0)
        {
            _send(false, messages, false);
        }
    }

    /**
     * Ends the batch of messages to be sent in a single request,
     * optionally sending messages present in the message queue depending
     * on the given argument.
     * @see #_startBatch()
     */
    function _endBatch()
    {
        --_batch;
        if (_batch < 0)
        {
            throw 'Calls to startBatch() and endBatch() are not paired';
        }

        if (_batch === 0 && !_isDisconnected() && !_internalBatch)
        {
            _flushBatch();
        }
    }

    /**
     * Sends the connect message
     */
    function _connect()
    {
        if (!_isDisconnected())
        {
            var message = {
                channel: '/meta/connect',
                connectionType: _transport.getType()
            };

            // In case of reload or temporary loss of connection
            // we want the next successful connect to return immediately
            // instead of being held by the server, so that connect listeners
            // can be notified that the connection has been re-established
            if (!_connected)
            {
                message.advice = { timeout: 0 };
            }

            _setStatus('connecting');
            _cometd._debug('Connect sent', message);
            _send(false, [message], true, 'connect');
            _setStatus('connected');
        }
    }

    function _delayedConnect()
    {
        _setStatus('connecting');
        _delayedSend(function()
        {
            _connect();
        });
    }

    function _updateAdvice(newAdvice)
    {
        if (newAdvice)
        {
            _advice = _cometd._mixin(false, {}, _config.advice, newAdvice);
            _cometd._debug('New advice', _advice);
        }
    }

    function _disconnect(abort)
    {
        _cancelDelayedSend();
        if (abort)
        {
            _transport.abort();
        }
        _clientId = null;
        _setStatus('disconnected');
        _batch = 0;
        _resetBackoff();

        // Fail any existing queued message
        if (_messageQueue.length > 0)
        {
            _handleFailure.call(_cometd, undefined, _messageQueue, 'error', 'Disconnected');
            _messageQueue = [];
        }
    }

    /**
     * Sends the initial handshake message
     */
    function _handshake(handshakeProps)
    {
        _clientId = null;

        _clearSubscriptions();

        // Reset the transports if we're not retrying the handshake
        if (_isDisconnected())
        {
            _transports.reset();
            _updateAdvice(_config.advice);
        }
        else
        {
            // We are retrying the handshake, either because another handshake failed
            // and we're backing off, or because the server timed us out and asks us to
            // re-handshake: in both cases, make sure that if the handshake succeeds
            // the next action is a connect.
            _updateAdvice(_cometd._mixin(false, _advice, {reconnect: 'retry'}));
        }

        _batch = 0;

        // Mark the start of an internal batch.
        // This is needed because handshake and connect are async.
        // It may happen that the application calls init() then subscribe()
        // and the subscribe message is sent before the connect message, if
        // the subscribe message is not held until the connect message is sent.
        // So here we start a batch to hold temporarily any message until
        // the connection is fully established.
        _internalBatch = true;

        // Save the properties provided by the user, so that
        // we can reuse them during automatic re-handshake
        _handshakeProps = handshakeProps;

        var version = '1.0';

        // Figure out the transports to send to the server
        var transportTypes = _transports.findTransportTypes(version, _crossDomain, _config.url);

        var bayeuxMessage = {
            version: version,
            minimumVersion: '0.9',
            channel: '/meta/handshake',
            supportedConnectionTypes: transportTypes,
            advice: {
                timeout: _advice.timeout,
                interval: _advice.interval
            }
        };
        // Do not allow the user to mess with the required properties,
        // so merge first the user properties and *then* the bayeux message
        var message = _cometd._mixin(false, {}, _handshakeProps, bayeuxMessage);

        // Pick up the first available transport as initial transport
        // since we don't know if the server supports it
        _transport = _transports.negotiateTransport(transportTypes, version, _crossDomain, _config.url);
        _cometd._debug('Initial transport is', _transport.getType());

        // We started a batch to hold the application messages,
        // so here we must bypass it and send immediately.
        _setStatus('handshaking');
        _cometd._debug('Handshake sent', message);
        _send(false, [message], false, 'handshake');
    }

    function _delayedHandshake()
    {
        _setStatus('handshaking');

        // We will call _handshake() which will reset _clientId, but we want to avoid
        // that between the end of this method and the call to _handshake() someone may
        // call publish() (or other methods that call _queueSend()).
        _internalBatch = true;

        _delayedSend(function()
        {
            _handshake(_handshakeProps);
        });
    }

    function _failHandshake(message)
    {
        _notifyListeners('/meta/handshake', message);
        _notifyListeners('/meta/unsuccessful', message);

        // Only try again if we haven't been disconnected and
        // the advice permits us to retry the handshake
        var retry = !_isDisconnected() && _advice.reconnect !== 'none';
        if (retry)
        {
            _increaseBackoff();
            _delayedHandshake();
        }
        else
        {
            _disconnect(false);
        }
    }

    function _handshakeResponse(message)
    {
        if (message.successful)
        {
            // Save clientId, figure out transport, then follow the advice to connect
            _clientId = message.clientId;

            var newTransport = _transports.negotiateTransport(message.supportedConnectionTypes, message.version, _crossDomain, _config.url);
            if (newTransport === null)
            {
                throw 'Could not negotiate transport with server; client ' +
                      _transports.findTransportTypes(message.version, _crossDomain, _config.url) +
                      ', server ' + message.supportedConnectionTypes;
            }
            else if (_transport !== newTransport)
            {
                _cometd._debug('Transport', _transport, '->', newTransport);
                _transport = newTransport;
            }

            // End the internal batch and allow held messages from the application
            // to go to the server (see _handshake() where we start the internal batch).
            _internalBatch = false;
            _flushBatch();

            // Here the new transport is in place, as well as the clientId, so
            // the listeners can perform a publish() if they want.
            // Notify the listeners before the connect below.
            message.reestablish = _reestablish;
            _reestablish = true;
            _notifyListeners('/meta/handshake', message);

            var action = _isDisconnected() ? 'none' : _advice.reconnect;
            switch (action)
            {
                case 'retry':
                    _resetBackoff();
                    _delayedConnect();
                    break;
                case 'none':
                    _disconnect(false);
                    break;
                default:
                    throw 'Unrecognized advice action ' + action;
            }
        }
        else
        {
            _failHandshake(message);
        }
    }

    function _handshakeFailure(xhr, message)
    {
        _failHandshake({
            successful: false,
            failure: true,
            channel: '/meta/handshake',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'retry',
                interval: _backoff
            }
        });
    }

    function _failConnect(message)
    {
        // Notify the listeners after the status change but before the next action
        _notifyListeners('/meta/connect', message);
        _notifyListeners('/meta/unsuccessful', message);

        // This may happen when the server crashed, the current clientId
        // will be invalid, and the server will ask to handshake again
        // Listeners can call disconnect(), so check the state after they run
        var action = _isDisconnected() ? 'none' : _advice.reconnect;
        switch (action)
        {
            case 'retry':
                _delayedConnect();
                _increaseBackoff();
                break;
            case 'handshake':
                // The current transport may be failed (e.g. network disconnection)
                // Reset the transports so the new handshake picks up the right one
                _transports.reset();
                _resetBackoff();
                _delayedHandshake();
                break;
            case 'none':
                _disconnect(false);
                break;
            default:
                throw 'Unrecognized advice action' + action;
        }
    }

    function _connectResponse(message)
    {
        _connected = message.successful;

        if (_connected)
        {
            _notifyListeners('/meta/connect', message);

            // Normally, the advice will say "reconnect: 'retry', interval: 0"
            // and the server will hold the request, so when a response returns
            // we immediately call the server again (long polling)
            // Listeners can call disconnect(), so check the state after they run
            var action = _isDisconnected() ? 'none' : _advice.reconnect;
            switch (action)
            {
                case 'retry':
                    _resetBackoff();
                    _delayedConnect();
                    break;
                case 'none':
                    _disconnect(false);
                    break;
                default:
                    throw 'Unrecognized advice action ' + action;
            }
        }
        else
        {
            _failConnect(message);
        }
    }

    function _connectFailure(xhr, message)
    {
        _connected = false;
        _failConnect({
            successful: false,
            failure: true,
            channel: '/meta/connect',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'retry',
                interval: _backoff
            }
        });
    }

    function _failDisconnect(message)
    {
        _disconnect(true);
        _notifyListeners('/meta/disconnect', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _disconnectResponse(message)
    {
        if (message.successful)
        {
            _disconnect(false);
            _notifyListeners('/meta/disconnect', message);
        }
        else
        {
            _failDisconnect(message);
        }
    }

    function _disconnectFailure(xhr, message)
    {
        _failDisconnect({
            successful: false,
            failure: true,
            channel: '/meta/disconnect',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failSubscribe(message)
    {
        _notifyListeners('/meta/subscribe', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _subscribeResponse(message)
    {
        if (message.successful)
        {
            _notifyListeners('/meta/subscribe', message);
        }
        else
        {
            _failSubscribe(message);
        }
    }

    function _subscribeFailure(xhr, message)
    {
        _failSubscribe({
            successful: false,
            failure: true,
            channel: '/meta/subscribe',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failUnsubscribe(message)
    {
        _notifyListeners('/meta/unsubscribe', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _unsubscribeResponse(message)
    {
        if (message.successful)
        {
            _notifyListeners('/meta/unsubscribe', message);
        }
        else
        {
            _failUnsubscribe(message);
        }
    }

    function _unsubscribeFailure(xhr, message)
    {
        _failUnsubscribe({
            successful: false,
            failure: true,
            channel: '/meta/unsubscribe',
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _failMessage(message)
    {
        _notifyListeners('/meta/publish', message);
        _notifyListeners('/meta/unsuccessful', message);
    }

    function _messageResponse(message)
    {
        if (message.successful === undefined)
        {
            if (message.data)
            {
                // It is a plain message, and not a bayeux meta message
                _notifyListeners(message.channel, message);
            }
            else
            {
                _cometd._debug('Unknown message', message);
            }
        }
        else
        {
            if (message.successful)
            {
                _notifyListeners('/meta/publish', message);
            }
            else
            {
                _failMessage(message);
            }
        }
    }

    function _messageFailure(xhr, message)
    {
        _failMessage({
            successful: false,
            failure: true,
            channel: message.channel,
            request: message,
            xhr: xhr,
            advice: {
                reconnect: 'none',
                interval: 0
            }
        });
    }

    function _receive(message)
    {
        message = _applyIncomingExtensions(message);
        if (message === undefined || message === null)
        {
            return;
        }

        _updateAdvice(message.advice);

        var channel = message.channel;
        switch (channel)
        {
            case '/meta/handshake':
                _handshakeResponse(message);
                break;
            case '/meta/connect':
                _connectResponse(message);
                break;
            case '/meta/disconnect':
                _disconnectResponse(message);
                break;
            case '/meta/subscribe':
                _subscribeResponse(message);
                break;
            case '/meta/unsubscribe':
                _unsubscribeResponse(message);
                break;
            default:
                _messageResponse(message);
                break;
        }
    }

    /**
     * Receives a message.
     * This method is exposed as a public so that extensions may inject
     * messages simulating that they had been received.
     */
    this.receive = _receive;

    _handleMessages = function(rcvdMessages)
    {
        _cometd._debug('Received', rcvdMessages);

        for (var i = 0; i < rcvdMessages.length; ++i)
        {
            var message = rcvdMessages[i];
            _receive(message);
        }
    };

    _handleFailure = function(conduit, messages, reason, exception)
    {
        _cometd._debug('handleFailure', conduit, messages, reason, exception);

        for (var i = 0; i < messages.length; ++i)
        {
            var message = messages[i];
            var channel = message.channel;
            switch (channel)
            {
                case '/meta/handshake':
                    _handshakeFailure(conduit, message);
                    break;
                case '/meta/connect':
                    _connectFailure(conduit, message);
                    break;
                case '/meta/disconnect':
                    _disconnectFailure(conduit, message);
                    break;
                case '/meta/subscribe':
                    _subscribeFailure(conduit, message);
                    break;
                case '/meta/unsubscribe':
                    _unsubscribeFailure(conduit, message);
                    break;
                default:
                    _messageFailure(conduit, message);
                    break;
            }
        }
    };

    function _hasSubscriptions(channel)
    {
        var subscriptions = _listeners[channel];
        if (subscriptions)
        {
            for (var i = 0; i < subscriptions.length; ++i)
            {
                if (subscriptions[i])
                {
                    return true;
                }
            }
        }
        return false;
    }

    function _resolveScopedCallback(scope, callback)
    {
        var delegate = {
            scope: scope,
            method: callback
        };
        if (_isFunction(scope))
        {
            delegate.scope = undefined;
            delegate.method = scope;
        }
        else
        {
            if (_isString(callback))
            {
                if (!scope)
                {
                    throw 'Invalid scope ' + scope;
                }
                delegate.method = scope[callback];
                if (!_isFunction(delegate.method))
                {
                    throw 'Invalid callback ' + callback + ' for scope ' + scope;
                }
            }
            else if (!_isFunction(callback))
            {
                throw 'Invalid callback ' + callback;
            }
        }
        return delegate;
    }

    function _addListener(channel, scope, callback, isListener)
    {
        // The data structure is a map<channel, subscription[]>, where each subscription
        // holds the callback to be called and its scope.

        var delegate = _resolveScopedCallback(scope, callback);
        _cometd._debug('Adding listener on', channel, 'with scope', delegate.scope, 'and callback', delegate.method);

        var subscription = {
            channel: channel,
            scope: delegate.scope,
            callback: delegate.method,
            listener: isListener
        };

        var subscriptions = _listeners[channel];
        if (!subscriptions)
        {
            subscriptions = [];
            _listeners[channel] = subscriptions;
        }

        // Pushing onto an array appends at the end and returns the id associated with the element increased by 1.
        // Note that if:
        // a.push('a'); var hb=a.push('b'); delete a[hb-1]; var hc=a.push('c');
        // then:
        // hc==3, a.join()=='a',,'c', a.length==3
        var subscriptionID = subscriptions.push(subscription) - 1;
        subscription.id = subscriptionID;
        subscription.handle = [channel, subscriptionID];

        _cometd._debug('Added listener', subscription, 'for channel', channel, 'having id =', subscriptionID);

        // The subscription to allow removal of the listener is made of the channel and the index
        return subscription.handle;
    }

    function _removeListener(subscription)
    {
        var subscriptions = _listeners[subscription[0]];
        if (subscriptions)
        {
            delete subscriptions[subscription[1]];
            _cometd._debug('Removed listener', subscription);
        }
    }

    //
    // PUBLIC API
    //

    /**
     * Registers the given transport under the given transport type.
     * The optional index parameter specifies the "priority" at which the
     * transport is registered (where 0 is the max priority).
     * If a transport with the same type is already registered, this function
     * does nothing and returns false.
     * @param type the transport type
     * @param transport the transport object
     * @param index the index at which this transport is to be registered
     * @return true if the transport has been registered, false otherwise
     * @see #unregisterTransport(type)
     */
    this.registerTransport = function(type, transport, index)
    {
        var result = _transports.add(type, transport, index);
        if (result)
        {
            this._debug('Registered transport', type);

            if (_isFunction(transport.registered))
            {
                transport.registered(type, this);
            }
        }
        return result;
    };

    /**
     * @return an array of all registered transport types
     */
    this.getTransportTypes = function()
    {
        return _transports.getTransportTypes();
    };

    /**
     * Unregisters the transport with the given transport type.
     * @param type the transport type to unregister
     * @return the transport that has been unregistered,
     * or null if no transport was previously registered under the given transport type
     */
    this.unregisterTransport = function(type)
    {
        var transport = _transports.remove(type);
        if (transport !== null)
        {
            this._debug('Unregistered transport', type);

            if (_isFunction(transport.unregistered))
            {
                transport.unregistered();
            }
        }
        return transport;
    };

    this.unregisterTransports = function()
    {
        _transports.clear();
    };

    this.findTransport = function(name)
    {
        return _transports.find(name);
    };

    /**
     * Configures the initial Bayeux communication with the Bayeux server.
     * Configuration is passed via an object that must contain a mandatory field <code>url</code>
     * of type string containing the URL of the Bayeux server.
     * @param configuration the configuration object
     */
    this.configure = function(configuration)
    {
        _configure.call(this, configuration);
    };

    /**
     * Configures and establishes the Bayeux communication with the Bayeux server
     * via a handshake and a subsequent connect.
     * @param configuration the configuration object
     * @param handshakeProps an object to be merged with the handshake message
     * @see #configure(configuration)
     * @see #handshake(handshakeProps)
     */
    this.init = function(configuration, handshakeProps)
    {
        this.configure(configuration);
        this.handshake(handshakeProps);
    };

    /**
     * Establishes the Bayeux communication with the Bayeux server
     * via a handshake and a subsequent connect.
     * @param handshakeProps an object to be merged with the handshake message
     */
    this.handshake = function(handshakeProps)
    {
        _setStatus('disconnected');
        _reestablish = false;
        _handshake(handshakeProps);
    };

    /**
     * Disconnects from the Bayeux server.
     * It is possible to suggest to attempt a synchronous disconnect, but this feature
     * may only be available in certain transports (for example, long-polling may support
     * it, callback-polling certainly does not).
     * @param sync whether attempt to perform a synchronous disconnect
     * @param disconnectProps an object to be merged with the disconnect message
     */
    this.disconnect = function(sync, disconnectProps)
    {
        if (_isDisconnected())
        {
            return;
        }

        if (disconnectProps === undefined)
        {
            if (typeof sync !== 'boolean')
            {
                disconnectProps = sync;
                sync = false;
            }
        }

        var bayeuxMessage = {
            channel: '/meta/disconnect'
        };
        var message = this._mixin(false, {}, disconnectProps, bayeuxMessage);
        _setStatus('disconnecting');
        _send(sync === true, [message], false, 'disconnect');
    };

    /**
     * Marks the start of a batch of application messages to be sent to the server
     * in a single request, obtaining a single response containing (possibly) many
     * application reply messages.
     * Messages are held in a queue and not sent until {@link #endBatch()} is called.
     * If startBatch() is called multiple times, then an equal number of endBatch()
     * calls must be made to close and send the batch of messages.
     * @see #endBatch()
     */
    this.startBatch = function()
    {
        _startBatch();
    };

    /**
     * Marks the end of a batch of application messages to be sent to the server
     * in a single request.
     * @see #startBatch()
     */
    this.endBatch = function()
    {
        _endBatch();
    };

    /**
     * Executes the given callback in the given scope, surrounded by a {@link #startBatch()}
     * and {@link #endBatch()} calls.
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to be executed within {@link #startBatch()} and {@link #endBatch()} calls
     */
    this.batch = function(scope, callback)
    {
        var delegate = _resolveScopedCallback(scope, callback);
        this.startBatch();
        try
        {
            delegate.method.call(delegate.scope);
            this.endBatch();
        }
        catch (x)
        {
            this._debug('Exception during execution of batch', x);
            this.endBatch();
            throw x;
        }
    };

    /**
     * Adds a listener for bayeux messages, performing the given callback in the given scope
     * when a message for the given channel arrives.
     * @param channel the channel the listener is interested to
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to call when a message is sent to the channel
     * @returns the subscription handle to be passed to {@link #removeListener(object)}
     * @see #removeListener(subscription)
     */
    this.addListener = function(channel, scope, callback)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }

        return _addListener(channel, scope, callback, true);
    };

    /**
     * Removes the subscription obtained with a call to {@link #addListener(string, object, function)}.
     * @param subscription the subscription to unsubscribe.
     * @see #addListener(channel, scope, callback)
     */
    this.removeListener = function(subscription)
    {
        if (!org.cometd.Utils.isArray(subscription))
        {
            throw 'Invalid argument: expected subscription, not ' + subscription;
        }

        _removeListener(subscription);
    };

    /**
     * Removes all listeners registered with {@link #addListener(channel, scope, callback)} or
     * {@link #subscribe(channel, scope, callback)}.
     */
    this.clearListeners = function()
    {
        _listeners = {};
    };

    /**
     * Subscribes to the given channel, performing the given callback in the given scope
     * when a message for the channel arrives.
     * @param channel the channel to subscribe to
     * @param scope the scope of the callback, may be omitted
     * @param callback the callback to call when a message is sent to the channel
     * @param subscribeProps an object to be merged with the subscribe message
     * @return the subscription handle to be passed to {@link #unsubscribe(object)}
     */
    this.subscribe = function(channel, scope, callback, subscribeProps)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        // Normalize arguments
        if (_isFunction(scope))
        {
            subscribeProps = callback;
            callback = scope;
            scope = undefined;
        }

        // Only send the message to the server if this client has not yet subscribed to the channel
        var send = !_hasSubscriptions(channel);

        var subscription = _addListener(channel, scope, callback, false);

        if (send)
        {
            // Send the subscription message after the subscription registration to avoid
            // races where the server would send a message to the subscribers, but here
            // on the client the subscription has not been added yet to the data structures
            var bayeuxMessage = {
                channel: '/meta/subscribe',
                subscription: channel
            };
            var message = this._mixin(false, {}, subscribeProps, bayeuxMessage);
            _queueSend(message);
        }

        return subscription;
    };

    /**
     * Unsubscribes the subscription obtained with a call to {@link #subscribe(string, object, function)}.
     * @param subscription the subscription to unsubscribe.
     */
    this.unsubscribe = function(subscription, unsubscribeProps)
    {
        if (arguments.length < 1)
        {
            throw 'Illegal arguments number: required 1, got ' + arguments.length;
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        // Remove the local listener before sending the message
        // This ensures that if the server fails, this client does not get notifications
        this.removeListener(subscription);

        var channel = subscription[0];
        // Only send the message to the server if this client unsubscribes the last subscription
        if (!_hasSubscriptions(channel))
        {
            var bayeuxMessage = {
                channel: '/meta/unsubscribe',
                subscription: channel
            };
            var message = this._mixin(false, {}, unsubscribeProps, bayeuxMessage);
            _queueSend(message);
        }
    };

    /**
     * Removes all subscriptions added via {@link #subscribe(channel, scope, callback, subscribeProps)},
     * but does not remove the listeners added via {@link addListener(channel, scope, callback)}.
     */
    this.clearSubscriptions = function()
    {
        _clearSubscriptions();
    };

    /**
     * Publishes a message on the given channel, containing the given content.
     * @param channel the channel to publish the message to
     * @param content the content of the message
     * @param publishProps an object to be merged with the publish message
     */
    this.publish = function(channel, content, publishProps)
    {
        if (arguments.length < 1)
        {
            throw 'Illegal arguments number: required 1, got ' + arguments.length;
        }
        if (!_isString(channel))
        {
            throw 'Illegal argument type: channel must be a string';
        }
        if (_isDisconnected())
        {
            throw 'Illegal state: already disconnected';
        }

        var bayeuxMessage = {
            channel: channel,
            data: content
        };
        var message = this._mixin(false, {}, publishProps, bayeuxMessage);
        _queueSend(message);
    };

    /**
     * Returns a string representing the status of the bayeux communication with the Bayeux server.
     */
    this.getStatus = function()
    {
        return _status;
    };

    /**
     * Returns whether this instance has been disconnected.
     */
    this.isDisconnected = _isDisconnected;

    /**
     * Sets the backoff period used to increase the backoff time when retrying an unsuccessful or failed message.
     * Default value is 1 second, which means if there is a persistent failure the retries will happen
     * after 1 second, then after 2 seconds, then after 3 seconds, etc. So for example with 15 seconds of
     * elapsed time, there will be 5 retries (at 1, 3, 6, 10 and 15 seconds elapsed).
     * @param period the backoff period to set
     * @see #getBackoffIncrement()
     */
    this.setBackoffIncrement = function(period)
    {
        _config.backoffIncrement = period;
    };

    /**
     * Returns the backoff period used to increase the backoff time when retrying an unsuccessful or failed message.
     * @see #setBackoffIncrement(period)
     */
    this.getBackoffIncrement = function()
    {
        return _config.backoffIncrement;
    };

    /**
     * Returns the backoff period to wait before retrying an unsuccessful or failed message.
     */
    this.getBackoffPeriod = function()
    {
        return _backoff;
    };

    /**
     * Sets the log level for console logging.
     * Valid values are the strings 'error', 'warn', 'info' and 'debug', from
     * less verbose to more verbose.
     * @param level the log level string
     */
    this.setLogLevel = function(level)
    {
        _config.logLevel = level;
    };

    /**
     * Registers an extension whose callbacks are called for every incoming message
     * (that comes from the server to this client implementation) and for every
     * outgoing message (that originates from this client implementation for the
     * server).
     * The format of the extension object is the following:
     * <pre>
     * {
     *     incoming: function(message) { ... },
     *     outgoing: function(message) { ... }
     * }
     * </pre>
     * Both properties are optional, but if they are present they will be called
     * respectively for each incoming message and for each outgoing message.
     * @param name the name of the extension
     * @param extension the extension to register
     * @return true if the extension was registered, false otherwise
     * @see #unregisterExtension(name)
     */
    this.registerExtension = function(name, extension)
    {
        if (arguments.length < 2)
        {
            throw 'Illegal arguments number: required 2, got ' + arguments.length;
        }
        if (!_isString(name))
        {
            throw 'Illegal argument type: extension name must be a string';
        }

        var existing = false;
        for (var i = 0; i < _extensions.length; ++i)
        {
            var existingExtension = _extensions[i];
            if (existingExtension.name === name)
            {
                existing = true;
                break;
            }
        }
        if (!existing)
        {
            _extensions.push({
                name: name,
                extension: extension
            });
            this._debug('Registered extension', name);

            // Callback for extensions
            if (_isFunction(extension.registered))
            {
                extension.registered(name, this);
            }

            return true;
        }
        else
        {
            this._info('Could not register extension with name', name, 'since another extension with the same name already exists');
            return false;
        }
    };

    /**
     * Unregister an extension previously registered with
     * {@link #registerExtension(name, extension)}.
     * @param name the name of the extension to unregister.
     * @return true if the extension was unregistered, false otherwise
     */
    this.unregisterExtension = function(name)
    {
        if (!_isString(name))
        {
            throw 'Illegal argument type: extension name must be a string';
        }

        var unregistered = false;
        for (var i = 0; i < _extensions.length; ++i)
        {
            var extension = _extensions[i];
            if (extension.name === name)
            {
                _extensions.splice(i, 1);
                unregistered = true;
                this._debug('Unregistered extension', name);

                // Callback for extensions
                var ext = extension.extension;
                if (_isFunction(ext.unregistered))
                {
                    ext.unregistered();
                }

                break;
            }
        }
        return unregistered;
    };

    /**
     * Find the extension registered with the given name.
     * @param name the name of the extension to find
     * @return the extension found or null if no extension with the given name has been registered
     */
    this.getExtension = function(name)
    {
        for (var i = 0; i < _extensions.length; ++i)
        {
            var extension = _extensions[i];
            if (extension.name === name)
            {
                return extension.extension;
            }
        }
        return null;
    };

    /**
     * Returns the name assigned to this Cometd object, or the string 'default'
     * if no name has been explicitly passed as parameter to the constructor.
     */
    this.getName = function()
    {
        return _name;
    };

    /**
     * Returns the clientId assigned by the Bayeux server during handshake.
     */
    this.getClientId = function()
    {
        return _clientId;
    };

    /**
     * Returns the URL of the Bayeux server.
     */
    this.getURL = function()
    {
        return _config.url;
    };

    this.getTransport = function()
    {
        return _transport;
    };

    this.getConfiguration = function()
    {
        return this._mixin(true, {}, _config);
    };

    this.getAdvice = function()
    {
        return this._mixin(true, {}, _advice);
    };

    // WebSocket handling for Firefox, which deploys WebSocket
    // under the name of MozWebSocket in Firefox 6, 7, 8 and 9
    org.cometd.WebSocket = window.WebSocket;
    if (!org.cometd.WebSocket)
    {
        org.cometd.WebSocket = window.MozWebSocket;
    }
};


}

if(!dojo._hasResource["dojo.io.script"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.io.script"] = true;
dojo.provide("dojo.io.script");

dojo.getObject("io", true, dojo);

/*=====
dojo.declare("dojo.io.script.__ioArgs", dojo.__IoArgs, {
	constructor: function(){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply to this
		//		type as well, EXCEPT "handleAs". It is not applicable to
		//		dojo.io.script.get() calls, since it is implied by the usage of
		//		"jsonp" (response will be a JSONP call returning JSON)
		//		or the response is pure JavaScript defined in
		//		the body of the script that was attached.
		//	callbackParamName: String
		//		Deprecated as of Dojo 1.4 in favor of "jsonp", but still supported for
		// 		legacy code. See notes for jsonp property.
		//	jsonp: String
		//		The URL parameter name that indicates the JSONP callback string.
		//		For instance, when using Yahoo JSONP calls it is normally,
		//		jsonp: "callback". For AOL JSONP calls it is normally
		//		jsonp: "c".
		//	checkString: String
		//		A string of JavaScript that when evaluated like so:
		//		"typeof(" + checkString + ") != 'undefined'"
		//		being true means that the script fetched has been loaded.
		//		Do not use this if doing a JSONP type of call (use callbackParamName instead).
		//	frameDoc: Document
		//		The Document object for a child iframe. If this is passed in, the script
		//		will be attached to that document. This can be helpful in some comet long-polling
		//		scenarios with Firefox and Opera.
		this.callbackParamName = callbackParamName;
		this.jsonp = jsonp;
		this.checkString = checkString;
		this.frameDoc = frameDoc;
	}
});
=====*/
(function(){
	var loadEvent = dojo.isIE ? "onreadystatechange" : "load",
		readyRegExp = /complete|loaded/;

	dojo.io.script = {
		get: function(/*dojo.io.script.__ioArgs*/args){
			//	summary:
			//		sends a get request using a dynamically created script tag.
			var dfd = this._makeScriptDeferred(args);
			var ioArgs = dfd.ioArgs;
			dojo._ioAddQueryToUrl(ioArgs);
	
			dojo._ioNotifyStart(dfd);

			if(this._canAttach(ioArgs)){
				var node = this.attach(ioArgs.id, ioArgs.url, args.frameDoc);

				//If not a jsonp callback or a polling checkString case, bind
				//to load event on the script tag.
				if(!ioArgs.jsonp && !ioArgs.args.checkString){
					var handle = dojo.connect(node, loadEvent, function(evt){
						if(evt.type == "load" || readyRegExp.test(node.readyState)){
							dojo.disconnect(handle);
							ioArgs.scriptLoaded = evt;
						}
					});
				}
			}

			dojo._ioWatch(dfd, this._validCheck, this._ioCheck, this._resHandle);
			return dfd;
		},
	
		attach: function(/*String*/id, /*String*/url, /*Document?*/frameDocument){
			//	summary:
			//		creates a new <script> tag pointing to the specified URL and
			//		adds it to the document.
			//	description:
			//		Attaches the script element to the DOM.  Use this method if you
			//		just want to attach a script to the DOM and do not care when or
			//		if it loads.
			var doc = (frameDocument || dojo.doc);
			var element = doc.createElement("script");
			element.type = "text/javascript";
			element.src = url;
			element.id = id;
			element.charset = "utf-8";
			return doc.getElementsByTagName("head")[0].appendChild(element);
		},
	
		remove: function(/*String*/id, /*Document?*/frameDocument){
			//summary: removes the script element with the given id, from the given frameDocument.
			//If no frameDocument is passed, the current document is used.
			dojo.destroy(dojo.byId(id, frameDocument));
			
			//Remove the jsonp callback on dojo.io.script, if it exists.
			if(this["jsonp_" + id]){
				delete this["jsonp_" + id];
			}
		},
	
		_makeScriptDeferred: function(/*Object*/args){
			//summary:
			//		sets up a Deferred object for an IO request.
			var dfd = dojo._ioSetArgs(args, this._deferredCancel, this._deferredOk, this._deferredError);
	
			var ioArgs = dfd.ioArgs;
			ioArgs.id = dojo._scopeName + "IoScript" + (this._counter++);
			ioArgs.canDelete = false;
	
			//Special setup for jsonp case
			ioArgs.jsonp = args.callbackParamName || args.jsonp;
			if(ioArgs.jsonp){
				//Add the jsonp parameter.
				ioArgs.query = ioArgs.query || "";
				if(ioArgs.query.length > 0){
					ioArgs.query += "&";
				}
				ioArgs.query += ioArgs.jsonp
					+ "="
					+ (args.frameDoc ? "parent." : "")
					+ dojo._scopeName + ".io.script.jsonp_" + ioArgs.id + "._jsonpCallback";
	
				ioArgs.frameDoc = args.frameDoc;
	
				//Setup the Deferred to have the jsonp callback.
				ioArgs.canDelete = true;
				dfd._jsonpCallback = this._jsonpCallback;
				this["jsonp_" + ioArgs.id] = dfd;
			}
			return dfd; // dojo.Deferred
		},
		
		_deferredCancel: function(/*Deferred*/dfd){
			//summary: canceller function for dojo._ioSetArgs call.
	
			//DO NOT use "this" and expect it to be dojo.io.script.
			dfd.canceled = true;
			if(dfd.ioArgs.canDelete){
				dojo.io.script._addDeadScript(dfd.ioArgs);
			}
		},
	
		_deferredOk: function(/*Deferred*/dfd){
			//summary: okHandler function for dojo._ioSetArgs call.
	
			//DO NOT use "this" and expect it to be dojo.io.script.
			var ioArgs = dfd.ioArgs;
	
			//Add script to list of things that can be removed.
			if(ioArgs.canDelete){
				dojo.io.script._addDeadScript(ioArgs);
			}
	
			//Favor JSONP responses, script load events then lastly ioArgs.
			//The ioArgs are goofy, but cannot return the dfd since that stops
			//the callback chain in Deferred. The return value is not that important
			//in that case, probably a checkString case.
			return ioArgs.json || ioArgs.scriptLoaded || ioArgs;
		},
	
		_deferredError: function(/*Error*/error, /*Deferred*/dfd){
			//summary: errHandler function for dojo._ioSetArgs call.
	
			if(dfd.ioArgs.canDelete){
				//DO NOT use "this" and expect it to be dojo.io.script.
				if(error.dojoType == "timeout"){
					//For timeouts, remove the script element immediately to
					//avoid a response from it coming back later and causing trouble.
					dojo.io.script.remove(dfd.ioArgs.id, dfd.ioArgs.frameDoc);
				}else{
					dojo.io.script._addDeadScript(dfd.ioArgs);
				}
			}
			console.log("dojo.io.script error", error);
			return error;
		},
	
		_deadScripts: [],
		_counter: 1,
	
		_addDeadScript: function(/*Object*/ioArgs){
			//summary: sets up an entry in the deadScripts array.
			dojo.io.script._deadScripts.push({id: ioArgs.id, frameDoc: ioArgs.frameDoc});
			//Being extra paranoid about leaks:
			ioArgs.frameDoc = null;
		},
	
		_validCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if dfd is still valid.
	
			//Do script cleanup here. We wait for one inflight pass
			//to make sure we don't get any weird things by trying to remove a script
			//tag that is part of the call chain (IE 6 has been known to
			//crash in that case).
			var _self = dojo.io.script;
			var deadScripts = _self._deadScripts;
			if(deadScripts && deadScripts.length > 0){
				for(var i = 0; i < deadScripts.length; i++){
					//Remove the script tag
					_self.remove(deadScripts[i].id, deadScripts[i].frameDoc);
					deadScripts[i].frameDoc = null;
				}
				dojo.io.script._deadScripts = [];
			}
	
			return true;
		},
	
		_ioCheck: function(/*Deferred*/dfd){
			//summary: inflight check function to see if IO finished.
			var ioArgs = dfd.ioArgs;
			//Check for finished jsonp
			if(ioArgs.json || (ioArgs.scriptLoaded && !ioArgs.args.checkString)){
				return true;
			}
	
			//Check for finished "checkString" case.
			var checkString = ioArgs.args.checkString;
			if(checkString && eval("typeof(" + checkString + ") != 'undefined'")){
				return true;
			}
	
			return false;
		},
	
		_resHandle: function(/*Deferred*/dfd){
			//summary: inflight function to handle a completed response.
			if(dojo.io.script._ioCheck(dfd)){
				dfd.callback(dfd);
			}else{
				//This path should never happen since the only way we can get
				//to _resHandle is if _ioCheck is true.
				dfd.errback(new Error("inconceivable dojo.io.script._resHandle error"));
			}
		},
	
		_canAttach: function(/*Object*/ioArgs){
			//summary: A method that can be overridden by other modules
			//to control when the script attachment occurs.
			return true;
		},
		
		_jsonpCallback: function(/*JSON Object*/json){
			//summary:
			//		generic handler for jsonp callback. A pointer to this function
			//		is used for all jsonp callbacks.  NOTE: the "this" in this
			//		function will be the Deferred object that represents the script
			//		request.
			this.ioArgs.json = json;
		}
	};
})();

}

if(!dojo._hasResource['dojox.cometd']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.cometd'] = true;
/*
 * Copyright (c) 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

dojo.provide('dojox.cometd');
dojo.registerModulePath('org','../org');



// Remap cometd JSON functions to dojo JSON functions
org.cometd.JSON.toJSON = dojo.toJson;
org.cometd.JSON.fromJSON = dojo.fromJson;

dojox.Cometd = function(name)
{
    var cometd = new org.cometd.Cometd(name);

    function LongPollingTransport()
    {
        var _super = new org.cometd.LongPollingTransport();
        var that = org.cometd.Transport.derive(_super);

        that.xhrSend = function(packet)
        {
            var deferred = dojo.rawXhrPost({
                url: packet.url,
                sync: packet.sync === true,
                contentType: 'application/json;charset=UTF-8',
                headers: packet.headers,
                postData: packet.body,
                withCredentials: true,
                handleAs: 'json',
                load: packet.onSuccess,
                error: function(error)
                {
                    packet.onError(error.message, deferred ? deferred.ioArgs.error : error);
                }
            });
            return deferred.ioArgs.xhr;
        };

        return that;
    }

    function CallbackPollingTransport()
    {
        var _super = new org.cometd.CallbackPollingTransport();
        var that = org.cometd.Transport.derive(_super);

        that.jsonpSend = function(packet)
        {
            var deferred = dojo.io.script.get({
                url: packet.url,
                sync: packet.sync === true,
                callbackParamName: 'jsonp',
                content: {
                    // In callback-polling, the content must be sent via the 'message' parameter
                    message: packet.body
                },
                load: packet.onSuccess,
                error: function(error)
                {
                    packet.onError(error.message, deferred ? deferred.ioArgs.error : error);
                }
            });
            return undefined;
        };

        return that;
    }

    // Registration order is important
    if (org.cometd.WebSocket)
    {
        cometd.registerTransport('websocket', new org.cometd.WebSocketTransport());
    }
    cometd.registerTransport('long-polling', new LongPollingTransport());
    cometd.registerTransport('callback-polling', new CallbackPollingTransport());

    return cometd;
};

// The default cometd instance
dojox.cometd = new dojox.Cometd();

// Create a compatibility API for dojox.cometd instance with
// the original API.

dojox.cometd._init = dojox.cometd.init;

dojox.cometd._unsubscribe = dojox.cometd.unsubscribe;

dojox.cometd.unsubscribe = function(channelOrToken, objOrFunc, funcName)
{
    if (typeof channelOrToken === 'string')
    {
        throw "Deprecated function unsubscribe(string). Use unsubscribe(object) passing as argument the return value of subscribe()";
    }

    dojox.cometd._unsubscribe(channelOrToken);
};

dojox.cometd._metaHandshakeEvent = function(event)
{
    event.action = "handshake";
    dojo.publish("/cometd/meta", [event]);
};

dojox.cometd._metaConnectEvent = function(event)
{
    event.action = "connect";
    dojo.publish("/cometd/meta", [event]);
};

dojox.cometd.addListener('/meta/handshake', dojox.cometd, dojox.cometd._metaHandshakeEvent);
dojox.cometd.addListener('/meta/connect', dojox.cometd, dojox.cometd._metaConnectEvent);

}

if(!dojo._hasResource["com.ibm.bpm.social.notifications"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.notifications"] = true;
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.notifications");

var bpm_social_messages = {};

var connectedToBayeuxServer = false;
var logoutOngoing = false;
var currentUser;
window["BPMCometDConnectionEstablished"] = false;





function connectToBayeuxServer(){
 if (navigator.userAgent.match(/iPad/i) && dojo.isSafari &&  !navigator.userAgent.match(/criOS/i)) {// If Safari on iPad don't use CometD but allow if Chrome on iPad
    return
  }

  var FUNC_NAME = "connectToBayeuxServer";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, "connecting to  Bayeux server", "notifications", FUNC_NAME);
  window["collaborationContainerDiv"] = dojo.byId("processPortalCollaborationBanner");

  
  
  
  

  
  bpm_social_messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

  // configure bayeux server url and other client options
  var configObject = {
    url: window.endpointURLObject.processPortalNotificationJS +"/cometd"
  };

  //workaround for defect 78650
  if (com.ibm.bpm.social.util.common.Util.isIPad) {
    configObject.maxNetworkDelay = 1000000;
  }
  
  dojox.cometd.configure(configObject);

  // disable the websocket transport
  dojox.cometd.websocketEnabled = false;

  if (typeof OnlineTrackingController === "function") {
    // Controller for online tracking
    // Trigger onConnectionBroken 1 minute after connection is lost.
    new OnlineTrackingController({
      cometd: dojox.cometd,
      userFunction: function(){
        var user = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (user != null) {
          return user["userName"];
        }
        else {
          return false;
        }
      },
      logFunction: function(message, cName, fName){
        com.ibm.bpm.social.util.common.Log.log(2, message, cName ? cName : "", fName ? fName : "");
      },
      timeout: 30000,
      update: true
    /* make sure new status is requested upon handshake not taking into account the timeout */});
  }

  // add a listener to the /meta/connect channel to inform us of connection events
  dojox.cometd.addListener("/meta/connect", handleOnMetaConnectMessage);
  dojox.cometd.addListener("/meta/disconnect", handleOnMetadisconnectMessage);

  // connect to bayuex server
  dojox.cometd.handshake();

  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function disconnectFromBayeuxServer() {
  logoutOngoing = true;
  if (typeof OnlineTrackingController === "function" && typeof OnlineTrackingController.instance === "object") {
    OnlineTrackingController.instance.logout();
  }
  dojox.cometd.disconnect();  
};

if (window.disconnectFromBayeuxServer == undefined) {
	window.disconnectFromBayeuxServer = disconnectFromBayeuxServer;
}

function handleOnMetadisconnectMessage(message){
  var FUNC_NAME = "handleOnMetadisconnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (logoutOngoing) {
    com.ibm.bpm.social.util.common.Log.log(2, "logout ongoing => no reconnection attempt", "notifications", FUNC_NAME);
  } else {
    // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
    // com.ibm.bpm.social.util.common.Log.log(2,"Meta Connect disconnect Message: "+ message, "notifications", FUNC_NAME);
    window["cometDForCollab"] = null;
    connectedToBayeuxServer = false;

    
    if (typeof (dojo.cookie("com_ibm_bpm_bspace_logout")) == "undefined" &&
      typeof (dojo.cookie("com_ibm_bpm_portal_unload")) == "undefined") {
      // if we're not logging out, reconnect to bayuex server
      dojox.cometd.handshake();
    }
  }   
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};
function handleOnMetaConnectMessage(message){
  var FUNC_NAME = "handleOnMetaConnectMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // ASZ: removed message because it will just be displayed as "Meta Connect Message: [object Object]" which only fills the console with useless info:
  // com.ibm.bpm.social.util.common.Log.log(2, "Meta Connect Message: " + message, "notifications", FUNC_NAME);
  var wasPreviouslyConnected = connectedToBayeuxServer;
  connectedToBayeuxServer = (message.successful === true);
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage connectedToBayeuxServer:"+connectedToBayeuxServer, "notifications", "handleOnMetadisconnectMessage");
  //com.ibm.bpm.social.util.common.Log.log(2,"handleOnMetaConnectMessage wasPreviouslyConnected:"+wasPreviouslyConnected, "notifications", "handleOnMetadisconnectMessage");
  if (!wasPreviouslyConnected && connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Success", "notifications", FUNC_NAME);

    handleConnectionSucceeded(message);
  }
  else if (wasPreviouslyConnected && !connectedToBayeuxServer) {
    com.ibm.bpm.social.util.common.Log.log(2, "handleOnMetaConnectMessage Broken", "notifications", FUNC_NAME);
    handleConnectionBroken(message);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleConnectionSucceeded(message){
  var FUNC_NAME = "handleConnectionSucceeded";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = true;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, setting windows variable cometd", "notifications",
    "handleConnectionSucceeded");
  window["cometDForCollab"] = dojox.cometd;
  com.ibm.bpm.social.util.common.Log.log(2, "handleConnectionSucceeded, calling subscribe to Channels", "notifications",
    "handleConnectionSucceeded");
  subscribeToChannels();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function getUserInfo(){
  var FUNC_NAME = "getUserInfo";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var userData = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return userData;
};

function handleConnectionBroken(message){
  var FUNC_NAME = "handleConnectionBroken";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  window["BPMCometDConnectionEstablished"] = false;
  window["cometDForCollab"] = null;

  //Do a check here to see if the reason behind the connection break is a 401 Response.  If that is the case,
  //then redirect to the login page.
  var xhr = message.xhr;
  if (typeof xhr != "undefined") {
    var status = xhr.status;
    if (typeof status != "undefined" && status == 401) {
      //Perform REDIRECT to login page
      com.ibm.bpm.social.util.common.Util.redirectToLoginPage();
    }
  }
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleFailedRestConnection(message){
  var FUNC_NAME = "handleFailedRestConnection";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.log(2, message, "notifications", FUNC_NAME);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToChannels(){
  var FUNC_NAME = "subscribeToChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //console.group("subscribeToChannels");
  if (!currentUser) {
    var userData = getUserInfo();
    currentUser = userData;
  }
  //com.ibm.bpm.social.util.common.Log.log(2, "User Info: " + userData, "notifications", "subscribeToChannels");
  //com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), "notifications", "subscribeToChannels");
  dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getUserNotificationChannelName(userData.userName), handleNotificationMessage);
  var membershipsAll = userData.memberships;
  
  //remove duplicates
  var memberships = [];
  dojo.forEach(membershipsAll, function(item, i) {
    if(dojo.indexOf(memberships, item)  == -1) {
      memberships.push(item);
    }
  });
  
  if (memberships) {
    //batch the subscriptions so that we are not making multiple HTTP requests for each group subscription
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        // com.ibm.bpm.social.util.common.Log.log(2, "Subscribing to channel " + com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), "notifications", "subscribeToChannels");
        dojox.cometd.subscribe(com.ibm.bpm.social.notifications.getGroupNotificationChannelName(memberships[i]), handleNotificationMessage);
      }
    });
  }

  //
  // subscribe to the task list channels in order for automatic refresh of the task list
  subscribeToTaskListChannels(currentUser.userName, memberships);

  console.groupEnd();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function subscribeToTaskListChannels(userName, memberships){
  var FUNC_NAME = "subscribeToTaskListChannels";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // task list push events for user
  dojox.cometd.subscribe('/bpm/process_portal/task_list/user/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName), handleTaskNotificationMessage);

  // task list push events for groups
  if (memberships) {
    dojox.cometd.batch(function(){
      for ( var i = 0; i < memberships.length; i++) {
        dojox.cometd.subscribe('/bpm/process_portal/task_list/group/' + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(memberships[i]), handleTaskNotificationMessage);
      }
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    switch (data.kind) {
      case "TASK_RESOURCE_ASSIGNED" :
        handle_TASK_RESOURCE_ASSIGNED(data);
        break;
      case "TASK_COLLABORATION_INVITE" :
        handle_TASK_COLLABORATION_INVITE(data);
        break;
      case "PROCESS_COMMENT_TAGGED" :
        handle_PROCESS_COMMENT_TAGGED(data);
        break;
      default:
        break;
    }
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handleTaskNotificationMessage(incomingMessage){
  var FUNC_NAME = "handleTaskNotificationMessage";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var data = incomingMessage.data;
  com.ibm.bpm.social.util.common.Log.log(2, "Incoming Notification Message: " + data, "notifications", FUNC_NAME);
  if (data) {
    dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_LIST_UPDATED, [incomingMessage]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_RESOURCE_ASSIGNED(data){
  var FUNC_NAME = "handle_TASK_RESOURCE_ASSIGNED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (!data.task.resource || !data.task.id) {
    return;
  }

  // check if task notification should be ignored, e.g. claim for inline completion
  if (!com.ibm.bpm.social.notifications._isIgnoreAssignedNotification(data.task.id)) {
    var shouldDisplayPopup = true;
    var notificationPopUpText = "";

    switch (data.task.resource.type) {
      case "ROLE" :
        //notificationMessage = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask,{'groupname': data.task.resource.displayName});
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var displayName =
          com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.resource.displayName);
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.groupReceivedNewTask, {
          'groupname': displayName,
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To My Group'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      case "USER" :
        //notificationMessage = bpm_social_messages.userReceivedNewTask;
        var link_id = getUniqueID("launch_task_action");
        //apply the BTD to the notification message
        var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
        notificationPopUpText = dojo.string.substitute(bpm_social_messages.userReceivedNewTask, {
          'taskName': taskName,
          'linkID': link_id
        });

        var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        if (usr != null && usr.userPreferences != null &&
          usr.userPreferences['Portal Notification New Task To Me'] == "false") {
          shouldDisplayPopup = false;
        }
        break;
      default:
        break;
    }

    if (shouldDisplayPopup) {
      //notificationPopUpText = "<div>${0} <a id='"+link_id+"' href='#' dojoattachevent='onclick: _handleActionClick'>${1}</a></div>";
      var aetherParams = {};
      aetherParams.message = notificationPopUpText;
      aetherParams.containerDivId = "messageNotificationDiv";
      aetherParams.position = "top-right";
      aetherParams.fade = true;
      aetherParams.parent = this;
      aetherParams.displayCloseIcon = true;
      aetherParams.eventParam = data;
      var message = new com.ibm.bpm.social.Aether(aetherParams);
      message.fireEvent = dojo.hitch(this, fireEvent);
    }
  }
  else {
    com.ibm.bpm.social.util.common.Log.log(2, "ignoring TASK_RESOURCE_ASSIGNED for " + data.task.id, "notifications",
      FUNC_NAME);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_TASK_COLLABORATION_INVITE(data){
  var FUNC_NAME = "handle_TASK_COLLABORATION_INVITE";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.resource.id);
  var task_link_id = getUniqueID("launch_task_action");
  var process_link_id = getUniqueID("launch_process_action");
  //apply BTD to the notifications message
  var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.name);
  var actor = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.resource.displayName);
  var processName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
  var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.task.collaboration.message);

  var notificationPopUpText = dojo.string.substitute(bpm_social_messages.collaborationInviteMessage, {
    'taskName': taskName,
    'taskLinkID': task_link_id,
    'processLinkID': process_link_id,
    'actor': actor,
    'processName': processName,
    'messageBody': com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.task.collaboration.message)
  });
  var aetherParams = {};
  aetherParams.message = notificationPopUpText;
  aetherParams.containerDivId = "messageNotificationDiv";
  aetherParams.position = "top-right";
  aetherParams.fade = true;
  aetherParams.parent = this;
  aetherParams.displayCloseIcon = true;
  if (invitorInfo && invitorInfo.image) {
    aetherParams.img_src = invitorInfo.image;
  }
  aetherParams.eventParam = data;
  var message = new com.ibm.bpm.social.Aether(aetherParams);
  message.fireEvent = dojo.hitch(this, fireEvent);
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function handle_PROCESS_COMMENT_TAGGED(data){
  var FUNC_NAME = "handle_PROCESS_COMMENT_TAGGED";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  //publish event to notify mentioned panel for refresh
  dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST, []);
  dojo.publish("/com/ibm/bpm/social/updateStream/" + data.process.instance.id, []);

  var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.process.instance.comment.author.id);
  com.ibm.bpm.social.util.common.Log.log(2, invitorInfo, "notifications", "handle_PROCESS_COMMENT_TAGGED");

  var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
  if (usr == null || usr.userPreferences == null || usr.userPreferences['Portal Notification Post Mention'] != "false") {
    var link_id = getUniqueID("launch_process_action");
    var taskName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.name);
    var actor =
      com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(data.process.instance.comment.author.name);
    var body = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(com.ibm.bpm.social.util.common.SimpleMarkupUtils.createSimpleMarkup(data.process.instance.comment.text));

    // Check if a user is mentioned in the text,
    // but make sure to not use the mentions text for other user then the logged in user.
    var owner= com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var messageText = bpm_social_messages.UserMentionedMessage;
    if (data.process.instance.comment.text && (data.process.instance.comment.text.indexOf("\"bpm-user-tag\"") < 0 ||
          data.process.instance.comment.text.indexOf("objId=\""+owner.userName+"\"") < 0)) {
      messageText = bpm_social_messages.UserCommentedMessage;
    }


    var notificationPopUpText = dojo.string.substitute(messageText, {
      'taskName': taskName,
      'linkID': link_id,
      'actor': actor,
      'messageBody': body
    });
    var aetherParams = {};
    aetherParams.message = notificationPopUpText;
    aetherParams.containerDivId = "messageNotificationDiv";
    aetherParams.position = "top-right";
    aetherParams.fade = true;
    aetherParams.parent = this;
    aetherParams.displayCloseIcon = true;
    if (invitorInfo && invitorInfo.image) {
      aetherParams.img_src = invitorInfo.image;
    }
    aetherParams.eventParam = data;
    var message = new com.ibm.bpm.social.Aether(aetherParams);
    message.fireEvent = dojo.hitch(this, fireEvent);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

function fireEvent(link_id, param){//custom override
  var FUNC_NAME = "fireEvent";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var eventType = link_id.split("::")[0];

  switch (eventType) {
    case "launch_task_action" :
      checkCollaborationSessionAvailablity(param.task.id);
      break;
    case "launch_process_action" :
      dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
        "instanceId": param.process.instance.id
      }]);
      break;
    default:
      break;
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};

/* checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
function checkCollaborationSessionAvailablity(taskID){
  var FUNC_NAME = "checkCollaborationSessionAvailablity";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // Response will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  if (taskID) {
    /* /bpm/notification/roster/task/ */
    dojo.xhrGet({
      //url: this.getProxyUrl()+"/bpm/notification/roster/task/"+taskID,
      url: window.endpointURLObject.processPortalNotificationJS +"/roster/task/" + taskID,
      handleAs: "json",
      preventCache: true,
      sync: true,
      headers: {
        "Content-Type": "application/json",
        "Accept": "application/json"
      },
      load: dojo.hitch(this, function(data, xhrArgs){
        checkNumberOfParticipants(data, xhrArgs, taskID);
      }),
      error: dojo.hitch(this, this.handleFailedRestConnection)
    });
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


/* checkNumberOfParticipants: See if there aren't too many participants in this task already. */
function checkNumberOfParticipants(data, xhrArgs, taskID){
  var FUNC_NAME = "checkNumberOfParticipants";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  // data will look something like this:
  // {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
  // or this:
  // {"message":"NO_SESSION_FOR_TASK"}
  if (!data) {
    return;
  }
  // If there's no collab session, then data won't contain an element called 'participants',
  // and in that case we go ahead and publish the show task topic.
  var doPublishShowTaskTopic = false;
  if (data.participants) {
    var participants = data.participants;
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    var currentUser;
    if (userProfileObj) {
      currentUser = userProfileObj.userName;
    }
    if (participants && (participants.length < 2 || _currentUserIsAlreadyInSession(currentUser, participants))) {
      // publish the show task topic
      doPublishShowTaskTopic = true;
    }
    else {
      com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(bpm_social_messages.collaborationSessionFull, function(){
        return;
      });
    }
  }
  else {
    doPublishShowTaskTopic = true;
  }
  if (doPublishShowTaskTopic) {

    //Defect 133537:  We should use the navigation API here instead of the show task completion topic, as this request
    //Might happen while on a dashboard page.  In which case, the proper pre requisite steps are not taken and the 
    //details page is not shown correctly.

    // publish the show task topic
    // dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
    //   "taskId": taskID
    // }]);

    dojo.publish(com.ibm.bpm.social.portal.Topics.NAVIGATION, 
      ['/dashboards/TWP/BPM_WORK?tw.local.view=taskcompletion&tw.local.taskid=' + taskID]);
  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
};


function _currentUserIsAlreadyInSession(currentUser, participants){
  var FUNC_NAME = "_currentUserIsAlreadyInSession";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  if (currentUser && participants) {
    for ( var i in participants) {
      if (participants[i].userName == currentUser) {
        com.ibm.bpm.social.util.common.Log.logExit(1, "returning true", "notifications", FUNC_NAME);
        return true;
      }
    }

  }
  com.ibm.bpm.social.util.common.Log.logExit(1, "returning false", "notifications", FUNC_NAME);
  return false;
};

function getUniqueID(prefix){
  var FUNC_NAME = "getUniqueID";
  com.ibm.bpm.social.util.common.Log.logEntry(1, "", "notifications", FUNC_NAME);
  var currentTime = new Date();
  com.ibm.bpm.social.util.common.Log.logExit(1, "", "notifications", FUNC_NAME);
  return prefix + "::" + currentTime.getTime();
};


/** ********************************COLLABORATION UTILS END************************************************** */

com.ibm.bpm.social.notifications._ignoreIncomingAssigned = new Array();

/**
 * Prepare ignore of TASK_RESOURCE_ASSIGNED notification for a task id, e.g. for inline completion
 *
 * @param {String} taskID
 * @returns {void} nothing
 */
com.ibm.bpm.social.notifications.setIgnoreAssignedNotification = function(taskID){
  com.ibm.bpm.social.util.common.Log.log(2, taskID, "notifications", "setIgnoreAssignedNotification");
  com.ibm.bpm.social.notifications._ignoreIncomingAssigned.push(taskID);
};

/**
 * @param taskID
 * @returns {Boolean} true if TASK_RESOURCE_ASSIGNED notification for a task id should be ignored
 */
com.ibm.bpm.social.notifications._isIgnoreAssignedNotification = function(taskID){
  var ignore = false;
  var index = dojo.indexOf(com.ibm.bpm.social.notifications._ignoreIncomingAssigned, taskID);
  if (index >= 0) {
    com.ibm.bpm.social.notifications._ignoreIncomingAssigned.splice(index, 1);
    ignore = true;
  }
  com.ibm.bpm.social.util.common.Log.log(2, taskID + " - " + ignore, "notifications", "_isIgnoreAssignedNotification");
  return ignore;
};

com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX = "/bpm/ProcessPortal/notifications";

com.ibm.bpm.social.notifications.getUserNotificationChannelName = function(userName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(userName);
}

com.ibm.bpm.social.notifications.getGroupNotificationChannelName = function(groupName) {
	return com.ibm.bpm.social.notifications._PORTAL_NOTIFICATION_CHANNEL_PREFIX + "/" + com.ibm.bpm.social.util.common.Util.escapeChannelSegment(groupName);
}

/** ********************************************************************************** */

dojo.addOnLoad(connectToBayeuxServer);

var unload = function(event){
  if(window.exitCollaboration) {
    window.exitCollaboration(event);
  }

  if (!window.logoutInProgress) {
    dojox.cometd.disconnect();
  }
};
dojo.addOnUnload(unload);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.common.TaskRestHandler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.common.TaskRestHandler"] = true;
// TaskRestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2011, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.common.TaskRestHandler");







dojo.declare("com.ibm.bpm.social.util.common.TaskRestHandler", com.ibm.bpm.social.util.common.RestHandler,
  {

    constructor: function constructor_TaskRestHandler(params){},

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getTask: function getTask_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/task";
      url += "/" + id;
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        // load: load,
        load: function(task, ioArgs){
          task = myself.addSystemProperties(task, systemID);
          load(task, ioArgs);
        },
        error: error
      });
    },

    /**
     * @param {String[]} idList
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * 
     */
    getTasksInBulkMode: function getTasksInBulkMode_TaskRestHandler(idList, load, error, async, parts){
      // TODO: re-enable federated mode once the REST Api supports a federated version of this

      // URL may look like this: /task?action=getdetails&taskIDs=2,3,5,7,11
      //           or like this: /task?action=getdetails&taskIDs=2,3,5,7,11&parts=data

      if (!idList || !idList.length || idList.length === 0) {
        // defensive - do nothing
        return;
      }

      var url = "/task";
      url += "?action=getdetails&taskIDs=";
      // we have at least one id
      url += idList[0];
      for ( var k = 1; k < idList.length; k++) {
        url += "," + idList[k];
      }

      if (parts && parts != "all") {
        url += "&parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },

    /**
     * @param taskID
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param systemID
     * @param domain
     * @returns {void} nothing
     */
    getActionsForTask: function getActionsForTask_TaskRestHandler(taskID, actionsFilter, load, error, async, systemID,
      domain){
      return this.getActions([{
        tkiid: taskID,
        systemID: systemID
      }], "tkiid", actionsFilter, load, error, async, domain);
    },

    /**
     * @param {Array} items
     * @param {String} idKey e.g TKIID or tkiid or id
     * @param actionsFilter
     * @param {Function} load
     * @param {Function} error
     * @param async
     * @param {String} domain, optional for federated backend and more than one item
     * @returns {dojo.Deferred}
     */
    getActions: function getActions_TaskRestHandler(items, idKey, actionsFilter, load, error, async, domain){
      com.ibm.bpm.social.util.common.Log.logEntry(2);
      if (actionsFilter != null) {
        com.ibm.bpm.social.util.common.Log.logObject(3, actionsFilter, "actionsFilter");
      }

      var deferred = null;

      var self = this;
      var idString = "";
      dojo.forEach(items, function(item){
        if (idString !== "") {
          idString += ",";
        }
        var id = item[idKey];
        if (id !== null) {
          idString += self.prefixID(item[idKey], domain, item.systemID);
        }
      });

      var url = "/task/actions?";
      var newLoad = load;
      if (this.isApiTypeFederated()) {
        if (domain) {
          url += "domain=" + domain + "&";
        }
        else if (items.length === 1 && items[0].systemID) {
          url += "systemID=" + items[0].systemID + "&";
          // insert the system id, in federated case the system id is not returned if no domain is specified
          newLoad = function(data, ioArgs){
            if (data && data.items) {
              for ( var i = 0; i < data.items.length; i++) {
                data.items[i].systemID = items[0].systemID;
              }
            }
            load(data, ioArgs);
          };

        }
        else {
          this.errorNoDomainOrSystemID();
        }
      }
      url += "taskIDs=" + encodeURIComponent(idString);

      if (actionsFilter) {
        var actionsFilterString = "";
        for ( var i in actionsFilter) {
          if (actionsFilterString != "") {
            actionsFilterString += ",";
          }
          actionsFilterString += actionsFilter[i];
        }
        url += "&actions=" + actionsFilterString;
      }

      deferred = this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });

      com.ibm.bpm.social.util.common.Log.logExit(2);
      return deferred;
    },

    /**
     * @param {String} id
     * @param {function} load
     * @param {function} error
     * @param {boolean} async
     * @param {String} parts one of "all", "data", "none" - default is "all"
     * @param {String} systemID federation only
     */
    getProcessActions: function getProcessActions_TaskRestHandler(id, load, error, async, parts, systemID){
      if (this.isApiTypeFederated()) {
        if (!systemID) {
          this.errorNoSystemID();
        }
      }

      var url = "/process";
      url += "/" + id + "/actions";
      url = this.addSystemID(url, systemID);

      if (parts && parts != "all") {
        url += "?parts=" + parts;
      }

      var sync = true;
      if (async) {
        sync = false;
      }
      var myself = this;
      this.performRequest("get", {
        url: url,
        sync: sync,
        load: load,
        error: error
      });
    },


    /**
     * @param id
     * @param load
     * @param error
     * @param priority e.g. Highest/10, High/20, Normal/30, Low/40 or Lowest/50
     * @param dueDate in ISO-8601 format (e.g. 2011-01-05T03:15:30.000-00:00)
     * @param systemID
     * @returns {void} nothing
     */
    updateTaskProperties: function updateTaskProperties_RestHandler(id, load, error, priority, dueDate, systemID){
      var url = "/task";
      url += "/" + id + "?action=update";

      if (priority) {
        url += "&priority=" + priority;
      }

      if (dueDate) {
        url += "&dueDate=" + dueDate;
      }

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Updates all tasks
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param {Object} content <br>
//       *        WPS API {"updatedTaskIDs": [..., ...], "updatedPropertyValues": {...}} <br>
//       *        Federated API {"tasks": [{taskID: ..., systemID: ...}, ...], "updatedPropertyValues": {...}}
//       * @param {String} domain federation only
//       * @returns {void} nothing
//       */
//      updateTasksData: function updateTasksData_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=update";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    claimTask: function claimTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=claim";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error,
        handleAs: "json"
      });
    },

    /**
     * Completes a simple inline task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param property The property for the json result.
     * @param result The value of the result property in the json result.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleInlineTask: function approveSimpleApprovalTask_TaskRestHandler(taskId, claimTask, property, result,
      comment, loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call            
      var data = {};
      data[property] = result;
      if (comment != "") {
        data["comment"] = comment;
      }
      var url = "/task/" + taskId + "?action=complete&params=" + window.encodeURIComponent(dojo.toJson(data));
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    /**
     * Completes a simple completion task.
     * 
     * @param taskId The task id.
     * @param claimTask {boolean} Whether the task needs to be claimed.
     * @param comment An optional comment.
     * @param loadHandler The load handler for the successful invocation.
     * @param errorHandler The error handler for the unsucessful invocation.
     * @returns {void} nothing
     */
    completeSimpleCompletionTask: function completeSimpleCompletionTask_TaskRestHandler(taskId, claimTask, comment,
      loadHandler, errorHandler, systemID){
      var errorOccurred = false;
      if (claimTask) {
        com.ibm.bpm.social.notifications.setIgnoreAssignedNotification(taskId);
        this.claimTask(taskId, null, function(error, ioArgs){
          if (typeof errorHandler === "function") {
            errorHandler(error, ioArgs);
          }
          errorOccurred = true;
        }, systemID);
      }
      if (errorOccurred) {
        return;
      }
      // Provide the data as escaped json to REST call                   
      var url = "/task/" + taskId + "?action=complete";
      if (comment != "") {
        var data = {
          "comment": comment
        };
        url = url + "&params=" + window.encodeURIComponent(dojo.toJson(data));
      }
      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: loadHandler,
        error: errorHandler,
        handleAs: "text"
      });
    },

    publishOnInstanceStream: function _publishOnInstanceStream_TaskRow(piid, comment, load, error, systemID){
      if (piid != null) {
        var url = "/process/" + piid + "?action=comment&comment=";
        url = url + window.encodeURIComponent(comment);
        url = this.addSystemID(url, systemID);
        this.performRequest("post", {
          url: url,
          sync: true,
          load: load,
          error: error,
          handleAs: "text"
        });
      }
    },

    // TODO remove or enable
//      /**
//       * Claims all tasks in content.
//       * 
//       * @param {Function} load
//       * @param {Function} error
//       * @param content {"taskIDs": [...]}
//       * @param {String} domain optional, needed for bulk calls to a federated system
//       * @returns {void} nothing
//       */
//      claimTasks: function claimTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=claim";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    cancelTask: function cancelTask_TaskRestHandler(id, load, error, systemID){
      var url = "/task";
      url += "/" + id + "?action=cancel";
      url = this.addSystemID(url, systemID);

      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    // TODO remove or enable
//      /**
//       * Cancels all tasks in content = {"taskIDs": [...]}.
//       */
//      cancelTasks: function cancelTasks_TaskRestHandler(load, error, content, domain){
//        var url = "/task?action=cancel";
//
//        if (this.isApiTypeFederated()) {
//          if (domain) {
//            url += "&domain=" + domain;
//          }
//          else {
//            this.errorNoDomain();
//          }
//        }
//
//        this.performRequest("rawPut", {
//          url: url,
//          sync: true,
//          load: load,
//          error: error,
//          putData: content,
//          contentType: "application/json; charset=UTF-8"
//        }, false, true);
//      },

    /**
     * 
     * @param tktid
     * @param tkiid
     * @param type
     * @param load
     * @param error
     * @param async
     * @param systemID
     * @returns {void} nothing
     */
    getClientSetting: function getClientSetting_TaskRestHandler(tktid, tkiid, type, load, error, async, systemID){
      //
      // Determine the system of the task
      //
      var wps = false;
      if (this.getSystems().isApiTypeFederated() && this.getSystems().isSystemWps(systemID)) {
        wps = true;
      }
      else if (this.getSystems().isApiTypeWps()) {
        wps = true;
      }
      //
      // Calculate the key for the cache.
      //
      var key = null;
      if (wps && tktid) {
        // WPS has client setting which are based on task templates
        // Therefore use template id if available (Take care of Ad hoc task which do not have a template id)      
        key = "/taskTemplate/" + tktid + "/clientSettings/" + type;
        key = this.getRequestUrl(key);
      }
      //
      // Set url for the call.
      //
      var url = null;
      if (tkiid) {
        // Call as specific as possible due to authorization reasons.
        // Because a user that is only OWNER or POTENTIAL_OWNER of a task can only retrieve the client settings 
        // of a task using the task resource and the tkiid 
        url = "/task/" + tkiid + "/clientSettings/" + type;
      }
      else {
        url = "/taskTemplate/" + tktid + "/clientSettings/" + type;
      }
      url = this.addSystemID(url, systemID);

      var requestLoad = load;
      if (key !== null && typeof load == "function") {
        // Handling for 200 (ok) is implemented, do not cache an 404 result,
        // because a 404 indicates a real error. One of the next calls for
        // this template should bring the real template.
        var expirationPeriod = 3600000;
        var data = com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.get(key);
        if (data) {
          // emulate 2.argument ??
          load.apply(null, [data.data,
                            {}]);
          return;
        }
        else {
          requestLoad = function(data, ioArgs){
            if (ioArgs.xhr.status == 200) {
              com.ibm.bpm.social.widgets.task.util.DataCache.INSTANCE.set(key, {
                data: data
              }, expirationPeriod);
            }
            return load.apply(this, [data,
                                     ioArgs]);
          };
        }
      }
      this.performRequest("get", {
        url: url,
        sync: !async,
        load: requestLoad,
        error: error
      });
    },

    /**
     * 
     * @param id
     * @param load
     * @param error
     * @param async
     * @returns {void} nothing
     */
    getTaskTemplateDetails: function getTaskTemplateDetails_RestHandler(tktid, load, error, async, systemID){
      var url = "/taskTemplate";
      url += "/" + tktid;

      url = this.addSystemID(url, systemID);

      this.performRequest("get", {
        url: url,
        sync: !async,
        load: load,
        error: error
      });
    },

    assignTaskBack: function assignTaskBack_RestHandler(tktid, load, error, systemID){
      var url = "/task";
      url += "/" + tktid + "?action=assign&back=true";

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToUser: function assignTaskToUser_RestHandler(id, load, error, user, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toUser=" + window.encodeURIComponent(user);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    assignTaskToGroup: function assignTaskToGroup_RestHandler(id, load, error, group, systemID){
      var url = "/task";
      url += "/" + id + "?action=assign&toGroup=" + window.encodeURIComponent(group);

      url = this.addSystemID(url, systemID);
      this.performRequest("put", {
        url: url,
        sync: true,
        load: load,
        error: error
      });
    },

    inviteUserToCollaborate: function inviteUserToCollaborate_RestHandler(id, load, error, userid, invitationMessage,
      systemID){
      var url = "/task/" + id;

      url = this.addSystemID(url, systemID);

      this.performRequest("post", {
        url: url,
        content: {
          action: 'invite',
          user: userid,
          message: invitationMessage
        },
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
          "Accept": "application/json"
        },
        sync: true,
        load: load,
        error: error
      });
    },

    statics: {
      _instance: null,

      _createInstance: function(params){
        if (this._instance === null) {
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
          com.ibm.bpm.social.util.common.Log.logObject(1, params, "params: ");
          // TODO handle params, e.g. 
//          this._instance.setIWidget(iWidget);
//          this._instance.setUrl("prefix", this.getServiceURLRoot());
//          this._instance.setUrl("userImageServicePrefix", this.getServiceEndpoint("userImageServiceUrlRoot"));
//          this._instance.vmmPrefix = this.getServiceEndpoint("vmmServiceUrlRoot");
        }
        else {
          // programming error
          var error = new Error("RestHandler instance has already been initialized!");
          com.ibm.bpm.social.util.common.Log.consoleError(error.message);
          throw error;
        }
        return this._instance;
      },

      _getInstance: function(){
        if (this._instance === null) {
          // TODO for now return default handler
          this._instance = new com.ibm.bpm.social.util.common.TaskRestHandler();
//        // programming error
//        var error = new Error("RestHandler instance must be initialized using 'createInstance'!");
//        com.ibm.bpm.social.util.common.Log.consoleError(error.message);
//        throw error;
        }
        return this._instance;
      }
    }
  });

com.ibm.bpm.social.util.common.TaskRestHandler.createInstance = function(params){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._createInstance();
};

com.ibm.bpm.social.util.common.TaskRestHandler.getInstance = function(){
  return com.ibm.bpm.social.util.common.TaskRestHandler.prototype.statics._getInstance();
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect");

//
// -- Class EventQueue
//
dojo.declare("com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect", null,
{
  _blankPageURL: dojo.moduleUrl("com.ibm.bpm.social.util.ui", "templates/blankPage.html"),   
  
  _isInDialog: false,
  
  constructor: function(/*string*/ iframePrefix, isInDialog)
  {
    this._lastShimmeredDomNode = null;
    this._isInDialog = isInDialog;
    
    this.IFramePool_size = 3;
    this.IFramePool_idPrefix = iframePrefix;
    this.IFramePool_freeIFrameName = iframePrefix + "FreeOne";
  },

  
  /**
   * Reapplies the background effect on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the effect reapply if the previous size was the same.
   */
  reapplyEffect: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
	  this.reapplyShimmersOn(domNode, optimize);
  },
  
  /** 
   * creates the background effect
   */
  createEffect: function(domNode)
  {
	  this.createShimmer(domNode);
  },
  
  /** 
   * removes the background effect
   */  
  removeEffect: function(domNode) 
  {
	  this.removeShimmer(domNode);
  },  
  
  /**
   * Reapplies a shimmer on the specified DOM node. Removes a previous one and applies a new one.
   * If the size is the same as last time called, does nothing.
   * 
   * @param optimize If true, will not do the shimmer reapply if the previous size was the same.
   */
  reapplyShimmersOn: function(/*DOMNode*/domNode, /*boolean*/ optimize)
  {
    // To optimize, compare with latest size.
    if (!optimize || !this._lastShimmeredDomNode || 
        domNode.offsetWidth != this._lastShimmeredDomNode.width || domNode.offsetHeight != this._lastShimmeredDomNode.height)
    {
      // Remove possible previous shimmer and add a new one with the new size.
      this.removeShimmer(domNode);
      this.createShimmer(domNode);
      
      // Remember the current this.resultsDiv size.
      this._lastShimmeredDomNode = {width:domNode.offsetWidth, height:domNode.offsetHeight};
    }
  },
  
  // @deprecated - use createEffect
  createShimmer: function(domNode)
  {
    this._log("createShimmer() - id: " + domNode.id);
    this._createShimmer_0(domNode);
    //this._createShimmerDashboard(domNode);
    //this._createGenericObjShimmer({domNode:domNode});
  },
  
  // @deprecated - use removeEffect  
  removeShimmer: function(domNode) 
  {
    this._log("removeShimmer() - id: " + domNode.id);
    this._removeShimmer_0(domNode);
    //this._removeShimmerDashboard(domNode);
    //this._removeGenericObjShimmer({domNode:domNode});
  },
  
  
  //////////////////////////////////////////////////////////////////////////////////////////////////
  // IMPLEMENTATION: Hand made version. The iframe appears OVER the div.
  //////////////////////////////////////////////////////////////////////////////////////////////////
  
  _createShimmer_0: function(domNode)
  {
    // LL
    var objParent = domNode.parentNode;    
    if(objParent){  
      try{
        objParent.removeChild(domNode);
        objParent.appendChild(domNode);
      }catch(err){
      }
    }  
    
    // Look for existing shimmer.
    var shimmerId = this._getShimmerId(domNode);
    var shimmer = document.getElementById(shimmerId);
    if(shimmer)
    {
      this.removeShimmer(domNode);
    }
    
    // Create new shimmer (need to use iframe, if use DIV does not work on Lotus Forms)
    shimmer = document.createElement('iframe');
    shimmer.id = shimmerId;
    shimmer.src = this._blankPageURL;
    //shimmer.src = "javascript:'<html></html>';"
    
//    var opacityVal = 0;//0 [0..10]
//    shimmer.style.opacity = opacityVal/10;
//    shimmer.style.filter = 'alpha(opacity=' + opacityVal*10 + ')';
    
    // Set sizes
    var inWidth = domNode.offsetWidth;
    var inHeight = domNode.offsetHeight;
    var inTop = this._getPositionTop(domNode);
    var inLeft = this._getPositionLeft(domNode);
    
    shimmer.style.position='absolute';
    shimmer.style.width=inWidth+"px";
    shimmer.style.height=inHeight+"px";
    shimmer.style.top=inTop+"px";
    shimmer.style.left=inLeft+"px";

    // defect 108900; account specifically for IE8
	//if (dojo.isIE == 8) {
	//	shimmer.style.zIndex='1'; //Defect 109956 Setting this to 1 caused the users to be hidden in IE8
	//							  // in the dropdown, had to backout this fix for this defect.
	//} else if (dojo.isIE && dojo.isIE < 8) {
    if (dojo.isIE) {
		shimmer.style.zIndex='0';
	} else {
		shimmer.style.zIndex='-1';
	}
    
    shimmer.setAttribute('frameborder','0');

    // Trick: Add the shimmer at the end or beginning, depending if this shimmer is used inside or not of a dialog.
    if (this._isInDialog)
    {
      document.body.appendChild(shimmer);
    }
    else
    {
      document.body.insertBefore(shimmer, document.body.firstChild);
    }

  
    this._log("Shimmer added to document body.");
  },
  
  
  /**
   * Local implementation.
   */
  _removeShimmer_0: function(domNode)
  {
    var shimmerId = this._getShimmerId(domNode);
    try 
    {
      var shimmer = document.getElementById(shimmerId);
      if (shimmer) 
      {
        document.body.removeChild(shimmer);
        this._log("Shimmer removed from document body.");
      }
    } 
    catch(e) 
    {
      this._log(e);
    }
  },
  
  
  _getShimmerId: function(domNode)
  {
    var shimmerId = (domNode.id) ? (domNode.id + "_shimmerutil_shimmer_") : "_unnamed_shimmerutil_shimmer_";
    return shimmerId;
  },
  
  
  _getPositionLeft: function(node)
  {
      var curleft = 0;
      if (node.offsetParent) 
      {
        while (1) 
        {
          curleft += node.offsetLeft;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
      } 
      else if (node.x) 
      {
          curleft += node.x;
      }
      return curleft;
  },
  
  
  _getPositionTop: function(node)
  {
    var curtop = 0;
    if (node.offsetParent) 
    {
        while (1) 
        {
          curtop += node.offsetTop;
          if (!node.offsetParent) {
            break;
          }
          node = node.offsetParent;
        }
    } 
    else if (node.y) 
    {
        curtop += node.y;
    }
    return curtop;
  },
  
  _log: function( message )
  {
    //console.log("ShimmerUtil (" + this.IFramePool_idPrefix + "): " + message);
  }
}
);


}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.UserChooser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.UserChooser"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.UserChooser");





/**
 * Dojo widget that allows the selection of one element from a list of valid ones or also the typing of a new element.
 * <p>
 *
 * The valid elements are provided in the constructor of this class, using the mandatory argument "ucModel".<br>
 * The ucModel should be an object that follows the contract/interface exemplified in the class
 * com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel.
 * <p>
 * Features:
 * <ol>
 *  <li>Embeddable UserChooser or ready to use UserChooserDialog</li>
 *  <li>Configurable model</li>
 *  <li>Configurable initially displayed suggestions</li>
 *  <li>Configurable delay between searchs</li>
 *  <li>Configurable minimum number of letters before starting search</li>
 *  <li>Configurable maximum number of results displayed at a time</li>
 *  <li>Allows to list users and/or groups</li>
 *  <li></li>
 * </ol>
 *
 * Arguments:
 * <ol>
 *  <li>ucModel (Mandatory): the source for the data.</li>
 *  <li>dropDownMaxSize (Optional): The maximum number of elements to display in the dropdown.</li>
 *  <li>usePointerStyle (Optional): Default is false.  If true, the ui will be styled as a pointer.
 *        If false, the ui will be styled as a combo box.</li>
 *  <li>searchDelay (Optional): The time to wait since a key is pressed to when a search is fired. In milliseconds.</li>
 *  <li>searchMinInputLength (Optional): The minimum length on the input to start a search.
 *      Values smaller than this in length will not perform any search.</li>
 *  <li>onChangeHandler (Optional): Handler function to be notified when a selection is done (or undone). 
 *                                  This handler should check
 *                                  the value of the passed in
 *                                  'choice' parameter.  If
 *                                  choice != null, the change
 *                                  is valid.</li>
 *  <li>onEnterHandler (Optional): Handler function for ENTER keystroke.</li>
 *  <li>onEscapeHandler (Optional): Handler function for ESCAPE keystroke.</li>
 * </ol>
 *
 */
dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.UserChooser", [dijit._Widget, dijit._Templated],
{

  templateString:"<div class=\"bpmSocialUserChooser\">\n  <div dojoAttachPoint=\"inputDiv\">\n  </div>\n  <div class=\"hidden noUsersFoundMessage\" dojoAttachPoint=\"noUsersFoundDiv\">\n  </div>  \n  <div class=\"hidden\" dojoAttachPoint=\"dropdownDiv\" > \n      <div class=\"pointer-up\">\n        <div class=\"pointer-up-inner\"></div>             \n        <div class=\"content\">\n            <div dojoAttachPoint=\"resultsDiv\" role=\"option\" class=\"uchDropdown\"></div>            \t\n        </div>            \n    </div>\n  </div>\n</div>\n",
  
  /**
   * Arguments and default values. See description in the header of this class.
   */
  ucModel: null,
  assignTaskidFilter: null,
  collabTaskidFilter: null,
  namesonly: true,
  dropDownMaxSize: (dojo.isSafari && (navigator.userAgent.indexOf('iPad') != -1))? 5:10,
  usePointerStyle: false,
  multiSelect: false,
  showProgressLabel: true,
  searchDelay: 800,
  searchMinInputLength: 1,
  onChangeHandler: null,
  onEnterHandler: null,
  onEscapeHandler: null,
  topic: "com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred",
  preloadInitialSuggestions: false,

  /**
   * Indicates if this instance is inside a USerChooserDialog.
   */
  isInDialog: false,

  /**
   * Private attributes.
   */
  _progressDiv: null,
  _inputWidth: 16,
  _eventConnections: null,
  _multiSelectConnections: null,
  _highlighter: null,
  _chosenItem: null,
  _chosenItems: null,
  _buttonNumber: 1,

  //base text direction user preference
  _btd: "",
  _bodyDir: "",

  /**
   * Indicates if in the middle of a search.
   */
  _performingSearch: false,

  /**
   * The result of the last query.
   */
  _lastQueryItems: [],

  /**
   * The value searched for in the previous attempt. Used to avoid performing the same search twice.
   */
  _lastSearchValue: "",

  _hasFocus: true,
  
  _justOpenedDropdown: true,// Defect 69068

  /**
   * Id of the last scheduled timer for a search.
   */
  _pendingId: null,
  _concurrentSearches: 0,
  _nextSearchId: 0,

  widgetsInTemplate: true,

  constructor: function()
  {
  },
  
  postCreate: function()
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this.inherited("postCreate", arguments);        
    this.setAssignTaskidFilter(this.assignTaskidFilter);
    this.setCollabTaskidFilter(this.collabTaskidFilter);
    if (this.multiSelect) {
        this.usePointerStyle = false;
    }
    //this._debugArguments();
    this._initLocalization();
    this._initUiElements();
    this._setHandlers();
    this._btd = com.ibm.bpm.social.util.common.Util.getBTDUserPreference();
    this._bodyDir = document.body.dir;
    if (this._bodyDir) {
        this._bodyDir == this._bodyDir.toLowerCase();
    } else {
        this._bodyDir == "ltr";
    }
    if (dojo.isIE == 8) {
    } else {
        var self = this;
        dojo.subscribe(this.topic, self._handleRestError);
    }
 
    if (this.preloadInitialSuggestions && this.ucModel.getInitialSuggestions) {
    	var model = this.ucModel;
    	var size = this.dropDownMaxSize + this._getMaxSizeFactor();
    	var af = this.assignTaskidFilter;
    	var cf = this.collabTaskidFilter;
        var no = this.namesonly;
    	window.setTimeout(function(){model.getInitialSuggestions(size,af,cf,no);},100);
    }
  },


  _debugArguments: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "dropDownMaxSize=" + this.dropDownMaxSize,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchDelay=" + this.searchDelay,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "searchMinInputLength=" + this.searchMinInputLength,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "assignTaskidFilter=" + this.assignTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "collabTaskidFilter=" + this.collabTaskidFilter,"UserChooser");
    com.ibm.bpm.social.util.common.Log.log(2, "namesonly=" + this.namesonly,"UserChooser");
  },


  _initLocalization: function()
  {
    
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
  },
  
  _getFormForElement: function(e) {
  		var form = e.parentNode;
  		while (form && (!form.nodeName || (form.nodeName.toLowerCase() != 'form'))) {
  			form = form.parentNode;
  		}
  		if (!form) {
  			form = document.body;
  		}
  		return form;
	},
  
  _initUiElements: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    if (!this.multiSelect) {
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="uchInput";
        this.chooserInput.autocomplete="off";
        dojo.attr(this.chooserInput,"aria-required","true");
        dojo.place(this.chooserInput, this.inputDiv, "first");
        this._progressDiv = document.createElement("div");
        dojo.place(this._progressDiv,this.inputDiv,"last");
    } else {
        var div = document.createElement("div");             
        div.className = "multiselect-container";
        var ul = document.createElement("ul");
        ul.className = "multiselect-choices";
        ul.id = "userchooserchosenitems";
        div.appendChild(ul);
        this.inputDiv.appendChild(div);
        var li = document.createElement("li");
        li.className = "multiselect-search";
        ul.appendChild(li);
        this.chooserInput = document.createElement("input");
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.chooserInput);
        this.chooserInput.type="text";
        this.chooserInput.className="multiselect-input";
        this.chooserInput.autocomplete="off";
        li.appendChild(this.chooserInput);
        this._progressDiv = document.createElement("div");
        li.appendChild(this._progressDiv);
    }
    this.noUsersFoundDiv.innerHTML = this._nlsMessages.noUsersFound;
    this.noUsersFoundDiv.id = "noUsersFound_"+this.id;
    this.chooserInput.focus();
    

    if (this.usePointerStyle) {
        dojo.addClass(this.dropdownDiv,"pointer");
    } else {
        dojo.addClass(this.dropdownDiv,"dropdown");
    }

       
    if (!this.chooserInput.id) {
        var labelCounter = 0; 
        while (dojo.byId("bpsace_userChooserLabel_" + labelCounter) != null) {
            labelCounter = labelCounter + 1;
        }
        com.ibm.bpm.social.util.common.Log.log(2, "labelCounter: "+labelCounter,"UserChooser");
        this.chooserInput.id = "bpsace_userChooserLabel_" + labelCounter;        
    }
    
    // RTC 44850 -- for A11y
    this.chooserInput.title = this._nlsMessages.typeInTextboxLabel;

    
    // Init highlighter
    this._highlighter = new com.ibm.bpm.social.util.ui.userchooser.Highlighter( this.resultsDiv, this._lastQueryItems );

		// Init shimmer.
    
    this._shimmerUtil = new com.ibm.bpm.social.util.ui.FadeOutBackgroundEffect("UCIFramePool_", this.isInDialog);
    
    // Set a unique id for some divs.
    if (!this.dropdownDiv.id)
    {
      this.dropdownDiv.id = this._generateUid();
    }
    if (!this.resultsDiv.id)
    {
      this.resultsDiv.id = this._generateUid();
    }    
    
    // Set into initial state.
    this._chooseItem(-1);
    this._hideResults();
    this._exitSearchingMode();
  },

  //removes duplicates from array 
  _unique: function(  /*{cn,userID,fullName,userName}[]*/ origArray, /*boolean*/ twoArrays ) {
		var newArray = [];
        var x,y,prop,entry,found,equal;		
		for (x=0; x < origArray.length; x++) {
		    found = false;
			entry = origArray[x];
            if (twoArrays) {            
    			for (y = 0; y < newArray.length; y++) {
    				equal = true;
    				for (prop in entry) {
    					if (newArray[y][prop] !== undefined) {
                            com.ibm.bpm.social.util.common.Log.log(2, "1["+prop+"]: "+entry[prop],"UserChooser");
                            com.ibm.bpm.social.util.common.Log.log(2, "2["+prop+"]: "+newArray[y][prop],"UserChooser");
    						if (entry[prop] != newArray[y][prop]) {
    							equal = false;
    							break;
    						}
    					} else {
    						equal = false;
    						break;
    					}
    				}//end for prop
    				if (equal) {
    					for (prop in newArray[y]) {
    						if (entry[prop] === undefined) {
    							equal = false;
    							break;
    						}
    					}
    				}
    			    if (equal) {
    				    found = true;
    				    break;
    				}
    			}//end for y
            }//end twoArrays
			if (!found && this.multiSelect) {//if multiSelect, remove items already selected
               if (this._chosenItems && this._chosenItems.length) {
                   for (y in this._chosenItems) {
                       equal = true;
                       for (prop in entry) {
                           if (this._chosenItems[y][prop] !== undefined) {
                               com.ibm.bpm.social.util.common.Log.log(2, "3["+prop+"]: "+entry[prop],"UserChooser");
                               com.ibm.bpm.social.util.common.Log.log(2, "4["+prop+"]: "+this._chosenItems[y][prop],"UserChooser");
                               if (entry[prop] != this._chosenItems[y][prop]) {
                                   com.ibm.bpm.social.util.common.Log.log(2, "5["+prop+"] not equal","UserChooser");
                                   equal = false;
                                   break;
                               }
                           } else {
                               com.ibm.bpm.social.util.common.Log.log(2, "6["+prop+"] undefined","UserChooser");
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           if (entry["cn"] === undefined && (entry["userID"] === undefined || entry["fullName"] === undefined)) {//need these 2 fields to determine a unique user
                               equal = false;
                               break;
                           }
                       }
                       if (equal) {
                           found = true;
                       }
                   }
               }
            }//end !found && multiSelect            
            if (!found) {
			   newArray.push(entry);
			}
		}
	 return newArray;
  },

  _requestSearch: function()
  {
    var searchValue = this._getSearchValue();
    com.ibm.bpm.social.util.common.Log.log(2,  "_requestSearch(): [" + searchValue + "]","UserChooser");
    if ( searchValue.length < this.searchMinInputLength ) {    
      com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. Input should be at least " + this.searchMinInputLength + " characters.","UserChooser");
      this._clearPendingId();
      this._hideResults();
    } else if (this._isInvalidWildcard(searchValue, true)) {
        com.ibm.bpm.social.util.common.Log.log(2,  "Ignoring search request. InvalidWildcard.","UserChooser");
        this._hideResults();
        // Unchoose previous selection
        this._chooseItem(-1);
    } else {
      // Enqueue search.
      this._requestDelayedSearch();
    }

    // Unchoose previous selection
    this._chooseItem(-1);
  },

  _isInvalidWildcard: function(searchString,showMessage) {
       //check for wildcard character in the middle of the search string
       var wildcard = searchString.indexOf('*');
       while (wildcard != -1) {
           if ((wildcard > 0) && (wildcard < (searchString.length-1))) {
               // we do not support wildcards in the middle of the string
               //var dlg = new com.ibm.bspace.common.util.widget.ErrorPopup();
               //dlg.setMessage(this._nlsMessages.invalidWildcardMessage, null, null, null);
               //dlg.destroyOnHide = true;
               //dlg.show();
               if (showMessage) {
                   
                   
                   com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(this._nlsMessages.invalidWildcardMessage);             
               }
               return (true);
           } else {
               wildcard = searchString.indexOf('*',wildcard+1);
           }
       }
       return (false);
  },

 /**
  * Returns search value trimmed of spaces.
  * */
  _getSearchValue: function()
  {
    var searchString = this.chooserInput.value;
    // 76246 convert the % character to * so that we are only using one wildcard char
    var wildcard = searchString.indexOf('%');
    while (wildcard != -1) {
    	searchString = searchString.substring(0,wildcard)+'*'+searchString.substring(wildcard+1);
        wildcard = searchString.indexOf('%',wildcard+1);
    }
    return searchString;
    //72937 return dojo.trim( this.chooserInput.value );
  },

   /**
   *Returns search value trimmed of leading and ending wild cards.
  *  */

  _getValueNoWildCards: function(value)
  {
      if (value.length > 2 && value.charAt(0) == '"' && value.charAt(value.length-1) == '"') {
          value = value.substring(1, value.length-1);
          com.ibm.bpm.social.util.common.Log.log(2, '_getValueNoWildCards quotes found: '+value,"UserChooser");
      }
      if (value.length > 1) {
          if (value.substr(0,1) == '*') {
              value = value.substr(1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards beg * found: "+value,"UserChooser");
          }
          if (value.length > 1 && value.indexOf('*',value.length-1) > -1) {
              value = value.substring(0,value.length-1);
              com.ibm.bpm.social.util.common.Log.log(2, "_getValueNoWildCards end * found: "+value,"UserChooser");
          }
      }
    return value;    
  },

   /**
    * Returns the id of the input field generated for the user chooser
    */
   getInputId: function()
   {
      return this.chooserInput.id;
   },
   
   /**
    * Sets the aria-labelled by field for the generated input field for the user chooser to the value in the input string
    * which should be the id field of the corresponding label tag
    */
   setAriaLabelledby: function( /*String*/ labelId ) {
   
   	if (this.chooserInput) {
   		
   		dojo.attr(this.chooserInput, "aria-labelledby", labelId);
   	}
   },

  /**
   * Handles a delay before calling _performSearch(). Cancels all pending requests before scheduling a new one.
   */
  _requestDelayedSearch: function()
  {

    // Cancel pending request if any.
    this._clearPendingId();

    // If different from previous search, do a search.
    var searchValue = this._getSearchValue();
    if (searchValue == this._lastSearchValue)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Ignoring search request. Search value is the same as last request: '" + searchValue + "'.","UserChooser");
    }
    else
    {
      if (searchValue == "")
      {
        this._processEmptyState();
      }
      else
      {
        var self = this;
        this._pendingId = window.setTimeout(function() 
                          {
                            self._lastSearchValue = searchValue;
                            self._performSearch(searchValue);
                          }, 
                          this.searchDelay);
      }
    } // else
  },

  
  _enterSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOn";
  },


  _exitSearchingMode: function()
  {
    this._progressDiv.className="uchProgressOff";
  },


  /**
   * Executes a search against VMM.
   */
  _performSearch: function( /*String*/ searchString )
  {
    var searchId = this._nextSearchId++;
    this._concurrentSearches++;
    com.ibm.bpm.social.util.common.Log.log(2, "START    search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches,"UserChooser");

    this._enterSearchingMode();

	//new code to check for mashups wild card setting
	com.ibm.bpm.social.util.common.Log.log(2, "get mashups service","UserChooser");
	var useWildCards = true;
	try {
		var cs = com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		if (cs == null)
		   com.ibm.bpm.social.util.common.Log.log(2, "could not find mashups service", "UserChooser");
		else {
		   var autoWildCard = cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.AUTO_WILDCARD_DISABLED);
		   com.ibm.bpm.social.util.common.Log.log(2, "autowildcard value =" + autoWildCard,"UserChooser");
		   if (autoWildCard == true) 
		   {
			   useWildCards = false;
		   }
		}
	} catch (e2) {
		// if we can't get mashups, then assume wildcards are ok
        com.ibm.bpm.social.util.common.Log.log(2, "error getting mashups service: "+e2 ,"UserChooser");
	}


	//end code to check for mashups wild card setting
    // Add wildcards
	var searchStringToUse;
	var equalSearchString;
	if (useWildCards == true) 
	{
       searchStringToUse = "*";
       if (searchString && searchString != "") {
    	   searchStringToUse = searchString;
		   if (searchString.length > 2 && searchString.charAt(0) == '"' && searchString.charAt(searchString.length-1) == '"') {
			   searchStringToUse = searchString.substring(1, searchString.length-1);
		   }
		   else {
			   if (searchString.charAt(0) != '*') {
				   searchStringToUse = '*' + searchStringToUse;
			   }
			   if (searchString.charAt(searchString.length-1) != '*') {
				   searchStringToUse = searchStringToUse + '*';
			   }
		   }
       }
	   equalSearchString = searchString;
	}
	else 
	{
	   searchStringToUse = searchString;
	}

	com.ibm.bpm.social.util.common.Log.log(2, "Search string is " + searchStringToUse,"UserChooser");

	var modelItems = [];
    // Request 1 more element than can be displayed, so to identify if there are more elements than we are displaying.
	if (useWildCards == true) {
		//78062 var modelItems1 = this.ucModel.query( equalSearchString, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		var modelItems2 = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
		
		/* 78062 if (modelItems1.length > 0) {
		   if (modelItems2.length > 0) {
		      modelItems = this._unique(modelItems1.concat(modelItems2),true);
		   } else {
		      modelItems = this._unique(modelItems1,false);
		   }
		} else */
        if (modelItems2.length > 0) {
			modelItems = this._unique(modelItems2,false);
		}
	} else {
		modelItems = this.ucModel.query( searchStringToUse, 0, this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly );
        modelItems = this._unique(modelItems,false);
	}

    //var modelItems = [];

    // Are we displaying all the items?
    var allItemsDisplayed = true;
    if ( modelItems.length >= (this.dropDownMaxSize + this._getMaxSizeFactor()) )
    {
      // There are more elements than the ones to be displayed. Set a flag indicating this.
      allItemsDisplayed = false;

      // Remove last unwanted element.
      modelItems.length = this.dropDownMaxSize;
    }
    else if ( modelItems.length <= this.dropDownMaxSize )
    {
      // We are ready to display all query elements. allItemsDisplayed is true by default.
    }
    else if ( modelItems.length <= 0 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Got " + modelItems.length + " results." ,"UserChooser");
      // No matches or ucModel (or VMM in the case of VMMModel) is down.
    }
    else
    {
      // Unexpected situation.
      // TODO use NLS. Use common dialogs.
      com.ibm.bpm.social.util.common.Log.log(2,  "*** ucModel is returning " + modelItems.length + " when the maximum allowed is " + this.dropDownMaxSize ,"UserChooser");
      modelItems.length = this.dropDownMaxSize;
    }

    // This search is finishing in sequence? If not, discard.
    var outOfSequenceSearch = (searchId < this._nextSearchId-1);
    if (outOfSequenceSearch)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Out of sequence search: " + searchId + ". Discarding results.","UserChooser");
    }
    else
    {
      // Populate dropdown with search result.
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
      this._exitSearchingMode();
    }
    
    this._concurrentSearches--;
    if (modelItems.length == 0) {
        dojo.removeClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"true","aria-describedby":this.noUsersFoundDiv.id});

    } else {
        dojo.addClass(this.noUsersFoundDiv,"hidden");
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    com.ibm.bpm.social.util.common.Log.log(2,  "FINISHED search (" + searchId + ") for [" + searchString + "]. _concurrentSearches=" + this._concurrentSearches + " : " + modelItems.length + " results.","UserChooser");
  },

  _getMaxSizeFactor: function() {
    var num = 1;
    if (this.multiSelect && this._chosenItems) {
        num = this._chosenItems.length + 1;
    }
    com.ibm.bpm.social.util.common.Log.log(2, "_getMaxSizeFactor: "+num,"UserChooser");
    return num;
  },

  _setModelItems: function( /*{cn,userID,fullName,userName}[]*/ modelItems )
  {
    this._lastQueryItems = modelItems;

    // Update highlighter model.
    this._highlighter.updateModel( modelItems );
    this._highlighter.reset();
  },


  _setModelItemsAndUpdateView: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*boolean*/ allItemsDisplayed )
  {
    this._setModelItems( modelItems );
    this._populateDropdown( modelItems, allItemsDisplayed );

    // The component can loose focus before the search is finished, so only show the dropdown if still 
    // with focus.
    if (this._hasFocus)
    {
      this._showResults();
    }
    this._highlighter.reset();

    // For BPM, we are not using the cn or userID fields for searching, so skip this for now
    // If the typed text is a value (cn or userID) in the model, then choose it.
    //this._tryAutomaticChoosing(modelItems);
  },


  /**
   * If the typed text is a value (cn or userID) in the model, then choose it.
   */  
  _tryAutomaticChoosing: function(/*{cn,userID,fullName,userName}[]*/ modelItems)
  {
    var searchValue = this._getValueNoWildCards(this._getSearchValue());
    
    // If there are items, try to choose automatically.
    if (modelItems.length > 0)
    {
      var index = this._searchModelForField( modelItems, searchValue, "cn" );
      com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (cn) on '" + searchValue + "': " + index ,"UserChooser");
      if (index != -1)
      {
        com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
        this._chooseItem( index );
      }
      else
      {
        // Search for id
        var index = this._searchModelForField( modelItems, searchValue, "userID" );
        com.ibm.bpm.social.util.common.Log.log(2, "Checking for auto choosing (userID) on '" + searchValue + "': " + index ,"UserChooser");
        if (index != -1)
        {
          com.ibm.bpm.social.util.common.Log.log(2, "Auto choosing item - first exact match: " + searchValue,"UserChooser");
          this._chooseItem( index );
        }
      }
    }
    else
    {
      // Set to true to enable free form choice.
      var freeFormChoiceEnabled = false;
      if (freeFormChoiceEnabled)
      {
        // If no values are returned from query (no actuall matches or ucModel down), allow free-form entry.
        com.ibm.bpm.social.util.common.Log.log(2, "Free-form choice made: " + searchValue,"UserChooser");
        this._chooseItemValue( {cn:"", userID:"", fullName: searchValue} );
      }
    }
  },
  

  /**
   * Searchs modelItems for an element with the same value in the searchField as specified.
   * The search is case insensitive.
   * @return The index of the found element or -1 if none.
   */
  _searchModelForField: function( /*{cn,userID,fullName,userName}[]*/ modelItems, /*String*/ searchValue, /*String*/ searchField )
  {
    var index = -1;
    if (searchValue && searchValue!="" && searchField && searchField!="")
    {
      var searchValueLC = searchValue.toLowerCase();
      for( var i=0; i<modelItems.length; i++ )
      {
        var currentValue = modelItems[i][searchField];
        if (currentValue && currentValue.toLowerCase && currentValue.toLowerCase() == searchValueLC)
        {
          index = i;
          break;
        }
      }
    }
    return index;
  },

  /**
   * Show the results if there is at least one item. Otherwise ignore.
   */
  _showResults: function()
  {
  	 
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    if ( this._lastQueryItems.length > 0 && this.resultsDiv.childNodes.length > 0 )
    {
      var optimizeShimmer;
      if ( dojo.hasClass(this.dropdownDiv,"hidden") )
      {
        dojo.removeClass(this.dropdownDiv,"hidden");

        // If just opened the dropdown, initialize highlight.
        this._highlighter.reset();
        this._justOpenedDropdown = true;// Defect 69068
        
        // Do not optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Not visible yet. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = false;
      }
      else
      {
        // Already visible and possible changed size. Optimize shimmer.
        com.ibm.bpm.social.util.common.Log.log(2, ">>> Already visible. this.resultsDiv.childNodes.length:" + this.resultsDiv.childNodes.length,"UserChooser");
        optimizeShimmer = true;
      }
      this._shimmerUtil.reapplyShimmersOn(this.resultsDiv, optimizeShimmer);
      // Defect 69068 User Chooser in Modify Task/Collaboration prefills the selection with first user from result
      // comment out the next line to prevent prefilling
      // this._chooseItem( this._highlighter.index, dojo.keys.DOWN_ARROW );//allow JAWS to speak this selection
    }
    else
    {
      this._hideResults();
    }
  },


  _hideResults: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    dojo.addClass(this.dropdownDiv,"hidden");
    this._shimmerUtil.removeShimmer(this.resultsDiv);
    this._lastSearchValue = null;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
  },


  /**
   * Generates a UID.
   */
  _generateUid: function() /*string*/
  {
    var newId =  "bspace.userchooser." + new Date().getTime();
    if (newId == this._lastGeneratedUid)
    {
      newId += ".x";
    }
    
    this._lastGeneratedUid = newId;
    return newId;
  },
  
  
  _resultsAreVisible: function()
  {
    return dojo.hasClass(this.dropdownDiv,"hidden") == false;
  },


  /**
   * Populates the dropdown with elements created from the specified items.
   */
  _populateDropdown: function( /*{cn,userID,fullName,userName}[]*/ items, /*boolean*/ allItemsDisplayed )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    // Disconnect previous
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = [];

    // Clean previous values.
    this._removeAllChildren(this.resultsDiv);

    // Add new values
    for( var i=0; i<items.length; i++ )
    {
      var qItem = items[i];

      // Calculate display string.
      var identifier = (qItem.fullName && qItem.fullName!="") 
      						? qItem.fullName 
      						: null;
      //format for BPMModel
      //  "userID":d2.userID,
      //  "fullName":d2.fullName,
      //  "userName":d2.userName} 
      //  var dispHtml = qItem.cn + (identifier ? ("&rlm; [ " + identifier + " ]") : "");
      var dispHtml = identifier;
      var rlm = "";
      if (document.body.dir == "rtl") {
          //rlm = "&rlm;";//after applying BTD, this doesn't seem to anything
      }
      if (identifier && qItem.userName && qItem.userName!="") {
          dispHtml = this._createDisplayName(identifier, qItem.userName );//identifier + (rlm+" [ " + qItem.userName + " ]") ;
      }
/* to test with userID      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + (rlm+" [ " + qItem.userID + " ]") ;
      }
*/
/*BSpace      if (identifier && qItem.userID && qItem.userID!="") {
          dispHtml = identifier + ("&rlm; [ " + qItem.userID + " ]") ;
      }
*/
      //format for other models
      var inputName = "";
      if (!dispHtml) {
          inputName = qItem.cn + (identifier ? rlm: "");
          dispHtml = this._createDisplayName(qItem.cn, identifier);//qItem.cn + (identifier ? (rlm+" [ " + identifier + " ]") : "");
      }
      this._setInputWidth(inputName);
      com.ibm.bpm.social.util.common.Log.log(2, "dispHtml: "+dispHtml,"UserChooser");

      // Bold matching substring
      if (this._lastSearchValue && this._lastSearchValue.length > 0)
      {
        dispHtml = this._emphasize( dispHtml, this._getValueNoWildCards(this._lastSearchValue) );
      }

      // Add element with display string.
      var item = this._addChild( this.resultsDiv, "div", dispHtml);
      item.className = "uchDropdownItem";
      
      
      // Hook event handlers.
      this._dropdownEventConnections.push(dojo.connect( item, "onmouseover", this, this._menuItemMouseOver ));
      this._dropdownEventConnections.push(dojo.connect( item, "onmousedown", this, this._menuItemMouseDown ));
    }
    
    // If needed, add a last DIV with a special message indicating that not all items are being displayed.
    if ( this.showProgressLabel && !allItemsDisplayed )
    {
      var msg = dojo.string.substitute( this._nlsMessages.progressLabel, {1:this.dropDownMaxSize} );
      var item = this._addChild( this.resultsDiv, "div", msg );
      item.className = "uchDropdownMessage";
    }
  },

  _createDisplayName: function(fullname, username) {
       if (this._btd == "default") {
           return fullname + (username ? (" [ " + username + " ]") : "");
       }
       var btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(fullname);
       if (this._btd == "ctx") {//must apply UCC to fullname and username separately
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               btdDir = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.getElementBTD(username);
               username = this._applyUCC(username, btdDir);
           }
           return fullname + (username ? (this._getMarker()+" [ " + username + " ]") : "");
       } else {
           fullname = this._applyUCC(fullname, btdDir);
           if (username) {
               username = this._applyUCC(username, btdDir);
           }
           return this._getUCCWrapper() + fullname + (username ? this._getMarker()+" [ " + username + " ]"+com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF : "");
       }
       return fullname + (username ? (" [ " + username + " ]") : "");
  },

  _applyUCC: function(text, btdDir) { 
       var controlChar = (btdDir == "ltr") ? 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE : 
           com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;

       //wrap the text in the control characters
       text = controlChar + text + com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.PDF;
       return text;       
  },

  _getMarker: function() {
    var marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRM;
    if (this._bodyDir == "rtl") {
        marker = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLM;
    }    
    return marker;
  },


  _getUCCWrapper: function() {
       if (this._bodyDir == "ltr") {
           return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.LRE;
       } 
       return com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.BidiConstants.RLE;
  },

  _setInputWidth: function(str) {       
       var len = str.length;
       if (len && len > this._inputWidth) {
           this._inputWidth = len;
           dojo.style(this.chooserInput,"minWidth",(len/2.15)+"em");
       }
  },

  /**
   * Emphasizes the matching text in the source text.
   */
  _emphasize: function( /*String*/ source, /*String*/ toMatch )
  {
    // Homogeinize strings to do case-insensitive matching.
    var sourceLC = source.toLowerCase();
    toMatch = toMatch.toLowerCase();
    var replaced = new String();
    var toMatchLength = toMatch.length;
    var idx = 0;
    while( idx < source.length )
    {
      var startIdx = idx;
      // Search on the lowercase version.
      idx = sourceLC.indexOf( toMatch, idx );
      if ( idx == -1 )
      {
        // Add previous text from the original version.
        var preFoundStr = source.substring( startIdx );
        replaced += preFoundStr;
        break;
      }
      else
      {
        // Add previous text
        var preFoundStr = source.substring(startIdx, idx);
        replaced += preFoundStr;

        // Add new replacement. Use text from original str, to match case.
        var foundStr = source.substr(idx, toMatch.length);
        replaced += "<b>" + foundStr + "</b>";

        idx += toMatchLength;
      }
    }

    com.ibm.bpm.social.util.common.Log.log(2, "_replaced('" + source + "', '" + toMatch + ")='" + replaced + "'","UserChooser");
    return replaced;
  },


  _processEmptyState: function()
  {
    if ( this._getSearchValue() == "" || this._isInvalidWildcard(this._getSearchValue()) )
    {
      // Show initial suggestions
      var modelItems = (this.ucModel.getInitialSuggestions) ? this.ucModel.getInitialSuggestions(this.dropDownMaxSize + this._getMaxSizeFactor(), this.assignTaskidFilter, this.collabTaskidFilter, this.namesonly) : [];
      
      
      // Are we displaying all the items?
      var allItemsDisplayed = true;
      if ( modelItems.length > this.dropDownMaxSize )
      {
        // There are more elements than the ones to be displayed. Set a flag indicating this.
        allItemsDisplayed = false;
  
        // Remove last unwanted element.
        modelItems.length = this.dropDownMaxSize;
      }
      
      this._setModelItemsAndUpdateView( modelItems, allItemsDisplayed );
    }
  },


  /**
   * Sets new handlers. The previous handlers will be disconnected.
   */
  _setHandlers: function()
  {
    // Disconnect previous
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = [];
    
    // Hook event handlers.
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeydown", this, this._onKeyDown ));
	this._eventConnections.push(dojo.connect( this.chooserInput, "onkeypress", this, this._onKeyPress ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onkeyup", this, this._onKeyUp ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onfocus", this, this._onUCFocus ));
    this._eventConnections.push(dojo.connect( this.chooserInput, "onblur", this, this._onUCBlur ));
  },



  /**
   * Special treatment to identify the correct DIV in the menu.
   *
   * Because of construction, is necessary to avoid the selection of an inner span in the DIV (this happens with the <B> tags inside the
   * DIV when bolding a part of the string).
   * If a SPAN is identified, is assumed that the parent is the div.
   * Note: this only makes sense according to the construction of the dropdown.
   * If that implementation is changed or the events associated with any other DIV here, this will not work anymore.
   *
   * TODO Find a better solution. This works on FF3 but not on IE7?
   */
  _getSmartTarget: function( /*Event*/ e )
  {
    var smartTarget = null;
    var target = e.target;
    if ( target )
    {
      if ( dojo.isIE )
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - IE" ,"UserChooser");
        smartTarget = target;
        while (smartTarget && (smartTarget.nodeName.toLowerCase() != "div")) {
        	smartTarget = smartTarget.parentNode;
        }
      }
      else
      {
        //com.ibm.bpm.social.util.common.Log.log(2,  "mouse event - not IE" ,"UserChooser");
        if(target.tagName == "B") {
          target = target.parentNode;
        }
        if ( target instanceof HTMLDivElement)
        {
          smartTarget = target;
        }
        else if (((HTMLSpanElement && target instanceof HTMLSpanElement) || target.tagName == "SPAN") && target.parentNode instanceof HTMLDivElement)
        {
          smartTarget = target.parentNode;
        }
        else
        {
          // Ignore
        }
      }
    }

    return smartTarget;
  },


  _menuItemMouseOver: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_menuItemMouseOver()" ,"UserChooser");
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._highlighter.moveSelectionToItem( smartTarget );
    }
  },


  _menuItemMouseDown: function( /*Event*/ e )
  {
    var smartTarget = this._getSmartTarget( e );
    if ( smartTarget )
    {
      this._hideResults();
      this._chooseItem( this._highlighter._getIndexForDivItem( smartTarget ) );
    }
  },

  _onKeyPress: function( /*Event*/ e) {
    com.ibm.bpm.social.util.common.Log.log(2, "keyCode="+e.keyCode+" charCode="+e.charCode,"UserChooser");
    if((e.charCode>0)) {
      setTimeout(dojo.hitch(this, "_requestSearch"), 10);
    }
  },

  _onKeyUp: function( /*Event*/ e )
  {
    if (e.keyCode == dojo.keys.BACKSPACE || e.keyCode == dojo.keys.CLEAR || e.keyCode == dojo.keys.DELETE) {
      this._requestSearch();
    }
  },


  _onKeyDown: function( /*Event*/ e )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_onKeyDown(): " + e.keyCode ,"UserChooser");
    var key = e.keyCode;
    //var key = (window.event) ? event.keyCode : e.keyCode;

    if (key == dojo.keys.DOWN_ARROW)
    {
      this._showResults();
      //First time dropdown is opened, use the down arrow key to select the first item
      //instead of moving to the next item
      if ( !this._justOpenedDropdown ) {// Defect 69068
      if ( this._highlighter.moveSelectionForward() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
      } else {
          this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
          this._justOpenedDropdown = false;// Defect 69068
      }
    }
    else if (key == dojo.keys.UP_ARROW)
    {
      this._showResults();
      if ( this._highlighter.moveSelectionBack() )
      {
        this._chooseItem( this._highlighter.index, key );//allow JAWS to speak this selection
      }
    }
    else if (key == dojo.keys.ENTER)
    {
      if ( this._resultsAreVisible() )
      {
        // Choose the currently selected item.
        this._hideResults();
        this._chooseItem( this._highlighter.index, key );
      }
      else
      {
        this._onEnterHandler();
      }
      dojo.stopEvent(e);
    }
    else if (key == dojo.keys.TAB)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
      }
    }
    else if (key == dojo.keys.ESCAPE)
    {
      if ( this._resultsAreVisible() )
      {
        this._hideResults();
        com.ibm.bpm.social.util.common.Log.log(2, "escape detected, unchoose item","UserChooser");
        this._clearPendingId();
        this._chooseItem(-1);//make unchoose item the default behavior for escape
        this._exitSearchingMode();
      }
      else
      {
        this._onEscapeHandler();
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _clearPendingId: function() {
     if (this._pendingId)
     {
        com.ibm.bpm.social.util.common.Log.log(2, "Canceling timer: '" + this._pendingId + "'.","UserChooser");
        window.clearTimeout(this._pendingId);
        this._pendingId = null;
     }
  },

  /**
   * Chooses the specified item as the user choice. The value in this._lastQueryItems in that index will be returned when getChoice
   * is called.
   * If itemIndex is -1, will unchoose any previous selection.
   */
  _chooseItem: function( /*Number*/ itemIndex, /*KeyCode*/ keyCode )
  {
    var item = (itemIndex in this._lastQueryItems) ? this._lastQueryItems[itemIndex] : null;
    this._chooseItemValue(item, keyCode);
  },


  /**
   * Chooses the specified item as the user choice. The specified item will be returned when getChoice is called.
   * If item is null, will unchoose any previous selection.
   */
  _chooseItemValue: function( /*{cn,userID,fullName,userName}*/ item, /*KeyCode*/ keyCode )
  {
    var unchoose = true;
    if (item)
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "Item chosen: " + item + ":" + dojo.toJson(item),"UserChooser");
      this._chosenItem = item;
      
      // Replace input string with match.
      if (item.cn && item.cn != "")
      {
        this.chooserInput.value = item.cn;
      } else if (item.fullName && item.fullName != "") {
    	this.chooserInput.value = item.fullName;
      }

      if (!keyCode || (keyCode && keyCode === dojo.keys.ENTER)) {
          this._onChangeHandler();
      }
      unchoose = false;
      
      // Forget last searched value.
      this._lastSearchValue = null;
    }

    if ( unchoose )
    {
      this._chosenItem = null;
      this._onChangeHandler();
    }
    //com.ibm.bpm.social.util.ui.TextField.setIncorrectEntry(this.chooserInput,unchoose);
  },

  setAssignTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.collabTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setAssignTaskidFilter found collabTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting collabTaskidFilter to null.","UserChooser");
         this.collabTaskidFilter = null;
     }
     this.assignTaskidFilter = taskid;
  },

  setCollabTaskidFilter: function(taskid) {
     if (taskid == "") {
         taskid = null;
     } else if (taskid && parseInt(taskid,10) != taskid) {
         taskid = null;
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter taskid "+taskid+" is not an integer, setting to null","UserChooser");
     } 
     if (taskid && this.assignTaskidFilter) {
         com.ibm.bpm.social.util.common.Log.log(2, "setCollabTaskidFilter found assignTaskidFilter taskid "+taskid+" NOT null. Cannot send both to REST services. Setting assignTaskidFilter to null.","UserChooser");
         this.assignTaskidFilter = null;
     }
     this.collabTaskidFilter = taskid;
  },

  _onChangeHandler: function() {  
    if (this.onChangeHandler) {
      if (this.multiSelect) {
           this._onChangeHandlerMultiselect();
          this.onChangeHandler(this._chosenItems);
      } else {      
          this.onChangeHandler(this._chosenItem);
      }
    }  
  },

  _onChangeHandlerMultiselect: function() {
   if (this._chosenItem != null) {
       if (!this._chosenItems) {
           this._chosenItems = new Array();
           if (!this._multiSelectConnections) {
               this._multiSelectConnections = new Array();
           }
       }
       var li = document.createElement("li");
       li.className = "choice";
       var span = document.createElement("span");
       var displayName =this._chosenItem.fullName;
       if (displayName == undefined) {
           displayName = this._chosenItem.userName;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.cn;
       }
       if (displayName == undefined) {
           displayName = this._chosenItem.userID;
       }
       span.innerHTML = displayName;
       li.appendChild(span);
       var button = document.createElement("button");
       button.className = "multiselect-delete-button";
       var idStr = this._getNextButtonId();
       button.id = idStr;
       span.id = idStr+"_span";
       dojo.attr(button, "aria-label", this._nlsMessages.deleteButtonLabel);
       dojo.attr(button, "aria-describedby", idStr+"_span");
       this._multiSelectConnections.push(dojo.connect( button, "onclick", this, this._multiSelectDeleteChoice ));
       var hcSpan = document.createElement("span");//a11y high contrast
       hcSpan.innerHTML = "X";
       hcSpan.className = "multi-select-delete-button-hc";
       button.appendChild(hcSpan);
       li.appendChild(button);
       dojo.place(li,"userchooserchosenitems",this._chosenItems.length);
       this._chosenItem = 
               {"cn":this._chosenItem.cn,
               "userID":this._chosenItem.userID,
               "fullName":this._chosenItem.fullName,
               "userName":this._chosenItem.userName,
               "buttonId": idStr};
       this._chosenItems.push(this._chosenItem);
       this._chosenItem = null;
       this.chooserInput.value = "";
       this._resetInputWidth();
   }
  },

  _resetInputWidth: function(){
       this._inputWidth = 16;
       dojo.style(this.chooserInput,"minWidth",this._inputWidth+"em");
  },

  _multiSelectDeleteChoice: function(e) {
    com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice id: "+e.target.id,"UserChooser");
    dojo.disconnect(e.target);
    //userId and fullName
    var node = document.getElementById(e.target.id);
    if (node) {//button anchor node
      node = node.parentNode;//parent li node
      if (node) {      
          node.parentNode.removeChild(node);
          for (var i in this._chosenItems) {
              var item = this._chosenItems[i];
              if (item.buttonId == e.target.id) {
                  this._chosenItems.splice(i,1);
                  //com.ibm.bpm.social.util.common.Log.log(2, "_multiSelectDeleteChoice deleted index: "+i+" _chosenItems.length: "+this._chosenItems.length,"UserChooser");
                 if (this._chosenItems.length == 0) {
                      this._chosenItems = null;
                  }
                  break;
              }
          }
      }
      this._onChangeHandler();
      dojo.stopEvent(e);
    }
  },

  _getNextButtonId: function() {
       var num = this._buttonNumber++;
       //com.ibm.bpm.social.util.common.Log.log(2, "_getNextButtonId num: "+num,"UserChooser");
       return "chosenitem_"+num;
  },

  _onEscapeHandler: function()
  {
    if (this.onEscapeHandler)
    {
      this.onEscapeHandler();
    } 
  },


  _onEnterHandler: function()
  {
    if (this.onEnterHandler)
    {
      this.onEnterHandler();
    }
  },


  _onUCFocus: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = true;
    if (this._chosenItem != null) {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    if ( this._getSearchValue() == "" )
    {
      this._processEmptyState();
    }
    else if (this._isInvalidWildcard(this._getSearchValue())) {
      this._processEmptyState();
    }
    else
    {
      if (!this._chosenItem) {
    	  this._requestSearch();
      }
    }
  },


  /**
   * Cannot use _onBlur name to avoid collision with Dojo core.
   */
  _onUCBlur: function( /*Event*/ e )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    this._hasFocus = false;
    this._hideResults();
    this._exitSearchingMode();
    if (this._chosenItem == null) {//if nothing selected, clear input field
        this.chooserInput.value = "";
    } else {
        dojo.attr(this.chooserInput,{"aria-invalid":"false","aria-describedby":""});
    }
    dojo.addClass(this.noUsersFoundDiv,"hidden");    

    // Defect 69068 - don't autoselect - may not be what the user wanted
    // If only one item in the last query, choose it.
    //if ( this._lastQueryItems.length == 1 )
    //{      
      //this._chooseItem( 0 );
    //}
  },
  
  _handleRestError: function() {
       com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
       this.chooserInput.value = "";
       this._exitSearchingMode();
       this._clearPendingId();
	   com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },

  /**
   * @return The currently choosen value or null if none.
   */
  getChoice: function() /*{cn,userID,fullName,userName}*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "getChoice(): " + dojo.toJson(this._chosenItem),"UserChooser");
    return this._chosenItem;
  },


  _addHtmlSelectOptions: function( /*DOMSelect*/ domSelect, /*String[]*/ options )
  {
    for( var i=0; i<options.length; i++ )
    {
      var option = document.createElement("option");
      option.value = options[i];
      option.innerHTML = options[i];
      domSelect.appendChild(option);
    }
  },


  _removeAllChildren: function( /*DOMSelect*/ domSelect )
  {
    while( domSelect.hasChildNodes() )
    {
      domSelect.removeChild( domSelect.lastChild );
    }
  },


  _addChild: function( parent, /*String*/ elementName, /*String?*/ innerHTML )
  {
    var newElem = document.createElement(elementName);
    parent.appendChild(newElem);

    if ( innerHTML )
    {
      newElem.innerHTML = innerHTML;
    }
    return newElem;
  },


  destroy: function()
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");

    // Destroy pending event handlers.
    this._disconnectEvents(this._eventConnections);
    this._eventConnections = null;
    this._disconnectEvents(this._dropdownEventConnections);
    this._dropdownEventConnections = null;
    dojo.unsubscribe(this.topic);
    this.inherited("destroy", arguments);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"UserChooser");
  },


  _disconnectEvents: function(/*handler[]*/ handlers)
  {
    if (handlers && handlers.length > 0)
    {
      com.ibm.bpm.social.util.common.Log.log(2, "Disconnecting " + handlers.length + " event handlers.","UserChooser");
      while(handlers.length)
      {
        dojo.disconnect(handlers.pop());
      }
    }
  },
  
  
  _log: function( message )
  {
    //console.log( message );
  }
}
);


dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.Highlighter", null,
{
  index: null,
  item: null,
  _resultsDiv: null,
  _model: null,

  constructor: function( resultsDiv, model )
  {
    this.index = null;
    this.item = null;
    this._resultsDiv = resultsDiv;
    this._model = model;
  },


  updateModel: function( model )
  {
    this._model = model;
  },


  reset: function()
  {
    // Unselect previous item.
    this.unselectCurrent();

    // If the model has elements, select first one. Otherwise, no one.
    if ( this._model && 0 < this._model.length )
    {
      this.index = 0;
      this.selectItemIndex( this.index );
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  unselectCurrent: function()
  {
    if (this.item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "_unselectCurrent(): " + this.item.innerHTML ,"UserChooser");
      this.item.className = "uchDropdownItem";
    }
    else
    {
      this.item =  null;
      this.index = null;
    }
  },


  /**
   * Selects the specified div, remembers the last selection and calculate the index.
   * It is expected that item is in this.resultsDiv.childNodes.
   */
  selectItem: function( /*HTMLDiv*/ item )
  {
    if (item)
    {
      //com.ibm.bpm.social.util.common.Log.log(2,  "selectItem(): "  + item.innerHTML,"UserChooser");
      item.className = "uchDropdownItemSelected";
      this.item = item;
      this.index = this._getIndexForDivItem( item );
    }
  },


  selectItemIndex: function( /*Number*/ itemIndex )
  {
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"UserChooser");
    var divItems = this._resultsDiv.childNodes;
    if ( itemIndex != null && itemIndex < divItems.length )
    {
      this.item = divItems[itemIndex];
      if ( this.item )
      {
        this.item.className = "uchDropdownItemSelected";
        this.index = itemIndex;
      }
      else
      {
        this.index = null;
        com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex ,"UserChooser");
      }
    }
    else
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found in index: " + itemIndex + ", divItems.length=" + divItems.length ,"UserChooser");
    }
  },


  moveSelectionToItemIndex: function( /*Number*/ itemIndex )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItemIndex(): " + itemIndex ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItemIndex( itemIndex );
  },


  moveSelectionToItem: function( /*HTMLDiv*/ item )
  {
    //com.ibm.bpm.social.util.common.Log.log(2,  "_moveSelectionToItem(): " + item.innerHTML ,"UserChooser");
    // Unselect previous one.
    this.unselectCurrent();

    // Select current one.
    this.selectItem( item );
  },


  _getIndexForDivItem: function( /*HTMLDiv*/ item )
  {
    var index = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf( this._resultsDiv.childNodes, item );
    if ( index == -1 )
    {
      com.ibm.bpm.social.util.common.Log.log(2,  "** Menu item not found: "  + item.innerHTML ,"UserChooser");
      com.ibm.bpm.social.util.common.Log.log(2,  "**: "  + dojo.toJson(item) ,"UserChooser");
    }

    return index;
  },


  moveSelectionForward: function()
  {
    var moved = false;
    if (this.index != null && this.index < this._model.length-1)
    {
      this.index++;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },


  moveSelectionBack: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2, "moveSelectionBack","UserChooser");
    var moved = false;
    if (this.index != null && 0<this.index)
    {
      this.index--;
      this.moveSelectionToItemIndex(this.index);
      moved = true;
    }

    return moved;
  },

  
  debug: function()
  {
    com.ibm.bpm.social.util.common.Log.log(2,  "index:" + this.index + ", item:" + this.item.innerHTML ,"UserChooser");
  },


  _log: function( message )
  {
    //console.log( "Highlighter: " + message );
  }
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil");

//
// -- Collection utilities.
//

/**
 * Sorts the specified array of objects using the objects field named as specified by sortField.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy = function( array, sortField ) /*void*/
{
  // Define sorter object.
  var sorter = {
    sortBy: function( a,b)
    {
      if ( a[sortField] == b[sortField] )
        return 0;
      else
        return (a[sortField] < b[sortField]) ? -1 : 1;
    }
  };

  // Sort.
  array.sort( sorter.sortBy );
};



/**
 * Filters the specified array.
 *
 * Creates a new array containing the elements in the original array that evaluates to true when applied the filter.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.filterArray = function( /*Array*/ array, booleanFilter ) /*Array*/
{
  var filtered = new Array();
  for (var i = 0; i<array.length; i++)
  {
    var context = this;
    if ( booleanFilter( array[i], context ) )
    {
      filtered.push( array[i] );
    }
  }

  return filtered;
};


/**
 * Partitions the list based on a specified field.
 * Each time the field value changes from the previous item in the list, the groupHandler is called.
 * For each element in the group, the elementHandler is called.
 *
 * The parameter array should be previously sorted by the specified field.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayInGroupsBy = function( array, field, groupHandler, unassignedGroupMessage )
{
  var currentGroupName = null;
  var currentGroup = null;
  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize group if necessary.
    if ( currentGroupName == null || currentGroupName != fieldValue )
    {
      if ( currentGroup != null )
      {
        // Notify of finished group.
        groupHandler( currentGroupName, currentGroup );
      }
      currentGroupName = fieldValue;
      currentGroup = new Array();
    }

    // Process element.
    currentGroup.push( elem );
  } // for

  // Process last group, if any.
  if ( currentGroup != null )
  {
    groupHandler( currentGroupName, currentGroup );
  }
};


/**
 * Creates a map from an array.
 * The returned map is JSON object with the keys being the values of the specified field
 * in the mapped elements.
 *
 * The parameter array will be sorted as a side effect.
 *
 * @return A not nullable "map", from string to objects originally in array.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.partitionArrayBy = function( array, field, unassignedGroupMessage )
{
  com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.sortArrayBy( array, field );
  var map = new String();

  for (var i = 0; i<array.length; i++)
  {
    var elem = array[i];
    var fieldValue = elem[field];

    // Treat the null or empty cases.
    if ( fieldValue == null || fieldValue == "" )
    {
      fieldValue = unassignedGroupMessage;
    }

    // Initialize bucket if necessary.
    var bucket = map[fieldValue];
    if ( bucket == null )
    {
      // No bucket yet. Create it.
      bucket = new Array();
      map[fieldValue] = bucket;
    }

    // Add element to bucket.
    bucket.push( elem );
  } // for

  return map;
};




/** 
 * Makes all elements in this array unique, i.e., removes all duplicate elements from array.
 * Assumes array is already sorted. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.unique = function(/*Array*/ array, compare)
{
  if (typeof(compare) == 'undefined') 
  {
    compare = com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend;
  }
  var dst = 0;  // Destination for elements
  var src = 0;  // Source of elements
  var limit = array.length - 1;
  while (src < limit) 
  {
    while (compare(array[src], array[src + 1]) == 0) 
    {
      if (++src == limit) 
      {
        break;
      }
    }
    array[dst++] = array[src++];
  }
  if (src == limit) 
  {
    array[dst++] = array[src];
  }
  array.length = dst;
}


/** 
 * Adds all the elements in the specified arrays to destArray. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.addAll = function(/*Array*/ destArray, /*Array[]*/ arrays) 
{
  for (var a = 0;  a < arrays.length;  a++) 
  {
    arr = arrays[a];
    for (var i = 0;  i < arr.length;  i++) 
    {
      destArray.push(arr[i]);
    }
  }
}


/** 
 * Compares two objects using built-in JavaScript operators. 
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.ascend = function(a, b) 
{
  if (a < b)
    return -1;
  else if (a > b)
    return 1;
  return 0;
}



/**
 * Will search the array until it matches the search critera.
 */
com.ibm.bpm.social.util.ui.userchooser.util.CollectionUtil.indexOf = function( /*Array*/ array, element /*, from*/ ) /*Number*/
{
  var length = array.length;

  var from = Number(arguments[2]) || 0;
  from = (from < 0) ? Math.ceil(from) : Math.floor(from);
  if (from < 0)
    from += length;

  for (; from < length; from++)
  {
    if (array[from] == element)
      return from;
  }
  return -1;
};

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.RestClient"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.util.RestClient"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.util.RestClient");



dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.util.RestClient", null,
{
  REASON_ADMINISTRATOR : 7,
  bpmServiceRoot: window.endpointURLObject.bpmRestJS,
  restgatewayerror: "Error 404: REST Service Gateway:",
  _queryIndex:0,

  constructor: function(bpmServiceRoot, iContext)
  {
	//com.ibm.bpm.social.util.common.Log.logLevel = 2;//enable to turn on logging
    com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    // Normalize service urls.
    if (bpmServiceRoot) {
        this.bpmServiceRoot = bpmServiceRoot;
    }
//PKL -- TODO bug
    if (iContext)
    {
      this.iContext= iContext;
    }

    this.bpmServiceRoot = this._normalizeUrl( this.bpmServiceRoot );
    this.bpmServiceRoot +="/v1";

    com.ibm.bpm.social.util.common.Log.log(2, "Using bpmServiceRoot=" + this.bpmServiceRoot ,"RestClient");

    this._bpmHeaders = {};
    this._bpmHeaders["accept"] = "application/json"; /*  "text/xml"; */
    if (!this.bpmServiceRoot) {
        this._bpmHeaders["content-type"] = "charset=utf-8";
    } else {
        this._bpmHeaders["content-type"] = "application/json";
    }
	
    
    this._nlsMessages = dojo.i18n.getLocalization("com.ibm.bpm.social.util.ui.userchooser", "UserChooser_nls");
    
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");

  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByFilter: function( /*String*/ name, /*int*/ offset, /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes ) 
  {
   //return this._searchBPMUsers( "filter=" + name, offset, size, attributes );
     return this._searchBPMUsers( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, attributes );
  },

  /**
   * Searchs BPM for users matching partially the specified name.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  searchBPMByName: function( /*string*/ name, /*int*/ offset, /*int*/ size, /*boolean*/ includeGroups, /*boolean*/ includeUsers, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly ) 
  {
    var searchGroups = includeGroups ? (includeGroups == true) : false;	// Default false
    var searchUsers = (includeUsers != null && includeUsers != undefined) ? (includeUsers == true) : true;	// Default true
    var cnsAll = [];
    
	this.errormsg = false;
	if (searchUsers)
	{
	    cnsAll = this.searchBPMByFilter( name, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly );	
    }
	this.errormsg = false;
	
    // Cut any extra not requested elements.
    if ( cnsAll.length > size )
    {
      cnsAll.length = size;
    }

    return cnsAll;
  },

  /**
   * Generic BPM query. Searchs BPM for users matching the search
   * attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPMUsers: function( /*String*/ searchAttributes, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*[String]*/ attributes, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    var includeAttributes = "";
    var extractor = "";
    return this._searchBPM( "users", searchAttributes, includeAttributes, extractor, offset, size, assignTaskidFilter, collabTaskidFilter, namesonly, loadCallback );
  },

  /**
   * Generic BPM query. Searchs BPM for users/groups matching the
   * search attributes.
   * <p>
   * Synchronic call.
   *
   * @return An array of results. Not null.
   */
  _searchBPM: function( /*String*/ entity, /*String*/ searchAttributes, /*String*/ attributes, /*function*/ extractor, /*int?*/ offset, /*int?*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly, /*function*/ loadCallback ) /*{cn,userID,fullName,userName}[]*/
  {
    com.ibm.bpm.social.util.common.Log.log(2, "Querying BPM for " + entity + ":'" + "filter=" +searchAttributes + "', maxresult="+size+", assignTaskidFilter="+assignTaskidFilter+" , collabTaskidFilter="+collabTaskidFilter+" ,namesonly: "+namesonly+" , using encodeURIComponent on "+searchAttributes ,"RestClient");

    var assignTaskString = "";
    if (assignTaskidFilter != null) {
        assignTaskString = "&assignTaskidFilter="+assignTaskidFilter;
    }
    var collabTaskString = "";
    if (collabTaskidFilter != null) {
        collabTaskString = "&collabTaskidFilter="+collabTaskidFilter;
    }
    var namesonlyString = "&namesonly=true";//performance
    if (namesonly != 'undefined' && (namesonly == false || namesonly == "false")) {
        namesonlyString = "";
    }
    //encode special character of userids in searchAttributes
    var url = this.bpmServiceRoot + "/" + entity + "?" + "filter=" +
              (window.encodeURIComponent(searchAttributes)) +
              "&maxresult=" + size +
              assignTaskString +
              collabTaskString +
              namesonlyString +
              "&parts=none";

    var results = []; // default value
    var self = this;
    var okHandler = function( /*String*/data, ioArgs )
    {
      com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM in okHandler, result is " + ioArgs.xhr.status,"RestClient");
      if (data)
      {
        // Collect desired information from results
        //console.log("_searchBPM data: "+dojo.fromJson(data));
        var d = dojo.fromJson(data);
        if (d.data && d.data.users) {        
            for (var i in d.data.users) {
                var d2 = d.data.users[i];
                if (!d2.isDisabled) {
                    com.ibm.bpm.social.util.common.Log.log(2,"...fullName: "+d2.fullName+", userName: " + d2.userName + ", userID: " + d2.userID,"RestClient");
                    results.push( 
                        {"cn":"",
                        "userID":d2.userID,
                        "fullName":d2.fullName,
                        "userName":d2.userName} 
                    );
                }
                //cn,userID,fullName,userName
            }
        }
      }
      //console.log("_searchBPM loadCallback: "+loadCallback); == undefined
      if (loadCallback) {
          loadCallback(results, ioArgs);
      }
    };
    var errorHandler = function( /*String*/data, ioArgs )
    {
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM In Error Handler, status is " + ioArgs.xhr.status,"RestClient");
       com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM responsetext " + ioArgs.xhr.responseText,"RestClient");
       var msg; 
       if ((ioArgs.xhr.status != 404) || ((ioArgs.xhr.status == 404) && (ioArgs.xhr.responseText.indexOf(self.restgatewayerror) >= 0)))  {
          if (!self.errormsg) {		     
             self.errormsg = true;
             var msg, msg2=null;
             if (ioArgs.xhr.responseText != null) {                 
                 var r1 = ioArgs.xhr.responseText.split(",");
                 for (var j=0; j<r1.length; j++) {
                      var r2 = r1[j].split(':');
                      if (r2[0] == '"errorMessage"') {
                          msg2 = r1[j].substr(16,r1[j].length-17);
                          break;
                      }
                 }
                 if (msg2 == null) {
                     msg2 = ioArgs.xhr.responseText;//responseText did not contain errorMessage, cannot format it
                 }
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:msg2} );
             } else {
                 msg = dojo.string.substitute( self._nlsMessages.internalError, {1:ioArgs.xhr.status} );
             }
             dojo.publish("com.ibm.bpm.social.util.ui.userchooser.util.RestClient.errorOccurred");
             com.ibm.bpm.social.util.common.Log.log(2,"_searchBPM publish errorOccurred topic","RestClient");
             
             com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showError(msg);             
          } 
        }
    };

    this._executeGet( url, okHandler, errorHandler, (loadCallback == null), this._bpmHeaders );

    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_searchBPM(" + searchAttributes + "," + offset + "," + size + ")=" + results.length ,"RestClient");
    com.ibm.bpm.social.util.common.Log.logEntry(2, "results: " + dojo.toJson( results ) ,"RestClient");
    return results;
  },

  //
  // -------- BPC
  //

  /**
   * Calculates the potential owners of the specified task.
   * <p>
   * Synchronic call.
   */
  queryTaskPotentialOwners: function( /*String*/ taskId, systemId, okHandler, errorHandler ) /*void*/
  {    
    var url = this.bpcServiceRoot + "/task/" + taskId + "/role/REASON_POTENTIAL_OWNER";
    if (systemId) {
        url+= "?systemID="+systemId;
    }
    this._executeGet( url, okHandler, errorHandler, true );
  },

  //
  // -------- Helpers
  //


  _executeGet: function( resourcePath, okHandler, errorHandler, sync, headers )
  {
  	com.ibm.bpm.social.util.common.Log.logEntry(2, "" ,"RestClient");
    this._queryIndex++;

    //var encodedUrl = encodeURI( resourcePath );
    var encodedUrl = resourcePath;
    
    // Rewrite only if needed.
    var needsRewrite = (encodedUrl.indexOf("http") == 0);

    var url = (needsRewrite && this.iContext) ? this.iContext.io.rewriteURI( encodedUrl ) : encodedUrl;
    com.ibm.bpm.social.util.common.Log.log(2, this._getOperationIdString() + "_executeGet(" + url + ")" ,"RestClient");

    if ( !sync )
    {
      sync = false;
    }
    
    var request =
    {
      url: url,
      load: okHandler,
      error: errorHandler,
	  timeout: 5000,
      sync: sync
    };

    if (headers)
    {
      request.headers = headers;
    }

    dojo.xhrGet(request);
	com.ibm.bpm.social.util.common.Log.logExit(2, "" ,"RestClient");
  },


  _getOperationIdString: function()
  {
    return "[" + this._queryIndex + "]";
  },


  /**
   * If url is not null, tries to remove the trailing '/' character and trim spaces.
   * If null, returns null.
   */
  _normalizeUrl: function( /*String*/ url ) /*String*/
  {
    var newUrl = url;
    if ( url )
    {
      url = dojo.trim( url );
      if( url.lastIndexOf("/") == url.length - 1 )
      {
        newUrl = url.substring(0, url.length - 1);
      }
    }

    return newUrl;
  },


  _log: function( message )
  {
    com.ibm.bpm.social.util.common.Log.log(2, message ,"RestClient");
  }
}
);


}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel"] = true;
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel");



dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel", null,
{
  constructor: function()
  {
    //this._log("constructor()");
  },
  
  
  /**
   * Searchs some repository for searchText value and returns the specified page.
   * <p>
   * Subclasses may override.
   * 
   * @return {cn,userID}[]
   */
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size ) /*{cn,userID}[]*/
  {
    return [];
  },
  
  
  /**
   * Here can return some initial suggestion, not necessarily bsacked up by VMM.
   * e.g., [{cn:"One",userID:1}, {cn:"Two",userID:2}, {cn:"Three",userID:3}]
   * <p>
   * Subclasses may override.
   * 
   * @param size The maximum number of suggestions.
   * @return {cn,userID}[]
   */
  getInitialSuggestions: function( /*int*/ size ) /*{cn,userID}[]*/
  {
    return [];
  },

  
  _removeWildcards: function( /*String*/ s ) /*String*/
  {
    var cleaned = new String();
    for(var i=0; i<s.length; i++) 
    {
      var c = s.charAt(i);
      if (c!="*")
      {
        cleaned += c;
      }
    }
    return cleaned;
  },  


  _log: function( message )
  {
    console.log( "AbstractModel: " + message );
  }  
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.BPMModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.userchooser.models.BPMModel"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.bpm.social.util.ui.userchooser.models.BPMModel");




dojo.declare( "com.ibm.bpm.social.util.ui.userchooser.models.BPMModel", com.ibm.bpm.social.util.ui.userchooser.models.AbstractModel,
{
  _tasksIds: null,
  _initialSuggestions: null,
  
  constructor: function( /*{bpmServiceRoot: String, includeGroups: boolean, includeUsers: boolean}*/ params )
  {
    //this._log("constructor()");
    this._includeGroups = params.includeGroups ? (params.includeGroups == true) : false;	// Default false
    this._includeUsers = (params.includeUsers != null && params.includeUsers != undefined) 
    							? (params.includeUsers == true) 
    							: true;	// Default true
    this._restClient = new com.ibm.bpm.social.util.ui.userchooser.util.RestClient(params.bpmServiceRoot, null);
  },
  
  
  query: function( /*String*/ searchText, /*int*/ offset, /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly ) /*{cn,userID}[]*/
  {
    return this._restClient.searchBPMByName( searchText, offset, size, this._includeGroups, this._includeUsers, assignTaskidFilter, collabTaskidFilter, namesonly );
  },
  
  getInitialSuggestions: function( /*int*/ size, /*int*/ assignTaskidFilter, /*int*/ collabTaskidFilter, /*boolean*/ namesonly  ) /*{cn,userID}[]*/
  {
	if (!this._initialSuggestions) {
		this._initialSuggestions = new Object();
	}
	var key = size + "_" + assignTaskidFilter + "_" + collabTaskidFilter;
	if (!this._initialSuggestions[key]) {
		this._initialSuggestions[key] = this._restClient.searchBPMByName( "*", 0, size, this._includeGroups, this._includeUsers, assignTaskidFilter, collabTaskidFilter, namesonly );	
	}
	var rc = new Array();
	for (var i=0; i<this._initialSuggestions[key].length; i++) {
		rc[i] = this._initialSuggestions[key][i];
	}
	return rc;
  },

  _log: function( message )
  {
    console.log( "BPMModel: " + message );
  }  
}
);

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.common.BizCard"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.common.BizCard"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.common.BizCard");







com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;

/**
 * Use the method to determine whether or not default actions should be used.
 * The purpose of this method is to abstract the different possible integrations
 * from widget-level code and centralize it in one place.
 */
com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard = function(){
	
	//TODO Possibly allow for parameters to check for only specific integrations...
	
	return typeof(connectionsBridge)=="undefined";
	
}

com.ibm.bpm.social.widgets.common.BizCard._connections = {};

com.ibm.bpm.social.widgets.common.BizCard.wrapElement = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapElement entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections(node, user);
		} 
		if(success) return;
	}
	
	node.innerHTML = "";
	
	dojo.addClass(node, "vcard");
	
	var nameNode = dojo.create("a", null, node);
	nameNode.href = "javascript:void(0);";
	nameNode.innerHTML = (typeof(user["fullName"])!="undefined") ? user["fullName"] : user["userName"];
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.wrapImage = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::wrapImage entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if(typeof(connectionsBridge)!="undefined"){
		var success = false;
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null && user["emailAddress"].length > 0){
			success = com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections(node, user);
		}
		if(success) return;
	}
	
	var container = dojo.create("span",{"class": "bpm-bizcard-container"}, node, "before");
	
	var nameNode = dojo.create("a", {"class": "bpm-bizcard-anchor"}, container);
	nameNode.href = "javascript:void(0);";
	
	dojo.place(nameNode, container, "only");
	
	dojo.place(node, nameNode, "only");
	
	var connect = dojo.connect(nameNode, "onclick", function(e){
		com.ibm.bpm.social.widgets.common.BizCard._showTooltip(nameNode, user);
		if (e) {
			dojo.stopEvent(e);
		}		
	});
	var connectId = ""+(new Date()).getTime();
	com.ibm.bpm.social.widgets.common.BizCard._connections[connectId] = connect;

	node.setAttribute("connectId", connectId);
	
};

com.ibm.bpm.social.widgets.common.BizCard.unwrapImage = function(node) {
	if(typeof(node)=="undefined" || node == null){
		return;
	}
	
	var connectId = node.getAttribute("connectId");
	if(typeof(connectId) != "undefined" && connectId != null) {
		node.setAttribute("connectId", null);
		dojo.disconnect(com.ibm.bpm.social.widgets.common.BizCard._connections[connectId]);
	}
	
	if(node.parentNode != null && typeof(node.parentNode) != "undefined" && dojo.hasClass(node.parentNode, "bpm-bizcard-anchor")) {
		var bizCard = node.parentNode.parentNode;
		if(bizCard != null && typeof(bizCard) != "undefined" && dojo.hasClass(bizCard, "bpm-bizcard-container vcard")) {
			if(bizCard.parentNode != null && typeof(bizCard.parentNode) != "undefined") {
				dojo.place(node, bizCard.parentNode);
				dojo.destroy(bizCard);
			}
		}
	}
};

com.ibm.bpm.social.widgets.common.BizCard._hideTooltip = function(){
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
};
/** Listeners to close the default business card **/
dojo.subscribe("com.ibm.bpm.socialPortal.PageTabSwitchEvent", com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.OPEN_PROCESS_DIAGRAM,com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
dojo.subscribe(com.ibm.bpm.social.portal.Topics.SIDEBAR_WIDGET_CHANGED, com.ibm.bpm.social.widgets.common.BizCard._hideTooltip);
/** End listeners to close the default business card **/

com.ibm.bpm.social.widgets.common.BizCard._showTooltip = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_showTooltip entering with node:{",node,"} and user:{",user,"}");	
	if(typeof(node)=="undefined" || typeof(user)=="undefined"){
		return;
	}
	
	if (com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog) {
		if(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy){
			com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.destroy();
		}
		com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = null;
	}
	
	var data = {
			'fullName': user['fullName'],
			'userName': user['userName'], 
			'displayName': user['fullName'],
			'title': typeof(user['userPreferences']['Title'])!='undefined' ? user['userPreferences']['Title'] : '',
			'taskEmailAddress': typeof(user['userPreferences']['Task Email Address'])!='undefined' ? user['userPreferences']['Task Email Address'] : '',
			'phoneNumber': typeof(user['userPreferences']['Phone Number'])!='undefined' ? user['userPreferences']['Phone Number'] : '',
			'includeTaskDetails': false
	};
	
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
	
	com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog = new dijit.TooltipDialog({
        content: com.ibm.bpm.social.widgets.common.BizCard._activityOwner.domNode,
        onMouseLeave: function(){
			dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        },
		onMouseEnter: function() {
			//If we get this event it means we definitely arent using an IPAD, so I can
			//cancel the timer going
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
        },
		//handled by bizCardConnect (defect 124992)
        //onClick: function(){
		//    console.log("in onclick");
        //	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
        //},
		onKeyDown: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				node.focus();
			}
		},
		onKeyUp: function(evt) {
			if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
				window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
			}
			//don't start the close timer just because of a key up (defect 124992)
			//com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();
			if(dojo.isIE || com.ibm.bpm.social.util.common.Util.isChrome) {
				var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
				if(keypressed == dojo.keys.TAB) {
					if(com.ibm.bpm.social.widgets.common.BizCard._activityOwner.handleKeyEvent(evt)) {
						dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
						node.focus();
					}
				}
			}
		}
    });
	
	//Need to close when user clicks outside of dialog, since we close when they click on the dialog
	//might as well go all the way with this.  Should clear up most issues with IPAD
	var _bizCardConnect = dojo.connect(window, "onmousedown", function(event) {
		try{
		    //only close the window when user clicks outside of the bizcard popup
			//Also use event.target and not event.originalTarget,which
			//is only supported by Firefox.
			//(defect 124992)
			if (dojo.isDescendant(event.target, com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog.domNode) == false) {
			//if(event.originalTarget.getAttribute("userName")==null){
				dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
				dojo.disconnect(_bizCardConnect);
			}
		} catch (e) {
			
		}
	});
	com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer();

	/** Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var _stack = [];
	while(dijit.popup._stack.length > 0){
		_stack.push(dijit.popup._stack.pop());
	}
	/** End Pt. 1: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	
	dijit.popup.open({
		popup: com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog,
		around: dojo.byId(node)
	});
	
	//Configure fix for accessibility defect 88867
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.setAttribute("tabindex", 0);
	com.ibm.bpm.social.widgets.common.BizCard._activityOwner.focusDiv.focus();
	
	
	node.setAttribute("tabindex", 0);
	/** Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/
	var tmp = dijit.popup._stack.pop();
	while(_stack.length > 0){
		dijit.popup._stack.push(_stack.pop());
	}
	dijit.popup._stack.push(tmp);
	/** End Pt. 2: This is a workaround from dijit.popup.open closing popups not in its direct hierarchy **/

};

/**
 * These timers are primarily used for the ipad, however it also affects desktop browser
 * behavior.  The issue is that if a user opens a dialog, but never enters it, then the dialog
 * just sticks around forever.  This will close the dialog after a certain amount of time
 * if the user doesn't "enter" the dialog
 */
com.ibm.bpm.social.widgets.common.BizCard._startCloseTimer = function() {
	com.ibm.bpm.social.widgets.common.BizCard._closeTimer = 
		window.setTimeout(com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer, 3000);
};	

com.ibm.bpm.social.widgets.common.BizCard._endCloseTimer = function() {
	if(typeof com.ibm.bpm.social.widgets.common.BizCard._closeTimer != "undefined") {
		window.clearTimeout(com.ibm.bpm.social.widgets.common.BizCard._closeTimer);
	}
	dijit.popup.close(com.ibm.bpm.social.widgets.common.BizCard._tooltipDialog);
};
/**
 * Product-specific integrations below
 * TODO: This could be refactored to allow for dynamic evaluation of available integrations
 */

/**
 * IBM Connections
 */

com.ibm.bpm.social.widgets.common.BizCard._wrapElement_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapElement_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["fullName"])!="undefined" 
			&& typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
				&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapElement(node, user["fullName"], user["emailAddress"]);
		}
	}
	return success;
};

com.ibm.bpm.social.widgets.common.BizCard._wrapImage_Connections = function(node, user){
	//console.debug("com.ibm.bpm.social.widgets.common.BizCard::_wrapImage_Connections entering with node:{",node,"} and user:{",user,"}");
	var success = false;
	if(typeof(connectionsBridge)!="undefined"){
		if(typeof(user["emailAddress"])!="undefined" && user["emailAddress"]!=null
			&& user["emailAddress"].length > 0){
			success = connectionsBridge.wrapImage(node, user["emailAddress"]);
		}
	}
	return success;
};



}

if(!dojo._hasResource["com.ibm.bpm.social.util.ui.Avatar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.util.ui.Avatar"] = true;
// RestHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// Copyright IBM Corporation 2013. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.util.ui.Avatar");










dojo.declare("com.ibm.bpm.social.util.ui.Avatar", [dijit._Widget,
                                                   dijit._Templated], {

  templateString: "",
  templateString:"<div class=\"avatarContainer\">\n\t<img dojoAttachPoint=\"img\"></img>\n\t<div class=\"onlineAwarenessIcon\" dojoAttachPoint=\"onlineIcon\"><div class=\"onlineAwarenessIconA11y\">&#9744</div></div>\n</div>\n",

  _onlineTracking: null,
  _userName: null,
  _useBizCard: false,

  /**
   * Initialize the object. Set the user object.
   * 
   * @param {String} userName : username of the user object
   */
  constructor: function constructor_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  postCreate: function postCreate_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //localization
    
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
       
    // Blank image must be set to support background image on the <img> tag
    this.img.src = dojo.moduleUrl("dojo", "resources/blank.gif");
    
    // Accessibility   
    dijit.setWaiRole(this.domNode, "img");
    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the user name.
   * 
   * @param {String} userName : username of the user object
   * @param {String} cssClass: css class for the image, as there are different sizes for example
   *        "bpm-social-default-avatar-medium"
   * @param {Boolean} useBizCard: not all avatars might use the Business Card functionality
   */
  setUser: function setUser_Avatar(user, cssClass, useBizCard){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
   
    dojo.addClass(this.img, cssClass);
    this._user = user;
    this._useBizCard = useBizCard;
    this._cssClass = cssClass;

    this._setUser();   

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Set the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  setCSSClass: function setCSSClass_Avatar(CSSClass){
    dojo.addClass(this.img, CSSClass);
  },


  /**
   * Remove the css class for the user.
   * 
   * @param {String} cssclass : name of the css class
   */
  unsetCSSClass: function unsetCSSClass_Avatar(CSSClass){
    dojo.removeClass(this.img, CSSClass);
  },

  /**
   * Create the object. Initialize the online tracking.
   * 
   * @returns {void} nothing
   */
  _setUser: function _setUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // set User
    if (this._user !== null) {
      this.img.src = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this._user["userName"]);
      this.img.alt = this._user["fullName"];
      
      // Accessibility         
      dijit.setWaiState(this.domNode, "label", this._user["fullName"]);

      //
      // Support for Online Awareness
      //      

      if (typeof OnlineTracking === "function") {
        this._deregisterUser();
        this._onlineTrackingConnects = [];
        var myself = this;
        this._onlineTracking = new OnlineTracking(this._user["userName"]);
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOnline", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.addClass(myself.onlineIcon, "online");
          myself.onlineIcon.title=myself._nlsResources["onlinetrackingTooltip"];    
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"] + ". " + myself._nlsResources["onlinetrackingTooltip"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onOffline", function(){
          dojo.removeClass(myself.onlineIcon, "online");
          dojo.addClass(myself.onlineIcon, "offline");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        this._onlineTrackingConnects.push(dojo.connect(myself._onlineTracking, "onConnectionBroken", function(){
          dojo.removeClass(myself.onlineIcon, "offline");
          dojo.removeClass(myself.onlineIcon, "online");
          myself.onlineIcon.removeAttribute("title");
          // Accessibility   
          dijit.setWaiState(myself.domNode, "label", myself._user["fullName"]);
        }));
        dojox.cometd.batch(function(){
          // Subscribe to appropriate channels
          myself._onlineTracking.activate();
        });

      }
      if (this._useBizCard) { 
    	com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.img, this._user);
    	
    	// Accessibility    	
    	dijit.setWaiState(this.img, "labelledby", this.id);  		
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy all connect from online tracking and also the online tracking object. After that a new
   * 
   * @returns {void} nothing
   */
  _deregisterUser: function _deregisterUser_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    if (this._onlineTrackingConnects) {
      for ( var i = 0; i < this._onlineTrackingConnects.length; i++) {
        dojo.disconnect(this._onlineTrackingConnects[i]);
      }
      this._onlineTrackingConnects = null;
    }
    if (this._onlineTracking != null) {
      this._onlineTracking.unregister();
      this._onlineTracking = null;
    }
    if (this._useBizCard) {
      com.ibm.bpm.social.widgets.common.BizCard.unwrapImage(this.img);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Destroy class object
   * 
   * @returns {void} nothing
   */
  destroy: function destroy_Avatar(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    this._deregisterUser();
    this.inherited(arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip");









dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-experts-expert-tooltip' role=\"dialog\" aria-describedby=\"activityOwner_tooltip_content\">\n    <div dojoAttachPoint='avatarContent'> </div>\t\n\t<div dojoAttachPoint=\"focusDiv\"></div>\n\t<div id=\"activityOwner_tooltip_content\" dojoAttachPoint=\"content\">\n\t\t<div class='displayName' dojoAttachPoint=\"displayNameField\"></div>\n\t\t<br>\n\t\t<div class='title' dojoAttachPoint=\"titleField\"></div>\n\t\t<div class='phoneNumber' dojoAttachPoint=\"phoneField\"></div>\n\t\t<div class='taskEmailAddress' dojoAttachPoint=\"emailField\"></div>\n\t\t<br>\n\t\t<div class='taskDetails' dojoAttachPoint=\"taskDetailsField\"></div>\n\t\t<div class='chat' dojoAttachPoint=\"chatStringField\"></div>\n\t</div>\n</div>\n",
	tkiid: null,
	livenames: null,	
	popup: null,
	userName: null,
	firstName: null,
	displayName: null,
	avgHours: 0,
	completed: 0,
	title: null,
	taskEmailAddress: null,
	phoneNumber: null,
	taskDetails: null,
	chatString: null,
	includeTaskDetails: true,
	tabInd: 0,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.livenames = {};
		
		if(this.displayName){
			this.displayNameField.innerHTML = this.displayName;
			this.firstName = this.displayName.split(" ")[0];
			//configure BTD on the displayNameField
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.displayNameField);
		}
		
		if(this.title) {
			this.titleField.innerHTML = this.title;
			//configure BTD on the title field
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.titleField);
		}
		
		if(this.phoneNumber) {
			this.phoneField.innerHTML = this.phoneNumber;
		}
		
		if(this.taskEmailAddress) {
			this.emailField.innerHTML = '<a href="mailto:'+this.taskEmailAddress+'">'+this.taskEmailAddress+'</a>';
		}
		
		if(this.includeTaskDetails){
			if(this.completed > 0){
				var data = {
						'userName': this.firstName,
						'average': this.avgHours,
						'completed': this.completed
				};
				this.taskDetailsField.innerHTML = dojo.replace(this.messages.expertsTooltip, data, /\$\{([^\}]+)\}/g);
			} else {
				this.taskDetailsField.innerHTML = this.messages.expertNoTaskData.replace("${user}", this.firstName);
			}
		}
				
		this._updateAvatar();
    	this._updateSametime();
	},
	
	_updateAvatar: function(){
		this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		this.avatar.setUser(user, "bpm-social-default-avatar-small", false);		
	},
	
	_updateSametime: function(){
		if(window.sametime){
			this.displayNameField.innerHTML = "";

			this.livenames[this.userName] = window.sametime.LiveName({'userId': this.userName, 'disableClicks': true, 'disableHoverBizCard': true},dojo.create("span", {}, this.displayNameField));
			this.livenames[this.userName].activate();
			
            this._createChatLinkSentence();            
			
			this._updateStatus();
			this.refresh = setInterval(dojo.hitch(this, this._updateStatus), 2000);
		}
	},

    _createChatLinkSentence: function() {
         //"${chat} ${with} ${user}"
         var s = this.messages.expertsChatTemplate.indexOf("${");
         var e = this.messages.expertsChatTemplate.indexOf("}");
         param = this.messages.expertsChatTemplate.substring(s+2,e);
         this._createChatLinkPart(param);
         for (var i=0; i<2; i++) {         
             s = this.messages.expertsChatTemplate.indexOf("${",s+1);
             e = this.messages.expertsChatTemplate.indexOf("}",e+1);
             param = this.messages.expertsChatTemplate.substring(s+2,e);
             this._createChatLinkPart(param);
         }         
    },

    _createChatLinkPart: function(param) {
         if (param == "chat") {
             this._createChatLink();
         } else if (param == "with") {
             this._createChatWith();
         } else if (param == "user") {
             this._createChatUser();
         } else {
             console.log("_createChatLinkPart WARNING invalid parameter: "+param);
         }
    },

    _createChatLink: function() {
			var link = dojo.create("a", {
				'href': 'javascript:void(0)',
				'username': this.userName,
				'class': 'chatLink',
             'innerHTML': this.messages.expertsChat + "&nbsp;"
			}, this.chatStringField);
			dojo.connect(link, 'onclick', this, this._handleChat);
    },

    _createChatWith: function() {
         dojo.create("span", {
             'innerHTML': this.messages.expertsChatCoda + "&nbsp;"
			}, this.chatStringField);

    },

    _createChatUser: function() {
         dojo.create("span", {
             'innerHTML': this.displayName + "&nbsp;"
         }, this.chatStringField);
	},
	
	_updateStatus: function(){
		dojo.removeClass(this.domNode, "online");
		dojo.removeClass(this.domNode, "offline");
		
		if(this.livenames && this.livenames[this.userName]){
			switch(this.livenames[this.userName].currentTextStyle){
				case "stproxy_liveNameAvailableText":
				case "stproxy_liveNameAwayText":
				case "stproxy_liveNameInMeetingText":
					dojo.addClass(this.domNode, "online");
					this.presenceStatus = "online";
					break;
				case "stproxy_liveNameOfflineText":
				case "stproxy_liveNameDndText":
				default:
					dojo.addClass(this.domNode, "offline");
					this.presenceStatus = "offline";
					break;
			}
		} else {
			dojo.style(this.chatStringField, "display", "none");
		}
		
	},
	
	_handleChat: function(e){
		var uname = e.target.getAttribute('username');
		//Following code was added for defect 112076 where the chat link would not work in IE9 or 10. In IE9 and IE10, 
		//we were getting a not logged in error, so added code to check the login status and login if we were not.
		if (dojo.isIE && !window.stproxy.isLoggedIn) {
			var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();
			window.stproxy.login.loginByToken(
				user.userName,
				window.stproxy.awareness.AVAILABLE,
				"",
				dojo.hitch(this, this.loginOK), 
				dojo.hitch(this, this.loginFailed)
			);
		}
		if(this.livenames[uname]){
			this.livenames[uname].startChat();
		}
	},
	
	
	loginOK: function(){
		//console.debug("Login succeeded...");
	},
		
	loginFailed: function(){
		console.error("Login failed...");
	},

	
	//Configure fix for accessibility defect 88867: Need to manually manage tab index and key events
	//Will Return a boolean determining whether the dialog should be closed.
	handleKeyEvent: function(evt) {
		console.log("gettingcalled");
		var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
		if(keypressed == dojo.keys.TAB) {
			console.log("tabkey!");
			if(evt.shiftKey) {
				console.log("decrement");
				this.tabInd--;
			} else {
				console.log("increment");
				this.tabInd++;
			}
			console.log("tabind = " + this.tabInd);
		}
		if (this.tabInd == -1 ||
			this.tabInd == 2 ||
			(this.tabInd == 1 && this.emailField.innerHTML == "")) {
			//close the dialog, because tab order is outside the dialog
			console.log("close");
			return true;
		}
		//otherwise let the tab order do its natural thing
		return false;
	},
				
	destroy: function() {
		if(this.refresh){
			clearInterval(this.refresh);
		}
		
		if (this.avatar) {
		   this.avatar.destroy();
		   this.avatar = null;
		}
		
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.experts.DialogExpertTile"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.experts.DialogExpertTile"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.DialogExpertTile");












/**
 * TODO This class needs to be consolidate with the ExpertTile.
 */
dojo.declare("com.ibm.bpm.social.widgets.experts.DialogExpertTile", [dijit._Widget,
                                                                     dijit._Templated], {

  templateString: "",
  templateString:"<div class='bpm-social-experts-expert' dojoAttachPoint='expert' dojoAttachEvent='onmouseenter:_doMouseEnter, onmouseleave:_doMouseLeave, onkeydown:_onKeyDown'>\n    <div dojoAttachPoint='avatarContent'> </div>\t\n\t<div class='bpm-social-experts-expert-data-div'>\t    \n\t\t<div class='bpm-social-experts-expert-name' dojoAttachPoint='expertname' role=\"listitem\"></div>\n\t\t<div class='bpm-social-experts-expert-details taskDetails' dojoAttachPoint='taskDetails'></div>\n\t\t<div class='bpm-social-experts-expert-actions-div'>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-mention' dojoAttachPoint=\"mentionDiv\">\n\t\t\t\t<a dojoAttachPoint='mention' href='javascript:void(0)' dojoAttachEvent='onclick:_handleMention'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator1' class='vertical-bar' style='display:block'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-help' dojoAttachPoint=\"helpDiv\">\n\t\t\t\t<a dojoAttachPoint='gethelp' href='javascript:void(0)' dojoAttachEvent='onclick:_handleGetHelp'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator2' class='vertical-bar'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-chat' dojoAttachPoint=\"chatDiv\">\n\t\t\t\t<a dojoAttachPoint='startchat' tabIndex='-1' href='javascript:void(0)' dojoAttachEvent='onclick:_handleChat'></a>\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n",

  /**
   * Input provided in the constructor based on the experts call.
   */
  avgHours: 0,
  completed: 0,
  email: null,
  fullName: null,
  workload: 0,
  userName: null,

  /**
   * Added information for fine tuning available functions also provided using the constructor.
   */
  categoryName: null, // If set, will be shown in the taskDetails section
  collaborate: false, // If true, collaboration action is shown  
  includeTaskDetails: false, // If true, shows the task details i.e. the statistics (avgHours, completed...)
  sidebar: false, // If true, assumes that it is rendered in the sidebar.
  taskId: null, // Added to allow for collaboration... required if includeTaskDetails == true


  livename: null, // Set by sametime integration logic
  imageUrl: null, // ??
  phoneNumber: null, // ??
  title: "", // ?
  refresh: null, // Used for timeout  handle
  
  /**
   * 
   */
  postCreate: function(){
    this.inherited(arguments);
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    
    //
    // extends of template
    //
    var div = document.createElement("div");
    dojo.addClass(div, "bpm-social-experts-expert-icon");
    this.expert.appendChild(div);
    var divA11y = document.createElement("div");
    dojo.addClass(divA11y, "bpm-social-experts-expert-icon-a11y");
    this.domNodeA11y = divA11y;
    div.appendChild(divA11y);
       
    //
    // Basic Setup
    //
    
    this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
    this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Create a unique id to be able to refer to the name field
    // It is dijit._Widget that has a unique id.
    var nameId = this.id + "_name";
    dojo.attr(this.expertname, "id", nameId);
    this.domNode.title = "";
    //
    // Retrieve preferences of current user -> this.userData
    // Retrieve user avatar and add it the div[#expert-avatar]
    //
    this._getUserData();
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    if (!this.fullName && user && user.fullName) {
      this.fullName = user.fullName;
    }
    this.expertname.innerHTML = this.fullName;

    if (window.sametime && this.fullName) {
      // the text-overflow:ellipsis style does not function properly
      // when there are span elements within the div, so clip the
      // text instead if we are using sametime
      this.expertname.style.textOverflow = 'clip';
      // Overwrite name and add the live name
      this.expertname.innerHTML = "";
      this.livename = window.sametime.LiveName({
        'userId': this.userName,
        'disableClicks': true,
        'disableHoverBizCard': true
      }, dojo.create("span", null, this.expertname));
      this.livename.activate();

      this._updateStatus();
      this.refresh = window.setInterval(dojo.hitch(this, this._updateStatus), 2000);
    }
    
    this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
    this.avatar.setUser(user, "bpm-social-default-avatar-medium", false);
    
    /** Business Card integration * */
    /** Default * */
    var node = this.expertname;
    if (this.livename != null) {
      var nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
      if (nodes.length == 1) {
        node = nodes[0];
        var img = dojo.create("img", {
          "src": dojo.moduleUrl("dojo", "resources/blank.gif"),
          "class": "connectionsOverlayImage"
        }, dojo.query(".stproxy_general", this.domNode)[0], "first");
        com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
        var divA11y = document.createElement("div");
        divA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
        divA11y.addClass("bpm-social-experts-expert-name-a11y");
        this.onlineTrackingDomNodeA11y = divA11y;
        img.appendChild(divA11y);
      }
    }

    /*
    // Do not show the BizCard as clicking on the tile selects the user and 
    // should not show the BizCard
    com.ibm.bpm.social.widgets.common.BizCard.wrapElement(node, user);
    com.ibm.bpm.social.widgets.common.BizCard.wrapImage(dojo.query("img", this.img)[0], user);
    */
    /** End Business Card integration * */


    //
    // Adjusting functionality based on configuration of instance.
    // Processing according to display in rtl direction
    //
    if (this.categoryName) {
      this.taskDetails.appendChild(document.createTextNode(this.categoryName));
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    else if (this.includeTaskDetails) {
      // If includeTaskDetails add content to dojoAttachPoint taskDetails
      var data;
      var firstName = this.fullName.split(" ")[0];
      this._updateUserData();
      if (this.completed == 0) {
        this.taskDetails.innerHTML = this.messages.expertNoTaskData.replace("${user}", firstName);
      }
      else {
        data = {
          'userName': firstName,
          'average': this.avgHours,
          'completed': this.completed
        };
        //defect 149810 Fix Expert tooltip wording
        this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip_short, data, /\$\{([^\}]+)\}/g);
      }
      dojo.attr(this.taskDetails, "id", "taskDetails_" + nameId);
      dijit.setWaiState(this.expertname, "describedby", "taskDetails_" + nameId);
    }
    // 
    // Add actions for mention | collaborate | chat
    // Remove mention if not in sidebar
    // 
    this.mention.innerHTML = "@" + this.messages.expertsMention;
    this.gethelp.innerHTML = this.messages.expertsCollaborate;
    this.startchat.innerHTML = this.messages.expertsChat;
    // Refering to the name field
    dijit.setWaiState(this.mention, "describedby", nameId);
    dijit.setWaiState(this.gethelp, "describedby", nameId);
    dijit.setWaiState(this.startchat, "describedby", nameId);
    this.actionSeparator1.innerHTML = this.messages.expertsSeparator;
    this.actionSeparator2.innerHTML = this.messages.expertsSeparator;
    // Downgrade actions if not in sidebar
    if (!this.sidebar) {
      this.mention.style.display = 'none';
      this.actionSeparator1.style.display = 'none';
    }
    //
    // The collaborate constructor parameter allows to configure whether collaboration action is supported.
    //
    if (!this.collaborate) {
      this.actionSeparator2.style.display = 'none';
      this.helpDiv.style.display = 'none';
    }
    //
    // Remove chat action
    //
    if (!(window.sameTime && this.fullName)) {
      dojo.style(this.chatDiv, "display", "none");
    }
  },

  _handleMention: function(e){
    var taskCompletionView = this._getTaskCompletionView();
    if (taskCompletionView) {
      // switch to the stream tab
      taskCompletionView.sidebar.selectTab(1);
      // set focus to input box
      taskCompletionView.stream._handlePostInitialFocus();
      // tag user
      taskCompletionView.stream.tagUser(this.userName, this.fullName);
    }
    dojo.stopEvent(e);
  },

  _handleGetHelp: function(e){
    var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
      "taskId": this.taskId
    });
    com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(taskData, {
      userName: this.userName,
      fullName: this.fullName
    });
    dojo.stopEvent(e);
  },

  _handleChat: function(e){
    if (this.livename && this.presenceStatus == "online") {
      this.livename.startChat();
    }
    dojo.stopEvent(e);
  },

  _showTooltip: function(){
    return;
  },

  _hideTooltip: function(){
    if (this.myTooltipDialog) {
      this.myTooltipDialog = null;
    }

  },

  _updateUserData: function(){
    var self = this;
    this.socialRestHandler.getExpertUserData(this.taskId, this.userName, function(data, xhrArgs){
      self.avgHours = data.avgHours;
      self.completed = data.completed;
    }, "", false);
  },

  _getUserData: function(){
    var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
    /*if(user && user.userPreferences && user.userPreferences.Image){
    	this.img.innerHTML = "<img alt='"+this.fullName+"' src='"+user.userPreferences.Image+"' />";
    }*/
    this.userData = user.userPreferences;
    if (!this.userData) {
      this.userData = new Object();
    }
  },

  _updateStatus: function(){
    dojo.removeClass(this.domNode, "online");
    dojo.removeClass(this.domNode, "offline");

    if (this.livename) {
      switch (this.livename.currentTextStyle) {
        case "stproxy_liveNameAvailableText" :
        case "stproxy_liveNameAwayText" :
        case "stproxy_liveNameInMeetingText" :
          dojo.addClass(this.domNode, "online");
          this.presenceStatus = "online";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-online"];
          }
          break;
        case "stproxy_liveNameOfflineText" :
        case "stproxy_liveNameDndText" :
        default:
          dojo.addClass(this.domNode, "offline");
          this.presenceStatus = "offline";
          if (this.onlineTrackingDomNodeA11y) {
            this.onlineTrackingDomNodeA11y.innerHTML = this._nlsResources["onlinetracking-offline"];
          }
          break;
      }
    }
    else {
      dojo.style(this.chatDiv, "display", "none");
    }

    /** Business Card integration * */
    if (!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard() && this.livename != null) {
      /** Account for Sametime DOM structure, which can be delayed in being created due to network latency * */
      var nodes = dojo.query(".stproxy_liveNamePadding .hasHover", this.expertname);
      if (nodes.length == 0) {
        nodes = dojo.query(".stproxy_liveNamePadding", this.expertname);
        if (nodes.length == 1) {
          var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
          if (!this.fullName) {
            this.fullName = user.fullName;
          }
          com.ibm.bpm.social.widgets.common.BizCard.wrapElement(nodes[0], user);
        }
      }
    }
    /** End Business Card integration * */

  },


  _getTaskCompletionView: function(){
    var node = this.domNode;
    while (node && !dojo.hasClass(node, 'workViewCompletion')) {
      node = node.parentNode;
    }
    if (node == null) {
      node = dojo.query('.workViewCompletion')[0];
    }
    var taskCompletionView = dijit.byId(node.id);
    return taskCompletionView;
  },


  _doMouseEnter: function(e){
    dojo.addClass(this.expert, 'hover');
  },

  _doMouseLeave: function(e){
    dojo.removeClass(this.expert, 'hover');
  },

  _onFocus: function(){
    dojo.addClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onBlur: function(){
    dojo.removeClass(this.domNode, 'bpm-social-experts-focus');
  },

  _onKeyDown: function(e){
    if (e.keyCode === dojo.keys.RIGHT_ARROW && this.livename) {
      this.startchat.focus();
    } 
  },
 
  destroy: function(){
    if (this.refresh) {
      window.clearInterval(this.refresh);
    }
    
    this._hideTooltip();

    if (this.avatar) {
        this.avatar.destroy();
        this.avatar = null;
    }

    this.inherited(arguments);
  }

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.InvitationDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.dialogs.InvitationDialog"] = true;
/*
 * Licensed Materials - Property of IBM 5725-C96 Copyright IBM Corporation 2012,2013. All Rights Reserved. US
 * Government Users Restricted Rights- Use, duplication or disclosure restricted by GSA ADP Schedule Contract
 * with IBM Corp.
 */
dojo.provide("com.ibm.bpm.social.widgets.dialogs.InvitationDialog");











/**
 * Invitation dialog used when inviting people to a group task.
 * 
 */
dojo.declare("com.ibm.bpm.social.widgets.dialogs.InvitationDialog", [com.ibm.bspace.common.util.ui.Dialog], {

  templateString:"<div dojoAttachPoint='content' class=\"bpm-task-invitation-dialog\">\t\t\n\t<div dojoAttachPoint=\"invitedUserSection\" class=\"bpm-task-invitation-dialog-invitedUsers-section\">\t\t\n\t\t<h2 title=\"${labelForInvitedUsersSubTitle}\" class=\"bpm-task-invitation-dialog-invitedUsers-title\" dojoAttachPoint=\"userChooserLabel\">${labelForInvitedUsersSubTitle}</h2>\t\t\t\t\t\t\t\t\t\n   </div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"collaboratorsSectionDummy\"></div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"suggestedCollaboratorsSection\" class=\"bpm-task-invitation-dialog-suggestions-section\" title=\"${labelForSuggestedCollaboratorTooltip}\">\n\t      <h2 class=\"bpm-task-invitation-dialog-suggestions-title\" dojoAttachPoint=\"suggestionsTitle\">${labelForSuggestedCollaboratorTitle}</h2>\t\t\n\t\t  <div dojoAttachPoint=\"suggestedCollaborators\"></div>\n\t\t<div>\n\t\t\t<div tabIndex=\"0\" dojoAttachPoint=\"showMore\" class=\"bpm-task-invitation-dialog-suggestions-more\">${labelForMore}</div>\n\t\t\t<div tabIndex=\"0\" dojoAttachPoint=\"showBack\" class=\"bpm-task-invitation-dialog-suggestions-back\">${labelForBack}</div>\t\t\t\n\t\t</div>\n\t\t<div class=\"bpm-task-invitation-dialog-suggestions-clear\"></div>\n\t</div>\n\t<div tabindex=\"-1\" dojoAttachPoint=\"invitedCollaboratorsSection\" class=\"bpm-task-invitation-dialog-suggestions-section\" title=\"${labelForInvitedCollaboratorTooltip}\">   \n\t      <h2 class=\"bpm-task-invitation-dialog-suggestions-title\">${labelForInvitedCollaboratorTitle}</h2>\t\t\n\t\t  <div dojoAttachPoint=\"invitedCollaborators\"></div>\n\t\t  \t\t<div>\n\t\t\t<div dojoAttachPoint=\"showMoreInvited\" class=\"bpm-task-invitation-dialog-suggestions-more\">${labelForMore}</div>\n\t\t\t<div dojoAttachPoint=\"showBackInvited\" class=\"bpm-task-invitation-dialog-suggestions-back\">${labelForBack}</div>\t\t\t\n\t\t</div>\n\t\t<div class=\"bpm-task-invitation-dialog-suggestions-clear\"></div>\n    </div>\t\n\t\t\n\t<div dojoAttachPoint=\"messageSection\" class=\"bpm-task-invitation-dialog-message-section\">\n\t\t<h2 class=\"bpm-task-invitation-dialog-message-title\">${labelForMessageSection}</h2>\n\t\t<label dojoAttachPoint=\"messageBoxLabel\" style=\"display: none\">${labelForMessageSection}</label>\n\t\t<textarea dojoAttachPoint=\"messageBox\" rows=\"3\" cols=\"85\" placeholder=\"${labelForMessageSectionPlaceholder}\"></textarea>\n\t</div>\n\t\n\t<div class=\"bpm-task-invitation-dialog-button-section\" style=\"background: white\">\n\t\t<button dojoAttachPoint=\"inviteButton\" class=\"bpm-task-invitation-dialog-button-invite\" title=\"${labelForInviteButtonTitle}\">${labelForInviteButton}</button>\n\t</div>\n\t\t\n</div>\n",

  constructor: function(parameters){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Life cycle management:
    // As dijit._Widget this.connect should be used in order have everything freed upon destruction.
    // The destroy method will be called in event handler to onHide.
    // Might not be used as the com.ibm.bspace.common.util.ui.Dialog.postCreate already calls Helper.wrapElement that 
    // wraps the elements out of the template and should take care of them when destroying.
    //
    this._wrappedElements = [];

    // TODO There could be a configurable limit. So far it is 5. If it changes we need to ask for that limit.
    this.COLLAB_LIMIT = 5;
    //
    // Looking up helper
    //
    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
    // Processing of parameters
    //
    var task = parameters.task;

    //
    // labels used in the template
    //    
    this.labelForInvitedUsers = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTitle"];
    this.labelForInvitedUsersSubTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle"];    
    this.labelForInvitedUsersSubTitleWithoutSection =
    this._nlsResources["task_dialog_startCollaboration_invitedUserSectionSubTitle_WithoutSection"];
    this.labelForInvitedUsersTitle = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTooltip"];
    this.labelForMessageSection = this._nlsResources["task_dialog_startCollaboration_messageSectionTitle"];
    this.labelForMessageSectionPlaceholder = this._nlsResources["task_dialog_startCollaboration_messageSectionPlaceholder"];
    // this.labelForMessageSectionTitle = this._nlsResources["task_dialog_startCollaboration_messageSectionTooltip"];
    this.labelForInviteButton = this._nlsResources["task_dialog_startCollaboration_saveButton"];
    this.labelForInviteButtonTitle = this._nlsResources["task_dialog_startCollaboration_saveButtonTooltip"];
    this.labelForSuggestedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_suggestedCollaborators"];
    this.labelForInvitedCollaboratorTitle = this._nlsResources["task_dialog_startCollaboration_invitedCollaborators"];
    this.labelForSuggestedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_suggestedCollaboratorsTooltip"];
    this.labelForInvitedCollaboratorTooltip =
      this._nlsResources["task_dialog_startCollaboration_invitedCollaboratorsTooltip"];
    this.labelForMore = this._nlsResources["task_dialog_startCollaboration_more"];
    this.labelForBack = this._nlsResources["task_dialog_startCollaboration_back"];
    //
    //
    // Normalizing properties because the properties used by query are different from
    // the properties returned by GET /task
    // Although there is currently no use case where the provided task is retrieved using a query.
    //
    if (typeof task.TKIID === "string") {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping;
    }
    else {
      this.mapping = com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping;
    }
    this._availableActions = parameters.availableActions;
    this._tkiid = task[this.mapping["TKIID"]];
    this._taskName = task[this.mapping["NAME"]];
    this._taskDisplayName = task[this.mapping["DISPLAY_NAME"]];
    this._processInstanceName = task[this.mapping["PI_NAME"]];
    this._piid = task[this.mapping["PIID"]];
    // Remember whether a user is expert or frequent collaborator
    this._userCategory = {};
    //
    // Assumption: task has already an owner. Otherwise inviting does not make sense.
    // 
    this._currentOwner = {
      value: task[this.mapping["OWNER"]],
      type: "user"
    };
    this.placeHolderState = {
      modified: true
    };
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  postCreate: function postCreate_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calling com.ibm.bspace.common.util.ui.Dialog.postCreate and thereby wrapping the radio buttons and buttons.
    // Because the Dialog.postCreate method calls com.ibm.bspace.common.util.ui.Helper.wrapElement which
    // finds all markup that needs to be wrapped.
    //
    this.inherited("postCreate", arguments);
    this.connect(this.popup, 'onHide', function(){
      // Basically this is also a dijit._Widget with a meaningful destroy method.
      // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
      setTimeout(dojo.hitch(this, this.destroy), 1000);
    });
    
    // Accessibility
    dijit.setWaiRole(this.suggestedCollaborators, "listbox");
    var suggestionsTitleId = this.id + "_suggestionsTitle";
    this.suggestionsTitle.id = suggestionsTitleId;
    dijit.setWaiState(this.suggestedCollaborators, "labelledby", suggestionsTitleId);
    dijit.setWaiRole(this.showMore, "button");     
    dijit.setWaiRole(this.showBack, "button");
    
    //
    // Custom Logic Starts Here
    //    
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  showDialog: function showDialog_InvitationDialog(user){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var expertsBoxDim = this._calculateSize();
    // get the number of experts per line before the invited users are inserted
    this._calculateNumberOfExperts();
    this._setupUserChooser(user);
    if (user == null) {
      // user is not already known
      this._setupSuggestedCollaborators();
    }
    // insert the invited users in the dialog, that can be done independent of the experts callback
    // this._setupInvitedUsers();
    this.connect(this.inviteButton, "onclick", "onInvite");
    // if there is a limit of allowed collaborators then we do not need to create the whole dialog. 
    // In that case do nothing.     
    this._setupMessageBox();
    // button is disabled 
    com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, true);
    // Remove the tooltip for the dialog
    this.domNode.removeAttribute("title");
    this.show();
    // Set the width of the dialog to assure that there is no white space,
    // if the title is too long.
    dojo.style(this.popup.domNode, "width", expertsBoxDim.w + "px");
    this._connectKey = dojo.connect(this.domNode, "onkeydown", this, "_onKeyDown");
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },


  destroy: function destroy_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);

    //
    // Assumption: Connections established with this.connect are disconnected in the destroy method of dijij._Widget
    //
    if (this._connectKey) {
      dojo.disconnect(this._connectKey);
      this._connectKey = null;
    }

    if (this._wrappedElements.length > 0) {
      for ( var i = 0; i < this._wrappedElements.length; i++) {
        com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements);
      }
    }
    if (this._expertsWidgets != null) {
      for ( var j = 0; j < this._expertsWidgets.length; j++) {
        var widget = this._expertsWidgets[j];
        widget.destroy();
      }
      this._expertsWidgets = null;
    }
    if (this._invitedWidgets != null) {
      for ( var j = 0; j < this._invitedWidgets.length; j++) {
        var widget = this._invitedWidgets[j];
        widget.destroy();
      }
      this._invitedWidgets = null;
    }
    this._restHandler = null;
    this._errorHandler = null;
    this._messageField = null;
    this.inherited("destroy", arguments);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _onKeyDown: function _onKeyDown_InvitationDialog(e){
    var key = e.keyCode;
    if (key == dojo.keys.ESCAPE) {
      this.hide();
    }
  },
 
  _calculateSize: function _calculateSize_InvitationDialog(){
    // calculate the size of the area necessary for the experts/invitee box
    var size = dojo.window.getBox();
    var width = size.w;
    var height = size.h;
    // this is the size we should use. Otherwise the dialog is smaller and has scrollbars
    var expertsBoxHeight = 300;
    var expertsBoxWidth = 675;
    // the width of the window plus 32 for the rest of the dialog need to be there
    if (width - 32 < expertsBoxWidth) {
      expertsBoxWidth = (width - 32) > 0 ? width - 32 : width;
    }
    // the height of the window plus 250 for the rest of the dialog needs to be there 
    if (height - 250 < expertsBoxHeight) {
      expertsBoxHeight = (height - 250) > 0 ? height - 250 : height;
    }
    // make the dummy element that size and make it visible till the dialog is shown
    dojo.style(this.content, "width", expertsBoxWidth + "px");
    dojo.style(this.collaboratorsSectionDummy, "height", expertsBoxHeight + "px");
    dojo.style(this.collaboratorsSectionDummy, "display", "block");
    
    return {"h": expertsBoxHeight, "w": expertsBoxWidth};
  },

  _calculateNumberOfExperts: function _calculateNumberOfExperts_InvitationDialog(){
    var expertsBoxWidth = 630;
    if (this.content && this.content.offsetWidth) {
      // the scrollbar also needs some px this reduce 15
      expertsBoxWidth = this.content.offsetWidth - 25;
    }
    // for 3 experts tiles we need 600
    if (expertsBoxWidth >= 630) {
      this._numberExpertsPerLine = 3;
    }
    else {
      // otherwise calculate how many fit
      var mod = expertsBoxWidth % 210;
      this._numberExpertsPerLine = ((expertsBoxWidth - mod) / 210);
    }
    // now make room for the real experts or just make it small
    dojo.style(this.collaboratorsSectionDummy, "height", 0 + "px");
  },

  _checkButtonEnablement: function _checkButtonEnablement_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var valid = true;
    valid = (this.userChooser.chooserInput.value.length > 0 && this.userChooser.getChoice() !== null);
    if (this.inviteButton) {
      com.ibm.bspace.common.util.ui.Button.setDisabled(this.inviteButton, !valid);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
  },

  _setupUserChooser: function _setupUserChooser_InvitationDialog(prepopulatedUseridString){
    //
    // TODO Make sure that (de-) selecting suggested collaborators plus entering users in the user chooser is synchronized.
    //
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    //
    // Create and configure the user chooser
    //
    var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
      includeGroups: false,
      includeUsers: true
    });
    var myself = this;
    myself._ucClickedFirstTime = false;
    var args = {
      searchMinInputLength: 0,
      ucModel: ucModel,
      _onUCFocus: function(e){
        if (!myself._ucClickedFirstTime) {
          myself._ucClickedFirstTime = true;
          dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
          myself.userChooser.chooserInput.value = "";
        }
        com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);

        // myself.userChooser.chooserInput.value = "";
        myself._checkButtonEnablement();
        for ( var j = 0; j < myself._expertsWidgets.length; j++) {
          var widget = myself._expertsWidgets[j];
          if (widget.isAdded) {
            widget.isAdded = false;
            dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
            widget.domNodeA11y.innerHTML = "";
          }
        }
      },
      _onUCBlur: function(e){
        var choice = myself.userChooser.getChoice();
        var valid = (myself.userChooser.chooserInput.value.length < 0 || choice !== null);
        if (!valid) {
          myself.userChooser.chooserInput.value = "";
          myself.userChooser._hideResults();
          myself._checkButtonEnablement();
        }
        else {
          // make sure only the user specified in the user chooser is marked for invite
          for ( var j = 0; j < myself._expertsWidgets.length; j++) {
            var widget = myself._expertsWidgets[j];
            if (widget.userName == choice.userName) {
              dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "&#10004;";
              widget.isAdded = true;
            }
            else if (widget.isAdded) {
              widget.isAdded = false;
              dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
              widget.domNodeA11y.innerHTML = "";
            }
          }
        }
        dojo.addClass(myself.userChooser.noUsersFoundDiv, "hidden");
      }
    };
    this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
    this.connect(this.userChooser, "onChangeHandler", "_checkButtonEnablement");
    //
    // initially show a help text in the input field
    //      
    this.userChooser.chooserInput.value = this._nlsResources["task_dialog_startCollaboration_invitedUseridFieldHelp"];
    dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    //
    // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
    //
    this.userChooser.setCollabTaskidFilter(this._tkiid);

    // allow to set a string in the UserChooser field, e.g. for calling this dialog
    // from the Experts List, thereby pre-populating the selected user from the Experts list.
    if (prepopulatedUseridString) {
      if (prepopulatedUseridString.fullName) {
        this.userChooser._chooseItemValue(prepopulatedUseridString);
      }
      else {
        this.userChooser.chooserInput.value = prepopulatedUseridString;
      }
      myself._ucClickedFirstTime = true;
      dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    }

    // A11y: add label for user chooser
    var ucLabel = document.createElement("label");
    ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
    ucLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_invitedUseridField_label"];
    ucLabel.style.display = "none";

    this.invitedUserSection.appendChild(ucLabel);
    this.invitedUserSection.appendChild(this.userChooser.domNode);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _setupMessageBox: function _setupMessageBox_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // 
    // Calculate the default invitation message
    //
    var taskLabel = this._taskDisplayName;
    if (!taskLabel || taskLabel === "") {
      taskLabel = this._taskName;
    }
    var instanceName = this._processInstanceName;
    if (!instanceName || instanceName === "") {
      instanceName = this._piid;
    }
    // Set id and for appropriately
    this.messageBox.id = this.popup.id + "_MessageBox";
    this.messageBoxLabel.htmlFor = this.popup.id + "_MessageBox";
    // Use placeholder instead
    /*
    var msg = dojo.string.substitute(this._nlsResources["task_dialog_startCollaboration_messageDefault"], {
        1: taskLabel,
        2: instanceName
      });
      this.messageBox.appendChild(document.createTextNode(msg));
    */
    //configure base text direction here:
    com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this.messageBox);
    if (dojo.isIE) {
      this._addPlaceholder(this.messageBox, this.labelForMessageSectionPlaceholder, this.placeHolderState);
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _addPlaceholder: function(textarea, placeHolderText, placeHolderState){
    var fontStyle = "normal";
    if (textarea.style.fontStyle != null) {
      fontStyle = textarea.style.fontStyle;
    }
    var setPlaceHolderText = function(){
      textarea.value = placeHolderText;      
      textarea.style.fontStyle = "italic";
    };
    var removePlaceHolderText = function(){
      textarea.value = "";      
      textarea.style.fontStyle = fontStyle;
    };
    // Initial Setup 
    placeHolderState.modified = false;
    setPlaceHolderText();
    // Focus received
    dojo.connect(textarea, "onfocus", function(){
      if (!placeHolderState.modified) {
        removePlaceHolderText();
      }
    });
    // Focus lost
    dojo.connect(textarea, "onblur", function(){
      if (textarea.value != "") {
        placeHolderState.modified = true;
      }
      else {
        placeHolderState.modified = false;
        setPlaceHolderText();
      }
    });

  },

  _setupSuggestedCollaborators: function _setupSuggestedCollaborators_InvitationDialog(){
    this._socialRestHandler.getExpertsForTask(this._tkiid, dojo.hitch(this, this._expertsCallback), function(error){
    // TODO Error Handling for experts call
    }, false /* not async */);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _removeCurrentUser: function _removeCurrentUser_InvitationDialog(group){
    if (group && this._currentUser) {
      for ( var i = 0; i < group.length; i++) {
        if (group[i].userName == this._currentUser) {
          group.splice(i, 1);
          i--;
        }
      }
    }
  },

  _removeDuplicateUsers: function _removeDuplicateUsers_InvitationDialog(systemIdentified, expertParticipant){
    // calculate the invited users to delete them form the experts lists
    // they are displayed separately.	
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    if (expertParticipant && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < expertParticipant.length; i++) {
        var userName = expertParticipant[i].userName;
        /*
            var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);        
            if (filter > -1) {
              expertParticipant.splice(i, 1); // splicing the current item shift the items to the left
              i--;
            }
        */
        this._userCategory[userName] = categoryName;
      }
    }
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    if (systemIdentified && this.task.collaboration && this.task.collaboration.currentUsers) {
      for ( var i = 0; i < systemIdentified.length; i++) {
        var userName = systemIdentified[i].userName;
        /*
        var filter = dojo.indexOf(this.task.collaboration.currentUsers, userName);
        */
        if (typeof this._userCategory[userName] === "string") {
          // Already an expert 
          systemIdentified.splice(i, 1); // splicing the current item shift the items to the left     
          i--;
        }
        if (typeof this._userCategory[userName] === "undefined") {
          // Rememeber the type which is not already "Expert"
          this._userCategory[userName] = categoryName;
        }
      }
    }
  },

  _expertsCallback: function _expertsCallback_InvitationDialog(data){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // Establish user's name
    var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
    if (userProfileObj) {
      this._currentUser = userProfileObj.userName;
    }

    var anker = this.suggestedCollaborators;
    var taskId = this._tkiid;
    var counter = 0;
    var systemIdentified = data["systemIdentifiedUsers"];
    var expertParticipantGroup = data["expertParticipantGroupUsers"];
    var numberExpertsPerLine = this._numberExpertsPerLine;
    var displayedUsers = {};
    // Keep track of ExpertTile objects in order to destroy them later
    var expertsWidgets = this._expertsWidgets = [];
    // TODO Check whether the categories (Frequent Collaborator, Experts) can also be user defined in the plugable member service
    var categoryName = this._nlsResources["task_dialog_startCollaboration_category_frequentCollaborator"];
    // remove the current user from the lists
    this._removeCurrentUser(systemIdentified);
    this._removeCurrentUser(expertParticipantGroup);
    // remove the user which are already invited
    // insert system suggested users        
    this._removeDuplicateUsers(systemIdentified, expertParticipantGroup);
    var tileLabel = this._nlsResources["task_dialog_startCollaborationSuggested_title"];
    var myself = this;
    dojo.forEach(systemIdentified, function(item){     
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
    categoryName = this._nlsResources["task_dialog_startCollaboration_category_expert"];
    // insert modeled experts
    dojo.forEach(expertParticipantGroup, function(item){      
      item['taskId'] = taskId;
      item['includeTaskDetails'] = false;
      item['sidebar'] = false;
      item['collaborate'] = false;
      item['categoryName'] = categoryName;
      var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
      myself._createAccForDialogExpertTile(ee, counter);
      counter++;
      ee.domNode.title = tileLabel;
      expertsWidgets.push(ee);
      anker.appendChild(ee.domNode);
      if (counter > numberExpertsPerLine) {
        dojo.style(ee.domNode, "display", "none");
      }
    });
        
    for ( var j = 0; j < this._expertsWidgets.length; j++) {
      var widget = this._expertsWidgets[j];
      this.connect(widget, "onClick", "_expertClicked");
    }
    
    if (counter == 0) {
      dojo.style(this.suggestedCollaboratorsSection, "display", "none");
      this.userChooserLabel.innerHTML=this.labelForInvitedUsersSubTitleWithoutSection;      
      this.userChooserLabel.setAttribute("title",this.labelForInvitedUsersSubTitleWithoutSection);
      com.ibm.bpm.social.util.common.Log.logExit(2);
      return;
    }
    else {
      dojo.style(this.suggestedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMore, "display", "none");
    }
    else {
      dojo.style(this.showMore, "display", "block");
    }
    
    this.connect(this.showMore, "onclick", function(){
    	this._showMore(anker);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });
    this.connect(this.showMore, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showMore(anker);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });
    
    this.connect(this.showBack, "onclick", function(){
    	this._showLess(anker, numberExpertsPerLine);
    	if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	}
    });   
    this.connect(this.showBack, "onkeydown", function(e){
    	if(e.keyCode === dojo.keys.ENTER) {
    	  this._showLess(anker, numberExpertsPerLine);
    	  if(counter > 0) {
    		expertsWidgets[0].domNode.focus();
    	  }
    	}
    });

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },
  
  _showMore: function _showMore_InvitationDialog(anker) {
	  dojo.forEach(anker.childNodes, function(node){
	        dojo.style(node, "display", "inline-block");
	  });
	  dojo.style(this.showBack, "display", "block");
	  dojo.style(this.showMore, "display", "none"); 
  },
  
  _showLess: function _showLess_InvitationDialog(anker, numberExpertsPerLine) {
	  var i = 0;
      dojo.forEach(anker.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBack, "display", "none");
      dojo.style(this.showMore, "display", "block");
  },
  
  _createAccForDialogExpertTile: function _setWaiForDialogExpertTile_InvitationDialog(widget, pos) {
	  dijit.setWaiRole(widget.domNode,"option");
	  dijit.setWaiState(widget.domNode, "selected", "false");	   
	  dijit.setWaiState(widget.domNode, "labelledby", widget.avatar.id + " " + widget.taskDetails.id);
      if(pos === 0) {
        dojo.attr(widget.domNode, 'tabindex', "0");
      } else {
        dojo.attr(widget.domNode, 'tabindex', "-1");    	  
      } 
    
      // Keyboard handling for list navigation
      // Key down: next expert
      // Key up: previous expert 
      // Key space: select      
      this.connect(widget, "onKeyDown", function(e) {
    	  var numberOfExperts = this._expertsWidgets.length;
    	  if(e.keyCode === dojo.keys.DOWN_ARROW && pos < numberOfExperts - 1) {
            this._expertsWidgets[pos + 1].domNode.focus(); 
    	  } else if(e.keyCode === dojo.keys.UP_ARROW && pos > 0) {
    	    this._expertsWidgets[pos - 1].domNode.focus(); 
      	  } else if(e.keyCode === dojo.keys.SPACE) {
      		 this._selectWidget(widget, false); 
      	  }
      });
  },

  _setupInvitedUsers: function _invitedUsers_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // show invited users in a box
    var anker2 = this.invitedCollaborators;
    var taskId = this._tkiid;
    // Keep track of ExpertTile objects in order to destroy them later
    var invitedWidgets = this._invitedWidgets = [];
    // create a list of the invited users
    var invitedUsers = [];
    if (this.task.collaboration.currentUsers && this.task.collaboration.currentUsers.length > 0) {
      for ( var i = 0; i < this.task.collaboration.currentUsers.length; i++) {
        if (this.task.collaboration.currentUsers[i] != this._currentUser) {
          var userName = this.task.collaboration.currentUsers[i];
          var user = {
            "userName": userName
          };
          if (typeof this._userCategory[userName] === "string") {
            user["categoryName"] = this._userCategory[userName];
          }
          invitedUsers.push(user);
        }
      }
    }
    var numberExpertsPerLine = this._numberExpertsPerLine;
    // insert invited users
    var myself = this;
    var counter = 0;
    if (invitedUsers.length > 0) {
      //anker2.style.display = "block";
      // if the box is already full then do not show these entries
      if (counter > numberExpertsPerLine) {
        this.invitedCollaboratorsSection.style.display = "none";
      }
      else {
        this.invitedCollaboratorsSection.style.display = "block";
      }
      dojo.forEach(invitedUsers, function(item){
        counter++;
        item['taskId'] = taskId;
        item['includeTaskDetails'] = false;
        item['sidebar'] = false;
        item['collaborate'] = false;
        var ee = new com.ibm.bpm.social.widgets.experts.DialogExpertTile(item);
        ee.domNode.title = myself._nlsResources["task_dialog_startCollaborationInvited_title"];
        invitedWidgets.push(ee);
        anker2.appendChild(ee.domNode);
        dojo.addClass(ee.domNode, "bpm-social-experts-expert-selected");
        ee.domNodeA11y.innerHTML = "&#10004;";
        if (counter > numberExpertsPerLine) {
          dojo.style(ee.domNode, "display", "none");
        }
      });
      for ( var j = 0; j < invitedWidgets.length; j++) {
        var widget = invitedWidgets[j];
        this.connect(widget, "onClick", "_invitedUserClicked");
      }
    }
    if (counter == 0) {
      dojo.style(this.invitedCollaboratorsSection, "display", "none");
    }
    else {
      dojo.style(this.invitedCollaboratorsSection, "display", "block");
    }
    if (counter <= this._numberExpertsPerLine) {
      dojo.style(this.showMoreInvited, "display", "none");
    }
    else {
      dojo.style(this.showMoreInvited, "display", "block");
    }
    
    // show more functions for invited 
    this.connect(this.showMoreInvited, "onclick", function(){
      dojo.forEach(anker2.childNodes, function(node){
        dojo.style(node, "display", "inline-block");
      });
      dojo.style(this.showBackInvited, "display", "block");
      dojo.style(this.showMoreInvited, "display", "none");
    });
    this.connect(this.showBackInvited, "onclick", function(){
      var i = 0;
      dojo.forEach(anker2.childNodes, function(node){
        i++;
        if (i > numberExpertsPerLine) {
          dojo.style(node, "display", "none");
        }
        else {
          dojo.style(node, "display", "inline-block");
        }
      });
      dojo.style(this.showBackInvited, "display", "none");
      dojo.style(this.showMoreInvited, "display", "block");
    });
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  _expertClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, false);
  },

  _invitedUserClicked: function _expertClicked_InvitationDialog(event){
    this._userClicked(event, true);
  },

  _userClicked: function _userClicked_InvitationDialog(event, alreadyInviteed){
    var widget = dijit.getEnclosingWidget(event.target);
    if (widget instanceof com.ibm.bpm.social.util.ui.Avatar) {
      widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
    }
    this._selectWidget(widget, alreadyInviteed);
  },
  
  _selectWidget: function _selectWidget_InvitationDialog(widget, alreadyInviteed){
    if (widget != null) {
      if (!alreadyInviteed && widget.isAdded) {
        // Remove
        widget.isAdded = false;
        dojo.removeClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "";
        dijit.setWaiState(widget.domNode, "selected", "false");
        // Is there a clear function on the user chooser
        this.userChooser.chooserInput.value = "";
        this._checkButtonEnablement();       
      }
      else {
        // only one invitee can be selected. Therefore deselect all the others
        // TODO : grey out the already invited users. They should be display only. 	
        for ( var j = 0; j < this._expertsWidgets.length; j++) {
          var w = this._expertsWidgets[j];
          if (w.userName != widget.userName) {
            w.isAdded = false;
            dojo.removeClass(w.domNode, "bpm-social-experts-expert-selected");
            dijit.setWaiState(w.domNode, "selected", "false");
            w.domNodeA11y.innerHTML = "";
          }
        }
        widget.isAdded = true;
        dojo.addClass(widget.domNode, "bpm-social-experts-expert-selected");
        widget.domNodeA11y.innerHTML = "&#10004;";
        dijit.setWaiState(widget.domNode, "selected", "true");
        var user = {
          "userName": widget.userName,
          "fullName": widget.fullName
        };
        //
        // TODO Validate logic for setting a user on the user chooser
        //
        if (user != null) {
          if (user.fullName != null) {
            this.userChooser._chooseItemValue(user);
          }
          else {
            // Does this make sense given that user is an object !              
            this.userChooser.chooserInput.value = user;
          }
        }
      }      
    }
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getMessageText: function _getMessageText_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var messageText = "[no message specified]"; // User should never see this - no NLS necessary
    if (this.messageBox) {
      if (this.placeHolderState.modified) {
        messageText = this.messageBox.value;
      }
      else {
        messageText = "";
      }
    }
    if (dojo.trim(messageText) === "") {
      messageText = null;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return messageText;
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getInvitedUserid: function _getInvitedUserid_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // TODO Check implications of selecting/entering groups instead of single user
    var choice = null;
    if (this.userChooser) {
      choice = this.userChooser.getChoice();
      if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
        return {
          value: choice.userName,
          type: "user"
        };
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return choice;
  },

  onInvite: function onInvite_InvitationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    var myself = this;

    // if nothing can be don in the dialog it is an cancel button

    var invitedUser = this._getInvitedUserid();
    var userid = "[not selected yet]";
    if (invitedUser) {
      userid = invitedUser.value;
    }
    var message = this._getMessageText();
    var tkiid = this._tkiid; // Need for error handler below
    // 
    // Rest call to backend 
    //    
    this._restHandler.inviteUserToCollaborate(this._tkiid, function(response){
      // Good case => not action necessary
      // insert the invited user in task object.
      if (myself.task.collaboration && myself.task.collaboration.currentUsers) {
        if (dojo.indexOf(myself.task.collaboration.currentUsers, userid) === -1) {
          myself.task.collaboration.currentUsers.push(userid);
        }
      }
    }, function(data, ioArgs){
      var messageParameters = {
        1: tkiid
      };
      // the errorCodeMap will overrule the statusCodeMap
      var errorCodeMap = {
        "CWTBG0007E": {
          key: "KEY_TaskInvitedUserDoesNotExist",
          parameters: {
            1: userid
          }
        },
        "CWTBG0561E": {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        "CWTBG0018E": {
          key: "KEY_TaskInviteTaskNotClaimed"
        },
        "CWTBG0048E": {
          key: "KEY_TaskInviteTaskNotFound"
        },
        "CWTBG0540E": {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      // for error codes not found in the errorCodeMapm use the statusCodeMap
      var statusCodeMap = {
        404: {
          key: "KEY_TaskInviteNotAllowedForTask"
        },
        403: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        401: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        400: {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      var errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();
      errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
    }, userid, message);

    this.hide();
    com.ibm.bpm.social.util.common.Log.logExit(1);
  }

});


com.ibm.bpm.social.widgets.dialogs.InvitationDialog.queryMapping = {
  "TKIID": "TKIID",
  "NAME": "NAME",
  "DISPLAY_NAME": "DISPLAY_NAME",
  "PI_NAME": "PI_NAME",
  "PIID": "PIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.dialogs.InvitationDialog.itemMapping = {
  "TKIID": "tkiid",
  "NAME": "name",
  "DISPLAY_NAME": "displayName",
  "PI_NAME": "processInstanceName",
  "PIID": "piid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};


/**
 * Opens the InvitationDialog
 * 
 * @param task
 * @param user An object with a userName and fullName property
 * @param user {void} nothing
 */
com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog = function(task, user){
  var nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
  var dialog = new com.ibm.bpm.social.widgets.dialogs.InvitationDialog({
    "title": nlsResources["task_dialog_startCollaboration_title"],
    //"style": "min-width: 50%",
    //"style": "min-height: 50%",
    "task": task
  });
  dialog.showDialog(user);
};

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.experts.ExpertTile"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.experts.ExpertTile"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.experts.ExpertTile");














dojo.declare(
	"com.ibm.bpm.social.widgets.experts.ExpertTile",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class='bpm-social-experts-expert' dojoAttachPoint='expert' dojoAttachEvent='onmouseenter:_doMouseEnter, onmouseleave:_doMouseLeave'>\n\t<div dojoAttachPoint='avatarContent'> </div>\t\n\t<div class='bpm-social-experts-expert-data-div'>\t    \n\t\t<div class='bpm-social-experts-expert-name' dojoAttachPoint='name' role=\"listitem\" tabindex=\"0\"></div>\n\t\t<div tabIndex='0' dojoAttachEvent='onfocus:_focusStart' class='bpm-social-experts-focus-control'></div>\n\t\t<div class='bpm-social-experts-expert-details taskDetails' dojoAttachPoint='taskDetails'></div>\n\t\t<div class='bpm-social-experts-expert-actions-div'>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-mention' dojoAttachPoint=\"mentionDiv\">\n\t\t\t\t<a dojoAttachPoint='mention' href='javascript:void(0)' dojoAttachEvent='onclick:_handleMention'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator1' class='vertical-bar' style='display:block'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-help' dojoAttachPoint=\"helpDiv\">\n\t\t\t\t<a dojoAttachPoint='gethelp' href='javascript:void(0)' dojoAttachEvent='onclick:_handleGetHelp'></a>\n\t\t\t</div>\n\t\t\t<div dojoAttachPoint='actionSeparator2' class='vertical-bar'></div>\n\t\t\t<div class='bpm-social-experts-expert-actions-item bpm-social-experts-expert-actions-item-chat' dojoAttachPoint=\"chatDiv\">\n\t\t\t\t<a dojoAttachPoint='startchat' href='javascript:void(0)' dojoAttachEvent='onclick:_handleChat'></a>\n\t\t\t</div>\n\t\t</div>\n\t\t<div tabIndex='0' dojoAttachEvent='onfocus:_focusEnd' class='bpm-social-experts-focus-control'></div>\n\t</div>\n</div>\n",
	completed: 0,
	avgHours: 0,
	workload: 0,
	userName: null,
	fullName: null,
	phoneNumber: null,
	email: null,
	imageUrl: null,
	taskId: null,
	livename: null,
	refresh: null,
	title: "",
	includeTaskDetails: false,
	sidebar: false,
	userDataUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/userdata?username={username}",
	collaborate: false,
	
    postCreate: function() {
    	this.inherited(arguments);
    	
    	
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.mention.innerHTML = "@"+this.messages.expertsMention;
		this.gethelp.innerHTML = this.messages.expertsCollaborate;
		this.startchat.innerHTML = this.messages.expertsChat;
		
		this.name.innerHTML = this.fullName;
        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.name);
		
		var curdate = new Date();
		var millisec = curdate.getTime();
		var nameId = "expertTile_name" + millisec;
		dojo.attr(this.name, "id", nameId);	
		dijit.setWaiState(this.mention, "describedby", nameId);
		dijit.setWaiState(this.gethelp, "describedby", nameId);
		dijit.setWaiState(this.startchat, "describedby", nameId);
		
		this.actionSeparator1.innerHTML = this.messages.expertsSeparator;
		this.actionSeparator2.innerHTML = this.messages.expertsSeparator;
		
		if (!this.sidebar) {
			this.mention.style.display = 'none';
			this.actionSeparator1.style.display = 'none';
		}
		
    	this._getUserData();
    	
    	this.domNode.title = "";
		
		if (!this.collaborate) {
			this.actionSeparator1.style.display = 'none';
			this.helpDiv.style.display = 'none';
		}
    	
		if(window.sametime && this.fullName){
			// the text-overflow:ellipsis style does not function properly
			// when there are span elements within the div, so clip the
			// text instead if we are using sametime
			this.name.style.textOverflow = 'clip';
			
			this.name.innerHTML = "";
			this.livename = window.sametime.LiveName({'userId': this.userName, 'disableClicks': true, 'disableHoverBizCard': true},dojo.create("span", null, this.name));
			this.livename.activate();
			
			this._updateStatus();
			this.refresh = setInterval(dojo.hitch(this, this._updateStatus), 2000);
		} else {
			dojo.style(this.chatDiv, "display", "none");
		}			   

		/** Business Card integration **/
		/** Default **/
		var node = this.name;
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		
		if(this.livename!=null){
			var nodes = dojo.query(".stproxy_liveNamePadding", this.name);
			if(nodes.length==1){
				node = nodes[0];
				var img = dojo.create("img", {
					"src": dojo.moduleUrl("dojo", "resources/blank.gif"),
					"class": "connectionsOverlayImage"
					},
					dojo.query(".stproxy_general", this.domNode)[0],
					"first");
				com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
			}
		}
		com.ibm.bpm.social.widgets.common.BizCard.wrapElement(node, user);
		com.ibm.bpm.social.widgets.common.BizCard.wrapImage(dojo.query("img", this.img)[0], user);

		/** End Business Card integration **/
		
		this.avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContent);
		this.avatar.setUser(user, "bpm-social-default-avatar-medium", true);

		this._updateUserData();
		var data;
		var firstName = this.fullName.split(" ")[0]; 
        firstName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(firstName);
		if(this.includeTaskDetails){
			if(this.completed==0){
				this.taskDetails.innerHTML = this.messages.expertNoTaskData.replace("${user}", firstName);
			} else {
				data = {
					'userName': firstName,
					'average': this.avgHours,
					'completed': this.completed
				};
				//defect 149810 Fix Expert tooltip wording
				if(this.sidebar){
				   this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip, data, /\$\{([^\}]+)\}/g);
				} else {
				   this.taskDetails.innerHTML = dojo.replace(this.messages.expertsTooltip_short, data, /\$\{([^\}]+)\}/g);
				}
			}
            dojo.attr(this.taskDetails,"id","taskDetails_"+nameId);
            dijit.setWaiState(this.name,"describedby","taskDetails_"+nameId);
		}
	},
	  
	_showTooltip: function(){
		
		return;
		/*
		if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
			** Do not show if Connections is enabled **
			return;
		}
		
		if (this.myTooltipDialog) {
			this.myTooltipDialog = null;
		}
		
		this._updateUserData();
		
		var firstName = this.fullName.split(" ")[0]; 
		
		var data = {
				'fullName': this.fullName,
				'userName': this.userName, 
				'avgHours': this.avgHours,
				'completed': this.completed,
				'displayName': this.fullName,
				'title': typeof(this.userData['Title'])!='undefined' ? 
						this.userData['Title'] : '',
				'taskEmailAddress': typeof(this.userData['Task Email Address'])!='undefined' ? 
						this.userData['Task Email Address'] : '',
				'phoneNumber': typeof(this.userData['Phone Number'])!='undefined' ? this.userData['Phone Number'] : '',
				'taskDetails': "",
				'chatString': '',
				'hr': "",
				'includeTaskDetails': false
		};
		
		this.activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
		
		this.myTooltipDialog = new dijit.TooltipDialog({
            content: this.activityOwner.domNode,
            onMouseLeave: function(){
                dijit.popup.close(this.myTooltipDialog);
            }
        });
		dijit.popup.open({
			popup: this.myTooltipDialog,
			around: dojo.byId('expert-avatar')
		});

		*/
		
	},
	
	_hideTooltip: function() {
		if (this.myTooltipDialog) {
			this.myTooltipDialog = null;
		}

	},
	
	//defect 149810 fix so that task completed is an integer and does not have decimal places
	_updateUserData: function() {
		var self = this;
		
		this.socialRestHandler.getExpertUserData(this.taskId, this.userName,
			function(data, xhrArgs) {
				if(data.avgHours < 1){
					self.avgHours = dojo.number.format( data.avgHours, {type: "decimal", pattern: "0.#", places: 1});
				} else {
					self.avgHours = dojo.number.format( data.avgHours, {places: 1 } );
				}
				self.completed = dojo.number.format( data.completed, {places: 0 } ); 
			}, "", false);
	},
	
	_getUserData: function(){
		var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
		this.userData = user.userPreferences;
		if (!this.userData) {
			this.userData = new Object();
		}		
	},
	
	_updateStatus: function(){
		dojo.removeClass(this.domNode, "online");
		dojo.removeClass(this.domNode, "offline");
		
		if(this.livename){
			switch(this.livename.currentTextStyle){
				case "stproxy_liveNameAvailableText":
				case "stproxy_liveNameAwayText":
				case "stproxy_liveNameInMeetingText":
					dojo.addClass(this.domNode, "online");
					this.presenceStatus = "online";
					break;
				case "stproxy_liveNameOfflineText":
				case "stproxy_liveNameDndText":
				default:
					dojo.addClass(this.domNode, "offline");
					this.presenceStatus = "offline";
					break;
			}
		} else {
			dojo.style(this.chatDiv, "display", "none");
		}
		
		/** Business Card integration **/
		if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard() && this.livename!=null){
			/** Account for Sametime DOM structure, which can be delayed in being created due to network latency **/
			var nodes = dojo.query(".stproxy_liveNamePadding .hasHover", this.name);
			if(nodes.length==0){
				nodes = dojo.query(".stproxy_liveNamePadding", this.name);
				if(nodes.length==1){
					var user = com.ibm.bpm.social.util.common.Util.getUser(this.userName);
					com.ibm.bpm.social.widgets.common.BizCard.wrapElement(nodes[0], user); 
				}
			}
		}
		/** End Business Card integration **/
		
	},
	
	_handleGetHelp: function(e){
		var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({"taskId": this.taskId});
		com.ibm.bpm.social.widgets.dialogs.InvitationDialog.showDialog(taskData, {
			userName: this.userName,
			fullName: this.fullName
		});		
		dojo.stopEvent(e);
	},
	
	_handleChat: function(e){
		if(this.livename && this.presenceStatus=="online"){
			this.livename.startChat();
		}
		dojo.stopEvent(e);
	},
	
	_getTaskCompletionView: function() {
		var node = this.domNode;
		while (node && !dojo.hasClass(node,'workViewCompletion')) {
			node = node.parentNode;
		}
		if (node == null) {
			node = dojo.query('.workViewCompletion')[0];
		}
		var taskCompletionView = dijit.byId(node.id);
		return taskCompletionView;
	},
	
	_handleMention: function(e){
		var taskCompletionView = this._getTaskCompletionView();
		if (taskCompletionView) {
			// switch to the stream tab
			taskCompletionView.sidebar.selectTab(1);
			// set focus to input box
			taskCompletionView.stream._handlePostInitialFocus();
			// tag user
			taskCompletionView.stream.tagUser(this.userName,this.fullName);
		}
		dojo.stopEvent(e);
	},
	
	_doMouseEnter: function(e){
		dojo.addClass(this.expert, 'hover');
	},
	
	_doMouseLeave: function(e){
		dojo.removeClass(this.expert, 'hover');
	},
	
	_onFocus: function() {
		dojo.addClass(this.domNode,'bpm-social-experts-focus');
	},
	
	_onBlur: function() {
		dojo.removeClass(this.domNode,'bpm-social-experts-focus');
	},
	
	_focusStart: function() {
		if (this.sidebar) {
			this.mention.focus();
		} else {
			this.gethelp.focus();
		}
	},
	
	_focusEnd: function() {
		if(this.livename){
			this.startchat.focus();
		} else {
			this.gethelp.focus();
		}
	},
	
	destroy: function() {
		if(this.refresh){
			clearInterval(this.refresh);
		}
		this._hideTooltip();
	    if (this.avatar) {
	        this.avatar.destroy();
	        this.avatar = null;
	    }
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip");
















dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templateString:"<div class=\"bpm-social-participants-activity-tooltip-content\" role=\"group\" aria-labelledby=\"activity_details_tooltip_title\" dojoAttachPoint=\"container\">\n<div class=\"mad-header\" role=\"presentation\">\n\t<div id=\"activity_details_tooltip_title\" class='title' dojoAttachPoint=\"header\" role=\"listitem\" tabindex=\"0\">{title}</div>\n\t<div class='counter-wrapper' dojoAttachPoint='wrapper' role=\"listitem\" tabindex=\"-1\">\n\t\t<span class=\"previous\" dojoAttachPoint=\"previous\">\n\t\t\t<img src='' dojoAttachPoint='previousImg' dojoAttachEvent=\"ondijitclick:_clickPrevious\" tabindex=\"-1\">\n\t\t</span>\n\t\t<span class='counter-scroll'>\n\t\t\t<span class='counter' dojoAttachPoint='counter'></span>\n\t\t</span>\n\t\t<span class=\"next\" dojoAttachPoint=\"next\">\n\t\t\t<img src='' dojoAttachPoint='nextImg' dojoAttachEvent=\"ondijitclick:_clickNext\" tabindex=\"-1\">\n\t\t</span>\n\t</div>\n\t<div style=\"clear: both;\"></div>\n\t<div class=\"content\" dojoAttachPoint='content' role=\"listitem\" aria-describedby=\"owner-text-wrapper start-date due-date\" tabindex=\"0\">\n\t\t<div class=\"owner-data\">\n\t\t\t<!-- <img class=\"ownerImg bpm-social-default-avatar-small\" dojoAttachPoint=\"ownerImg\" alt=\"\"/>  -->\n\t\t\t<div dojoAttachPoint=\"avatarContainer\" ></div>\t\n\t\t\t<div id=\"owner-text-wrapper\" class=\"owner-text-wrapper\" role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\">\n\t\t\t\t<span class=\"ownerField\" dojoAttachPoint=\"ownerField\">{owner}</span>\n\t\t\t\t<span class=\"ownerLabel\" dojoAttachPoint=\"ownerLabel\">{ownerLabel}</span>\n\t\t\t</div>\n\t\t\t</div>\n        <div>\n    \t\t<div role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\" id=\"start-date\"><label id=\"start_date_label\" dojoAttachPoint=\"startDateLabel\">{startDateLabel}</label><span id=\"start_date_field\" class='startDate' dojoAttachPoint=\"startDateField\">{startDate}</span></div>\n    \t\t<div role=\"listitem\" aria-live=\"polite\" aria-atomic=\"true\" id=\"due-date\"><label id=\"due_date_label\" dojoAttachPoint=\"dueEndDateLabel\">{dueEndDateLabel}</label><span id=\"due_date_field\" class='dueEndDate' dojoAttachPoint=\"dueEndDateField\">{dueEndDate}</span></div>\n\t\t</div>\n\t</div>\n</div>\n<hr>\n<div class=\"mad-body\">\n\t<span class='title' dojoAttachPoint=\"expertsTitle\" role=\"listitem\" tabindex=\"0\">{expertsTitle}</span>\n\t<div style=\"clear: both;\"></div>\n\t<div class='activityExperts'>\n\t\t<div class='grid' dojoAttachPoint=\"grid\"></div>\n\t\t<div class='bpm-social-experts-layout' dojoAttachPoint='autoExperts'>\n\t\t\t<div class='bpm-social-experts-layout-auto-none' dojoAttachPoint='noExperts' style='display:none;'>\n\t\t\t\t<div class='header' dojoAttachPoint='noExperts_header'></div>\n\t\t\t\t<div class='subscript' dojoAttachPoint='noExperts_subscript'></div>\n\t\t\t\t<div class='content' dojoAttachPoint='noExperts_content'></div>\n\t\t\t</div>\n\t\t</div>\n\t</div>\n</div>\n</div>\n\n",
	
	tasks: [],
	taskInstances: [],
	currentTaskIndex: 0,
	
	currentMaxIndex: 3,
	currentMinIndex: 0,
	
	livenames: null,	
	expertTiles: [],
	
    postCreate: function() {
    	this.inherited(arguments);
		this.taskInstances = [];
    	
		
		this.messages = dojo.i18n.getLocalization('com.ibm.bpm.social', 'bpmSocial');
		
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.expertsTitle.innerHTML = this.messages.activityExpertsTitle;
		
		this.startDateLabel.innerHTML = this.messages.startDate+":";
		this.ownerLabel.innerHTML = this.messages.owner;
		
		this.noExperts_header.innerHTML = this.messages.noExperts_AutoHeader;
		
        dojo.attr(this.wrapper,"aria-label",this.messages.activity_details_counter_label);

    		this.previousImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/blank.gif");
    		this.nextImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/blank.gif");
    	  this._avatar = new com.ibm.bpm.social.util.ui.Avatar({}, this.avatarContainer);

//        if (dojo.hasClass(document.body,"dijit_a11y")) {
//            this.ownerImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
//        }

        this.nextImg.alt = this.messages.next_button_label;
        dijit.setWaiRole(this.nextImg,"button");
        this.previousImg.alt = this.messages.previous_button_label;
        dijit.setWaiRole(this.previousImg,"button");

		if(this.tasks.length==1){
			dojo.addClass(this.container, 'singleUser');
			dojo.empty(this.wrapper);
		} else {
            dojo.attr(this.wrapper,"tabindex","0");
            if (dojo.hasClass(document.body,"dijit_a11y")) {
                this.previousImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Chevron_gray_left.png");
                this.nextImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Chevron_gray_right.png");
                //this.ownerImg.src = dojo.moduleUrl("com.ibm.bpm.social", "img/Avatar_30x30.png");
            }
			for(var idx = 0; idx < this.tasks.length; idx++){
				var t = dojo.create('a', {
					'class': 'task-instance',
					'onclick': dojo.hitch(this, this._taskInstanceSelected),
					'index': idx+1,
					'href': 'javascript:void(0)',
					'innerHTML': idx+1
				}, this.counter);
				if(idx==0){
					dojo.addClass(t, 'current');
                    dijit.setWaiState(t,"disabled","true");
				}
				this.taskInstances.push(t);
			}
            this._disablePreviousOrNext(this.previous,this.previousImg,true);            
			
			/*
			if(this.tasks.length < this.currentMaxIndex){
				dojo.addClass(this.next, 'disabled');
			}
			*/
		}
		
		this._setTaskDetails(this.tasks[this.currentTaskIndex]);
		
		setTimeout(dojo.hitch(this, function(){
			this._checkForCounterScroll(0);
		}), 100)
	},
	
	_updateTaskDetails: function(){
		dojo.query('a.task-instance.current', this.domNode).forEach(function(node){
            dojo.removeClass(node,'current');
            dijit.setWaiState(node,"disabled","false");
        });	
		dojo.query('a.task-instance[index='+(this.currentTaskIndex+1)+']', this.domNode).forEach(function(node){
            dojo.addClass(node,'current');
            dijit.setWaiState(node,"disabled","true");
        });	

		
		dojo.fx.chain([
		    dojo.animateProperty({
		    	node: this.content,
		        duration: 300,
		        properties: {
		        	color: {
		        		start: "black",
		            	end: "white"
		            }
		        },
		        onBegin: dojo.hitch(this, function(){
		        	com.ibm.bpm.social.widgets.common.BizCard._hideTooltip();
		        }),
		        onEnd: dojo.hitch(this, function(){
		        	this._setTaskDetails(this.tasks[this.currentTaskIndex]);
		        })
		    }),
		    dojo.animateProperty({
		        node: this.content,
		        duration: 300,
		        properties: {
		            color: {
		            	start: "white",
		            	end: "black"
		            }
		        }
		    })
		]).play();
	},
	
	_setTaskDetails: function(t){
		
		this.dueEndDateLabel.innerHTML = (t.completionTime==null ? this.messages.dueDate : this.messages.endDate)+":";
 	   
		this.header.innerHTML = t.name;
		this.header.title = t.name;

        com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.header);
        
        this._applyStylingForBTD(this.header);
		
        this.startDateField.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(t.startTime);
        
		this.dueEndDateField.innerHTML = com.ibm.bpm.social.util.common.Util.formatDate(t.completionTime==null ? t.dueTime : t.completionTime);
        
		var user = null;
		if(t.owner){
			user = com.ibm.bpm.social.util.common.Util.getUser(t.owner);
			this.ownerField.innerHTML = user.fullName;
      this._avatar.setUser(user, " bpm-social-default-avatar-small", true);
      this._avatar.setCSSClass("owner-visible");
			dojo.style(this.ownerLabel, "visibility", "visible");
			dojo.style(this.ownerField, "visibility", "visible");
			dojo.query(".owner-text-wrapper", this.domNode).removeClass("no-owner");
		} else if(t.assignedToDisplayName){
			this.ownerField.innerHTML = t.assignedToDisplayName;
			this._avatar.setCSSClass("owner-hidden");
			dojo.style(this.ownerLabel, "visibility", "hidden");
			dojo.style(this.ownerField, "visibility", "visible");
			dojo.query(".owner-text-wrapper", this.domNode).addClass("no-owner");
		} else {
      this._avatar.setCSSClass("owner-hidden");
			dojo.style(this.ownerLabel, "visibility", "hidden");
			dojo.style(this.ownerField, "visibility", "hidden");
			dojo.query(".owner-text-wrapper", this.domNode).addClass("no-owner");
		}

		// Set the BTD off the ownerField before adding the business card integration
		com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.ownerField);
		this._applyStylingForBTD(this.ownerField);
		if(user!=null){
			/** Business Card integration **/
			//com.ibm.bpm.social.widgets.common.BizCard.unwrapImage(this.ownerImg); //fixes defect 93774
			com.ibm.bpm.social.widgets.common.BizCard.wrapElement(this.ownerField, user);
			//com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.ownerImg, user);
			/** End Business Card integration **/
		}
        
    },

	/*
	 * Called indirectly from the surrounding TooltipDialog onShow Event
	 */
	getExperts: function(){
		this.socialRestHandler.getExpertsForTask(this.tasks[this.currentTaskIndex].tkiid, 
			dojo.hitch(this, "_appendExpertsToTooltip"),
			dojo.hitch(this, "_onXhrErrors"), false);
	},
	
	_appendExpertsToTooltip: function(data, xhrArgs){
        var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();        
        var currentUserId = userProfileObj.userID;
		var ee = null;
		if(data.systemIdentifiedUsers.length==0){
			dojo.style(this.noExperts, "display", "block");
			return;
		}
        var expertsCt = 0;
		for(var idx = 0; idx < data.systemIdentifiedUsers.length; idx++){
            //Do not show if the expert matches the current user.
            //We only show 6, but REST might return 7.
            if (currentUserId != data.systemIdentifiedUsers[idx]["userID"]) {
			data.systemIdentifiedUsers[idx]['taskId'] = this.tasks[this.currentTaskIndex].tkiid;
			data.systemIdentifiedUsers[idx]['includeTaskDetails'] = true;
			ee = new com.ibm.bpm.social.widgets.experts.ExpertTile(data.systemIdentifiedUsers[idx]);
			this.grid.appendChild(ee.domNode);
			this.expertTiles.push(ee);
                expertsCt++;
                if (expertsCt == 6) {
                    break;
                }
            }
		}
		
		var position=null;
		if (!!ee) {
			position = dojo.position(ee.domNode);
		}
		if (position == null || position.w <= 206) { // Check to see if the size of the experts node is larger than expected size
			if(expertsCt <= 2){
				dojo.addClass(this.grid, "singleRow");
			} else if(expertsCt <= 4){
				dojo.addClass(this.grid, "doubleRow");
			} else if(expertsCt <= 6){
				dojo.addClass(this.grid, "tripleRow");
			} else {
				dojo.addClass(this.grid, "tripleRow");
				dojo.addClass(this.grid, "scrollExperts");
			}
		} else { // width of the experts element > than specified width so only have 1 entry per row
			if(expertsCt <= 1){
				dojo.addClass(this.grid, "singleRow");
			} else if(expertsCt <= 2){
				dojo.addClass(this.grid, "doubleRow");
			} else if(expertsCt <= 3){
				dojo.addClass(this.grid, "tripleRow");
			} else {
				dojo.addClass(this.grid, "tripleRow");
				dojo.addClass(this.grid, "scrollExperts");
			}
		}
	},
	
	_onXhrErrors: function(data,xhrArgs) {
		// TODO: how do we show errors?
		console.error(data, xhrArgs);
	},
	
	_taskInstanceSelected: function(e){
		
		var oldIdx = this.currentTaskIndex;
		
		this.currentTaskIndex = e.target.getAttribute('index')-1;
		
		var scrollDir = (this.tasks.length < this.currentMaxIndex ? 0 : this.currentTaskIndex - oldIdx);
		
		this._checkForCounterScroll(scrollDir);
		
		this._updateTaskDetails();
	},
	
	_clickPrevious: function(e){
        dojo.stopEvent(e);
		
		if(dojo.hasClass(this.previous,'disabled')){
			return;
		}
        this._disablePreviousOrNext(this.next,this.nextImg,false);
		
		this.currentTaskIndex--;
		if(this.currentTaskIndex < 0){
			this.currentTaskIndex = 0;
		}
		
		if(this.currentTaskIndex==0){
            this._disablePreviousOrNext(this.previous,this.previousImg,true);
		}
		this._checkForCounterScroll(-1);
		this._updateTaskDetails();

	},
	
	_clickNext: function(e){
        dojo.stopEvent(e);
		
		if(dojo.hasClass(this.next, 'disabled')){
			return;
		}
        this._disablePreviousOrNext(this.previous,this.previousImg,false);
		
		this.currentTaskIndex++;
		if(this.currentTaskIndex > this.tasks.length-1){
			this.currentTaskIndex = this.tasks.length - 1;
		}
		
		if(this.currentTaskIndex==this.tasks.length - 1){
            this._disablePreviousOrNext(this.next,this.nextImg,true);
		}
		this._checkForCounterScroll(1);
		this._updateTaskDetails();
	},
	
	_checkForCounterScroll: function(dir) {
		/*if(dir==0){
			this._checkPreviousNextState();
			return;
		}*/
		
		//scroll to the correct page number
		var scrollToIndex = this.currentTaskIndex;
		if(this.currentTaskIndex == this.taskInstances.length-1) {
			scrollToIndex--;
		}
		else if(this.currentTaskIndex == 0) {
			scrollToIndex++;
		}
		var pageLink = this.taskInstances[scrollToIndex];
        if (!pageLink) {
            return;
        }
        
		var scrollAmt = 25 - pageLink.offsetLeft;
		if(this.currentTaskIndex == 0) {
			scrollAmt--; //forces the ipad to refresh, fixes defact 91836
		}
		dojo.query(this.counter).style('left', ""+scrollAmt+"px");
		
		//get the adjacent numbers
		var leftmost = (scrollToIndex-2 >= 0) ? this.taskInstances[scrollToIndex-2] : null;
		var leftish = (scrollToIndex-1 >= 0) ? this.taskInstances[scrollToIndex-1] : null;
		var center = this.taskInstances[scrollToIndex];
		var rightish = (scrollToIndex+1 < this.taskInstances.length) ? this.taskInstances[scrollToIndex+1] : null;
		var rightmost = (scrollToIndex+2 < this.taskInstances.length) ? this.taskInstances[scrollToIndex+2] : null;
		
		//hide anything that would get cut-off by the arrow buttons
		if(leftmost != null) {
			dojo.style(leftmost, "visibility", "hidden");
		}
		if(leftish != null) {
			dojo.style(leftish, "visibility", "");
		}
		dojo.style(center, "visibility", "");
		if(rightish != null) {
			dojo.style(rightish, "visibility", "");
		}
		if(rightmost != null) {
			dojo.style(rightmost, "visibility", "hidden");
		}
		/*
		var lclCounter = dojo.query(this.counter);
		
		//For some reason the pixels here are just a little off (by 1/2 a pixel)
		//so adjusting the pixel movements by one does not fix the issue of the
		//numbers scrolling out of visibility.  What is even stranger, the ipads
		//numbers scroll into invisibility the opposite direction.  Here I adjust
		//the scrolling every other time to adjust for this movement, and the 
		//numbers stay where they should.
		//Defect 93772
		var scrollOffset;
		if(this.currentTaskIndex < 10) {
			scrollOffset = 12;
		} else {
			scrollOffset = 18;
			if(com.ibm.bpm.social.util.common.Util.isIPad || dojo.isIE) {
				scrollOffset--;
				//every 10 times or so
				if((this.currentTaskIndex % 10) == 0) {
					scrollOffset++;
				}
			} else if((this.currentTaskIndex % 2) > 0) {
				scrollOffset++;
			}
		}
		
		//var scrollOffset = (this.currentTaskIndex < 10 ? 12 : 18); //Used to determine if we need a big scroll or smaller
		
		var curLeft = parseInt(lclCounter.style('left'));
		
		var offset;
		if(dir > 0) {
			if(this.currentTaskIndex >= this.currentMaxIndex){
				offset = (dir * -1 * scrollOffset) + curLeft;
				lclCounter.style('left', offset+'px');
				if(this.currentMaxIndex < this.tasks.length){
					this.currentMaxIndex += dir;
					this.currentMinIndex += dir;
				}
			}
		} else if(dir < 0){
			if(this.currentTaskIndex <= this.currentMaxIndex){
				offset = ((dir * -1 * scrollOffset) + curLeft);
				if(offset > 0) { 
					offset = 0;
				}
				lclCounter.style('left', offset+'px');
				
				if(this.currentMinIndex > 0){
					this.currentMaxIndex += dir;
					this.currentMinIndex += dir;
				}
			}
		}*/
		this._checkPreviousNextState();
	},
	
	_checkPreviousNextState: function(){
		if(this.currentTaskIndex == this.tasks.length-1){
			if(!dojo.hasClass(this.next, 'disabled')){
                this._disablePreviousOrNext(this.next,this.nextImg,true);
			}
		} else {
            this._disablePreviousOrNext(this.next,this.nextImg,false);
		}
			
		if(this.currentTaskIndex == 0){
			if(!dojo.hasClass(this.previous, 'disabled')){
                this._disablePreviousOrNext(this.previous,this.previousImg,true);
			}
		} else {
            this._disablePreviousOrNext(this.previous,this.previousImg,false);
		}
	},

    _disablePreviousOrNext: function(span,img,disable) {
         if (disable) {
             dojo.addClass(span, 'disabled');
             dijit.setWaiState(img,"disabled","true");
             dojo.attr(img,"tabindex","-1");
		} else {
             dojo.removeClass(span, 'disabled');
             dijit.setWaiState(img,"disabled","false");
             dojo.attr(img,"tabindex","0");
         }
	},
	
	_applyStylingForBTD: function(node) {
		var isDocLanguageRTL = ("rtl" == document.body.getAttribute('dir'));
		
		if (isDocLanguageRTL)
			dojo.style(node, "textAlign", "right");
		else
			dojo.style(node, "textAlign", "left");
	},
				
	destroy: function() {
		if(this.expertTiles && this.expertTiles.length>0){
			for(var idx = 0; idx < this.expertTiles; idx++){
				if(this.expertTiles[idx].destroy){
					this.expertTiles[idx].destroy();
				}				
			}
			this.expertTiles = {};
		}			
    if (this._avatar) {
      this._avatar.destroy();
      this._avatar = null;
    }
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton");









dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton",
	[dijit._Widget, dijit._Templated, dijit.form.DropDownButton],
	{

	label: '',
	tasks: [],
	displayDropDown: true,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.activityDetails = new com.ibm.bpm.social.widgets.diagram.ActivityDetailsTooltip({
			'tasks': this.tasks
		});
		
		this.dropDown = new dijit.TooltipDialog({
            'content': this.activityDetails.domNode,
            'class': 'bpm-social-participants-activity-tooltip'
        });
		
    	this.label = '';
 
    	this.inherited(arguments);
    	
    	dojo.addClass(this.domNode, 'bpm-social-participants-activity');

        dijit.setWaiRole(this.dropDown.containerNode,"role","presentation");//fix RPT violation
    	
    	if(this.displayDropDown){
			this.openDropDown();
		}
	},
	
	openDropDown: function(){
		this.inherited(arguments);        
		
		this.activityDetails.getExperts();
        this.activityDetails.header.focus();


	},
	
	closeDropDown: function(foo, force){
		if(force==true){
			this.inherited(arguments);
		}
	},
	
	destroy: function() {
		if(this.activityDetails && this.activityDetails.destroy){
			this.activityDetails.destroy();
		}
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton");













dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton",
	[dijit._Widget, dijit._Templated, dijit.form.DropDownButton],
	{

	label: '',
	activityOwner: null,	
	livenames: null,	
	currentUser: null,
	avgHours: 0,
	completed: 0,
	userImageSize: 24,
	popupUserImageSize: 48,
	container: null,
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
    	var data = {
    			'fullName': this.currentUser.fullName,
    			'userName': this.currentUser.userName,
    			'displayName': this.currentUser.fullName,
    			'title': this.currentUser.userPreferences['Title'],
    			'taskEmailAddress': this.currentUser.userPreferences['Task Email Address'],
    			'phoneNumber': this.currentUser.userPreferences['Phone Number'],
    			'avgHours': this.avgHours,
    			'completed': this.completed,
    			'chatString': ''
       	};
    	    	
		if(com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
			this.activityOwner = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip(data);
			this.dropDown = new dijit.TooltipDialog({
				'content': this.activityOwner.domNode,
				'class': 'bpm-social-participants-image-tooltip'
			});
			dojo.connect(this, 'onMouseEnter', this, 'openDropDown');
		}

    	this.label = '';
 
    	this.inherited(arguments);
    	
    	dojo.addClass(this.domNode, 'bpm-social-participants-image');

	    imgSrc = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this.currentUser.userName);

	    var props = {
       		'src': imgSrc,
       		'height': this.popupUserImageSize,
    		'width': this.popupUserImageSize,
    		'alt': this.currentUser.fullName
       	};

    	var textNode = dojo.query(".dijitButtonText", this.domNode)[0];
    	this.currentImage = dojo.create("img", props, textNode);

    	//if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
    	//	dojo.connect(this, 'onClick', this, function(e){
    	//		if(this.container!=null && typeof(this.container.destroy)!='undefined'){
    	//			this.container.destroy();
    	//		}
    	//	});
    	//}

		/** Business Card integration **/
    	if(!com.ibm.bpm.social.widgets.common.BizCard.useDefaultCard()){
    		var user = com.ibm.bpm.social.util.common.Util.getUser(this.currentUser.userName);
    		com.ibm.bpm.social.widgets.common.BizCard.wrapImage(this.currentImage, user);
    	}
		/** End Business Card integration **/
    },
	
	openDropDown: function(){
		if(this.dropDown){
			if(this.container){
				//callback to parent container to close other dropdowns
				this.container.closeDropdowns();
			}
			this.inherited(arguments);
		}
	},
	
	isDropDownOpen: function(){
		return dojo.query('.dijitHasDropDownOpen', this.domNode).length > 0;
	},
	
	destroy: function() {
		if(this.activityOwner && this.activityOwner.destroy){
			this.activityOwner.destroy();
			this.activityOwner = null;
		}
		
		this.inherited('destroy',arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer");











dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer",
	[com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{

	templateString: "",
	templateString:"<div class=\"bpm-social-participants-image-container hidden\" dojoAttachPoint=\"container\">\n<div dojoAttachPoint=\"scroll\" class=\"scroll\">\n<div dojoAttachPoint=\"innerScroll\" class=\"innerScroll\"></div>\n</div>\n</div>\n",
	tasks: [],
	dropdowns: [],
	displayedUsers: {},
	displayedUsersCount: 0,
	displayDropDown: false,
	displayDropDownIndex: null,
	currentWidth: 0,
	cssSingleIconWidth: 57,
	//userDataUrl: "/ProcessPortal/api/rest/task/{tid}/userdata?username={username}",
	userDataUrl: window.endpointURLObject.bpmRest +"/v1/social/task/{tid}/userdata?username={username}",
	
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
		
		this.displayedUsers = {};
		this.dropdowns = [];
		
		var currentUser, avgHours, completed;
		
		if(this.tasks.length==1){
			dojo.addClass(this.container, "singleUser");
			this.displayDropDown = true;
			this.displayDropDownIndex = 0;
		}
		
		for(var idx = 0; idx < this.tasks.length; idx++){
			
	    	currentUser = com.ibm.bpm.social.util.common.Util.getUser(this.tasks[idx].assignedTo,true);

	    	if(this.displayedUsers[currentUser.userName]){
	    		continue;
	    	} else {
	    		this.displayedUsers[currentUser.userName] = true;
	    		this.displayedUsersCount++;
	    	}
	    	
	    	avgHours = 0;
	    	completed = 0;
	    	
			this.socialRestHandler.getExpertUserData(this.tasks[idx].tkiid, currentUser.userName,
				function(data, xhrArgs) {
					avgHours = data.avgHours;
					completed = data.completed;
				}, "", false);
			
	   		//03. Create Dropdown Button (Image)
	   		 var button = new com.ibm.bpm.social.widgets.diagram.ActivityOwnerDropDownButton({
	   			'currentUser': currentUser,
	   			'avgHours': avgHours,
	   			'completed': completed,
	   			'container': this
	   		}, dojo.create("div", {}, this.innerScroll));
	   		 
	   		this.dropdowns.push(button);

	   		dojo.style(this.innerScroll, 'width', (this.displayedUsersCount * this.cssSingleIconWidth)+'px');
		}
		dojo.removeClass(this.container, "hidden");
		
		this.inherited('postCreate',arguments);
		
		if(this.displayDropDown){
			if(this.displayDropDownIndex && this.displayDropDownIndex < this.dropdowns.length){
				this.openDropDown(this.displayDropDownIndex);
			} else {
				this.openFirstDropDown();
			}
		}
	},
	
	openFirstDropDown: function(){
		this.openDropDown(0);
		
	},
	
	openDropDown: function(idx){
		if(idx < this.dropdowns.length){
			this.dropdowns[idx].openDropDown();
		}
	},

	getDropDowns: function(){
		return this.dropdowns;
	},
	
	closeDropdowns: function(){
		for(var idx = 0; idx < this.dropdowns.length; idx++){
			this.dropdowns[idx].closeDropDown();
		}
	},
	
	destroy: function() {
		if(this.dropdowns.length > 0){
			for(var idx = 0; idx < this.dropdowns.length; idx++){
				if(this.dropdowns[idx] && this.dropdowns[idx].destroy){
					this.dropdowns[idx].closeDropDown();
					this.dropdowns[idx].destroy();
				}
			}
		}
		this.inherited(arguments);
	}

});

}

if(!dojo._hasResource["com.ibm.bpm.social.widgets.diagram.Participants"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.bpm.social.widgets.diagram.Participants"] = true;
/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011,2013. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.Participants");















//BPMSocial Packages














//WebViewer Packages - Alternate syntax used to avoid dojo build detection
dojo["require"]("ibm_ilog.diagram.gfxlayout.CanvasLayout");

dojo["require"]("ibm_ilog.diagram.gfxlayout.GridLayout");

dojo["require"]("com.ibm.bpm.wpd.base.bootstrap");

dojo["require"]("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");

dojo["require"]("com.ibm.bpm.wpd.document.bpd.view.BPDViewer");

dojo["require"]("com.ibm.bpm.wpd.core.model.documenttype.providers.BaseDocTypeProvider");

dojo["require"]("com.ibm.bpm.wpd.document.bpd.model.ProcessInstanceDocTypeProvider");

dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.Participants",
	[com.ibm.bpm.social.widgets.common._RestWidget],
	{
	
	userid: null,
	taskid: null,
	instanceid: null,
	clickTargetNode: 1,
	clickTargetImage: 2,
	userImageSize: 20,
	popupUserImageSize: 48,
	livenames: {},
	expertsCache: {},  //Store responses from our /experts REST call, indexed by iid
	instDataCache: {}, //Store response from system /process REST call, indexed by iid
	userDataCache: {}, //Store responses from our /userdata REST call, indexed by userName
	provider: null,
	maoContainer: null,
	notificationCookieKey: 'com.ibm.bpm.social.Participants.NotificationViewed',
	
	serverSideImageRendering: (dojo.isIE <= 9 || com.ibm.bpm.social.util.common.Util.isIPad),
	
	lastClickedIsCounter: false,
	
	scrollHandlerInitialized: false,
	lastTransform: [],
	diagramHeight: 0,
	
	contextMenu: null,
	contextWrapper: null,
	contextButton: null,
	dropdownHelper: null,
	currentTask: null,
	actions: [],
	_lastMouseEvent: null,
	_lastSelectedNode: null,
	
	taskActionsUrl: window.endpointURLObject.bpmRest +"/v1/task/actions?taskIDs={tkiid}",
	
	criticalPathFlowIds: null,
		
    postCreate: function() {
		
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.taskRestHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
		
        dojo.style(this.domNode, "position", "relative");
        dojo.style(this.domNode, "height", "100%");
        dojo.style(this.domNode, "width", "100%");
		
        if (this.instanceid != null) {
    		this.instDataCache[this.instanceid] = com.ibm.bpm.social.util.common.Util.getInstanceData({'diagram': true, 'header': true, 'instanceId': this.instanceid, 'skipCache': true});
    		
            this.loadJsps();
            this.createDiagram();
            this.createFadingNotification();

            dojo.connect(this.domNode, 'onclick', this, function(e){
				if(this.nodeClicked==false && (this.maoContainer!=null || this.dropdownButton!=null) ){
					this._closeOverlays();
				} 
				this.nodeClicked = false;
			});
			
			this._initializeScrollHandler();
        }     
        
        if(dojo.isIE){
        	this.connect(this.domNode, "oncontextmenu", this.onContextMenu);
        }
	},
	
	resize: function(){
		
		if(this.view && this.view.resize){
			this.view.resize();
		}
		
	},

	loadJsps: function() {
		try {
			var httpRequest = new XMLHttpRequest();
			var url = window.endpointURLObject.processPortalSupportJS +"/jsp/diagrammerCss.jsp";
			httpRequest.open("GET", url, true);
			httpRequest.send();
		} catch (e) {
			console.error(e);
		}
	},

    createDiagram: function(){
           var project={};
           var state = "Portal";
           var style = "position: absolute; top: 0px; bottom: 0px; right:0px; left:0px; border:0px";
           
           // In IE align the diagram to the left, for some reason in IE RTL only (doesn't occur on other browsers) the diagram pane gets right shifted
           // In IE the diagram is actually an image with hotspots. In RTL the image is shifted but the hotspots aren't. Forcing the alignment left
           // address this
           if (dojo.isIE)
        	   style = style + "; text-align:left";
           
           var criticalPathFlowIds = new Array();
           if(this.criticalPathFlowIds!=null){
        	   criticalPathFlowIds = this.criticalPathFlowIds;
           }
           
           //dojo.registerModulePath("com.ibm.bpm.wpd", "/WebViewer/widgets/com/ibm/bpm/wpd");
           //dojo.registerModulePath("com.ibm.bpm.wpd.core.model.documenttype.providers", "/WebViewer/widgets/com/ibm/bpm/wpd/core/model/documenttype/providers");
           //dojo.registerModulePath("ibm_ilog.diagram", "/WebViewer/widgets/ibm_ilog/diagram");
           //
           //
           //dojo.require('ibm_ilog.diagram.gfxlayout.CanvasLayout');
           //dojo.require("com.ibm.bpm.wpd.document.bpd.view.BPDViewer");           
           //dojo.require('ibm_ilog.diagram.gfxlayout.CanvasLayout');
           //dojo.require("com.ibm.bpm.wpd.document.bpd.model.ProcessInstanceDocTypeProvider");

           var provider = new com.ibm.bpm.wpd.document.bpd.model.ProcessInstanceDocTypeProvider();
           provider.config = {
                   onNodeContextMenu : dojo.hitch(this, this.createContextMenu),
                   onNodeMouseLeave : dojo.hitch(this, this.hideItem),
                   onNodeClick : dojo.hitch(this, this.showItem),
                   showExecutionPath : true,
                   showCurrentActivites : true,
                   showNote : true,
                   showColor : true,
                   showToolbarOnBreadCrumb: true,
                   showSearchOnToolBar: false,
                   criticalPathFlowIds: criticalPathFlowIds,
                   additionalDataHandlers: [dojo.hitch(this, this.getUserInformation)],
                   userBTD: com.ibm.bpm.social.util.common.Util.getBTDUserPreference()
           };
           
           //if(com.ibm.bpm.social.util.common.Util.isIPad){
        	   //provider.config.onNodeClick = provider.config.onNodeContextMenu; disable this to fix defect 121601
           //}
           
           this.provider = provider;
           var providers = [provider];
           // this is a div in the dom tree to which the webviewer component will be attached to by dojo 		
           var t = dojo.create("div", {}, this.domNode);
           
           // BreadcrumbView will instantiate both the breadcrumb and the bpd diagram view
           //dojo.require("com.ibm.bpm.wpd.base.bootstrap");
           var view =  com.ibm.bpm.wpd.base.createWebViewerComponenet("com.ibm.bpm.wpd.core.view.BreadcrumbView", project, style, t, state, providers);
           
           this.view = view;
           
           if(this.serverSideImageRendering){
        	   var userAvatars = this.getUserInformation(null, this.instanceid, this.instanceid);
				window.SocialPortalUserAvatars = userAvatars;
           }
           
           var params = {
                   instanceIds : this.instanceid,//instanceIds,
                   type : "ProcessInstanceType",
                   bpdId : this.instanceid,//The value from property name bpdId is only used as a key for caching,  
                                       // any unique value pass in would be fine. I do not pass the value from this property to server.                  
                   instanceName : "iBPMinstanceName"//window.parent.instanceName
               };

         // fire the open doc event to open the diagram and the breadcrumb
         //dojo.require("ibm_ilog.diagram.gfxlayout.GridLayout");
         //dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
         (new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent(params)).dispatch();

    },
    
    _initializeScrollHandler: function(){
    	if(!this.serverSideImageRendering){
  		
    		var mod = dojo.query('svg > g > g:first-child > rect'); //'svg > g > g:first-child > rect'

    		if(mod.length==0 && this.scrollHandlerInitialized==false){
    			setTimeout(dojo.hitch(this, this._initializeScrollHandler), 1000);
    			return;
    		}

    		if(mod.length==1){
    			mod = mod[0];
    			
    			this.diagramHeight = 0;
    			var lanes = this.instDataCache[this.instanceid].diagram.lanes;
    			for (var x = 0; x < lanes.length; x++){
    				this.diagramHeight += lanes[x].height;
    			}
    			
    			dojo.connect(mod, (!dojo.isMozilla ? "onmousewheel" : "DOMMouseScroll"),this, dojo.hitch(this, this.__onScrollHandler));	
    			
    			this.scrollHandlerInitialized = true;
    		}
    		
    		var mod = dojo.query('svg > g > g:nth-child(2)'); //'svg > g > g:nth-child(2)'

    		if(mod.length > 0){
    			mod = mod[0];    			
    			dojo.connect(mod, (!dojo.isMozilla ? "onmousewheel" : "DOMMouseScroll"),this, dojo.hitch(this, this.__onScrollHandler));
    		}
    	}
    },
    
    __onScrollHandler: function(e){
    	try{
			  var scroll = e[(!dojo.isMozilla ? "wheelDelta" : "detail")] * (!dojo.isMozilla ? 1 : -1);
			  
			  var trans; 			  
			  if(e.currentTarget.tagName=="rect"){
				  trans = this._parseTransform(e.target.parentElement.nextSibling.getAttribute("transform"));
			  } else if(e.currentTarget.tagName=="g"){
				  trans = this._parseTransform(e.currentTarget.getAttribute("transform"));
			  } else {
				  return;
			  }

			  //Prevent scrolling past the top of the diagram
			  if( scroll < 0 && (this.lastTransform.length==0 || trans[5]==0 || (this.lastTransform.length > 0 && (trans[5] - this.lastTransform[5]) < 0) ) ){
				  dojo.stopEvent(e);
			  }
			  
			  var ownerHeight;
			  if(dojo.isIE){
				  ownerHeight = e.target.ownerSVGElement.height.baseVal.value;
			  } else {
				  ownerHeight = e.target.ownerSVGElement.getAttribute("height");
				  if(ownerHeight){
					  try{
						  ownerHeight = ownerHeight.replace("px", "");
						  ownerHeight = parseFloat(ownerHeight);
					  } catch (e) {
						  ownerHeight = -1;
					  }
			  		}
			  }

			  //Prevent scrolling past the bottom of the diagram
			  if( scroll > 0 && (trans[5] + this.diagramHeight) > ownerHeight && ownerHeight > -1){
				  dojo.stopEvent(e);
			  }
			  this.lastTransform = trans;
		  } catch (error){
			  console.error("Diagram Scroll Handler exception: ",error);
		  }
    },
    
    _parseTransform: function(string){
    	//Parses an SVG string in the form of "matrix(1.00000000,0.00000000,0.00000000,1.00000000,0.00000000,40.00000000)"
    	var x;
    	var pattern = /\d*\.\d*/g;
    	if(dojo.isIE){
    		pattern = /\d*/g;
    	}
    	
    	var transform = [];
    	
    	var bits = string.match(pattern);
    	if(dojo.isIE){
    		var bits2 = [];
    		for(x = 0; x < bits.length; x++){
    			if(bits[x]!=""){
    				bits2.push(bits[x]);
    			}
    		}
    		bits = bits2; 
    	}
    	
    	var num = 0;
    	for(x = 0; x < bits.length; x++){
    		try{
    			num = parseFloat(bits[x]);
    			transform.push(num);
    		} catch(e){
    			transform.push(0);
    		}	
    	}
    	return transform;
    },
    
    getUserInformation: function(model, instanceIds, bpdId){
        // Make ajax call, make sure it is synchronized call.                                      
        // The load handler is the function 'yourCallBack'.
        // Each time user drill in a subprocess, the user data should be retrieved for that subprocess.
        // So here we need an ajax call to get information for that bpdId only. 
        // This function will be called by our component each time when user open a top level instance or drill in a subprocess. 
        // You need to write a jsp or rest handler to get the information each time
        // 
        // In summary, getUserInformation is a function registered with the docTypeProvider, 
        // it will be invoked each time when user drill in a subprocess or open a top level bpd instance.  
        // In getUserInformation, your code will need to retrieve the information from server based on bpdId and instanceId.
        // 
        // I guess you have the instanceIds. but I pass it anyway
        // The model is also passed to you. you do not need to update the model directly.
        // please pass the model to provider.updateModel(model, 'decorateNodeWithImage', params );
        //
        if (!instanceIds) {
          instanceIds = this.instanceid;
        }
        if(!this.instDataCache[instanceIds]){
      	  this.instDataCache[instanceIds] = com.ibm.bpm.social.util.common.Util.getInstanceData({'diagram': true, 'header': true, 'instanceId': instanceIds});
        }              
        var ud = this.getUserData(model, this.instDataCache[instanceIds]);
        
        if(model==null){
        	return ud;
        }
    },
    
    getUserData: function(model,data){
         var params = [];
         var previousFlowObjectID = null;
         var tasks = data.tasks;
         for (var ct in tasks) {
        	 var owner = tasks[ct]["owner"];
             var avatarURL, flowObjectID, userData;
             if (owner) {
                 avatarURL = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(owner);
                 flowObjectID = tasks[ct]["flowObjectID"];
                 if (dojo.isIE == 9 || com.ibm.bpm.social.util.common.Util.isIPad) {
                     if (previousFlowObjectID == flowObjectID) {
                         params.pop();
                     }      
                     previousFlowObjectID = flowObjectID;
                 }
                 userData = {};
	             userData = { activityId: flowObjectID, imagePath: avatarURL };
	             params.push(userData);
             }
         }

       if(model!=null){
    	   this.provider.updateModel(model, 'decorateNodeWithImage', params );
       } else {
    	   return params;
       }
    },

    _extractClickData: function(node, e){
    	
    	//01. Clear previously created popups
		this._closeOverlays();

    	//02. Identify task object by clicked Node
		var nodeId = null;
		if(node.getData){
			var d = node.getData();
	    	nodeId = d.get('id')[0]; //"bpdid:5cd499bd441308ca:37d49876:12e27ee26e3:-67c6"	
		} else if (node.id) { //IE9
            nodeId = node.id;
        } else if (node.target) {            
			nodeId = node.target.getAttribute("name");
		}

        if (nodeId) {        
    	//TODO Possibly need to extract currently selected Instance ID if we are drilling down here
		var task = this.__getTaskObjectByNodeId(nodeId);
		
		var bb = {};
		if(this.serverSideImageRendering){
                var target = node.target ? node.target : e.target; //IE9 e.target
    			bb.x = target.offsetLeft;
    			bb.y = target.offsetTop + 25; //To account for the breadcrumb header
    			bb.width = target.offsetWidth;
    			bb.height = target.offsetHeight;
		} else {
			var diagram = node.viewer.ilogDiagrammer;
			bb = node.getBounds(diagram._graph);
			bb.y += 30; //To account for the breadcrumb header
		}
        }
		
		return { "node": nodeId, "task": task, "boundingBox": bb };
    },
    
    onContextMenu: function(e){ //Only invoked for an IE9-specific context menu
		if(window.currentDiagrammerNode!=null){
			this.createContextMenu(window.currentDiagrammerNode, e);
		}
    },
    
    // create a context menu for an active user task 
    createContextMenu: function(node, e) {

        if (!e) {
            console.log("Participants.createContextMenu event is null, returning.");
            return;
        }
		dojo.stopEvent(e);
    	this._lastMouseEvent = e;
    	this._lastSelectedNode = node;
    	
    	var click = this._extractClickData(node, e);
        if (!click.task) {
            console.log("Participants.createContextMenu click.task undefined, returning.");
            return;
        }

    	if(click.task.length==0){ //Task / Flow Object not found
    		console.log("Participants.createContextMenu::returning due to Task Flow Object not found");
    		return;
    	} /*else if(click.task[0].assignedToType!='user'){ //Task assigned to a group, which we don't display info for
    		console.log("showItem::returning due to non-user assignment");
    		return;
    	}*/
    	this.currentTask = click.task[0];

    	//1. create an invisible DIV at the point of the click
    	this._destroyMenu();
    	
    	if(this.contextWrapper!=null){
    		dojo.destroy(this.contextWrapper);
    	}
    	
		this.contextWrapper = dojo.create("div", { 
    		"style":{
    				"height": "2px",
    				"width": "2px",
    				"position": "absolute",
    				"top": (typeof(e)=="undefined" ? node.pageY : e.pageY)+"px",
    				"left": (typeof(e)=="undefined" ? node.pageX : e.pageX)+"px",
    				"zIndex": 990,
    				"display": "none",
    				"backgroundColor": "transparent"
    		}
    	}, dojo.body());

		if(com.ibm.bpm.social.util.common.Util.isIPad){
			var touchX = 0, touchY = 0;
			if(e.changedTouches.length > 0){
				touchX = e.changedTouches.item(0).pageX;
				touchY = e.changedTouches.item(0).pageY;
			}
			dojo.style(this.contextWrapper, "top", touchY+"px");
			dojo.style(this.contextWrapper, "left", touchX+"px");
		}
		
    	//2. create an actiondropdownhelper
    	if(this.dropdownHelper==null){
    		this.dropdownHelper = com.ibm.bpm.social.widgets.ActionDropDownHelper.getInstance();
    	}
		this.contextMenu = this.dropdownHelper.createDropDown(this.contextWrapper);
    	
    	//3. create and wrap the div with the new actionhandler for tasks
		this.taskRestHandler.getActionsForTask(click.task[0].tkiid, "",
			dojo.hitch(this, this._createContextMenu_onActionsResponse),
			dojo.hitch(this, function(data, xhrArgs) {
				console.error(data, xhrArgs);
			}), true);
    },
    
    _createContextMenu_onActionsResponse: function(data, xhrArgs){
    	for(var x = 0; x < data.tasks.length; x++){
    		if(data.tasks[x].tkiid==this.currentTask.tkiid){
    			this.actions[this.currentTask.tkiid] = data.tasks[x].actions; 
    			this._createContextMenu_withActions(data.tasks[x].actions);
    			break;
    		}
    	}
    },
    
    _createContextMenu_withActions: function(actions){
		if(actions==null && !com.ibm.bpm.social.util.common.Util.isIPad){
    		//Task not found in actions list
    		console.debug("Returning from createContextMenu due to Task Not Found");
    		return;
    	}
		
		if(actions==null){
			actions = {};
		}
    	
		var modifyTaskHandler = new com.ibm.bpm.social.widgets.task.util.ModifyTaskHandler({
          task: this.currentTask,
          availableActions: actions
        });
        this.connect(modifyTaskHandler, "onTaskModified", dojo.hitch(this, function(dueDateTime, priority, assignee){
        	
        	this._updateTaskProperties(this.currentTask.tkiid, dueDateTime, priority, assignee);
        	
        }));
		
        //
        // Create the menu and add the task actions
        //
        this.contextMenu = this.dropdownHelper._createDropDownForTask(this.contextWrapper, this.currentTask, actions, {
          modifyTask: function(){
            modifyTaskHandler.showDialog();
          },
          reassignBackToGroup: function(){
            modifyTaskHandler.reassignBackToGroup();
          },
          viewActivityDetails: dojo.hitch(this, function(){
        	this.showItem(this._lastSelectedNode, this._lastMouseEvent);
          })
        }, false /* no collaboration */, true /* omit instance link */);
        //
        // Add the process actions if necessary
        //
        if (this.contextMenu != null && typeof this.currentTask.piid === "string") {
        
          this.dropdownHelper.addProcessActions(this.contextMenu, this.instDataCache[this.currentTask.piid], {
            onCompletedCallback: function(){
              dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM);
            },
            onCanceledCallback: function(){
              dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM);
            },
			onMessageClosedCallback: function(){
              dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_PROCESS_DIAGRAM);
			}
          }, true /* use ad-hoc dialogs */, true /* omit instance link */, true /* omit process diagram */, true /* omit critical path */);
        }
        if (this.contextMenu != null) {
          com.ibm.bspace.common.util.ui.Helper.wrapElement(this.contextWrapper);
          dojo.style(this.contextWrapper, "display", "block");
          
          this.connect(this.contextMenu, "onItemClick", this._hideMenuAnchor);
          this.connect(this.contextMenu, "onCancel", this._hideMenuAnchor);
        }
    	
    	//4. open the menu... (71306 - Wrap in a block to check that menu has valid children before opening (or replace "Loading..." with a "No available actions" item)
        this.contextMenu._openMyself({
        	target: this.contextWrapper,
        	coords: dojo.coords(this.contextWrapper)
        });
        this.contextMenu.focusFirstChild();
    },
    
    _updateTaskProperties: function(tkiid, dueDateTime, priority, assignee){
    	
    	dojo.publish(com.ibm.bpm.social.portal.Topics.TASK_UPDATED+"."+tkiid, [{dueDateTime: dueDateTime ,priority: priority, assignee: assignee}]);
    	
    },
    
    _hideMenuAnchor: function(){
    	if(this.contextWrapper!=null){
    		dojo.style(this.contextWrapper, "display", "none");
    	}
    },
    
    _destroyMenu: function(){
    	if (this.contextMenu!=null && this.contextMenu.destroy){
    		this.contextMenu.destroy();
    		com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this.contextWrapper);
    		dojo.style(this.contextWrapper, "display", "none");
    		this.contextMenu = null;
		}
    },

    hideItem: function(node, e) {
    	//console.log("Test hideItem", node, e);
    },
    
    _closeOverlays: function(){
   	
    	if(this.dropdownButton && this.dropdownButton.destroy){
			this.dropdownButton.closeDropDown(null, true);
			this.dropdownButton.destroy();
    		this.dropdownButton = null;
    	}

		if(this.maoContainer && this.maoContainer.destroy){
			//TODO Close dropdown?
			this.maoContainer.destroy();
			this.maoContainer = null;
		}
		
    },

    showItem: function(node, e) {
		this.nodeClicked = true;
		
    	//00.  Close all previous overlays on a zoom
		if(node.viewer && node.viewer.zoomSlider){
			var connectId = dojo.connect(node.viewer.zoomSlider, 'onChange', this, function(evt){
				this._closeOverlays();
				dojo.disconnect(connectId);
			});
		}
    	
    	//00.  Close all previous overlays on a scroll
    	try{ 
    		//Connect to the 'DOMAttrModified' event of the <g> element that is getting moved horizontally or vertically when scroll is used
    		var mod = dojo.query('svg > g > g:nth-child(2)');
    		
    		if(mod.length==1){
    			mod = mod[0];
    			dojo.connect(mod,'DOMAttrModified', this,function(evt){
        			if(evt.attrName=='transform'){
        				this._closeOverlays();
        			}
        		});
    		}
    		
    	} catch (error) {
    		console.error(error);
    	}
    	
    	var click = this._extractClickData(node, e);
    	if(click.task.length==0){ //Task / Flow Object not found
    		console.log("showItem::returning due to Task Flow Object not found");
    		return;
    	} /*else if(click.task[0].assignedToType!='user'){ //Task assigned to a group, which we don't display info for
    		console.log("showItem::returning due to non-user assignment");
    		return;
    	}*/
 		
		//03. Perform action based on click target
    	/*
    	var tgt = this.clickTargetNode;
    	if(click.task[0].assignedToType=='user'){
    		tgt = this._determineClickTarget(node, e, click.boundingBox);
    	}
    	
    	switch(tgt){
    		case this.clickTargetNode:
    			this._handleNodeClick(node, e, click.task, click.boundingBox);
    			break;
    		case this.clickTargetImage:
    			this._handleImageClick(node, e, click.task, click.boundingBox);
    			break;
    	}*/
    	this._handleNodeClick(node, e, click.task, click.boundingBox);
    },

	destroy: function() {
		
		this._destroyMenu();
		
		if(this.view && this.view.destroyRecursive){
			this.view.destroyRecursive();
			this.view = null;
			try{	dojo.destroy("bpdEditor");	} catch(e){ }
		} 

		if(this.dropdownButton && this.dropdownButton.destroy){
			this.dropdownButton.destroy();
			this.dropdownButton = null;
		}			
			
		if(this.maoContainer && this.maoContainer.destroy){
			this.maoContainer.destroy();
			this.maoContainer = null;
		}
		
		this.inherited(arguments);
	},
	
	_determineClickTarget: function(node, evt, bb){
		var isDecorator = false;
		if(this.serverSideImageRendering) { //IE8
			//console.debug("_determineClickTarget for IE8...");
			this.lastClickedIsCounter = (node.target.getAttribute("counter")=="true");
			isDecorator = (node.target.getAttribute("clickTargetImage")=="true");
		} else if(dojo.isIE==9) { //IE9
			//console.debug("_determineClickTarget for IE9...");
			if(node._data.item.bpmn2TaskType=="UserTask"){
				var lcl = { 'x': bb.x, 'y': bb.y, 'height': bb.height, 'width': bb.width };
				lcl.y += (node._data.item.counter ? 15:0) + 45; //IE takes into account the header, whereas other browsers do not			
				this.lastClickedIsCounter = (typeof(node._data.item.counter)!="undefined");
				if (evt.pageX) {
					if ( (lcl.x < evt.pageX && evt.pageX < (lcl.x + 18) )  && ( lcl.y < evt.pageY && evt.pageY < (lcl.y + 18) ) ) {
						isDecorator = true;
					}
				}
			}
		} else { //FF, Safari
			//console.debug("_determineClickTarget for FF/Safari...");
			if(evt.target instanceof SVGImageElement){
				if(typeof(node._data.item.decorator)!="undefined"){
					var src = node._data.item.decorator[0];
					var tgt = evt.target.attributes.getNamedItem('xlink:href').nodeValue;
					if(src==tgt){
						isDecorator = true;                    
					}
				}
				this.lastClickedIsCounter = (typeof(node._data.item.counter)!="undefined");
			}
		}
		return (isDecorator ? this.clickTargetImage : this.clickTargetNode);
	},
	
	__getTaskObjectByNodeId: function(nodeId){
		var t = this.instDataCache[this.instanceid].tasks;
    	var taskObj = [];
    	for(var idx = 0; idx < t.length; idx++){
    		if(t[idx].flowObjectID==nodeId){
    			taskObj.push(t[idx]);
    		}
    	}
		return taskObj;
	},
	
	_handleNodeClick: function(node, e, taskObj, bb){
		//console.log("_handleNodeClick::entering...",node,e,taskObj,bb);
		
		var position = ["below"];
		
		//01. Determine location
		bb.x = bb.x + (bb.width/2) - 20;	//15px negative offset to account for
		//bb.y = bb.y + bb.height - 20;		//tooltip dropdown offset
		
		var dropdownHeight = 250; //Height + margins
		
		if(dojo.isIE){
			if(document.documentElement.offsetHeight - (bb.y + bb.height) < dropdownHeight){
				//Make sure enough room for dropdown
				//If not, move button to top of node
				bb.y = bb.y - 20;
				position = ["above"];
			} else {
				bb.y = bb.y + bb.height - 20;
			}
		} else {
			if( (window.innerHeight - bb.y - bb.height) < dropdownHeight ){
				//Make sure enough room for dropdown
				//If not, move button to top of node
				bb.y = bb.y - 20;
				position = ["above"];
			} else {
				bb.y = bb.y + bb.height - 20;
			}
		}
		
    	//02. Create dropdown
		this.dropdownButton = new com.ibm.bpm.social.widgets.diagram.ActivityDetailsDropDownButton({
			'tasks':taskObj,
			'style': "left:"+bb.x+"px; top:"+bb.y+"px;",
			'dropDownPosition': position
		}
		,dojo.create("div", {}, this.domNode)); //previously dojo.body() on IBPM
		
   	},

	_handleImageClick: function(node, e, taskObj, bb){
		//console.log("_handleImageClick::entering...",node,e,taskObj,bb);
    
		//01. Determine location
		bb.x = bb.x  - this.popupUserImageSize + 12;
		bb.y = bb.y  - this.popupUserImageSize + 10 + (this.lastClickedIsCounter==true ? ( (dojo.isIE < 9)?0:10):0);
		
    	//02. Create dropdown
		this.maoContainer = new com.ibm.bpm.social.widgets.diagram.MultipleActivityOwnerContainer({
			'tasks': taskObj,
			'style': "left:"+bb.x+"px; top:"+bb.y+"px;",
			'displayDropDown': true,
			'displayDropDownIndex': 0
		},dojo.create("div", {}, this.domNode));  //previously dojo.body() on IBPM
	},
		
	createFadingNotification: function(){
		//dojo.cookie(this.notificationCookieKey, "0", { 'expires': -1}); //NOTE: Delete for production
		if(!dojo.cookie(this.notificationCookieKey)){
			dojo.cookie(this.notificationCookieKey, "1", {'expires': 5});
			new com.ibm.bpm.social.Aether({
				'message': "<div>"+this.messages.participantsInfoText+"</div>",
				'position': 'top-center',
				'fade': true,
				'parent': this,
				'displayCloseIcon': true
			});
		}
	}
	
});

}


dojo.i18n._preloadLocalizations("com.ibm.bpm.nls.social_diagram", ["ROOT","ar","cs","da","de","el","en","es","fi","fr","he","he-il","hu","it","iw","iw-il","ja","ko","nb","nl","no","pl","pt","pt-br","ro","ru","sk","sv","tr","xx","zh","zh-tw"]);
