/*
	Copyright (c) 2004-2012, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

dojo.provide("ibm_ilog.diagram.ibm_ilog_diagram_base");
if(!dojo._hasResource["dojox.collections._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections._base"] = true;
dojo.provide("dojox.collections._base");

dojox.collections.DictionaryEntry=function(/* string */k, /* object */v){
	//	summary
	//	return an object of type dojox.collections.DictionaryEntry
	this.key=k;
	this.value=v;
	this.valueOf=function(){
		return this.value; 	//	object
	};
	this.toString=function(){
		return String(this.value);	//	string
	};
}

/*	Iterators
 *	The collections.Iterators (Iterator and DictionaryIterator) are built to
 *	work with the Collections included in this module.  However, they *can*
 *	be used with arrays and objects, respectively, should one choose to do so.
 */
dojox.collections.Iterator=function(/* array */arr){
	//	summary
	//	return an object of type dojox.collections.Iterator
	var a=arr;
	var position=0;
	this.element=a[position]||null;
	this.atEnd=function(){
		//	summary
		//	Test to see if the internal cursor has reached the end of the internal collection.
		return (position>=a.length);	//	bool
	};
	this.get=function(){
		//	summary
		//	Get the next member in the collection.
		if(this.atEnd()){
			return null;		//	object
		}
		this.element=a[position++];
		return this.element;	//	object
	};
	this.map=function(/* function */fn, /* object? */scope){
		//	summary
		//	Functional iteration with optional scope.
		return dojo.map(a, fn, scope);
	};
	this.reset=function(){
		//	summary
		//	reset the internal cursor.
		position=0;
		this.element=a[position];
	};
}

/*	Notes:
 *	The DictionaryIterator no longer supports a key and value property;
 *	the reality is that you can use this to iterate over a JS object
 *	being used as a hashtable.
 */
dojox.collections.DictionaryIterator=function(/* object */obj){
	//	summary
	//	return an object of type dojox.collections.DictionaryIterator
	var a=[];	//	Create an indexing array
	var testObject={};
	for(var p in obj){
		if(!testObject[p]){
			a.push(obj[p]);	//	fill it up
		}
	}
	var position=0;
	this.element=a[position]||null;
	this.atEnd=function(){
		//	summary
		//	Test to see if the internal cursor has reached the end of the internal collection.
		return (position>=a.length);	//	bool
	};
	this.get=function(){
		//	summary
		//	Get the next member in the collection.
		if(this.atEnd()){
			return null;		//	object
		}
		this.element=a[position++];
		return this.element;	//	object
	};
	this.map=function(/* function */fn, /* object? */scope){
		//	summary
		//	Functional iteration with optional scope.
		return dojo.map(a, fn, scope);
	};
	this.reset=function() {
		//	summary
		//	reset the internal cursor.
		position=0;
		this.element=a[position];
	};
};

}

if(!dojo._hasResource["dojox.collections.Dictionary"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections.Dictionary"] = true;
dojo.provide("dojox.collections.Dictionary");


dojox.collections.Dictionary=function(/* dojox.collections.Dictionary? */dictionary){
	//	summary
	//	Returns an object of type dojox.collections.Dictionary
	var items={};
	this.count=0;

	//	comparator for property addition and access.
	var testObject={};

	this.add=function(/* string */k, /* object */v){
		//	summary
		//	Add a new item to the Dictionary.
		var b=(k in items);
		items[k]=new dojox.collections.DictionaryEntry(k,v);
		if(!b){
			this.count++;
		}
	};
	this.clear=function(){
		//	summary
		//	Clears the internal dictionary.
		items={};
		this.count=0;
	};
	this.clone=function(){
		//	summary
		//	Returns a new instance of dojox.collections.Dictionary; note the the dictionary is a clone but items might not be.
		return new dojox.collections.Dictionary(this);	//	dojox.collections.Dictionary
	};
	this.contains=this.containsKey=function(/* string */k){
		//	summary
		//	Check to see if the dictionary has an entry at key "k".
		if(testObject[k]){
			return false;			// bool
		}
		return (items[k]!=null);	//	bool
	};
	this.containsValue=function(/* object */v){
		//	summary
		//	Check to see if the dictionary has an entry with value "v".
		var e=this.getIterator();
		while(e.get()){
			if(e.element.value==v){
				return true;	//	bool
			}
		}
		return false;	//	bool
	};
	this.entry=function(/* string */k){
		//	summary
		//	Accessor method; similar to dojox.collections.Dictionary.item but returns the actual Entry object.
		return items[k];	//	dojox.collections.DictionaryEntry
	};
	this.forEach=function(/* function */ fn, /* object? */ scope){
		//	summary
		//	functional iterator, following the mozilla spec.
		var a=[];	//	Create an indexing array
		for(var p in items) {
			if(!testObject[p]){
				a.push(items[p]);	//	fill it up
			}
		}
		dojo.forEach(a, fn, scope);
	};
	this.getKeyList=function(){
		//	summary
		//	Returns an array of the keys in the dictionary.
		return (this.getIterator()).map(function(entry){
			return entry.key;
		});	//	array
	};
	this.getValueList=function(){
		//	summary
		//	Returns an array of the values in the dictionary.
		return (this.getIterator()).map(function(entry){
			return entry.value;
		});	//	array
	};
	this.item=function(/* string */k){
		//	summary
		//	Accessor method.
		if(k in items){
			return items[k].valueOf();	//	object
		}
		return undefined;	//	object
	};
	this.getIterator=function(){
		//	summary
		//	Gets a dojox.collections.DictionaryIterator for iteration purposes.
		return new dojox.collections.DictionaryIterator(items);	//	dojox.collections.DictionaryIterator
	};
	this.remove=function(/* string */k){
		//	summary
		//	Removes the item at k from the internal collection.
		if(k in items && !testObject[k]){
			delete items[k];
			this.count--;
			return true;	//	bool
		}
		return false;	//	bool
	};

	if (dictionary){
		var e=dictionary.getIterator();
		while(e.get()) {
			 this.add(e.element.key, e.element.value);
		}
	}
};

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");


dojo.getObject("i18n", true, dojo);

/*=====
dojo.i18n = {
	// summary: Utility classes to enable loading of resources for internationalization (i18n)
};
=====*/

// when using a real AMD loader, dojo.i18n.getLocalization is already defined by dojo/lib/backCompat
dojo.i18n.getLocalization = dojo.i18n.getLocalization || function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		`dojo.requireLocalization()` or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (`var foo = { bar: dojo.i18n.getLocalization() }`).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in `dojo.addOnLoad()`), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
		var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE:
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		`tests/hello/nls/en-us/salutations.js` is loaded as the object
	//		`tests.hello.nls.salutations.en_us={...}` The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.

	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			//Using ["indexOf"] so customBase builds do not see
			//this as a dojo._base.array dependency.
			if(targetLocale["indexOf"](flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(dojo.config.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					hash = hash.root || hash;
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}

			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default.
	//
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = dojo.config.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = dojo.config.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource['ibm_ilog.diagram.util.ErrorReporter']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.ErrorReporter'] = true;
dojo.provide('ibm_ilog.diagram.util.ErrorReporter');

if(dojo.config._notDiagramForMobile)


(function(){
	var e = ibm_ilog.diagram.util.ErrorReporter;

	var B;

	e.getLocalizedString = function(id){
		// summary:
		//		Returns the localized string for the specified identifier.
		// description:
		//		This function normally fetches the specified string (using the dojo.i18n API)
		//		from the package name "ibm_ilog.diagram" and the bundle name "ErrorReporter".
		//		On mobile platforms (that is, if dojo.config.diagramForMobile is true),
		//		dojo.i18n is not used (to reduce the size of downloaded code), so the id is
		//		returned unchanged. To localize messages on mobile, the application should
		//		replace this function (ibm_ilog.diagram.util.ErrorReporter.getLocalizedString)
		//		by a function that requests the localized string from the server.
		if(dojo.config.diagramForMobile)
			return id;
		try { // workaround for i18n bug in dojo 1.7...
			if (!B) 
				B = dojo.i18n.getLocalization("ibm_ilog.diagram", "ErrorReporter");
			return B[id];
		} catch(error){
			return id;
		}
	};
	
	e._errors = {};
	
	e.error = function(id) {
		throw this.buildError.apply(this,arguments);
	};

	e.warn = function(id) {
		console.warn(this.buildDescription.apply(this,arguments));
	};

	e.buildError = function(id) {
		var e = new Error(this.buildDescription.apply(this,arguments));
		e._isIBMDiagramError = true;
		var ed = this._errors[id];
		if(ed!==undefined) {
			e.id = ed.id;
			e.errorCode = ed.errorCode; 
		}
		return e;
	};
	
	e.buildDescription = function(id) {
		var err = this._errors[id];
		if(!err) {
			err = this._errors.InvalidErrorId;
		}
		var description = err.descriptor.apply(this,arguments);
		return err.errorCode+ ": " + description + " ("+err.id+")";
	};

	
	e._errorAtObject = function(obj,args) {
		// TODO [AV] obj is ignored for now
		this.error.apply(this,args);
	};
	
	var toKw = function(args,names,start) {
		var kwArgs = {}
		start = start || 0;
		for(var i=0;i<names.length;i++) {
			kwArgs[names[i]] = args[start+i];
		}
		return kwArgs;
	};
	
	var buildStandardDescriptor = function(/*Array*/kwArgNames) {
		// 
		//	summary:
		//		Builds a standard error descriptor function.
		//
		//	description:
		//		This function takes names for the positional user arguments 
		//		in the ErrorReporter.error() call so that they can be referenced
		//		from the I18N string templates easier.
		//
		//	kwArgNames: Array
		//		list of the argument names for using them in the call to 
		//		dojo.string.substitute
		//
		return function(id) {
			return dojo.string.substitute(e.getLocalizedString(id),toKw(arguments,kwArgNames,1));
		};

	};
	
	e.declareError = function(errorCode,id,descriptor) {
		// 
		//	summary:
		//		Error declaration helper
		//
		//	errorCode: String
		//		The IBM standard error code.
		//
		//	id: String
		//		An alphabetic error name used to reference it from the 
		//		code; making it more readable.
		//
		//	descriptor: String|Array|Function
		//		The error message or message generator.
		//	
		//		It can be three: 
		//		* message itself (Non-I18Nized)
		//		* a function which handles the R.error() arguments directly
		//		* an array naming the arguments of the R.error() call; for 
		//		  which a function is built that calls dojo.string.substitute 
		//		  with these names for I18N. (We call this the standard descriptor)  
		//
		var error = {};
		error.errorCode = errorCode;
		error.id = id;
		if(dojo.isFunction(descriptor)) {
			error.descriptor = descriptor;
		} else if(dojo.isArray(descriptor)) { 
			error.descriptor = buildStandardDescriptor(descriptor);
		} else {
			error.descriptor = function() {
				return ""+descriptor;
			};
		}
		if(this._errors[id]!==undefined) {
			throw new Error(dojo.string.substitute(e.getLocalizedString("ErrorIdAlreadyDeclared"),id));
		}
		this._errors[id] = error;
	};
	
	//
	// ERROR DECLARATIONS
	//	

	var errorTable =
	[

	 
	 	//-------------------------------------------------------
		// ErrorReporter internals
	 	//
		{	name: "Unknown" },
		{ name: "InvalidErrorId", args:["errorId"] },
		{	name: "ErrorIdAlreadyDeclared", args:["errorId"] },

		//-------------------------------------------------------
		// Generic-JavaScript
		//
		{ name: "SubclassResponsibility", args:["methodName"] },
	 	{
		 	name: "NullPointerException",
			descriptor: function(id, objectName, where) {
				return where?
						dojo.string.substitute(e.getLocalizedString("NullPointerException2"),{objectName:objectName,where:where}):
						dojo.string.substitute(e.getLocalizedString("NullPointerException"),{objectName:objectName});
			}
		},
		{
		 	name: "Wrapped",
		 	descriptor: function(id, process, error) {
		 		if(error.message) {
					error = error.message;
				}
				return dojo.string.substitute(e.getLocalizedString(id),{process:process,error:error});
			}
		},
		{
		 	name: "UnexpectedArgument",
		 	descriptor: function(id, sender, functionName, argumentName, actualArgument) {
				// summary:
				// 		Throws an error indicating that an argument is not of the expected type or range. 
				// sender:
				//		The object that raised the error.
				// functionName:
				//		The name of the function in which the error occurred.
				// argumentName:
				//		The name of the argument that received a wrong value.
				// actualArgument:
				//		The argument value that was passed and detected as unexpected.
				return dojo.string.substitute(e.getLocalizedString(id),toKw(arguments,["sender","functionName","argumentName","actualArgument"],1));
			}
		},
		{
		 	name: "InvalidNumberOfArguments",
		 	descriptor: function(id, args) {				
				return dojo.string.substitute(e.getLocalizedString(id),{
					numberOfArgs: args.length,
					options: Array.prototype.slice.call(arguments,4).toString()});
			}
		},
		{
			name:"UnexpectedReturnValueType", args: ["sender", "functionName", "expectedType", "actualType"]
		},
		
		
		// ------------------------------------------------------
		// Dojo-version
		//
		{ name:"UnsupportedDojoFeature", args:["version"] },
		
		
		
		
		//-------------------------------------------------------
		// Generic-Graph/Diagram
		//
		{	name: "GraphElementExpected" },
		
		
		
		
		
		//-------------------------------------------------------
		// DataStore
		//
		{	name: "NotInDataStore" },
		{	name: "CannotUseAStringForAQuery" },
		{	name: "NodesStoreRequired" },
		{	name: "StoresMustSupportIdentity" },
		{	name: "StoresMustSupportRead" },
		{
			name: "UnexpectedErrorOnNodesStoreFetch",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return dojo.string.substitute(e.getLocalizedString(id),{errorString:error});
			}
		},
		{
			name: "UnexpectedErrorOnLinksStoreFetch",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return dojo.string.substitute(e.getLocalizedString(id),{errorString:error});
			}
		},

		
		
		
		//-------------------------------------------------------
		// Overview
		//
		{	name: "OverviewDiagramMissing", args: ["diagram"] },

		
		
		//-------------------------------------------------------
		// WAI
		//
		{	name: "MissingWaiNode", args: ["element"] },

		
		
		//-------------------------------------------------------
		// ServerSideLayout
		//
		{
			name: "UnsupportedAlgorithm",
			descriptor: function(id,algorithm){
				return dojo.string.substitute(e.getLocalizedString(id),{algorithmClass:algorithm.declaredClass});
			}
		},


		
		//-------------------------------------------------------
		// Interactors
		//
		{	name: "RestrictedToOneInteractor" },
		{
		 	name: "InvalidInteractorId",
			descriptor: function(id, interactorId) {
				return dojo.string.substitute(e.getLocalizedString(id),{interactorId:dojo.toJson(interactorId)});
			}
		},
		{	name: "InvalidInteractorEvent", args: ["name"] },
		{	name: "InvalidInputFilterSpec", args: ["name"] },



		
		//-------------------------------------------------------
		// UI events/connections
		//
		{	name: "ConnectionStillActive", args: ["id"]	},
		{	name: "EventConnectionFailed", args: ["eventName"] },

		
		
		
		//-------------------------------------------------------
		// Viewport
		//
		{	name: "MaxZoomBelowMin" },
		{	name: "MinZoomAboveMax" },

		
		//-------------------------------------------------------
		// Swim lanes
		//
		{	name: "SwimLanesRequireGfxLayout" },
		

		//-------------------------------------------------------
		// User-defined functions
		//
		{
			name: "DropUserFunctionError",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return dojo.string.substitute(e.getLocalizedString(id),{error:error});
			}
		},
		{
			name: "UndoUserActionError",
			descriptor: function(id, action, error){
				if(error.message) {
					error = error.message;
				}
				return dojo.string.substitute(e.getLocalizedString(id),{action:action,error:error});
			}
		},

		
		
		
		//-------------------------------------------------------
		//
		{	name: "LASTERRORCODE" }
	];
	
	//	
	//	Iterate the table and declare the errors.
	//
	
	for( var i = 0; i<errorTable.length; i++ ) {
		var code = "CWZDD"+(2000+i)+"E"; // Should we better put this on the table and take care of possible errors/repetitions/maintainance problems/etc.?
		// AVA: yes, it think it would be better. My experience: I got 
		// CWZDD2016E: A node or link layout algorithm is needed. 
		// Then, to debug, I grepped for "2016" everywhere and found nothing...
		var ed = errorTable[i];
		e.declareError(code,ed.name,ed.descriptor||ed.args||ed.description||[]);
	}
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.HandleSet']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.HandleSet'] = true;
dojo.provide('ibm_ilog.diagram.util.HandleSet');




(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;

dojo.declare("ibm_ilog.diagram.util.HandleSet", null, {

	_connectionHandles: null,

	constructor: function () {
		this._connectionHandles = new dojox.collections.Dictionary();
	},

	_connect: function (id, source, eventName, target, handlerName, connector, disconnector) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		var h;
		connector = connector || dojo.connect;
		disconnector = disconnector || dojo.disconnect;
		if (this._connectionHandles.containsKey(id)) {
			R.warn("ConnectionStillActive", id);
		} else {
			switch (arguments.length) {
			case 5:
			case 7:
				h = connector(source, eventName, target, handlerName);
				break;
			default:
				R.error("InvalidNumberOfArguments", arguments, 5, 7);
				break;
			}
		}

		if (h) {
			this._addHandle(id, h, source, disconnector);
		}
	},

	_isConnected: function (id) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		return this._connectionHandles.containsKey(id);
	},

	_disconnect: function (id) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		var handle = this._connectionHandles.item(id);
		if (this._connectionHandles.remove(id)) {
			handle.disconnector(handle.h, handle.src);
		}
	},

	_addHandle: function ( /*String*/ id, /*dojo handler*/ handle, /*Obj*/ src, /*Function*/ disconnector) {
		//
		// summary:
		//		add a handle to the handles list
		// handle:
		//		the handle to be added
		//
		this._connectionHandles.add(id, {
			h: handle,
			src: src,
			disconnector: disconnector
		});
	},


	_disconnectAll: function () {
		//
		// summary:
		//		release all the handles.
		//
		dojo.forEach(this._connectionHandles.getValueList(), function (handle) {
			handle.disconnector(handle.h, handle.src);
		});
		this._connectionHandles.clear();
	}
});

ibm_ilog.diagram.util.HandleSet.gfxConnector = function(src,evt,object,handler) {
	return src.connect(evt,object,handler);
};

ibm_ilog.diagram.util.HandleSet.gfxDisconnector = function(handle,src) {
	return src.disconnect(handle);
};


})();

}

if(!dojo._hasResource["dojox.collections.ArrayList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections.ArrayList"] = true;
dojo.provide("dojox.collections.ArrayList");


dojox.collections.ArrayList=function(/* array? */arr){
	//	summary
	//	Returns a new object of type dojox.collections.ArrayList
	var items=[];
	if(arr) items=items.concat(arr);
	this.count=items.length;
	this.add=function(/* object */obj){
		//	summary
		//	Add an element to the collection.
		items.push(obj);
		this.count=items.length;
	};
	this.addRange=function(/* array */a){
		//	summary
		//	Add a range of objects to the ArrayList
		if(a.getIterator){
			var e=a.getIterator();
			while(!e.atEnd()){
				this.add(e.get());
			}
			this.count=items.length;
		}else{
			for(var i=0; i<a.length; i++){
				items.push(a[i]);
			}
			this.count=items.length;
		}
	};
	this.clear=function(){
		//	summary
		//	Clear all elements out of the collection, and reset the count.
		items.splice(0, items.length);
		this.count=0;
	};
	this.clone=function(){
		//	summary
		//	Clone the array list
		return new dojox.collections.ArrayList(items);	//	dojox.collections.ArrayList
	};
	this.contains=function(/* object */obj){
		//	summary
		//	Check to see if the passed object is a member in the ArrayList
		for(var i=0; i < items.length; i++){
			if(items[i] == obj) {
				return true;	//	bool
			}
		}
		return false;	//	bool
	};
	this.forEach=function(/* function */ fn, /* object? */ scope){
		//	summary
		//	functional iterator, following the mozilla spec.
		dojo.forEach(items, fn, scope);
	};
	this.getIterator=function(){
		//	summary
		//	Get an Iterator for this object
		return new dojox.collections.Iterator(items);	//	dojox.collections.Iterator
	};
	this.indexOf=function(/* object */obj){
		//	summary
		//	Return the numeric index of the passed object; will return -1 if not found.
		for(var i=0; i < items.length; i++){
			if(items[i] == obj) {
				return i;	//	int
			}
		}
		return -1;	// int
	};
	this.insert=function(/* int */ i, /* object */ obj){
		//	summary
		//	Insert the passed object at index i
		items.splice(i,0,obj);
		this.count=items.length;
	};
	this.item=function(/* int */ i){
		//	summary
		//	return the element at index i
		return items[i];	//	object
	};
	this.remove=function(/* object */obj){
		//	summary
		//	Look for the passed object, and if found, remove it from the internal array.
		var i=this.indexOf(obj);
		if(i >=0) {
			items.splice(i,1);
		}
		this.count=items.length;
	};
	this.removeAt=function(/* int */ i){
		//	summary
		//	return an array with function applied to all elements
		items.splice(i,1);
		this.count=items.length;
	};
	this.reverse=function(){
		//	summary
		//	Reverse the internal array
		items.reverse();
	};
	this.sort=function(/* function? */ fn){
		//	summary
		//	sort the internal array
		if(fn){
			items.sort(fn);
		}else{
			items.sort();
		}
	};
	this.setByIndex=function(/* int */ i, /* object */ obj){
		//	summary
		//	Set an element in the array by the passed index.
		items[i]=obj;
		this.count=items.length;
	};
	this.toArray=function(){
		//	summary
		//	Return a new array with all of the items of the internal array concatenated.
		return [].concat(items);
	}
	this.toString=function(/* string */ delim){
		//	summary
		//	implementation of toString, follows [].toString();
		return items.join((delim||","));
	};
};

}

if(!dojo._hasResource['ibm_ilog.diagram.Selectable']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Selectable'] = true;
dojo.provide('ibm_ilog.diagram.Selectable');



dojo.declare("ibm_ilog.diagram.Selectable", [], {
	// summary:
	//		Selectable class group all the functionality to store and manage the selection state and style of a component.
	//      Link and Node must inherit this class in order to be selectable.
	
	// Allows to test the class faster that testing declaredClass
    _isIBMDiagramSelectable: true,
    
	// Internal selection state
	_selected: false,
	// enabled for select
	_selectable: true,
    // the list of styled
	_styledChildren : null,
   
    isSelected: function() {
	// summary:
	//		returns the selected state
	// returns:
	//      true if selected
		return this._selected;
	},
	setSelected: function(selected) {
		// summary:
		//		sets the selected state
		// selected:
		//		the new selection state
		// returns:
		//		true if the selection state could be changed
		this._selected = selected;
		this._changeSelectionStyle(selected);
		return true;
	},
	toggleSelected: function(){
		// summary:
		//		toggle the selection state
		// returns:
		//		true if the selection state could be changed
		var s = !this._selected;
		return this.setSelected(s);
	},
	setSelectable: function(selectable){
		// summary:
		//		enable or disable selecting
		// enabled:
		// 		true to enable selecting
		this._selectable = selectable;
	},
	isSelectable: function(){
		// summary:
		//		return if the selectable can be selected
		return this._selectable;
	},
	_changeSelectionStyle: function(selected){
		// summary:
		//		change the selection style
		// selected:
		//		the new selection state
		if(this._styledChildren == null){
			this._styledChildren = new dojox.collections.ArrayList();
			this._findAllStyledElements();
		}
		var iterator = this._styledChildren.getIterator();
		while(!iterator.atEnd()){
			var shape = iterator.get();
			this._applySelectedStyle(shape,selected);
		}
	},
	
	_applySelectedStyle: function(shape,selected){
		// summary:
		//		change the selection style of the given shape
		// shape:
		//		the shape to be styled
		// selected:
		//		the new selection state	
		if(selected){
			if(!shape.onSelectionStyle || shape.__selectedStyleChanged){ // Added to fix Defect 344
				if(shape.__selectedStyleChanged){
					this._applyStyle(shape,shape['standardStyle']);
				}
				shape.standardStyle = this._getStandardStyle(shape,shape['selectedStyle']);
				this._applyStyle(shape,shape['selectedStyle']);
				shape.onSelectionStyle = true;
			}
		}else{
			this._applyStyle(shape,shape['standardStyle']);
			shape.standardStyle = null;
			shape.onSelectionStyle = false;
		}
		delete shape.__selectedStyleChanged;
	},
	_findAllStyledElements: function(){
		this._findStyledChildren(this.children);
	},
	_findStyledChildren: function(childrenArray){
		// summary:
		//		find all the children that have to be styled when selecting, and save the standard style
		for (var c in childrenArray){
			var child = childrenArray[c];
			if(!ibm_ilog.diagram.Selectable.Util.isSelectableInstance(child)) {
					if (child['selectedStyle']) {
						this._styledChildren.add(child);
					}
					if (child['children']) {
						this._findStyledChildren(child['children']);
					}
			}
		}
	},
	//TODO this function should be in a tool
	_applyStyle:function(shape, style){
		// summary:
		//		sets a new style to the desired element
		// shape:
		//		the shape to apply the style
		// style:
		//      the new style to be applied
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var setter = 'set' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[setter]) {
					shape[setter](style[p]);
				}
			}
		}
	},
	_getStandardStyle:function(shape, style){
		// summary:
		//		gets the standard style of the given style element
		// shape:
		//		the shape to get the style
		// style:
		//      the style from where to get the elements
		var simpleStyle = {};
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var getter = 'get' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[getter]) {
					simpleStyle[p] = shape[getter]();
				}
			}
		}
		return simpleStyle;
	}
});


(function(){
	ibm_ilog.diagram.Selectable.Util = {
	isSelectableInstance: function(ge){
		// Summary:
		//		this function returns if the given element is an instance of Selectable.
		//		It is a workaround to avoid using isInstanceOf (not supported on dojo 1.3.2)
		if(ge['_isIBMDiagramSelectable']){
			return true;
		}
		return false;
	}
	}
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.Interactor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.Interactor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.Interactor');





(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;

dojo.declare("ibm_ilog.diagram.interactors.InteractorState", [], {
	//
	//    summary:
	//        Represents a state of an interactor in terms of
	//      the connections are active during it.
	//
	_connections: null,

	_id: "",

	_interactor: null,

	constructor: function (interactor, id, connections) {
		this._interactor = interactor;
		this._id = id;
		this._connections = connections;
	},
	
	addConnection: function(c) {
		this._connections.push(c);
	},

	_forEachId: function (fname, s) {
		dojo.forEach(this._connections, s[fname], s);
	},

	enter: function () {
		this._forEachId("_doConnect", this._interactor);
	},

	leave: function (host) {
		this._forEachId("_disconnect", this._interactor);
	}
});

var gfxCon = ibm_ilog.diagram.util.HandleSet.gfxConnector;
var gfxDis = ibm_ilog.diagram.util.HandleSet.gfxDisconnector;

dojo.declare("ibm_ilog.diagram.interactors.Interactor", [ibm_ilog.diagram.util.HandleSet], {
	//
	//    summary:
	//        The interactor defines the API and common functionality for all the event management interactors.
	//
	//    description:
	//        The 
	//

	//
	//    The map with the connections used by the interactor
	//
	_connections: null,

	//
	//    _enabled:
	//        temporarily ignore events, but keeps dojo.connections
	//
	_enabled: true,

	//
	//    _manager:
	//        the mouse interactor manager which coordinates the use of the interactor
	//
	_manager: null,
	
	//
	//    _keyManager:
	//        the keyboard interactor manager which coordinates the use of the interactor
	//
	_keyManager: null,
	
	//
	//    _states:
	//        the states declared for this interactor
	//
	_states: null,

	//
	//    _state:
	//        the current state of the interactor
	//
	_state: null,

	//
	//    _hotKeys:
	//        the current hotKeys of the interactor
	//			format
	//			{ keyCode1:[connectionId1,connectionId2], keyCode2:[connectionId2] }
	//
	_hotKeys: null,

	_declareState: function (id, cids) {
		//
		//    summary:
		//        declares a state of the interactor
		//
		this._states.add(id, new ibm_ilog.diagram.interactors.InteractorState(this, id, cids));
	},

	_goStateId: function (stateId) {
		//
		//    summary:
		//        enters the state with the given id
		//
		this._goState(this._states.item(stateId));
	},

	_goState: function (s) {
		//
		//    summary:
		//        enters the state s
		//
		if (this._state !== s) {
			if (this._state) {
				this._state.leave(this);
			}
			this._state = s;
			if (this._state) {
				this._state.enter(this);
			}
		}
	},

	_declareStates: function () {
		//
		//    summary:
		//        subclasses must override this method for declaring interactor states
		//
	},

	setManager: function (manager) {
		this._manager = manager;
	},
	
	setKeyManager: function (keyManager) {
		this._keyManager = keyManager;
		this.registerHotKeys();
	},
	
	_initialize: function () {
		//
		//    summary:
		//        basic internal initialization
		//
		this.setConnections(this.getDefaultConnections());
		this._states = new dojox.collections.Dictionary();
		this.initializeFilters();
		this._hotKeys = {};
		this._declareStates();
		return this;
	},

	_doConnect: function (id) {
		//
		//    summary:
		//        connects the connection declared with the given id, used by state transitions
		//
		var cdata = this._connections[id];
		if (cdata && !cdata.hotKey) {
			var target = cdata.target || this;
			if (cdata.connectTo in target) {
				var filteredHandler = cdata.filteredHandler;
				try{
					this._connect(id, cdata.src, cdata.srcEvt, this, filteredHandler, cdata.gfxConnect && gfxCon, cdata.gfxConnect && gfxDis);
				} catch(e) {
					R.warn("EventConnectionFailed", cdata.srcEvt);
				}
			} else {
				R.error("InvalidInteractorEvent", cdata.connectTo);
			}
		} else {
			R.error("InvalidInteractorId",id);
		}
	},
	
	keyPressed: function(ids,e){
		var managed = false;
		for(var index in ids){
			var cdata = this._connections[ids[index]];
			if(cdata.hotKey){
				managed = cdata.filteredHandler(e) || managed;
			}
		}
		return managed;
	},
	
	initializeFilters: function () {
		//
		//    summary:
		//       
		//
		for(var id in this._connections){
			var cdata = this._connections[id];
			var target = cdata.target || this;
			if (cdata.connectTo in target) {
				var filter = this._applyFilter(target, cdata.connectTo, cdata.filter);
				cdata.filteredHandler = filter;
			} else {
				R.error("InvalidInteractorEvent", cdata.connectTo);
			}
		}
	},

	registerHotKeys: function(){
		if(this._keyManager){
			this._keyManager.clearHotKeys(this.getKeyInteractorId(),this._hotKeys);
			this._hotKeys = {};
			for(var id in this._connections){
				var cdata = this._connections[id];
				var hk = cdata.hotKey;
				if(!(hk in this._hotKeys)){
					this._hotKeys[hk] = [];
				}
				this._hotKeys[hk].push(id);
			}
			this._keyManager.setHotKeys(this.getKeyInteractorId(),this._hotKeys);
		}
	},
	
	setEnabled: function (b) {
		if (this._enabled != b) {
			this._enabled = b;
			if (b) {
				this.onEnabled();
			}
			if (!b) {
				this.onDisabled();
			}
		}
	},

	getEnabled: function () {
		return this._enabled;
	},

	onEnabled: function () {},
	onDisabled: function () {},

	activate: function () {
		//
		// summary:
		//        Activates "idle" state
		//
		this._goStateId("idle");
	},

	deactivate: function () {
		//
		// summary:
		//        Disconnects all connections by going to the null state
		//
		this._goState(null);

		// shut down other temporary connections
		this._disconnectAll();
	},

	_fconnect: function (id, source, event, obj, handlerName) {
		//
		// summary:
		//        Makes a temporary connection with the standard enabled-only filter
		//
		switch (arguments.length) {
		case 5:
			return this._connect(id, source, event, obj, this._applyFilter(obj, handlerName));
		default:
			R.error("InvalidNumberOfArguments", arguments, 5);
		}
	},

	toString: function () {
		return this.declaredClass.split(".").pop();
	},

	setConnections: function (connections) {
		//
		//    summary:
		//        sets a new connections configuration
		//
		this._connections = connections;
	},

	overrideConnections: function (connections) {
		//
		//    summary:
		//        Override the default event map with the given handlers
		//
		this._connections = dojo.clone(this._connections);
		dojo.mixin(this._connections, connections);
	},

	getConnections: function () {
		//
		//    summary:
		//        return the eventMap
		//
		return this._connections;
	},

	getDefaultConnections: function () {
		//
		//    summary:
		//        This function has to be overriden by subclasses, and used to return the default event map for the Interactor
		//
		return {};
	},

	reactivate: function () {
		//
		// summary:
		//        release all the handles and reconnect.
		//
		this.deactivate();
		this.activate();
	},

	_basicBuildInputFilter: function(key,value) {
		if(value===undefined)
			R.error("InvalidInputFilterSpec",key);
		switch(key) {
		case "condition":
			return value;
		case "button":
			switch(value) {
			case 0:
				return this.eventButtonIsLeft;
			case 1:
				return this.eventButtonIsRight;
			case 2:
				return this.eventButtonIsMiddle;
			default:
				R.error("InvalidInputFilterSpec",key);
			}
		case "alt":
			return function(e) {
				return e.altKey === value;
			};
		case "ctrl":
			return function(e) {
				return e.ctrlKey === value;
			};
		case "shift":
			return function(e) {
				return e.shiftKey === value;
			};
		case "disabled":
			return function(e) {
				return false;
			};
		default:
			R.error("InvalidInputFilterSpec",key);
		}
	},
	
	_buildInputFilter: function(spec) {
		var filter = null;
		for(s in spec) {
			var newFilter = this._basicBuildInputFilter(s,spec[s]);  
			if(spec.hasOwnProperty(s)) {
				if(filter) {
					(function(oldFilter,newFilter) { // fresh scope
						filter = function(e) {
							return oldFilter(e) && newFilter(e);
						};
					})(filter,newFilter);
				} else {
					filter = newFilter;
				}
			}
		};
		return filter;
	},
	
	_assignEventInput: function(id,spec) {
		if('filter' in spec){
			this._connections[id].filter = this._buildInputFilter(spec.filter);
		}
		if('hotKey' in spec){
			this._connections[id].hotKey = spec.hotKey;
		}
	},	
	
	assignInputs: function(assignments) {
		for(id in assignments) {
			if(this._connections.hasOwnProperty(id)) {
				this._assignEventInput(id,assignments[id]);
			} else {
				R.error("InvalidInteractorEvent",id);
			}
		}
		this.initializeFilters();
		this.registerHotKeys();
		if (this._state !== null) {
			this.reactivate();
		}
	},
	
	_applyFilter: function (scope, handlerName, filter) {
		//
		// summary:
		//        if the filter function return true, calls the internal handler given as parameter
		// methodName:
		//        the handler to be called
		// filter:
		//         the filter function to be evaluated
		//
		var thiz = this;
		if (filter) {
			return function (e) {
				if (thiz._enabled && filter(e)) {
					scope[handlerName](e);
					return true;
				}
				return false;
			};
		} else {
			return function (e) {
				if (thiz._enabled) {
					scope[handlerName].apply(scope, arguments);
					return true;
				}
				return false;
			};
		}
	},

	_getGraphElementFromEventInRoot: function (event, root) {
		// Summary:
		//        Returns the GraphElement (i.e. Node or Link) containing the target of the specified event. 
		// event:
		//        The event.
		// root:
		//        The root GFX shape containing the graph elements.
		
		var gfxTarget = event.gfxTarget;
		if (gfxTarget) {
			// We are using a Dojo that adds a gfxTarget field to the event (Dojo 1.6+):
			// use this and look upwards to find the GraphElement.
			var ge = gfxTarget; // a GFX shape
			if (ibm_ilog.diagram.Selectable.Util.isSelectableInstance(ge)) {
				return ge;
			}
			if (!ge.getParent) 
				return null;
			for (var elem = ge.getParent(); elem && elem.getParent; elem = elem.getParent()) {
				if (ibm_ilog.diagram.Selectable.Util.isSelectableInstance(elem)) {
					return elem;
				}
				if (elem === root) 
					break;
			}
		}
		else {
			// We are using an older Dojo: use the old method, that is, search recursively
			// top-to-bottom from the root to find the GFX shape whose event source (= HTML node)
			// is the event target. This does not work for Canvas, so for Canvas we must be in
			// the case above (which means Canvas is not support for "old" Dojo versions).
			return this._getGraphElementFromHtml(event.target, root);
		}
		return null;
	},

	_getGraphElementFromHtml: function(html, root) {
		// Summary:
		//        Returns the GraphElement (i.e. Node or Link) containing the specified (HTML) event target.
		// html: Node
		//        The event target (an HTML node).
		// root:
		//        The root GFX shape containing the graph elements.
		
		// fix for touch event
		// on iOS the event target on a gfx text is a TextNode no the SVGText
		if (!ibm_ilog.diagram.isSilverlight && html.nodeType == 3) 
			html = html.parentNode;
		
		if (dojox.gfx.equalSources(root.getEventSource(), html)) {
			return root;
		} else {
			if (root['children']) {
				for (var index in root.children) {
					var ge = this._getGraphElementFromHtml(html, root.children[index]);
					if (ge) {
						if (ibm_ilog.diagram.Selectable.Util.isSelectableInstance(ge)) {
							return ge;
						}
						else {
							return root;
						}
					}
				}
			}
		}
		
		return null;
	},

	__eod: undefined
});

//////////////////////////////////////////////////////////////////////////////
//
//    DOM Events Helper Functions
//
// dojo 1.4 has this already, but we need to support 1.3.2
if (dojo.isIE < 9) {
	if(dojox.gfx.renderer == 'vml') {
		dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
			eventButtonIsLeft: function (e) {
				return e.button & 1;
			}, eventButtonIsMiddle: function (e) {
				return e.button & 4;
			}, eventButtonIsRight: function (e) {
				return e.button & 2;
			}
		});
	} else {
		if(dojox.gfx.renderer == 'silverlight') {
			dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
				eventButtonIsLeft: function (e) {
					return true;
				}, eventButtonIsMiddle: function (e) {
					return false;
				}, eventButtonIsRight: function (e) {
					return false;
				}
			});
		}
	}
} else {
	dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
		eventButtonIsLeft: function (e) {
			return e.button == 0;
		}, eventButtonIsMiddle: function (e) {
			return e.button == 1;
		}, eventButtonIsRight: function (e) {
			return e.button == 2;
		}
	});
}

if (dojo.isMozilla) {
	dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
		eventNameWheel: "DOMMouseScroll",
		eventWheelAmmount: function (e) {
			return -e.detail;
		}
	});
} else {
	dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
		eventNameWheel: "onmousewheel",
		eventWheelAmmount: function (e) {
			return e.wheelDelta;
		}
	});
}

dojo.extend(ibm_ilog.diagram.interactors.Interactor, {
	eventPageLocation: function (e) {
		//
		//    summary:
		//        returns pointer position associated with the given event
		//
		return {
			x: e.pageX,
			y: e.pageY
		};
	}, eventClientLocation: function (e) {
		//
		//    summary:
		//        returns pointer position associated with the given event
		//
		var r = dojo.coords(e.currentTarget);
		return {
			x: e.clientX - r.x,
			y: e.clientY - r.y
		};
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.Geometry']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.Geometry'] = true;
dojo.provide('ibm_ilog.diagram.util.Geometry');



(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;

ibm_ilog.diagram.util.Geometry = {};

var g = ibm_ilog.diagram.util.Geometry;


	//
	//	Representation for the empty rectangle
	//		
	g.EmptyRect = { x: 0, y: 0, width: 0, height: 0 };

	
  g.isEmptyRect = function( /*dojox.gfx.Rectangle*/ r ) {
	//	returns:
	//		true when r is the empty rectangle.
      return r.x === 0 && r.y === 0 && r.width === 0 && r.height === 0;
  };
  

  g.equalRect = function( /*dojox.gfx.Rectangle*/ r1, /*dojox.gfx.Rectangle*/ r2) {
  	//	returns:
  	//		true when the two rects are equal
      return (r1.x === r2.x) &&
         (r1.y === r2.y) &&
         (r1.width === r2.width) &&
         (r1.height === r2.height);
  };


	g.cloneRect = function ( /*dojox.gfx.Rectangle*/ r) {
    	return {
    		x:r.x,
    		y:r.y,
    		width:r.width,
    		height:r.height
    		};
		};
		
  g.Rect = function(x,y,w,h) {
  	//	x: Number
  	//	y: Number
  	//	w: Number
  	//	h: Number
  	var r = {};

  	if(x!==undefined) {
  		r.x = x;
  	}
  	
  	if(y!==undefined) {
  		r.y = y;
  	}
  	
  	if(w!==undefined) {
  		r.width = w;
  	}
  	
  	if(h!==undefined) {
  		r.height = h;
  	}
  	
  	return r;
  };
  
  g.standarizeRect = function(r) {
  	//	r:	dojox.gfx.Rectangle
  	return {
  		x:r.x||0,
  		y:r.y||0,
  		width:r.width||r.w||0,
  		height:r.height||r.h||0
  		};
  };
  
  g.rectAssign = function(a,b) {
  	a.x = b.x;
  	a.y = b.y;
  	a.width = b.width;
  	a.height = b.height;
  };
  
  g.rectIncomplete = function( r ) {
  	//	r:	dojox.gfx.Rectangle
      return r.x===undefined || r.y===undefined || r.width===undefined || r.height===undefined;
  };
  
  g.getRectCenter = function( r ) {
  	//	r:	dojox.gfx.Rectangle
  	//	returns:
  	//		a point equal to the center of the rectangle r
      if(ibm_ilog.diagram.util.Geometry.rectIncomplete(r)) {
      	return null;
      } else {
      	return { x: r.x + r.width / 2, y: r.y + r.height / 2 };
      }
  };

  g.rectCenteredAt = function( r, p, center ) {
  	//	r:	dojox.gfx.Rectangle
  	//	p:	dojox.gfx.Point
  	//	center: dojox.gfx.Point? 
  	//	returns:
  	//		a copy of r but centered at p
  	center = center || { x:r.width / 2, y:r.height / 2 };
      return { x: p.x - center.x, y: p.y - center.y, width:r.width, height:r.height};
  };

  g.rectIntersects = function(rect1,rect2) {
      return (rect1.x < (rect2.x + rect2.width)) &&
             (rect2.x < (rect1.x + rect1.width)) &&
             (rect1.y < (rect2.y + rect2.height)) &&
             (rect2.y < (rect1.y + rect1.height));        
  };

  
  g.addRect = function( /*dojox.gfx.Rectangle*/ a, /*dojox.gfx.Rectangle*/ b) {
  	//	returns:
  	//		a rectangle representing the addition of the two given.
      if (a === null && b === null) {
          return null;
      }
      if (a === null && b !== null) {
          return b;
      }
      if (b === null) {
          return a;
      }
      var single1 = Math.min(a.x, b.x);
      var single2 = Math.max(a.x + a.width, b.x + b.width);
      var single3 = Math.min(a.y, b.y);
      var single4 = Math.max(a.y + a.height, b.y + b.height);
      return { x: single1, y: single3, width: single2 - single1, height: single4 - single3 };
  };
  
  g.createRect = function(p1,p2){
  	var x = Math.min(p2.x,p1.x);
      var y = Math.min(p2.y,p1.y);
      var width = Math.abs(p2.x-p1.x);
      var height = Math.abs(p2.y-p1.y);
  	return {x:x,y:y,width:width,height:height};
  };
  
  g.containsRect = function(cr,r){
  	if(cr.x<=r.x && cr.y<=r.y 
  			&& (cr.width+cr.x)>=(r.x+r.width) && (cr.height+cr.y)>=(r.y+r.height)){
  		return true;
  	}
  	return false;
  };
  
  g.clipRect = function(cr,r){
  	// cr: clipping rectangle
  	// r: clipee
  	var res = {
  		x:Math.min(Math.max(cr.x,r.x),cr.x+cr.width),
  		y:Math.min(Math.max(cr.y,r.y),cr.y+cr.height)
  	};
  	
  	res.width = Math.max(0,Math.min(cr.x+cr.width,r.x+r.width)-res.x);
  	res.height= Math.max(0,Math.min(cr.y+cr.height,r.y+r.height)-res.y);

  	return res;
  };

  g.moveRect = function( /*dojox.gfx.Rectangle*/ r, /*dojox.gfx.Point*/ delta ) {
  	return { x: (r.x||0)+delta.x,
		     y: (r.y||0)+delta.y,
		 width: r.width,
		height: r.height };
  };
  
  g.relocateRect = function( /*dojox.gfx.Rectangle*/ r, /*dojox.gfx.Point*/ p ) {
  	return { x: p.x,
		     y: p.y,
		 width: r.width,
		height: r.height };
  };
  
  g.growRect = function( /*dojox.gfx.Rectangle*/ r, /*ibm_ilog.diagram.util.Geometry.Size*/ minSize ) {
  	return { x: r.x,
	         y: r.y,
	     width: Math.max(r.width,minSize.width),
	    height: Math.max(r.height,minSize.height) };
	};

	g.expandRect = function(rect,l,t,r,b) {
		switch(arguments.length) {
		case 2:
			t = r = b = l;
			break;
		case 3:
			b = t;
			r = l;
			break;
		case 5:
			break;
		default:
			R.error("InvalidNumberOfArguments",arguments,2,3,5);
			break;
		}
    	return { x: rect.x-l,
		         y: rect.y-t,
		     width: rect.width+l+r,
		    height: rect.height+t+b };
	};
	
	g.printRect = function(r) {
		var f = function (n){ return ibm_ilog.diagram.util._round(n,1); };
		return f(r.width)+" x "+f(r.height)+": ["+f(r.x)+","+f(r.x+r.width)+"] x ["+f(r.y)+","+f(r.y+r.height)+"]";
	};
    
	g.rectEqualSize = function(r1,r2,resolution) {
  	return Math.abs(r1.width - r2.width) < resolution && Math.abs(r1.height - r2.height) < resolution;
  };
  
  g.rectEqualOrigin = function(r1,r2,resolution) {
  	return Math.abs(r1.x - r2.x) < resolution && Math.abs(r1.y - r2.y) < resolution;
  };
  
  g._rangeCenteredOverride = function(result,oldv,newv,x,l) {
  	//
  	//	summary:    	
  	//		Returns new range based on the overriding of a base range with override values from newr.
  	//		Used as a helper function for rectCenteredOverride.
  	//
  	//	tags:
  	//		private
  	//
  	//	description:
  	//		Receives an incomplete range newr and assigns its values to oldr, returning the result. 
  	//		It does not change the original range. The semantics are fairly straighforward except for
  	//		one case: if the newr specifies a new [w] and does not specify the corresponding
  	//		[x], then the resulting rectangle is resized centered, instead of keeping its original 
  	//		location.
  	//
  	//	returns: ibm_ilog.diagram.Rectangle
  	//		The result 
  	//	result: ibm_ilog.diagram.Rectangle
  	//		Where the result will be stored 
  	//	newr: ibm_ilog.diagram.Rectangle
  	//		The incomplete rectangle that holds the new values
  	//	oldr: ibm_ilog.diagram.Rectangle
  	//		The fake asignee it will not be changed
  	//	x:	String
  	//		The name of the attribute representing the range origin
  	//	w:	String
  	//		The name of the attribute representing the range length
  	//
	if( newv[l] === undefined ) {
    	if( newv[x] === undefined ) {
    		result[x]=oldv[x];
    	} else {
  			result[x]=newv[x];
    	}
  		result[l]=oldv[l];
	} else {
    	if( newv[x] === undefined ) {
    		// keeps range center
    		result[x]=oldv[x]-(newv[l]-oldv[l])/2;
			
    		// TODO use pivot instead:
    		//
			//if(!newv.scalePivot)
			//	newv.scalePivot = {};
			//var p = newv.scalePivot[x] = newv.scalePivot[x] || (newv[x]+(newv[l]/2));
			//
			//result[x]= p-(newv[w]/oldv[w])*(p-oldv[x]);

			
    	} else {
  			result[x]=newv[x];
    	}
		result[l]=newv[l];
	}    	
  	return result;
  };
  
  g.rectCenteredOverride = function(oldr,newr) {
  	//
  	//	summary:    	
  	//		Returns new rectangle based on the overriding of a base rectangle with override values from newr.
  	//
  	//	description:
  	//		Receives an incomplete rectangle newr and assigns its values to oldr, returning the result. 
  	//		It does not change the original rectangle. The semantics are fairly straighforward except for
  	//		one case: if the newr specifies a new width or height and does not specify the corresponding
  	//		x or y, then the resulting rectangle is resized centered, instead of keeping its original 
  	//		location.
  	//
  	//	newr:
  	//		The incomplete rectangle that holds the new values
  	//	oldr:
  	//		The fake asignee it will not be changed
  	//
  	var result = newr.scalePivot?{scalePivot:newr.scalePivot}:{};
  	this._rangeCenteredOverride(result,oldr,newr,'y','height');
  	this._rangeCenteredOverride(result,oldr,newr,'x','width');
  	return result;
  };
  
  g.constrainRect = function(r,area,resize) {
  	var w = resize?Math.min(r.width,area.width):r.width;
  	var h = resize?Math.min(r.height,area.height):r.height;
		return {
			x:      Math.max(Math.min(r.x,area.x+area.width-w),area.x),
			y:      Math.max(Math.min(r.y,area.y+area.height-h),area.y),
			width:  w,
			height: h
		};
  };
  
  g.visitMembers = function(obj,visitor) {
  	// probably should be moved to a generic set of utils
	var r = {};
	for( _i in obj ) {
		r[_i]=visitor(obj[_i]);
	}
	return r;
  };
  
  g.getRectRatio = function(r) {
  	return 	r.width/r.height;
  };
  
  g.rectBlend = function(r1,r2,factor) {
	var cx = r1.x + (r2.x - r1.x) * factor;
	var cy = r1.y + (r2.y - r1.y) * factor;
	var cwidth = r1.width + (r2.width - r1.width) * factor;
	var cheight = r1.height + (r2.height - r1.height) * factor;
	return {x:cx,y:cy,width:cwidth,height:cheight};
  };
  
  g.containsPoint = function(r,p) {
  	return r.x<=p.x && r.y<=p.y && p.x<=(r.x+r.width) && p.y<=(r.y+r.height);
  };

  
/*=====
	dojo.declare("ibm_ilog.diagram.util.Geometry.Size", null, {
		// summary:
		//		An object with 'width' and 'height' properties used as argument for various ibm_ilog.diagram.util.Geometry methods.
		
		constructor: function( width, height ) {
			// width: Number
			// height: Number
			this.width = width;
			this.height = height;
		}
	});
=====*/
    
  g.printSize = function(p) {
  	//	p: ibm_ilog.diagram.util.Geometry.Size
  	return "("+p.width+","+p.height+")"
  };

  g.sizeLongestSide = function( /*ibm_ilog.diagram.util.Geometry.Size*/ sz ) {
  	return Math.max(sz.width,sz.height);
  };

  g.visitSize = function( /*Function*/ op, /*ibm_ilog.diagram.util.Geometry.Size*/ size ) {
  	return op(size.width,size.height);
  };

  g.isNullSize = function( /*dojox.gfx.Rectangle*/ r ) {
  	//
  	//	summary:
	//		returns true when r is a "null" rectangle: width and height are 0 or undefined
  	//
      return !r.width && !r.height;
  };

  g.Size = function(width,height) {
  	return {width:width,height:height};
  };

  g.equalSize = function(p1, p2) {
  	//	returns:
  	//		true when the two rects are equal
      return (p1.width === p2.width) &&
         (p1.height === p2.height);
  };
  
  g.floorSize = function( sz ) {
  	return {width:Math.floor(sz.width),height:Math.floor(sz.height)};
  };

  g.mulSize = function(p1, s) {
    return { width: p1.width*s, height: p1.height*s };
  };

  
  
  
  
  
  
  
  
  
  g.Point = function( /*Number*/ x, /*Number*/ y ) {
  	// x: Number
  	// y: Number
  	// returns: dojox.gfx.Point
  	return {x:x,y:y};
  };

  g.floorPoint = function( p ) {
  	// p: dojox.gfx.Point
  	// returns: dojox.gfx.Point
  	return {x:Math.floor(p.x),y:Math.floor(p.y)};
  };
  
  g.printPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: dojox.gfx.Point
  	return "("+p.x+","+p.y+")"
  };

  g.InvalidPoint = function() {
  	// returns: dojox.gfx.Point
  	return { x: Number.NEGATIVE_INFINITY, y: Number.POSITIVE_INFINITY };
  };

  g.EmptyPoint = function() {
  	// returns: dojox.gfx.Point
  	return { x: NaN, y: NaN };
  };
  
  g.PointZero = function() {
  	// returns: dojox.gfx.Point
  	return { x: 0, y: 0 };
  };
  
  g.getDistance = function(p1, p2) {
  	// p1: dojox.gfx.Point
  	// p2: dojox.gfx.Point
  	// returns: Number
      var a = p2.x - p1.x;
      var b = p2.y - p1.y;
      return Math.sqrt(a * a + b * b);
  };
  
  g.getPointToSegmentDistance = function(A,B,C){
  	// Summary:
  	//		return the distance between a segment and a point
  	//		A and B are the segment points
  	
  	// special case where A = B
  	if(this.equalPoint(A,B)){
  		return this.getDistance(A,C);
  	}
  	
  	var u = ((C.x-A.x)*(B.x-A.x)+(C.y-A.y)*(B.y-A.y))/(Math.pow(B.x-A.x,2)+Math.pow(B.y-A.y,2));
  	
  	// P is the nearest point of the segment line
  	var P = {};
  	
  	if(u>1){
  		// the point P is over the point B - distance = B-C
  		return this.getDistance(B,C);
  	}else{
  		if(u<0){
  			// the point P is behind the point A - distance = A-C
  			return this.getDistance(A,C);
  		}else{
  			// the point P is in between A and B - distance = C-P
  			P.x = A.x + u * ( B.x - A.x);
  			P.y = A.y + u * ( B.y - A.y);
  			return this.getDistance(P,C);
  		}
  	}
  };
  
  g.isInvalidPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: Boolean
      return p.x === Number.NEGATIVE_INFINITY && p.y === Number.POSITIVE_INFINITY;
  };

  
  g.isEmptyPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: Boolean
      return isNaN(p.x) && isNaN(p.y);
  };

  g.addPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: p1.x+p2.x, y: p1.y+p2.y };
  };

  g.subPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: p1.x-p2.x, y: p1.y-p2.y };
  };

  g.negPoint = function(p) {
  	//	p: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: -p.x, y: -p.y };
  };
  
  g.mulPoint = function(p1, s) {
  	//	p: dojox.gfx.Point
  	//	s: Number
  	//	returns: dojox.gfx.Point
      return { x: p1.x*s, y: p1.y*s };
  };

  g.equalPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: Boolean
  	//		true when the two rects are equal
    return (p1.x === p2.x) &&
       (p1.y === p2.y);
  };

//
//	MISCELANEOUS OPERATIONS
//
    
  g.getScale = function(x, y) {
  	//	x: Number
  	//	y: Number
  	//	returns: Number
		if (x === 0) {
		    return y;
		} else if (y === 0) {
		    return x;
		} else {
		    return Math.sqrt(x * x + y * y);
		}
  };
  
  g.getStrokeWidth = function(stroke, t) {
		var w = stroke.width || 1;
		var zoom = Math.min(t.xx, t.yy);
		var realWidth = w * zoom;
		//            if (_minWidth >= 0f && realWidth < _minWidth)
		//                realWidth = _minWidth;
		//            if (_maxWidth >= 0f && realWidth > _maxWidth)
		//                realWidth = _maxWidth;
		w = (zoom === 0) ? 0 : (realWidth / zoom);
		return w;
  };

	ibm_ilog.diagram.util._round = function(value, places){
		if(dojo.number && dojo.number.round)
			return dojo.number.round(value, places); // Number
		return (+value).toFixed(places)*1; // Number
	};

})();

}

if(!dojo._hasResource["dojox.gfx.matrix"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.matrix"] = true;
dojo.provide("dojox.gfx.matrix");

(function(){
	var m = dojox.gfx.matrix;

	// candidates for dojox.math:
	var _degToRadCache = {};
	m._degToRad = function(degree){
		return _degToRadCache[degree] || (_degToRadCache[degree] = (Math.PI * degree / 180));
	};
	m._radToDeg = function(radian){ return radian / Math.PI * 180; };

	m.Matrix2D = function(arg){
		// summary: a 2D matrix object
		// description: Normalizes a 2D matrix-like object. If arrays is passed,
		//		all objects of the array are normalized and multiplied sequentially.
		// arg: Object
		//		a 2D matrix-like object, a number, or an array of such objects
		if(arg){
			if(typeof arg == "number"){
				this.xx = this.yy = arg;
			}else if(arg instanceof Array){
				if(arg.length > 0){
					var matrix = m.normalize(arg[0]);
					// combine matrices
					for(var i = 1; i < arg.length; ++i){
						var l = matrix, r = dojox.gfx.matrix.normalize(arg[i]);
						matrix = new m.Matrix2D();
						matrix.xx = l.xx * r.xx + l.xy * r.yx;
						matrix.xy = l.xx * r.xy + l.xy * r.yy;
						matrix.yx = l.yx * r.xx + l.yy * r.yx;
						matrix.yy = l.yx * r.xy + l.yy * r.yy;
						matrix.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
						matrix.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
					}
					dojo.mixin(this, matrix);
				}
			}else{
				dojo.mixin(this, arg);
			}
		}
	};

	// the default (identity) matrix, which is used to fill in missing values
	dojo.extend(m.Matrix2D, {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0});

	dojo.mixin(m, {
		// summary: class constants, and methods of dojox.gfx.matrix

		// matrix constants

		// identity: dojox.gfx.matrix.Matrix2D
		//		an identity matrix constant: identity * (x, y) == (x, y)
		identity: new m.Matrix2D(),

		// flipX: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)
		flipX:    new m.Matrix2D({xx: -1}),

		// flipY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)
		flipY:    new m.Matrix2D({yy: -1}),

		// flipXY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)
		flipXY:   new m.Matrix2D({xx: -1, yy: -1}),

		// matrix creators

		translate: function(a, b){
			// summary: forms a translation matrix
			// description: The resulting matrix is used to translate (move) points by specified offsets.
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			if(arguments.length > 1){
				return new m.Matrix2D({dx: a, dy: b}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies offsets for both dimensions
			// b: null
			return new m.Matrix2D({dx: a.x, dy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		scale: function(a, b){
			// summary: forms a scaling matrix
			// description: The resulting matrix is used to scale (magnify) points by specified offsets.
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			if(arguments.length > 1){
				return new m.Matrix2D({xx: a, yy: b}); // dojox.gfx.matrix.Matrix2D
			}
			if(typeof a == "number"){
				// branch
				// a: Number: a uniform scaling factor used for the both coordinates
				// b: null
				return new m.Matrix2D({xx: a, yy: a}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies scale factors for both dimensions
			// b: null
			return new m.Matrix2D({xx: a.x, yy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		rotate: function(angle){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an angle of rotation in radians (>0 for CW)
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			return new m.Matrix2D({xx: c, xy: -s, yx: s, yy: c}); // dojox.gfx.matrix.Matrix2D
		},
		rotateg: function(degree){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.rotate() for comparison.
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			return m.rotate(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewX: function(angle) {
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({xy: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewXg: function(degree){
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewX() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewX(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewY: function(angle){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({yx: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewYg: function(degree){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewY() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewY(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		reflect: function(a, b){
			// summary: forms a reflection matrix
			// description: The resulting matrix is used to reflect points around a vector,
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of reflection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value

			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = 2 * a * b / n2;
			return new m.Matrix2D({xx: 2 * a2 / n2 - 1, xy: xy, yx: xy, yy: 2 * b2 / n2 - 1}); // dojox.gfx.matrix.Matrix2D
		},
		project: function(a, b){
			// summary: forms an orthogonal projection matrix
			// description: The resulting matrix is used to project points orthogonally on a vector,
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of projection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value

			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = a * b / n2;
			return new m.Matrix2D({xx: a2 / n2, xy: xy, yx: xy, yy: b2 / n2}); // dojox.gfx.matrix.Matrix2D
		},

		// ensure matrix 2D conformance
		normalize: function(matrix){
			// summary: converts an object to a matrix, if necessary
			// description: Converts any 2D matrix-like object or an array of
			//		such objects to a valid dojox.gfx.matrix.Matrix2D object.
			// matrix: Object: an object, which is converted to a matrix, if necessary
			return (matrix instanceof m.Matrix2D) ? matrix : new m.Matrix2D(matrix); // dojox.gfx.matrix.Matrix2D
		},

		// common operations

		clone: function(matrix){
			// summary: creates a copy of a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be cloned
			var obj = new m.Matrix2D();
			for(var i in matrix){
				if(typeof(matrix[i]) == "number" && typeof(obj[i]) == "number" && obj[i] != matrix[i]) obj[i] = matrix[i];
			}
			return obj; // dojox.gfx.matrix.Matrix2D
		},
		invert: function(matrix){
			// summary: inverts a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be inverted
			var M = m.normalize(matrix),
				D = M.xx * M.yy - M.xy * M.yx,
				M = new m.Matrix2D({
					xx: M.yy/D, xy: -M.xy/D,
					yx: -M.yx/D, yy: M.xx/D,
					dx: (M.xy * M.dy - M.yy * M.dx) / D,
					dy: (M.yx * M.dx - M.xx * M.dy) / D
				});
			return M; // dojox.gfx.matrix.Matrix2D
		},
		_multiplyPoint: function(matrix, x, y){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// x: Number: an x coordinate of a point
			// y: Number: a y coordinate of a point
			return {x: matrix.xx * x + matrix.xy * y + matrix.dx, y: matrix.yx * x + matrix.yy * y + matrix.dy}; // dojox.gfx.Point
		},
		multiplyPoint: function(matrix, /* Number||Point */ a, /* Number, optional */ b){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: Number: an x coordinate of a point
			// b: Number: a y coordinate of a point
			var M = m.normalize(matrix);
			if(typeof a == "number" && typeof b == "number"){
				return m._multiplyPoint(M, a, b); // dojox.gfx.Point
			}
			// branch
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: dojox.gfx.Point: a point
			// b: null
			return m._multiplyPoint(M, a.x, a.y); // dojox.gfx.Point
		},
		multiply: function(matrix){
			// summary: combines matrices by multiplying them sequentially in the given order
			// matrix: dojox.gfx.matrix.Matrix2D...: a 2D matrix-like object,
			//		all subsequent arguments are matrix-like objects too
			var M = m.normalize(matrix);
			// combine matrices
			for(var i = 1; i < arguments.length; ++i){
				var l = M, r = m.normalize(arguments[i]);
				M = new m.Matrix2D();
				M.xx = l.xx * r.xx + l.xy * r.yx;
				M.xy = l.xx * r.xy + l.xy * r.yy;
				M.yx = l.yx * r.xx + l.yy * r.yx;
				M.yy = l.yx * r.xy + l.yy * r.yy;
				M.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
				M.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
			}
			return M; // dojox.gfx.matrix.Matrix2D
		},

		// high level operations

		_sandwich: function(matrix, x, y){
			// summary: applies a matrix at a centrtal point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object, which is applied at a central point
			// x: Number: an x component of the central point
			// y: Number: a y component of the central point
			return m.multiply(m.translate(x, y), matrix, m.translate(-x, -y)); // dojox.gfx.matrix.Matrix2D
		},
		scaleAt: function(a, b, c, d){
			// summary: scales a picture using a specified point as a center of scaling
			// description: Compare with dojox.gfx.matrix.scale().
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			// c: Number: an x component of a central point
			// d: Number: a y component of a central point

			// accepts several signatures:
			//	1) uniform scale factor, Point
			//	2) uniform scale factor, x, y
			//	3) x scale, y scale, Point
			//	4) x scale, y scale, x, y

			switch(arguments.length){
				case 4:
					// a and b are scale factor components, c and d are components of a point
					return m._sandwich(m.scale(a, b), c, d); // dojox.gfx.matrix.Matrix2D
				case 3:
					if(typeof c == "number"){
						// branch
						// a: Number: a uniform scaling factor used for both coordinates
						// b: Number: an x component of a central point
						// c: Number: a y component of a central point
						// d: null
						return m._sandwich(m.scale(a), b, c); // dojox.gfx.matrix.Matrix2D
					}
					// branch
					// a: Number: a scaling factor used for the x coordinate
					// b: Number: a scaling factor used for the y coordinate
					// c: dojox.gfx.Point: a central point
					// d: null
					return m._sandwich(m.scale(a, b), c.x, c.y); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: Number: a uniform scaling factor used for both coordinates
			// b: dojox.gfx.Point: a central point
			// c: null
			// d: null
			return m._sandwich(m.scale(a), b.x, b.y); // dojox.gfx.matrix.Matrix2D
		},
		rotateAt: function(angle, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotate().
			// angle: Number: an angle of rotation in radians (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) rotation angle in radians, Point
			//	2) rotation angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.rotate(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an angle of rotation in radians (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotate(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		rotategAt: function(degree, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotateg().
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) rotation angle in degrees, Point
			//	2) rotation angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.rotateg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an angle of rotation in degrees (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotateg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXAt: function(angle, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewX().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewX(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewX(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXgAt: function(degree, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewXg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewXg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewXg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYAt: function(angle, a, b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewY().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewY(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewY(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYgAt: function(/* Number */ degree, /* Number||Point */ a, /* Number, optional */ b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewYg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point

			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewYg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewYg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		}

		//TODO: rect-to-rect mapping, scale-to-fit (isotropic and anisotropic versions)

	});
})();

// propagate Matrix2D up
dojox.gfx.Matrix2D = dojox.gfx.matrix.Matrix2D;

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.DragInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.DragInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.DragInteractor');






(function () {
	var g = ibm_ilog.diagram.util.Geometry;
	var m = dojox.gfx.matrix;

dojo.declare("ibm_ilog.diagram.interactors.DragInteractor", [ibm_ilog.diagram.interactors.Interactor], {

	//
	//	_viewport: ibm_ilog.diagram.widget.Diagram
	//		The viewport where the drag occurs
	//
	_viewport: null,

	//
	//	hasDragged: Boolean
	//		true if last drag start was followed by a move
	//
	hasDragged: false,

	//
	//	_transform: [dojox.gfx.matrix.Matrix2D]
	//		Transform used to transform UI coordinate system into moveable object's system
	//
	_transform: null,

	//
	//	_last: 
	//		Last event coordinate
	//		
	_last: null, 

	//
	//	_first: 
	//		The event coordinate at which the drag started
	//		
	_first: null,

	//
	//	_firstViewportT:
	//		The viewport transform at the point where _first was defined or updated.
	//
	_firstViewportT: null,
		
	//
	//	_current: 
	//		Current event coordinate
	//		
	_current: null,
	
	//
	//	_delta: 
	//		The difference between _current and _last
	//		
	_delta: null,
	
	//
	//	_dragCursor: 
	//		Optional cursor ID set while dragging
	//		
	_dragCursor: null,
	
	//
	//	_cursorDom: 
	//		The DOM style that controls the cursor we will update
	//		
	_cursorStyle: null,

	
	_declareStates: function () {
		var idleStates = ["start"];
		if (ibm_ilog.diagram.mobileEnabled)
			idleStates.push("touchStart");
		this._declareState("idle", idleStates);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart"]);
		if (ibm_ilog.diagram.mobileEnabled)
			this._declareState("activeTouch", ["touchFirst","touchMove","touchEnd"]);
	},

	_initialize: function () {
		if (!this._viewport) {
			this._getEventCoord = this.__getEventCoordNoViewport;
			this._cursorStyle = this._cursorStyle || document.body.style;
		} else {
			this._cursorStyle = this._cursorStyle || this._viewport.getDiagram().domNode.style;
		}
		this._last = g.Point(0, 0);
		this._first = this._last;
		this._current = this._last;
		this.inherited(arguments);

	},

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
		if (diagram instanceof ibm_ilog.diagram.widget.Diagram) {
			this._viewport = diagram.getViewport();
		} else if (diagram instanceof ibm_ilog.diagram.widget.Viewport) {
			this._viewport = diagram;
		}
		this._initialize();
		return this;
	},

	_getInitialEventSource: function () {
		return this._viewport.getEventSource();
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		var defaultConnections = {
			start: {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_dragPreStart",
				filter: this._buildInputFilter({button:0,ctrl:false,shift:false}),
				gfxConnect: true
			}, first: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_dragFirstMove"
			}, move: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_dragMove"
			}, end: {
				src: document,
				srcEvt: "onmouseup",
				connectTo: "_dragEnd"
			}, cancelOnDragStart: {
				src: document,
				srcEvt: "ondragstart",
				target: dojo,
				connectTo: "stopEvent"
			}, cancelOnSelectStart: {
				src: document,
				srcEvt: "onselectstart",
				target: dojo,
				connectTo: "stopEvent"
			}
		};
		if (ibm_ilog.diagram.mobileEnabled){
			var mobileDefaultConnections = {
				touchStart : {
					src: this._getInitialEventSource(),
					srcEvt: "ontouchstart",
					connectTo: "_dragTouchPreStart",
					gfxConnect: true
				}, touchFirst: {
					src: document,
					srcEvt: "ontouchmove",
					connectTo: "_dragTouchFirstMove"
				},  touchMove : {
					src: document,
					srcEvt: "ontouchmove",
					connectTo: "_dragTouchMove"
				}, touchEnd : {
					src: document,
					srcEvt: "ontouchend",
					connectTo: "_dragTouchEnd"
				}
			};
			dojo.mixin(defaultConnections,mobileDefaultConnections);
		};
		return defaultConnections;
	},

	activate: function () {
		//
		//	summary:
		//		Activates the receiver by connecting it to its sources of events.
		//
		this._goStateId("idle");
		this.inherited(arguments);
	},

	_dragPreStart: function (e) {
		//
		//	summary:
		//		Starts the drag process by storing initial coordinates for pointer and view rectangle
		//
		if (this._state._id !== "idle") console.warn("Unexpected active dragging.");

		dojo.fixEvent(e);
		
		this._goStateId("active");
		this._startDragGesture(e);
		this._updateCursor();
	},

	_startDragGesture: function(e){
		this._firstViewportT = this._viewport ? this._viewport.getViewportInverseTransform() : null;
		this._last = this._getEventCoord(e);
		this._first = this._last;
		this._current = this._last;
		this._delta = {
			x: 0,
			y: 0
		};
		this._totalDelta = this._delta;
		
	},

	_showDragCursor: function() {
		if(this._dragCursor && this._cursorStyle && this._cursorBackup == null) {
			this._cursorBackup = this._cursorStyle.cursor;
			this._cursorStyle.cursor = this._dragCursor;
		}
	},

	_hideDragCursor: function() {
		if(this._cursorBackup!==null) {
			if (this._cursorStyle)
				this._cursorStyle.cursor = this._cursorBackup;
			this._cursorBackup = null;
		}
	},
	
	_updateCursor:function() {
		if(this._state._id=="active") {
			this._showDragCursor();
		} else {
			this._hideDragCursor();
		}
	},
	
	_dragFirstMove: function (e) {
		if (this._manager) {
			this._activationHandle = this._manager.push(this);
		}
		this._disconnect("first");
		this.hasDragged = true;
	},

	_dragMove: function (e) {
		dojo.fixEvent(e);
		this._dragGesture(e);
	},

	_dragGesture: function (e){
		this._last = this._current;
		this._current = this._getEventCoord(e);
		this._delta = {
			x: this._current.x - this._last.x,
			y: this._current.y - this._last.y
		};
		this._totalDelta = {
			x: this._current.x - this._first.x,
			y: this._current.y - this._first.y
		};
		
	},
	_dragEnd: function (e) {
		dojo.fixEvent(e);

		if (this._activationHandle) {
			this._manager.pop(this._activationHandle);
			this._activationHandle = null;
		}
		this._goState(null);

		this.hasDragged = false;
		this._goStateId("idle");
		
		this._updateCursor();
	},

	_getEventCoord: function (e) {
		var pl = this._viewport.eventClientLocation(e);
		if (this._transform) {
			pl = m.multiplyPoint(this._transform, pl);
		}
		return pl;
	},

	__getEventCoordNoViewport: function (e) {
		var pl = this.eventPageLocation(e);
		if (this._transform) {
			pl = m.multiplyPoint(this._transform, pl);
		}
		return pl;
	},
	_dragTouchPreStart: function (e) {
		dojo.fixEvent(e);
		e.preventDefault();		
		//
		//	summary:
		//		Starts the drag process by storing initial coordinates for pointer and view rectangle
		//
		if (this._state._id !== "idle") console.warn("Unexpected active dragging.");

		
		this._goStateId("activeTouch");
		this._startDragGesture(e);
		this._updateCursor();
	},	
	
	_dragTouchFirstMove: function (e) {
		dojo.fixEvent(e);
		e.preventDefault();		
		
		this._disconnect("touchFirst");
		this.hasDragged = true;
	},

	_dragTouchMove: function (e) {
		dojo.fixEvent(e);
		e.preventDefault();		
		// if more than one finger don't do anything
		if (e.touches.length > 1){
			return false;
		}
		
		this._dragGesture(e);
		
	},	
	_dragTouchEnd: function (e) {
		dojo.fixEvent(e);
		if (e.touches.length > 0){
			// one finger left, restart the gesture
			this._startDragGesture(e);
			return false;
		}
		if (this._activationHandle) {
			this._manager.pop(this._activationHandle);
			this._activationHandle = null;
		}
		this._goState(null);

		this.hasDragged = false;
		this._goStateId("idle");
		
		this._updateCursor();

		
	}	


});

})();

}

if(!dojo._hasResource["dojox.gfx._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx._base"] = true;
dojo.provide("dojox.gfx._base");

(function(){
	var g = dojox.gfx, b = g._base;

	// candidates for dojox.style (work on VML and SVG nodes)
	g._hasClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary:
		//		Returns whether or not the specified classes are a portion of the
		//		class list currently applied to the node.
		// return (new RegExp('(^|\\s+)'+classStr+'(\\s+|$)')).test(node.className)	// Boolean
		var cls = node.getAttribute("className");
		return cls && (" " + cls + " ").indexOf(" " + classStr + " ") >= 0;  // Boolean
	};
	g._addClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary:
		//		Adds the specified classes to the end of the class list on the
		//		passed node.
		var cls = node.getAttribute("className") || "";
		if(!cls || (" " + cls + " ").indexOf(" " + classStr + " ") < 0){
			node.setAttribute("className", cls + (cls ? " " : "") + classStr);
		}
	};
	g._removeClass = function(/*DomNode*/node, /*String*/classStr){
		//	summary: Removes classes from node.
		var cls = node.getAttribute("className");
		if(cls){
			node.setAttribute(
				"className",
				cls.replace(new RegExp('(^|\\s+)' + classStr + '(\\s+|$)'), "$1$2")
			);
		}
	};

	// candidate for dojox.html.metrics (dynamic font resize handler is not implemented here)

	//	derived from Morris John's emResized measurer
	b._getFontMeasurements = function(){
		//	summary:
		//		Returns an object that has pixel equivilents of standard font
		//		size values.
		var heights = {
			'1em': 0, '1ex': 0, '100%': 0, '12pt': 0, '16px': 0, 'xx-small': 0,
			'x-small': 0, 'small': 0, 'medium': 0, 'large': 0, 'x-large': 0,
			'xx-large': 0
		};

		if(dojo.isIE){
			//	we do a font-size fix if and only if one isn't applied already.
			//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			dojo.doc.documentElement.style.fontSize="100%";
		}

		//	set up the measuring node.
		var div = dojo.create("div", {style: {
				position: "absolute",
				left: "0",
				top: "-100px",
				width: "30px",
				height: "1000em",
				borderWidth: "0",
				margin: "0",
				padding: "0",
				outline: "none",
				lineHeight: "1",
				overflow: "hidden"
			}}, dojo.body());

		//	do the measurements.
		for(var p in heights){
			div.style.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}

		dojo.body().removeChild(div);
		return heights; 	//	object
	};

	var fontMeasurements = null;

	b._getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = b._getFontMeasurements();
		}
		return fontMeasurements;
	};

	// candidate for dojox.html.metrics

	var measuringNode = null, empty = {};
	b._getTextBox = function(	/*String*/ text,
								/*Object*/ style,
								/*String?*/ className){
		var m, s, al = arguments.length;
		if(!measuringNode){
			measuringNode = dojo.create("div", {style: {
				position: "absolute",
				top: "-10000px",
				left: "0"
			}}, dojo.body());
		}
		m = measuringNode;
		// reset styles
		m.className = "";
		s = m.style;
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(al > 1 && style){
			for(var i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(al > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;

		if(m["getBoundingClientRect"]){
			var bcr = m.getBoundingClientRect();
			return {l: bcr.left, t: bcr.top, w: bcr.width || (bcr.right - bcr.left), h: bcr.height || (bcr.bottom - bcr.top)};
		}else{
			return dojo.marginBox(m);
		}
	};

	// candidate for dojo.dom

	var uniqueId = 0;
	b._getUniqueId = function(){
		// summary: returns a unique string for use with any DOM element
		var id;
		do{
			id = dojo._scopeName + "Unique" + (++uniqueId);
		}while(dojo.byId(id));
		return id;
	};
})();

dojo.mixin(dojox.gfx, {
	//	summary:
	// 		defines constants, prototypes, and utility functions

	// default shapes, which are used to fill in missing parameters
	defaultPath: {
		type: "path", path: ""
	},
	defaultPolyline: {
		type: "polyline", points: []
	},
	defaultRect: {
		type: "rect", x: 0, y: 0, width: 100, height: 100, r: 0
	},
	defaultEllipse: {
		type: "ellipse", cx: 0, cy: 0, rx: 200, ry: 100
	},
	defaultCircle: {
		type: "circle", cx: 0, cy: 0, r: 100
	},
	defaultLine: {
		type: "line", x1: 0, y1: 0, x2: 100, y2: 100
	},
	defaultImage: {
		type: "image", x: 0, y: 0, width: 0, height: 0, src: ""
	},
	defaultText: {
		type: "text", x: 0, y: 0, text: "", align: "start",
		decoration: "none", rotated: false, kerning: true
	},
	defaultTextPath: {
		type: "textpath", text: "", align: "start",
		decoration: "none", rotated: false, kerning: true
	},

	// default geometric attributes
	defaultStroke: {
		type: "stroke", color: "black", style: "solid", width: 1,
		cap: "butt", join: 4
	},
	defaultLinearGradient: {
		type: "linear", x1: 0, y1: 0, x2: 100, y2: 100,
		colors: [
			{ offset: 0, color: "black" }, { offset: 1, color: "white" }
		]
	},
	defaultRadialGradient: {
		type: "radial", cx: 0, cy: 0, r: 100,
		colors: [
			{ offset: 0, color: "black" }, { offset: 1, color: "white" }
		]
	},
	defaultPattern: {
		type: "pattern", x: 0, y: 0, width: 0, height: 0, src: ""
	},
	defaultFont: {
		type: "font", style: "normal", variant: "normal",
		weight: "normal", size: "10pt", family: "serif"
	},

	getDefault: (function(){
		var typeCtorCache = {};
		// a memoized delegate()
		return function(/*String*/ type){
			var t = typeCtorCache[type];
			if(t){
				return new t();
			}
			t = typeCtorCache[type] = new Function;
			t.prototype = dojox.gfx[ "default" + type ];
			return new t();
		}
	})(),

	normalizeColor: function(/*Color*/ color){
		//	summary:
		// 		converts any legal color representation to normalized
		// 		dojo.Color object
		return (color instanceof dojo.Color) ? color : new dojo.Color(color); // dojo.Color
	},
	normalizeParameters: function(existed, update){
		//	summary:
		// 		updates an existing object with properties from an "update"
		// 		object
		//	existed: Object
		//		the "target" object to be updated
		//	update:  Object
		//		the "update" object, whose properties will be used to update
		//		the existed object
		if(update){
			var empty = {};
			for(var x in existed){
				if(x in update && !(x in empty)){
					existed[x] = update[x];
				}
			}
		}
		return existed;	// Object
	},
	makeParameters: function(defaults, update){
		//	summary:
		// 		copies the original object, and all copied properties from the
		// 		"update" object
		//	defaults: Object
		//		the object to be cloned before updating
		//	update:   Object
		//		the object, which properties are to be cloned during updating
		if(!update){
			// return dojo.clone(defaults);
			return dojo.delegate(defaults);
		}
		var result = {};
		for(var i in defaults){
			if(!(i in result)){
				result[i] = dojo.clone((i in update) ? update[i] : defaults[i]);
			}
		}
		return result; // Object
	},
	formatNumber: function(x, addSpace){
		// summary: converts a number to a string using a fixed notation
		// x:			Number:		number to be converted
		// addSpace:	Boolean?:	if it is true, add a space before a positive number
		var val = x.toString();
		if(val.indexOf("e") >= 0){
			val = x.toFixed(4);
		}else{
			var point = val.indexOf(".");
			if(point >= 0 && val.length - point > 5){
				val = x.toFixed(4);
			}
		}
		if(x < 0){
			return val; // String
		}
		return addSpace ? " " + val : val; // String
	},
	// font operations
	makeFontString: function(font){
		// summary: converts a font object to a CSS font string
		// font:	Object:	font object (see dojox.gfx.defaultFont)
		return font.style + " " + font.variant + " " + font.weight + " " + font.size + " " + font.family; // Object
	},
	splitFontString: function(str){
		// summary:
		//		converts a CSS font string to a font object
		// description:
		//		Converts a CSS font string to a gfx font object. The CSS font
		//		string components should follow the W3C specified order
		//		(see http://www.w3.org/TR/CSS2/fonts.html#font-shorthand):
		//		style, variant, weight, size, optional line height (will be
		//		ignored), and family.
		// str: String
		//		a CSS font string
		var font = dojox.gfx.getDefault("Font");
		var t = str.split(/\s+/);
		do{
			if(t.length < 5){ break; }
			font.style   = t[0];
			font.variant = t[1];
			font.weight  = t[2];
			var i = t[3].indexOf("/");
			font.size = i < 0 ? t[3] : t[3].substring(0, i);
			var j = 4;
			if(i < 0){
				if(t[4] == "/"){
					j = 6;
				}else if(t[4].charAt(0) == "/"){
					j = 5;
				}
			}
			if(j < t.length){
				font.family = t.slice(j).join(" ");
			}
		}while(false);
		return font;	// Object
	},
	// length operations
	cm_in_pt: 72 / 2.54,	// Number: points per centimeter
	mm_in_pt: 7.2 / 2.54,	// Number: points per millimeter
	px_in_pt: function(){
		// summary: returns a number of pixels per point
		return dojox.gfx._base._getCachedFontMeasurements()["12pt"] / 12;	// Number
	},
	pt2px: function(len){
		// summary: converts points to pixels
		// len: Number: a value in points
		return len * dojox.gfx.px_in_pt();	// Number
	},
	px2pt: function(len){
		// summary: converts pixels to points
		// len: Number: a value in pixels
		return len / dojox.gfx.px_in_pt();	// Number
	},
	normalizedLength: function(len) {
		// summary: converts any length value to pixels
		// len: String: a length, e.g., "12pc"
		if(len.length == 0) return 0;
		if(len.length > 2){
			var px_in_pt = dojox.gfx.px_in_pt();
			var val = parseFloat(len);
			switch(len.slice(-2)){
				case "px": return val;
				case "pt": return val * px_in_pt;
				case "in": return val * 72 * px_in_pt;
				case "pc": return val * 12 * px_in_pt;
				case "mm": return val * dojox.gfx.mm_in_pt * px_in_pt;
				case "cm": return val * dojox.gfx.cm_in_pt * px_in_pt;
			}
		}
		return parseFloat(len);	// Number
	},

	// a constant used to split a SVG/VML path into primitive components
	pathVmlRegExp: /([A-Za-z]+)|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	pathSvgRegExp: /([A-Za-z])|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,

	equalSources: function(a, b){
		// summary: compares event sources, returns true if they are equal
		return a && b && a == b;
	},
	
	switchTo: function(renderer){
		var ns = dojox.gfx[renderer];
		if(ns){
			dojo.forEach(["Group", "Rect", "Ellipse", "Circle", "Line",
					"Polyline", "Image", "Text", "Path", "TextPath",
					"Surface", "createSurface"], function(name){
				dojox.gfx[name] = ns[name];
			});
		}
	}
});

}

if(!dojo._hasResource["dojox.gfx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx"] = true;
dojo.provide("dojox.gfx");




dojo.loadInit(function(){
	// Since loaderInit can be fired before any dojo.provide/require calls,
	// make sure the dojox.gfx object exists and only run this logic if dojox.gfx.renderer
	// has not been defined yet.
	var gfx = dojo.getObject("dojox.gfx", true), sl, flag, match;
	while(!gfx.renderer){
		// Have a way to force a GFX renderer, if so desired.
		// Useful for being able to serialize GFX data in a particular format.
		if(dojo.config.forceGfxRenderer){
			dojox.gfx.renderer = dojo.config.forceGfxRenderer;
			break;
		}
		var renderers = (typeof dojo.config.gfxRenderer == "string" ?
			dojo.config.gfxRenderer : "svg,vml,canvas,silverlight").split(",");
		for(var i = 0; i < renderers.length; ++i){
			switch(renderers[i]){
				case "svg":
					// the next test is from https://github.com/phiggins42/has.js
					if("SVGAngle" in dojo.global){
						dojox.gfx.renderer = "svg";
					}
					break;
				case "vml":
					if(dojo.isIE){
						dojox.gfx.renderer = "vml";
					}
					break;
				case "silverlight":
					try{
						if(dojo.isIE){
							sl = new ActiveXObject("AgControl.AgControl");
							if(sl && sl.IsVersionSupported("1.0")){
								flag = true;
							}
						}else{
							if(navigator.plugins["Silverlight Plug-In"]){
								flag = true;
							}
						}
					}catch(e){
						flag = false;
					}finally{
						sl = null;
					}
					if(flag){
						dojox.gfx.renderer = "silverlight";
					}
					break;
				case "canvas":
					if(dojo.global.CanvasRenderingContext2D){
						dojox.gfx.renderer = "canvas";
					}
					break;
			}
			if(gfx.renderer){
				break;
			}
		}
		break;
	}
	
	if(dojo.config.isDebug){
		console.log("gfx renderer = " + gfx.renderer);
	}

	// load & initialize renderer
	if(gfx[gfx.renderer]){
		// already loaded
		gfx.switchTo(gfx.renderer);
	}else{
		// load
		gfx.loadAndSwitch = gfx.renderer;
		dojo["require"]("dojox.gfx." + gfx.renderer);
	}
});

}

if(!dojo._hasResource["dojox.gfx.shape"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.shape"] = true;
dojo.provide("dojox.gfx.shape");



dojo.declare("dojox.gfx.shape.Shape", null, {
	// summary: a Shape object, which knows how to apply
	// graphical attributes and transformations

	constructor: function(){
		// rawNode: Node: underlying node
		this.rawNode = null;

		// shape: Object: an abstract shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		this.shape = null;

		// matrix: dojox.gfx.Matrix2D: a transformation matrix
		this.matrix = null;

		// fillStyle: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)
		this.fillStyle = null;

		// strokeStyle: Object: a stroke object
		//	(see dojox.gfx.defaultStroke)
		this.strokeStyle = null;

		// bbox: dojox.gfx.Rectangle: a bounding box of this shape
		//	(see dojox.gfx.defaultRect)
		this.bbox = null;

		// virtual group structure

		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		this.parent = null;

		// parentMatrix: dojox.gfx.Matrix2D
		//	a transformation matrix inherited from the parent
		this.parentMatrix = null;
	},

	// trivial getters

	getNode: function(){
		// summary: returns the current DOM Node or null
		return this.rawNode; // Node
	},
	getShape: function(){
		// summary: returns the current shape object or null
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		return this.shape; // Object
	},
	getTransform: function(){
		// summary: returns the current transformation matrix or null
		return this.matrix;	// dojox.gfx.Matrix2D
	},
	getFill: function(){
		// summary: returns the current fill object or null
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)
		return this.fillStyle;	// Object
	},
	getStroke: function(){
		// summary: returns the current stroke object or null
		//	(see dojox.gfx.defaultStroke)
		return this.strokeStyle;	// Object
	},
	getParent: function(){
		// summary: returns the parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		return this.parent;	// Object
	},
	getBoundingBox: function(){
		// summary: returns the bounding box or null
		//	(see dojox.gfx.defaultRect)
		return this.bbox;	// dojox.gfx.Rectangle
	},
	getTransformedBoundingBox: function(){
		// summary: returns an array of four points or null
		//	four points represent four corners of the untransformed bounding box
		var b = this.getBoundingBox();
		if(!b){
			return null;	// null
		}
		var m = this._getRealMatrix();
			gm = dojox.gfx.matrix;
		return [	// Array
				gm.multiplyPoint(m, b.x, b.y),
				gm.multiplyPoint(m, b.x + b.width, b.y),
				gm.multiplyPoint(m, b.x + b.width, b.y + b.height),
				gm.multiplyPoint(m, b.x, b.y + b.height)
			];
	},
	getEventSource: function(){
		// summary: returns a Node, which is used as
		//	a source of events for this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this.rawNode;	// Node
	},

	// empty settings

	setShape: function(shape){
		// summary: sets a shape object
		//	(the default implementation simply ignores it)
		// shape: Object: a shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.shape = dojox.gfx.makeParameters(this.shape, shape);
		this.bbox = null;
		return this;	// self
	},
	setFill: function(fill){
		// summary: sets a fill object
		//	(the default implementation simply ignores it)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient,
		//	dojox.gfx.defaultRadialGradient,
		//	dojox.gfx.defaultPattern,
		//	or dojo.Color)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!fill){
			// don't fill
			this.fillStyle = null;
			return this;	// self
		}
		var f = null;
		if(typeof(fill) == "object" && "type" in fill){
			// gradient or pattern
			switch(fill.type){
				case "linear":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					break;
				case "radial":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					break;
				case "pattern":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					break;
			}
		}else{
			// color object
			f = dojox.gfx.normalizeColor(fill);
		}
		this.fillStyle = f;
		return this;	// self
	},
	setStroke: function(stroke){
		// summary: sets a stroke object
		//	(the default implementation simply ignores it)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			return this;	// self
		}
		// normalize the stroke
		if(typeof stroke == "string" || dojo.isArray(stroke) || stroke instanceof dojo.Color){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		return this;	// self
	},
	setTransform: function(matrix){
		// summary: sets a transformation matrix
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.matrix = dojox.gfx.matrix.clone(matrix ? dojox.gfx.matrix.normalize(matrix) : dojox.gfx.matrix.identity);
		return this._applyTransform();	// self
	},

	_applyTransform: function(){
		// summary: physically sets a matrix

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this;	// self
	},

	// z-index

	moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToFront(this);
			this._moveToFront();	// execute renderer-specific action
		}
		return this;	// self
	},
	moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToBack(this);
			this._moveToBack();	// execute renderer-specific action
		}
		return this;
	},
	_moveToFront: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},
	_moveToBack: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},

	// apply left & right transformation

	applyRightTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on right side
		//	(this.matrix * matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	applyLeftTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on left side
		//	(matrix * this.matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([matrix, this.matrix]) : this;	// self
	},
	applyTransform: function(matrix){
		// summary: a shortcut for dojox.gfx.Shape.applyRightTransform
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},

	// virtual group methods

	removeShape: function(silently){
		// summary: removes the shape from its parent's list of shapes
		// silently: Boolean?: if true, do not redraw a picture yet
		if(this.parent){
			this.parent.remove(this, silently);
		}
		return this;	// self
	},
	_setParent: function(parent, matrix){
		// summary: sets a parent
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parent = parent;
		return this._updateParentMatrix(matrix);	// self
	},
	_updateParentMatrix: function(matrix){
		// summary: updates the parent matrix with new matrix
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parentMatrix = matrix ? dojox.gfx.matrix.clone(matrix) : null;
		return this._applyTransform();	// self
	},
	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix
		var m = this.matrix;
		var p = this.parent;
		while(p){
			if(p.matrix){
				m = dojox.gfx.matrix.multiply(p.matrix, m);
			}
			p = p.parent;
		}
		return m;	// dojox.gfx.Matrix2D
	}
});

dojox.gfx.shape._eventsProcessing = {
	connect: function(name, object, method){
		// summary: connects a handler to an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return arguments.length > 2 ?	// Object
			dojo.connect(this.getEventSource(), name, object, method) :
			dojo.connect(this.getEventSource(), name, object);
	},
	disconnect: function(token){
		// summary: connects a handler by token from an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		dojo.disconnect(token);
	}
};

dojo.extend(dojox.gfx.shape.Shape, dojox.gfx.shape._eventsProcessing);

dojox.gfx.shape.Container = {
	// summary: a container of shapes, which can be used
	//	as a foundation for renderer-specific groups, or as a way
	//	to logically group shapes (e.g, to propagate matricies)

	_init: function() {
		// children: Array: a list of children
		this.children = [];
	},

	// group management

	openBatch: function() {
		// summary: starts a new batch, subsequent new child shapes will be held in
		//	the batch instead of appending to the container directly
	},
	closeBatch: function() {
		// summary: submits the current batch, append all pending child shapes to DOM
	},
	add: function(shape){
		// summary: adds a shape to the list
		// shape: dojox.gfx.Shape: a shape
		var oldParent = shape.getParent();
		if(oldParent){
			oldParent.remove(shape, true);
		}
		this.children.push(shape);
		return shape._setParent(this, this._getRealMatrix());	// self
	},
	remove: function(shape, silently){
		// summary: removes a shape from the list
		// silently: Boolean?: if true, do not redraw a picture yet
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				if(silently){
					// skip for now
				}else{
					shape.parent = null;
					shape.parentMatrix = null;
				}
				this.children.splice(i, 1);
				break;
			}
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		this.children = [];
		return this;	// self
	},

	// moving child nodes

	_moveChildToFront: function(shape){
		// summary: moves a shape to front of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.push(shape);
				break;
			}
		}
		return this;	// self
	},
	_moveChildToBack: function(shape){
		// summary: moves a shape to back of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.unshift(shape);
				break;
			}
		}
		return this;	// self
	}
};

dojo.declare("dojox.gfx.shape.Surface", null, {
	// summary: a surface object to be used for drawings
	constructor: function(){
		// underlying node
		this.rawNode = null;
		// the parent node
		this._parent = null;
		// the list of DOM nodes to be deleted in the case of destruction
		this._nodes = [];
		// the list of events to be detached in the case of destruction
		this._events = [];
	},
	destroy: function(){
		// summary: destroy all relevant external resources and release all
		//	external references to make this object garbage-collectible
		dojo.forEach(this._nodes, dojo.destroy);
		this._nodes = [];
		dojo.forEach(this._events, dojo.disconnect);
		this._events = [];
		this.rawNode = null;	// recycle it in _nodes, if it needs to be recycled
		if(dojo.isIE){
			while(this._parent.lastChild){
				dojo.destroy(this._parent.lastChild);
			}
		}else{
			this._parent.innerHTML = "";
		}
		this._parent = null;
	},
	getEventSource: function(){
		// summary: returns a node, which can be used to attach event listeners
		return this.rawNode; // Node
	},
	_getRealMatrix: function(){
		// summary: always returns the identity matrix
		return null;	// dojox.gfx.Matrix2D
	},
	isLoaded: true,
	onLoad: function(/*dojox.gfx.Surface*/ surface){
		// summary: local event, fired once when the surface is created
		// asynchronously, used only when isLoaded is false, required
		// only for Silverlight.
	},
	whenLoaded: function(
		/*Object?*/ context,
		/*Function|String*/ method
	){
		var f = dojo.hitch(context, method);
		if(this.isLoaded){
			f(this);
		}else{
			var h = dojo.connect(this, "onLoad", function(surface){
				dojo.disconnect(h);
				f(surface);
			});
		}
	}
});

dojo.extend(dojox.gfx.shape.Surface, dojox.gfx.shape._eventsProcessing);

dojo.declare("dojox.gfx.Point", null, {
	// summary: a hypothetical 2D point to be used for drawings - {x, y}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2}.
});

dojo.declare("dojox.gfx.Rectangle", null, {
	// summary: a hypothetical rectangle - {x, y, width, height}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2, width: 100, height: 200}.
});

dojo.declare("dojox.gfx.shape.Rect", dojox.gfx.shape.Shape, {
	// summary: a generic rectangle
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Rect");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Ellipse", dojox.gfx.shape.Shape, {
	// summary: a generic ellipse
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Ellipse");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.rx, y: shape.cy - shape.ry,
				width: 2 * shape.rx, height: 2 * shape.ry};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Circle", dojox.gfx.shape.Shape, {
	// summary: a generic circle
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Circle");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.r, y: shape.cy - shape.r,
				width: 2 * shape.r, height: 2 * shape.r};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Line", dojox.gfx.shape.Shape, {
	// summary: a generic line
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Line");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {
				x:		Math.min(shape.x1, shape.x2),
				y:		Math.min(shape.y1, shape.y2),
				width:	Math.abs(shape.x2 - shape.x1),
				height:	Math.abs(shape.y2 - shape.y1)
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Polyline", dojox.gfx.shape.Shape, {
	// summary: a generic polyline/polygon
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Polyline");
		this.rawNode = rawNode;
	},
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object
		// points: Object: a polyline/polygon shape object
		// closed: Boolean: close the polyline to make a polygon
		if(points && points instanceof Array){
			// points: Array: an array of points
			this.inherited(arguments, [{points: points}]);
			if(closed && this.shape.points.length){
				this.shape.points.push(this.shape.points[0]);
			}
		}else{
			this.inherited(arguments, [points]);
		}
		return this;	// self
	},
	_normalizePoints: function(){
		// summary: normalize points to array of {x:number, y:number}
		var p = this.shape.points, l = p && p.length;
		if(l && typeof p[0] == "number"){
			var points = [];
			for(var i = 0; i < l; i += 2){
				points.push({x: p[i], y: p[i + 1]});
			}
			this.shape.points = points;
		}
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox && this.shape.points.length){
			var p = this.shape.points;
			var l = p.length;
			var t = p[0];
			var bbox = {l: t.x, t: t.y, r: t.x, b: t.y};
			for(var i = 1; i < l; ++i){
				t = p[i];
				if(bbox.l > t.x) bbox.l = t.x;
				if(bbox.r < t.x) bbox.r = t.x;
				if(bbox.t > t.y) bbox.t = t.y;
				if(bbox.b < t.y) bbox.b = t.y;
			}
			this.bbox = {
				x:		bbox.l,
				y:		bbox.t,
				width:	bbox.r - bbox.l,
				height:	bbox.b - bbox.t
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Image", dojox.gfx.shape.Shape, {
	// summary: a generic image
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.shape = dojox.gfx.getDefault("Image");
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	},
	setStroke: function(){
		// summary: ignore setting a stroke style
		return this;	// self
	},
	setFill: function(){
		// summary: ignore setting a fill style
		return this;	// self
	}
});

dojo.declare("dojox.gfx.shape.Text", dojox.gfx.shape.Shape, {
	// summary: a generic text
	constructor: function(rawNode){
		// rawNode: Node: a DOM Node
		this.fontStyle = null;
		this.shape = dojox.gfx.getDefault("Text");
		this.rawNode = rawNode;
	},
	getFont: function(){
		// summary: returns the current font object or null
		return this.fontStyle;	// Object
	},
	setFont: function(newFont){
		// summary: sets a font for text
		// newFont: Object: a font object (see dojox.gfx.defaultFont) or a font string
		this.fontStyle = typeof newFont == "string" ? dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

dojox.gfx.shape.Creator = {
	// summary: shape creators
	createShape: function(shape){
		// summary: creates a shape object based on its type; it is meant to be used
		//	by group-like objects
		// shape: Object: a shape descriptor object
		var gfx = dojox.gfx;
		switch(shape.type){
			case gfx.defaultPath.type:		return this.createPath(shape);
			case gfx.defaultRect.type:		return this.createRect(shape);
			case gfx.defaultCircle.type:	return this.createCircle(shape);
			case gfx.defaultEllipse.type:	return this.createEllipse(shape);
			case gfx.defaultLine.type:		return this.createLine(shape);
			case gfx.defaultPolyline.type:	return this.createPolyline(shape);
			case gfx.defaultImage.type:		return this.createImage(shape);
			case gfx.defaultText.type:		return this.createText(shape);
			case gfx.defaultTextPath.type:	return this.createTextPath(shape);
		}
		return null;
	},
	createGroup: function(){
		// summary: creates a group shape
		return this.createObject(dojox.gfx.Group);	// dojox.gfx.Group
	},
	createRect: function(rect){
		// summary: creates a rectangle shape
		// rect: Object: a path object (see dojox.gfx.defaultRect)
		return this.createObject(dojox.gfx.Rect, rect);	// dojox.gfx.Rect
	},
	createEllipse: function(ellipse){
		// summary: creates an ellipse shape
		// ellipse: Object: an ellipse object (see dojox.gfx.defaultEllipse)
		return this.createObject(dojox.gfx.Ellipse, ellipse);	// dojox.gfx.Ellipse
	},
	createCircle: function(circle){
		// summary: creates a circle shape
		// circle: Object: a circle object (see dojox.gfx.defaultCircle)
		return this.createObject(dojox.gfx.Circle, circle);	// dojox.gfx.Circle
	},
	createLine: function(line){
		// summary: creates a line shape
		// line: Object: a line object (see dojox.gfx.defaultLine)
		return this.createObject(dojox.gfx.Line, line);	// dojox.gfx.Line
	},
	createPolyline: function(points){
		// summary: creates a polyline/polygon shape
		// points: Object: a points object (see dojox.gfx.defaultPolyline)
		//	or an Array of points
		return this.createObject(dojox.gfx.Polyline, points);	// dojox.gfx.Polyline
	},
	createImage: function(image){
		// summary: creates a image shape
		// image: Object: an image object (see dojox.gfx.defaultImage)
		return this.createObject(dojox.gfx.Image, image);	// dojox.gfx.Image
	},
	createText: function(text){
		// summary: creates a text shape
		// text: Object: a text object (see dojox.gfx.defaultText)
		return this.createObject(dojox.gfx.Text, text);	// dojox.gfx.Text
	},
	createPath: function(path){
		// summary: creates a path shape
		// path: Object: a path object (see dojox.gfx.defaultPath)
		return this.createObject(dojox.gfx.Path, path);	// dojox.gfx.Path
	},
	createTextPath: function(text){
		// summary: creates a text shape
		// text: Object: a textpath object (see dojox.gfx.defaultTextPath)
		return this.createObject(dojox.gfx.TextPath, {}).setText(text);	// dojox.gfx.TextPath
	},
	createObject: function(shapeType, rawShape){
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method

		// SHOULD BE RE-IMPLEMENTED BY THE RENDERER!

		return null;	// dojox.gfx.Shape
	}
};

}

if(!dojo._hasResource["dojox.gfx.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.utils"] = true;
dojo.provide("dojox.gfx.utils");



(function(){
	var d = dojo, g = dojox.gfx, gu = g.utils;

	dojo.mixin(gu, {
		forEach: function(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ object,
			/*Function|String|Array*/ f, /*Object?*/ o
		){
			o = o || d.global;
			f.call(o, object);
			if(object instanceof g.Surface || object instanceof g.Group){
				d.forEach(object.children, function(shape){
					gu.forEach(shape, f, o);
				});
			}
		},

		serialize: function(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ object
		){
			var t = {}, v, isSurface = object instanceof g.Surface;
			if(isSurface || object instanceof g.Group){
				t.children = d.map(object.children, gu.serialize);
				if(isSurface){
					return t.children;	// Array
				}
			}else{
				t.shape = object.getShape();
			}
			if(object.getTransform){
				v = object.getTransform();
				if(v){ t.transform = v; }
			}
			if(object.getStroke){
				v = object.getStroke();
				if(v){ t.stroke = v; }
			}
			if(object.getFill){
				v = object.getFill();
				if(v){ t.fill = v; }
			}
			if(object.getFont){
				v = object.getFont();
				if(v){ t.font = v; }
			}
			return t;	// Object
		},

		toJson: function(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ object,
			/* Boolean? */ prettyPrint
		){
			return d.toJson(gu.serialize(object), prettyPrint);	// String
		},

		deserialize: function(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ parent,
			/* dojox.gfx.Shape || Array */ object
		){
			if(object instanceof Array){
				return d.map(object, d.hitch(null, gu.deserialize, parent));	// Array
			}
			var shape = ("shape" in object) ? parent.createShape(object.shape) : parent.createGroup();
			if("transform" in object){
				shape.setTransform(object.transform);
			}
			if("stroke" in object){
				shape.setStroke(object.stroke);
			}
			if("fill" in object){
				shape.setFill(object.fill);
			}
			if("font" in object){
				shape.setFont(object.font);
			}
			if("children" in object){
				d.forEach(object.children, d.hitch(null, gu.deserialize, shape));
			}
			return shape;	// dojox.gfx.Shape
		},

		fromJson: function(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ parent,
			/* String */ json){
			return gu.deserialize(parent, d.fromJson(json));	// Array || dojox.gfx.Shape
		},

		toSvg: function(/*GFX object*/surface){
			// summary:
			//		Function to serialize a GFX surface to SVG text.
			// description:
			//		Function to serialize a GFX surface to SVG text.  The value of this output
			//		is that there are numerous serverside parser libraries that can render
			//		SVG into images in various formats.  This provides a way that GFX objects
			//		can be captured in a known format and sent serverside for serialization
			//		into an image.
			// surface:
			//		The GFX surface to serialize.
			// returns:
			//		Deferred object that will be called when SVG serialization is complete.
		
			//Since the init and even surface creation can be async, we need to
			//return a deferred that will be called when content has serialized.
			var deferred = new dojo.Deferred();
		
			if(dojox.gfx.renderer === "svg"){
				//If we're already in SVG mode, this is easy and quick.
				try{
					var svg = gu._cleanSvg(gu._innerXML(surface.rawNode));
					deferred.callback(svg);
				}catch(e){
					deferred.errback(e);
				}
			}else{
				//Okay, now we have to get creative with hidden iframes and the like to
				//serialize SVG.
				if (!gu._initSvgSerializerDeferred) {
					gu._initSvgSerializer();
				}
				var jsonForm = dojox.gfx.utils.toJson(surface);
				var serializer = function(){
					try{
						var sDim = surface.getDimensions();
						var width = sDim.width;
						var	height = sDim.height;

						//Create an attach point in the iframe for the contents.
						var node = gu._gfxSvgProxy.document.createElement("div");
						gu._gfxSvgProxy.document.body.appendChild(node);
						//Set the node scaling.
						dojo.withDoc(gu._gfxSvgProxy.document, function() {
							dojo.style(node, "width", width);
							dojo.style(node, "height", height);
						}, this);

						//Create temp surface to render object to and render.
						var ts = gu._gfxSvgProxy[dojox._scopeName].gfx.createSurface(node, width, height);

						//It's apparently possible that a suface creation is async, so we need to use
						//the whenLoaded function.  Probably not needed for SVG, but making it common
						var draw = function(surface) {
							try{
								gu._gfxSvgProxy[dojox._scopeName].gfx.utils.fromJson(surface, jsonForm);

								//Get contents and remove temp surface.
								var svg = gu._cleanSvg(node.innerHTML);
								surface.clear();
								surface.destroy();
								gu._gfxSvgProxy.document.body.removeChild(node);
								deferred.callback(svg);
							}catch(e){
								deferred.errback(e);
							}
						};
						ts.whenLoaded(null,draw);
					 }catch (ex) {
						deferred.errback(ex);
					}
				};
				//See if we can call it directly or pass it to the deferred to be
				//called on initialization.
				if(gu._initSvgSerializerDeferred.fired > 0){
					serializer();
				}else{
					gu._initSvgSerializerDeferred.addCallback(serializer);
				}
			}
			return deferred; //dojo.Deferred that will be called when serialization finishes.
		},

		//iFrame document used for handling SVG serialization.
		_gfxSvgProxy: null,

		//Serializer loaded.
		_initSvgSerializerDeferred: null,

		_svgSerializerInitialized: function() {
			// summary:
			//		Internal function to call when the serializer init completed.
			// tags:
			//		private
			gu._initSvgSerializerDeferred.callback(true);
		},

		_initSvgSerializer: function(){
			// summary:
			//		Internal function to initialize the hidden iframe where SVG rendering
			//		will occur.
			// tags:
			//		private
			if(!gu._initSvgSerializerDeferred){
				gu._initSvgSerializerDeferred = new dojo.Deferred();
				var f = dojo.doc.createElement("iframe");
				dojo.style(f, {
					display: "none",
					position: "absolute",
					width: "1em",
					height: "1em",
					top: "-10000px"
				});
				var intv;
				if(dojo.isIE){
					f.onreadystatechange = function(){
						if(f.contentWindow.document.readyState == "complete"){
							f.onreadystatechange = function() {};
							intv = setInterval(function() {
								if(f.contentWindow[dojo._scopeName] &&
								   f.contentWindow[dojox._scopeName].gfx &&
								   f.contentWindow[dojox._scopeName].gfx.utils){
									clearInterval(intv);
									f.contentWindow.parent[dojox._scopeName].gfx.utils._gfxSvgProxy = f.contentWindow;
									f.contentWindow.parent[dojox._scopeName].gfx.utils._svgSerializerInitialized();
								}
							}, 50);
						}
					};
				}else{
					f.onload = function(){
						f.onload = function() {};
						intv = setInterval(function() {
							if(f.contentWindow[dojo._scopeName] &&
							   f.contentWindow[dojox._scopeName].gfx &&
							   f.contentWindow[dojox._scopeName].gfx.utils){
								clearInterval(intv);
								f.contentWindow.parent[dojox._scopeName].gfx.utils._gfxSvgProxy = f.contentWindow;
								f.contentWindow.parent[dojox._scopeName].gfx.utils._svgSerializerInitialized();
							}
						}, 50);
					};
				}
				//We have to load the GFX SVG proxy frame.  Default is to use the one packaged in dojox.
				var uri = (dojo.config["dojoxGfxSvgProxyFrameUrl"]||dojo.moduleUrl("dojox", "gfx/resources/gfxSvgProxyFrame.html"));
				f.setAttribute("src", uri);
				dojo.body().appendChild(f);
			}
		},

		_innerXML: function(/*Node*/node){
			// summary:
			//		Implementation of MS's innerXML function, borrowed from dojox.xml.parser.
			// node:
			//		The node from which to generate the XML text representation.
			// tags:
			//		private
			if(node.innerXML){
				return node.innerXML;	//String
			}else if(node.xml){
				return node.xml;		//String
			}else if(typeof XMLSerializer != "undefined"){
				return (new XMLSerializer()).serializeToString(node);	//String
			}
			return null;
		},

		_cleanSvg: function(svg) {
			// summary:
			//		Internal function that cleans up artifacts in extracted SVG content.
			// tags:
			//		private
			if(svg){
				//Make sure the namespace is set.
				if(svg.indexOf("xmlns=\"http://www.w3.org/2000/svg\"") == -1){
					svg = svg.substring(4, svg.length);
					svg = "<svg xmlns=\"http://www.w3.org/2000/svg\"" + svg;
				}
				//Do some other cleanup, like stripping out the
				//dojoGfx attributes.
				svg = svg.replace(/\bdojoGfx\w*\s*=\s*(['"])\w*\1/g, "");
			}
			return svg;  //Cleaned SVG text.
		}
	});
})();

}

if(!dojo._hasResource["dojox.html.metrics"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.html.metrics"] = true;
dojo.provide("dojox.html.metrics");

(function(){
	var dhm = dojox.html.metrics;

	//	derived from Morris John's emResized measurer
	dhm.getFontMeasurements = function(){
		//	summary
		//	Returns an object that has pixel equivilents of standard font size values.
		var heights = {
			'1em':0, '1ex':0, '100%':0, '12pt':0, '16px':0, 'xx-small':0, 'x-small':0,
			'small':0, 'medium':0, 'large':0, 'x-large':0, 'xx-large':0
		};
	
		if(dojo.isIE){
			//	we do a font-size fix if and only if one isn't applied already.
			//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
			dojo.doc.documentElement.style.fontSize="100%";
		}
	
		//	set up the measuring node.
		var div=dojo.doc.createElement("div");
		var ds = div.style;
		ds.position="absolute";
		ds.left="-100px";
		ds.top="0";
		ds.width="30px";
		ds.height="1000em";
		ds.borderWidth="0";
		ds.margin="0";
		ds.padding="0";
		ds.outline="0";
		ds.lineHeight="1";
		ds.overflow="hidden";
		dojo.body().appendChild(div);
	
		//	do the measurements.
		for(var p in heights){
			ds.fontSize = p;
			heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
		}
		
		dojo.body().removeChild(div);
		div = null;
		return heights; 	//	object
	};

	var fontMeasurements = null;
	
	dhm.getCachedFontMeasurements = function(recalculate){
		if(recalculate || !fontMeasurements){
			fontMeasurements = dhm.getFontMeasurements();
		}
		return fontMeasurements;
	};

	var measuringNode = null, empty = {};
	dhm.getTextBox = function(/* String */ text, /* Object */ style, /* String? */ className){
		var m, s;
		if(!measuringNode){
			m = measuringNode = dojo.doc.createElement("div");
			// Container that we can set contraints on so that it doesn't
			// trigger a scrollbar.
			var c = dojo.doc.createElement("div");
			c.appendChild(m);
			s = c.style;
			s.overflow='scroll';
			s.position = "absolute";
			s.left = "0px";
			s.top = "-10000px";
			s.width = "1px";
			s.height = "1px";
			s.visibility = "hidden";
			s.borderWidth = "0";
			s.margin = "0";
			s.padding = "0";
			s.outline = "0";
			dojo.body().appendChild(c);
		}else{
			m = measuringNode;
		}
		// reset styles
		m.className = "";
		s = m.style;
		s.borderWidth = "0";
		s.margin = "0";
		s.padding = "0";
		s.outline = "0";
		// set new style
		if(arguments.length > 1 && style){
			for(var i in style){
				if(i in empty){ continue; }
				s[i] = style[i];
			}
		}
		// set classes
		if(arguments.length > 2 && className){
			m.className = className;
		}
		// take a measure
		m.innerHTML = text;
		var box = dojo.position(m);
		// position doesn't report right (reports 1, since parent is 1)
		// So we have to look at the scrollWidth to get the real width
		// Height is right.
		box.w = m.parentNode.scrollWidth;
		return box;
	};

	//	determine the scrollbar sizes on load.
	var scroll={ w:16, h:16 };
	dhm.getScrollbar=function(){ return { w:scroll.w, h:scroll.h }; };

	dhm._fontResizeNode = null;

	dhm.initOnFontResize = function(interval){
		var f = dhm._fontResizeNode = dojo.doc.createElement("iframe");
		var fs = f.style;
		fs.position = "absolute";
		fs.width = "5em";
		fs.height = "10em";
		fs.top = "-10000px";
		if(dojo.isIE){
			f.onreadystatechange = function(){
				if(f.contentWindow.document.readyState == "complete"){
					f.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
				}
			};
		}else{
			f.onload = function(){
				f.contentWindow.onresize = f.contentWindow.parent[dojox._scopeName].html.metrics._fontresize;
			};
		}
		//The script tag is to work around a known firebug race condition.  See comments in bug #9046
		f.setAttribute("src", "javascript:'<html><head><script>if(\"loadFirebugConsole\" in window){window.loadFirebugConsole();}</script></head><body></body></html>'");
		dojo.body().appendChild(f);
		dhm.initOnFontResize = function(){};
	};

	dhm.onFontResize = function(){};
	dhm._fontresize = function(){
		dhm.onFontResize();
	}

	dojo.addOnUnload(function(){
		// destroy our font resize iframe if we have one
		var f = dhm._fontResizeNode;
		if(f){
			if(dojo.isIE && f.onresize){
				f.onresize = null;
			}else if(f.contentWindow && f.contentWindow.onresize){
				f.contentWindow.onresize = null;
			}
			dhm._fontResizeNode = null;
		}
	});

	dojo.addOnLoad(function(){
		// getScrollbar metrics node
		try{
			var n=dojo.doc.createElement("div");
			n.style.cssText = "top:0;left:0;width:100px;height:100px;overflow:scroll;position:absolute;visibility:hidden;";
			dojo.body().appendChild(n);
			scroll.w = n.offsetWidth - n.clientWidth;
			scroll.h = n.offsetHeight - n.clientHeight;
			dojo.body().removeChild(n);
			//console.log("Scroll bar dimensions: ", scroll);
			delete n;
		}catch(e){}

		// text size poll setup
		if("fontSizeWatch" in dojo.config && !!dojo.config.fontSizeWatch){
			dhm.initOnFontResize();
		}
	});
})();

}

if(!dojo._hasResource['ibm_ilog.diagram._base.gfxext']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram._base.gfxext'] = true;
dojo.provide('ibm_ilog.diagram._base.gfxext');





dojo.requireIf(dojo.config.isBidiInDojoDiagrammer, 'ibm_ilog.diagram.util.BidiUtil');

(function() {
	var g = dojox.gfx, gs = g.shape, is16 = dojo.version.major > 1 || (dojo.version.major == 1 && dojo.version.minor >= 6);
	var iid = ibm_ilog.diagram;

	var renderer = g.renderer;
	switch (renderer) {
		case 'vml':
			iid.isVml = true;
			break;
		case 'svg':
			iid.isSvg = true;
			if (dojox.gfx.svg.useSvgWeb) {              
				iid.isSvgWeb = true;
			}			
			break;
		case 'silverlight':
			iid.isSilverlight = true;
			break;
		case 'canvas':
		case 'canvasWithEvents':
			iid.isCanvas = true;
			break;
	}
	
	iid.mobileEnabled = iid.isSilverlight ? false : true;

	// VML texts fix in Dojo < 1.5
	if (dojo.isIE >= 8 && iid.isVml && dojo.version.major == 1 && dojo.version.minor < 5) {
		var s = document.createStyleSheet();
		s.addRule("v\\:path", "behavior:url(#default#VML); display:inline-block");
		s.addRule("v\\:textpath", "behavior:url(#default#VML); display:inline-block");
		s.addRule("v\\:text", "behavior:url(#default#VML); display:inline-block");
	}
	
	// VML Container.add fix for Dojo >= 1.6
	if(iid.isVml && dojo.version.major == 1 && dojo.version.minor >= 6){
		// Surface/Group.add in Dojo 1.6+ will call setShape and setFont recursively
		// on all children of the Group. This causes trouble in our code because we
		// trap these methods to notify changes in parents. To avoid this, we set
		// a global flag saying that we are inside add, and we should not propagate
		// notifications up. The flase is also checked in the MultilineText object
		// to avoid re-wrapping text.
		var oldAdd = dojox.gfx.vml.Surface.prototype.add;
		var myAdd = function(shape){
			var oldInAdd = iid._inContainerAdd;
			iid._inContainerAdd = true;
			oldAdd.call(this, shape);
			iid._inContainerAdd = oldInAdd;
			return this;	// self
		};
		
		dojox.gfx.vml.Surface.prototype.add = myAdd;
		dojox.gfx.vml.Group.prototype.add = myAdd;
	}
	
	//
	// Canvas fix
	//
	if (iid.isCanvas && dojo.version.major === 1 && dojo.version.minor <= 5) {
		// The _setFont method is missing !
		dojo.extend(gs.Text, {
			_setFont: function() {
			}
		});
	}
	
	
	//
	// uniqueId generator
	//
	var _currentId = 0;
	iid._IdGenerator = {};
	iid._IdGenerator.nextId = function(obj) {
		return ++_currentId;
	};
	
	// "invisible location" values:
	if(!iid.isVml){
		iid._invisibleLocation = -10000;
		iid._invisibleSize = 1010;
	} else {
		iid._invisibleLocation = 100000;
		iid._invisibleSize = 10;
	}
	
	iid._base.gfxext.patchMethod = function(/*Object*/type, /*String*/ method, /*Function*/ execBefore, /*Function*/ execAfter) {
		// summary:
		//     Patches the specified method of the given type so that the 'execBefore' (resp. 'execAfter') function is 
		//     called before (resp. after) invoking the legacy implementation.
		// description:
		//     The execBefore function is invoked with the following parameter:
		//     execBefore(method, arguments) where 'method' is the patched method name and 'arguments' the arguments received
		//     by the legacy implementation.
		//     The execAfter function is invoked with the following parameter:
		//     execBefore(method, returnValue, arguments) where 'method' is the patched method name, 'returnValue' the value
		//     returned by the legacy implementation and 'arguments' the arguments received by the legacy implementation.
		// type: Object: the type to patch.
		// method: String: the method name.
		// execBefore: Function: the function to execute before the legacy implementation. 
		// execAfter: Function: the function to execute after the legacy implementation.
		var old = type.prototype[method];
		type.prototype[method] = function() {
			var callee = method;
			if (execBefore) 
				execBefore.call(this, callee, arguments);
			var ret = old.apply(this, arguments);
			if (execAfter) 
				ret = execAfter.call(this, callee, ret, arguments) || ret;
			return ret;
		};
	};
	
	// SVGWeb fix
	if(iid.isSvgWeb){
		var svg = is16 ? g.svg : g;
		var oldcs = svg.createSurface;
		svg.createSurface = function(parentNode, width, height){
			var s = oldcs.call(this, parentNode, width, height);
			s._parent = dojo.byId(parentNode);
			return s;
		};
		if(svg != g)
			g.createSurface = svg.createSurface;
	}
	
	//
	// Silverlight event conversion fix
	// Fixes:
	//  1.  e.target should be mapped to EventArgs.Source
	//  2.  e.pageX/pageY are added so that it does not break the interactions code
	// COO: Code borrowed from dojox/gfx/silverlight.js
	if (iid.isSilverlight) {
		
		dojo.extend(dojox.gfx.Group, {
			_setFillAttr : function() {}
		});
		
		
	    var surfaces = dojox.gfx.silverlight.surfaces;
		var mouseFix = function(s, a) {
			var ev = {
				target: s,
				currentTarget: s,
				preventDefault: function() {
				},
				stopPropagation: function() {
				}
			};
			if (a) {
				try {
					// BEGIN PATCH
					if (a.source) // source is defined from Silverlight 2+
						ev.target = a.source;
					// END PATCH
					ev.ctrlKey = a.ctrl;
					ev.shiftKey = a.shift;
					var p = a.getPosition(null);
					ev.x = ev.offsetX = ev.layerX = p.x;
					ev.y = ev.offsetY = ev.layerY = p.y;
					// calculate clientX and clientY
					var parent = surfaces[s.getHost().content.root.name];
					var t = dojo.position(parent);
					ev.clientX = t.x + p.x;
					ev.clientY = t.y + p.y;
					// BEGIN PATCH 2
					var d = (dojo.doc.documentElement && dojo.doc.documentElement.scrollLeft != null) ? dojo.doc.documentElement : dojo.doc.body;
					ev.pageX = ev.clientX + d.scrollLeft;
					ev.pageY = ev.clientY + d.scrollTop;
					// END PATCH 2
				} catch (e) {
					// squelch bugs in MouseLeave's implementation
				}
			}
			return ev;
		};
		var keyFix = function(s, a) {
			var ev = {
				keyCode: a.platformKeyCode,
				ctrlKey: a.ctrl,
				shiftKey: a.shift
			};
			// BEGIN PATCH
			if (a.source) // source is defined from Silverlight 2+
				ev.target = a.source;
			// END PATCH
			return ev;
		};
		var eventNames = {
			onclick: {
				name: "MouseLeftButtonUp",
				fix: mouseFix
			},
			onmouseenter: {
				name: "MouseEnter",
				fix: mouseFix
			},
			onmouseleave: {
				name: "MouseLeave",
				fix: mouseFix
			},
			onmouseover: {
				name: "MouseEnter",
				fix: mouseFix
			},
			onmouseout: {
				name: "MouseLeave",
				fix: mouseFix
			},
			onmousedown: {
				name: "MouseLeftButtonDown",
				fix: mouseFix
			},
			onmouseup: {
				name: "MouseLeftButtonUp",
				fix: mouseFix
			},
			onmousemove: {
				name: "MouseMove",
				fix: mouseFix
			},
			onkeydown: {
				name: "KeyDown",
				fix: keyFix
			},
			onkeyup: {
				name: "KeyUp",
				fix: keyFix
			}
		};
		var eventsProcessing = {
			connect: function(name, object, method) {
				var token, n = name in eventNames ? eventNames[name] : {
					name: name,
					fix: function() {
						return {};
					}
				};
				if (arguments.length > 2) {
					token = this.getEventSource().addEventListener(n.name, function(s, a) {
						dojo.hitch(object, method)(n.fix(s, a));
					});
				} else {
					token = this.getEventSource().addEventListener(n.name, function(s, a) {
						object(n.fix(s, a));
					});
				}
				return {
					name: n.name,
					token: token
				};
			},
			disconnect: function(token) {
				this.getEventSource().removeEventListener(token.name, token.token);
			}
		};
		
		dojo.extend(is16 ? dojox.gfx.shape.Shape : dojox.gfx.Shape, eventsProcessing);
		dojo.extend(dojox.gfx.Surface, eventsProcessing);
		// COO END
	}
	
	//
	// Extends createShape to support our own shape types
	//
	
	var extendCreateShape = function(callee, ret, args) {
		if (!ret) {
			var sh = args[0];
			switch (sh.type) {
				case 'graph':
					ret = this.createObject(iid.Graph);
					break;
				case 'node':
					ret = this.createObject(iid.Node);
					break;
				case 'link':
					ret = this.createObject(iid.Link);
					break;
				case 'subgraph':
					ret = this.createObject(iid.Subgraph);
					break;
				case 'swimlane':
					ret = this.createObject(iid.SwimLane);
					break;
			}
		}
		return ret;
	};
	iid._base.gfxext.patchMethod(g.Surface, "createShape", null, extendCreateShape);
	iid._base.gfxext.patchMethod(g.Group, "createShape", null, extendCreateShape);
	
	var vmlFixList = new dojox.collections.ArrayList();
	
	var needsVmlFix = function(instance) {
		for (var i = 0; i < vmlFixList.count; ++i) {
			if (instance instanceof vmlFixList.item(i)) {
				return true;
			}
		}
		return false;
	};
	
	iid.gfxDeclaration = function(ctor) {
		ctor.nodeType = dojox.gfx.Group.nodeType;
		vmlFixList.add(ctor);
		return ctor;
	};
	
	iid.gfxDeclare = function(name, superclass, declarations) {
		//
		//	NOTE: warning, this may break dojo doc tools
		//
		var ctor = dojo.declare.apply(dojo, arguments);
		return iid.gfxDeclaration(ctor);
	};
	
	//
	// Fix VML createObject to perform additional initializations
	// performed in VML createObject implementation.
	//
	
	if (iid.isVml) {
		var fixVMLCreateObject = function(callee, ret, args) {
			var shape = ret;
			if ((iid.Node && (shape instanceof iid.Node)) ||
			(iid.Graph && (shape instanceof iid.Graph)) ||
			(iid.Subgraph && (shape instanceof iid.Subgraph)) ||
			(iid.Link && (shape instanceof iid.Link)) ||
			(needsVmlFix(shape))) {
			
				// Call Creator._overrideSize
				var node = shape.rawNode;
				this._overrideSize(node);
				
				// Need to do vml-specific createGroup tasks
				// see dojox.gfx.vml.js/createGroup()
				var r = shape.rawNode.ownerDocument.createElement("v:rect");
				r.style.left = r.style.top = 0;
				r.style.width = shape.rawNode.style.width;
				r.style.height = shape.rawNode.style.height;
				r.filled = r.stroked = "f";
				shape.rawNode.appendChild(r);
				shape.bgNode = r;
			}
		};
		iid._base.gfxext.patchMethod(g.Surface, 'createObject', null, fixVMLCreateObject);
		iid._base.gfxext.patchMethod(g.Group, 'createObject', null, fixVMLCreateObject);
	}
	
	//
	// Change notifications
	//
	
	iid._base.gfxext.propagateChange = function(source, reason) {
		for (var shape = source; shape && !shape._noOnChanged && shape.getParent; shape = shape.getParent()) {
			if (shape._onChanged) 
				shape._onChanged(source, reason);
		}
	};
	
	(function() {
		var dv = dojo.version, fixme = dv.major === 1 && (dv.minor < 4 || dv.patch === 0);
		var fixvml = iid.isVml;
		// a function used to propagate the change notification to the hierarchy
		var onChangeFunc = function(callee, ret, args) {
			if(iid._inContainerAdd) return;
			if (dojo.config.useGfxLayout) {
				this.__shapeSet(callee);
				// no need to invalidate top-level panels that are translated.
				var p = this.getParent();
				if (callee !== 'setTransform' || (p && p.getLayout && p.getLayout())) 
					this.invalidate();
			}
			iid._base.gfxext.propagateChange(this, callee);
		};
		// make it accessible from outside (used in MultilineText as it overrides the setShape method
		// and therefore delete the patch
		iid._base.gfxext.onChangePatch = onChangeFunc;
		 		
		// a patched onChangeFunc to fix bug in dojo < 1.4.1
		var onChangeFunc2 = function(callee, ret, args) {
			this.bbox = null;
			onChangeFunc.apply(this, arguments);
		};
		// a patched function for vml to be called before the original function 
		var vmlFixRawNodePath = function() {
			if (!this.rawNode.path) 
				this.rawNode.path = {};
		};
		// yet another patch for VML: dojo 1.3 does not like shapes with 0 width/height
		var vmlFixZeroSize = (iid.isVml && dojo.version.major == 1 && dojo.version.minor <= 3) ? 
			function(callee, args){
				var shape = args[0];
				if (shape) {
					if (shape.width === 0)
						shape.width = 1;
					if (shape.height === 0)
						shape.height = 1;
				}
			} : null;
		
		//Bidi support functions
		if (dojo.config.isBidiInDojoDiagrammer) {
		    //apply Bidi format on the text, using text direction and complex expression properties
    	    var bidiUpdateText = function(callee, args) {      
			    var sh = args[0];
			    if (sh === undefined){
				    return;
			    }
			    //for case the user call setShape()with argument containing a new textDir but not text, like text.setShape({textDir: "ltr"});
			    if (sh.text === undefined) {
				    if (this.getShape().__beforeBidi)
					    sh.text = this.getShape().__beforeBidi;
				    else {
					    if (this.getShape().text === undefined)
						    return;
					    sh.text = this.getShape().text;
				    }	
			    }
			    if (sh.textDir === undefined) {
			        if (sh.ceType) {
			           this.getShape().textDir = "ltr"; 
			        }
				    if (this.getShape().textDir === undefined) 
					    return;
				    sh.textDir = this.getShape().textDir; //for case like applyLayout(), when textDir is not set in argument shape.
			    }
                if (typeof(sh.text) != "string") {
				    sh.text = sh.text.toString();
                }
			    if (sh.__stopBidi || sh.text.length < 2) { //no need to apply bidi format
				    if (sh.__stopBidi) //flag __stopBidi can be used by user to prevent multiple transformation of the same text
					    delete sh.__stopBidi;
				    return;
			    }
			    
			    //store original, non processed text in __beforeBidi field
			    if (args[1] === undefined) {  
			    	if (sh.__beforeBidi) {    
			            this.getShape().__old = sh.__beforeBidi; //fix for print (deserialization), to fix second transformation of the same text during createText
			        }
			        sh.__beforeBidi = sh.text;
			    }  
			    else if (sh.__beforeBidi) // text direction was changed after the text was processed
			        sh.text = sh.__beforeBidi;  
			    
			    //need for additional/future shape processing, like applyLayout() 
                this.getShape().__beforeBidi = sh.__beforeBidi;  
			    
			    //complex expression case
			    if (sh.ceType) {
                    sh = iid.util.BidiUtil.complExp.call(this,sh);
                    this.getShape().ceType = sh.ceType;		            
                    return true;
			    }
			    sh = iid.util.BidiUtil.bidiFormat.call(this,sh);
			    return true;
		    };
      		
      		//set text direction to the shape, get if from argument, or
      		//from the parents, if possible.  
      		//then we call setShape with shape containing textDir parameter
      		var bidiUpdateShape = function(calee, textShape, args) {			
			    var sh = textShape.getShape();
			    var bidi = false;
			    if (!sh.textDir && args[0] && args[0].textDir)
				    sh.textDir = args[0].textDir;
    		  				
			    //trying to get textDir form parents			
			    if (!sh.textDir) {
				    bidi = true;
				    var sp = textShape;
				    var i = 0;
				    while (sp) {
					    if (sp.textDir) {
						    sh.textDir = sp.textDir;
						    break;
					    }
					    else if(sp.getTextDir) {
						    sh.textDir = sp.getTextDir();
						    break;
					    }
					    if (!sp.parent)
						    break;
					    sp = sp.getParent();
					    i++;
				    }
			    }
       			if (sh.__old){ //fix for print (deserialization), to fix second transformation of the same text during createText
    			    sh.text = sh.__old;
    			    delete sh.__old;
    			    if (args[0] && args[0].__beforeBidi)
    			        delete args[0].__beforeBidi;
    			    bidi = true;
    			} 			
			    if (sh.textDir && bidi) {
				    if (args[0] && args[0].ceType) //parameter for complex expression, for case, textDir was not set explicitly in the template
				        sh.ceType = args[0].ceType;	
				    textShape.setShape(sh);
			    }
    		
			    return textShape;
		    };	
		 }   
		iid._base.gfxext.patchMethod(dojox.gfx.Rect, "setShape", vmlFixZeroSize, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Circle, "setShape", null, fixme ? onChangeFunc2 : onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Ellipse, "setShape", null, fixme ? onChangeFunc2 : onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Line, "setShape", fixvml ? vmlFixRawNodePath : null, fixme ? onChangeFunc2 : onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Polyline, "setShape", fixvml ? vmlFixRawNodePath : null, fixme ? onChangeFunc2 : onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Path, "setShape", fixvml ? vmlFixRawNodePath : null);
		iid._base.gfxext.patchMethod(dojox.gfx.Image, "setShape", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Text, "setShape", dojo.config.isBidiInDojoDiagrammer ? bidiUpdateText : null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.TextPath, "setShape", null, onChangeFunc);
		
		iid._base.gfxext.patchMethod(dojox.gfx.Rect, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Circle, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Ellipse, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Line, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Polyline, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Image, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Text, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.TextPath, "setTransform", null, onChangeFunc);
		
		iid._base.gfxext.patchMethod(dojox.gfx.Group, "setTransform", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Group, "add", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Group, "remove", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Group, "clear", null, onChangeFunc);
		iid._base.gfxext.patchMethod(dojox.gfx.Group, "createText", null, dojo.config.isBidiInDojoDiagrammer ? bidiUpdateShape : null);  		
		
		iid._base.gfxext.patchMethod(dojox.gfx.Text, "setFont", null, onChangeFunc);
		
	})();
	
	//
	// dojox.gfx.matrix.Matrix2D extensions
	//
	
	dojo.mixin(dojox.gfx.matrix, {
		createFromRectangles: function(/* dojox.gfx.Rectangle */content, /*dojox.gfx.Rectangle */ viewport, /*Boolean*/ keepAspectRatio) {
			// summary:
			//        Creates a dojo.gfx.Matrix2d instance that allows to transform
			//        a rectangle so that it is visible in a viewport rectangle.
			// content:
			//        The content rectangle to display in the viewport.
			// viewport:
			//        The destination viewport.
			// keepAspectRatio:
			//		  If true, the resulting matrix preserves the aspect ratio, 
			//			so applying it to the content rectangle may not give exactly the viewport rectangle.
			//		  If false, the resulting matrix does not preserve the aspect ratio, 
			//			and it exactly transforms the content rectangle into the viewport rectangle.
			var sx = (Math.abs(viewport.width - content.width) < 0.001) ? 1 : viewport.width / content.width;
			var sy = (Math.abs(viewport.height - content.height) < 0.001) ? 1 : viewport.height / content.height;
			var t = new dojox.gfx.matrix.Matrix2D();
			if (keepAspectRatio) {
				var s = Math.min(sx, sy);
				t.xx = t.yy = s;
				t.dx = (viewport.x + viewport.width / 2 - s * (content.x + content.width / 2));
				t.dy = (viewport.y + viewport.height / 2 - s * (content.y + content.height / 2));
			} else {
				t.xx = sx;
				t.yy = sy;
				t.dx = viewport.x - sx * content.x;
				t.dy = viewport.y - sy * content.y;
			}
			t.xy = 0;
			t.yx = 0;
			return t;
		}
	});
	dojo.extend(dojox.gfx.matrix.Matrix2D, {
		inverse: function() {
			// summary:
			//        Returns the inverse transformation.
			return dojox.gfx.matrix.invert(this);
		},
		isScale: function() {
			// summary:
			//        Indicates whether this transform corresponds to a scaling operation.
			return this.xy === 0 && this.yx === 0;
		},
		isTranslation: function() {
			// summary:
			//        Indicates whether this transform corresponds to a translation operation.
			return this.xy === 0 && this.yx === 0 && this.xx === 1 && this.yy === 1;
		},
		isIdentity: function() {
			// summary:
			//        Indicates whether this transform corresponds to the identity operation.
			return this.xy === 0 && this.yx === 0 && this.xx === 1 && this.yy === 1 && this.dx === 0 && this.dy === 0;
		},
		isRotation: function() {
			// summary:
			//        Indicates whether this transform corresponds to a rotation operation.
			return (this.xy === 0 && this.yx === 0) || (this.yx * this.xx + this.xy * this.yy === 0);
		},
		isInvertible: function() {
			// summary:
			//        Indicates whether this transform is invertible.
			return this.xx * this.yy - this.yx * this.xy !== 0;
		},
		isSame: function(t) {
			// summary:
			//        Indicates whether this transform and the specified transform are the same.
			return this.xx === t.xx && this.xy === t.xy && this.yx === t.yx &&
			this.yy === t.yy &&
			this.dx === t.dx &&
			this.dy === t.dy;
		},
		transformPoints: function(points) {
			// summary:
			//        Applies this transformation to an array of points.
			return dojo.map(points, function(p) {
				return dojox.gfx.matrix.multiplyPoint(this, p);
			}, this);
		},
		transformPoint: function(/* dojox.gfx.Point */p) {
			// summary:
			//        Applies this transformation to the specified point.
			return dojox.gfx.matrix.multiplyPoint(this, p);
		},
		transformRectangle: function(/* dojox.gfx.Rectangle */rect) {
			// summary:
			//        Applies the transformation to a rectangle.
			// description:
			//        The method applies the transformation on all corners of the
			//        rectangle and returns the smallest rectangle enclosing the 4 transformed
			//        points.            
			rect = rect ||
			{
				x: 0,
				y: 0,
				width: 0,
				height: 0
			};
			if (this.isIdentity()) {
				return {
					'x': rect.x,
					'y': rect.y,
					'width': rect.width,
					'height': rect.height
				};
			}
			var m = dojox.gfx.matrix;
			var p0 = m.multiplyPoint(this, rect.x, rect.y);
			var p1 = m.multiplyPoint(this, rect.x, rect.y + rect.height);
			var p2 = m.multiplyPoint(this, rect.x + rect.width, rect.y);
			var p3 = m.multiplyPoint(this, rect.x + rect.width, rect.y + rect.height);
			var minx = Math.min(p0.x, Math.min(p1.x, Math.min(p2.x, p3.x)));
			var miny = Math.min(p0.y, Math.min(p1.y, Math.min(p2.y, p3.y)));
			var maxx = Math.max(p0.x, Math.max(p1.x, Math.max(p2.x, p3.x)));
			var maxy = Math.max(p0.y, Math.max(p1.y, Math.max(p2.y, p3.y)));
			var r = {};
			r.x = minx;
			r.y = miny;
			r.width = maxx - minx;
			r.height = maxy - miny;
			return r;
		},
		transformVector: function(/* dojox.gfx.Point || dojox.gfx.Size */p) {
			// summary:
			//        Applies the transformation to a vector.
			var x, y, r = {};
			if (this.isIdentity()) {
				return p;
			}
			if ('x' in p && 'y' in p) { // point
				x = p.x;
				y = p.y;
				r.x = (x * this.xx) + (y * this.xy);
				r.y = (x * this.yx) + (y * this.yy);
			} else if ('width' in p && 'height' in p) { // size
				x = p.width;
				y = p.height;
				r.width = (x * this.xx) + (y * this.xy);
				r.height = (x * this.yx) + (y * this.yy);
			}
			return r;
		}
	});
	
	// dojox.gfx.Text.getBoundingBox fix
	_isOrphan = function(shape) {
		// a Surface does not have getParent() method.
		/*
		 for (var p = shape.getParent();p && p.getParent; p=p.getParent()){
		 }
		 return p === null;
		 */
		var p = shape.rawNode;
		while (p && p !== document.body) {
			if (p.style && p.style.display === 'none') 
				return true;
			p = p.parentNode;
		}
		return p !== document.body;
	};
	
	dojo.extend(dojox.gfx.Text, {
		getBoundingBox: function() {
			var sh = this.getShape(), sz;
			var text = sh.text;
			
			if ( !text || text.length == 0 )
				return null;

			var dictionary = dojox.gfx.Text.textDictionary
			if ( dictionary && !dictionary.detected ){
				dictionary.detected = true;
				
			}
			
			
			if ( dictionary && ! dictionary.messageLogged ){
				dictionary.messageLogged = true;
				//console.debug( "Using server generated text co-ordinates for " +  dictionary.count + " items.");
			}
			
			if ( dictionary  && dictionary.containsKey( text ) ){
				var bounds = dictionary.item( text );
				//console.debug( "GfxExt - Cache hit for " + text + " as " + dojo.toJson(bounds) );
				return bounds;
			}

			sz = this.__getTextBBox(text);
			//console.debug( "GfxExt - Translate text " + text + " as " + dojo.toJson(sz) );
			
			return sz;
		},
		
		__getTextBBox: function(s) {
			if (!s || s.length == 0) 
				return null;
			if (iid.isSvg && !iid.isSvgWeb) {
				// try/catch the FF native getBBox error. cheaper than walking up in the DOM
				// hierarchy to check the conditions (bench show /10 )
				try {
					return this.rawNode.getBBox();
				} catch (e) {
					return null;
				}
			} else if (iid.isVml) {
				// mandatory fix or IE crash in the vgx.dll (vml impl.)
				if (_isOrphan(this)) {
					return null;
				}
				var rawNode = this.rawNode, _display = rawNode.style.display;
				rawNode.style.display = "inline";
				
				// [av] added ||0 to avoid the IE crashes.
				// TODO remove ||0 and solve NaNs from "auto" settings in IE properly
				var w = dojox.gfx.pt2px(parseFloat(rawNode.currentStyle.width)) || 0;
				var h = dojox.gfx.pt2px(parseFloat(rawNode.currentStyle.height)) || 0;
				var sz = {
					x: 0,
					y: 0,
					width: w,
					height: h
				};
				// in VML, the width/height we get are in view coordinates
				var m = this._getRealMatrix();
				if (m && !m.isIdentity()) {
					sz = m.inverse().transformRectangle(sz);
				}
				// It's impossible to get the x/y from the currentStyle.left/top,
				// because all negative coordinates are 'clipped' to 0.
				// (x:0 + translate(-100) -> x=0
				this._computeLocation(sz);
				rawNode.style.display = _display;
				return sz;
			} else if (iid.isSilverlight) {
				var bb = {
					width: this.rawNode.actualWidth,
					height: this.rawNode.actualHeight
				};
				return this._computeLocation(bb,0.75); // 0.75: rough approx. of ascent...	
			} else if (this.getTextWidth) {
				if(iid.isSvgWeb && !this.parent)
					return null;
				var w = this.getTextWidth();
				var font = this.getFont();
				var fz = font ? font.size : dojox.gfx.defaultFont.size;
				var h = dojox.gfx.normalizedLength(fz);
				// TODO check if 0,0 is ok or needs to be corrected.
				sz = {
					width: w,
					height: h
				};
				this._computeLocation(sz, 0.85); // 0.85: rough approx. of ascent...
				return sz;
			}
		},
		
		_computeLocation: function(sz, coef) {
			var width = sz.width, height = sz.height, sh = this.getShape(), align = sh.align;
			switch (align) {
				case 'end':
					sz.x = sh.x - width;
					break;
				case 'middle':
					sz.x = sh.x - width / 2;
					break;
				case 'start':
				default:
					sz.x = sh.x;
					break;
			}
			coef = coef || 1;
			sz.y = sh.y - height*coef; // rough approximation of the ascent!...
			return sz;
		}
	});
	
	//
	// selectedStyle property
	//
	
	var SelectedStyleMixin = {
		setSelectedStyle:function(style) {
			this.selectedStyle = style;
			this.onSelectedStyleChanged();
		},
		getSelectedStyle:function() {
			return this.selectedStyle;
		},
		onSelectedStyleChanged: function() {
			var ge = iid.GraphElement.byShape(this);
			if(ge) {
				this.__selectedStyleChanged = true; // flag to force re-applying of selected style
				ge._changeSelectionStyle(ge.isSelected());
			}
		},
		__eod: undefined
	};
	dojo.extend(is16 ? dojox.gfx.shape.Shape: dojox.gfx.Shape, SelectedStyleMixin);
	
	// VML/IE8: patch for cases where surface is created outside of the DOM.
	//
	if(iid.isVml && dojo.isIE >= 8){
		var _patchMethod = function(method, field){
			var shapeProto = is16 ? dojox.gfx.shape.Shape.prototype: dojox.gfx.Shape.prototype;
			var old = shapeProto[method];
			shapeProto[method] = function(a){
				var s = this.getParent();
				while(s && s.getParent) s = s.getParent();
				if(s && _isOrphan(s)){
					this[field] = a;
					if(s){
						if(s._parent && !s._timer){
							//console.log("watching orphan surface...");
							s._timer = setTimeout(dojo.hitch(s, _timer), 100);
						}
						return this;
					} else {
						return old.call(this, a); // no surface?
					}
				} else {
					return old.call(this, a);
				}
			};
		};
		var _timer = function(){
			var s = this;
			if(!s._parent){
				//console.log("surface disposed.");
				delete s._timer;
			} else if(_isOrphan(s)){
				// still not added: continue waiting
				//console.log("surface still orphan: continue waiting...");
				s._timer = setTimeout(dojo.hitch(s, _timer), 100);
			} else {
				delete s._timer;
				//console.log("surface added: fixing");
				// remove/add surface node (don't know why exactly but hey that's VML...)
				s._parent.removeChild(s.clipNode);
				s._parent.appendChild(s.clipNode);
				// reset all shapes and styles on all shapes
				dojox.gfx.utils.forEach(s, function(shape){
					if(shape != s){
						if(shape.setShape) shape.setShape(shape.shape);
						if(shape.setFill) shape.setFill(shape.fillStyle);
						if(shape.setStroke) shape.setStroke(shape.strokeStyle);
					}
				}, s);
			}
		};
		_patchMethod("setShape", "shape");
		_patchMethod("setFill", "fillStyle");
		_patchMethod("setStroke", "strokeStyle");
	}
		
})();

}

if(!dojo._hasResource['ibm_ilog.diagram._base.diagext']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram._base.diagext'] = true;
dojo.provide('ibm_ilog.diagram._base.diagext');

if(dojo.config.diagramForMobile === undefined){
	// If dojo.config.diagramForMobile is not defined explicitly,
	// try to detect automatically if we are on mobile.
	// Note that this auto-detection can always be disabled by setting
	// dojo.config.diagramForMobile explicitly to either true or false.
	if(dojox.mobile && dojox.mobile.View){
		// If we are using dojox.mobile then we are probably on mobile.
		dojo.config.diagramForMobile = true;
	} else {
		// Test some well-known user agent names:
		var nua = navigator.userAgent.toLowerCase();
		if((nua.indexOf('iphone') >= 0) ||
		   (nua.indexOf('ipod') >= 0) ||
		   (nua.indexOf('ipad') >= 0) ||
		   (nua.indexOf('android') >= 0) ||
		   (nua.indexOf('blackberry') >= 0)){
			dojo.config.diagramForMobile = true;
		}
	}
}
dojo.config._notDiagramForMobile = !dojo.config.diagramForMobile;





//
// TransformChain: intended to be used on Node/Link via dojo.extend
//

ibm_ilog.diagram.TransformChain = {
	// summary:
	//		Provides additional methods on GFX shapes to compute transforms.
	// tags:
	//		private

    getTransformToContainer: function(/*dojox.gfx.Group*/container) {
        // summary:
        //        Gets the transform from the parent coordinate system
        //        to the specified container coordinate system.
        // container: dojox.gfx.Group:
        //        the target container.
        
        if (container === null) {
            //TODO: ErrorReporter
            throw {
                'name': 'ArgumentNullException',
                message: "Null parameter: container"
            };
        }
        var m = dojox.gfx.matrix;
        if (container === this) {
            return m.identity;
        }
        var t1 = m.identity;
        // does not take into account shape transform
        var p = this.getParent();
        // TODO: no need to go up til the surface. Should stop to 
        // 1st common ancestor. check perf gain ?
        if (p !== null) {
            t1 = p._getRealMatrix() || m.identity;
        }
        var t2 = container._getRealMatrix() || m.identity;
        var t = m.multiply(t2.inverse(), t1);
        return t;
    },
    
    getShapeToContainerTransform: function(container) {
        // summary:
        //        Gets the transform from this shape coordinate system
        //        to the specified container coordinate system.
        // description:
        //        This method takes into account the shape transform.
        // container: dojox.gfx.Group:
        //        the target container.
        if (container === null) {
            //TODO: ErrorReporter
            throw {
                'name': 'ArgumentNullException',
                message: "Null parameter: container"
            };
        }
        var m = dojox.gfx.matrix;
        if (container === this) {
            return m.identity;
        }
        // TODO: no need to go up til the surface. Should stop to 
        // 1st common ancestor. check perf gain ? 
        // t1: transform from surface -> shape
        var t1 = this._getRealMatrix() || m.identity;
        // t2: transform from surface -> container
        var t2 = container._getRealMatrix() || m.identity;
        return m.multiply(t2.inverse(), t1);
    }
};
dojo.extend(dojo.version.major > 1 || (dojo.version.major == 1 && dojo.version.minor >= 6) ? dojox.gfx.shape.Shape : dojox.gfx.Shape, ibm_ilog.diagram.TransformChain);

//
// BoundingBox fix
//

(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var eq = g.equalRect;

	
ibm_ilog.diagram.BoundingBoxFix = {
	// summary:
	//		Fixes/provides additional methods on GFX groups for bounding box computations.
	// tags:
	//		private
	
    _computeBoundingBox: function() {
    	var bb = null;
    	var cs = this.children;
    	var ncs = this.children.length;
    	var c;
        for (var i = 0; i < ncs; ++i) {
        	var c = cs[i];
            var cbb = c.getBoundingBox();
            if (!cbb) {
            	continue;
            }
            var ct = c.getTransform();
            if (ct && !ct.isIdentity()) {
                cbb = ct.transformRectangle(cbb);
            }
            bb = bb ? g.addRect(bb, cbb) : cbb;
        }

    	this._bbCache = bb;
        if (bb) {
            var t = this.getTransform() || dojox.gfx.matrix.identity;
            if (t) 
                bb = t.transformRectangle(bb);
        }
    	this._lbbCache = bb;
    },

    _prepareBoundingBoxCache: function() {
		var oldbb = this._bbCache;
		var oldlbb = this._lbbCache;
		
		if(!oldbb || !this._bbCacheValid) {
	    	this._computeBoundingBox();
	    	this._bbCacheValid = true;
		} 
	},
	
	getBoundingBox: function() {
        // summary:
        //        Gets the bounding box of this dojox.gfx.Group. Returns null if the group is empty.
        // description:
        //        This method returns the global children bounding box without
        //        taken into account the transform of this Group.
		this._prepareBoundingBoxCache();

		return this._bbCache;
    },
    
    _onChanged: function(source,reason) {
    	this._bbCacheValid = false;
    	// 	[av]
    	// 	this method became an "inlined inherited" at:
    	//	- Node.js
    	//	- Graph.js
    },
    
    getLocalBoundingBox: function() {
        // summary:
        //        Gets the bounding box of this dojox.gfx.Group in the local coordinate system.
        // description:
        //        This method returns the global children bounding box 
        //        taken into account the transform of this Group.        
		this._prepareBoundingBoxCache();

		return this._lbbCache;
    }
};

dojo.extend(dojox.gfx.Group, ibm_ilog.diagram.BoundingBoxFix);

})();



}

if(!dojo._hasResource['ibm_ilog.diagram._base.dojoext']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram._base.dojoext'] = true;
dojo.provide('ibm_ilog.diagram._base.dojoext');

(function() {
	
	// Dojo patches and cross-version helper functions

	
	
	var dv = dojo.version;
	var de = ibm_ilog.diagram._base.dojoext;
	
	if( dv.major<=1 && dv.minor<5 ) {
			// TODO remove the last condition -- IBM Dojo 1.5 bug?
		de.SetAttrName = "attr";
		de.GetAttrName = "attr";
	} else {
		de.SetAttrName = "set";
		de.GetAttrName = "get";
	}

	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.base']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.base'] = true;
dojo.provide('ibm_ilog.diagram.base');






}

if(!dojo._hasResource["dojo.data.api.Request"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.api.Request"] = true;
dojo.provide("dojo.data.api.Request");



dojo.declare("dojo.data.api.Request", null, {
	//	summary:
	//		This class defines out the semantics of what a 'Request' object looks like
	//		when returned from a fetch() method.  In general, a request object is
	//		nothing more than the original keywordArgs from fetch with an abort function
	//		attached to it to allow users to abort a particular request if they so choose.
	//		No other functions are required on a general Request object return.  That does not
	//		inhibit other store implementations from adding extentions to it, of course.
	//
	//		This is an abstract API that data provider implementations conform to.
	//		This file defines methods signatures and intentionally leaves all the
	//		methods unimplemented.
	//
	//		For more details on fetch, see dojo.data.api.Read.fetch().

	abort: function(){
		//	summary:
		//		This function is a hook point for stores to provide as a way for
		//		a fetch to be halted mid-processing.
		//	description:
		//		This function is a hook point for stores to provide as a way for
		//		a fetch to be halted mid-processing.  For more details on the fetch() api,
		//		please see dojo.data.api.Read.fetch().
		throw new Error('Unimplemented API: dojo.data.api.Request.abort');
	}
});

}

if(!dojo._hasResource["dojo.data.api.Read"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.api.Read"] = true;
dojo.provide("dojo.data.api.Read");




dojo.declare("dojo.data.api.Read", null, {
	//	summary:
	//		This is an abstract API that data provider implementations conform to.
	//		This file defines methods signatures and intentionally leaves all the
	//		methods unimplemented.  For more information on the dojo.data APIs,
	//		please visit: http://www.dojotoolkit.org/node/98

	getValue: function(	/* item */ item,
						/* attribute-name-string */ attribute,
						/* value? */ defaultValue){
		//	summary:
		//		Returns a single attribute value.
		//		Returns defaultValue if and only if *item* does not have a value for *attribute*.
		//		Returns null if and only if null was explicitly set as the attribute value.
		//		Returns undefined if and only if the item does not have a value for the
		//		given attribute (which is the same as saying the item does not have the attribute).
		// description:
		//		Saying that an "item x does not have a value for an attribute y"
		//		is identical to saying that an "item x does not have attribute y".
		//		It is an oxymoron to say "that attribute is present but has no values"
		//		or "the item has that attribute but does not have any attribute values".
		//		If store.hasAttribute(item, attribute) returns false, then
		//		store.getValue(item, attribute) will return undefined.
		//
		//	item:
		//		The item to access values on.
		//	attribute:
		//		The attribute to access represented as a string.
		//	defaultValue:
		//		Optional.  A default value to use for the getValue return in the attribute does not exist or has no value.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		//	example:
		//	|	var darthVader = store.getValue(lukeSkywalker, "father");
		var attributeValue = null;
		throw new Error('Unimplemented API: dojo.data.api.Read.getValue');
		return attributeValue; // a literal, an item, null, or undefined (never an array)
	},

	getValues: function(/* item */ item,
						/* attribute-name-string */ attribute){
		//	summary:
		// 		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain
		//		many attribute values.
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		has a return of false, then store.getValues(item, attribute) will return [].)
		//
		//	item:
		//		The item to access values on.
		//	attribute:
		//		The attribute to access represented as a string.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		//	example:
		//	|	var friendsOfLuke = store.getValues(lukeSkywalker, "friends");
		var array = [];
		throw new Error('Unimplemented API: dojo.data.api.Read.getValues');
		return array; // an array that may contain literals and items
	},

	getAttributes: function(/* item */ item){
		//	summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		//
		//	item:
		//		The item to access attributes on.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		//	example:
		//	|	var array = store.getAttributes(kermit);
		var array = [];
		throw new Error('Unimplemented API: dojo.data.api.Read.getAttributes');
		return array; // array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute){
		//	summary:
		//		Returns true if the given *item* has a value for the given *attribute*.
		//
		//	item:
		//		The item to access attributes on.
		//	attribute:
		//		The attribute to access represented as a string.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		//	example:
		//	|	var trueOrFalse = store.hasAttribute(kermit, "color");
		throw new Error('Unimplemented API: dojo.data.api.Read.hasAttribute');
		return false; // boolean
	},

	containsValue: function(/* item */ item,
							/* attribute-name-string */ attribute,
							/* anything */ value){
		//	summary:
		//		Returns true if the given *value* is one of the values that getValues()
		//		would return.
		//
		//	item:
		//		The item to access values on.
		//	attribute:
		//		The attribute to access represented as a string.
		//	value:
		//		The value to match as a value for the attribute.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		//	example:
		//	|	var trueOrFalse = store.containsValue(kermit, "color", "green");
		throw new Error('Unimplemented API: dojo.data.api.Read.containsValue');
		return false; // boolean
	},

	isItem: function(/* anything */ something){
		//	summary:
		//		Returns true if *something* is an item and came from the store instance.
		//		Returns false if *something* is a literal, an item from another store instance,
		//		or is any object other than an item.
		//
		//	something:
		//		Can be anything.
		//
		//	example:
		//	|	var yes = store.isItem(store.newItem());
		//	|	var no  = store.isItem("green");
		throw new Error('Unimplemented API: dojo.data.api.Read.isItem');
		return false; // boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary:
		//		Returns false if isItem(something) is false.  Returns false if
		//		if isItem(something) is true but the the item is not yet loaded
		//		in local memory (for example, if the item has not yet been read
		//		from the server).
		//
		//	something:
		//		Can be anything.
		//
		//	example:
		//	|	var yes = store.isItemLoaded(store.newItem());
		//	|	var no  = store.isItemLoaded("green");
		throw new Error('Unimplemented API: dojo.data.api.Read.isItemLoaded');
		return false; // boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary:
		//		Given an item, this method loads the item so that a subsequent call
		//		to store.isItemLoaded(item) will return true.  If a call to
		//		isItemLoaded() returns true before loadItem() is even called,
		//		then loadItem() need not do any work at all and will not even invoke
		//		the callback handlers.  So, before invoking this method, check that
		//		the item has not already been loaded.
		// 	keywordArgs:
		//		An anonymous object that defines the item to load and callbacks to invoke when the
		//		load has completed.  The format of the object is as follows:
		//		{
		//			item: object,
		//			onItem: Function,
		//			onError: Function,
		//			scope: object
		//		}
		//	The *item* parameter.
		//		The item parameter is an object that represents the item in question that should be
		//		contained by the store.  This attribute is required.
		
		//	The *onItem* parameter.
		//		Function(item)
		//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
		//		parameter, the fully loaded item.
		//
		//	The *onError* parameter.
		//		Function(error)
		//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
		//		parameter, the error object
		//
		//	The *scope* parameter.
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
		//		In the body of the callback function, the value of the "this"
		//		keyword will be the scope object.   If no scope object is provided,
		//		the callback functions will be called in the context of dojo.global().
		//		For example, onItem.call(scope, item, request) vs.
		//		onItem.call(dojo.global(), item, request)
		if(!this.isItemLoaded(keywordArgs.item)){
			throw new Error('Unimplemented API: dojo.data.api.Read.loadItem');
		}
	},

	fetch: function(/* Object */ keywordArgs){
		//	summary:
		//		Given a query and set of defined options, such as a start and count of items to return,
		//		this method executes the query and makes the results available as data items.
		//		The format and expectations of stores is that they operate in a generally asynchronous
		//		manner, therefore callbacks are always used to return items located by the fetch parameters.
		//
		//	description:
		//		A Request object will always be returned and is returned immediately.
		//		The basic request is nothing more than the keyword args passed to fetch and
		//		an additional function attached, abort().  The returned request object may then be used
		//		to cancel a fetch.  All data items returns are passed through the callbacks defined in the
		//		fetch parameters and are not present on the 'request' object.
		//
		//		This does not mean that custom stores can not add methods and properties to the request object
		//		returned, only that the API does not require it.  For more info about the Request API,
		//		see dojo.data.api.Request
		//
		//	keywordArgs:
		//		The keywordArgs parameter may either be an instance of
		//		conforming to dojo.data.api.Request or may be a simple anonymous object
		//		that may contain any of the following:
		//		{
		//			query: query-object or query-string,
		//			queryOptions: object,
		//			onBegin: Function,
		//			onItem: Function,
		//			onComplete: Function,
		//			onError: Function,
		//			scope: object,
		//			start: int
		//			count: int
		//			sort: array
		//		}
		//		All implementations should accept keywordArgs objects with any of
		//		the 9 standard properties: query, onBegin, onItem, onComplete, onError
		//		scope, sort, start, and count.  Some implementations may accept additional
		//		properties in the keywordArgs object as valid parameters, such as
		//		{includeOutliers:true}.
		//
		//	The *query* parameter.
		//		The query may be optional in some data store implementations.
		//		The dojo.data.api.Read API does not specify the syntax or semantics
		//		of the query itself -- each different data store implementation
		//		may have its own notion of what a query should look like.
		//		However, as of dojo 0.9, 1.0, and 1.1, all the provided datastores in dojo.data
		//		and dojox.data support an object structure query, where the object is a set of
		//		name/value parameters such as { attrFoo: valueBar, attrFoo1: valueBar1}.  Most of the
		//		dijit widgets, such as ComboBox assume this to be the case when working with a datastore
		//		when they dynamically update the query.  Therefore, for maximum compatibility with dijit
		//		widgets the recommended query parameter is a key/value object.  That does not mean that the
		//		the datastore may not take alternative query forms, such as a simple string, a Date, a number,
		//		or a mix of such.  Ultimately, The dojo.data.api.Read API is agnostic about what the query
		//		format.
		//		Further note:  In general for query objects that accept strings as attribute
		//		value matches, the store should also support basic filtering capability, such as *
		//		(match any character) and ? (match single character).  An example query that is a query object
		//		would be like: { attrFoo: "value*"}.  Which generally means match all items where they have
		//		an attribute named attrFoo, with a value that starts with 'value'.
		//
		//	The *queryOptions* parameter
		//		The queryOptions parameter is an optional parameter used to specify optiosn that may modify
		//		the query in some fashion, such as doing a case insensitive search, or doing a deep search
		//		where all items in a hierarchical representation of data are scanned instead of just the root
		//		items.  It currently defines two options that all datastores should attempt to honor if possible:
		//		{
		//			ignoreCase: boolean, //Whether or not the query should match case sensitively or not.  Default behaviour is false.
		//			deep: boolean 	//Whether or not a fetch should do a deep search of items and all child
		//							//items instead of just root-level items in a datastore.  Default is false.
		//		}
		//
		//	The *onBegin* parameter.
		//		function(size, request);
		//		If an onBegin callback function is provided, the callback function
		//		will be called just once, before the first onItem callback is called.
		//		The onBegin callback function will be passed two arguments, the
		//		the total number of items identified and the Request object.  If the total number is
		//		unknown, then size will be -1.  Note that size is not necessarily the size of the
		//		collection of items returned from the query, as the request may have specified to return only a
		//		subset of the total set of items through the use of the start and count parameters.
		//
		//	The *onItem* parameter.
		//		function(item, request);
		//		If an onItem callback function is provided, the callback function
		//		will be called as each item in the result is received. The callback
		//		function will be passed two arguments: the item itself, and the
		//		Request object.
		//
		//	The *onComplete* parameter.
		//		function(items, request);
		//
		//		If an onComplete callback function is provided, the callback function
		//		will be called just once, after the last onItem callback is called.
		//		Note that if the onItem callback is not present, then onComplete will be passed
		//		an array containing all items which matched the query and the request object.
		//		If the onItem callback is present, then onComplete is called as:
		//		onComplete(null, request).
		//
		//	The *onError* parameter.
		//		function(errorData, request);
		//		If an onError callback function is provided, the callback function
		//		will be called if there is any sort of error while attempting to
		//		execute the query.
		//		The onError callback function will be passed two arguments:
		//		an Error object and the Request object.
		//
		//	The *scope* parameter.
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onComplete, onError, etc) will be invoked in the context of the scope
		//		object.  In the body of the callback function, the value of the "this"
		//		keyword will be the scope object.   If no scope object is provided,
		//		the callback functions will be called in the context of dojo.global().
		//		For example, onItem.call(scope, item, request) vs.
		//		onItem.call(dojo.global(), item, request)
		//
		//	The *start* parameter.
		//		If a start parameter is specified, this is a indication to the datastore to
		//		only start returning items once the start number of items have been located and
		//		skipped.  When this parameter is paired withh 'count', the store should be able
		//		to page across queries with millions of hits by only returning subsets of the
		//		hits for each query
		//
		//	The *count* parameter.
		//		If a count parameter is specified, this is a indication to the datastore to
		//		only return up to that many items.  This allows a fetch call that may have
		//		millions of item matches to be paired down to something reasonable.
		//
		//	The *sort* parameter.
		//		If a sort parameter is specified, this is a indication to the datastore to
		//		sort the items in some manner before returning the items.  The array is an array of
		//		javascript objects that must conform to the following format to be applied to the
		//		fetching of items:
		//		{
		//			attribute: attribute || attribute-name-string,
		//			descending: true|false;   // Optional.  Default is false.
		//		}
		//		Note that when comparing attributes, if an item contains no value for the attribute
		//		(undefined), then it the default ascending sort logic should push it to the bottom
		//		of the list.  In the descending order case, it such items should appear at the top of the list.
		//
		//	returns:
		//		The fetch() method will return a javascript object conforming to the API
		//		defined in dojo.data.api.Request.  In general, it will be the keywordArgs
		//		object returned with the required functions in Request.js attached.
		//		Its general purpose is to provide a convenient way for a caller to abort an
		//		ongoing fetch.
		//
		//		The Request object may also have additional properties when it is returned
		//		such as request.store property, which is a pointer to the datastore object that
		//		fetch() is a method of.
		//
		//	exceptions:
		//		Throws an exception if the query is not valid, or if the query
		//		is required but was not supplied.
		//
		//	example:
		//		Fetch all books identified by the query and call 'showBooks' when complete
		//		|	var request = store.fetch({query:"all books", onComplete: showBooks});
		//	example:
		//		Fetch all items in the story and call 'showEverything' when complete.
		//		|	var request = store.fetch(onComplete: showEverything);
		//	example:
		//		Fetch only 10 books that match the query 'all books', starting at the fifth book found during the search.
		//		This demonstrates how paging can be done for specific queries.
		//		|	var request = store.fetch({query:"all books", start: 4, count: 10, onComplete: showBooks});
		//	example:
		//		Fetch all items that match the query, calling 'callback' each time an item is located.
		//		|	var request = store.fetch({query:"foo/bar", onItem:callback});
		//	example:
		//		Fetch the first 100 books by author King, call showKing when up to 100 items have been located.
		//		|	var request = store.fetch({query:{author:"King"}, start: 0, count:100, onComplete: showKing});
		//	example:
		//		Locate the books written by Author King, sort it on title and publisher, then return the first 100 items from the sorted items.
		//		|	var request = store.fetch({query:{author:"King"}, sort: [{ attribute: "title", descending: true}, {attribute: "publisher"}], ,start: 0, count:100, onComplete: 'showKing'});
		//	example:
		//		Fetch the first 100 books by authors starting with the name King, then call showKing when up to 100 items have been located.
		//		|	var request = store.fetch({query:{author:"King*"}, start: 0, count:100, onComplete: showKing});
		//	example:
		//		Fetch the first 100 books by authors ending with 'ing', but only have one character before it (King, Bing, Ling, Sing, etc.), then call showBooks when up to 100 items have been located.
		//		|	var request = store.fetch({query:{author:"?ing"}, start: 0, count:100, onComplete: showBooks});
		//	example:
		//		Fetch the first 100 books by author King, where the name may appear as King, king, KING, kInG, and so on, then call showKing when up to 100 items have been located.
		//		|	var request = store.fetch({query:{author:"King"}, queryOptions:(ignoreCase: true}, start: 0, count:100, onComplete: showKing});
		//	example:
		//		Paging
		//		|	var store = new dojo.data.LargeRdbmsStore({url:"jdbc:odbc:foobar"});
		//		|	var fetchArgs = {
		//		|		query: {type:"employees", name:"Hillary *"}, // string matching
		//		|		sort: [{attribute:"department", descending:true}],
		//		|		start: 0,
		//		|		count: 20,
		//		|		scope: displayer,
		//		|		onBegin: showThrobber,
		//		|		onItem: displayItem,
		//		|		onComplete: stopThrobber,
		//		|		onError: handleFetchError,
		//		|	};
		//		|	store.fetch(fetchArgs);
		//		|	...
		//		and then when the user presses the "Next Page" button...
		//		|	fetchArgs.start += 20;
		//		|	store.fetch(fetchArgs);  // get the next 20 items
		var request = null;
		throw new Error('Unimplemented API: dojo.data.api.Read.fetch');
		return request; // an object conforming to the dojo.data.api.Request API
	},

	getFeatures: function(){
		//	summary:
		//		The getFeatures() method returns an simple keyword values object
		//		that specifies what interface features the datastore implements.
		//		A simple CsvStore may be read-only, and the only feature it
		//		implements will be the 'dojo.data.api.Read' interface, so the
		//		getFeatures() method will return an object like this one:
		//		{'dojo.data.api.Read': true}.
		//		A more sophisticated datastore might implement a variety of
		//		interface features, like 'dojo.data.api.Read', 'dojo.data.api.Write',
		//		'dojo.data.api.Identity', and 'dojo.data.api.Attribution'.
		return {
			'dojo.data.api.Read': true
		};
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		//	summary:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.
		//
		//	description:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.  In general, this API
		//		expects to recieve as a parameter a request object returned from a fetch.
		//		It will then close out anything associated with that request, such as
		//		clearing any internal datastore caches and closing any 'open' connections.
		//		For some store implementations, this call may be a no-op.
		//
		//	request:
		//		An instance of a request for the store to use to identify what to close out.
		//		If no request is passed, then the store should clear all internal caches (if any)
		//		and close out all 'open' connections.  It does not render the store unusable from
		//		there on, it merely cleans out any current data and resets the store to initial
		//		state.
		//
		//	example:
		//	|	var request = store.fetch({onComplete: doSomething});
		//	|	...
		//	|	store.close(request);
		throw new Error('Unimplemented API: dojo.data.api.Read.close');
	},

	getLabel: function(/* item */ item){
		//	summary:
		//		Method to inspect the item and return a user-readable 'label' for the item
		//		that provides a general/adequate description of what the item is.
		//
		//	description:
		//		Method to inspect the item and return a user-readable 'label' for the item
		//		that provides a general/adequate description of what the item is.  In general
		//		most labels will be a specific attribute value or collection of the attribute
		//		values that combine to label the item in some manner.  For example for an item
		//		that represents a person it may return the label as:  "firstname lastlame" where
		//		the firstname and lastname are attributes on the item.  If the store is unable
		//		to determine an adequate human readable label, it should return undefined.  Users that wish
		//		to customize how a store instance labels items should replace the getLabel() function on
		//		their instance of the store, or extend the store and replace the function in
		//		the extension class.
		//
		//	item:
		//		The item to return the label for.
		//
		//	returns:
		//		A user-readable string representing the item or undefined if no user-readable label can
		//		be generated.
		throw new Error('Unimplemented API: dojo.data.api.Read.getLabel');
		return undefined;
	},

	getLabelAttributes: function(/* item */ item){
		//	summary:
		//		Method to inspect the item and return an array of what attributes of the item were used
		//		to generate its label, if any.
		//
		//	description:
		//		Method to inspect the item and return an array of what attributes of the item were used
		//		to generate its label, if any.  This function is to assist UI developers in knowing what
		//		attributes can be ignored out of the attributes an item has when displaying it, in cases
		//		where the UI is using the label as an overall identifer should they wish to hide
		//		redundant information.
		//
		//	item:
		//		The item to return the list of label attributes for.
		//
		//	returns:
		//		An array of attribute names that were used to generate the label, or null if public attributes
		//		were not used to generate the label.
		throw new Error('Unimplemented API: dojo.data.api.Read.getLabelAttributes');
		return null;
	}
});

}

if(!dojo._hasResource["dojo.data.api.Write"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.api.Write"] = true;
dojo.provide("dojo.data.api.Write");




dojo.declare("dojo.data.api.Write", dojo.data.api.Read, {
	//	summary:
	//		This is an abstract API that data provider implementations conform to.
	//		This file defines function signatures and intentionally leaves all the
	//		functionss unimplemented.

	getFeatures: function(){
		//	summary:
		//		See dojo.data.api.Read.getFeatures()
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Write': true
		};
	},

	newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
		//	summary:
		//		Returns a newly created item.  Sets the attributes of the new
		//		item based on the *keywordArgs* provided.  In general, the attribute
		//		names in the keywords become the attributes in the new item and as for
		//		the attribute values in keywordArgs, they become the values of the attributes
		//		in the new item.  In addition, for stores that support hierarchical item
		//		creation, an optional second parameter is accepted that defines what item is the parent
		//		of the new item and what attribute of that item should the new item be assigned to.
		//		In general, this will assume that the attribute targetted is multi-valued and a new item
		//		is appended onto the list of values for that attribute.
		//
		//	keywordArgs:
		//		A javascript object defining the initial content of the item as a set of JavaScript 'property name: value' pairs.
		//	parentInfo:
		//		An optional javascript object defining what item is the parent of this item (in a hierarchical store.  Not all stores do hierarchical items),
		//		and what attribute of that parent to assign the new item to.  If this is present, and the attribute specified
		//		is a multi-valued attribute, it will append this item into the array of values for that attribute.  The structure
		//		of the object is as follows:
		//		{
		//			parent: someItem,
		//			attribute: "attribute-name-string"
		//		}
		//
		//	exceptions:
		//		Throws an exception if *keywordArgs* is a string or a number or
		//		anything other than a simple anonymous object.
		//		Throws an exception if the item in parentInfo is not an item from the store
		//		or if the attribute isn't an attribute name string.
		//	example:
		//	|	var kermit = store.newItem({name: "Kermit", color:[blue, green]});

		var newItem;
		throw new Error('Unimplemented API: dojo.data.api.Write.newItem');
		return newItem; // item
	},

	deleteItem: function(/* item */ item){
		//	summary:
		//		Deletes an item from the store.
		//
		//	item:
		//		The item to delete.
		//
		//	exceptions:
		//		Throws an exception if the argument *item* is not an item
		//		(if store.isItem(item) returns false).
		//	example:
		//	|	var success = store.deleteItem(kermit);
		throw new Error('Unimplemented API: dojo.data.api.Write.deleteItem');
		return false; // boolean
	},

	setValue: function(	/* item */ item,
						/* string */ attribute,
						/* almost anything */ value){
		//	summary:
		//		Sets the value of an attribute on an item.
		//		Replaces any previous value or values.
		//
		//	item:
		//		The item to modify.
		//	attribute:
		//		The attribute of the item to change represented as a string name.
		//	value:
		//		The value to assign to the item.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or if *attribute*
		//		is neither an attribute object or a string.
		//		Throws an exception if *value* is undefined.
		//	example:
		//	|	var success = store.set(kermit, "color", "green");
		throw new Error('Unimplemented API: dojo.data.api.Write.setValue');
		return false; // boolean
	},

	setValues: function(/* item */ item,
						/* string */ attribute,
						/* array */ values){
		//	summary:
		//		Adds each value in the *values* array as a value of the given
		//		attribute on the given item.
		//		Replaces any previous value or values.
		//		Calling store.setValues(x, y, []) (with *values* as an empty array) has
		//		the same effect as calling store.unsetAttribute(x, y).
		//
		//	item:
		//		The item to modify.
		//	attribute:
		//		The attribute of the item to change represented as a string name.
		//	values:
		//		An array of values to assign to the attribute..
		//
		//	exceptions:
		//		Throws an exception if *values* is not an array, if *item* is not an
		//		item, or if *attribute* is neither an attribute object or a string.
		//	example:
		//	|	var success = store.setValues(kermit, "color", ["green", "aqua"]);
		//	|	success = store.setValues(kermit, "color", []);
		//	|	if (success) {assert(!store.hasAttribute(kermit, "color"));}
		throw new Error('Unimplemented API: dojo.data.api.Write.setValues');
		return false; // boolean
	},

	unsetAttribute: function(	/* item */ item,
								/* string */ attribute){
		//	summary:
		//		Deletes all the values of an attribute on an item.
		//
		//	item:
		//		The item to modify.
		//	attribute:
		//		The attribute of the item to unset represented as a string.
		//
		//	exceptions:
		//		Throws an exception if *item* is not an item, or if *attribute*
		//		is neither an attribute object or a string.
		//	example:
		//	|	var success = store.unsetAttribute(kermit, "color");
		//	|	if (success) {assert(!store.hasAttribute(kermit, "color"));}
		throw new Error('Unimplemented API: dojo.data.api.Write.clear');
		return false; // boolean
	},

	save: function(/* object */ keywordArgs){
		//	summary:
		//		Saves to the server all the changes that have been made locally.
		//		The save operation may take some time and is generally performed
		//		in an asynchronous fashion.  The outcome of the save action is
		//		is passed into the set of supported callbacks for the save.
		//
		//	keywordArgs:
		//		{
		//			onComplete: function
		//			onError: function
		//			scope: object
		//		}
		//
		//	The *onComplete* parameter.
		//		function();
		//
		//		If an onComplete callback function is provided, the callback function
		//		will be called just once, after the save has completed.  No parameters
		//		are generally passed to the onComplete.
		//
		//	The *onError* parameter.
		//		function(errorData);
		//
		//		If an onError callback function is provided, the callback function
		//		will be called if there is any sort of error while attempting to
		//		execute the save.  The onError function will be based one parameter, the
		//		error.
		//
		//	The *scope* parameter.
		//		If a scope object is provided, all of the callback function (
		//		onComplete, onError, etc) will be invoked in the context of the scope
		//		object.  In the body of the callback function, the value of the "this"
		//		keyword will be the scope object.   If no scope object is provided,
		//		the callback functions will be called in the context of dojo.global.
		//		For example, onComplete.call(scope) vs.
		//		onComplete.call(dojo.global)
		//
		//	returns:
		//		Nothing.  Since the saves are generally asynchronous, there is
		//		no need to return anything.  All results are passed via callbacks.
		//	example:
		//	|	store.save({onComplete: onSave});
		//	|	store.save({scope: fooObj, onComplete: onSave, onError: saveFailed});
		throw new Error('Unimplemented API: dojo.data.api.Write.save');
	},

	revert: function(){
		//	summary:
		//		Discards any unsaved changes.
		//	description:
		//		Discards any unsaved changes.
		//
		//	example:
		//	|	var success = store.revert();
		throw new Error('Unimplemented API: dojo.data.api.Write.revert');
		return false; // boolean
	},

	isDirty: function(/* item? */ item){
		//	summary:
		//		Given an item, isDirty() returns true if the item has been modified
		//		since the last save().  If isDirty() is called with no *item* argument,
		//		then this function returns true if any item has been modified since
		//		the last save().
		//
		//	item:
		//		The item to check.
		//
		//	exceptions:
		//		Throws an exception if isDirty() is passed an argument and the
		//		argument is not an item.
		//	example:
		//	|	var trueOrFalse = store.isDirty(kermit); // true if kermit is dirty
		//	|	var trueOrFalse = store.isDirty();       // true if any item is dirty
		throw new Error('Unimplemented API: dojo.data.api.Write.isDirty');
		return false; // boolean
	}
});

}

if(!dojo._hasResource["dojo.data.api.Identity"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.api.Identity"] = true;
dojo.provide("dojo.data.api.Identity");




dojo.declare("dojo.data.api.Identity", dojo.data.api.Read, {
	//	summary:
	//		This is an abstract API that data provider implementations conform to.
	//		This file defines methods signatures and intentionally leaves all the
	//		methods unimplemented.

	getFeatures: function(){
		//	summary:
		//		See dojo.data.api.Read.getFeatures()
		return {
			 'dojo.data.api.Read': true,
			 'dojo.data.api.Identity': true
		};
	},

	getIdentity: function(/* item */ item){
		//	summary:
		//		Returns a unique identifier for an item.  The return value will be
		//		either a string or something that has a toString() method (such as,
		//		for example, a dojox.uuid.Uuid object).
		//	item:
		//		The item from the store from which to obtain its identifier.
		//	exceptions:
		//		Conforming implementations may throw an exception or return null if
		//		item is not an item.
		//	example:
		//	|	var itemId = store.getIdentity(kermit);
		//	|	assert(kermit === store.findByIdentity(store.getIdentity(kermit)));
		throw new Error('Unimplemented API: dojo.data.api.Identity.getIdentity');
		var itemIdentityString = null;
		return itemIdentityString; // string
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary:
		//		Returns an array of attribute names that are used to generate the identity.
		//		For most stores, this is a single attribute, but for some complex stores
		//		such as RDB backed stores that use compound (multi-attribute) identifiers
		//		it can be more than one.  If the identity is not composed of attributes
		//		on the item, it will return null.  This function is intended to identify
		//		the attributes that comprise the identity so that so that during a render
		//		of all attributes, the UI can hide the the identity information if it
		//		chooses.
		//	item:
		//		The item from the store from which to obtain the array of public attributes that
		//		compose the identifier, if any.
		//	example:
		//	|	var itemId = store.getIdentity(kermit);
		//	|	var identifiers = store.getIdentityAttributes(itemId);
		//	|	assert(typeof identifiers === "array" || identifiers === null);
		throw new Error('Unimplemented API: dojo.data.api.Identity.getIdentityAttributes');
		return null; // string
	},


	fetchItemByIdentity: function(/* object */ keywordArgs){
		//	summary:
		//		Given the identity of an item, this method returns the item that has
		//		that identity through the onItem callback.  Conforming implementations
		//		should return null if there is no item with the given identity.
		//		Implementations of fetchItemByIdentity() may sometimes return an item
		//		from a local cache and may sometimes fetch an item from a remote server,
		//
		// 	keywordArgs:
		//		An anonymous object that defines the item to locate and callbacks to invoke when the
		//		item has been located and load has completed.  The format of the object is as follows:
		//		{
		//			identity: string|object,
		//			onItem: Function,
		//			onError: Function,
		//			scope: object
		//		}
		//	The *identity* parameter.
		//		The identity parameter is the identity of the item you wish to locate and load
		//		This attribute is required.  It should be a string or an object that toString()
		//		can be called on.
		//
		//	The *onItem* parameter.
		//		Function(item)
		//		The onItem parameter is the callback to invoke when the item has been loaded.  It takes only one
		//		parameter, the item located, or null if none found.
		//
		//	The *onError* parameter.
		//		Function(error)
		//		The onError parameter is the callback to invoke when the item load encountered an error.  It takes only one
		//		parameter, the error object
		//
		//	The *scope* parameter.
		//		If a scope object is provided, all of the callback functions (onItem,
		//		onError, etc) will be invoked in the context of the scope object.
		//		In the body of the callback function, the value of the "this"
		//		keyword will be the scope object.   If no scope object is provided,
		//		the callback functions will be called in the context of dojo.global.
		//		For example, onItem.call(scope, item, request) vs.
		//		onItem.call(dojo.global, item, request)
		if(!this.isItemLoaded(keywordArgs.item)){
			throw new Error('Unimplemented API: dojo.data.api.Identity.fetchItemByIdentity');
		}
	}
});

}

if(!dojo._hasResource["dojo.data.api.Notification"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.api.Notification"] = true;
dojo.provide("dojo.data.api.Notification");




dojo.declare("dojo.data.api.Notification", dojo.data.api.Read, {
	//	summary:
	//		This is an abstract API that data provider implementations conform to.
	//		This file defines functions signatures and intentionally leaves all the
	//		functions unimplemented.
	//
	//	description:
	//		This API defines a set of APIs that all datastores that conform to the
	//		Notifications API must implement.  In general, most stores will implement
	//		these APIs as no-op functions for users who wish to monitor them to be able
	//		to connect to then via dojo.connect().  For non-users of dojo.connect,
	//		they should be able to just replace the function on the store to obtain
	//		 notifications.  Both read-only and read-write stores may implement
	//		this feature.  In the case of a read-only store, this feature makes sense if
	//		the store itself does internal polling to a back-end server and periodically updates
	//		its cache of items (deletes, adds, and updates).
	//
	//	example:
	//
	//	|	function onSet(item, attribute, oldValue, newValue) {
	//	|		//Do something with the information...
	//	|	};
	//	|	var store = new some.newStore();
	//	|	dojo.connect(store, "onSet", onSet);

	getFeatures: function(){
		//	summary:
		//		See dojo.data.api.Read.getFeatures()
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Notification': true
		};
	},

	onSet: function(/* item */ item,
					/* attribute-name-string */ attribute,
					/* object | array */ oldValue,
					/* object | array */ newValue){
		//	summary:
		//		This function is called any time an item is modified via setValue, setValues, unsetAttribute, etc.
		//	description:
		//		This function is called any time an item is modified via setValue, setValues, unsetAttribute, etc.
		//		Its purpose is to provide a hook point for those who wish to monitor actions on items in the store
		//		in a simple manner.  The general expected usage is to dojo.connect() to the store's
		//		implementation and be called after the store function is called.
		//
		//	item:
		//		The item being modified.
		//	attribute:
		//		The attribute being changed represented as a string name.
		//	oldValue:
		//		The old value of the attribute.  In the case of single value calls, such as setValue, unsetAttribute, etc,
		//		this value will be generally be an atomic value of some sort (string, int, etc, object).  In the case of
		//		multi-valued attributes, it will be an array.
		//	newValue:
		//		The new value of the attribute.  In the case of single value calls, such as setValue, this value will be
		//		generally be an atomic value of some sort (string, int, etc, object).  In the case of multi-valued attributes,
		//		it will be an array.  In the case of unsetAttribute, the new value will be 'undefined'.
		//
		//	returns:
		//		Nothing.
		throw new Error('Unimplemented API: dojo.data.api.Notification.onSet');
	},

	onNew: function(/* item */ newItem, /*object?*/ parentInfo){
		//	summary:
		//		This function is called any time a new item is created in the store.
		//		It is called immediately after the store newItem processing has completed.
		//	description:
		//		This function is called any time a new item is created in the store.
		//		It is called immediately after the store newItem processing has completed.
		//
		//	newItem:
		//		The item created.
		//	parentInfo:
		//		An optional javascript object that is passed when the item created was placed in the store
		//		hierarchy as a value f another item's attribute, instead of a root level item.  Note that if this
		//		function is invoked with a value for parentInfo, then onSet is not invoked stating the attribute of
		//		the parent item was modified.  This is to avoid getting two notification  events occurring when a new item
		//		with a parent is created.  The structure passed in is as follows:
		//		{
		//			item: someItem,							//The parent item
		//			attribute:	"attribute-name-string",	//The attribute the new item was assigned to.
		//			oldValue: something	//Whatever was the previous value for the attribute.
		//						//If it is a single-value attribute only, then this value will be a single value.
		//						//If it was a multi-valued attribute, then this will be an array of all the values minues the new one.
		//			newValue: something	//The new value of the attribute.  In the case of single value calls, such as setValue, this value will be
		//						//generally be an atomic value of some sort (string, int, etc, object).  In the case of multi-valued attributes,
		//						//it will be an array.
		//		}
		//
		//	returns:
		//		Nothing.
		throw new Error('Unimplemented API: dojo.data.api.Notification.onNew');
	},

	onDelete: function(/* item */ deletedItem){
		//	summary:
		//		This function is called any time an item is deleted from the store.
		//		It is called immediately after the store deleteItem processing has completed.
		//	description:
		//		This function is called any time an item is deleted from the store.
		//		It is called immediately after the store deleteItem processing has completed.
		//
		//	deletedItem:
		//		The item deleted.
		//
		//	returns:
		//		Nothing.
		throw new Error('Unimplemented API: dojo.data.api.Notification.onDelete');
	}
});

}

if(!dojo._hasResource["dojox.gfx.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.Mover"] = true;
dojo.provide("dojox.gfx.Mover");

dojo.declare("dojox.gfx.Mover", null, {
	constructor: function(shape, e, host){
		// summary: an object, which makes a shape follow the mouse,
		//	used as a default mover, and as a base class for custom movers
		// shape: dojox.gfx.Shape: a shape object to be moved
		// e: Event: a mouse event, which started the move;
		//	only clientX and clientY properties are used
		// host: Object?: object which implements the functionality of the move,
		//	 and defines proper events (onMoveStart and onMoveStop)
		this.shape = shape;
		this.lastX = e.clientX
		this.lastY = e.clientY;
		var h = this.host = host, d = document,
			firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "destroy"),
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo, "stopEvent"),
			dojo.connect(d, "onselectstart", dojo, "stopEvent"),
			firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary: event processor for onmousemove
		// e: Event: mouse event
		var x = e.clientX;
		var y = e.clientY;
		this.host.onMove(this, {dx: x - this.lastX, dy: y - this.lastY});
		this.lastX = x;
		this.lastY = y;
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(){
		// summary: it is meant to be called only once
		this.host.onFirstMove(this);
		dojo.disconnect(this.events.pop());
	},
	destroy: function(){
		// summary: stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.shape = null;
	}
});

}

if(!dojo._hasResource["dojox.gfx.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.Moveable"] = true;
dojo.provide("dojox.gfx.Moveable");



dojo.declare("dojox.gfx.Moveable", null, {
	constructor: function(shape, params){
		// summary: an object, which makes a shape moveable
		// shape: dojox.gfx.Shape: a shape object to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		delay: Number: delay move by this number of pixels
		//		mover: Object: a constructor of custom Mover
		this.shape = shape;
		this.delay = (params && params.delay > 0) ? params.delay : 0;
		this.mover = (params && params.mover) ? params.mover : dojox.gfx.Mover;
		this.events = [
			this.shape.connect("onmousedown", this, "onMouseDown")
			// cancel text selection and text dragging
			//, dojo.connect(this.handle, "ondragstart",   dojo, "stopEvent")
			//, dojo.connect(this.handle, "onselectstart", dojo, "stopEvent")
		];
	},

	// methods
	destroy: function(){
		// summary: stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, this.shape.disconnect, this.shape);
		this.events = this.shape = null;
	},

	// mouse event processors
	onMouseDown: function(e){
		// summary: event processor for onmousedown, creates a Mover for the shape
		// e: Event: mouse event
		if(this.delay){
			this.events.push(
				this.shape.connect("onmousemove", this, "onMouseMove"),
				this.shape.connect("onmouseup", this, "onMouseUp"));
			this._lastX = e.clientX;
			this._lastY = e.clientY;
		}else{
			new this.mover(this.shape, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary: event processor for onmousemove, used only for delayed drags
		// e: Event: mouse event
		if(Math.abs(e.clientX - this._lastX) > this.delay || Math.abs(e.clientY - this._lastY) > this.delay){
			this.onMouseUp(e);
			new this.mover(this.shape, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary: event processor for onmouseup, used only for delayed delayed drags
		// e: Event: mouse event
		this.shape.disconnect(this.events.pop());
		this.shape.disconnect(this.events.pop());
	},

	// local events
	onMoveStart: function(/* dojox.gfx.Mover */ mover){
		// summary: called before every move operation
		dojo.publish("/gfx/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove");
	},
	onMoveStop: function(/* dojox.gfx.Mover */ mover){
		// summary: called after every move operation
		dojo.publish("/gfx/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
	},
	onFirstMove: function(/* dojox.gfx.Mover */ mover){
		// summary: called during the very first move notification,
		//	can be used to initialize coordinates, can be overwritten.

		// default implementation does nothing
	},
	onMove: function(/* dojox.gfx.Mover */ mover, /* Object */ shift){
		// summary: called during every move notification,
		//	should actually move the node, can be overwritten.
		this.onMoving(mover, shift);
		this.shape.applyLeftTransform(shift);
		this.onMoved(mover, shift);
	},
	onMoving: function(/* dojox.gfx.Mover */ mover, /* Object */ shift){
		// summary: called before every incremental move,
		//	can be overwritten.

		// default implementation does nothing
	},
	onMoved: function(/* dojox.gfx.Mover */ mover, /* Object */ shift){
		// summary: called after every incremental move,
		//	can be overwritten.

		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojox.gfx.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.move"] = true;
dojo.provide("dojox.gfx.move");




}

if(!dojo._hasResource["dojox.string.Builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.string.Builder"] = true;
dojo.provide("dojox.string.Builder");

dojox.string.Builder = function(/*String?*/str){
	//	summary:
	//		A fast buffer for creating large strings.
	//
	//	length: Number
	//		The current length of the internal string.

	//	N.B. the public nature of the internal buffer is no longer
	//	needed because the IE-specific fork is no longer needed--TRT.
	var b = "";
	this.length = 0;
	
	this.append = function(/* String... */s){
		// summary: Append all arguments to the end of the buffer
		if(arguments.length>1){
			/*
				This is a loop unroll was designed specifically for Firefox;
				it would seem that static index access on an Arguments
				object is a LOT faster than doing dynamic index access.
				Therefore, we create a buffer string and take advantage
				of JS's switch fallthrough.  The peformance of this method
				comes very close to straight up string concatenation (+=).

				If the arguments object length is greater than 9, we fall
				back to standard dynamic access.

				This optimization seems to have no real effect on either
				Safari or Opera, so we just use it for all.

				It turns out also that this loop unroll can increase performance
				significantly with Internet Explorer, particularly when
				as many arguments are provided as possible.

				Loop unroll per suggestion from Kris Zyp, implemented by
				Tom Trenka.

				Note: added empty string to force a string cast if needed.
			 */
			var tmp="", l=arguments.length;
			switch(l){
				case 9: tmp=""+arguments[8]+tmp;
				case 8: tmp=""+arguments[7]+tmp;
				case 7: tmp=""+arguments[6]+tmp;
				case 6: tmp=""+arguments[5]+tmp;
				case 5: tmp=""+arguments[4]+tmp;
				case 4: tmp=""+arguments[3]+tmp;
				case 3: tmp=""+arguments[2]+tmp;
				case 2: {
					b+=""+arguments[0]+arguments[1]+tmp;
					break;
				}
				default: {
					var i=0;
					while(i<arguments.length){
						tmp += arguments[i++];
					}
					b += tmp;
				}
			}
		} else {
			b += s;
		}
		this.length = b.length;
		return this;	//	dojox.string.Builder
	};
	
	this.concat = function(/*String...*/s){
		//	summary:
		//		Alias for append.
		return this.append.apply(this, arguments);	//	dojox.string.Builder
	};
	
	this.appendArray = function(/*Array*/strings) {
		//	summary:
		//		Append an array of items to the internal buffer.

		//	Changed from String.prototype.concat.apply because of IE.
		return this.append.apply(this, strings);	//	dojox.string.Builder
	};
	
	this.clear = function(){
		//	summary:
		//		Remove all characters from the buffer.
		b = "";
		this.length = 0;
		return this;	//	dojox.string.Builder
	};
	
	this.replace = function(/* String */oldStr, /* String */ newStr){
		// 	summary:
		//		Replace instances of one string with another in the buffer.
		b = b.replace(oldStr,newStr);
		this.length = b.length;
		return this;	//	dojox.string.Builder
	};
	
	this.remove = function(/* Number */start, /* Number? */len){
		//	summary:
		//		Remove len characters starting at index start.  If len
		//		is not provided, the end of the string is assumed.
		if(len===undefined){ len = b.length; }
		if(len == 0){ return this; }
		b = b.substr(0, start) + b.substr(start+len);
		this.length = b.length;
		return this;	//	dojox.string.Builder
	};
	
	this.insert = function(/* Number */index, /* String */str){
		//	summary:
		//		Insert string str starting at index.
		if(index == 0){
			b = str + b;
		}else{
			b = b.slice(0, index) + str + b.slice(index);
		}
		this.length = b.length;
		return this;	//	dojox.string.Builder
	};
	
	this.toString = function(){
		//	summary:
		//		Return the string representation of the internal buffer.
		return b;	//	String
	};

	//	initialize the buffer.
	if(str){ this.append(str); }
};

}

if(!dojo._hasResource["dojox.string.tokenize"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.string.tokenize"] = true;
dojo.provide("dojox.string.tokenize");

dojox.string.tokenize = function(/*String*/ str, /*RegExp*/ re, /*Function?*/ parseDelim, /*Object?*/ instance){
	// summary:
	//		Split a string by a regular expression with the ability to capture the delimeters
	// parseDelim:
	//		Each group (excluding the 0 group) is passed as a parameter. If the function returns
	//		a value, it's added to the list of tokens.
	// instance:
	//		Used as the "this" instance when calling parseDelim
	var tokens = [];
	var match, content, lastIndex = 0;
	while(match = re.exec(str)){
		content = str.slice(lastIndex, re.lastIndex - match[0].length);
		if(content.length){
			tokens.push(content);
		}
		if(parseDelim){
			if(dojo.isOpera){
				var copy = match.slice(0);
				while(copy.length < match.length){
					copy.push(null);
				}
				match = copy;
			}
			var parsed = parseDelim.apply(instance, match.slice(1).concat(tokens.length));
			if(typeof parsed != "undefined"){
				tokens.push(parsed);
			}
		}
		lastIndex = re.lastIndex;
	}
	content = str.slice(lastIndex);
	if(content.length){
		tokens.push(content);
	}
	return tokens;
}

}

if(!dojo._hasResource["dojox.dtl._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl._base"] = true;
dojo.provide("dojox.dtl._base");




dojo.experimental("dojox.dtl");

(function(){
	var dd = dojox.dtl;

	dd.TOKEN_BLOCK = -1;
	dd.TOKEN_VAR = -2;
	dd.TOKEN_COMMENT = -3;
	dd.TOKEN_TEXT = 3;

	dd._Context = dojo.extend(function(dict){
		// summary: Pass one of these when rendering a template to tell the template what values to use.
		if(dict){
			dojo._mixin(this, dict);
			if(dict.get){
				// Preserve passed getter and restore prototype get
				this._getter = dict.get;
				delete this.get;
			}
		}
	},
	{
		push: function(){
			var last = this;
			var context = dojo.delegate(this);
			context.pop = function(){ return last; }
			return context;
		},
		pop: function(){
			throw new Error("pop() called on empty Context");
		},
		get: function(key, otherwise){
			var n = this._normalize;

			if(this._getter){
				var got = this._getter(key);
				if(typeof got != "undefined"){
					return n(got);
				}
			}

			if(typeof this[key] != "undefined"){
				return n(this[key]);
			}

			return otherwise;
		},
		_normalize: function(value){
			if(value instanceof Date){
				value.year = value.getFullYear();
				value.month = value.getMonth() + 1;
				value.day = value.getDate();
				value.date = value.year + "-" + ("0" + value.month).slice(-2) + "-" + ("0" + value.day).slice(-2);
				value.hour = value.getHours();
				value.minute = value.getMinutes();
				value.second = value.getSeconds();
				value.microsecond = value.getMilliseconds();
			}
			return value;
		},
		update: function(dict){
			var context = this.push();
			if(dict){
				dojo._mixin(this, dict);
			}
			return context;
		}
	});

	var smart_split_re = /("(?:[^"\\]*(?:\\.[^"\\]*)*)"|'(?:[^'\\]*(?:\\.[^'\\]*)*)'|[^\s]+)/g;
	var split_re = /\s+/g;
	var split = function(/*String|RegExp?*/ splitter, /*Integer?*/ limit){
		splitter = splitter || split_re;
		if(!(splitter instanceof RegExp)){
			splitter = new RegExp(splitter, "g");
		}
		if(!splitter.global){
			throw new Error("You must use a globally flagged RegExp with split " + splitter);
		}
		splitter.exec(""); // Reset the global

		var part, parts = [], lastIndex = 0, i = 0;
		while(part = splitter.exec(this)){
			parts.push(this.slice(lastIndex, splitter.lastIndex - part[0].length));
			lastIndex = splitter.lastIndex;
			if(limit && (++i > limit - 1)){
				break;
			}
		}
		parts.push(this.slice(lastIndex));
		return parts;
	}

	dd.Token = function(token_type, contents){
		this.token_type = token_type;
		this.contents = new String(dojo.trim(contents));
		this.contents.split = split;
		this.split = function(){
			return String.prototype.split.apply(this.contents, arguments);
		}
	}
	dd.Token.prototype.split_contents = function(/*Integer?*/ limit){
		var bit, bits = [], i = 0;
		limit = limit || 999;
		while(i++ < limit && (bit = smart_split_re.exec(this.contents))){
			bit = bit[0];
			if(bit.charAt(0) == '"' && bit.slice(-1) == '"'){
				bits.push('"' + bit.slice(1, -1).replace('\\"', '"').replace('\\\\', '\\') + '"');
			}else if(bit.charAt(0) == "'" && bit.slice(-1) == "'"){
				bits.push("'" + bit.slice(1, -1).replace("\\'", "'").replace('\\\\', '\\') + "'");
			}else{
				bits.push(bit);
			}
		}
		return bits;
	}

	var ddt = dd.text = {
		_get: function(module, name, errorless){
			// summary: Used to find both tags and filters
			var params = dd.register.get(module, name.toLowerCase(), errorless);
			if(!params){
				if(!errorless){
					throw new Error("No tag found for " + name);
				}
				return null;
			}

			var fn = params[1];
			var require = params[2];

			var parts;
			if(fn.indexOf(":") != -1){
				parts = fn.split(":");
				fn = parts.pop();
			}

			dojo["require"](require);

			var parent = dojo.getObject(require);

			return parent[fn || name] || parent[name + "_"] || parent[fn + "_"];
		},
		getTag: function(name, errorless){
			return ddt._get("tag", name, errorless);
		},
		getFilter: function(name, errorless){
			return ddt._get("filter", name, errorless);
		},
		getTemplate: function(file){
			return new dd.Template(ddt.getTemplateString(file));
		},
		getTemplateString: function(file){
			return dojo._getText(file.toString()) || "";
		},
		_resolveLazy: function(location, sync, json){
			if(sync){
				if(json){
					return dojo.fromJson(dojo._getText(location)) || {};
				}else{
					return dd.text.getTemplateString(location);
				}
			}else{
				return dojo.xhrGet({
					handleAs: (json) ? "json" : "text",
					url: location
				});
			}
		},
		_resolveTemplateArg: function(arg, sync){
			if(ddt._isTemplate(arg)){
				if(!sync){
					var d = new dojo.Deferred();
					d.callback(arg);
					return d;
				}
				return arg;
			}
			return ddt._resolveLazy(arg, sync);
		},
		_isTemplate: function(arg){
			return (typeof arg == "undefined") || (typeof arg == "string" && (arg.match(/^\s*[<{]/) || arg.indexOf(" ") != -1));
		},
		_resolveContextArg: function(arg, sync){
			if(arg.constructor == Object){
				if(!sync){
					var d = new dojo.Deferred;
					d.callback(arg);
					return d;
				}
				return arg;
			}
			return ddt._resolveLazy(arg, sync, true);
		},
		_re: /(?:\{\{\s*(.+?)\s*\}\}|\{%\s*(load\s*)?(.+?)\s*%\})/g,
		tokenize: function(str){
			return dojox.string.tokenize(str, ddt._re, ddt._parseDelims);
		},
		_parseDelims: function(varr, load, tag){
			if(varr){
				return [dd.TOKEN_VAR, varr];
			}else if(load){
				var parts = dojo.trim(tag).split(/\s+/g);
				for(var i = 0, part; part = parts[i]; i++){
					dojo["require"](part);
				}
			}else{
				return [dd.TOKEN_BLOCK, tag];
			}
		}
	}

	dd.Template = dojo.extend(function(/*String|dojo._Url*/ template, /*Boolean*/ isString){
		// template:
		//		The string or location of the string to
		//		use as a template
		var str = isString ? template : ddt._resolveTemplateArg(template, true) || "";
		var tokens = ddt.tokenize(str);
		var parser = new dd._Parser(tokens);
		this.nodelist = parser.parse();
	},
	{
		update: function(node, context){
			// node: DOMNode|String|dojo.NodeList
			//		A node reference or set of nodes
			// context: dojo._Url|String|Object
			//		The context object or location
			return ddt._resolveContextArg(context).addCallback(this, function(contextObject){
				var content = this.render(new dd._Context(contextObject));
				if(node.forEach){
					node.forEach(function(item){
						item.innerHTML = content;
					});
				}else{
					dojo.byId(node).innerHTML = content;
				}
				return this;
			});
		},
		render: function(context, /*concatenatable?*/ buffer){
			buffer = buffer || this.getBuffer();
			context = context || new dd._Context({});
			return this.nodelist.render(context, buffer) + "";
		},
		getBuffer: function(){
			
			return new dojox.string.Builder();
		}
	});

	var qfRe = /\{\{\s*(.+?)\s*\}\}/g;
	dd.quickFilter = function(str){
		if(!str){
			return new dd._NodeList();
		}

		if(str.indexOf("{%") == -1){
			return new dd._QuickNodeList(dojox.string.tokenize(str, qfRe, function(token){
				return new dd._Filter(token);
			}));
		}
	}

	dd._QuickNodeList = dojo.extend(function(contents){
		this.contents = contents;
	},
	{
		render: function(context, buffer){
			for(var i=0, l=this.contents.length; i<l; i++){
				if(this.contents[i].resolve){
					buffer = buffer.concat(this.contents[i].resolve(context));
				}else{
					buffer = buffer.concat(this.contents[i]);
				}
			}
			return buffer;
		},
		dummyRender: function(context){ return this.render(context, dd.Template.prototype.getBuffer()).toString(); },
		clone: function(buffer){ return this; }
	});

	dd._Filter = dojo.extend(function(token){
		// summary: Uses a string to find (and manipulate) a variable
		if(!token) throw new Error("Filter must be called with variable name");
		this.contents = token;

		var cache = this._cache[token];
		if(cache){
			this.key = cache[0];
			this.filters = cache[1];
		}else{
			this.filters = [];
			dojox.string.tokenize(token, this._re, this._tokenize, this);
			this._cache[token] = [this.key, this.filters];
		}
	},
	{
		_cache: {},
		_re: /(?:^_\("([^\\"]*(?:\\.[^\\"])*)"\)|^"([^\\"]*(?:\\.[^\\"]*)*)"|^([a-zA-Z0-9_.]+)|\|(\w+)(?::(?:_\("([^\\"]*(?:\\.[^\\"])*)"\)|"([^\\"]*(?:\\.[^\\"]*)*)"|([a-zA-Z0-9_.]+)|'([^\\']*(?:\\.[^\\']*)*)'))?|^'([^\\']*(?:\\.[^\\']*)*)')/g,
		_values: {
			0: '"', // _("text")
			1: '"', // "text"
			2: "", // variable
			8: '"' // 'text'
		},
		_args: {
			4: '"', // :_("text")
			5: '"', // :"text"
			6: "", // :variable
			7: "'"// :'text'
		},
		_tokenize: function(){
			var pos, arg;

			for(var i = 0, has = []; i < arguments.length; i++){
				has[i] = (typeof arguments[i] != "undefined" && typeof arguments[i] == "string" && arguments[i]);
			}

			if(!this.key){
				for(pos in this._values){
					if(has[pos]){
						this.key = this._values[pos] + arguments[pos] + this._values[pos];
						break;
					}
				}
			}else{
				for(pos in this._args){
					if(has[pos]){
						var value = arguments[pos];
						if(this._args[pos] == "'"){
							value = value.replace(/\\'/g, "'");
						}else if(this._args[pos] == '"'){
							value = value.replace(/\\"/g, '"');
						}
						arg = [!this._args[pos], value];
						break;
					}
				}
				// Get a named filter
				var fn = ddt.getFilter(arguments[3]);
				if(!dojo.isFunction(fn)) throw new Error(arguments[3] + " is not registered as a filter");
				this.filters.push([fn, arg]);
			}
		},
		getExpression: function(){
			return this.contents;
		},
		resolve: function(context){
			if(typeof this.key == "undefined"){
				return "";
			}

			var str = this.resolvePath(this.key, context);

			for(var i = 0, filter; filter = this.filters[i]; i++){
				// Each filter has the function in [0], a boolean in [1][0] of whether it's a variable or a string
				// and [1][1] is either the variable name of the string content.
				if(filter[1]){
					if(filter[1][0]){
						str = filter[0](str, this.resolvePath(filter[1][1], context));
					}else{
						str = filter[0](str, filter[1][1]);
					}
				}else{
					str = filter[0](str);
				}
			}

			return str;
		},
		resolvePath: function(path, context){
			var current, parts;
			var first = path.charAt(0);
			var last = path.slice(-1);
			if(!isNaN(parseInt(first))){
				current = (path.indexOf(".") == -1) ? parseInt(path) : parseFloat(path);
			}else if(first == '"' && first == last){
				current = path.slice(1, -1);
			}else{
				if(path == "true"){ return true; }
				if(path == "false"){ return false; }
				if(path == "null" || path == "None"){ return null; }
				parts = path.split(".");
				current = context.get(parts[0]);

				if(dojo.isFunction(current)){
					var self = context.getThis && context.getThis();
					if(current.alters_data){
						current = "";
					}else if(self){
						current = current.call(self);
					}else{
						current = "";
					}
				}

				for(var i = 1; i < parts.length; i++){
					var part = parts[i];
					if(current){
						var base = current;
						if(dojo.isObject(current) && part == "items" && typeof current[part] == "undefined"){
							var items = [];
							for(var key in current){
								items.push([key, current[key]]);
							}
							current = items;
							continue;
						}

						if(current.get && dojo.isFunction(current.get) && current.get.safe){
							current = current.get(part);
						}else if(typeof current[part] == "undefined"){
							current = current[part];
							break;
						}else{
							current = current[part];
						}

						if(dojo.isFunction(current)){
							if(current.alters_data){
								current = "";
							}else{
								current = current.call(base);
							}
						}else if(current instanceof Date){
							current = dd._Context.prototype._normalize(current);
						}
					}else{
						return "";
					}
				}
			}
			return current;
		}
	});

	dd._TextNode = dd._Node = dojo.extend(function(/*Object*/ obj){
		// summary: Basic catch-all node
		this.contents = obj;
	},
	{
		set: function(data){
			this.contents = data;
			return this;
		},
		render: function(context, buffer){
			// summary: Adds content onto the buffer
			return buffer.concat(this.contents);
		},
		isEmpty: function(){
			return !dojo.trim(this.contents);
		},
		clone: function(){ return this; }
	});

	dd._NodeList = dojo.extend(function(/*Node[]*/ nodes){
		// summary: Allows us to render a group of nodes
		this.contents = nodes || [];
		this.last = "";
	},
	{
		push: function(node){
			// summary: Add a new node to the list
			this.contents.push(node);
			return this;
		},
		concat: function(nodes){
			this.contents = this.contents.concat(nodes);
			return this;
		},
		render: function(context, buffer){
			// summary: Adds all content onto the buffer
			for(var i = 0; i < this.contents.length; i++){
				buffer = this.contents[i].render(context, buffer);
				if(!buffer) throw new Error("Template must return buffer");
			}
			return buffer;
		},
		dummyRender: function(context){
			return this.render(context, dd.Template.prototype.getBuffer()).toString();
		},
		unrender: function(){ return arguments[1]; },
		clone: function(){ return this; },
		rtrim: function(){
			while(1){
				i = this.contents.length - 1;
				if(this.contents[i] instanceof dd._TextNode && this.contents[i].isEmpty()){
					this.contents.pop();
				}else{
					break;
				}
			}

			return this;
		}
	});

	dd._VarNode = dojo.extend(function(str){
		// summary: A node to be processed as a variable
		this.contents = new dd._Filter(str);
	},
	{
		render: function(context, buffer){
			var str = this.contents.resolve(context);
			if(!str.safe){
				str = dd._base.escape("" + str);
			}
			return buffer.concat(str);
		}
	});

	dd._noOpNode = new function(){
		// summary: Adds a no-op node. Useful in custom tags
		this.render = this.unrender = function(){ return arguments[1]; }
		this.clone = function(){ return this; }
	}

	dd._Parser = dojo.extend(function(tokens){
		// summary: Parser used during initialization and for tag groups.
		this.contents = tokens;
	},
	{
		i: 0,
		parse: function(/*Array?*/ stop_at){
			// summary: Turns tokens into nodes
			// description: Steps into tags are they're found. Blocks use the parse object
			//		to find their closing tag (the stop_at array). stop_at is inclusive, it
			//		returns the node that matched.
			var terminators = {}, token;
			stop_at = stop_at || [];
			for(var i = 0; i < stop_at.length; i++){
				terminators[stop_at[i]] = true;
			}

			var nodelist = new dd._NodeList();
			while(this.i < this.contents.length){
				token = this.contents[this.i++];
				if(typeof token == "string"){
					nodelist.push(new dd._TextNode(token));
				}else{
					var type = token[0];
					var text = token[1];
					if(type == dd.TOKEN_VAR){
						nodelist.push(new dd._VarNode(text));
					}else if(type == dd.TOKEN_BLOCK){
						if(terminators[text]){
							--this.i;
							return nodelist;
						}
						var cmd = text.split(/\s+/g);
						if(cmd.length){
							cmd = cmd[0];
							var fn = ddt.getTag(cmd);
							if(fn){
								nodelist.push(fn(this, new dd.Token(type, text)));
							}
						}
					}
				}
			}

			if(stop_at.length){
				throw new Error("Could not find closing tag(s): " + stop_at.toString());
			}

			this.contents.length = 0;
			return nodelist;
		},
		next_token: function(){
			// summary: Returns the next token in the list.
			var token = this.contents[this.i++];
			return new dd.Token(token[0], token[1]);
		},
		delete_first_token: function(){
			this.i++;
		},
		skip_past: function(endtag){
			while(this.i < this.contents.length){
				var token = this.contents[this.i++];
				if(token[0] == dd.TOKEN_BLOCK && token[1] == endtag){
					return;
				}
			}
			throw new Error("Unclosed tag found when looking for " + endtag);
		},
		create_variable_node: function(expr){
			return new dd._VarNode(expr);
		},
		create_text_node: function(expr){
			return new dd._TextNode(expr || "");
		},
		getTemplate: function(file){
			return new dd.Template(file);
		}
	});

	dd.register = {
		_registry: {
			attributes: [],
			tags: [],
			filters: []
		},
		get: function(/*String*/ module, /*String*/ name){
			var registry = dd.register._registry[module + "s"];
			for(var i = 0, entry; entry = registry[i]; i++){
				if(typeof entry[0] == "string"){
					if(entry[0] == name){
						return entry;
					}
				}else if(name.match(entry[0])){
					return entry;
				}
			}
		},
		getAttributeTags: function(){
			var tags = [];
			var registry = dd.register._registry.attributes;
			for(var i = 0, entry; entry = registry[i]; i++){
				if(entry.length == 3){
					tags.push(entry);
				}else{
					var fn = dojo.getObject(entry[1]);
					if(fn && dojo.isFunction(fn)){
						entry.push(fn);
						tags.push(entry);
					}
				}
			}
			return tags;
		},
		_any: function(type, base, locations){
			for(var path in locations){
				for(var i = 0, fn; fn = locations[path][i]; i++){
					var key = fn;
					if(dojo.isArray(fn)){
						key = fn[0];
						fn = fn[1];
					}
					if(typeof key == "string"){
						if(key.substr(0, 5) == "attr:"){
							var attr = fn;
							if(attr.substr(0, 5) == "attr:"){
								attr = attr.slice(5);
							}
							dd.register._registry.attributes.push([attr.toLowerCase(), base + "." + path + "." + attr]);
						}
						key = key.toLowerCase()
					}
					dd.register._registry[type].push([
						key,
						fn,
						base + "." + path
					]);
				}
			}
		},
		tags: function(/*String*/ base, /*Object*/ locations){
			dd.register._any("tags", base, locations);
		},
		filters: function(/*String*/ base, /*Object*/ locations){
			dd.register._any("filters", base, locations);
		}
	}

	var escapeamp = /&/g;
	var escapelt = /</g;
	var escapegt = />/g;
	var escapeqt = /'/g;
	var escapedblqt = /"/g;
	dd._base.escape = function(value){
		// summary: Escapes a string's HTML
		return dd.mark_safe(value.replace(escapeamp, '&amp;').replace(escapelt, '&lt;').replace(escapegt, '&gt;').replace(escapedblqt, '&quot;').replace(escapeqt, '&#39;'));
	}

	dd._base.safe = function(value){
		if(typeof value == "string"){
			value = new String(value);
		}
		if(typeof value == "object"){
			value.safe = true;
		}
		return value;
	}
	dd.mark_safe = dd._base.safe;

	dd.register.tags("dojox.dtl.tag", {
		"date": ["now"],
		"logic": ["if", "for", "ifequal", "ifnotequal"],
		"loader": ["extends", "block", "include", "load", "ssi"],
		"misc": ["comment", "debug", "filter", "firstof", "spaceless", "templatetag", "widthratio", "with"],
		"loop": ["cycle", "ifchanged", "regroup"]
	});
	dd.register.filters("dojox.dtl.filter", {
		"dates": ["date", "time", "timesince", "timeuntil"],
		"htmlstrings": ["linebreaks", "linebreaksbr", "removetags", "striptags"],
		"integers": ["add", "get_digit"],
		"lists": ["dictsort", "dictsortreversed", "first", "join", "length", "length_is", "random", "slice", "unordered_list"],
		"logic": ["default", "default_if_none", "divisibleby", "yesno"],
		"misc": ["filesizeformat", "pluralize", "phone2numeric", "pprint"],
		"strings": ["addslashes", "capfirst", "center", "cut", "fix_ampersands", "floatformat", "iriencode", "linenumbers", "ljust", "lower", "make_list", "rjust", "slugify", "stringformat", "title", "truncatewords", "truncatewords_html", "upper", "urlencode", "urlize", "urlizetrunc", "wordcount", "wordwrap"]
	});
	dd.register.filters("dojox.dtl", {
		"_base": ["escape", "safe"]
	});
})();

}

if(!dojo._hasResource["dojox.dtl.Context"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.dtl.Context"] = true;
dojo.provide("dojox.dtl.Context");


dojox.dtl.Context = dojo.extend(function(dict){
	this._this = {};
	dojox.dtl._Context.call(this, dict);
}, dojox.dtl._Context.prototype,
{
	getKeys: function(){
		var keys = [];
		for(var key in this){
			if(this.hasOwnProperty(key) && key != "_this"){
				keys.push(key);
			}
		}
		return keys;
	},
	extend: function(/*dojox.dtl.Context|Object*/ obj){
		// summary: Returns a clone of this context object, with the items from the
		//		passed objecct mixed in.
		return  dojo.delegate(this, obj);
	},
	filter: function(/*dojox.dtl.Context|Object|String...*/ filter){
		// summary: Returns a clone of this context, only containing the items
		//		defined in the filter.
		var context = new dojox.dtl.Context();
		var keys = [];
		var i, arg;
		if(filter instanceof dojox.dtl.Context){
			keys = filter.getKeys();
		}else if(typeof filter == "object"){
			for(var key in filter){
				keys.push(key);
			}
		}else{
			for(i = 0; arg = arguments[i]; i++){
				if(typeof arg == "string"){
					keys.push(arg);
				}
			}
		}

		for(i = 0, key; key = keys[i]; i++){
			context[key] = this[key];
		}

		return context;
	},
	setThis: function(/*Object*/ _this){
		this._this = _this;
	},
	getThis: function(){
		return this._this;
	},
	hasKey: function(key){
		if(this._getter){
			var got = this._getter(key);
			if(typeof got != "undefined"){
				return true;
			}
		}

		if(typeof this[key] != "undefined"){
			return true;
		}

		return false;
	}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.data.dtlbinding']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.data.dtlbinding'] = true;
dojo.provide('ibm_ilog.diagram.data.dtlbinding');






(function() {

	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	var idb = ibm_ilog.diagram.data.dtlbinding = {
		//dtlExp: /[\s{,]([\w'"]*):["'\s]*{{(.+?)}}[\s'"]*|[\s{,][\w]*:\s*['"][^'"]*{{[^'"]*}}[^'"]*['"]|{%.+?%}/g,
		//dtlExp: /([\s*{,])([\w\s'"]*):([^\{\{,]*{{(([^'"|]*)[^'"]*)}}[A-Za-z\s'"]*)|{%.+?%}/g,
		dtlExp: /([{,])(\s*['"]?([\w]*)['"]?\s*:([^\{\{,]*{{([^|}]+)[^}}]*}}[^,}]*))/g,
		//arrays:                  ([{,])(\s*['"]?([\w]*)['"]?:([^\{\{]*{{([^|}]+)[^}}]*}}[^,}\]]*))
		itemPropExp: /{{([^\.]*)[\.]?([^|}]*)\|?[^}]*\}{2}/,
		replaceExp: "$1_binding_$3:{destPName:'$3',srcPName:'$5',value:$4},$2",
		_get: function(key) {
			if (this.length) {
				return (this[0] instanceof ibd.DataContext) ? this[0].get(key) : this[0][key];
			}
		}
	};
	
	$bind = function(expr) {
		var f = function() {
			return expr;
		};
		f.__is_idb = true;
		return f;
	};
	
	var dumpTpl = function(obj) {
		var recurse = arguments.callee, objtype = typeof obj;
		if (objtype == "number" || objtype == "boolean") {
			return obj + "";
		}
		if (obj === null) {
			return "null";
		}
		if (dojo.isString(obj)) {
			return dojo._escapeString(obj);
		}
		if (dojo.isFunction(obj)) {
			return obj.__is_idb ? obj.call() : null;
		}
		if (dojo.isArray(obj)) {
			var res = dojo.map(obj, function(o) {
				var val = recurse(o);
				return val;
			});
			return '[' + res.join(',') + ']';
		}
		var output = [], key, sep = "", newLine = "";
		for (key in obj) {
			var keyStr, val;
			keyStr = dojo._escapeString(key);
			val = recurse(obj[key]);
			if (typeof val != "string") {
				// skip non-serializable values
				continue;
			}
			output.push(newLine + keyStr + ":" + sep + val);
		}
		return "{" + output.join("," + sep) + newLine + "}"; // String
	};
	
	
	var _handlersMap = new dojox.collections.Dictionary();
	
	idb.connectShapeChanges = function(shape, ctx) {
		var handlers = _handlersMap.item(shape.getId());
		if (!handlers) {
			handlers = [];
			_handlersMap.add(shape.getId(), handlers);
		}
		var entry = {
			context: ctx,
			h: dojo.connect(shape, 'onPropertyChanged', ctx, onShapeChanged)
		};
		handlers.push(entry);
	};
	
	idb.disconnectShapeChanges = function(/*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ctx) {
		if (!ctx._shape) 
			return;
		var handlers = _handlersMap.item(ctx._shape.getId());
		if (handlers) {
			dojo.every(handlers, function(e, idx) {
				if (e.context === ctx) {
					dojo.disconnect(e.h);
					handlers.splice(idx, 1);
					return false;
				}
				return true;
			});
		}
	};
	
	function onShapeChanged(property, oldValue, newValue) {
		var ctx = this;
		if (ctx.__changeBatchLevel) {
			ctx.__changeBatchQueue.push(property);
		} else {
			updateShapeBoundProperty(ctx, property);
		}
	};
	
	var _storesList = [];
	
	function _findStoreEntry(store) {
		var entry = null;
		dojo.every(_storesList, function(e, idx) {
			if (e.store === store) {
				entry = e;
				entry.idx = idx;
				return false;
			}
			return true;
		});
		return entry;
	};
	
	idb.connectStoreChanges = function(item, store, /*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ ctx) {
		if (!store.getFeatures()["dojo.data.api.Notification"]) {
			return;
		}
		var entry = _findStoreEntry(store);
		if (!entry) {
			entry = {};
			entry.store = store;
			entry.handler = dojo.connect(store, "onSet", store, onStoreChanged);
			// a map between the item identity and the associated context
			entry.ctxMap = new dojox.collections.Dictionary();
			_storesList.push(entry);
		}
		var identity = store.getIdentity(item);
		// TODO: is it really a 1-1 relationship ? Not the case if two diagrams are connected to the same store in the same page.
		entry.ctxMap.add(identity, ctx);
	};
	
	idb.disconnectStoreChanges = function(/*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ctx) {
		if (!ctx._dataCtx) 
			return;
		var store = ctx._dataCtx.store;
		if (!store) 
			return;
		var entry = _findStoreEntry(store);
		if (entry) {
			var item = ctx._dataCtx.item;
			var identity = store.getIdentity(item);
			entry.ctxMap.remove(identity);
			// no more entries for this store ?
			if (entry.ctxMap.count === 0) {
				// then disconnect the store
				dojo.disconnect(entry.handler);
				// and remove the entry from the list
				_storesList.splice(entry.idx, 1);
			}
		}
	};
	
	function onStoreChanged(item, attribute, oldV, newV) {
		// Here, this = store;        
		var store = this;
		var entry = _findStoreEntry(store);
		if (!entry) 
			return; // should never happen
		var identity = store.getIdentity(item);
		var ctx = entry.ctxMap.item(identity);
		if (!ctx) 
			return;
		updateShapeBoundProperty(ctx, attribute);
	};
	
	function updateShapeBoundProperty(ctx, property) {
		var bindings = ctx._bindings;
		for (var i = 0; i < bindings.length; ++i) {
			var binding = bindings[i];
			if (binding.srcPName === property) {
				var shape = binding.target;
				var tpl = new dojox.dtl.Template(binding.value, true);
				var v = tpl.render(ctx);
				v = _fromJson(v);
				var s0 = binding.path.length > 0 ? binding.path[0] : binding.destPName;
				var pname = (s0.substr(0, 1).toUpperCase()) + s0.substr(1);
				var mainGetter = dojo.getObject('get' + pname, false, shape);
				if (!mainGetter) {
					continue;
				}
				// get a ref to the current toplevel property value. eg mainProp = shape.getStroke();
				var mainProp = mainGetter.call(shape);
				// go down the properties path and stop before the last element (e.g. mainProp.color)                    
				if (binding.path.length > 0) {
					var o = binding.path.length > 1 ? dojo.getObject(binding.path.slice(1).join('.'), false, mainProp) : mainProp;
					// set the bound field to the new value (e.g: mainProp.color.r = newValue)
					o[binding.destPName] = v;
				} else {
					mainProp = v;
				}
				// finally call the setter with the updated mainProp object (e.g: shape.setStroke(mainProp))
				var mainSetter = dojo.getObject('set' + pname, false, shape);
				if (mainSetter) {
				    if (dojo.config.isBidiInDojoDiagrammer &&  property === "textDir")
				        mainSetter.call(shape, mainProp, property);
				    else    				
					    mainSetter.call(shape, mainProp);
				}
			}
		}
	};
	
	// [av] TODO this is a generic array.contains implementation, maybe shoube placed somewhere else if needed.
	var __contains = function(a, v) {
		var i = a.length;
		while (i--) {
			if (a[i] === v) {
				return true;
			}
		}
		return false;
	};
	
	var __getChange = function(changes, shape, pname) {
		var c;
		for (var i = 0; i < changes.length; i++) {
			c = changes[i];
			if (c.shape == shape && c.pname == pname) {
				return c;
			}
		}
		return null;
	};
	
	function updateShapeBoundProperties(ctx, properties) {
	
		var complexChanges = [];
		var simpleChanges = [];
		
		dojo.forEach(ctx._bindings, function(binding) {
			if (__contains(properties, binding.srcPName)) {
				var shape = binding.target;
				var tpl = new dojox.dtl.Template(binding.value, true);
				var v = tpl.render(ctx);
				v = _fromJson(v);
				var s0 = binding.path.length > 0 ? binding.path[0] : binding.destPName;
				var pname = (s0.substr(0, 1).toUpperCase()) + s0.substr(1);
				
				// go down the properties path and stop before the last element (e.g. mainProp.color)                    
				if (binding.path.length > 0) {
				
					var mainGetter = dojo.getObject('get' + pname, false, shape);
					if (!mainGetter) {
						return;
					}
					
					var c = __getChange(complexChanges, shape, pname);
					if (!c) {
						c = {
							shape: shape,
							pname: pname,
							value: mainGetter.call(shape)
						};
						complexChanges.push(c);
					}
					
					var o = binding.path.length > 1 ? dojo.getObject(binding.path.slice(1).join('.'), false, c.value) : c.value;
					// set the bound field to the new value (e.g: value.color.r = newValue)
					o[binding.destPName] = v;
				} else {
					simpleChanges.push({
						shape: shape,
						pname: pname,
						value: v
					});
				}
			};
		});
		
		dojo.forEach(complexChanges.concat(simpleChanges), function(c) {
			var mainSetter = dojo.getObject('set' + c.pname, false, c.shape);
			if (mainSetter) {
				mainSetter.call(c.shape, c.value);
			}
		});
	};
	
	function _fromJson(v) {
		v = v.replace(/\\/g, '\\\\').replace(/\n/g, '\\n');
		if (v.charAt(0) == '"' && v.slice(-1) == '"') {
			v = '"' + v.slice(1, -1).replace(/"/g, '\\"') + '"';
		} else if (v.charAt(0) == "'" && v.slice(-1) == "'") {
			v = "'" + v.slice(1, -1).replace(/'/g, "\\'") + "'";
		}
		return dojo.fromJson(v);
	};
	
	ibm_ilog.diagram.data.dtlbinding.BoundContext = dojo.extend(function(shape, listen) {
		this._shape = shape;
		// we don't mixin this with shape because we don't want the context methods (like dispose())
		// to be replaced by same-name shape methods.
		dojox.dtl.Context.call(this, {});
		this._listen = listen;
		if (this._shape) {
			if (this._listen) {
				idb.connectShapeChanges(shape, this);
				if (shape.getData) {
					var dataCtx = shape.getData();
					if (dataCtx) {
						this._dataCtx = dataCtx;
						idb.connectStoreChanges(dataCtx.item, dataCtx.store, this);
					}
				}
			}
			this.setThis(this._shape); // to resolve function call
		}
	}, dojox.dtl.Context.prototype, {
	
		get: function(key, otherwise) {
			if (this._shape) {
				var val;
				// special case: 'context': to be able to reference the shape in a template filter (as parameter)
				// { prop : {{ foo | myfilter:context }} } 
				// where myfilter:function(val, arg)  <- arg= this._shape
				if (key == 'context') 
					return this._shape;
				// first try a getKey()
				var pname = key.slice(0, 1).toUpperCase() + key.slice(1);
				var getter = dojo.getObject('get' + pname, false, this._shape);
				if (getter && dojo.isFunction(getter)) {
					val = getter.call(this._shape);
					return val;
				}
				if (key in this._shape) {
					return this._shape[key];
				}
				// delegate to optional dict
				return dojox.dtl.Context.prototype.get.apply(this, arguments);
			}
		},
		dispose: function() {
			if (this._listen && this._shape) {
				idb.disconnectShapeChanges(this);
			}
			delete this._shape;
			if (this._listen) {
				idb.disconnectStoreChanges(this);
			}
			if (this._bindings) {
				this._bindings.length = 0;
			}
			delete this._bindings;
		},
		setBindings: function(bindings) {
			this._bindings = bindings;
		},
		endBatch: function() {
			this.__changeBatchLevel--;
			if (!this.__changeBatchLevel) {
				try {
					updateShapeBoundProperties(this, this.__changeBatchQueue);
				} catch (e) {
					R.error("Wrapped", "BoundContext # endBatch", e);
				} finally {
					delete this.__changeBatchQueue;
				}
			}
		},
		startBatch: function() {
			if (!this.__changeBatchLevel) {
				this.__changeBatchQueue = [];
				this.__changeBatchLevel = 1;
			} else {
				this.__changeBatchLevel++;
			}
		}
		
	});
	
	function processTemplate(template) {
		var tpl = template, isStr = dojo.isString(template), parse = isStr, useDtl = false;
		// a plain JS object -> dump to string except if it has no bindings
		if (!isStr && !template.noDtl) {
			// serialize the template object to string
			tpl = dumpTpl(template); /*String*/
			parse = true;
		}
		if (parse) {
			if (tpl.length > 0) {
				// detect whether dtl is used
				// need to use indexOf as the regexp may return result for str such as "{shape:{..}}"
				if (tpl.indexOf('{{') > 0 || tpl.indexOf("{%") > 0) {
					// some dtl markups have been found
					useDtl = true;
					var result = tpl.replace(idb.dtlExp, idb.replaceExp);
					if (result) {
						// escape the dtl expression so that it can be 'dejsonified'
						var valueExp = /(value):([^}}]*}}[^,}'"]*['"]*)/
						var values = result.match(valueExp);
						while (values) {
							var s = values[2];
							s = "'" + escape(s) + "'";
							result = result.replace(valueExp, "$1:" + s);
							values = result.match(valueExp);
						}
						tpl = result;
					}
				} else { // no DTL detects
					// no DTL and was a plain JS ? mark the template object for next passes
					if (!isStr) {
						tpl = template;
						template.noDtl = true;
					}
				}
			} else { // empty string -> make it plain object
				tpl = {};
				tpl.noDtl = true;
			}
		}
		return {
			tpl: tpl,
			useDtl: useDtl
		};
	};
	
	ibm_ilog.diagram.declareTemplate = function(template) {
		var r = processTemplate(template);
		return new idb.GfxTemplate(r.tpl, r.useDtl);
	};
	
	idb.GfxTemplate = dojo.extend(function(template, useDtl) {
		// 2 arguments:  the ctor expects in this case a preprocessed template
		// as first parameter, and whether it uses dtl as 2nd parameter.
		//
		// 1 argument : the ctor expects a raw template.
		
		this.template = null;
		this.useDtl = false;
		var i;
		
		if (arguments.length > 1) {
			this.useDtl = useDtl;
			if (useDtl) {
				this.template = new dojox.dtl.Template(template, true);
			} else {
				this.template = template; /*String || Object*/
			}
		} else {
			var r = processTemplate(template);
			this.useDtl = r.useDtl;
			if (this.useDtl) {
				this.template = new dojox.dtl.Template(r.tpl, true);
			} else {
				this.template = r.tpl; /*String || Object*/
			}
		}
	}, {
		render: function(context, parent) {
			// summary:
			//    Renders this template as a gfx shape.
			// context: dojox.dtl.Context:
			//    The binding context.
			// parent: dojox.gfx.Surface || dojox.gfx.Group
			//    The gfx parent.
			// returns: dojox.gfx.Shape
			//    The shape corresponding to the template definition. 
			
			var v;
			if (this.useDtl) {
				v = this.template.render(context); /*String*/
			} else {
				v = this.template; /* String || object */
			}
			var o, bindings;
			if (dojo.isString(v)) {
				o = dojo.fromJson(v);
				if (this.useDtl) 
					bindings = this._reduceBindings(o);
			} else {
				o = v;
			}
			var s = ibm_ilog.diagram.templating.instantiate(parent, o, bindings);
			if (bindings && context.setBindings) 
				context.setBindings(bindings);
			return s;
		},
		
		_reduceBindings: function(o, bindings) {
			var bindings = bindings || [];
			for (var p in o) {
				if (p.indexOf("_binding_") !== -1) {
					var b = o[p];
					b.value = unescape(b.value);
					b.idx = bindings.length;
					var pos = b.srcPName.indexOf('.');
					if (pos != -1) {
						b.srcPName = b.srcPName.slice(pos + 1);
					}
					bindings.push(b);
				} else if (dojo.isObject(o[p])) {
					this._reduceBindings(o[p], bindings);
				}
			}
			return bindings;
		}
	});
	
	dojo.declare('ibm_ilog.diagram.data.DataContext', null, {
		constructor: function(item, store) {
			this.item = item;
			this.store = store;
		},
		
		getValue: function(/*String*/attribute, /*value?*/ defaultValue) {
			// summary:
			//     Returns a single attribute value. This method is a shortcut to
			//     this.store.getValue(item, attribute, defaultValue).
			
			if (this.store && this.item) {
				return this.store.getValue(item, attribute, defaultValue);
			}
			return null;
		},
		
		get: function(key) {
			// summary:
			//     Internal method. Should not be called directly.
			
			var store = this.store;
			var item = this.item;
			if (key == "getLabel") {
				return store.getLabel(item);
			} else if (key == "getAttributes") {
				return store.getAttributes(item);
			} else if (key == "getIdentity") {
				if (store.getIdentity) {
					return store.getIdentity(item);
				}
				return "Store has no identity API";
			} else {
				var values = store.getValues(item, key);
				if (!values) {
					return;
				}
				if (!dojo.isArray(values)) {
					return new ibm_ilog.diagram.data.DataContext(values, store);
				}
				
				values = dojo.map(values, function(value) {
					if (dojo.isObject(value) && store.isItem(value)) {
						return new ibm_ilog.diagram.data.DataContext(value, store);
					}
					return value;
				});
				values.get = idb._get;
				return values;
			}
		}
	});
	ibm_ilog.diagram.data.DataContext.prototype.get.safe = true;
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.templating']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.templating'] = true;
dojo.provide('ibm_ilog.diagram.templating');





(function() {

	var R = ibm_ilog.diagram.util.ErrorReporter;
	var d = dojo, g = dojox.gfx, gu = g.utils, idt = ibm_ilog.diagram.templating;
	
	d.mixin(idt, {
	
		instantiate: function(/*dojox.gfx.Surface || dojox.gfx.Group */parent, /*Object*/ template, bindings) {
			// summary:
			//        Instantiate the specified template.
			// description:
			//        A template object describes both the contents of a shape and its
			//        appearance properties. The description object must conform to the
			//        gfx serialization specification. In addition, it supports the concept
			//        of attach point that exists in dijit with the dojoAttachPoint attribute.
			//        When this method is invoked, an instance of the template is created 
			//        and initialized. If attach points are defined in the template, the 
			//        corresponding properties are defined on the instance.
			// parent: dojox.gfx.Surface || dojox.gfx.Group:
			//        The container of the new shape.
			// template: Object :
			//        The description object.
			try {
				// flag used to skip _onChanged notification 
				parent._noOnChanged = true;
				// no validation during the deserialization
				if (parent.suspendInvalidate) 
					parent.suspendInvalidate();
				
				var t = idt._deserialize(parent, template, bindings);
				// create dojoAttachPoint fields
				if (d.isArray(t)) {
					d.forEach(t, function(s) {
						idt._makeAttachPoint(s, parent);
					});
				} else {
					idt._makeAttachPoint(t, parent);
				}
			} finally {
				delete parent._noOnChanged;
				// trigger layout
				if (parent.resumeInvalidate) 
					parent.resumeInvalidate(true);
				ibm_ilog.diagram._base.gfxext.propagateChange(parent, null);
			}
			return t;
		},
		
		_makeAttachPoint: function(s, root) {
			root = root || s;
			if (s._attachPoint) {
				root[s._attachPoint] = s;
				// track attachPoints to be able to dispose them 
				if (!('_attachPoints' in root)) 
					root._attachPoints = [];
				root._attachPoints.push(s._attachPoint);
				delete s._attachPoint;
			}
			// Group ?
			if ('children' in s) {
				d.forEach(s.children, function(c) {
					idt._makeAttachPoint(c, root);
				});
			}
		},
		
		_deserialize: function(/* dojox.gfx.Surface || dojox.gfx.Group */parent,/* Object */ object, bindings) {
			// summary:
			//        Deserializes the specified template object.
			// template: Object : the description object to deserialize.
			// returns:
			//        A gfx shape that represents the specified description.
			
			if (object instanceof Array) {
				return d.map(object, function(s) {
					return idt._deserialize(parent, s, bindings);
				}); // Array
			}
			try {
				var shape = ("shape" in object) ? parent.createShape(object.shape) : parent.createGroup();
			} catch (e) {
				R.error("Wrapped", "templating.js # _deserialize # createShape/group", e);
			}
			if (bindings && bindings.length > 0) {
				// TODO: bindings is not used here. Replace with a bool
				idt.resolveBindingRefs(shape, object, bindings);
			}
			try {
				if (shape.suspendInvalidate) 
					shape.suspendInvalidate();
				// flag used to skip _onChanged notification 
				shape._noOnChanged = true;
				for (var p in object) {
					if (p == 'shape') 
						continue;
					else if (p == "transform") {
						shape.setTransform(object.transform);
					} else if (p == "stroke") {
						shape.setStroke(object.stroke);
					} else if (p == "fill") {
						shape.setFill(object.fill);
					} else if (p == "font") {
						shape.setFont(object.font);
					} else if (p == "children") {
						d.forEach(object.children, function(c) {
							idt._deserialize(shape, c, bindings);
						});
					} else if (p == "selectedStyle") {
						shape.selectedStyle = object.selectedStyle;
					} else if (p == "dojoAttachPoint") {
						shape._attachPoint = object.dojoAttachPoint;
					} else { // extended attribute
						// first check if a setter exists
						var n = p.substr(0, 1).toUpperCase() + p.substr(1);
						var setter = shape['set' + n];
						if (dojo.isFunction(setter)) {
							setter.call(shape, object[p]);
						} else {
							// fallback
							shape[p] = object[p];
						}
					}
				}
			} finally {
				delete shape._noOnChanged;
				if (shape.resumeInvalidate) 
					shape.resumeInvalidate(false); // false = do not trigger a validate()
				ibm_ilog.diagram._base.gfxext.propagateChange(shape, null);
			}
			return shape; // dojox.gfx.Shape
		},
		
		resolveBindingRefs: function(shape, o, bindings, path) {
			path = path || [];
			for (var p in o) {
				if (p && p != "children") {
					if (p.indexOf("_binding_") !== -1) {
						var b = o[p];
						b.target = shape;
						b.path = dojo.clone(path);
						// not needed anymore
						delete o[p];
					} else if (dojo.isObject(o[p])) {
						var npath = dojo.clone(path);
						npath.push(p);
						this.resolveBindingRefs(shape, o[p], bindings, npath);
					}
				}
			}
		}
		
	});
	
	//
	// _DiagramTemplated
	//
	dojo.declare("ibm_ilog.diagram.templating._DiagramTemplated", null, {
	
		applyTemplate: function(template, context) {
			// summary:
			//    Applies the specified template on this instance.
			// template: Object or string :
			//    the template to apply.
			// context: Object : the runtime context. Optional. If not specified, the runtime context is this shape instance.
			if (template) {
				if (this.children.length > 0) {
					this.clear();
				}
				this._clearAttachPoints();
				this._disposeBoundCountext();
				var dtlTemplate = template instanceof ibm_ilog.diagram.data.dtlbinding.GfxTemplate ? template : new ibm_ilog.diagram.data.dtlbinding.GfxTemplate(template);
				this._boundContext = context || new ibm_ilog.diagram.data.dtlbinding.BoundContext(this, dtlTemplate.useDtl);
				try {
				
					if (this.suspendInvalidate) 
						this.suspendInvalidate();
					var content = dtlTemplate.render(this._boundContext, this);
					this.onTemplateApplied(template);
				} catch (e) {
					this.onTemplateError(template, e);
					R.error("Wrapped", "_DiagramTemplated.applyTemplate", e);
				} finally {
					if (this.resumeInvalidate) 
						this.resumeInvalidate(true);
				}
				return content;
			}
			return null;
		},
		
		onTemplateApplied: function(/*Object || String*/template) {
			// summary:
			//      Invoked when the template has been applied.
			// template: Object or string :
			//    the template to apply.
		},
		
		onTemplateError: function(/*Object || String*/template, /*Exception*/ e) {
			// summary:
			//      Invoked if an error is raised when applying the specified template .
			// template: Object or string :
			//    the template to apply.
		},
		
		_disposeBoundCountext: function() {
			if (this._boundContext && this._boundContext.dispose) {
				this._boundContext.dispose();
			}
		},
		
		_clearAttachPoints: function() {
			if (this._attachPoints) {
				dojo.forEach(this._attachPoints, function(p) {
					delete this[p];
				}, this);
				this._attachPoints.length = 0;
				delete this._attachPoints;
			}
		},
		
		layoutShape: function( /*{shape}*/target) {
			//
			//	No layout needed by default.
			//
		},
		
		shallowInstantiateAt: function(target, template) {
			//
			//	Instantiate a shallow version of the node, only visible shape is
			//	generated, otherwise the result has no unneeded members. This means
			//	the result may not be an instance of the class of the receiver.
			//
			if (template) {
					var dtlTemplate = template instanceof ibm_ilog.diagram.data.dtlbinding.GfxTemplate ? template : new ibm_ilog.diagram.data.dtlbinding.GfxTemplate(template);
					var context = new ibm_ilog.diagram.data.dtlbinding.BoundContext(this, dtlTemplate.useDtl);
					var content = dtlTemplate.render(context, target);
			}
			
			return target;
			
		},
		
		onPropertyChanged: function(property, oldValue, newValue) {
			// summary:
			//		Called when a bindable property has changed.
			// description:
			//		Bindable properties are properties defined on a Node or a Link
			//		and that can be used in templates using bindings.
		},
		
		_getBindableProperty: function(property) {
			// summary:
			//		The generic method that gets a bindable property.
			// tags:
			//		private
			return this["_" + property];
		},
		
		_setBindableProperty: function(property, newValue) {
			// summary:
			//		The generic method that sets a bindable property.
			// tags:
			//		private
			var oldValue = this["_" + property];
			if (newValue != oldValue) {
				this["_" + property] = newValue;
				this.onPropertyChanged(property, oldValue, newValue);
			}
		},
		
		startDTLBatch: function() {
			if (this._boundContext) {
				this._boundContext.startBatch();
			}
		},
		
		endDTLBatch: function() {
			if (this._boundContext) {
				this._boundContext.endBatch();
			}
		}
	});
	
	// Declares a bindable property by extending a class with the following:
	//	- the field that contains the value (initialized to the default value)
	//	- the setXyx and getXyz methods that get and set the value
	ibm_ilog.diagram.templating.declareBindableProperty = function(type, property, defaultValue) {
		// summary:
		//		Declares a bindable property for an object type.
		// description:
		//		Bindable properties are properties of a graphic object (Node, Link or Subgraph)
		//		that can be used to define bindings in templates.
		// type:
		//		The type to which the property must be added.
		// property:
		//		The name of the property. If the property name is 'foo', declaring the bindable property
		//		will extend the specified type with two methods: getFoo() and a setFoo(newValue).
		//		In the template of the object, you can define bindings like: text: '{{foo}}'.
		//		The initial value of the bindable property will be used when creating the object, and
		//		when the property is changed the template will be updated automatically.
		// defaultValue:
		//		The default value of the property.
		var extension = {};
		// value field
		extension["_" + property] = defaultValue;
		var uname = (property.substr(0, 1).toUpperCase()) + property.substr(1);
		// getter function
		extension["get" + uname] = function() {
			return this._getBindableProperty(property);
		};
		// setter function
		extension["set" + uname] = function(newValue) {
			this._setBindableProperty(property, newValue);
			return this;
		};
		// OK extend now.
		dojo.extend(type, extension);
	};
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.GraphElement']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.GraphElement'] = true;
dojo.provide('ibm_ilog.diagram.GraphElement');






(function() {

dojo.declare("ibm_ilog.diagram.GraphElement", [dojox.gfx.Group, ibm_ilog.diagram.Selectable, ibm_ilog.diagram.templating._DiagramTemplated], {
  //
	// summary:
  //		An element that can be contained in a Graph.
	//
    
	_connections: null,
	
	//
	//	_IBMDiagramType: String
	//		Graph element type, used to distinguish basic graph element types: node, subgraph, link. 
	//		Used to quickly index maps based on graph element types.
	//
	_IBMDiagramType: undefined,
	
	_isIBMDiagramGraphElement: true,
	
	toString: function() {
    	return this.declaredClass.split(".").pop()+" "+this.getId()+(this.getLabel?(": "+this.getLabel()):"");
    },
    constructor: function() {
        var id = ibm_ilog.diagram._IdGenerator.nextId(this).toString();
        this.getId = function() {
            return id;
        };
        this._connections = [];
        // do not add _labelH to _connections as it is the result of a dojo.connect,
		// while this._connections will be disposed via shape.disconnect. See comments
		// in addConnection.
		this._labelH = dojo.connect(this,"setLabel",this,function(){
        	dojo.publish('/ibm_diagram/GraphElement/labelChanged', [this]);
        });
    },
    dispose: function(options) {
    	//	summary:
			//		Dispose this GraphElement.
			//	description:
			//		This function should be called when the GraphElement is completely deleted, that is, when it is removed
			//		from its parent graph and it will never be used again.
			//	options:
    	//		options.noRecurse:
			//			Do not recurse children
    	this.beforeDisposing(this);
    	dojo.publish('/ibm_diagram/GraphElement/beforeDisposing',[this]);
    	this._dispose();
    	this.afterDisposing(this);
    	dojo.publish('/ibm_diagram/GraphElement/afterDisposing',[this]);
    	//free connections
    	this.disconnectAll();
    },
    beforeDisposing: function(graphElement) {
    	// summary:
        //		This function is called before disposing the GraphElement
    },
    afterDisposing: function(graphElement) {
    	// summary:
        //		This function is called after disposing the GraphElement
    	// description:
        //		This function free all possible listeners
    	
    	// free possible listeners
        this._disposeBoundCountext();
    },
    connect: function() {
    	// summary:
        //		override gfx.connect to add the handle to the internal list
    	var conn = this.inherited(arguments);
    	this._connections.push(conn);
    	return conn;
    },
    addConnection: function(connection){
    	// summary:
        //		this method should be called on every dojo.connect done to the graph element that
    	//		should be disconnected on dispose.
    	this._connections.push(connection);
		
		// PR: you should NOT called this method for dojo.connect handler. Indeed, the
		// disconnectAll() method calls this.disconnect() which does not always relies on dojo.disconnect,
		// depending on the renderer. In general: you should not mix dojo.connect/disconnect with shape.connect/disconnect.
		//
		// ==> This method should be removed.
    },
    disconnectAll: function() {
    	// summary:
        //		This function is disconnect every dojo.connect attached to this element
    	dojo.forEach(this._connections,dojo.hitch(this,function(conn){
    		this.disconnect(conn);
    		delete conn;
    	}));
		dojo.disconnect(this._labelH);
    	this._connections = [];
    },
    _dispose: function(graphElement) {
    	// summary:
        //		this function should be overriden by any GraphElement subclass that needs any specific disposing action.
    }
    
});

ibm_ilog.diagram.GraphElement.byShape = function(shape) {
	// summary:
	//		Returns the GraphElement that contains the specified shape, or null.
	
	for(var p = shape; p && p.getParent; p = p.getParent())
		if(p instanceof ibm_ilog.diagram.GraphElement)
			return p;
	return null;
};

})();

}

if(!dojo._hasResource["ibm_ilog.diagram.util.MultilineText"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm_ilog.diagram.util.MultilineText"] = true;
dojo.provide("ibm_ilog.diagram.util.MultilineText");




(function () {
	var iid = ibm_ilog.diagram;
	
iid.gfxDeclare("ibm_ilog.diagram.util.MultilineText", [ dojox.gfx.Group ], {
	// summary:
	//		A multi-line text shape for GFX.
	// description:
	//		The MultilineText shape is similar to a Text shape, except that it can display multiple lines of text.
	//		<p>
	//		The shape object of the MultilineText accepts the same attributes as a plain Text shape
	//		("text", "x", "y", "align", etc). The "type" property must be "mltext".
	//		</p>
	//		<p>
	//		The MultilineText shape also accepts an additional "separator" attribute, that
	//		defines the line separator. The default separator is "\n". You can specify a simple string, or
	//		a RegExp as separator (for example, "[ \t]+"). When the text contains one or more
	//		occurrences of the line separator, the text is broken into several lines. The first line is placed
	//		normally according to the x and y properties of the shape, and subsequent lines are stacked vertically
	//		below the first line, that is, the baseline of the second line is at (x, y+h), where h is the font height.
	//		You can also specify an additional spacing using the "spacing" property of the shape (0 by default).
	//		</p>
	//		<p>
	//		The MultilineText supports simple text wrapping. To enable text wrapping, set the "wrap" property of the shape to true.
	//		When text wrapping is enabled, the text will be automatically broken into lines if necessary to fit the width of the text.
	//		This will work only if the width of the shape is constrained in one of the following ways:
	//		<ul>
	//		<li>You can specify the "width" property of the shape to a fixed width.</li>
	//		<li>The shape can be contained in a Group that has a GFX layout, and that constrains the width of the shape.
	//		For example, if the MultilineText shape is contained in a Group with a GridLayout with a fixed column width,
	//		then the text will be wrapped to fit the column width. (Note: the width determined by the GFX layout has precedence
	//		over the "width" shape property.)</li>
	//		</ul>
	//		In addition to wrapping text, you can also wrap single words if they do not fit the text width. To do this,
	//		set the "wordWrap" property of the shape to true (word-wrapping is false by default).
	//		</p>
	//		<p>
	//		<strong>Note:</strong> The built-in text wrapping method is a simplistic convenience placeholder that only breaks lines at space characters.
	//		To wrap text correctly according to globalization rules, you will need to provide a more complete implementation.
	//		To do this, you should override the <code>split</code> and <code>join</code> methods (to use different word breaking rules)
	//		or the <code>wrap</code> method (to completely replace the text wrapping algorithm), through a <code>dojo.extend</code> call.
	//		</p>
	//		<p>
	//		The MultilineText shape is actually a GFX Group shape containing Text shapes that display the lines.
	//		Each Text line has the same attributes as the MultilineText shape: align, font, fill, etc. For example,
	//		if the MultilineText has align: 'left', then all the lines will be aligned on the left. Similarly,
	//		calling setFont() on the MultilineText shape sets the font of all the lines.
	//		</p>
	
	_font: null,
	
	_lastWrapWidth: -1,
	
	textDir: "",
	
	setTextDir: function(/*String*/textdir) {
	   // summary:
	   //       set text direction property, need for runtime change of text direction  
	   
	   this.textDir = textdir; 
	},	
	
	constructor: function(){
		// summary:
		//		Creates an empty MultilineText instance.
		
		this.shape = dojo.clone(iid.util.MultilineText.defaultMultilineText);
	},
	
	setShape: function(shape){
		// summary:
		//		Sets the attributes of the MultilineText shape.
		// description:
		//		The <code>shape</code> parameter is an object with the following properties:
		//		<table style="border-collapse:collapse;" border="1">
		//			<tr>
		//				<th><b>Property</b></th>	<th><b>Description</b></th>
		//			</tr>
		//			<tr>
		//				<td>type</td>				<td>must be <code>'mltext'</code></td>
		//			</tr>
		//			<tr>
		//				<td>text</td>				<td>The text string to display.</td>
		//			</tr>
		//			<tr>
		//				<td>x</td>					<td>Defines the horizontal position of the text.</td>
		//			</tr>
		//			<tr>
		//				<td>y</td>					<td>Defines the vertical position of the baseline of the first line of text.</td>
		//			</tr>
		//			<tr>
		//				<td>align</td>				<td>The horizontal alignment (see dojox.gfx.Text).</td>
		//			</tr>
		//			<tr>
		//				<td>decoration</td>			<td>(see dojox.gfx.Text)</td>
		//			</tr>
		//			<tr>
		//				<td>rotated</td>			<td>(see dojox.gfx.Text) Not supported for multi-line text.</td>
		//			</tr>
		//			<tr>
		//				<td>kerning</td>			<td>(see dojox.gfx.Text)</td>
		//			</tr>
		//			<tr>
		//				<td>separator</td>			<td>The line separator character or regular expression.</td>
		//			</tr>
		//			<tr>
		//				<td>spacing</td>			<td>The vertical line spacing.</td>
		//			</tr>
		//			<tr>
		//				<td>wrap</td>				<td>If <code>true</code>, enables text wrapping. The value can also be a custom text wrapping function.</td>
		//			</tr>
		//			<tr>
		//				<td>wordWrap</td>			<td>If <code>true</code>, enables word-wrapping, that is, breaking single-word lines that do not fit the specified width.</td>
		//			</tr>
		//			<tr>
		//				<td>width</td>				<td>Specifies a default width for text wrapping. Can be overridden by a GFX layout.</td>
		//			</tr>
		//		</table>
		// shape: Object
		//		A hash that contains the properties of the shape.
		
		if(this._noSetShape) return this; // see comment in _createLine before this.createText...
		if(iid._inContainerAdd) return this;
		
		var sh = this.shape = dojo.mixin(dojo.clone(iid.util.MultilineText.defaultMultilineText), shape);
		
		this._lastWrapWidth = -1;
		
		if(sh.wrap && sh.width > 0)
			this._doWrap(sh.width);
		else {
			var sep = sh.separator;
			if(sep.length > 1)
				sep = new RegExp(sep, "g");

			var lines = sh.text.split(sep);

			this._createLines(lines);
		}
		// propagate changes and invalidate
		iid._base.gfxext.onChangePatch.call(this, 'setShape');
		return this;
	},
	
	_createLines: function(lines)
	{
		// summary:
		//		Creates the Text shapes displaying the lines of text.
		
		if(this.children && this.children.length > 0)
			this.clear();
		
		var sh = this.shape;
		
		var y = sh.y;
		for(var i = 0; i < lines.length; i++){
			var textObj = this._createLine(lines[i], y);
			if(textObj){
				var bb = textObj.getBoundingBox();
				if(bb)
					y += bb.height + sh.spacing;
			}
		}
	},
	
	_createLine: function(text, y)
	{
		var sh = this.shape;
		
		var textShape = dojo.clone(dojox.gfx.defaultText);
		for(var p in textShape)
			if(p != "type" && sh[p])
				textShape[p] = sh[p];
		textShape.y = y;
		textShape.text = text;
		
		if (dojo.config.isBidiInDojoDiagrammer && sh.textDir)                            
			textShape.textDir = sh.textDir;	
		
		// This is a workaround for a very nasty bug with Dojo 1.6:
		// when adding a text child, Dojo now resets the shape and text of all other text children,
		// but it does so using a dojox.gfx.utils.forEach, which iterates also on the multiline shape itself
		// (because the forEach just tests if the shape has a getFont method, which the parent multiline shape
		// happens to have too...).
		// So to avoid this we disable setShape by a flag.
		this._noSetShape = true;
		var textObj = this.createText(textShape);
		delete this._noSetShape;
		if(textObj){
			// copy font and fill:
			var s;
			s = this.getFont(); if(s) textObj.setFont(s);
			s = this.getFill(); if(s) textObj.setFill(s);
		}	
		return textObj;
	},
	
	setFont: function(font){
		// summary: 
		//		Sets the font for text.
		// font: Object
		//		A font object (see dojox.gfx.defaultFont) or a font string
		
		if(iid._inContainerAdd) return this;
		
		this._font = font;
		
		var y = this.shape.y;
		for(var i = 0; i < this.children.length; i++){
			this.children[i].setFont(font);
		}
		// propagate changes and invalidate
		iid._base.gfxext.onChangePatch.call(this, 'setShape');
		return this;
	},
	
	getFont: function(){
		// summary:
		//		Returns the current font object or null.
		
		return this._font;
	},
	
	setFill: function(fill){
		// summary: 
		//		Sets the color for text.
		// fill: Object
		//		A fill object (see dojox.gfx.defaultFill) or a color string.
		
		this.inherited(arguments);
		for(var i = 0; i < this.children.length; i++)
			this.children[i].setFill(fill);
	},
	
	getBoundingBox: function(){
		var bb = this.inherited(arguments);
		if(!bb)
			bb = { x:0, y:0, width:1, height:1 };
		return bb;
	},

	_computePreferredSize: function(/*Size*/proposedSize) {
		// summary:
		//     Computes the preferred size of this shape. 
		// description:
		//     This method wraps the text if required.
		
		var width = proposedSize.width;
		if(width <= 0)
			width = this.shape.width;

    	if(this.shape.wrap && width > 0)
    		this._doWrap(width);
		
    	var sz = this.inherited(arguments);
    	
    	return sz;
	},
	
    applyLayout: function(/*Rectangle*/rect) {
    	// summary:
    	//		Layout this text in the given rectangle.
    	
    	if (this.shape.wrap) {
			var w = rect.width;
			var t = this.getTransform();
			if(t && !t.isTranslation())
				w = t.inverse().transformRectangle(rect).width;
			this._doWrap(w);
		}
    	
    	var x = rect.x;
    	var y = rect.y;
        var oldBounds = this.getLocalBoundingBox();
        if (oldBounds){
        	// horizontal alignment:
        	switch(this.shape.align){
        	case 'middle':
        		x += (rect.width - oldBounds.width)/2;
        		break;
        	case 'end':
        		x += (rect.width - oldBounds.width);
        		break;
        	}
        	// vertical alignment:
        	switch(this.valign || 'stretch'){
        	case 'center':
        	case 'stretch':
        		y += (rect.height - oldBounds.height)/2;
        		break;
        	case 'bottom':
        		y += rect.height - oldBounds.height;
        		break;
        	}
        	
            this.applyLeftTransform(dojox.gfx.matrix.translate(x-oldBounds.x, y-oldBounds.y));
        } else {
            this.setTransform(dojox.gfx.matrix.translate(x, y));
        }
    },
    
    _doWrap: function(width)
    {
    	if(width == this._lastWrapWidth)
    		return;
    	this._lastWrapWidth = width;
    	
    	if(this.shape.text == ""){
    		this._createLines([""]);
    		return;
    	}
    	
		var wrap = this.shape.wrap;
		
		if(wrap){
			try {
				if(this.suspendInvalidate) this.suspendInvalidate();
				this._noOnChanged=true;
				var lines = this.wrap(this.shape.text, width);
				this._createLines(lines);
			} catch(e){
			} finally {
				if(this.resumeInvalidate) this.resumeInvalidate(false);
				delete this._noOnChanged;
				this._bbCacheValid = false;
			}
		}
    },
    
    wrap: function(text, width)
    {
		// summary:
		//		Breaks the text into lines to fit the specified width.
		// description:
		//		This method implements a simplistic text wrapping algorithm that breaks 
		//		the specified text into lines to fit the specified width.
		//		This simple implementation breaks the text into words by calling the
		//		<code>split</code> and <code>join</code> methods.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// text: string
		//		The text to break into lines fitting the specified width.
		// width: Number
		//		The width in which the wrapped text should fit.
		// returns:
		//		An array of strings representing the wrapped lines of text.
		
        var lines = [];
        var sh = this.shape;
        
        // Keep "hard" line breaks: split the text into paragraphs first,
        // then wrap each paragraph separately.
		var sep = sh.separator;
		if(sep.length > 1)
			sep = new RegExp(sep, "g");
		var paragraphs = text.split(sep);
		
		var tmpText = null;
		
		for(var pi = 0; pi < paragraphs.length; pi++){
			// split into words
			var words = this.split(paragraphs[pi]);
			if (words.length > 0) {
				var line = null;
				// join words into lines until max width is exceeded
				var i0 = 0;
				tmpText = this._createLine("", 0);
				var bb;
				for (var i = 0; i < words.length; i++) {
					var word = words[i];
					if (!line) {
						line = word;
						// Do word-wrapping?
						if (sh.wordWrap) {
							tmpText.setShape({
								text: line
							});
							bb = tmpText.getBoundingBox();
							if (bb && bb.width > width) {
								var cutIndex = this.next(word, 0);
								if (cutIndex > 0) {
									for (var j = this.next(word, cutIndex); j >= 0; j = this.next(word, j)) {
										var w = word.substring(0, j);
										tmpText.setShape({
											text: w
										});
										bb = tmpText.getBoundingBox();
										if (bb && bb.width > width) 
											break;
										cutIndex = j;
									}
									lines.push(word.substring(0, cutIndex));
									words[i] = word.substring(cutIndex);
									line = null;
									i--;
								}
							}
						}
					} else {
						var newline = this.join(words.slice(i0, i + 1));
						tmpText.setShape({
							text: newline
						});
						bb = tmpText.getBoundingBox();
						if (bb && bb.width > width) {
							lines.push(line);
							i0 = i;
							line = null;
							i--;
						}
						else {
							line = newline;
						}
					}
				}
				if (line) 
					lines.push(line);
			}
		}
        
		if(tmpText) this.remove(tmpText);
		
        return lines; // Array
    },
    
    split: function(text)
    {
		// summary:
		//		Splits the text using a space character delimiter.
		// description:
		//		The default method used to break the text into words for text wrapping.
		//		This default implementation splits the text using a space character delimiter.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// text: string
		//		The text to split into words.
		// returns:
		//		An array of strings representing the possible line break positions.
		
    	return text.split(" "); // Array
    },
    
    join: function(words)
    {
		// summary:
		//		Joins the words using a space character delimiter.
		// description:
		//		The default method used to join word into lines for text wrapping.
		//		This default implementation joins the words using a space character delimiter.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// words: Array
		//		The array of words to join into a line string.
		// returns:
		//		The specified words joined into a string.
		
    	return words.join(" "); // string
    },
	
	next: function(word, index)
	{
		// summary:
		//		Returns the next character in the word after index where word-wrapping is allowed.
		// description:
		//		This method is called during word-wrapping, to get the next character position where word-wrapping would be allowed.
		//		The default implementation just increments the index. You can override this method to forbid word breaks on particular
		//		characters, or for globalization.
		// word: string
		//		The word that is being wrapped.
		// index: int
		//		The start index from which the next character if required.
		// returns:
		//		The index of the next character where a word break may occur, or -1 if the end of the string was reached.
		
		index++;
		return index < word.length ? index : -1; // int
	}
});

iid.util.MultilineText._nextId = 1;
iid.util.MultilineText.defaultMultilineText = dojo.mixin(dojo.clone(dojox.gfx.defaultText),
										{ type: 'mltext', separator: '\n', spacing: 0, wrap: false, wordWrap: false, width: -1 });
iid.util.MultilineText.nodeType = dojox.gfx.Group.nodeType;

var g = dojox.gfx;

//
// Extends createShape to support our own shape types
//

function extendCreateShape(shapeType) {
    var method = "createShape";
    var old = shapeType.prototype[method];
    shapeType.prototype[method] = function() {
        var ret = old.apply(this, arguments);
        if (!ret) {
            var sh = arguments[0];
            switch (sh.type) {
                case 'mltext':
                    ret = this.createObject(iid.util.MultilineText, sh);
                    if (dojo.config.isBidiInDojoDiagrammer) {
                        var sp = this;
    				    while (sp) {
    					    if (sp.textDir) {
    						    ret.textDir = sp.textDir;
    						    break;
    					    }
    					    else if(sp.getTextDir) {
    						    ret.textDir = sp.getTextDir();
    						    break;
    					    }
    					    if (!sp.parent)
    						    break;
    					    sp = sp.getParent();
    				    }
                    }                    
                    break;
            }
        }
        return ret;
    };
}
extendCreateShape(g.Surface);
extendCreateShape(g.Group);

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.templates.templates']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.templates.templates'] = true;
dojo.provide('ibm_ilog.diagram.templates.templates');



(function(){

	var t = ibm_ilog.diagram.templates;

if (dojo.config.useGfxLayout) {

	// NODE - GFX-LAYOUT TEMPLATE

t.defaultNodeTemplate = [{
    dojoAttachPoint: 'baseShape',
    shape:{
        width:80, height:40, r: (ibm_ilog.diagram.isVML ? "0" : "5"),
        type:'rect'
    },
    fill: '{{backgroundColor}}',
    stroke: {'color': '{{borderColor}}', 'width': 2 },
    selectedStyle: {
        fill: '{{selectedBackgroundColor}}'
    }
},{
    dojoAttachPoint: 'textShape', 
    margins:[5,2,5,2],
    shape:{
        type:'mltext',
        text:'{{label}}', 
        wrap: $bind('{{wrap}}'), width: 70,
        align:'middle'
    }, 
    editable: {
        id:'label',
        editor:'textbox',
        border:2,
        minRegionSize:{width:50,height:15},
        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
        layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}
    },
    fill:'{{textColor}}', 
    font:{type:'font', size:'10pt', family:'sans-serif'},
    selectedStyle: {
        fill: '{{selectedTextColor}}'
    }
}];

t.defaultOverviewNodeTemplate = [{
  dojoAttachPoint: 'baseShape',
  shape:{
      width:80, height:40, r: (dojo.isIE ? "0" : "5"),
      type:'rect'
  },
  fill: '{{backgroundColor}}',
  stroke: {'color': '{{borderColor}}', 'width': 2 }
}];

} else {

	// NODE - NON GFX-LAYOUT TEMPLATE

t.defaultNodeTemplate = [{
    dojoAttachPoint: 'baseShape',
    shape: {
        x: 0,
        y: 0,
        width: 80,
        height: 40,
        r: 5,
        type: 'rect'
    },
    fill: '{{backgroundColor}}',
    stroke: {
        'color': '{{borderColor}}',
        'width': 2
    },
    selectedStyle: {
        fill: '{{selectedBackgroundColor}}'
    }
}, {
   dojoAttachPoint: 'textShape',
   shape: {
       type: 'text',
       text: '{{label}}',
       x: 40,
       y: 24,
       align: 'middle'
   },
   editable: {
       id:'label',
       editor:'textbox',
       border:2,
       regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
       baseRegion:{x:20,width:40,y:11,height:15}
   },
   fill: '{{textColor}}',
   font: {
       type: 'font',
       size: '10pt',
       family: 'sans-serif'
       },
   selectedStyle: {
       fill: '{{selectedTextColor}}'
   }
}];

t.defaultOverviewNodeTemplate = [{
  dojoAttachPoint: 'baseShape',
  shape: {
      x: 0,
      y: 0,
      width: 80,
      height: 40,
      r: 5,
      type: 'rect'
  },
  fill: '{{backgroundColor}}',
  stroke: {
      'color': '{{borderColor}}',
      'width': 2
  }
}];

}

t.defaultLinkTemplate = [{
    dojoAttachPoint: '_path',
    shape: {
        type: 'polyline',
        points: [{
            x: 0,
            y: 0
        }, {
            x: 100,
            y: 0
        }]
    },
    stroke: {
        width: $bind("{{strokeWidth}}"),
        color: '{{strokeColor}}'
    },
    selectedStyle: {
        stroke: {
            width: $bind("{{selectedStrokeWidth}}"),
            color: '{{selectedStrokeColor}}'
        }
    }
}, {
    dojoAttachPoint: '_endArrow',
    shape: {
        type: 'polyline',
        points: [{
            x: -8,
            y: 4
        }, {
            x: 0,
            y: 0
        }, {
            x: -8,
            y: -4
        }]
    },
    fill: '{{strokeColor}}',
    selectedStyle: {
        fill: '{{selectedStrokeColor}}'
    }
}];

t.defaultOverviewLinkTemplate = [{
  dojoAttachPoint: '_path',
  shape: {
      type: 'polyline',
      points: [{
          x: 0,
          y: 0
      }, {
          x: 100,
          y: 0
      }]
  },
  stroke: {
      width: $bind("{{strokeWidth}}"),
      color: '{{strokeColor}}'
  }
}];

if (dojo.config.useGfxLayout) {

	// SUBGRAPH - GFX-LAYOUT TEMPLATE

    t.defaultSubgraphTemplate = [{
        dojoAttachPoint: '_expandedGroup',
        layout: {
            type: 'ibm_ilog.diagram.gfxlayout.GridLayout',
            rows: [{
                height: -1
            }, {
                height: -1
            }]
        },
        children: [{
            dojoAttachPoint: '_expandedBaseShape',
            row: 0,
            rowspan: -1,
            shape: {
                type: 'rect',
                width: 0,
                height: 0,
                r: 4
            },
            fill: '{{headerColor}}',
            selectedStyle: {
                fill: '{{selectedHeaderColor}}'
            },
            stroke: '#404040'
        }, {
			hfiller:true,
            dojoAttachPoint: '_header',
            row: 0,
            margins: [2, 0, 2, 2],
            layout: {
                type: 'ibm_ilog.diagram.gfxlayout.StackLayout',
                horizontal: true,
                gap: 2
            },
            children: [{
                dojoAttachPoint: '_collapseButton',
                valign: 'center',
                layout: {
                    type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
                },
                children: [{
                    shape: {
                        type: 'rect',
                        width: 12,
                        height: 12,
                        r: 4
                    },
                    fill: '#808080'
                }, {
                    valign: 'center',
                    halign: 'center',
                    shape: {
                        type: 'line',
                        x1: 3,
                        y1: 6,
                        x2: 9,
                        y2: 6
                    },
                    stroke: {
                        color: 'white',
                        width: 2
                    }
                }]
            }, {
            	layout: {
					type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
				},
				children:[{
                dojoAttachPoint: 'titleShape',
				valign: 'center', halign: 'left',
                shape: {
                    type: 'text',
                    text: '{{label}}',
                    align: 'start'
                },
                fill: '{{textColor}}',
                font: {
                    type: 'font',
                    size: '10pt',
                    family: 'sans-serif'
                },
                editable: {
                    id: 'label',
                    editor: 'textbox',
                    options: {
                        style: {
                            'text-align': 'left'
                        }
                    },
                    border: 2,
			        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
    		    	layoutProperties: { valign: 'center', halign: 'left'}
                },
                selectedStyle: {
                    fill: '{{selectedTextColor}}'
                }
				}]
            }]
        }, {
            dojoAttachPoint: '_graphBackground',
            row: 1,
            margins: 2,
            col: 0,
            shape: {
                type: 'rect',
                width: 0,
                height: 0
            },
            fill: '{{graphBackgroundColor}}'
        }, {
            dojoAttachPoint: '_graph',
            row: 1,
            margins: $bind('{{graphMargin}}'),
			minimumSize: { width: 80, height: 40 },
            col: 0,
            shape: {
                type: 'graph'
            }
        }]
    }, {
        dojoAttachPoint: '_collapsedGroup',
        layout: {
            type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
        },
        children: [{
            dojoAttachPoint: '_collapsedBaseShape',
            shape: {
                width: 80,
                height: 40,
                r: 5,
                type: 'rect'
            },
            fill: '{{backgroundColor}}',
            stroke: {
                'width': 2
            },
            selectedStyle: {
                fill: '{{selectedBackgroundColor}}'
            }
        }, {
            dojoAttachPoint: 'textShape',
            margins: 5,
            shape: {
                type: 'mltext',
                text: '{{label}}',
		        wrap: $bind('{{wrap}}'), width: 70,
                align: 'middle'
            },
            fill: '{{textColor}}',
            selectedStyle: {
                fill: '{{selectedTextColor}}'
            },
    		editable: {
        		id:'label',
    		    editor:'textbox',
    		    border:2,
    		    regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
    		    layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}
    		},
            font: {
                type: 'font',
                size: '10pt',
                family: 'sans-serif'
            }
        }, {
            margins: 4,
            valign: 'top',
            halign: 'left',
            dojoAttachPoint: '_expandButton',
            layout: {
                type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
            },
            children: [{
                shape: {
                    type: 'rect',
                    width: 12,
                    height: 12,
                    r: 4
                },
                fill: '#808080'
            }, {
                valign: 'center',
                halign: 'center',
                shape: {
                    type: 'line',
                    x1: 3,
                    y1: 6,
                    x2: 9,
                    y2: 6
                },
                stroke: 'white'
            }, {
                valign: 'center',
                halign: 'center',
                shape: {
                    type: 'line',
                    x1: 6,
                    y1: 3,
                    x2: 6,
                    y2: 9
                },
                stroke: {
                    color: 'white',
                    width: 2
                }
            }]
        }]
    }];
    
} else {

	// SUBGRAPH - NON GFX-LAYOUT TEMPLATE

	t.defaultSubgraphTemplate = [{
        dojoAttachPoint: '_expandedGroup',
        children: [{
            dojoAttachPoint: '_expandedBaseShape',
           shape: {
               type: 'rect',
               r: 5
           },
           fill: '{{headerColor}}',
           selectedStyle: {
               fill: '{{selectedHeaderColor}}'
           },
           stroke: '#404040'
       }, {
           dojoAttachPoint: '_graphBackground',
           shape: {
               type: 'rect'
           },
           fill: '{{graphBackgroundColor}}'
       }, {
            dojoAttachPoint: '_graph',
            shape: {
                type: 'graph'
            }
        }, {
           dojoAttachPoint: '_header',
           transform: { dx: -8, dy: -26 },
           children: [{
               dojoAttachPoint: '_collapseButton',
               children: [{
                   shape: {
                       type: 'rect',
                       x: 0,
                       y: 0,
                       width: 12,
                       height: 12,
                       r: 4
                   },
                   fill: '#808080'
               }, {
                   shape: {
                       type: 'line',
                       x1: 3,
                       y1: 6,
                       x2: 9,
                       y2: 6
                   },
                   stroke: {
                       color: 'white',
                       width: 2
                   }
               }]
           }, {
               dojoAttachPoint: 'titleShape',
               shape: {
                   type: 'text',
                   text: '{{label}}',
                   x: 15,
                   y: 10,
                   align: 'start'
               },
               fill: '{{textColor}}',
               font: {
                   type: 'font',
                   size: '10pt',
                   family: 'sans-serif'
               },
               editable: { 
                   id:'label', 
                   editor:'textbox', 
                   options:{style:{'text-align':'left'}},
                   border:2,
                   baseRegion:{x:20,width:40,y:0,height:15}
               },
               selectedStyle: {
                   fill: '{{selectedTextColor}}'
               }
           }]
       }]
    }, {
        dojoAttachPoint: '_collapsedGroup',
        children: [{
            dojoAttachPoint: '_collapsedBaseShape',
            shape: {
                x: 0,
                y: 0,
                width: 80,
                height: 40,
                r: 5,
                type: 'rect'
            },
            fill: '{{backgroundColor}}',
            stroke: {
                'width': 2
            },
            selectedStyle: {
                fill: '{{selectedBackgroundColor}}'
            }
        }, {
            dojoAttachPoint: 'textShape',
            shape: {
                type: 'text',
                text: '{{label}}',
                x: 40,
                y: 24,
                align: 'middle'
            },
            fill: '{{textColor}}',
            selectedStyle: {
                fill: '{{selectedTextColor}}'
            },
            editable:{
               id:'label',
               editor:'textbox',
               border:2,
               baseRegion:{x:20,width:40,y:11,height:15}
           },
            font: {
                type: 'font',
                size: '10pt',
                family: 'sans-serif'
            }
        }, {
            dojoAttachPoint: '_expandButton',
           transform: { dx:2, dy:2 },
            children: [{
                shape: {
                    type: 'rect',
                    x: 0,
                    y: 0,
                    width: 12,
                    height: 12,
                    r: 4
                },
                fill: '#808080'
            }, {
                shape: {
                    type: 'line',
                    x1: 3,
                    y1: 6,
                    x2: 9,
                    y2: 6
                },
                stroke: 'white'
            }, {
                shape: {
                    type: 'line',
                    x1: 6,
                    y1: 3,
                    x2: 6,
                    y2: 9
                },
                stroke: {
                    color: 'white',
                    width: 2
                }
            }]
        }]
    }];

}

t.defaultOverviewExpandedSubgraphTemplate = [{
	dojoAttachPoint: 'baseShape',
	shape: {
		type: 'rect'
	},
	fill: '{{graphBackgroundColor}}',
	stroke: '#404040'
}];

t.defaultOverviewCollapsedSubgraphTemplate = [{
	dojoAttachPoint: 'baseShape',
	shape: {
		type: 'rect'
	},
	fill: '{{backgroundColor}}',
	stroke: '#404040'
}];


t.defaultNodeFocusAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
   	dojoAttachPoint: 'baseShape',
   	shape: {
       x: $bind("{{elemBaseLeft}}-3"),
       y: $bind("{{elemBaseTop}}-3"),
       width: $bind("{{elemBaseWidth}}+6"),
       height: $bind("{{elemBaseHeight}}+6"),
       type: 'rect'
		},
		stroke: {
			color: 'black',
			style: 'ShortDot'
		}
	}]
}];

t.defaultAnnotationTargetAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
	   	dojoAttachPoint: 'baseShape',
	   	shape: {
	       x: $bind("{{elemBaseLeft}}"),
	       y: $bind("{{elemBaseTop}}"),
	       width: $bind("{{elemBaseWidth}}"),
	       height: $bind("{{elemBaseHeight}}"),
	       type: 'rect'
		}
	}]
}];

if(dojo.config.useGfxLayout) {

	// ANNOTATION - GFX-LAYOUT TEMPLATE

	t.defaultAnnotationAdornerTemplate = [{
		dojoAttachPoint: 'adornerRoot',
    transform:{dx: $bind("{{elemBaseRight}}+15"),dy: $bind("{{elemBaseTop}}-25")},
		layout:{ type:'ibm_ilog.diagram.gfxlayout.GridLayout'},
		children:[{
		   	dojoAttachPoint: 'baseShape',
		   	shape: {
		       width: 1,
		       height: 1,
		       type: 'rect'
			},
			stroke: {
				color: [0,0,0,1],
				style: 'ShortDot',
				width: 1
			},
			fill: [222,222,150,0.75]
		},{
		   dojoAttachPoint: 'textShape',
			valign:'center',
			halign:'center',
			margins:4,
		   shape: {
		       type: 'text',
		       text: '{{annotation|safe}}',
		       x: 5,
		       y: 15,
		       align: 'middle'
		   },
		   fill: [0,0,0],
		   font: {
		       type: 'font',
		       size: '10pt',
		       family: 'sans-serif'
		   }
		}
		]
	}];
} else {
	
	// ANNOTATION - NON GFX-LAYOUT TEMPLATE
	
	t.defaultAnnotationAdornerTemplate = [{
		dojoAttachPoint: 'adornerRoot',
    transform:{dx: $bind("{{elemBaseRight}}+15"),dy: $bind("{{elemBaseTop}}-25")},
		children:[{
	   	dojoAttachPoint: 'baseShape',
	   	shape: {
	       width: 150,
	       height: 20,
	       type: 'rect'
			},
			stroke: {
				color: [120,120,0,1],
				style: 'ShortDot',
				width: 2
			},
			fill: [120,120,0,0.5]
		},{
		   dojoAttachPoint: 'textShape',
		   shape: {
		       type: 'text',
		       text: '{{annotation|safe}}',
		       x: 5,
		       y: 15,
		       align: 'left'
		   },
		   fill: [0,0,0],
		   font: {
		       type: 'font',
		       size: '10pt',
		       family: 'sans-serif'
		   }
		}
		]
	}];
}	

t.defaultAnnotationLinkTemplate = [{
    dojoAttachPoint: '_path',
    shape: {
        type: 'polyline',
        points: [{
            x: 0,
            y: 0
        }, {
            x: 100,
            y: 0
        }]
    },
		stroke: {
			color: [0,0,0,1],
			style: 'ShortDot',
			width: 1
		},
    selectedStyle: {
        stroke: {
            width: $bind("{{selectedStrokeWidth}}"),
            color: '{{selectedStrokeColor}}'
        }
    }
}];

t.defaultLinkFocusAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
	    dojoAttachPoint: '_path',
	    shape: {
	        type: 'polyline',
	        points: [{
	            x: 0,
	            y: 0
	        }, {
	            x: 0,
	            y: 0
	        }]
	    },
	    stroke: {
			 width: 5,
	        color: [255,0,255,0.5]
	    }
	}]
}];
t.defaultLinkFocusAdornerTemplate.noDtl = true;

t.defaultAdornerTemplates = t.defaultAdornerTemplates || new dojox.collections.Dictionary();

t.defaultAdornerTemplates.add("NodeFocusAdorner",t.defaultNodeFocusAdornerTemplate);
t.defaultAdornerTemplates.add("LinkFocusAdorner",t.defaultLinkFocusAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationAdorner",t.defaultAnnotationAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationTargetAdorner",t.defaultAnnotationTargetAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationLink",t.defaultAnnotationLinkTemplate);

t.defaultSwimLaneTemplate = "[{" +
"    dojoAttachPoint: '_expandedGroup'," +
"    layout:{" +
"       type:'ibm_ilog.diagram.gfxlayout.GridLayout'," +
" {% if vertical %}" +
"       rows:[{height:-1},{}]" +
" {% else %}" +
"  {% if rightToLeft %}" +
"       cols:[{},{width:-1}]" +
"  {% else %}" +
"       cols:[{width:-1},{}]" +
"  {% endif %}" +
" {% endif %}" +
"    }," +
"    children: [{" +
"           dojoAttachPoint: '_expandedBaseShape'," +
" {% if vertical %}" +
"           row:0, rowspan:2," +
" {% else %}" +
"           column:0, colspan:2," +
" {% endif %}" +
"           shape:{"+
"               type:'rect', width:0, height:0"+
"           },"+
"           fill:'{{headerColor}}'," +
"           selectedStyle: { "+
"               fill: '{{selectedHeaderColor}}'"+
"           }," +
"           stroke:'#404040'" +
"       }, {" +
"           dojoAttachPoint: '_header'," +
" {% if vertical %}" +
"           row:0, margins:[2,0,2,2]," +
"  {% if nested %}" +
"   		minimumSize: { width:-1, height:16 }," +
"  {% endif %}" +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:1, margins:[2,2,0,2]," +
"  {% else %}" +
"           column:0, margins:[0,2,2,2]," +
"  {% endif %}" +
"  {% if nested %}" +
"   		minimumSize: { width:16, height:-1 }," +
"  {% endif %}" +
" {% endif %}" +
"           layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'" +
" {% if not nested %}" +
"               ," +
"  {% if vertical %}" +
"               horizontalFlow:true, cols:[{width:-1},{}]" +
"  {% else %}" +
"               horizontalFlow:false, rows:[{height:-1},{}]" +
"  {% endif %}" +
" {% endif %}" +
"           }," +
"           children:["+
" {% if not nested %}" +
"             {" +
"               dojoAttachPoint: '_collapseButton',"+
" {% if vertical %}" +
"               valign:'center', halign:'left'," +
" {% else %}" +
"               halign:'center', valign:'top'," +
" {% endif %}" +
"               layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"               children: [{" +
"                   shape: {" +
"                       type: 'rect'," +
"                       x: 0," +
"                       y: 0," +
"                       width: 12," +
"                       height: 12" +
"                   }," +
"                   fill: '#808080'" +
"               },{ " +
"                   valign:'center',halign:'center'," +
"                   shape: {" +
"                       type: 'line'," +
"                       x1: 3," +
"                       y1: 6," +
"                       x2: 9," +
"                       y2: 6" +
"                   }," +
"                   stroke: {" +
"                       color: 'white'," +
"                       width: 2" +
"                   }" +
"               }]" +
"           },"+
" {% endif %}" +
"           {" +
"            	layout: {" +
"					type: 'ibm_ilog.diagram.gfxlayout.GridLayout'" +
"				}," +
"				children:[{" +
"               dojoAttachPoint: 'titleShape'," +
" {% if vertical %}" +
"               valign:'center', halign:'stretch'," +
" {% else %}" +
"               halign:'center', valign:'stretch'," +
"  {% if rightToLeft %}" +
"           	transform: {xx: 0, xy: -1, yx: 1, yy: 0}," +
"  {% else %}" +
"           	transform: {xx: 0, xy: 1, yx: -1, yy: 0}," +
"  {% endif %}" +
" {% endif %}" +
"               shape: {" +
"                   type: 'text'," +
"                   text: '{{label}}'," +
" {% if vertical %}" +
"                   align: 'middle'" +
" {% else %}" +
"  {% if rightToLeft %}" +
"                   align: 'middle'" +
"  {% else %}" +
"                   align: 'middle'" +
"  {% endif %}" +
" {% endif %}" +
"               }," +
"               fill: '{{textColor}}'," +
"               font: {" +
"                   type: 'font'," +
"                   size: '10pt'," +
"                   family: 'sans-serif'" +
"               }," +
"               editable: { " +
"                   id:'label', " +
"                   editor:'textbox', " +
"                   options:{style:{'text-align':'left'}}," +
"                   border:2," +
"			        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}}," +
"    		    	layoutProperties: {" +
" {% if vertical %}" +
"               valign:'center', halign:'left'" +
" {% else %}" +
"               halign:'center', valign:'top'" +
" {% endif %}" +
"                   }" +
"               },"+
"               selectedStyle: {" +
"                  fill: '{{selectedTextColor}}'" +
"               }" +
"				}]" +
"           }]" +
"       }, {" +
"           dojoAttachPoint: '_graphRect'," +
" {% if vertical %}" +
"           row:1," +
" {% else %}" +
"  {% if not rightToLeft %}" +
"           column:1," +
"  {% endif %}" +
" {% endif %}" +
"           shape:{"+
"               type:'rect',"+
"               width:0,height:0"+
"           }" +
"       }, {" +
"           dojoAttachPoint: '_graphBackground'," +
" {% if vertical %}" +
"           row:1," +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:0," +
"  {% else %}" +
"           column:1," +
"  {% endif %}" +
" {% endif %}" +
"           margins:1," +
"           shape:{"+
"               type:'rect',"+
"               width:0,height:0"+
"           }," +
"           fill:'{{graphBackgroundColor}}'" +
"       }, {" +
" {% if vertical %}" +
"           row:1, column:0," +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:0," +
"  {% else %}" +
"           column:1," +
"  {% endif %}" +
"           row:0," +
" {% endif %}" +
"           margins:0," +
"           halign:'left', valign:'top'," +
"           children: [{" +
"               shape:{"+
"                   type:'rect',"+
"                   x:0,y:0,width:0,height:0"+
"               }" +
"           }, {" +
"               dojoAttachPoint: '_graph'," +
"               shape:{"+
"                   type:'graph'"+
"               }" +
"           }]" +
"       }]" +
"   }, {" +
"   dojoAttachPoint: '_collapsedGroup'," +
"   layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"   children:[{"+ 
"     layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
" {% if vertical %}" +
"     halign:'left'," +
" {% else %}" +
"     valign:'top'," +
" {% endif %}" +
"     children:[{"+ 
"		 dojoAttachPoint: '_collapsedBaseShape'," +
"        shape: {" +
"            x: 0," +
"            y: 0," +
"            width: 80," +
"            height: 40," +
"            r: 5," +
"            type: 'rect'" +
"        }," +
"        fill: '{{backgroundColor}}'," +
"        stroke: {" +
"            'width': 2" +
"        }," +
"        selectedStyle: {" +
"            fill: '{{selectedBackgroundColor}}'" +
"        }" +
"   }," +
"   {" +
"        dojoAttachPoint: 'textShape'," +
"        margins:5," +
"        shape: {" +
"            type: 'mltext'," +
"            text: '{{label}}'," +
"            x: 40," +
"            y: 24," +
"	         wrap: {{wrap}}, width: 70," +
"            align: 'middle'" +
"        }," +
"        fill: '{{textColor}}'," +
"        selectedStyle: {" +
"            fill: '{{selectedTextColor}}'" +
"        }," +
"        editable:{" +
"           id:'label'," +
"           editor:'textbox'," +
"           border:2," +
"   		regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}}," +
"   		layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}" +
"        },"+
"        font: {" +
"            type: 'font'," +
"            size: '10pt'," +
"            family: 'sans-serif'" +
"        }" +
"   }," +
"   {" +
"       margins:4," +
"       valign:'top'," +
" {% if vertical %}" +
"       halign:'left'," +
" {% else %}" +
"  {% if rightToLeft %}" +
"       halign:'right'," +
"  {% else %}" +
"       halign:'left'," +
"  {% endif %}" +
" {% endif %}" +
"       dojoAttachPoint: '_expandButton'," +
"       layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"       children:[{" +
"            shape: {" +
"                type: 'rect'," +
"                x: 0," +
"                y: 0," +
"                width: 12," +
"                height: 12," +
"                r: 4" +
"            }," +
"            fill: '#808080'" +
"        }, {" +
"            valign:'center',halign:'center'," +
"            shape: {" +
"                type: 'line'," +
"                x1: 3," +
"                y1: 6," +
"                x2: 9," +
"                y2: 6" +
"            }," +
"            stroke: 'white'" +
"        }, {" +
"           valign:'center',halign:'center'," +
"           shape: {" +
"                type: 'line'," +
"                x1: 6," +
"                y1: 3," +
"                x2: 6," +
"                y2: 9" +
"            }," +
"            stroke: {" +
"                color: 'white'," +
"                width: 2" +
"            }" +
"       }" +
"       ]" +
"     }" +
"     ]" +
"   }" +
"   ]" +
"}" +
"]";

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout._base']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout._base'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout._base');





dojo.requireIf(dojo.config.isBidiInDojoDiagrammer, 'ibm_ilog.diagram.util.BidiUtil');

(function(){
	var is16 = dojo.version.major > 1 || (dojo.version.major == 1 && dojo.version.minor >= 6);
	
	// ensures the explicitSize is updated when the font is changed.
	ibm_ilog.diagram._base.gfxext.patchMethod(dojox.gfx.Text, "setFont", null, function(){
		this.__shapeSet();
	});
	
	// Returns the size of the given string when rendered by the specified Text shape.
	var getTextSize = function(/*dojox.gfx.Text*/sh, /*String*/ txt){
		if (!txt || txt.length == 0) 
			return null;
		if (ibm_ilog.diagram.isSvg && !ibm_ilog.diagram.isSvgWeb) {
			if (!sh.rawNode.firstChild) 
				return null;
			//Fix for svg in FF. When the text starts with lamed, tzadik, may be other letters, the text width after Bidi conversion is smaller, than 
			//the width of the text before convertion, we have to apply Bidi conversion on the text before measurement, to prevent 
			//additional truncation
			if (dojo.config.isBidiInDojoDiagrammer && sh.getShape().__beforeBidi && dojo.isFF) {
				var shape = dojo.clone(sh.getShape());
				shape.text = txt;
				shape = ibm_ilog.diagram.util.BidiUtil.bidiFormat.call(sh, shape);
				txt = shape.text;
			}
			// summary: get the text width in pixels
			var rawNode = sh.rawNode, oldParent = rawNode.parentNode, _measurementNode = rawNode.cloneNode(true);
			_measurementNode.style.visibility = "hidden";
			// solution to the "orphan issue" in FF
			var _height = 0, _width = 0, _x = 0, _y = 0;
			_measurementNode.firstChild.nodeValue = txt;
			oldParent.appendChild(_measurementNode);
			try {
				var bbox = _measurementNode.getBBox();
				_width = parseInt(bbox.width, 10);
				_height = parseInt(bbox.height, 10);
							
			} catch (e){return null;}
			oldParent.removeChild(_measurementNode);
			return {
				width: _width,
				height: _height
			};
		} else if (ibm_ilog.diagram.isVml) {
			var oldShape = sh.getShape();
			
			var rawNode = sh.rawNode, _display = rawNode.style.display;
			sh._suspendInvalidate = true;
			sh.setShape({
				text: txt
			});
			sh.getTextWidth();
			var _width = dojox.gfx.pt2px(parseFloat(rawNode.currentStyle.width));
			var _height = dojox.gfx.pt2px(parseFloat(rawNode.currentStyle.height));
			// in VML, the width/height we get are in view coordinates
			var m = sh._getRealMatrix();
			if (m && !m.isIdentity()) {
				var sz = {
					x: 0,
					y: 0,
					width: _width,
					height: _height
				};
				sz = m.inverse().transformRectangle(sz);
				_width = sz.width;
				_height = sz.height;
			}
			sh.setShape(oldShape);
			sh._suspendInvalidate = false;
			return {
				width: _width,
				height: _height
			};
			
			/*
			 var rawNode = sh.rawNode, oldParent = rawNode.parentNode,
			 _measurementNode = rawNode.cloneNode(true);
			 oldParent.appendChild(_measurementNode);
			 _measurementNode.style.display = "inline";
			 // find text path
			 var t = null, c = _measurementNode.childNodes;
			 for(var i = 0; i < c.length; ++i){
			 var tag = c[i].tagName;
			 if(tag == "textpath"){
			 t = c[i];
			 }
			 }
			 t.string = txt;
			 t.on = true;
			 
			 
			 var bcr = _measurementNode.getBoundingClientRect();
			 console.log((bcr.right - bcr.left) + " " + (bcr.bottom - bcr.top));
			 return {x:bcr.left, y:bcr.top, width: bcr.width || (bcr.right - bcr.left), height: bcr.height || (bcr.bottom - bcr.top)};
			 */
		} else if (ibm_ilog.diagram.isSilverlight) {
			var oldtext = sh.rawNode.text;
			sh.rawNode.text = txt;
			var bb = {
				width: sh.rawNode.actualWidth,
				height: sh.rawNode.actualHeight
			};
			sh.rawNode.text = oldtext;
			return bb;
		} else if (sh.getTextWidth) {
			if(ibm_ilog.diagram.isSvgWeb && !sh.parent)
				return null;
			// this works for Canvas
			var oldtext = sh.shape.text;
			sh.shape.text = txt;
			var w = sh.getTextWidth();
			sh.shape.text = oldtext;
			var font = sh.getFont();
			var fz = font ? font.size : dojox.gfx.defaultFont.size;
			var h = dojox.gfx.normalizedLength(fz);
			return {
				width: w,
				height: h
			};
		}
	};
	
	//
	// An extension to gfx Shape classes to add features required by the layout system.
	//
	ibm_ilog.diagram.gfxlayout._LayoutShapeExt = {
	
		_explicitSize: null,
		
		// Called when a setShape (or any methods that modify the bbox) is invoked.
		__shapeSet: function(){
			if (this._suspendInvalidate) {
				return;
			}
			// getBoundingBox may return null (for ex when the Text is not in the hierarchy yet)
			var bbox = this.getBoundingBox();
			// in Chrome, a (0,0,0,0) bbox is returned for undetermined node state instead of null.
			// So we treat such a bbox as null to force the _expliciteSize computation later
			// (see defect 2105)
			if (bbox && bbox.x === 0 && bbox.y === 0 && bbox.width === 0 && bbox.height === 0)
				bbox = null;
			this._explicitSize = bbox;
		},
		
		getExplicitSize: function(){
			// summary:
			//     Returns the explicit size of this shape.
			// description: 
			//     This method returns the size as computed by the getBoundingBox() method
			//     of the shape explicit geometry (that is, the geometry explicitely set and
			//     not the geometry computed by the layout).
			return this._explicitSize;
		},
		
		getPreferredSize: function(){
			// summary:
			// returns the preferred size as computed by the computePreferredSize() method.
			
			return {
				width: this._preferredSize.width,
				height: this._preferredSize.height
			};
		},
		
		computePreferredSize: function(/*Size*/proposedSize){
			// summary:
			//     Computes the preferred size of this shape according to the specified proposed size,
			//     and ensures it is conformed with the minimum and maximum size of this shape.
			// proposedSize: Size: the proposed size.        
			
			var sz = this._computePreferredSize(proposedSize);
			this._preferredSize = this._checkMinMaxSize(sz);
			return {
				width: this._preferredSize.width,
				height: this._preferredSize.height
			};
		},
		
		/*protected*/
		_computePreferredSize: function(/*Size*/proposedSize){
			// summary:
			//     Computes the preferred size of this shape. 
			// description:
			//     This method is invoked by the public computePreferredSize() method. Subclasses should override
			//     this method for specific implementation.
			
			var rect = this._explicitSize;
			if (!rect) 
				rect = this._explicitSize = this.getBoundingBox();
			if (!rect) 
				rect = ibm_ilog.diagram.util.Geometry.EmptyRect;
			var t = this.getTransform();
			if (t) {
				// do not modify rect as it breaks under IE9
				var rt = {
					x: rect.x || 0,
					y: rect.y || 0,
					width: rect.width,
					height: rect.height
				}
				rect = t.transformRectangle(rt);
			}
			return {
				width: rect.width,
				height: rect.height
			};
		},
		
		_checkMinMaxSize: function(/*Size*/bounds){
			// summary:
			//     Ensures the specified bounds fits the minimum and maximum size.
			
			return this._checkMaxSize(this._checkMinSize(bounds));
		},
		
		_checkMaxSize: function(bounds){
			// summary:
			//     Ensures the specified bounds fits the maximum size.
			
			var sz = this.maximumSize;
			if (sz && sz.height) {
				bounds.height = Math.min(sz.height, bounds.height);
			}
			if (sz && sz.width) {
				bounds.width = Math.min(sz.width, bounds.width);
			}
			return bounds;
		},
		
		_checkMinSize: function(bounds){
			// summary:
			//     Ensures the specified bounds fits the minimum size.
			
			var sz = this.minimumSize;
			if (sz && sz.height) {
				bounds.height = Math.max(sz.height, bounds.height);
			}
			if (sz && sz.width) {
				bounds.width = Math.max(sz.width, bounds.width);
			}
			return bounds;
		},
		
		//_suspendInvalidate: false,
		_suspendInvalidate: 0,
		
		suspendInvalidate: function(){
			// summary:
			//    Suspends the invalidate propagation to the parent hierarchy.
			
			//this._suspendInvalidate = true;
			++this._suspendInvalidate;
		},
		
		resumeInvalidate: function(/*Boolean*/invalidate){
			// summary:
			//     Resumes the invalidate propagation to the parent hierarchy.
			// invalidate: Boolean: Indicates whether a call to invalidate() should be done.
			
			//this._suspendInvalidate = false;
			
			--this._suspendInvalidate;
			if (this._suspendInvalidate < 0) {
				this._suspendInvalidate = 0;
				// TODO: throw Error
			}
			if (invalidate && !this._suspendInvalidate) 
				//if (invalidate) 
				this.invalidate();
		},
		
		invalidate: function(){
			// summary:
			//     Invalidates this shape.
			// description:
			//     Call this method when a change occurs on this shape that could impact its
			//     bounding box. The default implementation propagtes the invalidation to
			//     the parent hierarchy.
			if (this._suspendInvalidate) 
				return;
			var p = this.getParent();
			if (p && p.invalidate) {
				// indicates the size must be recomputed for all the parent layout chain
				p.invalidate();
			}
		},
		
		layout: function(/*Rectangle*/rect){
			// summary:
			//     Lays out this shape into the specified rectangle.
			
			try {
				this.suspendInvalidate();
				this.applyLayout(rect);
			} finally {
				this.resumeInvalidate();
			}
		},
		
		/*protected*/
		applyLayout: function(/*Rectangle*/rect){
			// summary:
			//     Invoked by the public layout() method to layout this shape into this specified rect.
			//     Subclasses should override this method to implement their own logic. The default
			//     implementation computes the transform needed to fit the specified rect.
			var t = this.getTransform() || dojox.gfx.matrix.identity;
			// 'disable' the transform set previously by the layout
			// so here, t is all the transforms except the one from the layout
			if (this.__oldLayoutTransform && !this.__oldLayoutTransform.isIdentity()) {
				t = dojox.gfx.matrix.multiply(this.__oldLayoutTransform.inverse(), t);
			}
			var bbox = this.getBoundingBox();
			if (t && bbox) 
				bbox = t.transformRectangle(bbox);
			if (bbox) {
				this.__oldLayoutTransform = dojox.gfx.matrix.createFromRectangles(bbox, rect);
				t = dojox.gfx.matrix.multiply(this.__oldLayoutTransform,t);
				this.setTransform(t);
			} else {
				var dx = rect.x, dy = rect.y;
				if (dx != 0 || dy != 0) {
					this.__oldLayoutTransform = dojox.gfx.matrix.translate(dx, dy);
					t = dojox.gfx.matrix.multiply(this.__oldLayoutTransform, t);
					this.setTransform(t);
				} else {
					this.__oldLayoutTransform = null;
				}
				
			}
		}
	};
	
	dojo.extend(is16 ? dojox.gfx.shape.Shape : dojox.gfx.Shape, ibm_ilog.diagram.gfxlayout._LayoutShapeExt);
	
	//
	// A fix for missing gfx.Text.getBoundingBox
	//
	
	ibm_ilog.diagram.gfxlayout._ellipsis = String.fromCharCode(8230);
	
	var _truncate = function(txt, width){
		var t = txt, c = txt.length, tw = getTextSize(this, t);
		tw = tw ? tw.width : 0;
		while (c > 0 && tw > width) {
			// drop 2 chars
			c -= 2;
			// and count ellipsis
			t = txt.slice(0, c) + ibm_ilog.diagram.gfxlayout._ellipsis;
			tw = getTextSize(this, t);
			tw = tw ? tw.width : 0;
		}
		// t='...' and still > width
		if (tw > width) 
			t = '';
		return t;
	};
	
	dojo.extend(dojox.gfx.Text, {
	
		_truncateMode: 'default',
		
		_truncatesFn: _truncate,
		
		setTruncateMode: function(mode){
			// summary:
			// 		Sets the method used to truncate the text.
			// description:
			//		This method is called by the applyLayout() function when the text set on this shape
			//		is wider than the available space so that it fits into the specified space. By default,
			//		the text is truncated with ellipsis as much as needed to fit the available width. You can
			//		specify a custom function or disable the behavior. The custom Function should obey the
			//		following specification:
			//		function foo(text, width) {
			//			// summary: Returns the new text.
			//			// text: String : the text to truncate.
			//			// width: Number: the slot width.
			//			// returns : the truncated text.
			//		}
			// mode: null||'default'||Function
			//		null to disable, 'default' to enable the default mode, or a Function for a custom implementation.
			
			if (mode != this._truncateMode) {
				var old = this._truncateMode;
				this._truncateMode = mode;
				if (!mode) {
					this._truncatesFn = null;
				} else if (mode == 'default') {
					this._truncatesFn = _truncate;
				} else if (dojo.isFunction(mode)) {
					this._truncatesFn = mode;
				}
				this.invalidate();
			}
		},
		
		getTruncateMode: function(){
			// summary:
			//		Returns the truncate mode.
			
			return this._truncateMode;
		},
		
		applyLayout: function(rect){
			// fix for #1855: on IE, when the text is changed, setShape calls setTransform internally,
			// this will trigger a layout before __setShape was called, thus before ___realtext was set to the new text.
			// So, to know if __realtext is valid, we also check the sh.text against a __disptext (displayed text) property,
			// that we set (below in this method) to the actually displayed (truncated) text.
			var sh = this.getShape(), nsh = {}, txt = (this.__realtext && this.__disptext == sh.text) ? this.__realtext : sh.text, sz = getTextSize(this, txt), sbbox = this.getBoundingBox();
			// if bbox is null (means not in the dom or hidden)
			// do nothing
			if (!sbbox) 
				return;
			
			// If the text is transformed, first apply the inverse transform to the rect:
			var tr = this.getTransform();
			if (tr && !tr.isIdentity()) 
				rect = tr.inverse().transformRectangle(rect);
			
			// Compute the text origin
			
			var h = sz.height, w = sz.width;
			
			switch (sh.align) {
				case 'start':
					nsh.x = rect.x;
					break;
				case 'middle':
					nsh.x = rect.x + rect.width / 2;
					break;
				case 'end':
					nsh.x = rect.x + rect.width;
			}
			// Patch for Canvas: if dir="rtl", text is displayed rtl relative to the align position
			// (I guess maybe SVG should do the same, but the fact is that it does not.
			//  Maybe one day we'll need to do the same for SVG?...)
			if(ibm_ilog.diagram.isCanvas && (sh.align == 'start' || sh.align == 'end')){
				// go up to the GFX Surface to find a DOM node
				for(var p = this.getParent(); p; p = p.getParent()){
					if(p.declaredClass == dojox.gfx.Surface.prototype.declaredClass){
						// Look at the computed 'direction' style,
						// seems the easier way to get the inherited direction value...
						var style = dojo.getComputedStyle(p.rawNode);
						if(style && style.direction == "rtl"){
							if(sh.align == 'start')
								nsh.x += rect.width;
							else
								nsh.x -= rect.width;
						}
						break;
					}
				}
			}
			var ascent = sh.y - sbbox.y;
			nsh.y = rect.y + (rect.height - h) / 2 + ascent;
			
			// check if text is larger than the given rect. 
			// In this case: cut the text and add ellipsis
			if (this._truncatesFn && txt && txt.length > 0 && Math.round(rect.width) > 0 && w > rect.width) {
				txt = this._truncatesFn.call(this, txt, rect.width);
			}
			nsh.text = txt;
			this.__disptext = txt;
			if (dojo.config.isBidiInDojoDiagrammer) {
				if (sh.textDir) 
					nsh.textDir = sh.textDir;
				if (sh.ceType) 
					nsh.ceType = sh.ceType;
			}
			this.__setShapeInt = true;
			try {
				this.setShape(nsh);
			} finally {
				this.__setShapeInt = false;
			}
		},
		
		__shapeSet: function(callee){
			if (!this.__setShapeInt && callee === 'setShape') {
				if (dojo.config.isBidiInDojoDiagrammer) {
					var tt = this.getShape().__beforeBidi;
					this.__realtext = tt ? tt : this.getShape().text;
				} else 
					this.__realtext = this.getShape().text;
			}
			
			// under IE, we need to reset the explicitSize, as the text bbox may be computed
			// in the createShape() call, and at this time, the _suspendInvalidate is not called yet
			// and the node might be in an incomplete state.	
			if (ibm_ilog.diagram.isVml && this._suspendInvalidate) {
				this._explicitSize = null;
				return;
			}
			ibm_ilog.diagram.gfxlayout._LayoutShapeExt.__shapeSet.call(this);
		}
	});
	
	dojo.extend(dojox.gfx.Rect, {
		applyLayout: function(rect){
			var t = this.getTransform();
			if (t && !t.isIdentity()) {
				//TODO: if t is translation:=> do a resize instead
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(dojox.gfx.matrix.createFromRectangles(oldBounds, rect));
			} else {
				this.setShape(rect);
			}
		}
	});
	
	dojo.extend(dojox.gfx.Ellipse, {
		applyLayout: function(rect){
			var t = this.getTransform();
			//TODO: if t is translation:=> do a resize instead
			if (t && !t.isIdentity()) {
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(dojox.gfx.matrix.createFromRectangles(oldBounds, rect));
			} else {
				var sh = {
					cx: rect.x + rect.width / 2,
					cy: rect.y + rect.height / 2,
					rx: rect.width / 2,
					ry: rect.height / 2
				};
				this.setShape(sh);
			}
		}
	});
	
	dojo.extend(dojox.gfx.Circle, {
		applyLayout: function(rect){
			var t = this.getTransform();
			//TODO: if t is translation:=> do a resize instead
			if (t && !t.isIdentity()) {
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(dojox.gfx.matrix.createFromRectangles(oldBounds, rect));
			} else {
				var sh = {
					cx: rect.x + rect.width / 2,
					cy: rect.y + rect.height / 2,
					r: Math.min(rect.width / 2, rect.height / 2)
				};
				this.setShape(sh);
			}
		}
	});
	
	//
	// An extension to gfx Group to add layout capabilities.
	//
	ibm_ilog.diagram.gfxlayout._PanelExt = {
	
		_ibmd_layoutMgr: null,
		
		__shapeSet: function(){
		
		},
		
		setLayout: function(/*ibm_ilog.diagram.gfxlayout.Layout || Object*/layoutOpt){
			// summary:
			//     Sets the layout manager for this container.
			// layoutOpt: ibm_ilog.diagram.gfxlayout.Layout || layoutOpt:
			//     Either a layout instance or a plain JS object that describes the layout to instantiate and its parameters.
			//     In case it is a plain JS object, the object must contain a 'type' property of type String that gives the
			//     name of the Layout subclass to instantiate. The 'layoutOpt' parameter is then passed to the constructor to
			//     configure the layout.
			
			if (this._ibmd_layoutMgr) 
				this._ibmd_layoutMgr._detach();
			var layout = null;
			// layoutOpt is either a Layout instance or a plain JS object that describes the class and its parameters.
			if (layoutOpt instanceof ibm_ilog.diagram.gfxlayout.Layout) {
				layout = layoutOpt;
			} else if (layoutOpt) {
				var t = layoutOpt.type;
				if (!t) 
					ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "setLayout", "layoutOpt.type", layoutOpt);
				var type = dojo.getObject(t, false);
				if (dojo.isFunction(type)) {
					layout = new type(layoutOpt);
				} else {
					ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "setLayout", "layoutOpt.type", layoutOpt);
				}
			}
			this._ibmd_layoutMgr = layout;
			if (this._ibmd_layoutMgr) {
				this._ibmd_layoutMgr._attach(this);
				this._ibmd_layoutMgr.invalidate();
			}
		},
		
		getLayout: function(){
			// summary:
			//     Gets the layout manager set on this container.
			
			return this._ibmd_layoutMgr; /*ibm_ilog.diagram.gfxlayout.Layout*/
		},
		
		_needsLayout: false,
		
		needsLayout: function(){
			// summary:
			//     Marks this container as needed to be layed out the next pass.
			this._needsLayout = true;
		},
		
		_oldLayoutRect: null,
		
		applyLayout: function(/*Rectangle*/rect){
			// summary: Lays out this container.
			// rect: Rectangle: 
			
			if (this._ibmd_layoutMgr) {
				// check a layout is needed: either because computePreferredSize as marked it as needed, or
				// if the rect is different from the previous one.
				var doIt = this._needsLayout || !this._oldLayoutRect ||
				this._oldLayoutRect.width !== rect.width ||
				this._oldLayoutRect.height !== rect.height;
				if (doIt) {
					this._oldLayoutRect = dojo.clone(rect);
					try {
						// Delegate to the layout manager.
						// Layout is done relative to the parents (ie. (0,0) )
						this._ibmd_layoutMgr.doLayout({
							x: 0,
							y: 0,
							width: rect.width,
							height: rect.height
						});
					} finally {
						this._needsLayout = false;
					}
				}
				// then positioned this container to the right coord
				//
				// remove the previous transform set by the layout
				var t = this.getTransform() || dojox.gfx.matrix.identity;
				// 'disable' the transform set previously by the layout
				// so here, t is all the transforms except the one from the layout
				if (this.__oldLayoutTransform && !this.__oldLayoutTransform.isIdentity()) {
					t = dojox.gfx.matrix.multiply(this.__oldLayoutTransform.inverse(), t);
				}
				var bbox = this.getBoundingBox();
				if (t && bbox) 
					bbox = t.transformRectangle(bbox);
				var dx = rect.x, dy = rect.y;
				if (dx != 0 || dy != 0) {
					this.__oldLayoutTransform = dojox.gfx.matrix.translate(dx, dy);
					t = dojox.gfx.matrix.multiply(this.__oldLayoutTransform,t);
					var oldt = this.getTransform();
					if (!(oldt != null && oldt.isSame(t))) 
						this.setTransform(t);
				} else {
					this.__oldLayoutTransform = null;
				}
			} else {
				ibm_ilog.diagram.gfxlayout._LayoutShapeExt.applyLayout.call(this, rect);
			}
		},
		
		_recomputeSize: false,
		
		invalidate: function(){
			// summary:
			//     Invalidates this shape.
			
			if (this._suspendInvalidate) 
				return;
			var p = this.getParent();
			var lmgr = this._ibmd_layoutMgr;
			// do nothing if a layout is running or no parent and no layout        
			if ((!lmgr && !p) || (lmgr && lmgr.inLayout())) 
				return;
			// marks the panel so that its preferred size needs to be recomputed the next pass.
			this._recomputeSize = true;
			// propagate to the parent hierarchy
			if (p && p._ibmd_layoutMgr) 
				p.invalidate();
			else if (lmgr) 
				// no parent or no parent panel, invalidate this one
				ibm_ilog.diagram.gfxlayout.postInvalidate(this);
		},
		
		_oldProposedSize: null,
		
		_computePreferredSize: function(proposedSize){
			// summary:
			//     Computes the preferred size of this panel.
			
			var sz;
			// empty ? nothing to do
			if (this.children.length == 0) 
				return this.getExplicitSize();
			if (this._ibmd_layoutMgr) {
				// recompute the size only if it has been explicitely marked as needed
				// or if the proposed size is different from the previous one.
				if (this._explicitSize) {
					proposedSize = this._explicitSize;
				}
				if (this._recomputeSize === true) {
					try {
						sz = this._computeLayoutPreferredSize(proposedSize);
						this.needsLayout();
					} finally {
						this._recomputeSize = false;
					}
				} else if (!this._oldProposedSize ||
				proposedSize.width !== this._oldProposedSize.width ||
				proposedSize.height !== this._oldProposedSize.height) {
				
					sz = this._computeLayoutPreferredSize(proposedSize);
					this.needsLayout();
				} else {
					// do not recompute the size.
					sz = this.getPreferredSize();
				}
			} else {
				// call bbox
				sz = ibm_ilog.diagram.gfxlayout._LayoutShapeExt._computePreferredSize.call(this, proposedSize);
			}
			return sz;
		},
		
		_computeLayoutPreferredSize: function(proposedSize){
			// summary:
			//     Computes the preferred size with the layout.
			
			this._oldProposedSize = {
				width: proposedSize.width,
				height: proposedSize.height
			};
			var sz = this._ibmd_layoutMgr.computePreferredSize(proposedSize);
			var t = this.getTransform();
			if (t) {
				var rect = {
					x: 0,
					y: 0,
					width: sz.width,
					height: sz.height
				};
				rect = t.transformRectangle(rect);
				sz.width = rect.width;
				sz.height = rect.height;
			}
			return sz;
		},
		
		getExplicitSize: function(){
			// summary:
			//     Returns the explicit size of this container.
			
			if (!this._explicitSize) {
				this._explicitSize = {
					width: -1, // = auto
					height: -1 // = auto
				};
			}
			return {
				width: this._explicitSize.width,
				height: this._explicitSize.height
			};
		},
		
		setExplicitSize: function(size){
			// summary:
			//     Sets the explicit size of this shape.
			
			this._explicitSize = {
				width: size.width,
				height: size.height
			};
			this.invalidate();
		},
		
		_computeBoundingBox: function(){
			// if it has been marked as modified, ensure it is validated before computing the bounding box
			if (this._recomputeSize && this._ibmd_layoutMgr) 
				ibm_ilog.diagram.gfxlayout.doPendingValidate(this);
			var bbox = ibm_ilog.diagram.BoundingBoxFix._computeBoundingBox.call(this);
			if (bbox && this._ibmd_layoutMgr) {
				return this._ibmd_layoutMgr.adjustBBox(bbox);
			} else {
				return bbox;
			}
		}
	};
	
	dojo.extend(dojox.gfx.Group, ibm_ilog.diagram.gfxlayout._PanelExt);
	
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.Engine']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.Engine'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.Engine');



//
// public interface to the layout engine singleton.
//

ibm_ilog.diagram.gfxlayout.postInvalidate = function(/*dojox.gfx.Group*/obj) {
    // summary:
    //     Posts an asynchronous validation request for the specified Panel.
    ibm_ilog.diagram.gfxlayout.Engine.invalidate(/*dojox.gfx.Shape*/obj);
};

ibm_ilog.diagram.gfxlayout.doPendingValidate=function(/*dojox.gfx.Group*/obj){
    // summary:
    //     Processes immediately any validation request registered for the specified panel.
    ibm_ilog.diagram.gfxlayout.Engine.doPendingValidate(obj);
};

//
// Layout engine (Singleton).
// 
ibm_ilog.diagram.gfxlayout.Engine = (function() {
    // the singleton instance
    var instance = null;
    
    function Ctor() {
        
        // the validation requests queue
        var _queue = [];
        
        // Posts an asynchronous validation request for the specified shape.
        this.invalidate = function(/*dojox.gfx.Shape*/obj) {
            
            // ensures the shape is in the hierarchy
            if (this._isOrphan(obj)) {
                return;
            }
            // find the top-level parent
            var topPnl = obj;
            while (topPnl.getParent() &&
                   topPnl.getParent().getLayout &&
                   topPnl.getParent().getLayout()) {
                topPnl = topPnl.getParent();
            }
            // no -panel- found (ie Group with a layout mgr)
            if (!topPnl.getLayout || !topPnl.getLayout()) {
                return;
            }
            // invalidate() invoked during the layout process ?
            if (topPnl.getLayout().inLayout()) 
                return;
            // if the top panel found is already in the pending queue,
            // do nothing
            if (dojo.some(_queue, function(item) {
                if (item === topPnl) 
                    return true; // stop loop
                })) {
                return; // already in the queue, do nothing
            }
            // add it to the pending validation queue
            _queue.push(topPnl);
            // add a layout() request to the Batch
            ibm_ilog.diagram.util.Batch.add(this);
        };
        
        // Processes immediately the validation requests registered for the specified shape.
        this.doPendingValidate=function(/*dojox.gfx.Shape*/obj){
            var topPnl = obj;
            while (topPnl.getParent() &&
                   topPnl.getParent().getLayout()) {
                topPnl = topPnl.getParent();
            }
            if (topPnl && topPnl.getLayout()){
                if (topPnl.getLayout().inLayout()) 
                    return;
                // look up the panel in the queue
                var ql = _queue.length;
				for (var i=0;i<ql;++i){
                    if (_queue[i] === topPnl) {
                        _queue[i] = null;
                        // Layout it
                        this._layout(topPnl);
                        break;
                    }
                }
            }            
        };
        
        // Invoked by the Batch to trigger the pending layout() request
        this._endBatch = function() {
            this.layout();
        };
        
        this.layout = function() {
            // init a new pending queue
            var queue = _queue;
            _queue = [];
			var fl = this._layout;
            var ql = queue.length;
            for (var i = 0; i < ql; ++i) {
                var panel = queue[i];
                if (!panel) // may have been removed by a doPendingValidate()
                    continue;
                fl.call(this, panel);
            }
        };
        
        this._layout = function(/*dojox.gfx.Group*/panel){
            //console.log("Engine layout");
            // is it still in the hierarchy ? may have been removed since then
            if (this._isOrphan(panel)) {
                return;
            }
            // get the explicit size of the top level panel to validate.
            // by default it's w=-1 (auto) and h=-1 (auto)
            var size = panel.getExplicitSize();
            // computes children size
            size = panel.computePreferredSize(size);
            // layout the panel
            panel.layout({x:0, y:0, width:size.width, height:size.height});
        };
        
        this._isOrphan = function(shape){
            // a Surface does not have getParent() method.
            for (var p = shape.getParent();p && p.getParent; p=p.getParent()){
            }
            return p === null;
        };
        
    };
    
    return new function() {
        if (instance == null) {
            instance = new Ctor();
            instance.constructor = null;
        }
        return instance;
    };
})();


}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.Layout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.Layout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.Layout');



dojo.declare('ibm_ilog.diagram.gfxlayout.Layout', null, {

    // the Group attached to this layout
    _container: null,
    
    
    // summary:
    //     The base class for Layout implementations.
    // description:
    //     A Layout instance is responsible for arranging the children of a GFX container
    //     according to positioning and sizing constraints.
    
    constructor: function(args) {
        // summary:
        //     Creates and initializes a new ibm_ilog.diagram.gfxlayout.Layout instance.
        // args : Object : a mixin set of parameters.
        
        dojo.mixin(this, args);
    },
	
	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {}; /*Object*/
	},
    
    _attach: function(/*dojox.gfx.Group*/container) {
        // summary:
        //     Associates this layout to the specified container.
        // container: dojox.gfx.Group:
        //     The container to associate.
        
        this._container = container;
    },
    
    _detach: function() {
        // summary:
        //     Detach this layout from its associated container.
        
        this._container = null;
    },
    
    onLayoutPropertyChanged: function() {
        // summary:
        //     Fires a layout property changed event.
        // description:
        //     This method should be called by layout subclasses when a property change needs
        //     the layout to be refreshed. 
        
        this.invalidate();
    },
    
    invalidate: function() {
        // summary:
        //     Invalidates this layout.
        // description:
        //     This method post a layout request to the layout engine.
        
        if (this.__inLayout) 
            return;
        //ibm_ilog.diagram.gfxlayout.postInvalidate(this._container);
        this._container.invalidate();
    },
    
    
    computePreferredSize: function(proposedSize) {
        // summary:
        //     compute the preferred size of the associated container according to the layout strategy.
        
        return proposedSize;
    },
    
    adjustBBox : function(/*Rectangle*/bbox){
        // summary:
        //     Adjusts the specified rectangle according to the constraints of this layout.
        return bbox;
    },

    inLayout:function(){
        // summary:
        //     Indicates whether this layout is in a layout session.
        return this.__inLayout; 
    },
    
    doLayout: function(/*Rect*/contBounds) {
        // summary:
        //     Applies this layout to the associated container.
        
        try {
            this.__inLayout = true;
            this.layoutChildren(contBounds);        
        } finally {
            this.__inLayout = false;
        }
    },
    
    layoutChildren: function(/*Rect*/contBounds) {
        // summary:
        //     Applies this layout strategy to the associated container within the specified bounds.
        //     Subclass must override this method to implement their own layout logic. 
    },
    
    placeChildInSlot: function(/*dojox.gfx.Shape*/shape, /*dojox.gfx.Rectangle*/slotBounds, /*Boolean*/hAlign, /*Boolean*/vAlign) {
        // summary:
        //     Places the specified object in the given slotBounds.
        // description:
        //     This method should be invoked by layout implementation to place a shape within
        //     the given slot bounds, taking into account the vertical and horizontal alignment
        //     contraints of the shape (specified by means of the 'valign' and 'halign' properties
        //     on the shape object) and the margins, if any (specified by means of the 'margins'
        //     property.
        // shape : dojox.gfx.Shape: the shape.
        // slotBounds : Rectangle : the slot bounds (i.e the bounds where the shape must lie in)
        // hAlign: Boolean : indicates whether the shape must be aligned horizontally according to
        //      the value of its halign property.
        // vAlign: Boolean : indicates whether the shape must be aligned vertically according to
        //      the value of its valign property.
        
        var checkW = true, checkH = true, shapeBounds = shape.getPreferredSize();
        if (shape.margins){
            slotBounds = this._applyMargins(slotBounds, shape.margins);
        }
        if (hAlign) {
            if (!vAlign) {
                shapeBounds.y = slotBounds.y;
                shapeBounds.height = slotBounds.height;
            }
            var halign = shape.halign || 'stretch';
            switch (halign) {
                case 'left':
                    shapeBounds.x = slotBounds.x;
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    break;
                case 'center':
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    shapeBounds.x = slotBounds.x + (slotBounds.width - shapeBounds.width) / 2;
                    break;
                case 'right':
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    shapeBounds.x = slotBounds.x + slotBounds.width - shapeBounds.width;
                    break;
                default:
                case 'stretch':
                    shapeBounds.width = slotBounds.width;
			        shapeBounds = shape._checkMinMaxSize(shapeBounds);
                    shapeBounds.x = slotBounds.x + (slotBounds.width - shapeBounds.width)/2;
                    checkW = false;
					break;
            }
        }
        if (vAlign){
            if (!hAlign) {
                shapeBounds.x = slotBounds.x;
                shapeBounds.width = slotBounds.width;
            }
            var valign = shape.valign || 'stretch';
            switch (valign) {
                case 'top':
                    shapeBounds.y = slotBounds.y;
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    break;
                case 'center':
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    shapeBounds.y = slotBounds.y + (slotBounds.height - shapeBounds.height) / 2;
                    break;
                case 'bottom':
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    shapeBounds.y = slotBounds.y + slotBounds.height - shapeBounds.height;
                    break;
                default:
                case 'stretch':
                    shapeBounds.height = slotBounds.height;
			        shapeBounds = shape._checkMinMaxSize(shapeBounds);
                    shapeBounds.y = slotBounds.y + (slotBounds.height - shapeBounds.height)/2;
					checkH = false;
                    break;
            }
        }
        // check Min/Max size
		if (checkW || checkH)
        	shapeBounds = shape._checkMinMaxSize(shapeBounds);
        
        shape.layout(shapeBounds);
    },
    
    // margins stuff
    
    _inflateH: function(/*Number*/val, /*Object||Array*/margins) {
        // summary:
        //     Enlarge the specified value by the given horizontal margins.
        // val : Number : the value to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        if (dojo.isArray(margins)) {
            // left and right are indexes 0 and 2;
            if (margins.length > 0) 
                val += margins[0];
            if (margins.length > 2) 
                val += margins[2];
        } else {
            val += 2 * margins;
        }
        return val;
    },
    
    _inflateV: function(/*Number*/val, /*Object||Array*/margins) {
        // summary:
        //     Enlarge the specified value by the given vertical margins.
        // val : Number : the value to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively.

        if (dojo.isArray(margins)) {
            // top and bottom are indexes 1 and 3;
            if (margins.length > 1) 
                val += margins[1];
            if (margins.length > 3) 
                val += margins[3];
        } else {
            val += 2 * margins;
        }
        return val;
    },
    
    _inflateRect:function(/*dojox.gfx.Rectangle*/rect, /*Object||Array*/margins){
        // summary:
        //     Enlarge the specified rect by the given margins.
        // rect: Rectangle: the rectangle to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively, and
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        rect = dojo.clone(rect);
        if (dojo.isArray(margins)) {
            // left
            if (margins.length > 0) {
                rect.x -= margins[0];
                rect.width += margins[0];
            }
            // top
            if (margins.length > 1) {
                rect.y -= margins[1];
                rect.height += margins[1];
            }
            // right
            if (margins.length > 2) {
                rect.width += margins[2];
            }
            // bottom
            if (margins.length > 3) {
                rect.height += margins[3];
            }
        } else {
            rect.x -= margins;
            rect.width += 2 * margins;
            rect.y -= margins;
            rect.height += 2 * margins;
        }
        return rect;
    },
        
    _applyMargins: function(/*dojox.gfx.Rectangle*/rect, /*Object||Array*/margins) {
        // summary:
        //     Applies the specified margins to given rect. This operation is the
        //     opposite of the _inflateRect method.
        // rect: Rectangle: the rectangle to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively, and
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        rect = dojo.clone(rect);
        if (dojo.isArray(margins)) {
            // left
            if (margins.length > 0) {
                rect.x += margins[0];
                rect.width -= margins[0];
            }
            // top
            if (margins.length > 1) {
                rect.y += margins[1];
                rect.height -= margins[1];
            }
            // right
            if (margins.length > 2) {
                rect.width -= margins[2];
            }
            // bottom
            if (margins.length > 3) {
                rect.height -= margins[3];
            }
        } else {
            rect.x += margins;
            rect.width -= 2 * margins;
            rect.y += margins;
            rect.height -= 2 * margins;
        }
        if (rect.width<0)
            rect.width = 0;
        if (rect.height<0)
            rect.height = 0; 
        return rect;
    }

    
});

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.GridLayout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.GridLayout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.GridLayout');



dojo.declare('ibm_ilog.diagram.gfxlayout.GridLayout', ibm_ilog.diagram.gfxlayout.Layout, {
    // summary:
    //      A layout class that arranges children of a container as a grid.
    // description:
    //      The GridLayout class arranges children of a container as grid elements.
    //      <p>The grid structure is defined by the 'rows' and 'cols' constructor parameters. 
    //      These properties are arrays of descriptor objects that defined how the column or row
    //      size is computed. These descriptor objects support the following properties:
    //      'width': if the container width (resp. height) is not fixed, then the column width (resp. row height)
    //      depends on the value of this property. A value of -1 means the dimension will accomodate to the maximum
    //      children dimension. Any other positive number sets the column width (resp. row height).
    //      'weight': if the container width (resp. height) is fixed, this property defines how the space 
    //      remaining after all the fixed width's (resp. height's) have been substracted from the 
    //      container width (resp. height) is distributed accross the columns (resp. the rows).
    //      <p>
    //      The GridLayout class defines the paddingLeft, paddingRight, paddingTop and paddingBottom properties
    //      to specificy the internal margins of the panel from which the children are positionned. A convenient global padding
    //      property is also defined to set the same padding in all directions.
    //      <p>Children in the grid are placed into the grid cells according to the 'column' and 'row' properties of the children. 
    //      When these properties are not specified on a shape, the shape is automatically placed in the grid according
    //      to its order in the container children list. The direction of the placement flow is defined via the 'horizontalFlow'
    //      property, which is 'true' by default. For example, assuming the 'horizontalFlow' property is set to 'true' the children
    //      do not defined any specific row/col index, the first child will be placed at the (0,0) cell, the 2nd one at the (1,0) cell, and so on.
    //      <p>
    //      A child can span over multiple rows or columns depending on their 'colspan' or 'rowspan' properties. These properties indicates
    //      the number of columns or rows the child must overlap. A value of '-1' means the child should span over all the remaining columns or rows.
    //      <p>
    //      Children are arranged within the computed grid cell according to their 'valign' 
    //      (one of the 'top', 'center', 'bottom' and 'stretch' values), 'halign' (one of the 'left', 'center', 'right' and 'stretch' values) 
    //      and their 'margins' properties.
    
    // the rows and columns descriptor
    rows: null,
    cols: null,
    // padding
    padding: 0,
    paddingLeft: 0,
    paddingRight: 0,
    paddingTop: 0,
    paddingBottom: 0,
    
    horizontalFlow: true,
    
    constructor: function(args) {
        this.rows = this.rows || [];
        this.cols = this.cols || [];
        if (this.rows.length === 0) 
            this.rows[0] = {
                weight: 1,
                height: -1
            };
        if (this.cols.length === 0) 
            this.cols[0] = {
                weight: 1,
                width: -1
            };
        if (args && 'padding' in args) 
            this._setPadding(args.padding, false);
    },
	
	getConfig : function() {
		// summary:
		// 		Gets the current configuration of this layout instance.
		// Description:
		//		This method returns a descriptor of this layout current configuration
		//		as well as the type information so that it can be passed to the Group.setLayout() method
		//		(for example to make a copy the layout of a Group). 
		// returns: Object
		//	An object that represents the layout current configuration.
		var arg ={};
		arg.type = 'ibm_ilog.diagram.gfxlayout.GridLayout';
		if (this.padding)
			arg.padding = this.padding;
		else {
			arg.paddingLeft = this.paddingLeft;
			arg.paddingRight = this.paddingRight;
			arg.paddingTop = this.paddingTop;
			arg.paddingBottom = this.paddingBottom;
		}
		arg.rows = dojo.clone(this.rows);
		arg.cols = dojo.clone(this.cols);
		arg.horizontalFlow = this.horizontalFlow;
		return arg;	/*Object*/	
	},
    
    setHorizontalFlow: function(/*Boolean*/value) {
        // summary:
        //     Sets whether children with no specified 'row' or 'column' properties
        //     are arranged horizontally or vertically in the grid. 
        //  value: Boolean: the automatic flow direction. 
        
        if (value !== this.horizontalFlow) {
            var old = this.horizontalFlow;
            this.horizontalFlow = value;
            this.onLayoutPropertyChanged('horizontalFlow', old, value);
        }
    },
	
	getHorizontalFlow:function(){ 
		return this.horizontalFlow; 
	},
    
    setPadding: function(value) {
        // summary:
        //     Sets the padding of this layout.
        // description:
        //     This property is used to set the internal margins of this layout.
        //     The value can either be a 4-elements array whose items match
        //     respectively the left, top, right and bottom paddings ; or one integer
        //     that is applied to all paddings.
        this._setPadding(value, true);
    },
    _setPadding: function(value, notify) {
        if (dojo.isArray(value)) {
            var l = value.length, oldL = this.paddingLeft, oldR = this.paddingRight, oldT = this.paddingTop, oldB = this.paddingBottom;
            if (l > 0) 
                this.paddingLeft = value[0];
            if (l > 1) 
                this.paddingTop = value[1];
            if (l > 2) 
                this.paddingRight = value[2];
            if (l > 3) 
                this.paddingBottom = value[3];
            if (notify &&
            (this.paddlingLeft !== oldL ||
            this.paddingTop !== oldT ||
            this.paddingBottom !== oldB ||
            this.paddingRight !== oldR)) {
            
                this.onLayoutPropertyChanged();
            }
        } else {
            var old = this.padding;
            this.padding = this.paddingLeft = this.paddingRight = this.paddingBottom = this.paddingTop = value;
            if (notify && (value !== old)) {
                this.onLayoutPropertyChanged();
            }
        }
    },
    
    adjustBBox: function(/*dojox.gfx.Rectangle*/bbox) {
        // summary:
        //     Returns the container bbox adjusted according to the constraints of this layout.
        // bbox: Rectangle: the bounding box of the associated container.
        
        bbox.x -= this.paddingLeft;
        bbox.y -= this.paddingTop;
        bbox.width += this.paddingLeft + this.paddingRight;
        bbox.height += this.paddingTop + this.paddingBottom;
        return bbox;
    },
    
    _getColIdx: function(shape, shapeIdx) {
        var c = 0;
        if ('column' in shape) 
            c = shape.column;
        else if (this.horizontalFlow) 
            c = shapeIdx % this.cols.length;
        else 
            c = Math.floor(shapeIdx / this.rows.length);
        return Math.min(c, this.cols.length - 1);
    },
    
    _getRowIdx: function(shape, shapeIdx) {
        var r = 0;
        if ('row' in shape) 
            r = shape.row;
        else if (this.horizontalFlow) 
            r = Math.floor(shapeIdx / this.cols.length);
        else 
            r = shapeIdx % this.rows.length;
        return Math.min(r, this.rows.length - 1);
    },
    
    _getRowDesc: function(rowIdx) {
        return rowIdx < this.rows.length ? this.rows[rowIdx] : null;
    },
    
    _getColDesc: function(colIdx) {
        return colIdx < this.cols.length ? this.cols[colIdx] : null;
    },
    
    computePreferredSize: function(proposedSize) {
    
        var children = this._container.children, count = children.length, i = 0, r = 0, c = 0, sh, sb, rDesc, cDesc, rm, cm;
        var rowsMetric = [], colsMetric = [];
        
        for (i = 0; i < this.rows.length; ++i) {
            rDesc = this.rows[i];
            rm = {};
            rm.height = 0;
            if ('height' in rDesc && rDesc.height !== -1) 
                rm.height = rDesc.height;
            rowsMetric.push(rm);
        }
        for (i = 0; i < this.cols.length; ++i) {
            cDesc = this.cols[i];
            cm = {};
            cm.width = 0;
            if ('width' in cDesc && cDesc.width !== -1) {
                cm.width = cDesc.width;
            }
            colsMetric.push(cm);
        }
        
        /*
         var totalFixedW = 0, totalWeightW = 0, totalFixedH = 0, totalWeightH = 0;
         
         for (i = 0; i < this.rows.length; ++i) {
         rDesc = this.rows[i];
         rm = {};
         rm.count = 0;
         rm.weight = -1;
         if ('weight' in rDesc) {
         var weight = rDesc.weight;
         rm.weight = weight;
         totalWeightH += weight;
         }
         rm.height = -1;
         if ('height' in rDesc && rDesc.height !== -1)
         rm.height = rDesc.height;
         rowsMetric.push(rm);
         }
         for (i = 0; i < this.cols.length; ++i) {
         cDesc = this.cols[i];
         cm = {};
         cm.count = 0;
         cm.weight = -1;
         if ('weight' in cDesc) {
         var weight = cDesc.weight;
         cm.weight = weight;
         totalWeightW += weight;
         }
         cm.width = -1;
         if ('width' in cDesc && cDesc.width !== -1) {
         cm.width = cDesc.width;
         }
         colsMetric.push(cm);
         }
         */
        for (i = 0; i < count; ++i) {
            sh = children[i];
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            // get the corresponding column/row descriptors
            rDesc = this.rows[r];
            cDesc = this.cols[c];
            var sz = {};
            sz.width = ('width' in cDesc && cDesc.width !== -1) ? cDesc.width : -1;
            sz.height = ('height' in rDesc && rDesc.height !== -1) ? rDesc.height : -1;
            sb = sh.computePreferredSize(sz);
			// take into account the shape width only if it is not an horizontal filler
			if (!sh.hfiller && 'width' in cDesc) {
                // if column width not set, set to the maximum preferred width
                if (cDesc.width === -1) {
                    var w = sb.width;
                    if (sh.margins) {
                        w = this._inflateH(w, sh.margins);
                    }
                    if (w > colsMetric[c].width) 
                        colsMetric[c].width = w;
                } else {
                    colsMetric[c].width = cDesc.width;
                }
            }
			// take into account the shape height only if it is not a vertical filler
            if (!sh.vfiller && 'height' in rDesc) {
                // if not fixed height, get the maximum preferred height 
                if (rDesc.height === -1) {
                    var h = sb.height;
                    if (sh.margins) {
                        h = this._inflateV(h, sh.margins);
                    }
                    if (h > rowsMetric[r].height) 
                        rowsMetric[r].height = h;
                } else {
                    rowsMetric[r].height = rDesc.height;
                }
            }
        }
        
        /*
         for (i = 0; i < count; ++i) {
         sh = children[i];
         c = this._getColIdx(sh, i);
         r = this._getRowIdx(sh, i);
         // get the corresponding column/row metrics
         rm = rowsMetric[r];
         cm = colsMetric[c];
         rDesc = this.rows[r];
         cDesc = this.cols[c];
         ++cm.count;
         ++rm.count;
         var sz = {};
         sz.width = ('width' in cDesc && cDesc.width !== -1) ? cDesc.width : -1;
         sz.height = ('height' in rDesc && rDesc.height !== -1) ? rDesc.height : -1;
         sb = sh.computePreferredSize(sz);
         // if column width not set, set to the maximum preferred width
         // (except if a weight has been specified, in this case the width
         // will be computed later when the total fixed width will be known
         if (this.cols[c].width === -1 && cm.weight === -1) {
         var w = sb.width;
         if (sh.margins)
         w = this._inflateH(w, sh.margins);
         if (w > cm.width)
         cm.width = w;
         }
         // if not fixed height, get the maximum preferred height
         if (this.rows[r].height === -1 && rm.weight === -1) {
         var h = sb.height;
         if (sh.margins)
         h = this._inflateV(h, sh.margins);
         if (h > rm.height)
         rm.height = h;
         }
         }
         // here, we have all fixed size computing done.
         // now we can compute the 'weighted' sizes
         if (totalWeightW > 0) {
         // Compute columns width
         for (i = 0; i < colsMetric.length; ++i) {
         cm = colsMetric[i];
         // if a width has been computed previously for this column, sum it
         if (cm.width !== -1) {
         totalFixedW += cm.width;
         } else { // width not set yet, and column not empty
         if (cm.weight === -1 && cm.count > 0) {
         cm.weight = 1;
         ++totalWeightW;
         } else if (cm.weight !== -1 && cm.count === 0) {
         // the column is empty, should not be included in the total
         --totalWeightW;
         }
         }
         }
         var remainingW = Math.max(0, contBounds.width - totalFixedW);
         // then compute the 'weighted' width
         if (totalWeightW > 0) {
         var unitW = remainingW / totalWeightW;
         for (i = 0; i < colsMetric.length; ++i) {
         cm = colsMetric[i];
         if (cm.width === -1 && cm.count > 0) {
         cDesc = this.cols[i];
         var weight = cDesc.weight || 1;
         cm.width = cm.weight * unitW;
         }
         }
         }
         }
         if (totalWeightH > 0) {
         
         for (i = 0; i < rowsMetric.length; ++i) {
         rm = rowsMetric[i];
         // if a height has been computed previously for this row, sum it
         if (rm.height !== -1) {
         totalFixedH += rm.height;
         } else {
         if (rm.weight === -1) {
         rm.weight = 1;
         totalWeightH += 1;
         }
         }
         }
         var remainingH = Math.max(0, contBounds.height - totalFixedH);
         if (totalWeightH > 0) {
         var unitH = remainingH / totalWeightH;
         for (i = 0; i < rowsMetric.length; ++i) {
         rm = rowsMetric[i];
         if (rm.height === -1) {
         rDesc = this.rows[i];
         var weight = rDesc.weight || 1;
         rm.height = rm.weight * unitH;
         }
         }
         }
         }
         if (totalWeightH > 0 || totalWeightW > 0) {
         for (i = 0; i < count; ++i) {
         sh = children[i];
         c = this._getColIdx(sh, i);
         r = this._getRowIdx(sh, i);
         // get the corresponding column/row metrics
         rm = rowsMetric[r];
         cm = colsMetric[c];
         var sz = {
         width: cm.width,
         height: rm.height
         };
         sb = sh.getPreferredSize(sz);
         
         }
         }
         */
        var sz = {
            width: 0,
            height: 0
        };
        if (proposedSize.width === -1) {
        
            for (i = 0; i < colsMetric.length; ++i) 
                sz.width += colsMetric[i].width;
        } else {
            sz.width = proposedSize.width;
        }
        if (proposedSize.height === -1) {
            for (i = 0; i < rowsMetric.length; ++i) 
                sz.height += rowsMetric[i].height;
        } else {
            sz.height = proposedSize.height;
        }
        sz.width += this.paddingLeft + this.paddingRight;
        sz.height += this.paddingTop + this.paddingBottom;
        
        return sz;
        
    },
    
    layoutChildren: function(contBounds) {
    
        contBounds = dojo.clone(contBounds);
        var children = this._container.children, count = children.length, i = 0, r = 0, c = 0, sh, sb, rDesc, cDesc, cm, rm;
        var rowsMetric = [], colsMetric = [];
        var totalFixedW = 0, totalWeightW = 0, totalFixedH = 0, totalWeightH = 0;
        
        for (i = 0; i < this.rows.length; ++i) {
            rDesc = this.rows[i];
            rm = {};
            rm.count = 0;
            rm.weight = -1;
            if ('weight' in rDesc) {
                var weight = rDesc.weight;
                rm.weight = weight;
                totalWeightH += weight;
            }
            rm.height = -1;
            if ('height' in rDesc && rDesc.height !== -1) 
                rm.height = rDesc.height;
            rowsMetric.push(rm);
        }
        for (i = 0; i < this.cols.length; ++i) {
            cDesc = this.cols[i];
            cm = {};
            cm.count = 0;
            cm.weight = -1;
            if ('weight' in cDesc) {
                var weight = cDesc.weight;
                cm.weight = weight;
                totalWeightW += weight;
            }
            cm.width = -1;
            if ('width' in cDesc && cDesc.width !== -1) {
                cm.width = cDesc.width;
            }
            colsMetric.push(cm);
        }
        
        contBounds.width = Math.max(0, contBounds.width - this.paddingLeft - this.paddingRight);
        contBounds.height = Math.max(0, contBounds.height - this.paddingTop - this.paddingBottom);
        for (i = 0; i < count; ++i) {
            sh = children[i];
            sb = sh.getPreferredSize();
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            // get the corresponding column/row metrics
            rm = rowsMetric[r];
            cm = colsMetric[c];
            ++cm.count;
            ++rm.count;
            // if column width not set, set to the maximum preferred width
            // (except if a weight has been specified, in this case the width
            // will be computed later when the total fixed width will be known
            if (this.cols[c].width === -1 && cm.weight === -1) {
                var w = sb.width;
                if (sh.margins) 
                    w = this._inflateH(w, sh.margins);
                if (w > cm.width) 
                    cm.width = w;
            }
            // if not fixed height, get the maximum preferred height 
            if (this.rows[r].height === -1 && rm.weight === -1) {
                var h = sb.height;
                if (sh.margins) 
                    h = this._inflateV(h, sh.margins);
                if (h > rm.height) 
                    rm.height = h;
            }
        }
        // Compute columns width
        for (i = 0; i < colsMetric.length; ++i) {
            cm = colsMetric[i];
            // if a width has been computed previously for this column, sum it
            if (cm.width !== -1) {
                totalFixedW += cm.width;
            } else { // width not set yet, and column not empty
                if (cm.weight === -1 && cm.count > 0) {
                    cm.weight = 1;
                    ++totalWeightW;
                } else if (cm.weight !== -1 && cm.count === 0) {
                    // the column is empty, should not be included in the total
                    --totalWeightW;
                }
            }
        }
        var remainingW = Math.max(0, contBounds.width - totalFixedW);
        // then compute the 'weighted' width
        if (totalWeightW > 0) {
            var unitW = remainingW / totalWeightW;
            for (i = 0; i < colsMetric.length; ++i) {
                cm = colsMetric[i];
                if (cm.width === -1 && cm.count > 0) {
                    cDesc = this.cols[i];
                    var weight = cDesc.weight || 1;
                    cm.width = cm.weight * unitW;
                }
            }
        }
        
        for (i = 0; i < rowsMetric.length; ++i) {
            rm = rowsMetric[i];
            // if a height has been computed previously for this row, sum it
            if (rm.height !== -1) {
                totalFixedH += rm.height;
            } else {
                if (rm.weight === -1) {
                    rm.weight = 1;
                    totalWeightH += 1;
                }
            }
        }
        var remainingH = Math.max(0, contBounds.height - totalFixedH);
        if (totalWeightH > 0) {
            var unitH = remainingH / totalWeightH;
            for (i = 0; i < rowsMetric.length; ++i) {
                rm = rowsMetric[i];
                if (rm.height === -1) {
                    rDesc = this.rows[i];
                    var weight = rDesc.weight || 1;
                    rm.height = rm.weight * unitH;
                }
            }
        }
        // compute cell coord
        var currentX = /*contBounds.x + */ this.paddingLeft;
        for (i = 0; i < colsMetric.length; ++i) {
            cm = colsMetric[i];
            cm.x = currentX;
            currentX += cm.width;
        }
        var currentY = /*contBounds.y + */ this.paddingTop;
        for (i = 0; i < rowsMetric.length; ++i) {
            rm = rowsMetric[i];
            rm.y = currentY;
            currentY += rm.height;
        }
        // arrange children
        for (i = 0; i < count; ++i) {
            var slotBounds = {}, sh = children[i];
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            cm = colsMetric[c];
            rm = rowsMetric[r];
            slotBounds.x = cm.x;
            slotBounds.width = cm.width;
            slotBounds.y = rm.y;
            slotBounds.height = rm.height;
            // span
            var span = sh.colspan;
            if (span) {
                this._spanSlot(span, c, slotBounds, 'width', colsMetric);
            }
            span = sh.rowspan;
            if (span) {
                this._spanSlot(span, r, slotBounds, 'height', rowsMetric);
            }
            this.placeChildInSlot(sh, slotBounds, true, true);
        }
    },
    
    _spanSlot: function(span, dimIdx, slotBounds, dim, dimMetric) {
        if (span === -1) 
            span = dimMetric.length - dimIdx;
        if (span > 1) {
            var end = dimIdx + span;
            for (++dimIdx; dimIdx < end; ++dimIdx) {
                if (dimIdx < dimMetric.length) {
                    var dm = dimMetric[dimIdx];
                    slotBounds[dim] += dm[dim];
                }
            }
        }
        return slotBounds;
    }
});

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.StackLayout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.StackLayout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.StackLayout');



dojo.declare('ibm_ilog.diagram.gfxlayout.StackLayout', ibm_ilog.diagram.gfxlayout.Layout, {
    // summary:
    //      A layout class that arranges children of a container as a horizontal or vertical stack.
    // description:
    //      The StackLayout class arranges children of a container as a horizontal or vertical stack
    //      depending on the value of its horizontal property. By default, a StackLayout has a vertical flow.
    //      An horizontal stack layout align its children vertically in the computed layout slot according to
    //      their 'valign' (one of the 'top', 'center', 'bottom' and 'stretch' values) and their 'margins' properties.
    //      Respectively, a vertical stack layout align its children horizontally in the computed layout slot according to
    //      their 'halign' (one of the 'left', 'center', 'right' and 'stretch' values) and their 'margins' properties.
    //      The StackLayout class defines the paddingLeft, paddingRight, paddingTop and paddingBottom properties
    //      to specify the internal margins of the panel from which the children are positioned. A convenient global padding
    //      property is also defined to set the same padding in all directions.
    //      Space between children can be adjusted using the gap property.
    
    // orientation
    horizontal: false,
    // padding
    padding: 0,
    paddingLeft: 0,
    paddingRight: 0,
    paddingTop: 0,
    paddingBottom: 0,
    // margin between items
    gap: 0,
    
    constructor: function(args) {
        if (args && 'padding' in args) {
            // you cannot set both padding and paddingXXX at the same time.
            this._setPadding(args.padding, false);
        }
    },
	
	getConfig: function() {
		// summary:
		// 		Gets the current configuration of this layout instance.
		// Description:
		//		This method returns a descriptor of this layout current configuration
		//		as well as the type information so that it can be passed to the Group.setLayout() method
		//		(for example to make a copy the layout of a Group). 
		// returns: Object
		//	An object that represents the layout current configuration.
		var arg = {};
		arg.type = 'ibm_ilog.diagram.gfxlayout.StackLayout';
		if (this.padding) 
			arg.padding = this.padding;
		else {
			arg.paddingLeft = this.paddingLeft;
			arg.paddingRight = this.paddingRight;
			arg.paddingTop = this.paddingTop;
			arg.paddingBottom = this.paddingBottom;
		}
		arg.gap = this.gap;
		arg.horizontal = this.horizontal;
		return arg; /*Object*/
	},
    
    setGap: function(/*Integer*/value) {
        // summary:
        //     Sets the items gap.
        //  value: Integer: the gap between each items. 
        if (value !== this.gap) {
            var old = this.gap;
            this.gap = value;
            this.onLayoutPropertyChanged('gap', old, value);
        }
    },
    
    setHorizontal: function(/*Boolean*/value) {
        // summary:
        //     Sets the stack orientation.
        // value: Boolean: true for an horizontal layout, false otherwise.
        
        if (this.horizontal !== value) {
            var old = this.horizontal;
            this.horizontal = value;
            this.onLayoutPropertyChanged('horizontal', old, value);
        }
    },
    
    setPadding: function(value) {
        // summary:
        //     Sets the padding of this layout.
        // description:
        //     This property is used to set the internal margins of this layout.
        //     The value can either be a 4-elements array whose items match
        //     respectively the left, top, right and bottom paddings ; or one integer
        //     that is applied to all paddings.
        this._setPadding(value, true);
    },
    _setPadding: function(value, notify) {
        if (dojo.isArray(value)) {
            var l = value.length, oldL = this.paddingLeft, oldR = this.paddingRight, oldT = this.paddingTop, oldB = this.paddingBottom;
            if (l > 0) 
                this.paddingLeft = value[0];
            if (l > 1) 
                this.paddingTop = value[1];
            if (l > 2) 
                this.paddingRight = value[2];
            if (l > 3) 
                this.paddingBottom = value[3];
            if (notify &&
            (this.paddlingLeft !== oldL ||
            this.paddingTop !== oldT ||
            this.paddingBottom !== oldB ||
            this.paddingRight !== oldR)) {
            
                this.onLayoutPropertyChanged();
            }
        } else {
            var old = this.padding;
            this.padding = this.paddingLeft = this.paddingRight = this.paddingBottom = this.paddingTop = value;
            if (notify && (value !== old)) {
                this.onLayoutPropertyChanged();
            }
        }
    },
    
    adjustBBox: function(/*dojox.gfx.Rectangle*/bbox) {
        // summary:
        //     Returns the container bbox adjusted according to the constraints of this layout.
        // bbox: Rectangle: the bounding box of the associated container.
        
        bbox.x -= this.paddingLeft;
        bbox.y -= this.paddingTop;
        bbox.width += this.paddingLeft + this.paddingRight;
        bbox.height += this.paddingTop + this.paddingBottom;
        return bbox;
    },
    
    computePreferredSize: function(/*Size*/proposedSize) {
        // summary:
        //     Computes the preferred size of the associated container according to the layout strategy.
        
        var csz = {}, children = this._container.children, count = children.length, maxW = 0, maxH = 0, i, bounds = [];
        
        for (i = 0; i < count; ++i) {
            var s = children[i];
            var sb = s.computePreferredSize(proposedSize);            
			if (s.margins) {
                sb.width = this._inflateH(sb.width, s.margins);
                sb.height = this._inflateV(sb.height, s.margins);
            }
            if (!s.hfiller && sb.width > maxW) 
                maxW = sb.width;
            if (!s.vfiller && sb.height > maxH) 
                maxH = sb.height;
            bounds.push(dojo.clone(sb));
        }
        
        if (proposedSize.width !== -1 && proposedSize.height !== -1) {
            return proposedSize;
        }
        if (proposedSize.height !== -1) 
            maxH = proposedSize.height;
        if (proposedSize.width !== -1) 
            maxW = proposedSize.width;
        var lastCoord = 0;
        if (this.horizontal) {
            for (i = 0; i < count; ++i) {
                if (children[i].hfiller)
					continue;
                var sb = bounds[i];
				lastCoord += sb.width;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
            csz.width = lastCoord;
            csz.height = maxH;
        } else {
            for (i = 0; i < count; ++i) {
                if (children[i].vfiller)
					continue;
                var sb = bounds[i];
                lastCoord += sb.height;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
            csz.height = lastCoord;
            csz.width = maxW;
        }
        csz.width += this.paddingLeft + this.paddingRight;
        csz.height += this.paddingTop + this.paddingBottom;
        return csz;
    },
    
    layoutChildren: function(/*Size*/contBounds) {
        // summary:
        //     Lays out the children of the associated container.
        
        var children = this._container.children, count = children.length, i;
        contBounds = dojo.clone(contBounds);
        
        contBounds.width = Math.max(0, contBounds.width - this.paddingLeft - this.paddingRight);
        contBounds.height = Math.max(0, contBounds.height - this.paddingTop - this.paddingBottom);
        var lastCoord;
        if (this.horizontal) {
            lastCoord = /*contBounds.x +*/ this.paddingLeft;
            for (i = 0; i < count; ++i) {
                var s = children[i];
                var sb = s.getPreferredSize();
                var slot = {};
                slot.x = lastCoord;
                slot.y = /*contBounds.y + */ this.paddingTop;
                slot.height = contBounds.height;
				slot.width = sb.width;
                if (s.margins) 
                    slot.width = this._inflateH(slot.width, s.margins);
                // the elements in the stack must be clipped 
				if (slot.width + lastCoord > contBounds.width)
					slot.width = contBounds.width - lastCoord;
                this.placeChildInSlot(s, slot, false, true);
                lastCoord += slot.width;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
        } else {
            lastCoord = /*contBounds.y +*/ this.paddingTop;
            for (i = 0; i < count; ++i) {
                var s = children[i];
                var sb = s.getPreferredSize();
                var slot = {};
                slot.x = /*contBounds.x +*/ this.paddingLeft;
                slot.y = lastCoord;
                slot.width = contBounds.width;
                slot.height = sb.height;
                if (s.margins) 
                    slot.height = this._inflateV(slot.height, s.margins);
                // the elements in the stack must be clipped 
				if (slot.height + lastCoord > contBounds.height)
					slot.height = contBounds.height - lastCoord;
                this.placeChildInSlot(s, slot, true, false);
                lastCoord += slot.height;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
        }
    }
});

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.FillLayout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.FillLayout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.FillLayout');



dojo.declare('ibm_ilog.diagram.gfxlayout.FillLayout', ibm_ilog.diagram.gfxlayout.Layout, {
	
	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {type:'ibm_ilog.diagram.gfxlayout.FillLayout'}; /*Object*/
	},

    computePreferredSize: function(proposedSize) {
        var children = this._container.children, count = children.length, o, i, maxW = 0, maxH = 0;
        for (i = 0; i < count; ++i) {
            if (children[i].filler) 
                continue;
            var sz = children[i].computePreferredSize(proposedSize);
            if (sz.width > maxW) 
                maxW = sz.width;
            if (sz.height > maxH)
                maxH = sz.height;
        }
        return {
            width: proposedSize.width === -1 ? maxW : proposedSize.width,
            height: proposedSize.height === -1 ? maxH : proposedSize.height
        }
    },
    
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        for (i = 0; i < count; ++i) {
            children[i].layout(contBounds);
        }
    }
});

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout');

if (dojo.config.useGfxLayout) {
    
	
    
    
    
    
}

}

if(!dojo._hasResource['ibm_ilog.diagram.Node']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Node'] = true;
dojo.provide('ibm_ilog.diagram.Node');











(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	
dojo.declare("ibm_ilog.diagram.Node", [ibm_ilog.diagram.GraphElement], {
    // summary:
    //		A Node is a graphic object that can be connected to other nodes using Link objects.
    
    // Allows to test the class faster that testing declaredClass
    _isIBMDiagramNode: true,

  	//
  	//	TID: String
  	//		Graph element Type ID, used to distinguish basic graph 
    //		element types:
    //	
    //			N = Node
    //			S = Subgraph
    //			L = Link
    //	
  	//		Usually used as a very fast index into maps based on 
    //		ibm_diagram graph element types.
  	//
    TID: 'N',
    
    _ports: null,
 
    constructor: function() {
        if (dojo.config.useGfxLayout) {
            this.setLayout(new ibm_ilog.diagram.gfxlayout.GridLayout());
        }
    },
    
    //---------- Disposal ----------
    
    _dispose: function() {
        // summary:
        //		Dispose this Node.
        // description:
        //		This function should be called when the node is completely deleted, that is, when it is removed
        //		from its parent graph and it will never be used again.
        
        dojo.forEach(this.getLinks(), function(link) {
            if (link.getStartNode() == this) 
                link.setStartPort(null);
            if (link.getEndNode() == this) 
                link.setEndPort(null);
        });
    },
    
    // ---------- Ports management: ----------
    
    addPort: function(/*ibm_ilog.diagram.PortBase*/port) {
        // summary:
        //		Adds a port to this node.
        // port: ibm_ilog.diagram.PortBase:
        //		The port to add.
        if (!this._ports) 
            this._ports = [port];
        else 
            this._ports.push(port);
        port._setOwner(this);
    },
    
    removePort: function(/*ibm_ilog.diagram.PortBase*/port) {
        // summary:
        //		Removes a port from this node.
        // port: ibm_ilog.diagram.PortBase:
        //		The port to remove.
        if (this._ports) {
            var index = dojo.indexOf(this._ports, port);
            if (index >= 0) {
                port._setOwner(null);
                if (this._ports.length == 1) 
                    this._ports = null;
                else 
                    this._ports.splice(index, 1);
            }
        }
    },
    
    getPorts: function() {
        // summary:
        //		Returns the ports of this node.
        // returns:
        //		An array of PortBase objects.
        // description:
        //		Note that this method always returns a newly allocated array.
        var ports = [];
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) 
                ports.push(this._ports[i]);
        }
        return ports;
    },
    
    _invalidateLinks: function() {
        // tag: private
        // summary:
        //		Invalidates the shape of all the links connected to this node. 
        // description:
        //		This method is internal and should not be called directly by application code.
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) {
                this._ports[i].invalidateLinks();
            }
        }
    },
    
	_setParent: function(parent, matrix)
	{
		ibm_ilog.diagram.GraphElement.prototype._setParent.call(this, parent, matrix);
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) {
				var p = this._ports[i];
				if(p._links){
					for(var j = 0; j < p._links.length; j++)
						p._links[j]._forceCheckIntergraphLink();
				}
            }
        }
	},
	
    // ---------- Get connected links: ----------
    
    getLinks: function(starting, ending) {
        // summary: 
        //		Gets the links whose start and/or end nodes are equal to this node.
        // description:
        //		If starting is not false, the method returns all the links whose start nodes are this node.
        //		If ending is not false, the method returns all the links whose end nodes are this node.
        // starting: Boolean:
        //		If this argument is not false (e.g., true or undefined), links whose start nodes are equal to this node are returned.
        // ending: Boolean:
        //		If this argument is not false (e.g., true or undefined), links whose end nodes are equal to this node are returned.
        // returns:
        //		An array of ibm_ilog.diagram.Link objects.
        // example:
        //		node.getLinks()
        //		node.getLinks(true, true)
        //		node.getLinks(true)
        //				 These 3 calls return all the links connected to the node.
        // example:
        //		node.getLinks(true, false)
        //				 This call returns the links starting at the node.
        // example:
        //		node.getLinks(false)
        //		node.getLinks(false, true)
        //				 These 2 calls return the links ending at the node.
        // example:
        //		node.getLinks(false, false) 
        //				This call always returns an empty array.
        
        var result = [];
        if (this._ports !== null && (starting != false || ending != false)) {
            dojo.forEach(this._ports, function(p) {
                dojo.forEach(p.getLinks(), function(link) {
                    if ((starting != false && link.getStartPort() === p) ||
                    (ending != false && link.getEndPort() === p)) {
                        if (dojo.indexOf(result, link) < 0) 
                            result.push(link);
                    }
                }, this);
            }, this);
        }
        return result; // Array
    },
    
    // ---------- Base shape: ----------
    
    getBaseShape: function() {
        // summary:
        //		Gets the base shape of the node.
        // description:
        //		The base shape for various purposes, for example links will typically
        //		be connected to the base shape instead of the overall bounding rectangle of the node.
        //		By default, the base shape is the shape contained in the node template
        //		whose dojoAttachPoint property is set to "baseShape". In the default template,
        //		this is a rounded rectangle. If not baseshape exists in the template, the
        //		Node itself is returned.
        
        return this.baseShape || this;
    },
    
    // ---------- Bounds: ----------
    
    getBounds: function(/*dojox.gfx.Group*/parent) {
        // summary:
        //		Returns the bounds of this node in the coordinate space of the specified parent.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are returned.
        // returns:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the bounds of the node in the specified coordinate space.

        var rect = this.getLocalBoundingBox();
        if (rect && parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            rect = t.transformRectangle(rect);
        }
        return rect;
    },
    
    setBounds: function(rect, /*dojox.gfx.Group*/ parent) {
        // summary:
        //		Changes the transform of this node so that its bounds will be a given rectangle,
        //		in the coordinate space of the specified parent.
        // rect:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the new bounds of the node in the specified coordinate space.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are changed.
        
        if (parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            t = t.inverse();
            rect = t.transformRectangle(rect);
        }
        
        if (dojo.config.useGfxLayout) {
            // if this Node has a layout, set its clientRectangle
            if (this.getLayout()) {
                this.setExplicitSize(rect);
				this.move(rect.x, rect.y, parent);
            // maybe it contains a panel as only child
            } else if (this.children.length === 1 && this.children[0].children && this.children[0].getLayout()) {
                this.children[0].setExplicitSize(rect);
            // fallback: uses transform
            } else {
                var oldBounds = this.getLocalBoundingBox();
                if (oldBounds === null) {
                    this.applyLeftTransform(dojox.gfx.matrix.translate(rect.x, rect.y));
                } else if (rect.x != oldBounds.x ||
                rect.y != oldBounds.y ||
                rect.width != oldBounds.width ||
                rect.height != oldBounds.height) {
                    this.applyLeftTransform(dojox.gfx.matrix.createFromRectangles(oldBounds, rect));
                }
            }
        } else {
                var oldBounds = this.getLocalBoundingBox();
                if (oldBounds === null) {
                    this.applyLeftTransform(dojox.gfx.matrix.translate(rect.x, rect.y));
                } else if (rect.x != oldBounds.x ||
                           rect.y != oldBounds.y ||
                           rect.width != oldBounds.width ||
                           rect.height != oldBounds.height) {
                    this.applyLeftTransform(dojox.gfx.matrix.createFromRectangles(oldBounds, rect));
                }
        }
    },
    
    allowsResize: function() {
    	return false;
    },

    minimumSize: Size(10,10),
    
    move: function(/*number*/x, /*number*/ y, /*dojox.gfx.Group*/ parent) {
        // summary:
        //		Moves the node to the specified position.
        // description:
        //		This function changes the node's transform so that the top-left of its bounds
        //		(in the parent coordinate space) will be the specified 'x'and 'y' arguments
        
        
        if (parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            t = t.inverse();
            var p = {
                x: x,
                y: y
            };
            p = t.transformPoint(p);
            x = p.x;
            y = p.y;
        }

        var oldBounds = this.getLocalBoundingBox();
        if (oldBounds) {
            if (x != oldBounds.x || y != oldBounds.y) 
                this.applyLeftTransform(dojox.gfx.matrix.translate(x - oldBounds.x, y - oldBounds.y));
        } else {
            this.applyLeftTransform(dojox.gfx.matrix.translate(x, y));
        }
    },
    
    onBoundsChanged: function() {
        // summary:
        //		This event is called to signal that the bounds of the node have changed.
    },
    
    _onChanged: function(shape, reason) {
    	// This method is called a lot, keep it short and fast :).
    	
    	// <inlining this call from BoundingBoxFix at diagext.js>
    	// 
    	// this.inherited(arguments);
    	this._bbCacheValid = false;
    	// <end inline>

    	if(!ibm_ilog.diagram.Node._ChangesInvalidateLinksDisabled) {
    		this._invalidateLinks();
    	}
    	this.onBoundsChanged(shape,reason); // bounds probably changed
    },
    
    hitTest: function(p,graph,tolerance){
    	// Summary:
    	//		return if the given point hits the element
    	var g = ibm_ilog.diagram.util.Geometry;
    	
    	if(tolerance == null){
    		tolerance = 0;
    	}
    	var bounds = this.getBounds(graph);
    	if(!bounds) {
    		return false;
    	}
    	var bb = g.expandRect(bounds,tolerance);
    	if(g.containsPoint(bb,p)) {
    		var shape = this.getBaseShape();
    		if(shape != this){
    			var referencePoint = g.Point(0,0);
    			var clipConnectionPoint = this._getExternalPoint(bb);
    			// This solution only works with closed shapes
    			var completeResult = {};
    			ibm_ilog.diagram.util.ClippingUtil.clipToShape(this,graph,shape,bounds,referencePoint, clipConnectionPoint, p,completeResult);
    			// If the point is inside the node, the number of intersection must be even
    			if( (completeResult.numberOfResults%2) == 1){
					return true;
				}else{
					return false;
				}
    		}else{
    			return true;
    		}
    	}
    	return false;
    },
    
    _getExternalPoint: function(bounds){
    	var margin = 30;//this margin is to ensure that the point is outside the node shape (as the bb is not correctly on path shapes, this is big)
    	var g = ibm_ilog.diagram.util.Geometry;
    	return g.Point(bounds.x-margin,bounds.y-margin);
    },
    
    __eod: undefined
   
});

// Add TransformChain extension 
//dojo.extend(ibm_ilog.diagram.Node, ibm_ilog.diagram.TransformChain);

// Declare bindable properties
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "label", "");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "wrap", true);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "annotation", ""); // used by annotation adorners
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "tooltip", ""); // used to generate a tooltip on hover
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "backgroundColor", "#ADD7FF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "selectedBackgroundColor", "#0000FF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "borderColor", "#808080");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "textColor", "#000000");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "selectedTextColor", "#FFFFFF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "data", null);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Node, "textDir", "");

// Default node template
ibm_ilog.diagram.Node.defaultTemplate = ibm_ilog.diagram.declareTemplate(ibm_ilog.diagram.templates.defaultNodeTemplate);

ibm_ilog.diagram.Node.nodeType = dojox.gfx.Group.nodeType;
ibm_ilog.diagram.Node.defaultShape = {
    type: 'node'
};

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.ClippingUtil']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.ClippingUtil'] = true;
dojo.provide('ibm_ilog.diagram.util.ClippingUtil');

//


(function(){
	//var R = ibm_ilog.diagram.util.ErrorReporter;
	
	ibm_ilog.diagram.util.ClippingUtil = {
			
    getClippedPoint: function(bbox/*Rectangle*/,
                              p1/*Point*/, p2/*Point*/) { 
		// Summary:
	    // 		Returns the clipped point at rectangle (node bounding box), assuming 
	    // 		that p1 (attach point for a link in a graph) is inside the bounding box,
	    // 		and p2 (control point for a link) is outside that bounding box.
	    // bbox: 
		//		The rectangle (bounding box).
	    // p1: 
		//		The point (attach point) to which the unclipped line (link)
		//		is attached.
	    // p2:
		//		The auxiliary control point. For the purpose of
	    //      clipping, the line (link) can be considered as a ray that 
	    //      starts at the attach point and goes through the control point.
	    // return:
		//		The clipped point.
		//

      var intersectionPoints = [];
      var numIntersections = this.lineIntersectsRect(p1, p2, bbox, intersectionPoints);      
      // choose the result from the intersections
      return this.bestClipPointOnRay(p1, p2, intersectionPoints, numIntersections);
    },
			
	bestClipPointOnRay: function(startPoint/*Point*/, controlPoint/*Point*/,intersectionPoints/*Points Array*/,numIntersections){
		// Summary:
	    // 		Selects the best of n intersection points of a ray for clipping.
	    // 		If any intersection point is on the ray, it returns the one that is
	    // 		farthest from the start of the ray.
	    // 		If no intersection point is on the ray, it returns the one that is closest
	    // 		to the starting point of the ray.
	    // startPoint: 
		//		The starting point of the ray.
	    // controlPoint: 
		//		Another point of the ray.
	    // intersectionPoints:
		//		An array of intersection points. Typically,
	    //      these are the intersection points of a line through starting and
	    //      control points and the border shape of a node.
	    // numIntersections:
		//		The number of valid intersections in the array.
	    // return:
		//		The best of the intersection points.
		//
		      switch (numIntersections) {
		        case 0:
		          return {x:startPoint.x,y:startPoint.y};
		        case 1:
		          return intersectionPoints[0];
		      }
		      // case that we have 2 ore more intersections. Search the best one.
		      
		      var anyOnRay = false;
		      var closestPoint = null;
		      var farthestPoint = null;
		      var dx;
		      var dy;
		      var distSquare;
		      var closestDistSquare = Number.MAX_VALUE;
		      var farthestDistSquare = -1;
		      
		      for (var i = 0; i < numIntersections; i++) {
		        if (intersectionPoints[i] == null) continue;
		        dx = intersectionPoints[i].x - startPoint.x;
		        dy = intersectionPoints[i].y - startPoint.y;
		        distSquare = dx * dx + dy * dy;
		        if (this.pointOnRay(startPoint, controlPoint, intersectionPoints[i])) {
		          anyOnRay = true;
		          if (distSquare > farthestDistSquare) {
		            farthestDistSquare = distSquare;
		            farthestPoint = intersectionPoints[i];
		          }
		        }
		        else if (!anyOnRay && distSquare <= closestDistSquare) {
		          closestDistSquare = distSquare;
		          closestPoint = intersectionPoints[i];
		        }
		      }
		      
		      return anyOnRay ? farthestPoint : closestPoint;
		    },
		    
		    pointOnRay: function(p1/*Point*/, p2/*Point*/,testPoint/*Point*/){
		    	// Summary:
			    // 		Returns <code>true</code> if the input test point is on a ray.
			    // p1:
			    //		The start point of the ray.
			    // p2: 
			    // 		Another point on the ray.
			    // testPoint: 
			    //	 	The point to be tested.
		      var dx = (p2.x > p1.x ? p2.x - p1.x : p1.x - p2.x);
		      var dy = (p2.y > p1.y ? p2.y - p1.y : p1.y - p2.y);
		      
		      // compare in x or y direction. Choose the direction where the points
		      // have the larger span, to avoid rounding errors
		      if (dx > dy) {
		        if (p1.x < p2.x && p1.x <= testPoint.x)
		          return true;
		        if (p1.x > p2.x && p1.x >= testPoint.x)
		          return true;
		      }
		      else {
		        if (p1.y < p2.y && p1.y <= testPoint.y)
		          return true;
		        if (p1.y > p2.y && p1.y >= testPoint.y)
		          return true;
		      }
		      return false;
		    },
		    
 
		    
		    lineIntersectsSegment: function(p1/*Point*/,p2/*Point*/, p3/*Point*/, p4/*Point*/, res){
		    	// Summary:
			    // 		Calculates the intersection between a line and a line segment.
			    // p1: 
			    // 		 A point on the line.
			    // p2: 
			    // 		 Another point on the line.
			    // p3: 
			    // 		 The first end point of the line segment.
			    // p4: 
			    // 		 The second end point of the line segment.
			    // res: 
			    // 		 Container for the resulting intersection point.
			    // return: 
			    // 		 <code>true</code> if the line intersects the segment
			    //         at a unique point.
		      if (this.pointsEquals(p3,p4)) {
		        // test whether p3 is on the line p1-p2
		        if ((p3.x - p1.x) * (p2.y - p1.y) == (p2.x - p1.x) * (p3.y - p1.y)) {
		          res.x = p3.x;
		          res.y = p3.y;
		          return true;
		        }
		      }
		      else
		        if (this.GetLineIntersection(p1, p2, p3, p4, res) != null) {
		          var dx = (p4.x > p3.x ? p4.x - p3.x : p3.x - p4.x);
		          var dy = (p4.y > p3.y ? p4.y - p3.y : p3.y - p4.y);
		          
		          // compare in x or y direction. Choose the direction where the points
		          // have the larger span, to avoid rounding errors.
		          
		          if (dx > dy) {
		            if (p3.x <= res.x && res.x <= p4.x ||
		              p4.x <= res.x && res.x <= p3.x)
		              return true;
		          }
		          else {
		            if (p3.y <= res.y && res.y <= p4.y ||
		              p4.y <= res.y && res.y <= p3.y)
		              return true;
		          }
		        }
		      return false;
		    },
		    
		    
		    // Checking intersection with a polyline isn't that easy.
		    // First, we check the intersection with all polygon segments.
		    // However, it may happen that a line hits exactly the corner point
		    // between two adjacent polygon segments. Due to rounding errors, the
		    // intersection point may appear outside of both polygon segments (this
		    // happens frequently if the one segment is nearly horizontal and the other
		    // is nearly vertical. Therefore we need a special corner test.
		    //
		    // Example,    Due to rounding errors,
		    //             GetLineIntersction \
		    //             may return          \
		    //             this point ------->  X
		    //             which appears         +----------|
		    //             to be outside         |\
		    //             of both polygon       | \
		    //             segments              |  \
		    //                                   |   \
		    //                                   |    \ line
		    //                                   -     \
		    //
		    // If a line doesn't cross adjacent segments (a,b) and (b,c), but crosses
		    // the segment (a,c), we know that it MUST cross the polygon a-b-c at a
		    // point close to b, but we had rounding errors during the calculation that
		    // makes the calculated point appear outside (a,b) and (b,c). In this case,
		    // we report b as the intersection point.
		    

		    lineIntersectsPolyPoints: function(p1/*Point*/, p2/*Point*/, points, closed, res){
			    // Calculates the intersections between a line and a polyline or polygon.
			    // p1: 
			    // 		  One point of the line.
			    // p2: 
			    // 		  Another point on the line.
			    // points: 
			    // 		  The points of the polyline or polygon.
			    // closed: 
			    // 		  <code>true</code> if it is a polygon,
			    //        <code>false</code> if it is a polyline.
			    // res: 
			    // 		  Container for the resulting intersection points.
			    //            This container must must be large enough to get all
			    //            intersection points.
			    // return: 
			    // 		  The number of intersections.
		      var numIntersections = 0;
		      var pa = null;
		      var pb = null;
		      var pc = null;
		      var intersectsFirstSegment = false;
		      var intersectsAB = false;
		      var intersectsBC = false;
		      var prevCrossPoint = null;
		      var crossPoint = {};
		      var pab = {x:0,y:0};
		      var pbc = {x:0,y:0};
		      
		      for (var i = 0; i < points.length; i++) {
		        pc = points[i];
		        // segment test
		        if (pb != null) {
		          if (this.lineIntersectsSegment(p1, p2, pb, pc, crossPoint)) {
		            if (prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,crossPoint)) {
		              res[numIntersections++] = prevCrossPoint = crossPoint;
		              crossPoint = {};
		              intersectsBC = true;
		              if (i == 1)
		                intersectsFirstSegment = true;
		            }
		          }
		        }
		        // corner test
		        if (pa != null && !intersectsAB && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pb.x);
		          pab.y = 0.5 * (pa.y + pb.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if (prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pb))
		              res[numIntersections++] = prevCrossPoint = pb;
		          }
		        }
		        
		        pa = pb;
		        pb = pc;
		        intersectsAB = intersectsBC;
		      }
		      
		      var firstCrossPoint = (numIntersections > 0 ? res[0] : null);
		      
		      if (points.length > 2 && closed) {
		        pc = points[0];
		        // segment test
		        if (this.lineIntersectsSegment(p1, p2, pb, pc, crossPoint)) {
		          if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,crossPoint)) &&
		            (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,crossPoint))) {
		              res[numIntersections++] = crossPoint;
		              crossPoint = {};
		              intersectsBC = true;
		            }
		        }
		        // and two more corner tests
		        if (!intersectsAB && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pb.x);
		          pab.y = 0.5 * (pa.y + pb.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pb)) &&
		              (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,pb)))
		              res[numIntersections++] = prevCrossPoint = pb;
		          }
		        }
		        pa = points[0];
		        if (!intersectsFirstSegment && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pc.x);
		          pab.y = 0.5 * (pa.y + pc.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pc)) &&
		              (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,pc)))
		              res[numIntersections++] = pc;
		          }
		        }
		      }
		      
		      return numIntersections;
		    },
		    
    	lineIntersectsRect: function(p1/*Point*/, p2/*Point*/, 
                                 	 rect/*Rectangle*/,
                                 	 res) {
		// Summary:
		// 		Calculates the intersections between a line and a rectangle.
		// p1: 
		// 		One point of the line.
		// p2: 
		// 		Another point on the line.
		// rect: 
		// 		The rectangle to test.
		// res: 
		// 		Container of two for the resulting intersection points.
		// return: 
		// 		 The number of intersections.

      	// get the corner points of the bounding box
		var pts = [];
		pts[0] = {x:rect.x, y:rect.y};
		pts[1] = {x:rect.x + rect.width, y:pts[0].y};
		pts[2] = {x:pts[1].x, y:rect.y + rect.height};
		pts[3] = {x:rect.x, y:pts[2].y};
      
      	// get the intersections between the line through attach and control
      	// point and the polygon defined by pts.
      
      	return this.lineIntersectsPolyPoints(p1, p2, pts, true, res);
    	},
		    
		    lineIntersectsEllipse:function(p1/*Point*/, p2/*Point*/, rect/*dojox.gfx.Rectangle*/, res){
		    	// Summary:
			    // 		Calculates both intersections between a line and an ellipse.
			    // p1: 
			    // 		One point of the line.
			    // p2: 
			    // 		Another point on the line.
			    // rect: 
			    // 		The bounding box of the ellipse.
			    // res: 
			    // 		Container of two for the resulting intersection points.
			    // return: 
			    // 		 The number of intersections.
		      var dx = p2.x - p1.x;
		      var dy = p2.y - p1.y;
		      var a = 0.5 * rect.width;
		      var b = 0.5 * rect.height;
		      var cx = rect.x + a;
		      var cy = rect.y + b;
		      var x;
		      var y;
		      var f;
		      var g;
		      var z;
		      var sqrtz;
		      
		      if (dx == 0.0 && dy == 0.0)
		        return 0;
		      
		      if ((dx > 0.0 ? dx : -dx) > (dy > 0.0 ? dy : -dy)) {
		        f = dy/dx;
		        g = p1.y - cy - f * (p1.x - cx);
		        z = b * b + a * a * f * f - g * g;
		        if (z < 0.0)
		          return 0;
		        
		        if (z == 0.0) {
		          x = - f * g * a * a / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          return 1;
		        }
		        else {
		          sqrtz = Math.sqrt(z);
		          x = (a * b * sqrtz - f * g * a * a) / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          x = (- a * b * sqrtz - f * g * a * a) / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[1] = {x:(x + cx),y:(y + cy)};
		          return 2;
		        }
		      }
		      else {
		        f = dx/dy;
		        g = p1.x - cx - f * (p1.y - cy);
		        z = a * a + b * b * f * f - g * g;
		        if (z < 0.0)
		          return 0;
		        
		        if (z == 0.0) {
		          y = - f * g * b * b / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          return 1;
		        }
		        else {
		          sqrtz = Math.sqrt(z);
		          y = (a * b * sqrtz - f * g * b * b) / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          y = (- a * b * sqrtz - f * g * b * b) / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[1] = {x:(x + cx),y:(y + cy)};
		          return 2;
		        }
		      }
		    },
		    
		    lineIntersectsArc:function(p1/*Point*/, p2/*Point*/,rect/*dojox.gfx.Rectangle*/, startAngle, deltaAngle,res){
		    	// Summary:
		    	// 		Calculates both intersections between a line and an arc (a partial ellipse).
			    // p1: 
			    // 		 One point of the line.
			    // p2: 
			    // 		 Another point on the line.
			    // rect: 
			    // 		 The bounding box of the complete ellipse.
			    // startAngle: 
			    // 		 The angle of the starting point in degrees (0 .. 360).
			    // deltaAngle: 
			    // 		 The angle range of the arc from the starting point to
			    //                   the end point in degrees (0 .. 360).
			    // res: 
			    // 		 Container of two for the resulting intersection points.
			    // return: 
			    // 		 The number of intersections.
		      var tempRes = [];
		      var numPoints = this.lineIntersectsEllipse(p1, p2, rect, tempRes);
		      
		      var i;
		      var j = 0;
		      for (i = 0; i < numPoints; i++) {
		        if (this.ellipticAngle(rect, tempRes[i], startAngle) <= deltaAngle){
		        	j++;
		        	res.push(tempRes[i]);
		        }
		      }
		      
		      return j;
		    },
		    
		    lineIntersectsPartialEllipse: function(p1/*Point*/, p2/*Point*/,rx,ry, startPoint,endPoint,fa,fs,alpha,res){
		    	//Summary:
		    	// 		Calculates both intersections between a line and a partial ellipse.
		    	var center = this.ellipseEndpointToCenterParameterization(startPoint,endPoint,fa,fs,rx,ry,alpha);
		    	var rect = {x:center.x-(rx), y:center.y-(ry), width:rx*2, height:ry*2};
		    	
		    	var pp1 = this._rotatePointAt(p1,center,-alpha);
		    	var pp2 = this._rotatePointAt(p2,center,-alpha);
		    	
		    	var angle;
		    	var delta;
		    	if(fs==1){
		    		angle = this.ellipticAngle(rect, endPoint, 0);
		    		delta = this.ellipticAngle(rect, startPoint, angle);
		    	}else{
		    		angle = this.ellipticAngle(rect, startPoint, 0);
		    		delta = this.ellipticAngle(rect, endPoint, angle);
		    	}
		    	angle = angle + alpha;
		    	var length = res.length
		    	this.lineIntersectsArc(pp1, pp2,rect, angle, delta,res);
		    	if((res.length - length) >= 1){
		    		res[res.length-1] = this._rotatePointAt(res[res.length-1],center,alpha);
		    	}
		    	return res.length - length;
		    	
		    },
		    
		    _rotatePointAt: function(point,center,angle){
		    	var _m = dojox.gfx.matrix;
		    	var mRot = _m.rotategAt(angle,center);
		    	return _m.multiplyPoint(mRot,point);
		    },
		    
		    
		    lineIntersectsQuadSpline: function(p1/*Point*/, p2 /*Point*/,c/*Array*/,startIndex,res){
		    	// Summary:
		    	// 		Calculates the two intersections between a line and a quadratic Bezier spline.
			    // p1:
		    	// 		 One point of the line.
			    // p2:
		    	// 		 Another point on the line.
			    // c:
		    	// 		  An array of control points of the spline.
			    // startIndex:
		    	// 		 The index of the first valid control point in the array <code>c</code>.
			    //       There must be three valid control points of the spline in the array.
			    // res:
		    	// 		 Container of two for the resulting intersection points.
			    // return:
		    	// 		 The number of intersections.
		    	
		    	
		       // the spline parameters
		      var cx = c[startIndex].x;
		      var bx = 2 * (c[startIndex + 1].x - cx);
		      var ax = c[startIndex + 2].x - bx - cx;
		      var cy = c[startIndex].y;
		      var by = 2 * (c[startIndex + 1].y - cy);
		      var ay = c[startIndex + 2].y - by - cy;
		      
		      // the line parameters
		      var DistX = p2.x - p1.x;
		      var DistY = p2.y - p1.y;
		      
		      // is it a line? if not, return no solution
		      if (DistX == 0.0 && DistY == 0.0)
		        return 0;
		      
		      // the spline formula is (with t = 0..1):
		      //
		      //     x =  ax * t^2 + bx * t + cx
		      //     y =  ay * t^2 + by * t + cy
		      //
		      // calculate all parameters t of intersection points
		      
		      var numSolutions;
		      var t = [0,0];
		      var f;
		      var g;
		      
		      if ((DistX > 0.0 ? DistX : -DistX) > (DistY > 0.0 ? DistY : -DistY)) {
		        f = DistY/DistX;
		        g = p1.y - f * p1.x;
		        numSolutions = this.calcQuadSolution(f*ax-ay, f*bx-by, f*cx-cy+g, t);
		      }
		      else {
		        f = DistX/DistY;
		        g = p1.x - f * p1.y;
		        numSolutions = this.calcQuadSolution(f*ay-ax, f*by-bx, f*cy-cx+g, t);
		      }
		      
		      // Only those solutions between 0 and 1 are relevant
		      var numPoints = 0;
		      for (var i = 0; i < numSolutions; i++) {
		        if (0 <= t[i] && t[i] <= 1) {
		          res[numPoints++] ={x:(ax * t[i] * t[i] + bx * t[i] + cx),y:(ay * t[i] * t[i] + by * t[i] + cy)};
		        }
		      }
		      
		      return numPoints;
		    },
		   
		   lineIntersectsCubicSpline: function(p1/*Point*/, p2/*Point*/, c/*Array*/, startIndex, res){
			    // Summary:
			    // 		Calculates all three intersections between a line and a cubic Bezier spline.
			    // p1:
		    	// 		 One point of the line.
			    // p2:
		    	// 		 Another point on the line.
			    // c:
		    	// 		  An array of control points of the spline.
			    // startIndex:
		    	// 		 The index of the first valid control point in the array <code>c</code>.
			    //                   There must be four valid control points of the spline in the array.
			    // res:
		    	// 		 Container of three for the resulting intersection points.
			    // return:
		    	// 		 The number of intersections.
		    	
		    	// the spline parameters
		      var dx = c[startIndex].x;
		      var cx = 3 * (c[startIndex + 1].x - dx);
		      var bx = 3 * (c[startIndex + 2].x - c[startIndex + 1].x) - cx;
		      var ax = c[startIndex + 3].x - bx - cx - dx;
		      var dy = c[startIndex].y;
		      var cy = 3 * (c[startIndex + 1].y - dy);
		      var by = 3 * (c[startIndex + 2].y - c[startIndex + 1].y) - cy;
		      var ay = c[startIndex + 3].y - by - cy - dy;
		      
		      // the line parameters
		      var DistX = p2.x - p1.x;
		      var DistY = p2.y - p1.y;
		      
		      // is it a line? if not, return no solution
		      if (DistX == 0.0 && DistY == 0.0)
		        return 0;
		      
		      // the spline formula is (with t = 0..1):
		      //
		      //     x =  ax * t^3 + bx * t^2 + cx * t + dx
		      //     y =  ay * t^3 + by * t^2 + cy * t + dy
		      //
		      // calculate all parameters t of intersection points
		      
		      var numSolutions;
		      var t = [0,0,0];
		      var f;
		      var g;
		      
		      if ((DistX > 0.0 ? DistX : -DistX) > (DistY > 0.0 ? DistY : -DistY)) {
		        f = DistY/DistX;
		        g = p1.y - f * p1.x;
		        numSolutions = this.calcCubicSolution(f*ax-ay, f*bx-by, f*cx-cy, f*dx-dy+g, t);
		      }
		      else {
		        f = DistX/DistY;
		        g = p1.x - f * p1.y;
		        numSolutions = this.calcCubicSolution(f*ay-ax, f*by-bx, f*cy-cx, f*dy-dx+g, t);
		      }
		      
		      // Only those solutions between 0 and 1 are relevant
		      var numPoints = 0;
		      for (var i = 0; i < numSolutions; i++) {
		        if (0 <= t[i] && t[i] <= 1)
		          res[numPoints++] ={x:((ax * t[i] * t[i] * t[i] + bx * t[i] * t[i] + cx * t[i] + dx)),y:((ay * t[i] * t[i] * t[i] + by * t[i] * t[i] + cy * t[i] + dy))};
		      }
		      
		      return numPoints;
		    },

		    GetLineIntersection: function( p1LA/*Point*/, p2LA/*Point*/,p1LB/*Point*/, p2LB/*Point*/,result){
		    	// Summary:
			    // 		Returns the intersection point between two lines.
			    //  p1LA:
			    // 		 One point on the first line.
			    //  p2LA:
			    // 		 Another point on the first line.
			    //  p1LB:
			    // 		 One point on the second line.
			    //  p2LB:
			    // 		 Another point on the second line.
			    //  result:
			    // 		 If not <code>null</code>, this point is used a container
			    //               for the return value and is filled with the intersection
			    //               point if the lines intersect.
			    //               If <code>null</code>, a new point is allocated internally
			    //               and returned, if the lines intersect.
			    // return:
			    // 		 The unique intersection point, or <code>null</code> if the two lines are parallel.
			    
		      return this.GetLineIntersection1( p1LA.x, p1LA.y, p2LA.x, p2LA.y,p1LB.x, p1LB.y, p2LB.x, p2LB.y,result);
		    },
		    
		    
		    GetLineIntersection1: function(ax1, ay1, ax2, ay2, bx1, by1, bx2, by2, result){
		    	// Summary:
		    	// 		Returns the intersection point between two lines.
			    //  ax1:
			    // 		  The x coordinate of the first point on the first line.
			    //  ay1:
			    // 		  The y coordinate of the first point on the first line.
			    //  ax2:
			    // 		  The x coordinate of the second point on the first line.
			    //  ay2:
			    // 		  The y coordinate of the second point on the first line.
			    //  bx1:
			    // 		  The x coordinate of the first point on the second line.
			    //  by1:
			    // 		  The y coordinate of the first point on the second line.
			    //  bx2:
			    // 		  The x coordinate of the second point on the second line.
			    //  by2:
			    // 		  The y coordinate of the second point on the second line.
			    //  result:
			    // 		  If not <code>null</code>, this point is used a container
			    //               for the return value and is filled with the intersection
			    //               point if the lines intersect.
			    //               If <code>null</code>, a new point is allocated internally
			    //               and returned, if the lines intersect.
			    // return:
			    // 		  The unique intersection point, or <code>null</code> if the two lines are parallel.
		    	
		      var dx1 = ax2 - ax1;
		      var dy1 = ay2 - ay1;
		      var dx2 = bx2 - bx1;
		      var dy2 = by2 - by1;
		      
		      // if cross determinantes are equal, the segments are parallel
		      var dx1y2 = dx1 * dy2;
		      var dx2y1 = dx2 * dy1;
		      
		      if (dx1y2 == dx2y1)
		        return null;
		      
		      // for precision, first handle vertical case, then general case
		      if (ax1 == ax2)
		        result.x = ax1;
		      else if (bx1 == bx2)
		        result.x = bx1;
		      else
		        result.x = (ax1 * dx2y1 - bx1 * dx1y2 + (by1  - ay1) * dx1 * dx2) / (dx2y1 - dx1y2);
		      
		      // for precision, first handle horizontal case, then general case
		      if (ay1 == ay2)
		        result.y = ay1;
		      else if (by1 == by2)
		        result.y = by1;
		      else
		        result.y =  (ay1 * dx1y2 - by1 * dx2y1 + (bx1 - ax1) * dy1 * dy2) / (dx1y2 - dx2y1);
		      
		      return result;
		    },
		    
		 // --------------------------------------------------------------------------
		 // Auxiliaries related to arcs / ellipses
		    
		    ellipticAngle: function(rect/*dojox.gfx.Rectangle*/, p/*Point*/, startAngle){
		    // Summary:
		    // 		Returns the elliptic angle of point p relative to the center of rect where
		    // 		the rect is considered the bounding box of an ellipse and startAngle
		    // 		indicates the origin of the elliptic scale.
		    //  rect:
	    	// 		 The bounding box of the complete ellipse.
		    //  p:
	    	// 		 The point whose elliptic angle is calculated.
		    //  startAngle:
	    	// 		 The angle that defines the 0 point.
		    // return:
	    	// 		 The elliptic angle (relative to startAngle). The value is between 0 and 360.
		      var dx = p.x - rect.x - rect.width / 2;
		      var dy = rect.y + rect.height / 2 - p.y;
		      var x = dx / rect.width;
		      var y = dy / rect.height;
		      var a = 180.0 / Math.PI *  Math.atan2(y, x);
		      a -= startAngle;
		      while (a < 0.0) a += 360.0;
		      while (a > 360.0) a -= 360.0;
		      return a;
		    },
		    
		    retrieveArcPointFromAngle: function(rect/*dojox.gfx.Rectangle*/, angle){
		    // Summary:
	    	// 		Returns the point on an arc from an elliptic angle.
		    //  rect:
	    	// 		 The bounding box of the complete ellipse.
		    //  angle:
	    	// 		 The elliptic angle to be considered.
		    //  p:
	    	// 		 Container for the resulting point.	
		    	
		      angle = angle * Math.PI / 180.0;
		      var p ={};
		      p.x = rect.x + 0.5 * rect.width * (1 + Math.cos(angle));
		      p.y = rect.y + 0.5 * rect.height * (1 - Math.sin(angle));
		      return p;
		    },
		    
		    
		    arcStartPoint: function(rect/*dojox.gfx.Rectangle*/, startAngle){
		    // Summary:
		    // 		Returns the starting point of an arc (a partial ellipse).
		    //  rect:
		    // 		 The bounding box of the complete ellipse.
		    //  startAngle:
			// 		  The angle of the starting point in degrees (0 .. 360).
		    // return:
			// 		  The starting point.
		    	
		      var angle = startAngle;
		      var p = retrieveArcPointFromAngle(rect, angle);
		      
		      // now, EllipticAngle should return 0.0. However, due to rounding, it
		      // may return a value representing an angle close to 0.0. We can accept
		      // angles slightly larger than 0 degree, but not angles slightly smaller
		      // than 0 degree because they would create gaps between the arc and the
		      // point. Note that an angle smaller than 0 degree is calculated as
		      // a value around 359.
		      
		      var testAngle = ellipticAngle(rect, p, startAngle);
		      
		      if (testAngle > 180.0) {
		        // search a valid angle by choosing a delta offset
		        var delta = 0.0;
		        while (testAngle > 180.0) {
		          delta += 0.004;
		          angle = startAngle + delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // try to reduce the delta offset until the angle gets invalid
		        var bestDelta = delta;
		        while (testAngle < 180.0) {
		          bestDelta = delta;
		          delta = delta/8;
		          angle = startAngle + delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // now get the resulting point from the last valid angle
		        angle = startAngle + bestDelta;
		        p = retrieveArcPointFromAngle(rect, angle);
		      }
		      return p;
		    },
		    
		    
		    arcEndPoint: function(rect/*dojox.gfx.Rectangle*/, startAngle, deltaAngle){
		    // Summary:
		    // 		Returns the end point of an arc (a partial ellipse).
		    // rect:
		    // 		  The bounding box of the complete ellipse.
		    // startAngle:
		    // 		  The angle of the starting point in degrees (0 .. 360).
		    // deltaAngle:
		    // 		  The angle range of the arc from the starting point to the end point in degrees (0 .. 360).
		    // return:
		    // 		  The end point.
		    	
		      if (deltaAngle == 0.0)
		        return arcStartPoint(rect, startAngle);
		      
		      var angle = startAngle + deltaAngle;
		      var p = retrieveArcPointFromAngle(rect, angle);
		      
		      // now, EllipticAngle should return deltaAngle. However, due to rounding,
		      // it may return a value representing an angle close to deltaAngle. We can
		      // accept angles slightly smaller than deltaAngle, but not angles slightly
		      // larger than deltaAngle because they would create gaps between the arc
		      // and the point.
		      
		      var testAngle = ellipticAngle(rect, p, startAngle);
		      
		      if (testAngle > deltaAngle) {
		        // search a valid angle by choosing a delta offset
		        var delta = 0.0;
		        while (testAngle > deltaAngle) {
		          delta += 0.004;
		          angle = startAngle + deltaAngle - delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // try to reduce the delta offset until the angle gets invalid
		        var bestDelta = delta;
		        while (testAngle < deltaAngle) {
		          bestDelta = delta;
		          delta = delta/8;
		          angle = startAngle + deltaAngle - delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // now get the resulting point from the last valid angle
		        angle = startAngle + deltaAngle - bestDelta;
		        p = retrieveArcPointFromAngle(rect, angle);
		      }
		      return p;
		    },
		    
		    pointsEquals: function(p1,p2){
		    	return p1.x==p2.x && p1.y==p2.y;
		    },
		    
		    calcLinearSolution: function( a, b, result){
		    // Summary:
		    // 		 Calculates the solution of the equation a * x + b = 0.
		    //  a:
		    // 		 Parameter of the equation.
		    //  b:
		    // 		 Parameter of the equation.
		    //  result:
		    // 		 Container for the result. The array must have size 1.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    
		    	
		      if (a == 0.0) {
		        if (b == 0.0) return -1;
		        return 0;
		      }
		      result[0] = -b / a;
		      return 1;
		    },	
		    
		    calcQuadSolution: function( a, b, c, result) {
		    // Summary:
		    // 		 Calculates the solution of the equation a * x^2 + b * x + c = 0.
		    //  a:
		    // 		  Parameter of the equation.
		    //  b:
		    // 		  Parameter of the equation.
		    //  c:
		    // 		  Parameter of the equation.
		    //  result:
		    // 		  Container for the result. The array must have size 2.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    	
		    	
		      if (a == 0.0)
		        return this.calcLinearSolution(b, c, result);
		      
		      // solution is (+/- sqrt(b^2 - 4ac) - b)/(2a)
		      
		      var D = b * b - 4 * a * c;
		      if (D < 0) return 0;
		      if (D == 0) {
		        result[0] = -b / (2 * a);
		        return 1;
		      }
		      var sqrtD = Math.sqrt(D);
		      result[0] = (sqrtD - b) / (2 * a);
		      result[1] = (-sqrtD - b) / (2 * a);
		      return 2;
		    },	  
		    
		    calcCubicSolution: function(a, b, c, d, result){
		    // Summary:
		    // 		 Calculates the solution of the equation a * x^3 + b * x^2 + c * x + d = 0.
		    // a:
		    // 		  Parameter of the equation.
		    // b:
		    // 		  Parameter of the equation.
		    // c:
		    // 		  Parameter of the equation.
		    // d:
		    // 		  Parameter of the equation.
		    // result:
		    // 		  Container for the result. The array must have size 3.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    	
		    	
		      if (a == 0.0)
		        return this.calcQuadSolution(b, c, d, result);
		      
		      // We use a variant of the Cardan-Tartaglia formula (see any good
		      // mathematics formula collectione, e.g., Bronstein 1985, pp. 131-132)
		      
		      var oneThird = 1.0 / 3.0;
		      var z = b / (3 * a);
		      var r = 3 * a * a;
		      var p = (3 * a * c - b * b) / r;
		      var q = (2 * b * b * b - 9 * a * b * c + 9 * r * d) / (9 * r * a);
		      var sgn;
		      
		      // now, solve y^3 + p * y + q = 0  and x = y - z
		      
		      if (p == 0.0) {
		        // 1 solution: y = - q^(1/3)
		        sgn = (q >= 0.0 ? 1.0 : -1.0);
		        result[0] = -sgn * Math.pow(sgn * q, oneThird) - z;
		        return 1;
		      }
		      
		      var cubicPThird = p * p * p / 27;
		      var squareQHalf = q * q / 4;
		      var D = cubicPThird + squareQHalf;
		      
		      if (D < 0) {
		        // The formula of Cardan, Bombelli, and others.
		        // The background is basically the same as below, except that we
		        // need to get the square root of a negative number and hence end up
		        // in the complex number world. Luckily, the i-part of the complex
		        // numbers eliminate each other in the end result.
		        
		        // 3 solutions
		        var rho = Math.sqrt(-cubicPThird);
		        var phi = Math.acos(-q / (2 * rho));
		        sgn = (rho >= 0.0 ? 1.0 : -1.0);
		        var f = 2 * sgn * Math.pow(sgn * rho, oneThird);
		        result[0] = f * Math.cos(phi / 3) - z;
		        result[1] = f * Math.cos((phi + 2 * Math.PI) / 3) - z;
		        result[2] = f * Math.cos((phi + 4 * Math.PI) / 3) - z;
		        return 3;
		      } else {
		        // The original Cardan-Tartaglia formula. The background is this:
		        // (u + v)^3 - 3uv(u + v) - u^3 - v^3 = 0
		        // hence y = (u+v) is a solution iff p=-3uv and u^3+v^3+q = 0
		        // With v=-p/(3u) follows u^3-p^3/(27u^3)+q = 0
		        // that is u^6 + q u^3 - p^3/27 = 0
		        // With u1 = u^3 follows u1^2 + q u1 - p^3/27 = 0.
		        // This quadratic formula can be solved with the standard mechanism to u1,
		        // hence u and v can be obtained. The second solution can be found by
		        // factorizing the first solution.
		        
		        var u1 = (q > 0
		          ? // Avoid rounding errors due to cancellation
		          cubicPThird / (Math.sqrt(D) + q / 2)
		          : Math.sqrt(D) - q / 2);
		        sgn = (u1 >= 0.0 ? 1.0 : -1.0);
		        var u = sgn * Math.pow(sgn * u1, oneThird);
		        
		        // u cannot be zero, because p != 0. Only if p = 0 follows D = q/2
		        // hence u1 = 0 hence u = 0. But just to be sure of rounding problems,
		        // we repeat:
		        
		        if (u == 0.0) {
		          // see above, case p==0.0
		          sgn = (q >= 0.0 ? 1.0 : -1.0);
		          result[0] = -sgn * Math.pow(sgn * q, oneThird) - z;
		          return 1;
		        }
		        
		        var v = -p / (3 * u);
		        if (u == v) {
		          // 2 solutions
		          result[0] = u + v - z;
		          result[1] = -0.5 * (u + v) - z;
		          return 2;
		        } else {
		          // 1 solution
		          result[0] = u + v - z;
		          return 1;
		        }
		      }
		    },	  
		    
		    // --------------------------------------------------------------------------
			// Auxiliaries related to matrix
		    
		    multiplyVectorial: function(matrix,vector){
		    // Summary:
		    //		multiplies a 2x2 matrix towards a 2x1 vector
		    	var rta = [0,0];
		    	for(var i=0;i<2;i++){
		    		for(var j=0;j<2;j++){
		    			rta[i] = rta[i] + ( matrix[i][j] * vector[j]);
		    		}
		    	}
		    	return rta;
		    },
		    multiplyScalar: function(vector,scalar){
			    // Summary:
			    //		multiplies a matrix towards a scalar
			    	var rta = dojo.clone(vector);
			    	for(var i=0;i<vector.length;i++){
			    			rta[i] = vector[i]* scalar;
			    	}
			    	return rta;
			},
			add: function(v1,v2){
			    // Summary:
			    //		adds a 2x2 matrix towards a 2x2 matrix
			    	var rta = [0,0];
			    		for(var j=0;j<2;j++){
			    			rta[j] = v1[j] + v2[j];
			    		}
			    	return rta;
			    },
			
			ellipseEndpointToCenterParameterization: function(p1,p2,fa,fs,rx,ry,alphaDegrees){
				//Summary:
			    //		Change the parameterization of an ellipse, from endpoint to center.
			    var	alpha = this.toRadians(alphaDegrees); 
				
				//Step 1: Compute (x1', y1') according to the formula
				var m1 = [[Math.cos(alpha),Math.sin(alpha)],[-Math.sin(alpha),Math.cos(alpha)]];
				var v1 = [(p1.x-p2.x)/2,(p1.y-p2.y)/2];
				var pt = this.multiplyVectorial(m1,v1);
				
				//Step 2: Compute (cX', cY') according to the formula
				var t1 = Math.pow(rx,2)*Math.pow(ry,2);
				var t2 = Math.pow(rx,2)*Math.pow(pt[1],2);
				var t3 = Math.pow(ry,2)*Math.pow(pt[0],2);
				var sqrt = Math.pow((t1 - t2 - t3)/(t2 + t3),(1/2));
				var v2 =[rx*pt[1]/ry,-ry*pt[0]/rx];
				var ct = this.multiplyScalar(v2,sqrt);//(cX', cY')
				
				//the - sign is chosen if fa = fs   
				if(fa==fs){
					ct = this.multiplyScalar(ct,-1);
				}
				
				//Step 3: Compute (cX, cY) from (cX', cY')
				var m2 = [[Math.cos(alpha),-Math.sin(alpha)],[Math.sin(alpha),Math.cos(alpha)]];
				var v3 = [(p1.x+p2.x)/2,(p1.y+p2.y)/2];
				var center = this.add(this.multiplyVectorial(m2,ct),v3);
				
				return {x:center[0],y:center[1]};
				
			},
			ellipticalAngle: function(startPoint,endPoint,center){
				var v1 = {x:startPoint.x-center.x,y:startPoint.y-center.y};
				var v2 = {x:endPoint.x-center.x,y:endPoint.y-center.y};
				return vectorsAngle(v1,v2);
			},
			vectorsAngle: function(v1,v2){ 
				var t1 = v1.x*v2.x+v1.y*v2.y;
				var t2 = this.vectorLenght(v1)*this.vectorLenght(v2);
				return Math.acos(t1/t2);
			},
			vectorLenght: function(v){
				return Math.pow(Math.pow(v.x,2)+Math.pow(v.y,2),1/2);
			},
			toDegrees: function(radian){
				return (360*radian)/(2*Math.PI);
			},
			toRadians: function(degrees){
				return (degrees*2*Math.PI)/360;
			},
			
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////    GFX SHAPES CLIPPING FUNCTIONS     ///////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			
		clipToShape: function(node,graph,shape,bounds,unusedPoint, connectionPoint, referencePoint,completeResult){	
				
				var result = [],resultPoint,numberOfResults = 0, clipConnectionPoint = {}, points, poli, 
					s, stype = shape.getShape ? shape.getShape().type : null; 
				clipConnectionPoint.x = connectionPoint.x;
				clipConnectionPoint.y = connectionPoint.y;
				
				if(stype === 'rect'){
					s = shape.getShape();
					points = [{x:s.x,y:s.y},{x:s.x+s.width,y:s.y},{x:s.x+s.width,y:s.y+s.height},{x:s.x,y:s.y+s.height},{x:s.x,y:s.y}];
					poli = this._transformPoints(shape/*node*/,graph,points);
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,poli,true,result);
				}else if(stype === 'circle' || stype === 'ellipse'){
					numberOfResults = this.lineIntersectsEllipse(referencePoint,clipConnectionPoint,bounds,result);
				}else if(stype === 'polyline'){
					points = shape.getShape().points;
					poli = this._transformPoints(shape/*node*/,graph,points);
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,poli,true,result);
				}else if(stype === 'path'){
					numberOfResults = this._clipOnPath(node,graph,shape,unusedPoint, clipConnectionPoint, referencePoint,result);
				}else if (shape instanceof Object && shape.width && shape.height){
					// workaround to deal with raw rectangle
					s = shape;
					points = [{x:s.x,y:s.y},{x:(s.x+s.width),y:s.y},{x:(s.x+s.width),y:(s.y+s.height)},{x:s.x,y:(s.y+s.height)},{x:s.x,y:s.y}];
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,points,true,result);
				}
				
				if(completeResult){
					completeResult.points = this._filterPoints(result,clipConnectionPoint, referencePoint);
					completeResult.numberOfResults = completeResult.points.length;
				}
				
				if(numberOfResults>0){
					// FIXME using the original results because the bounding box is not being calculated correctly
					//var resultPoint = this.bestClipPointOnRay(referencePoint,clipConnectionPoint,completeResult.points,completeResult.numberOfResults);
					
					resultPoint = this.bestClipPointOnRay(clipConnectionPoint,referencePoint,result,numberOfResults);
					return resultPoint;
				}else{
					return null;
				}
			},
			
			_filterPoints: function(points,p1,p2){
				var g = ibm_ilog.diagram.util.Geometry
				var result = [];
				var rect = g.createRect(p1,p2);
				for(pn in points){
					var p = points[pn];
					if(g.containsPoint(rect,p)){
						result.push(p);
					}
				}
				return result;
			},
			
			_transformPoints: function(node,graph,points){
				// Summary:
				//		transform the given points, to the absolute graph coordinates
				
				var result = [];
				for(pn in points){
					var p = points[pn];
					p = this._transformPoint(node,graph,p);
					result.push(p);
				}
				return result;
			},
			
			_transformPoint: function(node,graph,point){
				// Summary:
				//		transform the given point, to the absolute graph coordinates
				var _m = dojox.gfx.matrix;
				//var p2 = _m.multiplyPoint(node.getTransform(), point);
				var p2 =  _m.multiplyPoint(node.getShapeToContainerTransform(graph), point);
				return p2;
			},
			
			_clipOnPath: function(node,graph,shape,unusedPoint, connectionPoint, referencePoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Path shape.
				var segments = shape.segments;
				var lastPoint = this._transformPoint(shape/*node*/,graph,{x:0,y:0});
				var reflectionControlPoint = {};
				var firstPointPath = this._transformPoint(shape/*node*/,graph,{x:0,y:0});
				for(var index in segments){
					var segment = segments[index];
					var action = segment.action;
					var args = segment.args;
						if( action == 'M'){ //absolute move
							lastPoint = {x:segments[index].args[0],y:segments[index].args[1]};
							reflectionControlPoint = {}; 
							firstPointPath = lastPoint; continue;}
						if( action == 'm'){ //relative move
							lastPoint =  this._changeToAbsolute( {x:segments[index].args[0],y:segments[index].args[1]},lastPoint);
							reflectionControlPoint = {}; 
							firstPointPath = lastPoint; continue;}
						if( action == 'Z' || action == 'z' ){ // close path
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,firstPointPath,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'L' ){//absolute line
							var p2 = {x:args[0],y:args[1]};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'l' ){//relative line
							var p2 = this._changeToAbsolute({x:args[0],y:args[1]},lastPoint);
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'H' ){//horizontal absolute line
							var p2 =  {x:args[0],y:lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'h' ){//horizontal relative line
							var p2 = {x:args[0]+lastPoint.x,y:lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'V' ){//vertical absolute line
							var p2 = {x:lastPoint.x,y:args[0]};
							lastPoint = this._clipOnLinearPath(shape/*node*/,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'v' ){//vertical relative line
							p2 = {x:lastPoint.x,y:args[0]+lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'C' || action == 'c' || action == 'S' || action == 's' ){ // cubic bezier
							lastPoint = this._clipOnCubicPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
						if( action == 'Q' || action == 'q' || action == 'T' || action == 't'){ // quadratic bezier
							lastPoint = this._clipOnQuadraticPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
						if( action == 'A' || action == 'a' ){ 
							lastPoint = this._clipOnArcPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
				}
				return intersections.length;
			},
			
			_clipOnLinearPath: function(node,graph,p1,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Linear Path shape.
				var crossPoint = {};
				
				if (this.lineIntersectsSegment(connectionPoint,referencePoint,this._transformPoint(node,graph,p1), this._transformPoint(node,graph,p2), crossPoint)) {
		            intersections.push(crossPoint);
				}
				reflectionControlPoint.action = 'l';
				return p2;
			},
			
			_clipOnQuadraticPath: function(node,graph,segment,connectionPoint,referencePoint,lastPoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Quadratic Bezier Path shape.
				var args = segment.args;
				var result = [];
				var p1;
				var p2;
				
				// DIFFERENCE BETWEEN Q (normal) AND T (smooth) 
				if( segment.action.toLowerCase() == 'q'){
					p1 = {x:args[0],y:args[1]};
					p2 = {x:args[2],y:args[3]};
				}else{ // case T
					p2 = {x:args[0],y:args[1]};
					if( reflectionControlPoint != {} && reflectionControlPoint.action.toLowerCase() == 'q' ){
						p1 = {x:lastPoint.x+reflectionControlPoint.x, y:lastPoint.y+reflectionControlPoint.y};
					}else{
						p1 = {x:lastPoint.x, y:lastPoint.y};
					}
				}
				
				// RELATIVE Transform
				if( segment.action == 'q'){
					p1 = this._changeToAbsolute(p1,lastPoint);
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				if( segment.action == 't'){
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				
				// set reflection point
				reflectionControlPoint.x = p2.x-p1.x;
				reflectionControlPoint.y = p2.y-p1.y;
				reflectionControlPoint.action = 'q';
				
				var points = [lastPoint,p1,p2];
				var controlPoints = this._transformPoints(node,graph,points);
				var n = this.lineIntersectsQuadSpline(referencePoint,connectionPoint,controlPoints,0,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				return p2;
			},
			
			_clipOnCubicPath: function(node,graph,segment,connectionPoint,referencePoint,lastPoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Cubic Bezier Path shape.
				
				var args = segment.args;
				var result = [];
				var p1;
				var p2;
				var p3;
				if( segment.action.toLowerCase() == 'c'){
					p1 = {x:args[0],y:args[1]};
					p2 = {x:args[2],y:args[3]};
					p3 = {x:args[4],y:args[5]};
				}else{ // case T
					p2 = {x:args[0],y:args[1]};
					p3 = {x:args[2],y:args[3]};
					if( reflectionControlPoint != {} && reflectionControlPoint.action.toLowerCase() == 'c' ){
						p1 = {x:lastPoint.x+reflectionControlPoint.x, y:lastPoint.y+reflectionControlPoint.y};
					}else{
						p1 = {x:lastPoint.x, y:lastPoint.y};
					}	
				}
				
				if( segment.action == 'c'){
					p1 = this._changeToAbsolute(p1,lastPoint);
					p2 = this._changeToAbsolute(p2,lastPoint);
					p3 = this._changeToAbsolute(p3,lastPoint);
				}
				if( segment.action == 's'){
					p2 = this._changeToAbsolute(p2,lastPoint);
					p3 = this._changeToAbsolute(p3,lastPoint);
				}
				
				reflectionControlPoint.x = p3.x-p2.x;
				reflectionControlPoint.y = p3.y-p2.y;
				reflectionControlPoint.action = 'c';
				
				var points = [lastPoint,p1,p2,p3];
				var controlPoints = this._transformPoints(node,graph,points);
				var n = this.lineIntersectsCubicSpline(referencePoint,connectionPoint,controlPoints,0,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				return p3;
			},
			
			_clipOnArcPath:function(node,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections){
				// Summary:
				//		return the connection point, clipping to a partial arc Path.
				var args = segment.args;
				var p2 = {x:args[5],y:args[6]};
				var rx = args[0];
				var ry = args[1];
				var	xAxisRotation = args[2];
				var largeArcFlag = args[3]; 
				var sweepFlag =	args[4];
				
				if(segment.action == 'a'){
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				var result = [];
				var n = this.lineIntersectsPartialEllipse(referencePoint,connectionPoint,rx,ry, this._transformPoint(node,graph,lastPoint),this._transformPoint(node,graph,p2),largeArcFlag,sweepFlag,xAxisRotation,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				
				reflectionControlPoint.action = 'a';
				
				return p2;
				
			},
			
			_changeToAbsolute: function(point, lastPoint){
				// Summary:
				//		return the given point, changing its relative coordinates to absolute, according to the last point.
				return {x: point.x+lastPoint.x, y:point.y+lastPoint.y}
			},		
			
			_eof:function(){}
	};

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.Port']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Port'] = true;
dojo.provide('ibm_ilog.diagram.Port');







dojo.declare("ibm_ilog.diagram.PortBase", null, {
	// summary:
	//		The PortBase class represents the base class for ports.
	// description:
	//		A port defines a connection point between a Link and a Node.
	//		The PortBase class is intended as the base class for subclasses that
	//		define different types of ports. PortBase instances should not be created
	//		directly, only its subclasses should be used.
	
	_owner: null,
	_links: null,
	_clipOnShape: null,
	
	getOwner: function() {
		// summary:
		//		Gets the owner Node of this port.
		// returns:
		//		The Node to which this port belongs.
		return this._owner;
	},
	
	_setOwner: function(owner) {
		// summary:
		//		Sets the owner Node of this port.
		// owner: ibm_ilog.diagram.Node:
		//		The new owner of this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		
		if (owner != this._owner) {
			this._owner = owner;
			this.invalidateLinks();
		}
	},
	
	getConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary: 
		//		Gets the connection point and the origin point for this port.
		// link: ibm_ilog.diagram.Link:
		//		The Link that is (or will be) connected to this port.
		// referencePoint:
		//		The opposite point of the segment connected to this port.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the connection point is relative to.
		// connectionPoint:
		//		An object (with 'x' and 'y' properties) into which the connection point will be stored.
		//		The connection point is the point where the link will start or end.
		// originPoint:
		//		An object (with 'x' and 'y' properties) into which the origin point will be stored.
		//		The origin point is the point towards which the link will 'aim' (typically the center of the node),
		//		and is used to determine the shape of orthogonal links for example.
		// returns:
		//		The 'connectionPoint' argument.
		// description:
		//		This is the central method of the PortBase class and is called by the Link object
		//		to get its start and end points. This method calls computeConnectionPoint, which is
		//		the method that actually calculates the connection point in the PortBase subclasses.
		//		Then, getConnectionPoint applies the clipping if getClipOnShape() is true.
		//		So, getConnectionPoint should not be overridden in subclasses, computeConnectionPoint
		//		is the method that is overridden instead.
		this.computeConnectionPoint(link, parent, referencePoint, connectionPoint, originPoint, start);
		if(this.getClipOnShape()){
			var p = this._clipToShape(link,parent,referencePoint, connectionPoint, originPoint, start);
			connectionPoint.x = p.x;
			connectionPoint.y = p.y;
		}
		return connectionPoint;
	},	
	
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary: 
		//		Computes the connection point and the origin point for a given subclass of PortBase.
		// link: ibm_ilog.diagram.Link:
		//		The Link that is (or will be) connected to this port.
		// referencePoint:
		//		The opposite point of the segment connected to this port.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the connection point is relative to.
		// connectionPoint:
		//		An object (with 'x' and 'y' properties) into which the connection point will be stored.
		//		The connection point is the point where the link will start or end.
		// originPoint:
		//		An object (with 'x' and 'y' properties) into which the origin point will be stored.
		//		The origin point is the point towards which the link will 'aim' (typically the center of the node),
		//		and is used to determine the shape of orthogonal links for example.
		// returns:
		//		The 'connectionPoint' argument.
		// description:
		//		This method is called by PortBase.getConnectionPoint, and should be overridden in
		//		PortBase subclasses to actually implement the calculation of the connection point.
		//		Clipping on the node's shape is implemented by getConnectionPoint, so
		//		computeConnectionPoint does not have to care about it.
		//		The base implementation for the PortBase class returns the center of the node for both
		//		the connection point and the origin point.
		
		var bounds = this.getConnectionBounds(parent);
		connectionPoint.x = originPoint.x = bounds.x + bounds.width / 2;
		connectionPoint.y = originPoint.y = bounds.y + bounds.height / 2;
		return connectionPoint;
	},
	
	getConnectionBounds: function(parent) {
		// summary:
		//		Gets the bounds of the owner node's base shape used to compute the connection point.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the bounds are relative to.
		// returns:
		//		A rectangle (with 'x', 'y', 'width' and 'height' properties) containing the bounds
		//		used to compute the connection point.
		// description:
		//		This method returns the bounds that will be used by the port to compute the connection and origin points.
		//		The default implementation returns the bounds of the base shape returned by getBaseShape().
		//		This method	can be overridden to return different bounds.
		
		if(!parent)
			parent = this._owner.getParent();
		
		var baseShape = this.getBaseShape();
        var rect = baseShape.getBoundingBox();
        var m = dojox.gfx.matrix;
        // t1: transform from surface -> base shape
        var t1 = baseShape._getRealMatrix() || m.identity;
        // t2: transform from surface -> parent
        var t2 = parent._getRealMatrix() || m.identity;
        var t = m.multiply(t2.inverse(), t1);
		rect = t.transformRectangle(rect);
        return rect;
	},
	
	needsReferencePoint: function() {
		// summary:
		//		Indicates whether or not this port needs a reference point to compute the connection point.
		// returns:
		//		True if this port needs a reference point to compute the connection point,
		//		or false otherwise.
		// description:
		//		This method is used by a PortBase subclass implementation to signal whether or not
		//		it needs (and uses) the referencePoint argument passed to the getConnectionPoint method.
		//		A subclass must return true if it actually uses the referencePoint, or false if it does
		//		not use it. If needsReferencePoint returns true, the caller of the getConnectionPoint method
		//		(usually the Link object) must pass a valid referencePoint. If needsReferencePoint returns false,
		//		the PortBase subclass guarantees that the referencePoint will never be used or referenced,
		//		so the caller may pass an invalid point, or a null or undefined object.
		//		By default the method returns true if getClipOnShape() returns true, of false otherwise.
		//		PortBase subclasses that always need a reference point must override this method and return true.
		
		return this._clipOnShape || false;
	},
	
	invalidateLinks: function() {
		// summary:
		//		Invalidates the shape of all the links connected to this port. 
		// description:
		//		Invalidates the shape of all the links connected to this port. 
		//		This method must be called by PortBase subclasses when a change of the port
		//		causes the connection point to change.
		if(this._links)
			for(var i = 0; i < this._links.length; i++)
				this._links[i].invalidateLinkShape();
				
	},
	
	_addLink: function(link) {
		// summary:
		//		Adds a link to the list of links connected to this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		if(!this._links)
			this._links = [ link ];
		else
			this._links.push(link);
	},
	
	_removeLink: function(link) {
		// summary:
		//		Removes a link from the list of links connected to this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		if (this._links) {
			var index = dojo.indexOf(this._links, link);
			if (index >= 0) {
				if(this._links.length == 1)
					this._links = null;
				else
					this._links.splice(index, 1);
			}
		}
	},
	
	getLinks: function() {
		// summary:
		//		Gets the links connected to this port.
		// returns:
		//		An array of Links connected to this port.
		// description:
		//		Note that this method always returns a newly allocated array.
		var links = [];
		if (this._links) {
			for(var i = 0; i < this._links.length; i++)
			links.push(this._links[i]);
		}
		return links;
	},
	
	getLinksCount: function() {
		// summary:
		//		Gets the number of links connected to this port.
		
		return this._links ? this._links.length : 0;
	},

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////   CLIPPING TOOLS ///////////////////////////////////////////////////////////////////	
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	setClipOnShape: function(clipOnShape){
		// summary:
		//		Determines whether the connection point is "clipped" on the base shape of the node.
		// description:
		//		If clipOnShape is true, the getConnectionPoint method will extend the segment defined
		//		by the reference point and the connection point so that the connection point is on the
		//		outline of the node's shape, so that the connection point always "touches" the node shape
		//		even if the node is not rectangular (for example, if the node has an elliptical of polygonal shape).
		//		If clipOnShape is false, the getConnectionPoint always returns the return of computeConnectionPoint
		//		without modifying it, and the point may not be exactly on the boundary on the node shape.
		// clipOnShape:
		//		If true, the connection point is clipped on the node's shape, otherwise the connection point
		//		is the point returned by computeConnectionPoint.
		
		if (clipOnShape != this._clipOnShape) {
			this._clipOnShape = clipOnShape;
			this.invalidateLinks();
		}
	},
	
	getClipOnShape: function(){
		// summary:
		//		Gets the flag that determines whether the connection point is "clipped" on the base shape of the node.
		// description:
		//		If clipOnShape is true, the getConnectionPoint method will extend the segment defined
		//		by the reference point and the connection point so that the connection point is on the
		//		outline of the node's shape, so that the connection point always "touches" the node shape
		//		even if the node is not rectangular (for example, if the node has an elliptical of polygonal shape).
		//		If clipOnShape is false, the getConnectionPoint always returns the return of computeConnectionPoint
		//		without modifying it, and the point may not be exactly on the boundary on the node shape.
		// returns:
		//		True if the connection point is clipped on the node's shape, or false otherwise.
		
		return this._clipOnShape;
	},
	
	getBaseShape : function() {
		// summary:
		//		Gets the shape contained in the owner node's template to which links should be connected.
		// description:
		//		The base shape is used by getConnectionBounds to compute the default connection bounds,
		//		and also when clipOnShape is true to clip the connection point if the base shape is
		//		non-rectangular.
		//		By default the base shape is determined by calling the getBaseShape() of the owner node,
		//		but this can be overridden to connect links to a different template part of the node.
		
		return this._owner.getBaseShape();
	},
	
	_clipToShape: function(link,parent,referencePoint, connectionPoint, originPoint, start){
		
		var node = this._owner;
		var shape = this.getBaseShape();
		var bounds = this.getConnectionBounds(parent);
		
		if(shape == node){
			var shape = ibm_ilog.diagram.util.Geometry.Rect(bounds.x,bounds.y,bounds.width,bounds.height);
		}
			
		var resClipPoint = ibm_ilog.diagram.util.ClippingUtil.clipToShape(node,parent,shape,bounds,originPoint, connectionPoint, referencePoint)

		return resClipPoint?resClipPoint:connectionPoint;
	}
		
});

dojo.declare("ibm_ilog.diagram.BasicPort", ibm_ilog.diagram.PortBase, {
	// summary:
	//		The BasicPort class represents ports that define fixed connection points
	//		relative to the rectangular bounds of a node.
	//		The 'position' property is an object with 'x' and 'y' properties representing
	//		ratios (between 0 and 1) of the width and height of the node. For example,
	//		the position { x: 0.5, y: 0.5} defines a connection point at the center of a node, and
	//		the position { x: 1, y: 0 } defines a connection point at the top-right of a node.
	//		The 'offset' property (another x/y object, but in pixels this time) can be used
	//		to alter slightly this relative position.
	
	// Allows to test the class faster that testing declaredClass
	_isIBMDiagramBasicPort: true,
	
	_position: null,
	_offset: null,
	_movable: true,
	
    constructor: function() {
		// summary:
		//		Creates a new BasicPort with a default position set to the center of the node.
		// description:
		//		The default position is { x: 0.5, y: 0.5 }, and the default offset is { x: 0, y: 0 }.
		this._position = { x: 0.5, y: 0.5 };
		this._offset = { x: 0, y: 0 };
	},
	
	getPosition: function() {
		// summary:
		//		Gets the position of this port.
		// returns:
		//		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the node width/height.
		return this._position;
	},
	
	setPosition: function(position) {
		// summary:
		//		Sets the position of this port.
		// position:
		//		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the node width/height.
		
		if(typeof(position) != 'object' || !('x' in position) || !('y' in position))
			ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument",this, "setPosition", "position", position);
		
		if (position.x != this._position.x ||
		    position.y != this._position.y) {
			this._position.x = position.x;
			this._position.y = position.y;
			this.invalidateLinks();
		}
	},
		
    isMovable: function() {
        // summary:
        //		Gets the movable state of this port.
        // returns:
        //		True if graph layout and user interaction are allowed to change the position 
	    //      of the port.
        //		False if graph layout and user interaction are not allowed to change the 
		//      position of the port.
        
		// AVA: the design of the class hierarchy for Ports is different in  
		// Diagram for Flex in the sense that the base class (PortBase) defines
		// a "move" function (default implementation is empty). Hence, the functions
		// "set/isMovable" plus the marker "supportsMove" are also defined at the
		// base class. This allows the code that uses the port to treat all ports in
		// a generic way, without the need to check the port class.
		// Here, the function which allows to move the port ("setPosition") exists
		// only on BasicPort, not on PortBase. Therefore, we cannot place "set/isMovable"
		// and "supportsMove" at PortBase. Not a big deal, but maybe it would be worth
		// to review the design.
		// With the current design, Diagram.flex' "supportsMove" makes no sense here,
		// therefore I only added "set/isMovable".  
        return this._movable ? true : false;
    },
    
    setMovable: function(m) {
        // summary:
        //		Sets the movable state of this port. The ports are movable by default.
        // m: Boolean:
        //		If 'm' is true, graph layout and user interaction are allowed to change the position 
	    //      of the port.
        //		If 'm' is false, graph layout and user interaction are not allowed to change the 
		//      position of the port.
        
        this._movable = m;
    },
	
	getOffset: function() {
		// summary:
		//		Gets the offset of this port.
		// returns:
		//		An object with 'x' and 'y' properties representing horizontal/vertical offsets in pixels.
		return this._offset;
	},
	
	setOffset: function(offset) {
		// summary:
		//		Sets the offset of this port.
		// offset:
		//		An object with 'x' and 'y' properties representing horizontal/vertical offsets in pixels.
		
		if(typeof(offset) != 'object' || !('x' in offset) || !('y' in offset))
			ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument",this, "setOffset", "offset", offset);
		
		if (offset.x != this._offset.x ||
		    offset.y != this._offset.y) {
			this._offset.x = offset.x;
			this._offset.y = offset.y;
			this.invalidateLinks();
		}
	},
	
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary:
		//		Implements the calculation of the connection and origin points for the BasicPort class.
		// description:
		//		The connection point is computed based on the position and offset properties.
		//		The origin point is always the center of the node.
		
		var bounds = this.getConnectionBounds(parent);
		connectionPoint.x = bounds.x + bounds.width * this._position.x;
		connectionPoint.y = bounds.y + bounds.height * this._position.y;
		originPoint.x = bounds.x + bounds.width/2;
		originPoint.y = bounds.y + bounds.height/2;
		connectionPoint.x += this._offset.x;
		connectionPoint.y += this._offset.y;
		return connectionPoint;
	}
});

dojo.declare("ibm_ilog.diagram.AutomaticPort", ibm_ilog.diagram.PortBase, {
	// summary:
	//		The AutomaticPort class represents a port that connects a link to the middle of the
	//		closest side of a node.
	
	needsReferencePoint: function(){
		// summary:
		//		Returns true to indicate that the automatic port needs a reference point
		//		to compute the connection point.
		return true;
	},
		
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary:
		//		Implements the calculation of the connection and origin points for the AutomaticPort class.
		// description:
		//		Computes the connection point as the middle of the side of the node that is closest
		//		to the reference point.
		//		The origin point is always the center of the node.
		
		var bounds = this.getConnectionBounds(parent);
		var cx = bounds.x + bounds.width/2;
		var cy = bounds.y + bounds.height/2;
		originPoint.x = cx;
		originPoint.y = cy;
		connectionPoint.x = cx;
		connectionPoint.y = cy;
		if (bounds.width > 0 && bounds.height > 0) {
			var dx = referencePoint.x - cx;
			var dy = referencePoint.y - cy;
			var refRatio = dx != 0 ? (dy > 0 ? dy : -dy) / (dx > 0 ? dx : -dx) : 2;
			if (refRatio > 1) {
				// above or below: determine which from sign of dy
				connectionPoint.y = bounds.y;
				if (dy > 0) 
					connectionPoint.y += bounds.height;
			}
			else {
				// left or right: determine which from sign of dx
				connectionPoint.x = bounds.x;
				if (dx > 0) 
					connectionPoint.x += bounds.width;
			}
		}
		return connectionPoint;
	}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.util.Batch']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.Batch'] = true;
dojo.provide('ibm_ilog.diagram.util.Batch');




// Automatic batch mechanism to group possibly costly changes in Diagram objects.

ibm_ilog.diagram.util.Batch = {
	
	// the current batch
	_batch : [],
	// the timer that will call endBatch
	_timer : null,
	
	add: function(obj){
		// summary:
		//		Adds the specified object to the current batch.
		// obj:
		//		The object to add to the current batch.
		// description:
		//		This method checks whether the specified object is already contained in the current
		//		batch, and if not adds it. When the first object is added, a timer is started so as
		//		to have the endBatch() function called automatically as soon as the current event loop
		//		is finished.
		//		This method is fast and can be called many times on an object without performance issues.
		
		if (!obj._inBatch) {
			ibm_ilog.diagram.util.Batch._batch.push(obj);
			obj._inBatch = true;
			if (!ibm_ilog.diagram.util.Batch._timer) {
				ibm_ilog.diagram.util.Batch._timer = setTimeout(ibm_ilog.diagram.util.Batch.endBatch, 0);
			}
		}
	},
	
	endBatch: function()
	{
		// summary:
		//		Calls the _endBatch() method of all objects in the current batch.
		// description:
		//		This function is normally called automatically by the batch timer.
		//		You can call it explicitly if you want to refresh all the objects immediately.
		//		It is also possible to have endBatch() called when a specified method of a specified
		//		class is called, using endBatchOn(type, method).
		
		try {
			// Objects may be batched while we are here, so we save the current batch list
			// in a local variable and allocate a new "public" batch list immediately.
			// We loop until the new batch list is empty.
			// TODO: test for infinite loops?
			var batch = ibm_ilog.diagram.util.Batch._batch;
			ibm_ilog.diagram.util.Batch._batch = [];
			while (batch.length > 0) {
				for (var i = 0; i < batch.length; i++) {
					var obj = batch[i];
					if (obj._inBatch) {
                        try {
                            obj._endBatch();
                        } finally {
                            obj._inBatch = false;
                        }
					}
				}
				batch = ibm_ilog.diagram.util.Batch._batch;
				ibm_ilog.diagram.util.Batch._batch = [];
			}
		} finally {
			ibm_ilog.diagram.util.Batch._timer = null;
		}
	},
	
	endBatchOn: function(type, method){
		// summary:
		//		Ensures that endBatch() is called when the specified method
		//		of the specified class is called.
		// description:
		//		You can call endBatchOn(type, method) to make sure all objects are refreshed immediately
		//		when the specified method is called.
		//		For example, by default, ibm_ilog.diagram.util.Batch.endBatchOn(dojox.gfx.Moveable, "onMove")
		//		is called to force refresh when an object is moved using a GFX Moveable object.
		//		Otherwise, links may seem to visually "lag behind" nodes.
		
		var old = type.prototype[method];
		type.prototype[method] = function(){
			var ret = old.apply(this, arguments);
			ibm_ilog.diagram.util.Batch.endBatch();
			return ret;
		}
	}
};

// Make sure the current batch ends immediately when an object is moved using a Moveable:

ibm_ilog.diagram.util.Batch.endBatchOn(dojox.gfx.Moveable, "onMove");

}

if(!dojo._hasResource['ibm_ilog.diagram.Link']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Link'] = true;
dojo.provide('ibm_ilog.diagram.Link');














ibm_ilog.diagram.LinkShapeType = {
    Straight: 1,
    Orthogonal: 2,
    OrthogonalEditable: 3,
    Oblique: 4,
    Free: 5
};

dojo.declare("ibm_ilog.diagram.Link", [ibm_ilog.diagram.GraphElement], {
    // summary:
    //		A Link is a graphic object that connects two Node objects.
    // description:
    //		The link is a templatable object that connects two nodes.
    //		A link is created by calling graph.createLink(template), where 'graph' is an ibm_ilog.diagram.Graph instance,
    //		and 'template' is a GFX definition of the shapes that make up the link.
    //		The template must contain at least a GFX shape that will represent the polyline shape of the link.
    //		This shape must have a "dojoAttachPoint: '_path'" property.
    //		The template can contain one or two shapes representing the start and/or end arrows
    //		of the link, with "dojoAttachPoint: '_startArrow'" and "dojoAttachPoint: '_endArrow'" properties.
	//		The template can also contain decorations, that is, shapes that will placed placed automatically
	//		along the link's path. Decorations are identified by a isDecoration property which must be set to true.
	//		Additional properties can be specified to define how a decoration is placed and aligned on the path.
	//		See the getDecorations() method for more details.
    
	// Allows to test the class faster that testing declaredClass
	_isIBMDiagramLink: true,
	
	//
	//	TID: String
	//		Graph element Type ID, used to distinguish basic graph 
  //		element types:
  //	
  //			N = Node
  //			S = Subgraph
  //			L = Link
  //	
	//		Usually used as a very fast index into maps based on 
  //		ibm_diagram graph element types.
	//
  TID: 'L',
	
    // properties
    _startPort: null,
    _endPort: null,
    _startPoint: {
        x: 0,
        y: 0
    },
    _endPoint: {
        x: 100,
        y: 0
    },
    _intermediatePoints: null,
    _shapeType: ibm_ilog.diagram.LinkShapeType.Free,
    _nodeDistance: 20,
    _hitTolerance: 0,
    
    // template parts
    _path: null,
    _startArrow: null,
    _endArrow: null,
    
    // internal state	
    _pathPoints: null,
    _pathPointsValid: false,
    _selectionPath: null,
	
	_isIntergraphLinkAtStart: false,
	_isIntergraphLinkAtEnd: false,

	_curveTension: 0,
	
    constructor: function() {
        // summary: Constructor.

        this._startPoint = {
            x: 0,
            y: 0
        };
        this._endPoint = {
            x: 100,
            y: 0
        };
    },
    
    //---------- Disposal ----------
    
    _dispose: function() {
        // summary:
        //		Dispose this Link.
        // description:
        //		This function should be called when the link is completely deleted, that is, when it is removed
        //		from its parent graph and it will never be used again.
        
        this.setStartPort(null);
        this.setEndPort(null);
    },
    
    invalidate:function(){},
    applyLayout:function(){},
    
    //---------- Start/end ports ----------
    
    getStartPort: function() {
        // summary:
        //		Gets the start port of the link.
        // returns:
        //		The PortBase object that defines the start point of the link.
        return this._startPort;
    },
    
    setStartPort: function(port) {
        // summary:
        //		Sets the start port of the link.
        // port: ibm_ilog.diagram.PortBase:
        //		The PortBase object that defines the start point of the link.
        if (this._startPort != port) {
            if (this._startPort)
				this._startPort._removeLink(this);
			this._checkIntergraphLink(this._startPort, port, true);
            this._startPort = port;
            if (this._startPort)
				this._startPort._addLink(this);
            this.invalidateLinkShape();
        }
        return this;
    },
    
    getEndPort: function() {
        // summary:
        //		Gets the end port of the link.
        // returns:
        //		The PortBase object that defines the end point of the link.
        return this._endPort;
    },
    
    setEndPort: function(port) {
        // summary:
        //		Sets the end port of the link.
        // port: ibm_ilog.diagram.PortBase:
        //		The PortBase object that defines the end point of the link.
        if (this._endPort != port) {
			if (this._endPort) 
				this._endPort._removeLink(this);
			this._checkIntergraphLink(this._endPort, port, false);
			this._endPort = port;
			if (this._endPort)
				this._endPort._addLink(this);
			this.invalidateLinkShape();
		}
        return this;
    },
	
    _forceCheckIntergraphLink: function(){
		this._isIntergraphLinkAtStart = this._forceCheckIntergraphLink1(this.getStartPort(),this._isIntergraphLinkAtStart);
		this._isIntergraphLinkAtEnd = this._forceCheckIntergraphLink1(this.getEndPort(),this._isIntergraphLinkAtEnd);
    },
    
    _forceCheckIntergraphLink1: function(port,isIntergraphLink){
    	if(port){
    		if(isIntergraphLink){
    			this._checkIntergraphLink1(port, true);
    		}
    		return this._checkIntergraphLink1(port, false);
		}
    	return isIntergraphLink;
    },
    
	_checkIntergraphLink: function(oldPort, newPort, start){
		// check cases when we know this is not necessary:
		if(oldPort && newPort && oldPort._owner == newPort._owner)
			return;
		// remove link from _integraphLinks arrays for old port:
		var isIntergraphLink = start ? this._isIntergraphLinkAtStart : this._isIntergraphLinkAtEnd;
		if (oldPort && isIntergraphLink)
			this._checkIntergraphLink1(oldPort, true);
		// add link from _integraphLinks arrays for new port:
		if (newPort) {
			isIntergraphLink = this._checkIntergraphLink1(newPort, false);
			if(start)
				this._isIntergraphLinkAtStart = isIntergraphLink;
			else
				this._isIntergraphLinkAtEnd = isIntergraphLink;
		}
	},
		
	_checkIntergraphLink1: function(port, cleanup) {

		// Is this an intergraph link?
		// (definition = link parent is an ancestor or port owner parent)
		var node = port.getOwner();
		var linkParent = this.getParent();
		if(node && linkParent){
			var nodeParent = node.getParent();
			if(nodeParent && nodeParent != linkParent){
				for(var p = nodeParent.getParent(); p && p.getParent; p = p.getParent()){
					if(p == linkParent){
						// OK, yes, this is an intergraph link.
						// record the link in the _intergraphLinks arrays of all subgraphs
						// between the node parent and the link parent
						for (var sub = nodeParent.getParent(); sub != linkParent; sub = sub.getParent()) {
							if (sub._isIBMDiagramSubgraph) {
								if (cleanup) {
									if (sub._intergraphLinks) {
										var index = dojo.indexOf(sub._intergraphLinks, this);
										if (index >= 0) {
											if(sub._intergraphLinks.length == 1)
												sub._intergraphLinks = undefined;
											else
												sub._intergraphLinks.splice(index, 1);
										}
									}
								}
								else {
									if (!sub._intergraphLinks) 
										sub._intergraphLinks = [this];
									else 
										if (dojo.indexOf(sub._intergraphLinks, this) < 0) 
											sub._intergraphLinks.push(this);
								}
							}
						}
						return true;
					}
				}
			}
		}
		return false;
	},
	
	_setParent: function(parent, matrix)
	{
		ibm_ilog.diagram.GraphElement.prototype._setParent.call(this, parent, matrix);
		this._forceCheckIntergraphLink();
	},
    
    //---------- Start/end nodes ----------
    
    getStartNode: function() {
        // summary:
        //		Gets the start node of the link.
        // returns:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a start port has been specified, this function returns the owner of the start port,
        //		otherwise it returns null.
        
        if (this._startPort) 
            return this._startPort.getOwner();
        else 
            return null;
    },
    
    setStartNode: function(/*ibm_ilog.diagram.Node*/node) {
        // summary:
        //		Sets the start node of the link.
        // node:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a start port has been specified, this function returns the owner of the start port,
        //		otherwise it returns null.
        
        if (node) {
            if (!this._startPort || this._startPort.getOwner() != node) {
                this.setStartPort(this._getOrCreateDefaultPort(node));
            }
        } else {
            this.setStartPort(null);
        }
        return this;
    },
    
    getEndNode: function() {
        // summary:
        //		Gets the end node of the link.
        // returns:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a end port has been specified, this function returns the owner of the end port,
        //		otherwise it returns null.
        
        if (this._endPort) 
            return this._endPort.getOwner();
        else 
            return null;
    },
    
    setEndNode: function(/*ibm_ilog.diagram.Node*/node) {
        // summary:
        //		Sets the end node of the link.
        // node:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a end port has been specified, this function returns the owner of the end port,
        //		otherwise it returns null.
        
        if (node) {
            if (!this._endPort || this._endPort.getOwner() != node) {
                this.setEndPort(this._getOrCreateDefaultPort(node));
            }
        } else {
            this.setEndPort(null);
        }
        return this;
    },
    
    _getOrCreateDefaultPort: function(/*ibm_ilog.diagram.Node*/node) {
        // tag: private
        // summary:
        //		Looks for an existing default port on the specified node
		///		(the default port is identified by the _isDefaultPort flag),
        //		if none exists create and add one.
        // node: ibm_ilog.diagram.Node:
        //		The node in which the default port should be looked up or created.
        
        var port;
        var ports = node.getPorts();
        for (var i = 0; i < ports.length; i++) {
            var port = ports[i];
            if (port._isDefaultPort) {
                return port;
            }
        }
        port = this._createDefaultPort(node);
		port._isDefaultPort = true;
        node.addPort(port);
        return port;
    },
	
	_createDefaultPort: function(/*ibm_ilog.diagram.Node*/node) {
        // tag: private
        // summary:
        //		Called by _getOrCreateDefaultPort to create the default port if none exists.
		//		By default, creates a new AutomaticPort.
        // node: ibm_ilog.diagram.Node:
        //		The node in which the default port should be created.
		return new ibm_ilog.diagram.AutomaticPort();
	},
    
    //---------- Start/end points ----------
    
    getFallbackStartPoint: function() {
        // summary:
        //		Gets the start point of the link.
        // returns:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The start point is used only when no start port is specified.
        return this._startPoint;
    },
    
    setFallbackStartPoint: function(point) {
        // summary:
        //		Sets the start point of the link.
        // point:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The start point is used only when no start port is specified.
        
        if (typeof(point) != 'object' || !('x' in point) || !('y' in point)) 
            ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "setFallbackStartPoint", "point", point);
        
        if (point.x != this._startPoint.x ||
        point.y != this._startPoint.y) {
            this._startPoint.x = point.x;
            this._startPoint.y = point.y;
            if (!this._startPort) 
                this.invalidateLinkShape();
            
        }
        return this;
    },
    
    getFallbackEndPoint: function() {
        // summary:
        //		Gets the end point of the link.
        // returns:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The end point is used only when no end port is specified.
        return this._endPoint;
    },
    
    setFallbackEndPoint: function(point) {
        // summary:
        //		Sets the end point of the link.
        // point:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The end point is used only when no end port is specified.
        
        if (typeof(point) != 'object' || !('x' in point) || !('y' in point)) 
            ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "setFallbackEndPoint", "point", point);
        
        if (point.x != this._endPoint.x ||
        point.y != this._endPoint.y) {
            this._endPoint.x = point.x;
            this._endPoint.y = point.y;
            if (!this._endPort) 
                this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Intermediate points ----------
    
    getIntermediatePoints: function() {
        // summary:
        //		Gets the intermediate points of the link.
        // returns:
        //		An array of objects with 'x' and 'y' properties.
        // description:
        //		The intermediate points define the additional points of the link (i.e., the points between the start and end points,
        //		which are always determined by the setStartPort()/setEndPort() or setFallbackStartPoint/setFallbackEndPoint() functions)
        //		when the link shape type is ibm_ilog.diagram.LinkShapeType.Free or ibm_ilog.diagram.LinkShapeType.OrthogonalEditable.
        //		For other shape types, the intermediate points are ignored since the link path is computed automatically.
        
        if (this._intermediatePoints !== null) {
            return dojo.clone(this._intermediatePoints);
        } else {
            return [];
        }
    },
    
    setIntermediatePoints: function(points) {
        // summary:
        //		Sets the intermediate points of the link.
        // points:
        //		An array of objects with 'x' and 'y' properties
        //		(can be null to remove all intermediate points).
        // description:
        //		The intermediate points define the additional points of the link (i.e., the points between the start and end points,
        //		which are always determined by the setStartPort()/setEndPort() or setFallbackStartPoint/setFallbackEndPoint() functions)
        //		when the link shape type is ibm_ilog.diagram.LinkShapeType.Free or ibm_ilog.diagram.LinkShapeType.OrthogonalEditable.
        //		For other shape types, the intermediate points are ignored since the link path is computed automatically.
        
        if (points !== null) {
            this._intermediatePoints = dojo.clone(points);
        } else {
            this._intermediatePoints = null;
        }
        if (this._shapeType == ibm_ilog.diagram.LinkShapeType.Free ||
        		this._shapeType == ibm_ilog.diagram.LinkShapeType.OrthogonalEditable) 
            this.invalidateLinkShape(); // in the other shapeTypes the interm points are not used
        return this;            
    },
    
    //---------- Shape type ----------
    
    getShapeType: function() {
        // summary:
        //		Gets the shape type of this link.
        // returns:
        //		One of the values defined in the ibm_ilog.diagram.LinkShapeType enumeration.
        // description:
        //		The shape type determines if and how the link automatically computes its shape
        //		based on its start and end port or points, and possibly its intermediate points.
        
        return this._shapeType;
    },
    
    setShapeType: function(/*int*/shapeType) {
        // summary:
        //		Sets the shape type of this link.
        // shapeType: int:
        //		One of the values defined in the ibm_ilog.diagram.LinkShapeType enumeration.
        // description:
        //		The shape type determines if and how the link automatically computes its shape
        //		based on its start and end port or points, and possibly its intermediate points.
        
        if (shapeType != this._shapeType) {
            this._shapeType = shapeType;
            this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Node distance ----------
    
    getNodeDistance: function() {
        // summary:
        //		Gets the minimum distance between this link and its start or end nodes when the link is orthogonal.
        // returns:
        //		A distance in pixels.
        // description:
        //		This property is used when the link shape type is Orthogonal, OrthogonalEditable or Oblique,
        //		and determines the minimum distance kept between the link and its start or end nodes.
        //		The default distance is 20.
        
        return this._nodeDistance;
    },
    
    setNodeDistance: function(/*number*/distance) {
        // summary:
        //		Sets the minimum distance between this link and its start or end nodes when the link is orthogonal.
        // distance: number:
        //		A distance in pixels.
        // description:
        //		This property is used when the link shape type is Orthogonal, OrthogonalEditable or Oblique,
        //		and determines the minimum distance kept between the link and its start or end nodes.
        
        if (distance != this._nodeDistance) {
            this._nodeDistance = distance;
            if (this._shapeType == ibm_ilog.diagram.LinkShapeType.Orthogonal ||
            this._shapeType == ibm_ilog.diagram.LinkShapeType.OrthogonalEditable ||
            this._shapeType == ibm_ilog.diagram.LinkShapeType.Oblique) 
                this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Curve tension (splines) ----------
	
	setCurveTension: function(/*number*/t) {
		// summary:
		//		Sets the curve tension to draw the link as a spline.
		// t: number:
		//		The curve tension, between 0 and 1.
		// description:
		//		This properties allows to draw curved links. By default, the curve tension is 0, which draws
		//		a plain polyline (not curved). If the curve tension is set to a value greater than 0, the
		//		link is drawn as a smooth spline that goes through the start point, then through the middle
		//		of each link segment (as computed according to the link shape type), and then to the end point.
		//		The curve tension determines the shape of the curve: small values will produce sharper bends,
		//		larger values will produce a smoother curve.
		//		The curve tension must be greater than 0 and less than 1
		//		(0.5 usually provides nicest results).
		//
		//		NOTE: If the _path part of the link template was a GFX Polyline shape, setting the curve tension to
		//			  a value greater than 0 automatically replaces it by a GFX Path shape. 
		
		if(t < 0) t = 0;
		if(t > 1) t = 1;
		if(t != this._curveTension){
			this._curveTension = t;
			// if the path is a polyline, replace it by a GFX Path.
			if (t > 0 && !(this._path instanceof dojox.gfx.Path)) {
				var stroke, selectedStyle, parent;
				if (this._path) {
					stroke = this._path.getStroke();
					selectedStyle = this._path.selectedStyle;
					parent = this._path.getParent();
					// Added to solve problem when link is selected
					standardStyle = this._path.standardStyle;
					if(this._styledChildren)
						this._styledChildren.remove(this._path);
					this._path.removeShape();
				}
				this._path = (parent || this).createPath();
				if(stroke)
					this._path.setStroke(stroke);
				if(selectedStyle)
					this._path.selectedStyle = selectedStyle;
				// Added to solve problem when link is selected
				if(standardStyle)
					this._path.standardStyle = standardStyle;
				if(this._styledChildren && this._path.standardStyle)
					this._styledChildren.add(this._path);
			}
			this.invalidateLinkShape();
		}
        return this;
	},
	
	getCurveTension: function() {
		// summary:
		//		Gets the curve tension.
		// returns: number:
		//		The curve tension.
		// description:
		//		See setCurveTension.
		
		return this._curveTension;
	},
    //---------- Path ----------

    layoutShape: function(target) {
    	//
    	//	_DiagramTemplated API:
    	//	
    	//		for externally instantiated templates
    	//
    	this._updatePath(target);
    },
    
    _updatePath: function(target) {
    	
    	var target = target || this;
    
        var points = this.applyPathToArrows(target);
    	this.applyPath(target._path,points,target._curveTension);
		// decorations:
		
		this._placeDecorations(target);
        this._updateSelectionPath(points,target,target._curveTension);
    },
    
    //-------------------------------------------------------	
    applyPathToArrows: function(target){
    	
    	var points = this._pathPoints;
        
        // arrows:
        if (points && points.length >= 2) {
            if ((target._startArrow && target._startArrow.getParent()) ||
            (target._endArrow && target._endArrow.getParent())) {
                points = dojo.clone(points); // end points will be modified
            }
            if (target._startArrow && target._startArrow.getParent()) {
            	this._updateArrow(target._startArrow, points[0], points[1],target);
            }
            if (target._endArrow && target._endArrow.getParent()) {
            	this._updateArrow(target._endArrow, points[points.length - 1], points[points.length - 2],target);
            }
        }
        return points;
    },
    
    applyPath: function(path,points,curveTension){
    	
        if (path instanceof dojox.gfx.Polyline) {
            path.setShape(points);
			path.bbox = null; // patch for GFX bug...
        } else {
            path.setShape("");
            if (points && points.length > 0) {
                path.moveTo(points[0].x, points[0].y);
				if (curveTension > 0 && points.length > 2) {
					var x = points[0].x;
					var y = points[0].y;
					var t = 1 - curveTension/2;
					for (var i = 0; i < points.length - 2; i++) {
						// first control point:
						var p1 = points[i + 1];
						var x1 = x + (p1.x - x) * t;
						var y1 = y + (p1.y - y) * t;
						// Compute next curve point:
						var p2 = points[i + 2];
						if (i < points.length - 3) {
							x = p1.x + (p2.x - p1.x) / 2;
							y = p1.y + (p2.y - p1.y) / 2;
						}
						else {
							x = p2.x;
							y = p2.y;
						}
						// second control point:
						var x2 = x + (p1.x - x) * t;
						var y2 = y + (p1.y - y) * t;
						path.curveTo(x1, y1, x2, y2, x, y);
					}
				}
				else {
					// polyline:
					for (var i = 1; i < points.length; i++) {
						path.lineTo(points[i].x, points[i].y);
					}
				}
            }
        }
	},
    //---------- Mouse tolerance ----------
    
    _updateSelectionPath: function(points,target,curveTension) {
    	var target = target || this;
        if (target._hitTolerance == 0) {
            if (target._selectionPath != null) {
            	target.remove(target._selectionPath);
            	target._selectionPath = null;
            }
        } else {
            this._updatePathShape(target,curveTension);
            this.applyPath(target._selectionPath,points,curveTension);
        }
    },
    
    _updatePathShape: function(target,curveTension){
		
    	if (target._selectionPath != null) {
        	if( ((curveTension > 0) && !(this._selectionPath instanceof dojox.gfx.Path)) || 
        	    ((curveTension == 0) && (this._selectionPath instanceof dojox.gfx.Path)) ){
        			target.remove(target._selectionPath);
        			target._selectionPath = null;
        	}
        }
    	if (target._selectionPath == null) {
    		if (curveTension > 0) {
    			target._selectionPath = target.createPath();
    		}else{
    			target._selectionPath = target.createPolyline();
    		}
    		target._selectionPath.setStroke(dojo.fromJson("{color: [255,255,255,0.001],width: " + this._hitTolerance + "}"));
    	}
	},
    
    getHitTolerance: function() {
        // Summary:
        //		gets the mouse tolerance.
        
        return this._hitTolerance;
    },
    
    setHitTolerance: function(tolerance) {
        // Summary: 
        //		sets the mouse tolerance. This value affects all the mouse events and the tolerance to consider it is over a link.
        //		The recomended values goes from 5 to 10 (bigger --> bigger tolerance).
        //		If 0, no tolerance is applied.
        
        this._hitTolerance = tolerance;
        this.invalidateLinkShape();
        return this;
    },
    
    //---------- Arrows ----------
    
    isStartArrowVisible: function() {
        // summary:
        //		Gets the visibility of the start arrow.
        // returns:
        //		True if the start arrow is defined in the link template and visible.
        //		False if the start arrow is either not defined in the link template or invisible.
        
        return (this._startArrow && this._startArrow.getParent()) ? true : false;
    },
    
    setStartArrowVisible: function(v) {
        // summary:
        //		Makes the start arrow visible or invisible.
        // v: Boolean:
        //		If no start arrow is defined in the link template
        //		(that is, if the template contains no shape with "dojoAttachPoint: '_startArrow'"),
        //		this method does nothing.
        //		Otherwise, if 'v' is true, the start arrow is shown (i.e., added to the link),
        //		and if 'v' is false, the start arrow is hidden (i.e. removed from the link).
        
        if (this._startArrow) {
            if (v) {
                if (!this._startArrow.getParent()) {
                    this.add(this._startArrow);
		            this.invalidateLinkShape();
                }
            } else {
                if (this._startArrow.getParent()) {
                    this.remove(this._startArrow);
		            this.invalidateLinkShape();
                }
            }
        }
        return this;
    },
    
    isEndArrowVisible: function() {
        // summary:
        //		Gets the visibility of the end arrow.
        // returns:
        //		True if the end arrow is defined in the link template and visible.
        //		False if the end arrow is either not defined in the link template or invisible.
        return (this._endArrow && this._endArrow.getParent()) ? true : false;
    },
    
    setEndArrowVisible: function(v) {
        // summary:
        //		Makes the end arrow visible or invisible.
        // v: Boolean:
        //		If no end arrow is defined in the link template
        //		(that is, if the template contains no shape with "dojoAttachPoint: '_endArrow'"),
        //		this method does nothing.
        //		Otherwise, if 'v' is true, the end arrow is shown (i.e., added to the link),
        //		and if 'v' is false, the end arrow is hidden (i.e. removed from the link).
        
        if (this._endArrow) {
            if (v) {
                if (!this._endArrow.getParent()) {
                    this.add(this._endArrow);
		            this.invalidateLinkShape();
                }
            } else {
                if (this._endArrow.getParent()) {
                    this.remove(this._endArrow);
		            this.invalidateLinkShape();
                }
            }
        }
        return this;
    },
    
    
    _updateArrow: function(arrow, endPoint, nextPoint,target) {
    	var target = target || this;
        // tag: private
        var _m = dojox.gfx.matrix;
        var dx = nextPoint.x - endPoint.x;
        var dy = nextPoint.y - endPoint.y;
		// hack for VML: because of roundings in transforms,
		// almost vertical/horizontal segments cause exceptions...
		if(dx > -0.01 && dx < 0.01) dx = 0;
		if(dy > -0.01 && dy < 0.01) dy = 0;
        var a = Math.atan2(-dy, -dx);
        var transforms = [_m.translate(endPoint.x, endPoint.y), _m.rotate(a)];
        var stroke = target._path.getStroke();
        if (stroke && stroke.width > 2) {
            transforms.push(_m.scale(1 + (stroke.width - 2) / 4)); // nicer than linear scaling...
        }
		arrow.setTransform(transforms);
		
        // shorten the segment so that the polyline's end does not show at the tip of the arrow
        var l = Math.sqrt(dx * dx + dy * dy);
        if (l > 0) {
            stroke = target._path.getStroke();
            var w = stroke ? stroke.width : 2;
            var r = w / l;
            endPoint.x += r * dx;
            endPoint.y += r * dy;
        }
    },
    
    //---------- Points computation ----------
    
    getPoints: function() {
        // summary:
        //		Gets the actual points of the link.
        // description:
        //		This function calls validateLinkShape() and returns the points that have been computed.
        
        this.validateLinkShape();
        return dojo.clone(this._pathPoints);
    },
    
    invalidateLinkShape: function() {
        // summary:
        //		Invalidates the shape of this link.
        // description:
        //		This function ensures that the shape of the link will be recomputed before the next redisplay of the link.
        //		This function is called automatically when the start or end port change, or their owner nodes change,
        //		or when a property of the link that determines the shape (such as the intermediate points) is changed.
        //		In most cases you do not need to call this function explicitly.
        
        this._pathPointsValid = false;
        ibm_ilog.diagram.util.Batch.add(this); // arrange so that _endBatch() is called soon
    },
    
    _endBatch: function() {
        // summary:
        //		This method will be called when invalidateLinkShape() has been called, and calls validateLinkShape().
        // tags:
        //		private
        
        this.validateLinkShape();
    },
    
    validateLinkShape: function() {
        // summary:
        //		Recomputes the shape of this link.
        // description:
        //		This function is called automatically when invalidateLinkShape() has been called, and before the
        //		link is redisplayed. 
        //		In most cases you do not need to call this function explicitly.
        
        if (!this._pathPointsValid) {
            this._computeLinkPoints();
            this._updatePath();
            this._pathPointsValid = true;
        }
    },
    
    revalidateLinkShape: function() {
        // summary:
        //		Recomputes the shape of this link.
        
    	this._pathPointsValid = false;
    	this.validateLinkShape();
    },

    _getParent: function() {
        // tag: private
        var p = this.getParent();
        if (p && !p.getParent) {
            p = null;
        }
        return p;
    },
    
    _computeLinkPoints: function() {
        // tag: private
        var _m = dojox.gfx.matrix;
        var startPoint = this.getFallbackStartPoint();
        var endPoint = this.getFallbackEndPoint();
        var startOrigin = startPoint;
        var endOrigin = endPoint;
        var cp, op, p, result;
        
        var parent = this._getParent();
        
        if (parent !== null) {
			
			var startPort = this._isIntergraphLinkAtStart ? this._getVisiblePort(this._startPort) : this._startPort;
			var endPort = this._isIntergraphLinkAtEnd ? this._getVisiblePort(this._endPort) : this._endPort;
			
            var t = _m.invert(this.getTransform());
            
            if (startPort !== null &&
            startPort.getOwner() !== null) {
                p = {
                    x: 0,
                    y: 0
                };
                
                if (startPort.needsReferencePoint()) {
                    p = this._getReferencePoint(false, startPort, endPort);
                    if (endPort === null) {
                        p = _m.multiplyPoint(this.getTransform(), p);
                    }
                }
                cp = {};
                op = {};
                result = startPort.getConnectionPoint(this, parent, p, cp, op, true);
                startPoint = _m.multiplyPoint(t, cp);
                startOrigin = _m.multiplyPoint(t, op);
            }
            
            if (endPort !== null &&
            endPort.getOwner() !== null) {
                p = {
                    x: 0,
                    y: 0
                };
                
                if (endPort.needsReferencePoint()) {
                    p = this._getReferencePoint(true, startPort, endPort);
                    if (startPort === null) {
                        p = _m.multiplyPoint(this.getTransform(), p);
                    }
                }
                cp = {};
                op = {};
                result = endPort.getConnectionPoint(this, parent, p, cp, op, false);
                endPoint = _m.multiplyPoint(t, cp);
                endOrigin = _m.multiplyPoint(t, op);
            }
        }
        
        this._setLinkPoints(startPoint, startOrigin, endPoint, endOrigin, startPort, endPort);
    },
	
	_getVisiblePort: function(port) {
		if(!port)
			return null;
		var owner = port.getOwner();
		if(!owner)
			return null;
		var topMostCollapsedNode = null;
		var linkParent = this.getParent();
		for(var p = owner.getParent(); p && p.getParent && p != linkParent; p = p._collapsedParent || p.getParent()){
			if(p._collapsedParent){
				for (var pp = p._collapsedParent; pp && pp.getParent; pp = pp.getParent()) {
					if (pp._isIBMDiagramSubgraph) {
						topMostCollapsedNode = pp;
						break;
					}
				}
			}
		}
		if (topMostCollapsedNode) {
			var collapsedPort = topMostCollapsedNode.getCollapsedPort(this, port);
			if(collapsedPort){
				if(!collapsedPort.getOwner())
					topMostCollapsedNode.addPort(collapsedPort);
				return collapsedPort;
			} 
			return this._getOrCreateDefaultPort(topMostCollapsedNode);
		}
		return port;
	},
	
    _getReferencePoint: function(start, startPort, endPort) {
        // tag: private
        var _m = dojox.gfx.matrix;
        var points = this._intermediatePoints;
        if ((this._shapeType === ibm_ilog.diagram.LinkShapeType.Free ||
        this._shapeType === ibm_ilog.diagram.LinkShapeType.OrthogonalEditable) &&
        points !== null &&
        points.length > 0) {
            var p = start ? points[points.length - 1] : points[0];
			var t = this.getTransform();
			if(t)
            	return _m.multiplyPoint(t, p);
			else
				return p;
        } else {
            var port = start ? startPort : endPort;
            if (port !== null && port.getOwner() !== null && !port.needsReferencePoint()) {
                cp = {};
                op = {};
                return port.getConnectionPoint(this, this.getParent(), null, cp, op, start);
            } else {
                var obj = start ? (startPort !== null ? startPort.getOwner() : null) : (endPort !== null ? endPort.getOwner() : null);
                if (obj !== null && obj.getParent() !== null) {
                    // Note by E.T
                    // testing obj.Parent != null is not enough because
                    // link from or to still may be not in same hierarchy
                    // but it fixes some basic cases.
                    var relBounds;
                    try {
                        relBounds = obj.getBounds(this.getParent());
                    } catch (Exception) {
                        // port owner not in same hierarchy.(for example not added yet)
						relBounds = null;
                    }
					if (relBounds != null) {
						return {
							x: relBounds.x + relBounds.width / 2,
							y: relBounds.y + relBounds.height / 2
						};
					} else {
						if (start) {
							return this.getFallbackStartPoint();
						}
						else {
							return this.getFallbackEndPoint();
						}
					}
                    
                } else {
                    if (start) {
                        return this.getFallbackStartPoint();
                    } else {
                        return this.getFallbackEndPoint();
                    }
                }
            }
        }
    },
    
    _setLinkPoints: function(startPoint, startOrigin, endPoint, endOrigin, startPort, endPort) {
        // tag: private
        
        var i;
        
        this._pathPoints = null;
        
        if (Math.abs(startPoint.x - endPoint.x) <= ibm_ilog.diagram.Link._constants._epsilon) {
            endPoint.x = startPoint.x;
        }
        if (Math.abs(startPoint.y - endPoint.y) <= ibm_ilog.diagram.Link._constants._epsilon) {
            endPoint.y = startPoint.y;
        }
        var points = [];
        switch (this._shapeType) {
            case ibm_ilog.diagram.LinkShapeType.Free:
                points.push(startPoint);
                if (this._intermediatePoints !== null) {
                    points.push.apply(points, this._intermediatePoints);
                }
                points.push(endPoint);
                break;
            case ibm_ilog.diagram.LinkShapeType.Straight:
                points.push(startPoint, endPoint);
                break;
            case ibm_ilog.diagram.LinkShapeType.Orthogonal:
            case ibm_ilog.diagram.LinkShapeType.OrthogonalEditable:
            case ibm_ilog.diagram.LinkShapeType.Oblique:
                // TODO (what?)
                
                var startDirection = !this._samePoint(startPoint, startOrigin) ? this._getDirection(startOrigin, startPoint) : this._getDirection(startPoint, endPoint);
                var endDirection = !this._samePoint(endPoint, endOrigin) ? this._getDirection(endOrigin, endPoint) : this._getDirection(endPoint, startPoint);
                
                points.push(startPoint);
                
                var startBounds, endBounds;
                var parent = this._getParent();
                if (parent !== null) {
                    if (startPort !== null) {
                        startBounds = startPort.getConnectionBounds(parent);
                    /*
                     } else if (this.getStart() !== null) {
                     startBounds = this.getStart().getBounds(parent);
                     */
                    } else {
                        startBounds = {
                            x: startPoint.x,
                            y: startPoint.y,
                            width: 0,
                            height: 0
                        };
                    }
                    if (endPort !== null) {
                        endBounds = endPort.getConnectionBounds(parent);
                    /*
                     } else if (this.getEnd() !== null) {
                     endBounds = this.getEnd().getBounds(parent);
                     */
                    } else {
                        endBounds = {
                            x: endPoint.x,
                            y: endPoint.y,
                            width: 0,
                            height: 0
                        };
                    }
                } else {
                    startBounds = {
                        x: startPoint.x,
                        y: startPoint.y,
                        width: 0,
                        height: 0
                    };
                    endBounds = {
                        x: endPoint.x,
                        y: endPoint.y,
                        width: 0,
                        height: 0
                    };
                }
                
                if (this._shapeType === ibm_ilog.diagram.LinkShapeType.OrthogonalEditable &&
                this._intermediatePoints !== null &&
                this._intermediatePoints.length > 0) {
                    var keepExisting = true;
                    
                    // Check that the existing segments are orthogonal.
                    //
                    if (keepExisting) {
                        for (i = 0; i < this._intermediatePoints.length - 1; i++) {
                            if (this._intermediatePoints[i].x != this._intermediatePoints[i + 1].x &&
                            this._intermediatePoints[i].y != this._intermediatePoints[i + 1].y) {
                                keepExisting = false;
                                break;
                            }
                        }
                    }
                    
                    if (keepExisting) {
                        // Keep the existing points,
                        // and just adjust the first/last segments.
                        //
                        points.push.apply(points, this._intermediatePoints);
                        //                        for each(var p in this._intermediatePoints) {
                        //							points.push(p);
                        //						}
                        //
                        points.push(endPoint);
                        
                        // adjust start:
                        //
                        this._adjustEndPoint(points, 0, 1);
                        
                        // adjust end:
                        //
                        this._adjustEndPoint(points, points.length - 1, -1);
                    } else {
                        this._computeOrthogonal(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points);
                        points.push(endPoint);
                    }
                } else {
                    this._computeOrthogonal(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points);
                    points.push(endPoint);
                }
                
                if (this._shapeType === ibm_ilog.diagram.LinkShapeType.Oblique) {
                
                    for (i = 0; i < points.length - 3; i++) {
                        var p1 = points[i];
                        var p2 = points[i + 1];
                        var p3 = points[i + 2];
                        var p4 = points[i + 3];
                        if (p1.x === p2.x && p3.x === p4.x) {
                            var dx = Math.abs(p3.x - p1.x) / 2;
                            var dy1 = p2.y - p1.y;
                            var dy2 = p4.y - p3.y;
                            if (((dy1 > 0 && dy2 > 0) || (dy1 < 0 && dy2 < 0)) &&
                            Math.abs(dy1) >= dx &&
                            Math.abs(dy2) >= dx) {
                                if (dy1 > 0) {
                                    points[i + 1] = {
                                        x: p2.x,
                                        y: p2.y - dx
                                    };
                                    points[i + 2] = {
                                        x: p3.x,
                                        y: p3.y + dx
                                    };
                                } else {
                                    points[i + 1] = {
                                        x: p2.x,
                                        y: p2.y + dx
                                    };
                                    points[i + 2] = {
                                        x: p3.x,
                                        y: p3.y - dx
                                    };
                                }
                            }
                        } else if (p1.y === p2.y && p3.y === p4.y) {
                            var dy = Math.abs(p3.y - p1.y) / 2;
                            var dx1 = p2.x - p1.x;
                            var dx2 = p4.x - p3.x;
                            if (((dx1 > 0 && dx2 > 0) || (dx1 < 0 && dx2 < 0)) &&
                            Math.abs(dx1) >= dy &&
                            Math.abs(dx2) >= dy) {
                                if (dx1 > 0) {
                                    points[i + 1] = {
                                        x: p2.x - dy,
                                        y: p2.y
                                    };
                                    points[i + 2] = {
                                        x: p3.x + dy,
                                        y: p3.y
                                    };
                                } else {
                                    points[i + 1] = {
                                        x: p2.x + dy,
                                        y: p2.y
                                    };
                                    points[i + 2] = {
                                        x: p3.x - dy,
                                        y: p3.y
                                    };
                                }
                            }
                        }
                    }
                    
                }
                
                break;
            default:
                break;
        }
        
        this._pathPoints = points;
    },
    
    _samePoint: function(p1, p2) {
        return (Math.abs(p1.x - p2.x) <= ibm_ilog.diagram.Link._constants._epsilon) &&
        (Math.abs(p1.y - p2.y) <= ibm_ilog.diagram.Link._constants._epsilon);
    },
    
    _getDirection: function(originPoint, connectionPoint) {
        // tag: private
        var dx = connectionPoint.x - originPoint.x;
        var dy = connectionPoint.y - originPoint.y;
        if (Math.abs(dx) >= Math.abs(dy)) {
            // Horizontal:
            if (dx > 0) {
                return ibm_ilog.diagram.Link._constants._right;
            } else {
                return ibm_ilog.diagram.Link._constants._left;
            }
        } else {
            // Vertical:
            if (dy > 0) {
                return ibm_ilog.diagram.Link._constants._bottom;
            } else {
                return ibm_ilog.diagram.Link._constants._top;
            }
        }
    },
    
    _adjustEndPoint: function(points, i, d) {
        // tag: private
        
        var p0 = points[i]; // The start or end point.
        var p1 = points[i + d]; // The point just next to the start/end point.
        // Do we need to adjust? Only if the first/last segment is not orthogonal.
        if (Math.abs(p0.x - p1.x) > ibm_ilog.diagram.Link._constants._epsilon && Math.abs(p0.y - p1.y) > ibm_ilog.diagram.Link._constants._epsilon) {
            // We must guess if the first/last segment must be horizontal or vertical.
            // For this, we cannot just look at the first segment, because the
            // first/last point may have been already modified by the interactor.
            // So, we look at the next segment, and we say that the first/last
            // segment must be orthogonal to it.
            //
            var p2 = points[i + d + d]; // The second point of the next segment.
            var nextSegmentIsHorizontal;
            var dx = Math.abs(p1.x - p2.x);
            var dy = Math.abs(p1.y - p2.y);
            if (dx < ibm_ilog.diagram.Link._constants._epsilon && dx < ibm_ilog.diagram.Link._constants._epsilon && points.length > 3) {
                // The next segment is null: this can happen if points
                // get aligned during interaction. In that case, we look at
                // the segment after the next segment.
                p1 = p2;
                p2 = points[i + d + d + d];
                dx = Math.abs(p1.x - p2.x);
                dy = Math.abs(p1.y - p2.y);
                nextSegmentIsHorizontal = dx < dy;
            } else {
                nextSegmentIsHorizontal = dx > dy;
            }
            
            if (nextSegmentIsHorizontal) {
                points[i + d] = {
                    x: p0.x,
                    y: p1.y
                };
            } else {
                points[i + d] = {
                    x: p1.x,
                    y: p0.y
                };
            }
        }
    },
    
    _rotatePosition: function(position, count) {
        // tag: private
        return (position + count) % 4;
    },
    
    _rotatePoint: function(p, count) {
        // tag: private
        var tmp;
        switch (count) {
            case 1:
                tmp = p.x;
                p.x = -p.y;
                p.y = tmp;
                break;
            case 2:
                p.x = -p.x;
                p.y = -p.y;
                break;
            case 3:
                tmp = p.x;
                p.x = p.y;
                p.y = -tmp;
                break;
        }
        return p;
    },
    _rotateRect: function(r, count) {
        // tag: private
        var tmp;
        switch (count) {
            case 1:
                tmp = r.x;
                r.x = -r.y - r.height;
                r.y = tmp;
                tmp = r.width;
                r.width = r.height;
                r.height = tmp;
                break;
            case 2:
                r.x = -r.x - r.width;
                r.y = -r.y - r.height;
                break;
            case 3:
                tmp = r.x;
                r.x = r.y;
                r.y = -tmp - r.width;
                tmp = r.width;
                r.width = r.height;
                r.height = tmp;
                break;
        }
        return r;
    },
    
    _computeOrthogonal: function(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points) {
        // tag: private
        
        // First, rotate everything so that the start point points to the top.
        //
        var rotateCount = 0;
        switch (startDirection) {
            case ibm_ilog.diagram.Link._constants._top:
                break;
            case ibm_ilog.diagram.Link._constants._bottom:
                rotateCount = 2;
                break;
            case ibm_ilog.diagram.Link._constants._left:
                rotateCount = 1;
                break;
            case ibm_ilog.diagram.Link._constants._right:
                rotateCount = 3;
                break;
        }
        
        startPoint = this._rotatePoint(dojo.clone(startPoint), rotateCount);
        startDirection = this._rotatePosition(startDirection, rotateCount);
        startBounds = this._rotateRect(dojo.clone(startBounds), rotateCount);
        endPoint = this._rotatePoint(dojo.clone(endPoint), rotateCount);
        endDirection = this._rotatePosition(endDirection, rotateCount);
        endBounds = this._rotateRect(dojo.clone(endBounds), rotateCount);
        
        // OK, now we start to enumerate all directions for the end point.
        //
        var x, y;
        switch (endDirection) {
            case ibm_ilog.diagram.Link._constants._top:
                if (endBounds.y + endBounds.height < startBounds.y) {
                    if ((endBounds.x > startPoint.x) || (endBounds.x + endBounds.width < startPoint.x)) {
                        //     ___          ___
                        //    |   |        |   |
                        //    |   E   or   E   |
                        //    |                |
                        //    S                S
                        
                        y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    } else {
                        //     ___          ___
                        //    |   |        |   |
                        //    |   E   or   E   |
                        //    |__            __|
                        //       |          |
                        //       S          S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        if (endPoint.x > startPoint.x) {
                            x = endBounds.x - this._nodeDistance;
                        } else {
                            x = endBounds.x + endBounds.width + this._nodeDistance;
                        }
                        points.push({
                            x: x,
                            y: y
                        });
                        y = endBounds.y - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else if (endBounds.y > startBounds.y + startBounds.height) {
                    if ((endPoint.x > startBounds.x + startBounds.width) || (endPoint.x < startBounds.x)) {
                        //     ___          ___
                        //    |   |        |   |
                        //    S   |   or   |   S
                        //        E        E
                        
                        y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    } else {
                        //     __          __
                        //    |  |        |  |
                        //    S  |   or   |  S
                        //      _|        |_
                        //     |            |
                        //     E            E
                        
                        y = startBounds.y - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        if (endPoint.x < startPoint.x) {
                            x = startBounds.x - this._nodeDistance;
                        } else {
                            x = startBounds.x + startBounds.width + this._nodeDistance;
                        }
                        points.push({
                            x: x,
                            y: y
                        });
                        y = (endPoint.y + startBounds.y + startBounds.height) / 2;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else {
                    //     _          _
                    //    | |        | |
                    //    S E   or   E S
                    
                    y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                }
                break;
            case ibm_ilog.diagram.Link._constants._bottom:
                if (endPoint.y < startPoint.y) {
                    if (endPoint.x !== startPoint.x) // straight link!...
                    {
                        //           E        E
                        //     ______|   or   |______
                        //    |                      |
                        //    S                      S
                        
                        y = (startPoint.y + endPoint.y) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else if ((endBounds.x > startBounds.x + startBounds.width) ||
                (endBounds.x + endBounds.width < startBounds.x)) {
                    //     ____                    ____
                    //    |    |    E   or   E    |    |
                    //    S    |____|        |____|    S
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    if (endBounds.x > startBounds.x + startBounds.width) {
                        x = (endBounds.x + startBounds.x + startBounds.width) / 2;
                    } else {
                        x = (endBounds.x + endBounds.width + startBounds.x) / 2;
                    }
                    points.push({
                        x: x,
                        y: y
                    });
                    y = endBounds.y + endBounds.height + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                } else {
                    //     ______          ______
                    //    |      |        |      |
                    //    S      |   or   |      S
                    //        E  |        |  E
                    //        |__|        |__|
                    
                    y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    if (endPoint.x > startPoint.x) {
                        x = Math.max(startBounds.x + startBounds.width, endBounds.x + endBounds.width) + this._nodeDistance;
                    } else {
                        x = Math.min(startBounds.x, endBounds.x) - this._nodeDistance;
                    }
                    points.push({
                        x: x,
                        y: y
                    });
                    y = endBounds.y + endBounds.height + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                }
                break;
            case ibm_ilog.diagram.Link._constants._left:
                if (endBounds.x < startPoint.x) {
                    if (endBounds.y + endBounds.height < startPoint.y) {
                        //    _E
                        //   |___
                        //       |
                        //       S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = endBounds.x - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    } else {
                        //     _____
                        //    |     |
                        //    |_E   |
                        //          S
                        
                        y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = Math.min(endBounds.x, startBounds.x) - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    }
                } else if (endPoint.y < startBounds.y) {
                    //     ____E
                    //    |
                    //    |
                    //    S
                    
                    points.push({
                        x: startPoint.x,
                        y: endPoint.y
                    });
                } else if (endBounds.x > startBounds.x + startBounds.width) {
                    //     ____
                    //    |    |
                    //    S    |___E
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = (startBounds.x + startBounds.width + endBounds.x) / 2;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                } else {
                    //     ___
                    //    |   |
                    //    |   S
                    //    |__E
                    
                    y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = Math.min(endBounds.x, startBounds.x) - this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                }
                break;
            case ibm_ilog.diagram.Link._constants._right:
                if (endBounds.x + endBounds.width > startPoint.x) {
                    if (endBounds.y + endBounds.height < startPoint.y) {
                        //      E_
                        //     ___|
                        //    |
                        //    S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = endBounds.x + endBounds.width + this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    } else {
                        //     _____
                        //    |     |
                        //    |   E_|
                        //    S
                        
                        y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = Math.max(endBounds.x + endBounds.width, startBounds.x + startBounds.width) + this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    }
                } else if (endPoint.y < startBounds.y) {
                    //    E____
                    //         |
                    //         |
                    //         S
                    
                    points.push({
                        x: startPoint.x,
                        y: endPoint.y
                    });
                } else if (endBounds.x + endBounds.width < startBounds.x) {
                    //          ____
                    //         |    |
                    //    E____|    S
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = (startBounds.x + endBounds.x + endBounds.width) / 2;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                } else {
                    //     ___
                    //    |   |
                    //    S   |
                    //     E__|
                    
                    y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = Math.max(endBounds.x + endBounds.width, startBounds.x + startBounds.width) + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                }
                break;
        }
        
        // Rotate the computed points back to the original orientation.
        //
        if (rotateCount !== 0) {
            rotateCount = 4 - rotateCount;
            for (var i = 1; i < points.length; i++) {
                var p = points[i];
                p = this._rotatePoint(p, rotateCount);
                points[i] = p;
            }
        }
    },
    _changeSelectionStyle: function(selected) {
        this.inherited(arguments);
        this.invalidateLinkShape();
    },
	
	// Link decorations:
	
	getDecorations: function() {
		// summary:
		//		Gets an array containing the decorations of the link.
		// returns: Array:
		//		An array of GFX shapes.
		// description:
		//		The link can contain shapes called "decorations". Decorations are automatically
		//		placed along the link path. To be considered as a decoration, a child shape
		//		must have the "isDecoration" set to true. The decoration can have the following properties
		//		that define their placement on the link:
		//		- position: A number that defines the position of the decoration along the link. The meaning
		//				of the position depends on the positionType property.
		//				The default position is 0.5, and 
		//				The default positionType is ibm_ilog.diagram.LinkDecorationPositionType.Relative,
		//				so by default a decoration is placed at the middle of the link.
		//		- positionType: Defines the way the position is interpreted. Possible values are:
		//			ibm_ilog.diagram.LinkDecorationPositionType.Relative:
		//				The position property is a ratio between 0 and 1 that defines a relative position on the
		//				link (0 = start of the link, 1 = of the link, 0.5 = middle of the link, etc).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromStart:
		//				The position is a fixed length that defines an offset from the start of the link
		//				(going forward from the start to the end of the link).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromMiddle:
		//				The position is a fixed length that defines an offset from the middle of the link
		//				(going forward from the start to the end of the link).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromEnd:
		//				The position is a fixed length that defines an offset from the end of the link
		//				(going backward from the end to the start of the link).
		//				The default positionType is ibm_ilog.diagram.LinkDecorationPositionType.Relative,
		//				and the default position is 0.5,
		//				so by default a decoration is placed at the middle of the link.
		//		- orthogonalAlignment: Defines the alignment of the decoration orthogonally to the link path.
		//				Possible values are:
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left:
		//				The decoration is placed on the "left" of the link, looking towards the end of the link
		//				(for example, if the link is horizontal and pointing to the right, the decoration will be
		//				above the link). The spacing between the decoration and the link is defined by the
		//				distance property.
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center:
		//				The decoration is centered on the link.
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Start:
		//				The decoration is placed on the "right" of the link, looking towards the end of the link
		//				(for example, if the link is horizontal and pointing to the right, the decoration will be
		//				below the link). The spacing between the decoration and the link is defined by the
		//				distance property.
		//				The default orthogonalAlignment is ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center.
		//		- parallelAlignment: Defines the alignment of the decoration along the link.
		//				Possible values are:
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
		//				The side of the decoration that is closest to the start of the link is aligned
		//				on the point defined by the position and positionType.
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle:
		//				The middle of the decoration is aligned
		//				on the point defined by the position and positionType.
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
		//				The side of the decoration that is closest to the end of the link is aligned
		//				on the point defined by the position and positionType.
		//				The default parallelAlignment is ibm_ilog.diagram.LinkDecorationParellelAlignment.Middle.
		//		- distance: Defines the distance between the decoration and the link when
		//				orthogonalAlignment is ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left
		//				or ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Right.
		//				The default distance is 0.
		//		- autoRotate: If autoRotate is true, the decoration is rotated according to the orientation
		//				of the link segment that contains the decoration position.
		//				By default, autoRotate is false.

		var decorations = [];
		
		var children = this.children;
		var childrenCount = children.length;
		
		for (var childIndex = 0; childIndex < childrenCount; childIndex++) {
			var item = children[childIndex];
			if (item.isDecoration) {
				decorations.push(item);
			}
		}
		
		return decorations;
	},
	
	_placeDecorations: function(target){
		// summary:
		//		Moves (and possibly rotates) a decoration along the link's path.
		// tags:
		//		private
		
		target = target || this;
		
		var points = null;
		var totalLength = 0;
		
		var children = target.children;
		var childrenCount = children.length;
		
		for (var childIndex = 0; childIndex < childrenCount; childIndex++) {
			var item = children[childIndex];
			if (item.isDecoration) {
				
				if (!points) {
					points = target._pathPoints;
					// compute total length of link
					for (var i = 0; i < points.length - 1; i++) 
						totalLength += ibm_ilog.diagram.util.Geometry.getDistance(points[i], points[i + 1]);
				}
				
				// get placement parameters and set default values
				
				var position = (item.position !== undefined) ? item.position : 0.5;
				var positionType = item.positionType || ibm_ilog.diagram.LinkDecorationPositionType.Relative;
				var positionLength = 0;
				var autoRotate = item.autoRotate;
				var distance = item.distance || 0;
				var orthogonalAlignment = item.orthogonalAlignment || ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center;
				var parallelAlignment = item.parallelAlignment || ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle;
				
				switch (positionType) {
					case ibm_ilog.diagram.LinkDecorationPositionType.Relative:
						positionLength = totalLength * position;
						break;
					case ibm_ilog.diagram.LinkDecorationPositionType.AbsoluteFromStart:
						positionLength = position;
						break;
					case ibm_ilog.diagram.LinkDecorationPositionType.AbsoluteFromMiddle:
						positionLength = totalLength / 2 + position;
						break;
					case ibm_ilog.diagram.LinkDecorationPositionType.AbsoluteFromEnd:
						positionLength = totalLength - position;
						break;
				}
				
				var currentLength = 0;
				var i;
				for (i = 0; i < points.length - 1; i++) {
					currentLength += ibm_ilog.diagram.util.Geometry.getDistance(points[i], points[i + 1]);
					if (positionLength < currentLength) 
						break;
				}
				if (i == points.length - 1) 
					i--;
				
				var d = ibm_ilog.diagram.util.Geometry.getDistance(points[i], points[i + 1]);
				
				var p;
				var dx, dy;
				
				if (d == 0) {
					p = {
						x: points[i].x,
						y: points[i].y
					};
					dx = 0;
					dy = 0;
				}
				else {
					var e = currentLength - positionLength;
					dx = points[i + 1].x - points[i].x;
					dy = points[i + 1].y - points[i].y;
					p = {
						x: points[i + 1].x - (dx * e) / d,
						y: points[i + 1].y - (dy * e) / d
					};
				}
				
				// Compute final position based on alignments and distance:
				//
				
				item.setTransform(null);
				
				var bounds = item.getBoundingBox() || {height: 0, width: 0, x:0, y:0};
				
				var rotationTransform = null;
				
				if (dy == 0 && !autoRotate) {
					// Horizontal segment:
					//
					var dir = dx >= 0 ? 1 : -1;
					switch (orthogonalAlignment) {
						case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center:
							break;
						case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left:
							p.y -= dir * (distance + bounds.height / 2);
							break;
						case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Right:
							p.y += dir * (distance + bounds.height / 2);
							break;
					}
					switch (parallelAlignment) {
						case ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle:
							break;
						case ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
							p.x += dir * (bounds.width / 2);
							break;
						case ibm_ilog.diagram.LinkDecorationParallelAlignment.End:
							p.x -= dir * (bounds.width / 2);
							break;
					}
				}
				else 
					if (dx == 0 && !autoRotate) {
						// Vertical segment:
						//
						dir = dy >= 0 ? 1 : -1;
						switch (orthogonalAlignment) {
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center:
								break;
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left:
								p.x += dir * (distance + bounds.width / 2);
								break;
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Right:
								p.x -= dir * (distance + bounds.width / 2);
								break;
						}
						switch (parallelAlignment) {
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle:
								break;
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
								p.y += dir * (bounds.height / 2);
								break;
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.End:
								p.y -= dir * (bounds.height / 2);
								break;
						}
					}
					else {
						// Oblique segment, or AutoRotate == true
						//
						
						var angle = Math.atan2(dy, dx);
						var t = dojox.gfx.matrix.rotateAt(-angle, p);
						var pbounds;
						if (autoRotate) 
							pbounds = dojo.clone(bounds);
						else 
							pbounds = t.transformRectangle(bounds);
						
						
						switch (orthogonalAlignment) {
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center:
								break;
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left:
								p.y -= distance + pbounds.height / 2;
								break;
							case ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Right:
								p.y += distance + pbounds.height / 2;
								break;
						}
						switch (parallelAlignment) {
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle:
								break;
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
								p.x += pbounds.width / 2;
								break;
							case ibm_ilog.diagram.LinkDecorationParallelAlignment.End:
								p.x -= pbounds.width / 2;
								break;
						}
						
						t = t.inverse();
						p = t.transformPoint(p);
						
						if (autoRotate) {
							var pi = Math.PI;
							if (angle > pi / 2 || angle < -pi / 2) {
								if (angle < 0) 
									angle += pi;
								else 
									angle -= pi;
								t = dojox.gfx.matrix.rotateAt(angle, {
									x: t.dx,
									y: t.dy
								});
							}
							item.setTransform(t);
							bounds = t.transformRectangle(bounds);
						}
					}
				
				// OK, move the decoration now, 'p' contains the location of the center of the decoration.
				
				dx = p.x - (bounds.x + bounds.width / 2)
				dy = p.y - (bounds.y + bounds.height / 2);
				
				if (dx != 0 || dy != 0) 
					item.applyLeftTransform(dojox.gfx.matrix.translate(dx, dy));
			}
		}
	},
	
// ---------- Bounds: ----------
    
    getBounds: function(/*dojox.gfx.Group*/parent) {
        // summary:
        //		Returns the bounds of this node in the coordinate space of the specified parent.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are returned.
        // returns:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the bounds of the node in the specified coordinate space.
        
        var rect = this.getLocalBoundingBox();
        if (rect && parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            rect = t.transformRectangle(rect);
        }
        return rect;
    },
    
    hitTest: function(p,graph,tolerance){
    	// Summary:
    	//		return if the given point hits the element
    	var g = ibm_ilog.diagram.util.Geometry;
    	
    	if(tolerance == null){
    		tolerance = this._hitTolerance;
    	}
    	
    	var bb = this.getBounds(graph);
    	if(!bb) {
    		return false;
    	}
    	bb = g.expandRect(bb,tolerance);
    	if(g.containsPoint(bb,p)) {
    		var path = this._path;
    		if (path instanceof dojox.gfx.Polyline) {
    			var t = this.getShapeToContainerTransform(graph).inverse();
    			p = t.transformPoint(p);
    			if(this._getPathDistance(path,p)<tolerance){
    				return true;
    			}
    		}else{
    			// TODO if the link is not a polyline, use just the bounding box
    			return true;
    		}
    	}
    	return false;
    },
    
    _getPathDistance: function(path,p){
    	// Summary:
    	//		calculates the distance from the polyline to the given point
    	//
    	var g = ibm_ilog.diagram.util.Geometry;
    	var points = path.getShape().points;
    	var distance = Number.MAX_VALUE;
    	for(var i=1;i<points.length;i++){
    		var a = points[i-1];
    		var b = points[i];
    		var d = g.getPointToSegmentDistance(a,b,p);
    		distance = Math.min(distance,d);
    	}
    	return distance;
    }
});

ibm_ilog.diagram.LinkDecorationPositionType = {
	Relative: 1,
	AbsoluteFromStart: 2,
	AbsoluteFromMiddle: 3,
	AbsoluteFromEnd: 4
};

ibm_ilog.diagram.LinkDecorationParallelAlignment = {
	Start: 1,
	Middle: 2,
	End: 3
};

ibm_ilog.diagram.LinkDecorationOrthogonalAlignment = {
	Left: 1,
	Center: 2,
	Right: 3
};

ibm_ilog.diagram.Link._constants = {
    _top: 0,
    _right: 1,
    _bottom: 2,
    _left: 3,
    
    _epsilon: 0.001
};

ibm_ilog.diagram.Link.nodeType = dojox.gfx.Group.nodeType;
ibm_ilog.diagram.Link.defaultShape = {
    type: 'link'
};

//Add TransformChain extension 
//dojo.extend(ibm_ilog.diagram.Link, ibm_ilog.diagram.TransformChain);

// Declare bindable properties
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "strokeColor", "#000000");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "strokeWidth", 2);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "selectedStrokeColor", "#0000FF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "selectedStrokeWidth", 3);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "data", null);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Link, "textDir", ""); 

ibm_ilog.diagram.Link.defaultTemplate = ibm_ilog.diagram.declareTemplate(ibm_ilog.diagram.templates.defaultLinkTemplate);

}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");



dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	//
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into
	//		these animations from the constructor (for example, `node`,
	//		`showDuration`, `hideDuration`).
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.fx.wipeIn,
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually.
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary: Toggle the node to showing
		// delay: Integer?
		//		Ammount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		// delay: Integer?
		//		Ammount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");




/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo,
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain,
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"],
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo.Animation[]*/ animations){
		// summary:
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel,
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = d.byId(args.node), s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo.Animation
	};

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it.
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()
		
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo.Animation
	};

	dojo.fx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node"
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = d.byId(args.node),
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	};

})();

}

if(!dojo._hasResource["dojox.gfx.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.fx"] = true;
dojo.provide("dojox.gfx.fx");



(function(){
	var d = dojo, g = dojox.gfx, m = g.matrix;

	// Generic interpolators. Should they be moved to dojox.fx?

	function InterpolNumber(start, end){
		this.start = start, this.end = end;
	}
	InterpolNumber.prototype.getValue = function(r){
		return (this.end - this.start) * r + this.start;
	};

	function InterpolUnit(start, end, units){
		this.start = start, this.end = end;
		this.units = units;
	}
	InterpolUnit.prototype.getValue = function(r){
		return (this.end - this.start) * r + this.start + this.units;
	};

	function InterpolColor(start, end){
		this.start = start, this.end = end;
		this.temp = new dojo.Color();
	}
	InterpolColor.prototype.getValue = function(r){
		return d.blendColors(this.start, this.end, r, this.temp);
	};

	function InterpolValues(values){
		this.values = values;
		this.length = values.length;
	}
	InterpolValues.prototype.getValue = function(r){
		return this.values[Math.min(Math.floor(r * this.length), this.length - 1)];
	};

	function InterpolObject(values, def){
		this.values = values;
		this.def = def ? def : {};
	}
	InterpolObject.prototype.getValue = function(r){
		var ret = dojo.clone(this.def);
		for(var i in this.values){
			ret[i] = this.values[i].getValue(r);
		}
		return ret;
	};

	function InterpolTransform(stack, original){
		this.stack = stack;
		this.original = original;
	}
	InterpolTransform.prototype.getValue = function(r){
		var ret = [];
		dojo.forEach(this.stack, function(t){
			if(t instanceof m.Matrix2D){
				ret.push(t);
				return;
			}
			if(t.name == "original" && this.original){
				ret.push(this.original);
				return;
			}
			if(!(t.name in m)){ return; }
			var f = m[t.name];
			if(typeof f != "function"){
				// constant
				ret.push(f);
				return;
			}
			var val = dojo.map(t.start, function(v, i){
							return (t.end[i] - v) * r + v;
						}),
				matrix = f.apply(m, val);
			if(matrix instanceof m.Matrix2D){
				ret.push(matrix);
			}
		}, this);
		return ret;
	};

	var transparent = new d.Color(0, 0, 0, 0);

	function getColorInterpol(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = g.normalizeColor(prop.start);
		}else{
			start = value = obj ? (name ? obj[name] : obj) : def;
		}
		if(prop.end){
			end = g.normalizeColor(prop.end);
		}else{
			if(!value){
				value = obj ? (name ? obj[name] : obj) : def;
			}
			end = value;
		}
		return new InterpolColor(start, end);
	}

	function getNumberInterpol(prop, obj, name, def){
		if(prop.values){
			return new InterpolValues(prop.values);
		}
		var value, start, end;
		if(prop.start){
			start = prop.start;
		}else{
			start = value = obj ? obj[name] : def;
		}
		if(prop.end){
			end = prop.end;
		}else{
			if(typeof value != "number"){
				value = obj ? obj[name] : def;
			}
			end = value;
		}
		return new InterpolNumber(start, end);
	}

	g.fx.animateStroke = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change stroke properties over time
		// example:
		//	|	dojox.gfx.fx.animateStroke{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"},
		//	|		width: {end: 15},
		//	|		join:  {values: ["miter", "bevel", "round"]}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, stroke;
		d.connect(anim, "beforeBegin", anim, function(){
			stroke = shape.getStroke();
			var prop = args.color, values = {}, value, start, end;
			if(prop){
				values.color = getColorInterpol(prop, stroke, "color", transparent);
			}
			prop = args.style;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.width;
			if(prop){
				values.width = getNumberInterpol(prop, stroke, "width", 1);
			}
			prop = args.cap;
			if(prop && prop.values){
				values.cap = new InterpolValues(prop.values);
			}
			prop = args.join;
			if(prop){
				if(prop.values){
					values.join = new InterpolValues(prop.values);
				}else{
					start = prop.start ? prop.start : (stroke && stroke.join || 0);
					end = prop.end ? prop.end : (stroke && stroke.join || 0);
					if(typeof start == "number" && typeof end == "number"){
						values.join = new InterpolNumber(start, end);
					}
				}
			}
			this.curve = new InterpolObject(values, stroke);
		});
		d.connect(anim, "onAnimate", shape, "setStroke");
		return anim; // dojo.Animation
	};

	g.fx.animateFill = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change fill color over time.
		//	Only solid fill color is supported at the moment
		// example:
		//	|	dojox.gfx.fx.animateFill{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		color: {start: "red", end: "green"}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, fill;
		d.connect(anim, "beforeBegin", anim, function(){
			fill = shape.getFill();
			var prop = args.color, values = {};
			if(prop){
				this.curve = getColorInterpol(prop, fill, "", transparent);
			}
		});
		d.connect(anim, "onAnimate", shape, "setFill");
		return anim; // dojo.Animation
	};

	g.fx.animateFont = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change font properties over time
		// example:
		//	|	dojox.gfx.fx.animateFont{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		variant: {values: ["normal", "small-caps"]},
		//	|		size:  {end: 10, units: "pt"}
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, font;
		d.connect(anim, "beforeBegin", anim, function(){
			font = shape.getFont();
			var prop = args.style, values = {}, value, start, end;
			if(prop && prop.values){
				values.style = new InterpolValues(prop.values);
			}
			prop = args.variant;
			if(prop && prop.values){
				values.variant = new InterpolValues(prop.values);
			}
			prop = args.weight;
			if(prop && prop.values){
				values.weight = new InterpolValues(prop.values);
			}
			prop = args.family;
			if(prop && prop.values){
				values.family = new InterpolValues(prop.values);
			}
			prop = args.size;
			if(prop && prop.units){
				start = parseFloat(prop.start ? prop.start : (shape.font && shape.font.size || "0"));
				end = parseFloat(prop.end ? prop.end : (shape.font && shape.font.size || "0"));
				values.size = new InterpolUnit(start, end, prop.units);
			}
			this.curve = new InterpolObject(values, font);
		});
		d.connect(anim, "onAnimate", shape, "setFont");
		return anim; // dojo.Animation
	};

	g.fx.animateTransform = function(/*Object*/ args){
		// summary:
		//	Returns an animation which will change transformation over time
		// example:
		//	|	dojox.gfx.fx.animateTransform{{
		//	|		shape: shape,
		//	|		duration: 500,
		//	|		transform: [
		//	|			{name: "translate", start: [0, 0], end: [200, 200]},
		//	|			{name: "original"}
		//	|		]
		//	|	}).play();
		if(!args.easing){ args.easing = d._defaultEasing; }
		var anim = new d.Animation(args), shape = args.shape, original;
		d.connect(anim, "beforeBegin", anim, function(){
			original = shape.getTransform();
			this.curve = new InterpolTransform(args.transform, original);
		});
		d.connect(anim, "onAnimate", shape, "setTransform");
		return anim; // dojo.Animation
	};
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.gfxlayout.CanvasLayout']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.gfxlayout.CanvasLayout'] = true;
dojo.provide('ibm_ilog.diagram.gfxlayout.CanvasLayout');



dojo.declare('ibm_ilog.diagram.gfxlayout.CanvasLayout', ibm_ilog.diagram.gfxlayout.Layout, {

	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {type:'ibm_ilog.diagram.gfxlayout.CanvasLayout'}; /*Object*/
	},

	computePreferredSize: function(proposedSize) {
		var children = this._container.children, count = children.length, i, bbox, sz;
		for (i = 0; i < count; ++i) {
			var child = children[i];
			sz = child.computePreferredSize({
				width: -1,
				height: -1
			});
			var rect = child.getBoundingBox();
			if (!rect) 
				continue;
			var t = child.getTransform();
			
			if (child.__oldLayoutTransform) {
				t = dojox.gfx.matrix.multiply(child.__oldLayoutTransform.inverse(), t);
			}
			
			if (t && !t.isIdentity()) 
				rect = t.transformRectangle(rect);
			rect.width = sz.width;
			rect.height = sz.height;
			bbox = bbox ? ibm_ilog.diagram.util.Geometry.addRect(rect, bbox) : rect;
		}
		return {
			width: bbox ? bbox.width : -1,
			height: bbox ? bbox.height : -1
		};
	},
	
	layoutChildren: function(contBounds) {
		var children = this._container.children, count = children.length, o, i;
		if (count === 0) 
			return;
		for (i = 0; i < count; ++i) {
			var child = children[i];
			// Process only panels (ie a Group with a Layout)
			if (!child.children || !child.getLayout()) 
				continue;
			var sz = child.getPreferredSize();
			child.layout({
				x: 0,
				y: 0,
				width: sz.width,
				height: sz.height
			});
		}
		// translate the graph so that the top-left corner of its content bbox
		// matches the layout origin (contBounds)
		var r = this._container.getBoundingBox();
		if (!r) 
			return;
		var t = this._container.getTransform();
		if (this._oldCanvasTransform) 
			t = dojox.gfx.matrix.multiply(this._oldCanvasTransform.inverse(), t);
		this._oldCanvasTransform = dojox.gfx.matrix.translate(contBounds.x - r.x, contBounds.y - r.y);
		t = dojox.gfx.matrix.multiply(t, this._oldCanvasTransform);
		this._container.setTransform(t);
	}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.Subgraph']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Subgraph'] = true;
dojo.provide('ibm_ilog.diagram.Subgraph');











if (dojo.config.useGfxLayout) {
    
}

(function(){
	var g = ibm_ilog.diagram.util.Geometry;

dojo.declare("ibm_ilog.diagram.Subgraph", [ibm_ilog.diagram.Node], {
    // summary:
    //		A Subgraph is a Node that contains a nested Graph.
    // description:
    //		Subgraph objects are used to implemented nested graphs, that is, graphs contained in graphs.
    //		A Subgraph can be collapsed (i.e., its nested graph is hidden and replaced by a plain node)
    //		or expanded (i.e., its nested graph is fully displayed).
    
    // Allows to test the class faster that testing declaredClass
    _isIBMDiagramSubgraph: true,

  	//
  	//	TID: String
  	//		Graph element Type ID, used to distinguish basic graph 
    //		element types:
    //	
    //			N = Node
    //			S = Subgraph
    //			L = Link
    //	
  	//		Usually used as a very fast index into maps based on 
    //		ibm_diagram graph element types.
  	//
    TID: 'S',

    // "attach points" (aka template parts),
    // initialized automatically when reading the template.
    _graph: null,
    _collapsedGroup: null,
    _expandedGroup: null,
    _expandButton: null,
    _collapseButton: null,
    
    _collapsed: false,
    
    _fixedPosition: null,
    _animate: true,
    
    allowsResize: function() {
    	return true;
    },
    
    dispose: function(options) {
    	//	summary:
			//		Dispose this GraphElement.
			//	description:
			//		This function should be called when the GraphElement is completely deleted, that is, when it is removed
			//		from its parent graph and it will never be used again.
			//	options:
    	//		options.noRecurse:
			//			Do not recurse children
    	if(!options || !options.noRecurse) {
	    	dojo.forEach(this._graph.children, function(c){
	    		c.dispose(options);
	    	});
    	}
    	ibm_ilog.diagram.GraphElement.prototype.dispose.apply(this,arguments);
    },

    _subgraphInitialize: function() {
        // summary:
        //		Performs post-creation initializations such as connecting events, etc.
        // tags:
        //		private
        
        this._fixedPosition = {
            x: 0,
            y: 0
        };
        
        this.baseShape = this._expandedBaseShape;
        
        if (this._graph) {
            this._graph._owningSubgraph = this;
            if (dojo.config.useGfxLayout)
                this._graph.setLayout(new ibm_ilog.diagram.gfxlayout.CanvasLayout());
        }
        this._hide(this._collapsedGroup);
        
		try { // throws errors in Silverlight for example
			if (this._expandedBaseShape) {
				this._expandedBaseShape.connect("ondblclick", this, this._collapseButtonClicked);
				if (ibm_ilog.diagram.mobileEnabled) 
					this._expandedBaseShape.connect("ontouchend", this, this._collapseButtonDoubleTap);
			}
			if (this.titleShape) {
				this.titleShape.connect("ondblclick", this, this._collapseButtonClicked);
				if (ibm_ilog.diagram.mobileEnabled) 
					this.titleShape.connect("ontouchend", this, this._collapseButtonDoubleTap);
			}
			
			if (this._collapsedBaseShape) {
				this._collapsedBaseShape.connect("ondblclick", this, this._expandButtonClicked);
				if (ibm_ilog.diagram.mobileEnabled) 
					this._collapsedBaseShape.connect("ontouchend", this, this._expandButtonDoubleTap);
			}
			if (this.textShape) {
				this.textShape.connect("ondblclick", this, this._expandButtonClicked);
				if (ibm_ilog.diagram.mobileEnabled) 
					this.textShape.connect("ontouchend", this, this._expandButtonDoubleTap);
				
			}
		} catch(err){}
        // setup mouse listeners on expand/collapse buttons:
        if (this._expandButton) {
            this._expandButton.connect("onclick", this, this._expandButtonClicked);
        	if (ibm_ilog.diagram.mobileEnabled) {
				try {
					this._expandButton.connect("ontouchend", this, this._expandButtonTouch);
				} catch (err) {}
			}
        }
        if (this._collapseButton) {
            this._collapseButton.connect("onclick", this, this._collapseButtonClicked);
        	if (ibm_ilog.diagram.mobileEnabled) {
				try {
					this._collapseButton.connect("ontouchend", this, this._collapseButtonTouch);
				} catch(err){}
			}
        }
		
        if (!dojo.config.useGfxLayout)
		  this._doLayout();
    },
    
    applyTemplate: function(/*String||Object*/template, /*Object||ibm_ilog.diagram.data.dtlbinding.BoundContext*/context) {
        // summary:
        //     Applies the specified template on this subgraph.
        
        var c = this.inherited(arguments);
        if (c)
            this._subgraphInitialize();
        return c;
    },
    
    getGraph: function() {
        // summary:
        //		Gets the nested graph contained in this Subgraph.
        // returns: ibm_ilog.diagram.Graph
        //		The Graph object contained in this Subgraph.
        return this._graph;
    },
    
    setAnimate: function(/*Boolean*/value){
        // summary:
        //     Sets whether the expand/collapse operation is animated. The default value is true.
        // value: Boolean
    	//		Indicates whether the expand/collapse operation is animated.
        this._animate = value;
        return this;
    },
    
    getAnimate: function() {
        // summary:
        //     Returns whether the expand/collapse operation is animated.
        return this._animate;
    },
    
    isCollapsed: function() {
        // summary:
        //		Gets the collapsed state of this Subgraph.
        // returns: Boolean:
        //		True if the subgraph is collapsed, or false if the subgraph is expanded.
        return this._collapsed;
    },
    setCollapsed: function(/*Boolean*/collapsed, /*Boolean*/animate) {
        // summary:
        //		Collapses or expands this subgraph.
        // collapsed: Boolean:
        //		If true, the subgraph is collapsed, otherwise it is expanded.
        // animate: Boolean
    	//		If specified, indicates whether the expand/collapse operation is animated.
    	//		If this argument is not specified, the <code>getAnimate()</code> flag is used instead.
        
        this._setCollapsed(collapsed, arguments.length == 2 ? animate : this._animate);
        return this;
    },
    _setCollapsed: function(/*Boolean*/v, /*Boolean*/animate) {
        // summary:
        //		Internal method to collapse/expand the subgraph.
        // description:
        //		This method does nothing if v is equal to isCollapsed().
        // v: Boolean:
        //		True to collapse, false to expand.
        // animate: Boolean:
        //		True to animate.
        // tags:
        //		private
        
        if (v != this._collapsed) {

					if(this._animating)
						return;
						
			  if(!this.canCollapse())
				return;
					
		      if (dojo.config.useGfxLayout) {
		          this.suspendInvalidate();
		      }
		  
		      this._collapsed = v;
		
		      this.baseShape = v ? this._collapsedBaseShape : this._expandedBaseShape;
		      
		      var toHide, toShow;
		      if (this._collapsed) {
		          toHide = this._expandedGroup;
		          toShow = this._collapsedGroup;
		      } else {
		          toHide = this._collapsedGroup;
		          toShow = this._expandedGroup;
		      }
		      
		      var bbox = toHide.getLocalBoundingBox();
		      
		      this._show(toShow);
		
		      this._realign(toShow, bbox);
			
		      if (animate) {
		        var anim = dojo.fx.combine([this.__animateShowHide(toShow, true), this.__animateShowHide(toHide, false)]);
		        dojo.connect(anim, "onAnimate", this, this._invalidateIntergraphLinks);
						this._animating = true;
		        this.onCollapsedChanged();
		          
		             	dojo.connect(anim, "onEnd", this, function(){
							if (dojo.config.useGfxLayout) {
								this.resumeInvalidate(true);
								ibm_ilog.diagram.gfxlayout.doPendingValidate(this);
							}
							delete this._animating;
			        this.onCollapsedAnimated();
						});
		                         
		        anim.play();
		      } else {
		        this.onCollapsedChanged();
		        
		        this._hide(toHide);
		        
		        this._invalidateIntergraphLinks();
		
		        if (dojo.config.useGfxLayout) {
		            this.resumeInvalidate(true);
					ibm_ilog.diagram.gfxlayout.doPendingValidate(this);
		        }
		        
		      }
		      this._changeSelectionStyle(this.isSelected());
        }
    },
    
	canCollapse: function() {
		// summary:
		//		Indicates if this Subgraph can be collapsed.
		// description:
		//		Indicates if this Subgraph can be collapsed.
		//		This method always returns true for the Subgraph class,
		//		it can be overridden in subclasses that do not support collapsing.
		// returns:
		//		True if this Subgraph can be collapsed, false otherwise.
		
		return true; // Boolean
	},
	
    getFixedPosition: function() {
        // summary:
        //		Gets the fixed position used to realign the subgraph when it is expanded/collapsed.
        // returns:
        //		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the subgraph width/height.
        return this._fixedPosition;
    },
    
    setFixedPosition: function(position) {
        // summary:
        //		Sets a fixed position used to realign the subgraph when it is expanded/collapsed.
        // description:
        //		When a subgraph is expanded or collapsed, it is automatically realigned so that its position
        //		in the new state (expanded or collapsed) matches the position in the previous state.
        //		This method specifies how the subgraph will be realigned by defining a fixed position relative
        //		to the subgraph width/height.
        //		For example, setFixedPosition({x:0, y:0}) (the default value)
        //		realigns the subgraph so that its top-left corner stays at the same position,
        //		and setFixedPosition({x:0.5, y:0.5})
        //		realigns the subgraph so that its center stays at the same position.
        // position:
        //		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the subgraph width/height.
        
        if (typeof(position) != 'object' || !('x' in position) || !('y' in position)) 
            ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "setFixedPosition", "position", position);
        
        this._fixedPosition.x = position.x;
        this._fixedPosition.y = position.y;
        return this;
    },
    
    
    _realign: function(group, bbox) {
        // summary:
        //		Realigns the position of the shown group
        //		based on the bbox of the hidden group
        // tag: private
        
        var bb = group.getLocalBoundingBox();
        if (!bb) return;
        var newx = bbox.x + this._fixedPosition.x * (bbox.width - bb.width);
        var newy = bbox.y + this._fixedPosition.y * (bbox.height - bb.height);
        if (bb.x != newx || bb.y != newy) 
            group.applyLeftTransform(dojox.gfx.matrix.translate(newx - bb.x, newy - bb.y));
    },
	
    _saveStyles: function(group) {
        // summary:
        //		Workaround for IE: removing a shape clears all its styles.
        // tags:
        //		private
        dojox.gfx.utils.forEach(group, function(shape) {
            if (shape.setFill) {
                shape.oldFill = shape.getFill();
            }
            if (shape.setStroke) {
                shape.oldStroke = shape.getStroke();
            }
            if (shape.setFont) {
                shape.oldFont = shape.getFont();
            }
            if (shape.setShape) {
                shape.oldShape = shape.getShape();
            }
        }, null);
    },
    
    _restoreStyles: function(group) {
        // summary:
        //		Workaround for IE: removing a shape clears all its styles.
        // tags:
        //		private
        dojox.gfx.utils.forEach(group, function(shape) {
            try {
                if (dojo.config.useGfxLayout)
                    shape.suspendInvalidate();
                
                if (shape.oldFill) {
                    shape.setFill(shape.oldFill);
                }
                if (shape.oldStroke) {
                    shape.setStroke(shape.oldStroke);
                }
                if (shape.oldFont) {
                    shape.setFont(shape.oldFont);
                }
                if (shape.oldShape) {
                    shape.setShape(shape.oldShape);
                }
            } finally {
                if (dojo.config.useGfxLayout) {
                    if (shape.oldShape && 'text' in shape.oldShape) 
                        shape._explicitSize = null;
                    shape.resumeInvalidate(true);
                }
            }
        }, null);
    },
    
    onCollapsedAnimated: function() {
      // summary:
      //		Called when the subgraph is about to be been expanded or collapsed.
    	dojo.publish("/ibm_diagram/Subgraph/collapsedAnimated",[this]);
    },

    onCollapsedChanged: function() {
        // summary:
        //		Called when the subgraph has been expanded or collapsed.
    	dojo.publish("/ibm_diagram/Subgraph/collapsedChanged",[this]);
    },
    
    __animateShowHide: function(group, show) {
        // summary:
        //		Creates and starts the expand/collapse animations.
        // tags:
        //		private
        var anim;
        var startScale, endScale;
        if (show) {
            startScale = 0.1;
            endScale = 1;
        } else {
            startScale = 1;
            endScale = 0.1;
        }
        var r = group.getLocalBoundingBox();
        var cx = r.x + r.width * this._fixedPosition.x;
        var cy = r.y + r.height * this._fixedPosition.y;
        anim = dojox.gfx.fx.animateTransform({
            shape: group,
            duration: 500,
            transform: [{
                name: "scaleAt",
                start: [startScale, startScale, cx, cy],
                end: [endScale, endScale, cx, cy]
            }, {
                name: "original"
            }]
        });
        if (!show) {
			group._transformBeforeAnim = group.getTransform();
            dojo.connect(anim, "onEnd", this, dojo.hitch(this, "__endHide", group));
        }
        return anim;
    },
    
    __endHide: function(group) {
        // summary:
        //		Called at the end of the expand/collapse animation.
        // tags:
        //		private
        group.setTransform(group._transformBeforeAnim);
		delete group._transformBeforeAnim;
        // call _hide() after setTransform (crash under IE)
        this._hide(group);
    },
    
    _hide: function(toHide) {
        if (toHide && toHide.getParent()) {
            if ((dojo.isChrome && ibm_ilog.diagram.isSvg) || ibm_ilog.diagram.isVml) {
                this._saveStyles(toHide);
            }
            toHide._collapsedParent = toHide.getParent();
            toHide._collapsedParent.remove(toHide);
        }
    },
    
    _show: function(toShow) {
        if (toShow && !toShow.getParent()) {
            toShow._collapsedParent.add(toShow);
            toShow._collapsedParent = undefined;
            if ((dojo.isChrome  && ibm_ilog.diagram.isSvg) || ibm_ilog.diagram.isVml) {
                this._restoreStyles(toShow);
            }
        }
    },
    
    _expandButtonClicked: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        this._setCollapsed(false, this._animate);
    },
    _collapseButtonClicked: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
        this._setCollapsed(true, this._animate);
    },
    
    _expandButtonTouch: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        this._setCollapsed(false, this._animate);
        if (e.preventDefault)
          e.preventDefault();
        
    },
    _expandButtonDoubleTap: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        
  		if(event.touches.length == 0){
  			var delay = new Date().getTime() - this._lastUp;
  			if(this._lastUp && (delay < 500)){
  				this._setCollapsed(false, this._animate);
  				delete this._lastUp;
  			}else{
  				this._lastUp = new Date().getTime();
  			}   
  		}			
      if (e.preventDefault)
        e.preventDefault();
        
    },
    _collapseButtonTouch: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
        this._setCollapsed(true, this._animate);
        if (e.preventDefault)
          e.preventDefault();
    },
    
    _collapseButtonDoubleTap: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
  		if(event.touches.length == 0){
  			var delay = new Date().getTime() - this._lastUp;
  			if(this._lastUp && (delay < 500)){
  				this._setCollapsed(true, this._animate);
  				delete this._lastUp;
  			}else{
  				this._lastUp = new Date().getTime();
  			}   
  		}			
      if (e.preventDefault)
        e.preventDefault();
        
    },
    
	getCollapsedPort: function(link, port) {
		// summary:
		//		Gets a port to which the specified intergraph link will be connected when this
		//		subgraph is collapsed.
		// link: ibm_ilog.diagram.Link:
		//		The intergraph link connected to a node contained in this subgraph.
		// port: ibm_ilog.diagram.PortBase:
		//		The port to which the intergraph link is normally connected when this subgraph is expanded.
		// description:
		//		When a subgraph is collapsed, intergraph links connected to nodes contained in the subgraph
		//		are automatically reconnected to a port of the collapsed subgraph itself.
		//		This method lets you customize which port will be used for this.
		//		If this method returns null (the default), an AutomaticPort will be used by default.
		
		return null;
	},
	
	propagateTextDir: function(/*String*/textdir) {
        // summary: 
        //      propagate a new text direction to children (nodes, links)
        // textdir: String, text direction 
	 
        if (dojo.config.isBidiInDojoDiagrammer) {
	   	    dojo.forEach(this.getGraph().children, function(n) {
	   	        if (n.setTextDir){
	   	            n.setTextDir(textdir);
	   	            if (n instanceof ibm_ilog.diagram.Subgraph)
	   	                n.propagateTextDir(textdir);
	   	        }    
	   	    });
        }	   	    
	},
	
    // 
    // Internal layout (i.e. layout the parts of the template)
    // 
    
    // TODO: should not be hard-coded...
    _graphPosition: null,
    _headerHeight: 20,
    _margin: 2,
    
    layoutShape: function( /*Object*/target, forceDoLayout){
        var c = target._collapsedGroup, e = target._expandedGroup, s, h;
        if (this._collapsed) {
            s = c;
            h = e;
        } else {
            s = e;
            h = c;
        }
        
        target._graphPosition = {x:0,y:0};
        if (forceDoLayout)
            this._doLayout(target);

        if(this._collapsed) {
            var hbb = h.getLocalBoundingBox();
            this._realign(s, hbb);
        }
        if (h) {
            h.getParent().remove(h);
        }        
    },
    
    applyLayout : function(rect){
        if (dojo.config.useGfxLayout) {
            ibm_ilog.diagram.Node.prototype.applyLayout.apply(this, arguments);
            // update the subgraph location depending on the graph content bbox
            // (_noTranslate: flag set in the SwimLane subclass)
            if (this.getLayout() && !this.isCollapsed() && !this._noTranslate && this._graph) {
                var graphBB = this._graph.getBoundingBox();
                if (!graphBB) // empty
                    return;
                var t = this._graph.getShapeToContainerTransform(this) || dojox.gfx.matrix.identity;
                var subBB = this.getBoundingBox();
                var r = t.transformRectangle(graphBB);
                var r2 = subBB;
                var dx =  graphBB.x - (r.x-r2.x);
                var dy =  graphBB.y - (r.y-r2.y);
                t = this._expandedGroup.getTransform();
                if (this._oldSubgraphTransform)
                    t = dojox.gfx.matrix.multiply(this._oldSubgraphTransform.inverse(), t);
                this._oldSubgraphTransform = dojox.gfx.matrix.translate(dx, dy);
                t = dojox.gfx.matrix.multiply(t, this._oldSubgraphTransform);
                this._settingTransformInApplyLayout = true;
                this._expandedGroup.suspendInvalidate();
				this._expandedGroup.setTransform(t);
                this._expandedGroup.resumeInvalidate();
                delete this._settingTransformInApplyLayout;
            }
            
			// realign collapsed group after layout
			if(this.getLayout() && this.isCollapsed()){
				this._realign(this._collapsedGroup, this._expandedGroup.getLocalBoundingBox());
			}
        }
    },
    
    _doLayout: function( /*Object*/target ) {
        // summary:
        //		Layout the part of the subgraph template according to the new bounds of the graph.
        
        //if (/*dojo.config.useGfxLayout*/this.getLayout())
        //    return;
        
        target = target || this;
        
        // get new graph bounds
        var graphBounds = this._graph.getLocalBoundingBox();
        if (!graphBounds) {
            graphBounds = {x:0, y:0, width:80, height:40};
    	} else {
            graphBounds = g.cloneRect(graphBounds);
        }
        if (target._header) {
            if (target._graphPosition) {
                var dx = graphBounds.x - target._graphPosition.x;
                var dy = graphBounds.y - target._graphPosition.y;
                target._header.applyLeftTransform(dojox.gfx.matrix.translate(dx, dy));
            } else {
            	target._graphPosition = {};
            }
            target._graphPosition.x = graphBounds.x;
            target._graphPosition.y = graphBounds.y;
        }
        
        var gm = this.getGraphMargin();
        graphBounds.x -= gm;
        graphBounds.y -= gm;
        graphBounds.width += 2 * gm;
        graphBounds.height += 2 * gm;
        
		var shape;
		
        // move/resize graph background rect
        if (target._graphBackground) {
            shape = target._graphBackground.getShape();
            shape.x = graphBounds.x;
            shape.y = graphBounds.y;
            shape.width = graphBounds.width;
            shape.height = graphBounds.height;
            target._graphBackground.setShape(shape);
        }
        
        // move/resize overall background rect
        var bg = target._expandedBaseShape || target._background; // try also _background for compatibility...
        if (bg) {
            shape = bg.getShape();
            shape.x = graphBounds.x - this._margin;
            shape.y = graphBounds.y - this._headerHeight;
            shape.width = graphBounds.width + 2 * this._margin;
            shape.height = graphBounds.height + this._headerHeight + this._margin;
            bg.setShape(shape);
        }
    },
    
    _graphChanged: function() {
        if (dojo.config.useGfxLayout) {
            if (this.getLayout()) 
                this.invalidate();
            else
                this._doLayout();
        } else {
            this._doLayout();
        }
        
    },
    
	setExplicitSize: function(sz)
	{
		// summary:
		//		Does nothing, Subgraphs cannot have an explicit size, their size is determined by their children.
		
		if(this._isIBMDiagramSwimLane)
			this.inherited(arguments);
	},
	
    // 
    // Intergraph links invalidation:
    // (intergraph links are added to the subgraph's _intergraphLinks array by the Link,
    // when setting the start/end ports).
    // 
    
    _onChanged: function(shape, reason) {
    	// This method is called a lot, keep it short and fast :).

    	//	[av] this.inherited(arguments); replaced by:
		ibm_ilog.diagram.Node.prototype._onChanged.apply(this,arguments);
        
    	if(!ibm_ilog.diagram.Node._ChangesInvalidateLinksDisabled) {
	    	if (shape == this && reason == "setTransform") {
	            this._invalidateIntergraphLinks();
	        }
    	}
    },
    
    _invalidateIntergraphLinks: function() {
        if (this._intergraphLinks) {
            dojo.forEach(this._intergraphLinks, function(link) {
                link.invalidateLinkShape();
            });
        }
    },
    
    getIntergraphLinks: function(){
    	// summary:
        //		return all the intergraph links
    	// return array of ibm_ilog.diagram.Link
        return this._intergraphLinks || [];
    },
    _findAllStyledElements: function() {
    	// summary:
        //		find all the styled elements, from collapsed and expanded groups
    	// override Selectable method
        if(this._collapsedGroup!=null){
        	this._findStyledChildren(this._collapsedGroup.children);
        }
        if(this._expandedGroup!=null){
        	this._findStyledChildren(this._expandedGroup.children);
        }
    },
    
    _applySelectedStyle: function(shape,selected){
    	// summary:
        //		apply the appropriate style to the shape, verifying if it is visible 
    	//      (some styles cannot be applied if no surface is associated )
    	// override Selectable method
    	if(this._isVisible(shape)){
    		this.inherited(arguments);
    	}
    },
    
    _isVisible: function(shape){
    	// summary:
        //		verify if the shape belongs to the shown group
    	var parent = shape.getParent();
    	while(parent!=null){
    		if(parent==this){
    			return true;
    		}
    		parent = parent.getParent();
    	}
    	return false;
    }
    
});

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "headerColor", "#6699FF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "selectedHeaderColor", "#0000FF");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "graphBackgroundColor", "#f8f8f8");
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "graphMargin", 10);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "textDir", "");

// The default template for Subgraphs
ibm_ilog.diagram.Subgraph.defaultTemplate = ibm_ilog.diagram.declareTemplate(ibm_ilog.diagram.templates.defaultSubgraphTemplate);

ibm_ilog.diagram.Subgraph.nodeType = dojox.gfx.Group.nodeType;
ibm_ilog.diagram.Subgraph.defaultShape = {
    type: 'subgraph'
};

})();

}

if(!dojo._hasResource["ibm_ilog.diagram.graphlayout.LayoutRunner"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm_ilog.diagram.graphlayout.LayoutRunner"] = true;
dojo.provide("ibm_ilog.diagram.graphlayout.LayoutRunner");



dojo.requireIf(!!dojo.global.ibm_ilog.graphlayout, 'ibm_ilog.diagram.graphlayout._base');

dojo.declare("ibm_ilog.diagram.graphlayout.LayoutRunner", null, {
	// summary:
	//		This class provides the graph layout API for a Graph object.
	// tags:
	//		private
	
	_nodeLayout: null,
	_linkLayout: null,
	_nodeLayoutActive: true,
	_linkLayoutActive: true,
	_automaticNodeLayout: false,
	_automaticLinkLayout: false,
	_animatedLayout: false,
	_autoFitOnLayout: false,
	_rml: null,
	_graphAdapter: null,
	_layoutProvider: null,
	
	setNodeLayout: function(/*ibm_ilog.graphlayout.GraphLayout*/layout){
		// summary:
		//		Sets the graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// layout: ibm_ilog.graphlayout.GraphLayout:
		//		The graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		if (layout != this._nodeLayout) {
			this._detachLayout(this._nodeLayout, true);
			this._nodeLayout = layout;
			this._attachLayout(this._nodeLayout, true);
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	getNodeLayout: function(){
		// summary:
		//		Gets the graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// returns: ibm_ilog.graphlayout.GraphLayout:
		//		The graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		return this._nodeLayout;
	},
	
	setNodeLayoutActive: function(active){
		// summary:
		//		Activates or deactivates the graph layout specified by setNodeLayout().
		// active: boolean:
		//		True to activate the graph layout, false to deactivate it.
		// description:
		//		If the graph layout is active, it will be performed when performGraphLayout() is called.
		//		For example, you can call setNodeLayoutActive(false) if you want to perform only a link layout.
		
		if (active != this._nodeLayoutActive) {
			this._nodeLayoutActive = active;
			// Update the multiple layout instance
			var rml = this._rml;
			if (rml != null) {
				var ml = rml.getLayout(null);
				ml.setFirstGraphLayoutActive(active);
			}
			// We must refresh the sub-layouts because they may be an inherited copy of this one.
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	isNodeLayoutActive: function(){
		// summary:
		//		Returns true if the graph layout is active, or false otherwise.
		// returns: boolean:
		//		True if the graph layout is active, false otherwise.
		
		return this._nodeLayoutActive;
	},
	
	setAutomaticNodeLayout: function(automatic){
		// summary:
		//		Determines if the graph layout must be executed automatically
		//		whenever the graph changes.
		// automatic: boolean:
		//		If true, the graph layout will be executed automatically whenever the graph changes.
		//		If false, the graph layout must be executed by calling performGraphLayout explicitly.
		
		if (automatic != this._automaticNodeLayout) {
			this._automaticNodeLayout = automatic;
			if (automatic) 
				this._checkAutoLayout();
		}
	},
	
	isAutomaticNodeLayout: function(){
		// summary:
		//		Returns the flag specifying if the graph layout must be executed automatically
		//		whenever the graph changes.
		// returns: Boolean
		//		True if the graph layout will be executed automatically whenever the graph changes.
		//		False if the graph layout is executed only by calling performGraphLayout explicitly.
		return this._automaticNodeLayout;
	},
	
	setLinkLayout: function(/*ibm_ilog.graphlayout.GraphLayout*/layout){
		// summary:
		//		Sets the link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// layout: ibm_ilog.graphlayout.GraphLayout:
		//		The link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		if (layout != this._linkLayout) {
			this._detachLayout(this._linkLayout, false);
			this._linkLayout = layout;
			this._attachLayout(this._linkLayout, false);
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	getLinkLayout: function(){
		// summary:
		//		Gets the link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// returns: ibm_ilog.graphlayout.GraphLayout:
		//		The link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		return this._linkLayout;
	},
	
	setLinkLayoutActive: function(active){
		// summary:
		//		Activates or deactivates the link layout specified by setLinkLayout().
		// active: boolean:
		//		True to activate the link layout, false to deactivate it.
		// description:
		//		If the link layout is active, it will be performed when performGraphLayout() is called.
		//		For example, you can call setLinkLayoutActive(false) if you want to perform only a graph layout.
		
		if (active != this._linkLayoutActive) {
			this._linkLayoutActive = active;
			// Update the multiple layout instance
			var rml = this._rml;
			if (rml != null) {
				var ml = rml.getLayout(null);
				ml.setSecondGraphLayoutActive(active);
			}
			// We must refresh the sub-layouts because they may be an inherited copy of this one.
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	isLinkLayoutActive: function(){
		// summary:
		//		Returns true if the link layout is active, or false otherwise.
		// returns: boolean:
		//		True if the link layout is active, false otherwise.
		
		return this._linkLayoutActive;
	},
	
	setAutomaticLinkLayout: function(automatic){
		// summary:
		//		Determines if the link layout must be executed automatically
		//		whenever the graph changes.
		// automatic: boolean:
		//		If true, the link layout will be executed automatically whenever the graph changes.
		//		If false, the link layout must be executed by calling performGraphLayout explicitly.
		
		if (automatic != this._automaticLinkLayout) {
			this._automaticLinkLayout = automatic;
			if (automatic) 
				this._checkAutoLayout();
		}
	},
	
	isAutomaticLinkLayout: function(){
		// summary:
		//		Returns the flag specifying if the link layout must be executed automatically
		//		whenever the graph changes.
		// returns Boolean:
		//		True if the link layout will be executed automatically whenever the graph changes.
		//		False if the link layout is executed only by calling performGraphLayout explicitly.
		return this._automaticLinkLayout;
	},
	
	setAnimatedLayout: function(animated){
		// summary:
		//		Determines if an animation must be displayed after a graph layout has been performed.
		// animated: boolean:
		//		If true, an animation moves the nodes and reshapes the links from their position/shape
		//		before the layout to their position/shape after the layout.
		
		this._animatedLayout = animated;
	},
	
	isAnimatedLayout: function(){
		// summary:
		//		Returns the flags that determines if an animation must be displayed after a graph layout has been performed.
		// returns: boolean:
		//		True if graph layout is animated, false otherwise.
		
		return this._animatedLayout;
	},
	
	setAutoFitOnLayout: function(autoFit){
		// summary:
		//		Determines if fitToContents is called after a graph layout is performed.
		// autoFit: boolean:
		//		If true, fitToContents() is automatically called to show all the graph after a graph layout is performed.
		
		this._autoFitOnLayout = autoFit;
	},
	
	isAutoFitOnLayout: function(){
		// summary:
		//		Returne the boolean that determines if fitToContents is called after a graph layout is performed.
		// returns: boolean:
		//		True if fitToContents() is automatically called to show all the graph after a graph layout is performed.
		return this._autoFitOnLayout;
	},
	
	performGraphLayout: function(/*boolean*/recursive){
		// summary:
		//		Executes the graph layout and/or the link layout specified by the setNodeLayout()
		//		and setLinkLayout methods.
		// description:
		//		If a non-null graph layout has been specified by calling setNodeLayout(),
		//		and if the graph layout is active (that is, unless setNodeLayoutActive(false) has been called),
		//		then the graph layout algorithm is performed on the contents of this graph.
		//		Then, if a non-null link layout has been specified by calling setLinkLayout(),
		//		and if the link layout is active (that is, unless setLinkLayoutActive(false) has been called),
		//		then the link layout algorithm is performed on the contents of this graph.
		// recursive: boolean:
		//		If true, the graph layout is executed recursively on all Subgraphs contained in this Graph.
		//		If false, the graph layout is executed only on this Graph.
		
		if(arguments.length == 0)
			recursive = true;
		this._performGraphLayout(false, recursive);
	},
	
	onLayoutAnimationStart: function(){
		// summary:
		//		Called when the graph layout animation starts, if isAnimatedGraphLayout() is true.
		this._inLayoutAnimation = true;
	},
	
	onLayoutAnimate: function(v){
		// summary:
		//		Called on each step of the graph layout animation, if isAnimatedGraphLayout() is true.
		// v: number:
		//		A value between 0 and 100 specifying the progress of the animation.
	},
	
	onLayoutAnimationEnd: function(){
		// summary:
		//		Called when the graph layout animation ends, if isAnimatedGraphLayout() is true.
		this._inLayoutAnimation = false;
	},
	
	_performGraphLayout: function(auto, recursive){
	
		if (!this._rml) 
			return;
		
		// summary:
		//		Internal function that actually performs the layout.
		// auto: boolean:
		//		True if an auto-layout must be performed, false if an explicit layout is performed.
		//		If auto = true, the _automaticNodeLayout/_automaticLinkLayout flags are tested
		//		 to see which layout is in automatic mode.
		// recursive: boolean:
		//		If true, the graph layout is executed recursively on all Subgraphs contained in this Graph.
		//		If false, the graph layout is executed only on this Graph.
		
		var autoGraph = this._automaticNodeLayout;
		var autoLink = this._automaticLinkLayout;
		var animated = this._animatedLayout;
		
		// clear the auto graph layout flags to avoid infinite loops:
		// node move -> _onChange -> _checkAutoLayout -> auto-layout -> node move -> etc
		this._automaticNodeLayout = false;
		this._automaticLinkLayout = false;
		
        if (dojo.config.useGfxLayout){
            ibm_ilog.diagram.gfxlayout.Engine.layout();
        }
        
		var animation;
		if (this._animatedLayout) {
			animation = new ibm_ilog.diagram.graphlayout.Animation(this, recursive);
			animation.beforeLayout(recursive);
		}
		
		
		var swimLanes;
		if(ibm_ilog.graphlayout.hierarchical &&
				ibm_ilog.graphlayout.hierarchical.HierarchicalLayout &&
				this._nodeLayout &&
				this._nodeLayout instanceof ibm_ilog.graphlayout.hierarchical.HierarchicalLayout &&
				ibm_ilog.diagram.SwimLane){
			swimLanes = ibm_ilog.diagram.SwimLane._beforeLayout(this, this._nodeLayout);
		}
		
		if (this._rml) {
			
			this._refreshSubLayouts();
			
			var layout;
			if (recursive) 
				layout = this._rml;
			else 
				layout = this._rml.getLayout(null);
			layout.performLayout(true);
		}
		
		if (this._animatedLayout) {
			animation.afterLayout(recursive);
			animation = null;
		}
		
		if (!auto) 
			ibm_ilog.diagram.util.Batch.endBatch();
		
		if(swimLanes)
			ibm_ilog.diagram.SwimLane._afterLayout(this, this._nodeLayout, swimLanes);
		
		if (this._autoFitOnLayout && !this._animatedLayout) // animation will do the fit
			this.fitToContents();
		
		this._automaticNodeLayout = autoGraph;
		this._automaticLinkLayout = autoLink;
	},
	
	_attachLayout: function(layout, first){
		// tags:
		//		private 
		
		if (!layout) 
			return;
		
		var rml = this._rml;
		if (!rml) {
			var provider = new ibm_ilog.diagram.graphlayout._LayoutProvider(this);
			rml = new ibm_ilog.graphlayout.RecursiveMultipleLayout(null, provider);
			rml.setFirstGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.MIXED);
			rml.setSecondGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.MIXED);
			rml.attach(new ibm_ilog.diagram.graphlayout.GraphAdapter(this, rml));
			this._rml = rml;
		}
		
		var ml = rml.getLayout(null);
		
		if (first) {
			ml.setFirstGraphLayout(layout);
			ml.setFirstGraphLayoutActive(this._nodeLayoutActive);
		}
		else {
			ml.setSecondGraphLayout(layout);
			ml.setSecondGraphLayoutActive(this._linkLayoutActive);
		}
		
		// Before 22 april 2011, it was: 
		// layout.attach(new ibm_ilog.diagram.graphlayout.GraphAdapter(this, layout));
		// Fix for the configuration of local parameters (e.g. Hierarchical constraints) in 
		// subgraphs:
		layout.attach(rml.getGraphModel());
		
		layout._graphParameterChangedConnect =
			dojo.connect(layout, "onParameterChanged", this, this._onLayoutParameterChanged);
	},
	
	_detachLayout: function(layout, first){
		// tags:
		//		private
		
		if (!layout) 
			return;
		
		var rml = this._rml;
		if (rml) {
			var ml = rml.getLayout(null);
			if (first) 
				ml.setFirstGraphLayout(null);
			else 
				ml.setSecondGraphLayout(null);
			
			if (!this._nodeLayout && !this._linkLayout) {
				rml.detach();
				this._rml = null;
			}
		}
		
		if(layout._graphParameterChangedConnect)
			dojo.disconnect(layout._graphParameterChangedConnect);
	},
	
	_onLayoutParameterChanged: function(parameterName){
		// summary:
		//		Invalidates subgraph layouts when a layout property is changed.
		
		if (this._inLayoutParameterChanged || this._inAutoLayout) 
			return;
		if (parameterName == "ParametersUpToDate" || // User changed some other property.
				// Constraints of Hierarchical layout are not copied from parent
				// layout to inherited layout, hence we do not need to invalidate 
				// sublayouts when constraints are added to a parent layout.
				parameterName == "Constraints")
			return;
			
		try {
			this._inLayoutParameterChanged = true;
			
			var rml = this._rml;
			if (rml != null && !rml.isLayoutRunning()) {
				var ml = rml.getLayout(null);
				if (ml != null && !ml.isLayoutRunning()) {
					// OK, we are not in auto-layout, no layout is running,
					// we are not called recursively, we can do our stuff.					
					this._invalidateSubLayouts();
				}
			}
		}
		finally {
			delete this._inLayoutParameterChanged;
		}
	},
	
	_getParentGraph: function(){
		if (this._owningSubgraph) 
			return this._owningSubgraph.getParent();
		else 
			return null;
	},
	
	_invalidateSubLayouts: function(){
		for (var p = this; p != null; p = p._getParentGraph()) 
			p._sublayoutsInvalid = true;
	},
	
	_refreshSubLayouts: function(){
		// summary:
		//		Clears the layouts allocated by the layout provider for the subgraphs of this
		//		container. This method must be called every time a sub-layout is subject to change,
		//		that is:
		//		- when the toplevel layout itself changes (i.e., the GraphLayout.LinkLayout properties),
		//		- when a property of a toplevle layout changes,
		//		- when the Graph/LinkLayoutActive properties change
		//		- when a subgraph is added (because the sublayout must be attached immediately
		//		  if we are in auto-layout, so that the layout is done is something changes in
		//		  the subgraph.
		
		if (!this._sublayoutsInvalid) 
			return;
		delete this._sublayoutsInvalid;
		
		var rml = this._rml;
		if (rml != null) {
			var model = rml.getGraphModel();
			
			// Call the recursive method to go down into the subgraphs of this
			// container.
			//
			this._refreshSubLayoutsRec(rml.getLayoutProvider(), model, model);
		}
		
		var parent = this._getParentGraph();
		if (parent != null) 
			parent._refreshSubLayouts();
	},
	
	_refreshSubLayoutsRec: function(provider, topmodel, model){
		// summary:
		//		Recursive helper method for the previous method.
		
		var subgraphs = model.getSubgraphs();
		for (var i = 0; i < subgraphs.length; i++) {
			var submodel = topmodel.getGraphModel(subgraphs[i]);
			provider.setPreferredLayout(submodel, null);
			provider.getGraphLayout(submodel);
			this._refreshSubLayoutsRec(provider, topmodel, submodel);
		}
	},
    
	_checkAutoLayout: function(){
		// summary:
		//		Checks if an auto-layout must be performed and, if yes, schedule it.
		
		if (!this._inLayoutAnimation &&
		((this._nodeLayout && this._automaticNodeLayout && this._nodeLayoutActive) ||
		(this._linkLayout && this._automaticLinkLayout && this._linkLayoutActive))) {
			ibm_ilog.diagram.util.Batch.add(this);
		}
	},
	
	_endBatch: function(){
	
		var rml = this._rml;
		if (!rml) 
			return;
		
		try {
			if (!this._automaticNodeLayout) 
				rml.setFirstGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.INACTIVE);
			if (!this._automaticLinkLayout) 
				rml.setSecondGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.INACTIVE);
	
			this._inAutoLayout = true;
			
			this._performGraphLayout(true, true);
			
		}
		finally {
			this._inAutoLayout = false;
			
			if (!this._automaticNodeLayout) 
				rml.setFirstGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.MIXED);
			if (!this._automaticLinkLayout) 
				rml.setSecondGraphLayoutActive(ibm_ilog.graphlayout.RecursiveMultipleLayout.MIXED);
		}
	},
	
	_getActualLayout: function(first){
		// tags:
		//		private
		var toplevelGraph = null;
		for (var graph = this; graph != null; graph = graph._getParentGraph()) 
			toplevelGraph = graph;
		if (toplevelGraph && toplevelGraph._rml) {
			toplevelGraph._refreshSubLayouts();
			var ml = toplevelGraph._rml.getLayout(this._owningSubgraph);
			if(ml){
				if(first)
					return ml.getFirstGraphLayout();
				else
					return ml.getSecondGraphLayout();
			}
		}
		return null;
	},
	
	_getActualNodeLayout: function(){
		// summary:
		//		Gets the node layout that is actually executed on this graph.
		//		The actual layout can be inherited from another graph higher in the hierarchy.
		// tags:
		//		private
		return this._getActualLayout(true);
	},
	
	_getActualLinkLayout: function(toplevelGraph){
		// summary:
		//		Gets the link layout that is actually executed on this graph.
		//		The actual layout can be inherited from another graph higher in the hierarchy.
		// tags:
		//		private
		return this._getActualLayout(false);
	}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.Graph']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Graph'] = true;
dojo.provide('ibm_ilog.diagram.Graph');













dojo.declare("ibm_ilog.diagram.Graph", [dojox.gfx.Group, ibm_ilog.diagram.graphlayout.LayoutRunner], {
	// summary:
	//		The Graph object acts as a container for Node and Link objects. 

  // Allows to test the class faster that testing declaredClass
	_isIBMDiagramGraph: true,
	
	// diagram widget if this graph is embedded in a Diagram widget
	_diagram: null,
	
	constructor: function() {
    var id = ibm_ilog.diagram._IdGenerator.nextId(this).toString();
    this.getId = function() {
        return id;
    };
	},
	
	createNode: function(/*String || Object*/template, context) {
		// summary:
		//		Creates a new node in this Graph.
		// template:
		//		The template of the new node. If not specified, the default node template is used.
		//      If null, then an empty node is created.
		// returns:
		//		The new node.
		
		var node = this._createShape(ibm_ilog.diagram.Node, template, ibm_ilog.diagram.Node.defaultTemplate, context);
		return node;
	},
	
	createLink: function(/*String || Object*/template, context) {
		// summary:
		//		Creates a new link in this Graph.
		// description:
		//      The description defined in the template parameter must specify which shape 
		//      in the template defines the link path by means of the dojoAttachPoint attribute set to '_path'.
		//      In addition, the optional start and end arrow shapes can be defined in the template
		//      object and in this case must be declared using the corresponding '_startArrow' and '_endArrow'
		//      dojoAttachPoint's. 
		// template:
		//		The template of the new link. If not specified, the default link template is used.
		// returns:
		//		The new link.
		
		var link = this._createShape(ibm_ilog.diagram.Link, template, ibm_ilog.diagram.Link.defaultTemplate, context);
		return link;
	},
	
	createSubgraph: function(/*String || Object*/template, context) {
		// summary:
		//		Creates a new subgraph in this Graph.
		// template:
		//		The template of the new subgraph. If not specified or null, the default subgraph template is used.
		// returns:
		//		The new subgraph.
		
		var subgraph = this._createShape(ibm_ilog.diagram.Subgraph, template, ibm_ilog.diagram.Subgraph.defaultTemplate, context);
		return subgraph;
	},
	
	createSwimLane: function(/*String || Object*/template, context) {
		// summary:
		//		Creates a new swim lane in this Graph.
		// template:
		//		The template of the new swim lane. If not specified or null, the default swim lane template is used.
		// returns:
		//		The new swim lane.
		
		var swimlane = this._createShape(ibm_ilog.diagram.SwimLane, template, ibm_ilog.diagram.SwimLane.defaultTemplate, context);
		return swimlane;
	},
	
	_createShape: function(type, template, defaultTemplate, context) {
		var shape = this.createObject(type);
		this._applyTemplate(shape, template, defaultTemplate, context);
		return shape;
	},
	
	_applyTemplate: function(shape, template, defaultTemplate, gfxContext) {
	    if (dojo.config.isBidiInDojoDiagrammer)
		    this._propogateTextDir(shape);	
		if (template === null) // null-> empty shape 
			return;
		if (!template) { // undefined-> default template fallback
			template = defaultTemplate;
		}
		if (!template) {
			ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedArgument", this, "createSubgraph", "template", template);
		}
		shape.applyTemplate(template, gfxContext);
		shape._template = template; // [AV] temporary, TBD with Patrick
		return shape;
	},
		
	_propogateTextDir: function(shape) { 
	    if (dojo.config.isBidiInDojoDiagrammer) {       
		    var tDir = this.textDir, gr = this;
		    while(!tDir){
			    if (!gr.getParent)
				    break;
			    gr = gr.getParent();
			    tDir = gr.textDir;
		    }		
		    if (tDir && shape.setTextDir)
			    shape.setTextDir(tDir);	
	    }		    	
	 },
	
	onChanged: function() {
		// summary:
		//		This event is called to signal that the contents of the graph have changed.
	},
	
	_onChanged: function(shape, reason) {
		// This method is called a lot, keep it short and fast :).
		
		// <inlining this call from BoundingBoxFix at diagext.js>
		// 
		// this.inherited(arguments);
		
		this._bbCacheValid = false;
		
		// <end inline>
		
		// TODO: aggregate changes
		this.onChanged();
		
		if (this._owningSubgraph) 
			this._owningSubgraph._graphChanged();
		
		// Auto-layout:
		// We do an auto-layout only if a node changes (basically when a node is moved),
		// to avoid infinite loops: node move -> auto-layout -> link change -> _onChanged -> etc
		if (shape._isIBMDiagramNode) 
			this._checkAutoLayout();
	},
	
	clear: function(/*boolean*/dispose) {
		// summary: 
		//     removes all shapes from a group/surface
		// dispose: boolean :
		//     Indicates whether the graph contents should be disposed. The default
		//     value is true.
		
		if (typeof(dispose) === 'undefined') 
			dispose = true;
		if (dispose) {
			dojo.forEach(this.children, function(s) {
				if (s.dispose && dojo.isFunction(s.dispose)) 
					s.dispose();
			}, this);
		}
		this.inherited(arguments);
	},
	
	textDir: "",  //text direction property
	
	setTextDir: function(/*String*/textdir) {
	    // summary: 
	    //      set text direction to graph, propagate a new text direction to children (subgraphs, nodes, links)
	    // textdir: String, text direction 
	 
        if (dojo.config.isBidiInDojoDiagrammer) {
            if (this.textDir == textdir)
                return;
	  	    this.textDir = textdir;
	   	    dojo.forEach(this.children, function(n) {
	   	        if (n.setTextDir) {
	   	            if (n instanceof ibm_ilog.diagram.Subgraph)
	   	                n.propagateTextDir(textdir);
	   	            n.setTextDir(textdir);
	   	        }    
	   	    });
        }	   	    
	},
	
	getTextDir: function() {
        // summary: 
	    //      returns text direction of the graph	
	    return this.textDir;
	},
		
	//------------------
	// Fit to contents
	//------------------
	
	fitToContents: function(margin, zoomOutOnly) {
		// summary:
		//		Changes the transform of this Graph so that the whole graph is visible in its parent surface.
		// margin: number:
		//		A margin (in pixels) to leave around the graph contents.
		// zoomOutOnly: boolean:
		//		If true, the graph will only be zoomed out, that is, if its contents are smaller than the surface,
		//			the graph will only be centered and not zoomed in.
		//		If false, the graph will be zoomed in or out to fit the size of the parent surface.
		if (this._diagram){
      var graphBounds = this.getBoundingBox();
      var viewBounds = this._diagram.getViewport().getViewRect();
		  if (zoomOutOnly){
				if (!margin) 
					margin = 10;
		    if (graphBounds.width + 2 * margin <= viewBounds.width && graphBounds.height + 2 * margin <= viewBounds.height){
		      var r = {};
          r.x = graphBounds.x - (viewBounds.width - graphBounds.width) / 2 + margin;
          r.y = graphBounds.y - (viewBounds.height - graphBounds.height)/ 2 + margin;
          r.width = viewBounds.width;
          r.height = viewBounds.height;
          this._diagram.getViewport().setViewRect(r);
          return;
		    }
		  } 
		  this._diagram.getViewport().setViewRect(ibm_ilog.diagram.util.Geometry.expandRect(graphBounds,margin),{center:true});
		} else
		  this.setTransform(this.getFitToContentsTransform(margin, zoomOutOnly));
	},
	
	getFitToContentsTransform: function(margin, zoomOutOnly) {
		// summary:
		//		Computes the transform to apply to this Graph so that the whole graph is visible in its parent surface.
		// returns: dojox.gfx.matrix.Matrix2D:
		//		The transform to apply to the graph. If the parent of the graph is not a GFX Surface,
		//		the identity matrix is returned.
		// margin: number:
		//		A margin (in pixels) to leave around the graph contents.
		// zoomOutOnly: boolean:
		//		If true, the graph will only be zoomed out, that is, if its contents are smaller than the surface,
		//			the graph will only be centered and not zoomed in.
		//		If false, the graph will be zoomed in or out to fit the size of the parent surface.
		
		var surface;
		for (surface = this.getParent(); surface != null; surface = surface.getParent ? surface.getParent() : null) {
			if (!surface.getParent) 
				break;
		}
		if (!surface) 
			return dojox.gfx.matrix.identity;
		
		ibm_ilog.diagram.util.Batch.endBatch(); // update links
		if (!margin) 
			margin = 10;
		
		var surfaceSize = surface.getDimensions();
		
		var graphBounds = this.getBoundingBox();
		var viewBounds = {
			x: margin,
			y: margin,
			width: surfaceSize.width - 2 * margin,
			height: surfaceSize.height - 2 * margin
		};
		
		var t;
		if (!zoomOutOnly || graphBounds.width > viewBounds.width || graphBounds.height > viewBounds.height) 
			t = dojox.gfx.matrix.createFromRectangles(graphBounds, viewBounds, true);
		else 
			t = dojox.gfx.matrix.translate((viewBounds.x + viewBounds.width / 2) - (graphBounds.x + graphBounds.width / 2), (viewBounds.y + viewBounds.height / 2) - (graphBounds.y + graphBounds.height / 2));
		
		return t;
	},
	
	
	//------------------
	// Hit testing API
	//------------------
	
	hitTest: function(p, tolerance) {
		// Summary:
		//		find the diagram elements bellow the given point
		//		The resulting array is order by the z-order of the elements
		var elements = [];
		if (tolerance == null) {
			tolerance = 2;
		}
		this._hitTest(this, p, elements, tolerance);
		return elements;
	},
	
	_hitTest: function(graph, p, elements, tolerance) {
		//	TODO [av] gino: I changed iteration to use ints and avoid parseInts. Also I 
		//	fixed a bug where inner shapes of subgraph are reported even when collapsed.
		//	
		//		for (var n in graph.children) {
		//			// we iterate in reverse order to return the last elements first (z-order)
		//			var c = graph.children[graph.children.length - 1 - parseInt(n)];
		//			if (c.hitTest && dojo.isFunction(c.hitTest)) {
		//				if (c.hitTest(p, this.getParent(), tolerance)) {
		//					if (c._isIBMDiagramSubgraph) {
		//						this._hitTest(c.getGraph(), p, elements);
		//					}
		//					elements.push(c);
		//				}
		//			}
		//		}
		for (var i=graph.children.length-1;i>=0;i--) {
			// we iterate in reverse order to return the last elements first (z-order)
			var c = graph.children[i];
			if (c.hitTest && dojo.isFunction(c.hitTest)) {
				if (c.hitTest(p, this.getParent(), tolerance)) {
					if (c._isIBMDiagramSubgraph && !c.isCollapsed()) {
						this._hitTest(c.getGraph(), p, elements);
					}
					elements.push(c);
				}
			}
		}
	},
	
	//------------------------------------
	// Shortcuts to enable selection/move
	//------------------------------------
	
	setSelect: function(select)
	{
		// summary:
		//		Enables or disables interactive selection on this Graph.
		// select: Boolean
		//		If true, the user will be able to select nodes by clicking on them.
		//		If false, selection will be disabled.
		
		if(select != this._select){
			if(select){
				if (!this._selInteractor) {
					dojo['require']("ibm_ilog.diagram.interactors.SelectionInteractor");
					dojo.addOnLoad(this, function(){
						this._selInteractor = new ibm_ilog.diagram.interactors.SelectionInteractor().initialize(this);
						if(this._select)
							this._selInteractor.activate();
					});
				}
				else {
					this._selInteractor.activate();
				}
			} else if(this._selInteractor){
				this._selInteractor.deactivate();
			}
			this._select = select;
		}
		return this;
	},
	
	isSelect: function()
	{
		// summary:
		//		Returns true if selection is enabled, or false if selection is disabled.
		
		return this._select || false; // Boolean
	},
	
	getSelection: function(){
		// summary:
		//		Returns the selection object, if selection is enabled in the graph (isSelect == true).
		if(this.isSelect() && this._selInteractor){
			return this._selInteractor.getSelection();
		}
		return null;
	},
	
	setMove: function(move)
	{
		// summary:
		//		Enables or disables interactive move on this Graph.
		// description:
		//		Enables or disables interactive move on this Graph.
		//		Note that enabling move also enables select automatically.
		// select: Boolean
		//		If true, the user will be able to move nodes by dragging them.
		//		If false, move will be disabled.
		
		if(move != this._move){
			if(move){
				this.setSelect(true);
				if (!this._moveInteractor) {
					dojo['require']("ibm_ilog.diagram.interactors.MoveInteractor");
					dojo.addOnLoad(this, function(){
						this._moveInteractor = new ibm_ilog.diagram.interactors.MoveInteractor().initialize(this._selInteractor.getSelection(), this);
						if(this._move)
							this._moveInteractor.activate();
					});
				}
				else {
					this._moveInteractor.activate();
				}
			} else if(this._moveInteractor){
				this._moveInteractor.deactivate();
			}
			this._move = move;
		}
		return this;
	},
	
	isMove: function()
	{
		// summary:
		//		Returns true if move is enabled, or false if move is disabled.
		
		return this._move || false;
	},
	
	//----------------------------------------
	// Shortcuts to get nodes/links/subgraphs
	//----------------------------------------
	
	getNodes: function()
	{
		// summary:
		//		Returns the nodes (that is, instances of ibm_ilog.diagram.Node) contained in this Graph.
		
		return dojo.filter(this.children, function(item) { return item._isIBMDiagramNode; }); // Array
	},
	
	getLinks: function()
	{
		// summary:
		//		Returns the links (that is, instances of ibm_ilog.diagram.Link) contained in this Graph.
		
		return dojo.filter(this.children, function(item) { return item._isIBMDiagramLink; }); // Array
	},
	
	getSubgraphs: function()
	{
		// summary:
		//		Returns the subgraphs (that is, instances of ibm_ilog.diagram.Subgraph) contained in this Graph.
		
		return dojo.filter(this.children, function(item) { return item._isIBMDiagramSubgraph; }); // Array
	},
	
	//--------------
	// Check moves:
	//--------------
	
	onElementMoving: function(element, transform, delta)
	{
		// summary:
		//		Called when an element is about to be moved by an interactor.
		// description:
		//		This event lets a Graph forbid or limit node moves.
		//		The event handler can modify the delta argument:
		//		if delta.x or delta.y are modified, the move is limited to the new delta
		//		(and other objects moved together with the specified element will also be limited accordingly).
		//		The default implementation checks the limits set by setMoveLimits(rect) (null by default -> no limit).
		// element: ibm_ilog.diagram.GraphElement
		//		The element being moved
		// transform: dojox.gfx.matrix.Matrix2D
		//		The original transform of the element.
		// delta: Point
		//		An object with x and y properties specifying the move delta.
		//		This argument can be modified by event handlers to forbid of limit the move.
		
		var limits = this.getMoveLimits();
		if (limits) {
			// delta is in untransformed element coords, so convert the limits accordingly...
			if (transform) 
				limits = transform.inverse().transformRectangle(limits);
			var bb = element.getBoundingBox();
			if (bb.x + delta.x + bb.width > limits.x + limits.width) 
				delta.x = limits.x + limits.width - bb.x - bb.width;
			if (bb.y + delta.y + bb.height > limits.y + limits.height) 
				delta.y = limits.y + limits.height - bb.y - bb.height;
			if (bb.x + delta.x < limits.x) 
				delta.x = limits.x - bb.x;
			if (bb.y + delta.y < limits.y) 
				delta.y = limits.y - bb.y;
		}
	},
	
	_moveLimits: null,
	
	setMoveLimits: function(rect)
	{
		// summary:
		//		Sets the move limits for this Graph.
		// description:
		//		This method lets you limit interactive node moves to a given rectangle in the Graph.
		// rect: Rectangle
		//		A rectangle (x/y/width/height) that specifies the region where nodes
		//		are allowed to be moved. If null, node moves are not limited.
		
		this._moveLimits = rect;
	},
	
	getMoveLimits: function()
	{
		// summary:
		//		Gets the move limits for this Graph.
		// description:
		//		See setMoveLimits().
		// returns: Rectangle
		//		A rectangle (x/y/width/height) that specifies the region where nodes
		//		are allowed to be moved. If null, node moves are not limited.
		//		The default is null (no limit).
		
		return this._moveLimits;
	},
	
	//--------------
	// toString()...
	//--------------
	
	toString: function() {
		return "Graph[" + this.children.length + "]" + (this._owningSubgraph ? (" at " + this._owningSubgraph.toString()) : "");
	}
});

ibm_ilog.diagram.Graph.nodeType = dojox.gfx.Group.nodeType;
ibm_ilog.diagram.Graph.defaultShape = {
	type: 'graph'
};

(function() {
	// Extend the GFX Surface and Group objects with a createGraph function:
	
	var graphCreator = {
		createGraph: function() {
			var graph = this.createObject(ibm_ilog.diagram.Graph);
			return graph;
		}
	};
	dojo.extend(dojox.gfx.Surface, graphCreator);
	dojo.extend(dojox.gfx.Group, graphCreator);
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.DataConnector']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.DataConnector'] = true;
dojo.provide('ibm_ilog.diagram.widget.DataConnector');



















/**
 * @author John Kelly
 *
 */
(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare('ibm_ilog.diagram.widget.DataConnector', null, {
		//
		//	summary: 
		//
		//		The Data Connector class encapsulates the relationship
		//		between data stores, nodes and links.
		//
		//	description:
		//		
		//		Based on information provided through setters, the Data Connector reads in
		//      information from one or two data stores, then builds a model of the relationship
		//      between the items. When the model is complete, the Data Connector creates
		//      Subgraphs, Nodes and Links as appropriate.
		//
		//      There may be up to two stores involved. 
		//      - A nodesStore whose items represent subgraphs or nodes.
		//      - A linksStore whose items describe the relationship between nodeStore items.
		//      Both stores must, at least, support both dojo.data.api.Read and dojo.data.api.Identity
		//      data store features, as defined at http://docs.dojocampus.org/dojo/data
		//
		//      The basic flow is as follows:
		//      1.Based on the nodesStore query and queryOptions provided, load all matching items
		//        into the model.
		//      2.If defined, load in items from the linksStore. Each item will then be examined for 
		//        attributes "start" and "end". The values of these attributes are the identities 
		//        of items in the nodesStore. This inter-item relationship information is added to the
		//        model. If any the items referred to has not already been loaded from the 
		//        nodesStore it is done so now.
		//      3.Based on any to/from relationship functions or fields defined, add inter-item 
		//        relationship information to the model.If any the items referred to has not 
		//        already been loaded from the nodesStore it is done so now. Repeat step 3 until
		//        all items that have been loaded have had there inter-item relationships resolved.
		//      4.Now the model is fully populated, any final information needed can be deduced directly
		//        from the model.
		//      5.Based on information in the model, and options provided to the class instance, Subgraphs
		//        Nodes and links are created.
		//
		//	unit test:
		//
		//      The associated unit testcase is ibm_diagram/tests/widget/DataConnector.html
		//  
		
		// Private properties   
		
		// ------------------------------------
		// Node-related
		// ------------------------------------
		
		//
		// The validated nodes query
		//
		_nodesQuery: null,
		
		//
		// The validated nodes query options object
		//
		_nodesQueryOptions: null,
		
		// A string that indicates either the data item field name defining the child
		// relationship or a function name that returns the origin of a link 
		// for the specified data item    
		_childBinding: null,
		_childBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the parent relationship or a function name that returns
		// the destination of a link for the specified data item.    
		_parentBinding: null,
		_parentBindingFunction: null,
		
		// A string that indicates either the data item field name defining the from
		// relationship or a function name that returns the origin of a link 
		// for the specified data item    
		_predecessorsBinding: null,
		_predecessorsBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the to relationship or a function name that returns
		// the destination of a link for the specified data item.    
		_successorsBinding: null,
		_successorsBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "x location" associated with a nodesStore data item or a named function that 
		// returns the "x location" associated with a nodesStore data item.    
		_xBinding: null,
		_xBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "y location" associated with a nodesStore data item or a named function that 
		// returns the "y location" associated with a nodesStore data item.    
		_yBinding: null,
		_yBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "node graph data" associated with a nodesStore data item or a named function that 
		// returns the "node graph data" associated with a nodesStore data item.    
		_nodesGraphBinding: null,
		_nodesGraphBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "link graph data" associated with a linksStore data item or a named function that 
		// returns the "link graph data" associated with a linksStore data item.    
		_linksGraphBinding: null,
		_linksGraphBindingFunction: null,
		
		// A boolean to determine what xBinding/yBinding values represent
		//  - If true, the values provided by xBinding/yBinding atributes represent the center of the node.
		//  - If false, the values provided by xBinding/yBinding atributes represent the top left-hand corner of the node.
		_centerOnLocation: false,
		
		//
		// An array of node object functions which will be used to persist
		// graphical information to the store, if required.
		//
		
		_nodesGraphFuncMap: null,
		_nodesGraphProperties: null,
		
		//
		// The validated node data store
		//
		_nodesStore: null,
		
		//
		// Flag to indicate how hierarchical data should be represented visually.
		// If true, it will represent the nodesStore items as ibm_ilog.diagram.Node instances 
		// connected by links. If false, it will represent non-leaf items as ibm_ilog.diagram.Subgraph 
		// instances and leaf items as ibm_ilog.diagram.Node instances.
		_createLinksForHierarchy: true,
		
		// Defines which of the dojo.data.api that the store supports
		// See http://docs.dojocampus.org/dojo/data/api
		_nodesStoreFeatures: null,
		
		//
		// the map of store items to shapes nodes
		//
		_model: null,
		
		// ------------------------------------
		// Link-related
		// ------------------------------------
		
		//
		// An array of link object functions which will be used to persist
		// graphical information to the store, if required.
		//
		_linksGraphFuncMap: null,
		_linksGraphProperties: null,
		
		//
		// The validated link data store
		//
		_linksStore: null,
		
		// Defines which of the dojo.data.api that the store supports
		// See http://docs.dojocampus.org/dojo/data/api
		_linksStoreFeatures: null,
		
		//
		// The validated links query object
		//
		_linksQuery: null,
		
		//
		// The validated links query options object
		//
		_linksQueryOptions: null,
		
		// A string that indicates either the data item field name defining the start
		// item in a link or a function name that returns the origin of a link 
		// for the specified data item    
		_startNodeBinding: null,
		_startNodeBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the end item in a link or a function name that returns
		// the destination of a link for the specified data item.    
		_endNodeBinding: null,
		_endNodeBindingFunction: null,
		
		//
		// Array of notification handles
		_nodesStoreConnectHandles: null,
		_linksStoreConnectHandles: null,
		
		constructor: function() {
			// summary: 
			//     Construct and instance of ibm_ilog.diagram.widget.DataConnector and 
			//     initialise it.              
			// tags:
			//	    private      
			this._initAttributes();
			this._startNodeBinding = ibm_ilog.diagram.widget.DataConnector.StartLinkFieldname;
			this._endNodeBinding = ibm_ilog.diagram.widget.DataConnector.EndLinkFieldname;
			this._model = new ibm_ilog.diagram.widget._DataModel();
			
		},
		
		_initAttributes: function() {
			// summary: 
			//     initialise attributes              
			// tags:
			//	    private      
			this._nodesStoreConnectHandles = null;
			this._linksStoreConnectHandles = null;
			this._createLinksForHierarchy = true;
			this._nodesQuery = null;
			this._nodesQueryOptions = null;
			this._parentBinding = null;
			this._parentBindingFunction = null;
			this._childBinding = null;
			this._childBindingFunction = null;
			this._predecessorsBinding = null;
			this._predecessorsBindingFunction = null;
			this._successorsBinding = null;
			this._successorsBindingFunction = null;
			this._xBinding = null;
			this._xBindingFunction = null;
			this._yBinding = null;
			this._yBindingFunction = null;
			this._nodesGraphBinding = null;
			this._nodesGraphBindingFunction = null;
			this._centerOnLocation = false;
			this._nodesGraphFuncMap = null;
			this._nodesGraphProperties = "";
			this._nodesStore = null;
			this._nodesStoreFeatures = null;
			this._linksGraphFuncMap = null;
			this._linksGraphProperties = "";
			this._linksStore = null;
			this._linksStoreFeatures = null;
			this._linksQuery = null;
			this._linksQueryOptions = null;
			this._linksGraphBinding = null;
			this._linksGraphBindingFunction = null;
			this._startNodeBinding = null;
			this._startNodeBindingFunction = null;
			this._endNodeBinding = null;
			this._endNodeBindingFunction = null;
		},
		
		_load: function() {
			// summary: 
			//     Validate options, loads data and creates the graph
			// tags:
			//	    private 
			if (this._isValidModel()) {
				this._reset();
				this._buildModelFromStores();
			}
		},
		
		_destroyDataConnector: function() {
			// summary: 
			//     Clear up all resources used by the DataConnector
			// tags:
			//	    private 
			this._reset();
			delete this._model;
			this._initAttributes();
		},
		
		_reset: function() {
			// summary: 
			//     Reset the current collection of nodes and links.
			// tags:
			//	    private 
			
			//this._model.initLinks();
			//this._model.initNodes();
			this._clearNodes();
		},
		
		_clearNodes: function() {
			// summary: 
			//     Clear the current collection of nodes created from the nodes store.
			// tags:
			//	    private

			// clear graphical shapes			
			this._model.forEachEntry(function(e) {
				// remove all links attached to this node (both linksStore and implicits)
				var links = e.node.getLinks();
				dojo.forEach(links, function(link) {
					this._linkDispose(link);
				}, this);
				// and dispose the node
				this._nodeDispose(e.node);
			}, this);
			// clear data model
			this._model.initNodes();
			// also the links since all the links shapes have been removed
			this._model.initLinks();
		},
		
		_clearLinks: function() {
			// summary: 
			//     Clear the current collection of links created from the link store
			// tags:
			//	    private
			
			this._model.forEachLinkEntry(function(e) {
				var link = e.link;
				var ld = link.getData();
				if (ld && ld.store && ld.store === this._linksStore) {
					this._linkDispose(link);
				}
			}, this);
			this._model.initLinks();
		},
		
		
		// ===============================================================
		// Internal child/parent binding functions
		// ===============================================================
		
		_childBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item) {
			// summary: 
			//     Find the items which are associated with the childBinding field.
			// tags:
			//	    private 
			return nodesStore.getValues(item, this._childBinding);
		},
		
		_parentBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item) {
			// summary: 
			//     Find the items which are associated with the parentBinding field.
			// tags:
			//	    private   
			return nodesStore.getValues(item, this._parentBinding);
		},
		
		
		// ===============================================================
		// Internal successors/from binding functions
		// ===============================================================
		
		_successorsBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item) {
			// summary: 
			//     Find the items which are associated with the successorsBinding field.
			// tags:
			//	    private 
			return nodesStore.getValues(item, this._successorsBinding);
		},
		
		_predecessorsBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item) {
			// summary: 
			//     Find the items which are associated with the predecessorsBinding field.
			// tags:
			//	    private   
			return nodesStore.getValues(item, this._predecessorsBinding);
		},
		
		// ===============================================================
		// Internal x/y binding functions
		// ===============================================================
		
		_xBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item,/* new value */ value) {
			// summary: 
			//     Get/Set the value of the xBinding field.
			// tags:
			//	    private
			var retVal;
			if (value) {
				retVal = nodesStore.setValue(item, this._xBinding, value);
			} else {
				retVal = nodesStore.getValue(item, this._xBinding);
				if (!retVal) {
					retVal = 0;
				}
			}
			return retVal;
		},
		
		_yBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item, /* new value */ value) {
			// summary: 
			//     Get/Set the value of the yBinding field.
			// tags:
			//	    private   
			var retVal;
			if (value) {
				retVal = nodesStore.setValue(item, this._yBinding, value);
			} else {
				retVal = nodesStore.getValue(item, this._yBinding);
				if (!retVal) {
					retVal = 0;
				}
				
			}
			return retVal;
			
		},
		
		// ===============================================================
		// Internal nodes/links graphical binding functions
		// ===============================================================
		
		_graphBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item,/* new value */ value) {
			// summary: 
			//     Get/Set the value of the nodesGraphBinding field.
			// tags:
			//	    private
			var retVal;
			if (value) {
				var jvalue = dojo.toJson(value);
				retVal = nodesStore.setValue(item, this._nodesGraphBinding, jvalue);
			} else {
				var retValj = nodesStore.getValue(item, this._nodesGraphBinding);
				retVal = dojo.fromJson(retValj);
			}
			return retVal;
		},
		
		_graphBindingByLinksStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item, /* new value */ value) {
			// summary: 
			//     Get/Set the value of the linksGraphBinding field.
			// tags:
			//	    private   
			var retVal;
			if (value) {
				var jvalue = dojo.toJson(value);
				retVal = nodesStore.setValue(item, this._linksGraphBinding, jvalue);
			} else {
				var retValj = nodesStore.getValue(item, this._linksGraphBinding);
				retVal = dojo.fromJson(retValj);
			}
			return retVal;
		},
		
		// ===============================================================
		// Setters and Getters
		// ===============================================================
		
		_setCreateLinksForHierarchy: function(/* Boolean */value) {
			// summary: 
			//     Sets the render mode.
			// tags:
			//	    private 
			this._createLinksForHierarchy = value;
			return value;
		},
		
		_getCreateLinksForHierarchy: function() {
			// summary: 
			//     Returns the render mode.
			// tags:
			//	    private 
			return this._createLinksForHierarchy;
		},
		
		_setNodesStore: function(/* dojo.data.api.Read */value) {
			// summary: 
			//     Sets the nodes store. The nodes store provides data item to be 
			//     represented as nodes in the diagram. It must implement, at least, 
			//     the dojo.data.api.Read and dojo.data.api.Indentity interfaces.
			// tags:
			//	    private 
			
			// If there is already a store associated already...
			if (this._nodesStore) {
				// clear graphical shapes
				this._clearNodes();
				// drop notifications from it
				this._unwireNodesStore();
			}
			
			if (value) {
				this._nodesStoreFeatures = value.getFeatures();
				if (this._isSupportedStore(this._nodesStoreFeatures)) {
					this._nodesStore = value;
				} else {
					this._nodesStore = null;
				}
			} else {
				this._nodesStore = null;
			}
			
			// wire notifications in 
			if (this._nodesStore) {
				this._wireNodesStore();
			}
			
			return this._nodesStore;
		},
		
		_getNodesStore: function() {
			// summary: 
			//     Returns the nodes store.
			// tags:
			//	    private 
			return this._nodesStore;
		},
		
		_setLinksStore: function(/* dojo.data.api.Read */value) {
			// summary: 
			//     Sets the links store. The links store provides data item to be represented
			//     as links in the diagram. It must implement at least the dojo.data.api.Read and
			//     dojo.data.api.Indentity interfaces. If no links store is specified, links
			//     are created implicitly from the nodes data store via the connections 
			//     definition as defined by the predecessorsBinding/successorsBinding properties.
			// tags:
			//	    private 
			
			// If there is already a store associated already......
			if (this._linksStore) {
				// clear graphical shapes
				this._clearLinks();
				// drop notifications from it
				this._unwireLinksStore();
			}
			
			if (value) {
				this._linksStoreFeatures = value.getFeatures();
				if (this._isSupportedStore(this._linksStoreFeatures)) {
					this._linksStore = value;
				} else {
					this._linksStore = null;
				}
			} else {
				this._linksStore = null;
			}
			// wire notifications in 
			if (this._linksStore) {
				this._wireLinksStore();
			}
			
			
			return this._linksStore;
		},
		
		_getLinksStore: function() {
			// summary: 
			//     Returns the links store.
			// tags:
			//	    private 
			return this._linksStore;
		},
		
		_setNodesQuery: function(value) {
			// summary: 
			//     Sets the nodes query.
			// tags:
			//	    private 
			var validatedQuery = null;
			if (!value || value instanceof Object) {
				validatedQuery = this._nodesQuery = value;
			} else {
				R.error("CannotUseAStringForAQuery");
			}
			return validatedQuery;
		},
		
		_getNodesQuery: function() {
			// summary: 
			//     Returns the nodes query.
			// tags:
			//	    private 
			return this._nodesQuery;
		},
		
		_setNodesQueryOptions: function(value) {
			// summary: 
			//     Sets the nodes query options.
			// tags:
			//	    private 
			var validatedQueryOptions = null;
			validatedQueryOptions = this._nodesQueryOptions = value;
			return validatedQueryOptions;
		},
		
		_getNodesQueryOptions: function() {
			// summary: 
			//     Returns the nodes query options.
			// tags:
			//	    private 
			return this.validatedQueryOptions;
		},
		
		_setChildBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover child items.
			// tags:
			//	    private 
			this._childBinding = value;
			if (this._childBinding instanceof Function) {
				this._childBindingFunction = value;
			} else {
				if (!this._childBinding || this._childBinding === "") {
					this._childBindingFunction = null;
				} else {
					this._childBindingFunction = this._childBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getChildBinding: function() {
			// summary: 
			//     Returns the childBinding.
			// tags:
			//	    private 
			return this._childBinding;
		},
		
		_setParentBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover parent items.
			// tags:
			//	    private 
			this._parentBinding = value;
			if (this._parentBinding instanceof Function) {
				this._parentBindingFunction = value;
			} else {
				if (!this._parentBinding || this._parentBinding === "") {
					this._parentBindingFunction = null;
				} else {
					this._parentBindingFunction = this._parentBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getParentBinding: function() {
			// summary: 
			//     Returns the parentBinding.
			// tags:
			//	    private 
			return this._parentBinding;
		},
		
		_setPredecessorsBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover items predecessors.
			// tags:
			//	    private 
			this._predecessorsBinding = value;
			if (this._predecessorsBinding instanceof Function) {
				this._predecessorsBindingFunction = value;
			} else {
				if (!this._predecessorsBinding || this._predecessorsBinding === "") {
					this._predecessorsBindingFunction = null;
				} else {
					this._predecessorsBindingFunction = this._predecessorsBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getPredecessorsBinding: function() {
			// summary: 
			//     Returns the predecessorsBinding.
			// tags:
			//	    private 
			return this._predecessorsBinding;
		},
		
		_setSuccessorsBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover items to link TO.
			// tags:
			//	    private 
			this._successorsBinding = value;
			if (this._successorsBinding instanceof Function) {
				this._successorsBindingFunction = value;
			} else {
				if (!this._successorsBinding || this._successorsBinding === "") {
					this._successorsBindingFunction = null;
				} else {
					this._successorsBindingFunction = this._successorsBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getSuccessorsBinding: function() {
			// summary: 
			//     Returns the successorsBinding.
			// tags:
			//	    private 
			return this._successorsBinding;
		},
		
		_setXBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an items x position.
			// tags:
			//	    private 
			this._xBinding = value;
			if (this._xBinding instanceof Function) {
				this._xBindingFunction = value;
			} else {
				if (!this._xBinding || this._xBinding === "") {
					this._xBindingFunction = null;
				} else {
					this._xBindingFunction = this._xBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getXBinding: function() {
			// summary: 
			//     Returns the xBinding.
			// tags:
			//	    private 
			return this._xBinding;
		},
		
		_setYBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an items y position.
			// tags:
			//	    private 
			this._yBinding = value;
			if (this._yBinding instanceof Function) {
				this._yBindingFunction = value;
			} else {
				if (!this._yBinding || this._yBinding === "") {
					this._yBindingFunction = null;
				} else {
					this._yBindingFunction = this._yBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getYBinding: function() {
			// summary: 
			//     Returns the yBinding.
			// tags:
			//	    private 
			return this._yBinding;
		},
		
		_setNodesGraphBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an node item's graph data.
			// tags:
			//	    private 
			this._nodesGraphBinding = value;
			if (this._nodesGraphBinding instanceof Function) {
				this._nodesGraphBindingFunction = value;
			} else {
				if (!this._nodesGraphBinding || this._nodesGraphBinding === "") {
					this._nodesGraphBindingFunction = null;
				} else {
					this._nodesGraphBindingFunction = this._graphBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getNodesGraphBinding: function() {
			// summary: 
			//     Returns the _nodesGraphBinding.
			// tags:
			//	    private 
			return this._nodesGraphBinding;
		},
		
		_setNodesGraphProperties: function(nodesGraphProperties) {
			// summary: 
			//     Set the list of nodes properties to save
			// tags:
			//	    private
			this._nodesGraphProperties = nodesGraphProperties;
			// Set map to null so it can be-evaluated the next time it it needed.
			this._nodesGraphFuncMap = null;
		},
		
		_getNodesGraphProperties: function() {
			// summary: 
			//     Get the list of nodes properties
			// tags:
			//	    private            
			return this._nodesGraphProperties;
		},
		
		_setCenterOnLocation: function(value) {
			// summary: 
			//     Sets the centerOnLocation value.
			// tags:
			//	    private 
			this._centerOnLocation = value;
			return value;
		},
		
		_getCenterOnLocation: function(value) {
			// summary: 
			//     Returns the centerOnLocation value.
			// tags:
			//	    private 
			return this._centerOnLocation;
		},
		
		
		_setLinksQuery: function(value) {
			// summary: 
			//     Sets the links query.
			// tags:
			//	    private 
			var validatedQuery = null;
			if (!value || value instanceof Object) {
				validatedQuery = this._linksQuery = value;
			} else {
				R.error("CannotUseAStringForAQuery");
			}
			return validatedQuery;
		},
		
		_getLinksQuery: function() {
			// summary: 
			//     Returns the links query.
			// tags:
			//	    private 
			return this._linksQuery;
		},
		
		_setLinksQueryOptions: function(value) {
			// summary: 
			//     Sets the links query options.
			// tags:
			//	    private 
			var validatedQueryOptions = null;
			validatedQueryOptions = this._linksQueryOptions = value;
			return validatedQueryOptions;
		},
		
		_getLinksQueryOptions: function() {
			// summary: 
			//     Returns the links query options.
			// tags:
			//	    private 
			return this.validatedQueryOptions;
		},
		
		_setLinksGraphBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an link item's graph data.
			// tags:
			//	    private 
			this._linksGraphBinding = value;
			if (this._linksGraphBinding instanceof Function) {
				this._linksGraphBindingFunction = value;
			} else {
				if (!this._linksGraphBinding || this._linksGraphBinding === "") {
					this._linksGraphBindingFunction = null;
				} else {
					this._linksGraphBindingFunction = this._graphBindingByLinksStoreFieldName;
				}
			}
			return value;
		},
		
		_getLinksGraphBinding: function() {
			// summary: 
			//     Returns the _linksGraphBinding.
			// tags:
			//	    private 
			return this._linksGraphBinding;
		},
		
		_setLinksGraphProperties: function(linksGraphProperties) {
			// summary: 
			//     Set the list of links properties to save
			// tags:
			//	    private
			
			this._linksGraphProperties = linksGraphProperties;
			// Set map to null so it can be-evaluated the next time it it needed.
			this._linksGraphFuncMap = null;
			
		},
		
		_getLinksGraphProperties: function() {
			// summary: 
			//     Get the list of links properties
			// tags:
			//	    private
			return this._linksGraphProperties;
		},
		
		_setStartNodeBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an item which starts a link.
			// tags:
			//	    private 
			this._startNodeBinding = value;
			if (this._startNodeBinding instanceof Function) {
				this._startNodeBindingFunction = value;
			}
			return value;
		},
		
		_getStartNodeBinding: function() {
			// summary: 
			//     Returns the startNodeBinding.
			// tags:
			//	    private 
			return this._startNodeBinding;
		},
		
		_setEndNodeBinding: function(value) {
			// summary: 
			//     Sets the information to tell us now to discover an item which ends a link.
			// tags:
			//	    private 
			this._endNodeBinding = value;
			if (this._endNodeBinding instanceof Function) {
				this._endNodeBindingFunction = value;
			}
			return value;
		},
		
		_getEndNodeBinding: function() {
			// summary: 
			//     Returns the endNodeBinding.
			// tags:
			//	    private 
			return this._endNodeBinding;
		},
		
		// ===============================================================
		// Validation functions
		// ===============================================================
		
		_isValidModel: function() {
			// summary: 
			//     Validates if we have enough information available
			//     to build a coherent collection of nodes and links
			// tags:
			//	    private 
			var isValid = false;
			
			if (this._nodesStore) {
				isValid = true;
			} else {
				R.error("NodesStoreRequired");
			}
			return isValid;
		},
		
		
		_isSupportedStore: function(features) {
			// summary:
			//     Validates if the store features presented are enough to
			//     provide minimum level of functionality.
			// tags:
			//	    private 
			var isSupported = false;
			
			// ensure required interfaces are supported by the model
			if (!features['dojo.data.api.Identity']) {
				R.error("StoresMustSupportIdentity");
			} else {
				if (!features['dojo.data.api.Read']) {
					R.error("StoresMustSupportRead");
				} else {
					isSupported = true;
				}
			}
			return isSupported;
		},
		
		// ===============================================================
		// Node store loading functions
		// ===============================================================
		
		_buildModelFromStores: function() {
			// summary: 
			//     If nodesStore available and query supplied, load items based on the 
			//     nodesQuery and nodesQueryOptions. Then attempt to 
			//     resolve  connections from links store.
			// tags:
			//	    private 
			
			if (this._nodesStore && this._nodesQuery) {
				this._nodesStore.fetch({
					query: this._nodesQuery,
					queryOptions: this._nodesQueryOptions,
					scope: this,
					onError: function(error, request) {
						this.onNodesStoreFetchError.apply(this, arguments);
					},
					onComplete: function(items, request) {
						for (var i = 0; i < items.length; i++) {
							this._addStoreItemToModel(items[i]);
						}
						this._resolveConnectionsFromLinksStore();
					}
				});
			} else {
				this._resolveConnectionsFromLinksStore();
			}
		},
		
		onStoreFetchError: function(error, request) {
		
		},
		
		onNodesStoreFetchError: function(error, request) {
			// if not on debugAtAllCosts, we won't see this
			R.warn("UnexpectedErrorOnNodesStoreFetch", error);
			this.onStoreFetchError.apply(this, arguments);
		},
		
		onLinksStoreFetchError: function(error, request) {
			// if not on debugAtAllCosts, we won't see this
			R.warn("UnexpectedErrorOnLinksStoreFetch", error);
			this.onStoreFetchError.apply(this, arguments);
		},
		
		_loadNodesStoreItem: function(itemToLoad) {
			// summary: 
			//     Load individual item from the nodesStore
			//     and add it to the model.
			// tags:
			//	    private 
			this._nodesStore.loadItem({
				item: itemToLoad,
				scope: this,
				onError: function(error, request) {
					throw error;
				},
				onItem: function(item, request) {
					this._addStoreItemToModel(item);
				}
			});
		},
		
		_loadNodesStoreItemById: function(ident) {
			// summary: 
			//     Load individual item from the nodesStore
			//     by it's identity and add it to the model.
			// tags:
			//	    private 
			this._nodesStore.fetchItemByIdentity({
				identity: ident,
				scope: this,
				onError: function(error, request) {
					this.onNodesStoreFetchError.apply(this, arguments);
				},
				onItem: function(item, request) {
					this._addStoreItemToModel(item);
				}
			});
		},
		
		// ===============================================================
		// Link store loading functions
		// ===============================================================
		_getStartLinkItemId: function(linkStoreItem) {
			// tags:
			//	    private 
			var startLinkItemId = null;
			
			if (this._startNodeBindingFunction) {
				var startLinkItem = this._startNodeBindingFunction.call(this, this._linksStore, linkStoreItem, this._nodesStore);
				startLinkItemId = this._nodesStore.getIdentity(startLinkItem);
			} else {
				startLinkItemId = this._linksStore.getValue(linkStoreItem, this._startNodeBinding);
			}
			return startLinkItemId;
		},
		
		_getEndLinkItemId: function(linkStoreItem) {
			// tags:
			//	    private 
			var endLinkItemId = null;
			
			if (this._endNodeBindingFunction) {
				var endLinkItem = this._endNodeBindingFunction.call(this, this._linksStore, linkStoreItem, this._nodesStore);
				endLinkItemId = this._nodesStore.getIdentity(endLinkItem);
			} else {
				endLinkItemId = this._linksStore.getValue(linkStoreItem, this._endNodeBinding);
			}
			return endLinkItemId;
		},
		
		_resolveConnectionsFromLinksStore: function() {
			// summary: 
			//     If linksStore available, load items based on the 
			//     linksQuery and linksQueryOptions. Add information
			//     about the item relationships into the model, Then 
			//     attempt to resolve connections from nodes store.
			// tags:
			//	    private 
			if (this._linksStore) {
				this._linksStore.fetch({
					query: this._linksQuery,
					queryOptions: this._linksQueryOptions,
					scope: this,
					onError: function(error, request) {
						this.onLinksStoreFetchError.apply(this, arguments);
					},
					onComplete: function(items, request) {
					
						for (var i = 0; i < items.length; i++) {
							var currentLinkStoreItem = items[i];
							var linkEntry = this._addLinkStoreItemToModel(currentLinkStoreItem);
							linkEntry.startId = this._getStartLinkItemId(currentLinkStoreItem);
							linkEntry.endId = this._getEndLinkItemId(currentLinkStoreItem);
							
							// If we have enough information to describe a link then
							// update the model. 
							if (linkEntry.startId !== null) {
								linkEntry.start = this._model.getEntry(linkEntry.startId);
							}
							if (linkEntry.endId !== null) {
								linkEntry.end = this._model.getEntry(linkEntry.endId);
							}
							
							if (!linkEntry.start || !linkEntry.end) {
								linkEntry.pending = true;
							}
						}
						this._resolveConnectionsFromNodesStore();
					}
				});
			} else {
				this._resolveConnectionsFromNodesStore();
			}
		},
		
		// ===============================================================
		// NodeStore Notification-related functions
		// ===============================================================
		
		_wireNodesStore: function() {
			// summary: 
			//     If the nodesStore supports notification, register to be informed of any changes
			// tags:
			//	    private  
			if (this._nodesStore) {
				var features = this._nodesStore.getFeatures();
				if (features['dojo.data.api.Notification']) {
					this._nodesStoreConnectHandles = new dojox.collections.ArrayList();
					this._nodesStoreConnectHandles.add(dojo.connect(this._nodesStore, "onNew", this, "_onNewNodeItem"));
					this._nodesStoreConnectHandles.add(dojo.connect(this._nodesStore, "onSet", this, "_onSetNodeItem"));
					this._nodesStoreConnectHandles.add(dojo.connect(this._nodesStore, "onDelete", this, "_onDeleteNodeItem"));
				}
				
				if (features['dojo.data.api.Write']) {
					this._nodesStoreConnectHandles.add(dojo.connect(this._nodesStore, "save", this, "_onSaveNodesStore"));
					this._nodesStoreConnectHandles.add(dojo.connect(this._nodesStore, "revert", this, "_onRevertNodesStore"));
					// the connections to _onChanged and onBoundsChanged node events
					this._onNodeChangedHandlers = {};
				}
				
			}			
		},
		
		_unwireNodesStore: function() {
			// summary:
			//     unregister any nodesStore notifications we may have set.
			// tags:
			//	    private 
			if (this._nodesStoreConnectHandles && this._nodesStoreConnectHandles.count > 0) {
				this._nodesStoreConnectHandles.forEach(function(entry) {
					dojo.disconnect(entry);
				}, this);
				this._nodesStoreConnectHandles = null;
			}
			// the content of _onNodeChangedHandlers is disposed on a per-node basis, in _nodeDispose()
			this._onNodeChangedHandlers = null;
		},
		
		_connectItemsInExistingDiagram: function(parentItem, childItem, direction, isHierarchicalConnection) {
			// summary:
			//     utility function which connects a new node into an existing diagram.
			// tags:
			//	    private 
			
			// console.info("_connectItemsInExistingDiagram");
			// console.info("\tparentItem=" + parentItem ? this._nodesStore.getIdentity(parentItem) : "null");
			// console.info("\childItem=" + childItem ? this._nodesStore.getIdentity(childItem) : "null");
			
			var parentNode = this._getNodeFromStoreItem(parentItem);
			var childNode = this._getNodeFromStoreItem(childItem);
			
			
			if (this._createLinksForHierarchy || !isHierarchicalConnection) {
				// console.info("_createLinksForHierarchy=true");
				if (!childNode) {
					var entry = this._addStoreItemToModel(childItem);
					childNode = entry.node = this.createNode(null, childItem);
					this._applyNodeStoreItemGraphics(entry.node);
				}
				if (!parentNode) {
					var entry = this._addStoreItemToModel(parentItem);
					parentNode = entry.node = this.createNode(null, parentItem);
					this._applyNodeStoreItemGraphics(entry.node);
				}
				if (direction) {
					var link = this.connectNodes(parentNode, childNode);
				} else {
					var link = this.connectNodes(childNode, parentNode);
				}
			} else {
				// console.info("_createLinksForHierarchy=false");
				if (!childNode) {
				
					if (parentNode instanceof ibm_ilog.diagram.Subgraph) {
						// console.info("parentNode is Subgraph");
						var entry = this._addStoreItemToModel(childItem);
						childNode = entry.node = this.createNode(null, childItem, parentNode.getGraph());
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						// console.info("parentNode is Node");
						// parent node needs to change from node to subgraph
						//
						var entry = this._addStoreItemToModel(childItem);
						var parentSubgraph = this.createSubgraph(null, parentItem, parentNode.getParent());
						// Set this subgraph in the the model
						this._setNodeFromStoreItem(parentItem, parentSubgraph);
						childNode = entry.node = this.createNode(null, childItem, parentSubgraph.getGraph());
						// then re-wire linked node to the new subgraph                        
						dojo.forEach(parentNode.getLinks(true, false), function(link) {
							var port = new ibm_ilog.diagram.AutomaticPort();
							parentSubgraph.addPort(port);
							link.setStartPort(port);
						});
						dojo.forEach(parentNode.getLinks(false, true), function(link) {
							var port = new ibm_ilog.diagram.AutomaticPort();
							parentSubgraph.addPort(port);
							link.setEndPort(port);
						});
						// dispose of the old parentNode
						this._nodeDispose(parentNode, parentItem);
						
						// position child node, then parent subgraph
						this._applyNodeStoreItemGraphics(childNode);
						this._applyNodeStoreItemGraphics(parentSubgraph);
						
					}
				} else {
					// Situation does not occur in current function usage..
				}
			}
		},
		
		_onNodeChangedHandlers : null,
		
		_addOnChangedHandler: function(node, event, handler){			
			var h = dojo.connect(node, event, this, handler);
			var hl = this._onNodeChangedHandlers[node.getId()];
			if (!hl) {
				hl = [];
				this._onNodeChangedHandlers[node.getId()] = hl;
			}
			hl.push(h);
		},
		
		_clearOnChangedHandlers:function(node) {
			if (!this._onNodeChangedHandlers)
				return;
			var id = node.getId(), hl = this._onNodeChangedHandlers[id];
			if (!hl)
				return;
			dojo.forEach(hl, function(h){
				dojo.disconnect(h);
			});
			hl.length=0;
			delete this._onNodeChangedHandlers[id];
		},
		
		// ===============================================================
		//  Function to extract/store graphical information in the nodes 
		//  store itself
		// ===============================================================
		_applyNodeStoreItemGraphics: function(node) {
			// summary:
			//     Applies any graphical information to a node/subgraph
			//     that is stored in the nodesStore item, specifically..
			//     (a) TBD from the specified function
			//     (b) x/y positioning.
			// tags:
			//	    private 
			if (this._nodesGraphBindingFunction || this._xBindingFunction || this._yBindingFunction) {
			
				var item = node.getData().item, isWrite = this._nodesStore.getFeatures()['dojo.data.api.Write'];
				
				if (this._nodesGraphBindingFunction) {
					var gfxInfo = this._nodesGraphBindingFunction.call(this, this._nodesStore, item);
					if (gfxInfo) {
						this._applyProperties(node, gfxInfo);
					}
					
					// If store supports write then monitor node for changes
					if (isWrite) {
						this._addOnChangedHandler(node, '_onChanged', function() {
							if (this._nodesGraphBindingFunction && node.getData && node.getData()) {
							
								var nitem = node.getData().item;
								if (nitem && this._nodesStore.isItem(nitem)) {
									try {
										this._internalSet = true;
										
										var gfxInfo = this._extractProperties(node, this._nodesGraphProperties);
										this._nodesGraphBindingFunction.call(this, this._nodesStore, nitem, gfxInfo);
									} finally {
										this._internalSet = false;
									}
								}
							}
						});
					}
				}
				if (this._xBindingFunction || this._yBindingFunction) {
				
					var posx = 0;
					var posy = 0;
					
					if (this._xBindingFunction) {
						posx = this._xBindingFunction.call(this, this._nodesStore, item);
					}
					if (this._yBindingFunction) {
						posy = this._yBindingFunction.call(this, this._nodesStore, item);
					}
					if (this._centerOnLocation) {
						var bounds = node.getBounds();
						posx = posx - (bounds.width / 2);
						posy = posy - (bounds.height / 2);
					}
					
					node.move(posx, posy);
					
					// If store supports write then monitor node for changes
					if (isWrite) {
						this._addOnChangedHandler(node, "onBoundsChanged", function(shape, reason) {
							if (reason === "setTransform" && shape.getData && shape.getData()) {
							
								var posx = 0;
								var posy = 0;
								
								var bounds = shape.getBounds();
								
								if (this._centerOnLocation) {
									posx = bounds.x + (bounds.width / 2);
									posy = bounds.y + (bounds.height / 2);
								} else {
									posx = bounds.x;
									posy = bounds.y;
								}
								
								var sitem = shape.getData().item;
								if (sitem && this._nodesStore.isItem(item)) {
									if (this._xBindingFunction) {
										// read the current x value and only write new one if they are different
										var curposx = this._xBindingFunction.call(this, this._nodesStore, sitem);
										if (curposx !== posx) {
											try {
												this._internalSet = true;												
												this._xBindingFunction.call(this, this._nodesStore, sitem, posx);
											} finally {
												this._internalSet = false;
												
											}
										}
									}
									if (this._yBindingFunction) {
										// read the current y value and only write new one if they are different
										var curposy = this._yBindingFunction.call(this, this._nodesStore, sitem);
										if (curposy !== posy) {
											try {
												this._internalSet = true;
												this._yBindingFunction.call(this, this._nodesStore, sitem, posy);
											} finally {
												this._internalSet = false;												
											}
										}										
									}
								}
							}
						});
					}
				}
			}
		},
		
		_connectItems: function(item, filteritem, direction) {
			// summary:
			//     Resolve and links through to/from/parent/child bindings
			//     filternode can be specified to limit resolution 
			//     between just the "item" node and a filter node
			// tags:
			//	    private 
			if (this._successorsBindingFunction) {
				var connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					dojo.forEach(connectingItems, function(connectingItem) {
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, direction, false);
						}
					}, this);
				}
			}
			
			if (this._predecessorsBindingFunction) {
				var connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					dojo.forEach(connectingItems, function(connectingItem) {
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, direction, false);
						}
					}, this);
				}
			}
			
			if (this._parentBindingFunction) {
				var connectingItems = this._parentBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					dojo.forEach(connectingItems, function(connectingItem) {
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, direction, true);
						}
					}, this);
				}
			}
			
			if (this._childBindingFunction) {
				var connectingItems = this._childBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					dojo.forEach(connectingItems, function(connectingItem) {
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, direction, true);
						}
					}, this);
				}
			}
			
		},
		
		_onNewNodeItem: function(newItem, parentInfo) {
			// summary:
			//     called when a new item added to the nodesStore.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onNewNodeItem id=" + this._nodesStore.getIdentity(newItem) + ",parentInfo=" + (parentInfo ? this._nodesStore.getIdentity(parentInfo.item) : "no parent"));
			
			// connect this item to others, possibly creating a node/subgraph in the process
			this._connectItems(newItem, null, true);
			
			// Check if a parent has been defined
			if (parentInfo) {
				var parentNode = this._getNodeFromStoreItem(parentInfo.item);
				if (parentNode) {
					this._connectItems(parentInfo.item, newItem, true);
				}
			}
			
			// if a node/subgraph is not yet created for this item. then create one
			if (!this._getNodeFromStoreItem(newItem)) {
				var entry = this._addStoreItemToModel(newItem);
				entry.node = this.createNode(null, newItem);
				this._applyNodeStoreItemGraphics(entry.node);
			}
			
			// Check if this item completes any pending links
			// Create links from lists in the link model 
			var nodeEntry = this._model.getEntry(this._nodesStore.getIdentity(newItem));
			this._model.forEachPendingLinkEntry(function(linkEntry, nodeEntry) {
				if (!linkEntry.start) {
					if (nodeEntry.id === linkEntry.startId) {
						linkEntry.start = nodeEntry;
					}
					
				}
				if (!linkEntry.end) {
					if (nodeEntry.id === linkEntry.endId) {
						linkEntry.end = nodeEntry;
					}
				}
				if (linkEntry.start && linkEntry.end) {
					// We have found a pending link which can be completed with this item 
					linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, linkEntry.item);
					this._applyLinkStoreItemGraphics(linkEntry.link);
					linkEntry.pending = false;
				}
			}, nodeEntry, this);
		},
		
		_onDeleteNodeItem: function(item) {
			// summary:
			//     called when an item is deleted from the nodesStore.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onDeleteNodeItem id=" + this._nodesStore.getIdentity(item));
			// We first need to iteratively process any child items, as these
			// would not have been loaded if this potential parent 
			// not existed originally. Unfortunately we can't use the _childBindingFunction
			// to achieve this, as the datastore considers that the item currently being 
			// processed has been deleted. We'll need to find the children from the model instead.
			var id = this._nodesStore.getIdentity(item);
			var shape = this._getNodeFromStoreItem(item);
			var dc = this;
			if (shape) {
				dojo.forEach(shape.getLinks(), function(link) {
					dc._linkDispose(link);
				});
				this._nodeDispose(shape, item);
			}
			this._model.removeEntry(id);
			
		},
		
		_getExtraElements: function(Array1, Array2) {
			// tags:
			//	    private 
			var extras = [];
			for (var i = 0; i < Array2.length; i++) {
				var arrlen = Array1.length;
				var j = 0;
				for (j = 0; j < arrlen; j++) {
					if (Array2[i] === Array1[j]) {
						break;
					}
				}
				if (j === arrlen) {
					// this element was not found
					extras.push(Array2[i]);
				}
			}
			return extras;
		},
		
		_onSetNodeItem: function(modifiedItem, attribute, oldValue, newValue) {
			// summary:
			//     called when an item in nodesStore changes.
			//     This code is only interested in changes in attributes related to
			//     - store item relationships.
			//     - store item label.
			//     - x/y position
			//
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSetNodeItem id=" + this._nodesStore.getIdentity(modifiedItem) + ",attribute=" + attribute + ",old=" + oldValue + ",new=" + newValue);
			
			if (this._internalSet) 
				return;
			
			var shape = this._getNodeFromStoreItem(modifiedItem);
			if (shape) {
			
				var oldState;
				var newState;
				
				if (this._nodesStore.isItem(oldValue)) {
					oldState = [oldValue];
				} else {
					oldState = oldValue || [];
				}
				
				if (this._nodesStore.isItem(newValue)) {
					newState = [newValue];
				} else {
					newState = newValue || [];
				}
				
				if (oldState instanceof Array && newState instanceof Array) {
					var newItems = this._getExtraElements(oldState, newState);
					var deletedItems = this._getExtraElements(newState, oldState);
					// Look to remove links between this shape and the deleted items
					// console.info("deletedItems.length=" + deletedItems.length);
					for (var i = 0; i < deletedItems.length; i++) {
						var shape2 = this._getNodeFromStoreItem(deletedItems[i]);
						// console.info("\titem=" + i + " shape2=" + shape2 + " item=" + deletedItems[i].name);
						var connections = shape.getLinks();
						for (var j = 0; j < connections.length; j++) {
							var link = connections[j];
							if (link.getEndNode() === shape2) {
								this._linkDispose(link);
							}
						}
						connections = shape2.getLinks();
						for (var j = 0; j < connections.length; j++) {
							var link = connections[j];
							if (link.getEndNode() === shape) {
								this._linkDispose(link);
							}
						}
					}
					// console.info("newItems.length=" + newItems.length);
					// Add links between this shape and the new item					
					for (var i = 0; i < newItems.length; i++) {
						this._connectItems(newItems[i], modifiedItem, true);
						this._connectItems(modifiedItem, newItems[i], false);
					}
					
				}
				
				// Update Node label
				// check if the updated attribute is involved in the label
				var attr = this._nodesStore.getLabelAttributes(modifiedItem);
				if (attr) {
					for (var i = 0; i < attr.length; ++i) {
						if (attr[i] === attribute) {
							var label = this._nodesStore.getLabel(modifiedItem);
							shape.setLabel(newValue);
							break;
						}
					}
				}
				
				// Update Node position
				this._applyNodeStoreItemGraphics(shape);
			}
		},
		
		_onSaveNodesStore: function() {
			// summary:
			//     called after the nodesStore has been saved.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSaveNodesStore");
			this.reset();
			this._load();
		},
		
		_onRevertNodesStore: function() {
			// summary:
			//     called after the nodesStore has reverted back to the last "saved" state.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onRevertNodesStore");
			this.reset();
			this._load();
		},
		
		// ===============================================================
		// LinkStore Notification-related functions
		// ===============================================================
		
		_wireLinksStore: function() {
			// summary: 
			//     If the nodesStore supports notification, register to be informed of any changes
			// tags:
			//	    private    
			if (this._linksStore){
				var features =this._linksStore.getFeatures();
				if (features['dojo.data.api.Notification']) {
					this._linksStoreConnectHandles = new dojox.collections.ArrayList();
					this._linksStoreConnectHandles.add(dojo.connect(this._linksStore, "onNew", this, "_onNewLinkItem"));
					this._linksStoreConnectHandles.add(dojo.connect(this._linksStore, "onSet", this, "_onSetLinkItem"));
					this._linksStoreConnectHandles.add(dojo.connect(this._linksStore, "onDelete", this, "_onDeleteLinkItem"));
				}
				
				if (features['dojo.data.api.Write']) {
					this._linksStoreConnectHandles.add(dojo.connect(this._linksStore, "save", this, "_onSaveLinksStore"));
					this._linksStoreConnectHandles.add(dojo.connect(this._linksStore, "revert", this, "_onRevertLinksStore"));
					this._onLinkShapeHandlers = {};
				}
			}
		},
		
		_unwireLinksStore: function() {
			// summary:
			//     unregister any nodesStore notifications we may have set.
			// tags:
			//	    private 
			if (this._linksStoreConnectHandles && this._linksStoreConnectHandles.count > 0) {
				this._linksStoreConnectHandles.forEach(function(entry) {
					dojo.disconnect(entry);
				}, this);
				this._linksStoreConnectHandles = null;
			}
			this._onLinkShapeHandlers = null;
		},
		
		_onNewLinkItem: function(newitem, parentInfo) {
			// summary:
			//     called when a new item added to the linksStore.
			// tags:
			//	    private 
			var id = this._linksStore.getIdentity(newitem);
			// console.info("DataConnector._onNewLinkItem id=" + id + " parentInfo=" + parentInfo);
			var linkEntry = this._addLinkStoreItemToModel(newitem);
			linkEntry.startId = this._getStartLinkItemId(newitem);
			linkEntry.endId = this._getEndLinkItemId(newitem);
			if (linkEntry.startId !== null && linkEntry.endId !== null) {
				linkEntry.start = this._model.getEntry(linkEntry.startId);
				linkEntry.end = this._model.getEntry(linkEntry.endId);
				if (linkEntry.start && linkEntry.end) {
					linkEntry.pending = false;
					linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, newitem);
					this._applyLinkStoreItemGraphics(linkEntry.link);
				} else {
					linkEntry.pending = true;
				}
			}
		},
		
		_onDeleteLinkItem: function(item) {
			// summary:
			//     called when an item is deleted from the linksStore.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onDeleteLinkItem id=" + id);
			var link = this._getLinkFromStoreItem(item);
			this._linkDispose(link, item);
		},
		
		_onSetLinkItem: function(item, attribute, oldValue, newValue) {
			// summary:
			//     called when an item in linksStore changes. 
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSetLinkItem id=" + this._linksStore.getIdentity(item) + ",attribute=" + attribute + ",old=" + oldValue + ",new=" + newValue);
			
			if (this._internalSet) 
				return;
			
			var id = this._linksStore.getIdentity(item);
			var linkEntry = this._model.getLinkEntry(id);
			if (linkEntry) {
				var oldStartLinkItemId = linkEntry.startId;
				var oldEndLinkItemId = linkEntry.endId;
				var newStartLinkItemId = this._getStartLinkItemId(item);
				var newEndLinkItemId = this._getEndLinkItemId(item);
				
				if ((oldStartLinkItemId !== newStartLinkItemId) || (oldEndLinkItemId !== newEndLinkItemId)) {
					// The start and/or end point for this link has been changed
					this._linkDispose(linkEntry.link, item);
					linkEntry.link = null;
					
					if (oldStartLinkItemId !== newStartLinkItemId) {
						linkEntry.startId = newStartLinkItemId;
						linkEntry.start = this._model.getEntry(newStartLinkItemId);
					}
					if (oldEndLinkItemId !== newEndLinkItemId) {
						linkEntry.endId = newEndLinkItemId;
						linkEntry.end = this._model.getEntry(newEndLinkItemId);
					}
					
					if (linkEntry.start && linkEntry.end) {
						linkEntry.pending = false;
						linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, item);
						this._applyLinkStoreItemGraphics(linkEntry.link);
					} else {
						linkEntry.pending = true;
					}
				}
			}
		},
		
		_onSaveLinksStore: function() {
			// summary:
			//     called after the linksStore has been saved.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSaveLinksStore");
			this.reset();
			this._load();
		},
		
		_onRevertLinksStore: function() {
			// summary:
			//     called after the linksStore has reverted back to the last "saved" state.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onRevertLinksStore");
			this.reset();
			this._load();
		},		
		
		// a simple hashtable (ie {} ) for invalidateLinkShape connect handlers
		_onLinkShapeHandlers : null,
		
		_addLinkShapeHandler: function(link, handler){			
			var h = dojo.connect(link, 'invalidateLinkShape', this, handler);
			this._onLinkShapeHandlers[link.getId()] = h;
		},
		
		_clearLinkShapeHandlers:function(link) {
			if (!this._onLinkShapeHandlers)
				return;
			var id = link.getId(), h = this._onLinkShapeHandlers[id];
			if (h)
				dojo.disconnect(h);
			delete this._onNodeChangedHandlers[id];
		},
		
		// ===============================================================
		//  Function to extract/store graphical information in the Links 
		//  datastore itself
		// ===============================================================
		_applyLinkStoreItemGraphics: function(link) {
			// summary:
			//     Applies any graphical information to a node/subgraph
			//     that is stored in the nodesStore item, specifically..
			//     (a) TBD from the specified function
			//     (b) x/y positioning.
			// tags:
			//	    private 
			
			if (link.getData) {
			
				var item = link.getData().item;
				
				if (this._linksGraphBindingFunction) {
					var gfxInfo = this._linksGraphBindingFunction.call(this, this._linksStore, item);
					if (gfxInfo) {
						this._applyProperties(link, gfxInfo);
					}
					
					// If store supports write then monitor link for changes
					if (this._linksStore.getFeatures()['dojo.data.api.Write']) {
						this._addLinkShapeHandler(link, function() {
							if (this._linksGraphBindingFunction && link.getData()) {
								var litem = link.getData().item;
								if (litem && this._linksStore.isItem(litem)) {
									try {
										this._internalSet = true;
										var gfxInfo = this._extractProperties(link, this._linksGraphProperties);
										this._linksGraphBindingFunction.call(this, this._linksStore, litem, gfxInfo);
									} finally {
										this._internalSet = false;
									}
								}
							}
						});
					}
				}
			}
		},
		
		// ===============================================================
		// Graphics-properties-related functions
		// ===============================================================
		
		_getPropFuncMap: function(o, proplist) {
			// summary: 
			//     For a particular list of properties return a map 
			//     of property names to function names
			// tags:
			//	    private         
			var funcMap = o._isIBMDiagramNode ? this._nodesGraphFuncMap : this._linksGraphFuncMap;
			if (!funcMap) {
				var props = proplist.split(",");
				funcMap = {};
				for (var i = 0; i < props.length; i++) {
					var getName = "get" + props[i].charAt(0).toUpperCase() + props[i].substring(1);
					if (o[getName] && (o[getName] instanceof Function)) {
						funcMap[props[i]] = getName;
					} else {
						var isName = "is" + props[i].charAt(0).toUpperCase() + props[i].substring(1);
						if (o[isName] && (o[isName] instanceof Function)) {
							funcMap[props[i]] = isName;
						}
					}
				}
				
				if (o._isIBMDiagramNode) 
					this._nodesGraphFuncMap = funcMap;
				else 
					this._linksGraphFuncMap = funcMap;
			}
			return funcMap;
		},
		
		_extractProperties: function(o, props) {
			// summary: 
			//     Extracts a subset of properties of an object, given a 
			//     list of getter functions. Function names are assumed to
			//     be of the form isXXX or getXXX
			// tags:
			//	    private 
			var extracted = {};
			
			var funcMap = this._getPropFuncMap(o, props);
			for (var pname in funcMap) {
				extracted[pname] = o[funcMap[pname]]();
			}
			return extracted;
		},
		
		_applyProperties: function(o, props) {
			// summary: 
			//     Applies properties to an object. Setter functions are 
			//     determined, based on the property names.
			//     for property name xxxx, setter function is setXxxx.
			// tags:
			//	    private 
			for (var prop in props) {
				if (props.hasOwnProperty(prop)) {
					var setfunc = 'set' + prop.charAt(0).toUpperCase() + prop.substring(1);
					if (o[setfunc] && (o[setfunc] instanceof Function)) {
						o[setfunc](props[prop]);
					}
				}
			}
		},
		
		// ===============================================================
		// Model-related functions
		// ===============================================================
		
		_addStoreItemToModel: function(item) {
			// summary: 
			//     Add a store item into the model.
			// tags:
			//	    private 
			
			var id = this._nodesStore.getIdentity(item);
			
			var entry = this._model.getEntry(id);
			
			if (!entry) {
				entry = this._model.addEntry(id);
			}
			this._model.setEntryItem(entry, item);
			this._completeModelConstruction();
			
			return entry;
		},
		
		_addLinkStoreItemToModel: function(item) {
			// summary: 
			//     Add a store item into the model.
			// tags:
			//	    private 
			
			var id = this._linksStore.getIdentity(item);
			
			var entry = this._model.getLinkEntry(id);
			
			if (!entry) {
				entry = this._model.addLinkEntry(id);
			}
			this._model.setLinkEntryItem(entry, item);
			return entry;
		},
		
		_completeModelConstruction: function() {
			// summary: 
			//     Called after the all model entries have been created
			// tags:
			//	    private 
			
			// When all model entries have been creates and their items loaded..            
			if (this._model.isModelComplete()) {
				//                
				// Assumptions made for hierarchies
				// - An entry with an empty to-list is a leaf node 
				// - The from-list length will by not longer than 1
				
				this._model.forEachEntry(function(entry) {
					var childlist = entry.childlist;
					var parentlist = entry.parentlist;
					if (childlist.length > 0) {
					
						for (var t = 0; t < childlist.length; t++) {
							var childEntry = this._model.getEntry(childlist[t]);
							childEntry.parent = entry;
							entry.children.push(childEntry);
						}
					}
					if (parentlist.length === 1) {
						var parentEntry = this._model.getEntry(parentlist[0]);
						entry.parent = parentEntry;
						parentEntry.children.push(entry);
					}
				}, this);
				
				this._onModelCreated(this._model);
				this._createShapesFromModel();
			}
		},
		
		_onModelCreated: function(/* ibm_ilog.diagram.widget._DataModel */model) {
			// summary: 
			//     Function which is called when the model is complete.
			// tags:
			//	    private 
		
			// model.dumpToConsole();
		},
		
		_resolveConnectionsFromNodesStore: function() {
			// summary: 
			//     Attempt to resolve (discover relationship information for) 
			//     nodesStore items. Add information about the item relationships
			//     into the model. Repeat until all are resolved.
			//     Note that item-connecting information can be provided either
			//     by registered binding functions or from the link store.
			// tags:
			//	    private        
			var unresolved = [];
			
			this._model.forEachUnresolvedEntry(function(entry) {
			
				var currentItem = entry.item;
				
				if (this._parentBindingFunction) {
					var connectingItems = this._parentBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						
						if (this._model.containsEntry(ident)) {
							entry.parentlist.push(ident);
						}
					}, this);
				}
				
				if (this._childBindingFunction) {
					var connectingItems = this._childBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.childlist.push(ident);
							
						}
					}, this);
				}
				
				if (this._successorsBindingFunction) {
					var connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.tolist.push(ident);
						}
					}, this);
				}
				
				if (this._predecessorsBindingFunction) {
					var connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, currentItem);
					dojo.forEach(connectingItems, function(connectingItem) {
						var ident = this._nodesStore.getIdentity(connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.fromlist.push(ident);
						}
					}, this);
				}
				
				entry.resolved = true;
			}, this);
			
			// If all items are resolved and the model is fully populated, otherwise add 
			// new entries in the model 
			if (unresolved.length === 0) {
				this._model.setAllEntriesAdded();
				this._completeModelConstruction();
			} else {
				for (var i = 0; i < unresolved.length; i++) {
					if (this._nodesStore.isItemLoaded(unresolved[i])) {
						this._addStoreItemToModel(unresolved[i]);
					} else {
						this._loadNodesStoreItem(unresolved[i]);
					}
				}
				this._resolveConnectionsFromNodesStore();
			}
		},
		
		// ===============================================================
		// Shape mapping functions
		// ===============================================================
		
		_getNodeFromStoreItem: function(item) {
			// summary:
			//        Gets the GFX object, for the given Node Data Item.
			// tags:
			//	    private 
			
			var node = null;
			var id = this._nodesStore.getIdentity(item);
			var nodeModelEntry = this._model.getEntry(id);
			if (nodeModelEntry) {
				node = nodeModelEntry.node;
			}
			return node;
		},
		
		_setNodeFromStoreItem: function(item, node) {
			// summary:
			//        Sets the GFX object, associating it with the given Node Data Item.
			// tags:
			//	    private 
			
			var id = this._nodesStore.getIdentity(item);
			var nodeModelEntry = this._model.getEntry(id);
			if (nodeModelEntry) {
				nodeModelEntry.node = node;
			}
		},
		
		
		_getLinkFromStoreItem: function(item) {
			// summary:
			//        Gets the GFX object, for the given Link Data Item.
			// tags:
			//	    private 
			var link = null;
			var id = this._linksStore.getIdentity(item);
			var linkModelEntry = this._model.getLinkEntry(id);
			if (linkModelEntry) {
				link = linkModelEntry.link;
			}
			return link;
		},
		
		// ===============================================================
		// Nodes, Subgraphs and Links creation
		// ===============================================================
		_createLinksFromModel: function() {
			// summary: 
			//     Based on options provided, create explicit links objects
			//     associated with the nodeStore items and their
			//     relationships.
			// tags:
			//	    private 
			
			// Create links from lists in the node model 
			this._model.forEachEntry(function(entry) {
				var currentNode;
				var childlist = entry.childlist;
				var parentlist = entry.parentlist;
				var tolist = entry.tolist;
				var fromlist = entry.fromlist;
				if (this._createLinksForHierarchy) {
				
					if (childlist.length + parentlist.length > 0) {
						currentNode = entry.node;
						var link;
						for (var t = 0; t < childlist.length; t++) {
							var childNode = this._model.getEntry(childlist[t]).node;
							link = this.connectNodes(currentNode, childNode);
						}
						for (var f = 0; f < parentlist.length; f++) {
							var parentNode = this._model.getEntry(parentlist[f]).node;
							link = this.connectNodes(parentNode, currentNode);
						}
					}
				}
				if (tolist.length + fromlist.length > 0) {
					currentNode = entry.node;
					var link;
					for (var t = 0; t < tolist.length; t++) {
						var toNode = this._model.getEntry(tolist[t]).node;
						link = this.connectNodes(currentNode, toNode);
					}
					for (var f = 0; f < fromlist.length; f++) {
						var fromNode = this._model.getEntry(fromlist[f]).node;
						link = this.connectNodes(fromNode, currentNode);
					}
				}
			}, this);
			
			// Create links from lists in the link model 
			this._model.forEachCompleteLinkEntry(function(entry) {
				entry.link = this.connectNodes(entry.start.node, entry.end.node, null, entry.item);
				this._applyLinkStoreItemGraphics(entry.link);
			}, this);
			
		},
		
		_createShapesFromModel: function() {
			// summary: 
			//     Based on options provided, create graphical objects
			//     associated with the nodeStore items and their
			//     relationships.
			// tags:
			//	    private 
			if (this._createLinksForHierarchy) {
				// Create nodes
				this._model.forEachEntry(function(entry) {
					entry.node = this.createNode(null, entry.item);
					this._applyNodeStoreItemGraphics(entry.node);
				}, this);
				this._createLinksFromModel();
			} else {
				// Create subgraphs/nodes
				this._model.forEachTopEntry(function(entry) {
					if (entry.isLeaf()) {
						entry.node = this.createNode(null, entry.item);
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						entry.node = this.createSubgraph(null, entry.item);
						this.createChildGraphics(entry);
						this._applyNodeStoreItemGraphics(entry.node);
					}
				}, this);
				this._createLinksFromModel();
			}
			
			this.onLoaded();
		},
		
		createChildGraphics: function(parent) {
			// summary: 
			//     For hierarchical data, recursively go through the tree
			//     and create subgraphs and nodes accordingly
			// tags:
			//	    private         
			var children = parent.children;
			for (var i = 0; i < children.length; i++) {
				var child = children[i];
				if (child.isLeaf()) {
					child.node = this.createNode(null, child.item, parent.node.getGraph());
					this._applyNodeStoreItemGraphics(child.node);
				} else {
					child.node = this.createSubgraph(null, child.item, parent.node.getGraph());
				}
				
				this.createChildGraphics(child);
			}
			this._applyNodeStoreItemGraphics(parent.node);
		},
		
		// ===============================================================
		// Nodes and Links disposal
		// ===============================================================
		_nodeDispose: function(node, item) {
			// summary:
			//     dispose of a node. 
			// tags:
			//	    private 
			
			if (node) {
				this._clearOnChangedHandlers(node);
				node.removeShape();
				node.dispose();
			}
		},
		
		_linkDispose: function(link, item) {
			// summary:
			//     dispose of a link. 
			// tags:
			//	    private 
			
			if (link) {
				this._clearLinkShapeHandlers(link);
				link.removeShape();
				link.dispose();
			}
		}
		
	});
	
	//
	//	ibm_ilog.diagram.widget.DataConnector.StartLinkFieldname: String
	//		The name of a field in a linksStore item which defines the start node of a link
	//      The value associated with this field is a String which represents the 
	//      identity of an item in the nodesStore.
	//
	ibm_ilog.diagram.widget.DataConnector.StartLinkFieldname = "start";
	
	//
	//	ibm_ilog.diagram.widget.DataConnector.EndLinkFieldname: String
	//		The name of a field in a linksStore item which defines the end node of  a link
	//      The value associated with this field is a String which represents the 
	//      identity of an item in the nodesStore.
	//
	ibm_ilog.diagram.widget.DataConnector.EndLinkFieldname = "end";
	
	dojo.declare('ibm_ilog.diagram.widget._DataModel', null, {
		//
		//	summary:
		//
		//		A _DataModel is a collection of _NodeModelEntry items.
		//
		//	description:
		//		
		//		The _DataModel is a half-way house between the store and canvas.
		// tags:
		//	    private 
		
		// nmodel : dojox.collections.Dictionary
		//          A collection of entries, each representing a nodesStore item, associated 
		//          graphical objects and its relationships between other nodes
		//          It is not considered complete until all entries have been added
		//          and the item for each entry has loaded, I.e. no pending loads  
		nmodel: null,
		
		// lmodel : dojox.collections.Dictionary
		//          A collection of entries, each representing a linksStore item, associated 
		//          graphical objects  
		lmodel: null,
		
		// pendingLoads : Integer
		//                Number of model entries which don't have associated items 
		pendingLoads: 0,
		
		// allEntriesAdded : Boolean
		//                When true, all datastores have been parsed and model entries
		//                have been created.
		
		allEntriesAdded: false,
		
		// initialLoadComplete : Boolean
		//                When true, it means all items loaded and associated graphics
		//                have been created.
		
		initialLoadComplete: false,
		
		constructor: function() {
			// summary:
			//     Creates an instance of a data model
			//
			// tags:
			//	    private 
			this.nmodel = new dojox.collections.Dictionary();
			this.lmodel = new dojox.collections.Dictionary();
			this.init();
		},
		
		init: function() {
			// tags:
			//	    private 
			
			this.initNodes();
			this.initLinks();
			
			
			this.pendingLoads = 0;
			
			
		},
		
		initNodes: function() {
			// tags:
			//	    private 
			dojo.forEach(this.nmodel.getKeyList(), function(key) {
				this.nmodel.remove(key);
			}, this);
			this.nmodel.clear();
			this.allEntriesAdded = false;
			this.initialLoadComplete = false;
		},
		
		initLinks: function() {
			// tags:
			//	    private 
			dojo.forEach(this.lmodel.getKeyList(), function(key) {
				this.lmodel.remove(key);
			}, this);
			this.lmodel.clear();
		},
		
		isModelComplete: function() {
			// summary:
			//     Check if the model is "complete" - I.e. all entries have been added
			//     and each entry has an associated item. 
			//     We only want this to return true for the first call.
			// tags:
			//	    private 
			var isComplete = (this.pendingLoads === 0 && this.allEntriesAdded && !this.initialLoadComplete);
			if (isComplete) {
				this.initialLoadComplete = true;
			}
			return isComplete;
		},
		
		setAllEntriesAdded: function() {
			// summary:
			//     Indicate to the model that all entries have been added.
			// tags:
			//	    private 
			this.allEntriesAdded = true;
		},
		
		incrPendingLoads: function() {
			// summary:
			//     Increase the count of item loads pending.
			// tags:
			//	    private 
			this.pendingLoads++;
		},
		
		decrPendingLoads: function() {
			// summary:
			//     Decrease the count of item loads pending.
			// tags:
			//	    private            
			this.pendingLoads--;
		},
		
		clear: function() {
			// summary:
			//     Clears the data model
			// tags:
			//	    private 
			this.init();
		},
		
		getEntry: function(id) {
			// summary:
			//     Get an entry, based on the nodeStore Identity
			// tags:
			//	    private         
			return this.nmodel.item(id);
		},
		
		getLinkEntry: function(id) {
			// summary:
			//     Get an entry, based on the linksStore Identity
			// tags:
			//	    private         
			return this.lmodel.item(id);
		},
		
		containsEntry: function(id) {
			// summary:
			//     Enquire if there is an entry in the data model which relates
			//     to a specified nodeStore Identity
			// tags:
			//	    private  
			return this.nmodel.containsKey(id);
		},
		
		addEntry: function(id) {
			// summary:
			//     Creates a _NodeModelEntry, associates it with a specified
			//     nodeStore Identity then adds it to the model
			// tags:
			//	    private    
			var entry = new ibm_ilog.diagram.widget._NodeModelEntry();
			
			this.nmodel.add(id, entry);
			entry.id = id;
			
			// Track the number of item loads pending
			this.incrPendingLoads();
			return entry;
		},
		
		removeEntry: function(id) {
			// summary:
			//     Removes the _NodeModelEntry, associated with a specified
			//     nodeStore Identity, from model
			// tags:
			//	    private    
			var entry = this.getEntry(id);
			this.nmodel.remove(id);
		},
		
		addLinkEntry: function(id) {
			// summary:
			//     Creates a _LinkModelEntry, associates it with a specified
			//     linkStore Identity then adds it to the model
			// tags:
			//	    private    
			var entry = new ibm_ilog.diagram.widget._LinkModelEntry();
			
			this.lmodel.add(id, entry);
			entry.id = id;
			return entry;
		},
		
		removeLinkEntry: function(id) {
			// summary:
			//     Removes the _LinkModelEntry, associated with a specified
			//     linkStore Identity, from model
			// tags:
			//	    private    
			var entry = this.getLinkEntry(id);
			this.lmodel.remove(id);
		},
		
		
		
		setEntryItem: function(entry, item) {
			// summary:
			//     Associates and entry with a datastore item
			//     and tracks the number of item loads pending.   
			//     IT IS VERY IMPORTANT THAT setEntryItem (rather than entry.item = item;)
			//     IS USED SO THE MODEL CAN KEEP TRACK OF WHICH ENTRIES DON'T HAVE ITEMS
			// tags:
			//	    private         
			entry.item = item;
			this.decrPendingLoads();
		},
		
		setLinkEntryItem: function(entry, item) {
			// summary:
			//     Associates and entry with a datastore item
			// tags:
			//	    private     
			entry.item = item;
		},
		
		forEachLinkEntry: function(func, scope) {
			// summary:
			//     Loops through each link entry in the model, calling a named function, in
			//     the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				func.call(scope, e.element.value);
			}
		},
		
		forEachCompleteLinkEntry: function(func, scope) {
			// summary:
			//     Loops through each link entry in the model which is not 'pending', calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.pending) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		forEachPendingLinkEntry: function(func, nodeEntry, scope) {
			// summary:
			//     Loops through each link entry in the model which is 'pending', calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				if (e.element.value.pending) {
					func.call(scope, e.element.value, nodeEntry);
				}
			}
		},
		
		forEachEntry: function(func, scope) {
			// summary:
			//     Loops through each entry in the model, calling a named function, in
			//     the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				func.call(scope, e.element.value);
			}
		},
		
		forEachTopEntry: function(func, scope) {
			// summary:
			//     Loops through each entry in the model which have no parents, 
			//     calling a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.parent) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		forEachUnresolvedEntry: function(func, scope) {
			// summary:
			//     Loops through each entry in the model which is not marked as resolved, calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.resolved) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		dumpToConsole: function() {
			// summary:
			//     Debug facility to dump the contents of the data model to the console.
			// tags:
			//	    private 
			
			// console.info("Model dump =================================Nodes and their relationships");
			
			this.forEachEntry(function(entry) {
				var leafStatus = entry.isLeaf() ? "(+leaf)" : "";
				var itemStatus = entry.item ? "(+item)" : "";
				var resolvedStatus = entry.resolved ? "(+resolved)" : "";
				// console.info("Item - " + entry.id + " " + itemStatus + " " + resolvedStatus + " " + leafStatus);
				var childlist = entry.childlist;
				for (var t = 0; t < childlist.length; t++) {
					// console.info("       Children : " + childlist[t]);
				}
				var parentlist = entry.parentlist;
				for (var f = 0; f < parentlist.length; f++) {
					// console.info("       Parent   : " + parentlist[f]);
				}
				
				var tolist = entry.tolist;
				for (var t = 0; t < tolist.length; t++) {
					// console.info("       To       : " + tolist[t]);
				}
				var fromlist = entry.fromlist;
				for (var f = 0; f < fromlist.length; f++) {
					// console.info("       From     : " + fromlist[f]);
				}
			}, this);
			
			// console.info("Model dump =================================Links");
			this.forEachLinkEntry(function(entry) {
				// console.info("Item - " + entry.id);
			}, this);
		}
	});
	
	dojo.declare('ibm_ilog.diagram.widget._NodeModelEntry', null, {
		//
		//	summary:
		//
		//		A DataModelEntry collects attributes related to a nodesStore item.
		//
		//	description:
		//		
		//		When completely populated, it forms a logical representation of a
		//      nodeStore item and its relationship with other items.
		
		id: "",
		item: null,
		node: null,
		resolved: false,
		childlist: [],
		parentlist: [],
		tolist: [],
		fromlist: [],
		parent: null,
		children: [],
		
		
		constructor: function() {
			// summary:
			//     Creates an instance of a data model entry.
			// tags:
			//	    private 
			this.id = "";
			this.item = null;
			this.node = null;
			this.resolved = false;
			this.childlist = [];
			this.parentlist = [];
			this.fromlist = [];
			this.tolist = [];
			this.parent = null;
			this.children = [];
		},
		
		isLeaf: function() {
			return (this.children.length === 0);
		}
		
	});
	
	dojo.declare('ibm_ilog.diagram.widget._LinkModelEntry', null, {
		//
		//	summary:
		//
		//		A DataModelEntry collects attributes related to a linksStore item.
		//
		//	description:
		//		
		//		When completely populated, it forms a logical representation of a
		//      nodeStore item and its relationship with other items.
		
		id: "",
		item: null,
		link: null,
		startId: null,
		start: null,
		endId: null,
		end: null,
		pending: false,
		
		constructor: function() {
			// summary:
			//     Creates an instance of a data model entry.
			// tags:
			//	    private 
			this.id = "";
			this.item = null;
			this.link = null;
			this.startId = null;
			this.start = null;
			this.endId = null;
			this.end = null;
			this.pending = false;
		}
		
	});
	
})();



}

if(!dojo._hasResource['ibm_ilog.diagram.util.Assert']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.Assert'] = true;
dojo.provide('ibm_ilog.diagram.util.Assert');



(function(){
	
	var a = ibm_ilog.diagram.util.Assert;

	// TODO comment.
	//
	//	Check Viewport.js for example.
	//
	//	The check and assert functions must be replaced but null functions for release deployments
	//
	//	Note: can't reference local vars, only vars through "this".
	
	if(typeof(djConfig) !== 'undefined' && djConfig.isDebug && dojo.isMozilla) {
	
		//
		// Tested Ok on Mozilla.
		//
		// Does not work on Chrome due to this chrome behavior:
		//
		//	 eval("this") -> this
		//	 check = eval;
		//	 check("this") -> window
		//

		a.check = eval;
		
		a.buildPredicate = function( methodName, e, args ) {
			var r = ""+ 
				"(dojo.hitch(this, function(r){" + 
					"if (!r) {" +
					"var R = ibm_ilog.diagram.util.ErrorReporter;" + 
						"R."+methodName+".apply(R,Array.prototype.slice.call(arguments,1));"+
					"}" +
					"return r;" +
				"}, ("+e+")";
			
			for(var i=1;i<args.length;i++) {
				r+= ", \"" + args[i].replace(/"/g, "\\\"") + "\"";
			}
	
			r+="))();";
	 
			return r;
		}
		
		a.assert = function(e) {
			return a.buildPredicate("error",e,arguments);
		};

		a.verify = function(e) {
			return a.buildPredicate("warn",e,arguments);
		};
		
	} else {
		//
		// Disable everything on debug
		//
		a.check = a.assert = function(){};
	}
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.GlobalInitialization']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.GlobalInitialization'] = true;
dojo.provide('ibm_ilog.diagram.util.GlobalInitialization');

(function(){
	
	var gi = ibm_ilog.diagram.util.GlobalInitialization;
	
	gi._initializing = 0;
	
	gi.then = function(cb) {
		//
		//	summary:
		//		Executes cb after all registered entities have initialized.
		//
		if(!gi._initializing) {
			cb();
		} else {
			var h = dojo.connect(gi,"_finishedInit",function(){
				cb();
				dojo.disconnect(h);
			});
		}
	};
	
	gi.beginInit = function() {
		//
		//	summary:
		//		Notify an entity begins initialization.
		//
		
		gi._initializing++;
	};
	
	gi._finishedInit = function() {

	};

	gi.endInit = function() {
		//
		//	summary:
		//		Notify an entity just finished initialization.
		//
		gi._initializing--;
		if(!gi._initializing) {
			gi._finishedInit();
		}
	};
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.DeferredInitialization']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.DeferredInitialization'] = true;
dojo.provide('ibm_ilog.diagram.util.DeferredInitialization');



(function(){
	
	var gi = ibm_ilog.diagram.util.GlobalInitialization;
	
	dojo.declare("ibm_ilog.diagram.util.DeferredInitialization",[],{
		//
		//	summary:
		//		Provide the users of the class with a whenLoaded(cb) API for
		//		waiting for instances to finish initialization. Also subscribes
		//		the instance to the global initialization coordinator for global
		//		synchrinzation.
		//
		__initialized: false,
		
		_beginInit: function() {
			//
			//	summary:
			//		Begins initialization process.
			//
			this.__initialized = false;
	    	gi.beginInit();
	    },
	    
	    _endInit: function() {
			//
			//	summary:
			//		Finishes initialization process, subsequently executing all 
	    	//		registered callbacks.
			//
			this.__initialized = true;
	    	gi.endInit();
	    },
        
        isInitialized: function(){
            // summary:
            //     Returns whether the initialization process is done.
            return this.__initialized;
        },
	    
	    _defaultErrorHandler: function() {
	    	console.log(arguments);
	    },
	    
	    whenLoaded: function(cb,eb) {
			//
			//	summary:
			//		Registers a callback to be executed after the instance has 
	    	//		been initialized, or executes immediately if already initialized.
			//
	    	if(this.__initialized) {
	    		cb();
	    	} else {
		    	var h = dojo.connect(this,"_endInit",function(){
		    		cb();
		    		dojo.disconnect(h);
		    	});
	    	}
	    	return this;
	    }
    
	});
	
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.Viewport']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.Viewport'] = true;
dojo.provide('ibm_ilog.diagram.widget.Viewport');









(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var CHECK = ibm_ilog.diagram.util.Assert.check,	assert = ibm_ilog.diagram.util.Assert.assert;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect,	Size = g.Size,	PointZero = g.PointZero,	Point = g.Point;

	dojo.declare('ibm_ilog.diagram.widget.Viewport', [ibm_ilog.diagram.util.DeferredInitialization], {
		//	summary:
		//		A viewport window to a rectangular area of the diagram that is drawn on the browser.
		//		The drawn area is usually controlled by two scrollbars, horizontal and vertical.
		//
		//	description:
		//
		//		The Viewport class is an abstract class that provides basic infrastructure over which
		//		to construct concrete Diagram viewports.
		//
		//		A Viewport object is a collaborator of the Diagram widget that abstracts this notion 
		//		of viewport. The Diagram widget can then deal with the the viewport independently of 
		//		how it is actually implemented. 
		//
		//		The main responsibility of the viewport object is to control the current viewable 
		//		rectangle of the Diagram and report updates to it.
		//	
		//		The following concept definitions help understand the Viewport API:
		//			
		//		-	Content Rectangle    The area inside which the viewport can move, mostly based 
		//								 on the content bounding box.
		//
		//		-	View Rectangle       The rectangular portion of the content that is currently 
		//			                     visible in the viewport.
		//
		//		-	Client Size          The size of the UI space that allows the contents to be 
		//			                     viewed.
		//
		//	authors: AV

		// ========================================================================================================
		//
		//	BASIC COLLABORATORS AND INITIALIZATION:
		//
		//	
		//
		//
		//	_diagram:
		//		The associated Diagram widget.
		//
		_diagram: null,

		//
		//	_viewportMatrix:
		//		The transformation matrix applied to the content contents for displaying the in the
		//		viewport. Used to transform model coordinates to view coordinates.
		//
		_viewportMatrix: dojox.gfx.matrix.identity,

		//
		//	_viewportMatrix:
		//		The inverse of the transformation matrix applied to the content contents for 
		//		displaying the in the viewport. Used to transform view coords into model coords.
		//
		_viewportMatrixInverse: dojox.gfx.matrix.identity,


		templateString: function () {
			//
			//	summary:
			//		This method is must return the template that the Diagram widget must use 
			//		in order to use the receiver as viewport.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			R.error("SubclassResponsibility","templateString");
		},

		//
		//	_needsHudRelayout:
		//		Flag to indicate that we need to relayout the hud and probably resize the surface also.
		//
		_needsHudRelayout: false,

		initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
			//
			//	summary:
			//		Constructs a Viewport for collaborating with the diagram.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			this._beginInit();

			this._hudComponents = [];

			this._diagram = diagram;

			// FIXME: This line has been commented out as it causes a failure on Firefox 4 beta 2
			//CHECK(assert("this._diagram", M, B, "NullPointerException", "diagram", "initializing viewport"));

			this._createSurface();

			this._surface.whenLoaded(this, function () {
				this.setViewRect(this._clientSize, {instant:true});
				this._endInit();
			});
		},

		_getSurfaceSize: function () {
			return this.getHudSize();
		},

		_updateSurface: function () {
			var hz = this._getSurfaceSize();

			if (!dojo.isIE && !ibm_ilog.diagram.isCanvas) {
				this.getSurface().setDimensions(hz.width + "px", hz.height + "px");
			} else {
				this.getSurface().setDimensions(hz.width, hz.height);
			}
		},

		_eventBackground: null,


		_eventfulContent: null,

		getEventSourceBackground: function () {
			return this._underlayLayer;
		},

		getEventSource: function () {
			return this._eventfulContent;
		},

		_createEventSourceShape: function () {
			this._eventBackground = this._underlayLayer.createRect();
			this._eventBackground.setFill([0, 0, 0, 0.0001]);
			this._updateEventSourceShape();
		},

		getBackgroundShape: function() {
			return this._eventBackground;
		},

		_updateEventSourceShape: function () {
			//	
			//	Updates the event source shape size to the current viewport navigable area
			//
			if (this._eventBackground) {
				var s = this.getHudSize();
				s.width = s.width || 1; // bug in IE6 if width == 0
				s.height = s.height || 1; // bug in IE6 if height == 0
				this._eventBackground.setShape(s);
			}
		},

		eventClientLocation: function (e) {
			//
			//	Returns the client location of a mouse event generated by the event source
			//
			var r = dojo.coords(this._surface._parent, true);
			if ((e.type !== "undefined") && (e.type === "touchstart" ||
					e.type === "touchmove" ||
					e.type === "touchend" ||
					e.type === "gesturechange")){
				return {
					x: e.touches[0].pageX - r.x,
					y: e.touches[0].pageY - r.y
				};
					
				}
			return {
				x: e.pageX - r.x,
				y: e.pageY - r.y
			};
		},


		eventContentLocation: function (e) {
			//
			//	Returns the content location of a mouse event generated by the event source
			//
			return this.pointClientToContent(this.eventClientLocation(e));
		},

		//
		//	The GFX rendering surface
		//
		_surface: null,

		_computeInitialSize: function () {
			var box = dojo.contentBox(this._diagram.domNode); 
 			var parentBox = null; 
 			if (!box.w || !box.h) // only if needed 
   			parentBox = this._diagram.getParent() ? dojo.contentBox(this._diagram.getParent().containerNode) : {}; 
			
			// Note the fallback on the height of box for the width and on width of box for the height. 
			// This is intentional. Comment from Abel:
			// It's really a very improbable fallback case; where if only one axis is specified or 
			// can be inferred; then we assume the surface to be a square. You can remove it if you
			// prefer a less confusing logic there. This fallback case has never been used in 
			// any test/sample
			return Size(box.w || parentBox.w || box.h || 256, box.h || parentBox.h || box.w || 256);
		},

		_createSurface: function () {

			var csz = this._computeInitialSize();

			if (!dojo.isIE) {
				this._surface = dojox.gfx.createSurface(this._diagram.canvasNode, csz.width + "px", csz.height + "px");
			} else {
				this._surface = dojox.gfx.createSurface(this._diagram.canvasNode, csz.width, csz.height);
			}

			this._surface.whenLoaded(this, function () {

				//_surface
				//	_hudLayer                 <- top-level HUD components
				//	_eventfulContent          <- content event source
				//		_overlayLayer           <- non-transformed overlay shapes (adorners, marquee, etc.)
				//		_transformedLayer       <- viewport transformation
				//			_transformedOverlay   <- non-content overlays (effects, controls, etc.)
				//			_content              <- content (Graph)
				//			_transformedUnderlay  <- non-content underlays (background, backlights)
				//	  _underlayLayer          <- non-transformed underlay shapes (background, grid, etc.)
				//	

				this._eventfulContent = this._surface.createGroup();

				// the viewport node is used for transforming the content to be shown into the viewport
				this._transformedLayer = this._eventfulContent.createGroup();

				// the overlay layer is for drawing adorners or other event-requiring stuff
				this._overlayLayer = this._eventfulContent.createGroup();
				this._overlayLayer.moveToFront();

				// the ui node is for drawing content that is not affected by the viewport transform
				this._hudLayer = this._surface.createGroup();
				this._hudLayer.moveToFront();

				// the content node is for user content that is affected by the viewport transformation
				this._content = this._transformedLayer.createGroup();

				this._transformedOverlay = this._transformedLayer.createGroup();
				this._transformedOverlay.moveToFront();

				this._transformedUnderlay = this._transformedLayer.createGroup();
				this._transformedUnderlay.moveToBack();

				this._underlayLayer = this._eventfulContent.createGroup();
				this._underlayLayer.moveToBack();

				this._createEventSourceShape();

				dojo.connect(this._content, "_onChanged", this, "_onContentChangedNotification");

				this.setSize(csz);
			});
		},

		getSurface: function () {
			return this._surface;
		},

		getHudLayer: function () {
			return this._hudLayer;
		},

		getOverlayLayer: function () {
			return this._overlayLayer;
		},

		getUnderlayLayer: function () {
			return this._underlayLayer;
		},

		getContent: function () {
			return this._content;
		},
        
		_showOverlays:function(v){
			if (v && this._overlayLayer.getParent() == null) {
		    this._eventfulContent.add(this._overlayLayer);
		    this._overlayLayer.moveToFront();
			} else if (!v) {
				this._eventfulContent.remove(this._overlayLayer);
			}
		},

		_setViewportTransform: function ( /*dojox.gfx.matrix*/ zt) {
			//
			//	returns:
			//		the transformation used for zooming
			//
			//	tags:
			//		private
			//
			this._transformedLayer.setTransform(zt);
		},

		getViewportTransform: function () {
			return this._viewportMatrix;
		},

		getViewportInverseTransform: function () {
			return this._viewportMatrixInverse;
		},

		destroy: function () {
			//
			//	summary:
			//		Gracefully destroys the receiver
			//
			if (this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
			}
			if (this._surface) {
				this._surface.destroy();
			}
		},


		// ========================================================================================================
		//
		//	HUD (Heads-up display)
		//
		_hudComponents: null,

		//
		//	_hudEnabled: 
		//		True if hud is enabled
		//
		_hudEnabled: true,

		//
		//	_hudZoomingOverlay: 
		//		True if hud zooming overlay effect is enabled
		//
		_hudZoomingOverlay: false,

		//
		//
		//	_hudTargetView:
		//		The visible Rect representing the target viewport
		//
		_hudTargetView: null,

		registerHudComponent: function (component) {
			this._hudComponents.push(component);
		},

		setHudZoomFeedbackEnabled: function ( /*Boolean*/ b) {
			this._hudZoomingOverlay = b;
		},

		isHudZoomFeedbackEnabled: function () {
			return this._hudZoomingOverlay;
		},

		setHudEnabled: function ( /*Boolean*/ b) {
			if (b != this._hudEnabled) {
				if (b) {
					this._surface.add(this._hudLayer);
					this._hudLayer.moveToFront();
					this._hudEnabled = true;
					//this._activateHud();
				} else {
					//this._deactivateHud();
					this._hudEnabled = false;
					this._surface.remove(this._hudLayer);
				}
			}
			return this;
		},

		isHudEnabled: function () {
			return this._hudEnabled;
		},

		_activateHud: function () {
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(true);
			//		});
		},

		_deactivateHud: function () {
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(false);
			//		});
			//		if(this._hudTargetView) {
			//			this._hudLayer.remove(this._hudTargetView);
			//		}
		},

		_hudClearZoomingOverlay: function () {
			if (this._hudTargetView) {
				this._transformedOverlay.remove(this._hudTargetView);
			}
		},

		_hudCreateZoomingOverlay: function () {
			if (this._hudEnabled && this._hudZoomingOverlay && this._animOrigin.width !== this._animTarget.width) {
				var w = Math.min(this._animTarget.width, this._animTarget.height) * 0.10;
				var len = w * 0.50;
				var p = this._animTarget.scalePivot;
				var x1 = p.x - w;
				var x2 = p.x + w;
				var y1 = p.y - w;
				var y2 = p.y + w;

				this._hudTargetView = this._transformedOverlay.createGroup();
				var style = {
					color: "black",
					width: 1 / this._zoom
				};
				this._hudTargetView.createPolyline({
					points: [{
						x: x1,
						y: y1 + len
					}, {
						x: x1,
						y: y1
					}, {
						x: x1 + len,
						y: y1
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2 - len,
						y: y1
					}, {
						x: x2,
						y: y1
					}, {
						x: x2,
						y: y1 + len
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2,
						y: y2 - len
					}, {
						x: x2,
						y: y2
					}, {
						x: x2 - len,
						y: y2
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x1 + len,
						y: y2
					}, {
						x: x1,
						y: y2
					}, {
						x: x1,
						y: y2 - len
					}]
				}).setStroke(style);
			}
		},




		// ========================================================================================================
		//
		//	CONTENT RECTANGLE:
		//
		//	The area inside which the viewport can move, mostly based on the content bounding box.
		//
		//	The content rectangle (or navigable area) is computed from two values, the content 
		//	bounding box and the current client size, and it is obtained by the following method:
			//	A percentage of the current client size, defined by _airFactor is added to the
		//	borders of the content bounding box.
		//	
		//	The two main events that change the content rectangle are:
		//		- a change in the content
		//			(_onContentBoundingBoxChanged)
		//		- a change in the size of the viewport
		//			(setClientSize)

		//
		//	_contentBoundingBox:
		//		The area inside which we can move
		//
		_contentBoundingBox: EmptyRect,

		//
		//	_contentRect:
		//		The area inside which we can move
		//
		_contentRect: EmptyRect,

		//
		//	_airFactor:
		//		The minimum navigable space to the sides of the current content bounds as a percentage
		//		with respect to the current model bounding box.
		//
		_airFactor: 0.0,


		_getContentAir: function () {
			return g.sizeLongestSide(this._contentBoundingBox) * this._airFactor;
		},

		//
		//	_refreshContentsTimer:
		//		The timer scheduled by the refreshContentNotification request
		//
		_refreshContentsTimer: null,

		//
		//	_refreshContentsInterval:
		//		Refresh rate used for monitoring changes in the content.
		//
		_refreshContentsInterval: 50,

		setRefreshContentsSuspended: function( /*Boolean*/ suspended ) {
			if(this._refreshContentsSuspended !== suspended) {
				this._refreshContentsSuspended = suspended;
				if(suspended) {
					clearTimeout(this._refreshContentsTimer);
					this._refreshContentsTimer = null;
				} else {
					this._refreshContents();
				}
			}
			
		},
		
		getRefreshContentsSuspended: function() {
			return this._refreshContentsSuspended;
		},

		setRefreshContentsInverval: function ( /*Number*/ msInterval) {
			this._refreshContentsInterval = msInterval;
			if (msInterval < 0 && this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
				this._refreshContentsTimer = null;
			}
			return this;
		},

		getRefreshContentsInterval: function () {
			return this._refreshContentsInterval;
		},

		_onContentChangedNotification: function () {
			//
			//	summary:
			//		Sets up a timer to give the originating process time to finish with all the changes
			//
			if (!this._refreshContentsSuspended && this._refreshContentsInterval >= 0 && !this._refreshContentsTimer) {
				this._refreshContentsTimer =
				setTimeout(
						dojo.hitch(this, "_refreshContents", this._refreshContents), this._refreshContentsInterval);
			}
		},
		
		refresh: function() {
			//
			// 	summary:
			//		Forces a refresh contents procedure, where the content rect is updated and consequently 
			//		associated scrollbars and other features. 
			//
			this._refreshContents();
		},

		_refreshContents: function () {
			//
			//	summary:
			//		Callback executed after the contentChangedNotification timer is finished.
			//
			if(!this._refreshContentsSuspended) {
				this._refreshContentsTimer = null;
				this._onContentBoundingBoxChanged();
			}
		},

		setAirFactor: function ( /*Number*/ factor) {
			//
			//	summary:
			//		Sets the content air percentage with respect to the client size
			//
			this._airFactor = factor;

			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged();
			}
			return this;
		},


		getAirFactor: function () {
			//
			//	summary:
			//		Returns the content air percentage with respect to the client size
			//
			return this._airFactor;
		},


		getContentRect: function () {
			//
			//	summary:
			//		The current content size in pixels, which defines the available navigation area.
			//
			//	description:
			//		This value is updated by content BB change notifications (see _onContentBoundingBoxChanged)
			//
			//
			return this._contentRect;
		},


		_onContentBoundingBoxChanged: function () {
			//
			//	summary:
			//		The function is executed when the content of the content has changed.
			//
			//	returns:
			//		True if the change affected content bounding box.
			//
			//	tags:
			//		private
			//
			this._contentBoundingBox = this._content.getBoundingBox() || EmptyRect;

			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged()
			}
		},


		_computeContentRect: function () {
			//
			//	summary:
			//		Updates the navigable area
			//
			//	tags:
			//		private
			//
			var newContentRect = g.expandRect(this._contentBoundingBox, this._getContentAir());

			if (!g.equalRect(this._contentRect, newContentRect)) {
				this._contentRect = newContentRect;
				this.onContentRectChanged();
				return true;
			} else {
				return false;
			}
		},

		onContentRectChanged: function () {
			//
			//	summary:
			//		Event triggered when the content rect has changed.
			//
		},

		_constraintToContent: function ( /*dojox.gfx.Rectangle*/ r) {
			//
			//	summary:
			//		Applies a constraining policy on view rectangle r, returning the result.
			//
			//	tags:
			//		private
			//
			var cx = this._contentRect.x;
			var cy = this._contentRect.y;
			var cw = this._contentRect.width;
			var ch = this._contentRect.height;
			var w = r.width;
			var h = r.height;
			var x = r.x;
			var y = r.y;

			// Correct aspect ratio of r enlarging shorter sides
			var csz = this.getClientSize();
			var vrat = g.getRectRatio(csz);
			var rrat = g.getRectRatio(r);
			if (rrat > vrat) {
				h = w / vrat;
			} else if (rrat < vrat) {
				w = h * vrat;
			}

			var pivotdelta;
			if (this._minZoom > 0 && csz.width / w < this._minZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._minZoom) : {
					x: 0,
					y: 0
				};

				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;

				w = csz.width / this._minZoom;
				h = csz.height / this._minZoom;
			}

			if (this._maxZoom > 0 && csz.width / w > this._maxZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._maxZoom) : {
					x: 0,
					y: 0
				};

				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;

				w = csz.width / this._maxZoom;
				h = csz.height / this._maxZoom;
			}

			return {
				x: x,
				y: y,
				width: w,
				height: h,
				scalePivot: r.scalePivot
			};
		},

		// ========================================================================================================
		//
		//	CLIENT SIZE:
		//
		//		The size of the UI space that allows the contents to be viewed.
		//
		//
		//	_clientSize:
		//		The size of current viewport
		//
		_clientSize: EmptyRect,

		//
		//	_resizeMode: Number
		//		Current resize mode of the vieport, one of:
		//		| 1: Keep zoom level, expand area
		//		| 2: Keep viewable area, change zoom level. Longest side defines zoom level.
		//
		_resizeMode: 1,

		setResizeMode: function ( /*Number*/ m) {
			this._resizeMode = m;
		},

		getResizeMode: function () {
			return this._resizeMode;
		},

		getClientSize: function () {
			//
			//	summary:
			//		Returns the current viewport client size
			//
			return {
				width: this._clientSize.width,
				height: this._clientSize.height
			};
		},


		setClientSize: function ( /*Size*/ sz) {
			//
			//	summary:
			//		Sets the Client Size
			//
			R.error("SubclassResponsibility","setClientSize");
		},


		onClientSizeChanged: function () {
			//
			//	summary:
			//		The viewport has been resized (implies the related onSourceChanged event)
			//
			this.onLayoutHud();

			// Possible HUD information: this is a test
			//this._updateHudZoom();
			this._updateEventSourceShape();
		},

		getHudSize: function () {
			return this.getClientSize();
		},

		setSize: function ( /*Size*/ sz) {
			//
			//	summary:
			//		Sets the Viewport Size
			//
			R.error("SubclassResponsibility","setSize");
		},

		// ========================================================================================================
		//
		//	VIEW RECTANGLE:
		//
		//	The rectangle that represents which part of the model is currently seen from the viewport.
		//
		//	
		//
		//	_viewRect:
		//		The bounds of current content
		//
		_viewRect: EmptyRect,

		//
		//	_animate:
		//		Boolean flag to indicate that viewport movements are animated by default.
		//
		_animate: false,

		//
		//	_animLength:
		//		The length of the animation in milliseconds
		//
		_animLength: 200,

		//
		//	_animSteps:
		//		The number of updates to the viewport during the animation
		//
		_animSteps: 10,

		//
		//	_animation:
		//		The animation object (while animating).
		//
		_animation: null,

		//
		//	_animOrigin:
		//		The animation origin (while animating).
		//
		_animOrigin: null,

		//
		//	_animTarget:
		//		The animation target view rectangle (while animating).
		//
		_animTarget: null,

		getAnimLength: function () {
			return this._animLength;
		},

		setAnimLength: function ( /*Number*/ milliseconds) {
			this._animLength = milliseconds;
			return this;
		},

		getAnimSteps: function () {
			return this._animSteps;
		},

		setAnimSteps: function ( /*Number*/ steps) {
			this._animSteps = steps;
			return this;
		},

		setAnimated: function ( /*Boolean*/ animated) {
			this._animated = animated;
			return this;
		},

		getAnimated: function () {
			return this._animated;
		},

		getViewRect: function () {
			//
			//	summary:
			//		The currently displayed area in diagram coordinates.
			//
			return this._viewRect;
		},

		_cloneRect: function (r) {
			//	[AV] note: this is 7x-9x faster than the dojo.clone equivalent or than iterating attributes:
			//		var r, rect = {x:4,y:7,width:10,height:123};
			//		var start = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = dojo.clone(rect); }
			//		var end = new Date().getTime();
			//		var start2 = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = V._cloneRect(rect); }
			//		var end2 = new Date().getTime();
			//		console.log("dojo.clone",end-start,"_cloneRect",end2-start2);
			var cr = {};
			if (r.x!==undefined) cr.x = r.x;
			if (r.y!==undefined) cr.y = r.y;
			if (r.width!==undefined) cr.width = r.width;
			if (r.height!==undefined) cr.height = r.height;
			if (r.scalePivot!==undefined) cr.scalePivot = r.scalePivot;
			return cr;
		},

		prepareViewRect: function ( /*dojox.gfx.Rectangle*/ r, /*dojox.gfx.Rectangle*/ viewRect /*=current*/ ) {
			//
			//	summary:
			//		Pre-apply corrections to r that would otherwise be applied at the moment r is set as 
			//		the View Rectangle
			//
			//	description:
			//		Some corrections may be applied to r before assigning it 
			//		as the current View Rectangle:
			//
			//    	- if width xor height are set, set the other based on current viewport aspect ratio
			//
			//    	- for each axis:
			//    		- if origin is not defined and length is, make origin so that center is preserved
			//    		- if origin is defined but width is not, keep current length
			//
			//    	- if both width and height are set but the aspect ratio is not correct, correct short 
			//    	  side to match correct aspect ratio and maintain visibility of desired area.
			//
			//    	- if r goes beyond height or width, constrain location to make r within
			//
			//    	- if r lies before the origin, set location to origin
			//
			//    	- apply zoom constraints
			//
			viewRect = viewRect || this._viewRect;
			r = this._cloneRect(r);
			r.scalePivot = r.scalePivot || g.getRectCenter(r) || g.getRectCenter(viewRect);

			if (r.width === undefined && r.height !== undefined) {
				r.width = r.height * (g.getRectRatio(this.getClientSize()));
			}

			if (r.height === undefined && r.width !== undefined) {
				r.height = r.width / (g.getRectRatio(this.getClientSize()));
			}

			r = g.rectCenteredOverride(viewRect, r);
			r = this._constraintToContent(r);

			return r;

		},


		stopAnimation: function () {
			if (this._animation) {
				this._animation.stop();
				this._animation = null;
			}
		},

		_startViewRectAnimation: function ( /*dojox.gfx.Rectangle*/ viewRect) {
			this._animOrigin = this._viewRect;
			this._animTarget = viewRect;
			this._hudCreateZoomingOverlay();
			var animation = new dojo._Animation({
				duration: this._animLength,
				rate: this._animLength / this._animSteps,
				curve: [0, 100],
				onStop: dojo.hitch(this, function (v) {
					animation = null;
					this._animTarget = null;
					this._animOrigin = null;
					this._hudClearZoomingOverlay();
				}),
				onAnimate: dojo.hitch(this, function (v) {
					if (this._animation === animation) {
						// this is checked because animation.stop() is 
						// not working when fired from within the first 
						// step.
						var completion = v / 100;
						this._viewRect = g.standarizeRect(g.rectBlend(this._animOrigin, this._animTarget, completion));
						this._applyViewRect(true, true);
						this.onViewRectChanged();
						if (v == 100) {
							this.stopAnimation();
						}
					} else {
						// Try to stop the animation again
						animation.stop();
					}
				})
			});
			this._animation = animation;
			this._animation.play();
		},

		setViewRect: function ( /*dojox.gfx.Rectangle*/ r, /*Obj?*/ options) {
			//
			//	summary:
			//		Sets the currently displayed area.
			//
			//	description:
			//		Some corrections are applied to r. See prepareViewRect.
			//
			// 	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
		  //    .center
		  //      causes the view rect to be centered
			//
			options = options || {};
			if(!r) {
				return;
			}
      var oldR = g.cloneRect(r);
			r = this.prepareViewRect(r);
      if (options.center){
        if (oldR.width <= r.width){
          r.x -= (r.width - oldR.width) / 2;
        }
        if (oldR.height <= r.height){
          r.y -= (r.height - oldR.height)/ 2;
        }
      }
			
			var sizeChanged = !g.rectEqualSize(this._viewRect, r, 1 / this._zoom);
			var originChanged = !g.rectEqualOrigin(this._viewRect, r, 1 / this._zoom);

			this.stopAnimation();

			if (sizeChanged || originChanged || options.forceApply) {
				if (!(this._animated || options.animate) || options.instant) {
					this._viewRect = g.standarizeRect(r);
          this._applyViewRect(sizeChanged, originChanged);
					this.onViewRectChanged();
				} else {
					this._startViewRectAnimation(r);
				}
			}

			return this._viewRect;
		},

		_applyViewRect: function ( /*Boolean*/ sizeChanged, /*Boolean*/ originChanged) {
			this._computeZoom();

			//
			//	Subclasses must extend with the required actions.
			//
		},

		clientToContent: function () {
			//
			//	summary:
			//		returns a matrix that transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse;
		},

		pointClientToContent: function ( /*Point*/ p ) {
			//
			//	summary:
			//		transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse.transformPoint(p);
		},

		contentToClient: function () {
			//
			//	summary:
			//		returns a matrix that transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix;
		},

		pointContentToClient: function ( /*Point*/ p ) {
			//
			//	summary:
			//		transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix.transformPoint(p);
		},

		onLayoutHud: function () {
			//this._updateHudZoom();
			this._updateSurface();
			this._needsHudRelayout = false;
		},

		onViewRectChanged: function onViewRectChanged() {
			//
			//	summary:
			//		The view rectangle has been updated. 
			//
			// scrollbars may have reappeared/disappeared 
			if (this._needsHudRelayout) {
				this.onLayoutHud();
				this._needsHudRelayout = false;
			}
		},


		_onViewRectConstraintsChanged: function () {
			//
			//	summary:
			//		The view rectangle constraints have been updated.
			//
			this.setViewRect(this._viewRect, {
				forceApply: true,
				instant: true
			});
		},


		// ========================================================================================================
		//
		//	ZOOM:
		//
		//	Viewport zoom is defined from the relation between the viewport size and the view rectangle.
		//
		//		

		//
		//	_zoom:
		//		Current zoom level factor (larger is closer)
		//
		_zoom: 1,


		//
		//	_maxZoom:
		//		Defines how much the view rectangle can zoom in. Unlimited if 0.
		//	
		_maxZoom: 0,


		//
		//	_minZoom:
		//		Defines how much the view rectangle can zoom out. If 0, the limit is based on the contentRectangle.
		//	
		_minZoom: 0,


		setMaxZoom: function ( /*Number*/ z) {
			//
			//	summary:
			//		Sets a maximum zoom and updates the view rectangle based on new constraints.
			//
			if(z<0) { // normalize
				z = 0;
			}

			if(z > 0 && this._minZoom>0 && z<this._minZoom) {
				z = this._maxZoom;
				R.warn("MaxZoomBelowMin");
			}
			this._maxZoom = z;
			this._onViewRectConstraintsChanged();
			return this;
		},


		getMinZoom: function () {
			//
			//	summary:
			//		Returns the current minimum zoom constraint.
			//
			return this._minZoom;
		},


		getMaxZoom: function () {
			//
			//	summary:
			//		Returns the current maximum zoom constraint.
			//
			return this._maxZoom;
		},


		setMinZoom: function ( /*Number*/ z) {
			//
			//	summary:
			//		Sets a minimum zoom and updates the view rectangle based on new constraints.
			//
			if(z<0) { // normalize
				z = 0;
			}

			if(z>0 && this._maxZoom>0 && z>this._maxZoom) {
				z = this._maxZoom;
				R.warn("MinZoomAboveMax");
			}
			this._minZoom = z;
			this._onViewRectConstraintsChanged();
		},


		setZoom: function ( /*Number*/ zoom,options) {
			//
			//	summary:
			//		This method is a helper that actually controls setSourceRect expanding or shrinking
			//		the view rectangle size to achieve the zoom effect.
			//
			//	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
			//
			this.setViewRect({
				height: (this.getClientSize().height / zoom)
			},options);
			return this;
		},


		getZoom: function () {
			//
			//	summary:
			//		Returns the current zoom level based on the current view rectangle and content size.
			//
			return this._zoom;
		},


		_computeZoom: function () {
			//
			//	summary:
			//		Computes and stores the current zoom factor by comparing viewport size and view rectangle size.
			//
			var newZoom = this._zoom;
			if (this._viewRect.width) {
				var vp = this.getClientSize();
				newZoom = Math.min(vp.width / this._viewRect.width, vp.height / this._viewRect.height);
			}
			this._zoom = newZoom;
		},





		// ========================================================================================================
		//
		//	MISCELANEOUS SERVICES:
		//
		//
		//	_focusAirFactor:
		//		The minimum space reserved for the view rect on the visualizeArea operation.
		//		Expressed in terms of a percentage of the size of the area.
		//
		_visualizeAirFactor: 1.50,


		visualizeArea: function ( /*dojox.gfx.Rectangle*/ r, /*{expandCurrent,airFactor}*/ options) {
			//
			//	summary:
			//		Makes the r rectangle visible, givin it an extra air between the 
			//		area and the borders of the viewport if possible. The view may be 
			//		zoomed out if required, but it is never zoomed in.
			//
			//	r:
			//		The area to make visible.
			//
			//	airFactor:
			//		Percent of the area that is reserved as air. Overrides the default air 
			//		factor defined bu the member _focusAirFactor.
			//
			options = options || {};
			var vr = this._viewRect;
			var airFactor = (options.airFactor !== undefined) ? (1 + 2 * options.airFactor) : this._visualizeAirFactor;

			if (options.expandCurrent) {
				this.setViewRect(g.addRect(g.expandRect(r, Math.max(r.width, r.height) * (airFactor - 1) / 2), vr),options);
			} else {
				var minSize = {
					width: Math.max(vr.width, r.width * airFactor),
					height: Math.max(vr.height, r.height * airFactor)
				};
				var centered = g.rectCenteredAt(vr, g.getRectCenter(r));
				this.setViewRect(g.rectCenteredOverride(centered, minSize),options);
			}
		},

		centerAt: function ( /*Point*/ p, options) {
			//
			//	summary:
			//		Centers the viewport at the requested point, without changing the zoom.
			//
			this.setViewRect(g.rectCenteredAt(this.getViewRect(), p),options);
		},

		zoomAt: function ( /*Point*/ p, /*Number*/ z, options) {
			//
			//	summary:
			//		Scales the current zooms by a given factor z, using point p as pivot.
			//
			var vr = this.getViewRect(); // current viewport pos
			var rp = g.subPoint(p, vr); // relative cp
			var newViewRect = g.subPoint(p, g.mulPoint(rp, 1 / z));
			newViewRect.height = vr.height / z;
			newViewRect.scalePivot = p;

			this.setViewRect(newViewRect,options);
		},

		reset: function () {
			//
			//	TODO: reset the receiver to its initial state
			//
		},

		_dumpState: function () {
			console.log("zoom:" + ibm_ilog.diagram.util._round(this._zoom, 3) + " view:" + g.printRect(this._viewRect) + " cnt:" + g.printRect(this._contentRect));
		},

		toString: function () {
			return "[Viewport]";
		},

		getDiagram: function () {
			return this._diagram;
		},
		
		bringIntoView: function(/* ibm_ilog.diagram.Node */ node, /* Number? */ margin) {
		  //
		  //  summary:
		  //     translate the view just enough so that a given element is fully visible
		  //
		  
		  var vr = this.getViewRect();
		  var bb = node.getBounds(this.getDiagram().getGraph());
		  if(margin === undefined) margin = 10;
		  
      bb = g.cloneRect(bb);
		  bb.x -= margin;
		  bb.y -= margin;
		  bb.width += 2*margin;
		  bb.height += 2*margin;
		  
		  var newvr = { x:vr.x, y:vr.y };
		  newvr.x = Math.max(newvr.x, bb.x + bb.width - vr.width);
		  newvr.y = Math.max(newvr.y, bb.y + bb.height - vr.height);
		  newvr.x = Math.min(newvr.x, bb.x);
		  newvr.y = Math.min(newvr.y, bb.y);

		  this.setViewRect(newvr);
		}
		

	});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.DivViewport']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.DivViewport'] = true;
dojo.provide('ibm_ilog.diagram.widget.DivViewport');






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

  function getScrollBarWidth () {  
      var a = document.createElement('p');  
      a.style.width = "100%";  
      a.style.height = "200px";  
      var b = document.createElement('div');  
      b.style.position = "absolute";  
      b.style.top = "0px";  
      b.style.left = "0px";  
      b.style.visibility = "hidden";  
      b.style.width = "200px";  
      b.style.height = "150px";  
      b.style.overflow = "hidden";  
      b.appendChild (a);  
      document.body.appendChild (b);  
      var w1 = a.offsetWidth;  
      b.style.overflow = 'scroll';  
      var w2 = a.offsetWidth;  
      if (w1 == w2) w2 = b.clientWidth;  
      document.body.removeChild (b);  
      return (w1 - w2);  
  };  
   
dojo.declare('ibm_ilog.diagram.widget.DivViewport', [ibm_ilog.diagram.widget.Viewport,ibm_ilog.diagram.util.HandleSet], {
	//
	//	summary:
	//		This is a Viewport class based on an HTML <div> which overflows with scrollbars.
	//
	//	description:
	//
	//		DivViewport depends on a div declared at Diagram.html with attachPoint=scrollbarsNode.
	//		This div encloses the diagram Surface. 
	//		
	//		The diagram Surface is extended to enclose the whole diagram tightly. The 
	//		div is constrained to the desired client size using its width and height 
	//		properties. The view rectangle is then associated with the div scroll 
	//		positions, scrollTop and scrollLeft. Setting the view rectangle then 
	//		updates scrollTop and scrollLeft.
	//
	//		The zoom level is derived from the view rectangle, but is mapped entirely 
	//		different than its position. Instead, the zoom controls a scale transformation 
	//		of the entire Diagram. If the view rectangle is small, the Diagram is zoomed 
	//		in to fit the requested portion into the viewport window. With this also the 
	//		Surface is scaled, as it is required that the entire Graph fits the surface 
	//		exactly so that the scrollTop/scrollLeft properties work.
	//		
	//	note:
	//		the zooming service required from the Diagram object should be at a UI 
	//		level and should not affect, for example, the bouding box of the the Graph 
	//		objects. One option is to define an intermediate node between the Surface root 
	//		and the Graph node, which controls this transformation.
	//
	
	//
	//	The DOMNode of the div to which we bound
	//
	_scrollbars: null,

	_scrollbarsContent: null,

	_scrollbarWidth: 0, // TODO compute this
	
	_scrollCache: PointZero,
	
	_hScrollbarVisible: true,
	
	_vScrollbarVisible: true,

	templateString: function() {
		return "<div style='overflow:hidden; position:relative;width:100%;height:100%;' tabindex='-1' role='presentation'>"+
		         "<div dojoAttachPoint='scrollbarsNode' style='z-index:0; position:absolute; top:0px; left:0px; overflow:auto' tabindex='-1' role='presentation'>"+
		           "<div dojoAttachPoint='scrollbarsContentNode' style='visibility:hidden' tabindex='-1' role='presentation'></div>"+
		         "</div>"+
		         "<div dojoAttachPoint='canvasNode' style='z-index:1; position:absolute; top:0px; left:0px'></div>"+
		       "</div>";
	},
	
	//
	//	_updatingScrollbars:
	//		true when scrollbar position is being updated and we will ignore related events
	//		useful in chrome since it reports events immediately.
	//
	_updatingScrollbars: false,

	_computeScrollbarWidth: function() {
		return this._scrollbarWidth = getScrollBarWidth();
	},

	getScrollbarWidth: function() {
		return this._scrollbarWidth || this._computeScrollbarWidth();
	},

	getHudSize: function() {
		var hsz = this.getClientSize();
		var sbw = this.getScrollbarWidth();
		if(!this._hScrollbarVisible) {
			hsz.height+=sbw;
		}
		if(!this._vScrollbarVisible) {
			hsz.width+=sbw;
		}
		return hsz;
	},
	
	updateScrollbarVisibilityFlags: function() {
		var hv = (this._scrollbars.scrollWidth>this._scrollbars.clientWidth);//(this._viewRect.width<this._contentRect.width);
  	if(this._hScrollbarVisible != hv) {
  		this._hScrollbarVisible = hv;
  		this._needsHudRelayout = true;
  	}
		var vv = (this._scrollbars.scrollHeight>this._scrollbars.clientHeight);//(this._viewRect.height<this._contentRect.height);
  	if(this._vScrollbarVisible != vv) {
  		this._vScrollbarVisible = vv;
  		this._needsHudRelayout = true;
  	}
	},

	_onContentBoundingBoxChanged: function(){
		//
		//	summary:
		//		The function is executed when the content of the content has changed.
		//
		//	returns:
		//		True if the change affected content bounding box.
		//
		//	tags:
		//		private
		//
		this.inherited(arguments)
	},

	initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Initializes a Viewport for collaborating with the diagram.
		//
		//	diagram: ibm_ilog.diagram.widget.Diagram
		//		The diagram
		//
		this._scrollbars = diagram.scrollbarsNode;		
		this._scrollbarsContent = diagram.scrollbarsContentNode;
		
		this.inherited(arguments);

		this._connect("onscroll",this._scrollbars,"onscroll",this,"_onScroll");
		if (!diagram.getParent())
			this._connect("onresize",dojo.isIE ? diagram.domNode : dojo.global,"onresize",this,"_onResize");
		
		var parent = dijit.getEnclosingWidget(diagram.domNode.parentNode);
		
		// we are in dojox.mobile
		if (parent && dojo.isFunction(parent.onAfterTransitionIn)){
			dojo.connect(parent,"onAfterTransitionIn",this,"_onAfterTransitionIn");
			
		};
		
		
	},

	_debug: false,
	
	_onScroll: function(e) {
		if(!this._updatingScrollbars) {
			var p = Point(this._scrollbars.scrollLeft,this._scrollbars.scrollTop);
			
			if(this._debug) {
				console.log("cached:",g.printRect(this._scrollCache),", scroll:",g.printPoint(p));
			}
			
			// The following condition traps bouncing events generated by 
			// the receiver updating the scrollbar programatically
			
			if(!g.equalPoint(this._scrollCache,p)) {
				var q = this._pointScrollToModel(p);
				this.setViewRect(q,{instant:true});
			}
		}
	},
	
	_onResize: function(e) {
		var csz = this._computeInitialSize();
		this.setSize(csz);
	},
	
	_onAfterTransitionIn: function(){
		this.updateScrollbarVisibilityFlags();	
		var csz = this._computeInitialSize();
		this.setSize(csz);
		
	},
	
	setClientSize: function( /*Size*/ sz ){
		//
		//	summary:
		//		Sets the client Size
		//
		this.setSize({width:sz.width+this.getScrollbarWidth(),height:sz.height+this.getScrollbarWidth()});
	},
	
	setSize: function( /*Size*/ sz ) {
		//
		//	summary:
		//		Sets the Viewport Size
		//
		var sbw = this.getScrollbarWidth();
		
		var style = {style:{width:sz.width+"px",height:sz.height+"px"}};

		//dojo.attr(this._diagram.domNode,style);
		dojo.attr(this._scrollbars,style);
		this._updateClientSize(Size(sz.width-sbw,sz.height-sbw));
		this._updateSurface();
	},
	

	_updateClientSize: function( /*Size*/ sz ) {
		//
		//	summary:
		//		Updates viewportRect from the div attributes
		//
		//	tags:
		//		private
		//
		var vpH = sz.height || this._scrollbars.clientHeight;
		var vpW = sz.width || this._scrollbars.clientWidth;
		
		var newClientSize = { width:vpW, height:vpH };


		if(!g.equalRect(this._clientSize,newClientSize)) {

			this._clientSize = newClientSize;
			
			this._computeContentRect();

			switch(this._resizeMode) {
			case 1: // keep zoom level
			  // Note that we do NOT set the height attribute. Explanation from Abel:
			  // setViewRect can handle missing variables and they are 
				// inferred. We never want to change the aspect ratio, so this is a 
				// way of specifying priority: "take whatever height you need, but 
				// set width to what I give you"
				this.setViewRect({
					x:this._viewRect.x,
					y:this._viewRect.y,
					width:(newClientSize.width/this.getZoom())},{instant:true});
				break;
			case 2:
				this._onViewRectConstraintsChanged();
				break;
			default:
				R.error("InvalidViewportResizeMode",this._resizeMode)
			}
			
			this.onClientSizeChanged();

		}
	},
	
	_pointContentToScroll: function(p) {
		var r = g.subPoint(this._viewportMatrix.transformPoint(p),this._transformedContentRect);
		if(this._scrollCache && this.isRtl()) {
			var corr = this._scrollCache.width-this._viewportMatrix.transformRectangle(this._viewRect).width;
			r.x = r.x - corr;
		}
		return r;
	},
	
	isRtl: function() {
		//	summary:
		//		returns true if running on RTL mode.
		//	note:
		//		Peller said in the forum that RTL is supported only if set at the body. This is a dijit limitation.
		return dojo.attr(dojo.doc.body,"dir")=="rtl";
	},
	
	_pointScrollToModel: function(p) {
		if(this._scrollCache && this.isRtl()) {
			var corr = this._scrollCache.width-this._viewportMatrix.transformRectangle(this._viewRect).width;
			p.x = corr+p.x;
		}		
		return this._viewportMatrixInverse.transformPoint(g.addPoint(this._transformedContentRect,p));
	},	
	
	onViewRectChanged: function onViewRectChanged(){
		this.updateScrollbarVisibilityFlags();	
		this.inherited(arguments);
	},

	_applyViewRect: function( /*Boolean*/ sizeChanged, /*Boolean*/ originChanged ){
		//
		//	summary:
		//		Updates the surface size and _transformedContentRect based on the _viewRect and
		//		_contentRect. Called after a change in zoom or Graph content.
		//
		//	tags:
		//		private
		//
		
		if(!this.__initialized) {
			return;
		}
		
		this.inherited(arguments);
		
		// create transformation, apply if to Graph, and compute new transformedContentRect
		var zt = dojox.gfx.matrix.scale(this._zoom);
		var tt = dojox.gfx.matrix.translate(g.negPoint(this._viewRect)); // uses rect origin
		var t  = dojox.gfx.matrix.multiply(zt,tt);

		this._transformedContentRect = t.transformRectangle(g.addRect(this._viewRect,this._contentRect));
		
		var scrollContentSize = g.floorSize(g.growRect(this._transformedContentRect,this.getClientSize()));
		          // this._transformedContentRect;
			 	  // [AV] replace by this go back to tight surface mode
		
		this._viewportMatrix = t;
		this._viewportMatrixInverse = t.inverse();

		var scrollCache = g.floorPoint(this._pointContentToScroll(this._viewRect));

		scrollCache.width=scrollContentSize.width;
		scrollCache.height=scrollContentSize.height;
		
		this._updatingScrollbars = true; // needed for chrome

		//this._disconnect("onscroll"); // does not seem to work for avoiding the event generated 
		                                // update in the following lines
		
		// "if" to avoid updating in the wrong order (shrinking width while position requires it, and viceversa.)
		// and to avoid updating unless strictly necessary
		if(scrollCache.width >= this._scrollCache.width) {
			if(this._scrollCache.width != scrollCache.width) {
				this._scrollbarsContent.style.width = scrollCache.width+"px";
			}
			if((this._scrollCache.x) != (scrollCache.x)) {
				this._scrollbars.scrollLeft = scrollCache.x;
			}
		} else {
			if(this._scrollCache.x != scrollCache.x) {
				this._scrollbars.scrollLeft = scrollCache.x;
			}
			if(this._scrollCache.width != scrollCache.width) {
				
				this._scrollbarsContent.style.width = scrollCache.width+"px";
			}
		}

		if(scrollCache.height >= this._scrollCache.height) {
			if(this._scrollCache.height != scrollCache.height) {
				this._scrollbarsContent.style.height = scrollCache.height+"px";
			}
			if(this._scrollCache.y != scrollCache.y) {
				this._scrollbars.scrollTop = scrollCache.y;
			}
		} else {
			if(this._scrollCache.y != scrollCache.y) {
				this._scrollbars.scrollTop = scrollCache.y;
			}
			if(this._scrollCache.height != scrollCache.height) {
				this._scrollbarsContent.style.height = scrollCache.height+"px";
			}
		}

		scrollCache.x = this._scrollbars.scrollLeft; // sometimes the assignment fails, not setting the correct value
		scrollCache.y = this._scrollbars.scrollTop;  // this gets the real into the cache
		
		this._scrollCache = scrollCache;

    this._updatingScrollbars = false;
		
		// give the change to the DOM to process the recent 
		// updates so we don't get their notifications
		//		setTimeout(dojo.hitch(this,function(){
		//			this._connect("onscroll",this._scrollbars,"onscroll",this,"_onScroll");
		//		}),1);

		this._setViewportTransform(t);
	}

	
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.OverviewRenderer']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.OverviewRenderer'] = true;
dojo.provide('ibm_ilog.diagram.overview.OverviewRenderer');



(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;

dojo.declare('ibm_ilog.diagram.overview.OverviewRenderer',null, {
	
	//
	//	summary:
	//		The abstract superclass of overview renderers. This class may only exist
	//		for documentation purposes only.
	//

	render: function( /*Object*/content, /*dojox.gfx.Group*/ output ) {
		R.error("SubclassResponsibility","render");
	}
		
});

})();

}

if(!dojo._hasResource["dojox.lang.functional.lambda"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.lambda"] = true;
dojo.provide("dojox.lang.functional.lambda");

// This module adds high-level functions and related constructs:
//	- anonymous functions built from the string

// Acknoledgements:
//	- lambda() is based on work by Oliver Steele
//		(http://osteele.com/sources/javascript/functional/functional.js)
//		which was published under MIT License

// Notes:
//	- lambda() produces functions, which after the compilation step are
//		as fast as regular JS functions (at least theoretically).

// Lambda input values:
//	- returns functions unchanged
//	- converts strings to functions
//	- converts arrays to a functional composition

(function(){
	var df = dojox.lang.functional, lcache = {};

	// split() is augmented on IE6 to ensure the uniform behavior
	var split = "ab".split(/a*/).length > 1 ? String.prototype.split :
			function(sep){
				 var r = this.split.call(this, sep),
					 m = sep.exec(this);
				 if(m && m.index == 0){ r.unshift(""); }
				 return r;
			};
			
	var lambda = function(/*String*/ s){
		var args = [], sects = split.call(s, /\s*->\s*/m);
		if(sects.length > 1){
			while(sects.length){
				s = sects.pop();
				args = sects.pop().split(/\s*,\s*|\s+/m);
				if(sects.length){ sects.push("(function(" + args + "){return (" + s + ")})"); }
			}
		}else if(s.match(/\b_\b/)){
			args = ["_"];
		}else{
			var l = s.match(/^\s*(?:[+*\/%&|\^\.=<>]|!=)/m),
				r = s.match(/[+\-*\/%&|\^\.=<>!]\s*$/m);
			if(l || r){
				if(l){
					args.push("$1");
					s = "$1" + s;
				}
				if(r){
					args.push("$2");
					s = s + "$2";
				}
			}else{
				// the point of the long regex below is to exclude all well-known
				// lower-case words from the list of potential arguments
				var vars = s.
					replace(/(?:\b[A-Z]|\.[a-zA-Z_$])[a-zA-Z_$\d]*|[a-zA-Z_$][a-zA-Z_$\d]*:|this|true|false|null|undefined|typeof|instanceof|in|delete|new|void|arguments|decodeURI|decodeURIComponent|encodeURI|encodeURIComponent|escape|eval|isFinite|isNaN|parseFloat|parseInt|unescape|dojo|dijit|dojox|window|document|'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"/g, "").
					match(/([a-z_$][a-z_$\d]*)/gi) || [], t = {};
				dojo.forEach(vars, function(v){
					if(!(v in t)){
						args.push(v);
						t[v] = 1;
					}
				});
			}
		}
		return {args: args, body: s};	// Object
	};

	var compose = function(/*Array*/ a){
		return a.length ?
					function(){
						var i = a.length - 1, x = df.lambda(a[i]).apply(this, arguments);
						for(--i; i >= 0; --i){ x = df.lambda(a[i]).call(this, x); }
						return x;
					}
				:
					// identity
					function(x){ return x; };
	};

	dojo.mixin(df, {
		// lambda
		rawLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns an object describing the function; functions are
			//		passed through unmodified.
			// description:
			//		This method is to normalize a functional representation (a
			//		text snippet) to an object that contains an array of
			//		arguments, and a body , which is used to calculate the
			//		returning value.
			return lambda(s);	// Object
		},
		buildLambda: function(/*String*/ s){
			// summary:
			//		builds a function from a snippet, returns a string, which
			//		represents the function.
			// description:
			//		This method returns a textual representation of a function
			//		built from the snippet. It is meant to be evaled in the
			//		proper context, so local variables can be pulled from the
			//		environment.
			s = lambda(s);
			return "function(" + s.args.join(",") + "){return (" + s.body + ");}";	// String
		},
		lambda: function(/*Function|String|Array*/ s){
			// summary:
			//		builds a function from a snippet, or array (composing),
			//		returns a function object; functions are passed through
			//		unmodified.
			// description:
			//		This method is used to normalize a functional
			//		representation (a text snippet, an array, or a function) to
			//		a function object.
			if(typeof s == "function"){ return s; }
			if(s instanceof Array){ return compose(s); }
			if(s in lcache){ return lcache[s]; }
			s = lambda(s);
			return lcache[s] = new Function(s.args, "return (" + s.body + ");");	// Function
		},
		clearLambdaCache: function(){
			// summary:
			//		clears internal cache of lambdas
			lcache = {};
		}
	});
})();

}

if(!dojo._hasResource["dojox.lang.functional.fold"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional.fold"] = true;
dojo.provide("dojox.lang.functional.fold");



// This module adds high-level functions and related constructs:
//	- "fold" family of functions

// Notes:
//	- missing high-level functions are provided with the compatible API:
//		foldl, foldl1, foldr, foldr1
//	- missing JS standard functions are provided with the compatible API:
//		reduce, reduceRight
//	- the fold's counterpart: unfold

// Defined methods:
//	- take any valid lambda argument as the functional argument
//	- operate on dense arrays
//	- take a string as the array argument
//	- take an iterator objects as the array argument (only foldl, foldl1, and reduce)

(function(){
	var d = dojo, df = dojox.lang.functional, empty = {};

	d.mixin(df, {
		// classic reduce-class functions
		foldl: function(/*Array|String|Object*/ a, /*Function*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left
			//	to right using a seed value as a starting point; returns the final
			//	value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var i, n;
			if(d.isArray(a)){
				// array
				for(i = 0, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				for(i = 0; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
			}else{
				// object/dictionary
				for(i in a){
					if(!(i in empty)){
						z = f.call(o, z, a[i], i, a);
					}
				}
			}
			return z;	// Object
		},
		foldl1: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left
			//	to right; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var z, i, n;
			if(d.isArray(a)){
				// array
				z = a[0];
				for(i = 1, n = a.length; i < n; z = f.call(o, z, a[i], i, a), ++i);
			}else if(typeof a.hasNext == "function" && typeof a.next == "function"){
				// iterator
				if(a.hasNext()){
					z = a.next();
					for(i = 1; a.hasNext(); z = f.call(o, z, a.next(), i++, a));
				}
			}else{
				// object/dictionary
				var first = true;
				for(i in a){
					if(!(i in empty)){
						if(first){
							z = a[i];
							first = false;
						}else{
							z = f.call(o, z, a[i], i, a);
						}
					}
				}
			}
			return z;	// Object
		},
		foldr: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left using a seed value as a starting point; returns the final
			//	value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			for(var i = a.length; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		foldr1: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left; returns the final value.
			if(typeof a == "string"){ a = a.split(""); }
			o = o || d.global; f = df.lambda(f);
			var n = a.length, z = a[n - 1], i = n - 1;
			for(; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		// JS 1.8 standard array functions, which can take a lambda as a parameter.
		reduce: function(/*Array|String|Object*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array
			//	(from left-to-right) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldl1(a, f) : df.foldl(a, f, z);	// Object
		},
		reduceRight: function(/*Array|String*/ a, /*Function|String|Array*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array
			//	(from right-to-left) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldr1(a, f) : df.foldr(a, f, z);	// Object
		},
		// the fold's counterpart: unfold
		unfold: function(/*Function|String|Array*/ pr, /*Function|String|Array*/ f,
						/*Function|String|Array*/ g, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by unfolding a value
			o = o || d.global; f = df.lambda(f); g = df.lambda(g); pr = df.lambda(pr);
			var t = [];
			for(; !pr.call(o, z); t.push(f.call(o, z)), z = g.call(o, z));
			return t;	// Array
		}
	});
})();

}

if(!dojo._hasResource["dojox.collections.Set"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections.Set"] = true;
dojo.provide("dojox.collections.Set");


(function(){
	var dxc=dojox.collections;
	dxc.Set=new (function(){
		function conv(arr){
			if(arr.constructor==Array){
				return new dojox.collections.ArrayList(arr);	//	dojox.collections.ArrayList
			}
			return arr;		//	dojox.collections.ArrayList
		}
		this.union = function(/* array */setA, /* array */setB){
			//	summary
			//	Return the union of the two passed sets.
			setA=conv(setA);
			setB=conv(setB);
			var result = new dojox.collections.ArrayList(setA.toArray());
			var e = setB.getIterator();
			while(!e.atEnd()){
				var item=e.get();
				if(!result.contains(item)){
					result.add(item);
				}
			}
			return result;	//	dojox.collections.ArrayList
		};
		this.intersection = function(/* array */setA, /* array */setB){
			//	summary
			//	Return the intersection of the two passed sets.
			setA=conv(setA);
			setB=conv(setB);
			var result = new dojox.collections.ArrayList();
			var e = setB.getIterator();
			while(!e.atEnd()){
				var item=e.get();
				if(setA.contains(item)){
					result.add(item);
				}
			}
			return result;	//	dojox.collections.ArrayList
		};
		this.difference = function(/* array */setA, /* array */setB){
			//	summary
			//	Returns everything in setA that is not in setB.
			setA=conv(setA);
			setB=conv(setB);
			var result = new dojox.collections.ArrayList();
			var e=setA.getIterator();
			while(!e.atEnd()){
				var item=e.get();
				if(!setB.contains(item)){
					result.add(item);
				}
			}
			return result;	//	dojox.collections.ArrayList
		};
		this.isSubSet = function(/* array */setA, /* array */setB) {
			//	summary
			//	Returns if set B is a subset of set A.
			setA=conv(setA);
			setB=conv(setB);
			var e = setA.getIterator();
			while(!e.atEnd()){
				if(!setB.contains(e.get())){
					return false;	//	boolean
				}
			}
			return true;	//	boolean
		};
		this.isSuperSet = function(/* array */setA, /* array */setB){
			//	summary
			//	Returns if set B is a superset of set A.
			setA=conv(setA);
			setB=conv(setB);
			var e = setB.getIterator();
			while(!e.atEnd()){
				if(!setA.contains(e.get())){
					return false;	//	boolean
				}
			}
			return true;	//	boolean
		};
	})();
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.util.GraphUtil']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.GraphUtil'] = true;
dojo.provide('ibm_ilog.diagram.util.GraphUtil');

if (!dojo.config.useGfxLayout) {
	
	
	
	
}






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var gfx = dojox.gfx;
	var foldl = dojox.lang.functional.foldl;
	var g = ibm_ilog.diagram.util.Geometry;
	
ibm_ilog.diagram.util.GraphUtil = {};

var gu = ibm_ilog.diagram.util.GraphUtil;

var dxcs = dojox.collections.Set;

gu.allGraphElements = function collectChildren(roots,result) {
	if(!result) {
		result = new dojox.collections.ArrayList();
	}
	dojo.forEach(roots, function(e){
		if(e._isIBMDiagramGraphElement && !result.contains(e)) {
			result.add(e);
			var g;
			if(e._isIBMDiagramSubgraph && (g=e.getGraph().children)) {
				gu.allGraphElements(g,result);
			}
		}
	});
	return result;
};

gu.maximals = function maximals(gs) {
	//
	//	summary:
	//		Returns only the maximal containers for an array of graph elements
	//	s: Array
	//		the array of graph elements
	//

	var r = [];
	var subgraphs = [];

	dojo.forEach(gs,function(e){
		if(e._isIBMDiagramSubgraph) {
			subgraphs.push(e);
		}
	});
	
	dojo.forEach(gs,function(e){
		var keep = true;
		
		for( var i=0; keep && i < subgraphs.length; ++i ) {
			var s = subgraphs[i];
			if(s!=e && gu.isContainedBy(e,s)) {
				keep = false;
			}
		}
		if(keep) {
			r.push(e);
		}
		
	},this);
	
	return r;
};

gu.getOwningSubgraph = function getOwningSubgraph(ge) {
	for (var p = gu.safeGetParent(ge); p !== null && p.declaredClass != 'dojox.gfx.Surface'; p = gu.safeGetParent(p)) {
		if (p._isIBMDiagramSubgraph) {
			return p;
		}
	}
	return null;
};

gu.isStableVisible = function(ge){
	//
	//	returns:
	//		true of the graph element is visible in the surface
	//
  for (var p = ge.getParent();p && p.getParent; p=p.getParent()){
  	if(p._isIBMDiagramSubgraph && p._animating) {
  		return false;
  	}
  }
  return p !== null;
};

var _pairLowestCommonAncestor = function _pairLowestCommonAncestor(ga, gb) {
	var gaa;
	if(ga==gb) {
		return ga;
	}
	if (gu.isContainedBy(gb, ga)) {
		return ga;
	}
	if (gu.isContainedBy(ga, gb)) {
		return gb;
	}
	if(gaa = gu.getOwningSubgraph(ga)) {
		return _pairLowestCommonAncestor(gu.safeGetParent(gaa),gb);
	} else {
		return ga;
	}
};

gu.lowestCommonAncestor = function lowestCommonAncestor() {
	var elements = arguments;
	if(dojo.isArray(arguments[0])) {
		elements = arguments[0];
	}
	var a = elements[0].getParent();
	for(var i=1;i<elements.length;i++) {
		a = _pairLowestCommonAncestor(a,gu.safeGetParent(elements[i]));
	}
	return a;
};

gu.applyStyle = function(shape,style) {
	if(style.fill) {
		shape.setFill(style.fill);
	}
	if(style.stroke) {
		shape.setStroke(style.stroke);
	}
	if(style.font) {
		shape.setStroke(style.font);
	}
	if(style.selectedStyle) {
		shape.selectedStyle = style.selectedStyle;
	}
	if(style.shape) {
		shape.setShape(style.shape);
	}
	if(style.editable && style.editable.style) {
		shape.editable = shape.editable || {};
		shape.editable.style = style.editable.style;
	}
};

gu.getSurface= function(ge) {
	while(ge && ge.getParent) {
		ge = ge.getParent();
	};
	return ge;
};

gu.forEach = function(ge,f,scope) {
	//
	// Simple forEach style iteration through Graph entities
	//	
	scope = scope || dojo.global;
	if(ge) {
		var isSurface = ge instanceof gfx.Surface;
		var isSubgraph = ge instanceof ibm_ilog.diagram.Subgraph;
		var isNode = ge instanceof ibm_ilog.diagram.Node;
		var isLink = ge instanceof ibm_ilog.diagram.Link;
		var isGroup = ge instanceof gfx.Group;
		
		if( isSurface || isLink || isNode || isSubgraph ) {
			f.call(scope,ge);
		}
		
		var subs = {};
		
		if( isSubgraph ) {
			subs = ge.getGraph().children;
		} else if( isGroup || isSurface ) {
			subs = ge.children;
		}

		dojo.forEach(subs,function(ge){
			gu.forEach(ge,f,scope);
		});
	}
};

gu.isContainedBy = function(element,container) {
	while(element){
		if(element==container) {
			return true;
		}
		element = gu.safeGetParent(element);
	}
	return false;
};

gu.safeGetParent = function(ge){
	if(ge.getParent && ge.getParent()){
		return ge.getParent();
	}else{
		if(ge._collapsedParent) {
			return ge._collapsedParent;
		}
	}
	return null;
};

gu.doAncestors = function(shape,cb,scope) {
	scope=scope||dojo.global;
	for(; shape && gu.safeGetParent(shape); shape = gu.safeGetParent(shape)){
		cb.call(scope,gu.safeGetParent(shape));
	}
};

gu.folders = function(kwArgs) {
	var ignore = function(){};
	return dojo.mixin({
		graph: ignore,
		group: ignore,
		sub:   ignore,
		node:  ignore,
		link:  ignore,
		other: ignore,
		child: ignore,
		cBase: null,
		accGraph: ignore,
		accSubgraph: ignore,
		accGroup: ignore		
	},
		kwArgs
	);
};

gu.fold = function fold(e,ps,folders,scope,stopRecursionAtCollapsedSub) {
	//
	//	summary:
	//		Graph data structure fold function. --Functional Programming--
	//
	//	description:
	//		Recurses a Graphic offering callbacks to the user at each structural joint. In addition
	//		to the standard folder functions for each kind of entity, this function supports
	//		a state that is built forward on composite nodes [ for processing root-to-leaf instead
	//		of only folding leaf-to-root ].
	//      Also, it allows optionally to stop the recursion at the level of collapsed subgraphs,
	//      which are treated as a node.
	//
	//	types:
	//		G == Graphic
	//		FG == folded graphic
	//		FC == folded child collection
	//		ST == forward-built state
	//
	//	e:  /*G*/
	//		The Graphic to recurse
	//
	//	ps: /*ST*/
	//		the initial parent state
	//
	//	folders: {
	//
	//		graph: function( ST, ST, G, FG ){ return FG; },
	//			Fold a Graph
	//
	//		group: function( ST, ST, G, FG ){ return FG; },
	//			Fold a Group
	//
	//		sub:   function( ST, ST, G, FG ){ return FG; },
	//			Fold a Subgraph (receives folded inner Graph)
	//
	//		node:  function( ST, G ){ return FG; },
	//			Fold a Node
	//
	//		link:  function( ST, G ){ return FG; },
	//			Fold a Group node
	//
	//		other: function( ST, G ){ return FG; },
	//			Fold an unknown node (eventual gfx.Rects, etc.)
	//
	//		child: function( ST, FC, FG ){ return FC; },
	//			Fold a folded Graphic FG into a folded child-collection FC, returning the new collection
	//
	//		cBase: FC
	//			The initially empty folded child collection (constant)
	//
	//		accSubgraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGroup: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//	}
	//
	//  stopRecursionAtCollapsedSub: 
	//      If true, the recursion stops at the level of collapsed subgraphs, which are
	//      treated as a node. 
	//
	//	NOTE: unstable API
	//
	scope = scope || dojo.global;
	
	var foldChildren = function(c,ns,stopRecursionAtCollapsedSub) {
			return foldl(c,
				function(z,j){
					return folders.child.call(scope,ns,z,gu.fold(j,ns,folders,scope,stopRecursionAtCollapsedSub));
				},folders.cBase);
	};
	
	if(e instanceof ibm_ilog.diagram.Subgraph) { 
	   if (stopRecursionAtCollapsedSub && e.isCollapsed()) {
	     return folders.node.call(scope,ps,e);
	   } else {
	   	 var ns = folders.accSubgraph.call(scope,ps,e); 
	     return folders.sub.call(scope,ps,ns,e,gu.fold(e.getGraph(),ns,folders,scope,stopRecursionAtCollapsedSub));	   	
	   } 
	}
	else if(e instanceof ibm_ilog.diagram.Node)     { return folders.node.call(scope,ps,e); }
	else if(e instanceof ibm_ilog.diagram.Link)     { return folders.link.call(scope,ps,e); }
	else if(e instanceof ibm_ilog.diagram.Graph)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
	else if(e instanceof gfx.Group)            { var ns = folders.accGroup.call(scope,ps,e); 
	                                             return folders.group.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
	else { return folders.other.call(scope,ps,e); }
};

// AVA: Added foldGraphModel because needed for the server-side layout support of
// swimlanes. In principle could be merged with "fold" (which browses a Graph), however
// the function is already complex without this merge, so I prefer to not go this way.
// Server-side layout does not use "fold" anymore, so could be removed, but maybe it
// will be useful for something else;

gu.foldGraphModel = function foldGraphModel(e,ps,folders,scope,stopRecursionAtCollapsedSub) {
	//
	//	summary:
	//		Graph model data structure fold function. --Functional Programming--
	//
	//	description:
	//		Recurses a graph layout model offering callbacks to the user at each structural joint. In addition
	//		to the standard folder functions for each kind of entity, this function supports
	//		a state that is built forward on composite nodes [ for processing root-to-leaf instead
	//		of only folding leaf-to-root ].
	//      Also, it allows optionally to stop the recursion at the level of collapsed subgraphs,
	//      which are treated as a node.
	//
	//	types:
	//		G == Graph layout model (subclass of ibm_ilog.graphlayout.AbstractGraphLayoutModel)
	//		FG == folded graph layout model
	//		FC == folded child collection
	//		ST == forward-built state
	//
	//	e:  /*G*/
	//		The graph layout model (subclass of ibm_ilog.graphlayout.AbstractGraphLayoutModel) to recurse
	//
	//	ps: /*ST*/
	//		the initial parent state
	//
	//	folders: {
	//
	//		graph: function( ST, ST, G, FG ){ return FG; },
	//			Fold a graph layout model (ibm_ilog.graphlayout.AbstractGraphLayoutModel)
	//
	//		group: function( ST, ST, G, FG ){ return FG; },
	//			Fold a Group
	//
	//		sub:   function( ST, ST, G, FG ){ return FG; },
	//			Fold a Subgraph (receives folded inner Graph)
	//
	//		node:  function( ST, G ){ return FG; },
	//			Fold a Node
	//
	//		link:  function( ST, G ){ return FG; },
	//			Fold a Group node
	//
	//		other: function( ST, G ){ return FG; },
	//			Fold an unknown node (eventual gfx.Rects, etc.)
	//
	//		child: function( ST, FC, FG ){ return FC; },
	//			Fold a folded AbstractGraphLayoutModel FG into a folded child-collection FC, returning the new collection
	//
	//		cBase: FC
	//			The initially empty folded child collection (constant)
	//
	//		accSubgraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGroup: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//	}
	//
	//  stopRecursionAtCollapsedSub: 
	//      If true, the recursion stops at the level of collapsed subgraphs, which are
	//      treated as a node. 
	//
	//	NOTE: unstable API
	//
	scope = scope || dojo.global;
	
	var foldChildren = function(c,ns,stopRecursionAtCollapsedSub) {
			return foldl(c,
				function(z,j){
					return folders.child.call(scope,ns,z,gu.foldGraphModel(j,ns,folders,scope,stopRecursionAtCollapsedSub));
				},folders.cBase);
	};
	
	if(e instanceof ibm_ilog.diagram.Subgraph) { 
	   if (stopRecursionAtCollapsedSub && e.isCollapsed()) {
	     return folders.node.call(scope,ps,e);
	   } else {
	   	 var ns = folders.accSubgraph.call(scope,ps,e); 
	     return folders.sub.call(scope,ps,ns,e,gu.foldGraphModel(new ibm_ilog.diagram.graphlayout.GraphAdapter(e.getGraph()),ns,folders,scope,stopRecursionAtCollapsedSub));	   	
	   } 
	}
	else if(e instanceof ibm_ilog.diagram.Node)     { return folders.node.call(scope,ps,e); }
	else if(e instanceof ibm_ilog.diagram.Link)     { return folders.link.call(scope,ps,e); }
	else if(e instanceof ibm_ilog.diagram.Graph)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
    else if(e instanceof ibm_ilog.diagram.graphlayout.GraphAdapter)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.getNodesAndAllLinks(),ns,stopRecursionAtCollapsedSub)); }												 
	else if(e instanceof gfx.Group)            { var ns = folders.accGroup.call(scope,ps,e); 
	                                             return folders.group.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
	else { return folders.other.call(scope,ps,e); }
};

///////////////////////////////////////////////////////////////////////////////
//
// 	fold example 1: logXml(graph)
//

gu.logXml = function(g) {
	
	var acc = function(indent,e) {
		console.log(indent+"<"+e.declaredClass.slice(12)+" id="+(e.getId?e.getId():"?")+">");
		return indent+" ";
	};
	
	var fold = function(parIndent,selfIndent,g) {
		console.log(parIndent+"</"+g.declaredClass.slice(12)+">");
	};
	
	var leaf = function(parIndent,e) {
		console.log(parIndent+"<"+e.declaredClass.slice(12)+" id="+(e.getId?e.getId():"?")+"/>");
	};
	
	var ignore = function(){};

	gu.fold(
		g,
		"", // accumulated forward-state is the indentation string
		{
			graph: fold,
			group: fold,
			sub:   fold,
			node:  leaf,
			link:  leaf,
			other: ignore,
			child: ignore,
			cBase: null,
			accGraph: acc,
			accGroup: acc,
			accSubgraph: acc
		},
		this
	);
};

///////////////////////////////////////////////////////////////////////////////
//
// 	fold example 2: countNodes(graph)
//

gu.countNodes = function(g) {
	
	var add = function(st,siblingSum,childSum) {
		return siblingSum+childSum;
	};
	
	var one = function(){ return 1; };

	var zero = function(){ return 0; };

	var pass = function(ps,ns,g,fc){ return fc; };

	var nothing = function(){};

	return gu.fold(
		g,
		undefined, // forward state not used
		{
			graph: pass,
			group: pass,
			sub:   pass,
			node:  one,
			link:  zero,
			other: zero,
			child: add,
			cBase: 0,
			accGraph: nothing,
			accGroup: nothing,
			accSubgraph: nothing
		},
		this
	);
};


gu.dispatchClass = function(ge,args,context,handlers) {
	return handlers[ge.TID].apply(context,args);
};

gu._howGotHere = function(shape) {
	//
	//	summary:
	//		Debugging helper. Dumps the offsets contributed by 
	//		each transformation in the ancestors of a shape.
	//
	var r, t;
	var print = ibm_ilog.diagram.util.Geometry.printPoint;
	
	while(shape) {
		r = shape._getRealMatrix();
		
		if(t = shape.getTransform && shape.getTransform()) {
			var p = {x:t.dx / t.xx, y:t.dy/t.yy};
			var d = null; 
			
			var p1 = gu.safeGetParent(shape);
			p1 = p1 && p1._getRealMatrix && p1._getRealMatrix();
			p1 = p1 || dojox.gfx.matrix.identity;
			p1 = p1 && p1.transformPoint({x:0,y:0});
			
			var p2 = r.transformPoint({x:0,y:0});
			
			console.log( p1?print({x:p2.x-p1.x,y:p2.y-p1.y}):"(-,-)", print(p));
		} else {
			console.log( "--" );
		}
		
		shape = gu.safeGetParent(shape);
	}
};


})();

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer'] = true;

dojo.provide('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer');











(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var CHECK = ibm_ilog.diagram.util.Assert.check, assert = ibm_ilog.diagram.util.Assert.assert;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;
dojo.declare('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer',ibm_ilog.diagram.overview.OverviewRenderer, {

	//
	//	summary:
	//		Uses custom templates to generate the overview image.
	//
	//	description:
	//		Iterates the given graph and reinstantiates the elements in the graph 
	//		with new templates. If no templates are specified, default ones are used.
	//	
	
	//
	//	node:  Function
	//		The node template function for the overview.
	//
	node: ibm_ilog.diagram.templates.defaultOverviewNodeTemplate,
	
	//
	//	link:  Function
	//		The link template function for the overview.
	//
	link: ibm_ilog.diagram.templates.defaultOverviewLinkTemplate,

	//
	//	expandedSubgraph:  Function
	//		The expanded subgraph template function for the overview.
	//
	expandedSubgraph: ibm_ilog.diagram.templates.defaultOverviewExpandedSubgraphTemplate,

	//
	//	collapsedSubgraph:  Function
	//		The collapsed subgraph template function for the overview.
	//
	collapsedSubgraph: ibm_ilog.diagram.templates.defaultOverviewCollapsedSubgraphTemplate,

	//
	//	stripTexts: Boolean
	//		Optional removal of the texts from the generated shapes.
	//
	stripTexts: true,

	constructor: function( kwArgs ) {
		if(kwArgs) {
			dojo.mixin(this,kwArgs);
		}
		
		this._toFunction("node");
		this._toFunction("link");
		this._toFunction("expandedSubgraph");
		this._toFunction("collapsedSubgraph");

	},
	
	_toFunction: function(name) {
		if(typeof this[name] != "function") {
			var value = this[name];
			this[name] = function(){
				return value;
			};
		}
	},

	templateFor: function(ge) {
		return gu.dispatchClass(ge,arguments,this,{
			N:this.node,
			S:function(ge) {
				return ge.isCollapsed()?this.collapsedSubgraph():this.expandedSubgraph();
			},
      L:this.link
		});
	},
	
	renderGraphElement: function(ge) {
		
		var template = this.templateFor(ge);

		var ng = this._output.createGroup();
		
		if(ge.getTransform){
			var t = ge.getTransform();
			if(t) {
				ng.setTransform(t);
			}
		}

		var r = ge.shallowInstantiateAt(ng,template);
		
		if(!ge._isIBMDiagramLink){
			var oldBB = r.getLocalBoundingBox();
			var newBB = ge.getLocalBoundingBox();
		
			if(dojo.config.useGfxLayout) {
				r.layout(newBB);
			} else {
				r.applyLeftTransform( m.createFromRectangles( oldBB, newBB ));
			}
		} else {
    	ge.layoutShape(r);
		}
	},

	render: function( graph, /*dojox.gfx.Group*/ output ) {
		
		graph = graph || this.graph;
		
		this._output = output;
		
		dojo.forEach(graph.children,this.renderGraphElement,this);

	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.HudComponent']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.HudComponent'] = true;
dojo.provide('ibm_ilog.diagram.widget.HudComponent');



dojo.declare('ibm_ilog.diagram.widget.HudComponent', null, {
	//
	//	summary:
	//		Superclass for Viewport overlaid components.
	//	
	
	//
	//	_layer: dojox.gfx.Group
	//			Viewport layer at which the component is created.
	//
	_layer: null,

	//
	//	_enabled: boolean
	//			True if enabled and visible.
	//
	_enabled: true,

	//
	//	_shape: dojox.gfx.Group
	//			Main shape of the component
	//
	_shape: null,

	constructor: function (viewport) {
		this._enabled = true;
		this._viewport = viewport;
		this._layer = viewport._hudLayer;
		this._shape = this._layer.createGroup();

		viewport.registerHudComponent(this);
	},

	_enable: function () {
		//
		//	summary:
		//			Enables the component and displays it.
		//
		this._enabled = true;
		this._layer.add(this._shape);
	},

	_disable: function () {
		//
		//	summary:
		//			Disables the component and hides it.
		//
		this._layer.remove(this._shape);
		this._enabled = false;
	},

	setEnabled: function ( /*boolean*/ enabled ) {
		//
		//	summary:
		//			Shows or hides the component, disabling possible connections and overhead while hidden.
		//
		if (!this._enabled && enabled) {
			this._enable();
		} else if (this._enabled && !enabled) {
			this._disable();
		}
	},

	isEnabled: function () {
		//
		//	summary:
		//			Returns true of the component is active and visible.
		//
		return this._enabled;
	}

});

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.Overview']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.Overview'] = true;
dojo.provide('ibm_ilog.diagram.overview.Overview');





(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	
	
dojo.declare('ibm_ilog.diagram.overview.Overview', [ibm_ilog.diagram.util.HandleSet], {
//		summary:
//			Draws the overview as a GFX group.
//		
//		architecture:
//			The overview is a mini-diagram with no zooming or panning options that always shows the 
//			entire diagram, like a map.
//	
//			An Overview object draws a map-like representation of the contents of a viewport. It can be used 
//			through the OverviewWidget or directly (for example for using it to draw a component of the Viewport HUD).
//	
//			An Overview relies on an OverviewRenderer which encapsulates the algorithm for generating 
//			the overview image. Different algorithms provide different representation.
//	
//		view rectangle:
//			The Overview also represents the current location of the Viewport with a GFX Rectangle. 
//			Whenever the viewport is repositioned or zoomed, the Viewport object will notify the event, to 
//			which the Overview will react updating a rectangle shape.
//	
//		navigation:
//			See pointers on how the user could interact with the Overview, see the subclasses of OverviewInteractor
//			and how the Overview widget uses them.
//	

	
	//	
	//	_diagram: ibm_ilog.diagram.widget.Viewport
	//		The viewport
	//
	_viewport: null,
	
	//	
	//	_diagram: ibm_ilog.diagram.overview.OverviewRenderer
	//		The overview renderer
	//
	_renderer: null,

	//	
	//	_output: dojox.gfx.Group
	//		The insertion point into the Gfx Surface
	//
	_output: null,
	
	//	
	//	_render: dojox.gfx.Group
	//		Where the overview image is generated
	//
	_render: null,

	//	
	//	_content: Object
	//		The content renderer (usually the Graph).
	//
	_content: null,

	//	
	//	_hud: dojox.gfx.Group
	//		Where the overlay shapes are drawn (e.g. the view rectangie)
	//
	_hud: null,

	//	
	//	_enabled: boolean
	//		When false, the receiver hides its contents and ignores change events. 
	//
	_enabled: true,	

	setEnabled: function( /*boolean*/ enabled ) {
		if(!enabled && this._enabled) {
			this._output.remove(this._hud);
			this._render.clear();
			this._output.remove(this._render);
			if(this._timerHandle) {
				clearTimeout(this._timerHandle);
				this._timerHandle = null;
			}
			this._unsubscribe();
		} else if(enabled && !this._enabled) {
			this._output.add(this._hud);
			this._output.add(this._render);
			this._subscribe();
		}
		this._enabled = enabled;
	},
	
	isEnabled: function() {
			return this._enabled;
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//	Initialization
	//

	constructor: function( /*ibm_ilog.diagram.widget.Viewport*/ viewport,
			                                        /*Object*/ content,
	             /*ibm_ilog.diagram.overview.OverviewRenderer*/ renderer, 
	                                   /*dojox.gfx.Group*/ output, 
	                                              /*Size*/ size )
	{
		//
		//	summary:
		//		Constructs an Overview for observing the given viewport, whose contents will be drawn in an area 
		//		of the given size using the given renderer, and places it at the output gfx Group.
		//
		this._viewport = viewport;
		this._renderer = renderer;
		this._content = content;
		this._size = size;
		this._output = output;
		
		this._render = output.createGroup();

		// create hud group after render group so it is at the front
		this._hud = output.createGroup();
		this._createViewRectShape();
		this._createEventSourceShape();

		this._subscribe();
	},
	
	setContent: function( /*Object*/ content) {
		this._content = content;
	},
	
	getContent: function() {
		return this._content;
	},

	setViewport: function( /*Object*/ viewport) {
		this._viewport = viewport;
	},
	
	getViewport: function() {
		return this._viewport;
	},

	_unsubscribe: function () {
		this._disconnectAll();
	},
	
	_subscribe: function () {
  	this._connect("contentChanged",this._viewport.getContent(),"_onChanged",this,"_onContentChangedNotification");
  	this._connect("viewRectChanged",this._viewport,"onViewRectChanged",this,"_updateViewRectShape");
  	this._onContentChangedNotification();
  	this._updateViewRectShape();
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//	Overview operation
	//

	generateContent: function() {

		//
		//	Draws the overview picture
		//
		
		var render = this._beginRender();
		
		this._renderer.render(this._content,render);
		
		this._endRender(render);
		
		this._adjustContent();

	},

	_beginRender: function() {
		if(this._debug) {
			return this._render;
		} else {
			var render = this._output.createGroup();
			var cr = this._viewport.getContentRect();
	
			// we render offscreen, and move it when it's done
			render.setTransform(dojox.gfx.matrix.translate(ibm_ilog.diagram._invisibleLocation,ibm_ilog.diagram._invisibleLocation));
			render.moveToBack();
			return render;
		}
	},
	
	geViewport: function() {
		return this._viewport;
	},
	
	_endRender: function( render ) {
		if(!this._debug) {
			if(this._render) {
				this._output.remove(this._render);
			}
			this._render = render;
			render.setTransform();
		}
	},

	setRenderer: function( /*ibm_ilog.diagram.overview.OverviewRenderer*/ renderer ) {
		this._renderer = renderer;
	},
	
	getRenderer: function() {
		//
		//	summary:
		//		Returns the current renderer object used at the receiver.
		//
		return this._renderer;
	},

	//	
	//	_zoom:
	//		The current zoom factor applied to the contents displayed by the
	//		overview to be fit in the current overview size.
	//
	_zoom: 1,
	
	getZoom: function() {
		return this._zoom;
	},

	//	
	//	_size: 
	//		The size of the overview
	//
	_size: Size(0,0),
	
	getSize: function() {
		return this._size;
	},

	setSize: function( /*Size*/ sz ) {
	    this._size = sz;
		this._adjustContent();
	},
	
	_adjustContent: function() {
		
		var aabb = this._viewport.getContentRect();
		if(aabb.width && aabb.height) {
			var sz = this._size;
			var xr = sz.width/aabb.width;
			var yr = sz.height/aabb.height;
			var cent = { x: 0, y:0 };
	
			if(xr<yr) {
				this._zoom = xr;
				cent.y = (aabb.width*(sz.height/sz.width)-aabb.height)/2;
			} else {
				this._zoom = yr;
				cent.x = (aabb.height*(sz.width/sz.height)-aabb.width)/2;
			}
	
			var zt = dojox.gfx.matrix.scale(this._zoom);
			var tt = dojox.gfx.matrix.translate({x:-aabb.x+cent.x,y:-aabb.y+cent.y}); // uses rect origin
			var t  = dojox.gfx.matrix.multiply(zt,tt);
			this._render.setTransform(t);
			
			this._updateEventSourceShape();
			this._updateViewRectShape();
		}
		this.onContentReady();
		
	},
	
	onContentReady: function() {
		
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_viewRectShape: 
	//		The GFX rectangle used to represent the view rectangle
	//
	_viewRectShape: null,

	//	
	//	_viewRectShapeFill:
	//		The fill of the view rect shape
	//	
	_viewRectFill:[0xAA,0xD7,0xFF,0.2],
	
	setViewRectFill: function(fill) {
		this._viewRectFill = fill;
		if(this._viewRectShape) {
			this._viewRectShape.setFill(fill);
		}
	},

	//	
	//	_viewRectShapeStroke:
	//		The stroke for the border of the view rect shape
	//
	_viewRectStroke:{color:[0xAA,0xD7,0xFF]},

	setViewRectStroke: function(stroke) {
		this._viewRectStroke = stroke;
		this._updateViewRectShape();
	},
	
	
	_createViewRectShape: function() {
		this._viewRectShape = this._hud.createRect();
		this._viewRectShape.setFill(this._viewRectFill);
		this._updateViewRectShape();
	},

	_updateViewRectShape: function() {
		//
		//	Updates the view rectangle
		//
		if(this._viewRectShape) {
			var t = this._render.getTransform();
			if(t) {
				var vr = g.clipRect(g.standarizeRect(this._size),t.transformRectangle(this._viewport.getViewRect()));
				this._viewRectShape.setShape(vr);
				this._viewRectShape.setStroke({color:this._viewRectStroke.color,width:(this._viewRectStroke.width||1)});

			}
		}
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_eventSourceShape:
	//		A shape used as an event generator for interaction. Current implementation uses a invisible rectangle 
	//		(created with alpha 0.0001) that catches mouse events in the entire group area.
	//
	_eventSourceShape: null,

	getEventSource: function() {
		return this._eventSourceShape;
	},

	_createEventSourceShape: function() {
		this._eventSourceShape = this._hud.createRect();
		this._eventSourceShape.setFill([0,0,0,0.0001]);
		this._updateEventSourceShape();
	},
	
	_updateEventSourceShape: function() {
		//	
		//	Updates the event source shape size to the current viewport navigable area
		//
		if(this._eventSourceShape) {
			this._eventSourceShape.setShape(g.standarizeRect(this._size));
		}
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_timerHandle:
	//		The timer scheduled by the refreshContentNotification request
	//
	_timerHandle:null,
	
	//
	//	_timerInterval:
	//		Refresh rate used for monitoring changes in the content.
	//
	_timerInterval: 1500,

	//
	//	_timerWaitForInteractions: boolean
	//		if true, the rendering timer waits for interactions to finish
	//
	_timerWaitForInteractions: true,

	setWaitForInteractions: function( bool ) {
		this._timerWaitForInteractions = bool ;
	},
	
	getWaitForInteractions: function() {
		return this._timerWaitForInteractions;
	},

	//
	//	_interactorManager: ibm_ilog.diagram.interactors.InteractorManager
	//		the interactor manager to wait for
	//
	_interactorManager: null,

	setInteractorManager: function( interactorManager ) {
		this._interactorManager = interactorManager;
	},
	
	getInteractorManager: function( interactorManager ) {
		return this._interactorManager;
	},

	setRefreshContentsInverval: function( /*Number*/ msInterval ) {
		this._timerInterval = msInterval;
		if(msInterval<0 && this._timerHandle) {
			clearTimeout(this._timerHandle);
			this._timerHandle = null;
		}
	},
	
	getRefreshContentsInverval: function() {
		return this._timerInterval;
	},

	_onContentChangedNotification: function(){
		//
		//	summary:
		//		Sets up a timer to give the originating process time to finish with all the changes
		//
		if(this._timerInterval>=0 && !this._timerHandle) {
			this._timerHandle = 
				setTimeout(dojo.hitch(this,this._timer),this._timerInterval);
		}
	},
	
	_timer: function() {
		//
		//	summary:
		//		Callback executed after the contentChangedNotification timer is finished.
		//
		this._timerHandle = null;
		if(this._timerWaitForInteractions && this._interactorManager && this._interactorManager.stackLength()) {
			this._onContentChangedNotification();
		} else {
			this.generateContent();
		}
	},

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	clientToContent:
	//		
	//
	clientToContent: function() {
		return this._render._getRealMatrix().inverse();
	},


	contentToClient: function() {
		return this._render._getRealMatrix();
	},

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	__eod:undefined
	
});


})();

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.OverviewInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.OverviewInteractor'] = true;
dojo.provide('ibm_ilog.diagram.overview.OverviewInteractor');





(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

	ibm_ilog.diagram.overview.OverviewInteractorMixin = {

		//
		//	summary:
		//		Abstract class with the basic behavior of an interactor which handles input into an Overview.
		//
		
		//
		//	_viewport: ibm_ilog.diagram.widget.Viewport
		//		The diagram viewport
		//
		_viewport: null,

		//
		//	_viewport: ibm_ilog.diagram.overview
		//		The overview
		//
		_overview: null,
		
		//
		//	_surface: dojox.gfx.Surface
		//		The surface
		//
		_surface: null,

		initialize: function( /*ibm_ilog.diagram.widget.Viewport*/viewport, /*ibm_ilog.diagram.overview*/ overview, /*dojox.gfx.Surface*/ surface ) {
			this._viewport = viewport;
			this._overview = overview;
			this._surface = surface;
			this._initialize();
			return this;
		},
		
		eventSurfaceLocation: function (e) {
			//
			//	Returns the client location of a mouse event generated by the event source
			//
			var r = dojo.coords(this._surface._parent, true);
			return {
				x: e.pageX - r.x,
				y: e.pageY - r.y
			};
		},		

		eventViewportLocation: function( /*Event*/ e ) {
			//
			//	summary:
			//		Returns the event location in content coordinates
			//
			var loc = this.eventSurfaceLocation(e);
			return this._overview.clientToContent().transformPoint(loc);
		}

	};
	
	dojo.declare("ibm_ilog.diagram.overview.OverviewInteractor", [ibm_ilog.diagram.interactors.Interactor],ibm_ilog.diagram.overview.OverviewInteractorMixin);

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.OverviewZoomInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.OverviewZoomInteractor'] = true;
dojo.provide('ibm_ilog.diagram.overview.OverviewZoomInteractor');





(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

dojo.declare("ibm_ilog.diagram.overview.OverviewZoomInteractor", [ibm_ilog.diagram.overview.OverviewInteractor], {
	//
	//	summary:
	//		A ZoomInteractor controls the process of viewport panning and zooming
	//
	//	_incrementFactor: Number
	//		The factor at which the zoom is incremented at each discrete step
	//
	_incrementFactor: 1.5,



	setIncrementFactor: function ( /*Number*/ factor) {
		//
		//	summary:
		//		Sets the increment factor for zooming discrete operations
		//
		this._incrementFactor = factor;
	},

	getIncrementFactor: function () {
		//
		//	summary:
		//		Returns the increment factor for zooming discrete operations
		//
		return this._incrementFactor;
	},

	_declareStates: function () {
		this._declareState("idle", ["wheelZoom"]);
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		Returns the default event mapping
		//
		return {
			wheelZoom: {
				src: this._overview.getEventSource(),
				srcEvt: this.eventNameWheel,
				connectTo: "_wheelZoom",
				filter: this._noneFilter,
				gfxConnect: true
			}
		};
	},

	_wheelZoom: function ( /*Event*/ e) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatchs the appropriate zooming operations
		//
		var v = this._incrementFactor;

		if (this.eventWheelAmmount(e) < 0) {
			v = 1 / v;
		}

		this._viewport.setZoom(this._viewport.getZoom() * v);

		if (e.preventDefault) {
			e.preventDefault();
		}

		//	TODO: do we need this?
		e.returnValue = false;
	}

});


})();

}

if(!dojo._hasResource['ibm_ilog.diagram.overview.OverviewPanInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.overview.OverviewPanInteractor'] = true;
dojo.provide('ibm_ilog.diagram.overview.OverviewPanInteractor');







(function() {
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry,	 Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	
dojo.declare("ibm_ilog.diagram.overview.OverviewPanInteractor",[ibm_ilog.diagram.interactors.DragInteractor], {
	//
	//	summary:
	//		An OverviewPanInteractor pans the viewport when the user drags on 
	//		the overview and centers the viewport on a point when the user clicks at 
	//		that point.	//

	_dragCursor: "move",


	//
	//	_constrain: Boolean
	//		Constrains view rectangle movements to the viewport's content area
	//
	_constrain: true,
	
	setConstrain: function( /*boolean*/ b ) {
		this._constrain = b;
	},
	
	getConstrain: function() {
		return this._constrain;
	},
	
	_initialize: function() {
		this.inherited(arguments);
		this._cursorStyle = gu.getSurface(this._overview._output).rawNode.style;
	},
	
	_declareStates: function () {
		this._declareState("idle", ["fit","start","mouseenter","mouseleave"]);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart","mouseenter","mouseleave"]);
	},
	
	getDefaultConnections: function() {
		var cs = this.inherited(arguments);
		cs.fit = {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_fitContents",
				filter: this._buildInputFilter({shift:true,button:0}),
				gfxConnect:true
			};
		cs.mouseenter = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseenter",
				connectTo: "_hover",
				gfxConnect:true
			};
		cs.mouseleave = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseleave",
				connectTo: "_leave",
				gfxConnect:true
			};
		return cs;
	},
	
	_updateCursor: function() {
		if(this._hovered || this._state._id=="active") {
			this._showDragCursor();
		} else {
			this._hideDragCursor();
		}
	},
	
	_hover: function() {
		this._hovered = true;
		this._updateCursor();
	},
	
	_leave: function() {
		this._hovered = false;
		this._updateCursor();
	},

	_getInitialEventSource: function() {
		return this._overview.getEventSource(); 
	},
	
	_fitContents: function(e) {
		this._viewport.setViewRect(this._viewport.getContentRect());
	},

	_dragPreStart: function(e) {
		this._cursorStyle = this._cursorStyle || gu.getSurface(this._overview._output);
		var v = this._viewport;
		this.inherited(arguments);
		this._firstViewRect = v.getViewRect();
	},
    
	_dragMove: function(e) {
    this.inherited(arguments);

		// scale delta to zoom level to get delta in model coords
		var panDelta = g.mulPoint(this._totalDelta,1/this._overview.getZoom());
		
		var newViewRect = this._constrainedViewRect(g.moveRect(this._firstViewRect,panDelta));
		
		this._viewport.setViewRect( newViewRect, {instant:true} );
  },
	
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current panning process.
		//
		var v = this._viewport;
		var dragged = this.hasDragged; // inherited will clear this;
		this.inherited(arguments); // clears hadDragged
		if(!dragged) {
			
			var newViewRect = this._constrainedViewRect(g.rectCenteredAt(this._viewport.getViewRect(), this.eventViewportLocation(e)));

			this._viewport.setViewRect( newViewRect );
		}
	},

  _constrainedViewRect: function( viewRect ) {
	  if(this._constrain) {
			viewRect = g.constrainRect(viewRect,g.addRect(this._viewport.getContentRect(),this._viewport.getViewRect()));
		}
	  return viewRect;
  }

});

// [AV] moved this superclass to a dojo.extend because it does not seem to work properly on dojo 1.3.2 
dojo.extend(ibm_ilog.diagram.overview.OverviewPanInteractor,ibm_ilog.diagram.overview.OverviewInteractorMixin);

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.InteractorManager']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.InteractorManager'] = true;
dojo.provide('ibm_ilog.diagram.interactors.InteractorManager');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var _Set = dojox.collections.ArrayList;
	var dxs = dojox.collections.Set;

	dojo.declare("ibm_ilog.diagram.interactors.InteractorManager", [], {
	//
	//	summary:
	//		Organizes interactors by id and is responible for distinguishing 
	//		the currently active interactors.
	//
	
	//
	//	_interactors: 
	//		Registered interactors
	_interactors:null,
	
	//
	//	_stack: 
	//		Holds active interaction (currently only one level is used)
	_stack:null,
	
	//
	//	_base: 
	//		Idle interactors, which are active when _stack is empty
	_base:null,
	
	constructor:function(){
		//
		//	summary:
		//		creates a new instance of InteractorManager
		//
		this._interactors = new dojox.collections.Dictionary();
		this._base = this._gatherInteractors([]);
		this._stack = [];
	},
	
	add:function(id,interactor){
		//
		//	summary:
		//		register a new interactor with the given id
		//
		this._interactors.add(id,interactor);
		interactor._manager = this;
	},

	get:function(id){
		//
		//	summary:
		//		Returns the interactor registered with the given id
		//
		var interactor = this._interactors.item(id);
		if(!interactor) {
			R.error("InvalidInteractorId",id);
		}
		return interactor;
	},
	
	top:function(){
		//
		//	summary:
		//		Returns the top level of the stack
		//
		return this._stack[this._stack.length-1];
	},

	activeInteractors:function(){
		//
		//	summary:
		//		Returns a set with the currently active interactors
		//
		return this._stack.length?this.top().interactors:this._base;
	},

	_createStackObject:function(interactors){
		//
		//	summary:
		//		Returns a new stack object (also used as handle for popping)
		//
		return {interactors:interactors, active:true, index:this._stack.length};
	},

	_activate: function() {
		//
		//	summary:
		//		Activate the active set (currently: either _base or _stack[0])
		//
		this.activeInteractors().forEach(function(i){
			i.activate();
		},this);
	},

	_deactivate: function() {
		//
		//	summary:
		//		Deactivate the active set (currently: either _base or _stack[0])
		//
		this.activeInteractors().forEach(function(i){
			i.deactivate();
		},this);
	},
	
	_gatherInteractors: function( /*array*/ maybeNames ) {
		//
		//	summary:
		//		Gather a set of interactor that are listed explicitly of 
		//      by name into a dojox.collections.Set
		//
		var interactors = new _Set();
		dojo.forEach(maybeNames,function(i){
			if(typeof i=="string") {
				interactors.add(this.get(i));
			} else if (i instanceof ibm_ilog.diagram.interactors.Interactor) {
				interactors.add(i);
			} else {
				R.error("UnexpectedArgument");
			}
		},this);
		return interactors;
	},	
	
	switchTo:function(interactor1,interactor2,etc){
		//
		//	summary:
		//		Changes the set of idle interactors that are ready to be used. 
		//		Has immediate effect only when there are no elements in the active _stack.
		//
		var interactors = this._gatherInteractors(arguments); 

		if(!this._stack.length) {
			this._deactivate();
			this._base = interactors;
			this._activate();
		} else {
			this._base = interactors;
		}
	},
	
	push:function(interactor1,interactor2,etc){
		//
		//	summary:
		//		Push a set of interactors into the active stack. (Although currently only 1 is supported)
		//
		if(this._stack.length) {
			R.error("RestrictedToOneInteractor");
		}
		
		var interactors = this._gatherInteractors(arguments);
		var keep = [];
		var old = this.activeInteractors();
		var so = this._createStackObject(interactors);
		this._stack.push(so);
		var n = this.activeInteractors();
		dxs.difference(old,n).forEach(function(i){
			i.deactivate();
		});
		
		return this.top(); // this is the handle for deactivation
	},

	pop:function(h){
		//
		//	summary:
		//		Deactivates and eventually pops the interactors previously pushed on the active stack
		//
		if(h.active) {
			h.active = false;
			if(h == this.top()) {
				var old = this.activeInteractors();
				var i = h.index;
				while(i>=0 && !this._stack[i].active) {
					this._stack.pop();
					i--;
				}
				var n = this.activeInteractors();

				dxs.difference(n,old).forEach(function(i){
					i.activate();
				});
			}
		}
	},
	
	stackLength: function(){
		return this._stack.length;
	},

	__eod:undefined

});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.OverviewHudComponent']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.OverviewHudComponent'] = true;
dojo.provide('ibm_ilog.diagram.widget.OverviewHudComponent');








(function () {
	var g = ibm_ilog.diagram.util.Geometry;

	dojo.declare('ibm_ilog.diagram.widget.OverviewHudComponent', ibm_ilog.diagram.widget.HudComponent, {

		//
		//	summary:
		//		A Viewport component holding an overview.
		//

		//
		//	_interactors: ibm_ilog.diagram.interactors.InteractorManager
		//		The InteractorManager used for the interactions in the component
		//
		_interactors: null,

		//
		//	_background: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_background: null,

		//
		//	_size: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_size: g.Size(0, 0),

		//
		//	_overview: ibm_ilog.diagram.overview.Overview
		//		The Overview object used in the component.
		//
		_overview: null,

		constructor: function (viewport, content, renderer, size) {
			//
			//	summary:
			//		Construct the component in the given viewport, with the given renderer, with the given size.
			//
			this._size = size;
			this._shape.setTransform(dojox.gfx.matrix.translate(ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleLocation));
			this._background = this._shape.createRect(size).setFill([0, 0, 0, 0.5]);

			this._overview = new ibm_ilog.diagram.overview.Overview(viewport, content, renderer, this._shape.createGroup(), this._size);
			this._viewport = viewport;
			this._update();

			this._interactors = new ibm_ilog.diagram.interactors.InteractorManager();
			this._interactors.add("pan", new ibm_ilog.diagram.overview.OverviewPanInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.add("zoom", new ibm_ilog.diagram.overview.OverviewZoomInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.switchTo("pan", "zoom");

			dojo.connect(viewport, "onClientSizeChanged", this, "_update");
			dojo.connect(viewport, "onLayoutHud", this, "_update");
		},
		
		
		setPan: function( b ) {
			//
			//	summary:
			//		Activate or deactivate panning in the overview
			//
			this._interactors.get("pan").setEnabled(b);
		},
		
		getPan: function() {
			//
			//	Returns:
			//		True if the receiver is activated for panning
			//
			this._interactors.get("pan").getEnabled();
		},
		
		setZoom: function( b ) {
			//
			//	summary:
			//		Activate or deactivate zooming in the overview
			//
			this._interactors.get("zoom").setEnabled(b);
		},
		
		getZoom: function() {
			//
			//	Returns:
			//		True if the receiver is activated for zooming
			//
			this._interactors.get("zoom").getEnabled();
		},
		
		getOverview: function() {
			//
			//	returns: ibm_ilog.diagram.overview.Overview
			//			The Overview object
			//
			return this._overview;
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._overview.setEnabled(true);
			this._update();
		},

		_disable: function () {
			//
			//	summary:
			//			Disables the component and hides it.
			//
			this._overview.setEnabled(false);
			this.inherited(arguments);
		},

		setSize: function (size) {
			//
			//	summary:
			//		Resizes the component
			//
			this._size = size;
			this._update();
		},

		getSize: function () {
			//
			//	summary:
			//		Returns the size of the component.
			//
			return this._size;
		},

		_update: function () {
			//
			//	summary:
			//		Updates the size and location of the component after resize of the 
			//		viewport or the overview component.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var sz = this._size;
				var hsz = this._viewport.getHudSize();
				this._overview.setSize(sz);
				this._background.setShape(sz);

				var location = { y: hsz.height - sz.height, x:0 };
				
				if(!this._viewport.isRtl()){
					location.x = hsz.width - sz.width;
				}

				this._shape.setTransform(dojox.gfx.matrix.translate(location));
			}
		}

	});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.ZoomHudComponent']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.ZoomHudComponent'] = true;
dojo.provide('ibm_ilog.diagram.widget.ZoomHudComponent');



(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter
	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;

	dojo.declare('ibm_ilog.diagram.widget.ZoomHudComponent', ibm_ilog.diagram.widget.HudComponent, {
		//
		//	summary:
		//		A viewport component that displays the current zoom factor.
		//

		//
		//	_currentZoom:
		//		The current zoom factor being displayed at the component
		//
		_currentZoom: 0,

		constructor: function (viewport) {
			this._shape.setTransform(dojox.gfx.matrix.translate(ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleLocation));
			this._shape.createRect({
				width: 30,
				height: 20
			}).setFill([0, 0, 0, 0.5]);
			this._shape.createText({
				x: 15,
				y: 15,
				align: "middle",
				text: "x" + ibm_ilog.diagram.util._round(this._zoom, 1)
			}).setFont({
				size: "10pt",
				family: "sans-serif"
			}).setStroke("white").setFill("white");
			this._updateLayout();
			this._updateValue();

			dojo.connect(viewport, "onLayoutHud", this, "_updateLayout");
			dojo.connect(viewport, "onViewRectChanged", this, "_updateValue");
			//dojo.connect(viewport,"onClientSizeChanged",this,"_updateLayout");
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._updateLayout();
			this._updateValue();
		},

		_updateValue: function () {
			//
			//	summary:
			//		Updates the value displayed at the component
			//
			if (this._enabled && this._viewport.isHudEnabled() && (this._viewport.getZoom() !== this._currentZoom)) {
				this._currentZoom = this._viewport.getZoom();
				this._shape.children[1].setShape({
					text: "x" + ibm_ilog.diagram.util._round(this._currentZoom, 1)
				});
			}
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var hudSize = this._viewport.getHudSize();
				var location;
				if(!this._viewport.isRtl()) {
					location = Point(hudSize.width - 30, 0);
				} else {
					location = Point(0,0);
				}
				this._shape.setTransform(dojox.gfx.matrix.translate(location));
			}
		}
	});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.PanInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.PanInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.PanInteractor');







(function () {
	var g = ibm_ilog.diagram.util.Geometry;

dojo.declare("ibm_ilog.diagram.interactors.PanInteractor", [ibm_ilog.diagram.interactors.DragInteractor], {
	//
	//	summary:
	//		A PanInteractor controls the process of viewport panning
	//
	//
	//	_panViewStart: Rect
	//		initial position of the view rectangle when the current pan process started
	//	
	_panViewLast: null,

	_dragCursor: "move",

	_getInitialEventSource: function () {
		return this._viewport.getEventSource();
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		var cs = this.inherited(arguments);
		cs.start.filter = this._buildInputFilter({shift:true,button:0});
		return cs;
	},

	_dragPreStart: function (e) {

		this.inherited(arguments);

		this._firstViewRect = this._viewport.getViewRect();

		var v = this._viewport;
	},

	_dragMove: function (e) {

		this.inherited(arguments);

		var v = this._viewport;

		var viewMovement = g.mulPoint(this._totalDelta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		this._panViewLast = v.setViewRect(g.moveRect(this._firstViewRect, viewMovement), {instant:true});
	},

	_dragEnd: function (e) {
		var v = this._viewport;

		this.inherited(arguments);
	},

	_dragTouchPreStart: function (e) {

		this.inherited(arguments);

		this._firstViewRect = this._viewport.getViewRect();

	},

	_dragTouchMove: function (e) {
		this.inherited(arguments);
		if (e.touches.length > 1){
			return false;
		}
		
		var v = this._viewport;

		var viewMovement = g.mulPoint(this._totalDelta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		this._panViewLast = v.setViewRect(g.moveRect(this._firstViewRect, viewMovement), {instant:true});
	},

	_dragTouchEnd: function (e) {
		this.inherited(arguments);
		if (e.touches.length > 0){
			this._firstViewRect = this._viewport.getViewRect();			
			return false;
		}
	},

	_setOnShapes: function ( /*Boolean*/ b) {
		// [AV] TODO this method needs refactoring
		if (b) {
			// background and shapes
			this._connections.start.src = this._viewport.getEventSource();
		} else {
			// only background
			this._connections.start.src = this._viewport.getEventSourceBackground();
		}
	},

	setOnShapes: function ( /*Boolean*/ b) {
		this._setOnShapes(b);
		this.reactivate();
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.ZoomInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.ZoomInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.ZoomInteractor');




(function(){
	var g = ibm_ilog.diagram.util.Geometry;

dojo.declare("ibm_ilog.diagram.interactors.ZoomInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//
	//	summary:
	//		A ZoomInteractor controls the process of viewport panning and zooming
	//

	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram to pan
	//
	_diagram: null,
	
	//
	//	_incrementFactor: Number
	//		The factor at which the zoom is incremented at each discrete step
	//
	_incrementFactor: 1.5,
	
	_p0X:0,_p0Y:0,_p1X:0,_p1Y:0,

	_startDistance: 0,
	
	_declareStates: function() {
		var idleStates = ["wheelZoom"];
		if (ibm_ilog.diagram.mobileEnabled)
			idleStates.push("touchStart");
		this._declareState("idle",idleStates);
		if (ibm_ilog.diagram.mobileEnabled)
			this._declareState("activeTouch",["touchMove","touchEnd"]);
	},
	
	activate: function() {
		this._goStateId("idle");
	},
	
	setIncrementFactor: function( /*Number*/ factor ) {
		//
		//	summary:
		//		Sets the increment factor for zooming discrete operations
		//
		this._incrementFactor = factor;
	},
	
	getIncrementFactor: function() {
		//
		//	summary:
		//		Returns the increment factor for zooming discrete operations
		//
		return this._incrementFactor;
	},
	
    initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Zoom interactor constructor, based on a target Diagram widget.
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},
    
    getDefaultConnections:function(){
		//
		//	summary:
		//		Returns the default event mapping
		//
    	var defaultConnections = {
    		wheelZoom:{
    			src:this._diagram.domNode,
    			srcEvt:this.eventNameWheel,
    			connectTo:"_wheelZoom",
    			filter:this._buildInputFilter(dojo.isWebKit?{shift:true}:{ctrl:true})}
    	};
    	
		if (ibm_ilog.diagram.mobileEnabled){
			var mobileDefaultConnections = {
				touchStart : {
					src: this._diagram.domNode,
					srcEvt: "ontouchstart",
					connectTo: "_touchStart"
				}, touchMove :{
					src: this._diagram.domNode,
					srcEvt: "ontouchmove",
					connectTo: "_touchMove"
				}, touchEnd: {
					src: this._diagram.domNode,
					srcEvt: "ontouchend",
					connectTo: "_touchEnd"
				}

			};
			dojo.mixin(defaultConnections,mobileDefaultConnections);
		};
    	return defaultConnections;
    },
    
    _wheelZoom: function( /*Event*/ e ) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatchs the appropriate zooming operations
		//

		var v = this._incrementFactor;
		
		if(this.eventWheelAmmount(e)<0) {
			v = 1/v;
		}

		this._zoomAt(this._viewport.eventClientLocation(e),v);

		if (e.preventDefault) {
			e.preventDefault();
		}
		//	TODO: do we need this?
		e.returnValue = false;

    },
    
    _zoomAt: function( /*Point*/ p, /*Number*/z ) {
		//
		//	summary:
		//		Performs a zoom operation maitaining a pivot point p in the same 
    	//		fixed position relative to the viewport.
		//
    	var V = this._viewport;
    	var cp = V.pointClientToContent(p); // content p    	
    	V.zoomAt(cp,z);
    },
    
    _touchStart: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length != 2)
    		return false;
    	
    	this._p0X = e.touches[0].pageX;
    	this._p0Y = e.touches[0].pageY;
    	this._p1X = e.touches[1].pageX;
    	this._p1Y = e.touches[1].pageY;
    	
    	this._startDistance = Math.sqrt(Math.pow(this._p1X - this._p0X,2) + Math.pow(this._p1Y - this._p0Y,2));
    	
		this._goStateId("activeTouch");
		
    },
    _touchMove: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length < 2)
    		return false;
    	
    	this._p0X = e.touches[0].pageX;
    	this._p0Y = e.touches[0].pageY;
    	this._p1X = e.touches[1].pageX;
    	this._p1Y = e.touches[1].pageY;
    	
    	var distance = Math.sqrt(Math.pow(this._p1X - this._p0X,2) + Math.pow(this._p1Y - this._p0Y,2));
    	var scale = distance / this._startDistance;
    	this._startDistance = distance;
    	
    	var p = {
        		pageX : (this._p0X + this._p1X)/2,
        		pageY : (this._p0Y + this._p1Y)/2
        	};
        	
    	this._zoomAt(this._viewport.eventClientLocation(p),scale);
    	
    },
    _touchEnd: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length < 2)
    		this._goStateId("idle");
    	return false;
    	
    }

    
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.Selection']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.Selection'] = true;
dojo.provide('ibm_ilog.diagram.Selection');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var gu = ibm_ilog.diagram.util.GraphUtil; 

dojo.declare("ibm_ilog.diagram.Selection", [], {
	
	_diagram: null,
	_selectionStrategy: null,
	_multiSelectionEnabled: null,
	_nodeSelectionEnabled: null,
	_linkSelectionEnabled: null,
	
	constructor: function(diagram,options) {
		// summary:
		//		creates a new instance
		this._multiSelectionEnabled = true;
		this._nodeSelectionEnabled = true;
		this._linkSelectionEnabled = true;
		this._diagram = diagram;	
		options = options || diagram || {selectionMode:ibm_ilog.diagram.Selection.standard};
		this.instantiateSelectionStrategy(diagram,options);
	},
	
	instantiateSelectionStrategy: function(diagram,options){
		var useIndividualConnect = !diagram!=null;
		
		switch( options.selectionMode){
			case ibm_ilog.diagram.Selection.standard:{
				if(diagram && diagram.useDataStore()){
					this._selectionStrategy = new ibm_ilog.diagram.StoreSelectionStrategy(this,useIndividualConnect);
				}else{
					this._selectionStrategy = new ibm_ilog.diagram.GFXSelectionStrategy(this,useIndividualConnect);
				}
				this.setMultiSelectionEnabled(true);
				break;}
			case ibm_ilog.diagram.Selection.gfx_single: {
				this._selectionStrategy = new ibm_ilog.diagram.GFXSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(false);
				break;}
			case ibm_ilog.diagram.Selection.gfx_multi: {
				this._selectionStrategy = new ibm_ilog.diagram.GFXSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(true);
				break;}
			case ibm_ilog.diagram.Selection.dataStore_single: {
				this._selectionStrategy = new ibm_ilog.diagram.StoreSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(false);
				break;}
			case ibm_ilog.diagram.Selection.dataStore_multi: {
				this._selectionStrategy = new ibm_ilog.diagram.StoreSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(true);
				break;}		
		}
		if(!useIndividualConnect){
			dojo.connect(diagram,'beforeGraphElementDispose',this,'onElementDisposed');
		}
		
	},
	add: function(elements,forceClear){
		// summary:
		// 		sets the desired as selected, and add it to the selected list (if not in
		// 		the list yet). Also connect the element to react to the selectableChanged method
		// elements: a single element or an array of elements
		var removed = [];
		if((!this._multiSelectionEnabled && this.get().count>0) || forceClear == true){
			removed = this._clear();
		}
		if(!dojo.isArray(elements)){
			elements = [elements];
		}
		var added = [];
		for( itemIndex in elements ){ 
		 		var item = elements[itemIndex];
				var rta = this._selectionStrategy.add(item);
				if(rta){
					added.push(rta);
				}		
		 }
		
		if(added.length>0 || removed.length>0){
			this.onSelectionChanged(added,removed);
			return true;
		}else{
			return false;
		}
	},
	remove: function(elements,isDisposed){
		// summary:
		// 		sets the desired as unselected, and remove it from the selected list (if in
		// 		the list).
		// elements: a single element or an array of elements
		if(!dojo.isArray(elements)){
			elements = [elements];
		}
		var removed = [];
		for( itemIndex in elements ){ 
		 		var item = elements[itemIndex];
				var rta = this._selectionStrategy.remove(item,isDisposed);
				if(rta){
					removed.push(item);
				}		
		 }
		
		if(removed.length>0){
			this.onSelectionChanged([],removed);
			return true;
		}else{
			return false;
		}
	},
	_clear: function(){
		// summary:
		// sets the selection state of all the elements in the selected list as
		// false, and clear the list. Return an array with the removed elements
		var removed = this._selectionStrategy.get().toArray();
		this._selectionStrategy.clear();
		return removed;
	},
	clear: function() {
		// summary:
		// sets the selection state of all the elements in the selected list as
		// false, and clear the list. If any element is removed from the selection, a notification is thrown.
		var removed = this._clear();
		if(removed.length>0){
			this.onSelectionChanged([],removed);
		}
	},
	get: function(){
		// summary:
		// gets the ArrayList of selected elements
		return this._selectionStrategy.get().clone();
	},
	fastGet: function(){
		// summary:
		// gets the ArrayList of selected elements
		return this._selectionStrategy.get();
	},
	onSelectionChanged: function(added,removed){
		
	},
	getMultiSelectionEnabled: function(){
		// summary:
		// gets if multi-Selection is enabled (true by default)
		return this._multiSelectionEnabled;
	},
	setMultiSelectionEnabled: function(enabled){
		// summary:
		// sets if multi-Selection is enabled (true by default)
		this._multiSelectionEnabled = enabled;
	},
	getNodeSelectionEnabled: function(){
		// summary:
		// gets if Node Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		return this._nodeSelectionEnabled;
	},
	setNodeSelectionEnabled: function(enabled){
		// summary:
		// sets if node Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		this._nodeSelectionEnabled = enabled;
	},
	getLinkSelectionEnabled: function(){
		// summary:
		// gets if link Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		return this._linkSelectionEnabled;
	},
	setLinkSelectionEnabled: function(enabled){
		// summary:
		// sets if link Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		this._linkSelectionEnabled = enabled;
	},
	_getDataItem: function(element){
		//if(element['isInstanceOf'] && element.isInstanceOf(ibm_ilog.diagram.Selectable)){
		if(ibm_ilog.diagram.Selectable.Util.isSelectableInstance(element)){
			return this._diagram.getBoundItem(element);
		}else{
			return element;
		}
	},
	_asGraphElement: function(element){
		//if(!element['isInstanceOf'] || !element.isInstanceOf(ibm_ilog.diagram.Selectable)){
		if(!ibm_ilog.diagram.Selectable.Util.isSelectableInstance(element)){
			return this._diagram.getGraphElementFromDataItem(element);
		}else{
			return element;
		}
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		eg1: an element with no data item cannot be added to a StoreSelectionStrategy
		//		eg: a link is trying to be added to the selection but link selection is disabled
		var ge = this._asGraphElement(element);
		return ((ge._isIBMDiagramNode && this._nodeSelectionEnabled) || (ge._isIBMDiagramLink && this._linkSelectionEnabled)) && this._selectionStrategy.accept(element);
	},
	onElementDisposed: function(graphElement) {
		this.remove(graphElement,true);
	},
	selectAll: function(graph,options) {
		//
		//	summary: 
		//		select all elements in a graph 
		//	graph:
		//		the graph containing all the items to be selected
		//	options:
		//		options.noCheck: if true, skip checking each item is accepted by the receiver
		//
		graph = graph || this._diagram.getGraph();
		
		var selectables = gu.allGraphElements(graph.children).toArray()
	
		if(!options || !options.noCheck) {
			selectables = dojo.filter(selectables,dojo.hitch(this,"accept"));
		}
		this.add(selectables,true);
	},	
	__eod: undefined
});

dojo.declare("ibm_ilog.diagram.AbstractSelectionStrategy", [], {
	_selectedHandle: null,
	_selectedElements: null,
	_selection: null,
	_individualConnect: null,
	
	constructor: function(selection,useIndividualConnect) {
		this._selection = selection;
		this._individualConnect = useIndividualConnect;
		this._selectedElements = new dojox.collections.ArrayList();
		if(this._individualConnect){
			this._selectedHandle = new dojox.collections.ArrayList();
		}
	},
	add: function(element){
	
	},
	remove: function(element,isDisposed){
		
	},
	disposeRemove: function(element){
		
	},
	get: function(){
		return this._selectedElements;
	},
	clear: function(){
		while(this._selectedElements.count>0){
			var element = this._selectedElements.item(this._selectedElements.count-1);
			this.remove(element);
		}
	},
	_addElement: function(element){
		this._selectedElements.add(element);
		var ge = this._selection._asGraphElement(element);
		if (this._individualConnect) {
			this._selectedHandle.add(dojo.connect(ge, 'beforeDisposing', this._selection, 'onElementDisposed'));
		}
	},
	_removeElement: function(element){
		// summary:
		//		remove the elements from the list, disconnect it and remove the handle.
			var index =  this._selectedElements.indexOf(element);
			this._selectedElements.removeAt(index);
			if (this._individualConnect) {
				dojo.disconnect(this._selectedHandle.item(index));
				this._selectedHandle.removeAt(index);
			}
	}
});

dojo.declare("ibm_ilog.diagram.GFXSelectionStrategy", [ibm_ilog.diagram.AbstractSelectionStrategy], {
	add: function(element){
		var ge = this._selection._asGraphElement(element);
		if(!this._selectedElements.contains(ge)){
			if(ge.setSelected(true)){
				this._addElement(ge);
				return ge;
			}
		}
		return null;
	},
	remove: function(element,isDisposed){
		var ge = this._selection._asGraphElement(element);
		if (this._selectedElements.contains(ge)) {
			if(!isDisposed){
				ge.setSelected(false);
			}
			this._removeElement(element);
			return ge;
		}
		return null;
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		(eg: an element with no data item cannot be added to a StoreSelectionStrategy)
		
		return true;
	}
});

dojo.declare("ibm_ilog.diagram.StoreSelectionStrategy", [ibm_ilog.diagram.AbstractSelectionStrategy], {
	add: function(element){
		var ge = this._selection._asGraphElement(element);
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			R.error("NotInDataStore");
		}
		if(!this._selectedElements.contains(dataItem)){
			if(ge == null || ge.setSelected(true)){
				this._addElement(dataItem);
				var dataItem = this._selection._getDataItem(element);
				return dataItem;	
			}
		}
		return null;
	},
	remove: function(element,isDisposed){
		var ge = this._selection._asGraphElement(element);
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			R.error("NotInDataStore");
		}
		if (this._selectedElements.contains(dataItem)) {
			if (ge != null && !isDisposed) {
				ge.setSelected(false);
			}
			this._removeElement(dataItem);
			return dataItem;
		}
		return null;
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		(eg: an element with no data item cannot be added to a StoreSelectionStrategy)
		
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			return false;
		}
		return true;
	}
});

ibm_ilog.diagram.Selection.standard = 0;
ibm_ilog.diagram.Selection.dataStore_single = 1;
ibm_ilog.diagram.Selection.dataStore_multi = 2;
ibm_ilog.diagram.Selection.gfx_single = 3;
ibm_ilog.diagram.Selection.gfx_multi = 4;


})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.SelectionInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.SelectionInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.SelectionInteractor');






dojo.declare("ibm_ilog.diagram.interactors.SelectionInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This function provides the functionality to manage selection events on links and nodes.
	//the selection object
	_selection: null,

	//the event generation root
	_root: null,

	initialize: function (root, selection, background) {
		// summary:
		//		creates a new instance, sets the graph element from where to connect and set the default event mapping
		if (selection == null) {
			this._selection = new ibm_ilog.diagram.Selection();
		} else {
			this._selection = selection;
		}
		this._root = root;
		this._background = background;
		this._initialize();
		if(background) {
			this._states.item("idle").addConnection("clear");
		}
		return this;
	},

	_declareStates: function () {
		var idleStates = ["select", "toggle","add"];
		if (ibm_ilog.diagram.mobileEnabled)
			idleStates.push("touchSelect");
		this._declareState("idle", idleStates);
		this._declareState("active", ["move", "end"]);
		this._declareState("activeTouch", ["touchEnd"]);
	},

	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			select: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectPress",
				filter: this._buildInputFilter({ctrl:false}),
				gfxConnect:true
			}, clear: {
				src: this._background,
				srcEvt: "onmousedown",
				connectTo: "_backgroundPress",
				filter: this._buildInputFilter({ctrl:false}),
				gfxConnect:true
			}, toggle: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectionTogglePress",
				filter: this._buildInputFilter({ctrl:true,button:0}),
				gfxConnect:true
			}, add: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectionAddPress",
				filter: this._buildInputFilter({ctrl:true,button:1}), // right button does not toggle out a selected entity
				gfxConnect:true
			}, move: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_mouseMoved"
			}, end: {
				src: document,
				srcEvt: "onmouseup",
				connectTo: "_selectRelease"
			}, touchSelect: {
				src: this._root,
				srcEvt: "touchstart",
				connectTo: "_touchSelectPress",
				gfxConnect:true
			}, touchEnd: {
				src: document,
				srcEvt: "touchend",
				connectTo: "_touchSelectRelease"
			}
		};
	},
	
	_getGraphElementFromEvent: function (e) {
		// Summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._root);
		return ge !== this._root ? ge : null;
	},
	
	getSelection: function () {
		// Summary:
		//		return the selection object
		return this._selection;
	},

	//
	//	_focusedEntity: dojox.gfx.shape
	//		The entity focused during a selection operation (null if none)
	//
	_focusedEntity: null,


	_selectPress: function (event) {
		// summary:
		//		clear the selection and select the given element.
		// not multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity !== null && !this._focusedEntity.isSelected() && this._focusedEntity.isSelectable()  && this._selection.accept(this._focusedEntity)) {
			this._selection.add(this._focusedEntity,true);
		} else {
			this._goStateId("active");
		}
	},

	_touchSelectPress: function (event) {
		//console.log('touchSelectPress');
		event.preventDefault();
		if (event.touches.length > 1)
			return false;
		//event.target = event.touches[0].target;
		// summary:
		//		clear the selection and select the given element.
		// not multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity !== null && !this._focusedEntity.isSelected() && this._focusedEntity.isSelectable()  && this._selection.accept(this._focusedEntity)) {
			this._selection.add(this._focusedEntity,true);
		} else {
			this._goStateId("activeTouch");
		}
	},

	_mouseMoved: function (e) {
		this._goStateId("idle");
	},

	_selectRelease: function (event) {
		this._goState(null);

		if (!this._focusedEntity) {
			this._selection.clear();
		} else {
			if(this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)){
				this._selection.add(this._focusedEntity,true);
			}
		}
		this._focusedEntity = null;
		this._goStateId("idle");
	},

	_touchSelectRelease: function (event) {
		if (event.touches.length > 0){
			return false;
		}
		this._goState(null);

		if (!this._focusedEntity) {
			this._selection.clear();
		} else {
			if(this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)){
				this._selection.add(this._focusedEntity,true);
			}
		}
		this._focusedEntity = null;
		this._goStateId("idle");
	},

	_selectionTogglePress: function (event,noRemove) {
		// summary:
		//		adds the given element to the selection (if not selected) or remove it from the selection (if selected)
		// multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity != null) {
			if (!this._focusedEntity.isSelected() && this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)) {
				this._selection.add(this._focusedEntity);
			} else {
				if(!noRemove) {
					this._selection.remove(this._focusedEntity);
				}
			}
		}
	},
	_selectionAddPress: function (event) {
		this._selectionTogglePress(event,true);
	},
	
	_backgroundPress: function(event) {
		this._focusedEntity = null;
		this._goStateId("active"); // will clear if released, cancelled if moved
	},

__eod:undefined });

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.StandardInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.StandardInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.StandardInteractor');

//
//
//
//(function(){
//	var R = ibm_ilog.diagram.util.ErrorReporter;
//
//dojo.declare("ibm_ilog.diagram.interactors.StandardInteractor", [ibm_ilog.diagram.interactors.Interactor], {
//	//
//	//	summary:
//	//		Composite Interactor used to resolve conflicts in the UI event mappings
//	//		into various interactors
//	//
//
//	//
//	//	The pan interactor
//	//
//	_pan: null,
//	
//	//
//	//	The zoom interactor
//	//
//	_zoom: null,
//
//	//
//	//	The selection interactor
//	//
//	_selection: null,
//	
//	//
//	//	The Diagram widget
//	//
//	_diagram: null,
//	
//    constructor: function(
//    		/*ibm_ilog.diagram.widget.Diagram*/ diagram,
//    		/*PanInteractor*/              pan,
//    		/*ZoomInteractor*/             zoom,
//    		/*SelectionInteractor*/        selection )
//    {
//
//		switch(arguments.length) {
//		case 1:
//			this._diagram = diagram;
//			this._pan = new ibm_ilog.diagram.interactors.PanInteractor(diagram);
//			this._selection = new ibm_ilog.diagram.interactors.SelectionInteractor(diagram.getViewport().getEventSource(),diagram.getSelection());
//			this._zoom = new ibm_ilog.diagram.interactors.ZoomInteractor(diagram);
//			break;
//		case 4:
//			this._diagram = diagram;
//			this._pan = pan;
//			this._selection = selection;
//			this._zoom = zoom;
//			break;
//		default:
//			R.error("InvalidNumberOfArguments",arguments,1,4);
//			break;
//		}
//
//		this.setConnections(this.getDefaultConnections());
//
//	},
//    
//	activate: function() {
//		//
//		//	summary:
//    	//		Connects the handlers with event sources as described by the eventMap
//		//		This is an override. Replaces activation of selectionInteractor by a local logic
//		//
//		this._pan.activate();
//		this._zoom.activate();
//		this._selection.activate();
//		
//		this.inherited(arguments);
//	},
//	
//	deactivate: function() {
//    	//
//    	// summary:
//    	//		release all the handles.
//    	//
//		this._pan.deactivate();
//		this._zoom.deactivate();
//		this._selection.deactivate();
//
//		this.inherited(arguments);
//	},
//
//    getDefaultConnections:function(){
//		//
//		// summary:
//		//		Returns a mapping used to take control of selection interactor events
//		//
//    	return {};
//    },
//
//    __eod:null
//
//});
//
//})();

}

if(!dojo._hasResource['ibm_ilog.diagram.style']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.style'] = true;
dojo.provide('ibm_ilog.diagram.style');

(function() {

    var ids = ibm_ilog.diagram.style;
    
    function getSetter(o, p) {
        var uname = p.length > 1 ? (p.substr(0, 1).toUpperCase()) + p.substr(1) : p.toUpperCase();
        var set = dojo.getObject('set' + uname, false, o);
        return set;
    };
    
    ids.applyTo = function(/*Object*/mixin, /*Object*/target) {
        // summary: applies the specified style to the given target.
        // mixin: Object: 
        //            A dictionary of property-value pairs that define global properties to apply on the specified target.
        //            Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
        //            for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
        //            the node). If no setters are found, the property is set to the target as a simple field.
        // target: Object: the target instance.
         
        if (!mixin || !target) 
            return;
        for (var p in mixin) {
            var setter = getSetter(target, p);
            if (setter) {
                var val = mixin[p];
                setter.call(target, val);
            } else {
                // no setter -> add as a simple attribute
				target[p]=val; 
            }
        }
    };
    
})();

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");



dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						var widget = hash[widgetId];
						if(widget){	// may be null on page w/multiple dojo's loaded
							outAry.push(widget);
						}
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				var body;
				try{
					// non-IE
					var contentDocument = elem.contentDocument;
					if("designMode" in contentDocument && contentDocument.designMode == "on"){
						return true;
					}
					body = contentDocument.body;
				}catch(e1){
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						body = elem.contentWindow.document.body;
					}catch(e2){
						return false;
					}
				}
				return body.contentEditable == 'true' || (body.firstChild && body.firstChild.contentEditable == 'true');
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex, radioSelected = {};
		function radioName(node) {
			// If this element is part of a radio button group, return the name for that group.
			return node && node.tagName.toLowerCase() == "input" &&
				node.type && node.type.toLowerCase() == "radio" &&
				node.name && node.name.toLowerCase();
		}
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE && child.scopeName !== "HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
					var rn = radioName(child);
					if(dojo.attr(child, "checked") && rn) {
						radioSelected[rn] = child;
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		function rs(node) {
			// substitute checked radio button for unchecked one, if there is a checked one with the same name.
			return radioSelected[radioName(node)] || node;
		}
		return { first: rs(first), last: rs(last), lowest: rs(lowest), highest: rs(highest) };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dojo.Stateful"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.Stateful"] = true;
dojo.provide("dojo.Stateful");



dojo.declare("dojo.Stateful", null, {
	// summary:
	//		Base class for objects that provide named properties with optional getter/setter
	//		control and the ability to watch for property changes
	// example:
	//	|	var obj = new dojo.Stateful();
	//	|	obj.watch("foo", function(){
	//	|		console.log("foo changed to " + this.get("foo"));
	//	|	});
	//	|	obj.set("foo","bar");
	postscript: function(mixin){
		if(mixin){
			dojo.mixin(this, mixin);
		}
	},
	
	get: function(/*String*/name){
		// summary:
		//		Get a property on a Stateful instance.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property on a Stateful object. The property may
		//		potentially be retrieved via a getter method in subclasses. In the base class
		// 		this just retrieves the object's property.
		// 		For example:
		//	|	stateful = new dojo.Stateful({foo: 3});
		//	|	stateful.get("foo") // returns 3
		//	|	stateful.foo // returns 3
		
		return this[name];
	},
	set: function(/*String*/name, /*Object*/value){
		// summary:
		//		Set a property on a Stateful instance
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a stateful object and notifies any watchers of
		// 		the property. A programmatic setter may be defined in subclasses.
		// 		For example:
		//	|	stateful = new dojo.Stateful();
		//	|	stateful.watch(function(name, oldValue, value){
		//	|		// this will be called on the set below
		//	|	}
		//	|	stateful.set(foo, 5);
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myObj.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)
		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks){
			this._watchCallbacks(name, oldValue, value);
		}
		return this;
	},
	watch: function(/*String?*/name, /*Function*/callback){
		// summary:
		//		Watches a property for changes
		//	name:
		//		Indicates the property to watch. This is optional (the callback may be the
		// 		only parameter), and if omitted, all the properties will be watched
		// returns:
		//		An object handle for the watch. The unwatch method of this object
		// 		can be used to discontinue watching this property:
		//		|	var watchHandle = obj.watch("foo", callback);
		//		|	watchHandle.unwatch(); // callback won't be called now
		//	callback:
		//		The function to execute when the property changes. This will be called after
		//		the property has been changed. The callback will be called with the |this|
		//		set to the instance, the first argument as the name of the property, the
		// 		second argument as the old value and the third argument as the new value.
		
		var callbacks = this._watchCallbacks;
		if(!callbacks){
			var self = this;
			callbacks = this._watchCallbacks = function(name, oldValue, value, ignoreCatchall){
				var notify = function(propertyCallbacks){
					if(propertyCallbacks){
                        propertyCallbacks = propertyCallbacks.slice();
						for(var i = 0, l = propertyCallbacks.length; i < l; i++){
							try{
								propertyCallbacks[i].call(self, name, oldValue, value);
							}catch(e){
								console.error(e);
							}
						}
					}
				};
				notify(callbacks['_' + name]);
				if(!ignoreCatchall){
					notify(callbacks["*"]); // the catch-all
				}
			}; // we use a function instead of an object so it will be ignored by JSON conversion
		}
		if(!callback && typeof name === "function"){
			callback = name;
			name = "*";
		}else{
			// prepend with dash to prevent name conflicts with function (like "name" property)
			name = '_' + name;
		}
		var propertyCallbacks = callbacks[name];
		if(typeof propertyCallbacks !== "object"){
			propertyCallbacks = callbacks[name] = [];
		}
		propertyCallbacks.push(callback);
		return {
			unwatch: function(){
				propertyCallbacks.splice(dojo.indexOf(propertyCallbacks, callback), 1);
			}
		};
	}
	
});

}

if(!dojo._hasResource["dijit._WidgetBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._WidgetBase"] = true;
dojo.provide("dijit._WidgetBase");





(function(){

dojo.declare("dijit._WidgetBase", dojo.Stateful, {
	// summary:
	//		Future base class for all Dijit widgets.
	//		_Widget extends this class adding support for various features needed by desktop.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling set('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// mix in our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo._mixin(this, params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			// Also calls custom setters for all attributes with custom setters.
			this._applyAttributes();

			// If srcNodeRef was specified, then swap out original srcNode for this widget's DOM tree.
			// For 2.0, move this after postCreate().  postCreate() shouldn't depend on the
			// widget being attached to the DOM since it isn't when a widget is created programmatically like
			// new MyWidget({}).   See #11635.
			var source = this.srcNodeRef;
			if(source && source.parentNode && this.domNode !== source){
				source.parentNode.replaceChild(this.domNode, source);
			}
		}

		if(this.domNode){
			// Note: for 2.0 may want to rename widgetId to dojo._scopeName + "_widgetId",
			// assuming that dojo._scopeName even exists in 2.0
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(this._getSetterAttributes(), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	_getSetterAttributes: function(){
		// summary:
		//		Returns list of attributes with custom setters for this widget
		var ctor = this.constructor;
		if(!ctor._setterAttrs){
			var r = (ctor._setterAttrs = []),
				attrs,
				proto = ctor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
		}
		return ctor._setterAttrs;	// String[]
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected

		if(!this.domNode){
			// Create root node if it wasn't created by _Templated
			this.domNode = this.srcNodeRef || dojo.create('div');
		}

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNode.  If RTL mode then also add with Rtl suffix.
		// TODO: make baseClass custom setter
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},

	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// GET/SET, CUSTOM SETTERS, ETC. ///////////////////

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.replaceClass(mapNode, value, this["class"]);
		this._set("class", value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribute of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this._set("style", value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.replaceClass(mapNode, value, this[attr]);
					break;
			}
		}, this);
	},

	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set.
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]);
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			this._set(name, value);
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	_set: function(/*String*/ name, /*anything*/ value){
		// summary:
		//		Helper function to set new value for specified attribute, and call handlers
		//		registered with watch() if the value has changed.
		var oldValue = this[name];
		this[name] = value;
		if(this._watchCallbacks && this._created && value !== oldValue){
			this._watchCallbacks(name, oldValue, value);
		}
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var handles = [dojo._connect(obj, event, this, method)];
		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var handle = dojo.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	},

	defer: function(fcn, delay){
		// summary:
		//		Wrapper to setTimeout to avoid deferred functions executing
		//		after the originating widget has been destroyed.
		//		Returns an object handle with a remove method (that returns null) (replaces clearTimeout).
		// fcn: function reference
		// delay: Optional number (defaults to 0)
		// tags:
		//		protected.
		var timer = setTimeout(dojo.hitch(this, 
			function(){ 
				timer = null;
				if(!this._destroyed){ 
					dojo.hitch(this, fcn)(); 
	}
			}),
			delay || 0
		);
		return {
			remove:	function(){
					if(timer){
						clearTimeout(timer);
						timer = null;
					}
					return null; // so this works well: handle = handle.remove();
				}
		};
	}
});

})();

}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");


dojo.getObject("window", true, dojo);

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view using minimal movement, if it is not already.
	
	// Don't rely on node.scrollIntoView working just because the function is there since
	// it forces the node to the page's bottom or top (and left or right in IE) without consideration for the minimal movement.
	// WebKit's node.scrollIntoViewIfNeeded doesn't work either for inner scrollbars in right-to-left mode
	// and when there's a fixed position scrollable element

	node = dojo.byId(node);
	var body, doc = node.ownerDocument || dojo.doc;

	// has() functions but without has() support
	if(!("rtl_adjust_position_for_verticalScrollBar" in dojo.window)){
		body = dojo.body();
		var	scrollable = dojo.create('div', {
				style: {overflow:'scroll', overflowX:'visible', direction:'rtl', visibility:'hidden', position:'absolute', left:'0', top:'0', width:'64px', height:'64px'}
			}, body, "last"),
			div = dojo.create('div', {
				style: {overflow:'hidden', direction:'ltr'}
			}, scrollable, "last");
		dojo.window.rtl_adjust_position_for_verticalScrollBar = dojo.position(div).x != 0;
		scrollable.removeChild(div);
		body.removeChild(scrollable);
	}

	if(!("position_fixed_support" in dojo.window)){
		// IE6, IE7+quirks, and some older mobile browsers don't support position:fixed
		body = dojo.body();
		var	outer = dojo.create('span', {
				style: {visibility:'hidden', position:'fixed', left:'1px', top:'1px'}
			}, body, "last"),
			inner = dojo.create('span', {
				style: {position:'fixed', left:'0', top:'0'}
			}, outer, "last");
		dojo.window.position_fixed_support = dojo.position(inner).x != dojo.position(outer).x;
		outer.removeChild(inner);
		body.removeChild(outer);
	}

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		body = doc.body || doc.getElementsByTagName("body")[0];
		var	html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE,
			isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if(node == body || node == html){ return; }
		if(!(dojo.isMozilla || isIE || isWK || dojo.isOpera) && ("scrollIntoView" in node)){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			rootWidth = Math.min(body.clientWidth || html.clientWidth, html.clientWidth || body.clientWidth),
			rootHeight = Math.min(body.clientHeight || html.clientHeight, html.clientHeight || body.clientHeight),
			scrollRoot = (isWK || backCompat) ? body : html,
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return (isIE <= 6 || (isIE == 7 && backCompat))
					? false
					: (dojo.window.position_fixed_support && (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do
		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el),
				rtl = dojo.getComputedStyle(el).direction.toLowerCase() == "rtl";
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE || isIE >= 9){ elPos.x = 0; } // older IE can have values > 0
				if(elPos.y < 0 || !isIE || isIE >= 9){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					if(rtl && dojo.window.rtl_adjust_position_for_verticalScrollBar){
						elPos.x += scrollBarSize;
					}
					elPos.w = clientSize;
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
//						t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				t = nodePos.y - elPos.y, // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			var s, old;
			if(r * l > 0 && (!!el.scrollLeft || el == scrollRoot || el.scrollWidth > el.offsetHeight)){
				s = Math[l < 0? "max" : "min"](l, r);
				if(rtl && ((isIE == 8 && !backCompat) || isIE >= 9)){ s = -s; }
				old = el.scrollLeft;
				el.scrollLeft += s;
				s = el.scrollLeft - old;
				nodePos.x -= s;
			}
			if(bot * t > 0 && (!!el.scrollTop || el == scrollRoot || el.scrollHeight > el.offsetHeight)){
				s = Math.ceil(Math[t < 0? "max" : "min"](t, bot));
				old = el.scrollTop;
				el.scrollTop += s;
				s = el.scrollTop - old;
				nodePos.y -= s;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");





// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
					if(sel.rangeCount){
						bm.mark = sel.getRangeAt(0).cloneRange();
					}
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				targetWindow.document.body.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					targetWindow.document.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.body.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.body.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget.set("focused", false);
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				widget.set("focused", true);
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");



dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
};

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary:
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");





dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, choices, layoutNode, /*Object*/ aroundNodeCoords){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner, size)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//		It also passes in the available size for the popup, which is useful for tooltips to
	//		tell them that their width is limited to a certain amount.   layoutNode() may return a value expressing
	//		how much the popup had to be modified to fit into the available space.   This is used to determine
	//		what the best placement is.
	// aroundNodeCoords: Object
	//		Size of aroundNode, ex: {w: 200, h: 50}

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;
		var overflow = 0;

		// calculate amount of space available given specified position of node
		var spaceAvailable = {
			w: corner.charAt(1) == 'L' ? (view.l + view.w) - pos.x : pos.x - view.l,
			h: corner.charAt(1) == 'T' ? (view.t + view.h) - pos.y : pos.y - view.t
		};

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltip's size changes based on position, due to triangle)
		if(layoutNode){
			var res = layoutNode(node, choice.aroundCorner, corner, spaceAvailable, aroundNodeCoords);
			overflow = typeof res == "undefined" ? 0 : res;
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY;

		overflow += (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow,
				spaceAvailable: spaceAvailable
			};
		}
		
		return !overflow;
	});

	// In case the best position is not the last one we checked, need to call
	// layoutNode() again.
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner, best.spaceAvailable, aroundNodeCoords);
	}

	// And then position the node.   Do this last, after the layoutNode() above
	// has sized the node, due to browser quirks when the viewport is scrolled
	// (specifically that a Tooltip will shrink to fit as though the window was
	// scrolled to the left).
	//
	// In RTL mode, set style.right rather than style.left so in the common case,
	// window resizes move the popup along with the aroundNode.
	var l = dojo._isBodyLtr(),
		s = node.style;
	s.top = best.y + "px";
	s[l ? "left" : "right"] = (l ? best.x : view.w - best.x - best.w) + "px";
	s[l ? "right" : "left"] = "auto";       // needed for FF or else tooltip goes to far left

	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var aroundNodePos = dojo.position(aroundNode, true);

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode, {w: width, h: height});
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below-alt":
				leftToRight = !leftToRight;
				// fall through
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above-alt":
				leftToRight = !leftToRight;
				// fall through
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");




dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");






/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 1000,

	_idGen: 1,

	_createWrapper: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups.
		//		Puts widget inside a wrapper DIV (if not already in one),
		//		and returns pointer to that wrapper DIV.

		var node = widget.domNode || widget,
			wrapper = widget.declaredClass ? widget._popupWrapper :
				node.parentNode && dojo.hasClass(node.parentNode, "dijitPopup") ? node.parentNode : null;

		if(!wrapper){
			// Create wrapper <div> for when this widget [in the future] will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{ display: "none"},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
		
		return wrapper;	// DOMNode
	},

	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Moves the popup widget off-screen.
		//		Do not use this method to hide popups when not in use, because
		//		that will create an accessibility issue: the offscreen popup is
		//		still in the tabbing order.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, {
			visibility: "hidden",
			top: "-9999px",		// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
			display: ""
		});
	},

	hide: function(/*dijit._Widget*/ widget){
		// summary:
		//		Hide this popup widget (until it is ready to be shown).
		//		Initialization for widgets that will be used as popups
		//
		// 		Also puts widget inside a wrapper DIV (if not already in one)
		//
		//		If popup widget needs to layout it should
		//		do so when it is made visible, and popup._onShow() is called.

		// Create wrapper if not already there
		var wrapper = this._createWrapper(widget);

		dojo.style(wrapper, "display", "none");
	},
		
	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);

		// If we are opening a new popup that isn't a child of a currently opened popup, then
		// close currently opened popup(s).   This should happen automatically when the old popups
		// gets the _onBlur() event, except that the _onBlur() event isn't reliable on IE, see [22198].
		while(stack.length && (!args.parent || !dojo.isDescendant(args.parent.domNode, stack[stack.length-1].widget.domNode))){
			dijit.popup.close(stack[stack.length-1].widget);
		}

		// Get pointer to popup wrapper, and create wrapper if it doesn't exist
		var wrapper = this._createWrapper(widget);


		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.display = "";
		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget?*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented.
		//		If no popup is specified, closes all popups.

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while((popup && dojo.some(stack, function(elem){return elem.widget == popup;})) ||
			(!popup && stack.length)){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Hide the widget and it's wrapper unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.hide(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE < 9){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/*DomNode*/ node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = (this.iframe = dijit._frames.pop());
		node.appendChild(iframe);
		if(dojo.isIE<7 || dojo.isQuirks){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		Resize the iframe so it's the same size as node.
		//		Needed on IE6 and IE/quirks because height:100% doesn't work right.
		if(this.iframe){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");




dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");



(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_quirks: d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla
		}; // no dojo unsupported browsers

	if(ie){
		classes["dj_ie"] = true;
		classes["dj_ie" + maj(ie)] = true;
		classes["dj_iequirks"] = d.isQuirks;
	}
	if(ff){
		classes["dj_ff" + maj(ff)] = true;
	}

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");




// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");



dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE < 9){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");



dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	hasWaiRole: function(/*Element*/ elem, /*String?*/ role){
		// summary:
		//		Determines if an element has a particular role.
		// returns:
		//		True if elem has the specific role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has a role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the role for an element (which should be a wai role).
		// returns:
		//		The role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.

			dojo.attr(elem, "role", role);
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");






////////////////// DEFERRED CONNECTS ///////////////////

// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

////////////////// ONDIJITCLICK SUPPORT ///////////////////

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

dojo.declare("dijit._Widget", dijit._WidgetBase, {
	// summary:
	//		Base class for all Dijit widgets.
	//
	//		Extends _WidgetBase, adding support for:
	//			- deferred connections
	//				A call like dojo.connect(myWidget, "onMouseMove", func)
	//				will essentially do a dojo.connect(myWidget.domNode, "onMouseMove", func)
	//			- ondijitclick
	//				Support new dojoAttachEvent="ondijitclick: ..." that is triggered by a mouse click or a SPACE/ENTER keypress
	//			- focus related functions
	//				In particular, the onFocus()/onBlur() callbacks.   Driven internally by
	//				dijit/_base/focus.js.
	//			- deprecated methods
	//			- onShow(), onHide(), onClose()
	//
	//		Also, by loading code in dijit/_base, turns on:
	//			- browser sniffing (putting browser id like .dj_ie on <html> node)
	//			- high contrast mode sniffing (add .dijit_a11y class to <body> if machine is in high contrast mode)
	

	////////////////// DEFERRED CONNECTS ///////////////////

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		this.inherited(arguments);

		if(this.domNode){
			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	////////////////// FOCUS RELATED ///////////////////
	// _onFocus() and _onBlur() are called by the focus manager

	// focused: [readonly] Boolean
	//		This widget or a widget it contains has focus, or is "active" because
	//		it was recently clicked.
	focused: false,

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	////////////////// DEPRECATED METHODS ///////////////////

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	////////////////// ONDIJITCLICK SUPPORT ///////////////////

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup.
		//		Events connected with `this.connect` are disconnected upon
		//		destruction.
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = this.inherited(arguments, [obj, event == "ondijitclick" ? "onclick" : event, method]);

		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(d.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							
							// Stop event to prevent scrolling on space key in IE.
							// But don't do this for _HasDropDown because it surpresses the onkeypress
							// event needed to open the drop down when the user presses the SPACE key.
							if(!("openDropDown" in this && obj == this._buttonNode)){
								e.preventDefault();
							}
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target == dijit._lastKeyDownNode &&	// === breaks greasemonkey
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
		}

		return handles;		// _Widget.Handle
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.widget.Diagram']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.widget.Diagram'] = true;
dojo.provide('ibm_ilog.diagram.widget.Diagram');

































dojo.requireIf(dojo.config.isBidiInDojoDiagrammer, 'ibm_ilog.diagram.util.BidiUtil');



(function() {
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry, Size = g.Size;
	var __set = ibm_ilog.diagram._base.dojoext.SetAttrName;
	var __get = ibm_ilog.diagram._base.dojoext.GetAttrName;

	var _emptyEntityHandler = function(entity, event) {
	};
	
	dojo.declare('ibm_ilog.diagram.widget.Diagram', [dijit._Widget, ibm_ilog.diagram.widget.DataConnector, ibm_ilog.diagram.util.DeferredInitialization], {
		// summary:
		//		The Diagram widget is the main entry point to the Dojo Diagrammer component.
		// description:
		//		The purpose of the Dojo Diagrammer component is to display <em>graphs</em> (or <em>diagrams</em>)
		//		(that is, graphical representations consisting of nodes connected by links)
		//		in a Web browser page. The Dojo Diagrammer component is fully implemented in JavaScript
		//		and is based on the Dojo Toolkit. Dojo Diagrammer uses DojoX GFX to render the diagram.
		//		<p>
		//		The Diagram widget is the Dijit widget that is commonly used as an entry point to Dojo Diagrammer,
		//		and provides the following main features:
		//		<ul>
		//		<li><strong>Data store connection</strong>
		//		The Diagram widget can load a Dojo data store that implements the dojo.data.api.Read interface.
		//		For this, the data store must be set as the 'nodesStore' attribute of the Diagram.
		//		Each item of the data store will typically be displayed as a node in the graph.
		//		The appearance of the nodes is defined by a <em>template</em>, specified using the 'nodeTemplate'
		//		attribute (or the nodeTemplateFunction to define different templates according to each item).
		//		The Diagram widget will also create links to represent the relationships between nodes.
		//		These relationships must be defined by specifying one or several of the following attributes:
		//		childBinding, parentBinding, successorsBinding, predecessorsBinding. For example, if
		//		successorsBinding attribute is set to "nextItems", the value if the "nextItems" property
		//		of the data items is expected to be an array of identifiers that define
		//		the set of items to which the current item should be connected by links.
		//		</li>
		//		<li><strong>Viewport (scrolling and zooming)</strong>
		//		If the diagram is larger than the client area of the Diagram, it can be scrolled using the
		//		scrollbars or using the mouse (the default interaction for panning is Shift-drag).
		//		The diagram can also be zoomed or unzoomed (the default interaction is Control-mouse wheel).
		//		Alternately, all viewport actions can be done by code through the viewport object, accessible by the getViewport()
		//		method.
		//		</li>
		//		<li><strong>Selection</strong>
		//		Nodes and links in the diagram can be selected. The set of selected objects can be accessed and modified
		//		through the selection object accessible by the getSelection() method.
		//		The appearance of selected objects can be customized through their template.
		//		</li>
		//		<li><strong>Event handling</strong>
		//		The Diagram handles mouse and keyboard events through objects called "interactors". Examples of interactors are
		//		the SelectionInteractor, the PanInteractor, the ZoomInteractor, etc. The predefined interactors can be
		//		enabled or disabled using attributes of the Diagram widget: 'select', 'zoom', 'pan', etc.
		//		In addition, the Diagram widget provides events to easily connect to mouse or key events on all the
		//		nodes or links of the diagram: onNodeClick, onNodeMouseDown, onLinkClick, etc.
		//		</li>
		//		<li><strong>Accessibility</strong>
		//		The Diagram widget provides accessibility services: the diagram can be described by a screen reader,
		//		and the user can navigate through the nodes and links using the keyboard.
		//		</li>
		//		<li><strong>Touch-enablement and mobile support</strong>
		//		The Diagram widget reacts to touch events on mobile platforms (or other touch-enabled devices):
		//		single touch-and-scroll pans the diagram, two-fingers pinch zooms the diagram.
		//		Dojo Diagrammer can be used with dojox.mobile.
		//		In addition, to optimize download size, some features are disabled on mobile:
		//		key events, focus adorners, accessibility, etc. You can explicitly disable these features by setting
		//		the 'diagramForMobile' flag to true in djConfig/dojoConfig. If diagramForMobile is not defined,
		//		the Diagram widget tries to auto-detect some well-known mobile user agents. Note that diagramForMobile
		//		is also set to true if the dojox.mobile module is loaded before the Diagram widget is loaded.
		//		You can set diagramForMobile to false explicitly to enable all features even on mobiles.
		//		</li>
		//		</ul>
		//		The Diagram widget uses the following basic graphic objects behind the scene to display the diagram:
		//		<ul>
		//		<li><strong>Graph</strong> (class ibm&#95;ilog.diagram.Graph): A subclass of dojox.gfx.Group that contains nodes and links. </li>
		//		<li><strong>Node</strong> (class ibm&#95;ilog.diagram.Node): Represents a node in a Graph, the contents of a Node are defined by a template. </li>
		//		<li><strong>Link</strong> (class ibm&#95;ilog.diagram.Link): Represents a link between two nodes. </li>
		//		<li><strong>Subgraph</strong> (ibm&#95;ilog.diagram.Subgraph): Represents a Node that contains a nested Graph. </li>
		//		</ul>
		//		It is possible to use the Dojo Diagramer component without using a Diagram widget.
		//		For this, you must create a GFX surface (or use an existing one), and create a Graph in it using surface.createGraph().
		//		You can then create nodes, links and subgraphs using the methods createNode, createLink and createSubgraph of the Graph
		//		object.
		
		_graph: null,
		
		_firePropertyChanged: function(pname, oldValue, newValue) {
			if (this.isInitialized()) {
				this.onPropertyChanged(pname, oldValue, newValue);
			}
		},
		
		onPropertyChanged: function(/*String*/name, /*Object*/ oldValue, /*Object*/ newValue) {
			// summary:
			//     An extension point invoked when a property has changed.
			// name: String: the property name.
			// oldValue: Object: the old property value.
			// newValue: Object: the new property value.
		},
		
		// Public properties
		
		// nodesStore: dojo.data.api.Read 
		//		A data store that provides data item to be represented as Nodes.
		//		Must implement at least the dojo.data.api.Read and dojo.data.api.Identity interfaces.
		nodesStore: null,
		
		_setNodesStoreAttr: function(value) {
			var old = this._getNodesStore();
			if (old !== value) {
				this.nodesStore = this._setNodesStore(value);
				this._firePropertyChanged('nodesStore', old, value);
			}
			return this;
		},
		
		// An object providing the query criteria for the nodes data store.
		nodesQuery: null, // cast the value as an Object
		
		_setNodesQueryAttr: function(value) {
			// use the DataConnector getter to get the real old value and not the one set in the mixin() step
			var old = this._getNodesQuery(); 
			this.nodesQuery = this._setNodesQuery(value);
			this._firePropertyChanged('nodesQuery', old, value);
			return this;
		},
		
		// nodesQueryOptions: Object
		//                    An object containing additional options for the node query.
		nodesQueryOptions: {}, // cast the value as an Object
		_setNodesQueryOptionsAttr: function(value) {
			// use the DataConnector getter to get the real old value and not the one set in the mixin() step
			var old = this._getNodesQueryOptions();
			this.nodesQueryOptions = this._setNodesQueryOptions(value);
			this._firePropertyChanged('nodesQueryOptions', old, value);
			return this;
		},
		
		_setBindingAttr: function(value, pname) {
			if (value) {
				var func = dojo.getObject(value); // global scope
				// We don't want "xBinding" treated as variable name only a Function or a String
				if (dojo.isFunction(func)) {
					value = func;
				}
			}
		    var uname = (pname.substr(0, 1).toUpperCase()) + pname.substr(1);
			// _s and _g refer to the corresponding functions defined in the DataConnector impl.
			// which all starts like '_setFoo'
			var _s = '_set' + uname;
			var _g = '_get' + uname;
			var old = this[_g]();
			if (old !== value) {
				this[pname] = this[_s](value);
				this._firePropertyChanged(pname, old, value);
			}
			return this;
		},
		
		// childBinding: String
		//     A string that indicates either the data item field name 
		//     defining the "child" relationship or a function name that returns the
		//     origin of a link for the specified data item.
		childBinding: "", // cast the value as a String
		
		_setChildBindingAttr: function(value) {
			// We don't want "xBinding" treated as variable name only a Function or a String
			return this._setBindingAttr(value, 'childBinding');
		},
		
		// parentBinding: String
		//     A string that indicates either the data item field name defining the
		//     "parent" relationship or a function name that returns the destination 
		//     of a link for the specified data item.
		parentBinding: "", // cast the value as a String
		
		_setParentBindingAttr: function(parentBinding) {
			return this._setBindingAttr(parentBinding, 'parentBinding');
		},
		
		// predecessorsBinding: String
		//     A string that indicates either the data item field name 
		//     defining the predecessors or a function name that returns the
		//     origins of a link for the specified data item.
		predecessorsBinding: "", // cast the value as a String
		
		_setPredecessorsBindingAttr: function(predecessorsBinding) {
			return this._setBindingAttr(predecessorsBinding, 'predecessorsBinding');
		},
		
		// successorsBinding: String
		//     A string that indicates either the data item field name defining the
		//     successors or a function name that returns the destination 
		//     of a link for the specified data item.
		successorsBinding: "", // cast the value as a String

		_setSuccessorsBindingAttr: function(successorsBinding) {
			return this._setBindingAttr(successorsBinding, 'successorsBinding');
		},
		
		// xBinding: String
		//     A string that indicates either the data item field name defining the
		//     "x location" associated with a nodesStore data item or a named function that 
		//     returns the "x location" associated with a nodesStore data item.
		xBinding: "", // cast the value as a String
		
		_setXBindingAttr: function(xBinding) {
			return this._setBindingAttr(xBinding, 'xBinding');
		},
		
		// yBinding: String
		//     A string that indicates either the data item field name defining the
		//     "y location" associated with a nodesStore data item or a named function that 
		//     returns the "y location" associated with a nodesStore data item.
		yBinding: "", // cast the value as a String

		_setYBindingAttr: function(yBinding) {
			return this._setBindingAttr(yBinding, 'yBinding');
		},
		
		// nodesGraphBinding: String
		//     A string that indicates either the data item field name defining the
		//     "node graphic data" associated with a nodeStore data item or a named function that 
		//     returns the "node graphic data" associated with a linksStore data item.
		nodesGraphBinding: "", // cast the value as a String

		_setNodesGraphBindingAttr: function(nodesGraphBinding) {
			return this._setBindingAttr(nodesGraphBinding, 'nodesGraphBinding');
		},
		
		// nodesGraphProperties: String
		//           A comma-separated list of properties to save if nodesGraphBinding is specified.
		nodesGraphProperties: "",
		
		_setNodesGraphPropertiesAttr: function(value) {
			var old = this._getNodesGraphProperties();
			if (old !== value) {
				this.nodesGraphProperties = this._setNodesGraphProperties(value);
				this._firePropertyChanged('nodesGraphProperties', old, value);
			}
			return this;
		},
		
		// centerOnLocation: Boolean
		//     If true, the values provided by xBinding/yBinding atributes represent the center of the node.
		//     If false, the values provided by xBinding/yBinding atributes represent the top left-hand corner of the node.
		centerOnLocation: false,
		
		_setCenterOnLocationAttr: function(value) {
			var old = this._getCenterOnLocation();
			if (old !== value) {
				this.centerOnLocation = this._setCenterOnLocation(value);
				this._firePropertyChanged('centerOnLocation', old, value);
			}
			return this;
		},
		
		// linksStore: dojo.data.api.Read
		//             A data store that provides data item to be represented as Links when the
		//             nodes relationship cannot be implicitly deducted from the nodes store data structure.
		//             Must implement at least the dojo.data.api.Read and
		//             dojo.data.api.Identity interfaces.
		linksStore: null,
		
		_setLinksStoreAttr: function(value) {
			var old = this._getLinksStore();
			if (old !== value) {
				this.linksStore = this._setLinksStore(value);
				this._firePropertyChanged('linksStore', old, value);
			}
			return this;
		},
		
		// linksQuery: Object
		//     An object providing the query criteria for the links datas store.		
		linksQuery: null, // cast the value as a String
		
		_setLinksQueryAttr: function(value) {
			var old = this._getLinksQuery();
			this.linksQuery = this._setLinksQuery(value);
			this._firePropertyChanged('linksQuery', old, value);
			return this;
		},
		
		// linksQueryOptions: Object
		//                    An object containing additional options for the links query.
		linksQueryOptions: {}, // cast the value as an Object

		_setLinksQueryOptionsAttr: function(value) {
			var old = this._getLinksQueryOptions();
			this.linksQueryOptions = this._setLinksQueryOptions(value);
			this._firePropertyChanged('linksQueryOptions', old, value);
			return this;
		},
		
		// linksGraphBinding: String
		//     A string that indicates either the data item field name defining the
		//     "link graphic data" associated with a linksStore data item or a named function that 
		//     returns the "link graphic data" associated with a linksStore data item.
		linksGraphBinding: "", // cast the value as a String
		
		_setLinksGraphBindingAttr: function(linksGraphBinding) {
			return this._setBindingAttr(linksGraphBinding, 'linksGraphBinding');
		},
		
		// linksGraphProperties: String
		//     A comma-separated list of properties to save if linksGraphBinding is specified.
		linksGraphProperties: "",
		
		_setLinksGraphPropertiesAttr: function(value) {
			var old = this._getLinksGraphProperties();
			if (old !== value) {
				this.linksGraphProperties = this._setLinksGraphProperties(value);
				this._firePropertyChanged('linksGraphProperties', old, value);
			}
			return this;
		},
		
		// startNodeBinding: String
		//              A string that indicates either the data item field name 
		//              defining the "start" of a link or a function name that returns the
		//              origin of a link for the specified data item.
		startNodeBinding: "start", // cast the value as a String and set the default as field with name of "start"
		
		_setStartNodeBindingAttr: function(startNodeBinding) {
			return this._setBindingAttr(startNodeBinding, 'startNodeBinding');
		},

		// endNodeBinding: String
		//     A string that indicates either the data item field name defining the
		//     "end" of a link or a function name that returns the destination 
		//     of a link for the specified data item.
		endNodeBinding: "end", // cast the value as a String and set the default as field with name of "end"

		_setEndNodeBindingAttr: function(endNodeBinding) {
			return this._setBindingAttr(endNodeBinding, 'endNodeBinding');
		},
		
		// createLinksForHierarchy: Boolean
		//     If true, the hierarchy is rendered using nodes and links.
		//     If false, it is rendered using subgraphs and nodes
		createLinksForHierarchy: true,
		
		_setCreateLinksForHierarchyAttr: function(value) {
			var old = this._getCreateLinksForHierarchy();
			if (old !== value) {
				this.createLinksForHierarchy = this._setCreateLinksForHierarchy(value);
				this._firePropertyChanged('createLinksForHierarchy', old, value);
			}
			return this;
		},
		
		// textDir: String
		//     A string that indicates the text direction of displayed text
		textDir: "",   
		
		_setTextDirAttr: function(value) {
		    if (dojo.config.isBidiInDojoDiagrammer) {
		        if (this._graph)
		            this._graph.setTextDir(value);
	  	        if (this._annotations) {
		           dojo.forEach(this._annotations,function(a) {
				        if (a.setTextDir)
				            a.setTextDir(value);
			            });
    	        }    
		    }
		},		
		
		//////////////////////////////////////////////////////////////////////////////
		//	Viewport Properties
		//

		viewportClass: "DivViewport",
		
		//
		//	viewportAirFactor: [markup] Boolean
		//		The minimum navigable space to the sides of the current content bounds as a percentage
		//		with respect to the current model bounding box.
		viewportAirFactor: 0.0,
		_setViewportAirFactorAttr: function(value) {
			return this._setViewportAttr("AirFactor",value); 
		},
		_getViewportAirFactorAttr: function(){
			return this._getViewportAttr("AirFactor");
		},

		//
		// viewportAnimated: [markup] Boolean
		//		Boolean flag to indicate that viewport movements are animated by default.
		viewportAnimated: false,
		_setViewportAnimatedAttr: function(value) {
			return this._setViewportAttr("Animated",value); 
		},
		_getViewportAnimatedAttr: function(){
			return this._getViewportAttr("Animated");
		},

		//
		// viewportMaxZoom: [markup] Number
		//		Defines how much the view rectangle can zoom in. Unlimited if 0.
		viewportMaxZoom: 0,
		_setViewportMaxZoomAttr: function(value) {
			return this._setViewportAttr("MaxZoom",value); 
		},
		_getViewportMaxZoomAttr: function(){
			return this._getViewportAttr("MaxZoom");
		},

		//
		//	viewportMinZoom: [markup] Number
		//		Defines how much the view rectangle can zoom out. If 0, the limit is based on the contentRectangle.
		//
		viewportMinZoom: 0,
		_setViewportMinZoomAttr: function(value) {
			return this._setViewportAttr("MinZoom",value); 
		},
		_getViewportMinZoomAttr: function(){
			return this._getViewportAttr("MinZoom");
		},

		//
		//	viewportResizeMode: [markup] Number
		//		Resize mode of the vieport. See Viewport.js
		//
		viewportResizeMode: 1,
		_setViewportResizeModeAttr: function(value) {
			return this._setViewportAttr("ResizeMode",value); 
		},
		_getViewportResizeModeAttr: function(){
			return this._getViewportAttr("ResizeMode");
		},

		
		
		// Initialize viewport properties.
		// The method is called during the postCreate stage to apply the markup properties
		// that have been deferred (viewport is not fully initialized when the custom
		// setters are invoked by the dijit attributes engine).
		_initializeViewportProperties: function(){
			if (this.overviewSize != undefined)
				this._setOverviewSizeAttr(this.overviewSize);
			if (this.overviewPan != undefined)
				this._setOverviewPanAttr(this.overviewPan);
			if (this.overviewZoom!= undefined)
				this._setOverviewZoomAttr(this.overviewZoom);
			if (this.showZoomFactor)
				this._setShowZoomFactorAttr(this.showZoomFactor);
			if (this.showZoomFeedback)
				this._setShowZoomFeedbackAttr(this.showZoomFeedback);
			if (this.overview)
				this._setOverviewAttr(this.overview);
			if (!this.hud)
				this._setHudAttr(this.hud);
		},

		
		
		//
		//	overviewSize: Size
		//		The size of the embedded Overview
		//		
		overviewSize: Size(100, 100),
		
		_setOverviewSizeAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewSizeAttr();
			this.overviewSize = value;
			if (this._hudOverview) {
				this._hudOverview.setSize(value);
			}
			this._firePropertyChanged('overviewSize', old, value);
			return this;
		},
		
		_getOverviewSizeAttr:function(){
			return this._hudOverview ? this._hudOverview.getSize() : this.overviewSize;
		},
		
		
		//
		//	overviewPan: boolean
		//		Activate or deactivate panning in the hud overview
		//		
		overviewPan: true,
		
		_setOverviewPanAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewPanAttr();
			this.overviewPan = value;
			if (this._hudOverview) {
				this._hudOverview.setPan(value);
			}
			this._firePropertyChanged('overviewPan', old, value);
			return this;
		},
		
		_getOverviewPanAttr:function(){
			return this._hudOverview ? this._hudOverview.getPan() : this.overviewPan;
		},

		
		//
		//	overviewZoom: boolean
		//		Activate or deactivate panning in the hud overview
		//		
		overviewZoom: true,
		
		_setOverviewZoomAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewZoomAttr();
			this.overviewZoom = value;
			if (this._hudOverview) {
				this._hudOverview.setZoom(value);
			}
			this._firePropertyChanged('overviewZoom', old, value);
			return this;
		},
		
		_getOverviewZoomAttr:function(){
			return this._hudOverview ? this._hudOverview.getZoom() : this.overviewZoom;
		},

		
		//
		//	showZoomFactor: [markup] Boolean
		//		Display the current zoom factor
		//		
		showZoomFactor: false,
		
		_setShowZoomFactorAttr: function(enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getShowZoomFactorAttr();
			if (old !== enabled) {
				this.showZoomFactor = enabled;
				if (!this._hudZoom) {
					this._hudZoom = new ibm_ilog.diagram.widget.ZoomHudComponent(this._viewport);
				}
				this._hudZoom.setEnabled(enabled);
				this._firePropertyChanged('showZoomFactor', old, enabled);
			}
			return this;
		},
		
		_getShowZoomFactorAttr: function() {
			return this._hudZoom ? this._hudZoom.isEnabled() : false;
		},
		
		
		//
		//	showZoomFeedback: [markup] Boolean
		//		Display a visual feedback while zooming with animation 
		//		
		showZoomFeedback: false,
		
		_setShowZoomFeedbackAttr: function(/*Boolean*/enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getShowZoomFeedbackAttr();
			if (old !== enabled) {
				this.showZoomFeedback = enabled;
				this._viewport.setHudZoomFeedbackEnabled(enabled);
				this._firePropertyChanged('showZoomFeedback', old, enabled);
			}
			return this;
		},
		
		_getShowZoomFeedbackAttr: function() {
			return this._viewportReady ? this._viewport.isHudZoomFeedbackEnabled() : false;
		},
		
		
		
		//
		//  overview: [markup] Boolean
		//      Use the embedded overview hud component
		//      
		overview: false,
		
		_setOverviewAttr: function(/*Boolean*/enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewAttr();
			if (old !== enabled) {
				this.overview = enabled;
				if (!this._hudOverview) {
					var renderer = this.overviewRenderer || new ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer();
					this._hudOverview = new ibm_ilog.diagram.widget.OverviewHudComponent(this._viewport, this._graph, renderer, this.overviewSize);
					this._hudOverview.getOverview().setInteractorManager(this._interactors);
					this._hudOverview.setPan(this.overviewPan);
					this._hudOverview.setZoom(this.overviewZoom);
				}
				this._hudOverview.setEnabled(enabled);
				this._firePropertyChanged('overview', old, enabled);
			}
			return this;
		},
		
		_getOverviewAttr: function() {
			return this._hudOverview ? this._hudOverview.isEnabled() : false; 
		},


		
		//
		//  overviewRenderer: ibm_ilog.diagram.overview.OverviewRenderer
		//      Use renderer used in the embedded overview
		//      
		overviewRenderer: false,
		
		_setOverviewRendererAttr: function(/*ibm_ilog.diagram.overview.OverviewRenderer*/renderer) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewRendererAttr();
			if (old !== renderer) {
				this.overviewRenderer = renderer;
				if (this._hudOverview) {
					this._hudOverview.getOverview().setRenderer(renderer);
				}
				this._firePropertyChanged('overviewRenderer', old, renderer);
			}
			return this;
		},
		
		_getOverviewRendererAttr: function() {
			return this._hudOverview ? this._hudOverview.getOverview().getRenderer() : this.overviewRenderer; 
		},
		
		
		//
		//  hud: [markup] Boolean
		//      Use to disable the entire viewport hud.
		//      
		
		hud: true,
		
		_setHudAttr: function(enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getHudAttr();
			if (old !== enabled) {
				this.hud = enabled;
				this._viewport.setHudEnabled(enabled);
				this._firePropertyChanged('hud', old, enabled);
			}
			return this;
		},
		
		_getHudAttr: function() {
			return this._viewportReady ? this._viewport.isHudEnabled() : true;
		},

		
		
		_setViewportAttr: function(uname, value) {
			if (!this._viewportReady)
				return;
			var old = this['_getViewport'+uname+'Attr']();
			if (old !== value) {
				var pname = 'viewport'+uname;
				this[pname] = value;
				this._viewport['set'+uname](value);
				this._firePropertyChanged(pname, old, value);
			}
			return this;
		},

		_getViewportAttr: function(uname) {
			var pname = 'viewport'+uname;
			if (!this._viewportReady) {
				return this[pname];
			} else {
				return this._viewport['get'+uname]();
			}
		},
		
		//////////////////////////////////////////////////////////////////////////////
		//  Interactor Properties
		//

		//
		//  pan: [markup] boolean
		//      enable or disable panning
		//
		pan: true,
		
		_setPanAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getPanAttr(); // use the getter as the interactor may have been configured directly
			if (old !== b) {
				this.pan = b;
				this._interactors.get('pan').setEnabled(b);
				this._firePropertyChanged('pan', old, b);
			}
			return this;
		},
		
		_getPanAttr: function() {
			return this._viewportReady ? this._interactors.get("pan").getEnabled() : true;
		},

		
		
		//
		//  pan: [markup] boolean
		//      enable or disable panning if clicking on shapes.
		//
		panOnShapes:false,
		
		_setPanOnShapesAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return;
			var old = this._getPanOnShapesAttr();
			if (old !== b) {
				this.panOnShapes = b;
				this._interactors.get("pan").setActOnShapes(b);
				this._firePropertyChanged('panOnShapes', old, b);
			}
			return this; 
		},
		
		_getPanOnShapesAttr: function( /*boolean*/b) {
			return this._viewportReady ? this._interactors.get("pan").getActOnShapes(b) : false;
		},
		
		
		
		
		//
		//  zoom: [markup] boolean
		//      enable or disable zooming
		//
		zoom: true,
		
		_setZoomAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getZoomAttr();
			if (old !== b) {
				this.zoom = b;
				this._interactors.get("zoom").setEnabled(b);
				this._firePropertyChanged('zoom', old, b);
			}
			return this;
		},
		
		_getZoomAttr: function() {
			return this._viewportReady ? this._interactors.get("zoom").getEnabled() : true;
		},
		
		
		
		
		//
		//  select: [markup] boolean
		//      enable or disable selecting entities
		//
		select: true,

		_setSelectAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getSelectAttr();
			if (old !== b) {
				this.select = b;
				this._interactors.get("select").setEnabled(b);
				this._firePropertyChanged('select', old, b);
			}
			return this;
		},
		_getSelectAttr: function() {
			return this._viewportReady ? this._interactors.get("select").getEnabled() : true;
		},
		
		
		//
		//  marquee: [markup] boolean
		//      enable or disable marquee selection
		//
		marquee: true,
		
		_setMarqueeAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMarqueeAttr();
			if (old !== b) {
				this.marquee = b;
				this._interactors.get("marquee").setEnabled(b);
				this._firePropertyChanged('marquee', old, b);
			}
			return this;
		},
		
		_getMarqueeAttr: function() {
			return this._viewportReady ? this._interactors.get("marquee").getEnabled() : true;
		},
		
		
		
		//
		//  wheelPan: [markup] boolean
		//      enable or disable scrolling with the mouse wheel (vertical panning)
		//
		wheelPan: true,
		
		_setWheelPanAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getWheelPanAttr();
			if (old !== b) {
				this.wheelPan = b;
				this._interactors.get("wheelPan").setEnabled(b);
				this._firePropertyChanged('wheelPan', old, b);
			}
			return this;
		},
		
		_getWheelPanAttr: function() {
			return this._viewportReady ? this._interactors.get("wheelPan").getEnabled() : true;
		},
		
		
		
		//
		//	adorners: [markup] boolean
		//		enable or disable adorners focus following mouse pointer
		//
		adorners: true,
		
		_setNodeAdornersFocusAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getNodeAdornersFocusAttr();
			if (old !== b) {
				this.adorners = b;
				this._interactors.get("adorners").setEnabled(b);
				this._firePropertyChanged('adorners', old, b);
			}
			return this;
		},
		
		_getNodeAdornersFocusAttr: function() {
			return this._viewportReady ? this._interactors.get("adorners").getEnabled() : true;
		},
		
		//
		//  moveFunction: Function
		//      This function is an optional callback to configure how the MoveInteractor applies the movement deltas on entities.
		//      function(graphElement,originalTransform,delta) {
		//          //  summary:
		//          //      Concrete the movement to a graph element.
		//          //  graphElement:
		//          //      The graph element to move.
		//          //  originalTransform:
		//          //      The matrix representing the location from which the delta is computed.
		//          //  delta:
		//          //      The ammount to translate the entity from its original location.
		//      },  
		//
		moveFunction: null,
		
		//
		//  isMovableFunction: Function
		//      This function is an optional callback to configure how the MoveInteractor tests movability of elements
		//      function(graphElement) {
		//          //  summary:
		//          //      Return true if the given graph element is allowed to move
		//          //  graphElement:
		//          //      The graph element to test.
		//      }
		//
		isMovableFunction: null,
		
		
		//////////////////////////////////////////////////////////////////////////////
		// Events properties
		
		//
		//  nodeMenu:
		//      An optional dijit.Menu for nodes (property)
		//
		nodeMenu: null,
		
		//
		//  linkMenu:
		//      An optional dijit.Menu for links (property)
		//
		linkMenu: null,
		
		//
		//  subgraphMenu:
		//      An optional dijit.Menu for subgraphs (property)
		//
		subgraphMenu: null,
		
		//
		//  backMenu:
		//      An optional dijit.Menu for the diagram background (property)
		//
		backMenu: null,
		
		//////////////////////////////////////////////////////////////////////////////
		// Styling Properties
		
		// nodeStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on nodes.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		nodeStyle: null,
		
		_setNodeStyleAttr: function(value) {
			var old = this._created ? this.nodeStyle : null;
			this.nodeStyle = value;
			this._firePropertyChanged('nodeStyle', old, value);
			return this;
		},
		
		// linkStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on links.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		linkStyle: null,
		
		_setLinkStyleAttr: function(value) {
			var old = this._created ? this.linkStyle : null;
			this.linkStyle = value;
			this._firePropertyChanged('linkStyle', old, value);
			return this;
		},
		
		// subgraphStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on subgraphs.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		subgraphStyle: null,
		
		_setSubgraphStyleAttr: function(value) {
			var old = this._created ? this.subgraphStyle : null;
			this.subgraphStyle = value;
			this._firePropertyChanged('subgraphStyle', old, value);
			return this;
		},
		
		//////////////////////////////////////////////////////////////////////////////
		// Templating Properties
		
		_setTemplateField: function(field, template) {
			var _field = '_' + field;
			this[_field] = this[field] = template;
			if (template) {
				if (dojo.isString(template) && template.length > 0) {
					// if no DTL, convert it to plain JS object to prevent useless costly parsing
					if (!template.match(/{{|{%/)) 
						this[_field] = dojo.fromJson(template);
				}
				if (!(this[_field] instanceof ibm_ilog.diagram.data.dtlbinding.GfxTemplate)) 
					this[_field] = new ibm_ilog.diagram.data.dtlbinding.GfxTemplate(this[_field]);
			}
		},
		
		// nodeTemplate: String:
		//    A description object that defines the graphical representation of nodes.
		nodeTemplate: '',
		
		_setNodeTemplateAttr: function(template) {
			var old = this._created ? this.nodeTemplate : '';
			this._setTemplateField('nodeTemplate', template);
			this._firePropertyChanged('nodeTemplate', old, template);
			return this;
		},
		
		// nodeTemplateFunction: Function
		//    an optional function that computes the node template based on the item.
		//    The function receives the following parameters:
		//    item : the data item to create node from
		//    diagram : a reference to this diagram widget.
		nodeTemplateFunction: null,
		
		// createNodeFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Node instance for a given data
		//    item.
		//    The function receives the following parameters:
		//    item : the data item associated with this node.
		//    templateObj : A template object.
		//    graph : the target graph to create the node from. 
		//    diagram : a reference to this diagram widget.
		createNodeFunction: null,
		
		
		// subgraphTemplate: String:
		//    A description object that defines the graphical representation of subgraphs.
		subgraphTemplate: '',
		
		_setSubgraphTemplateAttr: function(/*String||Object*/template) {
			var old = this._created ? this.subgraphTemplate : '';
			this._setTemplateField('subgraphTemplate', template);
			this._firePropertyChanged('subgraphTemplate', old, template);
			return this;
		},
		
		// subgraphTemplateFunction: Function
		//    an optional function that computes the subgraph template based on the item.
		//    The function receives the following parameters:
		//    item : the data item to create subgraph from.
		//    diagram : a reference to this diagram widget.
		subgraphTemplateFunction: null,
		
		// createSubgraphFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Subgraph instance for a given data
		//    item.
		//    The function receives the following parameters:
		//    item : associated with this node.
		//    templateObj : the template object.
		//    graph : the target graph to create the subgraph from.
		//    diagram : a reference to this diagram widget.
		createSubgraphFunction: null,
		
		// linkTemplate: String:
		//    A description object that defines the graphical representation of links.
		linkTemplate: '',
		
		_setLinkTemplateAttr: function(/*String||Object*/template) {
			var old = this._created ? this.linkTemplate : '';
			this._setTemplateField('linkTemplate', template);
			this._firePropertyChanged('linkTemplate', old, template);
			return this;
		},
		
		// linkTemplateFunction: Function
		//    an optional function that computes the link template dynamically based on the item.
		//    The function receives the following parameters:
		//    linkItem : the link data item, if any.
		//    startNode: the link start node.
		//    endNode: the link end node.
		//    diagram : a reference to this diagram widget.
		linkTemplateFunction: null,
		
		// createLinkFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Link instance for a given data
		//    item.  
		//    The function receives the following parameters:
		//    linkItem : the link data item, if any.
		//    startNode: the link start node.
		//    endNode: the link end node.
		//    templateObj : the template object.
		//    graph : the parent graph of the link.
		//    diagram : a reference to this diagram widget.
		createLinkFunction: null,
		
		
		//////////////////////////////////////////////////////////////////////////////
		// Graph GraphLayout Properties
		
		_initializeLayoutProps:function(){
			if (this.nodeLayout)
				this._setNodeLayoutAttr(this.nodeLayout);
			if (this.automaticNodeLayout)
				this._setAutomaticNodeLayoutAttr(this.automaticNodeLayout);
			if (!this.nodeLayoutActive)
				this._setNodeLayoutActiveAttr(this.nodeLayoutActive);
			if (this.linkLayout)
				this._setLinkLayoutAttr(this.linkLayout);
			if (this.automaticLinkLayout)
				this._setAutomaticLinkLayoutAttr(this.automaticLinkLayout);
			if (!this.linkLayoutActive)
				this._setLinkLayoutActiveAttr(this.linkLayoutActive);
		},
		
		// summary: ibm_ilog.graphlayout.GraphLayout :
		//    The graph layout algorithm that will be executed on the nodes of this diagram.
		nodeLayout: null,
		
		_setNodeLayoutAttr: function(/*ibm_ilog.graphlayout.GraphLayout*/layout) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.getNodeLayout();
			if (old !== layout) {
				this.nodeLayout = layout;
				this._graph.setNodeLayout(layout);
				this._firePropertyChanged('nodeLayout', old, layout);
			}
			return this;
		},
		
		_getNodeLayoutAttr: function() {
			return this._viewportReady ? this._graph.getNodeLayout() : null;
		},
		
		//		Determines if the node layout must be executed automatically
		//		whenever the graph changes.
		automaticNodeLayout: false,
		
		_setAutomaticNodeLayoutAttr: function(/*boolean*/automatic) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isAutomaticNodeLayout();
			if (old !== automatic) {
				this.automaticNodeLayout = automatic;
				this._graph.setAutomaticNodeLayout(automatic);
				this._firePropertyChanged('automaticNodeLayout', old, automatic);
			}
			return this;
		},
		
		_getAutomaticNodeLayoutAttr: function() {
			return this._viewportReady ? this._graph.isAutomaticNodeLayout() : false;
		},
		
		// summary:
		//		Activates or deactivates the graph layout specified by setNodeLayout().
		nodeLayoutActive:true,
		
		_setNodeLayoutActiveAttr: function(/*boolean*/active) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isNodeLayoutActive();
			if (old !== active){
				this.nodeLayoutActive = active;
				this._graph.setNodeLayoutActive(active);
				this._firePropertyChanged('nodeLayoutActive', old, active);
			}
			return this;
		},
		
		_getNodeLayoutActiveAttr: function() {
			return this._viewportReady ? this._graph.isNodeLayoutActive() : true;
		},
		
		// summary: ibm_ilog.graphlayout.GraphLayout :
		//    The graph layout algorithm that will be executed on the links of this diagram.
		linkLayout: null,
				
		_setLinkLayoutAttr: function(/*ibm_ilog.graphlayout.GraphLayout*/layout) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.getLinkLayout();
			if (old !== layout) {
				this.linkLayout = layout;
				this._graph.setLinkLayout(layout);
				this._firePropertyChanged('linkLayout', old, layout);
			}
			return this;
		},

		_getLinkLayoutAttr: function() {
			return this._viewportReady ? this._graph.getLinkLayout() : null;
		},

		//		Determines if the link layout must be executed automatically
		//		whenever the graph changes.
		automaticLinkLayout: false,
		
		_setAutomaticLinkLayoutAttr: function(/*boolean*/automatic) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isAutomaticLinkLayout();
			if (old !== automatic) {
				this.automaticLinkLayout = automatic;
				this._graph.setAutomaticLinkLayout(automatic);
				this._firePropertyChanged('automaticLinkLayout', old, automatic);
			}
			return this;
		},
		
		_getAutomaticLinkLayoutAttr: function() {
			return this._viewportReady ? this._graph.isAutomaticLinkLayout() : false;
		},
		
		// summary:
		//		Activates or deactivates the link layout specified by setLinkLayout().
		linkLayoutActive: true,
		
		_setLinkLayoutActiveAttr: function(/*boolean*/active) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isLinkLayoutActive();
			if (old !== active) {
				this.linkLayoutActive = active;
				this._graph.setLinkLayoutActive(active);
				this._firePropertyChanged('linkLayoutActive', old, active);
			}
			return this;
		},
		
		_getLinkLayoutActiveAttr: function() {
			return this._viewportReady ? this._graph.isLinkLayoutActive() : true;
		},
		
		// END OF PROPERTIES
		// =======================================================================
		
		
		// =======================================================================
		//
		// WIDGET LIFECYCLE:
		//
		//  The following group methods define the process of construction, initialization, deinitialization and 
		//  related. These are mostly hooks into the Dijit framework.
		//
		
		_connections: null,
		_subscriptions: null,
		
		constructor: function() {
			this._connections = [];
			this._subscriptions = [];
		},
		
		postMixInProperties: function() {
			this.inherited(arguments);
			
			this._beginInit();
			
			// Create Viewport, which defines the templateString to use.
			this._createViewport();
			
			// Create Selection object
			this._createSelection();
			
			// register node events
			this._initializeNodeEvents();
			
			// initialize annotations infrastructure
			this._initializeAnnotations();
			
			// initialize adorner infrastructure
			this._initializeAdorners();

			// create interactor manager
			this._interactors = new ibm_ilog.diagram.interactors.InteractorManager();

		},
		
		postCreate: function() {
		
			this.inherited(arguments);
			
			this._initializeViewport();

			this.getViewport().whenLoaded(dojo.hitch(this, function() {
				if (this["backMenu"]) {
					try {
						var thiz = this;
						this._viewport.getEventSourceBackground().connect("oncontextmenu", function(e) {
							thiz.lastMenuTarget = thiz._graph;
							thiz.lastMenuEvent = e;
						});
						this["backMenu"].bindDomNode(this._viewport.getEventSourceBackground().getEventSource());
					} catch (e) {
						R.warn("EventConnectionFailed", "oncontextmenu");
					}
				}
				
				this._graph = this.getViewport().getContent().createGraph();
				this._graph._diagram = this;
				if (dojo.config.isBidiInDojoDiagrammer)
				    this._graph.textDir = this.textDir;							
				
				if (!this.disableGraphWai) {
					this._initializeGraphWai();
				} else {
					this[__set]('tabIndex','0');
				}
				
				this._connections.push(dojo.connect(this._graph, "onLayoutAnimationStart", this, function() {
					this._viewport.setRefreshContentsSuspended(true);
				}));
				
				this._connections.push(dojo.connect(this._graph, "onLayoutAnimationEnd", this, function() {
					this._viewport.setRefreshContentsSuspended(false);
				}));
				
				this._viewportReady = true;
				
				// initialize viewport properties
				this._initializeViewportProperties();
				
				// initialize graph graphlayout properties
				this._initializeLayoutProps();
				
				// initialize interactors
				this._initializeInteractors();
				
				// initialize key interactors
				this._initializeKeyInteractors();
				
				this.enterStandardMode();
				
				// If there was a nodeStore defined on the widget then draw     
				if (this._getNodesStore()) {
					this.load();
				}
				
				this.postPostCreate();
				
				this._endInit();
			}));
		},
		
		postPostCreate: function() {
			//  summary:
			//      Hook executed just after Diagram postCreate. Used by DiagramEditor subclass.
		},
		
		
		onLoaded: function() {
			//  summary:
			//      Invoked when the diagram has loaded the data. 
		},
		
		resize: function(info) {
			// No size info passed: use the widget's marginBox
			if(!info || !info.w || !info.h || info.w <= 0 || info.h <= 0) 
			  info = dojo.marginBox(this.domNode);
			
			// We get negative sizes sometimes... (e.g. on Chrome/Canvas)
			if (!info || !info.w || !info.h || info.w <= 0 || info.h <= 0) 				
  	  	return;
	    
			// We get _viewport.getSurface() being null sometimes... (e.g. on IE8)
			if (this._viewport.getSurface()) {
				this._viewport.setSize({
					width: info.w,
					height: info.h
				});
			}
		},
		
		destroy: function() {
		
			if (this._interactors) {
				this._interactors.switchTo();
			}
			
			dojo.forEach(this._connections, dojo.disconnect, dojo);
			
			dojo.forEach(this._subscriptions, dojo.unsubscribe, dojo);
			
			this._destroyDataConnector();

			if (this._graph) {
				this._graph.clear(true); // true = call dispose() on children
			}
			this._graph = null;
			
			if (this._viewport) {
				this._viewport.destroy();
			}
			
			this.inherited(arguments);
		},
		
		reset: function() {
			// clear the selection
			this.getSelection().clear();
			// reset the dataconnector and the associated graphical shapes created from the stores
			this._reset();
			
			// clear any remaining shapes (those not created from datastores)
			this._graph.clear(/*dispose*/true);
			
			// Defect 1875
			// The layout instances do cleanup work when notified that the content of the
			// graph has changed. Hence, let's notify the layout:
			var nodeLayout = this._graph.getNodeLayout();
			var linkLayout = this._graph.getLinkLayout();
			var graphModelEvent = null;
			if (nodeLayout || linkLayout) {
				// We do not need a dojo.require for GraphLayoutModelEvent, because this is 
				// required by GraphLayout itself. Hence, if a layout instance is already
				// set on the graph, the event class has already been required. 
	  		graphModelEvent = new ibm_ilog.graphlayout.GraphLayoutModelEvent();
				graphModelEvent.setType(ibm_ilog.graphlayout.GraphLayoutModelEvent.STRUCTURE_CHANGED);
	  	}
			if (nodeLayout)
				nodeLayout.contentsChanged(graphModelEvent);
		  if (linkLayout)
				linkLayout.contentsChanged(graphModelEvent);
				
			this._viewport.reset();
			this.enterStandardMode();
			
			return this;
		},
		
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		// moved to _Diagram.js
		
		disableGraphWai: true,

		// =======================================================================
		//
		//  VIEWPORT:
		//
		//      The viewport is an abstraction used to control the currently visualized area of the model.
		//
		//      see:
		//      widget.Viewport, widget.DivViewport
		//
		//
		
		//
		//  _viewport: ibm_ilog.diagram.widget.Viewport
		//      The viewport
		//
		_viewport: null,
		
		getViewport: function() {
			// summary:
			//     Gets the Viewport instance.
			return this._viewport;
		},
		
		visualizeBounds: function( /*Node|Link|Subgraph*/graphic, /*{expandCurrent:false,airFactor}*/ options) {
			//
			//	summary:
			//		Makes the bounds of a graph object visible, givin it an extra air between the 
			//		area and the borders of the viewport if possible. The view may be zoomed out 
			//		if required, but it is never zoomed in.
			//
			//	graphic:
			//		The object to make visible
			//
			//	options:
			//		Viewport options kwArgs
			//
			var b = graphic.getBounds(this.getGraph());
			if (!b) {
				return;
			}
			this._viewport.visualizeArea(b, options);
		},
		
		centerOnNode: function( /*Node|Link*/nodeOrLink, /*{animate}*/ options) {
			//
			//	summary:
			//		Centers the viewport on an object of the graph. Does not change the zoom level.
			//
			nodeOrLink = this.asGraphElement(nodeOrLink);
			var b = nodeOrLink.getBounds(this.getGraph());
			if (!b) 
				return;
			if (nodeOrLink instanceof ibm_ilog.diagram.Node) {
				this._viewport.centerAt(g.getRectCenter(b), options);
			} else {
				if (nodeOrLink instanceof ibm_ilog.diagram.Link) {
					//
					//	TODO: temporary, correct this when it's ready
					//
					this._viewport.centerAt(g.getRectCenter(b), options);
				}
			}
			
		},
		
		_createViewport: function() {
			//
			//	Creates the Viewport object
			//
			//	tags:
			//		private
			//
			this._viewport = new (ibm_ilog.diagram.widget[this.viewportClass])();
			this.templateString = this._viewport.templateString();
			
			
		},
		
		buildRendering: function(){
			// there are no variables in the template so we can cache the DOM tree
			var node = dojo._toDom(this.templateString);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + this.templateString);
			}
			this.domNode = node;
			var nodes = node.getElementsByTagName("*");
			for (var i = -1; i < nodes.length; i++){
				var n = (i == -1) ? node : nodes[i];
				var attachPoint = n.getAttribute("dojoAttachPoint");
				if (attachPoint == 'canvasNode')
					dojo.style(n, "overflow",'hidden');
				this[attachPoint]= n;
			}
			
			//this.inherited(arguments);
			
			this._fillContent(this.srcNodeRef);
			
		},
		
		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},
		
		
		_initializeViewport: function() {
			//
			//	Creates the Viewport object
			//
			//	tags:
			//		private
			//
			this._viewport.initialize(this);
			
			this._viewport.setAirFactor(this.viewportAirFactor);
			this._viewport.setAnimated(this.viewportAnimated);
			this._viewport.setMaxZoom(this.viewportMaxZoom);
			this._viewport.setMinZoom(this.viewportMinZoom);
			this._viewport.setResizeMode(this.viewportResizeMode);
			
		},
		
		createAdorner: function(clazz, templates, rootId) {
			var a = this._adornerLayer.createObject(clazz);
			a.initialize(this, templates, rootId);
			return a;
		},
		
		destroyAdorner: function(a) {
			a.destroy();
			a.getParent().remove(a);
		},
		
	  fitToContents: function(margin, zoomOutOnly) {
	    this.getGraph().fitToContents(margin, zoomOutOnly);
	  },
	  
		// =======================================================================
		//
		// INTERACTORS:
		//
		//		An interactor is an abstraction of a user interface process. Each interactor controls some kind
		//		process. It is responsible for making it available and has all the UI logic pertinent to it.
		//
		//		The interactors are managed by an InteractionManager, which organizes them by id and is responible
		//		for distinguishing the currently active interactors.
		//
		//		The diagram widget uses interactors to provide the users with ways to interact with the Graph
		//		surface. It uses a PanInteractor to provide the user with a mode in which the surface
		//		can be navigating by grabbing it with the left mouse button and zoomed using the mouse wheel.
		//		It also uses a SelectionInteractor to provide the user the ability of selecting nodes by 
		//		left-clicking them.
		//
		//		see:
		//		event.Interactor, event.PanInteractor, event.SelectionInteractor, event.InteractorManager
		//
		
		//
		//	_interactors: ibm_ilog.diagram.interactors.InteractorManager
		//		The interactor manager
		//
		_interactors: null,
		
		
		_setupInteractors: function(ids) {
			dojo.forEach(ids, function(i) {
				this._interactors.get(i).setEnabled(this[i]);
			}, this);
		},
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new ibm_ilog.diagram.interactors.PanInteractor().initialize(this));
			this._interactors.add("select", new ibm_ilog.diagram.interactors.SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ibm_ilog.diagram.interactors.ZoomInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom"]);
		},
		
		startup: function() {
			this.inherited(arguments);
			
			//	[av] adorner creation fails on certain renderers/browsers if this is done before "startup".
			//this._createAdorners();
			this._viewport.whenLoaded(dojo.hitch(this, this._createAdorners));
			
			this._loadAnnotations();
		},
		
		getInteractor: function(id) {
			return this._interactors.get(id);
		},
		
		getInteractorManager: function() {
			return this._interactors;
		},

		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom");
		},
		
		assignInteractorInputs: function(iid, assignments) {
			this.getInteractor(iid).assignInputs(assignments);
		},
		
		setMarqueeMode: function( /*int*/b) {
			this._interactors.get("marquee").setMode(b);
		},
		
		enableMarqueePartialSelection: function( /*boolean*/b) {
			this._interactors.get("marquee").setPartialSelectionEnabled(b);
		},

		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		
		// moved to _Diagram.js
		
		_initializeKeyInteractors: function() {
		},
		
		// =======================================================================
		//
		//	GRAPH MOUSE EVENTS:
		//
		//		The user can connect to input events related to entities through the events defined in this 
		//		seciton. These are mapped to the corresponding DOM events when they occur on the corresponding
		//		entity (Node or Link).
		//
		
		onNodeContextMenu: function(n, e) {
			this.lastMenuTarget = n;
			this.lastMenuEvent = e;
		},
		onNodeClick: _emptyEntityHandler,
		onNodeDblClick: _emptyEntityHandler,
		onNodeMouseUp: _emptyEntityHandler,
		onNodeMouseDown: _emptyEntityHandler,
		onNodeMouseMove: _emptyEntityHandler,
		onNodeMouseEnter: _emptyEntityHandler,
		onNodeMouseLeave: _emptyEntityHandler,
		onNodeMouseOver: _emptyEntityHandler,
		onNodeMouseOut: _emptyEntityHandler,
		// Touch events for Node
		onNodeTouchStart: function (ge,event){
			this.onNodeMouseDown(ge,event);
		},
		onNodeTouchEnd: function (ge,event){
			this.onNodeMouseUp(ge,event);
			this.onNodeClick(ge,event);
			if(event.touches.length == 0){
				var delay = new Date().getTime() - this._lastUp;
				if(this._lastUp && (delay < 500)){
					this.onNodeDblClick(ge,event);
					delete this._lastUp;
				}else{
					this._lastUp = new Date().getTime();
				}   
			}			
		},
		onNodeTouchMove: function (ge,event){
			this.onNodeMouseMove(ge,event);
		},
		
		onLinkContextMenu: function(l, e) {
			this.lastMenuTarget = l;
			this.lastMenuEvent = e;
		},
		onLinkClick: _emptyEntityHandler,
		onLinkDblClick: _emptyEntityHandler,
		onLinkMouseUp: _emptyEntityHandler,
		onLinkMouseDown: _emptyEntityHandler,
		onLinkMouseMove: _emptyEntityHandler,
		onLinkMouseEnter: _emptyEntityHandler,
		onLinkMouseLeave: _emptyEntityHandler,
		onLinkMouseOver: _emptyEntityHandler,
		onLinkMouseOut: _emptyEntityHandler,
		
		// Touch events for Link
    onLinkTouchStart: function (ge,event){
      this.onLinkMouseDown(ge,event);
    },
    onLinkTouchEnd: function (ge,event){
        this.onLinkMouseUp(ge,event);
        this.onLinkClick(ge,event);
        if(event.touches.length == 0){
            var delay = new Date().getTime() - this._lastUp;
            if(this._lastUp && (delay < 500)){
                this.onLinkDblClick(ge,event);
                delete this._lastUp;
            }else{
                this._lastUp = new Date().getTime();
            }   
        }           
    },
    onLinkTouchMove: function (ge,event){
        this.onLinkMouseMove(ge,event);
    },
  
		
		onSubgraphContextMenu: function(s, e) {
			this.lastMenuTarget = s;
			this.lastMenuEvent = e;
		},
		onSubgraphClick: _emptyEntityHandler,
		onSubgraphDblClick: _emptyEntityHandler,
		onSubgraphMouseUp: _emptyEntityHandler,
		onSubgraphMouseDown: _emptyEntityHandler,
		onSubgraphMouseMove: _emptyEntityHandler,
		onSubgraphMouseEnter: _emptyEntityHandler,
		onSubgraphMouseLeave: _emptyEntityHandler,
		onSubgraphMouseOver: _emptyEntityHandler,
		onSubgraphMouseOut: _emptyEntityHandler,

    // Touch events for subGraph
    onSubgraphTouchStart: function (ge,event){
      this.onSubgraphMouseDown(ge,event);
    },
    onSubgraphTouchEnd: function (ge,event){
        this.onSubgraphMouseUp(ge,event);
        this.onSubgraphClick(ge,event);
        if(event.touches.length == 0){
            var delay = new Date().getTime() - this._lastUp;
            if(this._lastUp && (delay < 500)){
                this.onSubgraphDblClick(ge,event);
                delete this._lastUp;
            }else{
                this._lastUp = new Date().getTime();
            }   
        }           
    },
    onSubgraphTouchMove: function (ge,event){
        this.onSubgraphMouseMove(ge,event);
    },
  
        
		
		//
		//	lastMenuTarget:
		//		The entity on which the last/current menu was opened.
		//
		lastMenuTarget: null,
		
		//
		//	linkMenu:
		//		An optional dijit.Menu for links (property)
		//
		_initializeNodeEvents: function() {
			//
			// TODO analyze and maybe replace this connection method by
			// the deferred method used in _Widget.
			//
			// May also be replaced by a single connection plus a linear search on the geometry.
			//
			//	tags:
			//		private
			//
			
			var eventNames = ["ContextMenu", "Click", "DblClick", "MouseUp", "MouseDown", "MouseMove", "MouseEnter", "MouseLeave", "MouseOver", "MouseOut"];
			if (ibm_ilog.diagram.mobileEnabled){
				eventNames.push("TouchStart","TouchMove","TouchEnd");
			}
			var lcEventNames = dojo.map(eventNames, function(e) {
				return e.toLowerCase();
			});
			
			var self = this;
			
			// Connect to node events
			
			dojo.forEach(["Node", "Link", "Subgraph"], function(get) {
				var lget = get.toLowerCase();
				var evtPrefix = "on" + get;
				var menuName = lget + "Menu";
				this._connections.push(dojo.connect(this, "_on" + get + "Created", this, function(ge) {
					for (var i = 0; i < eventNames.length; i++) {
						(function() {
							var eventName = evtPrefix + eventNames[i];
							try {
								ge.connect("on" + lcEventNames[i], self, function(e) {
									return (this[eventName])(ge, e);
								});
							} catch (e) {
								R.warn("EventConnectionFailed", "on" + lcEventNames[i]);
							}
						})();
					}
					if (this[menuName]) {
						try {
							this[menuName].bindDomNode(ge.getEventSource());
						} catch(e){}
					}
				}));
				
			}, this);
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, function(graphElement) {
				this.beforeGraphElementDispose(graphElement);
			}));
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/afterDisposing', this, function(graphElement) {
				this.afterGraphElementDispose(graphElement);
			}));
			
		},
		
		beforeGraphElementDispose: function(graphElement) {
			//
			//	summary:
			//		this method is called before any graph element is disposed
		},
		
		afterGraphElementDispose: function(graphElement) {
			//
			//	summary:
			//		this method is called after any graph element is disposed
		},
		
		// =======================================================================
		//
		//	SELECTION:
		//
		//		The interface to the currently selected nodes/dataitems.
		//
		//		see:
		//		widget.SelectionInteractor, widget.Selection
		//
		//
		
		
		//
		//	_selection: ibm_ilog.diagram.Selection
		//		The selection object
		//
		_selection: null,
		
		getSelection: function() {
			//
			//	summary:
			//		Return the selection object
			//    	
			return this._selection;
		},
		
		// bindable property that defines the selection mode
		// See: ibm_diagram_Selection
		selectionMode: null,
		
		_createSelection: function() {
			//
			//	summary:
			//		Creates the selection object
			//    	
			if (this.selectionMode == null) {
				this.selectionMode = ibm_ilog.diagram.Selection.standard;
			}
			this._selection = new ibm_ilog.diagram.Selection(this);
			
		},
		
		
		// =======================================================================
		//
		//	ADORNERS FOCUS:
		//
		//		The interface to manage focus on an entity for operation
		//
		//		see:
		//		widget.AdornerFocusInteractor
		//
		//

		//
		//  _nodeAdornersFocus: ibm_ilog.diagram.Node|ibm_ilog.diagram.Subgraph
		//      The currently adorner-focused node/subgraph 
		//
		_nodeAdornersFocus: null,

		//
		//	getAssociatedNodeAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given node.
		//
		getAssociatedNodeAdornersFunction: function(node) {
			return [];
		},
		
		//
		//	getAssociatedLinkAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given link.
		//
		getAssociatedLinkAdornersFunction: function(link) {
			return [];
		},

		//
		//	getAssociatedSubgraphAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given subgraph.
		//
		getAssociatedSubgraphAdornersFunction: function(sg) {
			return [];
		},

		getNodeAdornersFocus: function() {
			//
			//	summary:
			//		Returns the current node adorners focus.
			//			
			return this._nodeAdornersFocus;
		},
		
		setNodeAdornersFocus: function(ge) {
			//
			//	summary:
			//		Sets the node adorners focus on a graph element. Used by the adorners focus interactor;
			//			
			var oldValue = this._nodeAdornersFocus;
			this._nodeAdornersFocus = ge;
			if (oldValue != ge) {
				this.onNodeAdornersFocusChanged(oldValue, ge);
			}
			return this;
		},
		
		onNodeAdornersFocusChanged: function(oldValue, newValue) {
			//
			//	summary:
			//		event that occurs when the adorners focus has changed.
			//			
			var adorners = [];
			if(newValue) {
				if(newValue._isIBMDiagramSubgraph) {
					adorners = this.getAssociatedSubgraphAdornersFunction(newValue);
				} else {
					adorners = this.getAssociatedNodeAdornersFunction(newValue);
				}
			}
			dojo.forEach(this._activeNodeAdorners,function(g){
				g.setAdorned(null);
			},this);
			
			this._activeNodeAdorners = adorners;

			dojo.forEach(this._activeNodeAdorners,function(g){
				g.setAdorned(newValue);
			},this);
		},

		
		//
		//  _nodeAdornersFocus: ibm_ilog.diagram.Link
		//      The currently adorner-focused link 
		//
		_linkAdornersFocus: null,
		
		getLinkAdornersFocus: function() {
			//
			//	summary:
			//		Returns the current link adorners focus.
			//			
			return this._linkAdornersFocus;
		},
		
		setLinkAdornersFocus: function(ge) {
			//
			//	summary:
			//		Sets the link adorners focus on a graph element. Used by the adorners focus interactor;
			//			
			var oldValue = this._linkAdornersFocus;
			this._linkAdornersFocus = ge;
			if (oldValue != ge) {
				this.onLinkAdornersFocusChanged(oldValue, ge);
			}
			return this;
		},
		onLinkAdornersFocusChanged: function(oldValue, newValue) {
			//
			//	summary:
			//		event that occurs when the link adorners focus has changed.
			//			
			var adorners = []; 
			if(newValue) {
				adorners = this.getAssociatedLinkAdornersFunction(newValue);
			}
			
			dojo.forEach(this._activeLinkAdorners,function(g){
				g.setAdorned(null);
			},this);
			
			this._activeLinkAdorners = adorners;

			dojo.forEach(this._activeLinkAdorners,function(g){
				g.setAdorned(newValue);
			},this);
		},

		mustFreeAdornersFocus: function(graphElement, point, margin, outOfBounds) {
			// Summary:
			//		This function is called from the adorners focus interactor to decide whether to free the adorners focus or not.
			
			// This special case is added to be able to modify intermediate point on spline links
			if (outOfBounds && graphElement._isIBMDiagramLink && graphElement.isSelected() && graphElement.getCurveTension() > 0) {
				return false;
			}
			return outOfBounds;
		},
		
		changeAdornersFocus: function(focus, newFocus, possibleCursors, mustchangeAdornersFocus) {
			// Summary:
			//		This function is called from the adorners focus interactor to decide whether to change the adorners focus or not.
			// return: 
			//		the new gfx to apply the cursor, or null if not need to change. 	
			if (mustchangeAdornersFocus) {
				return newFocus;
			} else {
				return null;
			}
		},
		
		// _lowCpuMode: Enables the use of some low cpu consuming functions.
		// Be aware that some of them have less precision than the high cpu consumption.
		_lowCpuMode: false,
		
		isLowCpuMode: function() {
			//	Summary:
			//		return true if the low cpu mode is enabled
			//
			return this._lowCpuMode;
		},
		
		
		enableLowCpuMode: function(enable) {
			//	Summary:
			//		enables or disables the low cpu mode
			//
			this._lowCpuMode = enable;
			this.lowCpuModeChanged();
		},
		
		lowCpuModeChanged: function() {
			//	Summary:
			//		this function is called when the low cpu mode is changed
			//
		},

		// =======================================================================
		//
		//	Adorners
		//

		// moved to _Diagram.js
		
		_initializeAdorners: function() {
		},
		
		_createAdorners: function() {
		},
		
		// =======================================================================
		//
		//	FOCUSED ELEMENT:
		//
		//		The interface to manage focus on an entity for operation and navigation
		//
		//		see:
		//		widget.FocusInteractor
		//
		//

		focus: true,

		//
		//  _focusedElement: /*GraphElement*/
		//      The graph element currently focused
		//
		_focusedElement: null,
		
		_focusedNodeAdorner: null,
		
		_focusedLinkAdorner: null,

		_currentFocusAdorner: null,
		
		getFocusedElement: function() {
			return this._focusedElement;
		},
		
		getFocusAdorner: function(ge) {
			var node = this._focusedNodeAdorner;
			var link = this._focusedLinkAdorner;
			
			return {N:node,S:node,L:link}[ge.TID];
		},
		
		setFocusedElement: function(ge) {
			var oldValue = this._focusedElement;
			this._focusedElement = ge;
			
			var adorner = null;
			if(ge) {
				adorner = this.getFocusAdorner(ge);
			}
			
			if (adorner) {
				adorner.setAdorned(ge);
			}
			if(adorner!=this._currentFocusAdorner && this._currentFocusAdorner) {
				this._currentFocusAdorner.setAdorned(null);
			}
			this._currentFocusAdorner = adorner;
			
			if (oldValue != ge) {
				this.onFocusedElementChanged(oldValue, ge);
			}
			
			// [av] make sure we get focus when user clicks the diagram.
			// the standard capture is hidden because the only element with
			// tabIndex 0 may be hidden under several layers.
			this._htmlRefocus();

			return this;
		},
		
		onFocusedElementChanged: function(oldValue, newValue) {
		},
		
		// =======================================================================
		//
		//	Layout
		//
		
		
		performGraphLayout: function() {
		// summary:
		//		Executes the graph layout and/or the link layout specified by the setNodeLayout()
		//		and setLinkLayout methods.
		// description:
		//		If a non-null graph layout has been specified by calling setNodeLayout(),
		//		and if the graph layout is active (that is, unless setNodeLayoutActive(false) has been called),
		//		then the graph layout algorithm is performed on the contents of this graph.
		//		Then, if a non-null link layout has been specified by calling setLinkLayout(),
		//		and if the link layout is active (that is, unless setLinkLayoutActive(false) has been called),
		//		then the link layout algorithm is performed on the contents of this graph.
			
			this._graph.performGraphLayout();
		},
		
		//
		// Input Events
		//
		
		// =======================================================================
		//
		//	DATA CONNECTION:
		//
		//		The interface to manage attributes related to the node and links data stores
		//
		//		see:
		//		widget.DataConnector
		//
		// =======================================================================
		
		load: function() {
			// summary: 
			//        loads the data stores and creates the diagram.
			
			// Temporary
			this._load();
		},
		
		// =======================================================================
		
		//
		// Templating
		//    
		
		_nodeTemplate: null,
		_subgraphTemplate: null,
		_linkTemplate: null,
		
		createTemplatedShape: function(/*Function*/shapeType, /*String||Object?*/template, /*ibm_ilog.diagram.Graph?*/onGraph, /*Object?*/style, /*dojo.data.api.Read?*/store, /*Object?*/item) {
			// summary:
			// 		Creates an instance of the specified type fully initialized.
			// description:
			//     This method creates an instance of the specified type and populates it with the specified template.
			//     If the template defines bindings to the shape or the data item (if any), bindings are resolved.
			// shapeType: Function
			//		The type of the instance.
			// template: String||Object
			//    a template defining the shape contents. If no template is specified, the default Subgraph template will be used. Optional.
			// onGraph: ibm_ilog.diagram.Graph
			//    the parent graph. Optional.
			// store : dojo.data.api.Read
			//	a data store. Optional.
			// item: dojo.data.api.Item:
			//    a data item associated with this shape. Optional.
			
			onGraph = onGraph || this._graph;
			// Creates the node instance
			var shape = onGraph.createObject(shapeType);
			// Set the data context
			if (store && item) {
				shape.setData(new ibm_ilog.diagram.data.DataContext(item, store));
			}
			// apply the style
			if (style) {
				ibm_ilog.diagram.style.applyTo(style, shape);
			}
			// Apply the template (and resolve bindings, if any)
			onGraph._applyTemplate(shape, template, null, null);
			return shape;
		},
		
		_resolveTemplate: function(func, prop, def) {
			var template = null;
			// is there a function to return a template ?
			if (func) {
				// Should return a String to enable DTL engine. An Object means no binding.
				template = func();
			}
			if (!template) {
				// let's try the property 
				template = prop;
			}
			if (!template) {
				//let's use the default one
				template = def;
			}
			return template;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onNodeCreated: function(node, diagram) {
			this.onNodeCreated(node, diagram);
		},
		
		onNodeCreated: function(node, diagram) {
			// summary:
			//    Invoked when the specified node has been created.
			// node: ibm_ilog.diagram.Node: the new node instance.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		},
		
		resolveNodeTemplate: function(/*dojo.data.Item*/item) {
			// summary:
			//    Returns the template to represent the specified node item.
			//    item: dojo.data.Item: the node data item.
			
			var func = this.nodeTemplateFunction ? dojo.partial(this.nodeTemplateFunction, item, this) : null;
			return this._resolveTemplate(func, this._nodeTemplate, ibm_ilog.diagram.Node.defaultTemplate);
		},
		
		createNode: function(/*Object or String*/template, /*dojo.data.api.Item*/ item, /*ibm_ilog.diagram.Graph*/ onGraph, /*Function?*/ nodeType) {
			// summary:
			//        Creates a Node that represents the specified data item.
			// template: Object or String 
			//		The template to use for this node. Optional.
			// item: dojo.data.api.Item 
			//		a data item associated with this node. Optional.
			// onGraph: ibm_ilog.diagram.Graph 
			//		The parent graph of the node. If null, the top-level graph is used. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        A ibm_ilog.diagram.Node instance.
			
			
			if (!onGraph) {
				onGraph = this._graph;
			}
			
			template = template || this.resolveNodeTemplate(item);
			
			var node = null;
			// let's try a custom node factory
			if (this.createNodeFunction) {
				node = this.createNodeFunction.call(null, item, template, onGraph, this);
				if (node && (!(node instanceof ibm_ilog.diagram.Node))) {
					ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedReturnValueType", this, createNodeFunction, "ibm_diagam.Node", typeof(node));
				}
				
			}
			if (!node) {
				nodeType = nodeType || ibm_ilog.diagram.Node;
				node = this.createTemplatedShape(nodeType, template, onGraph, this.nodeStyle, this.nodesStore, item);
			}
			
			// set Label property
			if (item) {
				var text = this.nodesStore.getLabel(item);
				if (text) {
					node.setLabel(text);
				}
			}
			
			// TODO: should we pass the store and item ? 
			this._onNodeCreated(node, this);
			
			return node;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onSubgraphCreated: function(subgraph, diagram) {
			this.onSubgraphCreated(subgraph, diagram);
		},
		
		onSubgraphCreated: function(/*ibm_ilog.diagram.Subgraph*/subgraph, /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
			// summary:
			//    Invoked when the specified subgraph object has been created and initialized. 
			//    The default implementation does nothing.
			// subgraph: ibm_ilog.diagram.Subgraph: the new subgraph instance.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		},
		
		resolveSubgraphTemplate: function(item) {
			// summary:
			//    Returns the template to represent the specified subgraph item.
			
			var func = this.subgraphTemplateFunction ? dojo.partial(this.subgraphTemplateFunction, item, this) : null;
			return this._resolveTemplate(func, this._subgraphTemplate, ibm_ilog.diagram.Subgraph.defaultTemplate);
		},
		
		createSubgraph: function(/*Object or String*/template, /*dojo.data.api.Item*/ item, /*ibm_ilog.diagram.Graph*/ onGraph, /*Function?*/ nodeType) {
			// summary:
			//        Creates a Subgraph that represents the specified data item.
			// template: Object or String The template to use for this subgraph. Optional.
			// item: dojo.data.api.Item a data item associated with this subgraph. Optional.
			// onGraph: ibm_ilog.diagram.Graph The parent graph of the subgraph. If null, the top-level graph is used. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        A ibm_ilog.diagram.Subgraph instance.
			
			if (!onGraph) {
				onGraph = this._graph;
			}
			
			template = template || this.resolveSubgraphTemplate(item);
			
			var subgraph = null;
			// let's try a custom subgraph factory
			if (this.createSubgraphFunction) {
				subgraph = this.createSubgraphFunction.call(null, item, template, onGraph, this);
				if (subgraph && (!(subgraph instanceof ibm_ilog.diagram.Subgraph))) {
					ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedReturnValueType", this, createSubgraphFunction, "ibm_diagam.Subgraph", typeof(subgraph));
				}
			}
			if (!subgraph) {
				nodeType = nodeType || ibm_ilog.diagram.Subgraph;
				subgraph = this.createTemplatedShape(nodeType, template, onGraph, this.subgraphStyle, this.nodesStore, item);
			}
			
			// set Label property
			if (item) {
				var text = this.nodesStore.getLabel(item);
				if (text) {
					subgraph.setLabel(text);
				}
			}
			
			
			this._onSubgraphCreated(subgraph, this);
			
			return subgraph;
		},
		
		connectNodes: function(/* ibm_ilog.diagram.Node */start, /* ibm_ilog.diagram.Node */ end, /*String||Object*/ template, /*dojo.data.Item*/ linkItem) {
			// summary:
			//        Connects the specified nodes.
			//        This method creates a ibm_ilog.diagram.Link instance invoking the createLink() method
			//        and connects its extremities to the specified nodes. Optionally, one may specify
			//        the linksStore item associated with this link.
			// start: ibm_ilog.diagram.Node : the link start node.
			// end: ibm_ilog.diagram.Node: the link end node.
			// template: Object or String : the template to use. Optional.
			// linkItem: dojo.data.Item: the data item corresponding to this link, if any. Optional.
			// returns: ibm_ilog.diagram.Link
			
			if (!linkItem) {
				linkItem = null;
			}
			
			var link = this.createLink(start, end, template, linkItem);
			if (!link.getStartPort()) {
				link.setStartNode(start);
			}
			if (!link.getEndPort()) {
				link.setEndNode(end);
			}
			return link;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onLinkCreated: function(link, start, end, diagram) {
			this.onLinkCreated(link, start, end, diagram);
		},
		
		onLinkCreated: function(/*ibm_ilog.diagram.Link*/link, /*ibm_ilog.diagram.Node*/ start, /*ibm_ilog.diagram.Node*/ end, /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
			// summary:
			//    Invoked when the specified link has been created.
			// link: ibm_ilog.diagram.Link: the new link instance.
			// start: ibm_ilog.diagram.Node: the link start node, if any.
			// end: ibm_ilog.diagram.Node: the link end node, if any.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		
		},
		
		resolveLinkTemplate: function(/*dojo.data.Item*/linkItem,/*ibm_ilog.diagram.Node*/ start, /*ibm_ilog.diagram.Node*/ end) {
			// summary:
			//    Returns the link template to represent the specified relationship.
			// linkItem: dojo.data.Item: the data item corresponding to this link, if any.
			// start: ibm_ilog.diagram.Node : the link start node.
			// end: ibm_ilog.diagram.Node: the link end node.
			
			var func = this.linkTemplateFunction ? dojo.partial(this.linkTemplateFunction, linkItem, start, end, this) : null;
			return this._resolveTemplate(func, this._linkTemplate, ibm_ilog.diagram.Link.defaultTemplate);
		},
		
		createLink: function(/* ibm_ilog.diagram.Node */start, /* ibm_ilog.diagram.Node */ end, /*Object or String*/ template, /*dojo.data.Item*/ linkItem, /*Function?*/ linkType) {
			// summary:
			//        Creates a Link to connect the specified nodes. Optionally, one may specify
			//        the linksStore item associated with this link. 
			// description: This method only creates the Link instance and does not connect the nodes. Caller may invoke
			//        the Diagram.connectNodes() method instead, which invokes this method and perfom the connection.
			// start: ibm_ilog.diagram.Node
			//		the link start node.
			// end: ibm_ilog.diagram.Node
			//		the link end node.
			// template: Object or String
			//		the template to use. Optional.
			// linkItem: dojo.data.Item
			//		the data item corresponding to this link, if any. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        a ibm_ilog.diagram.Link instance.
			
			if (!linkItem) {
				linkItem = null;
			}
			
			template = template || this.resolveLinkTemplate(linkItem, start, end);
			
			// let's try a custom factory
			var link = null;
			var graph = gu.lowestCommonAncestor(start, end);
			if (this.createLinkFunction) {
				link = this.createLinkFunction.call(null, linkItem, start, end, template, graph, this);
				if (link && (!(link instanceof ibm_ilog.diagram.Link))) {
					ibm_ilog.diagram.util.ErrorReporter.error("UnexpectedReturnValueType", this, createLinkFunction, "ibm_diagam.Link", typeof(link));
				}
			}
			if (!link) {
				linkType = linkType || ibm_ilog.diagram.Link;
				link = this.createTemplatedShape(linkType, template, graph, this.linkStyle, this.linksStore, linkItem);
			}
			this._onLinkCreated(link, start, end, this);
			return link;
		},
		
		
		//
		// Data Binding
		//
		
		getBoundItem: function(nodeOrLink) {
			// summary: 
			//     Returns the data item, if any, associated with the specified node or link.
			// nodeOrLink: ibm_ilog.diagram.Node || ibm_ilog.diagram.Link || ibm_ilog.diagram.Subgraph: 
			//     the node, link or subgraph instance.
			
			var data = nodeOrLink.getData();
			return data ? data.item : null;
		},
		
		//------------------
		// Image export API
		//------------------
		
		toSvg: function(/*Function*/onComplete, /*Function*/ onError, /*Boolean*/ all) {
			// summary:
			//     Export the graph as an SVG string.
			// description:
			//		Export the graph as an SVG string. This features requires Dojo 1.4.x or greater.
			// onComplete: Function
			//		The callback that receives the resulting SVG.
			// onError: Function
			//		The function called when the generation fails.
			// all: Boolean
			//		Indicates whether the whole graph should be visible in the resulting svg. If false, only the visible area is displayed. The default value is false. 
			
			if (!dojox.gfx.utils.toSvg) {
				R.error("UnsupportedDojoFeature", "1.4.0");
			}
			var svg, gr = this._graph;
			var oldhud = this._getHudAttr();
			// hide hud and adorners
			this._setHudAttr(false);
			this.getViewport()._showOverlays(false);

			// patch so that gfx (de)serialization does not crash on MultilineText deserialization :
			// The gfx serialize() method does not save the 'shape' property of a Group instance, but
			// will save the others 'default' gfx properties (fill, stroke, font...), so MultilineText instances
			// are serialized as Group instance :
			// { children : [....], transform:..., font: ... }.  (so, no shape:{type:'ml'}).
			// Therefore, when the json is deserialized, a Group instance is created, and setFont() is invoked
			// (which fails because Group.setFont does not exist).
			// Patch (Hack!!!!): we delete the getFont/setFont accessors so that the font is not serialized 
			if (!ibm_ilog.diagram.isSvg) {
				var getFont = ibm_ilog.diagram.util.MultilineText.prototype.getFont;
				delete ibm_ilog.diagram.util.MultilineText.prototype.getFont;
			}
			svg = dojox.gfx.utils.toSvg(ibm_ilog.diagram.util.GraphUtil.getSurface(gr));
			svg.addCallbacks(dojo.hitch(this, function(result) {
				// restore font accessors
				if (!ibm_ilog.diagram.isSvg) {
					ibm_ilog.diagram.util.MultilineText.prototype.getFont = getFont;
				}
				if (all) {
					var topg = this.getViewport().getEventSource();
					var bbox = topg.getLocalBoundingBox();
					/*
					 var tr = dojox.gfx.matrix.translate(dx, dy);
					 var t = gr.getTransform();
					 if (t) {
					 var m = new dojox.gfx.matrix.Matrix2D([tr, t]);
					 result = result.replace(/(<g.*transform="matrix\()[\s\d]*,[\s\d]*,[\s\d]*,[\s\d]*,[-\s\d]*,[-\s\d]*(\)" _ibd_prnid="__graph")/,
					 '$1'+m.xx + ',' + m.xy + ',' + m.yx + ',' + m.yy + ',' + m.dx + ',' + m.dy+'$2');
					 } else {
					 result = result.replace(/(<g _ibd_prnid="__graph")/, '$1 transform="matrix(1,0,0,1,'+(-tr.dx)+','+(-tr.dy)+')"');
					 }
					 */
					result = result.replace(/(<g)/, '$1 transform="matrix(1,0,0,1,' + (-bbox.x) + ',' + (-bbox.y) + ')"');
					// resize surface to graph content
					var e = /(<svg [\d\.:A-Za-z'"=\/]*\s*width=")\d*([\w"'\s]*height=")\d*/g; // change width/height to the graph size
					//var e = /(<svg [\d\.:A-Za-z'"=\/]*)\s*width="\d*[\w"'\s]*height="\d*[\w"']*/g;  // remove width/height
					var bb = "$1" + Math.round(bbox.width) + "$2" + Math.round(bbox.height);
					result = result.replace(e, bb);
				}
				// restore state
				this._setHudAttr(oldhud);
				this.getViewport()._showOverlays(true);
				onComplete.call(null, result);
			}), dojo.hitch(this, function(result) {
				// restore font accessors
				if (!ibm_ilog.diagram.isSvg) {
					ibm_ilog.diagram.util.MultilineText.prototype.getFont = getFont;
				}
				// restore state
				this._setHudAttr(oldhud);
				this.getViewport()._showOverlays(true);
				if (onError) {
					onError.call(null, result);
				}
			}));
		},
		
		
		//
		// Misc
		//
		
		getParent: function(){
			// summary:
			//		Returns the parent widget of this widget, assuming the parent
			//		specifies isContainer
			var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
			return parent && parent.isContainer ? parent : null;
		},
		
		getSurface: function() {
			// summary: 
			//        Returns the dojox.gfx.Surface object displaying the graph.
			// returns:
			//        A dojox.gfx.Surface instance. 
			return this._viewport.getSurface();
		},
		
		getGraph: function() {
			// summary:
			//        Returns the ibm_ilog.diagram.Graph instance that displays the graph.
			// returns:
			//        An ibm_ilog.diagram.Graph instance. 
			return this._graph;
		},
		
		getGraphTransform: function() {
			// summary:
			//        Returns the graph transform.
			// returns:
			//        A dojox.gfx.matrix.Matrix2D instance that represents the current graph transform.
			return this._graph.getTransform() || dojox.gfx.matrix.identity;
		},
		
		getGraphToDiagramTransform: function() {
			// summary:
			//		Returns the matrix used to transform coordinates of objects contained
			//		in this diagram's graph (returned by <code>getGraph()</code>) to coordinates
			//		relative to this Diagram object.
			// returns:
			//		A dojox.gfx.matrix.Matrix2D instance.
			
			return this._graph._getRealMatrix() || dojox.gfx.matrix.identity; // dojox.gfx.matrix.Matrix2D
		},
		
		getGraphToClientTransform: function() {
			// summary:
			//		Returns the matrix used to transform coordinates of objects contained
			//		in this diagram's graph (returned by <code>getGraph()</code>) to coordinates
			//		relative to the client browser's page.
			// returns:
			//		A dojox.gfx.matrix.Matrix2D instance.
			
			var t = this.getGraphToDiagramTransform();
			var rect = this.domNode.getBoundingClientRect();
			t = dojox.gfx.matrix.multiply(dojox.gfx.matrix.translate(rect.left, rect.top), t);
			return t; // dojox.gfx.matrix.Matrix2D
		},
		
		getNodeRect: function(node, inClient) {
			// summary:
			//		Returns the bounding rectangle of a node of the graph,
			//		in client (i.e., browser page) coordinates, or relative to the Diagram.
			// description:
			//		This method computes the bounding rectangle of a node contained in this Diagram widget.
			//		The returned rectangle is either relative to the whole browser page (if inClient is true or omitted),
			//		or relative to the bounds of the Diagram widget (if inClient is false).
			// node: ibm_ilog.diagram.Node
			//		A Node of the Graph displayed in this Diagram.
			// inClient: Boolean
			//		If true (the default), the returned rectangle is relative to the whole browser page.
			//		If false, the returned rectangle is relative to the bounds of the Diagram widget.
			// returns:
			//		An object with x, y, width and height properties.
			
			var bb = node.getBounds(this._graph);
			var t = (arguments.length == 1 || inClient) ? this.getGraphToClientTransform() : this.getGraphToDiagramTransform();
			bb = t.transformRectangle(bb);
			return bb; // Object
		},
		
		getGraphElementFromDataItem: function(item) {
			// summary:
			//        Gets the GFX object, for the given Data Item.
			var node = this._nodesStore ? this._getNodeFromStoreItem(item) : null;
			return node ? node : (this._linksStore ? this._getLinkFromStoreItem(item) : null);
		},
		
		asGraphElement: function(itemOrGraphElement) {
			// Summary:
			// 		return the gfx that corresponds to the given item, checking if it is not a gfx already
			if (ibm_ilog.diagram.Selectable.Util.isSelectableInstance(itemOrGraphElement)) {
				return itemOrGraphElement;
			} else {
				return this.getGraphElementFromDataItem(itemOrGraphElement);
			}
		},
		
		useDataStore: function() {
			// Summary:
			//		return if the Diagram is using node store.
			return this.nodesStore != null;
		},
		
		// =======================================================================
		//
		//  ANNOTATIONS:
		//
		
		//
		//	annotationsEnabled:
		//		Used to disable annotations.
		//
		annotationsEnabled: true,
		
		//
		//	createAnnotationAdornerFunction:
		//		Allows to specify what adorner to create for annotating the given GraphElement
		//
		createAnnotationAdornerFunction: function(ge, kwArgs) {
			//
			//	ge: the graph element
			//
			var layer = this._annotationLayer;
			
			var target = layer.createObject(ibm_ilog.diagram.adorners.AnnotationTargetAdorner);
			target.initialize(this, this._adornerTemplates, "AnnotationTargetAdorner");
			
			var annotation = layer.createObject(ibm_ilog.diagram.adorners.AnnotationAdorner);
			annotation.initialize(this, this._adornerTemplates, "AnnotationAdorner");
			
			var link = layer.createObject(ibm_ilog.diagram.Link);
			link.applyTemplate(this._adornerTemplates.item("AnnotationLink"));
			
			link.setShapeType(ibm_ilog.diagram.LinkShapeType.Orthogonal);
			
			var s = new ibm_ilog.diagram.AutomaticPort();
			var e = new ibm_ilog.diagram.AutomaticPort();
			
			s._setOwner(annotation);
			e._setOwner(target);
			
			link.setStartPort(s);
			link.setEndPort(e);
			
			target.setLink(link);
			annotation.setLink(link, target);
			
			return annotation;
		},
		
		//
		//	destroyAnnotationAdornerFunction:
		//		Counterpart to the createAnnotationAdornerFunction, this functions destroyed the previously created annotation.
		//
		destroyAnnotationAdornerFunction: function(annotation) {
			var layer = this._annotationLayer;
			layer.remove(annotation.getLink());
			this.destroyAdorner(annotation.getTarget());
			this.destroyAdorner(annotation);
		},

		//
		//	hasAnnotationFunction:
		//		Allows to specify when the adorner needs an annotation adorner created
		//
		hasAnnotationFunction: function(ge) {
			//
			//	ge: the graph element
			//
			return ge.getAnnotation() != "";
		},
		
		//
		//	loadAnnotationsFunction:
		//		Allows to create the annotations already associated with data already present in the ge.
		//
		loadAnnotationsFunction: function(ge) {
			//
			//	ge: the graph element
			//
		},
		
		_loadAnnotations: function() {
			//
			//	summary:
			//		Allows the user to create annotations based on ge/item data. This 
			//		method is executed after widget initialization is completed. 
			//
			gu.forEach(this._graph, this.loadAnnotationsFunction, this);
		},
		
		_annotations: null,
		
		_initializeAnnotations: function() {
			var v = this.getViewport();
			
			v.whenLoaded(dojo.hitch(this, function() {
				this._annotations = new dojox.collections.Dictionary();
				this._annotationLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		getAnnotationList: function(ge) {
			//
			//	returns:
			//		A dojox.collections.ArrayList with the annotations created for the graph element.
			//	note:
			//		Creates an empty list if none is registered yet.
			//
			var id = ge.getId();
			var ans;
			if (this._annotations.contains(id)) {
				ans = this._annotations.item(id);
			} else {
				this._annotations.add(id, ans = (new dojox.collections.ArrayList()));
			}
			return ans;
		},
		
		addAnnotation: function(ge, kwArgs) {
			//
			//	ge: ibm_ilog.diagram.GraphElement
			//		The entity being annotated
			//
			if (this.annotationsEnabled) {
				var a = this.createAnnotationAdornerFunction(ge, kwArgs);
				
				var ans = this.getAnnotationList(ge);
				
				ans.add(a);
				
				a.setAdorned(ge);
			}
		},
		
		removeAnnotation: function(annotation) {
			var ans = this.getAnnotationList(annotation.getAdorned());
			ans.remove(annotation);
			this.destroyAnnotationAdornerFunction(annotation);
		},
		
		// =======================================================================
		//
		//  TOOLTIPS:
		//
		
		// On mobile platforms, these functions can be used only if the application
		// explicitly requires dijit.Tooltip. On desktop, dijit.Tooltip is required
		// in _Diagram.js so the application has nothing to do.
		
		//
		//	getTooltipContentFunction:
		//		Callback function used to build the tooltip content associated with 
		//		the given graph element.  
		//
		getTooltipContentFunction: function(ge) {
			return ge.getTooltip ? ge.getTooltip() : "";
		},

		//
		//	getTooltipAroundPlacementFunction:
		//		Callback function used to compute the graph element bounding box used
		//		to compute the tooltip placement.
		//
		getTooltipAroundPlacementFunction: function(ge) {
			var r = dojo.coords(this.domNode), bb;
			bb = this._viewport.contentToClient().transformRectangle(ge.getBounds(this._graph));
			bb = g.moveRect(g.clipRect(g.standarizeRect(this._viewport.getClientSize()),bb),r);
			return bb;
		},

		showTooltip: function(ge, innerHTML, dontHideOnChange) {
			//
			//	summary:
			//		Opens the dijit master tooltip on a graph element.
			//
			//	description:
			//		Opens a tooltip for a graph element. The content is get using the 
			//		getTooltipContentFunction function. Optionally, 
			//		the content can be overriden using the 'innerHTML' argument.
			//
			//	ge: ibm_ilog.diagram.GraphElement
			//		the graph element
			//
			//	innerHTML: String?
			//		optional argument that overrides the content of the tooltip.
			//
			//	dontHideOnChange:  boolean
			//		option to disable auto-hiding of the tooltip then the element reports changes
			//

			if(!dijit.Tooltip)
				return;
			
			this.hideTooltip();
			
			innerHTML = innerHTML || this.getTooltipContentFunction(ge);
			if (innerHTML.length) {
				if (innerHTML.length) {

					var bb = this.getTooltipAroundPlacementFunction(ge);
					
					//Set text direction in tooltip
					if (dojo.config.isBidiInDojoDiagrammer) {  
					    var UCC = "";  
	                    var textdir = this.textDir;
	                    if (textdir == "auto")
	                        textdir = ibm_ilog.diagram.util.BidiUtil.checkDir(innerHTML);
	                    if (textdir == "rtl")
	                        UCC  = "\u202B"; //RLE
	                    else if (textdir == "ltr")
	                        UCC = "\u202A"; //LRE
	                    dijit.showTooltip(UCC+innerHTML, bb);      
    				} else	
						dijit.showTooltip(innerHTML, bb);
						
					this._tooltipHandle = bb;
					this._tooltipTarget = ge;

					if(!dontHideOnChange) {
						this._tooltipHideOnChange = dojo.connect(ge,"_onChanged",this,function() {
							this.hideTooltip();
							dojo.disconnect(this._tooltipHideOnChange);
						});
					}
				}
			}
		},
		
		hideTooltip: function() {
			//
			//	summary:
			//		Hides the dijit master tooltip, if currently in use by the Diagram.
			//
			if(this._tooltipHandle) {
				dijit.hideTooltip(this._tooltipHandle);
				if(this._tooltipHideOnChange) {
					dojo.disconnect(this._tooltipHideOnChange);
				}
				this._tooltipHandle = null;
				this._tooltipTarget= null;
			}
		},
		
		//
		//	_tooltipTarget: ibm_ilog.diagram.GraphElement?
		//		The current toolip target.
		//
		_tooltipOwner: null,
		
		//
		//	_tooltipHandle: Object?
		//		The tooltip handle, used for hiding.
		//
		_tooltipHandle: null,
		
		//
		//	_tooltipHideOnChange: Object?
		//		The tooltip hide on change connection handle.
		//
		_tooltipHideOnChange: null,
		
		// marks the end of definition to make life easier with trailing '},'...
		__eod: undefined
	});
	
	// Add _Diagram features to Diagram if not on mobile.
	// (done here or in _Diagram.js, depending on the load order - layer or not)
	if(dojo.config._notDiagramForMobile &&
			ibm_ilog.diagram.widget._Diagram &&
			ibm_ilog.diagram.widget.Diagram &&
			ibm_ilog.diagram.widget.Diagram.prototype &&
			!ibm_ilog.diagram.widget.Diagram.prototype._isDiagramExtended)
		dojo.extend(ibm_ilog.diagram.widget.Diagram, ibm_ilog.diagram.widget._Diagram);
})();

}

if(!dojo._hasResource["ibm_ilog.diagram.util.LimitedStack"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm_ilog.diagram.util.LimitedStack"] = true;
dojo.provide("ibm_ilog.diagram.util.LimitedStack");


dojo.declare("ibm_ilog.diagram.util.LimitedStack", [], {
	//
	// Summary:
	//		this class is similar to dojox.collections.Stack, but a maximun size can be set.
	//		When the maxium size is reached, the oldest elements are discarded.
	
		_stack: null,
		_maxSize: null,
		count: null,
		constructor: function(arr,maximumSize){
			this._stack= [];
			this.setMaxSize(maximumSize);
			if(arr){
				this._stack=this._stack.concat(arr);
			}
			this.count=this._stack.length;
		},
		clear: function(){
			//	summary
			//	Clear the internal array and reset the count
			this._stack=[];
			this.count=this._stack.length;
		},
		clone: function(){
			//	summary
			//	Create and return a clone of this Stack
			return new ibm_ilog.diagram.util.LimitedStack(this._stack);
		},
		contains: function(/* object */o){
			//	summary
			//	check to see if the stack contains object o
			for (var i=0; i<this._stack.length; i++){
				if (this._stack[i] == o){
					return true;	//	bool
				}
			}
			return false;	//	bool
		},
		copyTo: function(/* array */ arr, /* int */ i){
			//	summary
			//	copy the stack into array arr at index i
			arr.splice(i,0,this._stack);
		},
		forEach: function(/* function */ fn, /* object? */ scope){
			//	summary
			//	functional iterator, following the mozilla spec.
			dojo.forEach(this._stack, fn, scope);
		},
		getIterator: function(){
			//	summary
			//	get an iterator for this collection
			return new dojox.collections.Iterator(this._stack);	//	dojox.collections.Iterator
		},
		peek: function(){
			//	summary
			//	Return the next item without altering the stack itself.
			return this._stack[(this._stack.length-1)];	//	object
		},
		pop: function(){
			//	summary
			//	pop and return the next item on the stack
			var r=this._stack.pop();
			this.count=this._stack.length;
			return r;	//	object
		},
		push: function(/* object */ o){
			//	summary
			//	Push object o onto the stack
			//  If the maximum size is reached, the first object is removed (the older object)
			this.count=this._stack.push(o);
			if(this._maxSize && this.count>this._maxSize){
				this._stack.shift();
				this.count--;
			}
		},
		toArray: function(){
			//	summary
			//	create and return an array based on the internal collection
			return [].concat(this._stack);	//	array
		},
		setMaxSize: function(maxSize){
			// summary:
			//		sets maximum size of the stack. If null or 0, no maximum size is set and behaves as a traditional stack.
			this._maxSize = maxSize==0?null:maxSize;
			if(this._maxSize){
				while(this.count>this._maxSize){
					this._stack.shift();
					this.count--;
				}
			}
		},
		getMaxSize: function(){
			// summary:
			//		gets maximum size of the stack.
			return this._maxSize;
		}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.UndoManager']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.UndoManager'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.UndoManager');






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var gu = ibm_ilog.diagram.util.GraphUtil;

	dojo.declare("ibm_ilog.diagram.editor.undo.UndoManager", [], {
	//
	//	summary:
	//		this class is the central component of the Undo Management system.
	//	The undo management provides the user the ability to undo applied changes to the diagram, and after an undo operation, the redo of the undone
	//	action. This feature is available out of the box in the DiagramEditor, and support not only all standard changes that can be applied through 
	//  the DiagramEditor, but also any simple property change that the user could apply programmatically, with the corresponding undo object registration.
	//	More complex operation can also be supported by the Undo Management system, but the user would be responsible for the implementation of the custom undo action
	//
	_undoList:null,
	_redoList:null,
	_geList: null,
	_geReplacementList: null,
	_diagram: null,
	_maximumStackSize: null,
	
	constructor:function(diagram){
		//
		//	summary:
		//		creates a new instance of UndoManager
		//
		this._undoList = new ibm_ilog.diagram.util.LimitedStack();
		this._redoList = new ibm_ilog.diagram.util.LimitedStack();
		this._geReplacementList = new dojox.collections.Dictionary();
		this._geList = new dojox.collections.Dictionary();
		this._diagram = diagram;
		this._connectOnCreate();
	},
	
	_connectOnCreate: function(){
		dojo.connect(this._diagram,'onNodeCreated',this,this._registerGraphElement);
		dojo.connect(this._diagram,'onLinkCreated',this,this._registerGraphElement);
		dojo.connect(this._diagram,'onSubgraphCreated',this,this._registerGraphElement);
		dojo.connect(this._diagram,'beforeGraphElementDispose',this,function(ge){this._unregisterGraphElement(ge.getId());});
		
		//adds the top level graph
		// workaround to add id to topLevelGraph 
		var graph = this._diagram.getGraph();
		graph.getId = function(){return 'topLevelGraph';};
		this._registerGraphElement(graph,this._diagram);
	},
	
	_registerGraphElement: function(ge,diagram){
		//
		//	summary:
		//		register a new ge id
		var id = ge.getId();
		this._geList.add(id,ge);
	},
	
	_unregisterGraphElement: function(id){
		//
		//	summary:
		//		unregister a ge id
		this._geList.remove(id);
	},
	
	addAction:function(action){
		//
		//	summary:
		//		adds an action to the undo action stack (clearing the redo stack) 	
		this._undoList.push(action);
		this._redoList.clear();
		action.setUndoManager(this);
		this.onChange(action);
	},
	
	undoAction:function(){
		//
		//	summary:
		//		unstacks an action from the undo stack, apply the undo operation and stack it on the redo stack
		if(this._undoList.count > 0){
			var action = this._undoList.pop();
			action.undo();
			this._redoList.push(action);
			this.onChange(action);
		}
	},
	
	redoAction:function(){
		//
		//	summary:
		//		unstack an action from the redo stack, apply the redo operation and stack it on the undo stack
		if(this._redoList.count > 0){
			var action = this._redoList.pop();
			action.redo();
			this._undoList.push(action);
			this.onChange(action);
		}
		
	},
	
	getUndoStack:function(){
		//
		//	summary:
		//		returns a copy of the undo stack
		return this._undoList.clone();
	},
	
	getRedoStack:function(){
		//
		//	summary:
		//		returns a copy of the redo stack
		return this._redoList.clone();
	},

	getUndoStackCount:function(){
		//
		//	summary:
		//		returns the size of the undo stack
		return this._undoList.count;
	},
	
	getRedoStackCount:function(){
		//
		//	summary:
		//		returns the size of the redo stack
		return this._redoList.count;
	},

	onChange:function(action){
		//
		//	summary:
		//		this method is called when and undo or redo action is applied
	},
	
	getRegisteredGraphElement: function(id){
		//
		//	summary:
		//		gets the registered graph element corresponding to the given id, looking for the replacement if necessary
		if(!id){
			return null;
		}
		var replacementId = this.getRegisteredGraphElementReplacement(id);
		return this._geList.item(replacementId);
	},
	
	registerGraphElementReplacement: function(oldId,newId){
		//
		//	summary:
		//		register when a graph element id is replaced by another (when a ge is removed and by a undo/redo operation a replacement is created again, with a new Id)
		this._geReplacementList.add(oldId,newId);
	},
	
	getRegisteredGraphElementReplacement: function(id){
		//
		//	summary:
		//		gets the registered replacement for the given ge id. Look for it recursively so as to find the last replacement.
		//		if not replacement is registered the given ge id is returned.
		
		var newId = this._geReplacementList.item(id);
		if(newId && newId!=id){
			var otherId = this.getRegisteredGraphElementReplacement(newId);
			if(newId && otherId && newId!=otherId){
				newId = otherId;
				this._geReplacementList.add(id,newId);
			}
		}
		return newId?newId:id;
	},
	
	getRegisteredParent: function(id){
		//
		//	summary:
		//		return the graph parent for the given id.
		//		If the id correspond to a subgraph, the inner graph is returned.
		var parent = this.getRegisteredGraphElement(id);
		if(parent._isIBMDiagramSubgraph){
			return parent.getGraph();
		}
		return parent;
	},
	
	getParentId: function(parent){
		//
		//	summary:
		//		get the Parent id.
		//		If the given parent is a graph from a subgraph, return the subgraph id. 
		//		If the parent is the top level graph, the graph id is returned
		var sg = gu.getOwningSubgraph(parent);
		return sg?sg.getId():parent.getId();
	},
	
	setMaximumStackSize: function(stackSize){
		//
		//	summary:
		//		sets the maximum stack size
		this._maximumStackSize = stackSize;
		var notify = stackSize && (this._undoList.count>stackSize || this._undoList.count>stackSize);
		this._undoList.setMaxSize(stackSize);
		this._redoList.setMaxSize(stackSize);
		if(notify){
			this.onChange(null);
		}
	},
	
	getMaximumStackSize: function(){
		//
		//	summary:
		//		gets the maximum stack size
		return this._maximumStackSize;
	},
	
	getDiagram: function(){
		//
		//	summary:
		//		gets the used Diagram
		return this._diagram;
	},
	
	reset: function(){
		this._undoList.clear();
		this._redoList.clear();
		this._geReplacementList.clear();
		this._geList.clear();
		this.onChange(null);
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.Action']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.Action'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.Action');




(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.Action", [], {
		//
		//	summary:
		//		this is the base Undo action. This contains the base components to apply all undo/redo actions on the UndoManager.
		//		Any Undo Action should inherit from this class
		_undoFunction:null,
		_redoFunction: null,
		_label: null,
		_undoManager: null,
		_isRedoing: null,
		_isUndoing: null,
		
	constructor:function(label){
		//
		//	summary:
		//		creates a new instance, setting up the label	
		//
		this._label = label;
		this._isRedoing = false;
		this._isUndoing = false;
	},
		
	initialize: function(undoFunction,redoFunction){
		//
		//	summary:
		//		initialize the action, by setting the undo and redo functions
		this._undoFunction = undoFunction;
		this._redoFunction = redoFunction;
	},
	
	undo: function(){
		//
		//	summary:
		//		call the undo function
		this._isUndoing = true;
		if(this._undoFunction){
			this._undoFunction();
		}
		this._isUndoing = false;
	},
	
	redo: function(){
		//
		//	summary:
		//		call the redo function
		this._isRedoing = true;
		if(this._redoFunction){
			this._redoFunction();
		}
		this._isRedoing = false;
	},
	
	getLabel: function(){
		//
		//	summary:
		//		return the label. This should be used to be shown in the UI
		return this._label;
	},
	setLabel: function(label){
		//
		//	summary:
		//		sets the label that should be used to be shown in the UI
		this._label = label;
	},
	setUndoManager: function(undoManager){
		//
		//	summary:
		//		Sets the undo Manager
		this._undoManager = undoManager;
	},
	
	getUndoManager: function(){
		//
		//	summary:
		//		Gets the undo Manager. The UndoManager is set in the Action automatically when it is added to the undo stack
		return this._undoManager;
	},
	
	isUndoing: function(){
		//
		//	summary:
		//		return true if the action is executing an undo action
		return this._isUndoing;
	},
	
	isRedoing: function(){
		//
		//	summary:
		//		return true if the action is executing a redo action
		return this._isRedoing;
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.SimpleAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.SimpleAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.SimpleAction');





(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.SimpleAction", [ibm_ilog.diagram.editor.undo.Action], {
		//
		//	summary:
		//		This class is the simple undo action. It is used to undo/redo simple properties that can be 
		//		changed by setting up a property or calling a method with a simple argument.
		//
		_oldValue:null,
		_newValue: null,
		_modifiedElementId: null,
		_methodOrProperty: null,
		_cloneFunction:null,
		
	constructor:function(label){
		//
		//	summary:
		//		creates a new instance, setting up the label
		//
		this._label = label;
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setOldValue: function(oldValue){
		//
		//	summary:
		//		sets the old value
		this._oldValue = oldValue;
	},
	setNewValue: function(newValue){
		//
		//	summary:
		//		sets the new value
		this._newValue = newValue;
	},
	getOldValue: function(){
		//
		//	summary:
		//		gets the old value, cloning the set value if clone function is previously set
		return this._cloneFunction?this._cloneFunction.call(null,this._oldValue):this._oldValue;
	},
	getNewValue: function(){
		//
		//	summary:
		//		gets the new value, cloning the set value if clone function is previously set
		return this._cloneFunction?this._cloneFunction.call(null,this._newValue):this._newValue;
	},
	setModifiedElementId: function(modifiedElementId){
		//
		//	summary:
		//		sets the modified object
		this._modifiedElementId = modifiedElementId;
	},
	getModifiedElementId: function(){
		//
		//	summary:
		//		sets the modified object
		return this._modifiedElementId;
	},
	getModifiedElement: function(){
		//
		//	summary:
		//		gets the modified graph element, looking for its replacements if neccesary
		return this.getUndoManager().getRegisteredGraphElement(this._modifiedElementId);
	},
	setMethodOrProperty: function(methodOrProperty){
		//
		//	summary:
		//		sets the property or method name to be modified
		this._methodOrProperty = methodOrProperty;
	},
	setCloneFunction: function(cloneF){
		//
		//	summary:
		//		sets the clone function in case the value should be clone before the change action
		this._cloneFunction = cloneF;
	},
	_undoFunction: function(){
		this.changeFunction(this.getOldValue());
	},
	_redoFunction: function(){
		this.changeFunction(this.getNewValue());
	},
	changeFunction: function(value){	
		
		var modifiedObject = this.getModifiedElement();
		if( modifiedObject && this._methodOrProperty){
			if(modifiedObject[this._methodOrProperty]){
				if(dojo.isFunction(modifiedObject[this._methodOrProperty])){
					modifiedObject[this._methodOrProperty](value);
				}else{
					modifiedObject[this._methodOrProperty] = value;
				}
			}
		}
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.MultipleAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.MultipleAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.MultipleAction');




(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.MultipleAction", [ibm_ilog.diagram.editor.undo.Action], {
		//
		//	summary:
		//		this action groups a list of actions. It simple groups a set of actions in a single complex action.
		//
		_actionList:null,
		_reverseUndo: null,
		_reverseRedo: null,
	constructor:function(label){
		//
		//	summary:
		//		creates a  new instance, setting up the label
		//
		this._label = label;
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
		this._actionList = [];
		this._reverseUndo = true;
		this._reverseRedo = false;
	},
	addAction: function(action){
		//
		//	summary:
		//		adds an action to the list
		this._actionList.push(action);
	},
	_undoFunction: function(){
		var size = this._actionList.length;
		for(var index = 0;index<size;index++){
			var action = this._reverseUndo?this._actionList[size-index-1]:this._actionList[index];
			action.undo();
		}
	},
	_redoFunction: function(){
		var size = this._actionList.length;
		for(var index = 0;index<size;index++){
			var action = this._reverseRedo?this._actionList[size-index-1]:this._actionList[index];
			action.redo();
		}
	},
	setUndoManager: function(undoManager){
		//
		//	summary:
		//		sets the undo Manager to each individual action added to the list
		this._undoManager = undoManager;
		dojo.forEach(
				this._actionList,
			    function(item){
			        item.setUndoManager(undoManager);
			    }
			);
	},
	setReverseUndo:function(reverse){
		// summary:
		//		sets to apply the undo actions in reverse order
			this._reverseUndo = reverse;
	},
	getReverseUndo: function(){
		// summary:
		//		gets to apply the undo actions in reverse order
		return this._reverseUndo;
	},
	setReverseRedo:function(reverse){
		// summary:
		//		sets to apply the redo actions in reverse order
		this._reverseRedo = reverse;
	},
	getReverseRedo: function(){
		// summary:
		//		gets to apply the redo actions in reverse order
		return this._reverseRedo;
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.UserCustomizedAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.UserCustomizedAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.UserCustomizedAction');




(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.UserCustomizedAction", [ibm_ilog.diagram.editor.undo.Action], {
		//	
		// summary:
		//		this action is a simple action, that also called a userCustomized function on undo/redo.
		//		The functions onUndo and onRedo must be overwritten by subclasses
		//
		_redoUserFunction: null,
		_undoUserFunction: null,
		
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setUndoUserFunction: function(undoUserFunction){
		//	
		// summary:
		//		sets the undoUser function. The set function is called when undo function. The undoUserFunction must be set every time the user adds custom code on drop action.
		this._undoUserFunction = undoUserFunction;
	},
	getUndoUserFunction: function(){
		//	
		// summary:
		//		gets the undoUser function. This function is called when undo function.
		return this._undoUserFunction;
	},
	setRedoUserFunction: function(redoUserFunction){
		//	
		// summary:
		//		sets the undoUser function. The set function is called after the standard undo function is done.
		this._redoUserFunction = redoUserFunction;
	},
	getRedoUserFunction: function(){
		//	
		// summary:
		//		gets the redoUser function.
		return this._redoUserFunction;
	},
	_undoFunction: function(){
		this.undoFunction();
		if(this._undoUserFunction){
			try{
				this._undoUserFunction();
			 }catch(e){
	            	R.error("UndoUserActionError",this.getLabel(),e);
	         }
		}
	},
	_redoFunction: function(){
		this.redoFunction();
		if(this._redoUserFunction){
			try{
				this._redoUserFunction();
			}catch(e){
				R.error("UndoUserActionError",this.getLabel(),e);
			}
		}
	},
	undoFunction: function(){
		//	
		// summary:
		//		this function must be overwritten by subclasses, and is called on undo
	},
	redoFunction: function(){
		//	
		// summary:
		//		this function must be overwritten by subclasses, and is called on redo
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.UndoActionList']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.UndoActionList'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.UndoActionList');
(function(){
	ibm_ilog.diagram.editor.undo.UndoActionList.Drop = "Drop";
	ibm_ilog.diagram.editor.undo.UndoActionList.RemoveIntermediatePoint = "RemoveIntermediatePoint";
	ibm_ilog.diagram.editor.undo.UndoActionList.AddIntermediatePoint = "AddIntermediatePoint";
	ibm_ilog.diagram.editor.undo.UndoActionList.ModifyIntermediatePoint = "ModifyIntermediatePoint";
	ibm_ilog.diagram.editor.undo.UndoActionList.Connect = "Connect";
	ibm_ilog.diagram.editor.undo.UndoActionList.ReConnect = "ReConnect";
	ibm_ilog.diagram.editor.undo.UndoActionList.ReParent = "ReParent";
	ibm_ilog.diagram.editor.undo.UndoActionList.Resize = "Resize";
	ibm_ilog.diagram.editor.undo.UndoActionList.Move = "Move";
	ibm_ilog.diagram.editor.undo.UndoActionList.Delete = "Delete";
	ibm_ilog.diagram.editor.undo.UndoActionList.Paste = "Paste";
	ibm_ilog.diagram.editor.undo.UndoActionList.Align = "Align";
	ibm_ilog.diagram.editor.undo.UndoActionList.Group = "Group";
	ibm_ilog.diagram.editor.undo.UndoActionList.Ungroup = "Ungroup";
	ibm_ilog.diagram.editor.undo.UndoActionList.Reorder = "Reorder";
	ibm_ilog.diagram.editor.undo.UndoActionList.InvertLink = "InvertLink";
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.GroupAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.GroupAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.GroupAction');






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var eu = ibm_ilog.diagram.editor.EditingUtils;
	dojo.declare("ibm_ilog.diagram.editor.undo.GroupAction", [ibm_ilog.diagram.editor.undo.UserCustomizedAction], {
		//	
		// summary:
		//		this action undo/redo any Group action.
		//		This action must be modified for the user if any user defined action is done on a Group action.
		//
		
	_elements: null,
	_subgraph: null,
	_subgraphName: null,
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.Group;
	},
	setElements: function(elements){
		//	
		// summary:
		//		sets the elements list to be group
		this._elements = [];
		dojo.forEach(elements,function(ge){this._elements.push(ge.getId());},this);
	},
	getElements: function(){
		//	
		// summary:
		//		gets the elements list to be group
		return this._elements;
	},
	setSubgraph: function(subgraph){
		//	
		// summary:
		//		sets the subgraph where to group
		this._subgraph = subgraph.getId();
	},
	getSubgraph: function(){
		//	
		// summary:
		//		gets the subgraph where to group
		return this._subgraph;
	},
	redoFunction: function(){
		var elements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		var result = eu.groupElements(D,elements,this._subgraphName);
		this.getUndoManager().registerGraphElementReplacement(this._subgraph,result.subgraph.getId());
		this._subgraph = result.subgraph.getId();
	},
	undoFunction: function(){
		var newElements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		var sg = this.getUndoManager().getRegisteredGraphElement(this._subgraph);
		eu.groupReparent(D,newElements,sg.getParent(),sg);
		this._subgraphName = sg.getLabel();
		sg.getParent().remove(sg); 
		sg.dispose();
	},
	_getNewList: function(){
		var newElements = [];
		var elements = this.getElements();
		dojo.forEach(elements, dojo.hitch(this, function(item){
			var newItem = this.getUndoManager().getRegisteredGraphElement(item);
			newElements.push(newItem);
		}));
		return newElements;
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.UngroupAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.UngroupAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.UngroupAction');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var eu = ibm_ilog.diagram.editor.EditingUtils;
	var m  = dojox.gfx.matrix;
	dojo.declare("ibm_ilog.diagram.editor.undo.UngroupAction", [ibm_ilog.diagram.editor.undo.UserCustomizedAction], {
		//	
		// summary:
		//		this action undo/redo any Ungroup action.
		//		This action must be modified for the user if any user defined action is done on a Ungroup action.
		//
		
	_elements: null,
	_subgraph: null,
	_subgraphTransform: null,
	_deleteAction: null,
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.Group;
	},
	setElements: function(elements){
		//	
		// summary:
		//		sets the elements list to be group
		this._elements = [];
		dojo.forEach(elements,function(ge){this._elements.push(ge.getId());},this);
	},
	getElements: function(){
		//	
		// summary:
		//		gets the elements list to be group
		return this._elements;// summary:
		//		creates the group undo action.
		// subgraph: ibm_ilog.diagram.Subgraph
		//		the created subgraph, that groups the elements.
		// elements: array of ibm_ilog.diagram.GraphElement
		//		the elements that have been group.
	},
	setSubgraphId: function(subgraphId){
		//	
		// summary:
		//		sets the subgraph where to group
		this._subgraph = subgraphId;
	},
	getSubgraphId: function(){
		//	
		// summary:
		//		gets the subgraph where to group
		return this._subgraph;
	},
	setSubgraphTransform: function(subgraphT){
		//	
		// summary:
		//		sets the subgraph where to group
		this._subgraphTransform = subgraphT;
	},
	getSubgraphTransform: function(){
		//	
		// summary:
		//		gets the subgraph where to group
		return this._subgraphTransform;
	},
	setDeleteAction: function(deleteAction){
		//	
		// summary:
		//		sets the DeleteAction
		this._deleteAction = deleteAction;
		//this._deleteAction.getUndoManager = dojo.hitch(function(){this.getUndoManager},this);
	},
	getDeleteAction: function(){
		//	
		// summary:
		//		gets the DeleteAction
		return this._deleteAction;
	},
	setUndoManager: function(undoManager){
		//
		//	summary:
		//		Sets the undo Manager
		this.inherited(arguments);
		this._deleteAction.setUndoManager(undoManager);
	},
	redoFunction: function(){
		var elements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		var sg = this.getUndoManager().getRegisteredGraphElement(this._subgraph);
		eu.groupReparent(D,elements,sg.getParent(),sg);
		this._deleteAction.redo();
	},
	undoFunction: function(){
		var newElements = this._getNewList();
		var D = this.getUndoManager().getDiagram();
		this._deleteAction.undo();
		var sg = this.getUndoManager().getRegisteredGraphElement(this._subgraph);
		sg.setTransform(m.clone(this._subgraphTransform));
		var lca = ibm_ilog.diagram.util.GraphUtil.lowestCommonAncestor.apply(this,newElements);
		eu.groupReparent(D,newElements,sg,lca);
	},
	_getNewList: function(){
		var newElements = [];
		var elements = this.getElements();
		dojo.forEach(elements, dojo.hitch(this, function(item){
			var newItem = this.getUndoManager().getRegisteredGraphElement(item);
			newElements.push(newItem);
		}));
		return newElements;
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.ConnectAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.ConnectAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.ConnectAction');






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.ConnectAction", [ibm_ilog.diagram.editor.undo.UserCustomizedAction], {
		//	
		// summary:
		//		this action undo/redo any Connect action.
		//		This action must be modified for the user if any user defined action is done on a connect action.
		//
		_linkId: null,
		_parentId: null,
		_createPointFunction: null,
		_linkShapeType: null,
	constructor:function(diagram,link,parent){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.Connect;
		this._linkId = link.getId();
		this._parentId = diagram.getUndoManager().getParentId(parent);
		this._linkShapeType = link.getShapeType();
		var start = link.getStartPort();
		this._start = {
			auto: !start._isIBMDiagramBasicPort,
			position: start._isIBMDiagramBasicPort?start.getPosition():null,
			nodeId: start.getOwner().getId()	
		};
		
		var end = link.getEndPort();
		if(end){
			this._end = {
				auto: !end._isIBMDiagramBasicPort,
				position: end._isIBMDiagramBasicPort?end.getPosition():null,
				nodeId: end.getOwner().getId()	
			};
		}else{
			this._end = {
					point: dojo.clone(link.getFallbackEndPoint())
				};
		}
	},
	_clearPorts: function(){
		var port;
		var link = this.getUndoManager().getRegisteredGraphElement(this._linkId);
		port = link.getStartPort();
		link.setStartPort(null);
		//remove Port
		if(port && port._isIBMDiagramBasicPort){
			port.getOwner().removePort(port);
		}
		port = link.getEndPort();
		link.setEndPort(null);
		//remove Port
		if(port && port._isIBMDiagramBasicPort){
			port.getOwner().removePort(port);
		}
	},
	_createPort: function(automaticPortType,position){
		var port;
		if(automaticPortType || !position){
			port = new ibm_ilog.diagram.AutomaticPort();
		}else{
			port = new ibm_ilog.diagram.BasicPort();
			port.setPosition(position);
		}
		return port;
	},
	getLink: function(){
		//	
		// summary:
		//		returns the link, looking for replaces if necessary
		return this.getUndoManager().getRegisteredGraphElement(this._linkId);
	},
	undoFunction: function(){
		var parent = this.getUndoManager().getRegisteredParent(this._parentId);
		var link = this.getLink();
		parent.remove(link);
		this._clearPorts();
		link.dispose();
	},
	redoFunction: function(){
		var parent = this.getUndoManager().getRegisteredParent(this._parentId);
		var link = parent.createLink();
		this.getUndoManager().registerGraphElementReplacement(this._linkId,link.getId());
		link.setShapeType(this._linkShapeType);
		
		var startNode = this.getUndoManager().getRegisteredGraphElement(this._start.nodeId);
		var endNode = this.getUndoManager().getRegisteredGraphElement(this._end.nodeId);
		
		var startP = this._createPort(this._start.auto,this._start.position);
		startNode.addPort(startP);
		link.setStartPort(startP);
		
		if(endNode){
			var endP = this._createPort(this._end.auto,this._end.position);
			endNode.addPort(endP);
			link.setEndPort(endP);
		}else{
			link.setFallbackEndPoint(dojo.clone(this._end.point));
		}
		var D = this.getUndoManager().getDiagram();
		D._onLinkCreated(link,startNode,endNode,D);
		D.onLinkUpdated(D,link,true,this);
		this._linkId = link.getId();
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.DeleteAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.DeleteAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.DeleteAction');





(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
dojo.declare("ibm_ilog.diagram.editor.undo.DeleteAction", [ibm_ilog.diagram.editor.undo.Action], {
	_serialized: null,
	_deletedNodes: null,
	_diagram: null,
	_deletedLinksAction: null,
	
	constructor:function(diagram){
		this.setLabel(ibm_ilog.diagram.editor.undo.UndoActionList.Delete);
		this._diagram = diagram;
		this._deletedLinksAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Delete);
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setDeleted: function(targetSet){
		this._deletedNodes = {};
		var ts = [];
		for(var index in targetSet){
			var ge = targetSet[index];
			var parent = ge.getParent();
			if(ge._isIBMDiagramNode){
				this._deletedNodes[ge.getId()] = this._diagram.getUndoManager().getParentId(parent);
				ts.push(ge);
			}else{
				this._deletedLinksAction.addAction(new ibm_ilog.diagram.editor.undo.DeleteLinkAction(this._diagram,ge,parent));
			}
		}
		this._serialized = this._diagram._getRemoveSerializer().serialize(ts);
		this._deletedLinksAction.setUndoManager(this._diagram.getUndoManager());
	},
	_undoFunction: function(){
		var undoGroup = new ibm_ilog.diagram.editor.undo.MultipleAction("Dummy");
		dojo.forEach(this._serialized,function(item){
			var parent =  this.getUndoManager().getRegisteredParent(this._deletedNodes[item.id]); 
			this._diagram._getRemoveSerializer().deserialize([item],{graph:parent,undo:undoGroup});
		},this);
		this._deletedLinksAction.undo();
	},
	_redoFunction: function(){
		this._deletedLinksAction.redo();
		for(var elementId in this._deletedNodes){
			var ge = this.getUndoManager().getRegisteredGraphElement(elementId);
			ge.getParent().remove(ge);
			ge.dispose();
		}
	},
	_initializeSerializer: function(){
		this._serializer = new ibm_ilog.diagram.editor.DiagramSerializer(this._diagram,null,this);
	},
	onSerialize: function(graphElement,serializedData){
		
	},
	onDeserialize: function(serializedData,newGraphElement,container,undoAction){
		var um = this._diagram.getUndoManager();
		um.registerGraphElementReplacement(serializedData.id,newGraphElement.getId());
	}
});
	
dojo.declare("ibm_ilog.diagram.editor.undo.DeleteLinkAction", [ibm_ilog.diagram.editor.undo.ConnectAction], {
	undoFunction: function(){
		ibm_ilog.diagram.editor.undo.ConnectAction.prototype.redoFunction.apply(this, arguments);
	},
	redoFunction: function(){
		ibm_ilog.diagram.editor.undo.ConnectAction.prototype.undoFunction.apply(this, arguments);
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.ReorderAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.ReorderAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.ReorderAction');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.ReorderAction", [ibm_ilog.diagram.editor.undo.Action], {
		//
		//	summary:
		//		this action undo / redo a reordering action.
		_newElementList: null,
		_oldElementList: null,
	
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.Reorder;
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setNewElementList: function(elementList){
		//	
		// summary:
		//		sets the new element list
		this._newElementList = [];
		dojo.forEach(elementList,function(ge){this._newElementList.push(ge.getId());},this);
	},
	setOldElementList: function(elementList){
		//	
		// summary:
		//		sets the old element list
		this._oldElementList = [];
		dojo.forEach(elementList,function(ge){this._oldElementList.push(ge.getId());},this);
	},
	_undoFunction: function(){
		var elements = this._getList(this._oldElementList);
		ibm_ilog.diagram.editor.EditingUtils.setChildrenOrder(elements);
	},
	_redoFunction: function(){
		var elements = this._getList(this._newElementList);
		ibm_ilog.diagram.editor.EditingUtils.setChildrenOrder(elements);
	},
	_getList: function(elements){
		var newElements = [];
		dojo.forEach(elements, dojo.hitch(this, function(item){
			var newItem = this.getUndoManager().getRegisteredGraphElement(item);
			newElements.push(newItem);
		}));
		return newElements;
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.ChangeLinkIntermediatePointsAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.ChangeLinkIntermediatePointsAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.ChangeLinkIntermediatePointsAction');






(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.ChangeLinkIntermediatePointsAction", [ibm_ilog.diagram.editor.undo.Action], {
		//	
		// summary:
		//		this action undo/redo any Connect action.
		//		This action must be modified for the user if any user defined action is done on a connect action.
		//
		_linkId: null,
		_newPoint: null,
		_oldPoint:null,
		_index:null,
		_operation: null,
	constructor:function(diagram,link){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._linkId = link.getId();
		this.initialize(dojo.hitch(this,this._undoFunction),dojo.hitch(this,this._redoFunction));
	},
	setRemovePointOperation: function(index,oldPoint){
		this._operation = 0;
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.RemoveIntermediatePoint;
		this._index = index;
		this._oldPoint = dojo.clone(oldPoint);
	},
	setAddPointOperation: function(index,point){
		this._operation = 1;
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.AddIntermediatePoint;
		this._index = index;
		this._newPoint = dojo.clone(point);
	},
	setModifyPointOperation: function(index,newPoint,oldPoint){
		this._operation = 2;
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.ModifyIntermediatePoint;
		this._index = index;
		this._oldPoint = dojo.clone(oldPoint);
		this._newPoint = dojo.clone(newPoint);
	},
	getLink: function(){
		//	
		// summary:
		//		returns the link, looking for replaces if necessary
		return this.getUndoManager().getRegisteredGraphElement(this._linkId);
	},
	_undoFunction: function(){
		var link = this.getLink();
		switch(this._operation){
			case 0: // Remove
				this._addPoint(link,this._index,this._oldPoint);
				break;
			case 1: // Add
				this._removePoint(link,this._index);
				break;
			case 2: // Modify
				this._modifyPoint(link,this._index,this._oldPoint);
				break;
			}
	},
	_redoFunction: function(){
		var link = this.getLink();
		switch(this._operation){
			case 0: // Remove
				this._removePoint(link,this._index);
				break;
			case 1: // Add
				this._addPoint(link,this._index,this._newPoint);
				break;
			case 2: // Modify
				this._modifyPoint(link,this._index,this._newPoint);
				break;
			}
	},
	_removePoint: function(link,index){
		var intermediatePoints = link.getIntermediatePoints();
		intermediatePoints = intermediatePoints.slice(0,index).concat(intermediatePoints.slice(index+1))
		link.setIntermediatePoints(intermediatePoints);
	},
	_addPoint: function(link,index,point){
		var intermediatePoints = link.getIntermediatePoints();
		intermediatePoints = intermediatePoints.slice(0,index).concat([point]).concat(intermediatePoints.slice(index));
		link.setIntermediatePoints(intermediatePoints);
	},
	_modifyPoint: function(link,index,point){
		var intermediatePoints = link.getIntermediatePoints();
		intermediatePoints[index].x = point.x;
		intermediatePoints[index].y = point.y;
		link.setIntermediatePoints(intermediatePoints);
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.undo.InvertLinkAction']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.undo.InvertLinkAction'] = true;
dojo.provide('ibm_ilog.diagram.editor.undo.InvertLinkAction');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	
	dojo.declare("ibm_ilog.diagram.editor.undo.InvertLinkAction", [ibm_ilog.diagram.editor.undo.Action], {
		//
		//	summary:
		//		this action undo / redo a reparenting action.
		_modifiedElementId: null,
	
	constructor:function(){
		//	
		// summary:
		//		creates a new instance, setting the corresponding label
		this._label = ibm_ilog.diagram.editor.undo.UndoActionList.InvertLink;
		this.initialize(dojo.hitch(this,this._changeFunction),dojo.hitch(this,this._changeFunction));
	},
	setModifiedElementId: function(modifiedElementId){
		//	
		// summary:
		//		sets the modified element Id
		this._modifiedElementId= modifiedElementId;
	},
	_changeFunction: function(){
		var modifiedElement = this.getUndoManager().getRegisteredGraphElement(this._modifiedElementId);
		ibm_ilog.diagram.editor.EditingUtils.invertLink(modifiedElement);
	}
	});
	
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.editor.EditingUtils']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.editor.EditingUtils'] = true;
dojo.provide('ibm_ilog.diagram.editor.EditingUtils');

if (!dojo.config.useGfxLayout) {
	
	
	
	
}














(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;
	var g = ibm_ilog.diagram.util.Geometry;
	
ibm_ilog.diagram.editor.EditionUtils = {};

var eu = ibm_ilog.diagram.editor.EditingUtils;

//////////////////////////////////////////////////////////////////////////////
//		Deletion utilities
eu.deleteGraphElements = function deleteGraphElements(/*ibm_ilog.diagram.widget.Diagram*/diagram,/*ibm_ilog.diagram.GraphElement[]*/elements, /*Function?*/doDeleteFunction) {
	// summary:
	//		deletes the elements from the diagram
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to delete
	// elements: ibm_ilog.diagram.GraphElement[]
	//		the elements set to be deleted
	// doDeleteFunction: Function
	//		an optional Function to perform the deletion.
	// returns:
	//		The Action corresponding to the delete operation.
	
	doDeleteFunction = dojo.isFunction(doDeleteFunction) ? doDeleteFunction : eu.doDeleteAction;
	var targetSet = eu.prepareDeletedSet(diagram, elements);
	return doDeleteFunction(diagram, targetSet); // ibm_ilog.diagram.editor.undo.Action
};

eu.prepareDeletedSet = function(/*ibm_ilog.diagram.widget.Diagram*/diagram, /*ibm_ilog.diagram.GraphElement[]*/elements){
	// summary:
	//		Computes the set of GraphElement to delete based on the specified elements list.
	// description:
	//		This method processes the specified elements array to ensures children (if any) and connected links are deleted too.  
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram instance to remove the shapes from.
	// elements: ibm_ilog.diagram.GraphElement[]
	//		the elements set to be deleted
	// returns:
	//		the list of GraphElement to delete.
	
	var allElements = {};
	eu.findElementsToDelete(diagram,elements,allElements);
	
	var list = [];
	
	for(var ids in allElements){
		list.push(allElements[ids]);
	}
	eu._changeFocused(diagram,allElements);

	var targetSet = gu.maximals(list);
	return targetSet; // ibm_ilog.diagram.GraphElement[]
};

eu.doDeleteAction = function(/*ibm_ilog.diagram.widget.Diagram*/diagram, /*ibm_ilog.diagram.GraphElement[]*/shapes){
	// summary:
	//		Deletes the specified shapes from the given diagram and returns a corresponding undo Action.
	// description:
	//		This implementation removes the specified shapes from the diagram graph and dispose them.
	//		Note that it does not handle any data store-related tasks.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram instance to remove the shapes from.
	// shapes: ibm_ilog.diagram.GraphElement[]
	//		An array of ibm_ilog.diagram.GraphElement to remove.
	// returns:
	//		The action to undo the delete operation.
	
	var action = eu.createDeleteUndoAction(diagram,shapes);
	
	dojo.forEach(shapes,function(graphElement){
		graphElement.getParent().remove(graphElement);
		graphElement.dispose();
	});
	
	return action; // ibm_ilog.diagram.editor.undo.DeleteAction
};

eu._changeFocused = function _changeFocused(diagram,allElements){
	// summary:
	//		Change the focused element to the first available top parent available (if the focused element is been removed).
	var current = diagram.getFocusedElement();
	if(current && (current.getId() in allElements)){
		var children = diagram.getGraph().children;
		for(var i = 0 ; i<children.length ; i++){
			var child = children[i];
			if(child._isIBMDiagramNode && !(child.getId() in allElements)){
				diagram.setFocusedElement(child);
				return;
			}
		}
	}
};

eu.findElementsToDelete = function findElementsToDelete(diagram,elements,result){
	// summary:
	//		iterate over the elements and add to the result all the children and the connected links.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to delete
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements set to be deleted
	// result: array of ibm_ilog.diagram.GraphElement
	//		the resulting array.
	for(key in elements){
		var e = elements[key];
		var ge = diagram.asGraphElement(e);
		if(ge) {
			result[ge.getId()] = ge;
			if(ge._isIBMDiagramNode){
				dojo.forEach(ge.getLinks(),function(link){
					result[link.getId()] = link;
				});
				if(ge._isIBMDiagramSubgraph){
					eu.findElementsToDelete(diagram,ge.getGraph().children,result);
				}
			}
		}
	}
};

//////////////////////////////////////////////////////////////////////////////
// Group/Ungroup utilities

eu.groupElements = function groupElements(diagram,elements,subgraphName) {
	// summary:
	//		Creates a new subgraph, and moves all the elements inside it. 
	//		This method return a JSObject with the created subgraph and the Undo Action: { subgraph:sg , action:action }
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to group
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements set to be group
	// subgraphName: String
	//		the new subgraph label
	// return: { subgraph: ibm_ilog.diagram.Subgraph , action: ibm_ilog.diagram.editor.undo.GroupAction }
	var lca = gu.lowestCommonAncestor.apply(this,elements);
	
	var sg = diagram.createSubgraph(null,null,lca);
	
	sg.setLabel(subgraphName);
	
	eu.groupReparent(diagram,elements,sg,lca);

	var action = eu.createGroupUndoAction(sg,elements);
	
	return { subgraph:sg, action:action };
};

eu.ungroupElements = function ungroupElements(diagram,subgraph) {
	// summary:
	//		Deletes the subgraph and moves all its children to its parent.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to ungroup
	// subgraph: ibm_ilog.diagram.Subgraph
	//		the subgraph to be ungrouped
	
	var elements = [].concat(subgraph.getGraph().children);
	
	var sgTransform = m.clone(subgraph.getTransform());
	
	eu.groupReparent(diagram,elements,subgraph.getParent(),subgraph);
	
	var subgraphId = subgraph.getId();
	
	var deleteAction = eu.deleteGraphElements(diagram,[subgraph]);
	
	var action = eu.createUngroupUndoAction(diagram,subgraphId,elements,deleteAction,sgTransform);
	
	return { elements:elements, action:action };
};

eu.groupReparent = function groupReparent(diagram,elements,newParent,oldParent) {
	// summary:
	//		Change the parent of all the elements given by parameter.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to reparent
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be reparented.
	// newParent: ibm_ilog.diagram.Subgraph or ibm_ilog.diagram.Graph
	//		the new parent
	// oldParent: ibm_ilog.diagram.Subgraph or ibm_ilog.diagram.Graph
	//		the old parent
	var graph = newParent._isIBMDiagramSubgraph?newParent.getGraph():newParent;
	ibm_ilog.diagram.util.Batch.endBatch(); // make sure everything is up to date...
	dojo.forEach(elements,function(ge) {
		graph.add(ge);
	});
	dojo.forEach(elements,function(ge) {
		var op = oldParent;
		if(!oldParent){
			op = ge.getParent();
			if(op && op.getParent()){
				op = op.getParent();
			}
		}
		eu.reparentTransform(diagram,ge,newParent,op);
	});	
};

eu.reparentTransform = function reparentTransform(diagram,element,newParent,oldParent) {
	// summary:
	//		Apply the transformation to reparent an element without noticeable transformation change.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram from where to reparent
	// element: ibm_ilog.diagram.GraphElement
	//		the element to be reparented.
	// newParent: ibm_ilog.diagram.Subgraph or ibm_ilog.diagram.Graph
	//		the new parent
	// oldParent: ibm_ilog.diagram.Subgraph or ibm_ilog.diagram.Graph
	//		the old parent (can be null)
	if(oldParent){
		var oldT = oldParent.getShapeToContainerTransform(diagram.getGraph().getParent());
		element.applyLeftTransform(oldT);
	}
	newParent = newParent || diagram.getGraph();
	var newT = newParent.getShapeToContainerTransform(diagram.getGraph().getParent()).inverse();
	element.applyLeftTransform(newT);

};

//////////////////////////////////////////////////////////////////////////////
// Alignment functions

eu.alignLeft = function alignLeft(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement left side.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:b.x,y:0}; }); 
};
eu.alignRight = function alignRight(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement right side.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:b.x+b.width,y:0}; }); 
};
eu.alignHCenter = function alignHCenter(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement horizontal center.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:b.x+b.width/2,y:0}; }); 
};
eu.alignTop = function alignTop(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement top side.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:0,y:b.y}; }); 
};
eu.alignBottom = function alignBottom(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement bottom side.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:0,y:b.y+b.height}; }); 
};
eu.alignVCenter = function alignVCenter(diagram,elements,pivotElement) {
	// summary:
	//		Align the elements based on the pivotElement vertical center.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//	the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//	the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//	the element taken as pivot for alignment.
	return eu.alignWithPivot(diagram,elements,pivotElement,function(b){ return {x:0,y:b.y+b.height/2}; }); 
};

eu.alignWithPivot = function alignWithPivot(diagram,elements,pivotElement,getBBPivot) {
	// summary:
	//		Align the elements based on the pivotElement and the getBBPivot function.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are aligned.
	// elements: array of ibm_ilog.diagram.Node
	//		the target set to be aligned.
	// pivotElement: ibm_ilog.diagram.Node
	//		the element taken as pivot for alignment.
	// getBBPivot: Function
	//		this function determines the alignment position.	
	
	var V = diagram.getViewport();
	var graph = diagram.getGraph();
	var refbb = pivotElement.getBounds(graph);
	var refPivot = getBBPivot(refbb);
	
	var mAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Align);
	
	for(var i = 1; i<elements.length; i++) {
		var ge = elements[i];
		var oldTransform = dojox.gfx.matrix.clone(ge.getTransform());
		var gebb = ge.getBounds(graph);
		var delta = g.subPoint(refPivot,getBBPivot(gebb));
		ge.setTransform(m.multiply(ge.getTransform(),m.translate(delta)));
		mAction.addAction(eu.createMoveUndoAction(ge,oldTransform,ge.getTransform()));
	}
	return mAction;
};

//////////////////////////////////////////////////////////////////////////////
// Reorder z-order functions

eu.reorderedParents = function reorderedParents(graphElements) {
	// summary:
	//		given a list of GraphElements, return and array list with the parents of the elements, 
	//		and a dictionary of the elements, with the Id as keys.
	// graphElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be processed.
	// return: {parents:dojox.collections.ArrayList,ids:{}}
	var ids = {}; 
	var parents = new dojox.collections.ArrayList();
	var union = dojox.collections.Set.union;
	
	dojo.forEach(graphElements,function(e){
		ids[e.getId()]=e;
		parents = union(parents,[e.getParent()]);
	});
	
	return {parents:parents,ids:ids};
};

eu.reorderSelection = function reorderSelection(selectedElements,childrenReorderer) {
	// summary:
	//		get the selection, and iterates over the selection parents,
	//		calling the childrenReorderer function passed as parameter, 
	//		with each parent element and the list of selected elements ids.
	// selectedElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be arranged.
	// childrenReorderer: Function
	//		the function to be called with each selection parent.
	var o = eu.reorderedParents(selectedElements);
	var actions = []; 
	for(var i = 0; i<o.parents.count ; i++){
		var parent = o.parents.item(i);
		var action = new ibm_ilog.diagram.editor.undo.ReorderAction
		action.setOldElementList(parent.children);
		childrenReorderer(parent,o.ids);
		action.setNewElementList(parent.children);
		actions.push(action);
	}
	if(actions.length==1){
		return actions[0];
	}else{
		var action = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Reorder);
		dojo.forEach(actions,function(a){action.addAction(a);});
		return action;
	}
};

eu.setChildrenOrder = function setChildrenOrder(order) {
	// summary:
	//		order the elements in the order parameter, as they are passed.
	//		This cause a reverse z-order (the last element is the Front one, and the other way about).
	// order: array of ibm_ilog.diagram.GraphElement
	//		the elements to be order
	dojo.forEach(order,function(e){
		e.moveToFront();
	});
};

eu.classifyList = function classifyList(childList,in_b,scope) {
	// summary:
	//		evaluates each element in the with the in_b function, 
	//		and add it to the a list if true, and to the b one if false.
	// childList: array of ibm_ilog.diagram.GraphElement
	//		the elements to be order
	// in_b: Function
	//		the function to evaluate.
	// scope: Scope
	//		the scope where to evaluate the in_b function.
	var a = [], b = [];
	dojo.forEach(childList,function(e){
		if(in_b(e)) {
			b.push(e);
		} else {
			a.push(e);
		}
	},scope);
	return {a:a,b:b};
};

eu.sendToBack = function sendToBack(selectedElements) {
	// summary:
	//		Send the selected elements to back .
	// selectedElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be arranged
	return eu.reorderSelection(selectedElements,function(parent,ids){
		var p = eu.classifyList(parent.children,function(e) {
			return e.getId() in ids;
		});
		eu.setChildrenOrder(p.b.concat(p.a));
	});
};

eu.bringToFront = function bringToFront(selectedElements) {
	// summary:
	//		Send the selected elements to the front.
	// selectedElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be arranged
	return eu.reorderSelection(selectedElements,function(parent,ids){
		var p = eu.classifyList(parent.children,function(e) {
			return e.getId() in ids;
		});
		eu.setChildrenOrder(p.a.concat(p.b));
	});
};

eu.sendBackward = function sendBackward(selectedElements) {
	// summary:
	//		Send the selected elements one position back.
	// selectedElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be arranged.
	return  eu.reorderSelection(selectedElements,function(parent,ids){
		var cs = parent.children;
		var yielded = null;
		var newOrder = [];
		var restore = function() {
			if(yielded) {
				newOrder.push(yielded);
				yielded = null;
			}
		};
		for( var i = 0; i< cs.length; i++) {
			var e = cs[i];
			if(!(e.getId() in ids)) {
				restore();
				yielded = e;
			} else {
				newOrder.push(e);
				restore();
			}
		};
		restore();
		eu.setChildrenOrder(newOrder);
	});
};

eu.bringForward =  function bringForward(selectedElements) {
	// summary:
	//		Send the selected elements one position front.
	// selectedElements: array of ibm_ilog.diagram.GraphElement
	//		the elements to be arranged.
	return eu.reorderSelection(selectedElements,function(parent,ids){
		var cs = parent.children;
		var pushed = [];
		var newOrder = [];
		for( var i = 0; i< cs.length; i++) {
			var e = cs[i];
			if(e.getId() in ids) {
				pushed.push(e);
			} else {
				newOrder.push(e);
				newOrder = newOrder.concat(pushed);
				pushed = [];
			}
		};
		newOrder = newOrder.concat(pushed);
		eu.setChildrenOrder(newOrder);
	});
};

//////////////////////////////////////////////////////////////////////////////
//   DROP FUNCTIONS

eu.startDropAction = function startDropAction(droppedItem,p,D,elements,dropAction){
	// summary:
	//		starts a drop action. 
	//		This method cam be called externally, from the DropInteractor or from an DropAction (redo).
	// droppedItem: ibm_ilog.diagram.GraphElement
	//		the item that has been dropped.
	// p: Point
	//		the dropping point.
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements that have been detected behind the point
	// dropAction: ibm_ilog.diagram.editor.undo.UngroupAction
	//		the droped action involved in the operation
	var di = D.getDropInteractor();
	di.setDroppedItem(droppedItem);
	if (di._onDropFunction) {
        try{
        	di._onDropFunction.call(null,droppedItem,p,D,elements,dropAction);
        }catch(e){
        	R.error("DropUserFunctionError",e);
        }
	}
	if(!dropAction.isRedoing()){
		D.getUndoManager().addAction(dropAction);
	}
};

//////////////////////////////////////////////////////////////////////////////
//		Intermediate point operations

eu.addItermediatePoint = function AddItermediatePoint(D,link,point,index){
	// summary:
	//		adds a intermediate point to the given link, in the specified index.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are deleted.
	// link: ibm_ilog.diagram.Link
	//		the link to be modified
	// point: point
	//		the point to be added
	// index: integer
	//		the position where to be added (between the intermediate points).
	
	var intermediatePoints = link.getIntermediatePoints();
	intermediatePoints = intermediatePoints.slice(0,index).concat([point]).concat(intermediatePoints.slice(index));
	link.setIntermediatePoints(intermediatePoints);
	D.getUndoManager().addAction(eu.createAddItermediatePointUndoAction(D,link,point,index)); 
};
////////////////////////////////////////////////////////////////////////////////
// INVERT LINKS

eu.invertLink = function invertLink(link){
	var startPort = link.getStartPort();
	var endPort = link.getEndPort();
	link.setStartPort(endPort);
	link.setEndPort(startPort);
	return this.createInvertLinkUndoAction(link);
};


//////////////////////////////////////////////////////////////////////////////
//		CREATION OF UNDO ACTIONS

eu.createInvertLinkUndoAction = function createInvertLinkUndoAction(link){
	var action = new ibm_ilog.diagram.editor.undo.InvertLinkAction();
	action.setModifiedElementId(link.getId());
	return action;
};

eu.createDeleteUndoAction = function createDeleteUndoAction(diagram,targetSet){
	// summary:
	//		creates the move undo action, based on the target.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are deleted.
	// targetSet: array of ibm_ilog.diagram.GraphElement
	//		the target set to be deleted
	var action = new ibm_ilog.diagram.editor.undo.DeleteAction(diagram);
	action.setDeleted(targetSet);
	return action;
};

eu.createGroupUndoAction = function createGroupUndoAction(subgraph,elements){
	// summary:
	//		creates the group undo action.
	// subgraph: ibm_ilog.diagram.Subgraph
	//		the created subgraph, that groups the elements.
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements that have been group.
	var action = new ibm_ilog.diagram.editor.undo.GroupAction();
	action.setElements(elements);
	action.setSubgraph(subgraph);
	return action;
};

eu.createUngroupUndoAction = function createUngroupUndoAction(diagram,subgraphId,elements,deleteAction,sgTransform){
	// summary: ibm_ilog.diagram.editor.undo.UngroupAction
	//		creates the ungroup undo action.
	// diagram: ibm_ilog.diagram.widget.Diagram
	//		the diagram where the elements are ungroup.
	// subgraph: String
	//		the deleted subgraph Id.
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements that have been ungroup.
	// deleteAction: ibm_ilog.diagram.editor.undo.DeleteAction
	//		the created deleteAction, obtained when deleting the subgraph.
	var action = new ibm_ilog.diagram.editor.undo.UngroupAction();
	action.setElements(elements);
	action.setSubgraphId(subgraphId);
	action.setSubgraphTransform(sgTransform);
	action.setDeleteAction(deleteAction);
	return action;
};

eu.createMoveUndoAction = function createMoveUndoAction(target,oldTransformation, newTransformation){
	// summary:ibm_ilog.diagram.editor.undo.UngroupAction
	//		creates the move undo action, based on the target.
	// target: ibm_ilog.diagram.GraphElement
	//		the element that has been moved.
	// oldTransformation: dojox.gfx.matrix
	//		the old transformation matrix.
	// newTransformation: dojox.gfx.matrix
	//		the new transformation matrix.
	var cloneF = function(object){return m.clone(object);};
	
	var action = new ibm_ilog.diagram.editor.undo.SimpleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Move);
	action.setOldValue(m.clone(oldTransformation));
	action.setNewValue(m.clone(newTransformation));
	action.setMethodOrProperty('setTransform');
	action.setModifiedElementId(target.getId());
	action.setCloneFunction(cloneF);
	return action;
};

eu.createOnDropAction = function createOnDropAction(D,droppedItem,p,elements){
	// summary:ibm_ilog.diagram.editor.undo.DropAction
	//		creates the drop undo action.
	// droppedItem: ibm_ilog.diagram.GraphElement
	//		the item that has been dropped.
	// p: Point
	//		the dropping point.
	// elements: array of ibm_ilog.diagram.GraphElement
	//		the elements that have been detected behind the point
	var newElements = [];
	dojo.forEach(elements, function(item){
		newElements.push(item.getId());
	});
	var action = new ibm_ilog.diagram.editor.undo.DropAction();
	action.setDroppedItem(droppedItem);
	action.setDroppedPosition(p);
	action.setElementsList(newElements);
	action.setDropInteractor(D.getDropInteractor());
	return action;
};

eu.createAddItermediatePointUndoAction = function createAddItermediatePointUndoAction(diagram,link,point,index){
	// summary:
	//		creates the undo action
	var action = new ibm_ilog.diagram.editor.undo.ChangeLinkIntermediatePointsAction(diagram, link);
	action.setAddPointOperation(index,point);
	return action;
};

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.MoveInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.MoveInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.MoveInteractor');














(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;

/*=====
ibm_ilog.diagram.__MoveInteractorArgs = function() {
	//	doMove: Function?
	//		Concrete the movement to a graph element.
	//	isMovable: Function?
	//		Return true if the given graph element is allowed to move
	this.doMove = function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The amount to translate the entity from its original location.
	}
	this.isMovable = function(graphElement) {
		//	summary:
		//		Return true if the given graph element is allowed to move
		//	graphElement:
		//		The graph element to test.
	}
};	
=====*/

	var superclass = ibm_ilog.diagram.interactors.DragInteractor.prototype;

dojo.declare("ibm_ilog.diagram.interactors.MoveInteractor", [ibm_ilog.diagram.interactors.DragInteractor], {
	//
	//	summary:
	//		A MoveInteractor allows the user to move the currently selected entities.
	//
	
	//
	//	_selection: ibm_ilog.diagram.Selection
	//		The selection, object, whose entities are to be moved by the operation
	//
	_selection: null,

	//
	//	_viewport: ibm_ilog.diagram.widget.Diagram
	//		The viewport witht the content to move
	//
	_viewport: null,

	//
	//	_content: dojox.gfx.Group
	//		The root of the elements to make moveable, used as the source of events.
	//
	_content: null,

	//
	//	_targetSet: array
	//		The elements that are going to be moved.
	//
	_targetSet: null,
	
	//
	//	_dragCursor: string
	//		id of the cursor used while dragging
	//
	_dragCursor: "move",
		
	initialize: function( /*ibm_ilog.diagram.Selection*/ selection, /*ibm_ilog.diagram.Viewport|ibm_ilog.diagram.Graph*/ viewportOrContent, /*ibm_ilog.diagram.__MoveInteractorArgs*/ callbacks ) {
		//
		//	summary:
		//		Constructs the MoveInteractor
		//

		if(viewportOrContent instanceof ibm_ilog.diagram.widget.Diagram) {
			this._viewport = viewportOrContent.getViewport();
			this._content = this._viewport.getContent();
		} else if(viewportOrContent instanceof ibm_ilog.diagram.widget.Viewport) {
			this._viewport = viewportOrContent;
			this._content = this._viewport.getContent();
		} else if(viewportOrContent instanceof ibm_ilog.diagram.Graph) {
			this._content = gu.getSurface(viewportOrContent);
			this._cursorStyle = this._content.getEventSource().style; 
		} else {
			R.error("UnexpectedArgument");
		}

		if(callbacks) {
			if(callbacks.doMove) {
				this._doMove = callbacks.doMove;
			}
			
			if(callbacks.isMovable) {
				this._isMovable = callbacks.isMovable;
			}
		}
		this._selection = selection;
		this._initialize();
		return this;
	},

	_getInitialEventSource: function() {
		return this._content; 
	},
	
	activate: function() {
		//
		//	summary:
		//		Activates the receiver by connecting it to its sources of events.
		//
		this._fconnect("selection",this._selection,"onSelectionChanged",this,"_onSelectionChanged");
		this.inherited(arguments);
		if(this._enabled) {
			this._buildTargetSet();
		}
	},
	
	onEnabled: function() {
		this._buildTargetSet();
	},
	
	_onSelectionChanged: function() {
		this._buildTargetSet();
	},

	_isMovable: function(ge) {
		return true;
	},
	
	_buildTargetSet: function() {
		//
		//	summary:
		//		Builds the set of entities that are going to be affected by the drag.
		//
		//	description:
		//		Grab Nodes and Subgraphs from the _selection object. Their original 
		//    transforms are saved to be used in the drag process. We also store 
		//		viewport-to-element transforms since they are not likely to change
		//		while dragging (would require modifying the transformation of some
		//		of the parents of the selected shape). This might change if we plan
		//		to provide pan-while-moving.
		//

		var s = this._selection.fastGet();
		var gs = [];
		
		s.forEach(function(e){
			var ge = this._selection._asGraphElement(e);
			if(ge && this._isMovable(ge)) {
				// no link moving for now
				gs.push(ge);
			}
		},this);
		
		this._targetSet = dojo.map(gu.maximals(gs),function(e){
			return { t1:e.getTransform(), e:e, t:this.getTransformToElement(e) };
		},this);
		
	},
	
	getTransformToElement: function(e) {
		//	returns:
		//		a transform from viewport coords to element coords
		var rm = e._getRealMatrix();
		return (rm && rm.inverse());
	},

	_getGraphElementFromEvent: function (e) {
		//	summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._content);
		return ge !== this._content ? ge : null;
	},

	_dragPreStart: function(e) {
		
		var starter = this._getGraphElementFromEvent(e);
		if(starter && !starter._isIBMDiagramLink && starter.isSelectable()) {
			//
			// [AV]
			// We should not be able to click on a non-selectable/moveable entity and 
			// start the drag from there, even thouh other entities in the target set
			// may be moveable.
			// To avoid this, we peek what's right below the position where we started
			// the drag. It should be a node or subgraph, and should be selectable.
			//
			// note: We may have not selected the to-be-dragged entity yet. It might 
			//       be selected in this same click right after this handler. So we can't
			//       test for isSelected() here.
			// 
		
			this.inherited(arguments);
			this._doDragPreStart(e);
		}
	},
	
  _dragTouchPreStart: function (e) {
    e.target = e.touches[0].target;
    var starter = this._getGraphElementFromEvent(e);
    if(starter && !starter._isIBMDiagramLink && starter.isSelectable()) {
      //
      // [AV]
      // We should not be able to click on a non-selectable/moveable entity and 
      // start the drag from there, even thouh other entities in the target set
      // may be moveable.
      // To avoid this, we peek what's right below the position where we started
      // the drag. It should be a node or subgraph, and should be selectable.
      //
      // note: We may have not selected the to-be-dragged entity yet. It might 
      //       be selected in this same click right after this handler. So we can't
      //       test for isSelected() here.
      // 
      this.inherited(arguments);
      
      this._doDragPreStart(e);
    }
    
  },

  _doDragPreStart: function (e){
    
    dojo.forEach(this._targetSet,function(s){
      s.t = this.getTransformToElement(s.e);
      s.t1 = s.e.getTransform();
    },this);
    
    if(this._viewport) {
      // Monitor viewport transform changes while dragging, so that we can update 
      // the viewport-to-element transforms.
      this._connect("updateTransforms",this._viewport,"onViewRectChanged",this,"_viewportMoved");
    }
    
  },
  
	_viewportMoved: function(){
		var oldit = this._firstViewportT || m.identity;
		var newt = this._viewport.getViewportTransform();
		
		// compute the new location of the first click in client coords.
		this._first = newt.transformPoint(oldit.transformPoint(this._first));
		
		dojo.forEach(this._targetSet,function(s){
			s.t = this.getTransformToElement(s.e);
		},this);
		this._firstViewportT = this._viewport.getViewportInverseTransform();
	},

	_doMove: function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The ammount to translate the entity from its original location.
		graphElement.setTransform(m.multiply(originalTransform,m.translate(delta)));
	},
	
	_dragMove: function(e) {
		//
		//	summary:
		//		Applies the current movement delta to the selected objects.
		//
		//	notes:
		//		It ends up transforming the UI coords to model coords and back to scrollbars coords,
		//		May need to be refactored if this is a performance bottleneck
		//
		
		// this.inherited(arguments); 
		// replaced by the following for performance:
		superclass._dragMove.apply(this, arguments);
		//
		
    this._doDragMove(e);
	},
	
  _dragTouchMove: function (e) {
    //
    //  summary:
    //    Applies the current movement delta to the selected objects.
    //
    //  notes:
    //    It ends up transforming the UI coords to model coords and back to scrollbars coords,
    //    May need to be refactored if this is a performance bottleneck
    //
    
    // this.inherited(arguments); 
    // replaced by the following for performance:
    superclass._dragTouchMove.apply(this, arguments);
    //
    this._doDragMove(e);
  },	
  
  _doDragMove: function (e){
    // check move limits defined on the Graph if any:
    var current = this._current;
    dojo.forEach(this._targetSet,function(s){
      var delta = g.subPoint(m.multiplyPoint(s.t,current),m.multiplyPoint(s.t,this._first));
      var p = s.e.getParent();
      if (p && p.onElementMoving) {
        var oldx = delta.x;
        var oldy = delta.y;
        p.onElementMoving.call(p, s.e, s.t1, delta);
        if (delta.x != oldx || delta.y != oldy)
          current = m.multiplyPoint(s.t.inverse(),g.addPoint(delta, m.multiplyPoint(s.t,this._first)));
      }
    },this);
    
    dojo.forEach(this._targetSet,function(s){
      var delta = g.subPoint(m.multiplyPoint(s.t,current),m.multiplyPoint(s.t,this._first));
      this._doMove(s.e,s.t1,delta);
    },this);
    
    ibm_ilog.diagram.util.Batch.endBatch(); // update links immediately for better visual result
    
  },
	_dragEnd: function(e) {
		var dragged = this.hasDragged;
		this.inherited(arguments);
    this._doDragEnd(dragged);
	},
	
  _dragTouchEnd: function (e) {
    var dragged = this.hasDragged;
    this.inherited(arguments);
    this._doDragEnd(dragged);
  },
  
  _doDragEnd: function (dragged){
    this._disconnect("updateTransforms");
    
    if(dragged && this._viewport && this._viewport.getDiagram()._isIBMDiagramEditor){
      var um = this._viewport.getDiagram().getUndoManager();
      um.addAction(this._createUndoAction());
    }
    
  },
  
	_createUndoAction: function(){
		
		var mAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Move);
		
		dojo.forEach(this._targetSet,function(s){
			var action = ibm_ilog.diagram.editor.EditingUtils. createMoveUndoAction(s.e,s.t1,s.e.getTransform());
			mAction.addAction(action);
		});
		
		return mAction;
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.MarqueeSelectionInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.MarqueeSelectionInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.MarqueeSelectionInteractor');







(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;
	var g = ibm_ilog.diagram.util.Geometry;
	var superclass = ibm_ilog.diagram.interactors.DragInteractor.prototype;
	
dojo.declare("ibm_ilog.diagram.interactors.MarqueeSelectionInteractor", [ibm_ilog.diagram.interactors.DragInteractor], {
	//summary:
	//		This function provides the functionality to manage marquee selection events on links and nodes.
	_selection: null,
	_background: null,
	_marqueePointerStart: null,
	_marquee:null,
	_marqueeStyle: dojo.fromJson("{'fill':'rgba(0, 0, 255, 0.1)' , 'stroke':{'width': '1', 'color':'#0000FF'}}"),
	_marqueeMode: null,
	_partial: null,
	_mustClear: null,
	_declareStates: function() {
	// Summary:
	//		Add the crtl start to the original states
	this.inherited(arguments);	
		this._declareState("idle",["start","startCtrl"]);
	},
	
	getDefaultConnections:function(){
		//
		//	summary:
		//		returns the default event mapping
		//
    		var map = this.inherited(arguments);
    		map.startCtrl = {src:this._getInitialEventSource(),srcEvt:"onmousedown",connectTo:"_dragPreStart",filter:this._buildInputFilter({button:0,ctrl:true}),gfxConnect:true};
    		return map;
    },
    
	initialize: function(viewport,selection) {
		// summary:
		//		creates a new instance, sets the viewport appropriate elements from where to connect and set the default event mapping.
		//		If the given root is not a viewport, the standard selection is activated
		if(selection==null){	
			this._selection = new ibm_ilog.diagram.Selection();
		}else{
			this._selection = selection; 
		}
		this._background = viewport.getEventSourceBackground();
		this._marqueeMode = ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topParent;
		this._partial = true;
		this._viewport = viewport;
		this._initialize();
		return this;
	},
	
	_getInitialEventSource: function() {
		return this._background;
	},
	
	setMarqueeStyle: function(style/*a style json object*/){
    	//summary:
		//		set the marquee style
    	this._marqueeStyle = style;
    },
    
    getMode: function(){
    	//summary:
		//		gets the marquee mode
    	return this._marqueeMode;
    },
    
    setMode: function(mode){
    	//summary:
		//		sets the marquee mode
    	this._marqueeMode = mode;
    },
    isPartialSelectionEnabled: function(){
    	//summary:
		//		gets if the marquee selects partial bounded elements.
    	return this._partial;
    },
    
    setPartialSelectionEnabled: function(partial){
    	//summary:
		//		sets if the marquee selects partial bounded elements.
    	this._partial = partial;
    },
    _dragPreStart: function(e) {
		//	summary:
		//		Starts the marquee selection process by storing initial coordinates for pointer and view rectangle
		//
    	this.inherited(arguments);
    	if(!e.ctrlKey) { // ask for ctrl key
    		this._mustClear = true;
    	}else{
    		this._mustClear = false;
    	}
		this._marqueePointerStart = this._viewport.eventContentLocation(e);
			
    },
    
    _dragMove: function(e) {
		//
		//	summary:
		//		When the receiver is _marqueeActive, draw the corresponding marquee on the viewport UI layer.
    	//
    	// this.inherited(arguments); 
		// replaced by the following for performance:
		superclass._dragMove.apply(this, arguments);
		
    	var g = ibm_ilog.diagram.util.Geometry;
    	var p = this._viewport.eventContentLocation(e); // current mouse coord
        
        var r = g.createRect(this._marqueePointerStart,p);
        
        var transformedRect = this._transformRect(r);
        transformedRect.width = transformedRect.width || 1; // zero crash vml
        transformedRect.height= transformedRect.height|| 1; // zero crash vml
        	
        if(this._marquee==null){
        	this._marquee = this._viewport.getOverlayLayer().createRect( transformedRect );
        	this._applyStyle(this._marquee,this._marqueeStyle);
        }else{
        	this._marquee.setShape(transformedRect);
        }
        
        
    },
   
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current marquee selection process.
    	//
    	var dragged = this.hasDragged;
		this.inherited(arguments);
    	if(dragged){
    		var p2 = this._viewport.eventContentLocation(e);
    		this._selectBoundedElements(this._marqueePointerStart,p2);
    		this._viewport.getOverlayLayer().remove(this._marquee);
    		this._marquee = null;
    	}
    },
    
    _selectBoundedElements: function(p1,p2){
    	// Summary:
    	//		select all the elements that are completely bounded by the rectangle delimited by the given points
    	var g = ibm_ilog.diagram.util.Geometry;
    	
    	var rect = g.createRect(p1,p2);
    	var graph = this._viewport._diagram.getGraph();
    	var toBeAdded = [];
    	this._selectElements(graph,graph,rect,toBeAdded);
    	this._selection.add(toBeAdded,this._mustClear);
    },
    
    _selectElements: function(parentGraph,graph,rect,toBeAdded){
    	var anyAdded = false;
    	for(var n in graph.children){
    		var c = graph.children[n];
    		var b = c.getBounds(parentGraph);
    		
    		var isBounded = this._evaluatesBounded(parentGraph,c,b,rect);
    		var isTouched = this._partial && this._evaluatesTouched(parentGraph,c,b,rect);
    		var isSubgraph = c._isIBMDiagramSubgraph && !c.isCollapsed();
    		
    		var shouldBeIncluded = isBounded||isTouched;
    		
    		switch(this._marqueeMode){
    			case ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topLevelOnly:{ 
    				if(shouldBeIncluded){
    					this._addToSelection(c,toBeAdded);
    					anyAdded = true;}
    				break;}
    			case ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topParent:{ 
    				if( (isSubgraph && isBounded) || (!isSubgraph && shouldBeIncluded)){
    					this._addToSelection(c,toBeAdded);
    					anyAdded = true;
    				}else {if(isSubgraph){
    					anyAdded = this._selectElements(parentGraph,c.getGraph(),rect,toBeAdded) || anyAdded;
    					if(!anyAdded && shouldBeIncluded){
    						this._addToSelection(c,toBeAdded);
    						anyAdded = true;
    					}
    				}}
    				break;}
    			case ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.all:{ 
    				if(shouldBeIncluded){
    					this._addToSelection(c,toBeAdded);
    				anyAdded = true;}
    				if(isSubgraph){
    					anyAdded = this._selectElements(parentGraph,c.getGraph(),rect,toBeAdded) || anyAdded;}
    				break;}
    		}
    	}
    	return anyAdded;
    },
    _evaluatesBounded: function(parentGraph,element,bound,rect){
    	return bound && g.containsRect(rect,bound);
    },
    _evaluatesTouched: function (parentGraph,element,bound,rect){
    	if(element._isIBMDiagramLink){
    		var points = element._pathPoints;
    		var t = element.getShapeToContainerTransform(parentGraph);
    		var included = false;
    		dojo.forEach(points,function(p){
    				included = included || g.containsPoint(rect,t.transformPoint(p));
    		},this);
    		return included;
    	}else{
    		return bound && g.rectIntersects(bound,rect);
    	}
    },
    _addToSelection: function(element,toBeAdded){
    	if(this._selection.accept(element) && element.isSelectable()){
    		toBeAdded.push(element);
    	}
    },
  //TODO this function should be in a tool
    _transformRect: function(bb/*rectangle*/) {
    	//
		//	summary:
		//		transform the rectangle to absolute UI coordinates
    	//
    	var v = this._viewport, z = v.getZoom(), vr = v.getViewRect(), g = ibm_ilog.diagram.util.Geometry;
    	bb = g.moveRect(bb,g.negPoint(vr)); 		
    	bb.width = bb.width*z;
    	bb.height = bb.height*z;
    	bb.x = bb.x*z;
    	bb.y = bb.y*z;
		return bb;
	},
    //TODO this function should be in a tool
	_applyStyle:function(shape, style){
		// summary:
		//		sets a new style to the desired element
		// shape:
		//		the shape to apply the style
		// style:
		//      the new style to be applied
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var setter = 'set' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[setter]) {
					shape[setter](style[p]);
				}
			}
		}
	}

});

// ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topLevelOnly: number
//		in this mode, only the top level elements (the ones that are created in the graph) are selected.
//		This is the default value.
ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topLevelOnly = 0;

// ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topParent: number
//		in this mode, only the highest parent of the selected elements is selected, but not its children. 
//		For example, if a complete subgraph is bounded, just the subgraph is selected, but not its children.
//		In the case the subgraph is not completely bounded by the marquee, its bounded children are selected.
ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topParent = 1;

//ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.all: number
//		in this mode all the bounding elements are selected.
ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.all = 2;


})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.WheelPanInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.WheelPanInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.WheelPanInteractor');




(function(){
	var g = ibm_ilog.diagram.util.Geometry;

dojo.declare("ibm_ilog.diagram.interactors.WheelPanInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//
	//	summary:
	//		A WheelPanInteractor controls the process of panning the viewport by the mouse wheel
	//

	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram to pan
	//
	_diagram: null,
	
	//
	//	_incrementFactor: Number
	//		The amount at which pan is moved at each discrete step
	//
	_incrementFactor: 40,
	
	_declareStates: function() {
		this._declareState("idle",["wheelPan"]);
	},
	
	activate: function() {
		this._goStateId("idle");
	},
	
	setIncrementFactor: function( /*Number*/ factor ) {
		//
		//	summary:
		//		Sets the amount at which pan is moved at each discrete operation
		//
		this._incrementFactor = factor;
	},
	
	getIncrementFactor: function() {
		//
		//	summary:
		//		Returns the amount at which pan is moved at each discrete operation
		//
		return this._incrementFactor;
	},
	
    initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Wheel Pan interactor constructor, based on a target Diagram widget.
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},
    
    getDefaultConnections:function(){
		//
		//	summary:
		//		Returns the default event mapping
		//
    	return {
    		wheelPan:{src:this._diagram.domNode,srcEvt:this.eventNameWheel,connectTo:"_wheelPan",filter:this._buildInputFilter({ctrl:false,shift:false})}
    	};
    },
    
    _wheelPan: function( /*Event*/ e ) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatches the appropriate pan operations
		//
    	
    	var V = this._viewport;
		var delta = this._incrementFactor;
		
		if(this.eventWheelAmmount(e)<0) {
			delta = -delta;
		}

		var viewMovement = g.mulPoint(g.Point(0,delta),-1/V.getZoom()); // scale delta to zoom level to get delta in model coords

		// defect 72537 - ensure the user does not scroll beyond the content
		var viewRect = V.getViewRect();
		var contentRect = V.getContentRect();
		if ((viewRect.y + viewMovement.y) < 0)
			viewMovement.y = -viewRect.y;
		if ((viewRect.y + viewRect.height + viewMovement.y) > contentRect.height) 
			viewMovement.y = contentRect.height - viewRect.y - viewRect.height;
		
		V.setViewRect( g.moveRect(V.getViewRect(),viewMovement), {instant:true} );

		if (e.preventDefault) {
			e.preventDefault();
		}
		//	TODO: do we need this?
		e.returnValue = false;

    }
    
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.AdornerFocusInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.AdornerFocusInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.AdornerFocusInteractor');





(function () {
	var g = ibm_ilog.diagram.util.Geometry;
	var gu = ibm_ilog.diagram.util.GraphUtil;

dojo.declare("ibm_ilog.diagram.interactors.AdornerFocusInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//
	//	summary:
	//		This interactor sets the diagram widget adorners focus 
	//		according to current mouse location. 
	//
	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_diagram: null,

	_nodeMargin: 15,
	_linkMargin: 15,

	_lowCpuMode: null,

	_lastPosition: null,

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		this._lowCpuMode = this._diagram.isLowCpuMode();
		dojo.connect(this._diagram, 'lowCpuModeChange', this, function () {
			this._lowCpuMode = this._diagram.isLowCpuMode();
			this.deactivate();
			this._initialize();
		});
		this._lastPosition = {
			x: 0,
			y: 0
		};
		return this;
	},

	_declareStates: function () {
		if (this._lowCpuMode == true) {
			this._declareState("idle", ["nodeover", "linkover", "subgraphover", "testfreecursor","onLeaveSurface"]);
		} else {
			this._declareState("idle", ["detectOnOver","onLeaveSurface"]);
		}
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		return {
			nodeover: {
				src: this._diagram,
				srcEvt: "onNodeMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			linkover: {
				src: this._diagram,
				srcEvt: "onLinkMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			subgraphover: {
				src: this._diagram,
				srcEvt: "onSubgraphMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			testfreecursor: {
				gfxConnect: true,
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_testFreeCursor"
			},
			detectOnOver: {
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_detectOnOver",
				gfxConnect: true
			},
			onLeaveSurface: {
				src: this._diagram.domNode,
				srcEvt: "onmouseleave",
				connectTo: "_onLeaveSurface"
			}
		};
	},
	
	_onLeaveSurface: function(e) {
		// Check if we are completely outside of the surface,
		// in that case we get no move event so we must free the adorners.
		var out = false;
		var l = this._viewport.eventClientLocation(e);
		if(l.x <= 0 || l.y <= 0){
			out = true;
		} else {
			var d = this._viewport.getClientSize();
			if(l.x >= d.width || l.y >= d.height)
				out = true;
		}
		if(out){
			this._freeCursor();
			this._freeLinkCursor();
		}
	},

	_setNodeAdornersFocus: function (ge, e) {
		// Summary:
		//		Sets the appropriate cursor, according to the ge element. Release the previous cursor, id applied.
		//
		var cursorIsFree = this._diagram.getNodeAdornersFocus() == null;
		if (!ge._isIBMDiagramLink && (cursorIsFree || gu.isContainedBy(ge, this._diagram.getNodeAdornersFocus()))) {
			this._freeLinkCursor(null);
			this._diagram.setNodeAdornersFocus(ge);
			dojo.stopEvent(e);
		} else if (ge._isIBMDiagramLink) {
			this._freeCursor(null);
			this._diagram.setLinkAdornersFocus(ge);
			dojo.stopEvent(e);
		}
	},

	onEnabled: function () {
		this._goStateId("idle");
	},

	onDisabled: function () {
		this._goState(null);
		this._freeCursor();
	},

	_freeCursor: function (e) {
		// Summary:
		//		Free the node/subgraph cursor
		//
		this._diagram.setNodeAdornersFocus(null);
	},

	_freeLinkCursor: function (e) {
		// Summary:
		//		Free the link cursor
		//
		this._diagram.setLinkAdornersFocus(null);
	},

	_testFreeCursor: function (e) {
		// Summary:
		//		Free the cursor testing the bounding box (adding a margin)
		//
		var D = this._viewport.getDiagram();
		var c = this._diagram.getNodeAdornersFocus();
		if (c) {
			var outOfBounds;
			var bb = g.expandRect(c.getBounds(this._viewport.getSurface()), this._nodeMargin);
			var l = this._viewport.eventClientLocation(e);
			outOfBounds = !g.containsPoint(bb, l);
			if (D.mustFreeAdornersFocus(c, p, this._nodeMargin, outOfBounds)) {
				this._freeCursor();
			}
		}
		c = this._diagram.getLinkAdornersFocus();
		if (c) {
			var V = this._viewport;
			var p = V.eventContentLocation(e);
			outOfBounds = !c.hitTest(p, D.getGraph(), this._linkMargin);
			if (D.mustFreeAdornersFocus(c, p, this._linkMargin, outOfBounds)) {
				this._freeLinkCursor();
			}
		}
	},

	_detectOnOver: function (e) {
		//added to increase performance
		var current = {
			x: e.pageX,
			y: e.pageY
		};
		var delta = {
			x: Math.abs(current.x - this._lastPosition.x),
			y: Math.abs(current.y - this._lastPosition.y)
		};
		var minDelta = 2;
		if (delta.x < minDelta && delta.y < minDelta) {
			return;
		}

		this._testFreeCursor(e);
		var p = this._viewport.eventContentLocation(e);
		var ges = this._diagram.getGraph().hitTest(p, 4);
		var ge = this._changeFocus(ges);
		if (ge) {
			this._setNodeAdornersFocus(ge, e);
		}
		this._lastPosition = {
			x: current.x,
			y: current.y
		};

	},
	_changeFocus: function (ges) {
		if (ges.length > 0) {
			var D = this._viewport.getDiagram();
			var anyCursor = this._diagram.getNodeAdornersFocus() || this._diagram.getLinkAdornersFocus();
			var changeAdornersFocus = true;
			if (anyCursor && (anyCursor.isSelected() || anyCursor._isIBMDiagramLink)) {
				changeAdornersFocus = false;
			}

			var newge = ges[0];
			for (var index in ges) {
				if (ges[index].isSelected()) {
					newge = ges[index];
					break;
				}
			}
			/*
			var newge = null;
			for(var index in ges){
				if(ges[index].isSelected()){
					newge = ges[index];
					break;
				}else{
					if(!newge && ges[index]._isIBMDiagramNode){
						newge = ges[index];
					}
				}
			}
			newge = newge?newge:ges[0];
			*/
			if (newge._isIBMDiagramLink && !newge.isSelected()) {
				if (anyCursor && anyCursor._isIBMDiagramNode && !gu.isContainedBy(newge, anyCursor)) {
					changeAdornersFocus = false;
				}
			}

			return D.changeAdornersFocus(anyCursor, newge, ges, changeAdornersFocus);
		}
		return null;
	}
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.FocusInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.FocusInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.FocusInteractor');


//
//

(function () {
//	var g = ibm_ilog.diagram.util.Geometry;
//	var gu = ibm_ilog.diagram.util.GraphUtil;

dojo.declare("ibm_ilog.diagram.interactors.FocusInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//
	//	summary:
	//

	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_diagram: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_graph: null,

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		this._diagram = diagram;
		this._graph = diagram.getGraph();
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},

	_declareStates: function () {
		this._declareState("idle", ["changeFocus"]);
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		return {
			changeFocus: {
				src: this._viewport.getEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_changeFocus",
				gfxConnect: true
			}
		};
	},

	_getGraphElementFromEvent: function (e) {
		// Summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._graph);
		return ge !== this._graph ? ge : null;
	},
	
	_changeFocus: function (e) {

		this._diagram.setFocusedElement(this._getGraphElementFromEvent(e));

		dojo.stopEvent(e);
	},

	onEnabled: function () {
		this._goStateId("idle");
	},

	onDisabled: function () {
		this._goState(null);
	}
	
});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.KeyboardInteractorManager']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.KeyboardInteractorManager'] = true;
dojo.provide('ibm_ilog.diagram.interactors.KeyboardInteractorManager');





(function () {
	var R = ibm_ilog.diagram.util.ErrorReporter;

dojo.declare("ibm_ilog.diagram.interactors.KeyboardInteractorManager", [], {
	// summary:
	//		This class manage the keyInteractor, by connection to the Diagram onkeydown event and pushing it to the appropriate registered Interactors.
	// The associated diagram
	_diagram: null,

	// the event connection
	_connection: null,

	// a dictionary of keyInteractors, by id
	_keyInteractors: null,

	//a map of hoKeys with the keyInteractors connections to be launch in each case
	// format:
	//			{ 65: { interactorId1:[connection1,connection2] , interactorId2:[connection3] },
	//			  70: { interactorId2:[connection4] } }
	_hotKeys: null,

	constructor: function (diagram) {
		// summary:
		//		Creates a new instance.
		// diagram: ibm_ilog.diagram.widget.Diagram
		//		the associated diagram.
		this._diagram = diagram;
		this._keyInteractors = {};
		this._hotKeys = {};
	},
	activate: function () {
		// summary:
		//		Activate the Manager, creating the connection.
		if (!this._connection) {
			this._connection = dojo.connect(this._diagram.getKeyboardEventSource(), "onkeydown", this, "keyPressed");
		}
	},
	deactivate: function () {
		// summary:
		//		Deactivate the Manager, deleting the connection.
		dojo.disconnect(this._connection);
		this._connection = null;
	},
	add: function (id, keyInteractor) {
		// summary:
		//		Adds a new keyInteractor.
		// id: String
		//		the keyIntearctor id.		
		// keyInteractor: ibm_ilog.diagram.interactors.Interactor
		//		the keyIntearctor to be added.		
		this._keyInteractors[id] = keyInteractor;
		keyInteractor.setKeyManager(this);
	},
	get: function (id) {
		// summary:
		//		Gets a keyInteractor by its id.
		// id: String
		//		the keyIntearctor id.
		// return: ibm_ilog.diagram.interactors.Interactor
		return this._keyInteractors[id];
	},
	remove: function (id) {
		// summary:
		//		Removes a keyInteractor by its id.
		// id: String
		//		the keyIntearctor id.
		delete(this._keyInteractors[id]);
	},
	keyPressed: function (e) {
		// summary:
		//		This method is called when a key is pressed in the diagram.
		// e: Event object
		//		the event to be treated.
		// return boolean
		var wasHandled = false;

		if (e.keyCode in this._hotKeys) {
			for (var accId in this._hotKeys[e.keyCode]) {
				wasHandled = this._keyInteractors[accId].keyPressed(this._hotKeys[e.keyCode][accId], e) || wasHandled;
			}
		}

		// If handled, stop the event and prevent default
		if (wasHandled) {
			dojo.stopEvent(e);
			if (e.preventDefault) {
				e.preventDefault();
			}
		}
		return wasHandled;
	},

	setHotKeys: function (id, hotKeys) {
		// summary:
		//		This method is called to register the hotkeys to the manager. Ideally, this is called inside the Interactors at initialization time.
		// id: String
		//		the keyIntearctor id.
		// hotKeys: JS object
		//		a dictionary with the hotKeys to be register and the connections to be registered in each one. Format: {key1:[conn1,conn2], key2:[conn4]}
		for (var key in hotKeys) {
			if (!(key in this._hotKeys)) {
				this._hotKeys[key] = {};
			}
			this._hotKeys[key][id] = dojo.clone(hotKeys[key]);
		}
	},
	clearHotKeys: function (id, hotKeys) {
		// summary:
		//		This method is called to clear the hotkeys in the manager. Ideally, this is called inside the Interactors when trying to unregister it for removal or reconfiguration.
		// id: String
		//		the keyIntearctor id.
		// hotKeys: JS object
		//		a dictionary with the hotKeys to be register and the connections to be registered in each one. Format: {key1:[conn1,conn2], key2:[conn4]}
		for (var key in hotKeys) {
			delete this._hotKeys[key][id];
		}
	},

	__eod: undefined
});

})();

}

if(!dojo._hasResource["dojox.collections.Stack"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.collections.Stack"] = true;
dojo.provide("dojox.collections.Stack");


dojox.collections.Stack=function(/* array? */arr){
	//	summary
	//	returns an object of type dojox.collections.Stack
	var q=[];
	if (arr) q=q.concat(arr);
	this.count=q.length;
	this.clear=function(){
		//	summary
		//	Clear the internal array and reset the count
		q=[];
		this.count=q.length;
	};
	this.clone=function(){
		//	summary
		//	Create and return a clone of this Stack
		return new dojox.collections.Stack(q);
	};
	this.contains=function(/* object */o){
		//	summary
		//	check to see if the stack contains object o
		for (var i=0; i<q.length; i++){
			if (q[i] == o){
				return true;	//	bool
			}
		}
		return false;	//	bool
	};
	this.copyTo=function(/* array */ arr, /* int */ i){
		//	summary
		//	copy the stack into array arr at index i
		arr.splice(i,0,q);
	};
	this.forEach=function(/* function */ fn, /* object? */ scope){
		//	summary
		//	functional iterator, following the mozilla spec.
		dojo.forEach(q, fn, scope);
	};
	this.getIterator=function(){
		//	summary
		//	get an iterator for this collection
		return new dojox.collections.Iterator(q);	//	dojox.collections.Iterator
	};
	this.peek=function(){
		//	summary
		//	Return the next item without altering the stack itself.
		return q[(q.length-1)];	//	object
	};
	this.pop=function(){
		//	summary
		//	pop and return the next item on the stack
		var r=q.pop();
		this.count=q.length;
		return r;	//	object
	};
	this.push=function(/* object */ o){
		//	summary
		//	Push object o onto the stack
		this.count=q.push(o);
	};
	this.toArray=function(){
		//	summary
		//	create and return an array based on the internal collection
		return [].concat(q);	//	array
	};
}

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.Navigator']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.Navigator'] = true;
dojo.provide('ibm_ilog.diagram.interactors.Navigator');




(function(){
dojo.declare("ibm_ilog.diagram.interactors.Navigator",ibm_ilog.diagram.interactors.Interactor,{
	//summary:
	//		This Interactor manages the naviagation of the graph through the keyboard interaction.
	//		This Interactor have 5 connections: changeNavigatorStyle, clearFocusedElement, backward, forward, previous, next.
	
	_diagram: null,
	_strategy: null,
	_strategyNumber: null,
	
	initialize: function(diagram){
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this.setNavigationStyle(ibm_ilog.diagram.interactors.Navigator.TreeStyle);
		return this._initialize();
	},
	changeNavigatorStyle: function(e){
		// summary:
		//		switches the navigation style
		this.setNavigationStyle((this._strategyNumber+1)%2);
	},
	clearFocusedElement: function(){
		// summary:
		//		clears the focus.
		this._diagram.setFocusedElement(null);
	},
	next: function(e){
		// Summary:
		//		apply the next operation according to the navigation style.
		this._strategy.next(e);
	},
	previous: function(e){
		// Summary:
		//		apply the previous operation according to the navigation style.
		this._strategy.previous(e);
	},
	forward: function(e){
		// Summary:
		//		apply the forward operation according to the navigation style.
		this._strategy.forward(e);
	},
	backward: function(e){
		// Summary:
		//		apply the backward operation according to the navigation style.
		this._strategy.backward(e);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.Navigator.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			backward: {
				hotKey: dojo.keys.LEFT_ARROW,	
				connectTo: "backward",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, forward: {
				hotKey: dojo.keys.RIGHT_ARROW,	
				connectTo: "forward",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, previous: {
				hotKey: dojo.keys.UP_ARROW,	
				connectTo: "previous",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, next: {
				hotKey: dojo.keys.DOWN_ARROW,	
				connectTo: "next",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, changeNavigatorStyle: {
				hotKey: dojo.keys.F8,	
				connectTo: "changeNavigatorStyle"
			},clearFocusedElement: {
				hotKey: dojo.keys.ESCAPE,	
				connectTo: "clearFocusedElement"
			}
		};
	},
	setNavigationStyle: function(style){
		// Summary:
		//		sets the navigation style.
		// style: number
		//		one of the navigation styles
		this._strategyNumber = style;
		switch(style){
			case ibm_ilog.diagram.interactors.Navigator.TreeStyle: this._strategy = new ibm_ilog.diagram.interactors.TreeNavigator(this._diagram);break;
			case ibm_ilog.diagram.interactors.Navigator.GraphStyle: this._strategy = new ibm_ilog.diagram.interactors.GraphNavigator(this._diagram);break;
			default: this._strategy = new ibm_ilog.diagram.interactors.NullNavigator(this._diagram); this._strategyNumber = ibm_ilog.diagram.interactors.Navigator.NullStyle;break;
		}
	}
});

dojo.declare("ibm_ilog.diagram.interactors.NullNavigator",null,{
	//summary:
	//		This class implements the disabled Navigation style. Also works as the superclass for others Navigation-Styles
	next: function(e){
		// Summary:
		//		do Nothing
	},
	previous: function(e){
		// Summary:
		//		do Nothing
	},
	forward: function(e){
		// Summary:
		//		do Nothing
	},
	backward: function(e){
		// Summary:
		//		do Nothing
	},
	_getFirstNode: function(){
		var children = this._diagram.getGraph().children;
		for(var i = 0;i<children.length;i++){
			if(children[i]._isIBMDiagramNode){
				return children[i];
			}
		}
		return null;
	},
	_getLastNode: function(){
		var children = this._diagram.getGraph().children;
		for(var i = (children.length-1);i>=0;i--){
			if(children[i]._isIBMDiagramNode){
				return children[i];
			}
		}
		return null;
	},
	_getChildIndex: function(array,element){
		for(index in array){
			if(element == array[index]){
				return parseInt(index);
			}
		}
		return -1;
	},
	_getVisibleParent: function(ge){
		// summary: 
		//		gets the lowest level visible element. If the Graph element is visible, return the same argument.
		// return: ibm_ilog.diagram.GraphElement
		var topMostCollapsedNode = null;
		var graph = this._diagram.getGraph();
		for(var p = ge.getParent(); p && p.getParent && p != graph; p = p._collapsedParent || p.getParent()){
			if(p._collapsedParent){
				for (var pp = p._collapsedParent; pp && pp.getParent; pp = pp.getParent()) {
					if (pp._isIBMDiagramSubgraph) {
						topMostCollapsedNode = pp;
						break;
					}
				}
			}
		}
		return topMostCollapsedNode || ge;
	},
	setFocusedElement: function(ge){
		// Summary:
		//		sets the ge to the diagram
		this._diagram.setFocusedElement(ge);
	}
});

dojo.declare("ibm_ilog.diagram.interactors.GraphNavigator",[ibm_ilog.diagram.interactors.NullNavigator],{
	//summary:
	//		This class implements the Graph Navigation style. The navigation is done througth 
	//		connections and record the path to enable backward navigation.
	_diagram: null,
	_stack: null,
	_clearOnNotification: null,
	_connections: null,
	
	constructor: function(diagram){
		this._diagram = diagram;
		this._stack = new dojox.collections.Stack();
		this._clearOnNotification = true;
		this._connections = [];
		this._connections.push(dojo.connect(this._diagram,'onFocusedElementChanged',this, function(){
			if(this._clearOnNotification){
				this.resetStack();
			}
		}));
	},
	next: function(e){
		// description:
		//		If Node focused: focus the first link of the focused node, and remember the 
		//		focused node as being the owner of the links being navigated.
		//		If Link focused: focus the previous link of previously focused node 
		//		(the node whose links are being navigated). If the currently focused link is 
		//		the first of the list, it does nothing.
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._nextConnection(ge);
		}
	},
	previous: function(e){
		// description:
		//		If Node focused: focus the last link of the focused node and remember 
		//		the focused node as being the owner of the links being navigated.
		//		If Link focused: focus the next link of previously focused node (the node 
		//		whose links are being navigated). If the currently focused link is the last 
		//		of the list, it does nothing.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._previousConnection(ge);
		}
	},
	forward: function(e){
		// description:
		//		If Node focused: focus the last link that was followed backwards (if any) 
		//		to get to that node. It is then possible to keep on browsing other links of 
		//		the previous node that was focused before ending up on the current one (navigating backwards).
		//		If Link focused: focus the endNode of the currently focused link, if any.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._followConnectionForward(ge);
		}
	},
	backward: function(e){
		// Summary:
		//		If Node focused: focus the last link that was followed forward (if any) to 
		//		get to that node. It is then possible to keep on browsing other links of the 
		//		previous node that was focused before ending up on the current one (navigating forward).
		//		If Link focused: focus the startNode of the currently focused link, if any.		
		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._followConnectionBackwards(ge);
		}
	},
	_validateFocusedElement: function(ge){
		if(ge){
			var vp = this._getVisibleParent(ge);
			if(vp == ge){
				return true;
			}else{
				this.resetStack();
				this.setFocusedElement(vp);
				return false;
			}
		}else{
			this.resetStack();
			this.setFocusedElement(this._getFirstNode());
			return false;
		}
	},
	getFocusedElement: function(){
		// summary: 
		//		gets the focused GraphElement from the diagram
		// return: ibm_ilog.diagram.GraphElement
		return this._diagram.getFocusedElement();
	},
	_followConnectionBackwards: function(ge){
		if(ge._isIBMDiagramNode){
			if(this._stack.peek() && this._stack.peek()._isIBMDiagramLink){
				this.setFocusedElement(this._stack.pop());
			}
		}else{
			var node = this._getLinkedNode(ge,true);
			this.setFocusedElement(node);
			if(this._stack.peek() == node){
				this._stack.pop();
			}else{
				this._stack.push(ge);
			}
		}
	},
	_followConnectionForward: function(ge){
		if(ge._isIBMDiagramNode){
			if(this._stack.peek() && this._stack.peek()._isIBMDiagramLink){
				this.setFocusedElement(this._stack.pop());
			}
		}else{
			var node = this._getLinkedNode(ge,false);
			this.setFocusedElement(node);
			if(this._stack.peek() == node){
				this._stack.pop();
			}else{
				this._stack.push(ge);
			}
		}		
	},
	_nextConnection: function(ge){
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length>0){
				this.setFocusedElement(links[0]);
				this._stack.push(ge);
			}
		}else{
			var link = this._getFollowingLink(ge,1);
			if(link){
				this.setFocusedElement(link);
			}
		}
	},
	_previousConnection: function(ge){
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length>0){
				this.setFocusedElement(links[links.length-1]);
				this._stack.push(ge);
			}
		}else{
			var link = this._getFollowingLink(ge,-1);
			if(link){
				this.setFocusedElement(link);
			}
		}
	},
	resetStack: function(){
		// summary:
		//		clears the navigation stack
		this._stack.clear();
	},
	_getFollowingLink: function(ge,increment){
		var links = this._getElementLinks(this._stack.peek());
		var index = this._getChildIndex(links,ge); 
		for(var i = index+increment;i>=0 && i<links.length;i=i+increment){
			return links[i];
		}
		return null;
	},
	setFocusedElement: function(ge){
		// summary:
		//		calls the superclass method, but disable the clearNotification
		this._clearOnNotification = false;
		this.inherited(arguments);
		this._clearOnNotification = true;
	},
	_getLinkedNode: function(link,start){
		var port;
		if(start){
			port = link.getStartPort();
		}else{
			port = link.getEndPort();
		}
		port = link._getVisiblePort(port);
		return port.getOwner();
	},
	_getElementLinks: function(ge){
		var result = ge.getLinks();
		if(ge._isIBMDiagramSubgraph){
			result = result.concat(ge.getIntergraphLinks());
		}
		return result;
	}
});

dojo.declare("ibm_ilog.diagram.interactors.TreeNavigator",[ibm_ilog.diagram.interactors.NullNavigator],{
	//summary:
	//		This class implements the Tree Navigation style.
	_diagram: null,
	constructor: function(diagram){
		this._diagram = diagram;
	},
	next: function(e){
		// description:
		//		focus the next sibling of the currently selected Node. If there is no such sibling, 
		//		it focus the next sibling of the parent of the currently selected node, provided that the 
		//		parent it not the root graph. In this case, it does nothing.
				
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this.nextNode(ge);
		}
	},
	previous: function(e){
		// description:
		//		focus the next sibling of the currently selected Node. If there is no such sibling, 
		//		it focus the next sibling of the parent of the currently selected node, provided that the 
		//		parent it not the root graph. In this case, it does nothing.	
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this.previousNode(ge);
		}
	},
	forward: function(e){
		// description:
		//		opens the currently selected Node if it is a closed Subgraph. 
		//		If it is an opened Subgraph, or if it is not a Subgraph, it does nothing.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._openNode(ge);
		}
	},
	backward: function(e){
		// description:
		//		closes the currently selected Node if it is an opened Subgraph. If is is a closed subgraph,
		//		or if it is not a Subgraph, it focus the parent of the currently selected node, provided 
		//		that the parent it not the root graph. In this case, it does nothing.	
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._closeNode(ge);
		}
	},
	_validateFocusedElement: function(ge){
		if(ge){
			var vp = this._getVisibleParent(ge);
			if(vp == ge){
				return true;
			}else{
				this.setFocusedElement(vp);
				return false;
			}
		}else{
			this.setFocusedElement(this._getFirstNode());
			return false;
		}
	},
	nextNode: function(ge){
		this._nextNode(ge,1);
	},
	previousNode: function(ge){
		this._nextNode(ge,-1);
	},
	_nextNode: function(c,increment){
		if(c){
			if(c._isIBMDiagramNode){
				var p = c.getParent();
				var next = this._getFollowingNode(p.children,c,increment);
				if(next){
					this.setFocusedElement(next);
				}else{
					this._nextNode(p._owningSubgraph,increment);
				}
			}
		}else{
			if(increment>0){
				this.setFocusedElement(this._getFirstNode());
			}else{
				this.setFocusedElement(this._getLastNode());
			}
		}
	},
	_openNode: function(ge){
		if(ge && ge._isIBMDiagramSubgraph){
			if(ge.isCollapsed() && ge.canCollapse()){
				ge.setCollapsed(false);	
			}else{
				var children = ge.getGraph().children;
				if(children.length>0){
					this.setFocusedElement(children[0]);
					return;
				}
			}
		}
	},
	_closeNode: function(ge){
		var ge = this.getFocusedElement();
		if(ge && ge._isIBMDiagramNode){
			if(ge._isIBMDiagramSubgraph && !ge.isCollapsed() && ge.canCollapse()){
				ge.setCollapsed(true);	
			}else{
				var parent = ge.getParent()._owningSubgraph;
				if(parent){
					this.setFocusedElement(parent);
				}
			}
		}
	},
	getFocusedElement: function(){
		// summary: 
		//		gets the focused Node from the diagram
		// return: ibm_ilog.diagram.Node
		var ge = this._diagram.getFocusedElement();
		if(ge && !ge._isIBMDiagramLink) {
			return ge;
		}
		return null;
	},
	_getFollowingNode: function(array,element,increment){
		var index = this._getChildIndex(array,element); 
		for(var i = index+increment;i>=0 && i<array.length;i=i+increment){
			if(array[i]._isIBMDiagramNode){
				return array[i];
			}
		}
		return null;
	}
});
// null navigation style
ibm_ilog.diagram.interactors.Navigator.NullStyle = -1;
// tree navigation style
ibm_ilog.diagram.interactors.Navigator.TreeStyle = 0;
// graph navigation style
ibm_ilog.diagram.interactors.Navigator.GraphStyle = 1;

// the interactor Id
ibm_ilog.diagram.interactors.Navigator.KeyInteractorId = "Navigator";

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.SelectionKeyInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.SelectionKeyInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.SelectionKeyInteractor');





dojo.declare("ibm_ilog.diagram.interactors.SelectionKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the Selection through the keyboard interaction.
	//		This Interactor have 4 connections: singleSelect, multiSelect, selectAll and clearSelection.		
	
	_diagram: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		return this._initialize();
	},
	
	getSelection: function () {
		// Summary:
		//		return the selection object.
		// return: ibm_ilog.diagram.Selection
		return this._diagram.getSelection();
	},
	
	getKeyInteractorId: function(){
		// Summary:
		//		return the interactor Id.
		return ibm_ilog.diagram.interactors.SelectionKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			singleSelect: {
				hotKey:dojo.keys.SPACE,
				connectTo: "singleSelect",
				filter: this._buildInputFilter({ctrl:false})
			}, multiSelect: {
				hotKey:dojo.keys.SPACE,
				connectTo: "multiSelect",
				filter: this._buildInputFilter({ctrl:true})
			}, selectAll: {
				hotKey:65,/*letter a*/
				connectTo: "selectAll",
				filter: this._buildInputFilter({ctrl:true})
			}, clearSelection: {
				hotKey:dojo.keys.ESCAPE,
				connectTo: "clearSelection"
			}
		};
	},
	singleSelect: function(e){
		// summary:
	    //		This method clears the selection and adds the focused element to the selection.
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		var ge = this._diagram.getFocusedElement();
		if(ge && sel.accept(ge)){
			sel.add(ge,true);
		}
	},
	multiSelect: function(e){
		// summary:
	    //		This method clears the adds the focused element to the selection or removed it if selected.
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		var ge = this._diagram.getFocusedElement();
		if(ge){
			if(ge.isSelected()){
				sel.remove(ge);
			}else{
				if(sel.accept(ge)){
					sel.add(ge,false);
				}
			}
		}
	},
	selectAll: function(e){
		// summary:
	    //		This method adds all the graph elements to the selection
	    // e: Event object
	    //		the event to be treated.
		var graph = this._diagram.getGraph();
		this.getSelection().selectAll(graph);
	},
	clearSelection: function(e){
		// summary:
	    //		This method clears the selection
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		sel.clear();
	}
});

ibm_ilog.diagram.interactors.SelectionKeyInteractor.KeyInteractorId = "Selection";

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.MoveKeyInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.MoveKeyInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.MoveKeyInteractor');











(function(){
	var g = ibm_ilog.diagram.util.Geometry;
	var gu = ibm_ilog.diagram.util.GraphUtil;
	var m = dojox.gfx.matrix;
	
dojo.declare("ibm_ilog.diagram.interactors.MoveKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the graphElements movement through the keyboard interaction.
	//		This Interactor have 8 connections: moveLeft, moveRigth, moveUp, moveDown, fineMoveLeft, fineMoveRigth, fineMoveUp, fineMoveDown.
	
	_diagram: null,
	_increment: null,
	_fineIncrement: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this.setIncrement(10);
		this.setFineIncrement(1);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the move increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the move increment
		return this._increment;
	},
	setFineIncrement: function(increment){
		// summary: 
		//		sets the move fine increment
		// increment: integer
		//		the fine increment to be set
		this._fineIncrement = increment;
	},
	getFineIncrement: function(){
		// summary: 
		//		gets the move fine increment
		return this._fineIncrement;
	},
	moveUp: function(e){
		// summary:
	    //		This method moves the selected elements up
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:-this._increment});
	},
	moveDown: function(e){
		// summary:
	    //		This method moves the selected elements down
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:this._increment});
	},
	moveRight: function(e){
		// summary:
	    //		This method moves the selected elements rigth
	    // e: Event object
	    //		the event to be treated.
		this._move({x:this._increment,y:0});
	},
	moveLeft: function(e){
		// summary:
	    //		This method moves the selected elements left
	    // e: Event object
	    //		the event to be treated.
		this._move({x:-this._increment,y:0});
	},
	fineMoveUp: function(e){
		// summary:
	    //		This method finely moves the selected elements up
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:-this._fineIncrement});
	},
	fineMoveDown: function(e){
		// summary:
	    //		This method finely moves the selected elements down
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:this._fineIncrement});
	},
	fineMoveRight: function(e){
		// summary:
	    //		This method finely moves the selected elements rigth
	    // e: Event object
	    //		the event to be treated.
		this._move({x:this._fineIncrement,y:0});
	},
	fineMoveLeft: function(e){
		// summary:
	    //		This method finely moves the selected elements left
	    // e: Event object
	    //		the event to be treated.
		this._move({x:-this._fineIncrement,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.MoveKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			moveLeft: {
				hotKey: dojo.keys.LEFT_ARROW,
				connectTo: "moveLeft",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveLeft: {
				hotKey: null,
				connectTo: "fineMoveLeft",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveRight: {
				hotKey: dojo.keys.RIGHT_ARROW,
				connectTo: "moveRight",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveRight: {
				hotKey: null,
				connectTo: "fineMoveRight",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveUp: {
				hotKey: dojo.keys.UP_ARROW,
				connectTo: "moveUp",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveUp: {
				hotKey: null,
				connectTo: "fineMoveUp",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveDown: {
				hotKey: dojo.keys.DOWN_ARROW,
				connectTo: "moveDown",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveDown: {
				hotKey: null,
				connectTo: "fineMoveDown",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}
		};
	},
	_move: function(delta){
		var targetSet = this._buildTargetSet();
		
		if(targetSet.length == 0){
			return false;
		}
		
		// check move limits defined on the Graph if any:
		dojo.forEach(targetSet,function(s){
			var p = s.e.getParent();
			if (p && p.onElementMoving) {
				var oldx = delta.x;
				var oldy = delta.y;
				p.onElementMoving.call(p, s.e, s.t1, delta);
			}
		},this);
		
		dojo.forEach(targetSet,function(s){
			this._doMove(s.e,s.t1,delta);
		},this);
		
		ibm_ilog.diagram.util.Batch.endBatch(); // update links immediately for better visual result
		
		if(this._diagram._isIBMDiagramEditor){
			var um = this._diagram.getUndoManager();
			um.addAction(this._createUndoAction(targetSet));
		}
		return true;
	},
	_doMove: function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The ammount to translate the entity from its original location.
		graphElement.setTransform(m.multiply(originalTransform,m.translate(delta)));
	},
	_isMovable: function(ge) {
		return !(ge._isIBMDiagramLink);
	},
	getTransformToElement: function(e) {
		//	returns:
		//		a transform from viewport coords to element coords
		var rm = e._getRealMatrix();
		return (rm && rm.inverse());
	},
	_buildTargetSet: function() {
		//
		//	summary:
		//		Builds the set of entities that are going to be affected by the drag.
		//
		//	description:
		//		Grab Nodes and Subgraphs from the _selection object. Their original 
		//    transforms are saved to be used in the drag process. We also store 
		//		viewport-to-element transforms since they are not likely to change
		//		while dragging (would require modifying the transformation of some
		//		of the parents of the selected shape). This might change if we plan
		//		to provide pan-while-moving.
		//

		var selection = this._diagram.getSelection();
		var s = selection.fastGet();
		var gs = [];
		
		s.forEach(function(e){
			var ge = selection._asGraphElement(e);
			if(ge && this._isMovable(ge)) {
				// no link moving for now
				gs.push(ge);
			}
		},this);
		
		//if no selection, add the focused element
		if(gs.length == 0){
			var c = this._diagram.getFocusedElement();
			if(c && this._isMovable(c)){
				gs.push(c);
			}
		}
		
		var targetSet = dojo.map(gu.maximals(gs),function(e){
			return { t1:e.getTransform(), e:e, t:this.getTransformToElement(e) };
		},this);
		
		return targetSet;
	},
	_createUndoAction: function(targetSet){
		// summary:
		//		creates the undo action, based on the targetSet
		var mAction = new ibm_ilog.diagram.editor.undo.MultipleAction(ibm_ilog.diagram.editor.undo.UndoActionList.Move);
		
		dojo.forEach(targetSet,function(s){
			var action = ibm_ilog.diagram.editor.EditingUtils. createMoveUndoAction(s.e,s.t1,s.e.getTransform());
			mAction.addAction(action);
		});
		
		return mAction;
	}
});

ibm_ilog.diagram.interactors.MoveKeyInteractor.KeyInteractorId = "Move";
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.PanKeyInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.PanKeyInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.PanKeyInteractor');







dojo.declare("ibm_ilog.diagram.interactors.PanKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the ViewPort Pan movement through the keyboard interaction.
	//		This Interactor have 4 connections: panLeft, panRigth, panUp, panDown.
	
	_diagram: null,
	_increment: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		this.setIncrement(10);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the pan increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the pan increment
		return this._increment;
	},
	panUp: function(e){
		// summary:
	    //		This method pans the diagram up
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:-this._increment});
	},
	panDown: function(e){
		// summary:
	    //		This method pans the diagram down
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:this._increment});
	},
	panRight: function(e){
		// summary:
	    //		This method pans the diagram right
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:this._increment,y:0});
	},
	panLeft: function(e){
		// summary:
	    //		This method pans the diagram left
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:-this._increment,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			panLeft: {
				hotKey: dojo.keys.LEFT_ARROW,	
				connectTo: "panLeft",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panRight: {
				hotKey: dojo.keys.RIGHT_ARROW,	
				connectTo: "panRight",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panUp: {
				hotKey: dojo.keys.UP_ARROW,	
				connectTo: "panUp",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panDown: {
				hotKey: dojo.keys.DOWN_ARROW,	
				connectTo: "panDown",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}
		};
	},
	_pan: function(delta){
		var v = this._diagram.getViewport();
		var g = ibm_ilog.diagram.util.Geometry;
		var viewMovement = g.mulPoint(delta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		var firstViewRect = v.getViewRect();
		v.setViewRect(g.moveRect(firstViewRect, viewMovement), {instant:true});
	}
});

ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId = "Pan";

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.ZoomKeyInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.ZoomKeyInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.ZoomKeyInteractor');





dojo.declare("ibm_ilog.diagram.interactors.ZoomKeyInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the ViewPort Zoom through the keyboard interaction.
	//		This Interactor have 2 connections: zoomIn, zoomOut.
	
	_diagram: null,
	_incrementFactor: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		this.setIncrementFactor(1.5);
		return this._initialize();
	},
	setIncrementFactor: function(incrementFactor){
		// summary: 
		//		sets the zoom increment factor
		// increment: integer
		//		the increment factor to be set
		this._incrementFactor = incrementFactor;
	},
	getIncrementFactor: function(){
		// summary: 
		//		gets the zoom increment factor
		// return: integer
		return this._incrementFactor;
	},
	zoomOut: function(e){
		// summary:
	    //		This method zooms the diagram out
	    // e: Event object
	    //		the event to be treated.
		this._zoom(1/this._incrementFactor);
	},
	zoomIn: function(e){
		// summary:
	    //		This method zooms the diagram in
	    // e: Event object
	    //		the event to be treated.
		this._zoom(this._incrementFactor);
	},
	zoomReset: function(e){
		// summary:
	    //		This method reset the diagram zoom (set to 1)
	    // e: Event object
	    //		the event to be treated.
		var v = this._diagram.getViewport();
		var zoom = 1/v.getZoom();
		this._zoom(zoom);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.ZoomKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			zoomIn: {
				hotKey: dojo.keys.NUMPAD_PLUS,
				connectTo: "zoomIn",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomOut: {
				hotKey: dojo.keys.NUMPAD_MINUS,
				connectTo: "zoomOut",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomReset: {
				hotKey: dojo.keys.NUMPAD_0,
				connectTo: "zoomReset",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}
			
		};
	},
	_zoom: function(zoomFactor){
		var v = this._diagram.getViewport();
		var ge = this._diagram.getFocusedElement();
		 
		if(ge){
			var point = ibm_ilog.diagram.util.Geometry.getRectCenter(ge.getBounds(this._diagram.getGraph()));
			this._zoomAt(v,point,zoomFactor);
		}else{
			var currentZoom = v.getZoom();
	        var zoom = currentZoom * zoomFactor;
			v.setZoom(zoom, {
				instant: true
			});
		}
		
		
	},
	_zoomAt: function(viewport, /*Point*/ p, /*Number*/z ) {
		//
		//	summary:
		//		Performs a zoom operation maitaining a pivot point p in the same 
    	//		fixed position relative to the viewport.
		//
    	var cp = viewport.pointClientToContent(p); // content p    	
    	viewport.zoomAt(cp,z);
    }
});

ibm_ilog.diagram.interactors.ZoomKeyInteractor.KeyInteractorId = "Zoom";

}

if(!dojo._hasResource['ibm_ilog.diagram.interactors.SwitchModeInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.interactors.SwitchModeInteractor'] = true;
dojo.provide('ibm_ilog.diagram.interactors.SwitchModeInteractor');






dojo.declare("ibm_ilog.diagram.interactors.SwitchModeInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//summary:
	//		This Interactor manages the UndoManager through the keyboard interaction.
	//		This Interactor have 3 connections: switchEdition, enterEdition, enterNavigation.
	
	_diagram: null,
	_onNavigation: null,
	_navigationInteractorIds: null,
	_editionInteractorIds: null,
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this._navigationInteractorIds = [ibm_ilog.diagram.interactors.Navigator.KeyInteractorId];
		this._editionInteractorIds = [ibm_ilog.diagram.interactors.MoveKeyInteractor.KeyInteractorId];
		return this._initialize();
	},
	switchEdition: function(e){
		// summary:
	    //		switch between edition and navigation mode.
		if(this._onNavigation){
			this.enterEdition();
		}else{
			this.enterNavigation();
		}
	},
	enterEdition: function(){
		// summary:
	    //		enter edition mode
		this._setInteractorsEnabled(this._navigationInteractorIds,false);
		this._setInteractorsEnabled(this._editionInteractorIds,true);
		this._onNavigation = false;
	},
	enterNavigation: function(){
		// summary:
	    //		enter navigation mode
		this._setInteractorsEnabled(this._navigationInteractorIds,true);
		this._setInteractorsEnabled(this._editionInteractorIds,false);
		this._onNavigation = true;
	},
	addNavigationInteractorId: function(id){
		// summary:
		//		adds an interactor id, that will be enabled when entering on navigation mode
		this._navigationInteractorIds.push(id);
	},
	addEditionInteractorId: function(id){
		// summary:
		//		adds an interactor id, that will be enabled when entering on edition mode
		this._editionInteractorIds.push(id);
	},
	_setInteractorsEnabled: function(interactorsIds,enabled){
		dojo.forEach(interactorsIds,
				function(id){
						this._keyManager.get(id).setEnabled(enabled);
						},this);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ibm_ilog.diagram.interactors.SwitchModeInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			enterNavigation: {
				hotKey: null,
				connectTo: "enterNavigation",
				filter: this._buildInputFilter({})
			}, enterEdition: {
				hotKey: null,
				connectTo: "enterEdition",
				filter: this._buildInputFilter({})
			}, switchEdition: {
				hotKey: dojo.keys.F9,
				connectTo: "switchEdition"
			}
		};
	}
});

ibm_ilog.diagram.interactors.SwitchModeInteractor.KeyInteractorId = "EnterEdition";

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.TemplatedOverlay']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.TemplatedOverlay'] = true;
dojo.provide('ibm_ilog.diagram.adorners.TemplatedOverlay');

(function() {
    
    dojo.declare('ibm_ilog.diagram.adorners.TemplatedOverlay',[dojox.gfx.Group,ibm_ilog.diagram.templating._DiagramTemplated],{
    	
    	constructor: function() {
	        var id = ibm_ilog.diagram._IdGenerator.nextId(this).toString();
	        this.getId = function() {
	            return id;
	        }
    	}

    });
    
})();

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.Adorner']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.Adorner'] = true;
dojo.provide('ibm_ilog.diagram.adorners.Adorner');





(function () {

var
	R = ibm_ilog.diagram.util.ErrorReporter,
	g = ibm_ilog.diagram.util.Geometry,	Rect = g.Rect,	EmptyRect = g.EmptyRect, Size = g.Size,	PointZero = g.PointZero, Point = g.Point,
	gu = ibm_ilog.diagram.util.GraphUtil,
	m = dojox.gfx.matrix;

var __invisibleLocation = Rect(ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleLocation, ibm_ilog.diagram._invisibleSize, ibm_ilog.diagram._invisibleSize);
	
var a = ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.adorners.Adorner', [ibm_ilog.diagram.adorners.TemplatedOverlay, ibm_ilog.diagram.util.HandleSet], {
	//
	//	summary:
	//		A UI overlay associated with a graph entity.
	//
	//	description:
	//		The Adorner object provides a graphical decoration for a
	//		graph entity. The decoration is composed of a graphical 
	//		representation given by the adorner template and also by a set of smaller 
	//		AdornerHandle entities.
	//		Each adorner usually defines a fixed set of handles which can be used. AdornerHandles 
	//		usually provide an interactive service to the user. For example, the handle 
	//		can connect to the onmousedown event of its shapes to perform some kind of 
	//		action on the adorned. If this action leads to an interaction process that 
	//		is not instantaneous, then the AdornerHandle can push an Interactor object 
	//		into the interactors stack, blocking other conflicting interactors 
	//		temporarily while the action is performed. For example, during the creation 
	//		of links some interactions may be disabled.
	//
	//	construction:
	//		Adorners don't use parameterized constructors. Instead, they rely on an 
	//		initialize method. For example:
	//
	//			new ibm_ilog.diagram.editor.ResizeAdorner().initialize(diagram,
	//		                                                    templatePool,
	//			                                                  "ResizeAdorner0");
	//
	//		The reason for this is that the creation is implemented with a template 
	//		method, and thus the template code implemented in the superclass needs the 
	//		methods already overriden. Currently dojo executes superclass constructors 
	//		before overriding methods redeclared in the subclass. 
	//
	//	TODO:
	//    . update extendCreateShape for using deserialization
	//		. change individual props for composite: elemBounds instead of elemLeft, elemTop, elemW, elemH
	//

	//   
	//	_adorned:
	//		The adorned element
	//
	_adorned: null,

	//   
	//	_changeConnections:
	//		The connections handles of the current connections to change events related to the bound shape
	//
	_changeConnections: [],

	//   
	//	_elementBounds:
	//		The bounding box of the bound element, in Graph coordinates
	//
	_elementBounds: null,

	//   
	//	_viewport:
	//		The viewport at which the Adorner resides
	//
	_viewport: null,

	//   
	//	_templatePool:
	//		The dictionary used for assigning templates to the AdornerHandles the Adorner
	//
	_templatePool: null,

	//   
	//	_diagram:
	//		The diagram at which the Adorner resides
	//
	_diagram: null,

	//   
	_onEGDisposeConn: null,

	initialize: function (diagram, templatePool, templateId) {
		//
		//	summary:
		//		Initializes an Adorner after construction.
		//
		//	diagram:
		//		The Diagram Dijit
		//
		//	templatePool:
		//		The template pool. The adorner gets its template from this pool using the 
		//		templateId argument, and also for instantiating its handles with the 
		//		templates from the pool.
		//
		//	templateId:
		//		the Id for the adorner template in the pool
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._templatePool = templatePool;
		
		this._elementBoundsChanged();
		
        if (dojo.config.isBidiInDojoDiagrammer) {
		    var tDir = diagram._graph.textDir;          
		    if (tDir)                                   
			    this.setTextDir(tDir);
		}

		var template = templatePool.item(templateId);
		this.applyTemplate(template);
		
		//Added to remove the adorner if binded element is removed
		this._onEGDisposeConn = dojo.connect(this._diagram,'beforeGraphElementDispose',this,'_onElementDisposed');
	},

	getAdorned: function () {
		//
		// 	summary:
		//		gets the bounded element
		//
		return this._adorned;
	},

	getViewport: function () {
		//
		// 	summary:
		//		gets the viewport
		//
		return this._viewport;
	},
	
	getDiagram: function () {
		//
		// 	summary:
		//		gets the diagram
		//
		return this._diagram;
	},

	activate: function () {
		//
		// 	summary:
		//		activates the adorner
		//
	},

	deactivate: function () {
		//
		// 	summary:
		//		deactivates the adorner
		//
	},

	_invalidated: 0,
	_invalidationCallbackH: null,
	_invalidationCallback: function() {
		//
		//	summary:
		//		The callback used in the _invalidate() setTimeout. Also called by forceUpdate();
		//
		this._elementBoundsChanged();
		this._invalidationCallbackH = null;
		this._invalidated = 0;
	},
	
	_invalidate: function() {
		//
		// 	summary:
		//		Requests invalidation of the adorner shape. 
		//	description:
		//		Usually based on a change notification. The update is batched with a setTimeout(0).
		//		_invalidate() is used for changes that are reported a lot or those that don't really
		//		need an immediate update.
		//		forceUpdate() can be used to clear the timeout and update immediately.
		//
		if(!this._invalidated) {
			var thiz = this;
			this._invalidationCallbackH = setTimeout(dojo.hitch(this,"_invalidationCallback"),0);
		}
		this._invalidated++;
	},
	
	forceUpdate: function() {
		//
		//	summary:
		//		Used to clear the timeout and update the adorner shape immediately.
		//
		if(this._invalidationCallbackH) {
			clearTimeout(this._invalidationCallbackH);
		}
		this._invalidationCallback();
	},
	
	_reconnectNotifications: function () {
		//
		// 	summary:
		//		Reconnect to change notifications related to the bound element. 
		//

		if (this._adorned && !this._isConnected("v")) {
			this._connect("v", this._viewport, "onViewRectChanged", this,"_updateLayout");
		} else if(!this._adorned) {
			this._disconnect("v");
		}
		var cc = this._changeConnections;
		for (var i = 0; i < cc.length; i++) {
			dojo.disconnect(cc[i]);
		}
		var e = this._adorned;
		if (e) {
			this._changeConnections = cc = [];
			gu.doAncestors(e, function (sg) {
				if (sg._isIBMDiagramSubgraph) {
					var gr = sg.getGraph();
					cc.push(dojo.connect(sg, "onBoundsChanged", this, function (s, reason) {
						this._invalidate();
					}));
					cc.push(dojo.connect(sg, "onCollapsedChanged", this, function () {
						this._invalidate();
					}));
					cc.push(dojo.connect(sg, "onCollapsedAnimated", this, function () {
						this._invalidate();
					}));
					cc.push(dojo.connect(sg, "_setParent", this, function () {
						this._reconnectNotifications();
						this._invalidate();
					}));
				}
			}, this);

			cc.push(dojo.connect(e, "_setParent", this, function () {
				this._reconnectNotifications();
				this._invalidate();
			}));

			// last shape can receive child updates
			cc.push(dojo.connect(e, "onBoundsChanged", this, "forceUpdate"));
		}
	},

	setAdorned: function (element) {
		//
		//	summary:
		//		apply the adorner to the given element
		//
		if (this._adorned !== element) {
			if (this._adorned) {
				this.deactivate();
			}
			this._adorned = element;

			this._reconnectNotifications();
			this._elementBoundsChanged();

			if (this._adorned) {
				this.activate();
			}
		}
	},
	
	_required: function() {
		return this._adorned && gu.isStableVisible(this._adorned); 
	},

	_setFallbackBounds: function() {
		this._elementBounds = null;
		this._elementBaseBounds = null;
	},
	
	_computeElementBoundsChanged: function () {
		if(this._required()) {
			this._elementBounds = this._adorned.getBounds(this._diagram.getGraph().getParent());
			var bs = this._adorned.baseShape;
			if(bs) {
			  var pm = this._diagram.getGraph()._getRealMatrix().inverse();
			  var cm = bs._getRealMatrix() || m.identity;
			  var t = m.multiply(pm,cm);
			  this._elementBaseBounds = t.transformRectangle(bs.getBoundingBox());
			} else {
				this._elementBaseBounds = null;
			}
		} else {
			this._setFallbackBounds();
		}
	},

	_elementBoundsChanged: function () {
		this._computeElementBoundsChanged();
		this._updateLayout();
	},

	_updateLayout: function () {

		var bb = __invisibleLocation, bbb = __invisibleLocation;
		if (this._required()) {
			// There is an adorned, and so this._elementBounds has meaningful data 
			var vt = this._viewport.contentToClient();
			bb = vt.transformRectangle(this._elementBounds);
			bbb = vt.transformRectangle(this._elementBaseBounds || this._elementBounds);
		}
		this.startDTLBatch();
		
		this.setElemWidth(bb.width);
		this.setElemHeight(bb.height);
		this.setElemRight(bb.x + bb.width);
		this.setElemBottom(bb.y + bb.height);
		this.setElemLeft(bb.x);
		this.setElemTop(bb.y);
		this.setElemCenterX(bb.width / 2);
		this.setElemCenterY(bb.height / 2);
		this.setElemBaseWidth(bbb.width);
		this.setElemBaseHeight(bbb.height);
		this.setElemBaseRight(bbb.x + bbb.width);
		this.setElemBaseBottom(bbb.y + bbb.height);
		this.setElemBaseLeft(bbb.x);
		this.setElemBaseTop(bbb.y);
		this.setElemBaseCenterX(bbb.width / 2);
		this.setElemBaseCenterY(bbb.height / 2);

		this.endDTLBatch();
	},
	_onElementDisposed: function(graphElement){
		// Summary:
		//		this method is called when a graph element is 
		if(this._adorned && graphElement===this._adorned){
			this.setAdorned(null);
		}
	},
	
	destroy: function(){
		// summary:
		//		removes the diagram connection
		dojo.disconnect(this._onEGDisposeConn);
	},

	// ==============================================================
	//	These are copied from ibm_ilog.diagram.Node to support owning Ports/Links
	//
	getBounds: function(/*dojox.gfx.Group*/parent) { 
	  // summary:
	  //		Returns the bounds of this node in the coordinate space of the specified parent.
	  // parent: dojox.gfx.Group
	  //		The parent object that determines the coordinate space
	  //		the bounds are relative to. If null or undefined, the bounds of the node
	  //		relative to its parent are returned.
	  // returns:
	  //		A rectangle (i.e. an object with x/y/width/height properties) containing
	  //		the bounds of the node in the specified coordinate space.
	
		/*
		  var rect = this.getLocalBoundingBox();
		  if (rect && parent && parent != this.getParent()) {
		      var t = this.getTransformToContainer(parent);
		      rect = t.transformRectangle(rect);
		  }
		  */
		
		// Changed by Eric: the previous implementation seems wrong, it returns {x: ibm_ilog.diagram._invisibleLocation, y: ibm_ilog.diagram._invisibleLocation, etc }...
		// (looks like the _invisibleLocation thing?...)
		// The following, using the bounds of the baseShape, works better.
		// Abel, please check that this will always work...
			
		if(!parent)
			parent = this.getParent();
		
		var baseShape = this.getBaseShape();
		var rect = baseShape.getBoundingBox();
		var m = dojox.gfx.matrix;
		// t1: transform from surface -> base shape
		var t1 = baseShape._getRealMatrix() || m.identity;
		// t2: transform from surface -> parent
		var t2 = parent._getRealMatrix() || m.identity;
		var t = m.multiply(t2.inverse(), t1);
		rect = t.transformRectangle(rect);
		
		return rect;
	},
	
	getBaseShape: function() {
		return this.baseShape;
	},
	
	onChanged: function() {
		
	},
	
	_onChanged: function() {
		this.onChanged();
	},

__eod:undefined }));

ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseLeft", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseTop", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseBottom", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseRight", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseWidth", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseHeight", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseCenterX", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBaseCenterY", null);

ibm_ilog.diagram.templating.declareBindableProperty(a, "elemLeft", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemTop", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemBottom", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemRight", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemWidth", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemHeight", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemCenterX", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "elemCenterY", null);
ibm_ilog.diagram.templating.declareBindableProperty(a, "textDir", null);      

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.AdornerWithHandles']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.AdornerWithHandles'] = true;
dojo.provide('ibm_ilog.diagram.adorners.AdornerWithHandles');


(function () {

var sup = ibm_ilog.diagram.adorners.Adorner.prototype;

var a = ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.adorners.AdornerWithHandles', [ibm_ilog.diagram.adorners.Adorner], {

	//   
	//	_handles:
	//		The AdornerHandles created for the Adorner
	//
	_handles: null,
	
	initialize: function () {
		
		this._handles = new dojox.collections.ArrayList();
		sup.initialize.apply(this,arguments);
	},

	activate: function () {
		//
		// 	summary:
		//		activates all the adorner handles
		//
		this._handles.forEach(function (e) {
			e.activate();
		});
	},

	deactivate: function () {
		//
		// 	summary:
		//		deactivates all the adorner handles
		//
		this._handles.forEach(function (e) {
			e.deactivate();
		});
	},

	_activateHandle: function (attachPointName) {
		if (this[attachPointName] !== undefined) {
			this[attachPointName].activate();
		}
	},

	_deactivateHandle: function (attachPointName) {
		if (this[attachPointName] !== undefined) {
			this[attachPointName].deactivate();
		}
	},

	_createHandle: function (clazz, attachPointName) {
		if (this[attachPointName]) {
			var template = this._templatePool.item(this[attachPointName].template || clazz.templateId);
			var handle = this[attachPointName].createObject(clazz);
			handle.initialize(this, template);
			this[attachPointName] = handle;
			this._handles.add(handle);
			return handle;
		} else {
			return null;
		}
	},

__eod:undefined }));

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.LinkAdorner']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.LinkAdorner'] = true;
dojo.provide('ibm_ilog.diagram.adorners.LinkAdorner');




(function () {
var R = ibm_ilog.diagram.util.ErrorReporter;
var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
var gu = ibm_ilog.diagram.util.GraphUtil;

var par = ibm_ilog.diagram.adorners.Adorner.prototype;

ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.adorners.LinkAdorner', [ibm_ilog.diagram.adorners.AdornerWithHandles], {
	//
	//	summary:
	//		A UI overlay associated with a link graph entity.
	//
	//	description:
	//		The LinkAdorner object provides a graphical decoration for a
	//		link graph entity. See Adorner for further
	//
	_elementPath: null,
	_echoConnections: null,
	
	initialize: function (diagram, templatePool, templateId) {
		// Summary:
		//		initialize the adorner. Adds some mouse listeners to mouseup and mousedownto push the events to the links
		this.inherited(arguments);
		this._echoConnect();
	},
	
	_buildEchoConnection: function(eventName){
		try {
			this._echoConnections.push(this._path.connect(eventName, this, 'echoEvent'));
		} catch(e) {
			R.warn("EventConnectionFailed",eventName);
		}
	},
	
	_echoConnect: function(){
		if(this._echoConnections){
			dojo.forEach(this._echoConnections,function(conn){dojo.disconnect(conn);});
		}
		this._echoConnections = [];
		if (this['_path']) {
			this._buildEchoConnection('onmousedown');
			this._buildEchoConnection('onmouseup');
			this._buildEchoConnection('onclick');
			this._buildEchoConnection('oncontextmenu');
		}
	},
	
	echoEvent: function (e) {
		// Summary:
		//		Push the recieved event to the binded link
		 if (e.initMouseEvent) {        // Firefox, Google Chrome, Safari, Opera
			  var evt = document.createEvent('MouseEvents');
			  evt.initMouseEvent(e.type, e.bubbles, e.cancelable, e.view, e.detail, e.screenX, e.screenY, e.clientX, e.clientY, e.ctrlKey, e.altKey, e.shiftKey, e.metaKey, e.button, e.relatedTarget);
			  this.getAdorned().rawNode.dispatchEvent(evt);
		 }else{
			  if (document.createEventObject) {        //Internet Explorer
                  var mouseEvent = document.createEventObject (e);
                  mouseEvent.button = e.button;
                  this.getAdorned().rawNode.fireEvent('on'+e.type,mouseEvent);
              }
		}
		dojo.stopEvent(e);
	},
	
	_reconnectNotifications: function () {
		//
		// 	summary:
		//		Reconnect to change notifications related to the bound element. 
		//
		this.inherited(arguments);
		var e = this._adorned;
		if (e) {
			this._changeConnections.push(dojo.connect(e, "_updatePath", this, "_elementBoundsChanged"));
		}
	},

	_setFallbackBounds: function() {
		this._elementPath = [{
			x: ibm_ilog.diagram._invisibleLocation,
			y: ibm_ilog.diagram._invisibleLocation
		}, {
			x: ibm_ilog.diagram._invisibleLocation,
			y: ibm_ilog.diagram._invisibleLocation
		}]; // by default construct adorner out of sight 
		par._setFallbackBounds.apply(this,arguments);
	},
	
	_computeElementBoundsChanged: function () {
		//
		// 	summary:
		//		gets the Element Path from the bounded element
		//
		if (this._required()) {
			this._elementPath = this._adorned._pathPoints;
		}
		par._computeElementBoundsChanged.apply(this,arguments);
	},

	_updateLayout: function () {
		
		this.startDTLBatch();
		this.inherited(arguments);
		var path = dojo.clone(this._elementPath);
		if (this._required()) {
			var v = this._viewport,
				z = v.getZoom(),
				vr = v.getViewRect();
			var t = this._adorned.getShapeToContainerTransform(this._diagram.getGraph().getParent());

			path[0] = t.transformPoint(path[0]);
			path[0] = g.moveRect(path[0], g.negPoint(vr));
			path[0].x = path[0].x * z;
			path[0].y = path[0].y * z;

			path[path.length - 1] = t.transformPoint(path[path.length - 1]);
			path[path.length - 1] = g.moveRect(path[path.length - 1], g.negPoint(vr));
			path[path.length - 1].x = path[path.length - 1].x * z;
			path[path.length - 1].y = path[path.length - 1].y * z;
		}

		this.setLinkStartX(path[0].x);
		this.setLinkStartY(path[0].y);
		this.setLinkEndX(path[path.length - 1].x);
		this.setLinkEndY(path[path.length - 1].y);
		this.endDTLBatch();

		//update path
		if (this['_path'] != undefined) {
			if (this.getAdorned() != null) {
				var transformationFunction = dojo.hitch(this, function (point, list) {
					var v = this._viewport,
						z = v.getZoom(),
						vr = v.getViewRect();
					var t = this._adorned.getShapeToContainerTransform(v._diagram.getGraph().getParent());
					point = t.transformPoint(point);
					point = g.moveRect(point, g.negPoint(vr));
					point.x = point.x * z;
					point.y = point.y * z;
					return point;
				});
				var curveTension = this.getAdorned().getCurveTension();
				this._updatePathShape(curveTension);
				var points = dojo.map(this.getAdorned()._pathPoints,transformationFunction);
				this.getAdorned().applyPath(this._path, points,curveTension);
			} else {
				if (this._path instanceof dojox.gfx.Polyline) {
					this._path.setShape(path);
					this._path.bbox = null; // patch for GFX bug...
				} else {
					this._path.setShape("");
				}
			}
		}
	},
	
	_updatePathShape: function(curveTension){
		if (curveTension > 0 && !(this._path instanceof dojox.gfx.Path)) {
			var stroke, selectedStyle, parent;
			if (this._path) {
				stroke = this._path.getStroke();
				parent = this._path.getParent();
				this._path.removeShape();
			}
			this._path = (parent || this).createPath();
			this._echoConnect();
			if(stroke)
				this._path.setStroke(stroke);
		}
	}

}));

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.adorners.LinkAdorner, "linkStartX", null);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.adorners.LinkAdorner, "linkStartY", null);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.adorners.LinkAdorner, "linkEndX", null);
ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.adorners.LinkAdorner, "linkEndY", null);

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.MoveAnnotationInteractor']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.MoveAnnotationInteractor'] = true;
dojo.provide('ibm_ilog.diagram.adorners.MoveAnnotationInteractor');





(function () {
	var g = ibm_ilog.diagram.util.Geometry;
	var m = dojox.gfx.matrix;

dojo.declare("ibm_ilog.diagram.adorners.MoveAnnotationInteractor", [ibm_ilog.diagram.interactors.DragInteractor], {

	initialize: function ( viewport, annotation ) {
		//
		//	summary:
		//		Initializes the interactor to work with the given handle
		//
		this._annotation = annotation;
		this._viewport = viewport;
		this._initialize();
		return this;
	},
	
	_getInitialEventSource: function () {
		return this._annotation;
	},

	_refreshTransform: function () {
		this._transform = this._annotation._adorned._getRealMatrix().inverse();
	},

	activate: function () {
		this.inherited(arguments);
	},

	_dragPreStart: function () {
		this._refreshTransform();
		this._connect("updateTransforms", this._viewport, "onViewRectChanged", this, "_refreshTransform");

		this.inherited(arguments);
	},

	_dragEnd: function () {
		this._disconnect("updateTransforms");
		var dragged = this.hasDragged, action, D;
		this.inherited(arguments);
	},

	_dragMove: function () {
		this.inherited(arguments);
		var t = this._annotation.getTransform();
		t = m.multiply(t,m.translate(this._delta))
		this._annotation.setTransform(t);
	}

});

})();

}

if(!dojo._hasResource['ibm_ilog.diagram.adorners.AnnotationAdorner']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.adorners.AnnotationAdorner'] = true;
dojo.provide('ibm_ilog.diagram.adorners.AnnotationAdorner');






(function() {

	var g = ibm_ilog.diagram.util.Geometry, Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	var gu = ibm_ilog.diagram.util.GraphUtil;

ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.adorners.AnnotationTargetAdorner', [ibm_ilog.diagram.adorners.Adorner], {

	_link: null,
	
	setLink: function(link) {
		this._link = link;
	},
	
	_onChange: function() {
		if(this._link) {
			this._link.revalidateLinkShape(); // immediate
			this._link.invalidateLinkShape(); // .. and failproof
		}
	},

__eod: undefined}));


var sup = ibm_ilog.diagram.adorners.Adorner.prototype;

var a = ibm_ilog.diagram.gfxDeclaration(dojo.declare('ibm_ilog.diagram.adorners.AnnotationAdorner', [ibm_ilog.diagram.adorners.Adorner], {
	//
	//	summary:
	//		An Adorner which displays node annotations
	//
	
	//
	//	annotationChangedEvent:
	//		The event to watch for associated with the annotation changing its value. 
	//
	annotationChangedEvent: "setAnnotation", 
	
	//
	//	annotationGetter:
	//		The method to invoke in the adorned entity to get the annotation. This is used in 
	//		the default implementation of refreshAnnotation. 
	//
	annotationGetter: "getAnnotation",
	
	
	_moveInteractor: null,
	
	initialize: function() {
		this.inherited(arguments);
		this._moveInteractor = new ibm_ilog.diagram.adorners.MoveAnnotationInteractor();
		this._moveInteractor.initialize(this._viewport,this);
		this._moveInteractor.setManager(this._diagram._interactors);
		
	},
	
	_reconnectNotifications: function() {
		//
		//	Add an additional connection to watch for "annotation changed" events.
		//
		ibm_ilog.diagram.adorners.Adorner.prototype._reconnectNotifications.apply(this,arguments);
		var e = this._adorned;
		if (e) {
			this.refreshAnnotation();
			this._changeConnections.push(dojo.connect(e,this.annotationChangedEvent,this,"refreshAnnotation"));
		}
	},
	
	refreshAnnotation: function(){
		//
		//	summary:
		//		Update receiver with a new annotation
		//	description:
		//		Current implementation relies on the template referencing 
		//		the "annotation" bindable property of the receiver.
		//		Can be overriden by subclasses to produce specific 
		//		ways of	rendering the annotation like word wrap, etc.
		//
		this.setAnnotation(this._adorned[this.annotationGetter]());
	},

	activate: function () {
		this._moveInteractor.activate();
	},

	deactivate: function () {
		this._moveInteractor.deactivate();
	},
	
	_link: null,
	_target: null,

	_updateLayout: function () {
		sup._updateLayout.apply(this,arguments);
	},
	
	setLink: function(link,target) {
		this._target = target;
		this._link = link;
	},
	
	getLink: function() {
		return this._link;
	},
	
	getTarget: function() {
		return this._target;
	},
	
	setAdorned: function(adorned) {
		this.inherited(arguments);
		if(this._target) {
			this._target.setAdorned(adorned);
		}
	},
	
	getBaseShape: function() {
		return this.baseShape;
	},
	
	destroy: function() {
		sup.destroy.apply(this,arguments);
	},
	
	_onChanged: function() {
		if(this._link) {
			this._link.revalidateLinkShape(); // immediate
			this._link.invalidateLinkShape(); // .. and failproof
		}
	},
	
__eod: undefined }));

ibm_ilog.diagram.templating.declareBindableProperty(a, "annotation", "");

})();

}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");


dojo.getObject("date.stamp", true, dojo);

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.forEach(dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
};

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") +
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
};

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");




new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary:
	//		The Dom/Widget parsing package

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".	 interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }").
					//	Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var dummyClass = {}, instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	// TODO: remove this in 2.0, when we stop caching parameters.
	d.connect(d, "extend", function(){
		instanceClasses = {};
	});

	function getProtoInfo(cls, params){
		// cls: A prototype
		//		The prototype of the class to check props on
		// params: Object
		//		The parameters object to mix found parameters onto.
		for(var name in cls){
			if(name.charAt(0)=="_"){ continue; }	// skip internal properties
			if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
			params[name] = val2type(cls[name]);
		}
		return params;
	}

	function getClassInfo(/*String*/ className, /*Boolean*/ skipParamsLookup){
		// summary:
		//		Maps a widget name string like "dijit.form.Button" to the widget constructor itself,
		//		and a list of that widget's parameters and their types
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{
		//				cls: dijit.Button,
		//				params: { label: "string", disabled: "boolean"}
		//			}

		var c = instanceClasses[className];
		if(!c){
			// get pointer to widget class
			var cls = d.getObject(className), params = null;
			if(!cls){ return null; }		// class not defined [yet]
			if(!skipParamsLookup){ // from fastpath, we don't need to lookup the attrs on the proto because they are explicit
				params = getProtoInfo(cls.prototype, {})
			}
			c = { cls: cls, params: params };
			
		}else if(!skipParamsLookup && !c.params){
			// if we're calling getClassInfo and have a cls proto, but no params info, scan that cls for params now
			// and update the pointer in instanceClasses[className]. This happens when a widget appears in another
			// widget's template which still uses dojoType, but an instance of the widget appears prior with a data-dojo-type,
			// skipping this lookup the first time.
			c.params = getProtoInfo(c.cls.prototype, {});
		}
		
		return c;
	}

	this._functionFromScript = function(script, attrData){
		// summary:
		//		Convert a <script type="dojo/method" args="a, b, c"> ... </script>
		//		into a function
		// script: DOMNode
		//		The <script> DOMNode
		// attrData: String
		//		For HTML5 compliance, searches for attrData + "args" (typically
		//		"data-dojo-args") instead of "args"
		var preamble = "";
		var suffix = "";
		var argsStr = (script.getAttribute(attrData + "args") || script.getAttribute("args"));
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	};

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		See parse.args argument for details.

		var thelist = [],
		mixin = mixin||{};
		args = args||{};

		// TODO: for 2.0 default to data-dojo- regardless of scopeName (or maybe scopeName won't exist in 2.0)
		var attrName = (args.scope || d._scopeName) + "Type",	// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.
			var node, type, clsInfo, clazz, scripts, fastpath;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				fastpath = obj.fastpath;
				clsInfo = obj.clsInfo || (type && getClassInfo(type, fastpath));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes. no fastpath/data-dojo-type support here.
				node = obj;
				type = attrName in mixin ? mixin[attrName] : node.getAttribute(attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] :
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.	Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {};
				
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				d._mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				d._mixin(params, obj.inherited);
			}
			
			// mix things found in data-dojo-props into the params
			if(fastpath){
				var extra = node.getAttribute(attrData + "props");
				if(extra && extra.length){
					try{
						extra = d.fromJson.call(args.propsThis, "{" + extra + "}");
						d._mixin(params, extra);
					}catch(e){
						// give the user a pointer to their invalid parameters. FIXME: can we kill this in production?
						throw new Error(e.toString() + " in data-dojo-props='" + extra + "'");
					}
				}

				// For the benefit of _Templated, check if node has data-dojo-attach-point/data-dojo-attach-event
				// and mix those in as though they were parameters
				var attachPoint = node.getAttribute(attrData + "attach-point");
				if(attachPoint){
					params.dojoAttachPoint = attachPoint;
				}
				var attachEvent = node.getAttribute(attrData + "attach-event");
				if(attachEvent){
					params.dojoAttachEvent = attachEvent;
				}
				dojo.mixin(params, mixin);
			}else{
				// FIXME: we need something like "deprecateOnce()" to throw dojo.deprecation for something.
				// remove this logic in 2.0
				// read parameters (ie, attributes) specified on DOMNode

				var attributes = node.attributes;

				// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
				for(var name in clsInfo.params){
					var item = name in mixin ? { value:mixin[name], specified:true } : attributes.getNamedItem(name);
					if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
					var value = item.value;
					// Deal with IE quirks for 'class' and 'style'
					switch(name){
					case "class":
						value = "className" in mixin ? mixin.className : node.className;
						break;
					case "style":
						value = "style" in mixin ? mixin.style : (node.style && node.style.cssText); // FIXME: Opera?
					}
					var _type = clsInfo.params[name];
					if(typeof value == "string"){
						params[name] = str2obj(value, _type);
					}else{
						params[name] = value;
					}
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				// FIXME: drop event="" support in 2.0. use data-dojo-event="" instead
				var event = (script.getAttribute(attrData + "event") || script.getAttribute("event")),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script, attrData);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			// FIXME: in 2.0, drop jsId support. use data-dojo-id instead
			var jsname = (node.getAttribute(attrData + "id") || node.getAttribute("jsId"));
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if( !args.noStart && instance  &&
					dojo.isFunction(instance.startup) &&
					!instance._started &&
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(rootNode, args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them. Searches for either data-dojo-type="Class" or
		//		dojoType="Class" where "Class" is a a fully qualified class name,
		//		like `dijit.form.Button`
		//
		//		Using `data-dojo-type`:
		//		Attributes using can be mixed into the parameters used to instantitate the
		//		Class by using a `data-dojo-props` attribute on the node being converted.
		//		`data-dojo-props` should be a string attribute to be converted from JSON.
		//
		//		Using `dojoType`:
		//		Attributes are read from the original domNode and converted to appropriate
		//		types by looking up the Class prototype values. This is the default behavior
		//		from Dojo 1.0 to Dojo 1.5. `dojoType` support is deprecated, and will
		//		go away in Dojo 2.0.
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a
		//		`rootNode` member, that is used.
		//
		// args: Object
		//		a kwArgs object passed along to instantiate()
		//
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes.
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object.
		//			* template: Boolean
		//				If true, ignores ContentPane's stopParser flag and parses contents inside of
		//				a ContentPane inside of a template.   This allows dojoAttachPoint on widgets/nodes
		//				nested inside the ContentPane to work.
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//			* scope: String
		//				Root for attribute names to search for.   If scopeName is dojo,
		//				will search for data-dojo-type (or dojoType).   For backwards compatibility
		//				reasons defaults to dojo._scopeName (which is "dojo" except when
		//				multi-version support is used, when it will be something like dojo16, dojo20, etc.)
		//			* propsThis: Object
		//				If specified, "this" referenced from data-dojo-props will refer to propsThis.
		//				Intended for use from the widgets-in-template feature of `dijit._Templated`
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId('foo'));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		//	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}
		root = root ? dojo.byId(root) : dojo.body();
		args = args || {};

		var attrName = (args.scope || d._scopeName) + "Type",		// typically "dojoType"
			attrData = "data-" + (args.scope || d._scopeName) + "-";	// typically "data-dojo-"

		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode,			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				// TODO: what if this is a widget and dir/lang are declared in data-dojo-props?
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = (!parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser) || (args && args.template);

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					// FIXME: desupport dojoType in 2.0. use data-dojo-type instead
					var type, html5 = recurse && child.getAttribute(attrData + "type");
					if(html5){
						type = html5;
					}else{
						// fallback to backward compatible mode, using dojoType. remove in 2.0
						type = recurse && child.getAttribute(attrName);
					}
					
					var fastpath = html5 == type;

					if(type){
						// if dojoType/data-dojo-type specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							fastpath: fastpath,
							clsInfo: getClassInfo(type, fastpath), // note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Ignore bogus entries in inherited hash like {dir: ""}
		var inherited = {};
		if(args && args.inherited){
			for(var key in args.inherited){
				if(args.inherited[key]){ inherited[key] = args.inherited[key]; }
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root,
			inherited: inherited
		}, list);

		// go build the object instances
		var mixin = args && args.template ? {template: true} : null;
		return this.instantiate(list, mixin, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){
		if(dojo.config.parseOnLoad){
			dojo.parser.parse();
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.getObject("dijit.wai.onload") === dojo._loaders[0]){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");


dojo.getObject("string", true, dojo);

/*=====
dojo.string = {
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	//	summary:
	//		Efficiently replicate a string `n` times.
	//	str:
	//		the string to replicate
	//	num:
	//		number of times to replicate the string
	
	if(num <= 0 || !str){ return ""; }
	
	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	//	summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	//	text:
	//		the string to pad
	//	size:
	//		length to provide padding
	//	ch:
	//		character to pad, defaults to '0'
	//	end:
	//		adds padding at the end if true, otherwise pads at start
	//	example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template,
									/*Object|Array*/map,
									/*Function?*/	transform,
									/*Object?*/		thisObject){
	//	summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	//	template:
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive.
	//	map:
	//		hash to search for substitutions
	//	transform:
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	//	thisObject:
	//		where to look for optional format function; default to the global
	//		namespace
	//	example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	//	example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	//	example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ?
		dojo.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = dojo.getObject(key, false, map);
			if(format){
				value = dojo.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	//	summary:
	//		Trims whitespace from both sides of the string
	//	str: String
	//		String to be trimmed
	//	returns: String
	//		Returns the trimmed string
	//	description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	dojo.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");



/*=====
dojo.cache = {
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary:
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");







dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

/*=====
		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
 		_attachPoints: [],
 =====*/

/*=====
		// _attachEvents: [private] Handle[]
		//		List of connections associated with dojoAttachEvent=... in the
		//		template
 		_attachEvents: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
			this._attachEvents = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// Call down to _Widget.buildRendering() to get base classes assigned
			// TODO: change the baseClass assignment to attributeMap
			this.inherited(arguments);

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					template: true,
					inherited: {dir: this.dir, lang: this.lang},
					propsThis: this,	// so data-dojo-props of widgets in the template can reference "this" to refer to me
					scope: "dojo"	// even in multi-version mode templates use dojoType/data-dojo-type
				}));

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			//		Alternately, if rootNode is an array of widgets, then will process dojoAttachPoint
			//		etc. for those widgets.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && (getAttrFunc(baseNode, "dojoType") || getAttrFunc(baseNode, "data-dojo-type"))){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint") || getAttrFunc(baseNode, "data-dojo-attach-point");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent") || getAttrFunc(baseNode, "data-dojo-attach-event");;
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this._attachEvents.push(this.connect(baseNode, event, thisFunc));
						}
					}
				}

				// waiRole, waiState
				// TODO: remove this in 2.0, templates are now using role=... and aria-XXX=... attributes directicly
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			// And same for event handlers
			dojo.forEach(this._attachEvents, this.disconnect, this);
			this._attachEvents = [];
			
			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");





dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: dijit.defaultDuration,

		templateString: dojo.cache("dijit", "templates/Tooltip.html", "<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\"\r\n\t><div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" role='alert'></div\r\n\t><div class=\"dijitTooltipConnector\" dojoAttachPoint=\"connectorNode\"></div\r\n></div>\r\n"),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });
		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			// reset width; it may have been set by orient() on a previous tooltip show()
			this.domNode.width = "auto";

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, dijit.getPopupAroundAlignment((position && position.length) ? position : dijit.Tooltip.defaultPosition, !rtl), dojo.hitch(this, "orient"));

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ tooltipCorner, /*Object*/ spaceAvailable, /*Object*/ aroundNodeCoords){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.   It will also reduce the tooltip's
			//		width to whatever width is available
			// tags:
			//		protected
			this.connectorNode.style.top = ""; //reset to default
			
			//Adjust the spaceAvailable width, without changing the spaceAvailable object
			var tooltipSpaceAvaliableWidth = spaceAvailable.w - this.connectorNode.offsetWidth;

			node.className = "dijitTooltip " +
				{
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];
				
			// reduce tooltip's width to the amount of width available, so that it doesn't overflow screen
			this.domNode.style.width = "auto";
			var size = dojo.contentBox(this.domNode);
			
			var width = Math.min((Math.max(tooltipSpaceAvaliableWidth,1)), size.w);
			var widthWasReduced = width < size.w;
			
			this.domNode.style.width = width+"px";
						
			//Adjust width for tooltips that have a really long word or a nowrap setting
			if(widthWasReduced){
				this.containerNode.style.overflow = "auto"; //temp change to overflow to detect if our tooltip needs to be wider to support the content
				var scrollWidth = this.containerNode.scrollWidth;
				this.containerNode.style.overflow = "visible"; //change it back
				if(scrollWidth > width){
					scrollWidth = scrollWidth + dojo.style(this.domNode,"paddingLeft") + dojo.style(this.domNode,"paddingRight");
					this.domNode.style.width = scrollWidth + "px";
				}
			}
			
			// Reposition the tooltip connector.
			if(tooltipCorner.charAt(0) == 'B' && aroundCorner.charAt(0) == 'B'){
				var mb = dojo.marginBox(node);
				var tooltipConnectorHeight = this.connectorNode.offsetHeight;
				if(mb.h > spaceAvailable.h){
					// The tooltip starts at the top of the page and will extend past the aroundNode
					var aroundNodePlacement = spaceAvailable.h - (aroundNodeCoords.h / 2) - (tooltipConnectorHeight / 2);
					this.connectorNode.style.top = aroundNodePlacement + "px";
					this.connectorNode.style.bottom = "";
				}else{
					// Align center of connector with center of aroundNode, except don't let bottom
					// of connector extend below bottom of tooltip content, or top of connector
					// extend past top of tooltip content
					this.connectorNode.style.bottom = Math.min(
						Math.max(aroundNodeCoords.h/2 - tooltipConnectorHeight/2, 0),
						mb.h - tooltipConnectorHeight) + "px";
					this.connectorNode.style.top = "";
				}
			}else{
				// reset the tooltip back to the defaults
				this.connectorNode.style.top = "";
				this.connectorNode.style.bottom = "";
			}
			
			return Math.max(0, size.w - tooltipSpaceAvaliableWidth);
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip

			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
	// summary:
	//		Display tooltip w/specified contents in specified position.
	//		See description of dijit.Tooltip.defaultPosition for details on position parameter.
	//		If position is not specified then dijit.Tooltip.defaultPosition is used.
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode, position, rtl);
};

dijit.hideTooltip = function(aroundNode){
	// summary:
	//		Hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String|String[]
		//		Id of domNode(s) to attach the tooltip to.
		//		When user hovers over specified dom node, the tooltip will appear.
		connectId: [],

		// position: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on position parameter.
		position: [],

		_setConnectIdAttr: function(/*String*/ newId){
			// summary:
			//		Connect to node(s) (specified by id)

			// Remove connections to old nodes (if there are any)
			dojo.forEach(this._connections || [], function(nested){
				dojo.forEach(nested, dojo.hitch(this, "disconnect"));
			}, this);

			// Make connections to nodes in newIds.
			var ary = dojo.isArrayLike(newId) ? newId : (newId ? [newId] : []);
			this._connections = dojo.map(ary, function(id){
				var node = dojo.byId(id);
				return node ? [
					this.connect(node, "onmouseenter", "_onTargetMouseEnter"),
					this.connect(node, "onmouseleave", "_onTargetMouseLeave"),
					this.connect(node, "onfocus", "_onTargetFocus"),
					this.connect(node, "onblur", "_onTargetBlur")
				] : [];
			}, this);
	
			this._set("connectId", newId);

			this._connectIds = ary;	// save as array
		},

		addTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Attach tooltip to specified node if it's not already connected

			// TODO: remove in 2.0 and just use set("connectId", ...) interface

			var id = node.id || node;
			if(dojo.indexOf(this._connectIds, id) == -1){
				this.set("connectId", this._connectIds.concat(id));
			}
		},

		removeTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// TODO: remove in 2.0 and just use set("connectId", ...) interface
			
			var id = node.id || node,	// map from DOMNode back to plain id string
				idx = dojo.indexOf(this._connectIds, id);
			if(idx >= 0){
				// remove id (modifies original this._connectIds but that's OK in this case)
				this._connectIds.splice(idx, 1);
				this.set("connectId", this._connectIds);
			}
		},

		buildRendering: function(){
			this.inherited(arguments);
			dojo.addClass(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			dojo.forEach(dojo.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		_onTargetMouseEnter: function(/*Event*/ e){
			// summary:
			//		Handler for mouseenter event on the target node
			// tags:
			//		private
			this._onHover(e);
		},

		_onTargetMouseLeave: function(/*Event*/ e){
			// summary:
			//		Handler for mouseleave event on the target node
			// tags:
			//		private
			this._onUnHover(e);
		},

		_onTargetFocus: function(/*Event*/ e){
			// summary:
			//		Handler for focus event on the target node
			// tags:
			//		private

			this._focus = true;
			this._onHover(e);
		},

		_onTargetBlur: function(/*Event*/ e){
			// summary:
			//		Handler for blur event on the target node
			// tags:
			//		private

			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			// keep a tooltip open if the associated element still has focus (even though the
			// mouse moved away)
			if(this._focus){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target, this.position, !this.isLeftToRight());

			this._connectNode = target;
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				dijit.hideTooltip(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		onShow: function(target, position){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		uninitialize: function(){
			this.close();
			this.inherited(arguments);
		}
	}
);

// dijit.Tooltip.defaultPosition: String[]
//		This variable controls the position of tooltips, if the position is not specified to
//		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the following values:
//
//			* before: places tooltip to the left of the target node/widget, or to the right in
//			  the case of RTL scripts like Hebrew and Arabic
//			* after: places tooltip to the right of the target node/widget, or to the left in
//			  the case of RTL scripts like Hebrew and Arabic
//			* above: tooltip goes above target node
//			* below: tooltip goes below target node
//
//		The list is positions is tried, in order, until a position is found where the tooltip fits
//		within the viewport.
//
//		Be careful setting this parameter.  A value of "above" may work fine until the user scrolls
//		the screen so that there's no room above the target node.   Nodes with drop downs, like
//		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
//		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
//		is only room below (or above) the target node, but not both.
dijit.Tooltip.defaultPosition = ["after", "before"];

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");


dojo.getObject("regexp", true, dojo);

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
};

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression.
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
};

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression.
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
};

}

if(!dojo._hasResource["dojo.number"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.number"] = true;
dojo.provide("dojo.number");






dojo.getObject("number", true, dojo);

/*=====
dojo.number = {
	// summary: localized formatting and parsing routines for Number
}

dojo.number.__FormatOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means do not round.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	fractional: Boolean?
	//		If false, show no decimal places, overriding places and pattern settings.
	this.pattern = pattern;
	this.type = type;
	this.places = places;
	this.round = round;
	this.locale = locale;
	this.fractional = fractional;
}
=====*/

dojo.number.format = function(/*Number*/value, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		[Common Locale Data Repository](http://unicode.org/cldr) as well as the appropriate symbols and
	//		delimiters.
	//		If value is Infinity, -Infinity, or is not a valid JavaScript number, return null.
	// value:
	//		the number to be formatted

	options = dojo.mixin({}, options || {});
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value) || Math.abs(value) == Infinity){ return null; } // null
	return dojo.number._applyPattern(value, pattern, options); // String
};

//dojo.number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
dojo.number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

dojo.number._applyPattern = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatOptions?*/options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	// options: dojo.number.__FormatOptions?
	//		_applyPattern is usually called via `dojo.number.format()` which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group,
		decimal = options.customs.decimal,
		patternList = pattern.split(';'),
		positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}
	
	//TODO: support @ sig figs?
	var numberPatternRE = dojo.number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	if(options.fractional === false){ options.places = 0; }
	return pattern.replace(numberPatternRE,
		dojo.number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places, round: options.round}));
};

dojo.number.round = function(/*Number*/value, /*Number?*/places, /*Number?*/increment){
	//	summary:
	//		Rounds to the nearest value with the given number of decimal places, away from zero
	//	description:
	//		Rounds to the nearest value with the given number of decimal places, away from zero if equal.
	//		Similar to Number.toFixed(), but compensates for browser quirks. Rounding can be done by
	//		fractional increments also, such as the nearest quarter.
	//		NOTE: Subject to floating point errors.  See dojox.math.round for experimental workaround.
	//	value:
	//		The number to round
	//	places:
	//		The number of decimal places where rounding takes place.  Defaults to 0 for whole rounding.
	//		Must be non-negative.
	//	increment:
	//		Rounds next place to nearest value of increment/10.  10 by default.
	//	example:
	//		>>> dojo.number.round(-0.5)
	//		-1
	//		>>> dojo.number.round(162.295, 2)
	//		162.29  // note floating point error.  Should be 162.3
	//		>>> dojo.number.round(10.71, 0, 2.5)
	//		10.75
	var factor = 10 / (increment || 10);
	return (factor * +value).toFixed(places) / factor; // Number
};

if((0.9).toFixed() == 0){
	// (isIE) toFixed() bug workaround: Rounding fails on IE when most significant digit
	// is just after the rounding place and is >=5
	(function(){
		var round = dojo.number.round;
		dojo.number.round = function(v, p, m){
			var d = Math.pow(10, -p || 0), a = Math.abs(v);
			if(!v || a >= d || a * Math.pow(10, p + 1) < 5){
				d = 0;
			}
			return round(v, p, m) + (v > 0 ? d : -d);
		};
	})();
}

/*=====
dojo.number.__FormatAbsoluteOptions = function(){
	//	decimal: String?
	//		the decimal separator
	//	group: String?
	//		the group separator
	//	places: Number?|String?
	//		number of decimal places.  the range "n,m" will format to m places.
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
	this.decimal = decimal;
	this.group = group;
	this.places = places;
	this.round = round;
}
=====*/

dojo.number._formatAbsolute = function(/*Number*/value, /*String*/pattern, /*dojo.number.__FormatAbsoluteOptions?*/options){
	// summary:
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. `#,##0.00`)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split("."),
		comma = typeof options.places == "string" && options.places.indexOf(","),
		maxPlaces = options.places;
	if(comma){
		maxPlaces = options.places.substring(comma + 1);
	}else if(!(maxPlaces >= 0)){
		maxPlaces = (patternParts[1] || []).length;
	}
	if(!(options.round < 0)){
		value = dojo.number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split("."),
		fractional = valueParts[1] || "";
	if(patternParts[1] || options.places){
		if(comma){
			options.places = options.places.substring(0, comma);
		}
		// Pad fractional with trailing zeros
		var pad = options.places !== undefined ? options.places : (patternParts[1] && patternParts[1].lastIndexOf("0") + 1);
		if(pad > fractional.length){
			valueParts[1] = dojo.string.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		if(maxPlaces < fractional.length){
			valueParts[1] = fractional.substr(0, maxPlaces);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dojo.string.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(','),
		groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
dojo.number.__RegexpOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  Defined by pattern or Infinity if pattern not provided.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.places = places;
}
=====*/
dojo.number.regexp = function(/*dojo.number.__RegexpOptions?*/options){
	//	summary:
	//		Builds the regular needed to parse a number
	//	description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return dojo.number._parseInfo(options).regexp; // String
};

dojo.number._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale),
		bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale),
		pattern = options.pattern || bundle[(options.type || "decimal") + "Format"],
//TODO: memoize?
		group = bundle.group,
		decimal = bundle.decimal,
		factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dojo.regexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dojo.regexp.escapeString(pattern, '.')+")";
		return pattern.replace(dojo.number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false
				},

				parts = format.split('.'),
				places = options.places;

			// special condition for percent (factor != 1)
			// allow decimal places even if not specified in pattern
			if(parts.length == 1 && factor != 1){
			    parts[1] = "###";
			}
			if(parts.length == 1 || places === 0){
				flags.fractional = false;
			}else{
				if(places === undefined){ places = options.pattern ? parts[1].lastIndexOf('0') + 1 : Infinity; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length > 1){
				flags.groupSize = groups.pop().length;
				if(groups.length > 1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+dojo.number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/([\s\xa0]*)(\u00a4{1,3})([\s\xa0]*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1],
				symbol = dojo.regexp.escapeString(options[prop] || options.currency || "");
			before = before ? "[\\s\\xa0]" : "";
			after = after ? "[\\s\\xa0]" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
};

/*=====
dojo.number.__ParseOptions = function(){
	//	pattern: String?
	//		override [formatting pattern](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
	//		with this string.  Default value is based on locale.  Overriding this property will defeat
	//		localization.  Literal characters in patterns are not supported.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific (not yet supported), percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default.  Strict parsing requires input as produced by the format() method.
	//		Non-strict is more permissive, e.g. flexible on white space, omitting thousands separators
	//	fractional: Boolean?|Array?
	//		Whether to include the fractional portion, where the number of decimal places are implied by pattern
	//		or explicit 'places' parameter.  The value [true,false] makes the fractional portion optional.
	this.pattern = pattern;
	this.type = type;
	this.locale = locale;
	this.strict = strict;
	this.fractional = fractional;
}
=====*/
dojo.number.parse = function(/*String*/expression, /*dojo.number.__ParseOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale
	//		and follow the syntax described by
	//		[unicode.org TR35](http://www.unicode.org/reports/tr35/#Number_Format_Patterns)
    	//		Note that literal characters in patterns are not supported.
	// expression:
	//		A string representation of a Number
	var info = dojo.number._parseInfo(options),
		results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return absoluteMatch * info.factor; //Number
};

/*=====
dojo.number.__RealNumberRegexpFlags = function(){
	//	places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	//	decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	//	fractional: Boolean?|Array?
	//		Whether decimal places are used.  Can be true, false, or [true,
	//		false].  Default is [true, false] which means optional.
	//	exponent: Boolean?|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	//	eSigned: Boolean?|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
	this.places = places;
	this.decimal = decimal;
	this.fractional = fractional;
	this.exponent = exponent;
	this.eSigned = eSigned;
}
=====*/

dojo.number._realNumberRegexp = function(/*dojo.number.__RealNumberRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation

	// assign default values to missing parameters
	flags = flags || {};
	//TODO: use mixin instead?
	if(!("places" in flags)){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(!("fractional" in flags) || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(!("exponent" in flags)){ flags.exponent = [true, false]; }
	if(!("eSigned" in flags)){ flags.eSigned = [true, false]; }

	var integerRE = dojo.number._integerRegexp(flags),
		decimalRE = dojo.regexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){
					re = "(?:" + re + "\\d+)?";
				}else{
					re += "\\d{" + flags.places + "}";
				}
			}
			return re;
		},
		true
	);

	var exponentRE = dojo.regexp.buildGroupRE(flags.exponent,
		function(q){
			if(q){ return "([eE]" + dojo.number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return "";
		}
	);

	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
dojo.number.__IntegerRegexpFlags = function(){
	//	signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or `[true,false]`.
	//		Default is `[true, false]`, (i.e. will match if it is signed
	//		or unsigned).
	//	separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. `[",", ""]`,
	//		makes ',' optional.
	//	groupSize: Number?
	//		group size between separators
	//	groupSize2: Number?
	//		second grouping, where separators 2..n have a different interval than the first separator (for India)
	this.signed = signed;
	this.separator = separator;
	this.groupSize = groupSize;
	this.groupSize2 = groupSize2;
}
=====*/

dojo.number._integerRegexp = function(/*dojo.number.__IntegerRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression that matches an integer

	// assign default values to missing parameters
	flags = flags || {};
	if(!("signed" in flags)){ flags.signed = [true, false]; }
	if(!("separator" in flags)){
		flags.separator = "";
	}else if(!("groupSize" in flags)){
		flags.groupSize = 3;
	}

	var signRE = dojo.regexp.buildGroupRE(flags.signed,
		function(q){ return q ? "[-+]" : ""; },
		true
	);

	var numberRE = dojo.regexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:\\d+)";
			}

			sep = dojo.regexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			//TODO: should we continue to enforce that numbers with separators begin with 1-9?  See #6933
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	return signRE + numberRE; // String
};

}

if(!dojo._hasResource["ibm_ilog.diagram.widget._Diagram"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm_ilog.diagram.widget._Diagram"] = true;
dojo.provide("ibm_ilog.diagram.widget._Diagram");



















 // for compatibility...


 // for correct rounding

// This class contains the Diagram code that is conditionnally loaded
// only on desktop (non-mobile) platforms.

(function() {
	var gu = ibm_ilog.diagram.util.GraphUtil;
	//var R = ibm_ilog.diagram.util.ErrorReporter;

	var _defaultWaiLabelFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	var _defaultWaiDescriptionFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	// Extender object to add functionality on desktop (i.e., non-mobile) platforms:
	
	ibm_ilog.diagram.widget._Diagram = {
		
		_isDiagramExtended: true,
			
		// =======================================================================
		//
		// INTERACTORS:
		//
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new ibm_ilog.diagram.interactors.PanInteractor().initialize(this));
			this._interactors.add("select", new ibm_ilog.diagram.interactors.SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ibm_ilog.diagram.interactors.ZoomInteractor().initialize(this));
			
			this._interactors.add("move", new ibm_ilog.diagram.interactors.MoveInteractor().initialize(this._selection, this.getViewport(), {
				doMove: this.moveFunction,
				isMovable: this.isMovableFunction
			}));
			this._interactors.add("marquee", s = new ibm_ilog.diagram.interactors.MarqueeSelectionInteractor().initialize(this.getViewport(), this.getSelection()));
			this._interactors.add("wheelPan", new ibm_ilog.diagram.interactors.WheelPanInteractor().initialize(this));
			this._interactors.add("adorners", new ibm_ilog.diagram.interactors.AdornerFocusInteractor().initialize(this));
			this._interactors.add("focus", new ibm_ilog.diagram.interactors.FocusInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus"]);
			
			dojo.connect(this._graph, "onLayoutAnimationStart", this, function() {
				// TODO focusedElement also
				this.setNodeAdornersFocus(null);
				if (this._getNodeAdornersFocusAttr()) {
					this._setNodeAdornersFocusAttr(false);
					
					var h = dojo.connect(this._graph, "onLayoutAnimationEnd", this, function() {
						dojo.disconnect(h);
						this._setNodeAdornersFocusAttr(true);
					});
				}
			});
		},
		
		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus");
		},	
		
		//
		//  move: [markup] boolean
		//      enable or disable moving entities
		//
		move: false,
		
		_setMoveAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMoveAttr();
			if (old !== b) {
				this.move = b;
				this._interactors.get("move").setEnabled(b);
				this._firePropertyChanged('move', old, b);
			}
			return this;
		},
		
		_getMoveAttr: function() {
			return this._viewportReady ? this._interactors.get("move").getEnabled() : false;
		},
		
		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		_keyInteractors: null,
		
		_initializeKeyInteractors: function() {
			//
			//	tags:
			//		private
			//
			this._keyInteractors = new ibm_ilog.diagram.interactors.KeyboardInteractorManager(this);
			
			this._keyInteractors.add(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId, new ibm_ilog.diagram.interactors.Navigator().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.SelectionKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.SelectionKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.PanKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.PanKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.ZoomKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.ZoomKeyInteractor().initialize(this));
			this._keyInteractors.add(ibm_ilog.diagram.interactors.MoveKeyInteractor.KeyInteractorId, new ibm_ilog.diagram.interactors.MoveKeyInteractor().initialize(this));
			var edition = new ibm_ilog.diagram.interactors.SwitchModeInteractor();
			this._keyInteractors.add(ibm_ilog.diagram.interactors.SwitchModeInteractor.KeyInteractorId, edition.initialize(this));
			edition.enterNavigation();
			this._keyInteractors.activate();
		},
		
		getKeyInteractor: function(id) {
			return this._keyInteractors.get(id);
		},
	
		assignKeyInteractorInputs: function(iid, assignments) {
			this.getKeyInteractor(iid).assignInputs(assignments);
		},
		
		getKeyboardEventSource: function() {
			var wai = this._getWaiNode(this._graph);
			return wai || this.focusNode || this.domNode;
		},
		
		// =======================================================================
		//
		//	Adorners
		//
	
		_initializeAdorners: function() {
			var v = this.getViewport();
			
			this._adornerTemplates = ibm_ilog.diagram.templates.defaultAdornerTemplates.clone();
			
			v.whenLoaded(dojo.hitch(this, function() {
				this._adornerLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		
		_createAdorners: function() {
			this._focusedNodeAdorner = this.createAdorner(ibm_ilog.diagram.adorners.Adorner, this._adornerTemplates, "NodeFocusAdorner");
			
			this._focusedLinkAdorner = this.createAdorner(ibm_ilog.diagram.adorners.LinkAdorner, this._adornerTemplates, "LinkFocusAdorner");
	
		},
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		tabIndex: "-1",
		
		//
		//	disableGraphWai: boolean
		//		Disables the infrastructure used for assigning WAI roles. (Initialization-only attribute).
		//        
		disableGraphWai: false,
		
		//
		//	enableGraphWaiDescribedBy: boolean
		//		Used to enable description for wai structure
		//
		enableGraphWaiDescribedBy: false,
		
		//
		//	waiNodeLabelFunction: Function?
		//		This function is used to compute the WAI label text for a node.
		//
		waiNodeLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiLinkLabelFunction: Function?
		//		This function is used to compute the WAI label text for a link.
		//
		waiLinkLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiSubgraphLabelFunction: Function?
		//		This function is used to compute the WAI label text for a subgraph.
		//
		waiSubgraphLabelFunction: _defaultWaiLabelFunction,
		
		//	waiGraphLabelFunction: Function?
		//		This function is used to compute the WAI label for the graph.
		//
		waiGraphLabelFunction: function(graph) {
			return "Graph";
		},
		
		
		
		//
		//	waiNodeRoleFunction: Function?
		//		This function is used to compute the WAI role for a node.
		//
		waiNodeRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiLinkRoleFunction: Function?
		//		This function is used to compute the WAI role for a link.
		//
		waiLinkRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiSubgraphRoleFunction: Function?
		//		This function is used to compute the WAI role for a subgraph.
		//
		waiSubgraphRoleFunction: function(ge) {
			return "group";
		},
		
		//	waiGraphRoleFunction: Function?
		//		This function is used to compute the WAI role for the graph.
		//
		waiGraphRoleFunction: function(graph) {
			return "group";
		},
		
		
		
		//
		//	waiNodeDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a node.
		//
		waiNodeDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiLinkDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a link.
		//
		waiLinkDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiSubgraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a subgraph.
		//
		waiSubgraphDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//	waiGraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description for the graph.
		//
		waiGraphDescriptionFunction: function(graph) {
			return "";
		},
		
		
		
		_bindWaiNode: function( /*ibm_ilog.diagram.GraphElement|ibm_ilog.diagram.Graph*/ge, /*div*/ waiNode, /*label*/ waiDesc) {
			//
			//	summary:	
			//		Adds private WAI information to graph element and the WAI node. 
			//
			//	tags:
			//		private
			//
			ge.__diagramWaiNode = waiNode;
			ge.__diagramWaiDesc = waiDesc;
			waiNode.__graphElement = ge;
		},
		
		_onWaiFocus: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has received focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_onWaiBlur: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has lost focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_waiInvisibleStyle: {
			position: 'absolute',
			left: '0px',
			top: '0px',
			width: '100%',
			height: '100%',
			overflow:"visible"
		},
		
		_initializeGraphWai: function() {
			//
			//	summary:
			//		Initializes the infrastructure to support WAI screen reading on the graph elements.
			//
			//	tags:
			//		private
			//
			
			// when the widget receives focus, forward to appropiate WAI node 
			this._connections.push(dojo.connect(this.domNode, "onfocus", this, "_onDiagramFocus"));
			
			// create root node at a hidden position
			var waiRoot = dojo.create('div', {style:{
				position: 'absolute',
				left: '0px',
				top: '0px',
				width: '100%',
				height: '100%',
				overflow: "visible",
				'z-index': "-2"
			}}, this.domNode, "first");
			
			this._createWaiNode(this._graph, this.id, waiRoot, this.waiGraphRoleFunction(this._graph));
			
			this._graph.__diagramWaiNode.setAttribute('tabIndex','0');
			
			dijit.setWaiState(this._graph.__diagramWaiNode, "label", this.waiGraphLabelFunction(this._graph));
			if (this.enableGraphWaiDescribedBy) {
				this._graph.__diagramWaiDesc.innerHTML = this.waiGraphDescriptionFunction(this._graph);
			}
			
			// connect to onfocus/onblur
			if (!dojo.isIE) {
				waiRoot.addEventListener("focus", dojo.hitch(this, "_onWaiFocus"), true /*capturing phase*/);
				waiRoot.addEventListener("blur", dojo.hitch(this, "_onWaiBlur"), true /*capturing phase*/);
			} else {
				// use focusin focusout which DOES bubble up (IE does NOT support capturing phase)
				this._connections.push(dojo.connect(waiRoot, "focusin", dojo.hitch(this, "_onWaiFocus")));
				this._connections.push(dojo.connect(waiRoot, "focusout", dojo.hitch(this, "_onWaiBlur")));
			}
			
			this._connections.push(dojo.connect(this, "onNodeCreated", this, "_registerNodeWai"));
			this._connections.push(dojo.connect(this, "onSubgraphCreated", this, "_registerSubgraphWai"));
			this._connections.push(dojo.connect(this, "onLinkCreated", this, "_registerLinkWai"));
			
			this._connections.push(dojo.connect(this, "onFocusedElementChanged", this, "_updateElementWaiFocus"));
			//
			this._connections.push(dojo.connect(this.getSelection(), "onSelectionChanged", this, "_updateSelectedState"));
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, "_destroyWaiNode"));
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/refreshWaiLabel', this, "_updateWaiLabel"));
			
			this._subscriptions.push(dojo.subscribe('/ibm_diagram/GraphElement/labelChanged', this, "_updateWaiLabel"));
			
			this._subscriptions.push(dojo.subscribe("/ibm_diagram/Subgraph/collapsedChanged", this, "_updateSubgraphCollapsed"));
		},
		
		_getWaiNode: function(e) {
			//
			//	summary:
			//		Returns the WAI node associated with a graph element.
			//
			//	tags:
			//		private
			//
			return e.__diagramWaiNode;
		},
		
		_onDiagramFocus: function(subgraph) {
			//
			//	summary:
			//		Used when the widget receives HTML focus. Forwards it to the current focused element.
			//
			//	tags:
			//		private
			//
			//	TODO: check this is needed (maybe tabTndex is always enough).
			//
			var wainode = this._getWaiNode(this._focusedElement|| this._graph)
			if(wainode) {
				dijit.focus(wainode);
			}
		},
		
		_updateSubgraphCollapsed: function(subgraph) {
			//
			//	summary:
			//		Used when a subgraph collapsed state has changed. Updates the corresponding WAI property. 
			//
			//	tags:
			//		private
			//
			var wainode = this._getWaiNode(subgraph);
			if(wainode) {
				dijit.setWaiState(wainode, "expanded", !subgraph.isCollapsed());
			}
		},
		
		_updateSelectedState: function(selected, deselected) {
			//
			//	summary:
			//		Used a graph element is selected or deselected. Updates the correspondig WAI attribute.
			//
			//	tags:
			//		private
			//
			var selection = this.getSelection();
			dojo.forEach(deselected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						dijit.setWaiState(wai, "selected", false);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
			dojo.forEach(selected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						dijit.setWaiState(wai, "selected", true);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
		},
		
		_updateElementWaiFocus: function(old, current) {
			//
			//	summary:
			// 		Change wai focus to node associated with the current focused element position
			//
			//	tags:
			//		private
			//
			var oldWai = (old && this._getWaiNode(old)) || this._getWaiNode(this._graph);
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);

			//	TODO: [av] try aria-activedescendant instead?

			oldWai.setAttribute("tabIndex", "-1");
			curWai.setAttribute("tabIndex", "0");

			dijit.focus(curWai);
		},

		_htmlRefocus: function() {
			//
			//	summary:
			//		Set the HTML focus into the appropiate component based on the current widget state.
			//
			var current = this.getFocusedElement();  
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);
			dijit.focus(curWai);
		},
		
		_getFunctionTriad: function(pre, post) {
			//
			//	summary:
			// 		helper. returns a {N:L:S:} for use with GraphUtil.dispatchClass().
			//
			//	tags:
			//		private
			//
			if (pre && pre.length) {
				return {
					N: this[pre + "Node" + post + "Function"],
					L: this[pre + "Link" + post + "Function"],
					S: this[pre + "Subgraph" + post + "Function"]
				};
			} else {
				return {
					N: this["node" + post + "Function"],
					L: this["link" + post + "Function"],
					S: this["subgraph" + post + "Function"]
				};
			}
		},
		
		_updateWaiDescription: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI description text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiDescription) {
				ge.__diagramWaiDescription.innerHTML = gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Description"));
			}
		},
		
		_updateWaiLabel: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI label text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiNode) {
				dijit.setWaiState(ge.__diagramWaiNode, 'label', gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Label")));
			}
		},
		
		_createWaiNode: function(element, id, waiParent, role) {
			//
			//	summary:
			//		create node div for WAI
			//
			//	tags:
			//		private
			//
			var waiNodeId = id + "_waiNode";
			var waiLabelId = id + "_waiLabel";
			var waiDescId = id + "_waiDesc";
			
			var atts = dojo.mixin({
				id: waiNodeId,
				tabIndex: -1
			}, {style:this._waiInvisibleStyle});
			
			var geWaiNode = dojo.create("div", atts, waiParent);
			var geWaiDesc;
			if (this.enableGraphWaiDescribedBy) {
				geWaiDesc = dojo.create("label", dojo.mixin({
					id: waiDescId,
					'for': waiNodeId
				}, {style:this._waiInvisibleStyle}), waiParent);
			}
			
			dijit.setWaiRole(geWaiNode, role);
			
			if (this.enableGraphWaiDescribedBy) {
				dijit.setWaiState(geWaiNode, 'describedBy', waiDescId);
			}
			
			this._bindWaiNode(element, geWaiNode, geWaiDesc);
		},
		
		_registerGraphElementWai: function(ge, role) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Graph Element
			//
			//	tags:
			//		private
			//
			
			// get div parent of parent
			
			var container = ge.getParent(), parentWaiNode;
			
			if (!container || !container.__diagramWaiNode) {
				// R.warn("MissingWaiNode", container);
				return;
			}
			
			parentWaiNode = container.__diagramWaiNode;
			
			// create node div for WAI
			this._createWaiNode(ge, ge.getId(), parentWaiNode, role);
			
			this._updateWaiLabel(ge);
			this._updateWaiDescription(ge);
		},
		
		_destroyWaiNode: function(ge) {
			if (ge.__diagramWaiLabel) {
				dojo.destroy(ge.__diagramWaiLabel);
			}
			if (ge.__diagramWaiNode) {
				dojo.destroy(ge.__diagramWaiNode);
			}
		},
		
		_registerLinkWai: function(link) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Link
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(link, this.waiLinkRoleFunction());
			
		},
		
		_registerNodeWai: function(node) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Node
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(node, this.waiNodeRoleFunction());
			
		},
		
		_registerSubgraphWai: function(subgraph) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Subgraph
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(subgraph, this.waiSubgraphRoleFunction());
			
			this._updateSubgraphCollapsed(subgraph);
			subgraph.getGraph().__diagramWaiNode = subgraph.__diagramWaiNode;
			
		}
	};

	// Add _Diagram features to Diagram if not on mobile.
	// (done here or in Diagram.js, depending on the load order - layer or not)
	if(dojo.config._notDiagramForMobile &&
			ibm_ilog.diagram.widget._Diagram &&
			ibm_ilog.diagram.widget.Diagram &&
			ibm_ilog.diagram.widget.Diagram.prototype &&
			!ibm_ilog.diagram.widget.Diagram.prototype._isDiagramExtended)
		dojo.extend(ibm_ilog.diagram.widget.Diagram, ibm_ilog.diagram.widget._Diagram);
})();

}

if(!dojo._hasResource["ibm_ilog.diagram.SwimLane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm_ilog.diagram.SwimLane"] = true;
dojo.provide("ibm_ilog.diagram.SwimLane");



dojo.declare("ibm_ilog.diagram.SwimLane", [ibm_ilog.diagram.Subgraph], {
	// summary:
	//		Represents a swim lane in a Dojo Diagrammer graph.
	// description:
	//		The <code>SwimLane</code> object is a special type of <code>Subgraph</code>
	//		used to divide a diagram into horizontal or vertical "stripes" that typically
	//		represent the different participants in a business process or workflow diagram.
	//		The differences with a plain <code>Subgraph</code> object are:
	//		<ul>
	//		  <li>
	//		    The swim lane has its own position and size, it does not adapt automatically to its contents.
	//		  </li>
	//		  <li>
	//		    The position and orientation of the title bar can be changed using the <code>setVertical()</code>
	//		    and <code>setRightToLeft()</code> methods.
	//		  </li>
	//		  <li>
	//		    Nested swim lanes (that is, swim lanes contained in another swim lane) can be automatically
	//		    stacked inside their parent lane (see <code>setStackSublanes</code>).
	//		  </li>
	//		</ul>
	
	_vertical: false,
	_rightToLeft: false,
	_stackSublanes: true,
	
	_isIBMDiagramSwimLane: true,
	_flattenForGL: true,
	_noTranslate: true,
	
	constructor: function()
	{
		if(!dojo.config.useGfxLayout)
			ibm_ilog.diagram.util.ErrorReporter.error("SwimLanesRequireGfxLayout");
		this._explicitSize = { width: 100, height: 100};
	},
	
    applyTemplate: function(/*String||Object*/template, /*Object||ibm_ilog.diagram.data.dtlbinding.BoundContext*/context) {
        // summary:
        //     Applies the specified template on this subgraph.
        
        var c = this.inherited(arguments);
        this._template = template;
        return c;
    },
	
	_subgraphInitialize: function()
	{
		this.inherited(arguments);
		if(this._graph){
			this._graph.setLayout(new ibm_ilog.diagram._SwimLaneLayout());
			this._graph._flattenForGL = true; // we mark both the subgraph and its graph for efficiency
			var g = this._graph.getParent();
			if(g.declaredClass == dojox.gfx.Group.prototype.declaredClass)
				g.setLayout(new ibm_ilog.diagram._GroupLayout());
		}
	},
	
	setVertical: function(vertical)
	{
		// summary:
		//		Sets the orientation of the swim lane.
		// description:
		//		Sets the orientation of the swim lane.
		// vertical: Boolean:
		//		If <code>true</code>, the title of the swim lane will be displayed at the top of the graph.
		//		If <code>false</code>, the title of the swim lane will be displayed (vertically)
		//		on the left (if <code>isRightToLeft()</code> is <code>false</code>) or
		//		on the right (if <code>isRightToLeft()</code> is <code>true</code>)
		//		of the graph.
		
		if(vertical != this._vertical){
			this._vertical = vertical;
			this.applyTemplate(this._template);
		}
        return this;
	},
	
	getVertical: function()
	{
		// summary:
		//		Gets the orientation of the swim lane.
		// description:
		//		Gets the orientation of the swim lane (see <code>setVertical</code>).
		
		return this._vertical;
	},
	
	setRightToLeft: function(rightToLeft)
	{
		// summary:
		//		Sets the horizontal direction of the swim lane.
		// description:
		//		Sets the horizontal direction of the swim lane.
		//		
		//		This property has an effect only if <code>isVertical()</code> is <code>false</code>.
		// rightToLeft: Boolean:
		//		If <code>false</code>, the title of the swim lane will be displayed on the left of the graph.
		//		If <code>rightToLeft</code> is <code>true</code>, the title of the swim lane
		//		will be displayed on the right of the graph.
		
		if(rightToLeft != this._rightToLeft){
			this._rightToLeft = rightToLeft;
			this.applyTemplate(this._template);
		}
        return this;
	},
	
	getRightToLeft: function()
	{
		// summary:
		//		Gets the horizontal direction of the swim lane.
		// description:
		//		Gets the horizontal direction of the swim lane (see <code>setRightToLeft()</code>.
		
		return this._rightToLeft;
	},
	
	setStackSublanes: function(stack)
	{
		// summary:
		//		Sets the sub-lane stacking mode.
		// description:
		//		Sets the sub-lane stacking mode.
		//		
		//		Sub-lanes (that is, <code>SwimLane</code> objects contained in this <code>SwimLane</code>)
		//		are stacked vertically if <code>getVertical()</code> is <code>false</code>, or
		//		horizontally if <code>getVertical()</code> is <code>true</code>.
		// stack: Boolean:
		//		If <code>true</code>, sub-lanes will be stacked, otherwise they will be left to their
		//		current position and size.
		
		if(stack != this._stackSublanes){
			this._stackSublanes = stack;
			this.invalidate();
		}
        return this;
	},
	
	getStackSublanes: function()
	{
		// summary:
		//		Gets the sub-lane stacking mode.
		// description:
		//		Gets the sub-lane stacking mode (see <code>setStackSublanes()</code>).
		
		return this._stackSublanes;
	},
	
	getNested: function()
	{
		// summary:
		//		Returns true if this swim lane is a nested lane contained in another swim lane.
		
		var parent = this.getParent();
		return parent && parent._owningSubgraph && parent._owningSubgraph._isIBMDiagramSwimLane;
	},
	
	canCollapse: function() {
		// summary:
		//		Returns false if this swim lane is nested.
		// returns:
		//		True if this SwimLane is not nested, false if it is nested.
		
		return !this.getNested(); // Boolean
	},
	
    applyLayout: function(rect)
	{
		this.inherited(arguments);
		if (this._graphRect) {
			var gr = this._graphRect.getBoundingBox();
			var t = this._graphRect.getShapeToContainerTransform(this._graph);
			gr = t.transformRectangle(gr);
			this._graph.setMoveLimits(gr);
		}
	}
});

// The default template for SwimLanes
ibm_ilog.diagram.SwimLane.defaultTemplate = ibm_ilog.diagram.declareTemplate(ibm_ilog.diagram.templates.defaultSwimLaneTemplate);

ibm_ilog.diagram.SwimLane.nodeType = dojox.gfx.Group.nodeType;
ibm_ilog.diagram.SwimLane.defaultShape = {
    type: 'swimlane'
};

ibm_ilog.diagram.templating.declareBindableProperty(ibm_ilog.diagram.Subgraph, "poolSpacing", 20);

dojo.declare('ibm_ilog.diagram._SwimLaneLayout', ibm_ilog.diagram.gfxlayout.CanvasLayout, {
	// summary:
	//		A private subclass of CanvasLayout used in swim lanes to stack sub-lanes.
	// tags:
	//		private
	
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        if (count === 0)
            return;
        var sl = this._container._owningSubgraph;
        var pos = 0;
        for (i=0; i<count;++i) {
            var child  = children[i];
            // Process only panels (ie a Group with a Layout)
            if (!child.children || !child.getLayout())
                continue;
			var sz = child._isIBMDiagramSwimLane ?
						child.getExplicitSize() :
						child.getPreferredSize();
            var x = 0, y = 0, w = sz.width, h = sz.height;
            // Stack sub-lanes if required.
            if(sl && sl._stackSublanes && child._isIBMDiagramSwimLane){
				var graphRect = sl._graphRect || sl._graphBackground;
           		var gbb = graphRect.getBoundingBox();
            	if(sl._vertical){
            		x = pos;
            		h = gbb.height;
            		pos += w;
            	} else {
            		y = pos;
            		w = gbb.width;
            		pos += h;
            	}
            	child.setExplicitSize({width:w, height:h});
            	var gt = sl._graph.getTransform();
            	if(gt){
            		var p = {x:x, y:y};
            		p = gt.inverse().transformPoint(p);
            		x = p.x; y = p.y;
            	}
            	//child.move(x, y);
				// move the top-left point of the child lane's base shape instead,
				// for the case when nodes are outside of the swim lane
				var baseShape = child.getBaseShape();
		        var rect = baseShape.getBoundingBox();
				var t = baseShape.getShapeToContainerTransform(this._container);
				rect = t.transformRectangle(rect);
				var dx = x - rect.x;
				var dy = y - rect.y;
				if(dx !== 0 || dy !== 0)
					child.applyLeftTransform(dojox.gfx.matrix.translate(dx, dy));
            }
            child.layout({x:0,y:0,width:w,height:h});
        }
    }
});

dojo.declare('ibm_ilog.diagram._GroupLayout', ibm_ilog.diagram.gfxlayout.CanvasLayout, {
	// summary:
	//		A private subclass of Layout used in swim lanes...
	// tags:
	//		private
	
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        if (count === 0)
            return;
        for (i=0; i<count;++i) {
            var child  = children[i];
            // Process only panels (ie a Group with a Layout)
            if (!child.children || !child.getLayout())
                continue;
            var sz = child.getPreferredSize();
            child.layout({x:0,y:0,width:sz.width,height:sz.height});
        }
    }
});

dojo.mixin(ibm_ilog.diagram.SwimLane, {
	
	// automaticHierarchicalLayout: boolean
	//		Controls whether swim lanes are automatically laid out when a Hierarchical layout
	//		is performed on the graph. If the flag is set to false, the beforeHierarchicalLayout/afterHierarchicalLayout
	//		methods are called respectively before and after the layout, to let applications implement their own layout.
	automaticHierarchicalLayout: true,
	
	beforeHierarchicalLayout: function(graph, hl){
		// summary:
		//		Called if automaticHierarchicalLayout is set to false before a Hierarchical layout
		//		is performed on the graph.
		// description:
		//		Called if automaticHierarchicalLayout is set to false before a Hierarchical layout
		//		is performed on the graph. This method must return an  array of swim lanes, that will
		//		be passed back to afterHierarchicalLayout.
		// graph: ibm_ilog.diagram.Graph
		//		The toplevel graph on which the layout is performed.
		// hl: ibm_ilog.graphlayout.hierarchical.HierarchicalLayout
		//		The hierarchical layout that is being performed.
		// returns:
		//		An array of Swimlane objects that will be passed back to afterHierarchicalLayout.
		return []; // Array
	},
	
	afterHierarchicalLayout: function(graph, hl, swimlanes){
		// summary:
		//		Called if automaticHierarchicalLayout is set to false after a Hierarchical layout
		//		is performed on the graph.
		// description:
		//		Called if automaticHierarchicalLayout is set to false after a Hierarchical layout
		//		is performed on the graph. This method should perform a custom layout on the swim lanes.
		// graph: ibm_ilog.diagram.Graph
		//		The toplevel graph on which the layout is performed.
		// hl: ibm_ilog.graphlayout.hierarchical.HierarchicalLayout
		//		The hierarchical layout that is being performed.
		// swimlanes: Array
		//		The array of SwimLane obejcts that was returned by beforeHierarchicalLayout.
	},
	
	_beforeLayout: function(graph, hl)
	{
		// summary:
		//		Creates HierarchicalSwimLaneConstraint objects for all swim lanes in the graph.
		
		if (!this.automaticHierarchicalLayout) {
			return this.beforeHierarchicalLayout(graph, hl);
		}
		
		// collect all swim lanes
		var swimlanes = [];
		this._getSwimLanes(graph, swimlanes);
		
		var n = swimlanes.length;
		
		if(n == 0)
			return null;
		
		var l = swimlanes[0];

		var dir = l._vertical ?
				ibm_ilog.graphlayout.Direction.BOTTOM :
					(l._rightToLeft ?
							ibm_ilog.graphlayout.Direction.LEFT :
								ibm_ilog.graphlayout.Direction.RIGHT);
		
		if(dir == ibm_ilog.graphlayout.Direction.RIGHT)
			swimlanes.reverse();
		
		hl.setFlowDirection(dir);
		
		// create a swim lane constraint for each lane
		for(var i = 0; i < n; i++){
			l = swimlanes[i];
			
			// add all nodes in a node group
			var g = new ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup();
			var children = l._graph.children;
			for(var j = 0; j < children.length; j++){
				var node = children[j];
				if(node._isIBMDiagramNode && !node._isIBMDiagramSwimLane)
					g.add(node);
			}
			
			// compute margins
			var m = l.getGraphMargin();
			var wm = m;
			var em = m;
			if(l._isFirstLaneInPool || l._isLastLaneInPool){
				m = l.getPoolSpacing()/2;
				if(l._vertical || l._rightToLeft){
					if(l._isFirstLaneInPool)
						wm += m;
					if(l._isLastLaneInPool)
						em += m;
				} else {
					if(l._isFirstLaneInPool)
						em += m;
					if(l._isLastLaneInPool)
						wm += m;
				}
			}
			
			// create the constraint
			var c = new ibm_ilog.graphlayout.hierarchical.HierarchicalSwimLaneConstraint(g, 0, i, wm, em);
			l._hierarchicalConstraint = c;
			hl.addConstraint(c);
		}
		
		// return the swim lane (or null if empty)
		return swimlanes.length > 0 ? swimlanes : null;
	},
	
	_getSwimLanes: function(graph, list)
	{
		var children = graph.children;
		var n = children.length;
		for(var i = 0; i < n; i++){
			var child = children[i];
			if(child._isIBMDiagramSwimLane){
				// expand all swim lanes, collapsed lanes are not well supported by graph layout.
				if(child.isCollapsed())
					child.setCollapsed(false, false);
				var old = list.length;
				// first see if it has nested swim lanes in it:
				this._getSwimLanes(child._graph, list);
				if(list.length == old) // no: return it in the list.
					list.push(child);
				else {
					// don't stack lanes automatically during layout
					child._oldStackSublanes = child._stackSublanes;
					child.setStackSublanes(false);
				}
				// mark the first and last lane/sublane inside toplevel lanes,
				// so we can add spacing margins
				if(!child.getParent()._owningSubgraph){
					list[old]._isFirstLaneInPool = true;
					list[list.length-1]._isLastLaneInPool = true;
				}
			}
		}
		
		return list;
	},
	
	_afterLayout: function(graph, hl, swimlanes)
	{
		// summary:
		//		Updates the position/size of all swim lanes according to info computed by the hierarchical layout.
		
		if (!this.automaticHierarchicalLayout) {
			this.afterHierarchicalLayout(graph, hl, swimlanes);
			return;
		}
		
		// First update the swim lanes that actually contain nodes
		// (those that correspond to constraints)
		var n = swimlanes.length;
		var i, l;
		for(i = 0; i < n; i++){
			l = swimlanes[i];
			var c = l._hierarchicalConstraint;
			var bb = c.getCalcBoundingBox();
			
			// add north/south margins, and remove extra west/east margin
			// corresponding to pool spacing.
			var m = l.getGraphMargin();
			var wm = c.getWestMinMargin() - m;
			var em = c.getEastMinMargin() - m;
			if(l._vertical){
				bb.y -= m;
				bb.height += 2*m;
				bb.x += wm;
				bb.width -= wm + em;
			} else {
				bb.x -= m;
				bb.width += 2*m;
				bb.y += l._rightToLeft ? wm : em;
				bb.height -= wm + em;
			}
			this._moveResizeLane(graph, l, bb);
		}
		
		// make sure all sublanes have the correct size already...
		ibm_ilog.diagram.util.Batch.endBatch();
		
		// Now, update swim lanes that contain other swim lanes,
		// and that must just fit their children lanes
		this._updateParentLanes(graph);
		
		// cleanup
		for(i = 0; i < n; i++){
			l = swimlanes[i];
			hl.removeConstraint(l._hierarchicalConstraint);
			delete l._hierarchicalConstraint;
		}
	},
	
	_moveResizeLane: function(graph, sl, rect)
	{
		// summary:
		//		Moves/resizes a lane so that its 'graph rect' becomes rect.
		//		Rect is given in 'graph' coordinates.
		
		// we will work in the coord system of the lane, so transform
		// the rect (which is passed in graph coords) to this system:
		var t = sl.getTransformToContainer(graph);
		if(t)
			rect = t.inverse().transformRectangle(rect);
		
		// get current lane's subgraph rect, to compute header width/height
		var graphRect = sl._graphRect || sl._graphBackground;
		var gr = graphRect.getBoundingBox();
        var t = graphRect.getShapeToContainerTransform(sl) || dojox.gfx.matrix.identity;
		gr = t.transformRectangle(gr);

		// current lane bounds: we do not use the real bounds including all nodes,
		// because at this point nodes can be completely ouside of the lane, 
		// instead we get the current explicit size and the current transform...
		t = sl.getTransform();
		var bb = {
				x: t ? t.dx : 0,
				y: t ? t.dy : 0,
				width : sl.getExplicitSize().width || sl.getBounds().width,
				height : sl.getExplicitSize().height || sl.getBounds().height
		};
		
		// lane translation to get to the desired position
		// (taking into account subgraph offsets)
		var dx = rect.x - gr.x - bb.x;
		var dy = rect.y - gr.y - bb.y;
		
		// resize lane (also taking into account diff between graph rect and lane bbox)
		var sz = bb;
		var w = rect.width + sz.width - gr.width;
		var h = rect.height + sz.height - gr.height;
		sl.setExplicitSize({width:w, height:h});
		
		// translate the lane,
		sl.applyLeftTransform(dojox.gfx.matrix.translate(dx, dy));
		
		// and inversely translate its graph
		// so that nodes still appear at the same position (computed by the layout)
		
		sl._graph.applyLeftTransform(dojox.gfx.matrix.translate(-dx, -dy));
	},
	
	_updateParentLanes: function(graph)
	{
		var children = graph.children;
		var n = children.length;
		for(var i = 0; i < n; i++){
			var child = children[i];
			if(child._isIBMDiagramSwimLane && !child._hierarchicalConstraint){
				var graphBounds = child._graph.getBoundingBox();
				t = child._graph.getShapeToContainerTransform(graph);
				if(t)
					graphBounds = t.transformRectangle(graphBounds);
				this._moveResizeLane(graph, child, graphBounds);
				this._updateParentLanes(child._graph);
				child.setStackSublanes(child._oldStackSublanes);
				delete child._oldStackSublanes;
			}
		}
	}
});

}

if(!dojo._hasResource['ibm_ilog.diagram.util.PrintUtil']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['ibm_ilog.diagram.util.PrintUtil'] = true;
dojo.provide('ibm_ilog.diagram.util.PrintUtil');




(function(){
	var R = ibm_ilog.diagram.util.ErrorReporter;

	/*=====
	ibm_ilog.diagram.util.__PrintUtilArgs = function(){
		//	getTargetSurfaceDiv: Function?
		//		This is used to get the HTML div tag where the target surfaces are going to be drawn
		//	postProcessing: Function?
	 	//		This function is called after every target surface is created, to customize any part necessary
		
		// [av] TODO the following doc style is copied from dojo.xhr but is not currently working.
		getTargetSurfaceDiv = function(row,column) {
			// [Mandatory]
			// row: the row number
			// column: the column number
			// return: the div tag where the surface will be drawn
		},
		postProcessing= function(row,column) {
			// [optional]
			// row: the row number
			// column: the column number
			// surface: the the drawn surface
			// group: the complete gfx.group drawn (typically the complete graph)
		},
	};
=====*/
	
dojo.declare("ibm_ilog.diagram.util.PrintUtil",null,{
	constructor: function( /*ibm_ilog.diagram.util.__PrintUtilArgs*/ kwArgs ) {
		this._kwArgs = kwArgs;
	},
	printGraph: function(graph,bounding,width,height,pagesWidth,pagesHeight,paperColor){
		// Summary:
		// 		Separates the given graph boundingBox into pagesWidth horizontal surfaces and pagesHeight vertical surfaces.
		//
		// graph:
		// 		the graph to be separated into multiple surfaces
		// bounding:
		// 		[Rect] indicates the area to be show. If missing, the complete graph is shown
		// width:
		// 		the width in pixels of every single surface
		// height:
		// 		the height in pixels of every single surface
		// pagesWidth:
		// 		the amount of horizontal surfaces into be split
		// pagesHeight:
		// 		the amount of vertical surfaces into be split
		// paperColor:
		//		[Color] this paperColor is used to hide not required part of the graph, when the target area has different aspect 
		//		ratio than the source bounding. If missing, white is used as default color.
		
		if (!dojox.gfx.utils.toSvg) {
			R.error("UnsupportedDojoFeature", "1.4.0");
		}
		
		// patch so that gfx (de)serialization does not crash on MultilineText deserialization :
		// The gfx serialize() method does not save the 'shape' property of a Group instance, but
		// will save the others 'default' gfx properties (fill, stroke, font...), so MultilineText instances
		// are serialized as Group instance :
		// { children : [....], transform:..., font: ... }.  (so, no shape:{type:'ml'}).
		// Therefore, when the json is deserialized, a Group instance is created, and setFont() is invoked
		// (which fails because Group.setFont does not exist).
		// Patch (Hack!!!!): we delete the getFont/setFont accessors so that the font is not serialized 
		var getFont = ibm_ilog.diagram.util.MultilineText.prototype.getFont;
		delete ibm_ilog.diagram.util.MultilineText.prototype.getFont;
		
		bounding = bounding || graph.getBoundingBox();
		paperColor = paperColor || 'white';
		// Calculate the zoom and necessary to fit and center
		var tMatrix = this._calculateTransformToFit(bounding, width, height, pagesHeight, pagesWidth);
		//Fix to avoid null bounding [defect 725]
		bounding = bounding || {x:0,y:0,width:0,height:0};
		var userFunc = this._kwArgs;
		for (var r = 0; r < pagesHeight; r++) {
			for (var c = 0; c < pagesWidth; c++) {
				var div = userFunc.getTargetSurfaceDiv(r,c);
				
				//Option 1: with this lines the code fails on div.appendChild(tempDiv);
				/*
				var tempDiv = dojo.create("div");
				div.appendChild(tempDiv);
				var surf = dojox.gfx.createSurface(tempDiv,width,height);
				*/
				
				
				//Option 2: This Line fails on vml.js line 1036: r.appendChild(b); (V:GROUP append V:RECT)
				var surf = dojox.gfx.createSurface(div,width,height);
				
				//Create temp surface to render object to and render.
				//Option 3: This Line fails on vml.js line 1036: r.appendChild(b); (V:GROUP append V:RECT)
				/*
				var divWindow = this._getWindow(div.ownerDocument);
				this._setDojoScope(divWindow);
				var surf = divWindow[dojox._scopeName].gfx.createSurface(div, width, height);
				*/
				
				surf.whenLoaded(dojo.hitch(this, function(){
					var obj = dojox.gfx.utils.serialize(graph);
					var group = surf.createGroup();
					group.applyLeftTransform(tMatrix);
					group.applyLeftTransform(dojox.gfx.matrix.translate(width * c * -1, height * r * -1));
					dojox.gfx.utils.deserialize(group, obj);
					if(userFunc.postProcessing){
						userFunc.postProcessing(r,c,surf,group);
					}
					this._cropToFit(group,bounding,r,c,pagesHeight,pagesWidth,width,height,tMatrix.xx,paperColor);
				}));
			}
		}
		
		// restore font accessors
		ibm_ilog.diagram.util.MultilineText.prototype.getFont = getFont;
	},
	_getWindow:function(anyDocument){
		var divWindow = anyDocument.defaultView ||  anyDocument.parentWindow;
		return divWindow;
	},
	_setDojoScope: function(window){
		
		window[dojox._scopeName] = this._getWindow(document)[dojox._scopeName];
		window[dojo._scopeName] = this._getWindow(document)[dojo._scopeName];
	},
	
	printViewport: function(diagram,width,height,pagesWidth,pagesHeight,paperColor){
		// Summary:
		// 		Separates the given diagram viewport into pagesWidth horizontal surfaces and pagesHeight vertical surfaces.
		// diagram:
		// 		the diagram which viewport will be separated into multiple surfaces
		// width:
		// 		the width in pixels of every single surface
		// height:
		// 		the height in pixels of every single surface
		// pagesWidth:
		// 		the amount of horizontal surfaces into be split
		// pagesHeight:
		// 		the amount of vertical surfaces into be split
		// paperColor:
		//		[Color] this paperColor is used to hide not required part of the graph, when the target area has different aspect 
		//		ratio than the source bounding. If missing, white is used as default color.
		var graph = diagram.getGraph();
		var bounding = diagram.getViewport().getViewRect();
		this.printGraph(graph,bounding,width,height,pagesWidth,pagesHeight,paperColor);
	},
	
	_calculateTransformToFit: function(printBB,dimx, dimy, h, w){
		// Summary:
		// 		Calculates and returns the Transformation necessary to fit the pintBB area in the printable area,
		//		compound by the merge of (dimx * w) X (dimy * h)
		
		//Fix to avoid null transformation [defect 725]
		if(!printBB){
			return dojox.gfx.matrix.identity;
		}
		
		if (printBB.width && printBB.height) {
			var sz = {
				width: dimx * w,
				height: dimy * h
			};
			var xr = sz.width / printBB.width;
			var yr = sz.height / printBB.height;
			var cent = {
				x: 0,
				y: 0
			};
			
			var zoom;
			if (xr < yr) {
				zoom = xr;
				cent.y = (printBB.width * (sz.height / sz.width) - printBB.height) / 2;
			}
			else {
				zoom = yr;
				cent.x = (printBB.height * (sz.width / sz.height) - printBB.width) / 2;
			}
			
			var zt = dojox.gfx.matrix.scale(zoom);
			var tt = dojox.gfx.matrix.translate({
				x: - printBB.x + cent.x,
				y: - printBB.y + cent.y
			});
			
			var t = dojox.gfx.matrix.multiply(zt, tt);
			
			return t;
		}
	},
	_cropToFit: function(group,bounding,r,c,pagesHeight,pagesWidth,width,height,zoom,paperColor){
		// Summary:
		// 		Creates rectangles over the surrounding surfaces, to limit the printed area to the given bounding 
		//		(hiding the parts that are out of the bounding when the target aspect ratio do not match the original aspect ratio)
		var rectx = Math.max((bounding.width),(width * pagesWidth/zoom));
		var recty = Math.max((bounding.height),(height * pagesHeight/zoom));
		
		if(r==0){
			group.createRect({ x: bounding.x, y: bounding.y-recty, width: rectx, height: recty }).setFill(paperColor);
		}
		if(c==0){
			group.createRect({ x: bounding.x-rectx, y: bounding.y, width: rectx, height: recty }).setFill(paperColor);
		}
		if(r==(pagesHeight-1)){
			group.createRect({ x: bounding.x, y: bounding.y+bounding.height, width: rectx, height: recty }).setFill(paperColor);
		}
		if(c==(pagesWidth-1)){
			group.createRect({ x: bounding.x+bounding.width, y: bounding.y, width: rectx, height: recty }).setFill(paperColor);
		}
	}
});

})();

}


dojo.i18n._preloadLocalizations("ibm_ilog.diagram.nls.ibm_ilog_diagram_base", ["ROOT","ar","cs","da","de","el","en","es","fi","fr","he","he-il","hu","it","iw","iw-il","ja","ko","nb","nl","no","pl","pt","pt-br","ro","ru","sk","sv","tr","xx","zh","zh-tw"]);
